@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 // SOFTWARE. 
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or 
-// other dealings in this Software without prior written authorization from  
-// WideStudio Development Team.
 
 #ifndef WSCBASELIST_H
 #define WSCBASELIST_H
@@ -58,7 +54,7 @@ class WSCremoteInstanceData{
     WSDLEX32 WSCushort getPort();
 
 };
-
+#define WS_BASE_LIST_MAX_ARRAY 64
 class WSCbaseList{
 
  private:
@@ -68,11 +64,31 @@ class WSCbaseList{
   WSClistData*  _need_init;
   WSClistData   _event_list;
   WSClistData   _event_client_list;
+#ifndef NO_REMOTE_INSTANCE
   WSClistData   _remote_instance_list;
   WSClistData   _export_instance_list;
   WSClistData   _export_instance_update_list;
   WSClistData   _export_instance_delete_list;
   WSCbool       _remote_data_list_updated;
+#endif
+//  WSCbase*      _tmp_array[WS_BASE_LIST_MAX_ARRAY];
+//  WSCbase**     _tmp_list;
+//  long          _tmp_list_len;
+
+#ifndef NO_MWT_FUNCTION
+  WSClistData   _root_instance_list;
+  WSClistData   _selected_instance_list;
+  WSCulong      _target_instance_id;
+#endif
+#ifndef WS_EMBED
+  WSClistData   _event_proc_list;
+  WSClistData   _event_proc_fname_list;
+  WSClistData   _event_proc_trg_list;
+  WSClistData   _event_proc_data_list;
+  void (*_event_proc_handler)(char*,void*);
+  WSCstring     _instance_position_cname;
+  WSCbool       _instance_position_mode;
+#endif
 
  public:
   WSDLEX32 WSCbaseList();
@@ -91,9 +107,15 @@ class WSCbaseList{
   WSDLEX32 void execInitialize();
   WSDLEX32 void  setUserData(char*,void *);
   WSDLEX32 void* getUserData(char*);
+  WSDLEX32 void  setUserLongData(char*,WSCulong);
+  WSDLEX32 WSCulong  getUserLongData(char*);
+  WSDLEX32 void  setUserStringData(char*,char*);
+  WSDLEX32 char* getUserStringData(char*);
 
   WSDLEX32 WSClistData* getInstanceList(char*);
+#ifndef NO_REMOTE_INSTANCE
   WSDLEX32 WSCRbase*    getRemoteInstance(char*);
+#endif
 
   WSDLEX32 void         setSearchInstanceHandler(void*);
   WSDLEX32 WSCbase*     searchInstance(char* class_name,char* objname);
@@ -104,12 +126,25 @@ class WSCbaseList{
   WSDLEX32 void     execEvent(long);
   WSDLEX32 void     execEvent(char*);
 
+#ifndef WS_EMBED
+  WSDLEX32 void     addEventProc(char* fname,char*,long);
+  WSDLEX32 void     addEventProc(char* fname,char*,void*);
+  WSDLEX32 void     delEventProc(char* fname,char*);
+  WSDLEX32 void     addEventProc(void(*hd)(void*),char*,void*);
+  WSDLEX32 void     delEventProc(void(*hd)(void*),char*);
+  WSDLEX32 void     execEventProc(char*);
+  WSDLEX32 void     delAllEventProc();
+  WSDLEX32 void     setEventProcHandler(void(*hd)(char*,void*));
+#endif
+
+#ifndef NO_REMOTE_INSTANCE
   WSDLEX32 long     addRemoteInstanceData(WSCremoteInstanceData* idata);
   WSDLEX32 long     delRemoteInstanceData(WSCremoteInstanceData* idata);
   WSDLEX32 WSCbool  getRemoteInstanceListUpdated();
   WSDLEX32 void     setRemoteInstanceListUpdated(WSCbool);
+#endif
 
-
+#ifndef NO_REMOTE_INSTANCE
   WSDLEX32 long     addExportInstanceList(WSCbase* inst);
   WSDLEX32 long     delExportInstanceList(WSCbase* inst);
   WSDLEX32 long     addExportInstanceUpdateList(WSCbase* inst);
@@ -118,14 +153,42 @@ class WSCbaseList{
   WSDLEX32 long     addExportInstanceDeleteList(WSCbase* inst);
   WSDLEX32 long     delExportInstanceDeleteList(WSCbase* inst);
 
-
   WSDLEX32 WSClistData* getExportInstanceDeleteList();
   WSDLEX32 WSClistData* getExportInstanceUpdateList();
   WSDLEX32 WSClistData* getExportInstanceList();
+#endif
 
+#ifndef NO_MWT_FUNCTION
+#ifndef WS_EMBED
+  WSDLEX32 void      setInstanceId(long kind,WSCulong data);
+  WSDLEX32 WSCulong  getInstanceId(WSCulong id,long kind,WSCulong data);
+  WSDLEX32 WSCstring getInstanceData(WSCulong id,long kind,WSCulong data);
+  WSDLEX32 WSCbase*  getInstanceFromId(WSCulong id);
+  WSDLEX32 WSCulong  getIdFromInstance(WSCbase* inst);
+#endif
+#endif
+
+#ifndef NO_MWT_FUNCTION
+  WSDLEX32 static void      setInternalWindowCreateHandler(void(*)(WSCbase*));
+  WSDLEX32 static void*     getInternalWindowCreateHandler();
+  WSDLEX32 static void      setInternalWindowVisibleHandler(void(*)(WSCbase*));
+  WSDLEX32 static void*     getInternalWindowVisibleHandler();
+  WSDLEX32 static void      setInternalWindowUnvisibleHandler(void(*)(WSCbase*));
+  WSDLEX32 static void*     getInternalWindowUnvisibleHandler();
+  WSDLEX32 static void      setInternalWindowDestroyHandler(void(*)(WSCbase*));
+  WSDLEX32 static void*     getInternalWindowDestroyHandler();
+#endif
+#ifndef WS_EMBED
+  WSDLEX32 void      setNewInstancePositionMode(WSCbool fl);
+  WSDLEX32 WSCbool   getNewInstancePositionMode();
+  WSDLEX32 void      setNewInstanceClassName(char*);
+  WSDLEX32 WSCstring getNewInstanceClassName();
+#endif
+
+#ifndef NO_REMOTE_INSTANCE
   WSDLEX32 WSCremoteInstanceData*  getRemoteInstanceData(char* pname);
   WSDLEX32 WSClistData* getRemoteInstanceList();
-
+#endif
 };
 
 extern WSDLEX WSCbaseList* WSGIappObjectList();
