@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 // SOFTWARE. 
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall 
-// not be used in advertising or otherwise to promote the sale, use or other 
-// dealings in this Software without prior written authorization from  
-// Toshikaz Hirabayashi.
 
 #ifndef WSCBLINK_H
 #define WSCBLINK_H
@@ -44,14 +40,14 @@ class WSCblink{
     WSClistData  _blink_refreshes;
     long         _clock;
 
-    static void  _blink_proc(unsigned char,void*);
+    WSDLEX32 static void  _blink_proc(unsigned char,void*);
   public:
-    WSCblink();
-    virtual ~WSCblink();
-    long addBlinkInstance(WSCbase* ,unsigned char rate,WSCbool refresh);
-    long delBlinkInstance(WSCbase* );
-    WSCbool getBlinkFore(long rate);
-    WSClistData* getBlinkClients();
+    WSDLEX32 WSCblink();
+    WSDLEX32 virtual ~WSCblink();
+    WSDLEX32 long addBlinkInstance(WSCbase* ,unsigned char rate,WSCbool refresh);
+    WSDLEX32 long delBlinkInstance(WSCbase* );
+    WSDLEX32 WSCbool getBlinkFore(long rate);
+    WSDLEX32 WSClistData* getBlinkClients();
 };
 extern WSDLEX WSCblink* WSGIappBlink();
 #endif
