@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCCLASSINFOMATION_H
 #define WSCCLASSINFOMATION_H
@@ -41,50 +37,52 @@ class WSCclassInformation{
    WSCulong     _trigger_mask;
    WSCulong     _del_trigger_mask;
    WSClistData* _inf_table; 
-   WSClistData* _all_prop_list;
-   WSClistData* _prop_list_for_init;
+//   WSClistData* _all_prop_list;
+//   WSClistData* _prop_list_for_init;
+   WSClistData _all_prop_list;
+   WSClistData _prop_list_for_init;
    WSCindexData _all_ext_trigger_list;
    WSCindexData _ext_trigger_list;
    WSClistData  _del_ext_trigger_list;
 
    void* _bases;
-
-   void _create_inf_table();
+   WSCproperty* _cache;
+   WSDLEX32 void _create_inf_table();
 
  public:
-   char* cname;
+   const char* cname;
    long  cname_hash_value;
+   WSDLEX32 WSCclassInformation(const char* cname,WSCclassInformation* parent);
+   WSDLEX32 virtual ~WSCclassInformation();
 
-   WSCclassInformation(char* cname,WSCclassInformation* parent);
-   virtual ~WSCclassInformation();
+   WSDLEX32 WSCbool      getPropObj(char* prop_name,WSCproperty** prop);
+#ifndef WS_EMBED
+   WSDLEX32 WSCbool      getPropObj(char* prop_name,char** class_name);
+#endif
+   WSDLEX32 void         addPropObj(WSCproperty* prop);
+   WSDLEX32 void         setTrigger(long trg); 
+   WSDLEX32 void         setDelTrigger(long trg); 
+   WSDLEX32 WSCbool      existTrigger(long trg);
+   WSDLEX32 WSCbool      _existExtTrigger(long trg);
+   WSDLEX32 WSCbool      _existTrigger(long trg);
+   WSDLEX32 WSClistData* getHashTable();
+   WSDLEX32 WSClistData* getAllPropertyList();
+   WSDLEX32 WSClistData* getPropertyList();
+   WSDLEX32 WSClistData* getPropertyListForInit();
+   WSDLEX32 void*        getClassInstanceList();
+   WSDLEX32 void         setClassInstanceList(void*);
 
-   WSCbool      getPropObj(char* prop_name,WSCproperty** prop);
-   WSCbool      getPropObj(char* prop_name,char** class_name);
-   void         addPropObj(WSCproperty* prop);
-   void         setTrigger(long trg); 
-   void         setDelTrigger(long trg); 
-   WSCbool      existTrigger(long trg);
-   WSCbool      _existExtTrigger(long trg);
-   WSCbool      _existTrigger(long trg);
-   WSClistData* getHashTable();
-   WSClistData* getAllPropertyList();
-   WSClistData* getPropertyList();
-   WSClistData* getPropertyListForInit();
-   void*        getClassInstanceList();
-   void         setClassInstanceList(void*);
-
-   void         setExtTrigger(long trg,char* tname);
-   void         setDelExtTrigger(long trg);
-   WSCindexData* getExtTriggerList();
-   WSCindexData* getAllExtTriggerList();
-
-   WSCclassInformation* getParentClassInformation();
-};
-struct WSChashData{
-   WSChashData();
-   WSCclassInformation* _class_info;
-   WSCproperty* _property;
+   WSDLEX32 void         setExtTrigger(long trg,const char* tname);
+   WSDLEX32 void         setDelExtTrigger(long trg);
+   WSDLEX32 WSCindexData* getExtTriggerList();
+   WSDLEX32 WSCindexData* getAllExtTriggerList();
+   WSDLEX32 WSCclassInformation* getParentClassInformation();
 };
+//struct WSChashData{
+//   WSChashData();
+//   WSCclassInformation* _class_info;
+//   WSCproperty* _property;
+//};
 extern WSDLEX WSCclassInformation* WSGIappClassInformation();
 
 #define WSMFdefineUseDevice(CN,DN) \
@@ -95,45 +93,75 @@ extern WSDLEX WSCclassInformation* WSGIappClassInformation();
   if (_prop_init == False){ \
     _prop_init = True; \
     WSCproperty* prop; \
+    WSCbool cb_cast = canBurstCast(this); \
+    WSCclassInformation* cinf = getClassInformation(); \
+    char* cname = getClassName(); \
     prop = NULL;
 
+#ifndef WS_EMBED
 #define WSMFpropertyCreate(PropName,PropType,WorkVar,BuilderTitle) \
-  prop = new WSCproperty(PropName,getClassName()); \
-  getClassInformation()->addPropObj(prop); \
+  prop = new WSCproperty(PropName,cname); \
+  cinf->addPropObj(prop); \
   prop->setType( #PropType,_set_p_w ## WorkVar, _get_p_w  ## WorkVar, \
      _get_def_w ## WorkVar); \
+  prop->_can_burst_cast = cb_cast; \
   prop->_builder_title = BuilderTitle;
+#else
+#define WSMFpropertyCreate(PropName,PropType,WorkVar,BuilderTitle) \
+  prop = new WSCproperty(PropName,cname); \
+  cinf->addPropObj(prop); \
+  prop->setType( #PropType,_set_p_w ## WorkVar, _get_p_w  ## WorkVar, \
+     _get_def_w ## WorkVar); \
+  prop->_can_burst_cast = cb_cast;
+#endif
+
+#ifndef WS_EMBED
 #define WSMFpropertySetSelection(ValueNameList,ValueList) \
   if (ValueNameList != NULL || ValueList != NULL){ \
-    prop->setBuilderSelectValues(ValueNameList,ValueList); \
+    prop->_data_labels = ValueNameList; \
+    prop->_data = ValueList; \
   }
+#else
+#define WSMFpropertySetSelection(ValueNameList,ValueList)
+#endif
 
+#ifndef WS_EMBED
 #define WSMFpropertyVisibleOff(PropName) \
-  prop = new WSCproperty(PropName,getClassName()); \
+  prop = new WSCproperty(PropName,cname); \
   prop->_ignore = True; \
   prop->_builder_visible = False; \
+  prop->_can_burst_cast = canBurstCast(this); \
   getClassInformation()->addPropObj(prop);
+#else
+#define WSMFpropertyVisibleOff(PropName) 
+#endif
 
 #define WSMFpropertyInitExec(init) \
   prop->_init_execute = init;
 
+#ifndef WS_EMBED
 #define WSMFpropertyVisible(Vis) \
   prop->_builder_visible = Vis;
+#else
+#define WSMFpropertyVisible(Vis)
+#endif
 
 #define WSMFpropertyFileSelect(fl) \
   prop->_file_select = fl;
 
 #define WSMFpropertyValueChangeDef(CN,PropName,PropType) \
-  prop = new WSCproperty(PropName,getClassName()); \
-  getClassInformation()->addPropObj(prop); \
+  prop = new WSCproperty(PropName,cname); \
+  cinf->addPropObj(prop); \
   prop->_default_value_change = True; \
   prop->setType( #PropType,NULL, NULL, CN ## PropName ## _get_def_w); \
-  prop->_init_execute = True;
+  prop->_init_execute = True; \
+  prop->_can_burst_cast = cb_cast;
 
 #define WSMFpropertyDelete(PropName) \
-  prop = new WSCproperty(PropName,getClassName()); \
+  prop = new WSCproperty(PropName,cname); \
   prop->_not_use = True; \
-  getClassInformation()->addPropObj(prop);
+  prop->_can_burst_cast = cb_cast; \
+  cinf->addPropObj(prop); 
 
 #define WSMFpropertyCreateEnd };
 #define WSMFaddTrigger(TRG) getClassInformation()->setTrigger(TRG);
@@ -153,7 +181,9 @@ extern WSDLEX WSCclassInformation* WSGIappClassInformation();
     CN* obj = (CN*)ptr; \
     if ( obj->WorkVal != *(CT*)data){ \
       obj->WorkVal = *(CT*)data; \
-      obj->needUpdate(); \
+      if (obj->_update_registered == False && obj->_vis != False ){ \
+        obj->needUpdate(); \
+      } \
       obj->_absolute_draw = True; \
       obj->setWork ## PN(obj->WorkVal); \
     } \
@@ -201,12 +231,19 @@ extern WSDLEX WSCclassInformation* WSGIappClassInformation();
   void* CN::cast(char *class_name){ \
     if(class_name == NULL){ \
       return(this); \
-    }else if(!strcmp(class_name,#CN)){ \
+    }else if(!WSIFstrcmp(class_name,#CN)){ \
       return(this); \
     }else{ \
       return(VC::cast(class_name)); \
     } \
   } \
+  void* CN::cast2(char *class_name){ \
+    if(!WSIFstrcmp(class_name,#CN)){ \
+      return(this); \
+    }else{ \
+      return(VC::cast2(class_name)); \
+    } \
+  } \
   WSCclassInformation* CN::CN ## _info_ptr = NULL; \
   WSCclassInformation* CN::getClassInformation(){ \
     if (CN ## _info_ptr == NULL){ \
