@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCFORM_H
 #define WSCFORM_H
@@ -54,9 +50,10 @@ class WSCform :public WSCbase{
     short          _a1_color;
     short          _a2_color;
     short          _a3_color;
-    WSCuchar       _grad_no;
+#ifndef NO_GRADATION
     WSCuchar       _grad_margin;
-
+    unsigned       _grad_no:4;
+#endif
     unsigned  _anchor_top_use:1;
     unsigned  _anchor_bottom_use:1;
     unsigned  _anchor_left_use:1;
@@ -83,11 +80,13 @@ class WSCform :public WSCbase{
     WSMFpropertyDef(WSNbackPixmap,       short,     _bp_color    );
     WSMFpropertyDef(WSNshadowType,       char,      _shadow_type );
     WSMFpropertyDef(WSNvis,              WSCbool,   _vis         );
-    WSMFpropertyDef(WSNdet,              WSCbool,   _sensitive   );
+    WSMFpropertyDef(WSNdet,              WSCbool,   _sensitive_work );
     WSMFpropertyDef(WSNpixmapStyle,      WSCuchar,  _pixmap_style);
     WSMFpropertyDef(WSNmouse,   WSCushort,  _mouse_no    );
+#ifndef NO_GRADATION
     WSMFpropertyDef(WSNgradation,   WSCuchar,  _grad_no    );
     WSMFpropertyDef(WSNgradationMargin,   WSCuchar,  _grad_margin    );
+#endif
     WSMFpropertyDef(WSNanchorTop,     WSCushort,      _anchor_top  );
     WSMFpropertyDef(WSNanchorBottom,  WSCushort,      _anchor_bottom  );
     WSMFpropertyDef(WSNanchorLeft,    WSCushort,      _anchor_left  );
@@ -96,42 +95,49 @@ class WSCform :public WSCbase{
     WSMFpropertyDef(WSNanchorBottomFlag,  WSCbool,      _anchor_bottom_use  );
     WSMFpropertyDef(WSNanchorLeftFlag,    WSCbool,      _anchor_left_use  );
     WSMFpropertyDef(WSNanchorRightFlag,   WSCbool,      _anchor_right_use  );
+#ifndef NO_REMOTE_INSTANCE
     WSMFpropertyDef(WSNexport,   WSCbool,      _export_  );
+#endif
+    WSMFpropertyDef(WSNkeepPixmapCache,   WSCbool,      _keep_pixmap_cache  );
 
-    virtual long  _device_initialize();
-    virtual void  _exec_child_adjust_for_anchors();
+    WSDLEX32 virtual long  _device_initialize();
+    WSDLEX32 virtual void  _exec_child_adjust_for_anchors();
 static void      (*_ext_draw_proc)(void*);
-static WSCstring _policy_name;
+static char* _policy_name;
 
   public:
-    WSCform(WSCbase*,char*);
-    virtual ~WSCform();
+    WSDLEX32 WSCform(WSCbase*,char*);
+    WSDLEX32 virtual ~WSCform();
 
-    virtual void onResize(WSCrect*);
-    virtual void onVisibleChange(WSCbool);
-    virtual void onParentVisibleChange(WSCbool);
-    virtual void onGuiPolicyChange(long);
+    WSDLEX32 virtual void onResize(WSCrect*);
+    WSDLEX32 virtual void onVisibleChange(WSCbool);
+    WSDLEX32 virtual void onParentVisibleChange(WSCbool);
+    WSDLEX32 virtual void onGuiPolicyChange(long);
 
-    virtual void _adjust_for_anchors(WSCushort,WSCushort);
+    WSDLEX32 virtual void _adjust_for_anchors(WSCushort,WSCushort);
 
-    virtual long execEventProc(long ev,void*);
+    WSDLEX32 virtual long execEventProc(long ev,void*);
 
-    virtual long draw();
-    virtual long clear();
-    virtual long redraw();
-    virtual long cdraw();
+    WSDLEX32 virtual long draw();
+    WSDLEX32 virtual long clear();
+    WSDLEX32 virtual long redraw();
+    WSDLEX32 virtual long cdraw();
 //    virtual long execEventProc(long ev, void*);
 //    virtual long drawGriphand();
-    virtual void update();
-    virtual long getObjectType();
-    virtual WSClistData& getChildren();
-    virtual void setChildren(WSClistData&);
-    virtual void internalUpdate();
+    WSDLEX32 virtual void update();
+    WSDLEX32 virtual long getObjectType();
+    WSDLEX32 virtual WSClistData& getChildren();
+    WSDLEX32 virtual WSClistData* getChildrenPtr();
+    WSDLEX32 virtual void setChildren(WSClistData&);
+    WSDLEX32 virtual void internalUpdate();
+#ifndef NO_RESTACK_CHILD
+    WSDLEX32 virtual long restackChild(WSCbase*,int order);
+#endif
 
-    void execResizeEvent();
-    void setNoRefresh(WSCbool);
-    WSCbool getNoRefresh();
-    long getPixmapStyle();
+    WSDLEX32 void execResizeEvent();
+    WSDLEX32 void setNoRefresh(WSCbool);
+    WSDLEX32 WSCbool getNoRefresh();
+    WSDLEX32 long getPixmapStyle();
 
     WSMFguiClassDef(WSCform,WSCbase);
     WSMFsetUseDevice(formDev);
@@ -151,8 +157,10 @@ struct WSCform_draw_struct{
   short bottom_shadow_color;
   short back_pixmap;
   long shadow_type;
+#ifndef NO_GRADATION
   short gradation;
   short gradation_margin;
+#endif
 };
 
 
