@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCFUNCTIONLIST_H
 #define WSCFUNCTIONLIST_H
@@ -35,22 +31,22 @@ class WSCfunctionRegister{
   protected:
     WSCfunction* _function;
   public:
-    WSCfunctionRegister(char*,void*);
-    virtual ~WSCfunctionRegister();
+    WSDLEX32 WSCfunctionRegister(char*,void*);
+    WSDLEX32 virtual ~WSCfunctionRegister();
 };
 
 class WSCfunctionList{
   protected:
     WSClistData  _function_list;
   public:
-    void         addFunction(WSCfunction*);
-    WSCfunction* addFunction(char* name,void* proc);
+    WSDLEX32 void         addFunction(WSCfunction*);
+    WSDLEX32 WSCfunction* addFunction(char* name,void* proc);
 
-    WSCfunction* getFunction(char*);
-    void*        getFunctionAddr(char*);
+    WSDLEX32 WSCfunction* getFunction(char*);
+    WSDLEX32 void*        getFunctionAddr(char*);
 
-    void delFunction(WSCfunction*);
-    void delAll();
+    WSDLEX32 void delFunction(WSCfunction*);
+    WSDLEX32 void delAll();
 };
 extern WSDLEX WSCfunctionList*  WSGIappFunctionList();
 #endif
