@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCIMAGESET_H
 #define WSCIMAGESET_H
@@ -41,21 +37,21 @@ class WSCimageSet{
    WSCushort  _image_num;
    WSCindexData _default_image_handles;
  public:
-   WSCimageSet();
-   virtual ~WSCimageSet();
-   long  setCreateHandler(WSDimage* (*)());
+   WSDLEX32 WSCimageSet();
+   WSDLEX32 virtual ~WSCimageSet();
+   WSDLEX32 long  setCreateHandler(WSDimage* (*)());
 
-   short     getImageNo(char* iname);
-   WSDimage* getImage(short);
-   WSDimage* getImage(char* iname);
-   char*     getImageName(short);
-   long  setDefaultImageHandle(char* iname,void*);
-   void* getDefaultImageHandle(char* iname);
+   WSDLEX32 short     getImageNo(char* iname);
+   WSDLEX32 WSDimage* getImage(short);
+   WSDLEX32 WSDimage* getImage(char* iname);
+   WSDLEX32 char*     getImageName(short);
+   WSDLEX32 long  setDefaultImageHandle(char* iname,void*);
+   WSDLEX32 void* getDefaultImageHandle(char* iname);
 
-   long destroyImage(short);
-   long destroyImage(char* iname);
-   long update(char* iname);
-   long destroyAllImage();
+   WSDLEX32 long destroyImage(short);
+   WSDLEX32 long destroyImage(char* iname);
+   WSDLEX32 long update(char* iname);
+   WSDLEX32 long destroyAllImage();
 };
 extern WSDLEX WSCimageSet*  WSGIappImageSet();
 extern WSDLEX short WSGFimage(char*);
