@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #ifndef WSCLIST_H
 #define WSCLIST_H
@@ -50,8 +46,17 @@ class WSClist :public WSCscrForm{
      short       _select_fore_color;
      short       _icon_pixmap;
      short       _icon_pixmap_old;
+
+     char*     _upname;
+     char*     _dwname;
+     char*     _lname;
+     char*     _rname;
+     char*     _rtname;
+     char*     _tbname;
+     char*     _btbname;
+
      WSCushort   _w_bk;
-     WSCushort   _h_bk;
+//     WSCushort   _h_bk;
      WSCushort   _lb_width;
      WSCushort   _work_height_bk;
      short   _pre_select;
@@ -72,7 +77,9 @@ class WSClist :public WSCscrForm{
      unsigned    _need_ds_update:1;
      unsigned    _need_scr_redraw:1;
      unsigned    _need_update_btn_area:1;
-
+     unsigned    _enable_focus_move:1;
+     unsigned    _return_key_focus:1;
+ 
 
      WSMFpropertyDef(WSNitemHeight, WSCuchar, _lb_height );
      WSMFpropertyDef(WSNfont,   WSCuchar, _font   );
@@ -92,6 +99,16 @@ class WSClist :public WSCscrForm{
      WSMFpropertyDef(WSNenableInput,  WSCbool, _enable_input  );
      WSMFpropertyDef(WSNseparator,  char*, _separator  );
 
+    WSMFpropertyDef(WSNenableFocusMove,  WSCbool, _enable_focus_move );
+    WSMFpropertyDef(WSNupward,    char*,   _upname);
+    WSMFpropertyDef(WSNdownward, char*,   _dwname);
+    WSMFpropertyDef(WSNleftward,   char*,   _lname);
+    WSMFpropertyDef(WSNrightward,  char*,   _rname);
+    WSMFpropertyDef(WSNreturn,     char*,   _rtname);
+    WSMFpropertyDef(WSNtab,     char*,   _tbname);
+    WSMFpropertyDef(WSNbacktab,     char*,   _btbname);
+    WSMFpropertyDef(WSNreturnKeyFocus,   WSCbool, _return_key_focus);
+
     WSDLEX32 virtual void setWorkWSNwidth(WSCushort);
     WSDLEX32 virtual void setWorkWSNheight(WSCushort);
     WSDLEX32 virtual void setWorkWSNworkWidth(WSCushort);
@@ -113,11 +130,9 @@ class WSClist :public WSCscrForm{
 //    static void _mouse_press_work(WSCbase*);
 //    static void _mouse_release_work(WSCbase*);
 
-    WSDLEX32 virtual void onScrMousePress(WSCpoint*);
-    WSDLEX32 virtual void onScrMouseRelease(WSCpoint*);
-
     WSDLEX32 void _lb_adjust_work(long&,long,WSCbool,long&);
     WSDLEX32 virtual void _lb_adjust();
+    WSDLEX32 void _update_lb_height();
     WSDLEX32 void _data_from_file(char*);
     WSDLEX32 static void _input_focus_lose(WSCbase*);
     WSDLEX32 static void _input_fix_work(WSCbase*);
@@ -141,11 +156,17 @@ class WSClist :public WSCscrForm{
 //    virtual void onActivate();
     WSDLEX32 virtual void onSort();
     WSDLEX32 virtual void onItemSelected();
+    WSDLEX32 virtual void onItemPreSelected();
     WSDLEX32 virtual void onItemDoubleClicked();
     WSDLEX32 virtual void onInputFixed(WSCstring str,long pos,long column);
     WSDLEX32 virtual void onKey(WSDkeyboard*,WSCbool);
     WSDLEX32 virtual void onValueChange();
     WSDLEX32 virtual char* getDefaultProperty();
+    WSDLEX32 virtual void onScrMousePress(WSCpoint*);
+    WSDLEX32 virtual void onScrMouseRelease(WSCpoint*);
+    WSDLEX32 virtual void onFocusChange(WSCbool);
+    WSDLEX32 void _focus_move(long direct);
+
 
     WSDLEX32 void      setLabelHeight(WSCushort h);
     WSDLEX32 void      delAll();
@@ -159,7 +180,7 @@ class WSClist :public WSCscrForm{
     WSDLEX32 long      getPreSelectedPos();
     WSDLEX32 long      setPreSelectPos(long);
     WSDLEX32 long      getSelectedPos();
-    WSDLEX32 long      setSelectPos(long );
+    WSDLEX32 long      setSelectPos(long ,WSCbool create_event = True);
     WSDLEX32 WSCstring getSelectedItem();
     WSDLEX32 void      setTopPos(long);
     WSDLEX32 void      setBottomPos(long);
@@ -182,14 +203,22 @@ class WSClist :public WSCscrForm{
     WSDLEX32 long      setItemValue(long pos,long kind,long val);
     WSDLEX32 long      getItemValue(long pos,long kind,long* val);
     WSDLEX32 long      getSortPos();
+    WSDLEX32 char      getSortOrder(long col_no);
+    WSDLEX32 void      setSortPos(long col_no);
+    WSDLEX32 void      sort(long col_no,char order = WS_LIST_REVERSING_ORDER);
     WSDLEX32 WSCstring getInputString();
+    WSDLEX32 void      setInputString(WSCstring str);
+    WSDLEX32 long      getInputPos(long* pos,long* col);
+    WSDLEX32 long      setInputStatus(long status,long pos,long col=-1);
+
     WSDLEX32 void      cancelInput();
     WSDLEX32 char*     getSeparator();
     WSDLEX32 long      getSelectedNextPos(long pos);
     WSDLEX32 WSCstring getSelectedNextItem(long pos);
     WSDLEX32 WSCbool   getSelected(long pos);
-    WSDLEX32 WSCbool   setSelected(long pos,WSCbool);
-
+    WSDLEX32 WSCbool   setSelected(long pos,WSCbool,WSCbool create_event = True);
+    WSDLEX32 virtual WSCbool canFocus();
+    WSDLEX32 void    resetUpdateFlag();
 
     WSMFguiClassDef(WSClist,WSCscrForm);
 };
