@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCMESSAGEDIALOG_H
 #define WSCMESSAGEDIALOG_H
@@ -44,15 +40,17 @@ class WSCmessageDialog :public WSCbaseDialog{
       
 
   public:
-    WSCmessageDialog(WSCbase*,char*);
-    virtual ~WSCmessageDialog();
-    virtual  long initialize();
+    WSDLEX32 WSCmessageDialog(WSCbase*,char*);
+    WSDLEX32 virtual ~WSCmessageDialog();
+    WSDLEX32 virtual  long initialize();
 
 //    virtual void onResize(WSCrect* rect);
-    virtual void onActivate();
-    virtual void onGuiPolicyChange(long);
+    WSDLEX32 virtual void onActivate();
+    WSDLEX32 virtual void onGuiPolicyChange(long);
+
+    WSDLEX32 virtual void adjustForm();
+    WSDLEX32 virtual char* getDefaultProperty();
 
-    virtual void adjustForm();
     WSMFguiClassDef(WSCmessageDialog,WSCbaseDialog);
     WSMFversionDef(WSCmessageDialog);
 };
