@@ -22,10 +22,16 @@ struct _Col {
 	SQLCHAR			colName[32];
 	SQLSMALLINT		colNameLen;
 	SQLSMALLINT		colType;
-	SQLUINTEGER		colSize;
 	SQLSMALLINT		colScale;
+#ifndef SQL64
+	SQLUINTEGER		colSize;
 	SQLINTEGER		buffLen;
 	SQLINTEGER		len;
+#else
+	SQLULEN		colSize;
+	SQLLEN		buffLen;
+	SQLLEN		len;
+#endif
 	void*			buff;
 };
 
@@ -110,7 +116,6 @@ private:
 	void setValue(int index, WSCvariant* var);
 	void getError(SQLHANDLE	hstmt);
 	long getColTypeConv(long index);
-
 public:
 	WSCodbcDatabase*	_pDB;
 	SQLHANDLE		_hstmt;
@@ -123,6 +128,7 @@ public:
 	SQLCHAR			_szErrorMsg[SQL_MAX_MESSAGE_LENGTH + 1];
 	SQLINTEGER		_access;
 	SQLINTEGER		_Cursor;
+    long cancel();
 };
 
 #endif		// WSCODBCDATABASE_H
