@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2002 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCOPENGLFORM_H
 #define WSCOPENGLFORM_H
@@ -48,34 +44,37 @@ class WSCopenglForm   :public WSCform{
     unsigned  _context_created_fl:1;
 
 //    virtual long _device_initialize();
-    void _delete_context();
+    WSDLEX32 void _delete_context();
     long (*_create_context_handler)();
-    long _default_create_context();
-    long _create_context();
+    WSDLEX32 long _default_create_context();
+    WSDLEX32 long _create_context();
 
     long (*_update_context_handler)();
-    long _default_update_context();
-    long _update_context();
+    WSDLEX32 long _default_update_context();
+    WSDLEX32 long _update_context();
   public:
 
-    WSCopenglForm(WSCbase* base, char* name);
+    WSDLEX32 WSCopenglForm(WSCbase* base, char* name);
 
-    virtual ~WSCopenglForm();
-    virtual long draw();
-    virtual void update();
-    virtual long execEventProc(long,void*);
+    WSDLEX32 virtual ~WSCopenglForm();
+    WSDLEX32 virtual long draw();
+    WSDLEX32 virtual void update();
+    WSDLEX32 virtual long execEventProc(long,void*);
 
-    virtual void onVisibleChange(WSCbool);
-    virtual void onResize(WSCrect*);
-    void* getOpenGLContext();
+    WSDLEX32 virtual void onVisibleChange(WSCbool);
+    WSDLEX32 virtual void onParentVisibleChange(WSCbool);
+    WSDLEX32 virtual void onResize(WSCrect*);
+    WSDLEX32 virtual void onOpenGLInitialize();
+    WSDLEX32 virtual void onOpenGLDelete();
+    WSDLEX32 void* getOpenGLContext();
 
-    virtual long beginDraw();
-    virtual long endDraw();
+    WSDLEX32 virtual long beginDraw();
+    WSDLEX32 virtual long endDraw();
 
-    void setCreateContextHandler(long(*hd)());
-    void setUpdateContextHandler(long(*hd)());
-    void makeCurrent();
-    void noMakeCurrent();
+    WSDLEX32 void setCreateContextHandler(long(*hd)());
+    WSDLEX32 void setUpdateContextHandler(long(*hd)());
+    WSDLEX32 void makeCurrent();
+    WSDLEX32 void noMakeCurrent();
 
     WSMFguiClassDef(WSCopenglForm,WSCform);
     WSMFversionDef(WSCopenglForm);
