@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #ifndef WSCOPTION_H
 #define WSCOPTION_H
@@ -56,7 +52,7 @@ class WSCoption    :public WSCvlabel{
     WSDLEX32 void _menu_popdown();
     WSDLEX32 void _menu_update();
 WSDLEX32 static void      (*_ext_draw_proc)(void*);
-WSDLEX32 static WSCstring _policy_name;
+WSDLEX32 static char* _policy_name;
 
   public:
     WSDLEX32 WSCoption(WSCbase*,char*);
@@ -110,8 +106,10 @@ struct WSCoption_draw_struct{
   char* label_string;
   short label_pixmap;
   short blink_pixmap;
+#ifndef NO_GRADATION
   long  gradation;
   long  gradation_margin;
+#endif
   long  margin_left;
   long  margin_right;
   long  margin_top;
