@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCRADIOGROUP_H
 #define WSCRADIOGROUP_H
@@ -45,6 +41,15 @@ class WSCradioGroup   :public WSCform{
     short     _indicator_pixmap;
     short     _select_pixmap;
     short     _label_pixmap;
+
+     char*     _upname;
+     char*     _dwname;
+     char*     _lname;
+     char*     _rname;
+     char*     _rtname;
+     char*     _tbname;
+     char*     _btbname;
+     char*     _sc_str;
  
     unsigned  _font:4;
     unsigned  _margin:3;
@@ -54,6 +59,8 @@ class WSCradioGroup   :public WSCform{
     unsigned  _indicator_shadow_thick:4;
     unsigned  _indicator_on:1;
     unsigned  _orientation:1;
+    unsigned  _enable_focus_move:1;
+    unsigned  _return_key_focus:1;
 
     WSMFpropertyDef(WSNvalue,     WSCulong, _value );
     WSMFpropertyDef(WSNtitleString,   char*, _title_string );
@@ -71,27 +78,39 @@ class WSCradioGroup   :public WSCform{
     WSMFpropertyDef(WSNselectPixmap,    short,  _select_pixmap );
     WSMFpropertyDef(WSNlabelPixmap,    short,  _label_pixmap );
     WSMFpropertyDef(WSNorientation,     WSCuchar, _orientation  );
+    WSMFpropertyDef(WSNkeepPixmapCache,   WSCbool,  _keep_pixmap_cache    );
+
+    WSMFpropertyDef(WSNenableFocusMove,  WSCbool, _enable_focus_move );
+    WSMFpropertyDef(WSNupward,    char*,   _upname);
+    WSMFpropertyDef(WSNdownward, char*,   _dwname);
+    WSMFpropertyDef(WSNleftward,   char*,   _lname);
+    WSMFpropertyDef(WSNrightward,  char*,   _rname);
+    WSMFpropertyDef(WSNreturn,     char*,   _rtname);
+    WSMFpropertyDef(WSNtab,     char*,   _tbname);
+    WSMFpropertyDef(WSNbacktab,     char*,   _btbname);
+    WSMFpropertyDef(WSNreturnKeyFocus,   WSCbool, _return_key_focus);
+    WSMFpropertyDef(WSNshortcut,     char*,   _sc_str);
 
 
-
-    virtual void setWorkWSNbackColor(short);
-    virtual void  _adjust();
-    static void _value_ch_handler(WSCbase*);
+    WSDLEX32 virtual void setWorkWSNbackColor(short);
+    WSDLEX32 virtual void  _adjust();
+    WSDLEX32 static void _value_ch_handler(WSCbase*);
 
   public:
-    WSCradioGroup(WSCbase*,char*);
-    virtual ~WSCradioGroup();
+    WSDLEX32 WSCradioGroup(WSCbase*,char*);
+    WSDLEX32 virtual ~WSCradioGroup();
 
-    virtual void update();
-    virtual long draw();
+    WSDLEX32 virtual void update();
+    WSDLEX32 virtual long draw();
 
-    virtual void onValueChange();
-    virtual void onVisibleChange(WSCbool);
-    virtual char* getDefaultProperty();
+    WSDLEX32 virtual void onValueChange();
+    WSDLEX32 virtual void onVisibleChange(WSCbool);
+    WSDLEX32 virtual char* getDefaultProperty();
+    WSDLEX32 virtual void onFocusChange(WSCbool);
 
-    long setItemSensitive(short no, WSCbool flag);
-    WSCbool getItemSensitive(short no);
-    WSCvradio* getItem(short no);
+    WSDLEX32 long setItemSensitive(short no, WSCbool flag);
+    WSDLEX32 WSCbool getItemSensitive(short no);
+    WSDLEX32 WSCvradio* getItem(short no);
 
     WSMFguiClassDef(WSCradioGroup,WSCform);
     WSMFversionDef(WSCradioGroup);
