@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,20 +14,17 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #ifndef WSCSFORM_H
 #define WSCSFORM_H
 
 #include <WScom.h>
 #include <WSCform.h>
+#include <WSDkeyboard.h>
 
 #define WSCsform_VER  1010
 
@@ -76,6 +73,7 @@ WSDLEX32 static    void _move_adjust_work(void*);
     WSDLEX32 void _parse_pos(char*);
     WSDLEX32 void _clear_area();
     WSDLEX32 virtual void  _exec_child_adjust_for_anchors();
+    WSDLEX32 static WSCbool _special_key_handler(void*,WSDkeyboard*,WSCbool);
 
   public:
     WSDLEX32 WSCsform(WSCbase*,char*);
