@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCTYPESERVER_H
 #define WSCTYPESERVER_H
@@ -40,27 +36,28 @@ class WSCtypeServer{
   void  (*_delete_handler)(void *);
   void  (*_save_handler)(WSDserialize*,void *);
   void  (*_load_handler)(WSDserialize*,void *);
-  WSCstring _type_name;
+//  WSCstring _type_name;
 
  public:
-  WSCtypeServer();
-  virtual ~WSCtypeServer();
+  char* _type_name;
+  WSDLEX32 WSCtypeServer();
+  WSDLEX32 virtual ~WSCtypeServer();
 
-  void  setType(char* type);
-  void  setSaveHandler(void (*)(WSDserialize*,void*) );
-  void  setLoadHandler(void (*)(WSDserialize*,void*) );
-  void  setCreateHandler(void*(*)(void*));
-  void  setDeleteHandler(void(*)(void*));
+  WSDLEX32 void  setType(char* type);
+  WSDLEX32 void  setSaveHandler(void (*)(WSDserialize*,void*) );
+  WSDLEX32 void  setLoadHandler(void (*)(WSDserialize*,void*) );
+  WSDLEX32 void  setCreateHandler(void*(*)(void*));
+  WSDLEX32 void  setDeleteHandler(void(*)(void*));
 
-  void* getSaveHandler();
-  void* getLoadHandler();
-  long  getParentClassNum();
-  char* getParentClassName(long i);
-  WSCtypeServer* getParentClassServer(long,char** cname);
-  virtual void*  getParentClassPtr(char*,void*) = 0;
+  WSDLEX32 void* getSaveHandler();
+  WSDLEX32 void* getLoadHandler();
+  WSDLEX32 long  getParentClassNum();
+  WSDLEX32 char* getParentClassName(long i);
+  WSDLEX32 WSCtypeServer* getParentClassServer(long,char** cname);
+  WSDLEX32 virtual void*  getParentClassPtr(char*,void*) = 0;
 
-  void* createInstance(void*);
-  void  deleteInstance(void*);
+  WSDLEX32 void* createInstance(void*);
+  WSDLEX32 void  deleteInstance(void*);
   WSCbool isParentClass(char*);
   WSCbool isHit(char*);
 };
