@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCVIFIELD_H
 #define WSCVIFIELD_H
@@ -60,7 +56,7 @@ class WSCvifield   :public WSCvlabel{
     unsigned  _data_source_type:2;
     long      _select_pos1;
     long      _select_pos2;
-    unsigned  _kanji_in:1;
+    unsigned  _enable_fep:4;
     unsigned  _icursor:1;
     unsigned  _enable_focus_move:1;
     unsigned  _fspace:1;
@@ -83,7 +79,7 @@ class WSCvifield   :public WSCvlabel{
 
 //    WSMFpropertyDef(WSNcolumns,     WSCushort, _columns);
     WSMFpropertyDef(WSNcursorPos,   short,    _cur_pos);
-    WSMFpropertyDef(WSNkanjiIn,     WSCbool,  _kanji_in);
+    WSMFpropertyDef(WSNenableFep,     WSCbool,  _enable_fep);
     WSMFpropertyDef(WSNinterCur,    WSCbool,  _icursor);
     WSMFpropertyDef(WSNenableFocusMove,  WSCbool, _enable_focus_move );
     WSMFpropertyDef(WSNupward,    char*,   _upname);
@@ -104,82 +100,87 @@ class WSCvifield   :public WSCvlabel{
     WSMFpropertyDef(WSNdisplayOnly,  WSCbool, _display_only  );
 
 
-    virtual void setWorkWSNmaxLength(WSCushort data);
-    virtual void setWorkWSNlabelString(char* data);
-    virtual void getWorkWSNlabelString(char** data);
+    WSDLEX32 virtual void setWorkWSNmaxLength(WSCushort data);
+    WSDLEX32 virtual void setWorkWSNlabelString(char* data);
+    WSDLEX32 virtual void getWorkWSNlabelString(char** data);
 
-    static void _cursor_blink_handler(WSCuchar,void*);
-    static WSCbool _special_key_handler(void*,WSDkeyboard*,WSCbool);
-    static void _key_handler(void*,WSDkeyboard*,WSCbool);
+    WSDLEX32 static void _cursor_blink_handler(WSCuchar,void*);
+    WSDLEX32 static WSCbool _special_key_handler(void*,WSDkeyboard*,WSCbool);
+    WSDLEX32 static void _key_handler(void*,WSDkeyboard*,WSCbool);
 //    static void _focus_lose_handler(void*);
 
 
-    virtual void  _on_key_input(WSDkeyboard*,WSCbool); 
-    virtual long  _device_initialize();
+    WSDLEX32 virtual void  _on_key_input(WSDkeyboard*,WSCbool); 
+    WSDLEX32 virtual long  _device_initialize();
 
 //    char* _get_text();
-    void  _adjust_cbase();
-    void  _set_text(WSCushort*);
-    void  _update_text();
-    void  _fill_space();
-    void  _set_cur_pos(long absolute_pos);
-    void  _get_cur_pos(long* x,long* y,long* pos);
+    WSDLEX32 void  _adjust_cbase();
+    WSDLEX32 void  _set_text(WSCushort*);
+    WSDLEX32 void  _update_text();
+    WSDLEX32 void  _fill_space();
+    WSDLEX32 void  _set_cur_pos(long absolute_pos);
+    WSDLEX32 void  _get_cur_pos(long* x,long* y,long* pos);
 
 //    WSCushort* _get_disp_string();
-    WSCbool _return_key_ok();  
+    WSDLEX32 WSCbool _return_key_ok();  
 
-    void _fep_cursor_adjust(WSDkeyboard*);
-virtual long _cursor_move(char direction);
-    void _get_disp_cur_pos(long* x,long* y);
-    long _get_absolute_pos(long x,long y);
-    void _set_select_pos1(long pos);
-    void _set_select_pos2(long pos);
-    void _delete_select_string();
-    void _reset_select_string();
-    void _replace_string(WSCushort*);
-    void _adjust_max_len();
-    void _set_ds_instance();
+    WSDLEX32 void _fep_cursor_adjust(WSDkeyboard*);
+WSDLEX32 virtual long _cursor_move(char direction);
+    WSDLEX32 void _get_disp_cur_pos(long* x,long* y);
+    WSDLEX32 long _get_absolute_pos(long x,long y);
+    WSDLEX32 void _set_select_pos1(long pos);
+    WSDLEX32 void _set_select_pos2(long pos);
+    WSDLEX32 void _delete_select_string();
+    WSDLEX32 void _reset_select_string();
+    WSDLEX32 void _replace_string(WSCushort*);
+    WSDLEX32 void _adjust_max_len();
+    WSDLEX32 void _set_ds_instance();
+
+    WSDLEX32 void _focus_move_for_combo(long);
 
   public:
-    WSCvifield(WSCbase*,char*);
-    virtual ~WSCvifield();
+    WSDLEX32 WSCvifield(WSCbase*,char*);
+    WSDLEX32 virtual ~WSCvifield();
 
-    virtual void onCursorBlink();                  
-    virtual void onFocusChange(WSCbool);
-    virtual void onWindowFocusChange(WSCbool);
-    virtual void onSpecialFocusChange(WSCbool);
-    virtual void onMousePress(WSCpoint*);
-    virtual void onMouseRelease(WSCpoint*);
-    virtual void onMouseMove(WSCpoint*);
-    virtual void onKeyHook(WSDkeyboard*);
-    virtual void onKeyInput(WSDkeyboard*,WSCbool);
-    virtual void onKey(WSDkeyboard*,WSCbool);
-    virtual void onActivate();
-    virtual void onValueChange();
-    virtual void onSelectionChange(WSCbool);
-    virtual void onVisibleChange(WSCbool);
-    virtual void onParentVisibleChange(WSCbool);
+    WSDLEX32 virtual void onCursorBlink();                  
+    WSDLEX32 virtual void onFocusChange(WSCbool);
+    WSDLEX32 virtual void onWindowFocusChange(WSCbool);
+    WSDLEX32 virtual void onSpecialFocusChange(WSCbool);
+    WSDLEX32 virtual void onMousePress(WSCpoint*);
+    WSDLEX32 virtual void onMouseRelease(WSCpoint*);
+    WSDLEX32 virtual void onMouseMove(WSCpoint*);
+    WSDLEX32 virtual void onKeyHook(WSDkeyboard*);
+    WSDLEX32 virtual void onKeyInput(WSDkeyboard*,WSCbool);
+    WSDLEX32 virtual void onKey(WSDkeyboard*,WSCbool);
+    WSDLEX32 virtual void onActivate();
+    WSDLEX32 virtual void onValueChange();
+    WSDLEX32 virtual void onSelectionChange(WSCbool);
+    WSDLEX32 virtual void onVisibleChange(WSCbool);
+    WSDLEX32 virtual void onParentVisibleChange(WSCbool);
 
-    virtual void setData(WSCvariant*,long code = WS_EN_DEFAULT);
-    virtual long getDataSourceType();                                     
-    virtual char* getDataSourcePropertyName();
+    WSDLEX32 virtual void setData(WSCvariant*,long code = WS_EN_DEFAULT);
+    WSDLEX32 virtual long getDataSourceType();                                     
+    WSDLEX32 virtual char* getDataSourcePropertyName();
 
-    virtual long draw();
-    virtual void update();
+    WSDLEX32 virtual long draw();
+    WSDLEX32 virtual void update();
+
+    WSDLEX32 void setCursorFore(WSCbool fl);
+    WSDLEX32 WSCbool getCursorFore();
+    WSDLEX32 WSCbool setCurPos(long pos); //pos >= 0
+    WSDLEX32 void replaceSelectedString(char*,long encode = WS_EN_DEFAULT);
+    WSDLEX32 WSCstring getSelectedString();
+    WSDLEX32 void deleteSelectedString();
+    WSDLEX32 long setSelect(long,long);
+    WSDLEX32 long getSelectedPos();
+    WSDLEX32 long setFepStatus(WSCbool);
+    WSDLEX32 WSCushort* getBuf();
+    WSDLEX32 WSCstring getString();
+    WSDLEX32 void setBuf(WSCushort*);
+    WSDLEX32 void addString(char*,long encode = WS_EN_DEFAULT);
+    WSDLEX32 void setKeyInputDisable(WSCbool);
+    WSDLEX32 WSCbool getKeyInputDisable();
 
-    void setCursorFore(WSCbool fl);
-    WSCbool getCursorFore();
-    WSCbool setCurPos(long pos); //pos >= 0
-    void replaceSelectedString(char*,long encode = WS_EN_DEFAULT);
-    WSCstring getSelectedString();
-    void deleteSelectedString();
-    long setSelect(long,long);
-    long getSelectedPos();
-    long setFepStatus(WSCbool);
-    WSCushort* getBuf();
-    WSCstring getString();
-    void setBuf(WSCushort*);
-    void addString(char*,long encode = WS_EN_DEFAULT);
     WSMFguiClassDef(WSCvifield,WSCvlabel);
     WSMFversionDef(WSCvifield);
 };
