@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCVLINEGRAPH_H
 #define WSCVLINEGRAPH_H
@@ -63,20 +59,20 @@ class WSCvlineGraph :public WSCnwbase{
     WSMFpropertyDef(WSNdataSourceName,  char*, _data_source_name  );
     WSMFpropertyDef(WSNbackBlinkColor,      short,   _bg_blink_color      );
 
-    void _set_ds_instance();
+    WSDLEX32 void _set_ds_instance();
 
 
 
   public:
-    WSCvlineGraph(WSCbase* base, char* name);
-    virtual ~WSCvlineGraph();
+    WSDLEX32 WSCvlineGraph(WSCbase* base, char* name);
+    WSDLEX32 virtual ~WSCvlineGraph();
 
-    virtual long draw();
-    virtual void setData(WSCvariant*,long code = WS_EN_DEFAULT);
-    virtual long getDataSourceType();
-    virtual char* getDataSourcePropertyName();
-    virtual void update();
-    virtual char* getDefaultProperty();
+    WSDLEX32 virtual long draw();
+    WSDLEX32 virtual void setData(WSCvariant*,long code = WS_EN_DEFAULT);
+    WSDLEX32 virtual long getDataSourceType();
+    WSDLEX32 virtual char* getDataSourcePropertyName();
+    WSDLEX32 virtual void update();
+    WSDLEX32 virtual char* getDefaultProperty();
 
     WSMFguiClassDef(WSCvlineGraph, WSCnwbase);
     WSMFversionDef  (WSCvlineGraph);
