@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCVPOLY_H
 #define WSCVPOLY_H
@@ -43,12 +39,12 @@ class WSCvpoly :public WSCvpolyAttr{
     WSMFpropertyDef(WSNlineY,   char*, _data_y  );
     WSMFpropertyDef(WSNlineNum, WSCuchar, _line_num);
 
-    long	_set_data_num(short num);
+    WSDLEX32 long	_set_data_num(short num);
 
   public:
-    WSCvpoly(WSCbase* base, char* name);
-    virtual ~WSCvpoly();
-    virtual long draw();
+    WSDLEX32 WSCvpoly(WSCbase* base, char* name);
+    WSDLEX32 virtual ~WSCvpoly();
+    WSDLEX32 virtual long draw();
 
     WSMFguiClassDef(WSCvpoly, WSCvpolyAttr);
     WSMFversionDef  (WSCvpoly);
