@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCVSLIDER_H
 #define WSCVSLIDER_H
@@ -62,26 +58,27 @@ class WSCvslider  :public WSCnwbase{
     WSMFpropertyDef(WSNvalue,             short,     _value);
     WSMFpropertyDef(WSNorientation,       WSCuchar,  _orientation);
 
-    static void _timer_work(WSCuchar,void*);
+    WSDLEX32 static void _timer_work(WSCuchar,void*);
 
-    long _bar_in(short px,short py, short x,short y,WSCushort w,WSCushort h,
+    WSDLEX32 long _bar_in(short px,short py, short x,short y,WSCushort w,WSCushort h,
                   WSCushort st);
 
 
-    virtual long _device_initialize();
+    WSDLEX32 virtual long _device_initialize();
 
   public:
-    WSCvslider(WSCbase*,char*);
+    WSDLEX32 WSCvslider(WSCbase*,char*);
 
-    virtual void onActivate();
-    virtual void onDragInterval();
-    virtual void onMousePress(WSCpoint*);
-    virtual void onMouseMove(WSCpoint*);
-    virtual void onMouseRelease(WSCpoint*);
-    virtual void onValueChange();
+    WSDLEX32 virtual void onActivate();
+    WSDLEX32 virtual void onDragInterval();
+    WSDLEX32 virtual void onMousePress(WSCpoint*);
+    WSDLEX32 virtual void onMouseMove(WSCpoint*);
+    WSDLEX32 virtual void onMouseRelease(WSCpoint*);
+    WSDLEX32 virtual void onValueChange();
 
-    virtual long draw();
-    virtual long execEventProc(long ev,void* data);
+    WSDLEX32 virtual long draw();
+    WSDLEX32 virtual long execEventProc(long ev,void* data);
+    WSDLEX32 virtual char* getDefaultProperty();
 
     WSMFguiClassDef(WSCvslider,WSCnwbase);
     WSMFversionDef(WSCvslider);
