@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCWINDOW_H
 #define WSCWINDOW_H
@@ -36,7 +32,9 @@
 class WSCwindow  :public WSCbase{
 
   protected:
+#ifndef NO_WINDOW_TITLE
     char*      _title_string;
+#endif
     char*      _user_string;
     char*      _group_name;
     WSClistData _children;
@@ -55,11 +53,15 @@ class WSCwindow  :public WSCbase{
     short      _a3_color;
     WSCushort  _mouse_no;
     WSCushort  _margin_left;
-    WSCuchar   _grad_no;
-    WSCuchar   _grad_margin;
     WSCuchar   _bar_thick;
+#ifndef NO_GRADATION
+    WSCuchar   _grad_margin;
+    unsigned   _grad_no:4;
+#endif
     unsigned   _shadow_thick:4;
+#ifndef NO_WINDOW_TITLE
     signed      _title_bar:4;
+#endif
     signed     _shadow_type:4;
     unsigned   _unique:1;
     unsigned   _pixmap_style:2;
@@ -69,9 +71,12 @@ class WSCwindow  :public WSCbase{
     unsigned   _indicator:1;
     unsigned   _emboss:1;
     unsigned   _vis_ch_sequence:1;
+    unsigned   _enable_close:1;
 
     WSMFpropertyDef(WSNname,char*,     _instance_name); 
+#ifndef NO_WINDOW_TITLE
     WSMFpropertyDef(WSNtitleString,char*,     _title_string); 
+#endif
     WSMFpropertyDef(WSNuserString, char*,     _user_string ); 
     WSMFpropertyDef(WSNuserValue,  long,      _user_value  ); 
     WSMFpropertyDef(WSNx,      short,     _x           );
@@ -84,65 +89,84 @@ class WSCwindow  :public WSCbase{
     WSMFpropertyDef(WSNtopShadowColor,  short, _ts_no );
     WSMFpropertyDef(WSNbottomShadowColor,short,_bs_no );
     WSMFpropertyDef(WSNbackPixmap,  short,     _bp_no       );
+#ifndef NO_WINDOW_TITLE
     WSMFpropertyDef(WSNtitleBar,   char,      _title_bar   );
+#endif
     WSMFpropertyDef(WSNshadowType, char,      _shadow_type );
     WSMFpropertyDef(WSNvis,        WSCbool,   _vis     );
-    WSMFpropertyDef(WSNdet,        WSCbool,   _sensitive      );
+    WSMFpropertyDef(WSNdet,        WSCbool,   _sensitive_work );
     WSMFpropertyDef(WSNpixmapStyle,WSCuchar,  _pixmap_style);
     WSMFpropertyDef(WSNgroup,   char*,     _group_name  ); 
     WSMFpropertyDef(WSNunique,  WSCbool,   _unique      ); 
     WSMFpropertyDef(WSNmouse,   WSCushort,  _mouse_no    ); 
+#ifndef NO_GRADATION
     WSMFpropertyDef(WSNgradation,   WSCuchar,  _grad_no    );
     WSMFpropertyDef(WSNgradationMargin,   WSCuchar,  _grad_margin    );
+#endif
     WSMFpropertyDef(WSNexit,   WSCbool,  _exit    );
     WSMFpropertyDef(WSNindicatorOn,   WSCbool,  _indicator    );
     WSMFpropertyDef(WSNmargin,   WSCuchar,  _margin    );
     WSMFpropertyDef(WSNmarginLeft,   WSCushort,  _margin_left    );
     WSMFpropertyDef(WSNbarThickness,   WSCuchar,  _bar_thick    );
     WSMFpropertyDef(WSNemboss,   WSCbool,  _emboss    );
+#ifndef NO_REMOTE_INSTANCE
     WSMFpropertyDef(WSNexport,   WSCbool,  _export_    );
+#endif
+    WSMFpropertyDef(WSNkeepPixmapCache,   WSCbool,  _keep_pixmap_cache    );
 
-    void _exec_child_adjust_for_anchors();
+    WSDLEX32 void _exec_child_adjust_for_anchors();
 
-    virtual long _device_initialize();
+    WSDLEX32 virtual long _device_initialize();
 
-    static WSClistData*  _workwindow_list;
-    static WSClistData*  getWorkWindowList();
-    static void (*_exit_handler)();
-    static void _delay_visible(unsigned char,void*);
+    WSDLEX32 static WSClistData*  _workwindow_list;
+    WSDLEX32 static WSClistData*  getWorkWindowList();
+    WSDLEX32 static void (*_exit_handler)();
+    WSDLEX32 static void _delay_visible(unsigned char,void*);
   public:
-    WSCwindow(WSCbase*,char*);
-    virtual ~WSCwindow();
-    virtual long initialize();
-    virtual long execEventProc(long ev,void*);
+    WSDLEX32 WSCwindow(WSCbase*,char*);
+    WSDLEX32 virtual ~WSCwindow();
+    WSDLEX32 virtual long initialize();
+    WSDLEX32 virtual long execEventProc(long ev,void*);
 
-    virtual void onVisibleChange(WSCbool);
-    virtual void onResize(WSCrect*);
-    virtual void onParentVisibleChange(WSCbool);
-    virtual void onFocusChange(WSCbool);
-    virtual void onEditModeChange(WSCbool);
-    virtual void onDefaultColorChange();
+    WSDLEX32 virtual void onVisibleChange(WSCbool);
+    WSDLEX32 virtual void onResize(WSCrect*);
+    WSDLEX32 virtual void onParentVisibleChange(WSCbool);
+    WSDLEX32 virtual void onFocusChange(WSCbool);
+#ifndef WS_EMBED
+    WSDLEX32 virtual void onEditModeChange(WSCbool);
+#endif
+    WSDLEX32 virtual void onDefaultColorChange();
+    WSDLEX32 virtual void onWindowClose();
 
-    virtual long draw();
-    virtual long clear();
-    virtual long redraw();
-    virtual long cdraw();
-    virtual void update();
+    WSDLEX32 virtual long draw();
+    WSDLEX32 virtual long clear();
+    WSDLEX32 virtual long redraw();
+    WSDLEX32 virtual long cdraw();
+    WSDLEX32 virtual void update();
 
-    long setMapStatus(long);
+    WSDLEX32 long setMapStatus(long);
 
-    virtual long getObjectType();
-    virtual WSClistData& getChildren();
-    virtual void setChildren(WSClistData&);
-    virtual void adjustForm();
-    virtual void setVisible(WSCbool);
-    void execResizeEvent();
+    WSDLEX32 virtual long getObjectType();
+    WSDLEX32 virtual WSClistData& getChildren();
+    WSDLEX32 virtual WSClistData* getChildrenPtr();
+    WSDLEX32 virtual void setChildren(WSClistData&);
+    WSDLEX32 virtual void adjustForm();
+    WSDLEX32 virtual void setVisible(WSCbool);
+    WSDLEX32 virtual void internalUpdate();
+#ifndef NO_RESTACK_CHILD
+    WSDLEX32 virtual long restackChild(WSCbase*,int order);
+#endif
 
-static void setGlobalExitHandler(void(*)());
+    WSDLEX32 void execResizeEvent();
+
+WSDLEX32 static void setGlobalExitHandler(void(*)());
+
+    WSDLEX32 long setWindowStatus(long status);
+    WSDLEX32 long getWindowStatus();
+    WSDLEX32 long getPixmapStyle();
+    WSDLEX32 WSCbool getEnableClose();
+    WSDLEX32 void setEnableClose(WSCbool);
 
-    long setWindowStatus(long status);
-    long getWindowStatus();
-    long getPixmapStyle();
     WSMFguiClassDef(WSCwindow,WSCbase);
     WSMFsetUseDevice(windowDev);
     WSMFversionDef(WSCwindow);
