@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSDDRAGDROP_H
 #define WSDDRAGDROP_H
@@ -33,11 +29,11 @@ class WSCbase;
 class WSDdragDrop   :public WSDres{
 
   public:
-    static  void setCreateHandler(WSDdragDrop* (*)());
-    virtual long registerDragSite(WSCbase*,char* msg);
-    virtual long unregisterDragSite(WSCbase*);
-    virtual long registerDropSite(WSCbase*,char* msg,void(*)(WSCbase*,WSCbase*));
-    virtual long unregisterDropSite(WSCbase*);
+    WSDLEX32 static  void setCreateHandler(WSDdragDrop* (*)());
+    WSDLEX32 virtual long registerDragSite(WSCbase*,char* msg);
+    WSDLEX32 virtual long unregisterDragSite(WSCbase*);
+    WSDLEX32 virtual long registerDropSite(WSCbase*,char* msg,void(*)(WSCbase*,WSCbase*));
+    WSDLEX32 virtual long unregisterDropSite(WSCbase*);
 
     WSMFclassDef(WSDdragDrop,WSDres);
 };
