@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #ifndef WSNDEF_H
 #define WSNDEF_H
@@ -77,8 +73,8 @@ extern char* _WSNblinkRefreshing;
 extern char* _WSNpixmapStyle;
 #define       WSNlabelPixmap              _WSNlabelPixmap
 extern char* _WSNlabelPixmap;
-#define       WSNkanjiIn                  _WSNkanjiIn
-extern char* _WSNkanjiIn;
+#define       WSNenableFep                  _WSNenableFep
+extern char* _WSNenableFep;
 #define       WSNinterCur                 _WSNinterCur
 extern char* _WSNinterCur;
 #define       WSNinterval                 _WSNinterval
@@ -369,6 +365,8 @@ extern char* _WSNvcolumns;
 extern char* _WSNhcolumns;
 #define       WSNseparator                _WSNseparator
 extern char* _WSNseparator;
+#define       WSNseparator2                _WSNseparator2
+extern char* _WSNseparator2;
 #define       WSNfit                _WSNfit
 extern char* _WSNfit;
 #define       WSNip                _WSNip
@@ -403,6 +401,12 @@ extern char* _WSNsort;
 extern char* _WSNitem;
 #define       WSNfilter                  _WSNfilter
 extern char* _WSNfilter;
+#define       WSNpixmapOverlap                  _WSNpixmapOverlap
+extern char* _WSNpixmapOverlap;
+#define       WSNkeepPixmapCache                  _WSNkeepPixmapCache
+extern char* _WSNkeepPixmapCache;
+#define       WSNautoLineFeeding                  _WSNautoLineFeeding
+extern char* _WSNautoLineFeeding;
 
 #else //MSW
 
@@ -454,8 +458,8 @@ extern WSDLEX32 char* _WSNblinkRefreshing();
 extern WSDLEX32 char* _WSNpixmapStyle();
 #define       WSNlabelPixmap              _WSNlabelPixmap()
 extern WSDLEX32 char* _WSNlabelPixmap();
-#define       WSNkanjiIn                  _WSNkanjiIn()
-extern WSDLEX32 char* _WSNkanjiIn();
+#define       WSNenableFep                  _WSNenableFep()
+extern WSDLEX32 char* _WSNenableFep();
 #define       WSNinterCur                 _WSNinterCur()
 extern WSDLEX32 char* _WSNinterCur();
 #define       WSNinterval                 _WSNinterval()
@@ -745,6 +749,8 @@ extern WSDLEX32 char* _WSNvcolumns();
 #define       WSNhcolumns    _WSNhcolumns()
 extern WSDLEX32 char* _WSNhcolumns();
 #define       WSNseparator    _WSNseparator()
+extern WSDLEX32 char* _WSNseparator2();
+#define       WSNseparator2    _WSNseparator2()
 extern WSDLEX32 char* _WSNseparator();
 #define       WSNfit    _WSNfit()
 extern WSDLEX32 char* _WSNfit();
@@ -780,6 +786,12 @@ extern WSDLEX32 char* _WSNsort();
 extern WSDLEX32 char* _WSNitem();
 #define       WSNfilter         _WSNfilter()
 extern WSDLEX32 char* _WSNfilter();
+#define       WSNpixmapOverlap         _WSNpixmapOverlap()
+extern WSDLEX32 char* _WSNpixmapOverlap();
+#define       WSNkeepPixmapCache         _WSNkeepPixmapCache()
+extern WSDLEX32 char* _WSNkeepPixmapCache();
+#define       WSNautoLineFeeding         _WSNautoLineFeeding()
+extern WSDLEX32 char* _WSNautoLineFeeding();
 
 #endif //MSW
 
