@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,18 +14,15 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #ifndef WSCOM_H
 #define WSCOM_H
 
+
 #include <wsconfig.h>
 ///////////////////////////////////////
 // definition of directories and files
@@ -69,11 +66,13 @@
 
 #include <WSNdef.h>
 
+//#define THR_DBG
 //#define PDBG
 //#define MEMDBG
 #ifdef MEMDBG
 #include <stddef.h>
 extern void* operator new(size_t);
+extern void* operator new[](size_t);
 extern void operator delete(void*);
 extern void operator delete[](void*);
 #endif
@@ -89,6 +88,7 @@ extern void operator delete[](void*);
 /////////////////////////////////////
 
 #define WS_BASE_CLASS       "BASE-CLASS"
+#define WS_BASE_CPTR        "BASE-CPTR"
 #define WS_TRG_LIST         "TRG-LIST"
 #define WS_WIN_STORE        "WIN-STORE"
 #define WS_WIN_CLASS        "WIN-CLASS"
@@ -98,7 +98,7 @@ extern void operator delete[](void*);
 #define WS_EXT_VAR          "EXT-VAR"
 #define WS_PROPERTY_CHANGED "PROP-CHANGED"
 #define WS_GEOMETRY_UPDATE  "GEOMETRY-UPDATE"
-#define WS_DET              "WS-DET"
+//#define WS_DET              "WS-DET"
 #define WS_VIS              "WS-VIS"
 #define WS_PROP_LIST        "WS-PROP-LIST"
 #define WS_UTRG_LIST        "WS-UTRG-LIST"
@@ -109,6 +109,9 @@ extern void operator delete[](void*);
 #define WS_RCLASS_GENERATE  "WS-RCLASS-GENERATE"
 #define WS_RCLASS_NAME      "WS-RCLASS-NAME"
 #define WS_RCLASS_BNAME     "WS-RCLASS-BNAME"
+#define WS_NO_DEL_BUILDER_DATA     "WS-NO-DEL-BUILDER-DATA"
+#define WS_GRIPH_HANDLER "G-HANDLER"
+#define WS_GRIPH_SIZE 5
 
 // arc types
 #define WS_ARC_NONE     -1
@@ -125,6 +128,7 @@ extern void operator delete[](void*);
 #define WS_TYPE_MANAGER  0x0002
 #define WS_TYPE_FORM     0x0004
 #define WS_TYPE_SCR_FORM 0x0008
+#define WS_TYPE_MENU_WINDOW 0x0010
 
 #define WS_OOF_SUFFIX   ".oof"
 #define WS_OOF_SUFFIX2  "oof"
@@ -194,6 +198,7 @@ extern void operator delete[](void*);
 #define WS_EN_EUCKR       30
 #define WS_EN_EUCCN       40
 #define WS_EN_BIG5        50
+#define WS_EN_CP1251      60
 
 //#define WS_EN_LANG_NONE    0
 //#define WS_EN_LANG_JP      1
@@ -223,6 +228,7 @@ extern void operator delete[](void*);
 #define WSN_EN_EUCKR      "EUCKR"
 #define WSN_EN_EUCCN      "EUCCN"
 #define WSN_EN_BIG5       "BIG5"
+#define WSN_EN_CP1251     "CP1251"
 //#define WSN_EN_UTF8_JP    "UTF8_JP"
 
 typedef unsigned char  WSCbool;
@@ -234,6 +240,7 @@ typedef unsigned int   WSCuint;
 class WSCbase;
 class WSDmwindowDev;
 class WSDimage;
+class WSCprocedure;
 typedef void (*WSCeventProc)(WSCbase*);
 
 //kind of mouse button state ,return value of WSDmouse::getMouseStatus()
@@ -248,9 +255,12 @@ typedef void (*WSCeventProc)(WSCbase*);
 #define WS_MOUSE_BTN5      (1<<12)
 
 //kind of read write mode
-#define WS_READ      0
-#define WS_WRITE     1
-#define WS_RW        2
+#define WS_READ       0
+#define WS_READ_ONLY  0
+#define WS_WRITE      1
+#define WS_WRITE_ONLY 1
+#define WS_RW         2
+#define WS_RDWR       2
 
 //kind of indicator type
 #define WS_IN_OUT     (unsigned char)0
@@ -350,7 +360,7 @@ extern int gettimeofday(struct timeval *__tv,
 
 //events
 #define  WSEV_HASH_MAX_NUM        32
-#define  WSEV_NONE                -1
+#define  WSEV_NONE                (char)-1
 #define  WSEV_INITIALIZE           (long)0
 #define  WSEV_DELETE               (long)1
 #define  WSEV_ACTIVATE             (long)2
@@ -393,6 +403,14 @@ extern int gettimeofday(struct timeval *__tv,
 #define  WSEV_SCR_MOUSE_DOUBLE_CLICK  (long)51
 #define  WSEV_ITEM_DOUBLE_CLICKED    (long)52
 #define  WSEV_THREAD_STARTED      (long)53
+#define  WSEV_OPENGL_INITIALIZE      (long)54
+#define  WSEV_OPENGL_DELETE       (long)55
+#define  WSEV_WINDOW_CLOSED       (long)56
+#define  WSEV_SCR_FOCUS_CH       (long)57
+#define  WSEV_ITEM_PRE_SELECTED    (long)58
+#define  WSEV_GRAB_CANCELED        (long)59
+#define  WSEV_MENU_POPUP           (long)60
+
 
 #define  WSEV_NONE_BIT                0x00000000
 #define  WSEV_INITIALIZE_BIT          0x00000001
@@ -415,6 +433,17 @@ extern int gettimeofday(struct timeval *__tv,
 #define  WSEV_KEY_RELEASE_BIT         0x00020000
 #define  WSEV_KEY_HOOK_BIT            0x00040000
 
+#define  WS_FEP_START                 1
+#define  WS_FEP_STOP                  2
+#define  WS_FEP_NATIVE                1
+#define  WS_FEP_KATAKANA              3
+#define  WS_FEP_HKATAKANA             4
+
+#define WS_FEP_DISABLE                0
+#define WS_FEP_ENABLE                 1
+#define WS_FEP_KANJI_ON_FOCUS         2
+#define WS_FEP_KATAKANA_ON_FOCUS      3
+#define WS_FEP_HKATAKANA_ON_FOCUS     4
 
 //#ifndef MSW //if unix system
 #include <assert.h>
@@ -453,7 +482,11 @@ extern WSDLEX32 int _dbprintf_init();
 extern WSDLEX32 int _dbprintf(const char*,...);
 };
 #ifdef _WSWIN32
-#define dbprintf
+#  ifdef __GNUC__
+#    define dbprintf(x...)   
+#  else 
+#    define dbprintf
+#  endif
 #else
 #define dbprintf(...) 
 #endif
@@ -498,6 +531,7 @@ struct WSCrectList {
 class WSCstring;
 extern WSDLEX void  WSGFtrace(WSCstring);
 extern WSDLEX char* WSGFstrdup(const char*);
+extern WSDLEX char* WSGFstrdup(const char*,long len);
 extern WSDLEX long  WSGFstricmp(char*,char*);
 extern WSDLEX long  WSGFstrnccmp(char*,char*,long);
 extern WSDLEX void  WSGFtrimSpace(char*);
@@ -507,6 +541,7 @@ extern WSDLEX short WSGFindexColor(char*);
 extern WSDLEX short WSGFcolor(char*);
 extern WSDLEX char* WSGFcolorName(short);
 extern WSDLEX short WSGFimage(char*);
+extern WSDLEX short WSGFimage(char*,short);
 extern WSDLEX char* WSGFimageName(short);
 
 extern WSDLEX WSCuchar  WSGFblinkRate(long);
@@ -570,15 +605,38 @@ extern WSDLEX WSCushort* WSGFstrdupUCS2(WSCushort*);
 extern WSDLEX WSCushort* WSGFgetWordUCS2(WSCushort*,long pos,WSCushort*);
 extern WSDLEX long WSGFgetWordCharPosUCS2(WSCushort*,long pos,WSCushort*);
 extern WSDLEX WSCushort* WSGFgetUCS2(const char* text,long encode);
+extern WSDLEX long WSGFgetUcs2len(const char* text,long encode,long len);
 extern WSDLEX char* WSGFgetString(WSCushort* text,long encode);
 extern WSDLEX WSCushort* WSGFaddStringUCS2(WSCushort* str1,WSCushort* str2);
+extern WSDLEX WSCstring WSGFcreateLineFeedStringByWidth(WSCstring*,short fontno,WSCushort);
 extern WSDLEX WSDmwindowDev* WSGFgetAquaImage(short col,WSCushort,WSCushort,long);
 extern WSDLEX WSDmwindowDev* WSGFgetAquaMaskImage(WSCushort,WSCushort,long);
 
-
 //do not change this size
-#define WS_MAX_HASH_VALUE       64
+//optimized 64 -> 32
+//#define WS_MAX_HASH_VALUE       64
+#ifdef WS_EMBED
+#define WS_MAX_HASH_VALUE       16
+#else
+#define WS_MAX_HASH_VALUE       32
+#endif
+#ifdef NO_INLINE_HASH_VALUE
 extern WSDLEX long WSGFgetHashValue(char*);
+#else
+inline long WSGFgetHashValue(char* str){
+  WSCulong val = str[0];
+  register WSCulong len = 0;
+  while(str[len] != 0){
+    len++;
+  }
+  if (len != 0){
+//    val += str[len/2]*2;
+    val += str[len-1]*4;
+  }
+  return (val % WS_MAX_HASH_VALUE);
+//  return (str[0] % WS_MAX_HASH_VALUE);
+}
+#endif
 
 #ifndef WS_DEBUG
 #  ifdef __GNUC__
@@ -596,6 +654,11 @@ extern WSDLEX long WSGFgetHashValue(char*);
 
 extern WSDLEX WSCbase* WSGIappPropertyEditor();
 extern WSDLEX long      WSGFsetAppPropertyEditor(WSCbase*);
+#define WS_PE_PROPERTY_CHANGED 0
+#define WS_PE_INSTANCE_ACTIVATED 1
+#define WS_PE_INSTANCE_NOT_ACTIVATED 2
+extern WSDLEX void      WSGFsetPropertyEditorHandler(void(*)(WSCbase*,long));
+extern WSDLEX void*     WSGFgetPropertyEditorHandler();
 extern WSDLEX WSCbase* WSGIappWindowList();
 extern WSDLEX long      WSGFsetAppWindowList(WSCbase*);
 
@@ -610,6 +673,7 @@ extern WSDLEX char* WSGFultoa(WSCulong val);
 extern WSDLEX char* WSGFultoa(WSCulong val,char* buf);
 extern WSDLEX char* WSGFftoa(float);
 extern WSDLEX char* WSGFlftoa(double);
+extern WSDLEX long WSGFatol(char*);
 
 extern WSDLEX void WSGFgettimeofday(long* );
 
@@ -620,7 +684,13 @@ extern WSDLEX WSCstring* WSGFcreateWindowSrc(WSCbase*);
 extern WSDLEX WSCstring* WSGFcreateWindowSrcRb(WSCbase*);
 extern WSDLEX WSCstring* WSGFcreateWindowSrcPy(WSCbase*);
 extern WSDLEX WSCstring* WSGFcreateWindowSrcPerl(WSCbase*);
-extern WSDLEX WSCstring* WSGFcreateWindowSrcPerl_new(WSCbase*);
+extern WSDLEX WSCstring* WSGFcreateWindowSrcJava(WSCbase*,char* pkgname);
+extern WSDLEX WSCstring* WSGFcreateWindowSrcOcaml(WSCbase*);
+extern WSDLEX WSCstring* WSGFcreateWindowSrcMwtRb(WSCbase*);
+extern WSDLEX WSCstring* WSGFcreateWindowSrcMwtPy(WSCbase*);
+extern WSDLEX WSCstring* WSGFcreateWindowSrcMwtPerl(WSCbase*);
+extern WSDLEX WSCstring* WSGFcreateWindowSrcMwtJava(WSCbase*,char* pkgname);
+extern WSDLEX WSCstring* WSGFcreateWindowSrcMwtOcaml(WSCbase*);
 
 //extern WSDLEX short  WSGFgetBarPixmap(WSCushort,WSCuchar);
 extern WSDLEX void   WSGFmarkChanged(WSCbase*);
@@ -637,6 +707,14 @@ extern WSDLEX void     WSGFopCreate(WSCstring*,char*,char*);
 extern WSDLEX void     WSGFopCreateRb(WSCstring*,char*,char*);
 extern WSDLEX void     WSGFopCreatePy(WSCstring*,char*,char*);
 extern WSDLEX void     WSGFopCreatePerl(WSCstring*,char*,char*);
+extern WSDLEX void     WSGFopCreateJava(WSCstring*,char*,char*);
+extern WSDLEX void     WSGFopCreateOcaml(WSCstring*,char*,char*);
+extern WSDLEX void     WSGFopCreateMwtRb(WSCstring*,char*,char*);
+extern WSDLEX void     WSGFopCreateMwtPy(WSCstring*,char*,char*);
+extern WSDLEX void     WSGFopCreateMwtPerl(WSCstring*,char*,char*);
+extern WSDLEX void     WSGFopCreateMwtJava(WSCstring*,char*,char*);
+extern WSDLEX void     WSGFopCreateMwtOcaml(WSCstring*,char*,char*);
+extern WSDLEX void     WSGFsetJavaPackageNameHandler(void*);
 extern WSDLEX void     WSGFclassProcedureCreate(WSCstring*,char*,char*,char*);
 extern WSDLEX WSCbool  WSGFcheckNameString(char*);
 extern WSDLEX WSCbool  WSGFcheckPureNameString(char*);
@@ -646,7 +724,7 @@ extern WSDLEX long     WSGFcheckDiffFile(char*,void*,long len);
 extern WSDLEX long     WSGFcheckDiffTextFile(char*,char*);
 extern WSDLEX long     WSGFreplaceFile(char*,void*,long len,WSCbool same_save);
 extern WSDLEX long     WSGFreplaceTextFile(char*,char*,WSCbool same_save);
-extern WSDLEX void     WSGFsortCharList(WSClistData* src,WSClistData* dest);
+extern WSDLEX void     WSGFsortCharList(WSClistData* list);
 extern WSDLEX void     WSGFsetMargin(WSCbase*,WSCushort);
 extern WSDLEX void     WSGFdestroyPixmap(char*);
 extern WSDLEX void     WSGFdestroyPixmap(short);
@@ -659,27 +737,35 @@ extern WSDLEX WSCstring*    WSGFcreateWindowClassSrcP(WSCbase*);    //must free
 extern WSDLEX WSCstring*    WSGFupdateWindowClassHeader(WSCstring*,WSCbase*); //must return value
 extern WSDLEX WSCstring*    WSGFreadTextFile(char*,long encoding=WS_EN_DEFAULT); //must free return value
 extern WSDLEX WSCbool       WSGFcheckArrayName(char*,long*,char*);
+extern WSDLEX WSCbool       WSGFisArrayName(char*);
+extern WSDLEX long          WSGFgetArrayNo(char*);
+extern WSDLEX WSCstring     WSGFgetArrayName(char*);
 extern WSDLEX long          WSGFexistName(char*,WSCbase*);
 extern WSDLEX void          WSGFsetExistNameHandler(long (*)(char*,WSCbase*));
 extern WSDLEX void*         WSGFgetExistNameHandler();
 extern WSDLEX long          WSGFsendMessage(char*  message_key,char* data);
 extern WSDLEX char*         WSGFeventToName(long ev);
 extern WSDLEX long          WSGFnameToEvent(char* ev);
-extern WSDLEX WSCstring*    WSGFgetMouseCursorSaveData();
+//extern WSDLEX WSCstring*    WSGFgetMouseCursorSaveData();
 extern WSDLEX WSCstring*    WSGFgetFontSaveData();
-extern WSDLEX long          WSGFloadMouseCursorFile(char*);
+//extern WSDLEX long          WSGFloadMouseCursorFile(char*);
 extern WSDLEX long          WSGFloadMouseCursorData(char**);
 extern WSDLEX void          WSGFsetFontData(WSCstring* data);
-extern WSDLEX void          WSGFloadFontFile(char*);
+//extern WSDLEX void          WSGFloadFontFile(char*);
 extern WSDLEX void          WSGFloadFontData(char**);
 extern WSDLEX WSCbase*      WSGFgetFontEditor();
 extern WSDLEX void          WSGFsetFontEditor(WSCbase*);
-extern WSDLEX WSCbase*      WSGFgetClassBase(WSCbase*);
-extern WSDLEX void          WSGFresetClassBase(WSCbase*);
 extern WSDLEX void          WSGFsetDeleteHandler(void (*)(WSCbase*));
 extern WSDLEX void(* WSGFgetDeleteHandler() )(WSCbase*) ;
 extern WSDLEX void          WSGFsetCreateHandler(void (*)(WSCbase*));
 extern WSDLEX void          WSGFexecCreateHandler(WSCbase*);
+#ifndef WS_EMBED
+#ifndef NO_INDEX_VARIANT_DATA
+class WSCindexVariantData;
+extern WSDLEX void          WSGFsetCreateInstanceHandler(WSCbase*(*)(WSCbase*,WSCindexVariantData*));
+extern WSDLEX WSCbase*      WSGFexecCreateInstanceHandler(WSCbase*,WSCindexVariantData*);
+#endif
+#endif
 extern WSDLEX char*         WSGFtriggerToName(long ev);
 extern WSDLEX WSCstring     WSGFgetFileName(char*);
 extern WSDLEX WSCstring     WSGFgetDirName(char*);
@@ -701,6 +787,8 @@ extern WSDLEX void WSGFdrawStripes(WSDdev*,WSCushort,WSCushort,short,short,short
 
 extern WSDLEX long WSGFgetLanguage();
 extern WSDLEX void WSGFsetGetLanguageHandler(long(*)());
+extern WSDLEX long WSGFgetMapping();
+extern WSDLEX void WSGFsetGetMappingHandler(long(*)());
 
 extern WSDLEX void WSGFwsNsPluginInitialize();
 extern WSDLEX void WSGFwsIePluginInitialize();
@@ -715,6 +803,17 @@ extern WSDLEX void WSGFdrawModern1Shadow(WSDdev* dev,long shadow_type,WSCushort
    short x, short y, WSCushort w,WSCushort h,WSCbool nodark);
 
 extern WSDLEX void WSGFexit(int);
+extern WSDLEX void WSGFbeep();
+extern WSDLEX WSCbool WSGFgetWindowEnableClose(WSCbase*);
+extern WSDLEX void WSGFenablePropertyBuilderTitle();
+
+#ifndef WS_DIALOG_NONE
+#define WS_DIALOG_NONE  -2
+#endif
+
+#ifndef WS_DIALOG_DEFAULT
+#define WS_DIALOG_DEFAULT  -1
+#endif
 
 #ifndef WS_DIALOG_CANCEL
 #define WS_DIALOG_CANCEL  0
@@ -734,6 +833,7 @@ extern WSDLEX void WSGFexit(int);
 
 #define WS_TEXT    0
 #define WS_BINARY  1
+#define WS_VFS     2
 
 #define WS_NORMAL  0
 #define WS_TREE    1
@@ -748,6 +848,15 @@ extern WSDLEX void WSGFexit(int);
 #define WS_HAVE_CHILD   1
 #define WS_HAVE_BROS    2
 
+#define WS_LIST_RDWR     0
+#define WS_LIST_READONLY 1
+#define WS_LIST_NUMONLY  2
+
+#define WS_LIST_ASCENDING_ORDER  1
+#define WS_LIST_DESCENDING_ORDER 2
+#define WS_LIST_REVERSING_ORDER  3
+
+
 #define WS_OPEN          0
 #define WS_INDENT_LEVEL  1
 #define WS_ABSOLUTE_OPEN 2
@@ -766,8 +875,7 @@ extern WSDLEX void WSGFexit(int);
 #define WS_POLICY_ORIGINAL   1
 #define WS_POLICY_WINDOWS    2
 #define WS_POLICY_GRADATION  4
-#define WS_POLICY_AQUA       8
-#define WS_POLICY_OTHER     16
+#define WS_POLICY_OTHER      8
 
 #define WS_DATA_SOURCE_NONE      0
 #define WS_DATA_SOURCE_INSTANCE  1
@@ -926,13 +1034,26 @@ extern WSDLEX long WSGFgetEncoding(char* locale_name);
 
 #define WS_DB_ODBC     0
 #define WS_DB_POSTGRES 1
+#define WS_DB_MYSQL    2
+#define WS_DB_SQLITE   3
 
 //LANG
 #define WS_LANG_CPP    0
 #define WS_LANG_RUBY   1
 #define WS_LANG_PYTHON 2
 #define WS_LANG_PERL   3
+#define WS_LANG_JAVA   4
+#define WS_LANG_OCAML  5
 
+#define WSN_LANG_CPP    "C++"
+#define WSN_LANG_RUBY   "Ruby"
+#define WSN_LANG_PYTHON "Python"
+#define WSN_LANG_PERL   "Perl"
+#define WSN_LANG_JAVA   "Java"
+#define WSN_LANG_OCAML  "Ocaml"
+
+#define WS_MAPPING_MWT   0
+#define WS_MAPPING_MPFC  1
 
 //MOUSE
 #define WS_DEFAULT_MOUSE_ID "0   132        0          black      white"
@@ -965,4 +1086,173 @@ extern WSDLEX long WSGFgetEncoding(char* locale_name);
 #define WS_IMAGE_PNG      (3)
 #define WS_IMAGE_JPG      (4)
 
+
+#define WS_FONT_NORMAL     0x0000
+#define WS_FONT_BOLD       0x0001
+#define WS_FONT_ITALIC     0x0002
+
+#define WS_FONT_COUNT      23
+#define WS_FONT_NONE        0
+#define WS_FONT_ISO8859(n) (n) /* 1 - 15 */
+#define WS_FONT_UNICODE    16
+#define WS_FONT_KOI8R      17
+#define WS_FONT_JIS0201    18
+#define WS_FONT_JIS0208    19
+#define WS_FONT_KSC5601    20
+#define WS_FONT_GB2312     21
+#define WS_FONT_BIG5       22
+#define WS_FONT_CP1251     23
+
+//MWT Functions
+#define WS_ROOT_INSTANCE_ID   0
+#define WS_ROOT_PROPERTY_ID   0
+#define WS_ROOT_PROCEDURE_ID  0
+
+#define WS_NEXT_CHILD_INSTANCE_ID       1
+#define WS_PARENT_INSTANCE_ID           2
+
+#define WS_SET_ROOT_INSTANCE            3
+#define WS_REMOVE_ROOT_INSTANCE         4
+#define WS_SET_SELECTED_INSTANCE        5
+#define WS_REMOVE_SELECTED_INSTANCE     6
+#define WS_NEXT_SELECTED_INSTANCE_ID    7
+
+#define WS_INSTANCE_NAME                10
+#define WS_PROPERTY_NAME                11
+#define WS_PROPERTY_VALUE               12
+#define WS_PROPERTY_TYPE                13
+#define WS_PROPERTY_ATTRIBUTE           14
+#define WS_PROCEDURE_FUNCTION_NAME      15
+#define WS_PROPERTY_SELECT_VALUE_NAME   16
+#define WS_PROPERTY_SELECT_VALUE        17
+#define WS_PROPERTY_TITLE               18
+
+#define WS_PROPERTY_TYPE_COLOR          1
+#define WS_PROPERTY_TYPE_PIXMAP         2
+#define WS_PROPERTY_TYPE_FILE_SELECT    4
+
+
+#define WS_NEXT_PROPERTY_ID             20
+#define WS_NEXT_PROCEDURE_ID            21
+#define WS_NEXT_PROPERTY_SELECT_ITEM_ID 22
+
+
+#define WS_NEXT_TARGET_ID               30
+#define WS_TARGET_NATIVE                 0
+#define WS_TARGET_NAME                  31
+#define WS_TARGET_SUFFIX                32
+#define WS_ITEM_ROOT                     0
+
+#define WS_NEXT_LANG_ID                 33
+#define WS_LANG_STR                     34
+
+#define WS_NEXT_ENCODING_ID             35
+#define WS_ENCODING_NAME                36
+
+#define WS_DEFAULT_PROJECT_FILE_NAME    40
+#define WS_DEFAULT_PROJECT_FILE_NAME_S "$(WSDIR)/sys/data/locale/%s/default.prj"
+
+#define WS_LINK_OPT_DEF_FILE_NAME       41
+#define WS_LINK_OPT_DEF_FILE_NAME_S    "$(WSDIR)/sys/data/linkoptions"
+
+#define WS_LINK_OPTIONS                 42
+#define WS_STLINK_OPTIONS               43
+#define WS_NEXT_LINK_OPT_ID             44
+#define WS_LINK_OPT_NAME                45
+#define WS_TARGET_INSTANCE_ID           46
+#define WS_LINK_LIB_DEF_FILE_NAME       47
+#define WS_LINK_LIB_DEF_FILE_NAME_S     "$(WSDIR)/sys/config/libname"
+#define WS_LINK_LIB_NAME                48
+#define WS_LINK_CLASS_DEF_FILE_NAME     49
+#define WS_LINK_CLASS_DEF_FILE_NAME_S   "$(WSDIR)/sys/data/locale/%s/lib%d.data"
+
+#define WS_TARGET_DEF_FILE_NAME         50
+#define WS_TARGET_DEF_FILE_NAME_S       "$(WSDIR)/sys/data/targets"
+
+#define WS_ENV_DEF_FILE_NAME            51
+#ifdef MSW
+#define WS_ENV_DEF_FILE_NAME_S          "$(WINDIR)/.wsbuilderrc"
+#else
+#define WS_ENV_DEF_FILE_NAME_S          "$(HOME)/.wsbuilderrc"
+#endif
+
+#define WS_LIBNAME_DEF_FILE_NAME        52
+#define WS_LIBNAME_DEF_FILE_NAME_S      "$(WSDIR)/sys/config/libname"
+
+#define WS_TARGET_ENV_DEF_FILE_NAME     53
+#define WS_TARGET_ENV_DEF_FILE_NAME_S   "$(WSDIR)/sys/data/targets-env"
+#define WS_TARGET_ENV_TITLE             54
+#define WS_TARGET_ENV_NAME              55
+#define WS_UNDO_NUM                     56
+
+
+#define WS_OPERATION_SRC_COPY      0
+#define WS_OPERATION_XOR           1
+
+
+extern WSDLEX long WSGFloadProject(char* dname,char* fname,long encoding);
+extern WSDLEX long WSGFloadProject(char* path,long encoding);
+extern WSDLEX long WSGFreloadProject(WSCulong project,char* dname,char* fname,long encoding);
+extern WSDLEX long WSGFreloadProject(WSCulong project,char* path,long encoding);
+extern WSDLEX long WSGFdestroyProject(WSCulong project);
+extern WSDLEX long WSGFdumpProject(WSCulong project);
+extern WSDLEX long WSGFsaveProject(WSCulong project,char* dname,char* fname,long encoding);
+extern WSDLEX long WSGFsetProjectData(WSCulong prj,char* index,char* val,long encoding);
+extern WSDLEX WSCstring WSGFgetProjectData(WSCulong prj,char* index);
+extern WSDLEX WSCstring WSGFgetProjectEnvironmentData(char* index);
+extern WSDLEX long WSGFloadWinFiles(char* dname,char* fname,WSCulong prj);
+extern WSDLEX long WSGFloadProjectAndWinFiles(char* dname,char* fname,long encoding);
+extern WSDLEX long WSGFsaveWinFile(char* dname,WSCbase* win,long encoding);
+extern WSDLEX long WSGFsaveProjectEnvironment();
+extern WSDLEX long WSGFloadLinkOptions(WSCulong prj);
+
+extern WSDLEX long WSGFloadTargets(char* path,long encoding);
+extern WSDLEX long WSGFgetMgrData(long kind,WSCulong val);
+extern WSDLEX WSCstring WSGFgetMgrStrStrData(long kind,char* val);
+extern WSDLEX WSCstring WSGFgetMgrStrData(long kind,WSCulong val);
+extern WSDLEX long WSGFsetMgrStrData(long kind,char* val,long encoding);
+extern WSDLEX WSCprocedure* WSGFidToProcedure(WSCulong id);
+extern WSDLEX WSCulong WSGFprocedureToId(WSCprocedure*);
+
+extern "C" {
+extern WSDLEX void libwspostgres_func();
+extern WSDLEX void libwsmysql_func();
+extern WSDLEX void libwsodbc_func();
+};
+
+#ifndef USE_INLINE_STRCMP
+#define WSIFstrcmp strcmp
+#else
+inline long WSIFstrcmp(char* x,char* y){
+//  register WSCuchar* s1 = (WSCuchar*)x;
+//  register WSCuchar* s2 = (WSCuchar*)y;
+  register long cnt = 0;
+  register WSCuchar p1 = 0;
+  register WSCuchar p2 = 0;
+  do{
+      p1 = (WSCuchar)x[cnt];
+      p2 = (WSCuchar)y[cnt];
+//      p1 = (WSCuchar)*s1++;
+//      p2 = (WSCuchar)*s2++;
+      if (p1 == 0){
+         return p1 -p2;
+      }
+      cnt++;
+  } while (p1 == p2);
+  return p1 -p2;
+}
+#endif
+
+#define WS_APP_TYPE_GENERIC       0
+#define WS_APP_TYPE_LINUX_GTK     1
+#define WS_APP_TYPE_LINUX_MOTIF   2
+#define WS_APP_TYPE_MACOSX        3
+#define WS_APP_TYPE_WIN32         4
+#define WS_APP_TYPE_SOLARIS_GTK   5
+#define WS_APP_TYPE_SOLARIS_MOTIF 6
+
+#define LINK_LIB_POSTGRES  libwspostgres_func();
+#define LINK_LIB_MYSQL     libwsmysql_func();
+#define LINK_LIB_ODBC      libwsodbc_func();
+#define LINK_LIB_SQLITE    libwssqlite_func();
 #endif //WSCOMH
