@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #ifndef WSKEYSYM_H
 #define WSKEYSYM_H
@@ -482,6 +478,7 @@
 #define WSK_Super_R  XK_Super_R
 #define WSK_Hyper_L  XK_Hyper_L
 #define WSK_Hyper_R  XK_Hyper_R
+#define WSK_Apps     -1
 
 #define WSK_space               XK_space
 #define WSK_exclam              XK_exclam
@@ -581,6 +578,104 @@
 #define WSK_braceright          XK_braceright
 #define WSK_asciitilde          XK_asciitilde
 
+#define WSK_nobreakspace        0x0a0
+#define WSK_exclamdown          0x0a1
+#define WSK_cent                0x0a2
+#define WSK_sterling            0x0a3
+#define WSK_currency            0x0a4
+#define WSK_yen                 0x0a5
+#define WSK_brokenbar           0x0a6
+#define WSK_section             0x0a7
+#define WSK_diaeresis           0x0a8
+#define WSK_copyright           0x0a9
+#define WSK_ordfeminine         0x0aa
+#define WSK_guillemotleft       0x0ab
+#define WSK_notsign             0x0ac
+#define WSK_hyphen              0x0ad
+#define WSK_registered          0x0ae
+#define WSK_macron              0x0af
+#define WSK_degree              0x0b0
+#define WSK_plusminus           0x0b1
+#define WSK_twosuperior         0x0b2
+#define WSK_threesuperior       0x0b3
+#define WSK_acute               0x0b4
+#define WSK_mu                  0x0b5
+#define WSK_paragraph           0x0b6
+#define WSK_periodcentered      0x0b7
+#define WSK_cedilla             0x0b8
+#define WSK_onesuperior         0x0b9
+#define WSK_masculine           0x0ba
+#define WSK_guillemotright      0x0bb
+#define WSK_onequarter          0x0bc
+#define WSK_onehalf             0x0bd
+#define WSK_threequarters       0x0be
+#define WSK_questiondown        0x0bf
+#define WSK_Agrave              0x0c0
+#define WSK_Aacute              0x0c1
+#define WSK_Acircumflex         0x0c2
+#define WSK_Atilde              0x0c3
+#define WSK_Adiaeresis          0x0c4
+#define WSK_Aring               0x0c5
+#define WSK_AE                  0x0c6
+#define WSK_Ccedilla            0x0c7
+#define WSK_Egrave              0x0c8
+#define WSK_Eacute              0x0c9
+#define WSK_Ecircumflex         0x0ca
+#define WSK_Ediaeresis          0x0cb
+#define WSK_Igrave              0x0cc
+#define WSK_Iacute              0x0cd
+#define WSK_Icircumflex         0x0ce
+#define WSK_Idiaeresis          0x0cf
+#define WSK_Eth                 0x0d0
+#define WSK_Ntilde              0x0d1
+#define WSK_Ograve              0x0d2
+#define WSK_Oacute              0x0d3
+#define WSK_Ocircumflex         0x0d4
+#define WSK_Otilde              0x0d5
+#define WSK_Odiaeresis          0x0d6
+#define WSK_multiply            0x0d7
+#define WSK_Ooblique            0x0d8
+#define WSK_Ugrave              0x0d9
+#define WSK_Uacute              0x0da
+#define WSK_Ucircumflex         0x0db
+#define WSK_Udiaeresis          0x0dc
+#define WSK_Yacute              0x0dd
+#define WSK_Thorn               0x0de
+#define WSK_ssharp              0x0df
+#define WSK_agrave              0x0e0
+#define WSK_aacute              0x0e1
+#define WSK_acircumflex         0x0e2
+#define WSK_atilde              0x0e3
+#define WSK_adiaeresis          0x0e4
+#define WSK_aring               0x0e5
+#define WSK_ae                  0x0e6
+#define WSK_ccedilla            0x0e7
+#define WSK_egrave              0x0e8
+#define WSK_eacute              0x0e9
+#define WSK_ecircumflex         0x0ea
+#define WSK_ediaeresis          0x0eb
+#define WSK_igrave              0x0ec
+#define WSK_iacute              0x0ed
+#define WSK_icircumflex         0x0ee
+#define WSK_idiaeresis          0x0ef
+#define WSK_eth                 0x0f0
+#define WSK_ntilde              0x0f1
+#define WSK_ograve              0x0f2
+#define WSK_oacute              0x0f3
+#define WSK_ocircumflex         0x0f4
+#define WSK_otilde              0x0f5
+#define WSK_odiaeresis          0x0f6
+#define WSK_division            0x0f7
+#define WSK_oslash              0x0f8
+#define WSK_ugrave              0x0f9
+#define WSK_uacute              0x0fa
+#define WSK_ucircumflex         0x0fb
+#define WSK_udiaeresis          0x0fc
+#define WSK_yacute              0x0fd
+#define WSK_thorn               0x0fe
+#define WSK_ydiaeresis          0x0ff
+
+
 #endif //UNIX - Xwindow System
 
 //-----------------------------------------------------------------------------
@@ -760,10 +855,11 @@
 #define WSK_Meta_R				-1
 #define WSK_Alt_L				VK_MENU
 #define WSK_Alt_R				VK_MENU
-#define WSK_Super_L				-1
-#define WSK_Super_R				-1
+#define WSK_Super_L				VK_LWIN
+#define WSK_Super_R				VK_RWIN
 #define WSK_Hyper_L				-1
 #define WSK_Hyper_R				-1
+#define WSK_Apps				VK_APPS
 
 #define WSK_space				VK_SPACE
 #define WSK_exclam				0x121
@@ -863,6 +959,104 @@
 #define WSK_braceright			0x17d
 #define WSK_asciitilde			0x17e
 
+#define WSK_nobreakspace        0x1a0
+#define WSK_exclamdown          0x1a1
+#define WSK_cent                0x1a2
+#define WSK_sterling            0x1a3
+#define WSK_currency            0x1a4
+#define WSK_yen                 0x1a5
+#define WSK_brokenbar           0x1a6
+#define WSK_section             0x1a7
+#define WSK_diaeresis           0x1a8
+#define WSK_copyright           0x1a9
+#define WSK_ordfeminine         0x1aa
+#define WSK_guillemotleft       0x1ab
+#define WSK_notsign             0x1ac
+#define WSK_hyphen              0x1ad
+#define WSK_registered          0x1ae
+#define WSK_macron              0x1af
+#define WSK_degree              0x1b0
+#define WSK_plusminus           0x1b1
+#define WSK_twosuperior         0x1b2
+#define WSK_threesuperior       0x1b3
+#define WSK_acute               0x1b4
+#define WSK_mu                  0x1b5
+#define WSK_paragraph           0x1b6
+#define WSK_periodcentered      0x1b7
+#define WSK_cedilla             0x1b8
+#define WSK_onesuperior         0x1b9
+#define WSK_masculine           0x1ba
+#define WSK_guillemotright      0x1bb
+#define WSK_onequarter          0x1bc
+#define WSK_onehalf             0x1bd
+#define WSK_threequarters       0x1be
+#define WSK_questiondown        0x1bf
+#define WSK_Agrave              0x1c0
+#define WSK_Aacute              0x1c1
+#define WSK_Acircumflex         0x1c2
+#define WSK_Atilde              0x1c3
+#define WSK_Adiaeresis          0x1c4
+#define WSK_Aring               0x1c5
+#define WSK_AE                  0x1c6
+#define WSK_Ccedilla            0x1c7
+#define WSK_Egrave              0x1c8
+#define WSK_Eacute              0x1c9
+#define WSK_Ecircumflex         0x1ca
+#define WSK_Ediaeresis          0x1cb
+#define WSK_Igrave              0x1cc
+#define WSK_Iacute              0x1cd
+#define WSK_Icircumflex         0x1ce
+#define WSK_Idiaeresis          0x1cf
+#define WSK_Eth                 0x1d0
+#define WSK_Ntilde              0x1d1
+#define WSK_Ograve              0x1d2
+#define WSK_Oacute              0x1d3
+#define WSK_Ocircumflex         0x1d4
+#define WSK_Otilde              0x1d5
+#define WSK_Odiaeresis          0x1d6
+#define WSK_multiply            0x1d7
+#define WSK_Ooblique            0x1d8
+#define WSK_Ugrave              0x1d9
+#define WSK_Uacute              0x1da
+#define WSK_Ucircumflex         0x1db
+#define WSK_Udiaeresis          0x1dc
+#define WSK_Yacute              0x1dd
+#define WSK_Thorn               0x1de
+#define WSK_ssharp              0x1df
+#define WSK_agrave              0x1e0
+#define WSK_aacute              0x1e1
+#define WSK_acircumflex         0x1e2
+#define WSK_atilde              0x1e3
+#define WSK_adiaeresis          0x1e4
+#define WSK_aring               0x1e5
+#define WSK_ae                  0x1e6
+#define WSK_ccedilla            0x1e7
+#define WSK_egrave              0x1e8
+#define WSK_eacute              0x1e9
+#define WSK_ecircumflex         0x1ea
+#define WSK_ediaeresis          0x1eb
+#define WSK_igrave              0x1ec
+#define WSK_iacute              0x1ed
+#define WSK_icircumflex         0x1ee
+#define WSK_idiaeresis          0x1ef
+#define WSK_eth                 0x1f0
+#define WSK_ntilde              0x1f1
+#define WSK_ograve              0x1f2
+#define WSK_oacute              0x1f3
+#define WSK_ocircumflex         0x1f4
+#define WSK_otilde              0x1f5
+#define WSK_odiaeresis          0x1f6
+#define WSK_division            0x1f7
+#define WSK_oslash              0x1f8
+#define WSK_ugrave              0x1f9
+#define WSK_uacute              0x1fa
+#define WSK_ucircumflex         0x1fb
+#define WSK_udiaeresis          0x1fc
+#define WSK_yacute              0x1fd
+#define WSK_thorn               0x1fe
+#define WSK_ydiaeresis          0x1ff
+
+
 #endif //MS Windows System
 
 #ifdef BTRON //BTRON
@@ -1242,6 +1436,7 @@
 #define WSK_Super_R  -1
 #define WSK_Hyper_L  -1
 #define WSK_Hyper_R  -1
+#define WSK_Apps     -1
 
 #define WSK_space               TK_KSP
 #define WSK_exclam              TK_EXCL
