@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCCLASSPROPEDITOR_H
 #define WSCCLASSPROPEDITOR_H
@@ -89,44 +85,44 @@ class WSCclassPropEditor   :public WSCbaseDialog{
 
      WSMFpropertyDef( WSNclient,  void*,          _voidp_client ); 
 
-static void _addv_work(WSCbase*);
-static void _delv_work(WSCbase*);
-static void _upv_work(WSCbase*);
-static void _downv_work(WSCbase*);
+WSDLEX32 static void _addv_work(WSCbase*);
+WSDLEX32 static void _delv_work(WSCbase*);
+WSDLEX32 static void _upv_work(WSCbase*);
+WSDLEX32 static void _downv_work(WSCbase*);
 
-static void _set_cbtn_work(WSCbase*);
-static void _end_btn_work(WSCbase*);
-static void _ibtn1_work(WSCbase*);
-static void _ibtn2_work(WSCbase*);
-static void _ibtn3_work(WSCbase*);
-static void _dialog_set_work(WSCbase*);
-static void _dialog2_set_work(WSCbase*);
-static void _vsname_set_work(WSCbase*);
-static void _value_set_work(WSCbase*);
+WSDLEX32 static void _set_cbtn_work(WSCbase*);
+WSDLEX32 static void _end_btn_work(WSCbase*);
+WSDLEX32 static void _ibtn1_work(WSCbase*);
+WSDLEX32 static void _ibtn2_work(WSCbase*);
+WSDLEX32 static void _ibtn3_work(WSCbase*);
+WSDLEX32 static void _dialog_set_work(WSCbase*);
+WSDLEX32 static void _dialog2_set_work(WSCbase*);
+WSDLEX32 static void _vsname_set_work(WSCbase*);
+WSDLEX32 static void _value_set_work(WSCbase*);
 
 //static void _op_delete_callback(long,void*);
-static void _dialog1_resize_work(WSCbase*);
-static void _dialog2_resize_work(WSCbase*);
+WSDLEX32 static void _dialog1_resize_work(WSCbase*);
+WSDLEX32 static void _dialog2_resize_work(WSCbase*);
 //static void _op_us_dummy();
 //static void _op_cm_dummy(void*);
 //static void _op_cb_dummy(WSCbase*);
 //static void _op_rt_dummy(long,long*);
-virtual void onValueChange();
-virtual void onResize(WSCrect*);
-    void _adjust();
+WSDLEX32 virtual void onValueChange();
+WSDLEX32 virtual void onResize(WSCrect*);
+    WSDLEX32 void _adjust();
   public:
-    WSCclassPropEditor(WSCbase*,char*);
-    virtual  long initialize();
+    WSDLEX32 WSCclassPropEditor(WSCbase*,char*);
+    WSDLEX32 virtual  long initialize();
 //    virtual  long execEventProc(long,void*);
 
-    void setEditClient(WSCbase*);
-    WSCbase* getEditClient();
-    virtual void onGuiPolicyChange(long);
+    WSDLEX32 void setEditClient(WSCbase*);
+    WSDLEX32 WSCbase* getEditClient();
+    WSDLEX32 virtual void onGuiPolicyChange(long);
 
-    long getSelectedPropObj();
+    WSDLEX32 long getSelectedPropObj();
 
-    void updateProperties();
-    void setSrcPath(char*);
+    WSDLEX32 void updateProperties();
+    WSDLEX32 void setSrcPath(char*);
 
     WSMFguiClassDef(WSCclassPropEditor,WSCbaseDialog);
 };
