@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #ifndef WSCMESSAGEBOX_H
 #define WSCMESSAGEBOX_H
@@ -46,20 +42,20 @@ class WSCmessageBox  :public WSCbaseDialog {
      WSMFpropertyDef( WSNhbarValue, WSCulong,  _hvalue);
 
      
-     void _adjust();
+     WSDLEX32 void _adjust();
 
 //virtual void onVisibleChange(WSCbool);
 //virtual void onParentVisibleChange(WSCbool);
-virtual void onResize(WSCrect*);
-virtual void onActivate();
-virtual void onGuiPolicyChange(long);
+WSDLEX32 virtual void onResize(WSCrect*);
+WSDLEX32 virtual void onActivate();
+WSDLEX32 virtual void onGuiPolicyChange(long);
 
-static     void _textf_work(WSCbase*);
+WSDLEX32 static     void _textf_work(WSCbase*);
   public:
-    WSCmessageBox(WSCbase*,char*);
-    virtual ~WSCmessageBox();
-    virtual  long initialize();
-    virtual WSCbase* getPropertyInheritChild();
+    WSDLEX32 WSCmessageBox(WSCbase*,char*);
+    WSDLEX32 virtual ~WSCmessageBox();
+    WSDLEX32 virtual  long initialize();
+    WSDLEX32 virtual WSCbase* getPropertyInheritChild();
     WSCtextField* getTextField();
 
     WSMFguiClassDef(WSCmessageBox,WSCbaseDialog);
