@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCPROJECTEDITOR_H
 #define WSCPROJECTEDITOR_H
@@ -87,6 +83,7 @@ class WSCprojectEditor :public WSCbaseDialog{
     WSCvlabel* _pe_gt3;
     WSCvlabel* _pe_gt4;
     WSClist* _pe_elist;
+    WSClist* _pe_elist2;
     WSCvslider* _pe_sl1;
     WSCvslider* _pe_sl2;
     WSCvslider* _pe_sl3;
@@ -110,6 +107,8 @@ class WSCprojectEditor :public WSCbaseDialog{
     WSCvifield* _pe_ifi010;
     WSCvifield* _pe_ifi011;
     WSCvifield* _pe_ifi012;
+    WSCvifield* _pe_ifi013;
+    WSCvifield* _pe_ifi014;
     WSCvbtn*    fdiavbtn000;
     WSCvbtn*    fdiavbtn001;
     WSCvbtn*    fdiavbtn002;
@@ -125,6 +124,8 @@ class WSCprojectEditor :public WSCbaseDialog{
     WSCindexForm* index0052;
     WSCvbtn*    _pe_bt1;
     WSCvbtn*    _pe_btn3;
+    WSCvbtn*    _pe_bt2;
+    WSCvbtn*    _pe_btn4;
     WSCoption*  _opt1;
 //    WSCvradio*  _pe_radio1;
     WSCoption*  _opt4;
@@ -152,83 +153,87 @@ class WSCprojectEditor :public WSCbaseDialog{
     WSCstring _linkoptions_fname;
     WSCstring _default_prj_fname;
     long _target_no;
+    WSCvifield* _pe_ifi015;
 //    WSCinputDialog* _pe_inputd;
 //WSCcolorSelect* _pe_colord;
 
 //    WSMFpropertyDef( WSNfileName,   char*,   _file_name ); 
-    static void _ifnum_op(WSCbase*);
-    static void _sldnum_op(WSCbase*);
-    static void _feditop(WSCbase*);
-    static void _pe_btn1_work(WSCbase*);
-//    static void _pe_btn2_work(WSCbase*);
-    static void _pe_btn3_work(WSCbase*);
-    static void _policy_select_work(WSCbase*);
-    static void _pe_fbt1_work(WSCbase*);
-    static void _pe_fbt2_work(WSCbase*);
-    static void _pe_fbt3_work(WSCbase*);
-    static void _pe_fbt4_work(WSCbase*);
-    static void _pe_fbt5_work(WSCbase*);
-    static void _pe_fbt6_work(WSCbase*);
-    static void _pe_fbt7_work(WSCbase*);
-    static void _pe_fbt8_work(WSCbase*);
-    static void _pe_fbt9_work(WSCbase*);
-    static void _cpe_fbt_work(WSCbase*);
+    WSDLEX32 static void _ifnum_op(WSCbase*);
+    WSDLEX32 static void _sldnum_op(WSCbase*);
+    WSDLEX32 static void _feditop(WSCbase*);
+    WSDLEX32 static void _pe_btn1_work(WSCbase*);
+    WSDLEX32 static void _pe_btn2_work(WSCbase*);
+    WSDLEX32 static void _pe_btn3_work(WSCbase*);
+    WSDLEX32 static void _pe_btn4_work(WSCbase*);
+    WSDLEX32 static void _policy_select_work(WSCbase*);
+    WSDLEX32 static void _pe_fbt1_work(WSCbase*);
+    WSDLEX32 static void _pe_fbt2_work(WSCbase*);
+    WSDLEX32 static void _pe_fbt3_work(WSCbase*);
+    WSDLEX32 static void _pe_fbt4_work(WSCbase*);
+    WSDLEX32 static void _pe_fbt5_work(WSCbase*);
+    WSDLEX32 static void _pe_fbt6_work(WSCbase*);
+    WSDLEX32 static void _pe_fbt7_work(WSCbase*);
+    WSDLEX32 static void _pe_fbt8_work(WSCbase*);
+    WSDLEX32 static void _pe_fbt9_work(WSCbase*);
+    WSDLEX32 static void _cpe_fbt_work(WSCbase*);
 
-    static void _editor_option_work(WSCbase*);
-    static void _target_option_work(WSCbase*);
+    WSDLEX32 static void _editor_option_work(WSCbase*);
+    WSDLEX32 static void _target_option_work(WSCbase*);
 
-    virtual void onResize(WSCrect* rect);
-    virtual void onActivate();
-    long  _open(char* ,WSCindexData*,WSCindexData*);
-    long  _save(char* ,WSCindexData*,WSCindexData*);
-    void  _update_data(WSCindexData*,WSCindexData*);
-    void  _cancel_data(WSCindexData*,WSCindexData*);
-    static void  _compat_work(WSCbase*);
-    long  _reload_project();
-    void _loadLinkOptions(WSCstring*);
+    WSDLEX32 virtual void onResize(WSCrect* rect);
+    WSDLEX32 virtual void onActivate();
+    WSDLEX32 long  _open(char* ,WSCindexData*,WSCindexData*);
+    WSDLEX32 long  _save(char* ,WSCindexData*,WSCindexData*);
+    WSDLEX32 void  _update_data(WSCindexData*,WSCindexData*);
+    WSDLEX32 void  _cancel_data(WSCindexData*,WSCindexData*);
+    WSDLEX32 static void  _compat_work(WSCbase*);
+    WSDLEX32 long  _reload_project();
+    WSDLEX32 void _loadLinkOptions(WSCstring*);
 
   public:
-    WSCprojectEditor(WSCbase*,char*);
-    virtual ~WSCprojectEditor();
-    virtual  long initialize();
+    WSDLEX32 WSCprojectEditor(WSCbase*,char*);
+    WSDLEX32 virtual ~WSCprojectEditor();
+    WSDLEX32 virtual  long initialize();
 
-    virtual void adjustForm();
-    virtual void onGuiPolicyChange(long);
-    virtual void onVisibleChange(WSCbool);
+    WSDLEX32 virtual void adjustForm();
+    WSDLEX32 virtual void onGuiPolicyChange(long);
+    WSDLEX32 virtual void onVisibleChange(WSCbool);
 
-    void  setDefaultProjectName(char* );
-    long  open(char* );
-    long  save(char* );
-    long  open_global(char* );
-    long  save_global(char* );
+    WSDLEX32 void  setDefaultProjectName(char* );
+    WSDLEX32 long  open(char* );
+    WSDLEX32 long  save(char* );
+    WSDLEX32 long  open_global(char* );
+    WSDLEX32 long  save_global(char* );
 
-    WSCstring getProjectDataFromFile(char* fname,char* index);
-    char* getProjectData(char* index = NULL);
-    char* getProjectData(char* index,WSCindexData*,WSCindexData*);
-    void  updateData();
-    void  cancelData();
-    void  setProjectData(char*,char*);
-    void  setMenu(long no);
-    void  setUseDll(char*);
-    void  clearUseDll();
-    void  setObjectBox(WSCobjectBox*);
-    void  setUseLocaleName(char*);
-    char*  getUseLocaleName();
-    void  setEncodingMenu(char* menu,long default_value);
-    long getLanguage();
-    void setLanguage(long);
-    WSCbool getOldSwigMapping();
+    WSDLEX32 WSCstring getProjectDataFromFile(char* fname,char* index);
+    WSDLEX32 char* getProjectData(char* index = NULL);
+    WSDLEX32 char* getProjectData(char* index,WSCindexData*,WSCindexData*);
+    WSDLEX32 void  updateData();
+    WSDLEX32 void  cancelData();
+    WSDLEX32 void  setProjectData(char*,char*);
+    WSDLEX32 void  setMenu(long no);
+    WSDLEX32 void  setUseDll(char*);
+    WSDLEX32 void  setUseFiles(char*);
+    WSDLEX32 void  clearUseDll();
+    WSDLEX32 void  setObjectBox(WSCobjectBox*);
+    WSDLEX32 void  setUseLocaleName(char*);
+    WSDLEX32 char*  getUseLocaleName();
+    WSDLEX32 void  setEncodingMenu(char* menu,long default_value);
+    WSDLEX32 long getLanguage();
+    WSDLEX32 void setLanguage(long);
+//    WSDLEX32 WSCbool getOldSwigMapping(); //removed..
+    WSDLEX32 WSCbool useMpfcMapping(); //removed..
 
-    WSCstring getGuiPolicy();
-    void setGuiPolicy(char*);
+    WSDLEX32 WSCstring getGuiPolicy();
+    WSDLEX32 void setGuiPolicy(char*);
 
-    void loadEditorOptions(char*);
-    void loadLinkOptions(char*);
-    WSCstring getLinkOptions(long);
-    WSCstring getStaticLinkOptions(long);
+    WSDLEX32 void loadEditorOptions(char*);
+    WSDLEX32 void loadLinkOptions(char*);
+    WSDLEX32 WSCstring getLinkOptions(long);
+    WSDLEX32 WSCstring getStaticLinkOptions(long);
 
-    WSCstring getTargetSuffixName();
-    WSCstring getTargetEnvironment();
+    WSDLEX32 WSCstring getTargetSuffixName();
+    WSDLEX32 WSCstring getTargetEnvironment();
 
     WSMFguiClassDef(WSCprojectEditor,WSCbaseDialog);
     WSMFversionDef(WSCprojectEditor);
