@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #ifndef WSCPROPEDIT_H
 #define WSCPROPEDIT_H
@@ -66,45 +62,46 @@ class WSCpropertyEditor  :public WSCscrForm{
      WSCoption* newopti_008;
      WSCbool _update_sequence;
 
-     void setWorkWSNwidth(WSCushort);
-     void setWorkWSNheight(WSCushort);
+     WSDLEX32 void setWorkWSNwidth(WSCushort);
+     WSDLEX32 void setWorkWSNheight(WSCushort);
 
-     void _check_name(char* );
+     WSDLEX32 void _check_name(char* );
 
-    virtual void onVisibleChange(WSCbool);
-    virtual void onParentVisibleChange(WSCbool);
-    virtual void onActivate();
-    virtual void onValueChange();
+    WSDLEX32 virtual void onVisibleChange(WSCbool);
+    WSDLEX32 virtual void onParentVisibleChange(WSCbool);
+    WSDLEX32 virtual void onActivate();
+    WSDLEX32 virtual void onValueChange();
 
-     void updateGeometryProperties();
-static void _set_cbtn_work(WSCbase*);
+     WSDLEX32 void updateGeometryProperties();
+WSDLEX32 static void _set_cbtn_work(WSCbase*);
 
-static void _set_btn_work(WSCbase*);
-static void _color_set_work(WSCbase*,void*);
-static void _cset_btn_work(WSCbase*);
-static void _if_focus_on(WSCbase*);
-static void _exec_change_name_handler(WSCbase* this_instance);
-static void (*_ch_name_handler)(WSCbase* this_instance);
-static void _popup_prop_work(WSCbase*);
-static void _set_prop_work(WSCbase*);
-   void _sort_property(WSClistData* dest,WSClistData* src);
-static void _delay_update_work(WSCuchar clk,void*);
+WSDLEX32 static void _set_btn_work(WSCbase*);
+WSDLEX32 static void _color_set_work(WSCbase*,void*);
+WSDLEX32 static void _cset_btn_work(WSCbase*);
+WSDLEX32 static void _if_focus_on(WSCbase*);
+WSDLEX32 static void _exec_change_name_handler(WSCbase* this_instance);
+WSDLEX32 static void (*_ch_name_handler)(WSCbase* this_instance);
+WSDLEX32 static void _popup_prop_work(WSCbase*);
+WSDLEX32 static void _set_prop_work(WSCbase*);
+   WSDLEX32 void _sort_property(WSClistData* dest,WSClistData* src);
+WSDLEX32 static void _delay_update_work(WSCuchar clk,void*);
+    WSDLEX32 void _set_undo_proc();
  
   public:
-static void setChangeNameHandler(void (*)(WSCbase*));
+WSDLEX32 static void setChangeNameHandler(void (*)(WSCbase*));
 
-    WSCpropertyEditor(WSCbase*,char*);
-    virtual ~WSCpropertyEditor();
+    WSDLEX32 WSCpropertyEditor(WSCbase*,char*);
+    WSDLEX32 virtual ~WSCpropertyEditor();
 
-    virtual  long initialize();
-    virtual void onGuiPolicyChange(long);
+    WSDLEX32 virtual  long initialize();
+    WSDLEX32 virtual void onGuiPolicyChange(long);
 
 //    virtual long execEventProc(long,void*);
-    void updateProperties();
+    WSDLEX32 void updateProperties();
 
-    void setPropertyDataToClient();
+    WSDLEX32 void setPropertyDataToClient();
 
-virtual  void update();  //if GEOMETRY-UPDATE of user data is 1 then
+WSDLEX32 virtual  void update();  //if GEOMETRY-UPDATE of user data is 1 then
                          //execute updateGeometryProperties()
     WSMFguiClassDef(WSCpropertyEditor,WSCform);
 
