@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSCWINDOWEDITOR_H
 #define WSCWINDOWEDITOR_H
@@ -86,39 +82,39 @@ class WSCwindowEditor :public WSCscrForm{
     unsigned _editable:1;
     WSMFpropertyDef( WSNclient,  void*,          _voidp_client ); 
 
-    static void _prjtg_work(WSCbase*);
-    static void _vartg_work(WSCbase*);
-    static void _stg1_work(WSCbase*);
-    static void _psbt1_work(WSCbase*);
-    static void _psbt2_work(WSCbase*);
-    static void _psbt3_work(WSCbase*);
-    static void _psbt4_work(WSCbase*);
-    static void _vifi_work(WSCbase*);
-    static void _fbtn_work(WSCbase*);
-    static void _rctg_work(WSCbase*);
+    WSDLEX32 static void _prjtg_work(WSCbase*);
+    WSDLEX32 static void _vartg_work(WSCbase*);
+    WSDLEX32 static void _stg1_work(WSCbase*);
+    WSDLEX32 static void _psbt1_work(WSCbase*);
+    WSDLEX32 static void _psbt2_work(WSCbase*);
+    WSDLEX32 static void _psbt3_work(WSCbase*);
+    WSDLEX32 static void _psbt4_work(WSCbase*);
+    WSDLEX32 static void _vifi_work(WSCbase*);
+    WSDLEX32 static void _fbtn_work(WSCbase*);
+    WSDLEX32 static void _rctg_work(WSCbase*);
 
-    virtual void onActivate();
-    virtual void onVisibleChange(WSCbool);
-    virtual void onParentVisibleChange(WSCbool);
+    WSDLEX32 virtual void onActivate();
+    WSDLEX32 virtual void onVisibleChange(WSCbool);
+    WSDLEX32 virtual void onParentVisibleChange(WSCbool);
 
 
-    void _adjust();
-    virtual void setWorkWSNwidth(WSCushort);
-    virtual void setWorkWSNheight(WSCushort);
+    WSDLEX32 void _adjust();
+    WSDLEX32 virtual void setWorkWSNwidth(WSCushort);
+    WSDLEX32 virtual void setWorkWSNheight(WSCushort);
 
   public:
-    WSCwindowEditor(WSCbase*,char*);
-    virtual ~WSCwindowEditor();
+    WSDLEX32 WSCwindowEditor(WSCbase*,char*);
+    WSDLEX32 virtual ~WSCwindowEditor();
 
-    virtual  long initialize();
+    WSDLEX32 virtual  long initialize();
 //    virtual  long execEventProc(long,void*);
-    virtual void onGuiPolicyChange(long);
+    WSDLEX32 virtual void onGuiPolicyChange(long);
 
-    void setEditClient(WSCbase*);
-    WSCbase* getEditClient();
+    WSDLEX32 void setEditClient(WSCbase*);
+    WSDLEX32 WSCbase* getEditClient();
 
-    void setEditable(WSCbool);
-    WSCbool getEditable();
+    WSDLEX32 void setEditable(WSCbool);
+    WSDLEX32 WSCbool getEditable();
 
     WSMFguiClassDef(WSCwindowEditor,WSCscrForm);
     WSMFversionDef(WSCwindowEditor);
