@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall
-// not be used in advertising or otherwise to promote the sale, use or other
-// dealings in this Software without prior written authorization from
-// Toshikaz Hirabayashi.
 
 #ifndef WSDBTRONFORMDEV_H
 #define WSDBTRONFORMDEV_H
@@ -47,6 +43,8 @@ class WSDbtronFormDev               :public WSDwindowDev {
     WSCbool _added_to_child_dev_list;
     RLIST   _clip_fore_list[N_VIEWR];
     long    _clip_num;
+    WSDbtronFormDev* _mparent_dev;
+
 static WSCbool _expose_sequence;
 static WSClistData _geometry_update_list;
 static WSClistData _update_list;
@@ -139,7 +137,7 @@ virtual void _clear_area( long x,long y,WSCulong w,WSCulong h);
   virtual void getAbstPos(short*,short*);
 //  virtual void getAbstPosForGid(short*,short*);
   virtual WSCbool haveWindow();
-  WSClistData getChildDevList();
+  WSClistData* getChildDevList();
 
   void setDefaultRegion();
   void redrawAll();
@@ -148,7 +146,7 @@ virtual void _clear_area( long x,long y,WSCulong w,WSCulong h);
   static WSDbtronFormDev* getMouseGrabedDev(); 
   static void setMouseGrabedDev(WSDbtronFormDev* dev); 
   virtual void drawBackGround(long x,long y,WSCulong w,WSCulong h);
-  virtual void exposeExecute(short x,short y,WSCushort w,WSCushort h);
+//  virtual void exposeExecute(short x,short y,WSCushort w,WSCushort h);
   virtual void recursiveExposeExecute(short x,short y,WSCushort w,WSCushort h);
   virtual long exposeArea(long x,long y,WSCulong w,WSCulong h,WSCbool scaling = True);    
   void addUpdateList();
@@ -158,6 +156,8 @@ static void executeUpdate();
 static void setGlobalRegion(short x,short y,WSCushort,WSCushort);
 static void getGlobalRegion(short* x,short* y,WSCushort*,WSCushort*);
 virtual void recursive_update();
+virtual long detachClient();
+  void _clear_parent_cache();
 
   WSMFclassDef(WSDbtronFormDev,WSDwindowDev);
 };
