@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #ifndef WSDMACKEYBOARD_H
 #define WSDMACKEYBOARD_H
@@ -70,7 +66,7 @@ static void _status_draw(XIC,XPointer,XIMStatusDrawCallbackStruct );
 
 //    WSCbool deliverKeyEvent(XEvent* ev);
 
-    virtual long startKanji(WSDdev*,short x,short y,short fontno,short fgno,short bgno);
+    virtual long startFep(WSDdev*,short x,short y,short fontno,short fgno,short bgno);
     virtual long startAscii(WSDdev*,short x,short y,short fontno,short fgno,short bgno);
     virtual long setFepPos (WSDdev*,short x,short y,short fontno,short fgno,short bgno);
     virtual long resetFep(WSDdev*);
