@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall 
-// not be used in advertising or otherwise to promote the sale, use or other 
-// dealings in this Software without prior written authorization from  
-// Toshikaz Hirabayashi.
 
 
 #ifndef WSDWINCOLOR_H
@@ -34,15 +30,15 @@
 class WSDwinColor :public WSDcolor{
 
   protected:
-virtual    long initialize();
-virtual    long destroy();
+WSDLEX32 virtual    long initialize();
+WSDLEX32 virtual    long destroy();
 
   public:
-   WSDwinColor();
-   virtual ~WSDwinColor();
-   virtual long    getRGB(long*,long*,long*);
+   WSDLEX32 WSDwinColor();
+   WSDLEX32 virtual ~WSDwinColor();
+   WSDLEX32 virtual long    getRGB(long*,long*,long*);
 
    WSMFclassDef(WSDwinColor,WSDcolor);
 };
 
-#endif //WSDXCOLOR_H
+#endif //WSDWINCOLOR_H
