@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team 
-// shall not be used in advertising or otherwise to promote the sale, use or 
-// other dealings in this Software without prior written authorization from  
-// WideStudio Development Team.
 
 
 #ifndef WSDWINKEYBOARD_H
@@ -45,9 +41,10 @@ class WSDwinKeyboard  :public WSDkeyboard{
     static void _device_move_handler(WSDdev*,void*);
     WSDdev* _dev;
     WSCbool _focused;
+    WSCbool _fep_status_bk;
     char    _buf[32];
     long    _pos;
-
+    DWORD   _sent_bk;
   public:
     WSDLEX32 WSDwinKeyboard();
     WSDLEX32 virtual ~WSDwinKeyboard();
@@ -56,16 +53,21 @@ class WSDwinKeyboard  :public WSDkeyboard{
 
     WSDLEX32 WSCbool _dispatch_key(MSG*);
 
-    WSDLEX32 virtual long startKanji(WSDdev*,short x,short y,short fontno,short fgno,short bgno);
+    WSDLEX32 virtual long startFep(WSDdev*,short x,short y,short fontno,short fgno,short bgno);
     WSDLEX32 virtual long startAscii(WSDdev*,short x,short y,short fontno,short fgno,short bgno);
     WSDLEX32 virtual long setFepPos (WSDdev*,short x,short y,short fontno,short fgno,short bgno);
     WSDLEX32 virtual long resetFep(WSDdev*);
+    WSDLEX32 virtual long setFepStatus(long);
+
+    WSDLEX32 virtual void setEnableFep(WSDdev*,WSCbool);
 
     WSDLEX32 void setModeWindowPos(short x,short y,WSDdev*);
     WSDLEX32 virtual void setSelectedString(char*,long encode = WS_EN_DEFAULT);
     WSDLEX32 virtual char* getSelectedString(long encode = WS_EN_DEFAULT);
 WSMFclassDef(WSDwinKeyboard,WSDkeyboard);
-
+    WSDLEX32 void updateFepStatus();
+//    WSDLEX32 HIMC   getHimc();
+    WSDLEX32 void   setHwnd(HWND);
 };
-extern WSDwinKeyboard* WSGIwinKeyboard();
+extern WSDLEX32 WSDwinKeyboard* WSGIwinKeyboard();
 #endif
