@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 Toshikaz Hirabayashi
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,18 +14,14 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// TOSHIKAZ HIRABAYASHI BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
-// OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of Toshikaz Hirabayashi shall 
-// not be used in advertising or otherwise to promote the sale, use or other 
-// dealings in this Software without prior written authorization from  
-// Toshikaz Hirabayashi.
 
 
-#ifndef WSDXVPDEV_H
-#define WSDXVPDEV_H
+#ifndef WSDWINNWDEV_H
+#define WSDWINNWDEV_H
 
 #include <windows.h>
 #include <WSDnwDev.h>
@@ -36,19 +32,18 @@ class WSDwinnwDev               :public WSDnwDev {
   protected:
 static HBITMAP _pixmap;
 static HDC     _mdc;
-
-     virtual long getPixmap();
-     virtual long _copy_pixmap_to_window();
+ 
+     WSDLEX32 virtual long getPixmap();
+     WSDLEX32 virtual long _copy_pixmap_to_window();
 
   public:
-     WSDwinnwDev();
-     virtual ~WSDwinnwDev();
-     WSCbool getReady();
+     WSDLEX32 WSDwinnwDev();
+     WSDLEX32 virtual ~WSDwinnwDev();
+     WSDLEX32 WSCbool getReady();
 
-virtual long     getContextResource();
+WSDLEX32 virtual long     getContextResource();
 
 //----------------------------------------------//
-//NX`
      WSMFclassDef(WSDwinnwDev,WSDnwDev);
 };
 
