@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,21 +14,18 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #ifndef WSDXAPPDEV_H
 #define WSDXAPPDEV_H
 
 #include <X11/Intrinsic.h>
 #include <WSDappDev.h>
-
+class WSDsemaphore;
+class WSDmutex;
 class WSDxappDev               :public WSDappDev {
 
   private:
@@ -43,8 +40,18 @@ class WSDxappDev               :public WSDappDev {
      Atom         _accept_atom_for_sock;
      Atom         _read_event_for_execute;
      Atom         _callback_event_for_thread;
-  protected:
-
+     Atom         _lock_event_for_thread;
+     WSDsemaphore*    _sem_lock;
+     WSDsemaphore*    _sem_wait;
+     WSDmutex*    _mutex;
+     int          _msg_pipe[2];
+     int          _thr_id;
+     int          _lock_events;
+#ifndef NO_MWT_FUNCTION
+     Display*     _grabbed_disp;
+     WSCbool      _owner_grab;
+#endif
+     static void  _msg_recv(void*,int*,XtInputId*);
   public:
      WSDxappDev();
      virtual ~WSDxappDev();
@@ -77,9 +84,20 @@ virtual WSCushort getHeight();
     virtual long loadGuiPolicy();
     virtual long saveGuiPolicy();
 
+    virtual long lock();
+    virtual long unlock();
+    void    _init_lock();
+    int     _get_main_loop_thr_id();
+    void checkThreadSafe();
+#ifndef NO_MWT_FUNCTION
+    Display* getGrabbedDisplay();
+    void     setGrabbedDisplay(Display*);
+    WSCbool  getOwnerGrab();
+    void     setOwnerGrab(WSCbool fl);
+#endif
 //----------------------------------------------//
      WSMFclassDef(WSDxappDev,WSDappDev);
 };
 extern WSDxappDev* WSGIxwinAppDev();
-
+#define WS_CHECK_THREAD_SAFE  WSGIxwinAppDev()->checkThreadSafe();
 #endif
