@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #ifndef WSDXFONT_H
 #define WSDXFONT_H
@@ -33,18 +29,21 @@
 class WSDxfont               :public WSDfont {
 
  protected:
+#if 0
   WSCstring _ergs;
   WSCstring _krgs;
   WSCstring _grgs;
   WSCstring _efnd;
   WSCstring _kfnd;
   WSCstring _gfnd;
-
+#endif
   WSCstring _fmly;
+#if 0
   WSCstring _wght;
   WSCstring _slant;
-
-  long _xfont_set;
+#endif
+  XFontSet _xfont_set;
+  char* _font_set_name;
   long _font_height;
   virtual long initialize();
   virtual long destroy();
@@ -56,8 +55,11 @@ class WSDxfont               :public WSDfont {
   virtual long getStringHeight(WSCstring*);
   virtual long getStringWidthUCS2(WSCushort*);
   virtual long getStringHeightUCS2(WSCushort*);
+  XFontSet getXFontSet();
+  char* getFontSetName();
 
-  long getXFontSet();
+  virtual long getAscent();
+  virtual long getDescent();
 
   WSMFclassDef(WSDxfont,WSDfont);
 };
