@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #ifndef WSDXFORMDEV_H
 #define WSDXFORMDEV_H
@@ -40,6 +36,11 @@ class WSDxformDev               :public WSDwindowDev {
     short    _bg_pix;
     WSCushort _mno;
     long     _click_time;
+    long     _mouse_val;
+    unsigned _global_mouse:1;
+#ifndef NO_MWT_FUNCTION
+    unsigned _need_grab:1;
+#endif
 
 virtual long createWindow();
 virtual long createPixmap(); //call for Expose only...
@@ -89,6 +90,13 @@ virtual void _clear_area( long x,long y,WSCulong w,WSCulong h);
   virtual long raise();
   virtual long lower();
     void clearChildWidgets();
+#ifndef NO_RESTACK_CHILD
+  WSDLEX32 virtual void    restackChildren();
+#endif
+#ifndef NO_MWT_FUNCTION
+  WSDLEX32 WSCbool  getNeedGrab();
+#endif
+
   virtual void _clear_widget();
     WSMFclassDef(WSDxformDev,WSDwindowDev);
 };
