@@ -1,7 +1,7 @@
 #----------------------------------------------------------
 #Function for the event procedure
 #----------------------------------------------------------
-use mpfc;
+use mwt;
 
 sub btnop {
 	my ($object) = @_;
