@@ -1,14 +1,14 @@
 #----------------------------------------------------------
 #Function for the event procedure
 #----------------------------------------------------------
-use mpfc;
+use mwt;
 $count = 1;
 sub btnop {
 	my ($object) = @_;
 	#do something...
-	mpfc::WSCbase_setProperty($object,"labelString","hello!");
+	$object->setProperty("labelString","hello!");
 	$count +=1;
 	$str = "Btn pressed: $count";
-	mpfc::WSCbase_setProperty($newvlab_002,"labelString",$str);
+	$newvlab_002->setProperty("labelString",$str);
 }
 1;
