@@ -1,10 +1,10 @@
 #----------------------------------------------------------
 #Function for the event procedure
 #----------------------------------------------------------
-import mpfc
+import mwt
 
 def btnop(object):
 	#do something...
 	object.setProperty("labelString","hello")
 	return
-mpfc.WSGFfunctionRegister("btnop",btnop)
+mwt.WSGFfunctionRegister("btnop",btnop)
