@@ -1,7 +1,7 @@
 #----------------------------------------------------------
 #Function for the event procedure
 #----------------------------------------------------------
-import mpfc
+import mwt
 from newwin000 import newvlab_001
 
 def dotbtnep(object):
@@ -9,4 +9,4 @@ def dotbtnep(object):
 	newvlab_001.setVariantData("dot-input", True)
 	
 	return
-mpfc.WSGFfunctionRegister("dotbtnep",dotbtnep)
+mwt.WSGFfunctionRegister("dotbtnep",dotbtnep)
