@@ -1,7 +1,7 @@
 #----------------------------------------------------------
 #Function for the event procedure
 #----------------------------------------------------------
-import mpfc
+import mwt
 from newwin000 import *
 
 def resizeep(object):
@@ -15,10 +15,10 @@ def resizeep(object):
 
 	bheight = ((heigth - 60) - 10) / 5
 	bwidth = width / 4
-	for i in range(21):
-		newvbtn[i].setProperty("x",(5 + bwidth*(i / 5)))
-		newvbtn[i].setProperty("y",(60 + bheight * (i % 5)))
-		newvbtn[i].setProperty("width",bwidth)
-		newvbtn[i].setProperty("height", bheight)
+#	for i in range(21):
+#		newvbtn[i].setProperty("x",(5 + bwidth*(i / 5)))
+#		newvbtn[i].setProperty("y",(60 + bheight * (i % 5)))
+#		newvbtn[i].setProperty("width",bwidth)
+#		newvbtn[i].setProperty("height", bheight)
 	return
-mpfc.WSGFfunctionRegister("resizeep",resizeep)
+mwt.WSGFfunctionRegister("resizeep",resizeep)
