@@ -1,12 +1,12 @@
 #
 #WINDOW BUILDER PROJECT MASTER SRC
-import mpfc
+import mwt
 
-mpfc.WSGFdeviceInitialize()
-app = mpfc.WSGIappDev()
-mpfc.WSGFwsInitialize()
+mwt.WSGFdeviceInitialize()
+app = mwt.WSGIappDev()
+mwt.WSGFwsInitialize()
 
-mpfc.WSGFsetDefaultEncoding(20);
+mwt.WSGFsetDefaultEncoding(20);
 #--- OBJECT execute auto initialize ---
 import newwin000
 import resizeep
