@@ -3,11 +3,11 @@
 #----------------------------------------------------------
 def btnep(object)
   $val = $newdial_001.popup();
-  if $val == Mpfc::WS_DIALOG_OK
+  if $val == Mwt::WS_DIALOG_OK
      $newvlab_007.setProperty("labelString","DIALOG OK!");
-  elsif $val == Mpfc::WS_DIALOG_NO
+  elsif $val == Mwt::WS_DIALOG_NO
      $newvlab_007.setProperty("labelString","DIALOG NO!");
-  elsif $val == Mpfc::WS_DIALOG_CANCEL
+  elsif $val == Mwt::WS_DIALOG_CANCEL
      $newvlab_007.setProperty("labelString","DIALOG CANCEL!");
   end
 
