@@ -1,32 +1,32 @@
 #----------------------------------------------------------
 #Function for the event procedure
 #----------------------------------------------------------
-use mpfc;
+use mwt;
 sub update_ep {
   my ($object) = @_;
   $num = $#invader_list_x +1;
   if ($num == 0){
     $newvtim_001->setProperty("running",False);
-    mpfc::WSGIappMessageDialog()->setProperty("labelString",
+    mwt::WSGIappMessageDialog()->setProperty("labelString",
                               " ̰ϴ.");
-    mpfc::WSGIappMessageDialog()->setVisible($mpfc::True);
+    mwt::WSGIappMessageDialog()->setVisible($mwt::True);
   }
-  $key = mpfc::WSGIappKeyboard()->getKey();
-  if ( mpfc::WSGIappKeyboard()->isPressed() != False){
-    if ($key == $mpfc::WSK_Right){
+  $key = mwt::WSGIappKeyboard()->getKey();
+  if ( mwt::WSGIappKeyboard()->isPressed() != False){
+    if ($key == $mwt::WSK_Right){
       $my_base_x+=5;
       if ($my_base_x > 380){
         $my_base_x = 380;
       }
-    }elsif ($key == $mpfc::WSK_Left){
+    }elsif ($key == $mwt::WSK_Left){
       $my_base_x-=5;
       if ($my_base_x < 0){
         $my_base_x = 0;
       }
-    }elsif ($key == $mpfc::WSK_space){
+    }elsif ($key == $mwt::WSK_space){
       push(@my_missile_list_x, $my_base_x+5);
       push(@my_missile_list_y, $my_base_y);
-      mpfc::WSGIappKeyboard()->setKey(0);
+      mwt::WSGIappKeyboard()->setKey(0);
     }
   }
   $num = $#invader_missile_list_x +1;
@@ -36,9 +36,9 @@ sub update_ep {
     if ($my_base_x < $mpt_x && $mpt_x < $my_base_x + 10 &&
         $my_base_y < $mpt_y && $mpt_y < $my_base_y + 10){
       $newvtim_001->setProperty("running",False);
-      mpfc::WSGIappMessageDialog()->setProperty(
+      mwt::WSGIappMessageDialog()->setProperty(
                                  "labelString"," ƽϴ.");
-      mpfc::WSGIappMessageDialog()->setVisible($mpfc::True);
+      mwt::WSGIappMessageDialog()->setVisible($mwt::True);
       break;
     }
 
