@@ -1,3 +1,25 @@
+//
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
+//
+
   WSGIappLocaleSet()->setLocaleString("zh_CN","","");
   WSGIappLocaleSet()->setLocaleString("zh_CN","A3:WSSTR Can not create class: %s.\nPlease check the import library is right."," %s  \nȷ׷ӵǷȷ");
   WSGIappLocaleSet()->setLocaleString("zh_CN","B3:WSSTR The name %s is already used by another instance.","%s ظ");
@@ -99,6 +121,10 @@
   WSGIappLocaleSet()->setLocaleString("zh_CN","p3:WSSTR Create %s/%s.py...","%s/%s.py");
   WSGIappLocaleSet()->setLocaleString("zh_CN","o3:WSSTR Create %s.pl...","%s.pl");
   WSGIappLocaleSet()->setLocaleString("zh_CN","p3:WSSTR Create %s/%s.pl...","%s/%s.pl");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","o3:WSSTR Create %s.java...","%s.java");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","p3:WSSTR Create %s/%s.java...","%s/%s.java");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","o3:WSSTR Create %s.ml...","%s.ml");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","p3:WSSTR Create %s/%s.ml...","%s/%s.ml");
   WSGIappLocaleSet()->setLocaleString("zh_CN","q3:WSSTR Overwrite existing file: %s?","ȷϸ %s ļ");
   WSGIappLocaleSet()->setLocaleString("zh_CN","r3:WSSTR OK","");
   WSGIappLocaleSet()->setLocaleString("zh_CN","s3:WSSTR Please specify a target instance.","ָĿ");
@@ -132,7 +158,8 @@
   WSGIappLocaleSet()->setLocaleString("zh_CN","T3:WSSTR  Debug mode"," ģʽ");
   WSGIappLocaleSet()->setLocaleString("zh_CN","U3:WSSTR Application type","Application type");
   WSGIappLocaleSet()->setLocaleString("zh_CN","V3:WSSTR  Generate class library","");
-  WSGIappLocaleSet()->setLocaleString("zh_CN","W3:WSSTR Compiler","");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","W3:WSSTR C Compiler","C ");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","W3:WSSTR CPP Compiler","CPP ");
   WSGIappLocaleSet()->setLocaleString("zh_CN","X3:WSSTR Runtime mode,Debug mode,Runtime static mode,Debug static mode","ͨģʽ,ģʽ,̬ͨģʽ,̬ģʽ");
   WSGIappLocaleSet()->setLocaleString("zh_CN","Y3:WSSTR Include path","ض Include ·");
   WSGIappLocaleSet()->setLocaleString("zh_CN","Z3:WSSTR Compiler flags","ضѡ");
@@ -246,7 +273,16 @@
   WSGIappLocaleSet()->setLocaleString("zh_CN","P3:WSSTR Python(DBG)","Python(DBG)");
   WSGIappLocaleSet()->setLocaleString("zh_CN","P3:WSSTR Perl","Perl");
   WSGIappLocaleSet()->setLocaleString("zh_CN","P3:WSSTR Perl(DBG)","Perl(DBG)");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","P3:WSSTR Java","Java");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","P3:WSSTR Java(make)","Java(make)");
   WSGIappLocaleSet()->setLocaleString("zh_CN","f3:WSSTR Static option","ӷʽ");
   WSGIappLocaleSet()->setLocaleString("zh_CN","l3:WSSTR Statically linked","̬");
   WSGIappLocaleSet()->setLocaleString("zh_CN","l3:WSSTR Dynamically linked","̬");
-
+  WSGIappLocaleSet()->setLocaleString("zh_CN","K3:WSSTR Import source files","Դ");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","A3:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:","    :COPY:,ճ:PASTE:,SP,:PROP:,:OP:,:EXPROP:,SP,ɾ:DEL:");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","A3:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:","    :COPY:,ճ:PASTE:,SP,:PROP:,:OP:,:EXPROP:,SP,ǰ:FRONT:,ú:BACK:,SP,ɾ:DEL:");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","A:WSSTR  //please edit here...\n","  //༭\n");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","B:WSSTR\n\n//Initializing of class...\n","\n\n//ʼ\n");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","C:WSSTR//##### IMPORTANT: Do not edit from here  #######\n\n","//##### IMPORTANT  ²ɱ༭  #######\n\n");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","D:WSSTR//##### IMPORTANT: Do not edit to here  #######\n\n","//##### IMPORTANT  ˲ɱ༭     #########\n");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","E:WSSTR();\n\n    // class definitions/////////////////////////////////\n","();\n\n    // ////////////////////////////////////\n");
