@@ -1,3 +1,25 @@
+//
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
+//
+
   WSGIappLocaleSet()->setLocaleString("en_US","","");
   WSGIappLocaleSet()->setLocaleString("en_US","A3:WSSTR Can not create class: %s.\nPlease check the import library is right.","Can not create class: %s.\nPlease check the import library is right.");
   WSGIappLocaleSet()->setLocaleString("en_US","B3:WSSTR The name %s is already used by another instance.","The name %s is already used by another instance.");
@@ -99,6 +121,10 @@
   WSGIappLocaleSet()->setLocaleString("en_US","p3:WSSTR Create %s/%s.py...","Create %s/%s.py...");
   WSGIappLocaleSet()->setLocaleString("en_US","o3:WSSTR Create %s.pl...","Create %s.pl...");
   WSGIappLocaleSet()->setLocaleString("en_US","p3:WSSTR Create %s/%s.pl...","Create %s/%s.pl...");
+  WSGIappLocaleSet()->setLocaleString("en_US","o3:WSSTR Create %s.java...","Create %s.java...");
+  WSGIappLocaleSet()->setLocaleString("en_US","p3:WSSTR Create %s/%s.java...","Create %s/%s.java...");
+  WSGIappLocaleSet()->setLocaleString("en_US","o3:WSSTR Create %s.ml...","Create %s.ml...");
+  WSGIappLocaleSet()->setLocaleString("en_US","p3:WSSTR Create %s/%s.ml...","Create %s/%s.ml...");
   WSGIappLocaleSet()->setLocaleString("en_US","q3:WSSTR Overwrite existing file: %s?","Overwrite existing file: %s?");
   WSGIappLocaleSet()->setLocaleString("en_US","r3:WSSTR OK","OK");
   WSGIappLocaleSet()->setLocaleString("en_US","s3:WSSTR Please specify a target instance.","Please specify a target instance.");
@@ -132,7 +158,8 @@
   WSGIappLocaleSet()->setLocaleString("en_US","T3:WSSTR  Debug mode"," Debug mode");
   WSGIappLocaleSet()->setLocaleString("en_US","U3:WSSTR Application type","Application type");
   WSGIappLocaleSet()->setLocaleString("en_US","V3:WSSTR  Generate class library"," Generate class library");
-  WSGIappLocaleSet()->setLocaleString("en_US","W3:WSSTR Compiler","Compiler");
+  WSGIappLocaleSet()->setLocaleString("en_US","W3:WSSTR C Compiler","C Compiler");
+  WSGIappLocaleSet()->setLocaleString("en_US","W3:WSSTR CPP Compiler","CPP Compiler");
   WSGIappLocaleSet()->setLocaleString("en_US","X3:WSSTR Runtime mode,Debug mode,Runtime static mode,Debug static mode","Runtime mode,Debug mode,Runtime static mode,Debug static mode");
   WSGIappLocaleSet()->setLocaleString("en_US","Y3:WSSTR Include path","Include path");
   WSGIappLocaleSet()->setLocaleString("en_US","Z3:WSSTR Compiler flags","Compiler flags");
@@ -246,7 +273,16 @@
   WSGIappLocaleSet()->setLocaleString("en_US","P3:WSSTR Python(DBG)","Python(DBG)");
   WSGIappLocaleSet()->setLocaleString("en_US","P3:WSSTR Perl","Perl");
   WSGIappLocaleSet()->setLocaleString("en_US","P3:WSSTR Perl(DBG)","Perl(DBG)");
+  WSGIappLocaleSet()->setLocaleString("en_US","P3:WSSTR Java","Java") ;
+  WSGIappLocaleSet()->setLocaleString("en_US","P3:WSSTR Java(make)","Java(make)");
   WSGIappLocaleSet()->setLocaleString("en_US","f3:WSSTR Static option","Static option");
   WSGIappLocaleSet()->setLocaleString("en_US","l3:WSSTR Dynamically linked","Dynamically linked");
   WSGIappLocaleSet()->setLocaleString("en_US","l3:WSSTR Statically linked","Statically linked");
-
+  WSGIappLocaleSet()->setLocaleString("en_US","K3:WSSTR Import source files","Import source files");
+  WSGIappLocaleSet()->setLocaleString("en_US","A3:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:","Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:");
+  WSGIappLocaleSet()->setLocaleString("en_US","A3:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:","Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:");
+  WSGIappLocaleSet()->setLocaleString("en_US","A:WSSTR  //please edit here...\n","  //please edit here...\n");
+  WSGIappLocaleSet()->setLocaleString("en_US","B:WSSTR\n\n//Initializing of class...\n","\n\n//Initializing of class...\n");
+  WSGIappLocaleSet()->setLocaleString("en_US","C:WSSTR//##### IMPORTANT: Do not edit from here  #######\n\n","//##### IMPORTANT: Do not edit from here  #######\n\n");
+  WSGIappLocaleSet()->setLocaleString("en_US","D:WSSTR//##### IMPORTANT: Do not edit to here  #######\n\n","//##### IMPORTANT: Do not edit to here  #######\n\n");
+  WSGIappLocaleSet()->setLocaleString("en_US","E:WSSTR();\n\n    // class definitions/////////////////////////////////\n","();\n\n    // class definitions/////////////////////////////////\n");
