@@ -1,3 +1,25 @@
+//
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
+//
+
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","","");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","A3:WSSTR Can not create class: %s.\nPlease check the import library is right.","NX %s ł܂B\nǉR|[lg̐ݒ肪mFĂB");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","B3:WSSTR The name %s is already used by another instance.","IuWFNg %s ̂̂Ɩ̂dȂĂ܂B");
@@ -99,6 +121,10 @@
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","p3:WSSTR Create %s/%s.py...","%s/%s.py\n쐬܂.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","o3:WSSTR Create %s.pl...","%s.pl\n쐬܂.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","p3:WSSTR Create %s/%s.pl...","%s/%s.pl\n쐬܂.");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","o3:WSSTR Create %s.java...","%s.java\n쐬܂.");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","p3:WSSTR Create %s/%s.java...","%s/%s.java\n쐬܂.");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","o3:WSSTR Create %s.ml...","%s.ml\n쐬܂.");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","p3:WSSTR Create %s/%s.ml...","%s/%s.ml\n쐬܂.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","q3:WSSTR Overwrite existing file: %s?","%s\n݂܂Bu܂?");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","r3:WSSTR OK","쐬");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","s3:WSSTR Please specify a target instance.","ΏۃIuWFNgw肵ĂB");
@@ -132,7 +158,8 @@
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","T3:WSSTR  Debug mode"," fobO[h");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","U3:WSSTR Application type","rh^Cv");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","V3:WSSTR  Generate class library"," NXCuɂ");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","W3:WSSTR Compiler","RpC");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","W3:WSSTR C Compiler","C RpC");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","W3:WSSTR CPP Compiler","CPP RpC");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","X3:WSSTR Runtime mode,Debug mode,Runtime static mode,Debug static mode","ʏ탂[h,fobO[h,ʏX^eBbN[h,fobOX^eBbN[h");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","Y3:WSSTR Include path","ʃCN[hpX");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","Z3:WSSTR Compiler flags","ʃRpCtO");
@@ -246,6 +273,16 @@
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","P3:WSSTR Python(DBG)","Python(DBG)");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","P3:WSSTR Perl","Perl");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","P3:WSSTR Perl(DBG)","Perl(DBG)");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","P3:WSSTR Java","Java");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","P3:WSSTR Java(make)","Java(make)");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","f3:WSSTR Static option","N");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","l3:WSSTR Statically linked","X^eBbNN(DLL )");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","l3:WSSTR Dynamically linked","_Ci~bNN(DLL )");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","K3:WSSTR Import source files","ǉ\x83\\[XR[h");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","A3:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:","    :COPY:,\x93\\t:PASTE:,SP,vpeB:PROP:,vV[W:OP:,eݒ:EXPROP:,SP,폜:DEL:");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","A3:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:","    :COPY:,\x93\\t:PASTE:,SP,vpeB:PROP:,vV[W:OP:,eݒ:EXPROP:,SP,Oʂ:FRONT:,wʂ:BACK:,SP,폜:DEL:");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","A:WSSTR  //please edit here...\n","  //Lqĉ\n");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","B:WSSTR\n\n//Initializing of class...\n","\n\n//NX\n");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","C:WSSTR//##### IMPORTANT: Do not edit from here  #######\n\n","//##### IMPORTANT  ȉAҏWĂ͂Ȃ܂  #######\n\n");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","D:WSSTR//##### IMPORTANT: Do not edit to here  #######\n\n","//##### IMPORTANT  ܂ł͕ҏWĂ͂Ȃ܂     #########\n");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","E:WSSTR();\n\n    // class definitions/////////////////////////////////\n","();\n\n    //hEp錾 ////////////////////////////////////\n");
