@@ -1,3 +1,25 @@
+//
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
+//
+
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","","");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","A3:WSSTR Can not create class: %s.\nPlease check the import library is right.","クラス %s が生成できません。\n追加コンポーネント等の設定が正しいか確認してください。");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","B3:WSSTR The name %s is already used by another instance.","オブジェクト %s が他のものと名称が重なっています。");
@@ -99,6 +121,10 @@
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","p3:WSSTR Create %s/%s.py...","%s/%s.py\nを作成します.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","o3:WSSTR Create %s.pl...","%s.pl\nを作成します.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","p3:WSSTR Create %s/%s.pl...","%s/%s.pl\nを作成します.");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","o3:WSSTR Create %s.java...","%s.java\nを作成します.");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","p3:WSSTR Create %s/%s.java...","%s/%s.java\nを作成します.");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","o3:WSSTR Create %s.ml...","%s.ml\nを作成します.");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","p3:WSSTR Create %s/%s.ml...","%s/%s.ml\nを作成します.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","q3:WSSTR Overwrite existing file: %s?","%s\nが存在します。置き換えますか?");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","r3:WSSTR OK","作成");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","s3:WSSTR Please specify a target instance.","対象オブジェクトを指定してください。");
@@ -132,7 +158,8 @@
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","T3:WSSTR  Debug mode"," デバッグモード");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","U3:WSSTR Application type","ビルドタイプ");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","V3:WSSTR  Generate class library"," クラスライブラリにする");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","W3:WSSTR Compiler","コンパイラ");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","W3:WSSTR C Compiler","C コンパイラ");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","W3:WSSTR CPP Compiler","CPP コンパイラ");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","X3:WSSTR Runtime mode,Debug mode,Runtime static mode,Debug static mode","通常モード,デバッグモード,通常スタティックモード,デバッグスタティックモード");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","Y3:WSSTR Include path","個別インクルードパス");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","Z3:WSSTR Compiler flags","個別コンパイルフラグ");
@@ -246,6 +273,16 @@
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","P3:WSSTR Python(DBG)","Python(DBG)");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","P3:WSSTR Perl","Perl");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","P3:WSSTR Perl(DBG)","Perl(DBG)");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","P3:WSSTR Java","Java");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","P3:WSSTR Java(make)","Java(make)");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","f3:WSSTR Static option","リンク方式");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","l3:WSSTR Statically linked","スタティックリンク(DLL 無し)");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","l3:WSSTR Dynamically linked","ダイナミックリンク(DLL あり)");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","K3:WSSTR Import source files","追加ソースコード");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","A3:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:","複写    :COPY:,貼り付け:PASTE:,SP,プロパティ:PROP:,プロシージャ:OP:,各種設定:EXPROP:,SP,削除:DEL:");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","A3:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:","複写    :COPY:,貼り付け:PASTE:,SP,プロパティ:PROP:,プロシージャ:OP:,各種設定:EXPROP:,SP,前面へ:FRONT:,背面へ:BACK:,SP,削除:DEL:");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","A:WSSTR  //please edit here...\n","  //何か処理を記述して下さい\n");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","B:WSSTR\n\n//Initializing of class...\n","\n\n//クラス初期化\n");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","C:WSSTR//##### IMPORTANT: Do not edit from here  #######\n\n","//##### IMPORTANT  ここから以下、編集してはなりません  #######\n\n");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","D:WSSTR//##### IMPORTANT: Do not edit to here  #######\n\n","//##### IMPORTANT  ここまでは編集してはなりません     #########\n");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","E:WSSTR();\n\n    // class definitions/////////////////////////////////\n","();\n\n    //派生・継承宣言 ////////////////////////////////////\n");
