@@ -1,3 +1,25 @@
+//
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
+//
+
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","","");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","A3:WSSTR Can not create class: %s.\nPlease check the import library is right.","饹 %s Ǥޤ\nɲåݡ꤬ͥǧƤ");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","B3:WSSTR The name %s is already used by another instance.","֥ %s ¾ΤΤ̾ΤŤʤäƤޤ");
@@ -99,6 +121,10 @@
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","p3:WSSTR Create %s/%s.py...","%s/%s.py\nޤ.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","o3:WSSTR Create %s.pl...","%s.pl\nޤ.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","p3:WSSTR Create %s/%s.pl...","%s/%s.pl\nޤ.");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","o3:WSSTR Create %s.java...","%s.java\nޤ.");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","p3:WSSTR Create %s/%s.java...","%s/%s.java\nޤ.");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","o3:WSSTR Create %s.ml...","%s.ml\nޤ.");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","p3:WSSTR Create %s/%s.ml...","%s/%s.ml\nޤ.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","q3:WSSTR Overwrite existing file: %s?","%s\n¸ߤޤ֤ޤ?");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","r3:WSSTR OK","");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","s3:WSSTR Please specify a target instance.","оݥ֥ȤꤷƤ");
@@ -132,7 +158,8 @@
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","T3:WSSTR  Debug mode"," ǥХå⡼");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","U3:WSSTR Application type","ӥɥ");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","V3:WSSTR  Generate class library"," 饹饤֥ˤ");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","W3:WSSTR Compiler","ѥ");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","W3:WSSTR C Compiler","C ѥ");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","W3:WSSTR CPP Compiler","CPP ѥ");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","X3:WSSTR Runtime mode,Debug mode,Runtime static mode,Debug static mode","̾⡼,ǥХå⡼,̾凉ƥå⡼,ǥХåƥå⡼");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","Y3:WSSTR Include path","̥󥯥롼ɥѥ");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","Z3:WSSTR Compiler flags","̥ѥե饰");
@@ -246,6 +273,16 @@
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","P3:WSSTR Python(DBG)","Python(DBG)");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","P3:WSSTR Perl","Perl");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","P3:WSSTR Perl(DBG)","Perl(DBG)");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","P3:WSSTR Java","Java");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","P3:WSSTR Java(make)","Java(make)");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","f3:WSSTR Static option","");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","l3:WSSTR Statically linked","ƥå(DLL ̵)");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","l3:WSSTR Dynamically linked","ʥߥå(DLL )");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","K3:WSSTR Import source files","ɲå");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","A3:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:","ʣ    :COPY:,Žդ:PASTE:,SP,ץѥƥ:PROP:,ץ:OP:,Ƽ:EXPROP:,SP,:DEL:");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","A3:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:","ʣ    :COPY:,Žդ:PASTE:,SP,ץѥƥ:PROP:,ץ:OP:,Ƽ:EXPROP:,SP,̤:FRONT:,̤:BACK:,SP,:DEL:");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","A:WSSTR  //please edit here...\n","  //򵭽ҤƲ\n");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","B:WSSTR\n\n//Initializing of class...\n","\n\n//饹\n");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","C:WSSTR//##### IMPORTANT: Do not edit from here  #######\n\n","//##### IMPORTANT  ʲԽƤϤʤޤ  #######\n\n");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","D:WSSTR//##### IMPORTANT: Do not edit to here  #######\n\n","//##### IMPORTANT  ޤǤԽƤϤʤޤ     #########\n");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","E:WSSTR();\n\n    // class definitions/////////////////////////////////\n","();\n\n    //Ѿ ////////////////////////////////////\n");
