@@ -1,3 +1,25 @@
+//
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
+//
+
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","","");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","A3:WSSTR Can not create class: %s.\nPlease check the import library is right.","Ŭ  %s  մϴ.\n߰Ʈ  ´ Ȯּ.");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","B3:WSSTR The name %s is already used by another instance.","Ʈ  %s   ܰͰ ̸ ߺϰֽϴ.");
@@ -99,6 +121,10 @@
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","p3:WSSTR Create %s/%s.py...","%s/%s.py\n ۼϰڽϴ.");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","o3:WSSTR Create %s.pl...","%s.pl\n ۼϰڽϴ.");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","p3:WSSTR Create %s/%s.pl...","%s/%s.pl\n ۼϰڽϴ.");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","o3:WSSTR Create %s.java...","%s.java\n ۼϰڽϴ.");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","p3:WSSTR Create %s/%s.java...","%s/%s.java\n ۼϰڽϴ.");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","o3:WSSTR Create %s.ml...","%s.ml\n ۼϰڽϴ.");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","p3:WSSTR Create %s/%s.ml...","%s/%s.ml\n ۼϰڽϴ.");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","q3:WSSTR Overwrite existing file: %s?","%s\n ֽϴ.մϱ?");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","r3:WSSTR OK","ۼ");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","s3:WSSTR Please specify a target instance.","Ʈ  ּ.");
@@ -132,7 +158,8 @@
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","T3:WSSTR  Debug mode"," ׸");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","U3:WSSTR Application type","Application type");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","V3:WSSTR  Generate class library"," Ŭ̺귯 Ѵ.");
-  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","W3:WSSTR Compiler","Ϸ");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","W3:WSSTR C Compiler","C Ϸ");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","W3:WSSTR CPP Compiler","CPP Ϸ");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","X3:WSSTR Runtime mode,Debug mode,Runtime static mode,Debug static mode"," , , ƽ , ƽ ");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","Y3:WSSTR Include path","Ŭн");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","Z3:WSSTR Compiler flags","÷");
@@ -246,6 +273,16 @@
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","P3:WSSTR Python(DBG)","Python(DBG)");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","P3:WSSTR Perl","Perl");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","P3:WSSTR Perl(DBG)","Perl(DBG)");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","P3:WSSTR Java","Java");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","P3:WSSTR Java(make)","Java(make)");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","f3:WSSTR Static option","ũ");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","l3:WSSTR Statically linked","ƽ ũ(DLL )");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","l3:WSSTR Dynamically linked","̳ ũ(DLL ʿ)");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","K3:WSSTR Import source files","ҽ  ϱ");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","A3:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:","    :COPY:,ٿֱ:PASTE:,SP,Ƽ:PROP:,ν:OP:,:EXPROP:,SP,:DEL:");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","A3:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:","    :COPY:,ٿֱ:PASTE:,SP,Ƽ:PROP:,ν:OP:,:EXPROP:,SP,ո鿡:FRONT:,ڸ鿡:BACK:,SP,:DEL:");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","A:WSSTR  //please edit here...\n","  //ó ּ.\n");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","B:WSSTR\n\n//Initializing of class...\n","\n\n//Ŭʱȭ\n");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","C:WSSTR//##### IMPORTANT: Do not edit from here  #######\n\n","//##### IMPORTANT  ⼭ʹ ϸ ȵ˴ϴ.  #######\n\n");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","D:WSSTR//##### IMPORTANT: Do not edit to here  #######\n\n","//##### IMPORTANT   ϸ ȵ˴ϴ.     #########\n");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","E:WSSTR();\n\n    // class definitions/////////////////////////////////\n","();\n\n    //Ļ / ¼ ////////////////////////////////////\n");
