@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <advance/WSCclassPropEditor.h>
@@ -115,7 +111,6 @@ WSCclassPropEditor::WSCclassPropEditor(WSCbase* base,char* objname):
   WSMFpropertyCreateEnd
 }
 
-//ץѥƥѥץκ
 WSMFproperty( WSCclassPropEditor, WSNclient, void*,   _voidp_client,   NULL);
 
 void WSCclassPropEditor::setWorkWSNclient(void* data){
@@ -295,7 +290,7 @@ void WSCclassPropEditor::_dialog_set_work(WSCbase* obj){
 
   op_edit->_dialog1->setVisible(False);
 
-  if (no == -1 && op_edit->_client != NULL){ //ɲ
+  if (no == -1 && op_edit->_client != NULL){ //ɲ...
     WSClistData* property_list = (WSClistData*)op_edit->_client->getUserData(WS_PROP_LIST);
     if (property_list == NULL){
       property_list = new WSClistData();
