@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <advance/WSCclassRMethodEditor.h>
@@ -122,7 +118,6 @@ WSCclassRMethodEditor::WSCclassRMethodEditor(WSCbase* base,char* objname):
   WSMFpropertyCreateEnd
 }
 
-//ץѥƥѥץκ
 WSMFproperty( WSCclassRMethodEditor, WSNclient, void*,   _voidp_client,   NULL);
 
 void WSCclassRMethodEditor::setWorkWSNclient(void* data){
@@ -1141,7 +1136,9 @@ void WSCclassRMethodEditor::setEditClient(WSCbase* client){
       char* rbcname = (char*)client->getUserData(WS_RCLASS_BNAME);
       if (rbcname == NULL){
         WSCstring tmp;
-        tmp = client->getClassName();
+        WSCbase* b = WSGFgetClassBase(client);
+        tmp = b->getClassName();
+        WSGFresetClassBase(client);
         if (!strncmp((char*)tmp,"WSC",3)){
           tmp.replaceString("WSC","WSCR",1);
         }else{
