@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCclassInformation.h>
@@ -86,6 +82,8 @@ WSCinspector::WSCinspector(WSCbase* base,char* objname):
   _pos_bk = WSGFstrdup("");
   _pform = NULL;
   _value = (0x2 | 0x4);
+  _get_ext_files_proc = NULL;
+
   WSMFpropertyCreateStart
     WSMFparentCheckVerSrc(WSCinspector);
 //    WSMFpropertyCreate(WSNlabelString,   char*, _label_string,WSSlabelString);
@@ -166,7 +164,7 @@ void WSCinspector::setWorkWSNbarValue(char* data){
   //_pos_bk will be freed end of setProperty(WSNbarValue)
   // but _pos will not be freed.
   // so free _pos and _pos_bk
-  delete _pos;
+  delete[] _pos;
   if (tmp.getWords(",") < 4){
     long val = atol(tmp2);
     char buffer[32];
@@ -245,7 +243,7 @@ void WSCinspector::getWorkWSNbarValue(char** data){
   }else{
     ret << "," << tmp.getWord(1,",");
   }
-  delete _pos_bk;
+  delete[] _pos_bk;
   _pos_bk = WSGFstrdup(ret.getString());
   *data = _pos_bk;
 }
@@ -606,6 +604,7 @@ long WSCinspector::initialize(){
   _file_list->setPropertyV(WSNuseIcon,(WSCbool)True);
   _file_list->setPropertyV(WSNreverseSelect,(WSCbool)True);
   _file_list->setProperty(WSNpixmapStyle,WS_DYNAMIC_PIXMAP);
+  _file_list->setProperty(WSNreverseSelect,False);
   _file_list->setPropertyV(WSNuserValue,(long)1);
   _file_list->setVisible(True);
   _file_list->setUserData("inspector",this);
@@ -1602,6 +1601,12 @@ void WSCinspector::_lb_press_work(WSCbase* label){
   for(i=0; i<num; i++){
     WSCbase* lb = list->getLabel(i);
     WSCbase* client = (WSCbase*)lb->getUserData("client");
+    if (label == lb){
+      if (list->getSelectedPos() != i){
+        list->setSelectPos(i);
+        _lb_activate_work(label);
+      }
+    }
     if (client != NULL && WSGIappObjectList()->existInstance(client)){
       client->setUserData(WS_LIST_TREE_OPEN,lb->getUserData(WS_LIST_TREE_OPEN));
 //printf("WSCinspector::_lb_press_work %s open=%d\n",client->getInstanceName(),lb->getUserData(WS_LIST_TREE_OPEN));
@@ -1710,7 +1715,7 @@ void WSCinspector::updateList(){
     WSCstring fname(item->getInstanceName());
     fname << ".win";
     _file_list->addItem(fname);
-    _file_list->getLabel(-1)->setPropertyV(WSNlabelPixmap,"$(WSDIR)/sys/pixmaps/bi17.xpm");
+    _file_list->getLabel(-1)->setPropertyV(WSNlabelPixmap,"$(WSDIR)/sys/pixmaps/bi16.xpm");
     WSCbase* label = _file_list->getLabel(-1);
     label->setUserData(WS_LIST_TREE_OPEN,item->getUserData("file-list-tree-open"));
     _set_file_list_work(item);
@@ -1725,7 +1730,37 @@ void WSCinspector::updateList(){
 
     label->setUserData("client",(void*)item);
   }
+  if (_get_ext_files_proc != NULL){
+    char* files = _get_ext_files_proc();
+    WSCstring ext_files(files);
+    ext_files.delHeadSpace();
+    ext_files.delTailSpace();
+    if (strcmp((char*)ext_files,"")){
+      _file_list->addItem("External Files:");
+      _file_list->getLabel(-1)->setPropertyV(WSNlabelPixmap,"$(WSDIR)/sys/pixmaps/bi16.xpm");
 
+
+      int words = ext_files.getWords();
+      int i;
+      WSCstring file;
+      for(i=0; i< words; i++){
+        file << ext_files.getWord(i);
+        if (((char*)file)[strlen(file)-1] == '\\'){
+          ((char*)file)[strlen(file)-1] = ' ';
+          continue;
+        }
+        _file_list->addItem(file);
+        _file_list->getLabel(-1)->setPropertyV(WSNlabelPixmap,"$(WSDIR)/sys/pixmaps/bi17.xpm");
+        _file_list->setItemValue(-1,WS_INDENT_LEVEL,1);
+        WSCbase* label = _file_list->getLabel(-1);
+        WSCprocedure* op = new WSCprocedure("label_selected",WSEV_ACTIVATE);
+        op->setFunction(_flb_activate_work,"WSCinspector::_flb_activate_work");
+        label->addProcedure(op);
+        label->setProperty(WSNuserString,file);
+        file.clear();
+      }
+    }
+  }
   _list->updateList();
   _list->redraw();
   _list2->updateList();
@@ -1739,7 +1774,8 @@ void WSCinspector::_file_list_vis_work(WSCbase* obj){
     return;
   }
   WSCinspector* ins = (WSCinspector*)obj->getUserData("inspector");
-
+  ins->updateList();
+#if 0
   ins->_file_list->delAll();
   long i;
   long num = ins->_target_list->getNum();
@@ -1769,6 +1805,7 @@ void WSCinspector::_file_list_vis_work(WSCbase* obj){
 
   ins->_file_list->updateList();
   ins->_file_list->redraw();
+#endif
 }
 
 void WSCinspector::_set_file_list_work(WSCbase* item){
@@ -1804,6 +1841,12 @@ void WSCinspector::_set_file_list_work(WSCbase* item){
       }else
       if (lang == WS_LANG_PERL){
         srcname << proc->getFunctionName() << ".pl";
+      }else
+      if (lang == WS_LANG_JAVA){
+        srcname << proc->getFunctionName() << ".java";
+      }else
+      if (lang == WS_LANG_OCAML){
+        srcname << proc->getFunctionName() << ".ml";
       }
 
       WSCstring fname(srcname);
@@ -2042,6 +2085,9 @@ long WSCinspector::draw(){
 void WSCinspector::setLoadProjectHandler(void(*fn)(WSCinspector*,char*)){
   _load_project_fn = fn;
 }
+void WSCinspector::setExternalSourceFilesHandler(void* fn){
+  _get_ext_files_proc = (char*(*)())fn;
+}
 void WSCinspector::setSelectIndexTab(long no){
   _area2->setProperty(WSNvalue,no);
   setFormVisible(3,True);
