@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <advance/WSCprocedureEditor.h>
@@ -366,14 +362,14 @@ void WSCprocedureEditor::_dialog_set_work(WSCbase* obj){
     return;
   }
 
-  if (WSGFcheckNameString(pgname.getString()) != True){
+  if (WSGFcheckNameString(pgname.getString()) != False){
     long ret = WSGFexistName( pgname.getString(),NULL );
     if (ret == 0){
        //OK.
-    }else if (ret == WS_PROCEDURE_NAME){
-      op_edit->_msg->setPropertyV(WSNlabelString,WSGFgettext("k3:WSSTR Invalid procedure name.\nAlready used by another procedure name."));
-      op_edit->_msg->setVisible(True);
-      return;
+//    }else if (ret == WS_PROCEDURE_NAME){
+//      op_edit->_msg->setPropertyV(WSNlabelString,WSGFgettext("k3:WSSTR Invalid procedure name.\nAlready used by another procedure name."));
+//      op_edit->_msg->setVisible(True);
+//      return;
     }else if (ret == WS_OBJECT_NAME){
       op_edit->_msg->setPropertyV(WSNlabelString,WSGFgettext("l3:WSSTR Invalid procedure name.\nAlready used by another instance name."));
       op_edit->_msg->setVisible(True);
@@ -471,6 +467,32 @@ void WSCprocedureEditor::_dialog_set_work(WSCbase* obj){
         }else{
           sprintf(buffer,WSGFgettext("p3:WSSTR Create %s/%s.pl..."),op_edit->getSrcPath(),fname.getString());
         }
+      }else
+      if (lang == WS_LANG_JAVA){
+        if (!strcmp(op_edit->getSrcPath(),"")){
+          sprintf(buffer,"%s.java",fname.getString());
+        }else{
+          sprintf(buffer,"%s/%s.java",op_edit->getSrcPath(),fname.getString());
+        }
+        op_edit->_file_name.setString(buffer);
+        if (!strcmp(op_edit->getSrcPath(),"")){
+          sprintf(buffer,WSGFgettext("o3:WSSTR Create %s.java..."),fname.getString());
+        }else{
+          sprintf(buffer,WSGFgettext("p3:WSSTR Create %s/%s.java..."),op_edit->getSrcPath(),fname.getString());
+        }
+      }else
+      if (lang == WS_LANG_OCAML){
+        if (!strcmp(op_edit->getSrcPath(),"")){
+          sprintf(buffer,"%s.ml",fname.getString());
+        }else{
+          sprintf(buffer,"%s/%s.ml",op_edit->getSrcPath(),fname.getString());
+        }
+        op_edit->_file_name.setString(buffer);
+        if (!strcmp(op_edit->getSrcPath(),"")){
+          sprintf(buffer,WSGFgettext("o3:WSSTR Create %s.ml..."),fname.getString());
+        }else{
+          sprintf(buffer,WSGFgettext("p3:WSSTR Create %s/%s.ml..."),op_edit->getSrcPath(),fname.getString());
+        }
       }
 
       WSCbool fl = WSGFcheckExistFile(op_edit->_file_name.getString());
@@ -532,6 +554,7 @@ void WSCprocedureEditor::_dialog_src_work(WSCbase* obj){
   fname.replaceString(".rb","",0);
   fname.replaceString(".py","",0);
   fname.replaceString(".pl","",0);
+  fname.replaceString(".java","",0);
 
   WSCbool ok = WSGFcheckNameString(fname.getString());
   if (ok == False){
@@ -605,14 +628,41 @@ void WSCprocedureEditor::_dialog_src_work(WSCbase* obj){
       sprintf(buffer,WSGFgettext("p3:WSSTR Create %s/%s.pl..."),
                      op_edit->getSrcPath(),fname.getString());
     }
-
+  }else
+  if (lang == WS_LANG_JAVA){
+    if (!strcmp(op_edit->getSrcPath(),"")){
+      sprintf(buffer,"%s.java",fname.getString());
+    }else{
+      sprintf(buffer,"%s/%s.java",op_edit->getSrcPath(),fname.getString());
+    }
+    op_edit->_file_name.setString(buffer);
+    if (!strcmp(op_edit->getSrcPath(),"")){
+      sprintf(buffer,WSGFgettext("o3:WSSTR Create %s.java..."),
+                     fname.getString());
+    }else{
+      sprintf(buffer,WSGFgettext("p3:WSSTR Create %s/%s.java..."),
+                     op_edit->getSrcPath(),fname.getString());
+    }
+  }else
+  if (lang == WS_LANG_OCAML){
+    if (!strcmp(op_edit->getSrcPath(),"")){
+      sprintf(buffer,"%s.ml",fname.getString());
+    }else{
+      sprintf(buffer,"%s/%s.ml",op_edit->getSrcPath(),fname.getString());
+    }
+    op_edit->_file_name.setString(buffer);
+    if (!strcmp(op_edit->getSrcPath(),"")){
+      sprintf(buffer,WSGFgettext("o3:WSSTR Create %s.ml..."),
+                     fname.getString());
+    }else{
+      sprintf(buffer,WSGFgettext("p3:WSSTR Create %s/%s.ml..."),
+                     op_edit->getSrcPath(),fname.getString());
+    }
   }
 
 
 
 
-
-
   if ( (long)win->getUserData(WS_WIN_CLASS) != 0){ 
     WSCstring fn;
     fn.setString( op_edit->_if3->getProperty(WSNlabelString) );
@@ -623,6 +673,7 @@ void WSCprocedureEditor::_dialog_src_work(WSCbase* obj){
     fn.replaceString(".rb","",0);
     fn.replaceString(".py","",0);
     fn.replaceString(".pl","",0);
+    fn.replaceString(".java","",0);
     long lang = WSGFgetLanguage();
     if (lang == WS_LANG_CPP){
       WSGFclassProcedureCreate(&op_edit->_file_data,fn.getString(),op_edit->_client->getClassName(),win->getInstanceName());
@@ -632,20 +683,51 @@ void WSCprocedureEditor::_dialog_src_work(WSCbase* obj){
     if (lang == WS_LANG_PYTHON){
     }else
     if (lang == WS_LANG_PERL){
+    }else
+    if (lang == WS_LANG_JAVA){
+    }else
+    if (lang == WS_LANG_OCAML){
     }
   }else{
     long lang = WSGFgetLanguage();
+    long mapping = WSGFgetMapping();
     if (lang == WS_LANG_CPP){
       WSGFopCreate(&op_edit->_file_data,fname.getString(),op_edit->_client->getClassName());
     }else
     if (lang == WS_LANG_RUBY){
-      WSGFopCreateRb(&op_edit->_file_data,fname.getString(),op_edit->_client->getClassName());
+      if (mapping == WS_MAPPING_MPFC){
+        WSGFopCreateRb(&op_edit->_file_data,fname.getString(),op_edit->_client->getClassName());
+      }else{
+        WSGFopCreateMwtRb(&op_edit->_file_data,fname.getString(),op_edit->_client->getClassName());
+      }
     }else
     if (lang == WS_LANG_PYTHON){
-      WSGFopCreatePy(&op_edit->_file_data,fname.getString(),op_edit->_client->getClassName());
+      if (mapping == WS_MAPPING_MPFC){
+        WSGFopCreatePy(&op_edit->_file_data,fname.getString(),op_edit->_client->getClassName());
+      }else{
+        WSGFopCreateMwtPy(&op_edit->_file_data,fname.getString(),op_edit->_client->getClassName());
+      }
     }else
     if (lang == WS_LANG_PERL){
-      WSGFopCreatePerl(&op_edit->_file_data,fname.getString(),op_edit->_client->getClassName());
+      if (mapping == WS_MAPPING_MPFC){
+        WSGFopCreatePerl(&op_edit->_file_data,fname.getString(),op_edit->_client->getClassName());
+      }else{
+        WSGFopCreateMwtPerl(&op_edit->_file_data,fname.getString(),op_edit->_client->getClassName());
+      }
+    }else
+    if (lang == WS_LANG_JAVA){
+      if (mapping == WS_MAPPING_MPFC){
+        WSGFopCreateJava(&op_edit->_file_data,fname.getString(),op_edit->_client->getClassName());
+      }else{
+        WSGFopCreateMwtJava(&op_edit->_file_data,fname.getString(),op_edit->_client->getClassName());
+      }
+    }else
+    if (lang == WS_LANG_OCAML){
+      if (mapping == WS_MAPPING_MPFC){
+        WSGFopCreateOcaml(&op_edit->_file_data,fname.getString(),op_edit->_client->getClassName());
+      }else{
+        WSGFopCreateMwtOcaml(&op_edit->_file_data,fname.getString(),op_edit->_client->getClassName());
+      }
     }
   }
   op_edit->_msg->setPropertyV(WSNlabelPixmap,WS_DIALOG_PIX1);
@@ -779,6 +861,22 @@ void WSCprocedureEditor::_ibtn3_work(WSCbase* obj){
           fname = win->getInstanceName();
           fname << "_" << ac->getFunctionName() << ".pl";
         }
+      }else
+      if (lang == WS_LANG_JAVA){
+        if (win != NULL && (long)win->getUserData(WS_WIN_CLASS) == 0){
+          fname << ac->getFunctionName() << ".java";
+        }else{
+          fname = win->getInstanceName();
+          fname << "_" << ac->getFunctionName() << ".java";
+        }
+      }else
+      if (lang == WS_LANG_OCAML){
+        if (win != NULL && (long)win->getUserData(WS_WIN_CLASS) == 0){
+          fname << ac->getFunctionName() << ".ml";
+        }else{
+          fname = win->getInstanceName();
+          fname << "_" << ac->getFunctionName() << ".ml";
+        }
       }
 
 //printf("WSCprocedureEditor:: pgname=#%s#\n",(char*)fname);
