@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <advance/WSCprocessForm.h>
@@ -33,7 +29,9 @@
 #include <WSDmessage.h>
 #include <WSDfileSystem.h>
 #include <WSCfontSet.h>
+#ifndef _WSWINCE
 #include <signal.h>
+#endif
 #include <WSCtextField.h>
 #include <WSCbaseList.h>
 #include <WSDexecute.h>
@@ -90,6 +88,7 @@ WSCprocessForm::~WSCprocessForm(){
   delete _dir_name;
 }
 long WSCprocessForm::initialize(){
+  WSCtextField::initialize();
 #if 0
   _m1 = WSDmessage::getNewInstance();
   _m1->setupMessage("WL-SBYTES",_cb1,(void*)this);
@@ -101,7 +100,6 @@ long WSCprocessForm::initialize(){
   _m4->setupMessage("WL-END",_cb4,(void*)this);
 #endif
 
-  WSCtextField::initialize();
 
   WSCprocedure* op1 = new WSCprocedure("sop4",WSEV_KEY_HOOK);
   op1->setInternal(True);
@@ -126,6 +124,7 @@ void WSCprocessForm::_textf_work(WSCbase* textf){
 void WSCprocessForm::_receive_proc(WSDexecute* exec){
   WSCprocessForm* _this = (WSCprocessForm*)exec->getUserData(); 
   char data[2048];
+  memset(data,0,sizeof(data));
   char* ret = exec->gets(data,sizeof(data));
   WSCstring tmp;
   tmp.setString(data,WSGIappLocaleSet()->getSystemLocaleEncoding());
