@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <advance/WSCpropertyEditor.h>
@@ -53,6 +49,7 @@
 #include <WSCtextField.h>
 #include <WSCvkslabel.h>
 #include <WSDtimer.h>
+#include <WSCundoManager.h>
 
 #define WS_WARNING_STR  WSGFgettext("L3:WSSTR Warning")
 
@@ -334,7 +331,7 @@ long WSCpropertyEditor::initialize(){
   WSCscrForm::initialize();
   setProperty(WSNscrollHeight,WS_PELB_T);
 
-  //ٹ
+  // Warrning dialog...
   _msg = new WSCmessageDialog(NULL,(WSCstring)"pe-md" + getInstanceName());
   _msg->initialize();
   _msg->setPropertyV(WSNx,(short)300);
@@ -627,9 +624,10 @@ void WSCpropertyEditor::updateProperties(){
         }
         label->setPropertyV(WSNuserValue,(long)tmp);
 
-        char* bhtitle = (char*)label->getUserData("BH-TITLE");
+        char* bhtitle = (char*)label->getUserData("WSBH-TITLE");
         if (bhtitle != NULL){
-          delete bhtitle;
+          delete[] bhtitle;
+          label->setUserData("WSBH-TITLE",NULL);
         }
         char propname[128];
         sprintf(propname,WSGFgettext("n3:WSSTR Name:WSN%s  Type:%s"),
@@ -829,7 +827,7 @@ void WSCpropertyEditor::updateProperties(){
           while( labels[n] != NULL ){
             items << WSGFgettext(labels[n]) << ":";
             char addr[32];
-            WSGFultoa((WSCulong)pdata[n],addr);
+            WSGFltoa((WSCulong)pdata[n],addr);
             items << addr << ":";
 
             if (n == 0){
@@ -1027,6 +1025,17 @@ void WSCpropertyEditor::_check_name(char* name){
      _exec_change_name_handler(this);
   }
 }
+void WSCpropertyEditor::_set_undo_proc(){
+#ifndef NO_MWT_FUNCTION
+#ifndef NO_INDEX_VARIANT_DATA
+  if (_client != NULL){
+    WSCpropUndoProc* new_undo = new WSCpropUndoProc();
+    new_undo->setTarget(_client);
+    WSGIappUndoManager()->addUndoProc(new_undo);
+  }
+#endif
+#endif
+}
 void WSCpropertyEditor::setPropertyDataToClient(){
   if (_update_sequence != False){
     return;
@@ -1045,6 +1054,7 @@ void WSCpropertyEditor::setPropertyDataToClient(){
   WSGFmarkChanged(client);
   client->clear();
 
+
   WSCproperty* prop;
   WSCproperty* tmp;
   WSClistData property_list1;
@@ -1063,7 +1073,7 @@ void WSCpropertyEditor::setPropertyDataToClient(){
     }
 
     if (client->existProperty(tmp->getPropObjName()) == False){
-      continue; //ƤΤäʤ
+      continue; //ƤΤäʤ...
     }
 
     if (fl != False){
@@ -1109,30 +1119,58 @@ void WSCpropertyEditor::setPropertyDataToClient(){
             WSCstring val(ifd->getProperty(WSNlabelString));
             WSCstring iname( val );
             iname.delTailSpace();
+            WSCstring original_val = client->getProperty(pname);
+            if (strcmp((char*)original_val,(char*)iname)){
+              _set_undo_proc();
+            }
             client->setProperty(pname,iname.getString() );
           }else if ( tmp->isColorPropObj() != False){
             WSCstring cname(ifd->getProperty(WSNlabelString));
             cname.delTailSpace();
             if (atoi(cname.getString()) != 0){
-              client->setPropertyV(pname,(short)atoi(cname.getString()) );
+              short original_val;
+              client->getPropertyV(pname,&original_val);
+              short val = (short)atoi(cname.getString());
+              if (original_val != val){
+                _set_undo_proc();
+              }
+//              client->setPropertyV(pname,(short)atoi(cname.getString()) );
+              client->setPropertyV(pname,val);
             }else{
+              WSCstring original_val = client->getProperty(pname);
+              if (strcmp((char*)original_val,(char*)cname.getString())){
+                _set_undo_proc();
+              }
               client->setProperty(pname,cname.getString() );
             }
           }else if (!strcmp(pname,"name")){
             _check_name(ifd->getProperty(WSNlabelString));
           }else{
             if (tmp->getPropObjType() == WSTcharptr){
+              WSCstring val = ifd->getProperty(WSNlabelString);
+              WSCstring original_val = client->getProperty(pname);
+              if (strcmp((char*)original_val,(char*)val)){
+                _set_undo_proc();
+              }
               client->setProperty(pname,ifd->getProperty(WSNlabelString));
             }else{
               WSCstring prv(ifd->getProperty(WSNlabelString));
               prv.delTailSpace();
+              WSCstring original_val = client->getProperty(pname);
+              if (strcmp((char*)original_val,(char*)prv)){
+                _set_undo_proc();
+              }
               client->setProperty(pname,prv.getString());
             }
           }
         }else{ 
           long addr;
           opt->getPropertyV(WSNvalue,&addr);
-//printf("i=%d prop=%s opt=%s  addr=0x%x\n",i,tmp->getPropObjName(),(char*)opt->getProperty(WSNmenuItems),addr);
+//printf("i=%d prop=%s opt=%s  addr=0x%x %d\n",i,tmp->getPropObjName(),(char*)opt->getProperty(WSNmenuItems),addr,addr);
+          WSCstring original_val = client->getProperty(tmp->getPropObjName());
+          if (strcmp((char*)original_val,(char*)addr)){
+            _set_undo_proc();
+          }
           client->setProperty(tmp->getPropObjName(), (char*)addr);
           //bug fix for no calling WSCwindow::setWorkWSNvis()
           if (!strcmp(tmp->getPropObjName(),"vis")){
@@ -1353,7 +1391,7 @@ void WSCpropertyEditor::_popup_prop_work(WSCbase* lb){
     while( labels[n] != NULL ){
       items << WSGFgettext(labels[n]) << ":";
       char addr[32];
-      WSGFultoa((WSCulong)pdata[n],addr);
+      WSGFltoa((long)pdata[n],addr);
       items << addr << ":";
       if (n == 0){
         hit_data = (long)pdata[n];
