@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCclassInformation.h>
@@ -299,7 +295,7 @@ void WSCwindowEditor::_stg1_work(WSCbase* ptr){
       obj->_file_name->setPropertyV(WSNlabelString,(char*)str2);
       char* tmp = (char*)win->getUserData(WS_FILENAME);
       if (tmp != NULL){
-        delete tmp;
+        delete[] tmp;
         win->setUserData(WS_FILENAME,NULL);
       }
       if (strcmp((char*)str2,"")){
@@ -342,7 +338,7 @@ void WSCwindowEditor::_vifi_work(WSCbase* ptr){
   }
   char* cicon = (char*)obj->_client->getUserData(WS_CLASS_ICON);
   if (cicon != NULL){
-    delete cicon;
+    delete[] cicon;
     obj->_client->setUserData(WS_CLASS_ICON,(void*)0);
   }
   if (strcmp("",obj->_class_icon->getProperty(WSNlabelString))){
@@ -351,7 +347,7 @@ void WSCwindowEditor::_vifi_work(WSCbase* ptr){
 
   char* ccom = (char*)obj->_client->getUserData(WS_CLASS_COMMENT);
   if (ccom != NULL){
-    delete ccom;
+    delete[] ccom;
     obj->_client->setUserData(WS_CLASS_COMMENT,(void*)0);
   }
   if (strcmp("",obj->_class_cmmt->getProperty(WSNlabelString))){
@@ -360,7 +356,7 @@ void WSCwindowEditor::_vifi_work(WSCbase* ptr){
 
   char* cbase = (char*)obj->_client->getUserData(WS_WIN_CLASS_BASE);
   if (cbase != NULL){
-    delete cbase;
+    delete[] cbase;
     obj->_client->setUserData(WS_WIN_CLASS_BASE,(void*)0);
   }
   if (strcmp("",obj->_class_base->getProperty(WSNlabelString))){
@@ -457,7 +453,7 @@ void WSCwindowEditor::_vifi_work(WSCbase* ptr){
   fname.delTailSpace();
   char* cfname = (char*)obj->_client->getUserData(WS_FILENAME);
   if (cfname != NULL){
-    delete cfname;
+    delete[] cfname;
     obj->_client->setUserData(WS_FILENAME,(void*)0);
   }
   if (strcmp("",(char*)fname)){
