@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #ifndef NO_REMOTE_INSTANCE
@@ -450,6 +446,7 @@ void WSCRbase::onParentSensitiveChange(WSCbool fl){
   _db->save(WS_DcRChar,"fl",(void*)&fl);
   endRemoteCall();
 }
+#ifndef WS_EMBED
 void WSCRbase::onEditModeChange(WSCbool fl){
   if (_local != NULL){
     _local->onEditModeChange(fl);
@@ -465,6 +462,7 @@ void WSCRbase::onEditModeChange(WSCbool fl){
   _db->save(WS_DcRChar,"fl",(void*)&fl);
   endRemoteCall();
 }
+#endif
 void WSCRbase::onFocusChange(WSCbool fl){
   if (_local != NULL){
     _local->onFocusChange(fl);
@@ -882,6 +880,7 @@ WSDLEX32 void _WSCbase_remote_call(WSDserialize* db,WSCremoteCall* rc){
         inst->onParentSensitiveChange(fl);
       }
     }else
+#ifndef WS_EMBED
     if (!strcmp(rc->_op,"onEditModeChange")){
       WSCbool fl;
       db->load(WS_DcRChar,"fl",(void*)&fl);
@@ -890,6 +889,7 @@ WSDLEX32 void _WSCbase_remote_call(WSDserialize* db,WSCremoteCall* rc){
         inst->onEditModeChange(fl);
       }
     }else
+#endif
     if (!strcmp(rc->_op,"onFocusChange")){
       WSCbool fl;
       db->load(WS_DcRChar,"fl",(void*)&fl);
