@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCbaseDialog.h>
@@ -33,6 +29,7 @@
 #include <WSCvbtn.h>
 #include <WSDappDev.h>
 #include <WSCbaseList.h>
+#include <WSCfontSet.h>
 
 WSMFguiClassInitialize(WSCbaseDialog,WSCwindow);
 WSMFversion(WSCbaseDialog,WSCwindow);
@@ -65,6 +62,14 @@ WSCbaseDialog::WSCbaseDialog(WSCbase* base,char* objname):
   _popuped = 0;
   _h = 100;
   _indicator = 0;
+  _focus_btn = WS_DIALOG_DEFAULT;
+  _status = WS_DIALOG_CANCEL;
+  _enable_esc = True;
+  _enable_close_on_cancel = True;
+  _tb_name = NULL;
+  _btb_name = NULL;
+  _font = (char)WSGIappFontSet()->getDefaultFontNo();
+
   WSGIappObjectList()->addEvent(this,WSEV_GUI_POLICY_CH);
 
   WSMFpropertyCreateStart
@@ -86,6 +91,11 @@ WSCbaseDialog::WSCbaseDialog(WSCbase* base,char* objname):
       WSMFpropertySetSelection(WSRbool3,WSRbool3D);
     WSMFpropertyCreate(WSNreplyHandler, void*,_rep,WSSreplyHandler );
     WSMFpropertyVisible( False );
+    WSMFpropertyCreate(WSNkeepPixmapCache,WSCbool,_keep_pixmap_cache,WSSkeepPixmapCache );
+      WSMFpropertySetSelection(WSRbool4,WSRbool4D);
+    WSMFpropertyCreate(WSNtab,  char*,   _tb_name,WSStab);
+    WSMFpropertyCreate(WSNbacktab,  char*,   _btb_name,WSSbacktab);
+    WSMFpropertyCreate(WSNfont,         WSCuchar,   _font,WSSfont  );
 
     WSMFpropertyValueChangeDef(WSCbaseDialog,WSNshadowThickness,WSCuchar);
     WSMFpropertyValueChangeDef(WSCbaseDialog,WSNheight,WSCushort);
@@ -104,13 +114,25 @@ WSCbaseDialog::~WSCbaseDialog(){
   WSGIappDev()->setFocusDialog(this,False);
   WSGIappObjectList()->delEvent(this,WSEV_GUI_POLICY_CH);
   if (_ok_str != NULL){
-    delete _ok_str;
+    delete[] _ok_str;
     _ok_str = NULL;
   }
   if (_no_str != NULL){
-    delete _no_str;
+    delete[] _no_str;
     _no_str = NULL;
   }
+  if (_cancel_str != NULL){
+    delete[] _cancel_str;
+    _cancel_str = NULL;
+  }
+  if (_tb_name != NULL){
+    delete[] _tb_name;
+    _tb_name = NULL;
+  }
+  if (_btb_name != NULL){
+    delete[] _btb_name;
+    _btb_name = NULL;
+  }
 }
 WSMFproperty( WSCbaseDialog, WSNok, WSCbool,   _ok     ,True );
 WSMFproperty( WSCbaseDialog, WSNno, WSCbool,   _no     ,True );
@@ -122,6 +144,10 @@ WSMFproperty( WSCbaseDialog, WSNcancelString, char*,  _cancel_str,WSGFstrdup("Ca
 WSMFproperty( WSCbaseDialog, WSNreplyHandler, void*,  _rep,NULL );
 WSMFproperty( WSCbaseDialog, WSNmodal, WSCbool,  _modal,False );
 WSMFproperty( WSCbaseDialog, WSNdefaultPosition, WSCbool,  _default_pos,False );
+WSMFproperty( WSCbaseDialog, WSNkeepPixmapCache, WSCbool,  _keep_pixmap_cache,True );
+WSMFproperty( WSCbaseDialog, WSNtab,  char*,   _tb_name, NULL);
+WSMFproperty( WSCbaseDialog, WSNbacktab,  char*,   _btb_name, NULL);
+WSMFproperty( WSCbaseDialog, WSNfont,        WSCuchar, _font,(char)WSGIappFontSet()->getDefaultFontNo());
 
 void WSCbaseDialog::setWorkWSNreplyHandler(void*){}
 void WSCbaseDialog::getWorkWSNreplyHandler(void**){}
@@ -145,6 +171,7 @@ void WSCbaseDialog::setWorkWSNcancelString(char* str){
   }
 }
 void WSCbaseDialog::getWorkWSNcancelString(char**){}
+#ifndef NO_WINDOW_TITLE
 void WSCbaseDialog::setWorkWSNtitleString(char* str){
   WSCwindow::setWorkWSNtitleString(str);
   if (_title_labels1 != NULL){
@@ -152,6 +179,7 @@ void WSCbaseDialog::setWorkWSNtitleString(char* str){
     _title_labels2->setPropertyV(WSNlabelString,str);
   }
 }
+#endif
 void WSCbaseDialog::setWorkWSNok(WSCbool fl){
   onGuiPolicyChange(WSGIappDev()->getGuiPolicy());
   if (fl == False){
@@ -205,12 +233,14 @@ void WSCbaseDialog::setWorkWSNwidth(WSCushort width){
   if (_cmd_form != NULL){
     _cmd_form->setPropertyV(WSNwidth, (WSCushort)(width -2));
 //    _cmd_form->setPropertyV(WSNbackPixmap,WSGFgetBarPixmap(width,WS_HORIZONTAL));
+#ifndef NO_WINDOW_TITLE
     _title_labels1->clear();
     _title_labels2->clear();
     _title_labels1->setPropertyV(WSNwidth,(WSCushort)(_w - 20));
     _title_labels2->setPropertyV(WSNwidth,(WSCushort)(_w - 20));
     _title_labels1->redraw();
     _title_labels2->redraw();
+#endif
     long ediff = 0;
     if (_cancel != False){
       if (WSGIappDev()->getGuiPolicy() & WS_POLICY_ORIGINAL){
@@ -258,7 +288,9 @@ void WSCbaseDialog::setWorkWSNheight(WSCushort height){
   }
 }
 void WSCbaseDialog::setWorkWSNmodal(WSCbool fl){
+#ifndef WS_EMBED
   if (getPropertyEditMode() == False){
+#endif
     WSDdev* dev = getowndev();
     if (dev != NULL){
       dev->setValue(WSDEV_FLOAT_ON_TOP,&fl);
@@ -267,26 +299,58 @@ void WSCbaseDialog::setWorkWSNmodal(WSCbool fl){
     if (getVisible() != False){
       WSGIappDev()->setModalClient(this);
     }
+#ifndef WS_EMBED
   }
+#endif
 }
 void WSCbaseDialog::getWorkWSNmodal(WSCbool* fl){}
 void WSCbaseDialog::setWorkWSNdefaultPosition(WSCbool){}
 void WSCbaseDialog::getWorkWSNdefaultPosition(WSCbool*){}
-
+void WSCbaseDialog::setWorkWSNkeepPixmapCache(WSCbool){}
+void WSCbaseDialog::getWorkWSNkeepPixmapCache(WSCbool*){}
+void WSCbaseDialog::setWorkWSNtab(char* cl){
+  if (_end_btn != NULL){
+    _end_btn->setProperty(WSNtab,cl);
+  }
+}
+void WSCbaseDialog::getWorkWSNtab(char**){}
+void WSCbaseDialog::setWorkWSNbacktab(char* cl){
+  if (_ok_btn != NULL){
+    _ok_btn->setProperty(WSNbacktab,cl);
+  }
+}
+void WSCbaseDialog::getWorkWSNbacktab(char**){}
 long WSCbaseDialog::_device_initialize(){
   WSCwindow::_device_initialize();
   WSDdev* dev = getowndev();
   if (dev != NULL){
+#ifndef WS_EMBED
     if (getPropertyEditMode() == False){
+#endif
       WSCbool fl = _modal; 
       dev->setValue(WSDEV_FLOAT_ON_TOP,&fl);
       dev->setValue(WSDEV_MODAL,&fl);
+#ifndef WS_EMBED
     }
+#endif
   }else{
     return WS_ERR;
   }
   return WS_NO_ERR;
 }
+void WSCbaseDialog::setWorkWSNfont(WSCuchar fn){
+  if (_ok_btn != NULL){
+    _ok_btn->setProperty(WSNfont,fn);
+  }
+  if (_no_btn != NULL){
+    _no_btn->setProperty(WSNfont,fn);
+  }
+  if (_end_btn != NULL){
+    _end_btn->setProperty(WSNfont,fn);
+  }
+}
+void WSCbaseDialog::getWorkWSNfont(WSCuchar*){}
+
 static void _setup_btn(WSCbase* btn,WSCbase* dlg){
   btn->initialize();
   btn->setInternalObject(True);
@@ -335,6 +399,7 @@ long WSCbaseDialog::initialize(){
   WSGFsetMargin(_title_label,0);
   _title_label->setVisible(True);
 
+#ifndef NO_WINDOW_TITLE
   _title_labels1 = new WSCvlabel(_cmd_form,"bd-ts1");
   _title_labels1->setInternalObject(True);
   _title_labels1->initialize();
@@ -362,8 +427,14 @@ long WSCbaseDialog::initialize(){
   _title_labels2->setPropertyV(WSNshadowType,(char)WS_SHADOW_TRANS);
   _title_labels2->setVisible(True);
   WSGFsetMargin(_title_labels2,0);
-
-  _ok_btn = new WSCvbtn(_cmd_form,"bd-ok-btn");
+#endif
+  WSCstring ok_name;
+  ok_name << getInstanceName() << "-ok-btn";
+  WSCstring no_name;
+  no_name << getInstanceName() << "-no-btn";
+  WSCstring end_name;
+  end_name << getInstanceName() << "-end-btn";
+  _ok_btn = new WSCvbtn(_cmd_form,ok_name);
   _ok_btn->setInternalObject(True);
   _setup_btn(_ok_btn,this);
   if (WSGIappDev()->getGuiPolicy() & WS_POLICY_ORIGINAL){
@@ -375,7 +446,10 @@ long WSCbaseDialog::initialize(){
 //  _ok_btn->setPropertyV(WSNlabelPixmap,WS_BTN_PIX);
 //  _ok_btn->setProperty(WSNusePixmap,True);
   _ok_btn->setPropertyV(WSNlabelString,_ok_str);
-
+  _ok_btn->setProperty(WSNfont,_font);
+  if (_btb_name != NULL){
+    _ok_btn->setProperty(WSNbacktab,_btb_name);
+  }
   if (_ok != False){
     if (_no != False){
       if (WSGIappDev()->getGuiPolicy() & WS_POLICY_ORIGINAL){
@@ -392,8 +466,7 @@ long WSCbaseDialog::initialize(){
   op1->setInternal(True);
   op1->setFunction(_ok_btn_work,"_ok_btn_work");
   _ok_btn->addProcedure(op1);
-
-  _no_btn = new WSCvbtn(_cmd_form,"bd-no-btn");
+  _no_btn = new WSCvbtn(_cmd_form,no_name);
   _no_btn->setInternalObject(True);
   _setup_btn(_no_btn,this);
 //  _no_btn->setPropertyV(WSNwidth,     (WSCushort)26        );
@@ -401,23 +474,32 @@ long WSCbaseDialog::initialize(){
 //  _no_btn->setProperty(WSNusePixmap,True);
   _no_btn->setPropertyV(WSNlabelString,_no_str);
   _no_btn->setPropertyV(WSNwidth,     (WSCushort)D_BTN_WIDTH        );
+  _no_btn->setProperty(WSNfont,_font);
 
   WSCprocedure* op2 = new WSCprocedure("op1",WSEV_ACTIVATE);
   op2->setInternal(True);
   op2->setFunction(_no_btn_work,"_no_btn_work");
   _no_btn->addProcedure(op2);
+  _no_btn->setProperty(WSNbacktab,ok_name);
 
   if (_no != False){
     _no_btn->setVisible(True);
     _no_btn->setPropertyV(WSNx,         (short)(_w -40 - D_BTN_WIDTH)  );
   }
-  _end_btn = new WSCvbtn(_cmd_form,"bd-end-btn");
+  _end_btn = new WSCvbtn(_cmd_form,end_name);
   _end_btn->setInternalObject(True);
   _setup_btn(_end_btn,this);
   _end_btn->setPropertyV(WSNx,         (short)(_w -40)  );
   _end_btn->setPropertyV(WSNwidth,     (WSCushort)26        );
 //  _end_btn->setProperty(WSNusePixmap,True);
   _end_btn->setPropertyV(WSNlabelPixmap,WS_EBTN_PIX);
+  _end_btn->setProperty(WSNfont,_font);
+  if (_tb_name != NULL){
+    _end_btn->setProperty(WSNtab,_tb_name);
+  }
+  _end_btn->setProperty(WSNbacktab,no_name);
+  _ok_btn->setProperty(WSNtab,no_name);
+  _no_btn->setProperty(WSNtab,end_name);
   if (_cancel != False){
     _end_btn->setVisible(True);
   }
@@ -435,11 +517,19 @@ void WSCbaseDialog::onParentVisibleChange(WSCbool fl){
   WSCwindow::onParentVisibleChange(fl);
   if (fl != False){
     onGuiPolicyChange( WSGIappDev()->getGuiPolicy());
+#ifndef WS_EMBED
     if (_modal == True && getPropertyEditMode() == False){
       if (getVisible() != False){
         WSGIappDev()->setModalClient(this);
       }
     }
+#else
+    if (_modal == True){
+      if (getVisible() != False){
+        WSGIappDev()->setModalClient(this);
+      }
+    }
+#endif
   }else{
 //    if (_modal == True && getPropertyEditMode() == False &&
     if (this == WSGIappDev()->getModalClient()){
@@ -455,14 +545,31 @@ void WSCbaseDialog::onVisibleChange(WSCbool fl){
   if (fl != False){
     _status = 0;
     _status = WS_DIALOG_CANCEL;
+#ifndef WS_EMBED
     if (_modal == True && getPropertyEditMode() == False){
       WSGIappDev()->setModalClient(this);
     }
-
-    WSGIappDev()->setFocusDialog(this,True);
+#else
+    if (_modal == True){
+      WSGIappDev()->setModalClient(this);
+    }
+#endif
+    if (_enable_esc != False){
+      WSGIappDev()->setFocusDialog(this,True);
+    }
+#ifndef WS_EMBED
     if (_default_pos != False && getPropertyEditMode() == False){
-      short x = (WSGIappDev()->getWidth() - _w)/2;
-      short y = (WSGIappDev()->getHeight() - _h)/2;
+#else
+    if (_default_pos != False){
+#endif
+      WSCushort theight = 0;
+      if (getowndev() != NULL){
+        getowndev()->getValue(WSDEV_TITLE_HEIGHT,&theight);
+      }
+      short offsetx = WSGIappDev()->getXOffset();
+      short offsety = WSGIappDev()->getYOffset();
+      short x = (WSGIappDev()->getWidth() - _w)/2 + offsetx;
+      short y = (WSGIappDev()->getHeight() - theight - _h)/2 + offsety;
       setPropertyV(WSNx,x);
       setPropertyV(WSNy,y);
     }
@@ -478,18 +585,45 @@ void WSCbaseDialog::onVisibleChange(WSCbool fl){
     }
   }
   WSCwindow::onVisibleChange(fl);
+
+  if (_focus_btn == WS_DIALOG_DEFAULT){
+    if (_ok != False){
+      _ok_btn->setFocus();
+    }else
+    if (_no != False){
+      _no_btn->setFocus();
+    }else{
+      _end_btn->setFocus();
+    }
+  }else
+  if (_focus_btn == WS_DIALOG_CANCEL){
+    _end_btn->setFocus();
+  }else
+  if (_focus_btn == WS_DIALOG_NO){
+    if (_no != False){
+      _no_btn->setFocus();
+    }
+  }else
+  if (_focus_btn == WS_DIALOG_OK){
+    if (_ok != False){
+      _ok_btn->setFocus();
+    }
+  }
+
 }
 void WSCbaseDialog::onResize(WSCrect* rect){
   if (_w != rect->width){
     if (_cmd_form != NULL){
       _cmd_form->setPropertyV(WSNwidth, (WSCushort)(rect->width -2));
 //      _cmd_form->setPropertyV(WSNbackPixmap,WSGFgetBarPixmap(rect->width,WS_HORIZONTAL));
+#ifndef NO_WINDOW_TITLE
       _title_labels1->clear();
       _title_labels2->clear();
       _title_labels1->setPropertyV(WSNwidth,(WSCushort)(_w - 20));
       _title_labels2->setPropertyV(WSNwidth,(WSCushort)(_w - 20));
       _title_labels1->redraw();
       _title_labels2->redraw();
+#endif
       long ediff = 0;
       if (_cancel != False){
         if (WSGIappDev()->getGuiPolicy() & WS_POLICY_ORIGINAL){
@@ -556,36 +690,54 @@ void WSCbaseDialog::onActivate(){
 void WSCbaseDialog::_ok_btn_work(WSCbase* obj){
   WSCbaseDialog* dlb_base = (WSCbaseDialog*)obj->getUserData("DLG_BASE");
 //  dlb_base->_popuped = 0;
+#ifndef WS_EMBED
   if (dlb_base->getPropertyEditMode() == False){
+#endif
     dlb_base->_status = WS_DIALOG_OK;
     dlb_base->onActivate();
     if (dlb_base->getVisible() != False){
       dlb_base->_status = WS_DIALOG_CANCEL;
     }
 //    dlb_base->execEventProc(WSEV_ACTIVATE,NULL);
+#ifndef WS_EMBED
   }
+#endif
 }
 void WSCbaseDialog::_no_btn_work(WSCbase* obj){
   WSCbaseDialog* dlb_base = (WSCbaseDialog*)obj->getUserData("DLG_BASE");
 //  dlb_base->_popuped = 0;
+#ifndef WS_EMBED
   if (dlb_base->getPropertyEditMode() == False){
+#endif
     dlb_base->_status = WS_DIALOG_NO;
     dlb_base->onActivate();
     if (dlb_base->getVisible() != False){
       dlb_base->_status = WS_DIALOG_CANCEL;
     }
 //    dlb_base-> execEventProc(WSEV_ACTIVATE,NULL);
+#ifndef WS_EMBED
   }
+#endif
 }
 void WSCbaseDialog::_end_btn_work(WSCbase* obj){
   WSCbaseDialog* dlb_base = (WSCbaseDialog*)obj->getUserData("DLG_BASE");
-  dlb_base->_popuped = 0;
+  if (dlb_base->_enable_close_on_cancel != False){
+    dlb_base->_popuped = 0;
+  }
+#ifndef WS_EMBED
   if (dlb_base->getPropertyEditMode() == False){
+#endif
     dlb_base->_status = WS_DIALOG_CANCEL;
     dlb_base->onActivate();
 //    dlb_base-> execEventProc(WSEV_ACTIVATE,NULL);
-    dlb_base->setVisible(False);
+    if (dlb_base->_enable_close_on_cancel == False){
+      dlb_base->_popuped = 1;
+    }else{
+      dlb_base->setVisible(False);
+    }
+#ifndef WS_EMBED
   }
+#endif
 }
 long WSCbaseDialog::getStatus(){
   return _status;
@@ -599,24 +751,23 @@ long WSCbaseDialog::popup(){
     return WS_ERR;
   }
   _popuped = 1;
-  if (_ok != False){
-    _ok_btn->setFocus();
-  }else
-  if (_no != False){
-    _no_btn->setFocus();
-  }else{
-    _end_btn->setFocus();
+  if (_enable_esc != False){
+    WSGIappDev()->setFocusDialog(this,True);
   }
-  WSGIappDev()->setFocusDialog(this,True);
   while(1){
     if (_popuped == 0){
       break;
     }
-    WSGIappDev()->setFocusDialog(this,True);
+    if (_enable_esc != False){
+      WSGIappDev()->setFocusDialog(this,True);
+    }
     WSGIappDev()->dispatchEvent();
     WSGIappDevice()->clearDeleteList();
   }
   setVisible(False);
+  WSGIappDev()->dispatchEvent();
+  WSGIappDevice()->clearDeleteList();
+  WSGIappDev()->update();
   return _status;
 }
 
@@ -624,8 +775,10 @@ void WSCbaseDialog::onGuiPolicyChange(long policy){
   if (policy & WS_POLICY_ORIGINAL){
     D_BTN_WIDTH = 60;
     _title_label->setVisible(True);
+#ifndef NO_WINDOW_TITLE
     _title_labels1->setVisible(True);
     _title_labels2->setVisible(True);
+#endif
     _cmd_form->setPropertyV(WSNshadowThickness,(WSCuchar)1);
     _cmd_form->setProperty(WSNbackColor,WS_DF_DARKBACKCOLOR);
     if (policy & WS_POLICY_GRADATION){
@@ -648,8 +801,10 @@ void WSCbaseDialog::onGuiPolicyChange(long policy){
   }else{
     D_BTN_WIDTH = 80;
     _title_label->setVisible(False);
+#ifndef NO_WINDOW_TITLE
     _title_labels1->setVisible(False);
     _title_labels2->setVisible(False);
+#endif
     _cmd_form->setPropertyV(WSNshadowThickness,(WSCuchar)0);
     _cmd_form->setProperty(WSNbackColor,WS_DF_BACKCOLOR);
     _cmd_form->setProperty(WSNgradation,WS_GR_NONE);
@@ -711,10 +866,137 @@ void WSCbaseDialog::onGuiPolicyChange(long policy){
   _cmd_form->internalUpdate();
   WSCbase::onGuiPolicyChange(policy);
 }
+void WSCbaseDialog::onSpecialFocusChange(WSCbool fl){
+  if (fl != False){
+    if (_ok_btn != NULL){
+      WSCbool fl = _ok_btn->getProperty(WSNvis);
+      if (fl != False){
+        _ok_btn->setSpecialFocus();
+        WSCwindow::onSpecialFocusChange(fl);
+        return;
+      }
+    }
+    if (_no_btn != NULL){
+      WSCbool fl = _no_btn->getProperty(WSNvis);
+      if (fl != False){
+        _no_btn->setSpecialFocus();
+        WSCwindow::onSpecialFocusChange(fl);
+        return;
+      }
+    }
+    if (_end_btn != NULL){
+      WSCbool fl = _end_btn->getProperty(WSNvis);
+      if (fl != False){
+        _end_btn->setSpecialFocus();
+        WSCwindow::onSpecialFocusChange(fl);
+        return;
+      }
+    }
+  }
+  WSCwindow::onSpecialFocusChange(fl);
+}
 void WSCbaseDialog::onFocusChange(WSCbool fl){
   if (fl != False){
-    WSGIappDev()->setFocusDialog(this,True);
+    if (_enable_esc != False){
+      WSGIappDev()->setFocusDialog(this,True);
+    }
   }else{
     WSGIappDev()->setFocusDialog(this,False);
   }
 }
+void WSCbaseDialog::setFocusBtn(long fcmd){
+  switch(fcmd){
+    case WS_DIALOG_NONE:{
+      _focus_btn = fcmd;
+      break;
+    }
+    case WS_DIALOG_DEFAULT:{
+      _focus_btn = fcmd;
+      if (getVisible() != False){
+        if (_ok != False){
+          _ok_btn->setFocus();
+        }else
+        if (_no != False){
+          _no_btn->setFocus();
+        }else{
+          _end_btn->setFocus();
+        }
+      }
+      break;
+    }
+    case WS_DIALOG_CANCEL:{
+      _focus_btn = fcmd;
+      if (getVisible() != False){
+        _end_btn->setFocus();
+      }
+      break;
+    }
+    case WS_DIALOG_NO:{
+      _focus_btn = fcmd;
+      if (getVisible() != False){
+        if (_no != False){
+          _no_btn->setFocus();
+        }
+      }
+      break;
+    }
+    case WS_DIALOG_OK:{
+      _focus_btn = fcmd;
+      if (getVisible() != False){
+        if (_ok != False){
+          _ok_btn->setFocus();
+        }
+      }
+      break;
+    }
+  }
+}
+void WSCbaseDialog::setEnableEscapeKey(WSCbool fl){
+  _enable_esc = fl;
+  if (_enable_esc == False){
+    WSCbase* target = WSGIappDev()->getFocusDialog();
+    if (target == this){
+      WSGIappDev()->setFocusDialog(this,False);
+    }
+  }
+}
+WSCbool WSCbaseDialog::getEnableEscapeKey(){
+  return _enable_esc;
+}
+void WSCbaseDialog::setEnableCloseOnCancel(WSCbool fl){
+  _enable_close_on_cancel = fl;
+}
+WSCbool WSCbaseDialog::getEnableCloseOnCancel(){
+  return _enable_close_on_cancel;
+}
+WSCbase* WSCbaseDialog::getFocusMoveInternalInstance(long direction){
+  if (direction == WS_TAB){
+    WSCbool fl = _ok_btn->getProperty(WSNvis);
+    if (fl != False){
+      return _ok_btn;
+    }
+    fl = _no_btn->getProperty(WSNvis);
+    if (fl != False){
+      return _no_btn;
+    }
+    fl = _end_btn->getProperty(WSNvis);
+    if (fl != False){
+      return _end_btn;
+    }
+  }else
+  if (direction == WS_STAB){
+    WSCbool fl = _end_btn->getProperty(WSNvis);
+    if (fl != False){
+      return _end_btn;
+    }
+    fl = _no_btn->getProperty(WSNvis);
+    if (fl != False){
+      return _no_btn;
+    }
+    fl = _ok_btn->getProperty(WSNvis);
+    if (fl != False){
+      return _ok_btn;
+    }
+  }
+  return NULL;
+}
