@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,49 +14,61 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCclassInformation.h>
 
-inline long WSIFstrcmpA(char* x,char* y){
-  register WSCuchar* s1 = (WSCuchar*)x;
-  register WSCuchar* s2 = (WSCuchar*)y;
-  register WSCuchar p1 = 0;
-  register WSCuchar p2 = 0;
-  do{
-    p1 = (WSCuchar)*s1++;
-    p2 = (WSCuchar)*s2++;
-    if (p1 == 0){
-      return p1 -p2;
-    }
-  }while(p1 == p2);
-  return p1 -p2;
-}
-WSCclassInformation::WSCclassInformation(char* class_name,WSCclassInformation* parent){
-  cname = WSGFstrdup(class_name);
-  cname_hash_value = WSGFgetHashValue(class_name);
+WSCclassInformation::WSCclassInformation(const char* class_name,WSCclassInformation* parent){
+//  cname = WSGFstrdup(class_name);
+  cname = class_name;
+  cname_hash_value = WSGFgetHashValue((char*)class_name);
   _bases = NULL;
   _pclass_info = parent;
   _inf_table = NULL;
-  _all_prop_list = new WSClistData(64);
-  _prop_list_for_init = new WSClistData(64);
+//  _all_prop_list = new WSClistData(64);
+//  _prop_list_for_init = new WSClistData(64);
   _trigger_mask = 0;
   _del_trigger_mask = 0;
+  _ext_trigger_list.setNoAllocIndex(True);
+  _all_ext_trigger_list.setNoAllocIndex(True);
+  _cache = NULL;
+  _prop_list._seg_size = 64;
+  _prop_list_for_init._seg_size = 64;
+  _all_prop_list._seg_size = 64;
+  _all_ext_trigger_list.getList()->_seg_size = 16;
+  _ext_trigger_list.getList()->_seg_size = 8;
+  _del_ext_trigger_list._seg_size = 8;
 };
 
 WSCclassInformation::~WSCclassInformation(){
-  delete cname;
-  delete _prop_list_for_init;
-  delete _all_prop_list;
+//  delete cname;
+//  delete _prop_list_for_init;
+//  delete _all_prop_list;
+  long num=_prop_list.getNum();
+  WSCproperty** buf = (WSCproperty**)_prop_list._data;
+  long i;
+  for(i=0; i<num; i++){
+    WSCproperty* prop = (WSCproperty*)buf[i];
+    delete prop;
+  }
+  if (_inf_table != NULL){
+//    for(i=0; i<WS_MAX_HASH_VALUE; i++){
+//      WSClistData* list = &(_inf_table[i]);
+//      num = list->getNum();
+//      int j;
+//      for(j=0; j<num; j++){
+//        WSCproperty* prop = (WSCproperty*)(*list)[j];
+//        delete prop;
+//      }
+//    }
+    delete[] _inf_table;
+    _inf_table = NULL;
+  }
 }
 WSCclassInformation* WSCclassInformation::getParentClassInformation(){
   return _pclass_info;
@@ -76,61 +88,76 @@ WSClistData* WSCclassInformation::getHashTable(){
   }
   return _inf_table;
 }
+#ifndef WS_EMBED
 WSCbool WSCclassInformation::getPropObj(char* prop_name,char** class_name){
-  if (_inf_table == NULL){
-    _create_inf_table();
-  }
+//  if (_inf_table == NULL){
+//    _create_inf_table();
+//  }
   long val = WSGFgetHashValue(prop_name);
-  if (val < 0 || val > WS_MAX_HASH_VALUE -1){
-    return False;
-  }
-  long num = _inf_table[val].getNum();
-  WSChashData* data;
+  WSClistData* list = &_inf_table[val];
+  long num = (*list)._num;
   long i;
+  WSCproperty* prop = NULL;
+  WSCproperty** buf =(WSCproperty**)list->_data;
   for(i=0; i < num; i++){
-    data = (WSChashData*)_inf_table[val][i];
-    if (data->_property->property_name == prop_name){
-      *class_name = data->_class_info->cname;
+    i++;
+    prop = buf[i];
+    if (prop->property_name == prop_name){
+      WSCclassInformation* inf = (WSCclassInformation*)(*list)[i-1];
+      *class_name = (char*)inf->cname;
        return True;
     }
-  }
-  for(i=0; i < num; i++){
-    data = (WSChashData*)_inf_table[val][i];
-    if (data->_property->property_name[0] == prop_name[0] &&
-        !WSIFstrcmpA( data->_property->property_name, prop_name)){
-      *class_name = data->_class_info->cname;
+    if (prop->property_name[0] == prop_name[0] &&
+        !WSIFstrcmp( prop->property_name, prop_name)){
+      WSCclassInformation* inf = (WSCclassInformation*)(*list)[i-1];
+      *class_name = (char*)inf->cname;
       return True;
     }
   }
   return False;
 }
+#endif
 WSCbool WSCclassInformation::getPropObj(char* prop_name,WSCproperty** prop){
-  if (_inf_table == NULL){
-    _create_inf_table();
+//  if (_inf_table == NULL){
+//    _create_inf_table();
+//  }
+  if (_cache != NULL)
+    if (_cache->property_name == prop_name ||
+        (_cache->property_name[0] == prop_name[0] &&
+          !WSIFstrcmp( _cache->property_name, prop_name))){
+    *prop = _cache;
+    return True;
   }
   long val = WSGFgetHashValue(prop_name);
-  long num = _inf_table[val].getNum();
-  WSChashData* data;
+  WSClistData* list = &_inf_table[val];
+  long num = (*list)._num;
+//  WSChashData* data;
   long i;
+  WSCproperty* property = NULL;
+  WSCproperty** proplist = (WSCproperty**)list->_data;
   for(i=0; i < num; i++){
-    data = (WSChashData*)_inf_table[val][i];
-    if (data->_property->property_name == prop_name){
-      *prop = data->_property;
+    i++;
+    property = proplist[i];
+    if (property->property_name == prop_name){
+      *prop = property;
+      _cache = property;
       return True;
     }
   }
-  for(i=0; i < num ; i++){
-    data = (WSChashData*)_inf_table[val][i];
-    if (data->_property->property_name[0] == prop_name[0] &&
-        !WSIFstrcmpA( data->_property->property_name, prop_name)){
-      *prop = data->_property;
+  for(i=0; i < num; i++){
+    i++;
+    property = proplist[i];
+    if (property->property_name[0] == prop_name[0] &&
+        !WSIFstrcmp( property->property_name, prop_name)){
+      *prop = property;
+      _cache = property;
       return True;
     }
   }
   return False;
 }
-void WSCclassInformation::setExtTrigger(long trg,char* tname){
-  _ext_trigger_list.setData(tname,(void*)trg);
+void WSCclassInformation::setExtTrigger(long trg,const char* tname){
+  _ext_trigger_list.setData((char*)tname,(void*)trg);
 }
 WSCindexData* WSCclassInformation::getExtTriggerList(){
   return &_ext_trigger_list;
@@ -190,13 +217,13 @@ WSClistData* WSCclassInformation::getAllPropertyList(){
   if (_inf_table == NULL){
     _create_inf_table();
   }
-  return _all_prop_list;
+  return &_all_prop_list;
 }
 WSClistData* WSCclassInformation::getPropertyListForInit(){
   if (_inf_table == NULL){
     _create_inf_table();
   }
-  return _prop_list_for_init;
+  return &_prop_list_for_init;
 }
 void WSCclassInformation::setDelExtTrigger(long trg){
   _del_ext_trigger_list.add((void*)trg);
@@ -204,10 +231,16 @@ void WSCclassInformation::setDelExtTrigger(long trg){
 void WSCclassInformation::_create_inf_table(){
   if (_inf_table == NULL){
     _inf_table = new WSClistData[WS_MAX_HASH_VALUE];
+    int i;
+    for(i=0; i<WS_MAX_HASH_VALUE; i++){
+      _inf_table[i]._seg_size = 16;
+    }
   }
-  WSClistData not_use_list;
-  WSClistData defaultch_list;
-  WSClistData not_vis_list;
+  WSClistData not_use_list(16);
+  WSClistData defaultch_list(16);
+#ifndef WS_EMBED
+  WSClistData not_vis_list(16);
+#endif
   WSCclassInformation* inf = this;
   WSClistData* plist;
   WSCproperty* prop = NULL;
@@ -217,9 +250,11 @@ void WSCclassInformation::_create_inf_table(){
   while(1){
     if (inf != NULL){
       WSClistData* dtlist = &inf->_del_ext_trigger_list;
-      long dtnum = dtlist->getNum();
+//      long dtnum = dtlist->getNum();
+      long dtnum = dtlist->_num;
+      long* dtlist_buf = (long*)dtlist->_data;
       for(i=0; i<dtnum; i++){
-        long trg = (long)dtlist->getData(i);
+        long trg = dtlist_buf[i];
         long j;
         long anum = _all_ext_trigger_list.getNum();
         for(j=0; j < anum;j++){
@@ -240,27 +275,31 @@ void WSCclassInformation::_create_inf_table(){
 
 
       plist = &inf->_prop_list;
-      num = plist->getNum();
+      num = plist->_num;
+      WSCproperty** props = (WSCproperty**)(*plist)._data;
       for(i=0; i< num; i++){
-        prop = (WSCproperty*)(*plist)[i];
+        prop = props[i];
         if (prop->_default_value_change != False){
           defaultch_list.add(prop);
           continue;
         }
+#ifndef WS_EMBED
         if (prop->_ignore != False && prop->_builder_visible == False){
           not_vis_list.add(prop);
           continue;
         }
+#endif
         if (prop->_not_use != False){
           not_use_list.add(prop);
           continue;
         }
-        num2 = not_use_list.getNum();
+        num2 = not_use_list._num;
         reject = False;
+        WSCproperty** not_use_list2 = (WSCproperty**)not_use_list._data;
         for(j=0; j < num2; j++){
-          nprop = (WSCproperty*)not_use_list[j];
+          nprop = not_use_list2[j];
           if (nprop->property_name[0] == prop->property_name[0] &&
-              !WSIFstrcmpA(nprop->property_name,prop->property_name)){
+              !WSIFstrcmp(nprop->property_name,prop->property_name)){
             reject = True;
             j = num2;
           }
@@ -269,12 +308,13 @@ void WSCclassInformation::_create_inf_table(){
           continue;
         }
         WSCproperty* default_ch = NULL;
-        long dnum = defaultch_list.getNum();
+        long dnum = defaultch_list._num;
         long j;
+        WSCproperty** dprops = (WSCproperty**)defaultch_list._data;
         for(j=0; j<dnum; j++){
-          WSCproperty* dprop = (WSCproperty*)defaultch_list[j];
+          WSCproperty* dprop = dprops[j];
           if (dprop->property_name[0] == prop->property_name[0] &&
-              !WSIFstrcmpA(dprop->property_name,prop->property_name)){
+              !WSIFstrcmp(dprop->property_name,prop->property_name)){
             default_ch = dprop;
             break;
           }
@@ -289,14 +329,18 @@ void WSCclassInformation::_create_inf_table(){
           new_prop->setGetDefaultValueHandler(
 		           (void(*)(void*))default_ch->getGetDefaultValueHandler() );
           new_prop->_init_execute = default_ch->_init_execute;
+//This causes that the property editor does not display, removed.
+//          new_prop->_default_value_change = True;
           prop = new_prop;
         }
+#ifndef WS_EMBED
         WSCproperty* builder_vis_ch = NULL;
-        dnum = not_vis_list.getNum();
+        dnum = not_vis_list._num;
+        WSCproperty** nvprops = (WSCproperty**)not_vis_list._data;
         for(j=0; j<dnum; j++){
-          WSCproperty* dprop = (WSCproperty*)not_vis_list[j];
+          WSCproperty* dprop = nvprops[j];
           if (dprop->property_name[0] == prop->property_name[0] &&
-              !WSIFstrcmpA(dprop->property_name,prop->property_name)){
+              !WSIFstrcmp(dprop->property_name,prop->property_name)){
             builder_vis_ch = dprop;
             break;
           }
@@ -308,12 +352,15 @@ void WSCclassInformation::_create_inf_table(){
           new_prop->_builder_visible =  builder_vis_ch->_builder_visible;
           prop = new_prop;
         }
-        WSChashData* data = new WSChashData;
-        data->_class_info = inf;
-        data->_property = prop;
-        char* name = prop->property_name;
-        _inf_table[ WSGFgetHashValue(name) ].add(data);
-        _all_prop_list->add(prop);
+#endif
+//        WSChashData* data = new WSChashData;
+//        data->_class_info = inf;
+//        data->_property = prop;
+//        char* name = prop->property_name;
+        long hash_val = WSGFgetHashValue(prop->property_name);
+        _inf_table[ hash_val ].add(inf);
+        _inf_table[ hash_val ].add(prop);
+        _all_prop_list.add(prop);
       }
       inf = inf->_pclass_info;
     }else{
@@ -323,7 +370,7 @@ void WSCclassInformation::_create_inf_table(){
 
   inf = this;
   WSCproperty* vis = NULL;
-  WSClistData pl;
+  WSClistData pl(32);
   while(1){
     if (inf != NULL){
       pl.add(inf);
@@ -333,7 +380,7 @@ void WSCclassInformation::_create_inf_table(){
     inf = inf->_pclass_info;
   }
 
-  j = pl.getNum(); 
+  j = pl._num;
   j--;
   if (j < 0){ 
     return;
@@ -341,29 +388,34 @@ void WSCclassInformation::_create_inf_table(){
   WSCclassInformation* pinf;
   WSCproperty* _prop;
 
+  WSCclassInformation** pinfs = (WSCclassInformation**)pl._data;
   while(1){
-    pinf = (WSCclassInformation*)pl[j];
+    pinf = pinfs[j];
     if (pinf != NULL ){
       plist = &pinf->_prop_list;
-      num = plist->getNum();
+      num = plist->_num;
+      WSCproperty** props = (WSCproperty**)plist->_data;
       for(i=0; i< num; i++){
-        _prop = (WSCproperty*)(*plist)[i];
+        _prop = props[i];
         if (_prop->_default_value_change != False){
           continue;
         }
+#ifndef WS_EMBED
         if (_prop->_ignore != False && _prop->_builder_visible == False){
           continue;
         }
+#endif
         if (_prop->_not_use != False){
           continue;
         }
         WSCproperty* default_ch = NULL;
-        long dnum = defaultch_list.getNum();
+        long dnum = defaultch_list._num;
         long j;
+        WSCproperty** dprops = (WSCproperty**)defaultch_list._data;
         for(j=0; j<dnum; j++){
-          WSCproperty* dprop = (WSCproperty*)defaultch_list[j];
+          WSCproperty* dprop = dprops[j];
           if (dprop->property_name[0] == prop->property_name[0] &&
-              !WSIFstrcmpA(dprop->property_name,_prop->property_name)){
+              !WSIFstrcmp(dprop->property_name,_prop->property_name)){
             default_ch = dprop;
             break;
           }
@@ -381,13 +433,15 @@ void WSCclassInformation::_create_inf_table(){
           new_prop->_set_proc_addr = _prop->_set_proc_addr;
           _prop = new_prop;
         }
+#ifndef WS_EMBED
         WSCproperty* builder_vis_ch = NULL;
-        dnum = not_vis_list.getNum();
+        dnum = not_vis_list._num;
+        WSCproperty** nvprops = (WSCproperty**)not_vis_list._data;
         for(j=0; j<dnum; j++){
-          WSCproperty* dprop = (WSCproperty*)not_vis_list[j];
-          if (dprop->property_name[0] == prop->property_name[0] &&
-              !WSIFstrcmpA(dprop->property_name,_prop->property_name)){
-            builder_vis_ch = dprop;
+          WSCproperty* nvprop = nvprops[j];
+          if (nvprop->property_name[0] == prop->property_name[0] &&
+              !WSIFstrcmp(nvprop->property_name,_prop->property_name)){
+            builder_vis_ch = nvprop;
             break;
           }
         }
@@ -401,9 +455,10 @@ void WSCclassInformation::_create_inf_table(){
           new_prop->_builder_visible =  builder_vis_ch->_builder_visible;
           _prop = new_prop;
         }
-        if ( WSIFstrcmpA(_prop->property_name,"vis")){
+#endif
+        if ( WSIFstrcmp(_prop->property_name,"vis")){
           if ( _prop->_init_execute != False){
-            _prop_list_for_init->add((void*)_prop);       
+            _prop_list_for_init.add((void*)_prop);       
           }
         }else{
           if ( _prop->_init_execute != False){
@@ -420,13 +475,13 @@ void WSCclassInformation::_create_inf_table(){
     }
   }
   if (vis != NULL){
-    _prop_list_for_init->add((void*)vis);       
+    _prop_list_for_init.add((void*)vis);       
   }
 }
 void WSCclassInformation::addPropObj(WSCproperty* prop){
   _prop_list.add((void*)prop);
 }
-WSChashData::WSChashData(){
-   _class_info = NULL;
-   _property = NULL;
-}
+//WSChashData::WSChashData(){
+//   _class_info = NULL;
+//   _property = NULL;
+//}
