@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCconductor.h>
@@ -61,6 +57,7 @@ WSCconductor::WSCconductor(){
   _timeout = 10;
   _timer_id = 0;
   _need_update = False;
+  _net_server_handler_list.setNoAllocIndex(True);
 }
 WSCconductor::~WSCconductor(){
   if (_timer_id != 0){
@@ -88,10 +85,12 @@ long WSCconductor::del(WSCtypeServer* item){
 }
 WSCtypeServer* WSCconductor::searchTypeServer(char* type){
   long i;
-  long num = _type_server_list.getNum();
+  long num = _type_server_list._num;
+  WSCtypeServer** buf = (WSCtypeServer**)_type_server_list._data;
   for(i=0; i<num; i++){
-    WSCtypeServer* drcl = (WSCtypeServer*)_type_server_list[i];
-    if (drcl->isHit(type) != False){
+    WSCtypeServer* drcl = buf[i];
+//    if (drcl->isHit(type) != False){
+    if (!strcmp(drcl->_type_name,type)){
       return drcl;
     }
   }
@@ -99,7 +98,7 @@ WSCtypeServer* WSCconductor::searchTypeServer(char* type){
 }
 WSDserialize* WSCconductor::getSerialize(char* serialize_type){
   long i;
-  long num = _serialize_list.getNum();
+  long num = _serialize_list._num;
   WSDserialize* same_serialize = NULL;
   for(i=0; i< num; i++){
     WSDserialize* serialize = (WSDserialize*)_serialize_list[i];
@@ -136,7 +135,7 @@ char* WSCconductor::getSerializePath(){
 void WSCconductor::setSerializePath(char* path){
   _serialize_path.setString(path);
 }
-WSDserialize* WSCconductor::beginTransaction(char* database,char* fname){
+WSDserialize* WSCconductor::beginTransaction(char* database,char* fname,int fl){
   WSDserialize* serialize = getSerialize(database);
   if (serialize == NULL){
 WSMFtrace("WSCconductor::beginTransaction()  Data base does not exist.\n");
@@ -144,21 +143,21 @@ WSMFtrace("WSCconductor::beginTransaction()  Data base does not exist.\n");
   }
   serialize->setPath(getSerializePath());
 
-  long ret = serialize->beginTransaction(fname);
+  long ret = serialize->beginTransaction(fname,fl);
   if (ret != WS_NO_ERR){
 WSMFtrace("WSCconductor::beginTransaction()  Data base error.\n");
     return NULL;
   }
   return serialize;
 }
-WSDserialize* WSCconductor::beginTransaction(char* database,char* fname,WSDserialize* serialize){
+WSDserialize* WSCconductor::beginTransaction(char* database,char* fname,WSDserialize* serialize,int fl){
   if (serialize == NULL){
 WSMFtrace("WSCconductor::beginTransaction()  Data base does not exist.\n");
     return NULL;
   }
   serialize->setPath(getSerializePath());
 
-  long ret = serialize->beginTransaction(fname);
+  long ret = serialize->beginTransaction(fname,fl);
   if (ret != WS_NO_ERR){
 WSMFtrace("WSCconductor::beginTransaction()  Data base error.\n");
     return NULL;
