@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,23 +14,22 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCfontSet.h>
 static WSCfontSet* _the_app_font_list = NULL;
 
+WSCfontSet* get_new_font_set(){
+  return new WSCfontSet();
+}
 WSCfontSet* WSGIappFontSet(){
   if (_the_app_font_list == NULL){
-    _the_app_font_list = new WSCfontSet();
+    _the_app_font_list = get_new_font_set();
     if (_the_app_font_list == NULL){
       return NULL;
     }
@@ -42,6 +41,7 @@ WSCfontSet::WSCfontSet(){
   _create_handler = NULL;
   _setup_handler = NULL;
   _default_init_handler = NULL;
+  _font_list._seg_size = 16;
 }
 WSCfontSet::~WSCfontSet(){
   long i;
@@ -63,16 +63,17 @@ long WSCfontSet::addFont(WSDfont* font){
 WSDfont* WSCfontSet::getFont(short no){
   long i;
   long num = _font_list.getNum();
+  WSDfont** fonts = (WSDfont**)_font_list.getBuf();
   for(i=0; i<num; i++){
-    WSDfont* fon = (WSDfont*)_font_list[i];
-    if (fon->getId() == no){
+    WSDfont* fon = fonts[i];
+    if (fon->_id == no){
       return fon;
     }
   }
   no = _default_font;
   for(i=0; i<num; i++){
-    WSDfont* fon = (WSDfont*)_font_list[i];
-    if (fon->getId() == no){
+    WSDfont* fon = fonts[i];
+    if (fon->_id == no){
       return fon;
     }
   }
@@ -83,7 +84,7 @@ WSDfont* WSCfontSet::getDefaultFont(){
   long num = _font_list.getNum();
   for(i=0; i<num; i++){
     WSDfont* fon = (WSDfont*)_font_list[i];
-    if (fon->getId() == _default_font){
+    if (fon->_id == _default_font){
       return fon;
     }
   }
@@ -128,3 +129,10 @@ long WSCfontSet::getStringHeight(short no,char* s,long en){
   str.setString(s,en);
   return getStringHeight(no,&str); 
 }
+WSCbool WSCfontSet::getReady(short no,WSCbool loading){
+  WSDfont* font = getFont(no);
+  if (font == NULL){
+    return False;
+  }
+  return font->getReady(loading);
+}
