@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCform.h>
@@ -57,11 +53,14 @@ WSCform::WSCform(WSCbase* base,char* objname):WSCbase(base,objname){
   _bp_color = -1;
   _shadow_type = WS_SHADOW_OUT;
   _sensitive = True;
+  _sensitive_work = True;
   _pixmap_style = WS_DYNAMIC_PIXMAP;
   _no_refresh = False;
   _mouse_no     = 0;
+#ifndef NO_GRADATION
   _grad_no     = 0;
   _grad_margin   = 0;
+#endif
   _update_resize   = 0;
   _anchor_top = 0;
   _anchor_bottom= 0;
@@ -74,6 +73,8 @@ WSCform::WSCform(WSCbase* base,char* objname):WSCbase(base,objname){
   _a1_color = -1;
   _a2_color = -1;
   _a3_color = -1;
+  _children._seg_size = 8;
+
   WSGIappObjectList()->addEvent(this,WSEV_GUI_POLICY_CH);
 
   WSMFpropertyCreateStart
@@ -95,15 +96,17 @@ WSCform::WSCform(WSCbase* base,char* objname):WSCbase(base,objname){
       WSMFpropertySetSelection(WSRshadowType,WSRshadowTypeD);
     WSMFpropertyCreate(WSNvis,       WSCbool,      _vis,WSSvis );
       WSMFpropertySetSelection(WSRbool2,WSRbool2D);
-    WSMFpropertyCreate(WSNdet,              WSCbool,  _sensitive,WSSdet);
+    WSMFpropertyCreate(WSNdet,              WSCbool,  _sensitive_work,WSSdet);
       WSMFpropertySetSelection(WSRbool1,WSRbool1D);
     WSMFpropertyCreate(WSNpixmapStyle,      WSCuchar,  _pixmap_style,WSSpixmapStyle);
     WSMFpropertySetSelection(WSRpixmapStyle,WSRpixmapStyleD);
     WSMFpropertyCreate( WSNmouse, WSCushort,   _mouse_no,WSSmouse  );
 
+#ifndef NO_GRADATION
     WSMFpropertyCreate(WSNgradation,  WSCuchar,  _grad_no,WSSgradation);
       WSMFpropertySetSelection(WSRgradation,WSRgradationD);
     WSMFpropertyCreate(WSNgradationMargin,WSCuchar,_grad_margin,WSSgradationMargin);
+#endif
 
     WSMFpropertyCreate(WSNanchorTop, WSCushort, _anchor_top,WSSanchorTop);
     WSMFpropertyCreate(WSNanchorBottom,WSCushort,_anchor_bottom,WSSanchorBottom);
@@ -118,9 +121,12 @@ WSCform::WSCform(WSCbase* base,char* objname):WSCbase(base,objname){
     WSMFpropertySetSelection(WSRbool3, WSRbool3D);
     WSMFpropertyCreate(WSNanchorRightFlag,WSCbool,_anchor_right_use,WSSanchorRightFlag);
     WSMFpropertySetSelection(WSRbool3, WSRbool3D);
-
+#ifndef NO_REMOTE_INSTANCE
     WSMFpropertyCreate(WSNexport,WSCbool,_export_,WSSexport);
     WSMFpropertySetSelection(WSRbool1, WSRbool1D);
+#endif
+    WSMFpropertyCreate(WSNkeepPixmapCache,WSCbool,_keep_pixmap_cache,WSSkeepPixmapCache );
+      WSMFpropertySetSelection(WSRbool4,WSRbool4D);
 
     WSMFaddTrigger(WSEV_INITIALIZE  );
     WSMFaddTrigger(WSEV_MOUSE_MOVE  );
@@ -134,8 +140,8 @@ WSCform::WSCform(WSCbase* base,char* objname):WSCbase(base,objname){
     WSMFaddTrigger(WSEV_RESIZE      );
   WSMFpropertyCreateEnd
 }
-WSMFproperty(WSCform,WSNname, char*, _instance_name ,WSGFstrdup("")); 
-WSMFproperty(WSCform,WSNuserString, char*, _user_string ,WSGFstrdup("")); 
+WSMFproperty(WSCform,WSNname, char*, _instance_name ,NULL);
+WSMFproperty(WSCform,WSNuserString, char*, _user_string ,NULL);
 WSMFproperty(WSCform,WSNuserValue, long,  _user_value, 0); 
 WSMFproperty(WSCform,WSNx,   short, _x, 100 );
 WSMFproperty(WSCform,WSNy,   short, _y, 100 );
@@ -153,11 +159,13 @@ WSMFproperty(WSCform,WSNbottomShadowColor,short, _bs_color, WS_DF_BOTTOMSHADOWCO
 WSMFproperty(WSCform,WSNbackPixmap, short, _bp_color, -1);
 WSMFproperty(WSCform,WSNshadowType, char,    _shadow_type ,WS_SHADOW_OUT );
 WSMFproperty(WSCform,WSNvis,WSCbool, _vis ,False  );
-WSMFproperty(WSCform,WSNdet,WSCbool, _sensitive, True );
+WSMFproperty(WSCform,WSNdet,WSCbool, _sensitive_work, True );
 WSMFproperty(WSCform,WSNpixmapStyle, WSCuchar, _pixmap_style ,WS_DYNAMIC_PIXMAP);
 WSMFproperty(WSCform, WSNmouse,   WSCushort,     _mouse_no     ,0);
+#ifndef NO_GRADATION
 WSMFproperty(WSCform, WSNgradation, WSCuchar, _grad_no   ,0);
 WSMFproperty(WSCform, WSNgradationMargin, WSCuchar, _grad_margin ,0);
+#endif
 WSMFproperty( WSCform, WSNanchorTop,WSCushort,_anchor_top,0 );
 WSMFproperty( WSCform, WSNanchorBottom,WSCushort,_anchor_bottom,0 );
 WSMFproperty( WSCform, WSNanchorLeft,WSCushort,_anchor_left,0 );
@@ -166,8 +174,10 @@ WSMFproperty( WSCform, WSNanchorTopFlag,WSCbool,_anchor_top_use,False );
 WSMFproperty( WSCform, WSNanchorBottomFlag,WSCbool,_anchor_bottom_use,False );
 WSMFproperty( WSCform, WSNanchorLeftFlag,WSCbool,_anchor_left_use,False );
 WSMFproperty( WSCform, WSNanchorRightFlag,WSCbool,_anchor_right_use,False );
+#ifndef NO_REMOTE_INSTANCE
 WSMFproperty( WSCform, WSNexport,WSCbool,_export_,False );
-
+#endif
+WSMFproperty( WSCform, WSNkeepPixmapCache, WSCbool,  _keep_pixmap_cache,True );
 void WSCform::setWorkWSNname(char* ){}
 void WSCform::getWorkWSNname(char** ){}
 void WSCform::setWorkWSNuserString(char* ){}
@@ -230,10 +240,12 @@ void WSCform::setWorkWSNwidth(WSCushort data){
     _dev->setValue(WSDEV_WIDTH,(void*)&data);
     _geometry_changed = True;
     _update_resize = True;
+#ifndef NO_GRADATION
     if (_grad_no != 0){
       setAbsoluteDraw(True);
       needUpdate();
     }
+#endif
   }else{
     _geometry_changed = True;
     _update_resize = True;
@@ -254,10 +266,12 @@ void WSCform::setWorkWSNheight(WSCushort data){
     _dev->setValue(WSDEV_HEIGHT,(void*)&data);
     _geometry_changed = True;
     _update_resize = True;
+#ifndef NO_GRADATION
     if (_grad_no != 0){
       setAbsoluteDraw(True);
       needUpdate();
     }
+#endif
   }else{
     _geometry_changed = True;
     _update_resize = True;
@@ -298,24 +312,31 @@ void WSCform::setWorkWSNvis(WSCbool data){
 }
 void WSCform::getWorkWSNvis(WSCbool*){}
 void WSCform::setWorkWSNdet(WSCbool data){
+#ifndef WS_EMBED
   if (getPropertyEditMode() == False){
-    if (data == False){
-      _sensitive = True;
-    }else{
-      _sensitive = False;
-    }
+#endif
+//    if (data == False){
+//      _sensitive = True;
+//    }else{
+//      _sensitive = False;
+//    }
     setSensitive(data);
-  }else{
-    long val =(long)data;
-    setUserData(WS_DET,(void*)val);
+#ifndef WS_EMBED
+//  }else{
+//    long val =(long)data;
+//    setUserData(WS_DET,(void*)val);
+//    _sensitive = True;
   }
+#endif
 }
 void WSCform::getWorkWSNdet(WSCbool* data){
-  if (getPropertyEditMode() != False){
-    long tmp = (long)getUserData(WS_DET);
-    WSCbool fl = (WSCbool)tmp;
-    *data = fl;
-  }
+#ifndef WS_EMBED
+//  if (getPropertyEditMode() != False){
+//    long tmp = (long)getUserData(WS_DET);
+//    WSCbool fl = (WSCbool)tmp;
+//    *data = fl;
+//  }
+#endif
 }
 void WSCform::setWorkWSNpixmapStyle(WSCuchar data){
   if (_initialized != False && _dev != NULL){
@@ -333,10 +354,12 @@ void WSCform::setWorkWSNmouse(WSCushort data ){
   }
 }
 void WSCform::getWorkWSNmouse(WSCushort* ){}
+#ifndef NO_GRADATION
 void WSCform::setWorkWSNgradation(WSCuchar ){}
 void WSCform::getWorkWSNgradation(WSCuchar* ){}
 void WSCform::setWorkWSNgradationMargin(WSCuchar ){}
 void WSCform::getWorkWSNgradationMargin(WSCuchar* ){}
+#endif
 
 void WSCform::setWorkWSNanchorTop(WSCushort data){
   if (_anchor_top_use != False){
@@ -411,12 +434,15 @@ void WSCform::setWorkWSNanchorRightFlag(WSCbool data){
 }
 void WSCform::getWorkWSNanchorRightFlag(WSCbool* data){}
 
+#ifndef NO_REMOTE_INSTANCE
 void WSCform::setWorkWSNexport(WSCbool data){
   setExported(data);
 }
 void WSCform::getWorkWSNexport(WSCbool* data){
 }
-
+#endif
+void WSCform::setWorkWSNkeepPixmapCache(WSCbool){}
+void WSCform::getWorkWSNkeepPixmapCache(WSCbool*){}
 
 WSCbool WSCform::getNoRefresh(){
   return _no_refresh;
@@ -427,6 +453,9 @@ void WSCform::setChildren(WSClistData& src){
 WSClistData& WSCform::getChildren(){
   return _children;
 }
+WSClistData* WSCform::getChildrenPtr(){
+  return &_children;
+}
 void WSCform::onVisibleChange(WSCbool fl){
   long need_resize_event = 0;
   if (fl != False && getowndev() == NULL){
@@ -460,7 +489,9 @@ void WSCform::onParentVisibleChange(WSCbool fl){
 }
 void WSCform::onResize(WSCrect* area){
   WSCbool fl = False;
+#ifndef WS_EMBED
   if (getPropertyEditMode() == False){
+#endif
     _x = area->x;
     _y = area->y;
     if (_w != area->width){
@@ -471,7 +502,9 @@ void WSCform::onResize(WSCrect* area){
       _h = area->height;
       fl = True;
     }
+#ifndef WS_EMBED
   }
+#endif
   if (fl != False){
 #if 0
     WSDdev* dev = getowndev();
@@ -501,7 +534,7 @@ WSCform::~WSCform(){
     delete child;
   }
   if (_user_string != NULL){
-    delete _user_string;
+    delete[] _user_string;
     _user_string = NULL;
   }
 }
@@ -530,7 +563,7 @@ long WSCform::_device_initialize(){
   dev->setValue(WSDEV_BACKCOLOR,(void*)&_back_color);
   dev->setValue(WSDEV_BACK_PIXMAP,(void*)&_bp_color);
   dev->setValue(WSDEV_PIXMAP_STYLE,(void*)&pstyle); 
-  dev->setEnableEventBit(WSEV_EXPOSE_BIT | WSEV_RESIZE_BIT);
+  dev->setEnableEventBit(WSEV_EXPOSE_BIT | WSEV_RESIZE_BIT | WSEV_MOUSE_IN_BIT);
   if (getVisible() != False){
     dev->setVisible(True);
   }
@@ -595,8 +628,10 @@ dbprintf("WSCform::draw %s:%d %s call _exec_child_adjust_for_anchors()..\n",__FI
       st.top_shadow_color = _ts_color;
       st.bottom_shadow_color = _bs_color;
       st.back_pixmap = _bp_color;
+#ifndef NO_GRADATION
       st.gradation = _grad_no;
       st.gradation_margin = _grad_margin;
+#endif
       _ext_draw_proc(&st);
       return WS_NO_ERR;
     }  
@@ -619,23 +654,18 @@ dbprintf("WSCform::draw %s:%d %s dev not ready..\n",__FILE__,__LINE__,getInstanc
 
 
   if (_bp_color == -1){
+#ifndef NO_GRADATION
     if (_grad_no == 0){
-      dev->setForeColor(_back_color);
-      dev->drawFillRect(0,0,_w,_h);
-#ifndef WS_EMBED
-      if ((WSGIappDev()->getGuiPolicy() & WS_POLICY_AQUA) != 0 &&
-           _bp_color == -1){
-        if (_a1_color == -1){
-          WSGFgetShadowColor(_back_color,&_a1_color,&_a2_color,&_a3_color);
-        }
-        WSGFdrawStripes(dev,_w,_h,_a1_color,_a2_color,_a3_color);
-      }
 #endif
       dev->setForeColor(_back_color);
+      dev->drawFillRect(0,0,_w,_h);
+      dev->setForeColor(_back_color);
 
+#ifndef NO_GRADATION
     }else{
       dev->drawGradation(_grad_no,_ts_color,_back_color,_bs_color,0,0,_w,_h,_grad_margin);
     }
+#endif
   }
 
   if (getDotMode() == True){
@@ -661,15 +691,16 @@ void WSCform::_exec_child_adjust_for_anchors(){
       WSClistData children = getChildren();
       long num = children.getNum();
       long i;
+      WSCbase** buf = (WSCbase**)children._data;
       for(i=0; i<num; i++){
-        WSCbase* child = (WSCbase*)children[i];
+        WSCbase* child = buf[i];
         child->_adjust_for_anchors(_w,_h);
         if (getPixmapStyle() == WS_DYNAMIC_PIXMAP){
           if (child->isNeedUpdate() != False){
-            if ((long)child->getUserData("anchor update..") == 0){
-              child->setUserData("anchor update..",(void*)1);
+            if (child->_flag_for_anchor == False){
+              child->_flag_for_anchor = True;
               child->update();
-              child->setUserData("anchor update..",(void*)0);
+              child->_flag_for_anchor = False;
             }
           }
         }
@@ -734,12 +765,22 @@ long WSCform::redraw(){
   if (getVisible() == False){
     return WS_ERR;
   }
-  long er = dev->beginDraw(0,0,_w,_h,True);
-  if (er != WS_NO_ERR){
-    return WS_NO_ERR;
-  }
-  dev->clearArea(0,0,_w,_h,True);
-  dev->endDraw();
+//  long noclear = False;
+//  if (_pixmap_style == WS_DYNAMIC_PIXMAP){
+//    WSDwindowDev* wdev = (WSDwindowDev*)dev->cast("WSDwindowDev");
+//    if (wdev != NULL && wdev->getPixmapCreated() == False){
+//      noclear = True;
+//    }
+//  }
+
+//  long er = dev->beginDraw(0,0,_w,_h,True);
+//  if (er != WS_NO_ERR){
+//    return WS_NO_ERR;
+//  }
+//  if (noclear == False){
+    dev->clearArea(0,0,_w,_h,True);
+//  }
+//  dev->endDraw();
   return WS_NO_ERR;
 }
 
@@ -822,7 +863,7 @@ void WSCform::_adjust_for_anchors(WSCushort w,WSCushort h){
       if (_anchor_bottom_use != 0){
         _anchor_top_use = False;
         _anchor_bottom_use = False;
-        clear();
+//          clear();
         if (h < _anchor_bottom + _anchor_top){
           setProperty(WSNheight,1);
           resized = True;
@@ -887,16 +928,21 @@ long WSCform::getPixmapStyle(){
 }
 
 void(*WSCform::_ext_draw_proc)(void*) = NULL;
-WSCstring WSCform::_policy_name;
+char* WSCform::_policy_name = NULL;
 
 void WSCform::onGuiPolicyChange(long policy){
   WSCbase::onGuiPolicyChange(policy);
   if (WSGIappDev()->getExtGuiPolicySelected() == False){
     _ext_draw_proc = NULL;
   }else{
-    if (!strcmp((char*)_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
+    if (_policy_name != NULL &&
+        !strcmp(_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
     }else{
       _ext_draw_proc = (void(*)(void*))WSGIappDev()->getExtGuiPolicyProc("WSCform::draw");
+      if (_policy_name != NULL){
+        delete _policy_name;
+      }
+      _policy_name = WSGFstrdup(WSGIappDev()->getExtGuiPolicyName());
     }
   }
   needUpdate();
@@ -915,6 +961,43 @@ void WSCform::internalUpdate(){
     }
   }
 }
+#ifndef NO_RESTACK_CHILD
+long WSCform::restackChild(WSCbase* target,int order){
+  long num = _children.getNum();
+  if (order > num ){
+    order = num;
+  }
+  order = num - order;
+  long pos = -1;
+  long i;
+  for(i=0; i< num; i++){
+    WSCbase* child = (WSCbase*)_children[i];
+    if (child == target){
+      pos = i;
+      break;
+    }
+  }
+  if (pos == -1){
+    return WS_ERR;
+  }
+  if (order == pos+1){
+    return WS_NO_ERR;
+  }
+  _children.delPos(pos);
+  if (order > 0){
+    order--;
+  }
+  _children.add(target,order);
+  WSDdev* dev = getowndev();
+  if (dev != NULL){
+    WSDwindowDev* wdev = (WSDwindowDev*)dev->cast("WSDwindowDev");
+    if (wdev != NULL){
+      wdev->restackChildren();
+    }
+  }
+  return WS_NO_ERR;
+}
+#endif
 #ifdef WS_EMBED
 #ifndef NO_EXT_GUI_POLICY
 class _gui_policy_register_{
