@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,17 +14,13 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
-
 #include <WScom.h>
+#ifndef WS_EMBED
 #include <WSCgripHand.h>
 #include <WSCclassInformation.h>
 #include <WSDdev.h>
@@ -39,6 +35,12 @@
 #define WS_GH_COLOR2  WS_DF_DARKBACKCOLOR
 #define WS_GH_COLOR3  WS_DF_BOTTOMSHADOWCOLOR
 
+#ifndef NO_MWT_FUNCTION
+#ifndef NO_INDEX_VARIANT_DATA
+#include <WSCundoManager.h>
+#endif
+#endif
+
 WSMFguiClassInitialize(WSCgripHand,WSCform);
 
 WSCgripHand::WSCgripHand(WSCbase* base,char* objname):WSCform(base,objname){
@@ -83,6 +85,16 @@ void WSCgripHand::onMousePress(WSCpoint* ){
   _update_timer->setCont(True);
   _update_timer->setWorkProc(_grip_hand_update_proc,this);
   _update_timer->startTimer();
+#ifndef NO_MWT_FUNCTION
+#ifndef NO_INDEX_VARIANT_DATA
+  WSCbase* client = (WSCbase*)getUserData("CLIENT");
+  if (client != NULL){
+    WSCpropUndoProc* prop_undo = new WSCpropUndoProc();
+    prop_undo->setTarget(client);
+    WSGIappUndoManager()->addUndoProc(prop_undo);
+  }
+#endif
+#endif
 }
 void WSCgripHand::onMouseRelease(WSCpoint* ){
    if (_update_timer != NULL){
@@ -124,3 +136,4 @@ void WSCgripHand::_grip_hand_update_proc(void* data){
 long WSCgripHand::getObjectType(){
   return (WS_TYPE_MANAGER | WS_TYPE_FORM );
 }
+#endif //WS_EMBED
