@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,18 +14,15 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 
+#ifndef NO_INDEX_VARIANT_DATA
 #include <WSCindexVariantData.h>
 
 WSCindexVariantData::WSCindexVariantData(){
@@ -63,7 +60,7 @@ long WSCindexVariantData::del(char* index){
     char* ptr = (char*)_index[i];
     if (ptr[0] == index[0] && !strcmp(ptr,index)){
       char* str = (char*)_index[i];
-      delete str;
+      delete[] str;
       _index.delPos(i);
       WSCvariant* val = (WSCvariant*)_data[i];
       _data.delPos(i);
@@ -83,7 +80,7 @@ long WSCindexVariantData::delPos(long pos){
   }
   if (pos < _index.getNum()){
     char* str = (char*)_index[pos];
-    delete str;
+    delete[] str;
     _index.delPos(pos);
     WSCvariant* val = (WSCvariant*)_data[pos];
     _data.delPos(pos);
@@ -98,7 +95,7 @@ void WSCindexVariantData::clear(){
   long i;
   for(i=0; i<num; i++){
     char* index =(char*)_index[i];
-    delete index;
+    delete[] index;
     WSCvariant* val = (WSCvariant*)_data[i];
     delete val;
   }
@@ -192,3 +189,4 @@ WSCbool WSCindexVariantData::isExist(char* index){
   }
   return False;
 }
+#endif
