@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,22 +14,17 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 
 #include <WSClistData.h>
-
 WSClistData::WSClistData(){
-  _seg_size = 16;
+  _seg_size = 128;
   _num = 0;
   _data = (void**)NULL;
 }
@@ -44,7 +39,7 @@ WSClistData::WSClistData(long seg){
 
 WSClistData::~WSClistData(){
   if (_data != NULL){
-    delete _data;
+    delete[] _data;
     _data = NULL;;
   }
   _num = 0;
@@ -55,69 +50,70 @@ long WSClistData::getNum(){
 }
 long WSClistData::add(void* item,long pos){
   void**   newList = NULL;
-  long num = _num;
   long segment_size = _seg_size;
   if (segment_size == 0){
     segment_size = 0x100;
   }
-  if (pos == -1 || pos >= num){
+  if (pos == -1 || pos >= _num){
 
-    if (num % segment_size == 0){
-	  if (num != 0){
-        newList = new void*[ (num /segment_size +1) * segment_size ];
-        memcpy(newList,_data,sizeof(void*) * num);
-        delete _data;
+    if (_num % segment_size == 0){
+	  if (_num != 0){
+        newList = new void*[ (_num /segment_size +1) * segment_size ];
+        memcpy(newList,_data,sizeof(void*) * _num);
+        delete[] _data;
+        _data = newList;
 	  }else{
-        newList = new void*[ segment_size ];
+        if (_data == NULL){
+          _data = new void*[ segment_size ];
+        }
 	  }
-      _data = newList;
     }
-    _data[num] = item;
-    num++;
-    _num = num;
+    _data[_num] = item;
+    _num++;
     return 0;
   }
 
   if (pos < 0 ){
     pos = 0;
   }
-
-  if (num != 0){
-    if (num % segment_size == 0){
-       long newlen = (num /segment_size +1) * segment_size;
+  if (_num != 0){
+    if (_num % segment_size == 0){
+       long newlen = (_num /segment_size +1) * segment_size;
        newList = new void*[newlen];
        memcpy(newList,_data,sizeof(void*) * pos);
-       memcpy(&newList[pos+1],&_data[pos],sizeof(void*) * (num -pos));
-       delete _data;
+       memcpy(&newList[pos+1],&_data[pos],sizeof(void*) * (_num -pos));
+       delete[] _data;
        _data = newList;
     }else{
-       memmove(&_data[pos+1],&_data[pos],sizeof(void*) * (num - pos));
+       memmove(&_data[pos+1],&_data[pos],sizeof(void*) * (_num - pos));
     }
   }else{
-    newList = new void*[segment_size];
-    _data = newList;
+    if (_data == NULL){
+      _data = new void*[segment_size];
+    }
   }
 
   _data[pos] = item;
-  num++;
-  _num = num;
+  _num++;
   return 0;
 }
 
 long WSClistData::del(void* item){
-  long i,fl;
-  fl = -1;
+//  long i,fl;
+//  fl = -1;
+  long i;
   long num = _num;
   for(i =0; i < num; i++){
     if (_data[i] == item){
       if (num != 1){
-        memcpy(&(_data[i]),&(_data[i+1]),sizeof(void*) * (num - i -1));
+        memmove(&(_data[i]),&(_data[i+1]),sizeof(void*) * (num - i -1));
       }else{
-        delete _data;
+        delete[] _data;
         _data = NULL;
       }
-      num--;
-      _num = num;
+//      num--;
+//      _num = num;
+      _num--;
       return 0;
     }
   }
@@ -128,31 +124,29 @@ long WSClistData::delPos(long fl){
   if (fl  < 0){
     return -1;
   }
-  long num = _num;
-  if (num == 0){
+  if (_num == 0){
     return -1;
   }
   // over flow deleting pos  
-  if (fl > num-1){
+  if (fl > _num-1){
     return -1;
   }
 
-  if (num != 1){
-    memcpy(&(_data[fl]),&(_data[fl+1]),sizeof(void*) * (num - fl -1));
+  if (_num != 1){
+    memmove(&(_data[fl]),&(_data[fl+1]),sizeof(void*) * (_num - fl -1));
   }else{
-    delete _data;
+    delete[] _data;
     _data = NULL;
   }
-  num--;
-  _num = num;
+  _num--;
   return 0;
 }
 
 void WSClistData::clear(){
-  if (_data != NULL){
-    delete _data;
-    _data = NULL;;
-  }
+//  if (_data != NULL){
+//    delete _data;
+//    _data = NULL;;
+//  }
   _num = 0;
 }
 
@@ -251,7 +245,7 @@ static void* dummy;
 WSClistData& WSClistData::operator = (WSClistData& src){
   if (&src != this){
     if (_data){
-      delete _data;
+      delete[] _data;
       _data = NULL;
     }
     _num = src._num;
