@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,20 +14,18 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSClocaleSet.h>
 #include <WSDenv.h>
 
+long _wsgv_default_encoding = WS_EN_LOCALE;
+long _wsgv_system_default_encoding = WS_EN_NONE;
 
 static WSDcodeConvert* _the_app_code_convert = NULL;
 static WSDcodeConvert* (*_c_handler)() = NULL;
@@ -79,10 +77,11 @@ void WSClocaleSet::execInitializeProcs(){
 }
 #endif
 WSClocaleSet::~WSClocaleSet(){
-  long num = _locale_list.getNum();
+  long num = _locale_list._num;
   long i;
+  WSClocale** locales = (WSClocale**)_locale_list._data;
   for(i=0; i<num; i++){
-    WSClocale* locale = (WSClocale*)_locale_list[i];
+    WSClocale* locale = locales[i];
     if (locale != NULL){
       delete locale;
     }
@@ -102,10 +101,11 @@ void WSClocaleSet::initialize(){
 //  }
 //printf("WSClocaleSet::initialize systemlocale=%s\n",WSGFgetLocaleName(locale_value));
   long i;
-  long num = _locale_list.getNum();
+  long num = _locale_list._num;
+  WSClocale** locales = (WSClocale**)_locale_list._data;
   for(i=0; i<num; i++){
-    WSClocale* locale = (WSClocale*)_locale_list[i];
-    if (locale != NULL && !strcmp(locale->getLocaleName(),WSGIappCodeConvert()->getSystemLocaleName())){
+    WSClocale* locale = locales[i];
+    if (locale != NULL && !strcmp(locale->_name,WSGIappCodeConvert()->getSystemLocaleName())){
       _default_locale = locale;
       break;
     }
@@ -127,7 +127,7 @@ char* WSClocaleSet::getDefaultLocaleName(){
 //printf("WSClocaleSet::getDefaultLocaleName() 0x%x\n",_default_locale);
 //printf("WSClocaleSet::getDefaultLocaleName() 0x%x\n",_default_locale->getLocaleName());
 //printf("WSClocaleSet::getDefaultLocaleName() #%s#\n",_default_locale->getLocaleName());
-  return _default_locale->getLocaleName();
+  return _default_locale->_name;
 }
 
 WSClocale* WSClocaleSet::getDefaultLocale(){
@@ -137,11 +137,12 @@ WSClocale* WSClocaleSet::getDefaultLocale(){
   return _default_locale;
 }
 long WSClocaleSet::setDefaultLocale(char* locale_str){
-  long num = _locale_list.getNum();
+  long num = _locale_list._num;
   long i;
+  WSClocale** locales = (WSClocale**)_locale_list._data;
   for(i=0; i<num; i++){
-    WSClocale* locale = (WSClocale*)_locale_list[i];
-    if (locale != NULL && !strcmp(locale->getLocaleName(),locale_str)){
+    WSClocale* locale = locales[i];
+    if (locale != NULL && !strcmp(locale->_name,locale_str)){
       _default_locale = locale;
       return WS_NO_ERR;
     }
@@ -152,16 +153,18 @@ long WSClocaleSet::setDefaultLocale(char* locale_str){
   return WS_NO_ERR;
 }
 WSClocale* WSClocaleSet::getLocale(char* locale_str){
-  long num = _locale_list.getNum();
+  long num = _locale_list._num;
   long i;
+  WSClocale** locales = (WSClocale**)_locale_list._data;
   for(i=0; i<num; i++){
-    WSClocale* locale = (WSClocale*)_locale_list[i];
-    if (locale != NULL && !strcmp(locale->getLocaleName(),locale_str)){
+    WSClocale* locale = locales[i];
+    if (locale != NULL && !strcmp(locale->_name,locale_str)){
       return locale;
     }
   }
   return NULL;
 }
+
 void WSClocaleSet::setLocaleString(char* locale_str,char* index,char* str){
   WSClocale* locale = getLocale(locale_str);
 //printf("WSClocaleSet::setLocaleString locale=0x%x\n",locale);
@@ -182,10 +185,12 @@ char* WSClocaleSet::getLocaleString(char* locale_str,char* index,long encode){
 }
 WSClocale::WSClocale(){
 //printf("this=0x%x\n",this);
+  _name = NULL;
   _locale = WS_EN_NONE;
   long i;
   for(i=0; i<WS_MAX_HASH_VALUE; i++){
     _hash[i] = NULL;
+    _hash_string[i] = NULL;
   }
 }
 WSClocale::~WSClocale(){
@@ -194,6 +199,18 @@ WSClocale::~WSClocale(){
     if (_hash[i] != NULL){
       WSCindexData* list = (WSCindexData*)_hash[i];
       long num = list->getNum();
+#if 0
+      long j;
+      for(j=0; j<num; j++){
+        WSCstring* item = (WSCstring*)list->getData(j);
+        delete item;
+      }
+#endif
+      delete list;
+    }
+    if (_hash_string[i] != NULL){
+      WSCindexData* list = (WSCindexData*)_hash_string[i];
+      long num = list->getNum();
       long j;
       for(j=0; j<num; j++){
         WSCstring* item = (WSCstring*)list->getData(j);
@@ -202,6 +219,10 @@ WSClocale::~WSClocale(){
       delete list;
     }
   }
+  if (_name != NULL){
+    delete _name;
+    _name = NULL;
+  }
 }
 void WSClocale::setLocaleName(char* lname){
 //printf("this=0x%x\n",this);
@@ -212,11 +233,16 @@ void WSClocale::setLocaleName(char* lname){
     _locale = WS_EN_ISO8859_1; 
   }
 //printf("_locale=%d name=%s\n",_locale,WSGFgetLocaleName(_locale));
-  _name.setString(lname);
+//  _name.setString(lname);
+  if (_name != NULL){
+    delete _name;
+    _name = NULL;
+  }
+  _name = WSGFstrdup(lname);
 }
 
 char* WSClocale::getLocaleName(){
-  return (char*)_name;
+  return _name;
 }
 void WSClocale::setString(char* index,char* str){
 //printf("this=0x%x\n",this);
@@ -224,40 +250,29 @@ void WSClocale::setString(char* index,char* str){
   WSCindexData* list = _hash[val];
   if (list == NULL){
     list = new WSCindexData();
+    list->setNoAllocIndex(True);
     _hash[val] = list;
   }
+#if 0
   WSCstring* str2 = new WSCstring;
 //printf("setstring _locale=%d name=%s\n",_locale,WSGFgetLocaleName(_locale));
   str2->setString(str,_locale);
   list->setData(index,str2);
 //printf("this=0x%x index=%s str=%s list=0x%x hash=%d\n",this,index,str,list,val);
+#endif
+  list->setData(index,str);
 }
 char* WSClocale::getString(char* index,long encode){
   long val = WSGFgetHashValue(index);
 
   WSCindexData* list = _hash[val];
-//printf("this=0x%x index=%s list=0x%x hash=%d\n",this,index,list,val);
-  if (list == NULL){
-    WSCstring tmp(index);
+  char* ret = NULL;
+  if (list == NULL || (ret = (char*)list->getData(index)) == NULL ){
+    WSCstring tmp;
+    tmp.setStaticString(index);
     if (tmp.isExist(":WSSTR") <0){
     }else{
-//      if (!strcmp(getLocaleName(),"UTF8")){
-//        WSClocale* locale = WSGIappLocaleSet()->getLocale(WSN_EN_ISO8859_1);
-        WSClocale* locale = WSGIappLocaleSet()->getLocale("en_US");
-        if (locale != NULL && this != locale){
-          return locale->getString(index,encode);
-        }
-fprintf(stderr,"WSClocale::getString locale=%s index=%s not found.\n",getLocaleName(),index);
-//      }
-    }
-    return index;
-  }
-  WSCstring* ret = (WSCstring*)list->getData(index);
-  if (ret == NULL){
-    WSCstring tmp(index);
-    if (tmp.isExist(":WSSTR") <0){
-    }else{
-      WSClocale* locale = WSGIappLocaleSet()->getLocale(WSN_EN_ISO8859_1);
+      WSClocale* locale = WSGIappLocaleSet()->getLocale("en_US");
       if (locale != NULL && this != locale){
         return locale->getString(index,encode);
       }
@@ -265,7 +280,33 @@ fprintf(stderr,"WSClocale::getString locale=%s index=%s not found.\n",getLocaleN
     }
     return index;
   }
-  return ret->getString(encode);
+  if (encode == WS_EN_DEFAULT){
+    encode = _wsgv_default_encoding;
+  } 
+  if (encode == WS_EN_LOCALE){
+    encode = _wsgv_system_default_encoding;
+  } 
+  if (encode == WS_EN_NONE){
+    return ret;
+  }
+
+  if (_locale == encode){
+    return ret;
+  }
+
+  WSCindexData* list2 = _hash_string[val];
+  if (list2 == NULL){
+    list2 = new WSCindexData();
+    list2->setNoAllocIndex(True);
+    _hash_string[val] = list2;
+  }
+  WSCstring* str2 = (WSCstring*)list2->getData(index);
+  if (str2 == NULL){
+    str2 = new WSCstring;
+    str2->setString(ret,_locale);
+    list2->setData(index,str2);
+  }
+  return str2->getString(encode);
 }
 char* WSGFgettext(char* index){
 //printf("WSGFgettext index=%s\n",index);
@@ -280,7 +321,8 @@ fprintf(stderr,"ERROR. WSGFgettext locale C does not found.\n");
   }
 //  return locale->getString(index,WSGIappLocaleSet()->getSystemLocaleEncoding());
 //printf("WSGFgettext %d index=%s %s\n",WSGIappLocaleSet()->getDefaultEncoding(),index,locale->getString(index,WSGIappLocaleSet()->getDefaultEncoding()));
-  return locale->getString(index,WSGIappLocaleSet()->getDefaultEncoding());
+//  return locale->getString(index,WSGIappLocaleSet()->getDefaultEncoding());
+  return locale->getString(index,_wsgv_default_encoding);
 }
 long WSClocaleSet::getSystemLocaleEncoding(){
   return WSGIappCodeConvert()->getSystemEncoding();
@@ -296,10 +338,25 @@ long WSClocaleSet::getDefaultEncoding(){
 }
 void WSClocaleSet::setDefaultEncoding(long encoding){
   _encoding = encoding;
+  if (WSGIappLocaleSet() == this){
+    _wsgv_default_encoding = encoding;
+  }
 }
 void WSGFsetDefaultEncoding(long en){
+  _wsgv_default_encoding = en;
   WSGIappLocaleSet()->setDefaultEncoding(en);
 }
+long WSGFgetDefaultEncoding(){
+  return _wsgv_default_encoding;
+}
+void WSGFsetSystemDefaultEncoding(long en){
+  _wsgv_system_default_encoding = en;
+}
+long WSGFgetSystemDefaultEncoding(){
+  return _wsgv_system_default_encoding;
+}
+
+
 
 WSDcodeConvert::WSDcodeConvert(){
   strcpy(_locale_name,"en_US");
@@ -309,6 +366,7 @@ WSDcodeConvert::~WSDcodeConvert(){
 
 void WSDcodeConvert::setSystemLocaleName(char* val,long encoding){
   char buf[32];
+  memset(buf,0,sizeof(buf));
   strcpy(buf,val);
   int i;
   for(i=0; i<32; i++){
