@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCmessageDialog.h>
@@ -62,7 +58,7 @@ WSCmessageDialog* WSGIappMessageDialog(){
 WSCmessageDialog::WSCmessageDialog(WSCbase* base,char* objname):
                 WSCbaseDialog(base,objname){
 
-  _label_string = WSGFstrdup("");
+  _label_string = NULL;
   _label = NULL;
   _area1 = NULL;
   _no = 0;
@@ -80,12 +76,12 @@ WSCmessageDialog::WSCmessageDialog(WSCbase* base,char* objname):
 
 WSCmessageDialog::~WSCmessageDialog(){
   if (_label_string != NULL){
-    delete _label_string;
+    delete[] _label_string;
     _label_string = NULL;
   }
 }
 
-WSMFproperty( WSCmessageDialog, WSNlabelString, char*, _label_string,WSGFstrdup("") );
+WSMFproperty( WSCmessageDialog, WSNlabelString, char*, _label_string,NULL);
 WSMFproperty( WSCmessageDialog, WSNworkBackColor,short, _wbg_no ,WS_DF_DARKBACKCOLOR);
 
 void WSCmessageDialog::setWorkWSNlabelString(char* str){
@@ -133,7 +129,7 @@ long WSCmessageDialog::initialize(){
 //  _label->setPropertyV(WSNforeColor,"black");
   _label->setPropertyV(WSNmaxLength,(WSCushort)128);
 //  _label->setPropertyV(WSNbackColor,WSGFcolor("gray80"));
-  _label->setPropertyV(WSNlabelString,_label_string);
+//  _label->setPropertyV(WSNlabelString,_label_string);
   _label->setPropertyV(WSNshadowType,(char)WS_SHADOW_TRANS);
   _label->setVisible(True);
 
