@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCmfile.h>
@@ -50,6 +46,15 @@ long WSCmfile::open(void* data,long len){
   _buf_len = len;
   return WS_NO_ERR;
 }
+long WSCmfile::setBuf(void* data,long len){
+  if (_buf != NULL){
+    delete _buf;
+  }
+  _buf = (char*)data;
+  _seek_pt = 0;
+  _buf_len = len;
+  return WS_NO_ERR;
+}
 
 long WSCmfile::close(){
   if (_buf != NULL){
@@ -71,6 +76,7 @@ long WSCmfile::read(void* buf,long len){
     return 0;
   }
   memcpy(buf,&_buf[_seek_pt],clen);
+  _seek_pt += clen;
   return clen;
 }
 
@@ -86,6 +92,7 @@ long WSCmfile::write(void* buf,long len){
     _buf = newbuf;
   }
   memcpy(&_buf[_seek_pt],buf,clen);
+  _seek_pt += clen;
   return clen;
 }
 
