@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCnwbase.h>
@@ -33,6 +29,7 @@
 #include <WSCcolorSet.h>
 #include <WSCblink.h>
 #include <WSDtimer.h>
+#include <WSDmouse.h>
 
 WSMFguiClassInitialize(WSCnwbase, WSCbase);
 WSMFversion(WSCnwbase, WSCbase);
@@ -51,6 +48,9 @@ WSCnwbase::WSCnwbase(WSCbase* base, char* objname): WSCbase(base, objname){
   _h = 30;
   _bl_fl = False;
   _tw_fl = False;
+#ifndef NO_NWBASE_MOUSE_PROPERTY
+  _mouse_no = 0;
+#endif
 //  _fore_color = WSGFcolor("black");
 //  _blink_color = WSGFcolor("gray85");
   _fore_color = WS_DF_NWFORECOLOR;
@@ -59,6 +59,7 @@ WSCnwbase::WSCnwbase(WSCbase* base, char* objname): WSCbase(base, objname){
   _bl_refreshing = False;
   _vis = False;
   _sensitive = True;
+  _sensitive_work = True;
   _event_mask |= WSEV_EXPOSE;
   _mouse_grabed = 0;
   _anchor_top = 0;
@@ -82,7 +83,7 @@ WSCnwbase::WSCnwbase(WSCbase* base, char* objname): WSCbase(base, objname){
     WSMFpropertyCreate(WSNvis,             WSCbool, _vis,WSSvis );
     WSMFpropertySetSelection(WSRbool2, WSRbool2D);
 
-    WSMFpropertyCreate(WSNdet,             WSCbool, _sensitive,WSSdet);
+    WSMFpropertyCreate(WSNdet,             WSCbool, _sensitive_work,WSSdet);
     WSMFpropertySetSelection(WSRbool1, WSRbool1D);
     WSMFpropertyCreate(WSNx,             short,  _x,WSSx  );
     WSMFpropertyCreate(WSNy,             short,  _y,WSSy  );
@@ -110,15 +111,13 @@ WSCnwbase::WSCnwbase(WSCbase* base, char* objname): WSCbase(base, objname){
     WSMFpropertySetSelection(WSRbool3, WSRbool3D);
     WSMFpropertyCreate(WSNanchorRightFlag,WSCbool,_anchor_right_use,WSSanchorRightFlag);
     WSMFpropertySetSelection(WSRbool3, WSRbool3D);
-
+#ifndef NO_REMOTE_INSTANCE
     WSMFpropertyCreate(WSNexport,WSCbool,_export_,WSSexport);
     WSMFpropertySetSelection(WSRbool1, WSRbool1D);
-
-#ifdef WS_USE_MOUSE_IMAGE
-    WSMFpropertyCreate(WSNmouseImg,      short,  _mouse_img,WSSmouseImg );
-    WSMFpropertySetup(NULL, NULL);
 #endif
-
+#ifndef NO_NWBASE_MOUSE_PROPERTY
+    WSMFpropertyCreate(WSNmouse,   WSCushort,  _mouse_no,WSSmouse );
+#endif
     WSMFaddTrigger(WSEV_INITIALIZE  );
     WSMFaddTrigger(WSEV_EXPOSE      );
     WSMFaddTrigger(WSEV_VISIBLE_CH  );
@@ -133,12 +132,12 @@ WSCnwbase::WSCnwbase(WSCbase* base, char* objname): WSCbase(base, objname){
   WSMFpropertyCreateEnd
 }
 
-WSMFproperty( WSCnwbase, WSNname, char*, _instance_name ,WSGFstrdup(""));
-WSMFproperty( WSCnwbase, WSNuserString, char*, _user_string ,WSGFstrdup(""));
+WSMFproperty( WSCnwbase, WSNname, char*, _instance_name ,NULL);
+WSMFproperty( WSCnwbase, WSNuserString, char*, _user_string ,NULL);
 WSMFproperty( WSCnwbase, WSNuserValue,  long,  _user_value  ,0);
 WSMFproperty( WSCnwbase, WSNblinkRefreshing, WSCbool, _bl_refreshing ,False    );
 WSMFproperty( WSCnwbase, WSNvis,   WSCbool,  _vis ,False   );
-WSMFproperty( WSCnwbase, WSNdet,   WSCbool,  _sensitive  ,True   );
+WSMFproperty( WSCnwbase, WSNdet,   WSCbool,  _sensitive_work  ,True   );
 WSMFproperty( WSCnwbase, WSNx,           short,          _x           ,0     );
 WSMFproperty( WSCnwbase, WSNy,           short,          _y           ,0     );
 WSMFproperty( WSCnwbase, WSNwidth,       WSCushort, _w           ,100   );
@@ -150,8 +149,8 @@ WSMFproperty( WSCnwbase, WSNtwinBlink,   WSCbool,        _tw_fl       ,False );
 //WSMFproperty( WSCnwbase, WSNblinkColor,  short,        _blink_color   ,WSGFcolor("gray85") );
 WSMFproperty( WSCnwbase, WSNblinkColor,  short,          _blink_color       ,WS_DF_NWBACKCOLOR);
 WSMFproperty( WSCnwbase, WSNblinkRate,   WSCuchar,    _blink_rate     ,WS250MS );
-#ifdef WS_USE_MOUSE_IMAGE
-WSMFproperty( WSCnwbase, WSNmouseImg,    short,          _mouse_img   ,1     );
+#ifndef NO_NWBASE_MOUSE_PROPERTY
+WSMFproperty( WSCnwbase, WSNmouse,    WSCushort,          _mouse_no   ,0     );
 #endif
 WSMFproperty( WSCnwbase, WSNanchorTop,WSCushort,_anchor_top,0 );
 WSMFproperty( WSCnwbase, WSNanchorBottom,WSCushort,_anchor_bottom,0 );
@@ -161,8 +160,9 @@ WSMFproperty( WSCnwbase, WSNanchorTopFlag,WSCbool,_anchor_top_use,False );
 WSMFproperty( WSCnwbase, WSNanchorBottomFlag,WSCbool,_anchor_bottom_use,False );
 WSMFproperty( WSCnwbase, WSNanchorLeftFlag,WSCbool,_anchor_left_use,False );
 WSMFproperty( WSCnwbase, WSNanchorRightFlag,WSCbool,_anchor_right_use,False );
+#ifndef NO_REMOTE_INSTANCE
 WSMFproperty( WSCnwbase, WSNexport,WSCbool,_export_,False );
-
+#endif
 void WSCnwbase::setWorkWSNname(char*){}
 void WSCnwbase::getWorkWSNname(char**){}
 void WSCnwbase::setWorkWSNuserString(char*){}
@@ -184,33 +184,37 @@ void WSCnwbase::setWorkWSNvis(WSCbool data){
 
 void WSCnwbase::getWorkWSNvis(WSCbool*){} 
 void WSCnwbase::setWorkWSNdet(WSCbool data){
+#ifndef WS_EMBED
   if (getPropertyEditMode() == False){
-    if (data == False){
-      _sensitive = True;
-    }else{
-      _sensitive = False;
-    }
+#endif
     setSensitive(data);
-  }else{
-    long val =(long)data;
-    setUserData(WS_DET,(void*)val);
+#ifndef WS_EMBED
+//  }else{
+//    long val =(long)data;
+//    setUserData(WS_DET,(void*)val);
   }
+#endif
 }
 
 void WSCnwbase::getWorkWSNdet(WSCbool* data){
-  if (getPropertyEditMode() != False){
-    long tmp = (long)getUserData(WS_DET);
-    WSCbool fl = (WSCbool)tmp;
-    *data = fl;
-  }
+#ifndef WS_EMBED
+//  if (getPropertyEditMode() != False){
+//    long tmp = (long)getUserData(WS_DET);
+//    WSCbool fl = (WSCbool)tmp;
+//    *data = fl;
+//  }
+#endif
 }
 
 void WSCnwbase::setWorkWSNx(short){
-  WSCbase* parent = getParent();
-  if (parent != NULL && parent->adjustAnchor() != False){
-    if (_anchor_left_use != False){
-      _x = _anchor_left;
-    }else if (_anchor_right_use != False){
+  if (_anchor_left_use != False){
+    WSCbase* parent = getParent();
+    if (parent != NULL && parent->adjustAnchor() != False){
+        _x = _anchor_left;
+    }
+  }else if (_anchor_right_use != False){
+    WSCbase* parent = getParent();
+    if (parent != NULL && parent->adjustAnchor() != False){
       parent->setGeometryChanged(True);
       parent->needUpdate();
     }
@@ -220,11 +224,14 @@ void WSCnwbase::setWorkWSNx(short){
 
 void WSCnwbase::getWorkWSNx(short*){}
 void WSCnwbase::setWorkWSNy(short){
-  WSCbase* parent = getParent();
-  if (parent != NULL && parent->adjustAnchor() != False){
-    if (_anchor_top_use != False){
+  if (_anchor_top_use != False){
+    WSCbase* parent = getParent();
+    if (parent != NULL && parent->adjustAnchor() != False){
       _y = _anchor_top;
-    }else if (_anchor_bottom_use != False){
+    }
+  }else if (_anchor_bottom_use != False){
+    WSCbase* parent = getParent();
+    if (parent != NULL && parent->adjustAnchor() != False){
       parent->setGeometryChanged(True);
       parent->needUpdate();
       return; 
@@ -235,9 +242,9 @@ void WSCnwbase::setWorkWSNy(short){
 
 void WSCnwbase::getWorkWSNy(short*){}
 void WSCnwbase::setWorkWSNwidth(WSCushort){
-  WSCbase* parent = getParent();
-  if (parent != NULL && parent->adjustAnchor() != False){
-    if (_anchor_left_use != False && _anchor_right_use != False){
+  if (_anchor_left_use != False && _anchor_right_use != False){
+    WSCbase* parent = getParent();
+    if (parent != NULL && parent->adjustAnchor() != False){
       parent->setGeometryChanged(True);
       parent->needUpdate();
     }
@@ -247,9 +254,9 @@ void WSCnwbase::setWorkWSNwidth(WSCushort){
 
 void WSCnwbase::getWorkWSNwidth(WSCushort*){}
 void WSCnwbase::setWorkWSNheight(WSCushort){
-  WSCbase* parent = getParent();
-  if (parent != NULL && parent->adjustAnchor() != False){
-    if (_anchor_top_use != False && _anchor_bottom_use != False){
+  if (_anchor_top_use != False && _anchor_bottom_use != False){
+    WSCbase* parent = getParent();
+    if (parent != NULL && parent->adjustAnchor() != False){
       parent->setGeometryChanged(True);
       parent->needUpdate();
     }
@@ -258,15 +265,12 @@ void WSCnwbase::setWorkWSNheight(WSCushort){
 }
 
 void WSCnwbase::getWorkWSNheight(WSCushort*){}
-
-#ifdef WS_USE_MOUSE_IMAGE
-void WSCnwbase::setWorkWSNmouseImg(short){
+#ifndef NO_NWBASE_MOUSE_PROPERTY
+void WSCnwbase::setWorkWSNmouse(WSCushort){
 }
-
-void WSCnwbase::getWorkWSNmouseImg(short*){
+void WSCnwbase::getWorkWSNmouse(WSCushort*){
 }
 #endif
-
 void WSCnwbase::setWorkWSNforeColor(short){}
 void WSCnwbase::getWorkWSNforeColor(short*){}
 
@@ -381,14 +385,18 @@ void WSCnwbase::setWorkWSNanchorRightFlag(WSCbool data){
 }
 void WSCnwbase::getWorkWSNanchorRightFlag(WSCbool* data){}
 
+#ifndef NO_REMOTE_INSTANCE
 void WSCnwbase::setWorkWSNexport(WSCbool data){
   setExported(data);
 }
 void WSCnwbase::getWorkWSNexport(WSCbool* data){
 }
+#endif
 
 WSCnwbase::~WSCnwbase(){
+#ifndef WS_EMBED
   clearGriphand();
+#endif
 //  WSClistData* ghlist = (WSClistData*)getUserData("GH-LIST");
 //  if (ghlist != NULL){
 //    delete ghlist;
@@ -399,7 +407,7 @@ WSCnwbase::~WSCnwbase(){
     WSGIappBlink()->delBlinkInstance(this);
   }
   if (_user_string != NULL){
-    delete _user_string;
+    delete[] _user_string;
   }
 }
 
@@ -418,12 +426,27 @@ long WSCnwbase::_device_initialize(){
 
 long WSCnwbase::execEventProc(long ev, void* data){
   if (ev == WSEV_EXPOSE){
-    draw();
+    if (_x + _w < 0 || _y + _h < 0){
+      return WS_NO_ERR;
+    }
+    WSCbool absolute = getAbsoluteDraw();
+    long ret = draw();
+    if (ret == WS_ERR){
+      return WS_ERR;
+    }
+    WSDdev* dev = getowndev();
+    if (dev == NULL){
+      return WS_ERR;
+    }
+    if (absolute == False && dev->isExposed(_x,_y,_w,_h) == False){
+      return WS_NO_ERR;
+    }
     if (data == NULL){
+
       WSCrect area;
       area.setRect(_x,_y,_w,_h);
       onExpose(&area); 
-      draw();
+//      draw();
       if (_expose_op == 1){
         execProcedure(ev);
       }
@@ -558,7 +581,7 @@ WSMFtrace("WSCnwbase::addProcedure trigger=%d not supported.\n", trg);
     ac->setEffectiveTriggerMask( WSEV_MOUSE_MOVE_BIT | WSEV_MOUSE_IN_BIT | WSEV_MOUSE_OUT_BIT);
 
     _event_mask |= ac->getEffectiveTriggerMask();
-    _add_prcs(&_prc_list, ac);
+    _add_prcs(ac);
 
     WSDdev* dev = getowndev();
     if (dev != NULL){
@@ -776,6 +799,88 @@ void WSCnwbase::_adjust_for_anchors(WSCushort w,WSCushort h){
     return;
   }
 }
+#ifndef NO_NWBASE_MOUSE_PROPERTY
+WSCbase* WSCnwbase::_get_child_with_point(WSCpoint* pt,WSCbool in){
+  if (getParent() == NULL){
+    return NULL;
+  }
+  WSClistData* children = getParent()->getChildrenPtr();
+  if (children == NULL){
+    return NULL;
+  }
+  int num = children->getNum();
+  WSCbase* target = NULL;
+  int i;
+  WSCbase** child_l = (WSCbase**)children->getBuf();
+  for(i=0; i<num; i++){
+    WSCbase* child = child_l[i];
+    short x = 0;
+    short y = 0;
+    WSCushort w = 0;
+    WSCushort h = 0;
+    if (child->existProperty(WSNx) != False){
+      child->getPropertyV(WSNx,&x);
+    }
+    if (child->existProperty(WSNy) != False){
+      child->getPropertyV(WSNy,&y);
+    }
+    if (child->existProperty(WSNwidth) != False){
+      child->getPropertyV(WSNwidth,&w);
+    }
+    if (child->existProperty(WSNheight) != False){
+      child->getPropertyV(WSNheight,&h);
+    }
+    if (in == False){
+      if (x < pt->x && pt->x < x+w && y < pt->y && pt->y < y+h){
+        target = child;
+      }
+    }else{
+      if (x <= pt->x && pt->x <= x+w && y <= pt->y && pt->y <= y+h){
+        target = child;
+      }
+    }
+  }
+  return target;
+}
+void WSCnwbase::onMouseIn(WSCpoint* pt){
+  WSDdev* pdev = getParentDev();
+  if (pdev != NULL && pdev->getVisible() != False){
+//printf("WSCnwbase::onMouseIn val=%d\n",_mouse_no);
+    WSCpoint pt2;
+    pt2.x = pt->x + _x;
+    pt2.y = pt->y + _y;
+    WSCbase* target = _get_child_with_point(&pt2,True);
+    if (target == NULL || target == this){
+      pdev->setValue(WSDEV_MOUSE_NO,&_mouse_no);
+    }
+  }
+}
+void WSCnwbase::onMouseOut(){
+  WSCbase* parent = getParent();
+  if (parent == NULL){
+    return;
+  }
+  WSDdev* pdev = getParentDev();
+  if (pdev != NULL && pdev->getVisible() != False){
+    WSCpoint pt2;
+    short x,y;
+    WSGIappMouse()->getMousePosition(&x,&y,this);
+    pt2.x = x + _x;
+    pt2.y = y + _y;
+    WSCbase* target = _get_child_with_point(&pt2,False);
+    WSCushort val = 0;
+    if (target != NULL && target->existProperty(WSNmouse) != False){
+      val = target->getProperty(WSNmouse);
+//printf("WSCnwbase::onMouseOut val=%d\n",val);
+      pdev->setValue(WSDEV_MOUSE_NO,&val);
+    }else if (parent->existProperty(WSNmouse) != False){
+      val = parent->getProperty(WSNmouse);
+//printf("WSCnwbase::onMouseOut val2=%d\n",val);
+      pdev->setValue(WSDEV_MOUSE_NO,&val);
+    }
+  }
+}
+#endif
 long WSCnwbase::getBlinkRate(){
   return (long)_blink_rate;
 }
