@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCoption.h>
@@ -50,7 +46,7 @@ WSCoption::WSCoption(WSCbase* base, char* objname):WSCvlabel(base, objname){
   _menu = NULL;
   _press_state = False;
   _status_array = NULL;
-  _menu_str = WSGFstrdup("");
+  _menu_str = NULL;
   _value = 0;
   _shadow_type = WS_SHADOW_OUT;
   WSGIappObjectList()->addEvent(this,WSEV_GUI_POLICY_CH);
@@ -90,7 +86,7 @@ void WSCoption::setWorkWSNmenuItems(char* data){
   str.replaceString(",","\n",0);
 
   if (_status_array != NULL) {
-    delete _status_array;
+    delete[] _status_array;
     _status_array = NULL;
   }
   long num = str.getWords("\n");    
@@ -105,7 +101,7 @@ void WSCoption::setWorkWSNmenuItems(char* data){
     _status_array[i] = True;
     WSCstring sect = str.getWord(i,"\n");
     sect.replaceString(":","\n",0);
-    long value;
+    long value = 0;
     WSCstring sect_value = sect.getWord(1, "\n");
     sscanf(sect_value.getString(), "%ld", &value);
 
@@ -158,13 +154,15 @@ WSCoption::~WSCoption(){
     delete _menu;
   }
   if (_status_array != NULL) {
-    delete _status_array;
+    delete[] _status_array;
     _status_array = NULL;
   }
-  delete _menu_str;
+  delete[] _menu_str;
 }
 
 long WSCoption::initialize(){
+  WSCvlabel::initialize();
+
   _menu = new WSCpopupFrame(NULL, "popup-m");
   _menu->initialize();
   _menu->setInternalObject(True);
@@ -175,7 +173,6 @@ long WSCoption::initialize(){
   _menu->setEmboss(_emboss);
 
 
-  WSCvlabel::initialize();
   setProperty(WSNmenuItems,"item1:1:ep1_name,item2:2:ep2_name,item3:3:ep3_name");
 //  setProperty(WSNmenuItems,"item1:1:,item2:2:,item3:3:");
   return WS_NO_ERR;
@@ -187,6 +184,7 @@ long WSCoption::_device_initialize(){
     return WS_ERR;
   }
   dev->setEnableEventBit( WSEV_MOUSE_MOVE_BIT | WSEV_MOUSE_OUT_BIT |
+                          WSEV_MOUSE_IN_BIT |
                           WSEV_MOUSE_PRESS_BIT  | WSEV_MOUSE_RELEASE_BIT );
   return WS_NO_ERR;
 }
@@ -259,8 +257,10 @@ long WSCoption::draw(){
       st.label_string = _label_string;
       st.label_pixmap = _label_pixmap;
       st.blink_pixmap = _blink_pixmap;
+#ifndef NO_GRADATION
       st.gradation = _grad_no;
       st.gradation_margin = _grad_margin;
+#endif
       st.margin_left = _margin_left;
       st.margin_right = _margin_right;
       st.margin_top = _margin_top;
@@ -322,11 +322,15 @@ long WSCoption::draw(){
       dev->setBackColor(fgno);
     }
   }
+#ifndef NO_GRADATION
   if (_grad_no == 0){
+#endif
     dev->drawFillRect(st, st, w-st*2, h-st*2);
+#ifndef NO_GRADATION
   }else{
     dev->drawGradation(_grad_no,tsno,bgno,bsno,st, st, w-st*2, h-st*2,_grad_margin);
   }
+#endif
 
   WSDimage* image;
   if (bldraw && blinktype != WS_BACK ){
@@ -346,15 +350,25 @@ long WSCoption::draw(){
     if (_press_state != False) {
       type = WS_SHADOW_IN;
     }
+#ifndef WS_EMBED
     if (getSensitive() == False ||
-       (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0)){
+       (getPropertyEditMode() != False && _sensitive_work == False)){
       type = WS_SHADOW_OUT;
     }
+#else
+    if (getSensitive() == False){
+      type = WS_SHADOW_OUT;
+    }
+#endif
     WSGFdrawShadow(dev, type, st, fgno, tsno, bsno, bgno,0, 0, w, h);
   }
 
+#ifndef WS_EMBED
   if ( getSensitive() == False ||
-       (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0)){
+       (getPropertyEditMode() != False && _sensitive_work == False)){
+#else
+  if ( getSensitive() == False){
+#endif
     if (_emboss != False){
       if (bldraw && blinktype != WS_BACK && twinblink != False) {
         dev->setForeColor(blno);
@@ -380,8 +394,12 @@ long WSCoption::draw(){
     if (bldraw && blinktype != WS_BACK && twinblink != False) {
       dev->setForeColor(blbgno);
     } else {
+#ifndef WS_EMBED
       if( _emboss != False && (getSensitive() == False ||
-          (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0))){
+          (getPropertyEditMode() != False && _sensitive_work == False))){
+#else
+      if( _emboss != False && getSensitive() == False){
+#endif
         dev->setForeColor(bsno);
       }else{
         dev->setForeColor(bgno);
@@ -392,8 +410,12 @@ long WSCoption::draw(){
     if(bldraw && blinktype != WS_BACK && twinblink != False) {
       dev->setForeColor(blno);
     }else{
+#ifndef WS_EMBED
       if ( _emboss != False && (getSensitive() == False ||
-        (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0))){
+        (getPropertyEditMode() != False && _sensitive_work == False))){
+#else
+      if ( _emboss != False && getSensitive() == False){
+#endif
         dev->setForeColor(bsno);
       }else{
         dev->setForeColor(fgno);
@@ -431,8 +453,12 @@ void WSCoption::onMousePress(WSCpoint* point){
     return;
   }
   if (_press_state == False && (long)getUserData(WS_PM_POPUPED) == 0){
+#ifndef WS_EMBED
     if (getPropertyEditMode() == False ||
         (WSGIappMouse()->getTargetBtn() != WS_MOUSE_BTN3)){
+#else
+    if ( WSGIappMouse()->getTargetBtn() != WS_MOUSE_BTN3){
+#endif
       _menu_popup(point);
       _press_state = True;
       setAbsoluteDraw(True);
@@ -511,7 +537,7 @@ WSMFtrace("WSCoption::addProcedure trigger=%d not supported.\n", ac->getTrigger(
                  WSEV_MOUSE_RELEASE_BIT | WSEV_MOUSE_OUT_BIT );
 
     _event_mask |= ac->getEffectiveTriggerMask();
-    _add_prcs(&_prc_list, ac);
+    _add_prcs(ac);
 
     WSDdev* dev = getowndev();
     if (dev != NULL){
@@ -602,11 +628,13 @@ void WSCoption::_menu_popup(WSCpoint* ){
 
     WSCushort dispw = WSGIappDev()->getWidth();
     WSCushort disph = WSGIappDev()->getHeight();
-    if (px + ww > dispw) {
-      px -= (px + ww - dispw);
-    }
-    if (py + hh > disph) {
-      py -= (py + hh - disph);
+    if (px < dispw && py < disph){
+      if (px + ww > dispw) {
+        px -= (px + ww - dispw);
+      }
+      if (py + hh > disph) {
+        py -= (py + hh - disph);
+      }
     }
     if (num > 0 && ww > 0){
       _menu->setPropertyV(WSNmenuItems, _menu_str);
@@ -662,16 +690,21 @@ long WSCoption::getItemValue(long pos){
 }
 
 void(*WSCoption::_ext_draw_proc)(void*) = NULL;
-WSCstring WSCoption::_policy_name;
+char* WSCoption::_policy_name = NULL;
 
 void WSCoption::onGuiPolicyChange(long policy){
   WSCvlabel::onGuiPolicyChange(policy);
   if (WSGIappDev()->getExtGuiPolicySelected() == False){
     _ext_draw_proc = NULL;
   }else{
-    if (!strcmp((char*)_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
+    if (_policy_name != NULL &&
+        !strcmp(_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
     }else{
       _ext_draw_proc = (void(*)(void*))WSGIappDev()->getExtGuiPolicyProc("WSCoption::draw");
+      if (_policy_name != NULL){
+        delete _policy_name;
+      }
+      _policy_name = WSGFstrdup(WSGIappDev()->getExtGuiPolicyName());
     }
   }
   if (getVisible() != False){
