@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCprocedure.h>
@@ -40,10 +36,10 @@ WSCprocedure::WSCprocedure(char* opname,long trg){
 }
 WSCprocedure::~WSCprocedure(){
   if (_func_name != NULL){
-    delete _func_name;
+    delete[] _func_name;
   }
   if (_prc_name != NULL){
-    delete _prc_name;
+    delete[] _prc_name;
   }
 }
 void WSCprocedure::setUseFile(WSCbool fl){
@@ -54,7 +50,7 @@ WSCbool WSCprocedure::getUseFile(){
 }
 long WSCprocedure::setProcName(char* str){
   if (_prc_name != NULL){
-    delete _prc_name;
+    delete[] _prc_name;
   }
   _prc_name = WSGFstrdup(str);
   return WS_NO_ERR;
@@ -93,15 +89,15 @@ long WSCprocedure::getTrigger(){
 }
 
 long WSCprocedure::setEffectiveTriggerMask(long ev){
- _effective_trg_mask = ev;
- return WS_NO_ERR;
+  _effective_trg_mask = ev;
+  return WS_NO_ERR;
 }
 long WSCprocedure::getEffectiveTriggerMask(){
   return _effective_trg_mask;
 }
 long WSCprocedure::setFunction( void(*proc)(WSCbase*),char* fname){
   if (_func_name != NULL){
-    delete _func_name;
+    delete[] _func_name;
   }
   _func_name = WSGFstrdup(fname);
   _function = (void*)proc;
