@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,56 +14,79 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCstring.h>
 #include <WSClistData.h>
 #include <WSClocaleSet.h>
 
+#define WSMFcodeCheck_SJIS(code) \
+ (((WSCuchar)(code) & 0x80) == 0x00 ? WS_ASCII: \
+ (((WSCuchar)(code) >=(WSCuchar)0xa0) && ((WSCuchar)(code) <= (WSCuchar)0xdf) ? WS_HKANA: \
+ (((WSCuchar)(code) < (WSCuchar)0xf0) ? WS_KANJI : WS_GAIJI)))
+
+extern long _wsgv_default_encoding;
+extern long _wsgv_system_default_encoding;
+
 long WSCstring::getLines(){
   return getWords("\n");
 }
 void WSCstring::clear(){
   if (_str != NULL){
 //WSMFtrace("WSCstring::clear #%s# deleted=0x%x\n",_str,_str);
-    _str[0] = 0;
-    delete[] _str;
+    if (_static == False){
+//      _str[0] = 0;
+      delete[] _str;
+    }
     _str = NULL;
   }
+  _static = False;
   _seek_pt = 0;
   _encoding = WS_EN_DEFAULT;
   _clear_list();
 }
 void WSCstring::_clear_list(){
+  _clear_list_ex();
+  _add_str_work_len = 0;
+  _add_str_work = 0;
+  if (_ucs2 != NULL){
+    delete[] _ucs2;
+    _ucs2 = NULL;
+  }
+}
+void WSCstring::_clear_list_ex(){
 //printf("WSCstring::_clear_list()...this=0x%x\n",this);
   long i;
-  long num = _tmp_list.getNum();
-  for(i=0; i<num; i++){
-    char* str = (char*)_tmp_list[i];
-    str[0] = 0;
-    delete str;
+  long num = _tmp_list._num;
+  if (num != 0){
+    char** strs = (char**)_tmp_list._data;
+    for(i=0; i<num; i++){
+      i++;
+      char* str = strs[i];
+//      str[0] = 0;
+      delete[] str;
 //WSMFtrace("WSCstring::_clear_list deleted=0x%x\n",str);
+    }
+    _tmp_list.clear();
   }
-  _tmp_list.clear();
-  _tmp_encode.clear();
   _chars = -1;
 }
-
 WSCstring::WSCstring(){
 //printf("WSCstring::WSCstring() this=0x%x\n",this);
   _str = NULL;
   _encoding = WS_EN_DEFAULT;
   _seek_pt = 0;
   _chars = -1;
+  _add_str_work_len = 0;
+  _add_str_work = 0;
+  _static = False;
+  _ucs2 = NULL;
+  _tmp_list._seg_size = 4;
 }
 
 WSCstring::WSCstring(const char* str,long encode){
@@ -77,31 +100,51 @@ WSCstring::WSCstring(const char* str,long encode){
 //printf("WSCstring::WSCstring(char*) enc=%d str=0x%x #%s#\n",_encoding,str,_str);
   _seek_pt = 0;
   _chars = -1;
+  _add_str_work_len = 0;
+  _add_str_work = 0;
+  _static = False;
+  _ucs2 = NULL;
 }
 WSCstring::WSCstring(WSCstring& str){
 //printf("WSCstring::WSCstring(WSCstirng& str) this=0x%x\n",this);
   _encoding = str._encoding;
+  _static = str._static;
   if (str._str != NULL){
-    _str = WSGFstrdup(str._str);
+    if (str._static){
+      _str = str._str;
+    }else{
+      _str = WSGFstrdup(str._str);
+    }
   }else{
     _str = NULL;
   }
 //printf("WSCstring::WSCstring(WSCstring&) enc=%d #%s#\n",_encoding,_str);
   _seek_pt = 0;
   _chars = str._chars;
+  _add_str_work_len = 0;
+  _add_str_work = 0;
+  _ucs2 = NULL;
 }
 
 WSCstring::WSCstring(const WSCstring& str){
 //printf("WSCstring::WSCstring(WSCstirng& str) this=0x%x\n",this);
   _encoding = str._encoding;
+  _static = str._static;
   if (str._str != NULL){
-    _str = WSGFstrdup(str._str);
+    if (str._static){
+      _str = str._str;
+    }else{
+      _str = WSGFstrdup(str._str);
+    }
   }else{
     _str = NULL;
   }
 //printf("WSCstring::WSCstring(WSCstring&) enc=%d #%s#\n",_encoding,_str);
   _seek_pt = 0;
   _chars = str._chars;
+  _add_str_work_len = 0;
+  _add_str_work = 0;
+  _ucs2 = NULL;
 }
 
 WSCstring::WSCstring(WSCvariant& str,long encode){
@@ -110,6 +153,10 @@ WSCstring::WSCstring(WSCvariant& str,long encode){
 //printf("WSCstring::WSCstring(WSCvariant) enc=%d #%s#\n",_encoding,_str);
   _seek_pt = 0;
   _chars = -1;
+  _add_str_work_len = 0;
+  _add_str_work = 0;
+  _static = False;
+  _ucs2 = NULL;
 }
 
 
@@ -120,6 +167,10 @@ WSCstring::WSCstring(const WSCvariant& str,long encode){
 //printf("WSCstring::WSCstring(WSCvariant) enc=%d #%s#\n",_encoding,_str);
   _seek_pt = 0;
   _chars = -1;
+  _add_str_work_len = 0;
+  _add_str_work = 0;
+  _static = False;
+  _ucs2 = NULL;
 }
   
 long WSCstring::operator ==(const WSCstring& src){
@@ -136,16 +187,24 @@ WSCstring& WSCstring::operator =(const WSCstring& src){
 //printf("WSCstring operator =WSCstring&\n");
   if (&src != this){
     if (_str != NULL){
-      delete _str;
+      if (_static == False){
+        delete[] _str;
+      }
       _str = NULL;
     }
+    _static = src._static;
     if (src._str != NULL){ 
-      _str = WSGFstrdup(src._str);
+      if (_static){
+        _str = src._str;
+      }else{
+        _str = WSGFstrdup(src._str);
+      }
     }
     _encoding = src._encoding;
     _seek_pt = 0;
     _clear_list();
     _chars = src._chars;
+    _ucs2 = NULL;
   }
   return *this;
 }
@@ -242,16 +301,22 @@ WSCstring::~WSCstring(){
 long WSCstring::_get_encoding() const{
   long encoding = _encoding;
   if (encoding == WS_EN_DEFAULT){
-    encoding = WSGIappLocaleSet()->getDefaultEncoding();
+//    encoding = WSGIappLocaleSet()->getDefaultEncoding();
+    encoding = _wsgv_default_encoding;
   }
   if (encoding == WS_EN_LOCALE){
-    encoding = WSGIappLocaleSet()->getSystemLocaleEncoding();
+//    encoding = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    encoding = _wsgv_system_default_encoding;
   } 
   return encoding;
 }
 void WSCstring::delLineFeed(){
   if (_str != NULL){
-    long cnt =0;
+    if (_static != False){
+      _str = WSGFstrdup(_str);
+      _static = False;
+    }
+    long cnt = 0;
     long max = strlen(_str);
     long i;
     for(i=0; i < max; i++){
@@ -262,17 +327,18 @@ void WSCstring::delLineFeed(){
     }
     _str[cnt] = 0;
     _clear_list();
-    _chars = -1;
     return;
   }
 }
 
 char* WSCstring::_get_utf8_string(char* str,long encode){
   if (encode == WS_EN_DEFAULT){
-    encode = WSGIappLocaleSet()->getDefaultEncoding();
+//    encode = WSGIappLocaleSet()->getDefaultEncoding();
+    encode = _wsgv_default_encoding;
   } 
   if (encode == WS_EN_LOCALE){
-    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+//    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    encode = _wsgv_system_default_encoding;
   } 
   int len = WSGIappCodeConvert()->codeConvert(NULL,0,WS_EN_UTF8,str,-1,encode);
   char* ret = new char[len+1];
@@ -281,10 +347,12 @@ char* WSCstring::_get_utf8_string(char* str,long encode){
 }
 char* WSCstring::_get_mb_string(char* str,long encode){
   if (encode == WS_EN_DEFAULT){
-    encode = WSGIappLocaleSet()->getDefaultEncoding();
+//    encode = WSGIappLocaleSet()->getDefaultEncoding();
+    encode = _wsgv_default_encoding;
   } 
   if (encode == WS_EN_LOCALE){
-    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+//    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    encode = _wsgv_system_default_encoding;
   } 
   int len = WSGIappCodeConvert()->codeConvert(NULL,0,encode,str,-1,WS_EN_UTF8);
   char* ret = new char[len+1];
@@ -293,16 +361,20 @@ char* WSCstring::_get_mb_string(char* str,long encode){
 }
 char* WSCstring::_convert_string(const char* str,long from_encode,long to_encode){
   if (to_encode == WS_EN_DEFAULT){
-    to_encode = WSGIappLocaleSet()->getDefaultEncoding();
+//    to_encode = WSGIappLocaleSet()->getDefaultEncoding();
+    to_encode = _wsgv_default_encoding;
   } 
   if (to_encode == WS_EN_LOCALE){
-    to_encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+//    to_encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    to_encode = _wsgv_system_default_encoding;
   } 
   if (from_encode == WS_EN_DEFAULT){
-    from_encode = WSGIappLocaleSet()->getDefaultEncoding();
+//    from_encode = WSGIappLocaleSet()->getDefaultEncoding();
+    from_encode = _wsgv_default_encoding;
   } 
   if (from_encode == WS_EN_LOCALE){
-    from_encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+//    from_encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    from_encode = _wsgv_default_encoding;
   } 
   int len = WSGIappCodeConvert()->codeConvert(NULL,0,to_encode,str,-1,from_encode);
   char* ret = new char[len+1];
@@ -314,31 +386,77 @@ void WSCstring::addString(const char* string,long encode){
   if (string == NULL){
     return;
   }
+  if (string[0] == 0){
+    return;
+  }
   if (_str == NULL){
     setString(string,encode);
     return;
   }
+  if (_static != False){
+    _static = False;
+    _str = WSGFstrdup(_str);
+  }
   if (encode == WS_EN_DEFAULT){
-    encode = WSGIappLocaleSet()->getDefaultEncoding();
+//    encode = WSGIappLocaleSet()->getDefaultEncoding();
+    encode = _wsgv_default_encoding;
   }
   if (encode == WS_EN_LOCALE){
-    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+//    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    encode = _wsgv_system_default_encoding;
   } 
   if (encode == _get_encoding()){
-    long len = strlen(_str) + strlen(string);
-    char* new_str = new char[len+1];
+    long len = 0;
+    if (_add_str_work > 0){
+      len = _add_str_work;
+    }else{
+      len = strlen(_str);
+    }
+    long len2 = strlen(string);
+    if (len2 < 1){
+      return;
+    }
+#if 1
+    if (_add_str_work_len > 0 && _add_str_work_len > (WSCulong)(len + len2)){
+      _add_str_work = len + len2;
+      strcpy(&_str[len],string);
+      _chars = -1;
+      if ( _tmp_list.getNum() != 0){
+        _clear_list_ex();
+      }
+      return;
+    }else{
+      _add_str_work = len + len2;
+      char* str = new char[len + len2 + 1];
+      if (str == NULL){
+        return;
+      }
+      if (_str != NULL){
+//        strcpy(str,_str);
+        memcpy(str,_str,len); //faster..
+        strcpy(&str[len],string);
+      }else{
+        strcpy(str,string);
+      }
+      delete[] _str;
+      _str = str;
+      _clear_list();
+    }
+#else //very slow..
+    char* new_str = new char[len+len2+1];
     strcpy(new_str,_str);
     strcat(new_str,string);
     setString(new_str,encode);
-    delete new_str;
+    delete[] new_str;
+#endif
     return;
   }else{
     char* org = _convert_string(_str,_get_encoding(),WS_EN_UTF8);
     char* string2 = _convert_string(string,encode,WS_EN_UTF8);
     setString(org,WS_EN_UTF8);
     addString(string2,WS_EN_UTF8);
-    delete org;
-    delete string2;
+    delete[] org;
+    delete[] string2;
     return;
   }
 }
@@ -354,11 +472,16 @@ char* WSCstring::getString(long encode){
   if (_str == NULL){
     return "";
   }
+  if (encode == _encoding){
+    return _str;
+  }
   if (encode == WS_EN_DEFAULT){
-    encode = WSGIappLocaleSet()->getDefaultEncoding();
+//    encode = WSGIappLocaleSet()->getDefaultEncoding();
+    encode = _wsgv_default_encoding;
   } 
   if (encode == WS_EN_LOCALE){
-    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+//    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    encode = _wsgv_system_default_encoding;
   } 
   if (encode == WS_EN_NONE){
     return _str;
@@ -383,6 +506,10 @@ void WSCstring::cutString(WSCulong char_pos){
   if (_str == NULL){
     return;
   }
+  if (_static != False){
+    _str = WSGFstrdup(_str);
+    _static = False;
+  }
   char* str = _str;
   long encoding = _get_encoding();
   if (encoding != WS_EN_UTF8){
@@ -409,8 +536,9 @@ void WSCstring::cutString(WSCulong char_pos){
   if (encoding != WS_EN_UTF8){
     char* new_str = _convert_string(str,WS_EN_UTF8,encoding);
     setString(new_str,encoding);
-    delete new_str;
-    delete str;
+    delete[] new_str;
+    delete[] str;
+    return;
   }
   _clear_list();
 }
@@ -446,8 +574,8 @@ long WSCstring::_seek_next_line(){
   }
   WSCushort* ustr = WSGFgetUCS2(ret,encoding);
   long val = WSGFstrlenUCS2(ustr);
-  delete ustr;
-  delete ret;
+  delete[] ustr;
+  delete[] ret;
   return val;
 #if 0
   char* str = getString(WS_EN_EUCJP);
@@ -515,10 +643,12 @@ long WSCstring::getWordCharPos(long wd,const char* sep,long encode){
     return -1;
   }
   if (encode == WS_EN_DEFAULT){
-    encode = WSGIappLocaleSet()->getDefaultEncoding();
+//    encode = WSGIappLocaleSet()->getDefaultEncoding();
+    encode = _wsgv_default_encoding;
   }
   if (encode == WS_EN_LOCALE){
-    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+//    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    encode = _wsgv_system_default_encoding;
   } 
   long encoding = _get_encoding();
 
@@ -529,7 +659,7 @@ long WSCstring::getWordCharPos(long wd,const char* sep,long encode){
   WSCushort* sep1 = WSGFgetUCS2(sep,encode);
   long seplen = WSGFstrlenUCS2(sep1);
   if (seplen < 1){
-    delete sep1;
+    delete[] sep1;
 //printf("WSCstring::getWordCharPos1 wd=%d 0\n",wd);
 //fflush(stdout);
     return 0;
@@ -547,8 +677,8 @@ long WSCstring::getWordCharPos(long wd,const char* sep,long encode){
         i += seplen-1;
       }
       if (cnt == wd){
-        delete sep1;
-        delete str1;
+        delete[] sep1;
+        delete[] str1;
         if (cnt == 0){
 //printf("WSCstring::getWordCharPos2 wd=%d 0\n",wd);
 //fflush(stdout);
@@ -572,8 +702,8 @@ long WSCstring::getWordCharPos(long wd,const char* sep,long encode){
         cnt++;
       }
       if (cnt == wd){
-        delete sep1;
-        delete str1;
+        delete[] sep1;
+        delete[] str1;
         if (cnt == 0){
 //printf("WSCstring::getWordCharPos4 wd=%d 0\n",wd);
 //fflush(stdout);
@@ -590,16 +720,57 @@ long WSCstring::getWordCharPos(long wd,const char* sep,long encode){
 //fflush(stdout);
   return -1;
 }
-
+WSCbool _check_sjis_(const char* sep,const char* str,int len,int pos){
+  if (pos > 0 &&
+      !(sep[0] & 0x80 )&&
+      sep[1] == 0 &&
+      !(str[pos-1] & 0x80)){
+    return True;
+  }
+  long start_pos = 0;
+  if (pos > 0){
+    int i;
+    for(i=pos; i> 0;i--){
+      if (str[i] <0x40){
+        start_pos = i;
+        break;
+      }
+    }
+  }
+  long sep_type = WSMFcodeCheck_SJIS(sep[0]);
+//  long len = strlen(str);
+  if (sep_type == WS_ASCII){
+    long i;
+    for( i=start_pos; i<len; i++ ){
+      if (i==pos){
+        return True;
+      }else if (i > pos){
+        return False;
+      }
+      long type = WSMFcodeCheck_SJIS(*(str+i));
+      switch(type){
+        case WS_ASCII: break;
+        case WS_KANJI: i++;
+                       break;
+        case WS_HKANA: break;
+        case WS_GAIJI: i++;
+                       break;
+      }
+    }
+  }
+  return False;
+}
 long WSCstring::getWords(const char* sep,long encode){
   if (sep == NULL){
     return 1;
   }
   if (encode == WS_EN_DEFAULT){
-    encode = WSGIappLocaleSet()->getDefaultEncoding();
+//    encode = WSGIappLocaleSet()->getDefaultEncoding();
+    encode = _wsgv_default_encoding;
   }
   if (encode == WS_EN_LOCALE){
-    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+//    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    encode = _wsgv_system_default_encoding;
   } 
   long encoding = _get_encoding();
   if (encode != encoding){
@@ -616,21 +787,24 @@ long WSCstring::getWords(const char* sep,long encode){
   long seplen = strlen(sep);
   if (seplen < 1){
     if (encode != encoding){
-      delete (char*)sep;
+      delete[] (char*)sep;
     }
     return 1;
   }
   long i;
   long cnt = 1; //words
-
   for(i=0; i < len +1; i++){
     if ( str[i] == sep[0] && !strncmp(&str[i],sep,(size_t)seplen) ){
+      if (encoding == WS_EN_SJIS &&
+          sep[0] > 0x3f && seplen == 1 && _check_sjis_(sep,str,len,i) == False){
+        continue;
+      }
       cnt++;
       i += seplen-1;
     }
   }
   if (encode != encoding){
-    delete (char*)sep;
+    delete[] (char*)sep;
   }
   return cnt;
 }
@@ -641,10 +815,12 @@ WSCstring WSCstring::getWord(long pos,const char* sep,long encode){
     sep = "";
   }
   if (encode == WS_EN_DEFAULT){
-    encode = WSGIappLocaleSet()->getDefaultEncoding();
+//    encode = WSGIappLocaleSet()->getDefaultEncoding();
+    encode = _wsgv_default_encoding;
   }
   if (encode == WS_EN_LOCALE){
-    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+//    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    encode = _wsgv_system_default_encoding;
   } 
   long encoding = _get_encoding();
   if (encode != encoding){
@@ -660,7 +836,7 @@ WSCstring WSCstring::getWord(long pos,const char* sep,long encode){
   if (seplen < 1){
     _return_tmp.setString("",encoding);
     if (encode != encoding){
-      delete (char*)sep;
+      delete[] (char*)sep;
     } 
     return _return_tmp;
   }
@@ -670,15 +846,26 @@ WSCstring WSCstring::getWord(long pos,const char* sep,long encode){
   long hit_point=0;
   for(i=0; i < len +1; i++){
     if (str[i] == sep[0] && !strncmp(&str[i],sep,(size_t)seplen)  ){
+      if (encoding == WS_EN_SJIS && 
+          sep[0] > 0x3f && seplen == 1 && _check_sjis_(sep,str,len,i) == False){
+        continue;
+      }
       if (cnt == pos){
+#if 0
         long length = i +1 -hit_point;
         char* tmp = new char[length];
         strncpy(tmp,&str[hit_point],(size_t)(i-hit_point));
         tmp[i-hit_point] = 0;
         _return_tmp.setString(tmp,encoding);
         delete tmp;
+#endif
+//        char bk = str[i];
+//        str[i] = 0;
+//        _return_tmp.setString(&str[hit_point],encoding);
+//        str[i] = bk;
+        _return_tmp.setStringEx(&str[hit_point],i-hit_point,encoding);
         if (encode != encoding){
-          delete (char*)sep;
+          delete[] (char*)sep;
         } 
         return _return_tmp;
       }
@@ -689,20 +876,23 @@ WSCstring WSCstring::getWord(long pos,const char* sep,long encode){
   }
 
   if ( cnt == pos ){
+#if 0
     long length = len +1 -hit_point;
     char* tmp = new char[length];
     strcpy(tmp,&str[hit_point]);
 
     _return_tmp.setString(tmp,encoding);
     delete tmp;
+#endif
+    _return_tmp.setString(&str[hit_point],encoding);
     if (encode != encoding){
-      delete (char*)sep;
+      delete[] (char*)sep;
     } 
     return _return_tmp;
   }
   _return_tmp.setString("",encoding);
   if (encode != encoding){
-    delete (char*)sep;
+    delete[] (char*)sep;
   } 
   return _return_tmp;
 }
@@ -724,18 +914,18 @@ long WSCstring::getWordCharPos(long wd){
   WSCbool flag = False;
 
   for(i=0; i < len +1; i++){
-    if (flag == False && str[i] != ' ' && str[i] != '\t' && str[i] != '\n' && str[i] != 0){
+    if (flag == False && str1[i] != ' ' && str1[i] != '\t' && str1[i] != '\n' && str1[i] != 0){
       cnt++;
       flag = True;
-    }else if (flag != False && (str[i] == ' ' || str[i] == '\t' || str[i] == '\n' || str[i] == 0) ){
+    }else if (flag != False && (str1[i] == ' ' || str1[i] == '\t' || str1[i] == '\n' || str1[i] == 0) ){
       flag = False;
     }
     if (cnt == wd){
-      delete str1;
+      delete[] str1;
       return i;
     }
   }
-  delete str1;
+  delete[] str1;
   return -1;
 }
 
@@ -788,12 +978,19 @@ WSCstring WSCstring::getWord(long pos){
               str[i] == '\n' || str[i] == 0) ){
       flag = False;
       if (hit != False){
+#if 0
         long length = i +1 -hit_point;
         char* tmp = new char[length];
         strncpy(tmp,&str[hit_point],(size_t)(i-hit_point));
         tmp[i-hit_point] = 0;
         _return_tmp.setString(tmp,encoding);
         delete tmp;
+#endif
+//        char bk = str[i];
+//        str[i] = 0;
+//        _return_tmp.setString(&str[hit_point],encoding);
+//        str[i] = bk;
+        _return_tmp.setStringEx(&str[hit_point],i-hit_point,encoding);
         return _return_tmp;
       }
     }
@@ -837,17 +1034,22 @@ void WSCstring::insertString(WSCulong char_pos,WSCstring& src){
   strcpy(&new_str[pos + slen],&str[pos]);
 
   if (encoding != WS_EN_UTF8){
-    delete _str;
+    if (_static == False){
+      delete[] _str;
+    }
+    _static = False;
+    delete[] str;
     _str = _convert_string(new_str,WS_EN_UTF8,encoding);
-    delete new_str;
-    _chars = -1;
+    delete[] new_str;
   }else{
-    delete _str;
+    if (_static == False){
+      delete[] _str;
+    }
+    _static = False;
     _str = new_str;
   }
   _clear_list();
 }
-
 long WSCstring::isExist(const char* str,long encode){
   if (str == NULL || str[0] == 0){
     return -1;
@@ -859,10 +1061,12 @@ long WSCstring::isExist(const char* str,long encode){
     return -1;
   }
   if (encode == WS_EN_DEFAULT){
-    encode = WSGIappLocaleSet()->getDefaultEncoding();
+//    encode = WSGIappLocaleSet()->getDefaultEncoding();
+    encode = _wsgv_default_encoding;
   }
   if (encode == WS_EN_LOCALE){
-    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+//    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    encode = _wsgv_system_default_encoding;
   } 
   long encoding = _get_encoding();
   if (encoding != encode){
@@ -875,7 +1079,7 @@ long WSCstring::isExist(const char* str,long encode){
   long len = strlen(text) - slen +1;
   if (len < 1){
     if (encoding != encode){
-      delete (char*)str;
+      delete[] (char*)str;
     }
     return -1;
   }
@@ -884,23 +1088,17 @@ long WSCstring::isExist(const char* str,long encode){
     if (text[i] == str[0]){
       if ( !strncmp(&text[i],str,(size_t)slen) ){
         if (encoding != encode){
-          delete (char*)str;
+          delete[] (char*)str;
         }
         if (i == 0){
           return 0;
         }
-        char* tmp = WSGFstrdup(text);
-        tmp[i] = 0;
-        WSCushort* tmp2 = WSGFgetUCS2(tmp,encoding);
-        long ret = WSGFstrlenUCS2(tmp2);
-        delete tmp;
-        delete tmp2;
-        return ret;
+        return WSGFgetUcs2len(text,encoding,i);
       }
     }
   }
   if (encoding != encode){
-    delete (char*)str;
+    delete[] (char*)str;
   }
   return -1;
 
@@ -917,6 +1115,10 @@ void WSCstring::deleteChar(WSCulong char_pos){
   if ( _str == NULL ){
     return;
   }
+  if (_static != False){
+    _str = WSGFstrdup(_str);
+    _static = False;
+  }
   char* str = _str;
   long encoding = _get_encoding();
   if (encoding != WS_EN_UTF8){
@@ -929,18 +1131,17 @@ void WSCstring::deleteChar(WSCulong char_pos){
   while(1){
     if (len < 1){
       if (encoding != WS_EN_UTF8){
-        delete str;
+        delete[] str;
       }
       return;
     }
     if (cnt == char_pos){
       next_diff = WSGIappCodeConvert()->utf8CharLen(&str[pos],len);
-      memcpy(&str[pos],&str[pos+next_diff],len - next_diff+1);
+      memmove(&str[pos],&str[pos+next_diff],len - next_diff+1);
       if (encoding != WS_EN_UTF8){
-        delete _str;
+        delete[] _str;
         _str = _convert_string(str,WS_EN_UTF8,encoding);
-        delete str;
-        _chars = -1;
+        delete[] str;
       }
       _clear_list();
       return;
@@ -956,22 +1157,24 @@ void WSCstring::_add_string_to_list(char* pstr,long encode){
   long i;
   long num = _tmp_list.getNum();
   for(i=0; i<num; i++){
-    long encode1 = (long)_tmp_encode[i];
+    long encode1 = (long)_tmp_list[i];
+    i++;
     if (encode1 == encode){
       char* str1 = (char*)_tmp_list[i];
-      delete str1;
+      delete[] str1;
       _tmp_list.setData(i,pstr);
       return;
     }
   }
+  _tmp_list.add((void*)encode);
   _tmp_list.add((void*)pstr);
-  _tmp_encode.add((void*)encode);
 }
 char* WSCstring::_get_string_from_list(long encode){
   long i;
   long num = _tmp_list.getNum();
   for(i=0; i<num; i++){
-    long encode1 = (long)_tmp_encode[i];
+    long encode1 = (long)_tmp_list[i];
+    i++;
     if (encode1 == encode){
       char* str = (char*)_tmp_list[i];
       return str;
@@ -981,21 +1184,75 @@ char* WSCstring::_get_string_from_list(long encode){
 }
 void WSCstring::setString(const char* str,long encode){
   if (encode == WS_EN_DEFAULT){
-    encode = WSGIappLocaleSet()->getDefaultEncoding();
+//    encode = WSGIappLocaleSet()->getDefaultEncoding();
+    encode = _wsgv_default_encoding;
   } 
   if (encode == WS_EN_LOCALE){
-    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+//    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    encode = _wsgv_system_default_encoding;
   } 
   if (_str != NULL){
-    delete _str;
+    if (_static == False){
+      delete[] _str;
+    }
     _str = NULL;
   }
+  _static = False;
   _encoding = encode;
-  _str = WSGFstrdup(str);
-  _chars = -1;
+  if (str != NULL && str[0] != 0){
+    _str = WSGFstrdup(str);
+  }
   _clear_list();
   _seek_pt = 0;
 }
+void WSCstring::setStringEx(const char* str,long len,long encode){
+  if (encode == WS_EN_DEFAULT){
+//    encode = WSGIappLocaleSet()->getDefaultEncoding();
+    encode = _wsgv_default_encoding;
+  } 
+  if (encode == WS_EN_LOCALE){
+//    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    encode = _wsgv_system_default_encoding;
+  } 
+  if (_str != NULL){
+    if (_static == False){
+      delete[] _str;
+    }
+    _str = NULL;
+  }
+  _static = False;
+  _encoding = encode;
+  if (str != NULL && str[0] != 0){
+    _str = WSGFstrdup(str,len);
+  }
+  _clear_list();
+  _seek_pt = 0;
+}
+void WSCstring::setStaticString(const char* str,long encode){
+  if (encode == WS_EN_DEFAULT){
+//    encode = WSGIappLocaleSet()->getDefaultEncoding();
+    encode = _wsgv_default_encoding;
+  } 
+  if (encode == WS_EN_LOCALE){
+//    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    encode = _wsgv_system_default_encoding;
+  } 
+  if (_str != NULL){
+    if (_static == False){
+      delete[] _str;
+    }
+    _static = False;
+    _str = NULL;
+  }
+  _encoding = encode;
+  if (str != NULL && str[0] != 0){
+    _static = True;
+    _str = (char*)str;
+  }
+  _clear_list();
+  _seek_pt = 0;
+}
+
 
 long WSCstring::tell(){
   return _seek_pt;
@@ -1045,19 +1302,15 @@ WSCstring WSCstring::gets(){
     return _ret_tmp;
   }
   while( _str[ptr] != '\n' && _str[ptr] != 0 ) ptr++;
-  char* ret = new char[ptr - _seek_pt +1];
-  strncpy(ret,&(_str[_seek_pt]),(ptr - _seek_pt));
-  if (ptr-_seek_pt > 0 && ret[ptr-_seek_pt -1] == 13){ //0x0d ms line feed
-    ret[ptr-_seek_pt-1] = 0;
+  if (ptr-_seek_pt > 0 && _str[ptr-1] == 13){ //0x0d ms line feed
+    _ret_tmp.setStringEx(&(_str[_seek_pt]),ptr-_seek_pt-1,encoding);
+  }else{
+    _ret_tmp.setStringEx(&(_str[_seek_pt]),ptr-_seek_pt,encoding);
   }
-  ret[ptr-_seek_pt] = 0;
   _seek_pt = ptr;
   if (_str[_seek_pt] == '\n'){
-     _seek_pt++;
+    _seek_pt++;
   }
-  
-  _ret_tmp.setString(ret,encoding);
-  delete ret;
   return _ret_tmp;
 }
 
@@ -1070,10 +1323,12 @@ long WSCstring::replaceString(const char* srcchar,const char* destchar,long num,
   }
   long encoding = _get_encoding();
   if (encode == WS_EN_DEFAULT){
-    encode = WSGIappLocaleSet()->getDefaultEncoding();
+//    encode = WSGIappLocaleSet()->getDefaultEncoding();
+    encode = _wsgv_default_encoding;
   }
   if (encode == WS_EN_LOCALE){
-    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+//    encode = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    encode = _wsgv_system_default_encoding;
   } 
   if (encoding == WS_EN_SJIS && srcchar[0] == '\\'){
     WSCstring ustr;
@@ -1086,8 +1341,8 @@ long WSCstring::replaceString(const char* srcchar,const char* destchar,long num,
     char* src2 = _convert_string(srcchar,encode,encoding);
     char* dest2 = _convert_string(destchar,encode,encoding);
     long ret = replaceString(src2,dest2,num,encoding);
-    delete src2;
-    delete dest2;
+    delete[] src2;
+    delete[] dest2;
     return ret;
   }
   if (_str == NULL){
@@ -1141,9 +1396,11 @@ long WSCstring::replaceString(const char* srcchar,const char* destchar,long num,
     }
   }
   new_str[cnt] = 0;
-  delete _str;
+  if (_static == False){
+    delete[] _str;
+  }
+  _static = False;
   _str = new_str; 
-  _chars = -1;
   _clear_list();
   return count;
 }
@@ -1152,21 +1409,34 @@ void WSCstring::delHeadSpace(){
   if (_str == NULL){
     return;
   }
-  char* new_str = new char[strlen(_str)+32];
-
-  strcpy(new_str,_str);
   long len = strlen(_str);
+//  char* new_str = new char[len+1];
+//  strcpy(new_str,_str);
   long i;
 
   for(i=0; i < len; i++){
-    if (new_str[i] != ' '){
+    if (_str[i] != ' '){
       break;
     }
   }
-  strcpy(_str,&new_str[i]);
-  delete new_str;
+  if (i==0){
+    return;
+  }
+  if (_static != False){
+    _str = WSGFstrdup(&_str[i]);
+    _static = False;
+    _clear_list();
+    return;
+  }
+
+//  long j;
+//  for(j=0; j < len -i+1; j++){
+//    _str[j] = _str[j+i];
+//  }
+  memmove(_str,&_str[i],len -i+1);
+//  strcpy(_str,&new_str[i]);
+//  delete new_str;
   _clear_list();
-  _chars = -1;
   return;
 }
 
@@ -1174,24 +1444,32 @@ void WSCstring::delTailSpace(){
   if (_str == NULL){
     return;
   }
-  char* new_str = new char[strlen(_str)+32];
-
-  strcpy(new_str,_str);
+  if (_str[0] == 0){
+    return;
+  }
   long i;
   long len = strlen(_str);
-  for(i=len; i > -1; i--){
-    if (new_str[i] != ' ' && new_str[i] != '\n' && new_str[i] != 0){
+  for(i=len-1; i > -1; i--){
+    if (_str[i] != ' ' && _str[i] != '\n' && _str[i] != 0){
       break;
     }
   }
-  new_str[i+1] = 0;
-  delete _str;
-  _str = new_str;
-  _clear_list();
-  _chars = -1;
+  if (i != len -1){
+    if (_static == False){
+      _str[i+1] = 0;
+      _clear_list();
+    }else{
+      long encoding = _get_encoding();
+      setStringEx(_str,i+1,encoding);
+    }
+  }
   return; 
 }
 void WSCstring::to_upper(){
+  if (_static != False){
+    _str = WSGFstrdup(_str);
+    _static = False;
+  }
   long i;
   if (_str != NULL){
     long len = strlen(_str);
@@ -1204,6 +1482,10 @@ void WSCstring::to_upper(){
   _clear_list();
 }
 void WSCstring::to_lower(){
+  if (_static != False){
+    _str = WSGFstrdup(_str);
+    _static = False;
+  }
   long i;
   if (_str != NULL){
     long len = strlen(_str);
@@ -1219,6 +1501,10 @@ void WSCstring::deleteChars(WSCulong char_pos,WSCulong len){
   if ( _str == NULL ){
     return;
   }
+  if (_static != False){
+    _str = WSGFstrdup(_str);
+    _static = False;
+  }
   char* str = _str;
   long encoding = _get_encoding();
   if (encoding != WS_EN_UTF8){
@@ -1227,6 +1513,9 @@ void WSCstring::deleteChars(WSCulong char_pos,WSCulong len){
 //printf("WSCstring::... encoding=%d\n",encoding);
 //printf("WSCstring::... start=#%s#\n",_str);
 //fflush(stdout);
+  if ((WSCulong)getChars() < len){
+    len = getChars();
+  }
   long len2 = strlen(str);
   long len3 = len2;
   long pos = 0;
@@ -1235,7 +1524,7 @@ void WSCstring::deleteChars(WSCulong char_pos,WSCulong len){
   while(1){
     if (len2 < 1){
       if (encoding != WS_EN_UTF8){
-        delete str;
+        delete[] str;
       }
       return;
     }
@@ -1246,25 +1535,25 @@ void WSCstring::deleteChars(WSCulong char_pos,WSCulong len){
         if (cnt2 == len){
           break;
         }
-//printf("WSCstring::... len=%d cnt2=%d next_diff=%d str[i]=0x%x\n",len,cnt2,next_diff,str[next_diff]);
+//printf("WSCstring::... len=%d cnt2=%d next_diff=%d str[next_diff]=0x%x 0x%x 0x%x\n",len,cnt2,next_diff,str[next_diff],str[next_diff+1],str[next_diff+2]);
 //fflush(stdout);
         next_diff += WSGIappCodeConvert()->utf8CharLen(&str[next_diff],len2);
         cnt2++;
       }
 //printf("WSCstring::... len=%d len2=%d next_diff=%d slen=%d\n",len,len2,next_diff,len3+1 - next_diff);
 //fflush(stdout);
-      memcpy(&str[pos],&str[next_diff],len3+1 - next_diff);
+      memmove(&str[pos],&str[next_diff],len3+1 - next_diff);
 //printf("WSCstring::... 0x%x 0x%x 0x%x\n",str[next_diff -1],str[next_diff],str[next_diff+1]);
 //fflush(stdout);
       if (encoding != WS_EN_UTF8){
-        delete _str;
+        delete[] _str;
         _str = _convert_string(str,WS_EN_UTF8,encoding);
-        delete str;
+        delete[] str;
 //printf("WSCstring::... result=#%s#\n",_str);
 //printf("WSCstring::... result=#%s#\n",_convert_string(_str,encoding,WS_EN_EUCJP));
 //fflush(stdout);
-        _chars = -1;
       }
+      _clear_list();
       return;
     }
     next_diff = WSGIappCodeConvert()->utf8CharLen(&str[pos],len2);
@@ -1279,17 +1568,91 @@ long WSCstring::getEncoding(){
 long WSCstring::setEncoding(long enc){
   long encoding = enc;
   if (encoding == WS_EN_DEFAULT){
-    encoding = WSGIappLocaleSet()->getDefaultEncoding();
+//    encoding = WSGIappLocaleSet()->getDefaultEncoding();
+    encoding = _wsgv_default_encoding;
   }
   if (encoding == WS_EN_LOCALE){
-    encoding = WSGIappLocaleSet()->getSystemLocaleEncoding();
+//    encoding = WSGIappLocaleSet()->getSystemLocaleEncoding();
+    encoding = _wsgv_system_default_encoding;
   } 
   long encoding_bak = getEncoding();
   if (encoding_bak == encoding){
     return encoding;
   }
   char* new_str = _convert_string(_str,encoding_bak,encoding);
-  setString(new_str,encoding);
-  delete new_str;
+//  setString(new_str,encoding);
+  if (_static != False){
+    _static = False;
+    _str = new_str;
+  }else{
+    delete[] _str;
+    _str = new_str;
+  }
+
+  if (encoding == WS_EN_DEFAULT){
+    encoding = _wsgv_default_encoding;
+  } 
+  if (encoding == WS_EN_LOCALE){
+    encoding = _wsgv_system_default_encoding;
+  } 
+  _encoding = encoding;
+
+  _clear_list();
+  _seek_pt = 0;
+
+//  delete[] new_str;
   return encoding_bak;
 }
+void WSCstring::setAddStringWorkLen(long mlen){
+  _add_str_work_len = mlen+1;
+  _add_str_work = 0;
+  if (_str == NULL){
+    _static = False;
+    _str = new char[mlen+1];
+    if (_str != NULL){
+      _str[0] = 0;
+    }
+    return;
+  }
+  long len = strlen(_str);
+  if (len < mlen){
+    char* newstr = new char [mlen+1];
+    strcpy(newstr,_str);
+    if (_static == False){
+      delete[] _str;
+    }
+    _static = False;
+    _str = newstr;
+  }
+}
+long WSCstring::replaceWord(long num,const char* sep,const char* newstr,long encode){
+  if (num < 0){
+    return WS_ERR;
+  }
+  if (sep == NULL){
+    return WS_ERR;
+  }
+  if (newstr == NULL){
+    newstr = "";
+  }
+  WSCstring news(newstr,encode);
+  long words = getWords(sep,encode);
+  if (!(num < words)){
+    return WS_ERR;
+  }
+  long pos = getWordCharPos(num,sep,encode);
+  WSCstring word = getWord(num,sep,encode);
+  deleteChars(pos,word.getChars());  
+  insertString(pos,news);
+  return WS_NO_ERR;
+}
+
+WSCushort* WSCstring::getInternalUcs2(){
+  if (_str == NULL || _str[0] == 0){
+    return NULL;
+  }
+  if (_ucs2 == NULL){
+    _ucs2 = WSGFgetUCS2(_str,_encoding);
+  }
+  return _ucs2;
+}
