@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,30 +14,29 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCvariant.h>
-#include <WSCstring.h>
+//#include <WSCstring.h>
 #include <WSCbase.h>
 
-
 WSCvariant::~WSCvariant(){
-  clear();
+  if (_inited == False){
+    clear();
+  }
 }
 WSCvariant::WSCvariant(){
   _type = WSTlong;
   _data = (void*)0;
   _data2 = NULL;
   _type_name = NULL;
+  _no_allocated = False;
+  _inited = True;
 }
 WSCvariant::WSCvariant(char data){
   long data1 = (long)data;
@@ -45,6 +44,8 @@ WSCvariant::WSCvariant(char data){
   _data = (void*)data1;
   _data2 = NULL;
   _type_name = NULL;
+  _no_allocated = False;
+  _inited = True;
 }
 WSCvariant::WSCvariant(WSCuchar data){
   long data1 = (long)data;
@@ -52,6 +53,8 @@ WSCvariant::WSCvariant(WSCuchar data){
   _data = (void*)data1;
   _data2 = NULL;
   _type_name = NULL;
+  _no_allocated = False;
+  _inited = True;
 }
 WSCvariant::WSCvariant(short data){
   long data1 = (long)data;
@@ -59,6 +62,8 @@ WSCvariant::WSCvariant(short data){
   _data = (void*)data1;
   _data2 = NULL;
   _type_name = NULL;
+  _no_allocated = False;
+  _inited = True;
 }
 WSCvariant::WSCvariant(WSCushort data){
   long data1 = (long)data;
@@ -66,6 +71,8 @@ WSCvariant::WSCvariant(WSCushort data){
   _data = (void*)data1;
   _data2 = NULL;
   _type_name = NULL;
+  _no_allocated = False;
+  _inited = True;
 }
 
 WSCvariant::WSCvariant(long data){
@@ -73,24 +80,32 @@ WSCvariant::WSCvariant(long data){
   _data = (void*)data;
   _data2 = NULL;
   _type_name = NULL;
+  _no_allocated = False;
+  _inited = True;
 }
 WSCvariant::WSCvariant(WSCulong data){
   _type = WSTulong;
   _data = (void*)data;
   _data2 = NULL;
   _type_name = NULL;
+  _no_allocated = False;
+  _inited = True;
 }
 WSCvariant::WSCvariant(int data){
   _type = WSTint;
   _data = (void*)data;
   _data2 = NULL;
   _type_name = NULL;
+  _no_allocated = False;
+  _inited = True;
 }
 WSCvariant::WSCvariant(WSCuint data){
   _type = WSTuint;
   _data = (void*)data;
   _data2 = NULL;
   _type_name = NULL;
+  _no_allocated = False;
+  _inited = True;
 }
 
 
@@ -99,6 +114,8 @@ WSCvariant::WSCvariant(float data){
   memcpy(&_data,&data,sizeof(void*));
   _data2 = NULL;
   _type_name = NULL;
+  _no_allocated = False;
+  _inited = True;
 }
 WSCvariant::WSCvariant(double data){
   _type = WSTdouble;
@@ -106,6 +123,8 @@ WSCvariant::WSCvariant(double data){
   memcpy(_data,&data,sizeof(double));
   _data2 = NULL;
   _type_name = NULL;
+  _no_allocated = False;
+  _inited = False;
 }
 WSCvariant::WSCvariant(void* data,char* tname){
   _type = WSTvoidptr;
@@ -113,24 +132,37 @@ WSCvariant::WSCvariant(void* data,char* tname){
   _data2 = NULL;
   if (tname != NULL){
     _type_name = WSGFstrdup(tname);
+    _inited = False;
   }else{
     _type_name = NULL;
+    _inited = True;
   }
+  _no_allocated = False;
 }
 WSCvariant::WSCvariant(WSCbase* data){
   _type = WSTvoidptr;
   _data = (void*)data;
   _data2 = NULL;
   _type_name = WSGFstrdup("WSCbase*");
+  _no_allocated = False;
+  _inited = False;
 }
 
 WSCvariant::WSCvariant(char* data){
   _type = WSTcharptr;
-  WSCstring* str = new WSCstring;
-  str->setString(data);
-  _data = (void*)str;
+  _data = (void*)WSGFstrdup(data);
   _data2 = NULL;
   _type_name = NULL;
+  _no_allocated = False;
+  _inited = False;
+}
+WSCvariant::WSCvariant(const char* data){
+  _type = WSTcharptr;
+  _data = (void*)data;
+  _no_allocated = True;
+  _data2 = NULL;
+  _type_name = NULL;
+  _inited = True;
 }
 #if 0
 WSCvariant::WSCvariant(WSCstring data){
@@ -145,27 +177,32 @@ WSCvariant::WSCvariant(WSCstring data){
 WSCvariant::WSCvariant(const WSCvariant& src){
   _type = src._type;
   _type_name = NULL;
+  _no_allocated = False;
+  _inited = True;
+  _data2 = NULL;
   if (src._type_name != NULL){
     _type_name = WSGFstrdup(src._type_name);
+    _inited = False;
   }
   if (_type == WSTcharptr){
-    WSCstring* str = new WSCstring;
-    WSCstring* sstr = (WSCstring*)src._data;
-    str->setString(sstr->getString());
-    _data = (void*)str;
-    _data2 = NULL;
+    if (src._no_allocated == False){
+      _data = (void*)WSGFstrdup((char*)src._data);
+      _inited = False;
+    }else{
+      _no_allocated = True;
+      _data = (void*)src._data;
+    }
   }else if (_type == WSTdouble){
     _data = (void*)new double;
     memcpy(_data,src._data,sizeof(double));
-    _data2 = NULL;
+    _inited = False;
   }else{
     _data = src._data;
-    _data2 = NULL;
   }
 }
 void WSCvariant::clear(){
   if (_data2 != NULL){
-    delete _data2;
+    delete[] _data2;
     _data2 = NULL;
   }
   if (_type == WSTdouble){
@@ -173,9 +210,9 @@ void WSCvariant::clear(){
     _data = NULL;
   }else
   if (_type == WSTcharptr){
-    WSCstring* ptr = (WSCstring*)_data;
-    if (ptr != NULL){
-      delete ptr;
+    char* ptr = (char*)_data;
+    if (ptr != NULL && _no_allocated == False){
+      delete[] ptr;
     }
     _data = NULL;
   }
@@ -184,12 +221,10 @@ void WSCvariant::clear(){
     _type_name = NULL;
   }
   _type = WSTlong;
-  if (_data2 != NULL){
-    delete _data2;
-    _data2 = NULL;
-  }
+  _no_allocated = False;
+  _inited = True;
 }
-char WSCvariant::getChar(){
+char WSCvariant::getChar() const{
   if (_type == WSTchar || _type == WSTuchar ||
       _type == WSTshort || _type == WSTushort ||
       _type == WSTint || _type == WSTuint ||
@@ -208,13 +243,12 @@ char WSCvariant::getChar(){
     return (char)val;
   }
   if (_type == WSTcharptr){
-    WSCstring* str = (WSCstring*)_data;
-    return (char)atoi(str->getString());
+    return (char)atoi((char*)_data);
   }
   return 0;
 }
 
-WSCuchar WSCvariant::getUnsignedChar(){
+WSCuchar WSCvariant::getUnsignedChar() const{
   if (_type == WSTchar || _type == WSTuchar ||
       _type == WSTshort || _type == WSTushort ||
       _type == WSTint || _type == WSTuint ||
@@ -233,13 +267,12 @@ WSCuchar WSCvariant::getUnsignedChar(){
     return (WSCuchar)val;
   }
   if (_type == WSTcharptr){
-    WSCstring* str = (WSCstring*)_data;
-    return (WSCuchar)atoi(str->getString());
+    return (WSCuchar)atoi((char*)_data);
   }
   return 0;
 }
 
-short WSCvariant::getShort(){
+short WSCvariant::getShort() const{
   if (_type == WSTchar || _type == WSTuchar ||
       _type == WSTshort || _type == WSTushort ||
       _type == WSTint || _type == WSTuint ||
@@ -258,12 +291,11 @@ short WSCvariant::getShort(){
     return (short)val;
   }
   if (_type == WSTcharptr){
-    WSCstring* str = (WSCstring*)_data;
-    return (short)atoi(str->getString());
+    return (short)atoi((char*)_data);
   }
   return 0;
 }
-WSCushort WSCvariant::getUnsignedShort(){
+WSCushort WSCvariant::getUnsignedShort() const{
   if (_type == WSTchar || _type == WSTuchar ||
       _type == WSTshort || _type == WSTushort ||
       _type == WSTint || _type == WSTuint ||
@@ -282,12 +314,11 @@ WSCushort WSCvariant::getUnsignedShort(){
     return (WSCushort)val;
   }
   if (_type == WSTcharptr){
-    WSCstring* str = (WSCstring*)_data;
-    return (WSCushort)atoi(str->getString());
+    return (WSCushort)atoi((char*)_data);
   }
   return 0;
 }
-long WSCvariant::getLong(){
+long WSCvariant::getLong() const{
   if (_type == WSTchar || _type == WSTuchar ||
       _type == WSTshort || _type == WSTushort ||
       _type == WSTint || _type == WSTuint ||
@@ -309,12 +340,11 @@ long WSCvariant::getLong(){
     return (long)_data;
   }
   if (_type == WSTcharptr){
-    WSCstring* str = (WSCstring*)_data;
-    return (long)atoi(str->getString());
+    return (long)atol((char*)_data);
   }
   return 0;
 }
-WSCulong WSCvariant::getUnsignedLong(){
+WSCulong WSCvariant::getUnsignedLong() const{
   if (_type == WSTchar || _type == WSTuchar ||
       _type == WSTshort || _type == WSTushort ||
       _type == WSTint || _type == WSTuint ||
@@ -333,12 +363,11 @@ WSCulong WSCvariant::getUnsignedLong(){
     return (WSCulong)val;
   }
   if (_type == WSTcharptr){
-    WSCstring* str = (WSCstring*)_data;
-    return (WSCulong)atoi(str->getString());
+    return (WSCulong)WSGFatol((char*)_data);
   }
   return 0;
 }
-int WSCvariant::getInt(){
+int WSCvariant::getInt() const{
   if (_type == WSTchar || _type == WSTuchar ||
       _type == WSTshort || _type == WSTushort ||
       _type == WSTint || _type == WSTuint ||
@@ -357,12 +386,11 @@ int WSCvariant::getInt(){
     return (int)val;
   }
   if (_type == WSTcharptr){
-    WSCstring* str = (WSCstring*)_data;
-    return (int)atoi(str->getString());
+    return (int)atoi((char*)_data);
   }
   return 0;
 }
-WSCuint WSCvariant::getUnsignedInt(){
+WSCuint WSCvariant::getUnsignedInt() const{
   if (_type == WSTchar || _type == WSTuchar ||
       _type == WSTshort || _type == WSTushort ||
       _type == WSTint || _type == WSTuint ||
@@ -381,13 +409,12 @@ WSCuint WSCvariant::getUnsignedInt(){
     return (WSCuint)val;
   }
   if (_type == WSTcharptr){
-    WSCstring* str = (WSCstring*)_data;
-    return (WSCuint)atoi(str->getString());
+    return (WSCuint)WSGFatol((char*)_data);
   }
   return 0;
 }
 
-float WSCvariant::getFloat(){
+float WSCvariant::getFloat() const{
   if (_type == WSTchar || _type == WSTuchar ||
       _type == WSTshort || _type == WSTushort ||
       _type == WSTint || _type == WSTuint ||
@@ -406,12 +433,11 @@ float WSCvariant::getFloat(){
     return (float)val;
   }
   if (_type == WSTcharptr){
-    WSCstring* str = (WSCstring*)_data;
-    return (float)atof(str->getString());
+    return (float)atof((char*)_data);
   }
   return 0;
 }
-double WSCvariant::getDouble(){
+double WSCvariant::getDouble() const{
   if (_type == WSTchar || _type == WSTuchar ||
       _type == WSTshort || _type == WSTushort ||
       _type == WSTint || _type == WSTuint ||
@@ -430,13 +456,12 @@ double WSCvariant::getDouble(){
     return (double)val;
   }
   if (_type == WSTcharptr){
-    WSCstring* str = (WSCstring*)_data;
-    return atof(str->getString());
+    return atof((char*)_data);
   }
   return (double)0;
 }
 
-WSCbase* WSCvariant::getInstancePtr(){
+WSCbase* WSCvariant::getInstancePtr() const{
   if (_type == WSTvoidptr ){
     if (!strcmp(_type_name,"WSCbase*")){
       return (WSCbase*)_data;
@@ -445,7 +470,7 @@ WSCbase* WSCvariant::getInstancePtr(){
   return NULL;
 }
 
-void* WSCvariant::getVoidPtr(){
+void* WSCvariant::getVoidPtr() const{
   if (_type == WSTchar || _type == WSTuchar ||
       _type == WSTshort || _type == WSTushort ||
       _type == WSTint || _type == WSTuint ||
@@ -465,13 +490,15 @@ void* WSCvariant::getVoidPtr(){
     return (void*)val2;
   }
   if (_type == WSTcharptr){
-    WSCstring* str = (WSCstring*)_data;
-    return (void*)atoi(str->getString());
+    return (void*)WSGFatol((char*)_data);
   }
   return (void*)0;
 }
 
 char* WSCvariant::getCharPtr(){
+  if (_type == WSTcharptr){
+    return (char*)_data;
+  }else
   if (_type == WSTchar || _type == WSTuchar ||
       _type == WSTshort || _type == WSTushort ||
       _type == WSTint || _type == WSTuint ||
@@ -481,11 +508,12 @@ char* WSCvariant::getCharPtr(){
       return _data2;
     }
     char buffer[64];
-    sprintf(buffer,"%d",(long)_data);
+    sprintf(buffer,"%ld",(long)_data);
     _data2 = WSGFstrdup(buffer);    
+    _inited = False;
     return _data2;
 //    return WSGFltoa((long)_data);
-  }
+  }else
   if (_type == WSTfloat){
     if (_data2 != NULL){
       return _data2;
@@ -495,8 +523,9 @@ char* WSCvariant::getCharPtr(){
     char buffer[128];
     sprintf(buffer,"%10.10lf",val);
     _data2 = WSGFstrdup(buffer);    
+    _inited = False;
     return _data2;
-  }
+  }else
   if (_type == WSTdouble){
     if (_data2 != NULL){
       return _data2;
@@ -505,19 +534,16 @@ char* WSCvariant::getCharPtr(){
     char buffer[128];
     sprintf(buffer,"%10.10lf",val);
     _data2 = WSGFstrdup(buffer);    
+    _inited = False;
     return _data2;
   }
-  if (_type == WSTcharptr){
-    WSCstring* str = (WSCstring*)_data;
-    return str->getString();
-  }
   return "";
 }
 
-long WSCvariant::getType(){
+long WSCvariant::getType() const{
   return _type;
 }
-char* WSCvariant::getTypeName(){
+char* WSCvariant::getTypeName() const{
   if (_type_name != NULL){
     return _type_name;
   }
@@ -546,69 +572,110 @@ static char fl = False;
 }
 
 void WSCvariant::setValue(char val){
-  clear();
+  if (_inited == False){
+    clear();
+  }
   long val2 = (long)val;
   _data = (void*)val2;
   _type = WSTchar;
 }
 void WSCvariant::setValue(WSCuchar val){
-  clear();
+  if (_inited == False){
+    clear();
+  }
   long val2 = (long)val;
   _data = (void*)val2;
   _type = WSTuchar;
 }
 void WSCvariant::setValue(short val){
-  clear();
+  if (_inited == False){
+    clear();
+  }
   long val2 = (long)val;
   _data = (void*)val2;
   _type = WSTshort;
 }
 void WSCvariant::setValue(WSCushort val){
-  clear();
+  if (_inited == False){
+    clear();
+  }
   long val2 = (long)val;
   _data = (void*)val2;
   _type = WSTushort;
 }
 void WSCvariant::setValue(long val){
-  clear();
+  if (_inited == False){
+    clear();
+  }
   _data = (void*)val;
   _type = WSTlong;
 }
 void WSCvariant::setValue(WSCulong val){
-  clear();
+  if (_inited == False){
+    clear();
+  }
   _data = (void*)val;
   _type = WSTulong;
 }
 void WSCvariant::setValue(int val){
-  clear();
+  if (_inited == False){
+    clear();
+  }
   long val2 = (long)val;
   _data = (void*)val2;
   _type = WSTint;
 }
 void WSCvariant::setValue(WSCuint val){
-  clear();
+  if (_inited == False){
+    clear();
+  }
   long val2 = (long)val;
   _data = (void*)val2;
   _type = WSTuint;
 }
 void WSCvariant::setValue(char* val){
-  clear();
+//  if (_type == WSTcharptr && _data != NULL &&
+//      strlen(val) <= strlen((char*)_data) ){
+//    strcpy((char*)_data,val);
+//    return;
+//  }
+  if (_inited == False){
+    clear();
+  }
   _type = WSTcharptr;
-  WSCstring* str = new WSCstring;
-  str->setString(val);
-  _data = (void*)str;
+  _data = (void*)WSGFstrdup(val);
   _type_name = NULL;
+  _inited = False;
+  _no_allocated = False;
+}
+void WSCvariant::setValue(const char* val){
+  if (_inited == False){
+    clear();
+  }
+  _type = WSTcharptr;
+  _data = (void*)val;
+  _type_name = NULL;
+  _no_allocated = True;
 }
 void WSCvariant::setValue(void* val,char* cname){
-  clear();
+  if (_inited == False){
+    clear();
+  }
   _data = val;
   _type = WSTvoidptr;
+  if (_type_name != NULL){
+    delete _type_name;
+    _type_name = NULL;
+  }
   if (cname != NULL){
     _type_name = WSGFstrdup(cname);
+    _inited = False;
   }
 }
 void WSCvariant::setValue(WSCbase* val){
-  clear();
+  if (_inited == False){
+    clear();
+  }
   _data = (void*)val;
   _type = WSTvoidptr;
   if (_type_name != NULL){
@@ -616,17 +683,23 @@ void WSCvariant::setValue(WSCbase* val){
     _type_name = NULL;
   }
   _type_name = WSGFstrdup("WSCbase*");
+  _inited = False;
 }
 void WSCvariant::setValue(float val){
-  clear();
+  if (_inited == False){
+    clear();
+  }
   memcpy(&_data,&val,sizeof(void*));
   _type = WSTfloat;
 }
 void WSCvariant::setValue(double val){
-  clear();
+  if (_inited == False){
+    clear();
+  }
   _data =(void*) new double;
   memcpy(_data,&val,sizeof(double));
   _type = WSTdouble;
+  _inited = False;
 }
 WSCvariant& WSCvariant::operator =(const WSCvariant& src){
   if (&src != this){
@@ -635,15 +708,20 @@ WSCvariant& WSCvariant::operator =(const WSCvariant& src){
     _type_name = NULL;
     if (src._type_name != NULL){
       _type_name = WSGFstrdup(src._type_name);
+      _inited = False;
     }
     if (_type == WSTcharptr){
-      WSCstring* str = new WSCstring;
-      WSCstring* sstr = (WSCstring*)src._data;
-      str->setString(sstr->getString());
-      _data = (void*)str;
+      _no_allocated = src._no_allocated;
+      if (_no_allocated == False){
+        _data = (void*)WSGFstrdup((char*)src._data);
+        _inited = False;
+      }else{
+         _data = (void*)src._data;
+      }
     }else if (_type == WSTdouble){
       _data = (void*)new double;
       memcpy(_data,src._data,sizeof(double));
+      _inited = False;
     }else{
       _data = src._data;
     }
@@ -694,6 +772,10 @@ WSCvariant& WSCvariant::operator =(char* val){
   setValue(val);
   return *this;
 }
+WSCvariant& WSCvariant::operator =(const char* val){
+  setValue(val);
+  return *this;
+}
 WSCvariant& WSCvariant::operator =(WSCbase* val){
   setValue(val);
   return *this;
@@ -708,40 +790,40 @@ WSCvariant& WSCvariant::operator =(void* val){
   setValue(val,NULL);
   return *this;
 }
-WSCvariant::operator char (){
+WSCvariant::operator char () const{
   return getChar();
 }
-WSCvariant::operator WSCuchar (){
+WSCvariant::operator WSCuchar () const{
   return getUnsignedChar();
 }
-WSCvariant::operator short (){
+WSCvariant::operator short () const{
   return getShort();
 }
-WSCvariant::operator WSCushort (){
+WSCvariant::operator WSCushort () const{
   return getUnsignedShort();
 }
-WSCvariant::operator long (){
+WSCvariant::operator long () const{
   return getLong();
 }
-WSCvariant::operator WSCulong (){
+WSCvariant::operator WSCulong () const{
   return getUnsignedLong();
 }
-WSCvariant::operator int (){
+WSCvariant::operator int () const{
   return getInt();
 }
-WSCvariant::operator WSCuint (){
+WSCvariant::operator WSCuint () const{
   return getUnsignedInt();
 }
-WSCvariant::operator float (){
+WSCvariant::operator float () const{
   return getFloat();
 }
-WSCvariant::operator double (){
+WSCvariant::operator double () const{
   return getDouble();
 }
 WSCvariant::operator char* (){
   return getCharPtr();
 }
-WSCvariant::operator WSCbase* (){
+WSCvariant::operator WSCbase* () const{
   return getInstancePtr();
 }
 #if 0
@@ -751,7 +833,18 @@ WSCvariant::operator WSCstring(){
   return str;
 }
 #endif
-WSCvariant::operator void* (){
+WSCvariant::operator void* () const{
   return getVoidPtr();
 }
-
+WSCbool WSCvariant::getNoAllocated() const{
+  return _no_allocated;
+}
+void WSCvariant::setNoAllocated(WSCbool fl){
+  _no_allocated = fl;
+}
+char* WSCvariant::getCharPtrConst() const{
+  if (_type == WSTcharptr){
+    return (char*)_data;
+  }
+  return "";
+}
