@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCvbtn.h>
@@ -39,6 +35,7 @@
 #include <WSDappDev.h>
 #include <WSDmwindowDev.h>
 #include <WSCguiPolicy.h>
+#include <ctype.h>
 
 WSMFguiClassInitialize(WSCvbtn, WSCvlabel);
 WSMFversion(WSCvbtn, WSCvlabel);
@@ -53,17 +50,20 @@ WSCvbtn::WSCvbtn(WSCbase* base, char* objname):
 
   _push_pixmap = -1;
   _ef_move = True;
-  _td_name = WSGFstrdup("");
-  _bd_name = WSGFstrdup("");
-  _ld_name = WSGFstrdup("");
-  _rd_name = WSGFstrdup("");
-  _tb_name = WSGFstrdup("");
-  _btb_name = WSGFstrdup("");
+  _td_name = NULL;
+  _bd_name = NULL;
+  _ld_name = NULL;
+  _rd_name = NULL;
+  _tb_name = NULL;
+  _btb_name = NULL;
+  _sc_str = NULL;
   _emboss = True;
   _focus_border = True;
-  _mask_images = NULL;
   _ext_draw_proc = NULL;
   _no_background = False;
+  _pixmap_overlapped = True;
+  _alt_focus_registered = False;
+  _mouse_in = False;
   WSGIappObjectList()->addEvent(this,WSEV_GUI_POLICY_CH);
 
   WSMFpropertyCreateStart
@@ -80,6 +80,9 @@ WSCvbtn::WSCvbtn(WSCbase* base, char* objname):
     WSMFpropertyCreate(WSNbacktab,  char*,   _btb_name,WSSbacktab);
 //    WSMFpropertyCreate(WSNreturn,  char*,  _rt_name,WSSreturn);
     WSMFpropertyValueChangeDef(WSCvbtn,WSNemboss,WSCbool);
+    WSMFpropertyCreate(WSNpixmapOverlap, WSCbool, _pixmap_overlapped,WSSpixmapOverlap);
+    WSMFpropertySetSelection(WSRbool3, WSRbool3D);
+    WSMFpropertyCreate(WSNshortcut,  char*,   _sc_str,WSSshortcut);
 
     WSMFpropertyDelete(WSNshadowType);
 
@@ -96,13 +99,15 @@ WSCvbtn::WSCvbtn(WSCbase* base, char* objname):
 
 WSMFproperty(WSCvbtn, WSNpushPixmap,      short,   _push_pixmap, -1);
 WSMFproperty(WSCvbtn, WSNenableFocusMove, WSCbool, _ef_move, True);
-WSMFproperty(WSCvbtn, WSNupward,    char*,   _td_name, WSGFstrdup(""));
-WSMFproperty(WSCvbtn, WSNdownward, char*,   _bd_name, WSGFstrdup(""));
-WSMFproperty(WSCvbtn, WSNleftward,   char*,   _ld_name, WSGFstrdup(""));
-WSMFproperty(WSCvbtn, WSNrightward,  char*,   _rd_name, WSGFstrdup(""));
-WSMFproperty(WSCvbtn, WSNtab,  char*,   _tb_name, WSGFstrdup(""));
-WSMFproperty(WSCvbtn, WSNbacktab,  char*,   _btb_name, WSGFstrdup(""));
+WSMFproperty(WSCvbtn, WSNupward,    char*,   _td_name, NULL);
+WSMFproperty(WSCvbtn, WSNdownward, char*,   _bd_name, NULL);
+WSMFproperty(WSCvbtn, WSNleftward,   char*,   _ld_name, NULL);
+WSMFproperty(WSCvbtn, WSNrightward,  char*,   _rd_name, NULL);
+WSMFproperty(WSCvbtn, WSNtab,  char*,   _tb_name, NULL);
+WSMFproperty(WSCvbtn, WSNbacktab,  char*,   _btb_name, NULL);
+WSMFproperty(WSCvbtn, WSNpixmapOverlap, WSCbool, _pixmap_overlapped, True);
 //WSMFproperty(WSCvbtn, WSNreturn, char*,   _rt_name, WSGFstrdup(""));
+WSMFproperty(WSCvbtn, WSNshortcut,  char*,   _sc_str, NULL);
 
 void WSCvbtn::setWorkWSNpushPixmap(short) {}
 void WSCvbtn::getWorkWSNpushPixmap(short*) {}
@@ -120,52 +125,56 @@ void WSCvbtn::setWorkWSNtab(char*) {}
 void WSCvbtn::getWorkWSNtab(char**) {}
 void WSCvbtn::setWorkWSNbacktab(char*) {}
 void WSCvbtn::getWorkWSNbacktab(char**) {}
+void WSCvbtn::setWorkWSNpixmapOverlap(WSCbool) {}
+void WSCvbtn::getWorkWSNpixmapOverlap(WSCbool*) {}
+void WSCvbtn::setWorkWSNshortcut(char* data) {
+  if (_alt_focus_registered == False){
+      _alt_focus_registered = True;
+    WSGIappDev()->addAltFocusInstance(this);
+  }
+}
+void WSCvbtn::getWorkWSNshortcut(char**) {}
 
 WSCvbtn::~WSCvbtn() {
   WSGIappObjectList()->delEvent(this,WSEV_GUI_POLICY_CH);
-
+  if (_sc_str != NULL){
+    delete[] _sc_str;
+    _sc_str = NULL;
+  }
+  if (_alt_focus_registered != False){
+    _alt_focus_registered = False;
+    WSGIappDev()->delAltFocusInstance(this);
+  }
   if (getFocus() != False){
     WSGIappKeyboard()->resetFocusInputWorkProc();
   }
+  if (WSGIappKeyboard()->getSpecialFocusInstance() == this){
+    WSGIappKeyboard()->resetSpecialInputWorkProc();
+  }
   if (_td_name != NULL){
-    delete _td_name;
+    delete[] _td_name;
     _td_name = NULL;
   }
   if (_bd_name != NULL){
-    delete _bd_name;
+    delete[] _bd_name;
     _bd_name = NULL;
   }
   if (_ld_name != NULL){
-    delete _ld_name;
+    delete[] _ld_name;
     _ld_name = NULL;
   }
   if (_rd_name != NULL){
-    delete _rd_name;
+    delete[] _rd_name;
     _rd_name = NULL;
   }
   if (_tb_name != NULL){
-    delete _tb_name;
+    delete[] _tb_name;
     _tb_name = NULL;
   }
   if (_btb_name != NULL){
-    delete _btb_name;
+    delete[] _btb_name;
     _btb_name = NULL;
   }
-  if (_mask_images != NULL){
-    if (_mask_images[0] != NULL){
-      delete _mask_images[0];
-    }
-    if (_mask_images[1] != NULL){
-      delete _mask_images[1];
-    }
-    if (_mask_images[2] != NULL){
-      delete _mask_images[2];
-    }
-    if (_mask_images[3] != NULL){
-      delete _mask_images[3];
-    }
-    delete _mask_images;
-  }
 }
 WSCbool WSCvbtn::_special_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool fl) {
   WSCvbtn* obj = (WSCvbtn*)ptr;
@@ -174,15 +183,16 @@ WSCbool WSCvbtn::_special_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool
   if (obj->getVisible() == False || obj->getSensitive() == False) {
     return True;
   }
-  if (fl == True && key == WSK_Return) {
+  if (fl == True && (key == WSK_Return || key == WSK_KP_Enter)) {
     if (obj->_press_state == False) {
       obj->_press_state = True;
       obj->setAbsoluteDraw(True);
       obj->redraw();
     }
+    obj->execEventProc(WSEV_KEY_PRESS, keyboard);
     return False;
   } else
-  if (fl == False && key == WSK_Return) {
+  if (fl == False && (key == WSK_Return || key == WSK_KP_Enter)) {
     if (obj->_press_state == True) {
       obj->_press_state = False;
       obj->setAbsoluteDraw(True);
@@ -191,6 +201,7 @@ WSCbool WSCvbtn::_special_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool
         obj->onActivate();
       }
     }
+    obj->execEventProc(WSEV_KEY_RELEASE, keyboard);
     return False;
   }
   return True;
@@ -204,14 +215,14 @@ void WSCvbtn::_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool fl) {
     return;
   }
 
-  if (fl == True && key == WSK_Return) {
+  if (fl == True && (key == WSK_Return || key == WSK_KP_Enter)) {
     if (obj->_press_state == False) {
       obj->_press_state = True;
       obj->setAbsoluteDraw(True);
       obj->redraw();
     }
   } else
-  if (fl == False && key == WSK_Return) {
+  if (fl == False && (key == WSK_Return || key == WSK_KP_Enter)) {
     if (obj->_press_state == True) {
       obj->_press_state = False;
       obj->setAbsoluteDraw(True);
@@ -314,26 +325,62 @@ void WSCvbtn::_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool fl) {
   }
 }
 
-void WSCvbtn::onKey(WSDkeyboard* ,WSCbool) {}
-//void WSCvbtn::onKeyRelease(WSDkeyboard*) {}
-#if 0
+void WSCvbtn::onKey(WSDkeyboard* keyb,WSCbool fl) {
+  if (_sc_str == NULL){
+    return;
+  }
+  if (getVisible() == False){
+    return;
+  }
+  if (getSensitive() == False){
+    return;
+  }
+  if (fl == False && _press_state != False) {
+    _press_state = False;
+    setAbsoluteDraw(True);
+    redraw();
+  }
+  WSCbool altkey = keyb->withAlt();
+  if (altkey == False){
+    return;
+  }
+  char* text = keyb->getText();
+  char buf[2];
+  buf[0] = (char)toupper(text[0]);
+  buf[1] = 0;
+  char buf2[2];
+  buf2[0] = (char)toupper(_sc_str[0]);
+  buf2[1] = 0;
+  if (fl == True && !strcmp(buf,buf2)){
+    if (fl != False && _press_state == False) {
+      _press_state = True;
+      setAbsoluteDraw(True);
+      redraw();
+      setFocus();
+    }else{
+      if (_press_state != False) {
+        _press_state = False;
+        setAbsoluteDraw(True);
+        redraw();
+      }
+    }
+    onActivate();
+    keyb->setText("");
+  }
+}
 long WSCvbtn::execEventProc(long ev, void* data) {
-
   if (ev == WSEV_KEY_PRESS) {
-//    onKeyPress((WSDkeyboard*)data);
-    onKey((WSDkeyboard*)data,True);
+    onKey(WSGIappKeyboard(),True);
   }else
   if (ev == WSEV_KEY_RELEASE) {
-//    onKeyRelease((WSDkeyboard*)data);
-    onKey((WSDkeyboard*)data,False);
-//  }else
-//  if (ev == WSEV_ACTIVATE) {
-//    onActivate();
+    onKey(WSGIappKeyboard(),False);
+  }else
+  if (ev == WSEV_ACTIVATE) {
+    onActivate();
   }
 
   return WSCvlabel::execEventProc(ev, data);
 }
-#endif
 long WSCvbtn::_device_initialize() {
   WSDdev* dev = getowndev();
   if (dev == NULL) {
@@ -353,7 +400,7 @@ void WSCvbtn::onFocusChange(WSCbool fl){
     return;
   }
   if (fl != False){
-    WSGIappKeyboard()->setFocusInputWorkProc(_key_handler, NULL, this);
+    WSGIappKeyboard()->setFocusInputWorkProc(_key_handler, NULL, this,this);
     if (getowndev() != NULL) {
       WSGIappKeyboard()->resetFep(getowndev());
     }
@@ -371,7 +418,7 @@ void WSCvbtn::onSpecialFocusChange(WSCbool fl){
     return;
   }
   if (fl != False){
-    WSGIappKeyboard()->setSpecialInputWorkProc(_special_key_handler, this);
+    WSGIappKeyboard()->setSpecialInputWorkProc(_special_key_handler, this,this);
     if (getowndev() != NULL) {
       WSGIappKeyboard()->resetFep(getowndev());
     }
@@ -420,13 +467,16 @@ long WSCvbtn::draw(){
       st.top_shadow_color = _ts_color;
       st.bottom_shadow_color = _bs_color;
       st.label_string = _label_string;
+      st.shortcut_string = _sc_str;
       st.label_pixmap = _label_pixmap;
       st.blink_pixmap = _blink_pixmap;
       st.push_pixmap = _push_pixmap;
       st.press_state = _press_state;
       st.orientation = _orientation;
+#ifndef NO_GRADATION
       st.gradation = _grad_no;
       st.gradation_margin = _grad_margin;
+#endif
       st.margin_left = _margin_left;
       st.margin_right = _margin_right;
       st.margin_top = _margin_top;
@@ -437,22 +487,24 @@ long WSCvbtn::draw(){
       st.reverse_flag = _rv_fl;
       st.enable_focus_move = _ef_move;
       st.no_background = _no_background;
+      st.pixmap_overlapped = _pixmap_overlapped;
+      st.auto_line_feed = _auto_line_feed;
       _ext_draw_proc(&st);
       return WS_NO_ERR;
     }
   }
 
+
+//  WSCbool absolute = getAbsoluteDraw();
+  if (_absolute_draw == False && dev->isExposed(_x, _y, _w, _h) == False) {
+    return WS_NO_ERR;
+  }
+
   short x = _x;
   short y = _y;
   WSCushort w = _w;
   WSCushort h = _h;
-  long policy = WSGIappDev()->getGuiPolicy();
-
-
-  WSCbool absolute = getAbsoluteDraw();
-  if (absolute == False && dev->isExposed(x, y, w, h) == False) {
-    return WS_NO_ERR;
-  }
+  long gpolicy       = WSGIappGuiPolicy()->getGuiPolicy();
   WSCushort st       = _shadow_thick;
   long blinking      = _bl_fl;
   long blinkfore     = WSGIappBlink()->getBlinkFore(_blink_rate);
@@ -465,37 +517,8 @@ long WSCvbtn::draw(){
   short backblinkno  = _bg_blink_color;
   short tsno         = _ts_color;
   short bsno         = _bs_color;
-
-#ifndef WS_EMBED
-  if (policy & WS_POLICY_AQUA){
-//    WSDimage* image = WSGIappImageSet()->getImage(WS_AQUA_BTN_IMAGE);
-//    WSDimage* mask = WSGIappImageSet()->getImage(WS_AQUA_BTN_MASK_IMAGE);
-//    WSCushort pw = image->getImageWidth();
-//    WSCushort ph = image->getImageHeight();
-    if (_mask_images != NULL){
-      WSCulong val = (WSCulong)((WSCulong)w << 16 + h);
-      WSCulong val2 = (WSCulong)_mask_images[4];
-      if (val != val2){
-//printf("WSCvbtn::draw size changed...%d %d \n",val,val2);
-        delete _mask_images[0];
-        delete _mask_images[1];
-        delete _mask_images[2];
-        delete _mask_images[3];
-        delete _mask_images;
-        _mask_images = NULL;
-      }
-    }
-    if (_mask_images == NULL){
-//printf("WSCvbtn::draw images allocated...\n");
-      _mask_images = new WSDmwindowDev*[5];
-      _mask_images[0] = WSGFgetAquaImage(tsno,w,h,1);
-      _mask_images[1] = WSGFgetAquaImage(WS_DF_AQUAFORECOLOR,w,h,1);
-      _mask_images[2] = WSGFgetAquaImage(WS_DF_AQUAHIGHLIGHTCOLOR,w,h,1);
-      _mask_images[3] = WSGFgetAquaMaskImage(w,h,1);
-      _mask_images[4] = (WSDmwindowDev*)((WSCulong)((WSCulong)w << 16 + h));
-    }
-  }
-#endif
+  short alignment    = getAlignment();
+  short sensitive    = getSensitive();
 
   if ( shadowtype == WS_SHADOW_NONE &&
         blinking  != False &&
@@ -504,15 +527,28 @@ long WSCvbtn::draw(){
     return WS_NO_ERR;
   }
 
-  WSCstring str;
-  str.setString(_label_string);
+  WSCstring* str;
+//  str.setStaticString(_label_string);
+  WSCstring tstr;
   if (_orientation == WS_VERTICAL) {
-    WSGFcreateVerticalString(&str);
+    str = &tstr;
+    str->setString(_label_string);
+    WSGFcreateVerticalString(&_str);
   } else {
-    str.replaceString("\\n", "\n", 0);
+    str = &_str;
+    str->replaceString("\\n", "\n", 0);
+    if (_auto_line_feed != False){
+      WSCushort ww = w;
+      if (w > st*2){
+        ww -= st*2;
+      }
+      WSCstring str2 = WSGFcreateLineFeedStringByWidth(str,_font,ww);
+      str = &tstr;
+      str->setString(str2);
+    }
   }
 
-  long err = dev->beginDraw(x, y, w, h, absolute);
+  long err = dev->beginDraw(x, y, w, h, _absolute_draw);
   if (err != WS_NO_ERR){
     return WS_NO_ERR;
   }
@@ -537,23 +573,11 @@ long WSCvbtn::draw(){
       dev->setForeColor(bgno);
     }
   }
+#ifndef NO_GRADATION
   if (_grad_no == 0){
-    if (!(policy & WS_POLICY_AQUA) || _label_pixmap != -1||
-     _push_pixmap != -1 || _blink_pixmap != -1){
-      dev->drawFillRect(0, 0, w, h);
-    }else{
-#ifndef WS_EMBED
-      if (_press_state != False) {
-        _mask_images[2]->copyToWindowWithMask(dev,0,0,w,h,1,1,_mask_images[3]);
-      }else if (_mouse_in != False){
-        _mask_images[2]->copyToWindowWithMask(dev,0,0,w,h,0,0,_mask_images[3]);
-      }else if (getFocus() != False){
-        _mask_images[1]->copyToWindowWithMask(dev,0,0,w,h,0,0,_mask_images[3]);
-      }else{
-        _mask_images[0]->copyToWindowWithMask(dev,0,0,w,h,0,0,_mask_images[3]);
-      }
 #endif
-    }
+    dev->drawFillRect(0, 0, w, h);
+#ifndef NO_GRADATION
   }else{
     if (_press_state == True) {
       dev->drawGradation(_grad_no,tsno,bgno,bsno,0, 1, w, h,_grad_margin);
@@ -561,7 +585,7 @@ long WSCvbtn::draw(){
       dev->drawGradation(_grad_no,tsno,bgno,bsno,0, 0, w, h,_grad_margin);
     }
   }
-
+#endif
   short          xx = st + _margin_left;
   short          yy = st + _margin_top;
   WSCushort ww = w - st*2;
@@ -581,8 +605,7 @@ long WSCvbtn::draw(){
   }
 
   WSDimage* image = NULL;
-  if (!(policy & WS_POLICY_AQUA) || _label_pixmap != -1||
-     _push_pixmap != -1 || _blink_pixmap != -1){
+  if ( _label_pixmap != -1|| _push_pixmap != -1 || _blink_pixmap != -1){
     if (blinking  != False &&
         blinktype != WS_BACK &&
         blinkfore != False ){
@@ -592,7 +615,54 @@ long WSCvbtn::draw(){
         if (_press_state == True){
           dy++;
         }
-        dev->drawImage (xx, dy, ww, hh, image, getAlignment());
+        if (_pixmap_overlapped != False){
+          dev->drawImage (xx, dy, ww, hh, image, alignment);
+        }else{
+//--
+/*
+    When there was both label pixmap and label string, then
+    we have to put them together in correct places. There are
+    nine options.
+*/
+
+    // First two options: Label is centered. All goes as original.
+
+    if (alignment == WS_LEFT || alignment == WS_RIGHT){
+          dev->drawImage (xx, dy, ww, hh, image, alignment);
+    }
+
+    // Next, the special option: Label is centered and in middle. Here,
+    // we need to center label pixmap and label string.
+
+    if (alignment == WS_CENTER){
+      dev->drawImage (xx + (ww - WSGIappFontSet()->getFont(_font)->getStringWidth(str)
+        - image->getImageWidth())/2, dy, ww, hh, image, WS_LEFT);
+    }
+
+    // Next, three options: Label is on top. Here, we center
+    // pixmap on bottom middle of the area, letting free the top
+    // middle of the area for label string.
+  
+    if (alignment == WS_LEFT_TOP){
+      dev->drawImage (xx, dy+(hh/2), ww, hh/2, image, WS_LEFT);
+    }else if (alignment == WS_TOP){
+      dev->drawImage (xx, dy+(hh/2), ww, hh/2, image, WS_CENTER);
+    }else if (alignment == WS_RIGHT_TOP){
+      dev->drawImage (xx, dy+(hh/2), ww, hh/2, image, WS_RIGHT);
+    }
+    // Third three options: Label is on botton. Here, we center
+    // pixmap on top middle of the area, letting free the bottom
+    // middle of the area for label string.
+
+    if (alignment == WS_LEFT_BOTTOM){
+      dev->drawImage (xx, dy, ww, hh/2, image, WS_LEFT);
+    }else if (alignment == WS_BOTTOM){
+      dev->drawImage (xx, dy, ww, hh/2, image, WS_CENTER);
+    }else if (alignment == WS_RIGHT_BOTTOM){
+      dev->drawImage (xx, dy, ww, hh/2, image, WS_RIGHT);
+    }
+//--
+        }
       }
     }else{
       short dy=yy;
@@ -613,23 +683,78 @@ long WSCvbtn::draw(){
         }
       }
       if (drawfl != False){
-        dev->drawImage (xx, dy, ww, hh, image, getAlignment());
+        if (_pixmap_overlapped != False){
+          dev->drawImage (xx, dy, ww, hh, image, alignment);
+        }else{
+//--
+/*
+    When there was both label pixmap and label string, then
+    we have to put them together in correct places. There are
+    nine options.
+*/
+
+    // First two options: Label is centered. All goes as original.
+
+    if (alignment == WS_LEFT ||
+        alignment == WS_RIGHT){
+          dev->drawImage (xx, dy, ww, hh, image, alignment);
+    }
+
+    // Next, the special option: Label is centered and in middle. Here,
+    // we need to center label pixmap and label string.
+
+    if (alignment == WS_CENTER){
+      dev->drawImage (xx + (ww - WSGIappFontSet()->getFont(_font)->getStringWidth(_str)
+        - image->getImageWidth())/2, dy, ww, hh, image, WS_LEFT);
+    }
+
+    // Next, three options: Label is on top. Here, we center
+    // pixmap on bottom middle of the area, letting free the top
+    // middle of the area for label string.
+
+    if (alignment == WS_LEFT_TOP){
+      dev->drawImage (xx, dy+(hh/2), ww, hh/2, image, WS_LEFT);
+    }else if (alignment == WS_TOP){
+      dev->drawImage (xx, dy+(hh/2), ww, hh/2, image, WS_CENTER);
+    }else if (alignment == WS_RIGHT_TOP){
+      dev->drawImage (xx, dy+(hh/2), ww, hh/2, image, WS_RIGHT);
+    }
+    // Third three options: Label is on botton. Here, we center
+    // pixmap on top middle of the area, letting free the bottom
+    // middle of the area for label string.
+
+    if (alignment == WS_LEFT_BOTTOM){
+      dev->drawImage (xx, dy, ww, hh/2, image, WS_LEFT);
+    }else if (alignment == WS_BOTTOM){
+      dev->drawImage (xx, dy, ww, hh/2, image, WS_CENTER);
+    }else if (alignment == WS_RIGHT_BOTTOM){
+      dev->drawImage (xx, dy, ww, hh/2, image, WS_RIGHT);
+    }
+//--
+        }
       }
     }
   }
   WSCbool specialfocus= getSpecialFocus(); 
-  if ((!(policy & WS_POLICY_AQUA) || _label_pixmap != -1||
-     _push_pixmap != -1 || _blink_pixmap != -1) && st > 0){
+  if (st > 0){
     char	type = WS_SHADOW_OUT;
-    if (getSensitive() == False ||
-        (getPropertyEditMode() != False && getUserData(WS_DET) == 0)) {
+#ifndef WS_EMBED
+    if (sensitive == False ||
+        (getPropertyEditMode() != False && _sensitive_work == False)) {
       type = WS_SHADOW_OUT;
     }else if (_press_state == True) {
       type = WS_SHADOW_IN;
     }
+#else
+    if (sensitive == False){
+      type = WS_SHADOW_OUT;
+    }else if (_press_state == True) {
+      type = WS_SHADOW_IN;
+    }
+#endif
     if (specialfocus == False){
       WSGFdrawBtnShadow(dev, type, st, tsno, tsno, bsno, bgno,0, 0, w, h);
-    }else if (WSGIappDev()->getGuiPolicy() & WS_POLICY_ORIGINAL){
+    }else if (gpolicy & WS_POLICY_ORIGINAL){
       WSGFdrawBtnShadow(dev, type, st, tsno, tsno, bsno, bgno,0, 0, w, h);
     }else if (_focus_border == False) {
       WSGFdrawBtnShadow(dev, type, st, tsno, tsno, bsno, bgno,0, 0, w, h);
@@ -644,8 +769,24 @@ long WSCvbtn::draw(){
     }    
   }
 
-  if (getSensitive() == False ||
-      (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0)){
+  if (_sc_str != NULL){
+    WSCstring _accel_char(_sc_str);
+    _accel_char.to_lower();
+
+    WSCstring _accel_str(*str);
+    _accel_str.to_lower();
+    long _char_pos = _accel_str.isExist(_accel_char);
+    if(_char_pos > -1 ){
+      dev->setUnderLineChar(_char_pos,1);
+    }
+  }
+
+#ifndef WS_EMBED
+  if (sensitive == False ||
+      (getPropertyEditMode() != False && _sensitive_work == False)){
+#else
+  if (sensitive == False){
+#endif
     if (_emboss != False){
 
       if (blinking  != False &&
@@ -663,13 +804,17 @@ long WSCvbtn::draw(){
           } else {
             dev->setForeColor(tsno);
           }
-          dev->drawString(xx+1, yy+1, ww, hh, &str, _font, getAlignment(), -1, False);
+          if (_pixmap_overlapped == False && image){
+          }else{
+            dev->drawString(xx+1, yy+1, ww, hh, str, _font, alignment, -1, False);
+          }
         }
       }
     }else{
       dev->setHatchPattern(8);
     }
   }
+
 #if 0
   if (specialfocus != False && _focus_border != False && 
       (WSGIappDev()->getGuiPolicy() & WS_POLICY_ORIGINAL)){
@@ -697,8 +842,12 @@ long WSCvbtn::draw(){
         twinblink != False) {
       dev->setForeColor(backblinkno);
     } else {
-      if( _emboss != False && (getSensitive() == False ||
-          (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0))){
+#ifndef WS_EMBED
+      if( _emboss != False && (sensitive == False ||
+          (getPropertyEditMode() != False && _sensitive_work == False))){
+#else
+      if( _emboss != False && sensitive == False){
+#endif
         dev->setForeColor(bsno);
       }else{
         dev->setForeColor(bgno);
@@ -711,8 +860,12 @@ long WSCvbtn::draw(){
         twinblink != False) {
       dev->setForeColor(blinkcolorno);
     }else{
-      if ( _emboss != False && (getSensitive() == False ||
-        (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0))){
+#ifndef WS_EMBED
+      if ( _emboss != False && (sensitive == False ||
+        (getPropertyEditMode() != False && _sensitive_work == False))){
+#else
+      if ( _emboss != False && sensitive == False){
+#endif
         dev->setForeColor(bsno);
       }else{
         dev->setForeColor(fgno);
@@ -726,26 +879,151 @@ long WSCvbtn::draw(){
       twinblink == False ){
       //do nothing...
   }else{
-    if (_press_state == True) {
-      dev->drawString(xx, yy+1, ww, hh, &str,  _font, getAlignment(), -1, False);
+    if (_pixmap_overlapped != False){
+      if (_press_state == True) {
+        dev->drawString(xx, yy+1, ww, hh, str,  _font, alignment, -1, False);
+      }else{
+        dev->drawString(xx, yy, ww, hh, str, _font, alignment, -1, False);
+      }
     }else{
-      dev->drawString(xx, yy, ww, hh, &str, _font, getAlignment(), -1, False);
+//--
+/*
+    When there was both label pixmap and label string, then
+    we have to put them together in correct places. There are
+    several options.
+*/
+   if (image){
+
+    int new_xx=0, new_yy=0, new_ww=0, new_hh=0, new_alignment=0;
+
+    new_alignment = getAlignment();
+
+    switch (alignment)
+    {
+
+    // When label is in middle, we draw string at left, right or over
+    // the label pixmap.
+
+      case WS_LEFT:
+       new_xx = xx + image->getImageWidth();
+       new_yy = yy;
+       new_ww = ww - image->getImageWidth();
+       new_hh = hh;
+       break;
+      case WS_CENTER:
+       new_xx = xx + (ww - WSGIappFontSet()->getFont(_font)->getStringWidth(str)
+        - image->getImageWidth())/2 + image->getImageWidth();
+       new_yy = yy;
+       new_ww = ww;
+       new_hh = hh;
+       new_alignment = WS_LEFT;
+       break;
+      case WS_RIGHT:
+       new_xx = xx;
+       new_yy = yy;
+       new_ww = ww - image->getImageWidth();
+       new_hh = hh;
+       break;
+
+    // When label is on top, we draw string inside the
+    // top middle area.
+      case WS_LEFT_TOP:
+       new_xx = xx;
+       new_yy = yy;
+       new_ww = ww;
+       new_hh = hh/2;
+       new_alignment = WS_LEFT;
+       break;
+      case WS_TOP:
+       new_xx = xx;
+       new_yy = yy;
+       new_ww = ww;
+       new_hh = hh/2;
+       new_alignment = WS_CENTER;
+       break;
+      case WS_RIGHT_TOP:
+       new_xx = xx;
+       new_yy = yy;
+       new_ww = ww;
+       new_hh = hh/2;
+       new_alignment = WS_RIGHT;
+       break;
+
+    // When label is on bottom, we draw string inside the
+    // bottom middle area.
+      case WS_LEFT_BOTTOM:
+       new_xx = xx;
+       new_yy = yy + (hh/2);
+       new_ww = ww;
+       new_hh = hh/2;
+       new_alignment = WS_LEFT;
+       break;
+      case WS_BOTTOM:
+       new_xx = xx;
+       new_yy = yy + (hh/2);
+       new_ww = ww;
+       new_hh = hh/2;
+       new_alignment = WS_CENTER;
+       break;
+      case WS_RIGHT_BOTTOM:
+       new_xx = xx;
+       new_yy = yy + (hh/2);
+       new_ww = ww;
+       new_hh = hh/2;
+       new_alignment = WS_RIGHT;
+       break;
+
+      default:
+        break;
+    }
+#ifndef WS_EMBED
+    if (sensitive == False ||
+      (getPropertyEditMode() != False && _sensitive_work == False)){
+#else
+    if (sensitive == False){
+#endif
+      if (_emboss != False){
+
+        if (twinblink == False || blinking == False){
+          if (blinking  != False &&
+              blinkfore != False &&
+              blinktype != WS_BACK &&
+              twinblink != False) {
+            dev->setForeColor(blinkcolorno);
+          } else {
+            dev->setForeColor(tsno);
+          }
+          dev->drawString(new_xx+1, new_yy+1, new_ww, new_hh, str, _font, new_alignment, -1, False);
+        }
+      }
+    }
+    if (_press_state == True) {
+      dev->drawString(new_xx, new_yy+1, new_ww, new_hh, str,  _font, new_alignment, -1, False);
+    }else{
+      dev->drawString(new_xx, new_yy, new_ww, new_hh, str, _font, new_alignment, -1, False);
+    }
+
+  }else{
+    if (_press_state == True) {
+      dev->drawString(xx, yy+1, ww, hh, str,  _font, alignment, -1, False);
+    }else{
+      dev->drawString(xx, yy, ww, hh, str, _font, alignment, -1, False);
 
     }
-    if (!(policy & WS_POLICY_AQUA)){
-      // draw dashed focused line...
-      if (getFocus() != False && _ef_move != False && _focus_border != False) {
-        if (w > 4 + st*2 && h > 4 + st*2) {
-          // 褹ϣ˸
-          dev->setLineWidth(0);
-          // 褹˸
-          dev->setLineDashType(8);
-          if (_press_state == True) {
-            //1 dot below
-            dev->drawRect(st + 2, st + 3, w - st * 2 - 4, h - st * 2 - 4);
-          } else {
-            dev->drawRect(st + 2, st + 2, w - st * 2 - 4, h - st * 2 - 4);
-          }
+  }
+//--
+
+    }
+    // draw dashed focused line...
+    if (getFocus() != False && _ef_move != False && _focus_border != False) {
+      if (w > 4 + st*2 && h > 4 + st*2) {
+        dev->setLineWidth(0);
+        dev->setLineDashType(8);
+        if (_press_state == True) {
+          //1 dot below
+          dev->drawRect(st + 2, st + 3, w - st * 2 - 4, h - st * 2 - 4);
+        } else {
+          dev->drawRect(st + 2, st + 2, w - st * 2 - 4, h - st * 2 - 4);
         }
       }
     }
@@ -809,10 +1087,6 @@ void WSCvbtn::onMouseRelease(WSCpoint* point){
 
 void WSCvbtn::onMouseIn(WSCpoint* pt){
   _mouse_in = True;
-  if (WSGIappDev()->getGuiPolicy() & WS_POLICY_AQUA){
-    setAbsoluteDraw(True);
-    redraw();
-  }
   WSCvlabel::onMouseIn(pt);
 }
 void WSCvbtn::onMouseOut(){
@@ -821,9 +1095,6 @@ void WSCvbtn::onMouseOut(){
     _press_state = False;
     setAbsoluteDraw(True);
     redraw();
-  }else if (WSGIappDev()->getGuiPolicy() & WS_POLICY_AQUA){
-    setAbsoluteDraw(True);
-    redraw();
   }
   WSCvlabel::onMouseOut();
 }
@@ -839,7 +1110,7 @@ WSMFtrace("WSCvbtn::addProcedure trigger=%d not supported.\n", ac->getTrigger())
                                  WSEV_MOUSE_RELEASE_BIT | WSEV_MOUSE_OUT_BIT );
 
     _event_mask |= ac->getEffectiveTriggerMask();
-    _add_prcs(&_prc_list, ac);
+    _add_prcs(ac);
 
     WSDdev* dev = _dev;
     if (dev != NULL){
@@ -864,16 +1135,22 @@ void WSCvbtn::setNoBackground(WSCbool fl){
   _no_background = fl;
 }
 void(*WSCvbtn::_ext_draw_proc)(void*) = NULL;
-WSCstring WSCvbtn::_policy_name;
+char* WSCvbtn::_policy_name = NULL;
 
 void WSCvbtn::onGuiPolicyChange(long policy){
   WSCvlabel::onGuiPolicyChange(policy);
   if (WSGIappDev()->getExtGuiPolicySelected() == False){
     _ext_draw_proc = NULL;
   }else{
-    if (!strcmp((char*)_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
+    if (_policy_name != NULL &&
+        !strcmp(_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
     }else{
       _ext_draw_proc = (void(*)(void*))WSGIappDev()->getExtGuiPolicyProc("WSCvbtn::draw");
+      if (_policy_name != NULL){
+        delete _policy_name;
+      }
+      _policy_name = WSGFstrdup(WSGIappDev()->getExtGuiPolicyName());
+
     }
   }
   if (getVisible() != False){
