@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCvlabel.h>
@@ -64,7 +60,7 @@ WSCvlabel::WSCvlabel(WSCbase* base, char* objname):
   _select_fore_color = 0;
 
   _max_length = WS_VLABEL_MAXLEN_DEFAULT;
-  _label_string = WSGFstrdup("");
+  _label_string = NULL;
   _alignment_v = WS_CENTER;
   _alignment_h = WS_CENTER;
   _margin_top = 2;
@@ -73,10 +69,15 @@ WSCvlabel::WSCvlabel(WSCbase* base, char* objname):
   _margin_right = 2;
   _label_pixmap = -1;
   _blink_pixmap = -1;
+#ifndef NO_GRADATION
   _grad_no = 0;
   _grad_margin = 0;
+#endif
   _font = (char)WSGIappFontSet()->getDefaultFontNo();
   _orientation = WS_HORIZONTAL;
+  _auto_line_feed = False;
+  _need_clear = False;
+  _hilight_draw = 0;
 
   WSMFpropertyCreateStart
     WSMFparentCheckVerSrc(WSCvlabel);
@@ -112,10 +113,15 @@ WSCvlabel::WSCvlabel(WSCbase* base, char* objname):
     WSMFpropertyCreate(WSNfont,         WSCuchar,   _font,WSSfont  );
     WSMFpropertyCreate(WSNlabelPixmap,  short,     _label_pixmap,WSSlabelPixmap);
     WSMFpropertyCreate(WSNblinkPixmap,  short,   _blink_pixmap,WSSblinkPixmap);
-
+#ifndef NO_GRADATION
     WSMFpropertyCreate(WSNgradation,   WSCuchar,   _grad_no,WSSgradation);
       WSMFpropertySetSelection( WSRgradation, WSRgradationD);
     WSMFpropertyCreate(WSNgradationMargin,   WSCuchar,   _grad_margin,WSSgradationMargin);
+#endif
+    WSMFpropertyCreate(WSNkeepPixmapCache,WSCbool,_keep_pixmap_cache,WSSkeepPixmapCache );
+      WSMFpropertySetSelection(WSRbool4,WSRbool4D);
+    WSMFpropertyCreate(WSNautoLineFeeding,WSCbool,_auto_line_feed,WSSautoLineFeeding );
+      WSMFpropertySetSelection(WSRbool4,WSRbool4D);
 
 
   WSMFpropertyCreateEnd
@@ -136,7 +142,7 @@ WSMFproperty(WSCvlabel, WSNbackBlinkColor,    short,        _bg_blink_color, WS_
 WSMFproperty(WSCvlabel, WSNreverseFlag,       WSCbool,        _rv_fl, False              );
 WSMFproperty(WSCvlabel, WSNemboss,            WSCbool,        _emboss, False );
 WSMFproperty(WSCvlabel, WSNusePixmap,         WSCbool,       _use_pix, False );
-WSMFproperty(WSCvlabel, WSNlabelString, char*,    _label_string,  WSGFstrdup(""));
+WSMFproperty(WSCvlabel, WSNlabelString, char*,    _label_string,NULL);
 WSMFproperty(WSCvlabel, WSNorientation, WSCuchar, _orientation,     WS_HORIZONTAL );
 WSMFproperty(WSCvlabel, WSNalignmentV,  WSCuchar, _alignment_v,   WS_CENTER     );
 WSMFproperty(WSCvlabel, WSNmarginTop,   WSCuchar,_margin_top,    2             );
@@ -148,9 +154,12 @@ WSMFproperty(WSCvlabel, WSNmaxLength,   WSCushort,_max_length,    WS_VLABEL_MAXL
 WSMFproperty(WSCvlabel, WSNfont,        WSCuchar, _font,(char)WSGIappFontSet()->getDefaultFontNo());
 WSMFproperty(WSCvlabel, WSNlabelPixmap,  short,   _label_pixmap,  -1  );
 WSMFproperty(WSCvlabel, WSNblinkPixmap,  short,   _blink_pixmap,  -1  );
+#ifndef NO_GRADATION
 WSMFproperty(WSCvlabel, WSNgradation,    WSCuchar, _grad_no,  0  );
 WSMFproperty(WSCvlabel, WSNgradationMargin,    WSCuchar, _grad_margin,  0  );
-
+#endif
+WSMFproperty(WSCvlabel, WSNkeepPixmapCache, WSCbool,  _keep_pixmap_cache,True );
+WSMFproperty(WSCvlabel, WSNautoLineFeeding, WSCbool,  _auto_line_feed,False );
 void WSCvlabel::setWorkWSNshadowThickness(WSCuchar){}
 void WSCvlabel::getWorkWSNshadowThickness(WSCuchar*){}
 void WSCvlabel::setWorkWSNshadowType(char stype){
@@ -161,6 +170,9 @@ void WSCvlabel::setWorkWSNshadowType(char stype){
       _dev->setValue(WSDEV_USE_PIXMAP,(void*)&fl);
     }
   }
+  if (stype == WS_SHADOW_TRANS){
+    _need_clear = True;
+  }
 }
 void WSCvlabel::getWorkWSNshadowType(char*){}
 void WSCvlabel::setWorkWSNbackColor(short){}
@@ -189,7 +201,12 @@ void WSCvlabel::setWorkWSNusePixmap(WSCbool fl) {
 }
 
 void WSCvlabel::getWorkWSNusePixmap(WSCbool*){}
-void WSCvlabel::setWorkWSNlabelString(char* ){}
+void WSCvlabel::setWorkWSNlabelString(char* data){
+  _str.setStaticString(_label_string);
+  if (_shadow_type == WS_SHADOW_TRANS){
+    _need_clear = True;
+  }
+}
 void WSCvlabel::getWorkWSNlabelString(char**){}
 void WSCvlabel::setWorkWSNalignmentV(WSCuchar){}
 void WSCvlabel::getWorkWSNalignmentV(WSCuchar*){}
@@ -218,14 +235,20 @@ void WSCvlabel::setWorkWSNlabelPixmap(short){}
 void WSCvlabel::getWorkWSNlabelPixmap(short*){}
 void WSCvlabel::setWorkWSNblinkPixmap(short no){}
 void WSCvlabel::getWorkWSNblinkPixmap(short*){}
+#ifndef NO_GRADATION
 void WSCvlabel::setWorkWSNgradation(WSCuchar no){}
 void WSCvlabel::getWorkWSNgradation(WSCuchar*){}
 void WSCvlabel::setWorkWSNgradationMargin(WSCuchar no){}
 void WSCvlabel::getWorkWSNgradationMargin(WSCuchar*){}
+#endif
+void WSCvlabel::setWorkWSNkeepPixmapCache(WSCbool){}
+void WSCvlabel::getWorkWSNkeepPixmapCache(WSCbool*){}
+void WSCvlabel::setWorkWSNautoLineFeeding(WSCbool){}
+void WSCvlabel::getWorkWSNautoLineFeeding(WSCbool*){}
 
 WSCvlabel::~WSCvlabel() {
   if (_label_string != NULL) {
-    delete _label_string;
+    delete[] _label_string;
     _label_string = (char*)NULL;
   }
 }
@@ -277,6 +300,10 @@ dbprintf("WSCvlabel::draw %s:%d %s dev=NULL\n",__FILE__,__LINE__,getInstanceName
 dbprintf("WSCvlabel::draw %s:%d %s not exposed..\n",__FILE__,__LINE__,getInstanceName());
     return WS_NO_ERR;
   }
+  if (_shadow_type == WS_SHADOW_TRANS && _need_clear != False &&
+      absolute != False && dev->isExposed(x, y, w, h) == False){
+    return WS_NO_ERR;
+  }
 
   WSCushort st       = _shadow_thick;
   long blinking      = _bl_fl;
@@ -303,15 +330,29 @@ dbprintf("WSCvlabel::draw %s:%d %s not draw..\n",__FILE__,__LINE__,getInstanceNa
     return WS_NO_ERR;
   }
 
-  WSCstring str;
-  str.setString(_label_string);
+//  WSCstring str;
+//  str.setStaticString(_label_string);
+//printf("WSCvlabel::draw str=%s\n",str.getString());fflush(stdout);
+  WSCstring* str;
+  WSCstring tstr;
   if (_orientation == WS_VERTICAL) {
-    WSGFcreateVerticalString(&str);
+    str = &tstr;
+    str->setString(_label_string);
+    WSGFcreateVerticalString(str);
   } else {
-    str.replaceString("\\n", "\n", 0);
-    str.replaceString("\\\n", "\\n", 0);
+    str = &_str;
+    str->replaceString("\\n", "\n", 0);
+    str->replaceString("\\\n", "\\n", 0);
+    if (_auto_line_feed != False){
+      WSCushort ww = w;
+      if (w > st*2){
+        ww -= st*2;
+      }
+      WSCstring str2 = WSGFcreateLineFeedStringByWidth(str,_font,ww);
+      str = &tstr;
+      str->setString(str2);
+    }
   }
-
   long err = dev->beginDraw(x, y, w, h, absolute);
   if (err != WS_NO_ERR){
 dbprintf("WSCvlabel::draw %s:%d %s not ready..\n",__FILE__,__LINE__,getInstanceName());
@@ -369,21 +410,41 @@ dbprintf("WSCvlabel::draw %s:%d %s not ready..\n",__FILE__,__LINE__,getInstanceN
         dev->setForeColor(bgno);
       }
     }
+#ifndef NO_GRADATION
     if (_grad_no == 0){
+#endif
       dev->drawFillRect( 0, 0, w, h);
+#ifndef NO_GRADATION
     }else{
       dev->drawGradation(_grad_no,tsno,bgno,bsno,0, 0, w, h,_grad_margin);
     }
+#endif
   }
-  if (_select_color != 0){
-    dev->setForeColor(_select_color);
+  if (_select_color != 0 || _hilight_draw != False){
+    if (_select_color != 0){
+      dev->setForeColor(_select_color);
+    }else{
+      dev->setForeColor(fgno);
+    }
     if (_vk_draw == WS_TREE){
       short x1 = hh + hh/4 + hh*indent_level;
       if (_use_icon != 0){
         x1 = hh*2 + hh/4 + hh*indent_level;
       }
       if ( w - x1 > 0){
-        dev->drawFillRect( x1, 0, w -x1, h);
+        if (_select_color != 0){
+          if (_hilight_draw == False){
+            dev->drawFillRect( x1, 0, w -x1, h);
+          }else{
+            dev->drawFillRect( x1+1, 1, w -x1-2, h-2);
+          }
+        }
+        if (_hilight_draw != False){
+          dev->setForeColor(_select_color);
+          dev->setLineDashType(8);
+          dev->drawRect( x1, 0, w -x1, h);
+          dev->setLineDashType(0);
+        }
       }
     }else{
       short x1 = 0;
@@ -391,7 +452,18 @@ dbprintf("WSCvlabel::draw %s:%d %s not ready..\n",__FILE__,__LINE__,getInstanceN
         x1 = hh + hh/4 ;
       }
       if ( w - x1 > 0){
-        dev->drawFillRect( x1, 0, w -x1, h);
+        if (_select_color != 0){
+          if (_hilight_draw == False){
+            dev->drawFillRect( x1, 0, w -x1, h);
+          }else{
+            dev->drawFillRect( x1+1, 1, w -x1-2, h-2);
+          }
+        }
+        if (_hilight_draw != False){
+          dev->setLineDashType(8);
+          dev->drawRect( x1, 0, w -x1, h);
+          dev->setLineDashType(0);
+        }
       }
     }
   }
@@ -450,7 +522,7 @@ dbprintf("WSCvlabel::draw %s:%d %s not ready..\n",__FILE__,__LINE__,getInstanceN
     }else{
       if (_tw_fl == False || _bl_fl == False){
         if (_vk_draw != WS_VERBOSE){
-          dev->drawString(xx+1, yy+1, ww, hh, &str, _font,
+          dev->drawString(xx+1, yy+1, ww, hh, str, _font,
                            getAlignment(), -1, False);
         }else{
           WSCstring sep(",");
@@ -466,7 +538,7 @@ dbprintf("WSCvlabel::draw %s:%d %s not ready..\n",__FILE__,__LINE__,getInstanceN
           long i;
           short xx1 = xx;
           for(i=0; i<num2; i++){
-            WSCstring tmp = str.getWord(i,sep);
+            WSCstring tmp = str->getWord(i,sep);
             WSCstring pos("0");
             if(i > 0){
               pos = _bar_pos.getWord(i-1,sep);
@@ -487,7 +559,7 @@ dbprintf("WSCvlabel::draw %s:%d %s not ready..\n",__FILE__,__LINE__,getInstanceN
               }
             }
             dev->drawString(xx1 +1 + atoi((char*)pos), yy+1,
-                            wt, hh, &str, _font, getAlignment(), -1, False);
+                            wt, hh, str, _font, getAlignment(), -1, False);
             xx1 = st + _margin_left;
           }
         }
@@ -555,16 +627,18 @@ dbprintf("WSCvlabel::draw %s:%d %s not ready..\n",__FILE__,__LINE__,getInstanceN
 //        }else
 //        if (WSGIappDev()->getGuiPolicy() & WS_POLICY_WINDOWS){
           dev->setForeColor(fgno2);
-          dev->drawRect( mx + hh/4 +_margin_left - 4,
-                         hh/2+_margin_top -4,9,9);
-          dev->drawLine( mx + hh/4 +_margin_left - 2,
+          long i_w2 = (hh/2 -1)/2;
+          long i_w = i_w2*2 +1;
+          dev->drawRect( mx + hh/4 +_margin_left - i_w2,
+                         hh/2+_margin_top -i_w2,i_w,i_w);
+          dev->drawLine( mx + hh/4 +_margin_left - i_w2/2,
                          hh/2+_margin_top ,
-                         mx + hh/4 +_margin_left + 3,
+                         mx + hh/4 +_margin_left + i_w2/2+1,
                          hh/2+_margin_top );
           dev->drawLine( mx + hh/4 +_margin_left,
-                         hh/2+_margin_top -2,
+                         hh/2+_margin_top -i_w2/2,
                          mx + hh/4 +_margin_left,
-                         hh/2+_margin_top +3);
+                         hh/2+_margin_top +i_w2/2+1);
 
 ////          WSGFdrawShadow(dev, WS_SHADOW_OUT, 2, tsno, tsno, bsno,bgno,
 ////                  mx + _margin_left, hh/4 + _margin_top, hh/2, hh/2);
@@ -576,11 +650,13 @@ dbprintf("WSCvlabel::draw %s:%d %s not ready..\n",__FILE__,__LINE__,getInstanceN
 //        }else
 //        if (WSGIappDev()->getGuiPolicy() & WS_POLICY_WINDOWS){
           dev->setForeColor(fgno2);
-          dev->drawRect( mx + hh/4 +_margin_left - 4,
-                         hh/2+_margin_top -4,9,9);
-          dev->drawLine( mx + hh/4 +_margin_left - 2,
+          long i_w2 = (hh/2 -1)/2;
+          long i_w = i_w2*2 +1;
+          dev->drawRect( mx + hh/4 +_margin_left - i_w2,
+                         hh/2+_margin_top -i_w2,i_w,i_w);
+          dev->drawLine( mx + hh/4 +_margin_left - i_w2/2,
                          hh/2+_margin_top ,
-                         mx + hh/4 +_margin_left + 3,
+                         mx + hh/4 +_margin_left + i_w2/2+1,
                          hh/2+_margin_top );
 
 ////          WSGFdrawShadow(dev, WS_SHADOW_IN, 2, tsno, tsno, bsno,bgno,
@@ -615,7 +691,7 @@ dbprintf("WSCvlabel::draw %s:%d %s not ready..\n",__FILE__,__LINE__,getInstanceN
   }
   if ( !( bldraw && blinktype != WS_BACK && _tw_fl == False) ){
     if (_vk_draw != WS_VERBOSE){
-      dev->drawString(xx, yy, ww, hh, &str, _font, getAlignment(), -1, False);
+      dev->drawString(xx, yy, ww, hh, str, _font, getAlignment(), -1, False);
     }else{
       WSCstring sep(",");
       if (getParent() != NULL){
@@ -630,7 +706,7 @@ dbprintf("WSCvlabel::draw %s:%d %s not ready..\n",__FILE__,__LINE__,getInstanceN
       long i;
       short xx1 = xx;
       for(i=0; i<num2 + 1; i++){
-        WSCstring tmp = str.getWord(i,sep);
+        WSCstring tmp = str->getWord(i,sep);
         WSCstring pos("0");
         if(i > 0){
           pos = _bar_pos.getWord(i-1,sep);
@@ -696,3 +772,6 @@ WSCstring WSCvlabel::get_bar_pos(){
 void WSCvlabel::set_bar_pos(WSCstring val){
   _bar_pos = val;
 }
+WSCstring* WSCvlabel::getInternalString(){
+  return &_str;
+}
