@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSkeysym.h>
@@ -138,13 +134,16 @@ long WSCvradio::draw(){
       st.bottom_shadow_color = _bs_color;
       st.label_string = _label_string;
       st.select_string = _select_string;
+      st.shortcut_string = _sc_str;
       st.label_pixmap = _label_pixmap;
       st.blink_pixmap = _blink_pixmap;
       st.select_pixmap = _select_pixmap;
       st.press_status = _press_status;
       st.orientation = _orientation;
+#ifndef NO_GRADATION
       st.gradation = _grad_no;
       st.gradation_margin = _grad_margin;
+#endif
       st.margin_left = _margin_left;
       st.margin_right = _margin_right;
       st.margin_top = _margin_top;
@@ -174,15 +173,15 @@ long WSCvradio::draw(){
 
   WSCstring str;
   if (_press_status == False){
-    str.setString( _label_string );
+    str.setStaticString( _label_string );
   }else{
     if (_select_string == NULL){
-      str.setString( _label_string );
+      str.setStaticString( _label_string );
     }else{
       if (_select_string[0] == 0 ){
-        str.setString( _label_string );
+        str.setStaticString( _label_string );
       }else{
-        str.setString( _select_string );
+        str.setStaticString( _select_string );
       }
     }
   }
@@ -218,8 +217,11 @@ long WSCvradio::draw(){
     }
   }
 
+#ifndef NO_GRADATION
   if (_grad_no == 0){
+#endif
     dev->drawFillRect(0, 0, w, h);
+#ifndef NO_GRADATION
   }else{
     if (_press_status == True  && _indicator_on == False){
       dev->drawGradation(_grad_no,tsno,bgno,bsno,0, 1, w, h,_grad_margin);
@@ -227,7 +229,7 @@ long WSCvradio::draw(){
       dev->drawGradation(_grad_no,tsno,bgno,bsno,0, 0, w, h,_grad_margin);
     }
   }
-
+#endif
   short offsetx = 0;
   short left_margin = _margin_left;
   short right_margin = _margin_right;
@@ -289,8 +291,25 @@ long WSCvradio::draw(){
 //    }
 //    WSGFdrawBtnShadow(dev, type, st, _fore_color, tsno, bsno,bgno, 0, 0, w, h);
 //  }
+
+  if (_sc_str != NULL){
+    WSCstring _accel_char(_sc_str);
+    _accel_char.to_lower(); 
+
+    WSCstring _accel_str(str);
+    _accel_str.to_lower();
+    long _char_pos = _accel_str.isExist(_accel_char);
+    if(_char_pos > -1 ){
+      dev->setUnderLineChar(_char_pos,1);
+    }
+  }
+
+#ifndef WS_EMBED
   if (getSensitive() == False ||
-      (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0)){
+      (getPropertyEditMode() != False && _sensitive_work == False)){
+#else
+  if (getSensitive() == False){
+#endif
     if (_emboss != False){
       if (_bl_fl != False &&
         blinkfore != False &&
@@ -329,8 +348,12 @@ long WSCvradio::draw(){
         _tw_fl == True){
       dev->setForeColor(_bg_blink_color);
     }else{
+#ifndef WS_EMBED
       if ( _emboss != False && (getSensitive() == False ||
-          (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0))){
+          (getPropertyEditMode() != False && _sensitive_work == False))){
+#else
+      if ( _emboss != False && getSensitive() == False){
+#endif
         dev->setForeColor(_bs_color);
       } else {
         dev->setForeColor(_back_color);
@@ -343,8 +366,12 @@ long WSCvradio::draw(){
         _tw_fl == True){
       dev->setForeColor(_blink_color);
     }else{
+#ifndef WS_EMBED
       if ( _emboss != False && (getSensitive() == False ||
-          (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0))){
+          (getPropertyEditMode() != False && _sensitive_work == False))){
+#else
+      if ( _emboss != False && getSensitive() == False){
+#endif
          dev->setForeColor(_bs_color);
       }else{
          dev->setForeColor(_fore_color);
@@ -357,7 +384,7 @@ long WSCvradio::draw(){
         _bl_type != WS_BACK &&
         blinkfore != False &&
         _tw_fl == False){
-        // 褷ʤ
+        // 褷ʤ...
     }else{
 
 
@@ -369,8 +396,8 @@ long WSCvradio::draw(){
                       &str, _font, getAlignment(), -1, False);
       }
 
-      //եΥ饤
-      // եưݤĤξΤ
+      //եΥ饤...
+      // եưݤĤξΤ...
       if (getFocus() == True && _enable_focus_move == True){
         dev->setLineWidth( 1 );
         dev->setLineDashType( 8 );
@@ -454,16 +481,21 @@ long WSCvradio::draw(){
 
 }
 void(*WSCvradio::_ext_draw_proc)(void*) = NULL;
-WSCstring WSCvradio::_policy_name;
+char* WSCvradio::_policy_name = NULL;
 
 void WSCvradio::onGuiPolicyChange(long policy){
   WSCvlabel::onGuiPolicyChange(policy);
   if (WSGIappDev()->getExtGuiPolicySelected() == False){
     _ext_draw_proc = NULL;
   }else{
-    if (!strcmp((char*)_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
+    if (_policy_name != NULL &&
+        !strcmp(_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
     }else{
       _ext_draw_proc = (void(*)(void*))WSGIappDev()->getExtGuiPolicyProc("WSCvradio::draw");
+      if (_policy_name != NULL){
+        delete _policy_name;
+      }
+      _policy_name = WSGFstrdup(WSGIappDev()->getExtGuiPolicyName());
     }
   }
   if (getVisible() != False){
