@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCvtoggle.h>
@@ -39,6 +35,7 @@
 #include <WSCblink.h>
 #include <WSDappDev.h>
 #include <WSCguiPolicy.h>
+#include <ctype.h>
 
 WSMFguiClassInitialize(WSCvtoggle, WSCvlabel);
 WSMFversion(WSCvtoggle, WSCvlabel);
@@ -72,17 +69,21 @@ WSCvtoggle::WSCvtoggle(WSCbase* base, char* objname):
   _indicator_pixmap = -1;
   _select_pixmap   = -1;
   _unique          = False;
-  _group_name      = WSGFstrdup("");
+  _group_name      = NULL;
   _id              = 0;
   _select_reset    = False;
   _enable_focus_move = True;
-  _td_name = WSGFstrdup("");
-  _bd_name = WSGFstrdup("");
-  _ld_name = WSGFstrdup("");
-  _rd_name = WSGFstrdup("");
-  _tb_name = WSGFstrdup("");
-  _btb_name = WSGFstrdup("");
+  _td_name = NULL;
+  _bd_name = NULL;
+  _ld_name = NULL;
+  _rd_name = NULL;
+  _tb_name = NULL;
+  _btb_name = NULL;
+  _sc_str = NULL;
   _toggle_status = False;
+  _return_key_focus = False;
+  _alt_focus_registered = False;
+
   WSGIappObjectList()->addEvent(this,WSEV_GUI_POLICY_CH);
 
   WSMFpropertyCreateStart
@@ -118,6 +119,10 @@ WSCvtoggle::WSCvtoggle(WSCbase* base, char* objname):
     WSMFpropertyCreate(WSNstatus, WSCbool,  _toggle_status,WSSstatus);
     WSMFpropertySetSelection(WSRbool3, WSRbool3D);
 
+    WSMFpropertyCreate(WSNreturnKeyFocus,WSCbool,_return_key_focus,WSSreturnKeyFocus );
+      WSMFpropertySetSelection(WSRbool1,WSRbool1D);
+    WSMFpropertyCreate(WSNshortcut,  char*,   _sc_str,WSSshortcut);
+
     WSMFpropertyDelete(WSNshadowType);
 
     WSMFaddTrigger(WSEV_VALUE_CH    );
@@ -131,7 +136,7 @@ WSCvtoggle::WSCvtoggle(WSCbase* base, char* objname):
   _indicator_type = WS_IN_OUT;
 }
 
-WSMFproperty( WSCvtoggle, WSNselectString,     char*,     _select_string    ,WSGFstrdup("")      );
+WSMFproperty( WSCvtoggle, WSNselectString,     char*,     _select_string    ,NULL      );
 WSMFproperty( WSCvtoggle, WSNindicatorOn,      WSCbool,   _indicator_on     ,True    );
 WSMFproperty( WSCvtoggle, WSNindicatorType,    WSCuchar,      _indicator_type   ,WS_IN_OUT );
 WSMFproperty( WSCvtoggle, WSNindicatorSize,    WSCuchar,     _indicator_size   ,20      );
@@ -144,17 +149,19 @@ WSMFproperty( WSCvtoggle, WSNindicatorPixmap,  short, _indicator_pixmap ,-1
 WSMFproperty( WSCvtoggle, WSNselectPixmap,     short,     _select_pixmap    ,-1      );
 //WSMFproperty( WSCvtoggle, WSNindicatorImageType, char,     _indicator_image_type, WS_PIXMAP);
 WSMFproperty( WSCvtoggle, WSNunique,           WSCbool,   _unique           ,False);
-WSMFproperty( WSCvtoggle, WSNgroup,            char*,     _group_name       ,WSGFstrdup(""));
+WSMFproperty( WSCvtoggle, WSNgroup,            char*,     _group_name       ,NULL);
 WSMFproperty( WSCvtoggle, WSNid,               short,     _id               ,0);
 WSMFproperty( WSCvtoggle, WSNselectReset,      WSCbool,   _select_reset     ,False);
 WSMFproperty( WSCvtoggle, WSNenableFocusMove,  WSCbool,   _enable_focus_move,True    );
-WSMFproperty(WSCvtoggle, WSNupward,    char*,   _td_name, WSGFstrdup(""));
-WSMFproperty(WSCvtoggle, WSNdownward, char*,   _bd_name, WSGFstrdup(""));
-WSMFproperty(WSCvtoggle, WSNleftward,   char*,   _ld_name, WSGFstrdup(""));
-WSMFproperty(WSCvtoggle, WSNrightward,  char*,   _rd_name, WSGFstrdup(""));
-WSMFproperty(WSCvtoggle, WSNtab,  char*,   _tb_name, WSGFstrdup(""));
-WSMFproperty(WSCvtoggle, WSNbacktab,  char*,   _btb_name, WSGFstrdup(""));
-WSMFproperty( WSCvtoggle, WSNstatus,  WSCbool,   _toggle_status,False    );
+WSMFproperty(WSCvtoggle, WSNupward,    char*,   _td_name, NULL);
+WSMFproperty(WSCvtoggle, WSNdownward, char*,   _bd_name, NULL);
+WSMFproperty(WSCvtoggle, WSNleftward,   char*,   _ld_name, NULL);
+WSMFproperty(WSCvtoggle, WSNrightward,  char*,   _rd_name, NULL);
+WSMFproperty(WSCvtoggle, WSNtab,  char*,   _tb_name, NULL);
+WSMFproperty(WSCvtoggle, WSNbacktab,  char*,   _btb_name, NULL);
+WSMFproperty(WSCvtoggle, WSNstatus,  WSCbool,   _toggle_status,False    );
+WSMFproperty(WSCvtoggle, WSNreturnKeyFocus, WSCbool, _return_key_focus, False);
+WSMFproperty(WSCvtoggle, WSNshortcut,  char*,   _sc_str, NULL);
 
 void WSCvtoggle::setWorkWSNselectString(char*){}
 void WSCvtoggle::getWorkWSNselectString(char** ){}
@@ -196,6 +203,9 @@ void WSCvtoggle::setWorkWSNtab(char*){}
 void WSCvtoggle::getWorkWSNtab(char**){} 
 void WSCvtoggle::setWorkWSNbacktab(char*){} 
 void WSCvtoggle::getWorkWSNbacktab(char**){} 
+void WSCvtoggle::setWorkWSNreturnKeyFocus(WSCbool){}
+void WSCvtoggle::getWorkWSNreturnKeyFocus(WSCbool*){}
+
 void WSCvtoggle::setWorkWSNstatus(WSCbool fl){
   if (getInitialized() != False){
     if (fl != getStatus()){
@@ -205,48 +215,67 @@ void WSCvtoggle::setWorkWSNstatus(WSCbool fl){
 }
 void WSCvtoggle::getWorkWSNstatus(WSCbool* fl){
   *fl = getStatus();
+  _toggle_status = *fl;
 }
+void WSCvtoggle::setWorkWSNshortcut(char* data) {
+  if (_alt_focus_registered == False){
+      _alt_focus_registered = True;
+    WSGIappDev()->addAltFocusInstance(this);
+  }
+}
+void WSCvtoggle::getWorkWSNshortcut(char**) {}
 
 WSCvtoggle::~WSCvtoggle(){
   WSGIappObjectList()->delEvent(this,WSEV_GUI_POLICY_CH);
+  if (_sc_str != NULL){
+    delete[] _sc_str;
+    _sc_str = NULL;
+  }
+  if (_alt_focus_registered != False){
+    _alt_focus_registered = False;
+    WSGIappDev()->delAltFocusInstance(this);
+  }
 
   if (getFocus() != False){
     WSGIappKeyboard()->resetFocusInputWorkProc();
+    setFocus(False);
+  }
+  if (_return_key_focus != False && getSpecialFocus() != False){
+    setSpecialFocus(False);
   }
   _get_toggle_list()->del(this);
   if (_select_string != NULL){
-    delete _select_string;
+    delete[] _select_string;
     _select_string = NULL;
   }
   if (_group_name != NULL){
-    delete _group_name;
+    delete[] _group_name;
     _group_name = NULL;
   }
   if (_td_name != NULL){
-    delete _td_name;
+    delete[] _td_name;
     _td_name = NULL;
   }
   if (_bd_name != NULL){
-    delete _bd_name;
+    delete[] _bd_name;
     _bd_name = NULL;
   }
   if (_ld_name != NULL){
-    delete _ld_name;
+    delete[] _ld_name;
     _ld_name = NULL;
   }
   if (_rd_name != NULL){
-    delete _rd_name;
+    delete[] _rd_name;
     _rd_name = NULL;
   }
   if (_tb_name != NULL){
-    delete _tb_name;
+    delete[] _tb_name;
     _tb_name = NULL;
   }
   if (_btb_name != NULL){
-    delete _btb_name;
+    delete[] _btb_name;
     _btb_name = NULL;
   }
-
 }
 
 void WSCvtoggle::_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool fl){
@@ -260,7 +289,7 @@ void WSCvtoggle::_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool fl){
   if (obj->getSensitive() == False){
     return;
   }
-  if (fl == True && key == WSK_Return){
+  if (fl == True && (key == WSK_Return || key == WSK_KP_Enter || key == WSK_space)){
     if (obj->_press_status == False){
       obj->setStatus(True, True);
       obj->draw();
@@ -275,7 +304,10 @@ void WSCvtoggle::_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool fl){
        WSCbase* target = obj->getFocusMoveInstance(WS_LEFT);
        if (target != NULL){
          target->setFocus();
+         obj->_status_move(target);
          target->draw();
+       }else{
+         obj->_extra_focus_move(WS_LEFT);
        }
      }
   }else if (fl == True && (key == WSK_Tab && keyboard->withShift())) {
@@ -286,6 +318,8 @@ void WSCvtoggle::_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool fl){
        if (target != NULL){
          target->setFocus();
          target->draw();
+       }else{
+         obj->_extra_focus_move(WS_STAB);
        }
      }
   }else if (fl == True && key == WSK_Tab ) {
@@ -296,6 +330,8 @@ void WSCvtoggle::_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool fl){
        if (target != NULL){
          target->setFocus();
          target->draw();
+       }else{
+         obj->_extra_focus_move(WS_TAB);
        }
      }
 
@@ -306,7 +342,10 @@ void WSCvtoggle::_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool fl){
        WSCbase* target = obj->getFocusMoveInstance(WS_RIGHT);
        if (target != NULL){
          target->setFocus();
+         obj->_status_move(target);
          target->draw();
+       }else{
+         obj->_extra_focus_move(WS_RIGHT);
        }
      }
 
@@ -317,7 +356,10 @@ void WSCvtoggle::_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool fl){
        WSCbase* target = obj->getFocusMoveInstance(WS_UP);
        if (target != NULL){
          target->setFocus();
+         obj->_status_move(target);
          target->draw();
+       }else{
+         obj->_extra_focus_move(WS_UP);
        }
      }
 
@@ -328,12 +370,15 @@ void WSCvtoggle::_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool fl){
        WSCbase* target = obj->getFocusMoveInstance(WS_DOWN);
        if (target != NULL){
          target->setFocus();
+         obj->_status_move(target);
          target->draw();
+       }else{
+         obj->_extra_focus_move(WS_DOWN);
        }
      }
 
   }
-  //ܡɥ٥Ȥȯ
+  //ܡɥ٥Ȥȯ...
   if (fl == True){
     obj->onKey(keyboard,True);
     obj->execProcedure(WSEV_KEY_PRESS);
@@ -342,6 +387,66 @@ void WSCvtoggle::_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool fl){
     obj->execProcedure(WSEV_KEY_RELEASE);
   }
 }
+void WSCvtoggle::_extra_focus_move(long direct){
+  WSCbase* parent = getParent();
+  if (parent != NULL && parent->cast("WSCradioGroup") != NULL){
+    WSCbool fl = parent->getProperty(WSNenableFocusMove);
+    if (fl == False){
+      return;
+    }
+    WSCbase* target = parent->getFocusMoveInstance(direct);
+    if (target != NULL){
+      target->setFocus();
+      target->redraw();
+    }
+  }
+}
+void WSCvtoggle::_status_move(WSCbase* base){
+  if (_group_name == NULL || _group_name[0] == 0){
+    return;
+  }
+  WSCvtoggle* target = (WSCvtoggle*)base->cast("WSCvtoggle");
+  if (target == NULL){
+    return;
+  }
+  if (target->_group_name == NULL || target->_group_name[0] == 0){
+    return;
+  }
+  if (!strcmp(target->_group_name,_group_name)){
+    if (_unique != False && target->_unique != False){
+      target->setStatus(True,True);
+    }
+    if (target->_unique == False){
+      target->setSpecialFocus();
+    }
+  }
+}
+long WSCvtoggle::_set_status(WSCbool state, WSCbool create_event){
+  if (state != False){
+    state = True;
+  }
+  if (state == True){
+    _press_status = True;
+    setAbsoluteDraw(True);
+    needUpdate();
+
+    if (create_event == True){
+      onValueChange(_press_status);
+      execProcedure(WSEV_VALUE_CH);
+    }
+  }else{
+    _press_status = False;
+    setAbsoluteDraw(True);
+    needUpdate();
+
+    if (create_event == True){
+      onValueChange(_press_status);
+      execProcedure(WSEV_VALUE_CH);
+    }
+  }
+  return WS_NO_ERR;
+}
+
 
 long WSCvtoggle::setStatus(WSCbool state, WSCbool create_event){
   if (state != False){
@@ -358,6 +463,7 @@ long WSCvtoggle::setStatus(WSCbool state, WSCbool create_event){
         WSCvtoggle* tgl = (WSCvtoggle*)_get_toggle_list()->getData(i);
         if (_group_name != NULL && _group_name[0] != 0 && 
             tgl != this &&
+            tgl->_group_name != NULL && tgl->_group_name[0] != 0 &&
             !strcmp(tgl->_group_name, _group_name) ){
           WSCbool bak = tgl->_select_reset;
           tgl->_select_reset = True;
@@ -369,6 +475,7 @@ long WSCvtoggle::setStatus(WSCbool state, WSCbool create_event){
         WSCvtoggle* tgl = (WSCvtoggle*)_get_toggle_list()->getData(i);
         if (_group_name != NULL && _group_name[0] != 0 && 
             tgl != this &&
+            tgl->_group_name != NULL && tgl->_group_name[0] != 0 &&
             !strcmp(tgl->_group_name, _group_name) ){
           WSCbool bak = tgl->_select_reset;
           tgl->_select_reset = True;
@@ -383,7 +490,7 @@ long WSCvtoggle::setStatus(WSCbool state, WSCbool create_event){
       execProcedure(WSEV_VALUE_CH);
     }
   }else{
-    if (_group_name != NULL && !strcmp(_group_name, "")
+    if (_group_name == NULL || (_group_name != NULL && !strcmp(_group_name, ""))
          || _unique == False || _select_reset == True ){
       _press_status = False;
       setAbsoluteDraw(True);
@@ -399,8 +506,9 @@ long WSCvtoggle::setStatus(WSCbool state, WSCbool create_event){
       for(i=0; i < num; i++){
         WSCvtoggle* tgl = (WSCvtoggle*)_get_toggle_list()->getData(i);
         if (_group_name != NULL && _group_name[0] != 0 && 
-           tgl != this &&
-           !strcmp(tgl->_group_name, _group_name) ){
+            tgl != this &&
+            tgl->_group_name != NULL && tgl->_group_name[0] != 0 &&
+            !strcmp(tgl->_group_name, _group_name) ){
           WSCbool bak = tgl->_select_reset;
           tgl->_select_reset = True;
           tgl->setStatus(False, True);
@@ -413,7 +521,50 @@ long WSCvtoggle::setStatus(WSCbool state, WSCbool create_event){
   return WS_NO_ERR;
 }
 
-void WSCvtoggle::onKey(WSDkeyboard*,WSCbool){
+void WSCvtoggle::onKey(WSDkeyboard* keyb,WSCbool fl){
+  if (_sc_str == NULL){
+    return;
+  }
+  if (getVisible() == False){
+    return;
+  }
+  if (getSensitive() == False){
+    return;
+  }
+  WSCbool altkey = keyb->withAlt();
+  if (altkey == False){
+    return;
+  }
+  char* text = keyb->getText();
+  char buf[2];
+  buf[0] = (char)toupper(text[0]);
+  buf[1] = 0;
+  char buf2[2];
+  buf2[0] = (char)toupper(_sc_str[0]);
+  buf2[1] = 0;
+  if (fl == True && !strcmp(buf,buf2)){
+    WSCbool status = getStatus();
+    if (status == False){
+      setStatus(True,True);
+      setFocus();
+    }else{
+      setStatus(False,True);
+    }
+    keyb->setText("");
+  }
+}
+long WSCvtoggle::execEventProc(long ev, void* data) {
+  if (ev == WSEV_KEY_PRESS) {
+    onKey(WSGIappKeyboard(),True);
+  }else
+  if (ev == WSEV_KEY_RELEASE) {
+    onKey(WSGIappKeyboard(),False);
+  }else
+  if (ev == WSEV_VALUE_CH){
+    onValueChange(getStatus());
+  }
+
+  return WSCvlabel::execEventProc(ev, data);
 }
 
 long WSCvtoggle::_device_initialize(){
@@ -422,6 +573,7 @@ long WSCvtoggle::_device_initialize(){
     return WS_ERR;
   }
   dev->setEnableEventBit( WSEV_MOUSE_MOVE_BIT | WSEV_MOUSE_OUT_BIT |
+                          WSEV_MOUSE_IN_BIT |
                           WSEV_MOUSE_PRESS_BIT | WSEV_MOUSE_RELEASE_BIT );
 
   WSCvlabel::_device_initialize();
@@ -433,12 +585,21 @@ void WSCvtoggle::onFocusChange(WSCbool fl){
     return;
   }
   if (fl == True){
-    WSGIappKeyboard()->setFocusInputWorkProc(_key_handler,NULL, this);
+    WSGIappKeyboard()->setFocusInputWorkProc(_key_handler,NULL, this,this);
     if (getowndev() != NULL){
       WSGIappKeyboard()->resetFep(getowndev());
     }
+    if (_return_key_focus != False){
+      WSGIappKeyboard()->resetSpecialInputWorkProc();
+      setSpecialFocus(True);
+    }
+
   }else{
     WSGIappKeyboard()->resetFocusInputWorkProc();
+    if (_return_key_focus != False){
+      WSGIappKeyboard()->resetSpecialInputWorkProc();
+      setSpecialFocus(False);
+    }
   }
   setAbsoluteDraw(True);
   needUpdate();
@@ -513,13 +674,16 @@ long WSCvtoggle::draw(){
       st.bottom_shadow_color = _bs_color;
       st.label_string = _label_string;
       st.select_string = _select_string;
+      st.shortcut_string = _sc_str;
       st.label_pixmap = _label_pixmap;
       st.blink_pixmap = _blink_pixmap;
       st.select_pixmap = _select_pixmap;
       st.press_status = _press_status;
       st.orientation = _orientation;
+#ifndef NO_GRADATION
       st.gradation = _grad_no;
       st.gradation_margin = _grad_margin;
+#endif
       st.margin_left = _margin_left;
       st.margin_right = _margin_right;
       st.margin_top = _margin_top;
@@ -549,15 +713,15 @@ long WSCvtoggle::draw(){
 
   WSCstring str;
   if (_press_status == False){
-    str.setString( _label_string );
+    str.setStaticString( _label_string );
   }else{
     if (_select_string == NULL){
-      str.setString( _label_string );
+      str.setStaticString( _label_string );
     }else{
       if (_select_string[0] == 0 ){
-        str.setString( _label_string );
+        str.setStaticString( _label_string );
       }else{
-        str.setString( _select_string );
+        str.setStaticString( _select_string );
       }
     }
   }
@@ -592,8 +756,11 @@ long WSCvtoggle::draw(){
       dev->setForeColor(bgno);
     }
   }
+#ifndef NO_GRADATION
   if (_grad_no == 0){
+#endif
     dev->drawFillRect(0, 0, w, h);
+#ifndef NO_GRADATION
   }else{
     if (_press_status == True  && _indicator_on == False){
       dev->drawGradation(_grad_no,tsno,bgno,bsno,0, 1, w, h,_grad_margin);
@@ -601,7 +768,7 @@ long WSCvtoggle::draw(){
       dev->drawGradation(_grad_no,tsno,bgno,bsno,0, 0, w, h,_grad_margin);
     }
   }
-
+#endif
   short offsetx = 0;
   short left_margin = _margin_left;
   short right_margin = _margin_right;
@@ -665,8 +832,24 @@ long WSCvtoggle::draw(){
     WSGFdrawBtnShadow(dev, type, st, _fore_color, tsno, bsno,bgno, 0, 0, w, h);
   }
 
+  if (_sc_str != NULL){
+    WSCstring _accel_char(_sc_str);
+    _accel_char.to_lower();
+
+    WSCstring _accel_str(str);
+    _accel_str.to_lower();
+    long _char_pos = _accel_str.isExist(_accel_char);
+    if(_char_pos > -1 ){
+      dev->setUnderLineChar(_char_pos,1);
+    }
+  }
+
+#ifndef WS_EMBED
   if (getSensitive() == False ||
-      (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0)){
+      (getPropertyEditMode() != False && _sensitive_work == False)){
+#else
+  if (getSensitive() == False){
+#endif
     if (_emboss != False){
       if (_bl_fl != False &&
         blinkfore != False &&
@@ -706,8 +889,12 @@ long WSCvtoggle::draw(){
         _tw_fl == True){
       dev->setForeColor(_bg_blink_color);
     }else{
+#ifndef WS_EMBED
       if ( _emboss != False && (getSensitive() == False ||
-          (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0))){
+          (getPropertyEditMode() != False && _sensitive_work == False))){
+#else
+      if ( _emboss != False && getSensitive() == False){
+#endif
         dev->setForeColor(_bs_color);
       } else {
         dev->setForeColor(_back_color);
@@ -720,8 +907,12 @@ long WSCvtoggle::draw(){
         _tw_fl == True){
       dev->setForeColor(_blink_color);
     }else{
+#ifndef WS_EMBED
       if ( _emboss != False && (getSensitive() == False ||
-          (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0))){
+          (getPropertyEditMode() != False && _sensitive_work == False))){
+#else
+      if ( _emboss != False && getSensitive() == False){
+#endif
          dev->setForeColor(_bs_color);
       }else{
          dev->setForeColor(_fore_color);
@@ -735,7 +926,7 @@ long WSCvtoggle::draw(){
         _bl_type != WS_BACK &&
         blinkfore != False &&
         _tw_fl == False){
-        // 褷ʤ
+        // 褷ʤ...
     }else{
 
 
@@ -747,8 +938,8 @@ long WSCvtoggle::draw(){
                       &str, _font, getAlignment(), -1, False);
       }
 
-      //եΥ饤
-      // եưݤĤξΤ
+      //եΥ饤...
+      // եưݤĤξΤ...
       if (getFocus() == True && _enable_focus_move == True){
         dev->setLineWidth( 1 );
         dev->setLineDashType( 8 );
@@ -775,7 +966,7 @@ long WSCvtoggle::draw(){
   }
 
   if (_indicator_on == True){
-    //󥸥κɸη׻
+    //󥸥κɸη׻...
     char itype = _indicator_type;
     WSCushort isize = _indicator_size;
     short ypos = (h - isize)/2;
@@ -784,7 +975,7 @@ long WSCvtoggle::draw(){
         ypos = 0;
     }
 
-    //󥸥طʤ
+    //󥸥طʤ...
     WSDimage* image;
     if (_press_status == True ){
       if (_select_pixmap == -1){
@@ -870,7 +1061,7 @@ WSMFtrace("WSCvtoggle::addProcedure trigger=%d not supported.\n", ac->getTrigger
                                  WSEV_MOUSE_RELEASE_BIT | WSEV_MOUSE_OUT_BIT );
 
     _event_mask |= ac->getEffectiveTriggerMask();
-    _add_prcs(&_prc_list, ac);
+    _add_prcs(ac);
 
     WSDdev* dev = getowndev();
     if (dev != NULL){
@@ -895,7 +1086,8 @@ long WSCvtoggle::getGroupValue(){
   for(i=0; i < num; i++){
     WSCvtoggle* tgl = (WSCvtoggle*)_get_toggle_list()->getData(i);
     if (_group_name != NULL && _group_name[0] != 0 && 
-       !strcmp(tgl->_group_name, _group_name) ){
+        tgl->_group_name != NULL && tgl->_group_name[0] != 0 &&
+        !strcmp(tgl->_group_name, _group_name) ){
        short id = tgl->_id;
        if (tgl->getStatus() == True){
          if (_unique == True){
@@ -921,26 +1113,27 @@ long WSCvtoggle::setGroupValue(long value){
   long i, num = _get_toggle_list()->getNum();
   for(i=0; i < num; i++){
     WSCvtoggle* tgl = (WSCvtoggle*)_get_toggle_list()->getData(i);
-    if (_group_name != NULL && _group_name[0] != 0 && 
-       !strcmp(tgl->_group_name, _group_name) ){
+    if (_group_name != NULL && _group_name[0] != 0 &&
+        tgl->_group_name != NULL && tgl->_group_name[0] != 0 &&
+        !strcmp(tgl->_group_name, _group_name) ){
        short id = tgl->_id;
        if (_unique == True){ //ξ...
-         WSCbool ubak = tgl->_unique;
-         tgl->_unique = False;
+//         WSCbool ubak = tgl->_unique;
+//         tgl->_unique = False;
          if (id == value){
-           WSCbool bak = tgl->_select_reset;
-           tgl->_select_reset = True;
-           tgl->setStatus(True, True);
-           tgl->_select_reset = bak;
+//           WSCbool bak = tgl->_select_reset;
+//           tgl->_select_reset = True;
+           tgl->_set_status(True, True);
+//           tgl->_select_reset = bak;
          }else{
-           WSCbool bak = tgl->_select_reset;
-           tgl->_select_reset = True;
-           tgl->setStatus(False, True);
-           tgl->_select_reset = bak;
+//           WSCbool bak = tgl->_select_reset;
+//           tgl->_select_reset = True;
+           tgl->_set_status(False, True);
+//           tgl->_select_reset = bak;
          }
-         tgl->_unique = ubak;
-         tgl->setAbsoluteDraw(True);
-         tgl->needUpdate();
+//         tgl->_unique = ubak;
+//         tgl->setAbsoluteDraw(True);
+//         tgl->needUpdate();
          tgl->draw();
        }else{      //¿Ťξ...
          long vl;
@@ -967,16 +1160,21 @@ long WSCvtoggle::setGroupValue(long value){
 }
 
 void(*WSCvtoggle::_ext_draw_proc)(void*) = NULL;
-WSCstring WSCvtoggle::_policy_name;
+char* WSCvtoggle::_policy_name = NULL;
 
 void WSCvtoggle::onGuiPolicyChange(long policy){
   WSCvlabel::onGuiPolicyChange(policy);
   if (WSGIappDev()->getExtGuiPolicySelected() == False){
     _ext_draw_proc = NULL;
   }else{
-    if (!strcmp((char*)_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
+    if (_policy_name != NULL &&
+        !strcmp(_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
     }else{
       _ext_draw_proc = (void(*)(void*))WSGIappDev()->getExtGuiPolicyProc("WSCvtoggle::draw");
+      if (_policy_name != NULL){
+        delete _policy_name;
+      }
+      _policy_name = WSGFstrdup(WSGIappDev()->getExtGuiPolicyName());
     }
   }
   if (getVisible() != False){
