@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCwindow.h>
@@ -34,11 +30,12 @@
 #include <WSDappDev.h>
 #include <WSDtimer.h>
 #include <WSDwindowDev.h>
+#include <WSCundoManager.h>
 
 WSMFguiClassInitialize(WSCwindow,WSCbase);
 WSMFversion(WSCwindow,WSCbase);
 WSMFdefineUseDevice(WSCwindow,windowDev)
-void _set_active_instance(WSCbase* win);
+extern WSDLEX32 void _set_active_instance(WSCbase* win);
 void (*WSCwindow::_exit_handler)() = NULL;
 
 WSClistData* WSCwindow::_workwindow_list = NULL;
@@ -54,7 +51,9 @@ WSCwindow::WSCwindow(WSCbase* base,char* objname):
 
   getWorkWindowList()->add(this);
 
+#ifndef NO_WINDOW_TITLE
   _title_string = NULL;
+#endif
   _user_string  = NULL;
   _user_value   = 0;
   _x            = 0;
@@ -71,12 +70,17 @@ WSCwindow::WSCwindow(WSCbase* base,char* objname):
   _ts_no        = WS_DF_TOPSHADOWCOLOR;
   _bs_no        = WS_DF_BOTTOMSHADOWCOLOR;
   _bp_no        = -1;
+#ifndef NO_WINDOW_TITLE
   _title_bar    = WS_FULL_TITLE;
+#endif
   _shadow_type  = 0;
+#ifndef NO_GRADATION
   _grad_no     = 0;
   _grad_margin   = 0;
+#endif
   _vis          = False;
   _sensitive    = True;
+  _sensitive_work = True;
   _pixmap_style = WS_DIRECT_WINDOW;
   _unique       = False;
   _group_name   = NULL;
@@ -94,12 +98,16 @@ WSCwindow::WSCwindow(WSCbase* base,char* objname):
   _a2_color = -1;
   _a3_color = -1;
   _vis_ch_sequence = False;
+  _enable_close = True;
+  _children._seg_size = 8;
 
   WSMFpropertyCreateStart
     WSMFparentCheckVerSrc(WSCwindow);
 
     WSMFpropertyCreate(WSNname,char*, _instance_name, WSSname);
+#ifndef NO_WINDOW_TITLE
     WSMFpropertyCreate(WSNtitleString,char*, _title_string, WSStitleString);
+#endif
     WSMFpropertyCreate(WSNuserString, char*, _user_string,WSSuserString );
     WSMFpropertyCreate(WSNuserValue,  long,  _user_value, WSSuserValue  );
     WSMFpropertyCreate(WSNx, short, _x,WSSx );
@@ -112,13 +120,15 @@ WSCwindow::WSCwindow(WSCbase* base,char* objname):
     WSMFpropertyCreate(WSNtopShadowColor,short,_ts_no,WSStopShadowColor );
     WSMFpropertyCreate(WSNbottomShadowColor,short,_bs_no,WSSbottomShadowColor  );
     WSMFpropertyCreate(WSNbackPixmap, short, _bp_no,WSSbackPixmap  );
+#ifndef NO_WINDOW_TITLE
     WSMFpropertyCreate(WSNtitleBar, char, _title_bar,WSStitleBar );
       WSMFpropertySetSelection(WSRtitleBar,WSRtitleBarD);
+#endif
     WSMFpropertyCreate(WSNshadowType, char,_shadow_type,WSSshadowType );
       WSMFpropertySetSelection(WSRshadowType,WSRshadowTypeD);
     WSMFpropertyCreate(WSNvis, WSCbool,_vis,WSSvis );
       WSMFpropertySetSelection(WSRbool2,WSRbool2D);
-    WSMFpropertyCreate(WSNdet,  WSCbool,_sensitive,WSSdet );
+    WSMFpropertyCreate(WSNdet,  WSCbool,_sensitive_work,WSSdet );
       WSMFpropertySetSelection(WSRbool1,WSRbool1D);
     WSMFpropertyCreate(WSNpixmapStyle, WSCuchar, _pixmap_style,WSSpixmapStyle);
       WSMFpropertySetSelection(WSRpixmapStyle,WSRpixmapStyleD);
@@ -127,9 +137,11 @@ WSCwindow::WSCwindow(WSCbase* base,char* objname):
       WSMFpropertySetSelection(WSRbool3,WSRbool3D);
     WSMFpropertyCreate( WSNmouse, WSCushort,   _mouse_no,WSSmouse  );
 
+#ifndef NO_GRADATION
     WSMFpropertyCreate(WSNgradation,  WSCuchar,  _grad_no,WSSgradation);
       WSMFpropertySetSelection(WSRgradation,WSRgradationD);
     WSMFpropertyCreate(WSNgradationMargin,WSCuchar,_grad_margin,WSSgradationMargin);
+#endif
     WSMFpropertyCreate( WSNexit,  WSCbool, _exit,WSSexit  );
       WSMFpropertySetSelection(WSRbool3,WSRbool3D);
     WSMFpropertyCreate( WSNindicatorOn,  WSCbool, _indicator,WSSindicatorOn  );
@@ -139,8 +151,12 @@ WSCwindow::WSCwindow(WSCbase* base,char* objname):
     WSMFpropertyCreate( WSNemboss,  WSCbool, _emboss,WSSemboss  );
       WSMFpropertySetSelection(WSRbool3,WSRbool3D);
     WSMFpropertyCreate( WSNbarThickness,  WSCuchar, _bar_thick,WSSbarThickness  );
+#ifndef NO_REMOTE_INSTANCE
     WSMFpropertyCreate(WSNexport,WSCbool,_export_,WSSexport);
     WSMFpropertySetSelection(WSRbool1, WSRbool1D);
+#endif
+    WSMFpropertyCreate(WSNkeepPixmapCache,WSCbool,_keep_pixmap_cache,WSSkeepPixmapCache );
+      WSMFpropertySetSelection(WSRbool4,WSRbool4D);
 
       WSMFaddTrigger(WSEV_INITIALIZE  );
       WSMFaddTrigger(WSEV_FOCUS_CH  );
@@ -156,14 +172,17 @@ WSCwindow::WSCwindow(WSCbase* base,char* objname):
       WSMFaddTrigger(WSEV_EXPOSE      );
 
       WSMFaddExtTrigger(WSEV_EXIT,"EXIT" );
+      WSMFaddExtTrigger(WSEV_WINDOW_CLOSED,"WINDOW-CLOSED");
 
   WSMFpropertyCreateEnd
 
 }
 
-WSMFproperty(WSCwindow, WSNname,char*,    _instance_name,WSGFstrdup("")); 
-WSMFproperty(WSCwindow, WSNtitleString,char*,    _title_string,WSGFstrdup("")); 
-WSMFproperty(WSCwindow, WSNuserString, char*,    _user_string ,WSGFstrdup("")); 
+WSMFproperty(WSCwindow, WSNname,char*,    _instance_name,NULL);
+#ifndef NO_WINDOW_TITLE
+WSMFproperty(WSCwindow, WSNtitleString,char*,    _title_string,NULL);
+#endif
+WSMFproperty(WSCwindow, WSNuserString, char*,    _user_string ,NULL);
 WSMFproperty(WSCwindow, WSNuserValue,  long,     _user_value  ,0      ); 
 WSMFproperty(WSCwindow, WSNx,          short,    _x           ,0    );
 WSMFproperty(WSCwindow, WSNy,          short,    _y           ,0    );
@@ -179,24 +198,30 @@ WSMFproperty(WSCwindow, WSNbackColor,  short,    _bg_no ,WS_DF_BACKCOLOR);
 WSMFproperty(WSCwindow, WSNtopShadowColor,  short,_ts_no ,WS_DF_TOPSHADOWCOLOR);
 WSMFproperty(WSCwindow, WSNbottomShadowColor, short,_bs_no ,WS_DF_BOTTOMSHADOWCOLOR);
 WSMFproperty(WSCwindow, WSNbackPixmap,        short,_bp_no    ,-1    );
+#ifndef NO_WINDOW_TITLE
 WSMFproperty(WSCwindow, WSNtitleBar,   char,     _title_bar    ,WS_FULL_TITLE  );
+#endif
 WSMFproperty(WSCwindow, WSNshadowType, char,     _shadow_type  ,0      );
 WSMFproperty(WSCwindow, WSNvis,        WSCbool,  _vis   ,False   );
-WSMFproperty(WSCwindow, WSNdet,        WSCbool,  _sensitive ,True  );
+WSMFproperty(WSCwindow, WSNdet,        WSCbool,  _sensitive_work ,True  );
 WSMFproperty(WSCwindow, WSNpixmapStyle,WSCuchar, _pixmap_style ,WS_DIRECT_WINDOW);
 WSMFproperty(WSCwindow, WSNunique,  WSCbool,  _unique       ,False);
-WSMFproperty(WSCwindow, WSNgroup,   char*,    _group_name   ,WSGFstrdup(""));
+WSMFproperty(WSCwindow, WSNgroup,   char*,    _group_name   ,NULL);
 WSMFproperty(WSCwindow, WSNmouse,   WSCushort,     _mouse_no     ,0);
+#ifndef NO_GRADATION
 WSMFproperty(WSCwindow, WSNgradation, WSCuchar, _grad_no   ,0);
 WSMFproperty(WSCwindow, WSNgradationMargin, WSCuchar, _grad_margin ,0);
+#endif
 WSMFproperty(WSCwindow, WSNexit,  WSCbool,  _exit       ,False);
 WSMFproperty(WSCwindow, WSNindicatorOn,  WSCbool,  _indicator   ,True);
 WSMFproperty(WSCwindow, WSNmargin,  WSCuchar,  _margin       ,2);
 WSMFproperty(WSCwindow, WSNmarginLeft,  WSCushort,  _margin_left       ,2);
 WSMFproperty(WSCwindow, WSNemboss,  WSCbool,  _emboss       ,False);
 WSMFproperty(WSCwindow, WSNbarThickness,  WSCuchar,  _bar_thick   ,16);
+#ifndef NO_REMOTE_INSTANCE
 WSMFproperty(WSCwindow, WSNexport,  WSCbool,  _export_   ,False);
-
+#endif
+WSMFproperty(WSCwindow, WSNkeepPixmapCache, WSCbool,  _keep_pixmap_cache,True );
 void WSCwindow::setWorkWSNemboss(WSCbool data){}
 void WSCwindow::getWorkWSNemboss(WSCbool* data){}
 
@@ -215,6 +240,7 @@ void WSCwindow::getWorkWSNmarginLeft(WSCushort* data){}
 void WSCwindow::setWorkWSNname(char* data){}
 void WSCwindow::getWorkWSNname(char** data){}
 
+#ifndef NO_WINDOW_TITLE
 void WSCwindow::setWorkWSNtitleString(char* data){
   if (_initialized != False && _dev != NULL){
     _dev->setValue(WSDEV_TITLE_STRING,(void*)data);
@@ -223,7 +249,7 @@ void WSCwindow::setWorkWSNtitleString(char* data){
 
 void WSCwindow::getWorkWSNtitleString(char** ){
 }
-
+#endif
 void WSCwindow::setWorkWSNuserString(char* ){
 }
 
@@ -237,39 +263,47 @@ void WSCwindow::getWorkWSNuserValue(long* ){
 }
 
 void WSCwindow::setWorkWSNx(short data){
+#ifndef WS_EMBED
   if (getPropertyEditMode() != False){
     long tmp =(long)data;
     setUserData("WW-X",(void*)tmp);
     return;
   }
+#endif
   if (_initialized != False && _dev != NULL){
       _dev->setValue(WSDEV_X,(void*)&data);
       _geometry_changed = True;
   }
 }
 void WSCwindow::getWorkWSNx(short* data){
+#ifndef WS_EMBED
   if (getPropertyEditMode() != False){
     long tmp =(long)getUserData("WW-X");
     short val = (short)tmp;
     *data = val;
   }
+#endif
 }
 void WSCwindow::setWorkWSNy(short data){
+#ifndef WS_EMBED
   if (getPropertyEditMode() != False){
     long tmp =(long)data;
     setUserData("WW-Y",(void*)tmp);
   }
+#endif
   if (_initialized != False && _dev != NULL){
     _dev->setValue(WSDEV_Y,(void*)&data);
     _geometry_changed = True;
   }
 }
 void WSCwindow::getWorkWSNy(short* data){
+#ifndef WS_EMBED
   if (getPropertyEditMode() != False){
     long tmp =(long)getUserData("WW-Y");
     short val = (short)tmp;
     *data = val;
   }
+#endif
 }
 void WSCwindow::setWorkWSNwidth(WSCushort data){
   if (_initialized != False && _dev != NULL){
@@ -314,8 +348,11 @@ void WSCwindow::setWorkWSNbackPixmap(short data){
 }
 
 void WSCwindow::getWorkWSNbackPixmap(short* ){}
+#ifndef NO_WINDOW_TITLE
 void WSCwindow::setWorkWSNtitleBar(char data){
+#ifndef WS_EMBED
   if (getPropertyEditMode() == False){
+#endif
     if (_dev == NULL){
       return;
     }
@@ -331,6 +368,7 @@ void WSCwindow::setWorkWSNtitleBar(char data){
       }
       _dev->setValue(WSDEV_TITLE_BAR,(void*)&val);
 	}
+#ifndef WS_EMBED
   }else{
     long val = data;
     setUserData("WW-TITLEB",(void*)val);
@@ -343,30 +381,38 @@ void WSCwindow::setWorkWSNtitleBar(char data){
     char dat = WS_FULL_TITLE;
     _dev->setValue(WSDEV_TITLE_BAR,(void*)&dat);
   }
+#endif
 }
-
 void WSCwindow::getWorkWSNtitleBar(char* data){
+#ifndef WS_EMBED
   if (getPropertyEditMode() != False){
     long tmp =(long)getUserData("WW-TITLEB");
     char fl = (char)tmp;
     *data = fl;
   }
+#endif
 }
+#endif //NO_WINDOW_TITLE
 
 void WSCwindow::setWorkWSNshadowType(char ){}
 void WSCwindow::getWorkWSNshadowType(char* ){}
 void WSCwindow::setWorkWSNvis(WSCbool data){
 //printf("WSCwindow:setWorkWSNvis %s editmode=%d %d\n",getInstanceName(),getPropertyEditMode(),data);
+#ifndef WS_EMBED
   if (getPropertyEditMode() == False){
+#endif
     setVisible( data );
+#ifndef WS_EMBED
   }else{
     _vis = _vis_bk;
     long val = (long)data;
     setUserData(WS_VIS,(void*)val);
   }
+#endif
 }
 
 void WSCwindow::getWorkWSNvis(WSCbool* data){
+#ifndef WS_EMBED
 //printf("WSCwindow::getWorkWSNvis %d\n",_vis);
   if (getPropertyEditMode() != False){
     long tmp = (long)getUserData(WS_VIS);
@@ -374,7 +420,12 @@ void WSCwindow::getWorkWSNvis(WSCbool* data){
 //printf("WSCwindow::getWorkWSNvis ret=%d\n",fl);
     *data = fl;
   }
+#endif
 }
+
+void WSCwindow::onWindowClose(){
+}
+#ifndef WS_EMBED
 void WSCwindow::onEditModeChange(WSCbool fl){
 //printf("WSCwindow::onEditModeChange %d\n",fl);
   WSCbase::onEditModeChange(fl);
@@ -391,34 +442,48 @@ void WSCwindow::onEditModeChange(WSCbool fl){
       setUserData(WS_VIS,(void*)1);
     }
     if (_sensitive == False){
-      setUserData(WS_DET,(void*)0);
-    }else{
-      setUserData(WS_DET,(void*)1);
+      setSensitive(True);
+      _sensitive_work = False;
     }
+//    if (_sensitive == False){
+//      setUserData(WS_DET,(void*)0);
+//    }else{
+//      setUserData(WS_DET,(void*)1);
+//    }
+#ifndef NO_WINDOW_TITLE
     val = _title_bar;
     setUserData("WW-TITLEB",(void*)val);
+#endif //NO_WINDOW_TITLE
   }
 }
+#endif
 void WSCwindow::setWorkWSNdet(WSCbool data){
+#ifndef WS_EMBED
   if (getPropertyEditMode() == False){
-    if (data == False){
-      _sensitive = True;
-    }else{
-      _sensitive = False;
-    }
+#endif
+//    if (data == False){
+//      _sensitive = True;
+//    }else{
+//      _sensitive = False;
+//    }
     setSensitive(data);
-  }else{
-    long val = (long)data;
-    setUserData(WS_DET,(void*)val);
+#ifndef WS_EMBED
+//  }else{
+//    long val = (long)data;
+//    setUserData(WS_DET,(void*)val);
+//    _sensitive = True;
   }
+#endif
 }
 
 void WSCwindow::getWorkWSNdet(WSCbool* data){
-  if (getPropertyEditMode() != False){
-    long tmp = (long)getUserData(WS_DET);
-    WSCbool fl = (WSCbool)tmp;
-    *data = fl;
-  }
+#ifndef WS_EMBED
+//  if (getPropertyEditMode() != False){
+//    long tmp = (long)getUserData(WS_DET);
+//    WSCbool fl = (WSCbool)tmp;
+//    *data = fl;
+//  }
+#endif
 }
 
 void WSCwindow::setWorkWSNpixmapStyle(WSCuchar data){
@@ -438,18 +503,24 @@ void WSCwindow::setWorkWSNmouse(WSCushort data ){
   }
 }
 void WSCwindow::getWorkWSNmouse(WSCushort* ){}
+#ifndef NO_GRADATION
 void WSCwindow::setWorkWSNgradation(WSCuchar ){}
 void WSCwindow::getWorkWSNgradation(WSCuchar* ){}
 void WSCwindow::setWorkWSNgradationMargin(WSCuchar ){}
 void WSCwindow::getWorkWSNgradationMargin(WSCuchar* ){}
+#endif
 void WSCwindow::setWorkWSNexit(WSCbool){}
 void WSCwindow::getWorkWSNexit(WSCbool* ){}
 
+#ifndef NO_REMOTE_INSTANCE
 void WSCwindow::setWorkWSNexport(WSCbool data){
   setExported(data);
 }
 void WSCwindow::getWorkWSNexport(WSCbool* data){
 }
+#endif
+void WSCwindow::setWorkWSNkeepPixmapCache(WSCbool){}
+void WSCwindow::getWorkWSNkeepPixmapCache(WSCbool*){}
 
 long WSCwindow::initialize(){
   WSCbase::initialize();
@@ -476,14 +547,16 @@ WSCwindow::~WSCwindow(){
     _set_active_instance(NULL);
   }
   getWorkWindowList()->del(this);
+#ifndef NO_WINDOW_TITLE
   if (_title_string != NULL){
-    delete _title_string;
+    delete[] _title_string;
   }
+#endif
   if (_user_string != NULL){
-    delete _user_string;
+    delete[] _user_string;
   }
   if (_group_name != NULL){
-    delete _group_name;
+    delete[] _group_name;
   }
 }
 
@@ -493,6 +566,9 @@ void WSCwindow::setChildren(WSClistData& src){
 WSClistData& WSCwindow::getChildren(){
   return _children;
 }
+WSClistData* WSCwindow::getChildrenPtr(){
+  return &_children;
+}
 
 long WSCwindow::_device_initialize(){
   WSDdev* dev = getowndev();
@@ -500,7 +576,10 @@ long WSCwindow::_device_initialize(){
     return WS_ERR;
   }
 
+#ifndef WS_EMBED
   if (getPropertyEditMode() == False){
+#endif
+#ifndef NO_WINDOW_TITLE
     char val = _title_bar;
     if (_title_bar == WS_NO_MANAGE){
       char val = WS_NO_TITLE;
@@ -508,7 +587,12 @@ long WSCwindow::_device_initialize(){
     }else{
       _dev->setValue(WSDEV_TITLE_BAR,(void*)&val);
     }
+#endif
+#ifndef WS_EMBED
   }
+#endif
+#ifndef NO_WINDOW_TITLE
+#ifndef WS_EMBED
   if (_title_bar == WS_NO_MANAGE && getPropertyEditMode() == False){
       WSCbool fl = True;
       dev->setValue(WSDEV_NO_FRAME,(void*)&fl);
@@ -516,6 +600,16 @@ long WSCwindow::_device_initialize(){
       WSCbool fl = False;
       dev->setValue(WSDEV_NO_FRAME,(void*)&fl);
   }
+#else
+  if (_title_bar == WS_NO_MANAGE){
+      WSCbool fl = True;
+      dev->setValue(WSDEV_NO_FRAME,(void*)&fl);
+  }else{
+      WSCbool fl = False;
+      dev->setValue(WSDEV_NO_FRAME,(void*)&fl);
+  }
+#endif
+#endif
 
   dev->setValue(WSDEV_MOUSE_NO,(void*)&_mouse_no);
   dev->setValue(WSDEV_X,(void*)&_x);
@@ -527,13 +621,14 @@ long WSCwindow::_device_initialize(){
 
   char pixmap_style = getPixmapStyle();
   dev->setValue(WSDEV_PIXMAP_STYLE,(void*)&pixmap_style);
+#ifndef NO_WINDOW_TITLE
   if (_title_string != NULL){
     dev->setValue(WSDEV_TITLE_STRING,(void*)_title_string);
   }else{
     dev->setValue(WSDEV_TITLE_STRING,(void*)"");
   }
-
-  dev->setEnableEventBit(WSEV_EXPOSE_BIT | WSEV_RESIZE_BIT);
+#endif
+  dev->setEnableEventBit(WSEV_EXPOSE_BIT | WSEV_RESIZE_BIT | WSEV_MOUSE_IN_BIT);
   if (_parent != NULL){
     dev->setValue(WSDEV_POPUP_PARENT,(void*)_parent);
   }
@@ -631,23 +726,18 @@ dbprintf("WSCwindow::draw %s:%d %s not ready\n",__FILE__,__LINE__,getInstanceNam
 
 
   if (_bp_no == -1){
+#ifndef NO_GRADATION
     if (_grad_no == 0){
-      dev->setForeColor(_bg_no);
-      dev->drawFillRect(0,0,_w,_h);
-#ifndef WS_EMBED
-      if ((WSGIappDev()->getGuiPolicy() & WS_POLICY_AQUA) != 0 && _bp_no == -1){
-        if (_a1_color == -1){
-          WSGFgetShadowColor(_bg_no,&_a1_color,&_a2_color,&_a3_color);
-        }
-        WSGFdrawStripes(dev,_w,_h,_a1_color,_a2_color,_a3_color);
-      }
 #endif
       dev->setForeColor(_bg_no);
+      dev->drawFillRect(0,0,_w,_h);
+      dev->setForeColor(_bg_no);
+#ifndef NO_GRADATION
     }else{
       dev->drawGradation(_grad_no,_ts_no,_bg_no,_bs_no,0,0,_w,_h,_grad_margin);
     }
+#endif
   }
-
   if (getDotMode() == True){
     dev->setForeColor(_fg_no);
     long i,j;
@@ -738,19 +828,43 @@ void WSCwindow::onResize(WSCrect* area){
     fl = True;
   }
   adjustForm();
+#ifndef WS_EMBED
   if (getPropertyEditMode() != False){
+#ifndef NO_MWT_FUNCTION
+#ifndef NO_INDEX_VARIANT_DATA
+    WSCundoProc* undo = WSGIappUndoManager()->getLastUndoProc();
+    WSCbool need_add = False;
+    if (getVisible() != False){
+      if (undo != NULL){
+        WSCpropUndoProc* undop = (WSCpropUndoProc*)undo->cast("WSCpropUndoProc");
+        WSCbase* target = undop->getTarget();
+        if (target != this){
+          need_add = True;
+        }
+      }else{
+        need_add = True;
+      }
+      if (need_add != False){
+        if (fl != False){
+          WSCpropUndoProc* new_undo = new WSCpropUndoProc();
+          new_undo->setTarget(this);
+          WSGIappUndoManager()->addUndoProc(new_undo);
+        }
+      }
+    }
+#endif   
+#endif   
     drawGriphand();
     if (fl != False){
-#ifndef WS_EMBED
       WSGFmarkChanged(this);
       WSCbase* prope = WSGIappPropertyEditor();
       if (prope != NULL){
         prope->setUserData(WS_GEOMETRY_UPDATE,(void*)1);
         prope->update();
       }
-#endif
     }
   }
+#endif
   if (fl != False){
     WSDdev* dev = getowndev();
     if (dev != NULL || getPixmapStyle() != WS_DIRECT_WINDOW){
@@ -793,14 +907,20 @@ void WSCwindow::onVisibleChange(WSCbool fl){
     execEventProc(WSEV_RESIZE,&rect);
   }
   if (fl != False){
+#ifndef NO_WINDOW_TITLE
     char title_bar = _title_bar;
+#ifndef WS_EMBED
     if (getPropertyEditMode() != False){
       title_bar = WS_ONLY_TITLE;
     }else{
+#endif
       if (title_bar == WS_NO_MANAGE){
         title_bar = WS_NO_TITLE;
       }
+#ifndef WS_EMBED
     }
+#endif
+#endif //NO_WINDOW_TITLE
     if (_unique != False && _group_name != NULL && strcmp(_group_name,"")){
       long num = getWorkWindowList()->getNum();
       long i;
@@ -812,7 +932,9 @@ void WSCwindow::onVisibleChange(WSCbool fl){
       }
     }
   }else if (_exit != False){
+#ifndef WS_EMBED
     if (getPropertyEditMode() == False){
+#endif
       if (_exit_handler != NULL){
         _exit_handler();
         execProcedure(WSEV_EXIT);
@@ -834,7 +956,9 @@ void WSCwindow::onVisibleChange(WSCbool fl){
           }
         }
       }
+#ifndef WS_EMBED
     }
+#endif
   }
 }
 
@@ -877,7 +1001,7 @@ long WSCwindow::clear(){
   }
 
   if (getVisible() == False){
-    return WS_ERR; //Ƥʤä饯ꥢʤ
+    return WS_ERR; //Ƥʤä饯ꥢʤ...
   }
 
   long err = dev->beginDraw(0,0,_w,_h,True);
@@ -897,15 +1021,24 @@ long WSCwindow::redraw(){
   }
 
   if (getVisible() == False){
-    return WS_ERR; //Ƥʤä饯ꥢʤ
+    return WS_ERR;
   }
+//  long noclear = False;
+//  if (_pixmap_style == WS_DYNAMIC_PIXMAP){
+//    WSDwindowDev* wdev = (WSDwindowDev*)dev->cast("WSDwindowDev");
+//    if (wdev != NULL && wdev->getPixmapCreated() == False){
+//      noclear = True;
+//    }
+//  }
 
-  long err = dev->beginDraw(0,0,_w,_h,True);
-  if (err != WS_NO_ERR){
-      return WS_NO_ERR;
-  }
-  dev->clearArea(0,0,_w,_h,True);
-  dev->endDraw();
+//  long err = dev->beginDraw(0,0,_w,_h,True);
+//  if (err != WS_NO_ERR){
+//      return WS_NO_ERR;
+//  }
+//  if (noclear == False){
+    dev->clearArea(0,0,_w,_h,True);
+//  }
+//  dev->endDraw();
 
   return WS_NO_ERR;
 }
@@ -923,6 +1056,7 @@ long WSCwindow::setMapStatus(long val){
 
 void WSCwindow::onParentVisibleChange(WSCbool fl){
   WSCbase::onParentVisibleChange(fl);
+#ifndef WS_EMBED
   if (fl != False){
     if (getPropertyEditMode() != False){
       setVisible(True);
@@ -932,6 +1066,7 @@ void WSCwindow::onParentVisibleChange(WSCbool fl){
       setVisible(False);
     }
   }
+#endif
 }
 static WSCbase* _active_window = NULL;
 static WSCbase* _from_window = NULL;
@@ -942,7 +1077,7 @@ void _set_active_instance(WSCbase* win){
 }
 
 long WSGFsetActiveWindow(WSCbase* win){
-  if ( win == NULL){  //NULLϲΰ̣
+  if ( win == NULL){  //NULLϲΰ̣...
     _from_window = _active_window;
     _active_window = NULL;
     return WS_NO_ERR;
@@ -952,7 +1087,7 @@ long WSGFsetActiveWindow(WSCbase* win){
     if (win->getVisible() == False){
       return WS_ERR;
     }
-    //եΤäƤ륪֥ȤΥե򳰤
+    //եΤäƤ륪֥ȤΥե򳰤...
     WSCbase* focusobj = WSCbase::getFocusInstance();
     if (focusobj != NULL){
       focusobj->setFocus(False);
@@ -964,9 +1099,10 @@ long WSGFsetActiveWindow(WSCbase* win){
     }
     char fl = True;
     dev->setValue(WSDEV_WIN_ACTIVE,(void*)&fl);
-    //setValue() 塢ǥХWinActive ٥Ȥȯ
-    //WSCworkWindow ʤɤΥ٥ȤȤˡ_set_active_instance()
-    //򤷤ƼʬϿƤ롣
+    //setValue() 塢ǥХWinActive ٥Ȥȯ...
+    //WSCworkWindow ʤɤΥ٥ȤȤˡ_set_active_instance()...
+    //򤷤ƼʬϿƤ롣...
+    return WS_NO_ERR;
   }
   return WS_ERR;
 }
@@ -975,7 +1111,7 @@ WSCbase* WSGFgetPreActiveWindow(){
 }
 
 WSCbase* WSGFgetActiveWindow(){
-  //ɥǤ˾äƤ ACTIVE Ǥʤ!
+  //ɥǤ˾äƤ ACTIVE Ǥʤ!...
   if (_active_window != NULL && _active_window->getVisible() == False){
       _active_window = NULL;
   }
@@ -998,17 +1134,17 @@ void WSCwindow::_exec_child_adjust_for_anchors(){
       WSClistData children = getChildren();
       long num = children.getNum();
       long i;
+      WSCbase** buf = (WSCbase**)children._data;
       for(i=0; i<num; i++){
-        WSCbase* child = (WSCbase*)children[i];
+        WSCbase* child = buf[i];
 dbprintf("WSCwindow::onResize %s:%d %s child=%s call adjust_for_anchors\n",__FILE__,__LINE__,getInstanceName(),child->getInstanceName());
         child->_adjust_for_anchors(_w,_h);
         if (getPixmapStyle() == WS_DYNAMIC_PIXMAP){
           if (child->isNeedUpdate() != False){
-            if ((long)child->getUserData("anchor update..") == 0){
-dbprintf("WSCwindow::onResize %s:%d %s child=%s call update\n",__FILE__,__LINE__,getInstanceName(),child->getInstanceName());
-              child->setUserData("anchor update..",(void*)1);
+            if (child->_flag_for_anchor == False){
+              child->_flag_for_anchor = True;
               child->update();
-              child->setUserData("anchor update..",(void*)0);
+              child->_flag_for_anchor = False;
             }
           }
         }
@@ -1054,6 +1190,11 @@ long WSCwindow::execEventProc(long ev, void* data){
       }
     }
     return WS_NO_ERR;
+  }else
+  if (ev == WSEV_WINDOW_CLOSED){
+    onWindowClose();
+    execProcedure(WSEV_WINDOW_CLOSED);
+    return WS_NO_ERR;
   }
   return WSCbase::execEventProc(ev,data);
 }
@@ -1099,4 +1240,46 @@ void WSCwindow::internalUpdate(){
     }
   }
 }
-
+#ifndef NO_RESTACK_CHILD
+long WSCwindow::restackChild(WSCbase* target,int order){
+  long num = _children.getNum();
+  if (order > num ){
+    order = num;
+  }
+  order = num - order;
+  long pos = -1;
+  long i;
+  for(i=0; i< num; i++){
+    WSCbase* child = (WSCbase*)_children[i];
+    if (child == target){
+      pos = i;
+      break;
+    }
+  }
+  if (pos == -1){
+    return WS_ERR;
+  }
+  if (order == pos+1){
+    return WS_NO_ERR;
+  }
+  _children.delPos(pos);
+  if (order > 0){
+    order--;
+  }
+  _children.add(target,order);
+  WSDdev* dev = getowndev();
+  if (dev != NULL){
+    WSDwindowDev* wdev = (WSDwindowDev*)dev->cast("WSDwindowDev");
+    if (wdev != NULL){
+      wdev->restackChildren();
+    }
+  }
+  return WS_NO_ERR;
+}
+#endif
+WSCbool WSCwindow::getEnableClose(){
+  return _enable_close;
+}
+void WSCwindow::setEnableClose(WSCbool fl){
+  _enable_close = fl;
+}
