@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSDappDev.h>
@@ -52,8 +48,10 @@ WSDappDev::WSDappDev(){
 WSMFtrace("Error! too many instances of WSDappDev.\n");
     WSGFexit(1);
   }
+  _name = NULL;
   _modal_client = NULL;
   _modal_proc = NULL;
+  _popup_modal_client = NULL;
   _iprm = 0;
   _cprm = 0;
   _policy = -1;
@@ -62,12 +60,22 @@ WSMFtrace("Error! too many instances of WSDappDev.\n");
   _mode = WS_MODE_NORMAL;
   _alt_context = NULL;
   _alt_context2 = NULL;
+  _rebase_handle = 0;
   _under_exec_init = False;
+  _exit_event_loop = False;
+  _locked = False;
 }
 
 WSDappDev::~WSDappDev(){
+  if (_name != NULL){
+    delete _name;
+    _name = NULL;
+  }
 //  destroy();
 }
+void WSDappDev::exitEventLoop(){
+  _exit_event_loop = True;
+}
 int WSDappDev::getArgc(){
   return _iprm;
 }
@@ -76,11 +84,18 @@ char** WSDappDev::getArgv(){
 }
 
 void WSDappDev::setInstanceName(char* name){
-  _name.setString(name);
+  if (_name != NULL){
+    delete _name;
+    _name = NULL;
+  }
+  _name = WSGFstrdup(name);
 }
 
 char* WSDappDev::getInstanceName(){
-  return _name.getString();
+  if (_name != NULL){
+    return _name;
+  }
+  return "";
 }
 
 long WSDappDev::initialize(){
@@ -150,10 +165,16 @@ void WSDappDev::setInitHandler(void (*hd)() ){
 WSCushort WSDappDev::getWidth(){
   return (WSCushort)1024;
 }
+WSCushort WSDappDev::getXOffset(){
+  return (WSCushort)0;
+}
 
 WSCushort WSDappDev::getHeight(){
   return (WSCushort)768;
 }
+WSCushort WSDappDev::getYOffset(){
+  return (WSCushort)0;
+}
 
 WSCbool WSDappDev::getTraceMode(){
   return _trace_output;
@@ -192,6 +213,12 @@ void WSDappDev::setModalClient(WSCbase* client){
 WSCbase* WSDappDev::getModalClient(){
   return _modal_client;
 }
+void WSDappDev::setPopupModalClient(WSCbase* client){
+  _popup_modal_client = client;
+}
+WSCbase* WSDappDev::getPopupModalClient(){
+  return _popup_modal_client;
+}
 void WSDappDev::update(){
 WSMFtrace("WSDappDev::update() This system does not supprt.\n");
 }
@@ -268,6 +295,17 @@ void WSDappDev::delMenu(WSCbase* menu){
 WSClistData& WSDappDev::getMenuList(){
   return _pld_menu_list;
 }
+void WSDappDev::addAltFocusInstance(WSCbase* inst){
+  _alt_focus_list.add((void*)inst);
+}
+void WSDappDev::delAltFocusInstance(WSCbase* inst){
+  _alt_focus_list.del((void*)inst);
+}
+WSClistData& WSDappDev::getAltFocusInstanceList(){
+  return _alt_focus_list;
+}
+
+
 void WSDappDev::exit(long val){
   WSGFexit(val);
 }
@@ -289,7 +327,14 @@ void WSDappDev::setContext2(void* context){
 void* WSDappDev::getContext2(){
   return _alt_context2;
 }
-
+#ifndef NO_MWT_FUNCTION
+void WSDappDev::setRebaseHandle(long context){
+  _rebase_handle = context;
+}
+long WSDappDev::getRebaseHandle(){
+  return _rebase_handle;
+}
+#endif
 WSCstring WSDappDev::getExtGuiPolicyName(){
   return WSGIappGuiPolicy()->getExtGuiPolicyName();
 #if 0
@@ -369,3 +414,16 @@ WSCbool WSDappDev::getUnderExecInitialize(){
 WSCbool WSDappDev::getInitializeExecuted(){
   return _init_executed;
 }
+long WSDappDev::lock(){
+  fprintf(stderr,"WSDappDev::lock() this version does not supported.\n");
+  return WS_ERR;
+}
+long WSDappDev::unlock(){
+  fprintf(stderr,"WSDappDev::unlock() this version does not supported.\n");
+  return WS_ERR;
+}
+WSCbool WSDappDev::getLockStatus(){
+  return _locked;
+}
+void WSDappDev::checkThreadSafe(){
+}
