@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSDdev.h>
@@ -62,6 +58,9 @@ WSDdev::WSDdev(){
   _y = 0;
   _deleted = 0;
   _visible = 0;
+  _operation = WS_OPERATION_SRC_COPY;
+  _pixmap_style = WS_DIRECT_WINDOW;
+  _expose_sequence = False;
   setInitialized(0);
 }
 
@@ -116,7 +115,15 @@ void WSDdev::setXOffsetPtr(short* ptr){
 void WSDdev::setYOffsetPtr(short* ptr){
   _y_offset_ptr = ptr;
 }
-
+WSCbool WSDdev::getScaled(){
+  if (_scale_ptr == NULL){
+    return False;
+  }
+  if (*_scale_ptr == 1.0){
+    return False;
+  }
+  return True;
+}
 double WSDdev::getScale(){
   if (_scale_ptr == NULL){
     return (double)1;
@@ -183,10 +190,17 @@ long WSDdev::execEvent(long /*ev*/,void* /*data*/){
 
 WSCbool WSDdev::isExposed(short x,short y,WSCushort w,WSCushort h,WSCbool scaling){
   if (scaling != False){
-    x = (short)((x + getXOffset()) * getScale());
-    y = (short)((y + getYOffset()) * getScale());
-    w = (WSCushort)(w * getScale());
-    h = (WSCushort)(h * getScale());
+    WSCbool scaled = getScaled();
+    if ( scaled != False){
+      double scale = getScale();
+      x = (short)((x + getXOffset())* scale);
+      y = (short)((y + getYOffset())* scale);
+      w = (WSCushort)(w * scale);
+      h = (WSCushort)(h * scale);
+    }else{
+      x = x + getXOffset();
+      y = y + getYOffset();
+    }
   }
 
   short ex,ey;
@@ -212,48 +226,71 @@ long WSDdev::informEventToClient(long ev_kind,void* data){
   WSCbase* mcl = WSGIappDev()->getModalClient();
   if (mcl != NULL && mcl != _client && ev_kind != WSEV_RESIZE){
 //WSMFtrace("WSDdev::informEventToClient modal\n");
-    void(*proc)(WSDdev*,long,void*) = (void(*)(WSDdev*,long,void*))WSGIappDev()->getModalProc();
-    WSCbase* cl = _client->getParent();
-    if (cl == NULL){
-      if (proc != NULL){
-        proc(this,ev_kind,data);
-        return WS_NO_ERR;
-      }
-      return WS_ERR;
-    }
-    long fl = False;
-    while(1){
-      if (cl == mcl){
-        fl = True;
+    WSCbool val = False;
+    WSDdev* d = this;
+    while(d != NULL){
+      d->getValue(WSDEV_GRAB_POINTER,&val);
+      if (val != False){
         break;
       }
-      cl = cl->getParent();
+      d = d->getParentDev();
+    }
+    if (val == False){
+      void(*proc)(WSDdev*,long,void*) = (void(*)(WSDdev*,long,void*))WSGIappDev()->getModalProc();
+      WSCbase* cl = _client->getParent();
       if (cl == NULL){
-        break;
+        if (proc != NULL){
+          proc(this,ev_kind,data);
+          return WS_NO_ERR;
+        }
+        return WS_ERR;
+      }
+      long fl = False;
+      while(1){
+        if (cl == mcl){
+          fl = True;
+          break;
+        }
+        cl = cl->getParent();
+        if (cl == NULL){
+          break;
+        }
+      }
+      if (fl == False){
+        if (proc != NULL){
+          proc(this,ev_kind,data);
+          return WS_NO_ERR;
+        }
+        return WS_ERR;
       }
     }
-    if (fl == False){
-      if (proc != NULL){
-        proc(this,ev_kind,data);
-        return WS_NO_ERR;
-      }
-      return WS_ERR;
-    }
+  }
+  WSCbase* pmcl = WSGIappDev()->getPopupModalClient();
+  if (pmcl != NULL && pmcl != _client){
+    return WS_ERR;
   }
 
   if (ev_kind == WSEV_MOUSE_IN || ev_kind == WSEV_MOUSE_MOVE ||
             ev_kind == WSEV_MOUSE_PRESS || ev_kind == WSEV_MOUSE_RELEASE){
      WSCpoint point;
      WSCpoint* src = (WSCpoint*)data;
-     double scale = getScale();
-     if (scale != 1){
+     WSCbool scaled = getScaled();
+     if (scaled != False){
+       double scale = getScale();
        point.setPoint((short)(src->x / scale) - getXOffset(),(short)(src->y / scale) - getYOffset()); 
      }else{
        point.setPoint(src->x - getXOffset(),src->y - getYOffset()); 
      }
-     if (_client->getSensitive() != False || _client->getPropertyEditMode() != False){
+#ifndef WS_EMBED
+     if (_client->getSensitive() != False
+         || _client->getPropertyEditMode() != False){
        _client->execEventProc(ev_kind,&point);
      }
+#else
+     if (_client->getSensitive() != False){
+       _client->execEventProc(ev_kind,&point);
+     }
+#endif
      return WS_ERR;
   }
   return  _client->execEventProc(ev_kind,data);
@@ -348,13 +385,22 @@ WSDdraw* WSDdev::getNativeDraw(){
   return _native_draw;
 }
 
+long WSDdev::internalBeginDraw(short x,short y,WSCushort w,WSCushort h,WSCbool absolute,WSCbool scaling){
+  return beginDraw(x,y,w,h,absolute,scaling);
+}
 long WSDdev::beginDraw(short x,short y,WSCushort w,WSCushort h,WSCbool absolute,WSCbool scaling){
   _absolute_draw = absolute;
   if (scaling != False){
-    x = (short)((x + getXOffset())*getScale());
-    y = (short)((y + getYOffset())*getScale());
-    w = (WSCushort)(w * getScale());
-    h = (WSCushort)(h * getScale());
+    if (getScaled() == False){
+      x += getXOffset();
+      y += getYOffset();
+    }else{
+      double scale = getScale();
+      x = (short)((x + getXOffset())*scale);
+      y = (short)((y + getYOffset())*scale);
+      w = (WSCushort)(w * scale);
+      h = (WSCushort)(h * scale);
+    }
   }
 
   WSDdev* pdev = getParentDev();
@@ -399,6 +445,9 @@ long WSDdev::beginDraw(short x,short y,WSCushort w,WSCushort h,WSCbool absolute,
 
   return WS_NO_ERR;
 }
+long WSDdev::internalEndDraw(){
+  return endDraw();
+}
 long WSDdev::endDraw(){
   _absolute_draw = False;
   WSDdraw* draw = getDraw();
@@ -480,8 +529,9 @@ long WSDdev::drawArc(short x,short y,WSCushort w,WSCushort h,short a1,short a2){
   if (draw == NULL){
     return WS_ERR;
   }
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawArc((short)(x * scale) +_x, 
                   (short)(y * scale) +_y,
                   (WSCushort)(w * scale), 
@@ -498,8 +548,9 @@ long WSDdev::drawFillArc(short x,short y,WSCushort w,WSCushort h,short a1,short
     return WS_ERR;
   }
 
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawFillArc((short)(x * scale) +_x, 
                       (short)(y * scale) +_y,
                       (WSCushort)(w * scale), 
@@ -525,8 +576,9 @@ long WSDdev::drawLines(WSCpoint* points,long num){
   }
 
   long i;
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     for(i = 0; i < num; i++){
       pt[i].x = (short)(points[i].x * scale) +_x;
       pt[i].y = (short)(points[i].y * scale) +_y;
@@ -550,8 +602,9 @@ long WSDdev::drawLine(short x1,short y1,short x2,short y2){
     return WS_ERR;
   }
 
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawLine((short)(x1 * scale) +_x, (short)(y1 * scale) +_y,
                    (WSCushort)(x2 * scale) +_x,
                    (WSCushort)(y2 * scale) +_y);
@@ -566,8 +619,9 @@ long WSDdev::drawRect(short x,short y,WSCushort w,WSCushort h){
   if (draw == NULL){
     return WS_ERR;
   }
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawRect((short)(x * scale) +_x, (short)(y * scale) +_y,
                    (WSCushort)(w * scale),
                    (WSCushort)(h * scale));
@@ -581,8 +635,9 @@ long WSDdev::drawFillRect(short x,short y,WSCushort w,WSCushort h){
   if (draw == NULL){
     return WS_ERR;
   }
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawFillRect((short)(x * scale) +_x, (short)(y * scale) +_y,
                        (WSCushort)(w * scale),
                        (WSCushort)(h * scale));
@@ -598,8 +653,9 @@ long WSDdev::drawGradation(long type,short col1,short col2,short col3,
   if (draw == NULL){
     return WS_ERR;
   }
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawGradation(type,col1,col2,col3,
                        (short)(x * scale) +_x, (short)(y * scale) +_y,
                        (WSCushort)(w * scale),
@@ -664,8 +720,9 @@ long WSDdev::drawRects(WSCrect* rects,long num){
   }
 
   long i;
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     for(i = 0; i < num; i++){
       rt[i].x = (short)(rects[i].x * scale) +_x;
       rt[i].y = (short)(rects[i].y * scale) +_y;
@@ -700,8 +757,9 @@ long WSDdev::drawFillRects(WSCrect* rects,long num){
     rt = r1;
   }
   long i;
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     for(i = 0; i < num; i++){
       rt[i].x = (short)(rects[i].x * scale) +_x;
       rt[i].y = (short)(rects[i].y * scale) +_y;
@@ -736,8 +794,9 @@ long WSDdev::drawPoly(WSCpoint* points,long num){
     pt = p1;
   }
   long i;
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     for(i = 0; i < num; i++){
       pt[i].x = (short)(points[i].x * scale) +_x;
       pt[i].y = (short)(points[i].y * scale) +_y;
@@ -769,8 +828,9 @@ long WSDdev::drawFillPoly(WSCpoint* points,long num){
   }
 
   long i;
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     for(i = 0; i < num; i++){
       pt[i].x = (short)(points[i].x * scale) +_x;
       pt[i].y = (short)(points[i].y * scale) +_y;
@@ -793,8 +853,9 @@ long WSDdev::drawImage(short x,short y,WSCushort w,WSCushort h,WSDimage* image,c
   if (draw == NULL){
     return WS_ERR;
   }
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawImage( (short)(x * scale) +_x,
                      (short)(y * scale) +_y,
                      (WSCushort)(w * scale),
@@ -809,8 +870,9 @@ long WSDdev::drawImage(short x,short y,short sx,short sy,WSCushort w,WSCushort h
   if (draw == NULL){
     return WS_ERR;
   }
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawImage( (short)(x * scale) +_x,
                      (short)(y * scale) +_y,
                      sx,sy,
@@ -826,8 +888,9 @@ long WSDdev::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDimage
   if (draw == NULL){
     return WS_ERR;
   }
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawStretchedImage( (short)(x * scale) +_x,
                      (short)(y * scale) +_y,
                      (WSCushort)(w * scale),
@@ -843,8 +906,9 @@ long WSDdev::drawString(long x,long y,WSCulong w,WSCulong h,WSCstring* str,char
   if (draw == NULL){
     return WS_ERR;
   }
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawString( (long)(x * scale) +_x, 
                       (long)(y * scale) +_y,
                       (WSCulong)(w * scale), 
@@ -862,10 +926,11 @@ long WSDdev::drawString(long x,long y,WSCulong w,WSCulong h,char font,char align
   if (draw == NULL){
     return WS_ERR;
   }
-  double scale = getScale();
   WSCstring tmp;
   tmp.setString(str,encoding);
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawString( (long)(x * scale) +_x, 
                       (long)(y * scale) +_y,
                       (WSCulong)(w * scale), 
@@ -882,8 +947,9 @@ long WSDdev::drawUString(long x,long y,WSCulong w,WSCulong h,WSCushort* str,char
   if (draw == NULL){
     return WS_ERR;
   }
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawUString( (long)(x * scale) +_x, 
                       (long)(y * scale) +_y,
                       (WSCulong)(w * scale), 
@@ -901,8 +967,9 @@ long WSDdev::drawFillString(long x,long y,WSCulong w,WSCulong h,WSCstring* str,c
   if (draw == NULL){
     return WS_ERR;
   }
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawFillString( (long)(x * scale) +_x, 
                           (long)(y * scale) +_y,
                           (WSCulong)(w * scale), 
@@ -921,8 +988,9 @@ long WSDdev::drawFillString(long x,long y,WSCulong w,WSCulong h,char font,char a
   }
   WSCstring tmp;
   tmp.setString(str,encoding);
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawFillString( (long)(x * scale) +_x, 
                           (long)(y * scale) +_y,
                           (WSCulong)(w * scale), 
@@ -939,8 +1007,9 @@ long WSDdev::drawFillUString(long x,long y,WSCulong w,WSCulong h,WSCushort* str,
   if (draw == NULL){
     return WS_ERR;
   }
-  double scale = getScale();
-  if (scale != 1){
+  WSCbool scaled = getScaled();
+  if (scaled != False){
+    double scale = getScale();
     draw->drawFillUString( (long)(x * scale) +_x, 
                           (long)(y * scale) +_y,
                           (WSCulong)(w * scale), 
@@ -954,7 +1023,6 @@ long WSDdev::drawFillUString(long x,long y,WSCulong w,WSCulong h,WSCushort* str,
 
 long WSDdev::setRegion(short x,short y,WSCushort w, WSCushort h){
   WSDdraw* draw = getDraw();
-  double scale = getScale();
 
   short ex,ey;
   WSCushort ew,eh;
@@ -963,6 +1031,19 @@ long WSDdev::setRegion(short x,short y,WSCushort w, WSCushort h){
     return WS_ERR;
   }
 
+  WSCbool scaled = getScaled();
+
+  if (scaled != False){
+    double scale = getScale();
+    x=(short)(x * scale) +_x;
+    y=(short)(y * scale) +_y;
+    w=(WSCushort)(w * scale);
+    h=(WSCushort)(h * scale);
+  }else{
+    x+=_x;
+    y+=_y;
+  }
+
   long ox,oy;
   WSCulong ow,oh;
   WSGFandArea(x,y,w,h,ex,ey,ew,eh,&ox,&oy,&ow,&oh);
@@ -974,13 +1055,8 @@ long WSDdev::setRegion(short x,short y,WSCushort w, WSCushort h){
     h = oh;
   }
 
-  if (scale != 1){
-    draw->setRegion( (short)(x * scale) +_x,(short)(y * scale) +_y,
-                     (WSCushort)(w * scale),
-                     (WSCushort)(h * scale));
-  }else{
-    draw->setRegion(x+_x,y+_y,w,h);
-  }
+  draw->setRegion(x,y,w,h);
+
   return WS_NO_ERR;
 }
 
@@ -989,8 +1065,9 @@ long WSDdev::getExposedArea(short* x,short* y, WSCushort* w, WSCushort* h,WSCboo
   if (dev != NULL){
       dev->getExposedArea(x,y,w,h,False);
       if (scaling != False){
-        double scale = getScale();
-        if (scale != 1){
+        WSCbool scaled = getScaled();
+        if (scaled != False){
+          double scale = getScale();
           *x = (short)(*x / scale) - getXOffset();
           *y = (short)(*y / scale) - getYOffset();
           *w = (WSCushort)(*w / scale);
@@ -1077,13 +1154,42 @@ WSCbool WSDdev::isIndependent(){
 
 void WSDdev::deleteInstance(){
   if (_deleted == 0){
-    WSGIappDevice()->addDeleteInstance(this);
     detachClient();
     setInitialized(False);
+    _deleted = 1;
+    WSGIappDevice()->addDeleteInstance(this);
   }
-  _deleted = 1;
 }
 void WSDdev::getWH(WSCushort* w,WSCushort* h){
   *w = _w;
   *h = _h;
 }
+long WSDdev::setOperation(WSCuchar op){
+  WSDdraw* draw = getDraw();
+  if (draw == NULL){
+    return WS_ERR;
+  }
+  _operation = op;
+  draw->setOperation(op);
+  return WS_NO_ERR;
+}
+WSCuchar WSDdev::getOperation(){
+  return _operation;
+}
+void WSDdev::setExposeSequence(WSCbool fl){
+  _expose_sequence = fl;
+}
+WSCbool WSDdev::getExposeSequence(){
+  return _expose_sequence;
+}   
+char WSDdev::getPixmapStyle(){
+  return _pixmap_style;
+}
+
+void WSDdev::setUnderLineChar(long ustart,long ulen){
+  WSDdraw* draw = getDraw();
+  if (draw == NULL){
+    return;
+  }
+  draw->setUnderLineChar(ustart,ulen);
+}
