@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSDmouse.h>
@@ -46,17 +42,31 @@ WSMFtrace("WSGImouse() ERROR. cannot create WSDmouse class instance.\n");
   return _the_app_mouse;
 }
 
-struct WSCmouseData{
-  long id;
-  long data;
-  WSCstring str;
+WSCmouseData::WSCmouseData(){
+  id = 0;
+  data = 0;
+  name = NULL;
 };
+void WSCmouseData::setName(char* n){
+  if (name != NULL){
+    delete[] name;
+  }
+  name = WSGFstrdup(n);
+}
+WSCmouseData::~WSCmouseData(){
+  if (name != NULL){
+    delete[] name;
+  }
+}
 
 
 WSDmouse::WSDmouse(){
   _status = 0;
   _target_btn = 0 ;
   _fclient = NULL;
+  _global_proc = NULL;
+  _global_mouse_id = -1;
+  _global_mouse_sensitive = True;
 }
 void WSDmouse::initialize(){}
 
@@ -80,6 +90,7 @@ void WSDmouse::delMouseCursor(long pos){
   if (pos < num){
     WSCmouseData* mouse = (WSCmouseData*)_mlist[pos];
     _mlist.del((void*)mouse);
+    delete mouse;
   }
 }
 void WSDmouse::setMouseCursor(long){}
@@ -87,12 +98,13 @@ void WSDmouse::setupMouseCursor(char*){}
 void WSDmouse::addMouseCursor(long id,long data,char* str){
   WSCmouseData* mouse = NULL;
   long i;
-  long num = _mlist.getNum();
+  long num = _mlist._num;
+  WSCmouseData** mouse_list = (WSCmouseData**)_mlist._data;
   for(i=0; i<num; i++){
-    WSCmouseData* mouse1 = (WSCmouseData*)_mlist[i];
+    WSCmouseData* mouse1 = mouse_list[i];
     if (mouse1->id == id){
       mouse1->data = data;
-      mouse1->str.setString(str);
+      mouse1->setName(str);
       _mlist.del((void*)mouse1);
       mouse = mouse1;
       break;
@@ -102,11 +114,12 @@ void WSDmouse::addMouseCursor(long id,long data,char* str){
     mouse = new WSCmouseData;
     mouse->data = data;
     mouse->id = id;
-    mouse->str.setString(str);
+    mouse->setName(str);
   }
-  num = _mlist.getNum();
+  num = _mlist._num;
+  WSCmouseData** mouse_list2 = (WSCmouseData**)_mlist._data;
   for(i=0; i<num; i++){
-    WSCmouseData* mouse1 = (WSCmouseData*)_mlist[i];
+    WSCmouseData* mouse1 = mouse_list2[i];
     if (mouse1->id < id){
       continue;
     }
@@ -115,7 +128,9 @@ void WSDmouse::addMouseCursor(long id,long data,char* str){
   } 
   _mlist.add((void*)mouse);
 }
-
+void WSDmouse::setGlobalMouseHook(WSCbool (*func)(long,WSCbase*)){
+  _global_proc = func;
+}
 long WSDmouse::getMouseCursorId(long pos){
   long num = _mlist.getNum();
   if (pos < 0){
@@ -132,27 +147,36 @@ long WSDmouse::getMouseCursorId(long pos){
 char* WSDmouse::getMouseCursorName(long id){
   long num = _mlist.getNum();
   long i;
+  WSCmouseData** mouse_list = (WSCmouseData**)_mlist.getBuf();
   for(i=0; i<num; i++){
-    WSCmouseData* mouse = (WSCmouseData*)_mlist[i];
+    WSCmouseData* mouse = mouse_list[i];
     if (mouse->id == id){
-      return mouse->str.getString();
+      if (mouse->name != NULL){
+        return mouse->name;
+      }else{
+        return "";
+      }
     }
   }
   return NULL;
 }
 
 long WSDmouse::getMouseCursor(long id){
+  if (_global_mouse_id != -1){
+    id = _global_mouse_id;
+  }
   long num = _mlist.getNum();
   long i;
+  WSCmouseData** mouse_list = (WSCmouseData**)_mlist.getBuf();
   for(i=0; i<num; i++){
-    WSCmouseData* mouse = (WSCmouseData*)_mlist[i];
+    WSCmouseData* mouse = mouse_list[i];
     if (mouse->id == id){
       return mouse->data;
     }
   }
   id = 0;
   for(i=0; i<num; i++){
-    WSCmouseData* mouse = (WSCmouseData*)_mlist[i];
+    WSCmouseData* mouse = mouse_list[i];
     if (mouse->id == id){
       return mouse->data;
     }
@@ -180,4 +204,40 @@ void WSDmouse::setMouseFocusClient(WSCbase* cl){
 WSCbase* WSDmouse::getMouseFocusClient(){
   return _fclient;
 }
-
+void WSDmouse::setMouseUngrabClient(WSCbase* cl){
+  _ungrab_client = cl;
+}
+WSCbase* WSDmouse::getMouseUngrabClient(){
+  return _ungrab_client;
+}
+void* WSDmouse::getGlobalMouseHook(){
+  return (void*)_global_proc;
+}
+WSCbool WSDmouse::execGlobalMouseHook(long ev,WSCbase* base){
+  if (_global_proc != NULL){
+    return _global_proc(ev,base);
+  }
+  return True;
+}
+void WSDmouse::resetGlobalMouseId(){
+  _global_mouse_id = -1;
+}
+void WSDmouse::setGlobalMouseId(short id){
+  _global_mouse_id = id;
+}
+short WSDmouse::getGlobalMouseId(){
+  return _global_mouse_id;
+}
+WSCbool WSDmouse::getGlobalMouseIdStatus(){
+  if (_global_mouse_id == -1){
+    return False;
+  }else{
+    return True;
+  }
+}
+void WSDmouse::setGlobalMouseSensitive(WSCbool fl){
+  _global_mouse_sensitive = fl;
+}
+WSCbool WSDmouse::getGlobalMouseSensitive(){
+  return _global_mouse_sensitive;
+}
