@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,17 +14,14 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
+#ifndef NO_PSDRAW
 #include <WSDprDev.h>
 #include <WSCdevice.h>
 #include <WSDdraw.h>
@@ -54,29 +51,43 @@ long WSDprDev::getValue(long kind,void* data){
 }
 
 long WSDprDev::setValue(long kind,void* data){
-  long val = *(long*)data;
+  long val;
   switch(kind){
-    case WSDEV_PR_TYPE:
+    case WSDEV_PR_TYPE:{
+      val = *(long*)data;
       _type = val;
       break;
-    case WSDEV_PR_FORMAT:
+    }
+    case WSDEV_PR_FORMAT:{
+      val = *(long*)data;
       _format = val;
       break;
-    case WSDEV_WORK_X:
+    }
+    case WSDEV_WORK_X:{
+      val = *(long*)data;
       _work_x = val;
       break;
-    case WSDEV_WORK_Y:
+    }
+    case WSDEV_WORK_Y:{
+      val = *(long*)data;
       _work_y = val;
       break;
-    case WSDEV_WORK_WIDTH:
+    }
+    case WSDEV_WORK_WIDTH:{
+      val = *(long*)data;
       _work_width = val;
       break;
-    case WSDEV_WORK_HEIGHT:
+    }
+    case WSDEV_WORK_HEIGHT:{
+      val = *(long*)data;
       _work_height = val;
       break;
-    case WSDEV_PR_ORIENTATION:
+    }
+    case WSDEV_PR_ORIENTATION:{
+      val = *(long*)data;
       _orientation = val;
       break;
+    }
   }
   return WS_NO_ERR;
 }
@@ -202,3 +213,4 @@ long WSDprDev::getSpecialResource(){
 WSCbool WSDprDev::isExposed(short,short,WSCushort,WSCushort,WSCbool){
   return True;
 }
+#endif //NO_PSDRAW
