@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,16 +14,14 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
+#include <WScom.h>
+#ifndef NO_PSDRAW
 #include <WSDfont.h>
 #include <WSCfontSet.h>
 #include <WSDcolor.h>
@@ -201,6 +199,16 @@ void putcolor(WSDcolor* color,WSCstring* str){
   (*str) << (double)g/256 << " "; 
   (*str) << (double)b/256 << " setrgbcolor\n"; 
 }
+
+void putregion(short x,short y,WSCushort w,WSCushort h,WSCstring* str){
+  (*str) << x << " " <<  y << " "
+	 << w << " " <<  h << " ws_cliprect\n";
+}
+
+void putregion(WSCstring* str){
+  (*str) << "ws_clipregion\n";
+}
+
 void drawfill(long no,long x,long y,WSCulong w,WSCulong h,WSCstring* str){
   long x1 = x;
   if ( x%16 != 0){
@@ -219,6 +227,11 @@ void drawfill(long no,long x,long y,WSCulong w,WSCulong h,WSCstring* str){
 }
 
 long WSDpsdraw::setRegion(short x,short y,WSCushort w,WSCushort h){
+  WSCstring str;
+  str << x << " " <<  y << " "
+      << w << " " <<  h << " ws_setregion\n";
+  _clipping = True;
+  _file->write((char*)str,strlen((char*)str));
   return WS_NO_ERR;
 }
 
@@ -229,19 +242,24 @@ long WSDpsdraw::drawArc(short x,short y,WSCushort w,WSCushort h,short a1,short a
   if ((a2 - a1)/64/360 != 0){
     WSCstring str;
     putcolor(_fg_color,&str);
+    str << "gsave\n";
+    if( _clipping )putregion(&str);// putregion(1)
     str << "gsave gsave newpath\n";
     str << x+w/2 << " " << y+h/2 << " ";
-    str << w << " " << h << " ws_ellipse\n";
+    str << w/2 << " " << h/2 << " ws_ellipse\n";
     if (_fill_type != 0){
       str << "closepath clip newpath\n";
       drawfill(_fill_type,x,y,w,h,&str);
     }
     str << " stroke grestore grestore\n";
+    str << "grestore\n";// correcpoiding to gsave just before putregion(1)
 
     _file->write((char*)str,strlen((char*)str));
     return WS_NO_ERR;
   }
   WSCstring str;
+  str << "gsave\n";
+  if( _clipping )putregion(&str);// putregion(2)
   str << "gsave gsave newpath\n";
   str << x+w/2 << " " << y + h/2 << " " << w/2 << " " << h/2 << " "
       << -a1/64 << " " << -a2/64 << " ws_arc\n";
@@ -252,6 +270,7 @@ long WSDpsdraw::drawArc(short x,short y,WSCushort w,WSCushort h,short a1,short a
     drawfill(_fill_type,x,y,w,h,&str);
   }
   str << "stroke grestore grestore\n";
+  str << "grestore\n";// correcpoiding to gsave just before putregion(2)
 
   _file->write((char*)str,strlen((char*)str));
   return WS_NO_ERR;
@@ -263,7 +282,9 @@ long WSDpsdraw::drawFillArc(short x,short y,WSCushort w,WSCushort h,short a1,sho
   if ((a2 - a1)/64/360 != 0){
     WSCstring str;
     putcolor(_fg_color,&str);
-    str << "gsave\nnewpath ";
+    str << "gsave\n";
+    if( _clipping )putregion(&str);
+    str << "newpath ";
     str << x+w/2 << " " << y+h/2 << " ";
     if (_fill_type == 0){
       str << w/2 << " " << h/2 << " ws_ellipse fill\n";
@@ -279,7 +300,9 @@ long WSDpsdraw::drawFillArc(short x,short y,WSCushort w,WSCushort h,short a1,sho
   if (kind == WS_PI){
     WSCstring str;
     putcolor(_fg_color,&str);
-    str << "gsave\nnewpath\n";
+    str << "gsave\n";
+    if( _clipping )putregion(&str);
+    str << "newpath\n";
     str << x+w/2 << " " << y+h/2 << " moveto\n";
     str << x+w/2 << " " << y+h/2 << " " << w/2 << " " << h/2 << " "
         << -a1/64 << " " << -a2/64 << " ws_arc\n";
@@ -294,7 +317,9 @@ long WSDpsdraw::drawFillArc(short x,short y,WSCushort w,WSCushort h,short a1,sho
   }else{
     WSCstring str;
     putcolor(_fg_color,&str);
-    str << "gsave\nnewpath\n";
+    str << "gsave\n";
+    if( _clipping )putregion(&str);
+    str << "newpath\n";
     str << x+w/2 << " " << y+h/2 << " " << w/2 << " " << h/2 << " ";
     if (_fill_type == 0){
       str  << -a1/64 << " " << -a2/64 << " ws_arc fill\n";
@@ -313,7 +338,9 @@ long WSDpsdraw::drawLine(short x1,short y1,short x2,short y2){
   }
   WSCstring str;
   putcolor(_fg_color,&str);
-  str << "gsave newpath\n";
+  str << "gsave\n";
+  if( _clipping )putregion( &str );
+  str << "newpath\n";
   str << x1 << " " << y1 << " moveto\n";
   str << x2 << " " << y2 << " lineto\n";
   str << _line_width << " setlinewidth\n";
@@ -378,7 +405,9 @@ long WSDpsdraw::drawFillRect(short x,short y,WSCushort w,WSCushort h){
   }
   WSCstring str;
   putcolor(_fg_color,&str);
-  str << "gsave newpath\n";
+  str << "gsave\n";
+  if( _clipping )putregion( &str );
+  str << "newpath\n";
   str << x << " " << y << " moveto " << x+w << " " << y << " lineto ";
   str << x+w << " " << y+h << " lineto " << x << " " << y+h << " lineto\n";
   if ( _fill_type == 0){
@@ -409,7 +438,7 @@ long WSDpsdraw::drawFillRects(WSCrect* rects,long num){
   }
   long i;
   for(i=0; i<num; i++){
-    drawRect(rects[i].x,rects[i].y,rects[i].width,rects[i].height);
+    drawFillRect(rects[i].x,rects[i].y,rects[i].width,rects[i].height);
   }
   return WS_NO_ERR;
 }
@@ -441,7 +470,9 @@ long WSDpsdraw::drawFillPoly(WSCpoint* points,long num){
   long i = 0;
   WSCstring str;
   putcolor(_fg_color,&str);
-  str << "gsave newpath\n";
+  str << "gsave\n";
+  if( _clipping )putregion(&str);
+  str << "newpath\n";
   str << points[0].x << " " << points[0].y << " moveto\n";
   for(i=1; i<num;i++){
     str << points[i].x << " " << points[i].y << " lineto\n";
@@ -473,293 +504,351 @@ long WSDpsdraw::drawFillPoly(WSCpoint* points,long num){
   return WS_NO_ERR;
 }
 
-long WSDpsdraw::drawImage(short x,short y,WSCushort w,WSCushort h,WSDimage* image,char align){
-
-  if (_file == NULL){
-    return WS_ERR;
-  }
-  WSCushort pw = image->getImageWidth();
-  WSCushort ph = image->getImageHeight();
-  short xoff = 0;
-  short yoff = 0;
-  if (align == WS_CENTER){
-    xoff = ((short)w - (short)pw)/2;
-    yoff = ((short)h - (short)ph)/2;
-  }else
-  if (align == WS_LEFT){
-    yoff = ((short)h - (short)ph)/2;
-  }else
-  if (align == WS_LEFT_TOP){
-    xoff = 0;
-    yoff = 0;
-  }else
-  if (align == WS_TOP){
-    xoff = ((short)w - (short)pw)/2;
-    yoff = 0;
-  }else
-  if (align == WS_RIGHT){
-    xoff = ((short)w - (short)pw);
-    yoff = ((short)h - (short)ph)/2;
-  }else
-  if (align == WS_BOTTOM){
-    xoff = ((short)w - (short)pw)/2;
-    yoff = ((short)h - (short)ph);
-  }else
-  if (align == WS_LEFT_BOTTOM){
-    xoff = 0;
-    yoff = ((short)h - (short)ph);
-  }else
-  if (align == WS_RIGHT_TOP){
-    xoff = ((short)w - (short)pw);
-    yoff = 0;
-  }else
-  if (align == WS_RIGHT_BOTTOM){
-    xoff = ((short)w - (short)pw);
-    yoff = ((short)h - (short)ph);
-  }
-
-  WSClistData rlist[CBUF_MAX];
-  WSClistData glist[CBUF_MAX];
-  WSClistData blist[CBUF_MAX];
-  long ccnt_a[CBUF_MAX];
-  long ccnt = 0;
-  long i;
-  for(i=0; i<CBUF_MAX; i++){
-    ccnt_a[i] = 0;
-  }
+long
+psmakemdev( WSDimage *image, WSCbool stretch, short xoff, short yoff, WSCushort pw, WSCushort ph,
+	   WSCushort w, WSCushort h, WSDmwindowDev **mdevret, unsigned char **maskbufret ){
   WSDmwindowDev* mdev = WSDmwindowDev::getNewInstance();
   if (mdev == NULL){
     return WS_ERR;
   }
-//printf("WSDpsdraw::drawImage this=0x%x\n",this);
+
+  short cno_black=WSGIappColorSet()->getColorNo("#000000");
+  short cno_white=WSGIappColorSet()->getColorNo("#FFFFFF");
+
+  WSCushort wb=(w+8-1)/8;
+  unsigned char *maskbuf=new unsigned char [wb*h];
+  unsigned char *img1=new unsigned char [w*h*3];
+
+  long x1,y1;
+
   WSDdraw* bk = WSDdev::getDraw();
   WSDdev::setDraw(WSDdev::getNativeDraw());
-
   mdev->createPixmap(w,h);
   mdev->beginDraw(0,0,w,h);
-  mdev->setForeColor(_bg_cno);
-  mdev->drawRect(0,0,w,h);
-  mdev->drawImage(xoff,yoff,w,h,image,WS_LEFT_TOP);
+  mdev->setForeColor(cno_black);
+  mdev->drawFillRect(0,0,w,h);
+  if( stretch ){
+      mdev->drawStretchedImage(0,0,w,h,image);
+  }else{
+      mdev->drawImage(xoff,yoff,pw,ph,image,WS_LEFT_TOP);
+  }
+  mdev->endDraw();
+  mdev->initBuffer();
+  for( y1=0 ; y1<h ; y1++ ){
+      for( x1=0 ; x1<w ; x1++ ){
+	  mdev->getBufferRGB(x1,y1,
+	     &img1[(y1*w+x1)*3],&img1[(y1*w+x1)*3+1],&img1[(y1*w+x1)*3+2]);
+      }
+  }
+  mdev->beginDraw(0,0,w,h);
+  mdev->setForeColor(cno_white);
+  mdev->drawFillRect(0,0,w,h);
+  if( stretch ){
+      mdev->drawStretchedImage(0,0,w,h,image);
+  }else{
+      mdev->drawImage(xoff,yoff,pw,ph,image,WS_LEFT_TOP);
+  }
   mdev->endDraw();
   mdev->initBuffer();
   WSDdev::setDraw(bk);
 
-  WSCulong* buf = new WSCulong[w*h];
-
-  long x1,y1;
-  for(x1=0; x1<w; x1++){
-    for(y1=0; y1<h; y1++){
-      WSCuchar r = 0,g = 0,b = 0;
-      mdev->getBufferRGB(x1,y1,&r,&g,&b);
-      r &=0xe0;
-      g &=0xe0;
-      b &=0xe0;
-      long hash = (r+g+b)&0xff;
-      long fl = 0;
-      long c;
-      for(c=0; c<ccnt_a[hash]; c++){
-        WSCulong r1 = (WSCulong)(rlist[hash])[c];
-        WSCulong g1 = (WSCulong)(glist[hash])[c];
-        WSCulong b1 = (WSCulong)(blist[hash])[c];
-        if ((WSCuchar)r1 == r && (WSCuchar)g1 == g && (WSCuchar)b1 == b){
-          fl = 1;
-          buf[y1*w + x1] = c*256 + hash;
-          break;
-        }
+  // get mask bitmap : this will be failed when really random dithering when drawImage
+  // currentlly no plathome independent method to know whether mask is valid or not and also no
+  // method to get mask data directly.
+  WSCbool needmask=False;
+  for( y1=0 ; y1<h ; y1++ ){
+      for( x1=0 ; x1<wb ; x1++ )maskbuf[y1*wb+x1]=0;
+      for( x1=0 ; x1<w ; x1++ ){
+	  WSCuchar r = 0,g = 0,b = 0;
+	  mdev->getBufferRGB(x1,y1, &r, &g, &b );
+	  if( img1[(y1*w+x1)*3]==r && img1[(y1*w+x1)*3+1]==g && img1[(y1*w+x1)*3+2]==b ){
+	      maskbuf[y1*wb+(x1/8)]|=1<<(7-(x1%8));
+	  }else{
+	      needmask=True;
+	  }
       }
-      if (fl == 0){
-        WSCulong r1 = r;
-        WSCulong g1 = g;
-        WSCulong b1 = b;
-        rlist[hash].add((void*)r1);
-        glist[hash].add((void*)g1);
-        blist[hash].add((void*)b1);
-        buf[y1*w + x1] = ccnt_a[hash] * 256 + hash;
-        ccnt++;
-        ccnt_a[hash]++;
-      }
-    }
   }
-
-  WSCstring str;
-  str << "gsave\n";
-  str << "3 " << ccnt << " mul array ws_setpixels\n";
-
-  _file->write((char*)str,strlen((char*)str));
-  long j;
-  WSCulong cnt = 0;
-  for(i=0; i<CBUF_MAX; i++){
-    for(j=0; j<ccnt_a[i]; j++){
-      WSCulong r = (WSCulong)(rlist[i])[j];
-      WSCulong g = (WSCulong)(glist[i])[j];
-      WSCulong b = (WSCulong)(blist[i])[j];
-      char buffer[256];
-      sprintf(buffer,"%d [ %lf %lf %lf ] ws_setpix\n",3*cnt,
-           (double)r/256,(double)g/256,(double)b/256);
-      _file->write(buffer,strlen(buffer));
-      (rlist[i]).setData(j,(void*)cnt);
-      cnt++;
-    }
+  delete [] img1;
+  if( ! needmask ){
+      delete [] maskbuf;
+      maskbuf=NULL;
   }
-  long bit = 8;
-  if (cnt > 256){
-    bit = 32; 
-  }
-  str = "";
-  str << x << " " << y << " translate 1.00 1.00 scale 0 rotate\n";
-  str << w << " " << h << " " << bit << "\n[1 0 0 1 0 0]\n";
-  str << "{currentfile ws_bwpicstr readhexstring pop 0 get ws_colorspot}\n";
-  str << "false 3 colorimage\n";
-  _file->write((char*)str,strlen(str));
-//printf("here2\n");
-  for(y1=0; y1<h; y1++){
-    for(x1=0; x1<w; x1++){
-      WSCulong val =  buf[y1*w + x1] / 256;
-      WSCulong hash =  buf[y1*w + x1] & 0xff;
-      WSCulong num = (WSCulong)(rlist[hash])[val];
-      char tmp[32];
-      if (bit == 8){
-        sprintf(tmp,"%02x",num);
-      }else{
-        sprintf(tmp,"%04x",num);
-      }
-      _file->write((char*)tmp,strlen(tmp));
-    }
-    _file->write((char*)"\n",strlen("\n"));
-  }
-  str = "";
-  str << "grestore\n";
-  _file->write((char*)str,strlen((char*)str));
-
-  delete buf;
+  *mdevret=mdev;
+  *maskbufret=maskbuf;
   return WS_NO_ERR;
 }
+
+void
+psoutimage( WSDmwindowDev *mdev, WSDfile *file, WSCbool clipping,
+	   short x, short y, WSCushort w, WSCushort h,
+	   short dx, short dy, unsigned char *maskbuf ){
+    WSClistData rlist[CBUF_MAX];
+    WSClistData glist[CBUF_MAX];
+    WSClistData blist[CBUF_MAX];
+    long ccnt_a[CBUF_MAX];
+    long ccnt = 0;
+    long i;
+    for(i=0; i<CBUF_MAX; i++){
+	ccnt_a[i] = 0;
+    }
+
+    WSCulong* buf = new WSCulong[w*h];
+
+    long x1,y1;
+    for(x1=0; x1<w; x1++){
+	for(y1=0; y1<h; y1++){
+	    WSCuchar r = 0,g = 0,b = 0;
+	    mdev->getBufferRGB(x+x1,y+y1,&r,&g,&b);
+	    r &=0xe0;
+	    g &=0xe0;
+	    b &=0xe0;
+	    long hash = (r+g+b)&0xff;
+	    long fl = 0;
+	    long c;
+	    for(c=0; c<ccnt_a[hash]; c++){
+		WSCulong r1 = (WSCulong)(rlist[hash])[c];
+		WSCulong g1 = (WSCulong)(glist[hash])[c];
+		WSCulong b1 = (WSCulong)(blist[hash])[c];
+		if ((WSCuchar)r1 == r && (WSCuchar)g1 == g && (WSCuchar)b1 == b){
+		    fl = 1;
+		    buf[y1*w + x1] = c*256 + hash;
+		    break;
+		}
+	    }
+	    if (fl == 0){
+		WSCulong r1 = r;
+		WSCulong g1 = g;
+		WSCulong b1 = b;
+		rlist[hash].add((void*)r1);
+		glist[hash].add((void*)g1);
+		blist[hash].add((void*)b1);
+		buf[y1*w + x1] = ccnt_a[hash] * 256 + hash;
+		ccnt++;
+		ccnt_a[hash]++;
+	    }
+	}
+    }
+
+    WSCstring str;
+    str << "gsave\n";
+    if( clipping )putregion(&str);
+    str << "3 " << ccnt << " mul array ws_setpixels\n";
+
+    file->write((char*)str,strlen((char*)str));
+    long j;
+    WSCulong cnt = 0;
+    for(i=0; i<CBUF_MAX; i++){
+	for(j=0; j<ccnt_a[i]; j++){
+	    WSCulong r = (WSCulong)(rlist[i])[j];
+	    WSCulong g = (WSCulong)(glist[i])[j];
+	    WSCulong b = (WSCulong)(blist[i])[j];
+	    char buffer[256];
+	    sprintf(buffer,"%d [ %lf %lf %lf ] ws_setpix\n",3*cnt,
+		    (double)r/256,(double)g/256,(double)b/256);
+	    file->write(buffer,strlen(buffer));
+	    (rlist[i]).setData(j,(void*)cnt);
+	    cnt++;
+	}
+    }
+    long bit = 8;
+    if (cnt > 256){
+	bit = 32; 
+    }
+    str = "";
+    str << dx << " " << dy << " translate 1.00 1.00 scale 0 rotate\n";
+    str << w << " " << h ;
+    if( maskbuf ){
+	str << " ws_maskedimage\n";
+    }else{
+	str << " ws_unmaskedimage\n";
+    }
+    file->write((char*)str,strlen(str));
+    for(y1=0; y1<h; y1++){
+	for(x1=0; x1<w; x1++){
+	    WSCulong val =  buf[y1*w + x1] / 256;
+	    WSCulong hash =  buf[y1*w + x1] & 0xff;
+	    WSCulong num = (WSCulong)(rlist[hash])[val];
+	    char tmp[32];
+	    if (bit == 8){
+		sprintf(tmp,"%02x",num);
+	    }else{
+		sprintf(tmp,"%04x",num);
+	    }
+	    file->write((char*)tmp,strlen(tmp));
+	}
+	file->write((char*)"\n",strlen("\n"));
+    }
+
+    if( maskbuf ){
+	WSCushort wb=(w+8-1)/8;
+	for(y1=0; y1<h; y1++){
+	    char tmp[32];
+	    for(x1=0; x1<wb; x1++){
+		sprintf( tmp, "%02x", maskbuf[y1*wb+x1] );
+		file->write((char*)tmp,strlen(tmp));
+	    }
+	    file->write((char*)"\n",strlen("\n"));
+	}
+    }
+
+    str = "";
+    str << "grestore\n";
+    file->write((char*)str,strlen((char*)str));
+    delete buf;
+}
+
+long WSDpsdraw::drawImage(short x,short y,WSCushort w,WSCushort h,WSDimage* image,char align){
+  if (_file == NULL){
+    return WS_ERR;
+  }
+  if (image == NULL){
+      return WS_ERR;
+  }
+  if (image->getValue1() == 0 ||
+      image->getValue1() == -1 ){
+      return WS_ERR;
+  }
+
+  WSCushort pw = image->getImageWidth();
+  WSCushort ph = image->getImageHeight();
+
+  WSCushort mw = w ;
+  WSCushort mh = h ;
+
+  short xoff = 0;
+  short yoff = 0;
+  short mxoff = 0 ;
+  short myoff = 0 ;
+
+  if (align == WS_CENTER){
+      xoff = ((short)w - (short)pw)/2;
+      yoff = ((short)h - (short)ph)/2;
+  }else
+  if (align == WS_LEFT){
+      yoff = ((short)h - (short)ph)/2;
+  }else
+  if (align == WS_LEFT_TOP){
+      xoff = 0;
+      yoff = 0;
+  }else
+  if (align == WS_TOP){
+      xoff = ((short)w - (short)pw)/2;
+      yoff = 0;
+  }else
+  if (align == WS_RIGHT){
+      xoff = ((short)w - (short)pw);
+      yoff = ((short)h - (short)ph)/2;
+  }else
+  if (align == WS_BOTTOM){
+      xoff = ((short)w - (short)pw)/2;
+      yoff = ((short)h - (short)ph);
+  }else
+  if (align == WS_LEFT_BOTTOM){
+      xoff = 0;
+      yoff = ((short)h - (short)ph);
+  }else
+  if (align == WS_RIGHT_TOP){
+      xoff = ((short)w - (short)pw);
+      yoff = 0;
+  }else
+  if (align == WS_RIGHT_BOTTOM){
+      xoff = ((short)w - (short)pw);
+      yoff = ((short)h - (short)ph);
+  }
+  if( mw>pw ){
+      mw=pw;
+      mxoff=xoff;
+  }
+  if( mh>ph ){
+      mh=ph;
+      myoff=yoff;
+  }
+
+  WSDmwindowDev *mdev=0;
+  unsigned char *maskbuf=0;
+
+  if( psmakemdev( image, False, xoff-mxoff, yoff-myoff, pw, ph,
+		 mw, mh, &mdev, &maskbuf )==WS_ERR ){
+      return WS_ERR;
+  }
+
+  psoutimage( mdev, _file, _clipping, 0, 0, mw, mh, x+mxoff, y+myoff, maskbuf );
+
+  if( maskbuf )delete [] maskbuf;
+  if( mdev )delete mdev;
+
+  return WS_NO_ERR;
+}
+
 long WSDpsdraw::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDimage* image){
   if (_file == NULL){
     return WS_ERR;
   }
   if (image == NULL){
-    return WS_ERR;
+      return WS_ERR;
   }
   if (image->getValue1() == 0 ||
       image->getValue1() == -1 ){
-    return WS_ERR;
-  }
-  WSClistData rlist[CBUF_MAX];
-  WSClistData glist[CBUF_MAX];
-  WSClistData blist[CBUF_MAX];
-  long ccnt_a[CBUF_MAX];
-  long ccnt = 0;
-  long i;
-  for(i=0; i<CBUF_MAX; i++){
-    ccnt_a[i] = 0;
-  }
-  WSDmwindowDev* mdev = WSDmwindowDev::getNewInstance();
-  if (mdev == NULL){
-    return WS_ERR;
-  }
-//printf("WSDpsdraw::drawImage this=0x%x\n",this);
-  WSDdraw* bk = WSDdev::getDraw();
-  WSDdev::setDraw(WSDdev::getNativeDraw());
-
-  mdev->createPixmap(w,h);
-  mdev->beginDraw(0,0,w,h);
-  mdev->drawStretchedImage(0,0,w,h,image);
-  mdev->endDraw();
-  mdev->initBuffer();
-  WSDdev::setDraw(bk);
-
-  WSCulong* buf = new WSCulong[w*h];
-
-  long x1,y1;
-  for(x1=0; x1<w; x1++){
-    for(y1=0; y1<h; y1++){
-      WSCuchar r = 0,g = 0,b = 0;
-      mdev->getBufferRGB(x1,y1,&r,&g,&b);
-      r &=0xe0;
-      g &=0xe0;
-      b &=0xe0;
-      long hash = (r+g+b)&0xff;
-      long fl = 0;
-      long c;
-      for(c=0; c<ccnt_a[hash]; c++){
-        WSCulong r1 = (WSCulong)(rlist[hash])[c];
-        WSCulong g1 = (WSCulong)(glist[hash])[c];
-        WSCulong b1 = (WSCulong)(blist[hash])[c];
-        if ((WSCuchar)r1 == r && (WSCuchar)g1 == g && (WSCuchar)b1 == b){
-          fl = 1;
-          buf[y1*w + x1] = c*256 + hash;
-          break;
-        }
-      }
-      if (fl == 0){
-        WSCulong r1 = r;
-        WSCulong g1 = g;
-        WSCulong b1 = b;
-        rlist[hash].add((void*)r1);
-        glist[hash].add((void*)g1);
-        blist[hash].add((void*)b1);
-        buf[y1*w + x1] = ccnt_a[hash] * 256 + hash;
-        ccnt++;
-        ccnt_a[hash]++;
-      }
-    }
+      return WS_ERR;
   }
 
-  WSCstring str;
-  str << "gsave\n";
-  str << "3 " << ccnt << " mul array ws_setpixels\n";
+  WSDmwindowDev *mdev;
+  unsigned char *maskbuf;
 
-  _file->write((char*)str,strlen((char*)str));
-  long j;
-  WSCulong cnt = 0;
-  for(i=0; i<CBUF_MAX; i++){
-    for(j=0; j<ccnt_a[i]; j++){
-      WSCulong r = (WSCulong)(rlist[i])[j];
-      WSCulong g = (WSCulong)(glist[i])[j];
-      WSCulong b = (WSCulong)(blist[i])[j];
-      char buffer[256];
-      sprintf(buffer,"%d [ %lf %lf %lf ] ws_setpix\n",3*cnt,
-           (double)r/256,(double)g/256,(double)b/256);
-      _file->write(buffer,strlen(buffer));
-      (rlist[i]).setData(j,(void*)cnt);
-      cnt++;
-    }
+  if( psmakemdev( image, True, 0, 0, 0, 0, w, h, &mdev, &maskbuf )==WS_ERR ){
+      return WS_ERR;
   }
-  long bit = 8;
-  if (cnt > 256){
-    bit = 32; 
-  }
-  str = "";
-  str << x << " " << y << " translate 1.00 1.00 scale 0 rotate\n";
-  str << w << " " << h << " " << bit << "\n[1 0 0 1 0 0]\n";
-  str << "{currentfile ws_bwpicstr readhexstring pop 0 get ws_colorspot}\n";
-  str << "false 3 colorimage\n";
-  _file->write((char*)str,strlen(str));
-//printf("here2\n");
-  for(y1=0; y1<h; y1++){
-    for(x1=0; x1<w; x1++){
-      WSCulong val =  buf[y1*w + x1] / 256;
-      WSCulong hash =  buf[y1*w + x1] & 0xff;
-      WSCulong num = (WSCulong)(rlist[hash])[val];
-      char tmp[32];
-      if (bit == 8){
-        sprintf(tmp,"%02x",num);
-      }else{
-        sprintf(tmp,"%04x",num);
-      }
-      _file->write((char*)tmp,strlen(tmp));
-    }
-    _file->write((char*)"\n",strlen("\n"));
-  }
-  str = "";
-  str << "grestore\n";
-  _file->write((char*)str,strlen((char*)str));
 
-  delete buf;
+  psoutimage( mdev, _file, _clipping, 0, 0, w, h, x, y, maskbuf );
+
+  if( maskbuf )delete [] maskbuf;
+  if( mdev )delete mdev;
+
   return WS_NO_ERR;
 }
 
+long
+WSDpsdraw::copyPixmap( WSDmwindowDev *mdev, short x, short y,
+		        WSCushort w, WSCushort h, short dx, short dy, WSDmwindowDev *maskmdev ){
+    if (mdev == NULL){
+	return WS_ERR;
+    }
+    if (_file == NULL){
+	return WS_ERR;
+    }
+
+    unsigned char *maskbuf=NULL;
+    if( maskmdev ){
+	WSCushort wb=(w+8-1)/8;
+	maskbuf=new unsigned char [wb*h];
+
+	long x1,y1;
+
+	WSCbool needmask=False;
+	for(y1=0; y1<h; y1++){
+	    for( x1=0; x1<wb ; x1++ )maskbuf[y1*wb+x1]=0;
+	    for( x1=0; x1<w; x1++ ){
+		WSCuchar r = 0,g = 0,b = 0;
+		maskmdev->getBufferRGB(x+x1,y+y1,&r,&g,&b);
+		if( r || g || b ){
+		    maskbuf[y1*wb+(x1/8)]|=1<<(7-(x1%8));
+		}else{
+		    needmask=True;
+		}
+	    }
+	}
+	if( ! needmask ){
+	    delete [] maskbuf;
+	    maskbuf=NULL;
+	}
+    }
+
+    psoutimage( mdev, _file, _clipping, x, y, w, h, dx, dy, maskbuf );
+
+    if( maskbuf )delete [] maskbuf;
+
+    return WS_NO_ERR;
+}
 
 long WSDpsdraw::drawImage(short x,short y,short sx,short sy,WSCushort w,WSCushort h,WSDimage* image){
   if (_file == NULL){
@@ -811,66 +900,183 @@ long WSDpsdraw::drawString(long x,long y,WSCulong w,WSCulong h,
   }
   long fheight = fn->getFontHeight();
 
+  if (str->isExist("\n") == -1){
+    // single line
+    WSCrect   area;
+    area.setRect(x,y,w,h);
+    WSCpoint  point;
+    WSGFcalcOneLineStringAddr(str,&area,align,fn,&point);
+    long swidth=fn->getStringWidth(str);
+
+    WSCstring str1;
+    putcolor(_fg_color,&str1);
+    str1 <<"/Times-Roman-Ryumin-Light-EUC-H /Times-Roman /Ryumin-Light-EUC-H eucfont\n/Times-Roman-Ryumin-Light-EUC-H findfont [";
+
+    str1 << fheight << " 0 0 -";
+    str1 << fheight << " 0 0] ws_makefont setfont\ngsave\n";
+    if( _clipping )putregion(&str1);
+    putregion(x,y,w,h,&str1);
+    str1 << point.x << " " << point.y+fheight << " moveto (";
+
+    WSCstring tmp1;
+    if (WSGIappLocaleSet()->getSystemLocaleEncoding() == WS_EN_EUCJP){
+      char* ptr = str->getString(WS_EN_EUCJP);
+      long num = strlen(ptr);
+      long i;
+      char buffer[16];
+      for(i=0; i<num; i++){
+        if ((unsigned char)ptr[i] & 0x80 ){
+          sprintf(buffer,"\\%o",(unsigned char)(ptr[i]));
+        }else
+        if ( ptr[i]=='(' || ptr[i]==')' || ptr[i]=='\\' ){
+          sprintf(buffer,"\\%c",ptr[i]);
+        }else{
+          buffer[0] = ptr[i];
+          buffer[1] = 0;
+        }
+        tmp1 << buffer;
+      }
+    }else
+    if (WSGIappLocaleSet()->getSystemLocaleEncoding() == WS_EN_SJIS){
+      char* ptr = str->getString(WS_EN_EUCJP);
+      long num = strlen(ptr);
+      long i;
+      char buffer[16];
+      for(i=0; i<num; i++){
+        if ((unsigned char)ptr[i] & 0x80){
+          sprintf(buffer,"\\%o",(unsigned char)(ptr[i]));
+        }else
+        if ( ptr[i]=='(' || ptr[i]==')' || ptr[i]=='\\' ){
+          sprintf(buffer,"\\%c",ptr[i]);
+        }else{
+          buffer[0] = ptr[i];
+          buffer[1] = 0;
+        }
+        tmp1 << buffer;
+      }
+    }else{
+      char* ptr = str->getString();
+      long num = strlen(ptr);
+      long i;
+      char buffer[16];
+      for(i=0; i<num; i++){
+        if ((unsigned char)ptr[i] & 0x80){
+          sprintf(buffer,"\\%o",(unsigned char)(ptr[i]));
+        }else
+        if ( ptr[i]=='(' || ptr[i]==')' || ptr[i]=='\\' ){
+          sprintf(buffer,"\\%c",ptr[i]);
+        }else{
+          buffer[0] = ptr[i];
+          buffer[1] = 0;
+        }
+        tmp1 << buffer;
+      }
+    }
+
+    str1 << tmp1;
+    str1 << ") " << swidth << " " << fheight << " ws_whshow\ngrestore\n";
+
+    _file->write((char*)str1,strlen((char*)str1));
+
+    return WS_NO_ERR;
+  }
+  // multiline
+
+  long num;
+  WSCrect area;
+  area.setRect(x,y,w,h);
+  WSCpoint* points;
+
+  WSGFcalcStringAddr(str,&area,align,fn,&points,&num);
+  long i;
+  str->seek(0);
+  long lpos =0;
+  long prelpos =0;
 
   WSCstring str1;
   putcolor(_fg_color,&str1);
   str1 <<"/Times-Roman-Ryumin-Light-EUC-H /Times-Roman /Ryumin-Light-EUC-H eucfont\n/Times-Roman-Ryumin-Light-EUC-H findfont [";
-
   str1 << fheight << " 0 0 -";
-  str1 << fheight << " 0 0] makefont setfont\ngsave\n";
-  str1 << x << " " << y+fheight << " moveto (";
+  str1 << fheight << " 0 0] ws_makefont setfont\ngsave\n";
+  if( _clipping )putregion(&str1);
+  putregion(x,y,w,h,&str1);
 
-  WSCstring tmp1;
-  if (WSGIappLocaleSet()->getSystemLocaleEncoding() == WS_EN_EUCJP){
-    char* ptr = str->getString(WS_EN_EUCJP);
-    long num = strlen(ptr);
-    long i;
-    char buffer[16];
-    for(i=0; i<num; i++){
-      if ((unsigned char)ptr[i] & 0x80){
-        sprintf(buffer,"\\%o",(unsigned char)(ptr[i]));
-      }else{
-        buffer[0] = ptr[i];
-        buffer[1] = 0;
+  for(i=0; i < num; i++){
+    WSCstring lstr;
+    lstr = str->gets();
+
+    long swidth=fn->getStringWidth(&lstr);
+
+    lpos += prelpos;
+    prelpos = lstr.getChars() +1;
+    str1 << points[i].x << " " << points[i].y+fheight << " moveto (";
+
+    WSCstring tmp1;
+    if (WSGIappLocaleSet()->getSystemLocaleEncoding() == WS_EN_EUCJP){
+      char* ptr = lstr.getString(WS_EN_EUCJP);
+      long num = strlen(ptr);
+      long i;
+      char buffer[16];
+      for(i=0; i<num; i++){
+        if ((unsigned char)ptr[i] & 0x80){
+          sprintf(buffer,"\\%o",(unsigned char)(ptr[i]));
+        }else
+        if ( ptr[i]=='(' || ptr[i]==')' || ptr[i]=='\\' ){
+          sprintf(buffer,"\\%c",ptr[i]);
+        }else{
+          buffer[0] = ptr[i];
+          buffer[1] = 0;
+        }
+        tmp1 << buffer;
       }
-      tmp1 << buffer;
-    }
-  }else
-  if (WSGIappLocaleSet()->getSystemLocaleEncoding() == WS_EN_SJIS){
-    char* ptr = str->getString(WS_EN_EUCJP);
-    long num = strlen(ptr);
-    long i;
-    char buffer[16];
-    for(i=0; i<num; i++){
-      if ((unsigned char)ptr[i] & 0x80){
-        sprintf(buffer,"\\%o",(unsigned char)(ptr[i]));
-      }else{
-        buffer[0] = ptr[i];
-        buffer[1] = 0;
+    }else
+    if (WSGIappLocaleSet()->getSystemLocaleEncoding() == WS_EN_SJIS){
+      char* ptr = lstr.getString(WS_EN_EUCJP);
+      long num = strlen(ptr);
+      long i;
+      char buffer[16];
+      for(i=0; i<num; i++){
+        if ((unsigned char)ptr[i] & 0x80){
+          sprintf(buffer,"\\%o",(unsigned char)(ptr[i]));
+        }else
+        if ( ptr[i]=='(' || ptr[i]==')' || ptr[i]=='\\' ){
+          sprintf(buffer,"\\%c",ptr[i]);
+        }else{
+          buffer[0] = ptr[i];
+          buffer[1] = 0;
+        }
+        tmp1 << buffer;
       }
-      tmp1 << buffer;
-    }
-  }else{
-    char* ptr = str->getString();
-    long num = strlen(ptr);
-    long i;
-    char buffer[16];
-    for(i=0; i<num; i++){
-      if ((unsigned char)ptr[i] & 0x80){
-        sprintf(buffer,"\\%o",(unsigned char)(ptr[i]));
-      }else{
-        buffer[0] = ptr[i];
-        buffer[1] = 0;
+    }else{
+      char* ptr = lstr.getString();
+      long num = strlen(ptr);
+      long i;
+      char buffer[16];
+      for(i=0; i<num; i++){
+        if ((unsigned char)ptr[i] & 0x80){
+          sprintf(buffer,"\\%o",(unsigned char)(ptr[i]));
+        }else
+        if ( ptr[i]=='(' || ptr[i]==')' || ptr[i]=='\\' ){
+          sprintf(buffer,"\\%c",ptr[i]);
+        }else{
+          buffer[0] = ptr[i];
+          buffer[1] = 0;
+        }
+        tmp1 << buffer;
       }
-      tmp1 << ptr;
     }
+
+    str1 << tmp1;
+    str1 << ") " << swidth << " " << fheight << " ws_whshow\n";
   }
-
-  str1 << tmp1;
-  str1 << ") show\ngrestore\n";
+  str1 << "grestore\n";
 
   _file->write((char*)str1,strlen((char*)str1));
 
+  str->seek(0);
+  if (num != 0){
+    delete[] points;
+  }
   return WS_NO_ERR;
 }
 
@@ -989,9 +1195,9 @@ long WSDpsdraw::outputHeader(){
       scaley = 1.0;
     }
     str << scalex << " " << scaley << " scale\n";
-    str << _work_x * _work_width / pointw << " "
-        << _work_y * _work_height / pointh << " translate\n";
-
+    str << -_work_x << " "
+        << -_work_y << " translate\n";
+    putregion( _work_x, _work_y, _work_width, _work_height, &str);
   }else{
     str << " 90 rotate\n";
     str << " 72 0 mul 72 0 mul translate\n";
@@ -1059,9 +1265,9 @@ long WSDpsdraw::outputHeader(){
     }
 
     str << scalex << " " << scaley << " scale\n";
-    str << _work_x * _work_width / pointw << " "
-        << _work_y * _work_height / pointh << " translate\n";
-
+    str << -_work_x << " "
+        << -_work_y << " translate\n";
+    putregion( _work_x, _work_y, _work_width, _work_height, &str);
   }
   _file->write((char*)str,strlen((char*)str));
   _file->write((char*)ws_ps_header2,strlen((char*)ws_ps_header2));
@@ -1077,7 +1283,7 @@ long WSDpsdraw::outputFooter(){
   return WS_NO_ERR;
 }
 
-char* ws_ps_header = ""
+char* ws_ps_header = "%!\n"
 "/ws_dict 6 dict def\n"
 "ws_dict begin\n"
 "/copyfont {\n"
@@ -1090,6 +1296,32 @@ char* ws_ps_header = ""
 "    end\n"
 "} bind def\n"
 "/eucfont {\n"
+"  dup type /nametype eq { findfont } if\n"
+"  dup /FMapType get 9 eq {\n"
+"  dup /CMap get /CMapName get exch\n"
+"  dup /FDepVector get 0 get\n"
+"  dup /CIDSystemInfo get 3 dict copy\n"
+"  /CIDInit /ProcSet findresource begin\n"
+"    12 dict begin\n"
+"      begincmap\n"
+"        4 -1 roll usecmap\n"
+"        [ exch null ] /CIDSystemInfo exch def\n"
+"        /CMapName /ws_EUCFontCMap def\n"
+"        /CMapVersion 1 def\n"
+"        /CMapType 1 def\n"
+"        /WMode 0 def\n"
+"        1 usefont\n"
+"        1 beginbfrange\n"
+"          <20> <7F> <20>\n"
+"        endbfrange\n"
+"        endcmap\n"
+"        currentdict CMapName exch /CMap defineresource pop\n"
+"      end\n"
+"    end\n"
+"    exch /kanjifont exch definefont pop\n"
+"    exch [ 3 1 roll ]\n"
+"    /ws_EUCFontCMap exch composefont pop\n"
+"  }{\n"
 "    12 dict begin\n"
 "        dup type /nametype eq { findfont } if\n"
 "        dup /WMode known {\n"
@@ -1124,6 +1356,43 @@ char* ws_ps_header = ""
 "        currentdict\n"
 "    end\n"
 "    definefont pop\n"
+"  } ifelse\n"
+"} def\n"
+"\n"
+"% assuming a3>0 lly<=0\n"
+"% font - num\n"
+"/ws_fontDescent {\n"
+"    dup /FontType get 0 ne {\n"
+"	dup /FontBBox get 1 get	exch\n"
+"	/FontMatrix get 3 get\n"
+"	mul neg\n"
+"    } {\n"
+"	dup /FDepVector get\n"
+"	0 exch {\n"
+"	    ws_fontDescent\n"
+"	    dup 2 index gt { exch } if pop\n"
+"	} forall\n"
+"	exch /FontMatrix get 3 get mul\n"
+"    } ifelse\n"
+"} def\n"
+"\n"
+"% font array - font\n"
+"/ws_makefont {\n"
+"    exch dup ws_fontDescent\n"
+"    2 index 3 get mul\n"
+"    [ 4 -1 roll aload pop 8 -1 roll add ]\n"
+"    makefont\n"
+"} def\n"
+"\n"
+"/ws_whshow {\n"
+"  gsave currentpoint translate\n"
+"  2 index stringwidth pop dup 0 eq { pop pop pop pop } { dup\n"
+"  4 -1 roll exch div 3 1 roll\n"
+"  newpath 0 0 moveto 3 index true charpath pathbbox newpath\n"
+"  pop 3 -1 roll pop 3 -1 roll sub neg\n"
+"  dup 0 ge { add } { pop } ifelse\n"
+"  2 copy ge { pop pop 1.0 } { div } ifelse\n"
+"  scale 0 0 moveto show } ifelse grestore\n"
 "} def\n"
 "\n"
 "/ws_ellipsedict 6 dict def\n"
@@ -1293,6 +1562,66 @@ char* ws_ps_header = ""
 "   end\n"
 " } def\n"
 "\n"
+#if PSDRAW_SAVE_REGION_CTM
+"/ws_regiondict 5 dict def\n"
+#else
+"/ws_regiondict 4 dict def\n"
+#endif
+"/ws_setregion {\n"
+"    ws_regiondict begin\n"
+"    /_clip_h exch def\n"
+"    /_clip_w exch def\n"
+"    /_clip_y exch def\n"
+"    /_clip_x exch def\n"
+#if PSDRAW_SAVE_REGION_CTM
+"    /_clip_CTM matrix currentmatrix def\n"
+#endif
+"    end\n"
+"} def\n"
+"\n"
+"/ws_cliprect {\n"
+"    newpath 4 2 roll 2 copy moveto\n"
+"    exch 3 index add exch 2 copy lineto\n"
+"    3 -1 roll add 2 copy lineto\n"
+"    3 1 roll sub neg exch lineto closepath\n"
+"    clip newpath\n"
+"} def\n"
+"\n"
+"/ws_clipregion {\n"
+"    ws_regiondict begin\n"
+"    currentdict /_clip_x known {\n"
+#if PSDRAW_SAVE_REGION_CTM
+"	matrix currentmatrix _clip_CTM setmatrix\n"
+#endif
+"	_clip_x _clip_y _clip_w _clip_h end\n"
+"	ws_cliprect\n"
+#if PSDRAW_SAVE_REGION_CTM
+"	setmatrix\n"
+#endif
+"    } if\n"
+"} def \n"
+"\n"
+"/ws_unmaskedimage\n"
+"{ 8 [1 0 0 1 0 0]\n"
+"{currentfile ws_bwpicstr readhexstring pop 0 get ws_colorspot}\n"
+"false 3 colorimage\n"
+"} def\n"
+"\n"
+"/ws_maskedimagedict 2 dict def\n"
+"\n"
+"/ws_maskedimage {\n"
+"ws_maskedimagedict begin /imgh exch def /imgw exch def\n"
+"<< /PatternType 1 /PaintType 1 /TilingType 1\n"
+"/BBox [ 0 0 imgw imgh ] /XStep imgw /YStep imgh\n"
+"/PaintProc { pop\n"
+"imgw imgh ws_unmaskedimage\n"
+"} >> matrix makepattern setpattern\n"
+"gsave newpath 0 0 moveto 0 1 lineto closepath clip\n"
+"newpath 2 0 moveto 2 1 lineto closepath fill\n"
+"grestore imgw imgh true [ 1 0 0 1 0 0 ]\n"
+"{ currentfile ws_bwpicstr readhexstring pop } imagemask\n"
+"end\n"
+"} def\n"
 "\n"
 "end\n"
 "\n"
@@ -1311,19 +1640,19 @@ grestore\n\
 ws_savedpage restore\n\
 end\n\
 showpage\n";
-
+#if 0 //removed..
 char* ws_ps_text2 = "\
  0 0 -";
 
 char* ws_ps_text3 = "\
- 0 0] makefont setfont\n\
+ 0 0] ws_makefont setfont\n\
    gsave\n";
 
 char* ws_ps_text4 = "\
  moveto (";
 
 char* ws_ps_text5 = "\
-) show\n\
+) ws_whshow\n\
    grestore\n";
-
-
+#endif
+#endif //NO_PSDRAW
