@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSDserialize.h>
@@ -32,36 +28,51 @@
 
 WSMFclassInit(WSDserialize,WSCroot);
 
+#define WS_SERIALIZE_SHORT_SIZE   2
+#define WS_SERIALIZE_INT_SIZE     4
+#define WS_SERIALIZE_LONG_SIZE    4
+#define WS_SERIALIZE_FLOAT_SIZE   4
+#define WS_SERIALIZE_DOUBLE_SIZE  8
+
 WSCserializeData::WSCserializeData(){
-  _data_name = NULL;
+  _data_name1 = NULL;
   _data_name_len = 0;
   _type = WS_DcLong;
   _type_name_len = 0;
-  _type_name = NULL;
+  _type_name1 = NULL;
   _data_len = 0;
-  _data = NULL;
+  _data1 = NULL;
   _cache_ptr = 0;
+  _children._seg_size = 16;
 };
 
 void WSCserializeData::clearChildren(){
-  long num = _children.getNum();
+  long num = _children._num;
+#if 1
+  if (num > 0){
+    WSCserializeData* serialize = (WSCserializeData*)_children.getData(0);
+    delete serialize;
+  }
+#endif
+#if 0
   long i;
   for(i=0; i< num; i++){
     WSCserializeData* serialize = (WSCserializeData*)_children.getData(i);
     delete serialize;
   }
+#endif
   _children.clear();
 };
 
 WSCserializeData::~WSCserializeData(){
-  if (_data != NULL){
-    delete (char*)_data;
+  if (_data1 != NULL && _data1 != _data2){
+    delete (char*)_data1;
   }
-  if (_type_name != NULL){
-    delete _type_name;
+  if (_type_name1 != NULL && _type_name1 != _type_name2){
+    delete _type_name1;
   }
-  if (_data_name != NULL){
-    delete _data_name;
+  if (_data_name1 != NULL && _data_name1 != _data_name2){
+    delete _data_name1;
   }
 
   clearChildren();
@@ -73,21 +84,21 @@ long WSCserializeData::size(){
 #else
    long ret = 1; //data_name_len
 
-   if (_data_name != NULL){
-     ret += strlen(_data_name); //data_name
+   if (_data_name1 != NULL){
+     ret += strlen(_data_name1); //data_name
    }
 
    ret += 2; //type,_type_name_len
 
-   if (_type_name != NULL){
-     ret += strlen(_type_name); //_type_name
+   if (_type_name1 != NULL){
+     ret += strlen(_type_name1); //_type_name
    }
 
    ret += 4; //_data_len
 
    ret += _data_len; //_data
 
-   long num = _children.getNum();
+   long num = _children._num;
    long i;
    for(i=0; i< num; i++){
      WSCserializeData* cdata = (WSCserializeData*)_children.getData(i);
@@ -104,15 +115,15 @@ long WSCserializeData::write(WSDserialize* serialize){
 //printf("WSCserializeData::write len=%d %d\n",sizeof(char),_data_name_len);
   serialize->_write(&_data_name_len,sizeof(char));
 //printf("WSCserializeData::write len=%d #%s#\n",_data_name_len,_data_name);
-  serialize->_write(_data_name,_data_name_len);
+  serialize->_write(_data_name1,_data_name_len);
 
   serialize->_write(&_type,sizeof(char));
   serialize->_write(&_type_name_len,sizeof(char));
 
   if (_type == WS_DcObject){
-    serialize->_write(_type_name,_type_name_len);
+    serialize->_write(_type_name1,_type_name_len);
     char tmp2[4];
-    long num = _children.getNum();
+    long num = _children._num;
     WSGIbinType()->memcpy4b(tmp2,num);
     long ret = serialize->_write(tmp2,sizeof(char)*4);
     long i;
@@ -126,7 +137,7 @@ long WSCserializeData::write(WSDserialize* serialize){
     char tmp[4];
     WSGIbinType()->memcpy4b(tmp,_data_len);
     serialize->_write(tmp,sizeof(char)*4);
-    long ret = serialize->_write(_data,_data_len);
+    long ret = serialize->_write(_data1,_data_len);
     return ret;
   }
 #endif
@@ -142,18 +153,22 @@ long WSCserializeData::read(WSDserialize* serialize){
   if (len < (long)sizeof(char)){
     return 0;
   }
-  if (_data_name != NULL){
-    delete _data_name;
-    _data_name = NULL;
+  if (_data_name1 != NULL && _data_name1 != _data_name2){
+    delete _data_name1;
+    _data_name1 = NULL;
   }
-  _data_name = new char[_data_name_len+1];
-  len = serialize->_read(_data_name,_data_name_len);
+  if (_data_name_len < WSCSERIALIZEDATA_MAX_BUF_LEN){
+    _data_name1 = _data_name2;
+  }else{
+    _data_name1 = new char[_data_name_len+1];
+  }
+  len = serialize->_read(_data_name1,_data_name_len);
 //printf("WSCserializeData::read len=%d _data_name_len\n",len,_data_name_len);
   if (len <_data_name_len){
-    _data_name[0] = 0;
+    _data_name1[0] = 0;
     return 0;
   }
-  _data_name[_data_name_len] = 0;
+  _data_name1[_data_name_len] = 0;
 //printf("WSCserializeData::read len=%d #%s#\n",_data_name_len,_data_name);
 
   len = serialize->_read(&_type,sizeof(char));
@@ -167,18 +182,22 @@ long WSCserializeData::read(WSDserialize* serialize){
     return 0;
   }
 
-  if (_type_name != NULL){
-    delete _type_name;
-    _type_name = NULL;
+  if (_type_name1 != NULL && _type_name1 != _type_name2){
+    delete _type_name1;
+    _type_name1 = NULL;
   }
   if (_type == WS_DcObject){
-    _type_name = new char[_type_name_len+1];
-    len = serialize->_read(_type_name,_type_name_len);
+    if (_type_name_len < WSCSERIALIZEDATA_MAX_BUF_LEN){
+      _type_name1 = _type_name2;
+    }else{
+      _type_name1 = new char[_type_name_len+1];
+    }
+    len = serialize->_read(_type_name1,_type_name_len);
     if (len < _type_name_len){
-      _type_name[0] = 0;
+      _type_name1[0] = 0;
       return 0;
     }
-    _type_name[_type_name_len] = 0;
+    _type_name1[_type_name_len] = 0;
 
     char tmp2[4];
     long ret = serialize->_read(tmp2,sizeof(char)*4);
@@ -187,7 +206,19 @@ long WSCserializeData::read(WSDserialize* serialize){
     }
     long num;
     WSGIbinType()->memscan4b(&num,tmp2);
-
+#if 1
+    long i;
+    clearChildren();
+    WSCserializeData* dt = new WSCserializeData[num];
+    for(i=0; i< num; i++){
+      len = dt[i].read(serialize);
+      if (len == 0){
+        return 0;
+      }
+      _children.add(&dt[i]);
+    }
+#endif
+#if 0
     long i;
     WSCserializeData* dt;
     for(i=0; i< num; i++){
@@ -198,22 +229,27 @@ long WSCserializeData::read(WSDserialize* serialize){
       }
       _children.add(dt);
     }
+#endif
     return ret;
   }else{
     char tmp[4];
     len = serialize->_read(tmp,sizeof(char)*4);
-    if (len < (long)sizeof(char)*4){
-      delete (char*)_data;
-      _data = NULL;
+    if (len < (long)sizeof(char)*4 && _data1 != NULL && _data1 != _data2){
+      delete (char*)_data1;
+      _data1 = NULL;
       return 0;
     }
     WSGIbinType()->memscan4b(&_data_len,tmp);
-    if (_data != NULL){
-      delete (char*)_data;
-      _data = NULL;
+    if (_data1 != NULL && _data1 != _data2){
+      delete (char*)_data1;
+      _data1 = NULL;
     }
-    _data = new char[_data_len];
-    long ret = serialize->_read(_data,_data_len);
+    if (_data_len < WSCSERIALIZEDATA_DATA_LEN+1){
+      _data1 = _data2;
+    }else{
+      _data1 = new char[_data_len];
+    }
+    long ret = serialize->_read(_data1,_data_len);
     if(ret < _data_len){
       return 0;
     }
@@ -234,7 +270,7 @@ WSDserialize::WSDserialize(){
 
 WSDserialize::~WSDserialize(){
   long i;
-  long num = _data_list.getNum();
+  long num = _data_list._num;
   WSCserializeData* dt;
   for(i=0; i< num; i++){
     dt  = (WSCserializeData*)_data_list.getData(i);
@@ -256,129 +292,171 @@ long WSDserialize::_write_data(WSCserializeData* dt,char* type,void* data, long
 #ifdef NO_FILE_SERIALIZE
   return WS_ERR_NOTYPE;
 #else
-  if (dt->_data != NULL){
-      delete (char*)dt->_data;
-      dt->_data = NULL;
+  if (dt->_data1 != NULL && dt->_data1 != dt->_data2){
+      delete (char*)dt->_data1;
+      dt->_data1 = NULL;
   }
   if (type[0] == '#'){
     if (!strcmp(type,WS_DcRString)){
-      char* d = WSGFstrdup((char*)data);
-      dt->_type = WS_DcString;
       dt->_type_name_len = 0;
-      dt->_data_len = strlen(d)+1;
-      dt->_data = d;
+      dt->_type = WS_DcString;
+      dt->_data_len = strlen((char*)data)+1;
+      if (dt->_data_len < WSCSERIALIZEDATA_DATA_LEN){
+        dt->_data1 = dt->_data2;
+        strcpy((char*)dt->_data1,(char*)data);
+      }else{
+        dt->_data1 = WSGFstrdup((char*)data);
+      }
     }else
     if (!strcmp(type,WS_DcRShort)){
-      char* d = new char[2*items];
-      short* dat = (short*)data;
-      long i;
-      for(i=0; i < items; i++){
-        WSGIbinType()->memcpy2b(&d[2*i],dat[i]);
-      }
       dt->_type = WS_DcShort;
       dt->_type_name_len = 0;
-      dt->_data_len = 2*items;
-      dt->_data = d;
+      dt->_data_len = WS_SERIALIZE_SHORT_SIZE*items;
+      if (dt->_data_len < WSCSERIALIZEDATA_DATA_LEN+1){
+        dt->_data1 = dt->_data2;
+      }else{
+        dt->_data1 = new char[WS_SERIALIZE_SHORT_SIZE*items];
+      }
+      short* dat = (short*)data;
+      long i;
+      char* d = (char*)dt->_data1;
+      for(i=0; i < items; i++){
+        WSGIbinType()->memcpy2b(&d[WS_SERIALIZE_SHORT_SIZE*i],dat[i]);
+      }
     }else
     if (!strcmp(type,WS_DcRUShort)){
-      char* d = new char[2*items];
-      WSCushort* dat = (WSCushort*)data;
-      long i;
-      for(i=0; i < items; i++){
-        WSGIbinType()->memcpy2b(&d[2*i],dat[i]);
-      }
       dt->_type = WS_DcUShort;
       dt->_type_name_len = 0;
-      dt->_data_len = 2*items;
-      dt->_data = d;
+      dt->_data_len = WS_SERIALIZE_SHORT_SIZE*items;
+      if (dt->_data_len < WSCSERIALIZEDATA_DATA_LEN+1){
+        dt->_data1 = dt->_data2;
+      }else{
+        dt->_data1 = new char[WS_SERIALIZE_SHORT_SIZE*items];
+      }
+      WSCushort* dat = (WSCushort*)data;
+      long i;
+      char* d = (char*)dt->_data1;
+      for(i=0; i < items; i++){
+        WSGIbinType()->memcpy2b(&d[WS_SERIALIZE_SHORT_SIZE*i],dat[i]);
+      }
     }else
     if (!strcmp(type,WS_DcRChar)){
-      char* d = new char[items];
-      memcpy(d,data,items);
       dt->_type = WS_DcChar;
       dt->_type_name_len = 0;
       dt->_data_len = items;
-      dt->_data = d;
+      if (dt->_data_len < WSCSERIALIZEDATA_DATA_LEN+1){
+        dt->_data1 = dt->_data2;
+      }else{
+        dt->_data1 = new char[items];
+      }
+      memcpy(dt->_data1,data,items);
     }else
     if (!strcmp(type,WS_DcRLong)){
-      char* d = new char[4*items];
-      long* dat = (long*)data;
-      long i;
-      for(i=0; i < items; i++){
-        WSGIbinType()->memcpy4b(&d[4*i],dat[i]);
-      }
       dt->_type = WS_DcLong;
       dt->_type_name_len = 0;
-      dt->_data_len = 4*items;
-      dt->_data = d;
+      dt->_data_len = WS_SERIALIZE_LONG_SIZE*items;
+      if (dt->_data_len < WSCSERIALIZEDATA_DATA_LEN+1){
+        dt->_data1 = dt->_data2;
+      }else{
+        dt->_data1 = new char[WS_SERIALIZE_LONG_SIZE*items];
+      }
+      long* dat = (long*)data;
+      char* d = (char*)dt->_data1;
+      long i;
+      for(i=0; i < items; i++){
+        WSGIbinType()->memcpy4b(&d[WS_SERIALIZE_LONG_SIZE*i],dat[i]);
+      }
     }else
     if (!strcmp(type,WS_DcRUChar)){
-      char* d = new char[items];
-      memcpy(d,data,items);
       dt->_type = WS_DcUChar;
       dt->_type_name_len = 0;
       dt->_data_len = items;
-      dt->_data = d;
+      if (dt->_data_len < WSCSERIALIZEDATA_DATA_LEN+1){
+        dt->_data1 = dt->_data2;
+      }else{
+        dt->_data1 = new char[items];
+      }
+      memcpy(dt->_data1,data,items);
     }else
     if (!strcmp(type,WS_DcRULong)){
-      char* d = new char[4*items];
-      WSCulong* dat = (WSCulong*)data;
-      long i;
-      for(i=0; i < items; i++){
-        WSGIbinType()->memcpy4b(&d[4*i],dat[i]);
-      }
       dt->_type = WS_DcULong;
       dt->_type_name_len = 0;
-      dt->_data_len = 4*items;
-      dt->_data = d;
+      dt->_data_len = WS_SERIALIZE_LONG_SIZE*items;
+      if (dt->_data_len < WSCSERIALIZEDATA_DATA_LEN+1){
+        dt->_data1 = dt->_data2;
+      }else{
+        dt->_data1 = new char[WS_SERIALIZE_LONG_SIZE*items];
+      }
+      WSCulong* dat = (WSCulong*)data;
+      char* d = (char*)dt->_data1;
+      long i;
+      for(i=0; i < items; i++){
+        WSGIbinType()->memcpy4b(&d[WS_SERIALIZE_LONG_SIZE*i],dat[i]);
+      }
     }else
     if (!strcmp(type,WS_DcRInt) ){
-      char* d = new char[4*items];
-      int* dat = (int*)data;
-      long i;
-      for(i=0; i < items; i++){
-        WSGIbinType()->memcpy4b(&d[4*i],dat[i]);
-      }
       dt->_type = WS_DcInt;
       dt->_type_name_len = 0;
-      dt->_data_len = 4*items;
-      dt->_data = d;
+      dt->_data_len = WS_SERIALIZE_INT_SIZE*items;
+      if (dt->_data_len < WSCSERIALIZEDATA_DATA_LEN+1){
+        dt->_data1 = dt->_data2;
+      }else{
+        dt->_data1 = new char[WS_SERIALIZE_INT_SIZE*items];
+      }
+      int* dat = (int*)data;
+      char* d = (char*)dt->_data1;
+      long i;
+      for(i=0; i < items; i++){
+        WSGIbinType()->memcpy4b(&d[WS_SERIALIZE_INT_SIZE*i],dat[i]);
+      }
     }else
     if (!strcmp(type,WS_DcRUInt)){
-      char* d = new char[4*items];
-      WSCuint* dat = (WSCuint*)data;
-      long i;
-      for(i=0; i < items; i++){
-        WSGIbinType()->memcpy4b(&d[4*i],dat[i]);
-      }
       dt->_type = WS_DcUInt;
       dt->_type_name_len = 0;
-      dt->_data_len = 4*items;
-      dt->_data = d;
+      dt->_data_len = WS_SERIALIZE_INT_SIZE*items;
+      if (dt->_data_len < WSCSERIALIZEDATA_DATA_LEN+1){
+        dt->_data1 = dt->_data2;
+      }else{
+        dt->_data1 = new char[WS_SERIALIZE_INT_SIZE*items];
+      }
+      WSCuint* dat = (WSCuint*)data;
+      char* d = (char*)dt->_data1;
+      long i;
+      for(i=0; i < items; i++){
+        WSGIbinType()->memcpy4b(&d[WS_SERIALIZE_INT_SIZE*i],dat[i]);
+      }
     }else
     if (!strcmp(type,WS_DcRFloat)){
-      char* d = new char[4*items];
-      float* dat = (float*)data;
-      long i;
-      for(i=0; i < items; i++){
-        WSGIbinType()->memcpy4b(&d[4*i],(long)dat[i]);
-      }
       dt->_type = WS_DcFloat;
       dt->_type_name_len = 0;
-      dt->_data_len = 4*items;
-      dt->_data = d;
-    }else
-    if (!strcmp(type,WS_DcRDouble)){
-      char* d = new char[8*items];
-      double* dat = (double*)data;
+      dt->_data_len = WS_SERIALIZE_FLOAT_SIZE*items;
+      if (dt->_data_len < WSCSERIALIZEDATA_DATA_LEN+1){
+        dt->_data1 = dt->_data2;
+      }else{
+        dt->_data1 = new char[WS_SERIALIZE_FLOAT_SIZE*items];
+      }
+      float* dat = (float*)data;
+      char* d = (char*)dt->_data1;
       long i;
       for(i=0; i < items; i++){
-        WSGIbinType()->memcpy8b(&d[8*i],&dat[i]);
+        WSGIbinType()->memcpy4b(&d[WS_SERIALIZE_FLOAT_SIZE*i],(long)dat[i]);
       }
+    }else
+    if (!strcmp(type,WS_DcRDouble)){
       dt->_type = WS_DcDouble;
       dt->_type_name_len = 0;
-      dt->_data_len = 8*items;
-      dt->_data = d;
+      dt->_data_len = WS_SERIALIZE_DOUBLE_SIZE*items;
+      if (dt->_data_len < WSCSERIALIZEDATA_DATA_LEN+1){
+        dt->_data1 = dt->_data2;
+      }else{
+        dt->_data1 = new char[WS_SERIALIZE_DOUBLE_SIZE*items];
+      }
+      double* dat = (double*)data;
+      char* d = (char*)dt->_data1;
+      long i;
+      for(i=0; i < items; i++){
+        WSGIbinType()->memcpy8b(&d[WS_SERIALIZE_DOUBLE_SIZE*i],&dat[i]);
+      }
     }
   }else{
 
@@ -387,7 +465,7 @@ long WSDserialize::_write_data(WSCserializeData* dt,char* type,void* data, long
       return WS_ERR_NOTYPE;
     }
 
-    //DEVELOLP//XXXXXXXXXXXXXXX   ηѴǤʤ
+    //DEVELOLP//XXXXXXXXXXXXXXX   ηѴǤʤ...
     long num = sv->getParentClassNum();
     long i;
     for(i=0; i< num; i++){
@@ -404,7 +482,7 @@ long WSDserialize::_write_data(WSCserializeData* dt,char* type,void* data, long
     svh = (void (*)(WSDserialize*,void*))sv->getSaveHandler();
     WSCserializeData* bk = _current_data;
     _current_data = dt;
-    svh(this,data); //ϥɥǥ
+    svh(this,data); //ϥɥǥ...
     _current_data = bk;
   }
   return WS_NO_ERR;
@@ -421,7 +499,7 @@ long WSDserialize::_write_sub_class(WSCserializeData* dt,char* type,void* data,l
     return WS_ERR_NOTYPE;
   }
 
-  //DEVELOP//XXXXXXXXXXXXXXX   ηѴǤʤ
+  //DEVELOP//XXXXXXXXXXXXXXX   ηѴǤʤ...
   long num = sv->getParentClassNum();
   long i;
   for(i=0; i< num; i++){
@@ -438,7 +516,7 @@ long WSDserialize::_write_sub_class(WSCserializeData* dt,char* type,void* data,l
   svh = (void (*)(WSDserialize*,void*))sv->getSaveHandler();
   WSCserializeData* bk = _current_data;
   _current_data = dt;
-  svh(this,data); //ϥɥǥ
+  svh(this,data); //ϥɥǥ...
   _current_data = bk;
   return WS_NO_ERR;
 #endif
@@ -449,35 +527,50 @@ long WSDserialize::_save(WSClistData* dt_list,char* type,char* name,void* data){
   return WS_ERR;
 #else
 
-  long num = dt_list->getNum();
+  long num = dt_list->_num;
   long i;
   //DEVELOP//®򤹤 XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
   for(i=0; i< num; i++){
     WSCserializeData* dt = (WSCserializeData*)dt_list->getData(i);
-    if (!strcmp(dt->_data_name,name)){
+    if (!strcmp(dt->_data_name1,name)){
       dt->clearChildren();
       dt->_type = WS_DcObject;
       dt->_type_name_len = strlen(type);
       dt->_data_len = 0;
-      if (dt->_type_name != NULL){
-          delete dt->_type_name;
-          dt->_type_name = NULL;
+      if (dt->_type_name1 != NULL && dt->_type_name1 != dt->_type_name2){
+          delete dt->_type_name1;
+          dt->_type_name1 = NULL;
+      }
+      if (dt->_type_name_len < WSCSERIALIZEDATA_MAX_BUF_LEN){
+        dt->_type_name1 = dt->_type_name2;
+        strcpy(dt->_type_name1,type);
+      }else{
+        dt->_type_name1 = WSGFstrdup(type);
       }
-      dt->_type_name = WSGFstrdup(type);
       _write_data(dt,type,data,1);
       return WS_NO_ERR;
     }
   }
 
   WSCserializeData* dt = new WSCserializeData;
-  dt->_data_name = WSGFstrdup(name);
   dt->_data_name_len = strlen(name);
+  if (dt->_data_name_len < WSCSERIALIZEDATA_MAX_BUF_LEN ){
+    dt->_data_name1 = dt->_data_name2;
+    strcpy(dt->_data_name1,name);
+  }else{
+    dt->_data_name1 = WSGFstrdup(name);
+  }
   dt->_type = WS_DcObject;
-  dt->_type_name = WSGFstrdup(type);
   dt->_type_name_len = strlen(type);
+  if (dt->_type_name_len < WSCSERIALIZEDATA_MAX_BUF_LEN ){
+    dt->_type_name1 = dt->_type_name2;
+    strcpy(dt->_type_name1,type);
+  }else{
+    dt->_type_name1 = WSGFstrdup(type);
+  }
   dt->_data_len = 0;
 
-   dt_list->add(dt);
+  dt_list->add(dt);
   _write_data(dt,type,data,1);
   return WS_NO_ERR;
 #endif
@@ -519,87 +612,87 @@ long WSDserialize::_read_data(WSCserializeData* dt,char* type,void* data, long i
 #else
   if (type[0] == '#'){
     if (!strcmp(type,WS_DcRString)){
-      //TEST  //array ̤б
-      strcpy((char*)data,(char*)dt->_data);
+      //TEST  //array ̤б...
+      strcpy((char*)data,(char*)dt->_data1);
     }else
     if (!strcmp(type,WS_DcRShort)){
       long i;
-      char* dat = (char*)dt->_data;
+      char* dat = (char*)dt->_data1;
       short* outd = (short*)data;
       for(i=0; i < items; i++){
-        WSGIbinType()->memscan2b((short*)&outd[i],&dat[2*i]);
+        WSGIbinType()->memscan2b((short*)&outd[i],&dat[WS_SERIALIZE_SHORT_SIZE*i]);
       }
     }else
     if (!strcmp(type,WS_DcRUShort)){
       long i;
-      char* dat = (char*)dt->_data;
+      char* dat = (char*)dt->_data1;
       WSCushort* outd = (WSCushort*)data;
       for(i=0; i < items; i++){
-        WSGIbinType()->memscan2b((short*)&outd[i],&dat[2*i]);
+        WSGIbinType()->memscan2b((short*)&outd[i],&dat[WS_SERIALIZE_SHORT_SIZE*i]);
       }
     }else
     if (!strcmp(type,WS_DcRChar)){
-      memcpy(data,dt->_data,items);
+      memcpy(data,dt->_data1,items);
     }else
     if (!strcmp(type,WS_DcRLong)){
       long i;
-      char* dat = (char*)dt->_data;
+      char* dat = (char*)dt->_data1;
       long* outd = (long*)data;
       for(i=0; i < items; i++){
-        WSGIbinType()->memscan4b(&outd[i],&dat[4*i]);
+        WSGIbinType()->memscan4b(&outd[i],&dat[WS_SERIALIZE_LONG_SIZE*i]);
       }
     }else
     if (!strcmp(type,WS_DcRUChar)){
-      memcpy(data,dt->_data,items);
+      memcpy(data,dt->_data1,items);
     }else
     if (!strcmp(type,WS_DcRULong)){
       long i;
-      char* dat = (char*)dt->_data;
+      char* dat = (char*)dt->_data1;
       long* outd = (long*)data;
       for(i=0; i < items; i++){
-        WSGIbinType()->memscan4b(&outd[i],&dat[4*i]);
+        WSGIbinType()->memscan4b(&outd[i],&dat[WS_SERIALIZE_LONG_SIZE*i]);
       }
     }else
     if (!strcmp(type,WS_DcRInt) ){
       long i;
-      char* dat = (char*)dt->_data;
+      char* dat = (char*)dt->_data1;
       long* outd = (long*)data;
       for(i=0; i < items; i++){
-        WSGIbinType()->memscan4b(&outd[i],&dat[4*i]);
+        WSGIbinType()->memscan4b(&outd[i],&dat[WS_SERIALIZE_INT_SIZE*i]);
       }
     }else
     if (!strcmp(type,WS_DcRUInt)){
       long i;
-      char* dat = (char*)dt->_data;
+      char* dat = (char*)dt->_data1;
       long* outd = (long*)data;
       for(i=0; i < items; i++){
-        WSGIbinType()->memscan4b(&outd[i],&dat[4*i]);
+        WSGIbinType()->memscan4b(&outd[i],&dat[WS_SERIALIZE_INT_SIZE*i]);
       }
     }else
     if (!strcmp(type,WS_DcRFloat)){
       long i;
-      char* dat = (char*)dt->_data;
+      char* dat = (char*)dt->_data1;
       float* outd = (float*)data;
       for(i=0; i < items; i++){
-        WSGIbinType()->memscan4b((long*)&outd[i],&dat[4*i]);
+        WSGIbinType()->memscan4b((long*)&outd[i],&dat[WS_SERIALIZE_FLOAT_SIZE*i]);
       }
     }else
     if (!strcmp(type,WS_DcRDouble)){
       long i;
-      char* dat = (char*)dt->_data;
+      char* dat = (char*)dt->_data1;
       double* outd = (double*)data;
       for(i=0; i < items; i++){
-        WSGIbinType()->memscan8b(&outd[i],&dat[8*i]);
+        WSGIbinType()->memscan8b(&outd[i],&dat[WS_SERIALIZE_DOUBLE_SIZE*i]);
       }
     }
   }else{
-    //DEVELOP//XXXXXXXXXXXXXXX   ȤǤбˤ롣
-    WSCtypeServer* sv = WSGIconductor()->searchTypeServer(dt->_type_name);
+    //DEVELOP//XXXXXXXXXXXXXXX   ȤǤбˤ롣...
+    WSCtypeServer* sv = WSGIconductor()->searchTypeServer(dt->_type_name1);
     if (sv == NULL){
       return WS_ERR_NOTYPE;
     }
 
-    //DEVELOP//XXXXXXXXXXXXXXX   ηѴǤʤ
+    //DEVELOP//XXXXXXXXXXXXXXX   ηѴǤʤ...
     long num = sv->getParentClassNum();
     long i;
     for(i=0; i< num; i++){
@@ -616,7 +709,7 @@ long WSDserialize::_read_data(WSCserializeData* dt,char* type,void* data, long i
     lvh = (void (*)(WSDserialize*,void*))sv->getLoadHandler();
     WSCserializeData* bk = _current_data;
     _current_data = dt;
-    lvh(this,data); //ϥɥǥ
+    lvh(this,data); //ϥɥǥ...
     _current_data = bk;
   }
   return WS_NO_ERR;
@@ -632,7 +725,7 @@ long WSDserialize::_read_sub_class(WSCserializeData* dt,char* type,void* data,lo
     return WS_ERR_NOTYPE;
   }
 
-  //DEVELOP//XXXXXXXXXXXXXXX   ηѴǤʤ
+  //DEVELOP//XXXXXXXXXXXXXXX   ηѴǤʤ...
   long num = sv->getParentClassNum();
   long i;
   for(i=0; i< num; i++){
@@ -649,7 +742,7 @@ long WSDserialize::_read_sub_class(WSCserializeData* dt,char* type,void* data,lo
   lvh = (void (*)(WSDserialize*,void*))sv->getLoadHandler();
   WSCserializeData* bk = _current_data;
   _current_data = dt;
-  lvh(this,data); //ϥɥǥ
+  lvh(this,data); //ϥɥǥ...
   _current_data = bk;
   return WS_NO_ERR;
 #endif
@@ -660,19 +753,19 @@ long WSDserialize::_load(WSCserializeData* pdt,WSClistData* dt_list,char* type,c
 #ifdef NO_FILE_SERIALIZE
   return WS_ERR_NOOBJ;
 #else
-  long num = dt_list->getNum();
+  long num = dt_list->_num;
 //printf("here0..num=%d\n",num);
   long i;
   if (pdt != NULL && pdt->_cache_ptr < (WSCulong)num){
-    //󸡺꤫鸡
-    //ʤȤȡ̤Ͻ֤˥ǡɤ߽ФΤ
-    //곺Ψƹ⤤
+    //󸡺꤫鸡...
+    //ʤȤȡ̤Ͻ֤˥ǡɤ߽ФΤ...
+    //곺Ψƹ⤤...
     for( i= pdt->_cache_ptr; i< num; i++){ 
       WSCserializeData* dt = (WSCserializeData*)dt_list->getData(i);
-      if (dt->_data_name == NULL){
+      if (dt->_data_name1 == NULL){
         continue;
       }
-      if ((dt->_data_name[0] == name[0] && !strcmp(dt->_data_name,name)) ||
+      if ((dt->_data_name1[0] == name[0] && !strcmp(dt->_data_name1,name)) ||
           (name[0] == '*' && name[1] == 0 )){
         pdt->_cache_ptr++;
 //printf("here1..\n");
@@ -684,11 +777,11 @@ long WSDserialize::_load(WSCserializeData* pdt,WSClistData* dt_list,char* type,c
 
   for(i=0; i< num; i++){
     WSCserializeData* dt = (WSCserializeData*)dt_list->getData(i);
-    if (dt->_data_name == NULL){
+    if (dt->_data_name1 == NULL){
       continue;
     }
 //printf("dt->_data_name=#%s# #%s#\n",dt->_data_name,name);
-    if ((dt->_data_name[0] == name[0] && !strcmp(dt->_data_name,name)) ||
+    if ((dt->_data_name1[0] == name[0] && !strcmp(dt->_data_name1,name)) ||
           (name[0] == '*' && name[1] == 0 )){
 //    if (!strcmp(dt->data_name,name) || !strcmp(name,"*")){
       if (pdt != NULL){
@@ -736,25 +829,26 @@ long WSDserialize::_load_alloc(WSCserializeData* pdt,WSClistData* dt_list,char*
 #ifdef NO_FILE_SERIALIZE
    return WS_ERR_NOTYPE;
 #else
-  long num = dt_list->getNum();
+  long num = dt_list->_num;
   long i;
   WSCulong cpoint = 0;
   if (pdt != NULL && pdt->_cache_ptr < (WSCulong)num){
-    //󸡺꤫鸡
-    //ʤȤȡ̤Ͻ֤˥ǡɤ߽ФΤ
-    //곺Ψƹ⤤
+    //󸡺꤫鸡...
+    //ʤȤȡ̤Ͻ֤˥ǡɤ߽ФΤ...
+    //곺Ψƹ⤤...
     cpoint = 0;
   }
 
+  WSCserializeData** buf = (WSCserializeData**)dt_list->_data;
   for( i= cpoint; i< num; i++){ 
-    WSCserializeData* dt = (WSCserializeData*)dt_list->getData(i);
-    if (dt->_data_name == 0){
+    WSCserializeData* dt = buf[i];
+    if (dt->_data_name1 == NULL){
       continue;
     }
-    if ((dt->_data_name[0] == name[0] && !strcmp(dt->_data_name,name)) ||
+    if ((dt->_data_name1[0] == name[0] && !strcmp(dt->_data_name1,name)) ||
           (name[0] == '*' && name[1] == 0 )){
 //    if (!strcmp(_dt->data_name,name) || !strcmp(name,"*") ){
-      char* target_type = dt->_type_name;
+      char* target_type = dt->_type_name1;
       if (target_type == NULL){
         switch(dt->_type){
           case WS_DcInt:    target_type = "#int";break;
@@ -768,32 +862,32 @@ long WSDserialize::_load_alloc(WSCserializeData* pdt,WSClistData* dt_list,char*
         }
       }
 
-      if (!strcmp(type,target_type)){ //ƱǤ
-        if (type[0] == '#'){   //ǥեȤηǤ
+      if (!strcmp(type,target_type)){ //ƱǤ...
+        if (type[0] == '#'){   //ǥեȤηǤ...
           if (!strcmp(type,WS_DcRString)){
-              *ptr = new char[strlen((char*)dt->_data)+1];
-              strcpy((char*)*ptr,(char*)dt->_data);
+              *ptr = new char[strlen((char*)dt->_data1)+1];
+              strcpy((char*)*ptr,(char*)dt->_data1);
               return WS_NO_ERR;
           }else if (!strcmp(type,WS_DcRShort)){
               *ptr = new char[2];
-              char* dat = (char*)dt->_data;
+              char* dat = (char*)dt->_data1;
               short* outd = (short*)*ptr;
               WSGIbinType()->memscan2b(outd,dat);
               return WS_NO_ERR;
           }else if (!strcmp(type,WS_DcRChar)){
               *ptr = new char[1];
-              *((char*)*ptr) = *((char*)dt->_data);
+              *((char*)*ptr) = *((char*)dt->_data1);
               return WS_NO_ERR;
           }else if (!strcmp(type,WS_DcRLong) || !strcmp(type,WS_DcRInt) || !strcmp(type,WS_DcRFloat)){
               *ptr = new char[4];
-              char* dat = (char*)dt->_data;
+              char* dat = (char*)dt->_data1;
               long* outd = (long*)*ptr;
               WSGIbinType()->memscan4b(outd,dat);
               return WS_NO_ERR;
 
           }else if (!strcmp(type,WS_DcRDouble)){
             *ptr = new char[8];
-            char* dat = (char*)dt->_data;
+            char* dat = (char*)dt->_data1;
             WSGIbinType()->memscan8b(*ptr,dat);
             return WS_NO_ERR;
           }
@@ -802,7 +896,7 @@ long WSDserialize::_load_alloc(WSCserializeData* pdt,WSClistData* dt_list,char*
           }
           _read_data(dt,target_type,*ptr,1);
           return WS_NO_ERR;
-        }else{ //桼ηǤ
+        }else{ //桼ηǤ...
           WSCtypeServer* sv = WSGIconductor()->searchTypeServer(target_type);
           if (sv == NULL){
             return WS_ERR_NOTYPE;
@@ -818,7 +912,7 @@ long WSDserialize::_load_alloc(WSCserializeData* pdt,WSClistData* dt_list,char*
           return WS_NO_ERR;
         }
       }else{
-        if (type[0] == '#'){   //ǥեȤηǤФ(㤦)
+        if (type[0] == '#'){   //ǥեȤηǤФ(㤦)...
           *ptr = NULL;
           return WS_ERR_INVALID_DATA;
         }
@@ -835,7 +929,7 @@ long WSDserialize::_load_alloc(WSCserializeData* pdt,WSClistData* dt_list,char*
           pdt->_cache_ptr = i+1;
         }
         _read_data(dt,target_type,tmp,1);
-        *ptr = sv->getParentClassPtr(type,tmp); //㤦ΤǥС
+        *ptr = sv->getParentClassPtr(type,tmp); //㤦ΤǥС...
         if (*ptr == NULL){
 //          delete tmp;
           sv->deleteInstance(tmp);
@@ -846,12 +940,12 @@ long WSDserialize::_load_alloc(WSCserializeData* pdt,WSClistData* dt_list,char*
     }
   }
 
-  //ǽ餫鸡Ƥ⸫Ĥʤä
+  //ǽ餫鸡Ƥ⸫Ĥʤä...
   if (pdt == NULL || cpoint == 0){
     return WS_ERR_NOOBJ;
   }
 
-  //Ƹʤ
+  //Ƹʤ...
   pdt->_cache_ptr = 0;
   return _load_alloc(pdt,dt_list, type, name, ptr, construct_data);
 #endif
@@ -909,16 +1003,17 @@ long WSDserialize::_search_instance(WSClistData* dt_list,char* type,char mode,ch
 #ifdef NO_FILE_SERIALIZE
   return WS_ERR_NOOBJ;
 #else
-  long n = dt_list->getNum();
+  long n = dt_list->_num;
   long i;
   WSClistData nlist;
   WSClistData clist;
 
 
   //DEVEOP//®򤹤 XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
+  WSCserializeData** dt_buf = (WSCserializeData**)dt_list->_data;
   for(i=0; i< n; i++){
-    WSCserializeData* dt = (WSCserializeData*)dt_list->getData(i);
-    char* target_type = dt->_type_name;
+    WSCserializeData* dt = dt_buf[i];
+    char* target_type = dt->_type_name1;
     if (target_type == NULL){
       switch(dt->_type){
         case WS_DcInt:    target_type = "#int";break;
@@ -934,7 +1029,7 @@ long WSDserialize::_search_instance(WSClistData* dt_list,char* type,char mode,ch
     if ( !strcmp(type,target_type)){ 
     //ƱǤ ....
        nlist.add(type);
-       clist.add(dt->_data_name);
+       clist.add(dt->_data_name1);
     }else if ( target_type[0] == '#' ){ 
     //C++ ɤηʤФ ...
        continue;
@@ -948,11 +1043,11 @@ long WSDserialize::_search_instance(WSClistData* dt_list,char* type,char mode,ch
        if (sv->isParentClass(type) != False){
        //Ѿطä ...
          clist.add(target_type);
-         nlist.add(dt->_data_name);
+         nlist.add(dt->_data_name1);
        }
     }
   }
-  n = nlist.getNum();
+  n = nlist._num;
   *num = n;
 
   if (n != 0){
@@ -984,12 +1079,17 @@ void WSDserialize::setUnderTransaction(WSCbool fl){
   }
 }
 
-long WSDserialize::beginTransaction(char* field){
+long WSDserialize::beginTransaction(char* field,int fl){
 #ifdef NO_FILE_SERIALIZE
   return WS_ERR;
 #else
   _field.setString(field);
-  long ret = _open(field,WS_RW);
+  long ret = 0;
+  if (fl == WS_READ_ONLY){
+    ret = _open(field,WS_READ_ONLY);
+  }else{
+    ret = _open(field,WS_RW);
+  }
   if (ret != WS_NO_ERR){
     return ret;
   }
@@ -1024,7 +1124,7 @@ long WSDserialize::endTransaction(){
   }
   _need_save = False;
   _seek(0);
-  long num = _data_list.getNum();
+  long num = _data_list._num;
   char tmp[4];
   WSGIbinType()->memcpy4b(tmp,num);
   _write(tmp,sizeof(char)*4);
@@ -1045,4 +1145,3 @@ long WSDserialize::endTransaction(){
   return WS_NO_ERR;
 #endif
 }
-
