@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSDwindowDev.h>
@@ -54,8 +50,30 @@ WSDwindowDev::WSDwindowDev(){
   _exposed_w = 0;
   _exposed_h = 0;
   _pixmap_style = WS_DIRECT_WINDOW;
+  _grabbed = False;
+  _dev_array = NULL;
+  _array_num = 0;
 }
+WSDwindowDev::~WSDwindowDev(){
+  int i;
+  int num = _event_clients.getNum();
+  WSDdev** event_clients_array = (WSDdev**)_event_clients.getBuf();
+  for(i=0; i<num; i++){
+    WSDdev* dev = event_clients_array[i];
+    if (dev->getParentDev() == this){
+      dev->setParentDev(NULL);
+    }
+    if (dev->getEventParentDev() == this){
+      dev->setEventParentDev(NULL);
+    }
+  }
 
+  if (_dev_array != NULL){
+    delete[] _dev_array;
+    _dev_array = NULL;
+    _array_num = 0;
+  }
+}
 #if 0
 static void (*_mouse_in_handler)(WSDdev*) = NULL;
 static void (*_mouse_press_handler)(WSDdev*) = NULL;
@@ -98,9 +116,15 @@ void WSDwindowDev::getDispAddr(short* ox,short* oy){
    short ax;
    short ay;
    getAbsoluteAddr(_w_x,_w_y,&ax,&ay);
+#ifndef NO_MWT_FUNCTION
+   long dx,dy;
+   getDiffCoordinates(&dx,&dy);
+   *ox = px + ax + dx;
+   *oy = py + ay + dy;
+#else
    *ox = px + ax + _diff_x;
    *oy = py + ay + _diff_y;
-
+#endif
    return;
 }
 
@@ -114,7 +138,13 @@ long WSDwindowDev::getValue(long kind,void* data){
              *(char*)data = _expose_sequence;
              break;
     case WSDEV_PIXMAP_STYLE:
-	         *(char*)data = getPixmapStyle();
+	         *(char*)data = _pixmap_style;
+             break;
+    case WSDEV_GRAB_POINTER:
+	         *(char*)data = _grabbed;
+             break;
+    case WSDEV_TITLE_HEIGHT: 
+             *(WSCushort*)data = 0;
              break;
   }
   return WS_NO_ERR;
@@ -130,13 +160,13 @@ long WSDwindowDev::setValue(long kind,void* data){
              _w_y = *(short*)data;
              break;
     case WSDEV_WIDTH:
-             if (getPixmapStyle() != WS_DIRECT_WINDOW){
+             if (_pixmap_style != WS_DIRECT_WINDOW){
                 destroyPixmap();
              }
              _w_w = *(WSCushort*)data;
              break;
     case WSDEV_HEIGHT:
-             if (getPixmapStyle() != WS_DIRECT_WINDOW){
+             if (_pixmap_style != WS_DIRECT_WINDOW){
                 destroyPixmap();
              }
              _w_h = *(WSCushort*)data;
@@ -149,7 +179,7 @@ long WSDwindowDev::setValue(long kind,void* data){
              break;
     case WSDEV_PIXMAP_STYLE:
              val = *(char*)data;
-             if (getPixmapStyle() != WS_DIRECT_WINDOW &&
+             if (_pixmap_style != WS_DIRECT_WINDOW &&
                  val == WS_DIRECT_WINDOW){
                destroyPixmap();
              }
@@ -171,6 +201,9 @@ long WSDwindowDev::setValue(long kind,void* data){
     case WSDEV_MOUSE_NO: 
              _mouse_no = *(WSCushort*)data;
              break;
+    case WSDEV_GRAB_POINTER:
+             _grabbed = *(WSCbool*)data;
+             break;
   }
   return WS_NO_ERR;
 }
@@ -192,7 +225,9 @@ void WSDwindowDev::onMousePress(WSCpoint* pt){
 //  if (_mouse_press_handler != NULL){
 //    _mouse_press_handler(this);
 //  }
-  execEvent(WSEV_MOUSE_PRESS,pt);
+  if (WSGIappMouse()->execGlobalMouseHook(WSEV_MOUSE_PRESS,getAttachedClient()) != False){
+    execEvent(WSEV_MOUSE_PRESS,pt);
+  }
 }
 
 void WSDwindowDev::onMouseDoubleClick(WSCpoint* pt){
@@ -206,22 +241,30 @@ void WSDwindowDev::onMouseRelease(WSCpoint* pt){
 //  if (_mouse_release_handler != NULL){
 //    _mouse_release_handler(this);
 //  }
-  execEvent(WSEV_MOUSE_RELEASE,pt);
+  if (WSGIappMouse()->execGlobalMouseHook(WSEV_MOUSE_RELEASE,getAttachedClient()) != False){
+    execEvent(WSEV_MOUSE_RELEASE,pt);
+  }
 }
 
 void WSDwindowDev::onMouseIn(WSCpoint* pt){
 //  if (_mouse_in_handler != NULL){
 //    _mouse_in_handler(this);
 //  }
-  execEvent(WSEV_MOUSE_IN,pt);
+  if (WSGIappMouse()->execGlobalMouseHook(WSEV_MOUSE_IN,getAttachedClient()) != False){
+    execEvent(WSEV_MOUSE_IN,pt);
+  }
 }
 
 void WSDwindowDev::onMouseOut(){
-  execEvent(WSEV_MOUSE_OUT,NULL);
+  if (WSGIappMouse()->execGlobalMouseHook(WSEV_MOUSE_OUT,getAttachedClient()) != False){
+    execEvent(WSEV_MOUSE_OUT,NULL);
+  }
 }
 
 void WSDwindowDev::onMouseMove(WSCpoint* pt){
-   execEvent(WSEV_MOUSE_MOVE,pt);
+  if (WSGIappMouse()->execGlobalMouseHook(WSEV_MOUSE_MOVE,getAttachedClient()) != False){
+    execEvent(WSEV_MOUSE_MOVE,pt);
+  }
 }
 
 void WSDwindowDev::onResize(WSCrect* area){
@@ -241,12 +284,21 @@ void WSDwindowDev::onResize(WSCrect* area){
 }
 
 WSCbool WSDwindowDev::getReady(){
-  long d1 = getDeviceResource();
-  long d2 = getWindowResource();
-  long d3 = getContextResource();
+//  long d1 = getDeviceResource();
+//  long d2 = getWindowResource();
+//  long d3 = getContextResource();
 //printf("WSDwindowDev::getReady %d %d %d\n",d1,d2,d3);
-  if (d1 == -1 || d2 == -1 || d3 == -1){
-     return False;
+//  if (d1 == -1 || d2 == -1 || d3 == -1){
+//     return False;
+//  }
+  if (getDeviceResource() == -1){
+    return False;
+  }
+  if (getWindowResource() == -1){
+    return False;
+  }
+  if (getContextResource() == -1){
+    return False;
   }
   return True;
 }
@@ -254,7 +306,7 @@ WSCbool WSDwindowDev::getReady(){
 void WSDwindowDev::onExpose(WSCrect* area){
    _expose_sequence = True;
    setExposedArea( area->x, area->y, area->width, area->height );
-   if ( getPixmapStyle() != WS_DIRECT_WINDOW ){
+   if ( _pixmap_style != WS_DIRECT_WINDOW ){
      if ( _pixmap_created == False ){
        if (createPixmap() == WS_NO_ERR){
          _pixmap_created = True;
@@ -273,14 +325,13 @@ void WSDwindowDev::onExpose(WSCrect* area){
    setExposedArea( 0,0,0,0 );
    return;
 }
-
-long WSDwindowDev::beginDraw(short x,short y,
+long WSDwindowDev::internalBeginDraw(short x,short y,
                               WSCushort w,WSCushort h,
                               WSCbool absolute,WSCbool scaling){
-
   if (scaling != False){
-    double scale = getScale();
-    if ( scale != 1){
+    WSCbool scaled = getScaled();
+    if ( scaled != False){
+      double scale = getScale();
       x = (short)((x + getXOffset()) * scale);
       y = (short)((y + getYOffset()) * scale);
       w = (WSCushort)(w * scale);
@@ -291,7 +342,7 @@ long WSDwindowDev::beginDraw(short x,short y,
     }
   }
 
-  if (getPixmapStyle() != WS_DIRECT_WINDOW){
+  if (_pixmap_style != WS_DIRECT_WINDOW){
      _pix_draw_x = x;
      _pix_draw_y = y;
      _pix_draw_w = w;
@@ -315,7 +366,59 @@ long WSDwindowDev::beginDraw(short x,short y,
     return WS_ERR;
   }
   if (getReady() == False){
-    if (getPixmapStyle() == WS_DYNAMIC_PIXMAP && 
+    if (_pixmap_style == WS_DYNAMIC_PIXMAP && 
+        _pixmap_created == False  && createPixmap() == WS_NO_ERR){
+          _pixmap_created = True;
+    }else{
+      return WS_ERR;
+    }
+  }
+  return WS_NO_ERR;
+}
+ 
+long WSDwindowDev::beginDraw(short x,short y,
+                              WSCushort w,WSCushort h,
+                              WSCbool absolute,WSCbool scaling){
+
+  if (scaling != False){
+    WSCbool scaled = getScaled();
+    if ( scaled != False){
+      double scale = getScale();
+      x = (short)((x + getXOffset()) * scale);
+      y = (short)((y + getYOffset()) * scale);
+      w = (WSCushort)(w * scale);
+      h = (WSCushort)(h * scale);
+    }else{
+      x = x + getXOffset();
+      y = y + getYOffset();
+    }
+  }
+
+  if (_pixmap_style != WS_DIRECT_WINDOW){
+     _pix_draw_x = x;
+     _pix_draw_y = y;
+     _pix_draw_w = w;
+     _pix_draw_h = h;
+  }
+
+  WSDdraw* draw = getDraw();
+  if (draw == NULL){
+    return WS_ERR;
+  }
+  _absolute_draw = absolute;
+
+  _x = x; 
+  _y = y; 
+  _w = w; 
+  _h = h; 
+  short ex,ey;
+  WSCushort ew,eh;
+  long ret = getExposedArea(&ex,&ey,&ew,&eh);
+  if (ret != WS_NO_ERR){
+    return WS_ERR;
+  }
+  if (getReady() == False){
+    if (_pixmap_style == WS_DYNAMIC_PIXMAP && 
         _pixmap_created == False  && createPixmap() == WS_NO_ERR){
           _pixmap_created = True;
     }else{
@@ -344,6 +447,15 @@ long WSDwindowDev::beginDraw(short x,short y,
   return WS_NO_ERR;
 }
 
+long WSDwindowDev::internalEndDraw(){
+  if (_absolute_draw != False && _pixmap_style != WS_DIRECT_WINDOW &&
+      _expose_sequence == False){
+     _copy_area( _pix_draw_x, _pix_draw_y, _pix_draw_w, _pix_draw_h,
+                _pix_draw_x, _pix_draw_y,WS_PIXMAP_TO_WINDOW);
+  }
+  _absolute_draw = False;
+  return WS_NO_ERR;
+}
 long WSDwindowDev::endDraw(){
   WSDdraw* draw = getDraw();
   if (draw == NULL){
@@ -352,7 +464,7 @@ long WSDwindowDev::endDraw(){
   }
   draw->endDraw();
 
-  if (getAbsoluteDraw() != False && getPixmapStyle() != WS_DIRECT_WINDOW &&
+  if (_absolute_draw != False && _pixmap_style != WS_DIRECT_WINDOW &&
       _expose_sequence == False){
      _copy_area( _pix_draw_x, _pix_draw_y, _pix_draw_w, _pix_draw_h,
                 _pix_draw_x, _pix_draw_y,WS_PIXMAP_TO_WINDOW);
@@ -376,16 +488,17 @@ short WSDwindowDev::getBackColorNo(){
 void WSDwindowDev::setBackColorNo(short bg){
   _bg_no = bg;
 }
-
+#if 0 //move to WSDdev.cpp
 void WSDwindowDev::setExposeSequence(WSCbool fl){
   _expose_sequence = fl;
 }
 WSCbool WSDwindowDev::getExposeSequence(){
   return _expose_sequence;
 }
+#endif
 long WSDwindowDev::exposeArea(long x,long y,WSCulong w,WSCulong h,WSCbool scaling){
   if (getReady() == False){
-    if (getPixmapStyle() == WS_DYNAMIC_PIXMAP && 
+    if (_pixmap_style == WS_DYNAMIC_PIXMAP && 
         _pixmap_created == False  && createPixmap() == WS_NO_ERR){
           _pixmap_created = True;
     }else{
@@ -396,8 +509,9 @@ long WSDwindowDev::exposeArea(long x,long y,WSCulong w,WSCulong h,WSCbool scalin
     return WS_ERR;
   }
   if (scaling != False){
-    double scale = getScale();
-    if ( scale != 1){
+    WSCbool scaled = getScaled();
+    if ( scaled != False){
+      double scale = getScale();
       x = (short)((x + getXOffset())* scale);
       y = (short)((y + getYOffset())* scale);
       w = (WSCushort)(w * scale);
@@ -407,8 +521,15 @@ long WSDwindowDev::exposeArea(long x,long y,WSCulong w,WSCulong h,WSCbool scalin
       y = y + getYOffset();
     }
   }
-
-  if (getPixmapStyle() == WS_DIRECT_WINDOW){
+  if (x < 0 && w + x < w){
+    w = w + x;
+    x = 0;
+  }
+  if (y < 0 && h + y < h){
+    h = h + y;
+    y = 0;
+  }
+  if (_pixmap_style == WS_DIRECT_WINDOW){
 //printf("WSDwindowDev::expose %d,%d,%d,%d\n",x,y,w,h);
     exposeExecute(x,y,w,h);
   }else{
@@ -422,9 +543,13 @@ long WSDwindowDev::exposeArea(long x,long y,WSCulong w,WSCulong h,WSCbool scalin
 
 long WSDwindowDev::clearArea(long x,long y,WSCulong w,WSCulong h,WSCbool fl,WSCbool scaling){
   if (getReady() == False){
-    if (getPixmapStyle() == WS_DYNAMIC_PIXMAP && 
+    if (_pixmap_style == WS_DYNAMIC_PIXMAP && 
         _pixmap_created == False  && createPixmap() == WS_NO_ERR){
           _pixmap_created = True;
+      if (scaling != False){
+        _copy_area(x,y,w,h,x,y,WS_PIXMAP_TO_WINDOW);
+        return WS_NO_ERR;
+      }
     }else{
       return WS_ERR;
     }
@@ -433,8 +558,9 @@ long WSDwindowDev::clearArea(long x,long y,WSCulong w,WSCulong h,WSCbool fl,WSCb
     return WS_ERR;
   }
   if (scaling != False){
-    double scale = getScale();
-    if ( scale != 1){
+    WSCbool scaled = getScaled();
+    if ( scaled != False){
+      double scale = getScale();
       x = (short)((x + getXOffset())* scale);
       y = (short)((y + getYOffset())* scale);
       w = (WSCushort)(w * scale);
@@ -444,14 +570,12 @@ long WSDwindowDev::clearArea(long x,long y,WSCulong w,WSCulong h,WSCbool fl,WSCb
       y = y + getYOffset();
     }
   }
-
-  if (getPixmapStyle() == WS_DIRECT_WINDOW){
+  if (_pixmap_style == WS_DIRECT_WINDOW){
     _clear_area(x,y,w,h);
     if ( fl != False ){
       exposeExecute(x,y,w,h);
     }
-
-  }else{ //ԥޥåפξ
+  }else{ //pixmap style..
     _setup_pixmap(x,y,w,h,_bg_no,_bp_no);
     if ( fl != False ){ 
       exposeExecute(x,y,w,h);
@@ -462,7 +586,7 @@ long WSDwindowDev::clearArea(long x,long y,WSCulong w,WSCulong h,WSCbool fl,WSCb
 }
 long WSDwindowDev::copyArea(long x,long y,WSCulong w,WSCulong h,long dx,long dy){
   if (getReady() == False){
-    if (getPixmapStyle() == WS_DYNAMIC_PIXMAP && 
+    if (_pixmap_style == WS_DYNAMIC_PIXMAP && 
         _pixmap_created == False  && createPixmap() == WS_NO_ERR){
           _pixmap_created = True;
     }else{
@@ -472,8 +596,12 @@ long WSDwindowDev::copyArea(long x,long y,WSCulong w,WSCulong h,long dx,long dy)
   if (getReady() == False){
     return WS_ERR;
   }
-  double scale = getScale();
-  if ( scale != 1){
+  if (x == dx && y == dy){
+    return WS_NO_ERR;
+  }
+  WSCbool scaled = getScaled();
+  if ( scaled != False){
+    double scale = getScale();
     x = (short)((x + getXOffset())* scale);
     y = (short)((y + getYOffset())* scale);
     w = (WSCushort)(w * scale);
@@ -507,17 +635,27 @@ void WSDwindowDev::exposeExecute(short x,short y,WSCushort w,WSCushort h){
       _expose_sequence = ebk;
       setExposedArea(oldx,oldy,oldw,oldh);
 }
-
 long WSDwindowDev::execEvent(long ev_kind,void* data){
   if (getAttachedClient() == NULL){
     return WS_NO_ERR;
   }
   long i;
   long num = _event_clients.getNum();
-  WSDdev**  array = new WSDdev*[num];
+
+  if (_dev_array == NULL){
+    _dev_array = new WSDdev*[num];
+    _array_num = num;
+  }
+  if (_array_num < num){
+    if (_dev_array != NULL){
+      delete[] _dev_array;
+    }
+    _dev_array = new WSDdev*[num];
+    _array_num = num;
+  }
 
   if (num > 0){
-    memcpy(array,_event_clients.getBuf(),sizeof(void*)*num);
+    memcpy(_dev_array,_event_clients._data,sizeof(void*)*num);
   }
   long ev_bit = WSGFeventToMaskBit(ev_kind);
   _event_delivering_sequence = True;
@@ -530,7 +668,7 @@ long WSDwindowDev::execEvent(long ev_kind,void* data){
 //printf("WSDwindowDev::execEvent ev=%d %s\n",ev_kind,getAttachedClient()->getInstanceName());
 
   for(i=0; i < num; i++){
-    WSDdev* dev = array[i];
+    WSDdev* dev = _dev_array[i];
     if (dev == this && (ev_bit & exclusive_mask)){
 //printf("WSDwindowDev::execEv   skipped.\n");
       need_execute_later = 1;
@@ -541,8 +679,10 @@ long WSDwindowDev::execEvent(long ev_kind,void* data){
       WSClistData* list = (WSClistData*)_client->getUserData(DELETED_CL_LIST);
       long j;
       long num2 = list->getNum();
+      WSDdev** dlist = (WSDdev**)(*list)._data;
       for(j=0; j < num2; j++){
-        WSDdev* data = (WSDdev*)(*list)[j];
+//        WSDdev* data = (WSDdev*)(*list)[j];
+        WSDdev* data = dlist[j];
         if (data == dev){
           deleted = 1;
           break;
@@ -567,8 +707,10 @@ long WSDwindowDev::execEvent(long ev_kind,void* data){
       list = (WSClistData*)_client->getUserData(DELETED_CL_LIST);
       long j;
       long num2 = list->getNum();
+      WSDdev** dlist = (WSDdev**)(*list).getBuf();
       for(j=0; j < num2; j++){
-        WSDdev* data = (WSDdev*)(*list)[j];
+//        WSDdev* data = (WSDdev*)(*list)[j];
+        WSDdev* data = dlist[j];
         if (data == this){
           deleted = 1;
         } 
@@ -587,7 +729,7 @@ long WSDwindowDev::execEvent(long ev_kind,void* data){
     }
   }
 
-  delete array;
+//  delete array;
   _event_delivering_sequence = False;
 //printf("WSDwindowDev::execEvent t2=%d %s\n",WSGFclocktime() - t1,getAttachedClient()->getInstanceName());
   return WS_NO_ERR;
@@ -595,8 +737,9 @@ long WSDwindowDev::execEvent(long ev_kind,void* data){
 
 WSCbool WSDwindowDev::isExposed(short x,short y,WSCushort w,WSCushort h,WSCbool scaling){
   if (scaling != False){
-    double scale = getScale();
-    if ( scale != 1){
+    WSCbool scaled = getScaled();
+    if ( scaled != False){
+      double scale = getScale();
       x = (short)((x + getXOffset())* scale);
       y = (short)((y + getYOffset())* scale);
       w = (WSCushort)(w * scale);
@@ -704,8 +847,9 @@ long WSDwindowDev::setEvent(WSDdev* client_dev,WSCbool fl){
 
 long WSDwindowDev::getExposedArea(short* x,short* y,WSCushort* w,WSCushort* h,WSCbool scaling){
   if (scaling != False){
-    double scale = getScale();
-    if ( scale != 1){
+    WSCbool scaled = getScaled();
+    if ( scaled != False){
+      double scale = getScale();
       *x = (short)((_exposed_x - getXOffset()) * scale);
       *y = (short)((_exposed_y - getYOffset()) * scale);
       *w = (WSCushort)(_exposed_w * scale);
@@ -833,9 +977,10 @@ void WSDwindowDev::setPixmapStyle(char val){
   _pixmap_style = val;
 }
 
-char WSDwindowDev::getPixmapStyle(){
-  return _pixmap_style;
-}
+//move to WSDdev.cpp
+//char WSDwindowDev::getPixmapStyle(){
+//  return _pixmap_style;
+//}
 
 void WSDwindowDev::getWindowSize(short* x,short* y,WSCushort* w,WSCushort* h){
   *x = _w_x;
@@ -872,7 +1017,14 @@ long WSDwindowDev::destroyWindow(){
 long WSDwindowDev::destroyContext(){
   return WS_NO_ERR;
 }
-
+WSCbool WSDwindowDev::getPixmapCreated(){
+  return _pixmap_created;
+}
+#ifndef NO_RESTACK_CHILD
+void WSDwindowDev::restackChildren(){
+  return;
+}
+#endif
 #if 0
 WSDdev* WSDwindowDev::getDependDev(){
   return _depend_dev;
@@ -887,3 +1039,9 @@ WSCbool WSDwindowDev::isIndependent(){
   return False;
 }
 #endif
+#ifndef NO_MWT_FUNCTION
+void WSDwindowDev::getDiffCoordinates(long* x,long* y){
+  *x = _diff_x;
+  *y = _diff_y;
+}
+#endif
