@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WSNdef.h>
 
@@ -52,7 +48,7 @@ char* _WSNblinkColor             ="blinkColor";
 char* _WSNblinkRefreshing        ="blinkRefreshing";
 char* _WSNpixmapStyle            ="pixmapStyle";
 char* _WSNlabelPixmap            ="labelPixmap";
-char* _WSNkanjiIn                ="kanjiIn";
+char* _WSNenableFep                ="enableFep";
 char* _WSNinterCur               ="interCur";
 char* _WSNinterval               ="interval";
 char* _WSNcont                   ="cont";
@@ -199,6 +195,7 @@ char* _WSNrefreshing ="refreshing";
 char* _WSNvcolumns ="vcolumns";
 char* _WSNhcolumns ="hcolumns";
 char* _WSNseparator ="separator";
+char* _WSNseparator2 ="separator2";
 char* _WSNfit ="fit";
 char* _WSNip ="ip";
 char* _WSNport ="port";
@@ -216,6 +213,9 @@ char* _WSNsortItems ="sortItems";
 char* _WSNsort ="sort";
 char* _WSNitem ="item";
 char* _WSNfilter ="filter";
+char* _WSNpixmapOverlap ="pixmapOverlap";
+char* _WSNkeepPixmapCache ="keepPixmapCache";
+char* _WSNautoLineFeeding ="autoLineFeeding";
 
 #else //MSW
 
@@ -243,7 +243,7 @@ char* _WSNblinkColor             (){ return "blinkColor";};
 char* _WSNblinkRefreshing        (){ return "blinkRefreshing";};
 char* _WSNpixmapStyle            (){ return "pixmapStyle";};
 char* _WSNlabelPixmap            (){ return "labelPixmap";};
-char* _WSNkanjiIn                (){ return "kanjiIn";};
+char* _WSNenableFep                (){ return "enableFep";};
 char* _WSNinterCur               (){ return "interCur";};
 char* _WSNinterval               (){ return "interval";};
 char* _WSNcont                   (){ return "cont";};
@@ -390,6 +390,7 @@ char* _WSNrefreshing        (){ return "refreshing";};
 char* _WSNvcolumns        (){ return "vcolumns";};
 char* _WSNhcolumns        (){ return "hcolumns";};
 char* _WSNseparator       (){ return "separator";};
+char* _WSNseparator2       (){ return "separator2";};
 char* _WSNfit       (){ return "fit";};
 char* _WSNip       (){ return "ip";};
 char* _WSNport     (){ return "port";};
@@ -407,5 +408,8 @@ char* _WSNsortItems     (){ return "sortItems";};
 char* _WSNsort     (){ return "sort";};
 char* _WSNitem     (){ return "item";};
 char* _WSNfilter     (){ return "filter";};
+char* _WSNpixmapOverlap     (){ return "pixmapOverlap";};
+char* _WSNkeepPixmapCache     (){ return "keepPixmapCache";};
+char* _WSNautoLineFeeding     (){ return "autoLineFeeding";};
 
 #endif //MSW
