@@ -1,3 +1,25 @@
+//
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
+//
+
 #include <WScom.h>
 #include <WSCvbtn.h>
 #include <WSDappDev.h>
@@ -10,9 +32,34 @@
 #include <WSCpopupFrame.h>
 #include <WSDmouse.h>
 #include <WSCguiPolicy.h>
+//--
+#include <WSCfontSet.h>
+//--
+
+#ifdef WS_EMBED
+#ifndef NO_EXT_GUI_POLICY
+void _libws_extgui_policy(){};
+#endif
+#endif
+
 
 short WSGFgetParentBackColor(WSCbase* client){
   WSCbase* parent = client->getParent();
+  if (parent == NULL){
+    return WSGIappColorSet()->getColorNo("DEF1");
+  }
+  if (parent->existProperty(WSNworkBackColor) != False){
+    short cno=0;
+    parent->getPropertyV(WSNworkBackColor,&cno);
+    return cno;
+  }
+  if (parent->existProperty(WSNbackColor) != False){
+    short cno=0;
+    parent->getPropertyV(WSNbackColor,&cno);
+    return cno;
+  }
+  return WSGIappColorSet()->getColorNo("DEF1");
+#if 0
   WSCstring parent_bg("DEF1");
   if (parent != NULL){
     parent_bg = (char*)parent->getProperty(WSNbackColor);
@@ -21,6 +68,7 @@ short WSGFgetParentBackColor(WSCbase* client){
     }
   }
   return WSGIappColorSet()->getColorNo(parent_bg);
+#endif
 }
 
 void WSGFgetFixedRGB(long r,long g,long b,long v,long* r1,long*g1,long* b1){
@@ -201,7 +249,6 @@ void WSGFdrawModern1Shadow(WSDdev* dev,long shadow_type,WSCushort stick,
     dev->drawFillRect(x,y,w,st);
 #endif
     return;
-
   }else{
     WSGFdrawShadow(dev,shadow_type,stick,
                    fg_no, ts_no, bs_no, bg_no,
@@ -225,11 +272,26 @@ void WSCvbtn_draw_mdr1(void* ptr){
        st->twin_blink == False){
     return;
   }
-  WSCstring str = st->label_string;
+//  WSCstring str;
+//  str.setStaticString(st->label_string);
+  WSCstring* _str;
+  WSCstring tstr;
   if (st->orientation == WS_VERTICAL) {
-    WSGFcreateVerticalString(&str);
+    _str = &tstr;
+    _str->setString(st->label_string);
+    WSGFcreateVerticalString(_str);
   } else {
-    str.replaceString("\\n", "\n", 0);
+    _str = st->_this->getInternalString();
+    _str->replaceString("\\n", "\n", 0);
+    if (st->auto_line_feed != False){
+      WSCushort ww = st->width;
+      if (st->width > st->shadow_thickness*2){
+        ww -= st->shadow_thickness*2;
+      }
+      WSCstring str2 = WSGFcreateLineFeedStringByWidth(_str,st->font,ww);
+      _str = &tstr;
+      _str->setString(str2);
+    }
   }
 
   long err = dev->beginDraw(st->x, st->y, st->width, st->height, absolute);
@@ -270,7 +332,11 @@ void WSCvbtn_draw_mdr1(void* ptr){
   if (st->no_background != False){
     dev->drawFillRect(0, 0, w, h);
   }
+#ifndef NO_GRADATION
   if (st->gradation == 0 && st->no_background == False){
+#else
+  if (st->no_background == False){
+#endif
     short ts1 = WSGFgetFixedColor(bgc,32);
     short bs1 = WSGFgetFixedColor(bgc,-32);
     if (st->shadow_thickness != 0){
@@ -287,6 +353,7 @@ void WSCvbtn_draw_mdr1(void* ptr){
     }
   }
 
+#ifndef NO_GRADATION
   if (st->gradation != 0){
     if (st->press_state == True) {
       dev->drawGradation(st->gradation,st->top_shadow_color,
@@ -298,7 +365,7 @@ void WSCvbtn_draw_mdr1(void* ptr){
                          st->width, st->height,st->gradation_margin);
     }
   }
-
+#endif
   WSCushort stick = st->shadow_thickness;
   short          xx = stick + st->margin_left;
   short          yy = stick + st->margin_top;
@@ -327,7 +394,54 @@ void WSCvbtn_draw_mdr1(void* ptr){
       if (st->press_state == True){
         dy++;
       }
-      dev->drawImage (xx, dy, ww, hh, image, st->_this->getAlignment());
+      if (st->pixmap_overlapped != False){
+        dev->drawImage (xx, dy, ww, hh, image, st->_this->getAlignment());
+      }else{
+//--
+/*
+    When there was both label pixmap and label string, then
+    we have to put them together in correct places. There are
+    nine options.
+*/
+    // First two options: Label is centered. All goes as original.
+
+    if (st->_this->getAlignment() == WS_LEFT ||
+        st->_this->getAlignment() == WS_RIGHT){
+          dev->drawImage (xx, dy, ww, hh, image, st->_this->getAlignment());
+    }
+
+    // Next, the special option: Label is centered and in middle. Here,
+    // we need to center label pixmap and label string.
+
+    if (st->_this->getAlignment() == WS_CENTER){
+      dev->drawImage (xx + (ww - WSGIappFontSet()->getFont(st->font)->getStringWidth(_str)
+        - image->getImageWidth())/2, dy, ww, hh, image, WS_LEFT);
+    }
+
+    // Next, three options: Label is on top. Here, we center
+    // pixmap on bottom middle of the area, letting free the top
+    // middle of the area for label string.
+  
+    if (st->_this->getAlignment() == WS_LEFT_TOP){
+      dev->drawImage (xx, dy+(hh/2), ww, hh/2, image, WS_LEFT);
+    }else if (st->_this->getAlignment() == WS_TOP){
+      dev->drawImage (xx, dy+(hh/2), ww, hh/2, image, WS_CENTER);
+    }else if (st->_this->getAlignment() == WS_RIGHT_TOP){
+      dev->drawImage (xx, dy+(hh/2), ww, hh/2, image, WS_RIGHT);
+    }
+    // Third three options: Label is on botton. Here, we center
+    // pixmap on top middle of the area, letting free the bottom
+    // middle of the area for label string.
+
+    if (st->_this->getAlignment() == WS_LEFT_BOTTOM){
+      dev->drawImage (xx, dy, ww, hh/2, image, WS_LEFT);
+    }else if (st->_this->getAlignment() == WS_BOTTOM){
+      dev->drawImage (xx, dy, ww, hh/2, image, WS_CENTER);
+    }else if (st->_this->getAlignment() == WS_RIGHT_BOTTOM){
+      dev->drawImage (xx, dy, ww, hh/2, image, WS_RIGHT);
+    }
+//--
+      }
     }
   }else{
     short dy=yy;
@@ -348,7 +462,55 @@ void WSCvbtn_draw_mdr1(void* ptr){
       }
     }
     if (drawfl != False){
-      dev->drawImage (xx, dy, ww, hh, image, st->_this->getAlignment());
+      if (st->pixmap_overlapped != False){
+        dev->drawImage (xx, dy, ww, hh, image, st->_this->getAlignment());
+      }else{
+//--
+/*
+    When there was both label pixmap and label string, then
+    we have to put them together in correct places. There are
+    nine options.
+*/
+
+    // First two options: Label is centered. All goes as original.
+
+    if (st->_this->getAlignment() == WS_LEFT ||
+        st->_this->getAlignment() == WS_RIGHT){
+          dev->drawImage (xx, dy, ww, hh, image, st->_this->getAlignment());
+    }
+
+    // Next, the special option: Label is centered and in middle. Here,
+    // we need to center label pixmap and label string.
+
+    if (st->_this->getAlignment() == WS_CENTER){
+      dev->drawImage (xx + (ww - WSGIappFontSet()->getFont(st->font)->getStringWidth(_str)
+        - image->getImageWidth())/2, dy, ww, hh, image, WS_LEFT);
+    }
+
+    // Next, three options: Label is on top. Here, we center
+    // pixmap on bottom middle of the area, letting free the top
+    // middle of the area for label string.
+
+    if (st->_this->getAlignment() == WS_LEFT_TOP){
+      dev->drawImage (xx, dy+(hh/2), ww, hh/2, image, WS_LEFT);
+    }else if (st->_this->getAlignment() == WS_TOP){
+      dev->drawImage (xx, dy+(hh/2), ww, hh/2, image, WS_CENTER);
+    }else if (st->_this->getAlignment() == WS_RIGHT_TOP){
+      dev->drawImage (xx, dy+(hh/2), ww, hh/2, image, WS_RIGHT);
+    }
+    // Third three options: Label is on botton. Here, we center
+    // pixmap on top middle of the area, letting free the bottom
+    // middle of the area for label string.
+
+    if (st->_this->getAlignment() == WS_LEFT_BOTTOM){
+      dev->drawImage (xx, dy, ww, hh/2, image, WS_LEFT);
+    }else if (st->_this->getAlignment() == WS_BOTTOM){
+      dev->drawImage (xx, dy, ww, hh/2, image, WS_CENTER);
+    }else if (st->_this->getAlignment() == WS_RIGHT_BOTTOM){
+      dev->drawImage (xx, dy, ww, hh/2, image, WS_RIGHT);
+    }
+//--
+      }
     }
   }
 
@@ -360,8 +522,12 @@ void WSCvbtn_draw_mdr1(void* ptr){
 
   if ( stick > 0){
     char    type = WS_SHADOW_OUT;
+#ifndef WS_EMBED
     if (st->_this->getSensitive() == False ||
-        (st->_this->getPropertyEditMode() != False && st->_this->getUserData(WS_DET) == 0)) {
+        (st->_this->getPropertyEditMode() != False && st->_this->_get_sensitive_work() == False)) {
+#else
+    if (st->_this->getSensitive() == False){
+#endif
       type = WS_SHADOW_OUT;
     }else if (st->press_state == True) {
       type = WS_SHADOW_IN;
@@ -369,21 +535,29 @@ void WSCvbtn_draw_mdr1(void* ptr){
 
     short pbg = WSGFgetParentBackColor(st->_this);
     if (specialfocus == False ||st->focus_border == False) {
+#ifndef NO_GRADATION
       if (st->gradation == 0){
+#endif
         WSGFdrawModern1BtnShadow(dev, type, stick, tsno, tsno, bsno, bgc, pbg,
                                0, 0, w, h,False,False);
+#ifndef NO_GRADATION
       }else{
         WSGFdrawModern1BtnShadow(dev, type, stick, tsno, tsno, bsno, bgc, pbg,
                                0, 0, w, h,False,True);
       }
+#endif
     }else{
+#ifndef NO_GRADATION
       if (st->gradation == 0){
+#endif
         WSGFdrawModern1BtnShadow(dev, type, stick, tsno, tsno, bsno, bgc, pbg,
                                1, 1, w-2, h-2,False,False);
+#ifndef NO_GRADATION
       }else{
         WSGFdrawModern1BtnShadow(dev, type, stick, tsno, tsno, bsno, bgc, pbg,
                                1, 1, w-2, h-2,False,True);
       }
+#endif
     }
     if (specialfocus != False && st->focus_border != False ){
       dev->setForeColor(WS_DF_DARKBOTTOMSHADOWCOLOR);
@@ -393,8 +567,23 @@ void WSCvbtn_draw_mdr1(void* ptr){
     }
   }
 
+  if (st->shortcut_string != NULL){
+    WSCstring _accel_char(st->shortcut_string);
+    _accel_char.to_lower();
+    WSCstring _accel_str(*_str);
+    _accel_str.to_lower();
+    long _char_pos = _accel_str.isExist(_accel_char);
+    if(_char_pos > -1 ){
+      dev->setUnderLineChar(_char_pos,1);
+    }
+  }
+
+#ifndef WS_EMBED
   if (st->_this->getSensitive() == False ||
-      (st->_this->getPropertyEditMode() != False && (long)st->_this->getUserData(WS_DET) == 0)){
+      (st->_this->getPropertyEditMode() != False && (long)st->_this->_get_sensitive_work() == False)){
+#else
+  if (st->_this->getSensitive() == False){
+#endif
     if (st->emboss != False){
 
       if (st->blinking  != False &&
@@ -412,7 +601,7 @@ void WSCvbtn_draw_mdr1(void* ptr){
           } else {
             dev->setForeColor(tsno);
           }
-          dev->drawString(xx+1, yy+1, ww, hh, &str, st->font, st->_this->getAlignment(), -1, False);
+          dev->drawString(xx+1, yy+1, ww, hh, _str, st->font, st->_this->getAlignment(), -1, False);
         }
       }
     }else{
@@ -427,8 +616,12 @@ void WSCvbtn_draw_mdr1(void* ptr){
         st->twin_blink != False) {
       dev->setForeColor(st->back_blink_color);
     } else {
+#ifndef WS_EMBED
       if( st->emboss != False && (st->_this->getSensitive() == False ||
-          (st->_this->getPropertyEditMode() != False && (long)st->_this->getUserData(WS_DET) == 0))){
+          (st->_this->getPropertyEditMode() != False && (long)st->_this->_get_sensitive_work() == False))){
+#else
+      if( st->emboss != False && st->_this->getSensitive() == False){
+#endif
         dev->setForeColor(bsno);
       }else{
         dev->setForeColor(bgno);
@@ -441,8 +634,12 @@ void WSCvbtn_draw_mdr1(void* ptr){
         st->twin_blink != False) {
       dev->setForeColor(st->blink_color);
     }else{
+#ifndef WS_EMBED
       if ( st->emboss != False && (st->_this->getSensitive() == False ||
-        (st->_this->getPropertyEditMode() != False && (long)st->_this->getUserData(WS_DET) == 0))){
+        (st->_this->getPropertyEditMode() != False && (long)st->_this->_get_sensitive_work() == False))){
+#else
+      if ( st->emboss != False && st->_this->getSensitive() == False){
+#endif
         dev->setForeColor(bsno);
       }else{
         dev->setForeColor(fgno);
@@ -455,19 +652,128 @@ void WSCvbtn_draw_mdr1(void* ptr){
       st->twin_blink == False ){
       //do nothing...
   }else{
-    if (st->press_state == True) {
-      dev->drawString(xx, yy+1, ww, hh, &str,  st->font, st->_this->getAlignment(), -1, False);
+    if (st->pixmap_overlapped != False){
+      if (st->press_state == True) {
+        dev->drawString(xx, yy+1, ww, hh, _str,  st->font, st->_this->getAlignment(), -1, False);
+      }else{
+        dev->drawString(xx, yy, ww, hh, _str, st->font, st->_this->getAlignment(), -1, False);
+      }
     }else{
-      dev->drawString(xx, yy, ww, hh, &str, st->font, st->_this->getAlignment(), -1, False);
+/*
+    When there was both label pixmap and label string, then
+    we have to put them together in correct places. There are
+    several options.
+*/
 
+   if (image){
+
+    int new_xx, new_yy, new_ww, new_hh, new_alignment;
+
+    new_alignment = st->_this->getAlignment();
+
+    switch (st->_this->getAlignment())
+    {
+
+    // When label is in middle, we draw string at left, right or over
+    // the label pixmap.
+
+      case WS_LEFT:
+       new_xx = xx + image->getImageWidth();
+       new_yy = yy;
+       new_ww = ww - image->getImageWidth();
+       new_hh = hh;
+       break;
+      case WS_CENTER:
+       new_xx = xx + (ww - WSGIappFontSet()->getFont(st->font)->getStringWidth(_str)
+        - image->getImageWidth())/2 + image->getImageWidth();
+       new_yy = yy;
+       new_ww = ww;
+       new_hh = hh;
+       new_alignment = WS_LEFT;
+       break;
+      case WS_RIGHT:
+       new_xx = xx;
+       new_yy = yy;
+       new_ww = ww - image->getImageWidth();
+       new_hh = hh;
+       break;
+
+    // When label is on top, we draw string inside the
+    // top middle area.
+
+      case WS_LEFT_TOP:
+       new_xx = xx;
+       new_yy = yy;
+       new_ww = ww;
+       new_hh = hh/2;
+       new_alignment = WS_LEFT;
+       break;
+      case WS_TOP:
+       new_xx = xx;
+       new_yy = yy;
+       new_ww = ww;
+       new_hh = hh/2;
+       new_alignment = WS_CENTER;
+       break;
+      case WS_RIGHT_TOP:
+       new_xx = xx;
+       new_yy = yy;
+       new_ww = ww;
+       new_hh = hh/2;
+       new_alignment = WS_RIGHT;
+       break;
+
+    // When label is on bottom, we draw string inside the
+    // bottom middle area.
+
+      case WS_LEFT_BOTTOM:
+       new_xx = xx;
+       new_yy = yy + (hh/2);
+       new_ww = ww;
+       new_hh = hh/2;
+       new_alignment = WS_LEFT;
+       break;
+      case WS_BOTTOM:
+       new_xx = xx;
+       new_yy = yy + (hh/2);
+       new_ww = ww;
+       new_hh = hh/2;
+       new_alignment = WS_CENTER;
+       break;
+      case WS_RIGHT_BOTTOM:
+       new_xx = xx;
+       new_yy = yy + (hh/2);
+       new_ww = ww;
+       new_hh = hh/2;
+       new_alignment = WS_RIGHT;
+       break;
+
+      default:
+        break;
+    }
+        if (st->press_state == True) {
+          dev->drawString(new_xx, new_yy+1, new_ww, new_hh, _str,  st->font, new_alignment, -1, False);
+        }else{
+          dev->drawString(new_xx, new_yy, new_ww, new_hh, _str, st->font, new_alignment, -1, False);
+        }
+
+  }else{
+    if (st->press_state == True) {
+      dev->drawString(xx, yy+1, ww, hh, _str,  st->font, st->_this->getAlignment(), -1, False);
+    }else{
+      dev->drawString(xx, yy, ww, hh, _str, st->font, st->_this->getAlignment(), -1, False);
+
+    }
+  }
+//--
     }
 
     // draw dashed focused line...
     if (st->_this->getFocus() != False && st->enable_focus_move != False) {
       if (w > 4 + stick*2 && h > 4 + stick*2) {
-        // 褹ϣ˸
+        // 褹ϣ˸...
         dev->setLineWidth(0);
-        // 褹˸
+        // 褹˸...
         dev->setLineDashType(8);
         if (st->press_state == True) {
           //1 dot below
@@ -504,14 +810,18 @@ void WSCform_draw_mdr1(void* ptr){
   st->_this->setAbsoluteDraw(False);
 
   if (st->back_pixmap == -1){
+#ifndef NO_GRADATION
     if (st->gradation == 0){
+#endif
       dev->setForeColor(st->back_color);
       dev->drawFillRect(0,0,st->width,st->height);
       dev->setForeColor(st->back_color);
 
+#ifndef NO_GRADATION
     }else{
       dev->drawGradation(st->gradation,st->top_shadow_color,st->back_color,st->bottom_shadow_color,0,0,st->width,st->height,st->gradation_margin);
     }
+#endif
   }
   if (st->_this->getDotMode() == True){
     dev->setForeColor(st->fore_color);
@@ -581,14 +891,17 @@ void WSCvtoggle_draw_mdr1(void* ptr){
 
   char* _label_string = dt->label_string;
   char* _select_string = dt->select_string;
+  char* _shortcut_string = dt->shortcut_string;
   short _label_pixmap = dt->label_pixmap;
   short _blink_pixmap = dt->blink_pixmap;
   short _select_color = dt->select_color;
   short _select_pixmap = dt->select_pixmap;
   short _press_status = dt->press_status;
   long  _orientation = dt->orientation;
+#ifndef NO_GRADATION
   long  _grad_no = dt->gradation;
   long  _grad_margin = dt->gradation_margin;
+#endif
   long  _margin_left = dt->margin_left;
   long  _margin_right = dt->margin_right;
   long  _margin_top = dt->margin_top;
@@ -609,15 +922,15 @@ void WSCvtoggle_draw_mdr1(void* ptr){
 
   WSCstring str;
   if (_press_status == False){
-    str.setString( _label_string );
+    str.setStaticString( _label_string );
   }else{
     if (_select_string == NULL){
-      str.setString( _label_string );
+      str.setStaticString( _label_string );
     }else{
       if (_select_string[0] == 0 ){
-        str.setString( _label_string );
+        str.setStaticString( _label_string );
       }else{
-        str.setString( _select_string );
+        str.setStaticString( _select_string );
       }
     }
   }
@@ -652,8 +965,11 @@ void WSCvtoggle_draw_mdr1(void* ptr){
       dev->setForeColor(bgno);
     }
   }
+#ifndef NO_GRADATION
   if (_grad_no == 0){
+#endif
     dev->drawFillRect(0, 0, w, h);
+#ifndef NO_GRADATION
   }else{
     if (_press_status == True  && _indicator_on == False){
       dev->drawGradation(_grad_no,tsno,bgno,bsno,0, 1, w, h,_grad_margin);
@@ -661,7 +977,7 @@ void WSCvtoggle_draw_mdr1(void* ptr){
       dev->drawGradation(_grad_no,tsno,bgno,bsno,0, 0, w, h,_grad_margin);
     }
   }
-
+#endif
   short offsetx = 0;
   short left_margin = _margin_left;
   short right_margin = _margin_right;
@@ -723,17 +1039,37 @@ void WSCvtoggle_draw_mdr1(void* ptr){
       type = WS_SHADOW_IN;
     }
     short pbg = WSGFgetParentBackColor(_this);
+#ifndef NO_GRADATION
     if (_grad_no == 0){
+#endif
       WSGFdrawModern1BtnShadow(dev, type, st, _fore_color, tsno, bsno,bgno,pbg,
                              0, 0, w, h,False,False);
+#ifndef NO_GRADATION
     }else{
       WSGFdrawModern1BtnShadow(dev, type, st, _fore_color, tsno, bsno,bgno,pbg,
                              0, 0, w, h,False,True);
     }
+#endif
   }
 
+  if (_shortcut_string != NULL){
+    WSCstring _accel_char(_shortcut_string);
+    _accel_char.to_lower(); 
+
+    WSCstring _accel_str(str);
+    _accel_str.to_lower();
+    long _char_pos = _accel_str.isExist(_accel_char);
+    if(_char_pos > -1 ){
+      dev->setUnderLineChar(_char_pos,1);
+    }
+  }
+
+#ifndef WS_EMBED
   if (_this->getSensitive() == False ||
-      (_this->getPropertyEditMode() != False && (long)_this->getUserData(WS_DET) == 0)){
+      (_this->getPropertyEditMode() != False && (long)_this->_get_sensitive_work() == False)){
+#else
+  if (_this->getSensitive() == False){
+#endif
     if (_emboss != False){
       if (_bl_fl != False &&
         blinkfore != False &&
@@ -773,8 +1109,12 @@ void WSCvtoggle_draw_mdr1(void* ptr){
         _tw_fl == True){
       dev->setForeColor(_bg_blink_color);
     }else{
+#ifndef WS_EMBED
       if ( _emboss != False && (_this->getSensitive() == False ||
-          (_this->getPropertyEditMode() != False && (long)_this->getUserData(WS_DET) == 0))){
+          (_this->getPropertyEditMode() != False && (long)_this->_get_sensitive_work() == False))){
+#else
+      if ( _emboss != False && _this->getSensitive() == False){
+#endif
         dev->setForeColor(_bs_color);
       } else {
         dev->setForeColor(_back_color);
@@ -787,8 +1127,12 @@ void WSCvtoggle_draw_mdr1(void* ptr){
         _tw_fl == True){
       dev->setForeColor(_blink_color);
     }else{
+#ifndef WS_EMBED
       if ( _emboss != False && (_this->getSensitive() == False ||
-          (_this->getPropertyEditMode() != False && (long)_this->getUserData(WS_DET) == 0))){
+          (_this->getPropertyEditMode() != False && (long)_this->_get_sensitive_work() == False))){
+#else
+      if ( _emboss != False && _this->getSensitive() == False){
+#endif
          dev->setForeColor(_bs_color);
       }else{
          dev->setForeColor(_fore_color);
@@ -802,7 +1146,7 @@ void WSCvtoggle_draw_mdr1(void* ptr){
         _bl_type != WS_BACK &&
         blinkfore != False &&
         _tw_fl == False){
-        // 褷ʤ
+        // 褷ʤ...
     }else{
 
 
@@ -814,8 +1158,8 @@ void WSCvtoggle_draw_mdr1(void* ptr){
                       &str, _font, _this->getAlignment(), -1, False);
       }
 
-      //եΥ饤
-      // եưݤĤξΤ
+      //եΥ饤...
+      // եưݤĤξΤ...
       if (_this->getFocus() == True && _enable_focus_move == True){
         dev->setLineWidth( 1 );
         dev->setLineDashType( 8 );
@@ -842,7 +1186,7 @@ void WSCvtoggle_draw_mdr1(void* ptr){
   }
 
   if (_indicator_on == True){
-    //󥸥κɸη׻
+    //󥸥κɸη׻...
     char itype = _indicator_type;
     WSCushort isize = _indicator_size;
     short ypos = (h - isize)/2;
@@ -851,7 +1195,7 @@ void WSCvtoggle_draw_mdr1(void* ptr){
         ypos = 0;
     }
 
-    //󥸥طʤ
+    //󥸥طʤ...
     WSDimage* image;
     if (_press_status == True ){
       if (_select_pixmap == -1){
@@ -880,15 +1224,19 @@ void WSCvtoggle_draw_mdr1(void* ptr){
       if  ( itype == WS_IN){
         type = WS_SHADOW_IN;
       }
+#ifndef NO_GRADATION
       if (_grad_no == 0){
+#endif
         WSGFdrawModern1BtnShadow(dev, type, _indicator_shadow_thick,
                      _fore_color, _ts_color, _bs_color,_back_color,_back_color,
                      xpos, ypos , isize, isize,False,False);
+#ifndef NO_GRADATION
       }else{
         WSGFdrawModern1BtnShadow(dev, type, _indicator_shadow_thick,
                      _fore_color, _ts_color, _bs_color,_back_color,_back_color,
                      xpos, ypos , isize, isize,False,True);
       }
+#endif
     }
   }
 
@@ -930,6 +1278,7 @@ void WSCvradio_draw_mdr1(void* ptr){
   short _bs_color    = dt->bottom_shadow_color;
 
   char* _label_string = dt->label_string;
+  char* _shortcut_string = dt->shortcut_string;
   char* _select_string = dt->select_string;
   short _label_pixmap = dt->label_pixmap;
   short _blink_pixmap = dt->blink_pixmap;
@@ -937,8 +1286,10 @@ void WSCvradio_draw_mdr1(void* ptr){
   short _select_pixmap = dt->select_pixmap;
   short _press_status = dt->press_status;
   long  _orientation = dt->orientation;
+#ifndef NO_GRADATION
   long  _grad_no = dt->gradation;
   long  _grad_margin = dt->gradation_margin;
+#endif
   long  _margin_left = dt->margin_left;
   long  _margin_right = dt->margin_right;
   long  _margin_top = dt->margin_top;
@@ -960,15 +1311,15 @@ void WSCvradio_draw_mdr1(void* ptr){
 
   WSCstring str;
   if (_press_status == False){
-    str.setString( _label_string );
+    str.setStaticString( _label_string );
   }else{
     if (_select_string == NULL){
-      str.setString( _label_string );
+      str.setStaticString( _label_string );
     }else{
       if (_select_string[0] == 0 ){
-        str.setString( _label_string );
+        str.setStaticString( _label_string );
       }else{
-        str.setString( _select_string );
+        str.setStaticString( _select_string );
       }
     }
   }
@@ -982,7 +1333,6 @@ void WSCvradio_draw_mdr1(void* ptr){
   if (err != WS_NO_ERR){
     return;
   }
-
   _this->WSCbase::update();
   _this->setAbsoluteDraw(False);
 
@@ -1004,8 +1354,11 @@ void WSCvradio_draw_mdr1(void* ptr){
     }
   }
 
+#ifndef NO_GRADATION
   if (_grad_no == 0){
+#endif
     dev->drawFillRect(0, 0, w, h);
+#ifndef NO_GRADATION
   }else{
     if (_press_status == True  && _indicator_on == False){
       dev->drawGradation(_grad_no,tsno,bgno,bsno,0, 1, w, h,_grad_margin);
@@ -1013,7 +1366,7 @@ void WSCvradio_draw_mdr1(void* ptr){
       dev->drawGradation(_grad_no,tsno,bgno,bsno,0, 0, w, h,_grad_margin);
     }
   }
-
+#endif
   short offsetx = 0;
   short left_margin = _margin_left;
   short right_margin = _margin_right;
@@ -1067,8 +1420,25 @@ void WSCvradio_draw_mdr1(void* ptr){
       dev->drawImage (sx, dy, sw, sh, image, _this->getAlignment());
     }
   }
+
+  if (_shortcut_string != NULL){
+    WSCstring _accel_char(_shortcut_string);
+    _accel_char.to_lower();
+
+    WSCstring _accel_str(str);
+    _accel_str.to_lower();
+    long _char_pos = _accel_str.isExist(_accel_char);
+    if(_char_pos > -1 ){
+      dev->setUnderLineChar(_char_pos,1);
+    }
+  }
+
+#ifndef WS_EMBED
   if (_this->getSensitive() == False ||
-      (_this->getPropertyEditMode() != False && (long)_this->getUserData(WS_DET) == 0)){
+      (_this->getPropertyEditMode() != False && (long)_this->_get_sensitive_work() == False)){
+#else
+  if (_this->getSensitive() == False){
+#endif
     if (_emboss != False){
       if (_bl_fl != False &&
         blinkfore != False &&
@@ -1107,8 +1477,12 @@ void WSCvradio_draw_mdr1(void* ptr){
         _tw_fl == True){
       dev->setForeColor(_bg_blink_color);
     }else{
+#ifndef WS_EMBED
       if ( _emboss != False && (_this->getSensitive() == False ||
-          (_this->getPropertyEditMode() != False && (long)_this->getUserData(WS_DET) == 0))){
+          (_this->getPropertyEditMode() != False && (long)_this->_get_sensitive_work() == False))){
+#else
+      if ( _emboss != False && _this->getSensitive() == False){
+#endif
         dev->setForeColor(_bs_color);
       } else {
         dev->setForeColor(_back_color);
@@ -1121,8 +1495,12 @@ void WSCvradio_draw_mdr1(void* ptr){
         _tw_fl == True){
       dev->setForeColor(_blink_color);
     }else{
+#ifndef WS_EMBED
       if ( _emboss != False && (_this->getSensitive() == False ||
-          (_this->getPropertyEditMode() != False && (long)_this->getUserData(WS_DET) == 0))){
+          (_this->getPropertyEditMode() != False && (long)_this->_get_sensitive_work() == False))){
+#else
+      if ( _emboss != False && _this->getSensitive() == False){
+#endif
          dev->setForeColor(_bs_color);
       }else{
          dev->setForeColor(_fore_color);
@@ -1135,7 +1513,7 @@ void WSCvradio_draw_mdr1(void* ptr){
         _bl_type != WS_BACK &&
         blinkfore != False &&
         _tw_fl == False){
-        // 褷ʤ
+        // 褷ʤ...
     }else{
 
 
@@ -1147,8 +1525,8 @@ void WSCvradio_draw_mdr1(void* ptr){
                       &str, _font, _this->getAlignment(), -1, False);
       }
 
-      //եΥ饤
-      // եưݤĤξΤ
+      //եΥ饤...
+      // եưݤĤξΤ...
       if (_this->getFocus() == True && _enable_focus_move == True){
         dev->setLineWidth( 1 );
         dev->setLineDashType( 8 );
@@ -1251,7 +1629,6 @@ void WSCvradio_draw_mdr1(void* ptr){
 
         }
       }
-
       WSGFdrawModern1Shadow(dev, WS_SHADOW_IN, _shadow_thick,
                    _fore_color, _ts_color, _bs_color,_back_color,_back_color,
                    xpos, ypos , isize, isize,False);
@@ -1315,8 +1692,10 @@ void WSCoption_draw_mdr1(void* ptr){
   short _label_pixmap = dt->label_pixmap;
   short _blink_pixmap = dt->blink_pixmap;
   short _press_state = dt->press_state;
+#ifndef NO_GRADATION
   long  _grad_no = dt->gradation;
   long  _grad_margin = dt->gradation_margin;
+#endif
   long  _margin_left = dt->margin_left;
   long  _margin_right = dt->margin_right;
   long  _margin_top = dt->margin_top;
@@ -1386,7 +1765,9 @@ void WSCoption_draw_mdr1(void* ptr){
       dev->setBackColor(fgno);
     }
   }
+#ifndef NO_GRADATION
   if (_grad_no == 0){
+#endif
     short ts1 = WSGFgetFixedColor(bgno,32);
     short bs1 = WSGFgetFixedColor(bgno,-32);
 
@@ -1403,10 +1784,11 @@ void WSCoption_draw_mdr1(void* ptr){
       dev->drawFillRect(0, 0, w, h);
     }
 //    dev->drawFillRect(st, st, w-st*2, h-st*2);
+#ifndef NO_GRADATION
   }else{
     dev->drawGradation(_grad_no,tsno,bgno,bsno,st, st, w-st*2, h-st*2,_grad_margin);
   }
-
+#endif
   WSDimage* image;
   if (bldraw && blinktype != WS_BACK ){
     if (_blink_pixmap != -1 && twinblink != False){
@@ -1425,22 +1807,34 @@ void WSCoption_draw_mdr1(void* ptr){
     if (_press_state != False) {
       type = WS_SHADOW_IN;
     }
+#ifndef WS_EMBED
     if (_this->getSensitive() == False ||
-       (_this->getPropertyEditMode() != False && (long)_this->getUserData(WS_DET) == 0)){
+       (_this->getPropertyEditMode() != False && (long)_this->_get_sensitive_work() == False)){
+#else
+    if (_this->getSensitive() == False){
+#endif
       type = WS_SHADOW_OUT;
     }
     short pbgno = WSGFgetParentBackColor(_this);
+#ifndef NO_GRADATION
     if (_grad_no == 0){
+#endif
       WSGFdrawModern1BtnShadow(dev, type, st, fgno, tsno, bsno, bgno, pbgno,
                           0, 0, w, h, False,False);
+#ifndef NO_GRADATION
     }else{
       WSGFdrawModern1BtnShadow(dev, type, st, fgno, tsno, bsno, bgno, pbgno,
                           0, 0, w, h, False,True);
     }
+#endif
   }
 
+#ifndef WS_EMBED
   if ( _this->getSensitive() == False ||
-       (_this->getPropertyEditMode() != False && (long)_this->getUserData(WS_DET) == 0)){
+       (_this->getPropertyEditMode() != False && (long)_this->_get_sensitive_work() == False)){
+#else
+  if ( _this->getSensitive() == False){
+#endif
     if (_emboss != False){
       if (bldraw && blinktype != WS_BACK && twinblink != False) {
         dev->setForeColor(blno);
@@ -1466,8 +1860,12 @@ void WSCoption_draw_mdr1(void* ptr){
     if (bldraw && blinktype != WS_BACK && twinblink != False) {
       dev->setForeColor(blbgno);
     } else {
+#ifndef WS_EMBED
       if( _emboss != False && (_this->getSensitive() == False ||
-          (_this->getPropertyEditMode() != False && (long)_this->getUserData(WS_DET) == 0))){
+          (_this->getPropertyEditMode() != False && (long)_this->_get_sensitive_work() == False))){
+#else
+      if( _emboss != False && _this->getSensitive() == False){
+#endif
         dev->setForeColor(bsno);
       }else{
         dev->setForeColor(bgno);
@@ -1478,8 +1876,12 @@ void WSCoption_draw_mdr1(void* ptr){
     if(bldraw && blinktype != WS_BACK && twinblink != False) {
       dev->setForeColor(blno);
     }else{
+#ifndef WS_EMBED
       if ( _emboss != False && (_this->getSensitive() == False ||
-        (_this->getPropertyEditMode() != False && (long)_this->getUserData(WS_DET) == 0))){
+        (_this->getPropertyEditMode() != False && (long)_this->_get_sensitive_work() == False))){
+#else
+      if ( _emboss != False && _this->getSensitive() == False){
+#endif
         dev->setForeColor(bsno);
       }else{
         dev->setForeColor(fgno);
@@ -1598,7 +2000,7 @@ void WSCpopupFrame_draw_mdr1(void* ptr){
          py < _y + posy + _item_height){
       _this->set_value(0);
       WSCstring sect_value = sect.getWord(1, "\n");
-      _this->set_value(atoi(sect_value.getString()));
+      _this->set_value(WSGFatol(sect_value.getString()));
       _this->set_value_effective(True);
     }else{
       WSGFdrawShadow(dev, _shadow_type, st,
