@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSClocaleSet.h>
@@ -70,41 +66,48 @@ void _libws_lang_init_proc5(){
 void _libws_lang_init_proc6(){
 #ifndef _WSWIN32
 #include "Espanol.cpp"
+#include "Espanol-ar.cpp"
+#endif
+}
+#endif
+#ifndef NO_ES_UTF8_MSG
+void _libws_lang_init_proc7(){
+#ifndef _WSWIN32
+#include "Espanol-utf8.cpp"
+#include "Espanol-ar-utf8.cpp"
 #endif
 }
 #endif
 
-
-class _libws_lang_init_{
-public:
-  _libws_lang_init_(){
+void WSGFenablePropertyBuilderTitle(){
 #ifndef NO_ENGLISH_MSG
-  WSGIappInitializeManager()->addInitializeProc(_libws_lang_init_proc);
+  _libws_lang_init_proc();
 #endif
 #ifndef NO_JAPANESE_EUC_MSG
-  WSGIappInitializeManager()->addInitializeProc(_libws_lang_init_proc1);
+  _libws_lang_init_proc1();
 #endif
 #ifndef NO_JAPANESE_SJIS_MSG
-  WSGIappInitializeManager()->addInitializeProc(_libws_lang_init_proc2);
+  _libws_lang_init_proc2();
 #endif
 #ifndef NO_JAPANESE_UTF8_MSG
-  WSGIappInitializeManager()->addInitializeProc(_libws_lang_init_proc3);
+  _libws_lang_init_proc3();
 #endif
 #ifndef NO_KSC5601_MSG
-  WSGIappInitializeManager()->addInitializeProc(_libws_lang_init_proc4);
+  _libws_lang_init_proc4();
 #endif
 #ifndef NO_GB2312_MSG
-  WSGIappInitializeManager()->addInitializeProc(_libws_lang_init_proc5);
+  _libws_lang_init_proc5();
 #endif
 #ifndef NO_ES_MSG
-  WSGIappInitializeManager()->addInitializeProc(_libws_lang_init_proc6);
+  _libws_lang_init_proc6();
+#endif
+#ifndef NO_ES_UTF8_MSG
+  _libws_lang_init_proc7();
 #endif
-  };
 };
 
 #ifdef WS_EMBED
 //dummy reference to link
 void _libws_lang(){}
 #endif
-static _libws_lang_init_ _run_libws_lang_init;
-
+//static _libws_lang_init_ _run_libws_lang_init;
