@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCfileSelect.h>
@@ -31,7 +27,7 @@
 #include <WSDdev.h>
 #include <WSCscrForm.h>
 #include <WSCvkflabel.h>
-#include <WSCoption.h>
+//#include <WSCoption.h>
 #include <WSCvtoggle.h>
 #include <WSCvifield.h>
 #include <WSCvbtn.h>
@@ -171,15 +167,21 @@ void WSCfileSelect::setWorkWSNdirName(char* data){
   char buf[2048];
   char buf2[2048];
   if (_dir_if != NULL){
-    if (data[0] == '.' && data[1] !=  '.'){
-      GETCWD(buf,2048);
-      sprintf(buf2,"%s%s",buf,&data[1]);
-      data = buf2;
-    }else if (data[0] == '.' && data[1] ==  '.'){
-      GETCWD(buf,2048);
-      sprintf(buf2,"%s/%s",buf,data);
-      data = buf2;
+#ifndef WS_EMBED
+    if (getPropertyEditMode() == False){
+#endif
+      if (data[0] == '.' && data[1] !=  '.'){
+        GETCWD(buf,2048);
+        sprintf(buf2,"%s%s",buf,&data[1]);
+        data = buf2;
+      }else if (data[0] == '.' && data[1] ==  '.'){
+        GETCWD(buf,2048);
+        sprintf(buf2,"%s/%s",buf,data);
+        data = buf2;
+      }
+#ifndef WS_EMBED
     }
+#endif
     WSCstring tmp(data);
     tmp.replaceString("\\","/",0);
     _dir_if->setPropertyV(WSNlabelString,tmp.getString());
@@ -190,7 +192,7 @@ void WSCfileSelect::setWorkWSNdirName(char* data){
 }
 void WSCfileSelect::getWorkWSNdirName(char** data){
   if (_dir_if != NULL){
-    delete _dir_name;
+    delete[] _dir_name;
     char* dt;
     _dir_if->getPropertyV(WSNlabelString,&dt);
     WSCstring str(dt);
@@ -210,7 +212,7 @@ void WSCfileSelect::setWorkWSNfileName(char* data){
 }
 void WSCfileSelect::getWorkWSNfileName(char** data){
   if (_file_if != NULL){
-    delete _file_name;
+    delete[] _file_name;
     char* dt;
     _file_if->getPropertyV(WSNlabelString,&dt);
     WSCstring str(dt);
@@ -231,7 +233,7 @@ void WSCfileSelect::setWorkWSNmaskFileName(char* data){
 }
 void WSCfileSelect::getWorkWSNmaskFileName(char** data){
   if (_mask_if != NULL){
-    delete _mask_name;
+    delete[] _mask_name;
     char* dt;
     _mask_if->getPropertyV(WSNlabelString,&dt);
     _mask_name = WSGFstrdup(dt);
@@ -379,7 +381,7 @@ void WSCfileSelect::_update_list(){
         flabel->setUserData("FILE",(void*)flg);
         flabel->setVisible(True);
 //        flabel->redraw();
-        delete (char*)dir_list->getData(i);
+        delete[] (char*)dir_list->getData(i);
       }
       delete dir_list;
     }
@@ -399,13 +401,22 @@ void WSCfileSelect::_update_list(){
       }
 #endif
       if (file_list != NULL){
-        num = file_list->getNum();
         WSCstring tmp(getProperty(WSNmaskFileName));
-        tmp.delHeadSpace();
-        tmp.delTailSpace();
-        tmp.delLineFeed();
-        WSCstring mask(".");
-        mask.addString(tmp);
+
+        WSClistData mask_list;
+        num = tmp.getWords(",");
+        for(i=0; i<num; i++){
+          WSCstring str1 = tmp.getWord(i,",");
+          str1.delHeadSpace();
+          str1.delTailSpace();
+          str1.delLineFeed();
+          if (!strcmp((char*)str1,"")){
+            continue;
+          }
+          WSCstring* mask = new WSCstring(".");
+          mask->addString(str1);
+          mask_list.add((void*)mask);
+        }
         num = file_list->getNum();
         for(i=0; i<num; i++){
           if (i+dnum > WS_FS_MAX_FILE_COUNTS){
@@ -413,26 +424,40 @@ void WSCfileSelect::_update_list(){
           }
           char* filename = (char*)file_list->getData(i);
           WSCstring tmp2(filename);
-          long pos = tmp2.isExist(mask.getString());
-          if (strcmp(tmp.getString(),"")){
+          WSCbool hit = False;
+          int j;
+          int masks = mask_list.getNum();
+          if (masks == 0){
+            hit = True;
+          }
+          for(j=0; j<masks; j++){
+            WSCstring mask = *(WSCstring*)mask_list[j];
+            long pos = tmp2.isExist(mask.getString());
+            if (strcmp(tmp.getString(),"")){
 #ifdef MSW
-            if ( pos == -1){
-              WSCstring mask2(mask);
-              mask2.to_upper();
-              pos = tmp2.isExist(mask2.getString());
-            }
+              if ( pos == -1){
+                WSCstring mask2(mask);
+                mask2.to_upper();
+                pos = tmp2.isExist(mask2.getString());
+              }
 #endif
-            if ( pos == -1){
-              continue;
-            }else{
-              WSCstring mask2(mask.getString());
-              mask2.addString(".");
-              // except for back files as XXX.mask.bk,etc..
-              if (tmp2.isExist(mask2.getString()) != -1){
+              if ( pos == -1){
                 continue;
+              }else{
+                WSCstring mask2(mask.getString());
+                mask2.addString(".");
+                // except for back files as XXX.mask.bk,etc..
+                if (tmp2.isExist(mask2.getString()) != -1){
+                  continue;
+                }
+                hit = True;
+                break;
               }
             }
           }
+          if (hit == False){
+            continue;
+          }
           WSCbase* icon  = (WSCbase*)_icon_list.getData(cnt+dnum);
           WSCbase* flabel = (WSCbase*)_label_list.getData(cnt+dnum);
           if (icon == NULL){
@@ -490,9 +515,14 @@ void WSCfileSelect::_update_list(){
           cnt++;
         }
         for(i=0; i<num; i++){
-          delete (char*)file_list->getData(i);
+          delete[] (char*)file_list->getData(i);
         }
         delete file_list;
+
+        num = mask_list.getNum();
+        for(i=0; i<num; i++){
+          delete (WSCstring*)mask_list[i];
+        }
       }
     }
   }
@@ -595,9 +625,13 @@ static long      bk_time = 0;
        void* hdp;
        fs->getPropertyV(WSNreplyHandler,&hdp);
        hd = (void(*)(WSCfileSelect*))hdp;
+#ifndef WS_EMBED
        if (fs->getPropertyEditMode() == False){
          fs->setVisible(False);
        }
+#else
+       fs->setVisible(False);
+#endif
        fs->_status = WS_DIALOG_OK;
        if (hd != NULL){
          hd(fs);
@@ -831,9 +865,9 @@ void WSCfileSelect::_mkdir_btn_work(WSCbase* obj){
 
     if (ret != WS_NO_ERR){
       WSCstring message;
-      message << WSGFgettext("F:WSSTRCannot create directory:\n");
+      message << WSGFgettext("F:WSSTRCannot create directory:");
       message << fs->_mkdir_dlg->getProperty(WSNlabelString);
-      message << WSGFgettext("G:WSSTRCannot create directory:\n");
+      message << WSGFgettext("G:WSSTR.");
 
       if (modal != False){
         fs->setProperty(WSNmodal,False);
@@ -1396,8 +1430,43 @@ void WSCfileSelect::onVisibleChange(WSCbool fl){
       tlb->setFocus();
     }
     _ok_btn->setSpecialFocus();
-    _need_update_fl = True; //for win32
-    needUpdate();
+    switch(_focus_btn){
+      case WS_DIALOG_DEFAULT:{
+        if (getVisible() != False){
+          if (_ok != False){
+            _ok_btn->setSpecialFocus();
+          }else
+          if (_no != False){
+            _no_btn->setSpecialFocus();
+          }else{
+            _end_btn->setSpecialFocus();
+          }
+        }
+        break;
+      }
+      case WS_DIALOG_CANCEL:{
+        if (getVisible() != False){
+          _end_btn->setSpecialFocus();
+        }
+        break;
+      }
+      case WS_DIALOG_NO:{
+        if (getVisible() != False){
+          if (_no != False){
+            _no_btn->setSpecialFocus();
+          }
+        }
+        break;
+      }
+      case WS_DIALOG_OK:{
+        if (getVisible() != False){
+          if (_ok != False){
+            _ok_btn->setSpecialFocus();
+          }
+        }
+        break;
+      }
+    }
   }
 }
 char* WSCfileSelect::getFileName(){
