@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCindexForm.h>
@@ -49,6 +45,8 @@ WSCindexForm::WSCindexForm(WSCbase* base,char* objname):
   _menu_string = WSGFstrdup("index1,index2");
   _orientation = WS_HORIZONTAL;
   _flipped = False;
+  _use_icon = False;
+  _sc_str = NULL;
 
   WSMFpropertyCreateStart
     WSMFparentCheckVerSrc(WSCindexForm);
@@ -60,7 +58,9 @@ WSCindexForm::WSCindexForm(WSCbase* base,char* objname):
     WSMFpropertyCreate(WSNvalue, WSCuchar, _value,WSSvalue);
     WSMFpropertyCreate(WSNflip, WSCbool, _flipped,WSSflip);
       WSMFpropertySetSelection(WSRbool1, WSRbool1D);
-
+    WSMFpropertyCreate(WSNuseIcon,  WSCbool, _use_icon, WSSuseIcon );
+      WSMFpropertySetSelection(WSRbool1,WSRbool1D);
+    WSMFpropertyCreate(WSNshortcut, char*, _sc_str,WSSshortcut);
 
 //    WSMFpropertyVisibleOff(WSNshadowType);
 
@@ -76,6 +76,12 @@ WSMFproperty( WSCindexForm, WSNmenuItems,  char*, _menu_string, WSGFstrdup("inde
 WSMFproperty( WSCindexForm, WSNorientation, WSCuchar, _orientation, WS_HORIZONTAL);
 WSMFproperty( WSCindexForm, WSNvalue, WSCuchar, _value, 0);
 WSMFproperty( WSCindexForm, WSNflip, WSCbool, _flipped, 0);
+WSMFproperty( WSCindexForm, WSNuseIcon, WSCbool,   _use_icon,   False);
+WSMFproperty( WSCindexForm, WSNshortcut, char*, _sc_str, NULL);
+void WSCindexForm::setWorkWSNshortcut(char* data) {
+    _need_adjust = 1;
+}
+void WSCindexForm::getWorkWSNshortcut(char**) {}
 
 
 void WSCindexForm::setWorkWSNbarThickness(WSCushort){
@@ -102,8 +108,22 @@ void WSCindexForm::setWorkWSNmenuItems(char* data){
 }
 void WSCindexForm::getWorkWSNmenuItems(char** ){}
 WSCindexForm::~WSCindexForm(){
+  if (_menu_string != NULL){
+    delete[] _menu_string;
+    _menu_string = NULL;
+  }
+  if (_sc_str != NULL){
+    delete[] _sc_str;
+    _sc_str = NULL;
+  }
 }
 
+void WSCindexForm::setWorkWSNuseIcon(WSCbool){
+  _adjust();
+}
+void WSCindexForm::getWorkWSNuseIcon(WSCbool*){}
+
+
 void WSCindexForm::_index_op(WSCbase* btn){
   WSCindexForm* obj = (WSCindexForm*)btn->getUserData("parent-form**");
   WSCulong val;
@@ -258,6 +278,10 @@ void WSCindexForm::_vis_adjust(){
 void WSCindexForm::_adjust(){
   _need_adjust = 0;
   WSCstring tmp(_menu_string);
+  WSCstring sc_str;
+  if (_sc_str != NULL){
+    sc_str = _sc_str;
+  }
   long tabs = tmp.getWords(",");
   long btns = _btns.getNum();
   long i;
@@ -290,7 +314,27 @@ void WSCindexForm::_adjust(){
     if (!strcmp(tmp2.getString(),"")){
       btn->setPropertyV(WSNlabelString,"index");
     }else{
-      btn->setPropertyV(WSNlabelString,tmp2.getString());
+      if (_use_icon == False){
+        btn->setPropertyV(WSNlabelString,tmp2.getString());
+        btn->setPropertyV(WSNpixmapOverlap,True);
+      }else{
+        WSCstring lb = tmp2.getWord(1,":");
+        WSCstring icon = tmp2.getWord(0,":");
+        btn->setPropertyV(WSNlabelString,lb.getString());
+        btn->setPropertyV(WSNlabelPixmap,icon.getString());
+        btn->setPropertyV(WSNpixmapOverlap,False);
+        if (_orientation == WS_HORIZONTAL){
+          btn->setPropertyV(WSNalignmentH,WS_CENTER);
+          btn->setPropertyV(WSNalignmentV,WS_CENTER);
+        }else{
+          btn->setPropertyV(WSNalignmentH,WS_CENTER);
+          btn->setPropertyV(WSNalignmentV,WS_BOTTOM);
+        }
+      }
+    }
+    WSCstring sc = sc_str.getWord(i,",");
+    if (strcmp(sc.getString(),"")){
+      btn->setProperty(WSNshortcut,sc);
     }
     WSCulong val;
     btn->getPropertyV(WSNuserValue,(long*)&val);
@@ -526,6 +570,18 @@ long WSCindexForm::draw(){
   if (dev == NULL){
     return WS_ERR;
   }
+
+  if (_update_resize != False){
+    _update_resize = False;
+    execResizeEvent();
+  }
+
+  if (_geometry_changed != False){
+dbprintf("WSCindexForm::draw %s:%d %s call _exec_child_adjust_for_anchors()..\n",__FILE__,__LINE__,getInstanceName());
+    _exec_child_adjust_for_anchors();
+  }
+
+
 //  if (_shadow_thick > 0){
   if (WSGIappDev()->getExtGuiPolicySelected() != False){
     if (_ext_draw_proc == NULL){
@@ -547,8 +603,10 @@ long WSCindexForm::draw(){
       st.top_shadow_color = _ts_color;
       st.bottom_shadow_color = _bs_color;
       st.back_pixmap = _bp_color;
+#ifndef NO_GRADATION
       st.gradation = _grad_no;
       st.gradation_margin = _grad_margin;
+#endif
       st.bar_thickness = _bar_thick;
       st.orientation = _orientation;
       st.flipped = _flipped;
@@ -580,13 +638,17 @@ long WSCindexForm::draw(){
     setAbsoluteDraw(False);
 
     if (_bp_color == -1){
+#ifndef NO_GRADATION
       if (_grad_no == 0){
+#endif
         dev->setForeColor(_back_color);
         dev->drawFillRect(0,0,_w,_h);
+#ifndef NO_GRADATION
       }else{
         dev->drawGradation(_grad_no,_ts_color,_back_color,_bs_color,
                             0,0,_w,_h,_grad_margin);
       }
+#endif
     }
     WSGFdrawBtnShadow(dev,_shadow_type,_shadow_thick,
                        _fore_color,_ts_color,_bs_color,_back_color,0,0,_w,_h);
@@ -659,16 +721,21 @@ long WSCindexForm::draw(){
   return WS_NO_ERR;
 }
 void(*WSCindexForm::_ext_draw_proc)(void*) = NULL;
-WSCstring WSCindexForm::_policy_name;
+char* WSCindexForm::_policy_name = NULL;
 
 void WSCindexForm::onGuiPolicyChange(long policy){
   WSCbase::onGuiPolicyChange(policy);
   if (WSGIappDev()->getExtGuiPolicySelected() == False){
     _ext_draw_proc = NULL;
   }else{
-    if (!strcmp((char*)_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
+    if (_policy_name != NULL &&
+        !strcmp(_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
     }else{
       _ext_draw_proc = (void(*)(void*))WSGIappDev()->getExtGuiPolicyProc("WSCindexForm::draw");
+      if (_policy_name != NULL){
+        delete _policy_name;
+      }
+      _policy_name = WSGFstrdup(WSGIappDev()->getExtGuiPolicyName());
     }
   }
   needUpdate();
