@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCinputDialog.h>
@@ -43,7 +39,7 @@ WSMFpropertyValueChange(WSCinputDialog,WSNno,WSCbool,False);
 WSCinputDialog::WSCinputDialog(WSCbase* base,char* objname):
                 WSCbaseDialog(base,objname){
 
-  _label_string = WSGFstrdup("");
+//  _label_string = NULL;
   _label = NULL;
   _title = NULL;
   _area1 = NULL;
@@ -51,7 +47,7 @@ WSCinputDialog::WSCinputDialog(WSCbase* base,char* objname):
   _no = False;
   WSMFpropertyCreateStart
     WSMFparentCheckVerSrc(WSCinputDialog);
-    WSMFpropertyCreate(WSNlabelString,   char*, _label_string,WSSlabelString);
+//    WSMFpropertyCreate(WSNlabelString,   char*, _label_string,WSSlabelString);
     WSMFpropertyCreate(WSNworkBackColor,  short, _wbg_no,WSSworkBackColor );
 
     WSMFpropertyValueChangeDef(WSCinputDialog,WSNno,WSCbool);
@@ -61,22 +57,23 @@ WSCinputDialog::WSCinputDialog(WSCbase* base,char* objname):
 }
 
 WSCinputDialog::~WSCinputDialog(){
-  if (_label_string != NULL){
-    delete _label_string;
-    _label_string = NULL;
-  }
+//  if (_label_string != NULL){
+//    delete _label_string;
+//    _label_string = NULL;
+//  }
 }
 
-WSMFproperty( WSCinputDialog, WSNlabelString, char*, _label_string,WSGFstrdup("") );
+//WSMFproperty( WSCinputDialog, WSNlabelString, char*, _label_string,NULL);
 WSMFproperty( WSCinputDialog, WSNworkBackColor,short, _wbg_no ,WS_DF_DARKBACKCOLOR);
-
+#ifndef NO_WINDOW_TITLE
 void WSCinputDialog::setWorkWSNtitleString(char* str){
   WSCbaseDialog::setWorkWSNtitleString(str);
   if (_title != NULL){
     _title->setProperty(WSNlabelString,str);
   }
 }
-
+#endif
+#if 0
 void WSCinputDialog::setWorkWSNlabelString(char* str){
   if (_label != NULL){
     _label->setPropertyV(WSNlabelString,str);
@@ -95,7 +92,7 @@ void WSCinputDialog::getWorkWSNlabelString(char** str){
     *str = _label_string;
   }
 }
-
+#endif
 void WSCinputDialog::setWorkWSNforeColor(short no){
   if (_title != NULL){
     _title->setProperty(WSNforeColor,no);
@@ -112,6 +109,10 @@ void WSCinputDialog::setWorkWSNworkBackColor(short no){
 }
 void WSCinputDialog::getWorkWSNworkBackColor(short* ){}
 
+WSCbase* WSCinputDialog::getPropertyInheritChild(){
+  return _label;
+}
+
 long WSCinputDialog::initialize(){
   WSCbaseDialog::initialize();
 
@@ -141,9 +142,9 @@ long WSCinputDialog::initialize(){
   _label->setPropertyV(WSNfillSpace,(WSCbool)False);
   _label->setPropertyV(WSNinterCur,(WSCbool)True);
   _label->setPropertyV(WSNmaxLength,(WSCushort)128);
-  _label->setPropertyV(WSNcolumns,(WSCushort)20);
+//  _label->setPropertyV(WSNcolumns,(WSCushort)20);
   _label->setPropertyV(WSNshadowType,(char)WS_SHADOW_IN);
-  _label->setPropertyV(WSNlabelString,_label_string );
+//  _label->setPropertyV(WSNlabelString,_label_string );
   _label->setVisible(True);
  
   WSCprocedure* lbop = new WSCprocedure("_label_focus_work",WSEV_PARENT_VISIBLE_CH);
