@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCvlabel.h>
@@ -37,8 +33,17 @@
 #include <WSDmouse.h>
 #include <WSDkeyboard.h>
 #include <WSCvscrBar.h>
+#ifdef WS_EMBED
+#include <WSCvklabel.h>
+#endif
+#include <WSCvifield.h>
 
 //#define TDBG
+#ifdef MSW
+#define LIST_MAX_HEIGHT 24000
+#else
+#define LIST_MAX_HEIGHT 32760
+#endif
 
 #define SELECT_STATUS "ls-selected"
 WSMFguiClassInitialize(WSClist,WSCscrForm);
@@ -47,6 +52,15 @@ WSMFpropertyValueChange( WSClist, WSNscrollHeight , WSCushort,20);
 
 WSClist::WSClist(WSCbase* base,char* objname):
                 WSCscrForm(base,objname){
+  _register_focus_move();
+  _upname = NULL;
+  _dwname = NULL;
+  _lname = NULL;
+  _rname = NULL;
+  _rtname = NULL;
+  _tbname = NULL;
+  _btbname = NULL;
+  _enable_focus_move = False;
 
   _lb_height = 20;
   _scroll_height = 20;
@@ -69,9 +83,9 @@ WSClist::WSClist(WSCbase* base,char* objname):
   _icon_pixmap = -1;
   _icon_pixmap_old = -1;
 //  _virtual_scr = 1;
-  _need_update = 0;
-  _data = WSGFstrdup("");
-  _data_source_name = WSGFstrdup("");
+  _need_update = 1;
+  _data = NULL;
+  _data_source_name = NULL;
   _data_source_type = WS_DATA_SOURCE_NONE;
   _hbar_visible = False;
   _lb_width = 1024;
@@ -84,6 +98,11 @@ WSClist::WSClist(WSCbase* base,char* objname):
   _need_update_btn_area = False;
   _wy_pos_bak = 0;
   _pre_select = -1;
+//  _w_bk = -1;
+  _w_bk = 0;
+  _sort_pos = 0;
+//  _h_bk = -1;
+  _return_key_focus = 0;
 
   WSMFpropertyCreateStart
     WSMFpropertyCreate(WSNseparator,  char*, _separator, WSSseparator );
@@ -110,7 +129,21 @@ WSClist::WSClist(WSCbase* base,char* objname):
     WSMFpropertyCreate(WSNenableInput,  WSCbool, _enable_input, WSSenableInput );
       WSMFpropertySetSelection(WSRbool3,WSRbool3D);
 
-//    WSMFpropertyDelete(WSNvirtualScroll);
+    WSMFpropertyCreate(WSNupward, char*, _upname,WSSupward);
+    WSMFpropertyCreate(WSNdownward,char*, _dwname,WSSdownward);
+    WSMFpropertyCreate(WSNleftward,char*,_lname,WSSleftward);
+    WSMFpropertyCreate(WSNrightward, char*,_rname,WSSrightward);
+    WSMFpropertyCreate(WSNreturn,char*,_rtname,WSSreturn);
+    WSMFpropertyCreate(WSNtab,char*,_tbname,WSStab);
+    WSMFpropertyCreate(WSNbacktab,char*,_btbname,WSSbacktab);
+    WSMFpropertyCreate(WSNenableFocusMove, WSCbool,_enable_focus_move,WSSenableFocusMove);
+    WSMFpropertySetSelection(WSRbool1,WSRbool1D);
+
+    WSMFpropertyCreate(WSNreturnKeyFocus,WSCbool,_return_key_focus,WSSreturnKeyFocus );
+      WSMFpropertySetSelection(WSRbool1,WSRbool1D);
+
+
+    WSMFpropertyDelete(WSNvirtualScroll);
     WSMFpropertyVisibleOff(WSNscrollHeight);
     WSMFpropertyVisibleOff(WSNworkHeight);
     WSMFpropertyValueChangeDef( WSClist,WSNscrollHeight ,WSCushort);
@@ -120,6 +153,7 @@ WSClist::WSClist(WSCbase* base,char* objname):
     WSMFaddTrigger(WSEV_KEY_RELEASE);
     WSMFaddExtTrigger(WSEV_SORT,"SORT");
     WSMFaddExtTrigger(WSEV_ITEM_SELECTED,"ITEM-SELECTED");
+    WSMFaddExtTrigger(WSEV_ITEM_PRE_SELECTED,"ITEM-PRE-SELECTED");
     WSMFaddExtTrigger(WSEV_INPUT_FIXED,"INPUT-FIXED");
     WSMFaddExtTrigger(WSEV_ITEM_DOUBLE_CLICKED,"ITEM-DOUBLE-CLICKED");
 
@@ -139,13 +173,25 @@ WSMFproperty( WSClist, WSNreverseSelect, WSCbool,_reverse_select,True);
 WSMFproperty( WSClist, WSNtitleHeight,  WSCuchar, _padding, 0);
 WSMFproperty( WSClist, WSNtitleString,  char*, _title_string,WSGFstrdup("title"));
 WSMFproperty( WSClist, WSNbarValue,  char*, _bar_pos,WSGFstrdup("20"));
-WSMFproperty( WSClist, WSNdata,  char*, _data,WSGFstrdup(""));
+WSMFproperty( WSClist, WSNdata,  char*, _data,NULL);
 WSMFproperty( WSClist, WSNdataSource,  WSCuchar, _data_source_type,WS_DATA_SOURCE_NONE);
-WSMFproperty( WSClist, WSNdataSourceName,  char*, _data_source_name,WSGFstrdup(""));
+WSMFproperty( WSClist, WSNdataSourceName,  char*, _data_source_name,NULL);
 WSMFproperty( WSClist, WSNenableInput,  WSCbool, _enable_input,False);
 WSMFproperty( WSClist, WSNseparator,  char*, _separator,WSGFstrdup(","));
 
-void WSClist::setWorkWSNitemHeight(WSCuchar){}
+WSMFproperty( WSClist, WSNenableFocusMove, WSCbool,    _enable_focus_move, False );
+WSMFproperty( WSClist, WSNupward,    char*,   _upname, NULL);
+WSMFproperty( WSClist, WSNdownward, char*,   _dwname, NULL);
+WSMFproperty( WSClist, WSNleftward,   char*,   _lname, NULL);
+WSMFproperty( WSClist, WSNrightward,  char*,   _rname, NULL);
+WSMFproperty( WSClist, WSNreturn, char*,   _rtname, NULL);
+WSMFproperty( WSClist, WSNtab, char*,   _tbname, NULL);
+WSMFproperty( WSClist, WSNbacktab, char*,   _btbname, NULL);
+WSMFproperty( WSClist, WSNreturnKeyFocus, WSCbool, _return_key_focus, False);
+
+void WSClist::setWorkWSNitemHeight(WSCuchar){
+  _update_lb_height();
+}
 void WSClist::getWorkWSNitemHeight(WSCuchar*){}
 
 void WSClist::setWorkWSNiconPixmap(short){
@@ -199,16 +245,21 @@ void WSClist::setWorkWSNbarValue(char* val){
   }
 }
 void WSClist::getWorkWSNbarValue(char** val){
+#if 0 //Removed: _btn_area valuech changes _bar_pos value, so setProperty deletes junk pointer of _bar_pos
   WSCstring tmp;
   if (_btn_area != NULL){
+    tmp = _btn_area->getProperty(WSNbarValue);
     if (_bar_pos != NULL){
-      delete _bar_pos;
+      if (!strcmp((char*)tmp,_bar_pos)){
+        return;
+      }
+      delete[] _bar_pos;
       _bar_pos = NULL;
     }
-    tmp = _btn_area->getProperty(WSNbarValue);
     _bar_pos = WSGFstrdup((char*)tmp);
     *val = _bar_pos;
   }
+#endif
 }
 
 void WSClist::setWorkWSNtitleString(char*){
@@ -255,9 +306,41 @@ void WSClist::setWorkWSNbottomShadowColor(short val){
 }
 void WSClist::setWorkWSNfont(WSCuchar){}
 void WSClist::getWorkWSNfont(WSCuchar*){}
-void WSClist::setWorkWSNselectColor(short){}
+void WSClist::setWorkWSNselectColor(short){
+  if (getVisible() != False){
+    long i;
+    long num = _lb_list.getNum();
+    for(i=0;i<num;i++){
+      WSCbase* lb = (WSCbase*)_lb_list[i];
+      long fl = (long)lb->getUserData(SELECT_STATUS);
+      if (fl == 0){
+        lb->setPropertyV(WSNselectColor,(short)0);
+        lb->setPropertyV(WSNselectForeColor,(short)0);
+      }else{
+        lb->setPropertyV(WSNselectColor,(short)_select_color);
+        lb->setPropertyV(WSNselectForeColor,(short)_select_fore_color);
+      }
+    }
+  }
+}
 void WSClist::getWorkWSNselectColor(short*){}
-void WSClist::setWorkWSNselectForeColor(short){}
+void WSClist::setWorkWSNselectForeColor(short){
+  if (getVisible() != False){
+    long i;
+    long num = _lb_list.getNum();
+    for(i=0;i<num;i++){
+      WSCbase* lb = (WSCbase*)_lb_list[i];
+      long fl = (long)lb->getUserData(SELECT_STATUS);
+      if (fl == 0){
+        lb->setPropertyV(WSNselectColor,(short)0);
+        lb->setPropertyV(WSNselectForeColor,(short)0);
+      }else{
+        lb->setPropertyV(WSNselectColor,(short)_select_color);
+        lb->setPropertyV(WSNselectForeColor,(short)_select_fore_color);
+      }
+    }
+  }
+}
 void WSClist::getWorkWSNselectForeColor(short*){}
 void WSClist::setWorkWSNtype(WSCuchar fl){
   if (fl == WS_NORMAL || fl == WS_TREE){
@@ -295,7 +378,7 @@ void WSClist::setWorkWSNtype(WSCuchar fl){
 void WSClist::_btn_area_valuech(WSCbase* ptr){
   WSClist* obj = (WSClist*)ptr->getUserData("p-list");
   if (obj->_bar_pos != NULL){
-    delete obj->_bar_pos;
+    delete[] obj->_bar_pos;
     obj->_bar_pos = NULL;
   }
   WSCstring tmp;
@@ -303,7 +386,7 @@ void WSClist::_btn_area_valuech(WSCbase* ptr){
   obj->_bar_pos = WSGFstrdup((char*)tmp);
   obj->_redraw_scr();
 //  obj->_scr_area->cdraw(); 
-  obj->redraw(); 
+//  obj->redraw(); 
 }
 void WSClist::getWorkWSNtype(WSCuchar*){
 }
@@ -313,6 +396,8 @@ void WSClist::setWorkWSNreverseSelect(WSCbool){}
 void WSClist::getWorkWSNreverseSelect(WSCbool*){}
 void WSClist::setWorkWSNenableInput(WSCbool){}
 void WSClist::getWorkWSNenableInput(WSCbool*){}
+void WSClist::setWorkWSNreturnKeyFocus(WSCbool){}
+void WSClist::getWorkWSNreturnKeyFocus(WSCbool*){}
 
 void WSClist::setWorkWSNwidth(WSCushort data){
   WSCscrForm::setWorkWSNwidth(data);
@@ -323,7 +408,7 @@ void WSClist::setWorkWSNwidth(WSCushort data){
       if ( _scr_area != NULL &&
            _btn_area != NULL){
         if (_bar_pos != NULL){
-          delete _bar_pos;
+          delete[] _bar_pos;
           _bar_pos = NULL;
         }
         WSCstring tmp;
@@ -435,9 +520,9 @@ void WSClist::_set_data(char* data){
   str.setString(data);
   delAll();
   if (_data != NULL){
-    delete _data;
+    delete[] _data;
   }
-  _data = WSGFstrdup((char*)str);
+  _data = WSGFstrdup(data);
   while(!str.eof()){
     WSCstring line = str.gets();
     WSCstring word1 = line.getWord(0,_separator);
@@ -495,7 +580,9 @@ void WSClist::setWorkWSNdata(char* data){
   _clear_ds_instance();
 //  if (_data_source_type == WS_DATA_SOURCE_NONE){
     WSCstring tmp;
-    tmp.setString(_data);
+    if (_data != NULL){
+      tmp.setString(_data);
+    }
     tmp.replaceString("\\n","\n",0);
     _set_data(tmp);
 //  }
@@ -507,6 +594,11 @@ void WSClist::_clear_ds_instance(){
   }
 }
 void WSClist::_set_ds_instance(){
+  if (_data_source_name == NULL){
+    _need_ds_update = True;
+    delAll();
+    return;
+  }
   WSCbase* serv = WSGIappObjectList()->searchInstance("WSCbase", _data_source_name);
   if (serv != NULL){
     setDataSourceServer(serv);
@@ -547,14 +639,16 @@ WSCstring WSClist::_get_data(){
   return ret;
 }
 void WSClist::getWorkWSNdata(char** ptr){
+#ifndef WS_EMBED
   if (getPropertyEditMode() != False &&
       _data_source_type != WS_DATA_SOURCE_NONE){
-    *ptr = "";
+    *ptr = NULL;
     return;
   }else{
+#endif
     WSCstring tmp = _get_data();
     if (_data != NULL && strcmp(_data,tmp.getString())){
-      delete _data;
+      delete[] _data;
       _data = WSGFstrdup(tmp.getString());
       *ptr = _data;
       return;
@@ -563,7 +657,9 @@ void WSClist::getWorkWSNdata(char** ptr){
       _data = WSGFstrdup(tmp.getString());
       *ptr = _data;
     }
+#ifndef WS_EMBED
   }
+#endif
 }
 void WSClist::setWorkWSNdataSource(WSCuchar){
   if (_data_source_name != NULL && _data_source_name[0] != 0){
@@ -589,7 +685,9 @@ void WSClist::setWorkWSNdataSourceName(char*){
     _set_ds_instance();
   }else
   if (_data_source_type == WS_DATA_SOURCE_FILE){
-    _data_from_file(_data_source_name);
+    if (_data_source_name != NULL){
+      _data_from_file(_data_source_name);
+    }
   }
 }
 void WSClist::getWorkWSNdataSourceName(char**){}
@@ -605,6 +703,24 @@ void WSClist::setWorkWSNseparator(char* data){
 }
 void WSClist::getWorkWSNseparator(char**){}
 
+void WSClist::setWorkWSNenableFocusMove(WSCbool){}
+void WSClist::getWorkWSNenableFocusMove(WSCbool* fl){}
+void WSClist::setWorkWSNupward(char*){}
+void WSClist::getWorkWSNupward(char**){}
+void WSClist::setWorkWSNdownward(char*){}
+void WSClist::getWorkWSNdownward(char**){}
+void WSClist::setWorkWSNleftward(char*){}
+void WSClist::getWorkWSNleftward(char**){}
+void WSClist::setWorkWSNrightward(char*){}
+void WSClist::getWorkWSNrightward(char**){}
+void WSClist::setWorkWSNreturn(char*){}
+void WSClist::getWorkWSNreturn(char**){}
+void WSClist::setWorkWSNtab(char*){}
+void WSClist::getWorkWSNtab(char**){}
+void WSClist::setWorkWSNbacktab(char*){}
+void WSClist::getWorkWSNbacktab(char**){}
+
+
 void WSClist::_data_from_file(char* fname){
   WSCstring* str = WSGFreadTextFile(fname);
   if (str != NULL){
@@ -613,26 +729,55 @@ void WSClist::_data_from_file(char* fname){
     delete str;
   }
 }
-
+#define WS_LIST_INPUT_STATUS "WS-input-status"
+struct _input_status_ {
+  _input_status_(){
+    pos = -1;
+    col = -1;
+    status = 0;
+  };
+  short pos;
+  short col;
+  WSCulong status;
+};
 WSClist::~WSClist(){
+  if (getFocus() != False){
+    setFocus(False);
+  }
+
+  if (_return_key_focus != False && getSpecialFocus() != False){
+    setSpecialFocus(False);
+  }
+
+  WSClistData* list = (WSClistData*)getUserData(WS_LIST_INPUT_STATUS);
+  if (list != NULL){
+    int i;
+    int num = list->getNum();
+    _input_status_** buf = (_input_status_**)list->getBuf();
+    for(i=0; i<num; i++){
+      _input_status_* status = buf[i];
+      delete status;
+    }
+    delete list;
+  }
   if (_bar_pos != NULL){
-    delete _bar_pos;
+    delete[] _bar_pos;
     _bar_pos = NULL;
   }
   if (_title_string != NULL){
-    delete _title_string;
+    delete[] _title_string;
     _title_string = NULL;
   }
   if (_data != NULL){
-    delete _data;
+    delete[] _data;
     _data = NULL;
   }
   if (_data_source_name != NULL){
-    delete _data_source_name;
+    delete[] _data_source_name;
     _data_source_name = NULL;
   }
   if (_separator != NULL){
-    delete _separator;
+    delete[] _separator;
     _separator = NULL;
   }
 }
@@ -737,6 +882,35 @@ WSCstring WSClist::getInputString(){
   }
   return str;
 }
+void WSClist::setInputString(WSCstring str){
+  if (_input_instance != NULL){
+    _input_instance->setProperty(WSNlabelString,str);
+  }
+}
+long WSClist::getInputPos(long* pos,long* col){
+  if (_input_instance != NULL){
+    long p = (long)_input_instance->getUserData("No.");
+    if (_disp_type != WS_VERBOSE){
+      if (pos != NULL){
+        *pos = p;
+      }
+      if (col != NULL){
+        *col = 0;
+      }
+      return WS_NO_ERR;
+    }else{
+      if (pos != NULL){
+        *pos = p;
+      }
+      long bno = (long)_input_instance->getUserData("bno");
+      if (col != NULL){
+        *col = bno;
+      }
+      return WS_NO_ERR;
+    }
+  }
+  return WS_ERR;
+}
 void WSClist::replaceItem(char* item,long pos,long column){
 
   WSCstring str2(item);
@@ -764,15 +938,16 @@ void WSClist::replaceItem(char* item,long pos,long column){
 }
 void WSClist::onInputFixed(WSCstring str,long pos,long column){
   execProcedure(WSEV_INPUT_FIXED);
+  WSCstring str2 = getInputString();
   if (_disp_type == WS_VERBOSE){
-    replaceItem(str,pos,column);
+    replaceItem(str2,pos,column);
   }else{
-    replaceItem(str,pos);
+    replaceItem(str2,pos);
   }
 }
 void WSClist::_input_fix_work(WSCbase* ifield){
   long key = WSGIappKeyboard()->getKey();
-  if (key == WSK_Return){
+  if (key == WSK_Return || key == WSK_KP_Enter){
     WSCbase* target = (WSCbase*)ifield->getUserData("selected-target");
     if (target == NULL){
       return;
@@ -789,6 +964,22 @@ void WSClist::_input_fix_work(WSCbase* ifield){
     }
     list->_adjust_input(target,pos,False,0,0);
   }
+  long val = (long)ifield->getUserData("WS_LIST_NUMONLY");
+  if (val != 0){
+    if (WSGIappKeyboard()->isCursorKey() != False){
+      return;
+    }else
+    if ( (key >= WSK_0    && key <= WSK_9   ) ||
+         (key >= WSK_KP_0 && key <= WSK_KP_9) ||
+          key == WSK_plus      || key == WSK_minus  ||
+          key == WSK_BackSpace || key == WSK_Delete || key == WSK_Insert ||
+          key == WSK_space     || key == WSK_Up     || key == WSK_Down   ||
+          key == WSK_Left      || key == WSK_Right  || key == WSK_Return ){
+      return;
+    }else{
+      WSGIappKeyboard()->setKey(0);
+    }
+  }
 }
 void WSClist::_input_focus_lose(WSCbase* ifield){
   if (ifield->getFocus() == False){
@@ -919,6 +1110,28 @@ void WSClist::_adjust_input(WSCbase* target,long pos,WSCbool vis,short x,short y
     _input_instance->setProperty(WSNmarginLeft,0);
     _input_instance->setProperty(WSNmarginTop,0);
     _input_instance->setProperty(WSNmarginBottom,0);
+    _input_instance->setUserData("WS_LIST_NUMONLY",(void*)0);
+
+    WSClistData* list = (WSClistData*)getUserData(WS_LIST_INPUT_STATUS);
+    if (list != NULL){
+      long i;
+      long num = list->getNum();
+      _input_status_** buf = (_input_status_**)list->getBuf();
+      for(i=0; i<num; i++){
+        _input_status_* item = buf[i];
+        if (item->pos == pos && item->col == bno &&
+            item->status == WS_LIST_READONLY){
+          _input_instance->setVisible(False);
+          return;
+        }
+        if (item->pos == pos && item->col == bno &&
+            item->status == WS_LIST_NUMONLY){
+          _input_instance->setUserData("WS_LIST_NUMONLY",(void*)1);
+          break;
+        }
+      }
+    }
+
     _input_instance->setVisible(True);
     _input_instance->setSpecialFocus(True);
     _input_instance->setFocus(True);
@@ -963,7 +1176,8 @@ void WSClist::_adjust(){
     op2->setFunction(_mouse_release_work,"_mouse_release_work");
     scr->addProcedure(op2);
 #endif
-    _input_instance = WSCbase::getNewInstance("WSCvifield",this,"list-in");
+//    _input_instance = WSCbase::getNewInstance("WSCvifield",this,"list-in");
+    _input_instance = new WSCvifield(this,"list-in");
     _input_instance->initialize();
     _input_instance->setInternalObject(True);
   }
@@ -993,6 +1207,7 @@ void WSClist::_adjust(){
   _need_update_btn_area = False;
   _btn_area->setPropertyV(WSNwidth,(WSCushort)_scr_area_w);
   _btn_area->setPropertyV(WSNheight,(WSCushort)_padding);
+  _btn_area->setPropertyV(WSNbarValue,_bar_pos);
 
   _btn_area->setPropertyV(WSNbackColor,(short)_back_color);
   _btn_area->setPropertyV(WSNforeColor,(short)_fore_color);
@@ -1010,7 +1225,7 @@ void WSClist::_adjust(){
     WSCbase* item = (WSCbase*)children[i];
     if (item->cast("WSCgripHand") != NULL){
       continue;
-    }else if (i < btnnum){
+    }else if (cnt < btnnum){
       WSCstring tmp = title.getWord(cnt,_separator);
       item->setPropertyV(WSNlabelString,(char*)tmp);
       item->setPropertyV(WSNbackColor,(short)_back_color);
@@ -1068,6 +1283,19 @@ void WSClist::_btn_activate(WSCbase* btn){
 }
 void WSClist::onSort(){
   long sort_pos = getSortPos(); 
+  long sort_direct = (long)getUserData("SORT-DIR");
+  long sort_retry = (long)getUserData("SORT-RETRY");
+//  if (sort_retry == 0 && sort_pos+1 != sort_direct && 
+//                         sort_pos+1 != -sort_direct ){
+//    sort_retry = 1;
+//  }
+  if (sort_pos+1 == sort_direct){
+    setUserData("SORT-DIR",(void*)(-(sort_pos+1)));
+    sort_direct = -1;
+  }else{
+    setUserData("SORT-DIR",(void*)(sort_pos+1));
+    sort_direct = 1;
+  }
 
   long num = _lb_list.getNum();
   WSClistData list;
@@ -1084,6 +1312,26 @@ void WSClist::onSort(){
     for(j=0; j<inum; j++){
       WSCbase* lb = (WSCbase*)list[j];
       char* istr = (char*)lb->getUserData("SORT-STR");
+      if (strcmp(istr,(char*)sortstr)==0){
+        WSCstring tstr = lb->getProperty(WSNlabelString);
+        if (strcmp(tstr,str) > 0){
+          list.add((void*)item,j);
+          added = 1;
+          break;
+        }
+      }else
+#if 0
+      if (strcmp(istr,(char*)sortstr)==0 && sort_direct == 1 && sort_retry == 0){
+        list.add((void*)item,j);
+        added = 1;
+        break;
+      }else
+      if (strcmp(istr,(char*)sortstr)==0 && sort_direct != 1 && sort_retry != 0){
+        list.add((void*)item,j);
+        added = 1;
+        break;
+      }else
+#endif
       if (strcmp(istr,(char*)sortstr)>0){
         list.add((void*)item,j);
         added = 1;
@@ -1099,11 +1347,16 @@ void WSClist::onSort(){
   for(i=0; i<num; i++){
     WSCbase* lb = (WSCbase*)list[i];
     char* istr = (char*)lb->getUserData("SORT-STR");
-    delete istr;
-    _lb_list.add(lb);
+    delete[] istr;
+    if (sort_direct == -1){
+      _lb_list.add(lb,0);
+    }else{
+      _lb_list.add(lb);
+    }
   }
   _lb_adjust();
 
+  setUserData("SORT-RETRY",(void*)0);
 
 //  execEventProc(WSEV_SORT,NULL);
   execProcedure(WSEV_SORT);
@@ -1111,6 +1364,9 @@ void WSClist::onSort(){
 long WSClist::getSortPos(){
   return _sort_pos;
 }
+void WSClist::setSortPos(long pos){
+  _sort_pos = pos;
+}
 void WSClist::setLabelClass(char* cname){
   _lb_class.setString(cname);
 }
@@ -1140,8 +1396,12 @@ void WSClist::addItem(char* str,long pos){
   cancelInput();
 //   WSCushort sw;
 //   getPropertyV(WSNworkWidth,&sw);
+#ifdef WS_EMBED
+   WSCbase* lb = new WSCvklabel( this,"lblist-l");
+#else
    WSCbase* lb = WSCbase::getNewInstance(_lb_class.getString(),
                                      this,"lblist-l");
+#endif
    if (lb == NULL){
 WSMFtrace("WSClist::addItem Error. Can not create %s class instance.\n",(char*)_lb_class);
      return;
@@ -1185,8 +1445,10 @@ void WSClist::update(){
   if (_need_update_btn_area != False){
     _adjust();
   }
+//printf("WSClist::update _need_update=%d _need_scr_redraw=%d\n",_need_update,_need_scr_redraw);
   if ( _need_update != 0){
     updateList();
+#if 0 //move to updateList()..
     if (_pixmap_style == WS_DYNAMIC_PIXMAP){
       WSDdev* dev = _scr_area->getdev();
       if (dev != NULL){
@@ -1202,11 +1464,21 @@ void WSClist::update(){
       _redraw_scr();
       _need_scr_redraw = False;
     }
+#endif
   }
   WSCscrForm::update();
 }
+void WSClist::_update_lb_height(){
+  long num = _lb_list.getNum();
+  long i;
+  for(i=0; i< num; i++){
+    WSCbase* label = (WSCbase*)_lb_list[i];
+    label->setProperty(WSNheight,_lb_height);
+  }
+}
 void WSClist::updateList(){
-//  cancelInput(); ->NG
+
+//  cancelinput(); ->ng
    _need_update = 0;
   long num = _lb_list.getNum();
   long i;
@@ -1237,7 +1509,25 @@ void WSClist::updateList(){
 
 //  setAbsoluteDraw(True);
 //  _scr_area->redraw();
-//  cdraw();
+//  cdraw();_
+//printf("WSClist::updateList _need_scr_redraw=%d\n",_need_scr_redraw);
+  //move from update()..
+  if (_pixmap_style == WS_DYNAMIC_PIXMAP){
+    WSDdev* dev = _scr_area->getdev();
+    if (dev != NULL){
+      char fl = WS_DIRECT_WINDOW;
+      dev->setValue(WSDEV_PIXMAP_STYLE,&fl);
+      fl = WS_DYNAMIC_PIXMAP;
+      dev->setValue(WSDEV_PIXMAP_STYLE,&fl);
+    }
+  //move from update()..
+    _redraw_scr();
+    _need_scr_redraw = False;
+  }
+  if (_need_scr_redraw != False){
+    _redraw_scr();
+    _need_scr_redraw = False;
+  }
 }
 
 void WSClist::delAll(){
@@ -1261,7 +1551,7 @@ void WSClist::delAll(){
     }
   }
   if (_data != NULL){
-    delete _data;
+    delete[] _data;
   }
   _data = NULL;
   _need_update = 1;
@@ -1357,6 +1647,9 @@ WSCstring WSClist::getSelectedItem(){
 long WSClist::getTopPos(){
 //  long hnum = _v_scr_slider_sz / _lb_height;
   long topbase = _wy_pos / _lb_height;
+  if (_wy_pos % _lb_height){
+    topbase++;
+  }
   long num = _lb_list.getNum();
   long i;
   long cnt =0;
@@ -1381,18 +1674,47 @@ void WSClist::setTopPos(long pos){
   long cnt =0;
   for(i=0; i<num; i++){
     WSCbase* item = (WSCbase*)_lb_list[i];
-    if (cnt == pos){
-      setProperty(WSNvbarValue,_lb_height*i);
+    if (i == pos){
+      _v_scr->_set_internal_value(_lb_height*cnt);
+      long bk = _scroll_height;
+      _scroll_height = 0;
+      _v_scr->onValueChange();
+      _scroll_height = bk;
+      _v_scr->setAbsoluteDraw(True);
+      _v_scr->draw();
+//      setProperty(WSNvbarValue,_lb_height*cnt);
       return;
     }
     if (item->getVisible() != False){
       cnt++;
     }
   }
-  setProperty(WSNvbarValue,_lb_height*i);
+  _v_scr->_set_internal_value(_lb_height*cnt);
+  _v_scr->onValueChange();
+  _v_scr->setAbsoluteDraw(True);
+  _v_scr->draw();
+//  setProperty(WSNvbarValue,_lb_height*cnt);
 }
 
 long WSClist::getBottomPos(){
+  long topbase = _wy_pos / _lb_height;
+  topbase += (_v_scr_slider_sz / _lb_height-1);
+  long num = _lb_list.getNum();
+  long i;
+  long cnt =0;
+  for(i=0; i<num; i++){
+    WSCbase* item = (WSCbase*)_lb_list[i];
+    if (cnt == topbase){
+      return i;
+    }
+    if (item->getVisible() != False){
+      cnt++;
+    }
+  }
+  return num -1;
+
+
+#if 0
   long pos = getTopPos();
   long hnum = _v_scr_slider_sz / _lb_height;
   long pos2 = pos + hnum -1;
@@ -1404,9 +1726,49 @@ long WSClist::getBottomPos(){
     return max-1;
   }
   return 0;
+#endif
 }
 
 void WSClist::setBottomPos(long pos){
+  if (pos == -1){
+    pos = _lb_list.getNum();
+  }
+  long num = _lb_list.getNum();
+  long i;
+  long cnt =0;
+//  long hnum = _v_scr_slider_sz / _lb_height;
+//  long diff = _v_scr_slider_sz % _lb_height;
+  for(i=0; i<num; i++){
+    WSCbase* item = (WSCbase*)_lb_list[i];
+    if (i == pos){
+      long val = _lb_height*(cnt+1) - _v_scr_slider_sz;
+      if (val < 0){
+        val = 0;
+      }
+      _v_scr->_set_internal_value(val);
+      long bk = _scroll_height;
+      _scroll_height = 0;
+      _v_scr->onValueChange();
+      _scroll_height = bk;
+      _v_scr->setAbsoluteDraw(True);
+      _v_scr->draw();
+//      setProperty(WSNvbarValue,val);
+      return;
+    }
+    if (item->getVisible() != False){
+      cnt++;
+    }
+  }
+  long val = _lb_height*(cnt+1) - _v_scr_slider_sz;
+  if (val < 0){
+    val = 0;
+  }
+  _v_scr->_set_internal_value(val);
+  _v_scr->onValueChange();
+  _v_scr->setAbsoluteDraw(True);
+  _v_scr->draw();
+//  setProperty(WSNvbarValue,val);
+#if 0
   if (pos == -1){
     pos = _lb_list.getNum();
   }
@@ -1416,6 +1778,7 @@ void WSClist::setBottomPos(long pos){
     pos2 = 0;
   }
   setTopPos(pos2);
+#endif
 }
 
 long WSClist::setPreSelectPos(long pos){
@@ -1427,9 +1790,10 @@ long WSClist::setPreSelectPos(long pos){
     pos = num -1;
   }
   _pre_select = pos;
+  onItemPreSelected();
   return WS_NO_ERR;
 }
-long WSClist::setSelectPos(long pos){
+long WSClist::setSelectPos(long pos,WSCbool create_event){
   long num =  _lb_list.getNum();
   if (pos > num -1){
     return WS_ERR;
@@ -1439,6 +1803,13 @@ long WSClist::setSelectPos(long pos){
   }
   _pre_select = pos;
   if (_mselect == False){
+#ifndef NO_INDEX_VARIANT_DATA
+    setVariantData("mselect-pos1",pos);
+    setVariantData("mselect-pos2",pos);
+#else
+    setUserData("mselect-pos1",(void*)pos);
+    setUserData("mselect-pos2",(void*)pos);
+#endif
     long i;
     long num = _lb_list.getNum();
     for(i=0;i<num;i++){
@@ -1498,10 +1869,12 @@ long WSClist::setSelectPos(long pos){
       lb->setUserData(SELECT_STATUS,(void*)1);
     }
   }
-  if (_activate){
-    onActivate();
+  if (create_event != False){
+    if (_activate){
+      onActivate();
+    }
+    onItemSelected();
   }
-  onItemSelected();
   needUpdate();
   _need_update = 1;
   return WS_NO_ERR;
@@ -1518,6 +1891,9 @@ void WSClist::setSelectItemChanged(WSCbool fl){
 void WSClist::onItemSelected(){
   execProcedure(WSEV_ITEM_SELECTED);
 }
+void WSClist::onItemPreSelected(){
+  execProcedure(WSEV_ITEM_PRE_SELECTED);
+}
 void WSClist::onItemDoubleClicked(){
   execProcedure(WSEV_ITEM_DOUBLE_CLICKED);
 }
@@ -1550,7 +1926,9 @@ long WSClist::draw(){
   }
   return WSCscrForm::draw();
 }
-
+void WSClist::resetUpdateFlag(){
+  _need_update = False;
+}
 long WSClist::setItemVisible(long pos,WSCbool fl){
   long num = _lb_list.getNum();
   if (num <1){
@@ -1578,6 +1956,9 @@ long WSClist::setItemValue(long pos,long kind,long val){
     pos = num -1;
   }
   WSCbase* item = (WSCbase*)_lb_list.getData(pos);
+  if (item == NULL){
+    return WS_ERR;
+  }
   if (kind == WS_OPEN){
     _need_update = 1;
     if (val == 0){
@@ -1628,6 +2009,9 @@ long WSClist::getItemValue(long pos,long kind,long* val){
     return WS_ERR;
   }
   WSCbase* item = (WSCbase*)_lb_list.getData(pos);
+  if (item == NULL){
+    return WS_ERR;
+  }
   if (kind == WS_OPEN){
     *val = (long)item->getUserData(WS_LIST_TREE_OPEN);
     return WS_NO_ERR;
@@ -1780,12 +2164,10 @@ void WSClist::_lb_adjust_work(long& pos,long indent,WSCbool vis,long& cnt){
   }
 }
 void WSClist::_lb_adjust(){
-//printf("_lb_adjust\n");
   long num = _lb_list.getNum();
   if (num <1){
-//    setPropertyV(WSNworkHeight,(WSCushort)1);
     setPropertyV(WSNworkHeight,(WSCushort)_scr_area_h);
-    _work_height_bk = _scr_area_h;
+    _work_height_bk = _work_height;
     if (getVisible() != False){
       _redraw_scr();
     }
@@ -1820,13 +2202,19 @@ void WSClist::_lb_adjust(){
     if (lbh > 0xffff){
       lbh = 0xffff;
     }
-    if (_scr_area_h > lbh){
-      lbh = _scr_area_h;
-    }
+//    if (_scr_area_h > lbh){
+//      lbh = _scr_area_h;
+//    }
     if (lbh != _work_height_bk){
       setProperty(WSNworkHeight,lbh);
       setAbsoluteDraw(False);
-      _work_height_bk = lbh;
+      _work_height_bk = _work_height;
+    }
+    if (_scr_area_h > lbh){
+      lbh = _scr_area_h;
+      setProperty(WSNworkHeight,lbh);
+      setAbsoluteDraw(False);
+      _work_height_bk = _work_height;
     }
   }else{
     long pos = 0;
@@ -1846,7 +2234,7 @@ void WSClist::_lb_adjust(){
       setAbsoluteDraw(False);
       _scr_area->setProperty(WSNworkHeight,lbh);
       _scr_area->setAbsoluteDraw(False);
-      _work_height_bk = lbh;
+      _work_height_bk = _work_height;
     }
 //    if (cnt < num){
 //      WSCrect area;
@@ -1862,7 +2250,7 @@ void WSClist::_lb_adjust(){
       _scr_area->redraw();
     }
   }
-  if ( cnt*_lb_height > 32760 ){
+  if ( cnt*_lb_height > LIST_MAX_HEIGHT || _pixmap_style == WS_DIRECT_WINDOW){
     if (_no_scroll == False){
       _set_no_scroll(True);
       _adjust_no_scroll();
@@ -1875,7 +2263,11 @@ void WSClist::_lb_adjust(){
       }
     }
   }else{
-    _set_no_scroll(False);
+    if (_no_scroll != False){
+      _set_no_scroll(False);
+      _lb_adjust();
+      return;
+    }
     if (cnt*_lb_height < _wy_pos + _v_scr_slider_sz){
       WSDdev* dev = _scr_area->getowndev();
       if (dev != NULL){
@@ -1916,7 +2308,6 @@ void WSClist::_adjust_no_scroll(){
     }
     return;
   }
-
   long i;
   long cnt=0;
   for(i=0; i<num; i++){
@@ -1932,6 +2323,7 @@ void WSClist::_adjust_no_scroll(){
         }
       }else if (_lb_height * cnt + _lb_height > _wy_pos ){
         item->setPropertyV(WSNy, (short)( cnt * _lb_height - _wy_pos));
+        item->setPropertyV(WSNx, (short)( -_wx_pos));
 //        item->WSCbase::update(); 
       }else{
         short val;
@@ -1984,6 +2376,46 @@ void WSClist::onValueChange(){
     }
   }
 }
+void WSClist::onFocusChange(WSCbool fl){
+  if (fl != False){
+    if (_return_key_focus != False){
+      WSGIappKeyboard()->resetSpecialInputWorkProc();
+      setSpecialFocus(True);
+    }
+
+    long pos = getSelectedPos();
+    if (pos != -1){
+      WSCbase* lb = getLabel(pos);
+      if (lb != NULL){
+        lb->setFocus(fl);
+      }
+    }else{
+      long num = getNum();
+      if (num > 0){
+        setSelectPos(0);
+        WSCbase* lb = getLabel(0);
+        if (lb != NULL){
+          lb->setFocus(fl);
+        }
+      }
+    }
+  }else{
+    if (_return_key_focus != False){
+      WSGIappKeyboard()->resetSpecialInputWorkProc();
+      setSpecialFocus(False);
+    }
+  }
+}
+void WSClist::_focus_move(long direct){
+  if (_enable_focus_move == False){
+    return;
+  }
+  WSCbase*  target = getFocusMoveInstance(direct);
+  if (target != NULL) {
+    target->setFocus();
+    target->redraw();
+  }
+}
 
 long WSClist::getSelectedNextPos(long pos){
   long i;
@@ -1997,8 +2429,8 @@ long WSClist::getSelectedNextPos(long pos){
   }
   return -1;
 }
-WSCbool WSClist::setSelected(long pos,WSCbool fl){
-  long i;
+WSCbool WSClist::setSelected(long pos,WSCbool fl,WSCbool create_event){
+//  long i;
   long num = _lb_list.getNum();
   if (pos < 0){
     pos = num-1;
@@ -2007,23 +2439,24 @@ WSCbool WSClist::setSelected(long pos,WSCbool fl){
     WSCbase* lb = (WSCbase*)_lb_list[pos];
     long ret = (long)lb->getUserData(SELECT_STATUS);
     if (fl == False){
-      lb->setUserData(SELECT_STATUS,(void*)0);
       lb->setPropertyV(WSNselectColor,(short)0);
       lb->setPropertyV(WSNselectForeColor,(short)0);
-      if ((long)lb->getUserData(SELECT_STATUS) == 0){
+      if (ret != 0){
         lb->setUserData(SELECT_STATUS,(void*)0);
         _select_item_changed = 1;
-        if (_activate){
-          onActivate();
+        if (create_event != False){
+          if (_activate){
+            onActivate();
+          }
+          onItemSelected();
         }
-        onItemSelected();
         needUpdate();
         _need_update = 1;
       }
     }else{
       WSCbool sl = getSelected(pos);
       if (sl == False){
-        setSelectPos(pos);
+        setSelectPos(pos,create_event);
       }
     }
     return (WSCbool)ret;
@@ -2031,7 +2464,7 @@ WSCbool WSClist::setSelected(long pos,WSCbool fl){
   return False;
 }
 WSCbool WSClist::getSelected(long pos){
-  long i;
+//  long i;
   long num = _lb_list.getNum();
   if (pos < 0){
     pos = num-1;
@@ -2054,6 +2487,76 @@ WSCstring WSClist::getSelectedNextItem(long pos){
 char* WSClist::getDefaultProperty(){
   return WSNdata;
 }
+char WSClist::getSortOrder(long col_no){
+  long sort_direct = (long)getUserData("SORT-DIR");
+  if (col_no+1 != sort_direct){
+    return WS_LIST_DESCENDING_ORDER;
+  }else{
+    return WS_LIST_ASCENDING_ORDER;
+  }
+}
+void WSClist::sort(long pos,char sort_order ){
+  long dpos = getSortPos();
+  setSortPos(pos);
+  if (sort_order == WS_LIST_REVERSING_ORDER){
+    onSort();
+  }else
+  if (sort_order == WS_LIST_ASCENDING_ORDER){
+    long sortd = (long)getUserData("SORT-DIR");
+    if (sortd == pos+1 && dpos == pos){
+       setUserData("SORT-RETRY",(void*)1);
+    }
+    setUserData("SORT-DIR",(void*)(-(pos+1)));
+    onSort();
+  }else
+  if (sort_order == WS_LIST_DESCENDING_ORDER){
+    long sortd = (long)getUserData("SORT-DIR");
+    if (sortd != pos+1 && dpos == pos){
+       setUserData("SORT-RETRY",(void*)1);
+    }
+    setUserData("SORT-DIR",(void*)(pos+1));
+    onSort();
+  }
+}
 
-
-
+WSCbool WSClist::canFocus(){
+  int i;
+  int num = _lb_list.getNum();
+  for(i=0; i<num; i++){
+    WSCbase* lb = (WSCbase*)_lb_list[i];
+    if (lb == NULL){
+      continue;
+    }
+    if (lb->getVisible() == False){
+      continue;
+    }
+    return True;
+  }
+  return False;
+}
+long WSClist::setInputStatus(long status,long pos,long col){
+  WSClistData* list = (WSClistData*)getUserData(WS_LIST_INPUT_STATUS);
+  if (list == NULL){
+    list = new WSClistData();
+    setUserData(WS_LIST_INPUT_STATUS,(void*)list);
+  }
+  if (list != NULL){
+    int i;
+    int num = list->getNum();
+    _input_status_** buf = (_input_status_**)list->getBuf();
+    for(i=0; i<num; i++){
+      _input_status_* item = buf[i];
+      if (item->pos == pos && item->col == col){
+        item->status = status;
+        return WS_NO_ERR;
+      }
+    }
+  }
+  _input_status_* item = new _input_status_;
+  item->pos = pos;
+  item->col = col;
+  item->status = status;
+  list->add((void*)item);
+  setUserData(WS_LIST_INPUT_STATUS,(void*)list);
+  return WS_NO_ERR;
+}
