@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCpulldownMenu.h>
@@ -50,6 +46,7 @@ WSMFpropertyValueChange(WSCpulldownMenu,WSNbackColor,short,WS_DF_MENUBACKCOLOR);
 WSMFpropertyValueChange(WSCpulldownMenu,WSNtopShadowColor,short,WS_DF_MENUTOPSHADOWCOLOR);
 WSMFpropertyValueChange(WSCpulldownMenu,WSNbottomShadowColor,short,WS_DF_MENUBOTTOMSHADOWCOLOR);
 
+static WSCpulldownMenu* _popup_instance=NULL;
 
 WSCpulldownMenu::WSCpulldownMenu(WSCbase* base, char* objname):
                 WSCvlabel(base, objname){
@@ -74,6 +71,9 @@ WSCpulldownMenu::WSCpulldownMenu(WSCbase* base, char* objname):
   _mouse_no_check = 0;
   _ld_name = WSGFstrdup("");
   _rd_name = WSGFstrdup("");
+  _item_height = -1;
+  _separator = WSGFstrdup(":"); 
+  _separator2 = WSGFstrdup(","); 
 
   WSGIappDev()->addMenu(this);
   WSGIappObjectList()->addEvent(this,WSEV_GUI_POLICY_CH);
@@ -85,6 +85,9 @@ WSCpulldownMenu::WSCpulldownMenu(WSCbase* base, char* objname):
     WSMFpropertyCreate(WSNshortcut, char*, _sc_str,WSSshortcut);
     WSMFpropertyCreate(WSNleftward,   char*,   _ld_name,WSSleftward);
     WSMFpropertyCreate(WSNrightward,  char*,   _rd_name,WSSrightward);
+    WSMFpropertyCreate(WSNitemHeight,  short,  _item_height,WSSitemHeight);
+    WSMFpropertyCreate(WSNseparator,  char*,   _separator,WSSseparator);
+    WSMFpropertyCreate(WSNseparator2,  char*,   _separator2,WSSseparator2);
 
     WSMFpropertyDelete(WSNorientation);
     WSMFpropertyDelete(WSNalignmentV);
@@ -99,6 +102,7 @@ WSCpulldownMenu::WSCpulldownMenu(WSCbase* base, char* objname):
     WSMFpropertyValueChangeDef(WSCpulldownMenu,WSNbottomShadowColor,short);
 
     WSMFaddTrigger(WSEV_ACTIVATE  );
+    WSMFaddExtTrigger(WSEV_MENU_POPUP,"MENU-POPUP");
 
     WSMFdelTrigger(WSEV_MOUSE_MOVE  );
     WSMFdelTrigger(WSEV_MOUSE_PRESS   );
@@ -111,6 +115,9 @@ WSMFproperty( WSCpulldownMenu, WSNshortcut, char*, _sc_str, WSGFstrdup(""));
 WSMFproperty( WSCpulldownMenu, WSNmenuItems, char*, _menu_str, WSGFstrdup("item1(key1):ep1_name:key1,item2(key2):ep2_name:key2,SP,item3(key3):ep3_name:key3")); 
 WSMFproperty(WSCpulldownMenu, WSNleftward,   char*,   _ld_name, WSGFstrdup(""));
 WSMFproperty(WSCpulldownMenu, WSNrightward,  char*,   _rd_name, WSGFstrdup(""));
+WSMFproperty(WSCpulldownMenu, WSNitemHeight, short,   _item_height, -1);
+WSMFproperty(WSCpulldownMenu, WSNseparator,  char*,   _separator, WSGFstrdup(":"));
+WSMFproperty(WSCpulldownMenu, WSNseparator2,  char*,   _separator2, WSGFstrdup(","));
 
 void WSCpulldownMenu::setWorkWSNshadowThickness(WSCuchar data){
   if (data == 0){
@@ -135,8 +142,25 @@ void WSCpulldownMenu::setWorkWSNleftward(char*) {}
 void WSCpulldownMenu::getWorkWSNleftward(char**) {}
 void WSCpulldownMenu::setWorkWSNrightward(char*) {}
 void WSCpulldownMenu::getWorkWSNrightward(char**) {}
+void WSCpulldownMenu::setWorkWSNitemHeight(short ){}
+void WSCpulldownMenu::getWorkWSNitemHeight(short* ){}
+void WSCpulldownMenu::setWorkWSNseparator(char* data) {
+  if (_menu != NULL){
+    _menu->setPropertyV(WSNseparator, data);
+  }
+}
+void WSCpulldownMenu::getWorkWSNseparator(char**) {}
+void WSCpulldownMenu::setWorkWSNseparator2(char* data) {
+  if (_menu != NULL){
+    _menu->setPropertyV(WSNseparator2, data);
+  }
+}
+void WSCpulldownMenu::getWorkWSNseparator2(char**) {}
 
 WSCpulldownMenu::~WSCpulldownMenu(){
+  if (_popup_instance == this){
+    _popup_instance = NULL;
+  }
   WSGIappObjectList()->delEvent(this,WSEV_GUI_POLICY_CH);
   WSGIappDev()->delMenu(this);
   if (_ptarget == this){
@@ -151,17 +175,18 @@ WSCpulldownMenu::~WSCpulldownMenu(){
     delete _menu;
     _menu = NULL;
   }
-  delete _sc_str;
-  delete _menu_str;
+  delete[] _sc_str;
+  delete[] _menu_str;
   if (_ld_name != NULL){
-    delete _ld_name;
+    delete[] _ld_name;
   }
   if (_rd_name != NULL){
-    delete _rd_name;
+    delete[] _rd_name;
   }
 }
 
 long WSCpulldownMenu::initialize(){
+  WSCvlabel::initialize();
   WSCstring name;
   name = (WSCstring)"pm-" + getInstanceName();
   _menu = new WSCpulldownMenuPopup(NULL, name);
@@ -172,7 +197,6 @@ long WSCpulldownMenu::initialize(){
   _menu->setPropertyV(WSNwidth, (WSCushort)10);
   _menu->setPropertyV(WSNheight, (WSCushort)10);
 
-  WSCvlabel::initialize();
   return WS_NO_ERR;
 }
 
@@ -256,8 +280,10 @@ long WSCpulldownMenu::draw(){
       st.blink_pixmap = _blink_pixmap;
       st.press_state = _press_state;
       st.orientation = _orientation;
+#ifndef NO_GRADATION
       st.gradation = _grad_no;
       st.gradation_margin = _grad_margin;
+#endif
       st.margin_left = _margin_left;
       st.margin_right = _margin_right;
       st.margin_top = _margin_top;
@@ -330,9 +356,12 @@ long WSCpulldownMenu::draw(){
   if ( h > margin_top + margin_bottom){
     sh = h - margin_top - margin_bottom;
   }
-
+#ifndef WS_EMBED
   if( getSensitive() == False ||
-     (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0)){
+     (getPropertyEditMode() != False && _sensitive_work == False)){
+#else
+  if( getSensitive() == False){
+#endif
     if (_emboss != False){
       if (_bl_fl != False &&
         blinkfore != False &&
@@ -352,7 +381,7 @@ long WSCpulldownMenu::draw(){
       }else{
         // ɽʸ...
         if (_tw_fl == False || _bl_fl == False){
-
+#if 0
 //--
 
 	// Accelerator key char underline show
@@ -421,8 +450,20 @@ long WSCpulldownMenu::draw(){
 	}
 
 //--
-          
-	  dev->drawString(sx+1, sy+1, sw, sh, &str, _font, getAlignment(), -1, False);
+#endif
+          if (_sc_str != NULL){
+            WSCstring _accel_char(_sc_str);
+            _accel_char.to_lower();
+      
+            WSCstring _accel_str(str);
+            _accel_str.to_lower();
+            long _char_pos = _accel_str.isExist(_accel_char);
+            if(_char_pos > -1 ){
+              dev->setUnderLineChar(_char_pos,1);
+            }
+          }
+
+          dev->drawString(sx+1, sy+1, sw, sh, &str, _font, getAlignment(), -1, False);
         }
       }
     }else{
@@ -439,8 +480,12 @@ long WSCpulldownMenu::draw(){
         _tw_fl != False) {
       dev->setForeColor(_bg_blink_color);
     } else {
+#ifndef WS_EMBED
       if ( _emboss != False && (getSensitive() == False ||
-          (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0))){
+          (getPropertyEditMode() != False && _sensitive_work == False))){
+#else
+      if ( _emboss != False && getSensitive() == False){
+#endif
         dev->setForeColor(_bs_color);
       } else {
         dev->setForeColor(_back_color);
@@ -454,8 +499,12 @@ long WSCpulldownMenu::draw(){
         _tw_fl != False) {
       dev->setForeColor(_blink_color);
     } else {
+#ifndef WS_EMBED
       if ( _emboss != False && (getSensitive() == False ||
-          (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0))){
+          (getPropertyEditMode() != False && _sensitive_work == False))){
+#else
+      if ( _emboss != False && getSensitive() == False){
+#endif
          dev->setForeColor(_bs_color);
       }else{
          dev->setForeColor(_fore_color);
@@ -473,10 +522,10 @@ long WSCpulldownMenu::draw(){
       blinkfore == False &&
       _bl_type != WS_BACK &&
       _tw_fl == False){
-     // 褷ʤ
+     // 褷ʤ...
   }else{
-    // ɽʸ
-
+    // ɽʸ...
+#if 0
 //--
 
 	// Accelerator key char underline show
@@ -545,6 +594,18 @@ long WSCpulldownMenu::draw(){
 	}
 
 //--
+#endif
+    if (_sc_str != NULL){
+      WSCstring _accel_char(_sc_str);
+      _accel_char.to_lower();
+
+      WSCstring _accel_str(str);
+      _accel_str.to_lower();
+      long _char_pos = _accel_str.isExist(_accel_char);
+      if(_char_pos > -1 ){
+        dev->setUnderLineChar(_char_pos,1);
+      }
+    }
 
     dev->drawString(sx, sy, sw, sh, &str, _font, getAlignment(), -1, False);
   }
@@ -557,6 +618,9 @@ long WSCpulldownMenu::draw(){
 void WSCpulldownMenu::onActivate(){
   execProcedure(WSEV_ACTIVATE);
 }
+void WSCpulldownMenu::onMenuPopup(){
+  execProcedure(WSEV_MENU_POPUP);
+}
 
 void WSCpulldownMenu::onVisibleChange(WSCbool fl){
   if (fl == False){
@@ -565,6 +629,7 @@ void WSCpulldownMenu::onVisibleChange(WSCbool fl){
     if (_grabed != 0){
       WSDdev* pdev = getParentDev();
       WSCbool fl = False;
+      WSGIappMouse()->setMouseUngrabClient(NULL);
       pdev->setValue(WSDEV_GRAB_POINTER,&fl);
       _grabed = 0;
       _mouse_grabed = 0;
@@ -583,6 +648,7 @@ void WSCpulldownMenu::onParentVisibleChange(WSCbool fl){
     if (_grabed != 0){
       WSDdev* pdev = getParentDev();
       WSCbool fl = False;
+      WSGIappMouse()->setMouseUngrabClient(NULL);
       pdev->setValue(WSDEV_GRAB_POINTER,&fl);
       _grabed = 0;
       _mouse_grabed = 0;
@@ -602,8 +668,12 @@ void WSCpulldownMenu::onMousePress(WSCpoint* point){
     return;
   }
   if (_press_state == False && (long)getUserData(WS_PM_POPUPED) == 0){
+#ifndef WS_EMBED
     if (getPropertyEditMode() == False ||
         (WSGIappMouse()->getTargetBtn() != WS_MOUSE_BTN3)){
+#else
+    if (WSGIappMouse()->getTargetBtn() != WS_MOUSE_BTN3){
+#endif
       if (_ptarget != NULL){
         _ptarget->onMouseRelease(point);
       }
@@ -622,25 +692,40 @@ void WSCpulldownMenu::onMouseRelease(WSCpoint* point){
     return;
   }
   if (_press_state != False){
+#ifndef WS_EMBED
     if (getPropertyEditMode() == False ||
         (WSGIappMouse()->getTargetBtn() != WS_MOUSE_BTN3)){
+#else
+    if (WSGIappMouse()->getTargetBtn() != WS_MOUSE_BTN3){
+#endif
       short px,py;
       getMouseAddr(&px,&py);
+#ifndef WS_EMBED
       if (_grabed == 0 &&
           -1 < px && px < (signed)_w && -1 < py && py < (signed)_h &&
           getPropertyEditMode() == False){
+#else
+      if (_grabed == 0 &&
+          -1 < px && px < (signed)_w && -1 < py && py < (signed)_h){
+#endif
         WSDdev* pdev = getParentDev();
         WSCbool fl = True;
+        WSGIappMouse()->setMouseUngrabClient(this);
         pdev->setValue(WSDEV_GRAB_POINTER,&fl);
         _grabed = 1;
         _mouse_grabed = 1;
 //        _out_side_mouse_move = True;
         _out_side_mouse_release = True;
         _out_side_mouse_press = True;
+#ifndef WS_EMBED
       }else if (_grabed == 0 && _mouse_no_check == True &&
           getPropertyEditMode() == False){
+#else
+      }else if (_grabed == 0 && _mouse_no_check == True){
+#endif
         WSDdev* pdev = getParentDev();
         WSCbool fl = True;
+        WSGIappMouse()->setMouseUngrabClient(this);
         pdev->setValue(WSDEV_GRAB_POINTER,&fl);
         _grabed = 1;
         _mouse_grabed = 1;
@@ -650,11 +735,23 @@ void WSCpulldownMenu::onMouseRelease(WSCpoint* point){
         if (_grabed != 0){
           WSDdev* pdev = getParentDev();
           WSCbool fl = False;
+          WSGIappMouse()->setMouseUngrabClient(NULL);
           pdev->setValue(WSDEV_GRAB_POINTER,&fl);
           _grabed = 0;
           _out_side_mouse_press = False;
         }
         _menu_popdown();
+
+        long val = getValue();
+        if (getValueToggleMenu(val) != False &&
+            getValueSensitive(val) != False){
+          WSCbool fl = getValueToggleState(val);
+          if (fl == False){
+            setValueToggleState(val,True);
+          }else{
+            setValueToggleState(val,False);
+          }
+        }
         execProcedure(WSEV_VALUE_CH);
 
         setAbsoluteDraw(True);
@@ -699,7 +796,7 @@ WSMFtrace("WSCpulldownMenu::addProcedure trigger=%d not supported.\n", ac->getTr
                                  WSEV_MOUSE_RELEASE_BIT | WSEV_MOUSE_OUT_BIT );
 
     _event_mask |= ac->getEffectiveTriggerMask();
-    _add_prcs(&_prc_list, ac);
+    _add_prcs(ac);
 
     WSDdev* dev = getowndev();
     if (dev != NULL){
@@ -716,9 +813,9 @@ long WSCpulldownMenu::beginCascade(char* lb){
   str.delHeadSpace();
   str.delTailSpace();
   if (strcmp(str.getString(),"")){
-    str += ",";
+    str += _separator2;
   }
-  str = str + lb + ":{";
+  str = str + lb + _separator + "{";
   setPropertyV(WSNmenuItems,str.getString());
   return WS_NO_ERR;
 }
@@ -759,21 +856,29 @@ void WSCpulldownMenu::_menu_popup(WSCpoint*){
     spobj->setFocus(False);
   }
 
-  WSGIappKeyboard()->setFocusInputWorkProc(_key_handler,NULL,this);
+  onMenuPopup();
+
+  _popup_instance = this;
+  WSGIappKeyboard()->setFocusInputWorkProc(_key_handler,NULL,this,this);
 
   if (_menu != NULL){
     WSCstring str(_menu_str);  
-    long num = str.getWords(",");    
+    long num = str.getWords(_separator2);    
 
     short len = 0;
     short ww = 0;
     short hh = st*2;
     long i;
     WSDfont* fn = WSGIappFontSet()->getFont(_font);
+    //--- [20041015]
+    // We use this variable to determinate if menu will use
+    // any pixmap on any option
+    WSCbool use_pixmap = False;
+    //---
     for (i=0; i<num; i++) {
-      WSCstring sect = str.getWord(i, ",");
-      WSCstring sect_str = sect.getWord(0, ":");
-      WSCstring op_name = sect.getWord(1, ":");
+      WSCstring sect = str.getWord(i, _separator2);
+      WSCstring sect_str = sect.getWord(0, _separator);
+      WSCstring op_name = sect.getWord(1, _separator);
 
       if (!strcmp(sect_str.getString(),"SP") ||
           !strcmp(sect_str.getString(),"SEPARATOR")){
@@ -784,7 +889,7 @@ void WSCpulldownMenu::_menu_popup(WSCpoint*){
       }else{
         if ( !strncmp(op_name.getString(),"{",1) ){
           do_cascade = True;
-          long wpos = str.getWordCharPos(i, ",");
+          long wpos = str.getWordCharPos(i, _separator2);
           WSCstring tmp(str.getString());
           tmp.deleteChars(0,wpos);
           wpos = tmp.getWordCharPos(1,"{");
@@ -807,14 +912,30 @@ void WSCpulldownMenu::_menu_popup(WSCpoint*){
             if (tstr[j] == '{'){
               layers++;
             }
-            if (tstr[j] == ','){
+            if (tstr[j] == _separator2[0]){
               num_of_separator++;
             }
           }
           i += num_of_separator;
         }
-        hh += _h;
+        if (_item_height > -1){
+          hh += _item_height;
+        }else{
+          hh += _h;
+        }
       }
+//--- [20041015]
+      // Here, we have to determinate if puldownMenuPopup of first level
+      // will have pixmaps. We need to activate use_pixmap only if there
+      // is a fifth option in menu item string and if this menu item string
+      // doesn't content a sub-menu. This is, if this menu item string
+      // hasn't got "{" symbol at second option.
+      WSCstring tmp_second_opt = sect.getWord(1, _separator);
+      if ((strlen(sect.getWord(4, _separator).getString()) > 0) &&
+        !(strchr(tmp_second_opt.getString(), '{'))){
+        use_pixmap = True;
+      }
+//---
 
       if (len < (short)strlen(sect_str.getString())) {
         len = (short)strlen(sect_str.getString());
@@ -823,22 +944,40 @@ void WSCpulldownMenu::_menu_popup(WSCpoint*){
         ww = (short)fn->getStringWidth(&sect_str) + st * 2 + st * 2 + smargin;
       }
     }
+    //--- [20041015]
+    // Here, if there are pixmaps, we need to make pulldownMenuPopup
+    // 24 pixles + smargin widest
+    if (use_pixmap != False){
+      WSCstring space(" ");
+      WSCushort smargin = WSGIappFontSet()->getStringWidth(_font,&space);
+      ww = ww + 32 + smargin;
+    }
+    //---
 
     if (do_cascade != False){
       ww += 30;
     }
-
-    if (px + ww > WSGIappDev()->getWidth()) {
-      px -= (px + ww - WSGIappDev()->getWidth());
+    WSCulong dw = WSGIappDev()->getWidth();
+    if (px < dw){
+      if (px + ww > dw){
+        px -= (px + ww - dw);
+      }
     }
     py += _h;
-    if (py + hh > WSGIappDev()->getHeight()) {
-      py -= (py + hh - WSGIappDev()->getHeight());
+    WSCulong dh = WSGIappDev()->getHeight();
+    if (py < dh){
+      if (py + hh > dh) {
+        py -= (py + hh - dh);
+      }
     }
 
     _menu->setPropertyV(WSNx, (short)(px));
     _menu->setPropertyV(WSNy, (short)(py));
-    _menu->setPropertyV(WSNmenuItemHeight,  _h);
+    if (_item_height > -1){
+      _menu->setProperty(WSNmenuItemHeight,  _item_height);
+    }else{
+      _menu->setProperty(WSNmenuItemHeight,  _h);
+    }
     _menu->setPropertyV(WSNfont, (WSCuchar)_font);
     _menu->setPropertyV(WSNforeColor, _fore_color);
     _menu->setPropertyV(WSNbackColor, _back_color);
@@ -853,9 +992,18 @@ void WSCpulldownMenu::_menu_popup(WSCpoint*){
     _menu->setPropertyV(WSNbottomShadowColor, _bs_color);
     _menu->setPropertyV(WSNmenuItems, _menu_str);
     _menu->setPropertyV(WSNheight, (WSCushort)(hh));
-    _menu->setPropertyV(WSNwidth, (WSCushort)(ww));
+    if (_menu->isToggle() == False || use_pixmap != False){
+      _menu->setPropertyV(WSNwidth, (WSCushort)(ww));
+    }else{
+      _menu->setPropertyV(WSNwidth, (WSCushort)(ww+32));
+    }
     _menu->setPropertyV(WSNpixmapStyle, (WSCuchar)(WS_DYNAMIC_PIXMAP));
+    _menu->set_id_value(0);
     _menu->setVisible(True);
+    //--- [20041015]
+    _menu->set_use_pixmap(use_pixmap);
+    //---
+
 //    WSGFstretchPopup(_menu);
   }
 }
@@ -879,6 +1027,7 @@ void WSCpulldownMenu::_menu_update(){
 }
 
 void WSCpulldownMenu::_menu_popdown(){
+  _popup_instance = NULL;
   WSGIappKeyboard()->resetFocusInputWorkProc();
 
   _out_side_mouse_press = False;
@@ -898,6 +1047,13 @@ long WSCpulldownMenu::setValueSensitive(short id, WSCbool fl) {
   }
   return ret;
 }
+WSCbool WSCpulldownMenu::getValueSensitive(short id) {
+  long ret = False;
+  if (_menu != NULL){
+    ret = _menu->getValueSensitive(id);
+  }
+  return ret;
+}
 
 long WSCpulldownMenu::setItemSensitive(short no, WSCbool fl) {
   if (no < 0){
@@ -915,6 +1071,40 @@ WSCbool* WSCpulldownMenu::getItemSensitive() {
   }
   return NULL;
 }
+long WSCpulldownMenu::setValueToggleMenu(short no, WSCbool fl) {
+  if (no < 0){
+    return WS_ERR;
+  }
+  if (_menu != NULL){
+    return _menu->setValueToggleMenu(no,fl);
+  }
+  return WS_ERR;
+}
+long WSCpulldownMenu::setValueToggleState(short no, WSCbool fl) {
+  if (no < 0){
+    return WS_ERR;
+  }
+  if (_menu != NULL){
+    return _menu->setValueToggleState(no,fl);
+  }
+  return WS_ERR;
+}
+WSCbool WSCpulldownMenu::getValueToggleMenu(short id) {
+  long ret = False;
+  if (_menu != NULL){
+    ret = _menu->getValueToggleMenu(id);
+  }
+  return ret;
+}
+WSCbool WSCpulldownMenu::getValueToggleState(short id) {
+  long ret = False;
+  if (_menu != NULL){
+    ret = _menu->getValueToggleState(id);
+  }
+  return ret;
+}
+
+
 
 long WSCpulldownMenu::getValue(){
   if (_menu != NULL){
@@ -923,13 +1113,23 @@ long WSCpulldownMenu::getValue(){
   }
   return 0;
 }
-
-long WSCpulldownMenu::addItem(char* lb,char* op,char* sc,long id){
+//--- [20041015}
+// OLD CODE
+//long WSCpulldownMenu::addItem(char* lb,char* op,char* sc,long id){
+// NEW CODE
+long WSCpulldownMenu::addItem(char* lb,char* op,char* sc,long id,char *px){
+//---
   WSCstring str(_menu_str);
   if (_menu_str[0] != 0 && _menu_str[strlen(_menu_str)-1] != '{'){
-    str += ",";
+    str += _separator2;
   }
-  str = str + lb + ":" + op + ":" + sc + ":" + WSGFltoa(id);
+//--- [20041015]
+  if (px == NULL){
+    str = str + lb + _separator + op + _separator + sc + _separator + WSGFltoa(id);
+  }else{
+    str = str + lb + _separator + op + _separator + sc + _separator + WSGFltoa(id) + _separator + px;
+  }
+//---
   setPropertyV(WSNmenuItems,str.getString());
   return WS_NO_ERR;
 }
@@ -941,10 +1141,73 @@ long WSCpulldownMenu::getItems(){
   return 0;
 }
 long WSCpulldownMenu::execEventProc(long ev,void* data){
+dbprintf("WSCpulldownMenu::execEventProc ev=%d\n",ev);
+  if (ev == WSEV_GRAB_CANCELED){
+dbprintf("WSCpulldownMenu::execEventProc _grabed=%d\n",_grabed);
+    getParent()->setUserData("PLD-PUSHED",(void*)0);
+    if (_grabed != 0){
+      WSDdev* pdev = getParentDev();
+      WSCbool fl = False;
+      WSGIappMouse()->setMouseUngrabClient(NULL);
+      if (pdev != NULL){
+        pdev->setValue(WSDEV_GRAB_POINTER,&fl);
+      }
+      _grabed = 0;
+      _out_side_mouse_press = False;
+    }
+    _menu_popdown();
+  
+    setAbsoluteDraw(True);
+    redraw();
+  }
+#ifndef WS_EMBED
   if (ev == WSEV_KEY_PRESS && getPropertyEditMode() == False &&
       getVisible() != False){
+#else
+  if (ev == WSEV_KEY_PRESS && getVisible() != False){
+#endif
     long key = WSGIappKeyboard()->getKey();
     char* text = WSGIappKeyboard()->getText();
+
+    if (_popup_instance != NULL){
+      if (_popup_instance->_menu != NULL &&
+          _popup_instance->_menu->getVisible() != False){
+        char buf[2];
+        buf[0] = (char)toupper(text[0]);
+        buf[1] = 0;
+        WSCpulldownMenu* _target = _popup_instance;
+//        WSCstring pname = _target->_get_proc_name_by_sc(
+//                                            _popup_instance->_menu_str,buf);
+        int act_val = -1;
+        WSCstring pname = _target->_get_proc_name_by_sc(buf,&act_val);
+        if (strcmp(buf,"") && strcmp(pname,"")){
+          WSCpoint pt;
+          pt.x = 0;
+          pt.y = 0;
+          _target->onMousePress(&pt);
+          _target->onMouseRelease(&pt);
+
+          if (_target->getValueToggleMenu(act_val) != False &&
+              _target->getValueSensitive(act_val) != False){
+            WSCbool fl = _target->getValueToggleState(act_val);
+            if (fl == False){
+              _target->setValueToggleState(act_val,True);
+            }else{
+              _target->setValueToggleState(act_val,False);
+            }
+          }
+          _target->execProcedure(WSEV_VALUE_CH);
+
+          if (_target->getSensitive() != False){
+            _target->onActivate();
+            _target->execProcedure((char*)pname);
+          }
+          WSGIappKeyboard()->setText("");
+          return WSCvlabel::execEventProc(ev,data);
+        }
+      }
+    }
+
     if (key != 0 && text[0] != 0 && _sc_str != NULL){
       int val1 = toupper(text[0]);
       int val2 = toupper(_sc_str[0]);
@@ -972,14 +1235,30 @@ void WSCpulldownMenu::_key_handler(void* ptr,WSDkeyboard* keyboard,WSCbool fl){
       char buf[2];
       buf[0] = (char)toupper(text[0]);
       buf[1] = 0;
-      WSCstring pname = obj->_get_proc_name_by_sc(obj->_menu_str,buf);
+//      WSCstring pname = obj->_get_proc_name_by_sc(obj->_menu_str,buf);
+      int act_val = -1;
+      WSCstring pname = obj->_get_proc_name_by_sc(buf,&act_val);
       if (strcmp(buf,"") && strcmp(pname,"")){
         WSCpoint pt;
         pt.x = 0;
         pt.y = 0;
         obj->onMousePress(&pt);
         obj->onMouseRelease(&pt);
-        obj->execProcedure((char*)pname);
+        if (obj->getValueToggleMenu(act_val) != False &&
+            obj->getValueSensitive(act_val) != False){
+          WSCbool fl = obj->getValueToggleState(act_val);
+          if (fl == False){
+            obj->setValueToggleState(act_val,True);
+          }else{
+            obj->setValueToggleState(act_val,False);
+          }
+        }
+        obj->execProcedure(WSEV_VALUE_CH);
+
+        if (obj->getSensitive() != False){
+          obj->onActivate();
+          obj->execProcedure((char*)pname);
+        }
       }else
       if (key == WSK_Up || key == WSK_Down || key == WSK_Left || key == WSK_Right){
         obj->_menu->setUserData("KEY_PROCESSED",(void*)0);
@@ -1013,7 +1292,11 @@ void WSCpulldownMenu::_key_handler(void* ptr,WSDkeyboard* keyboard,WSCbool fl){
           }
         }
       }else
-      if (key == WSK_Escape){
+      if (key == WSK_Escape || key == WSK_Super_L || key == WSK_Super_R ||
+          key == WSK_Control_L || key == WSK_Control_R || key == WSK_Apps ||
+          key == WSK_Alt_L || key == WSK_Alt_R ||
+          key == WSK_Meta_L || key == WSK_Meta_R ){
+
         obj->_menu->setSelected(False);
         WSCpoint pt;
         pt.x = 0;
@@ -1022,7 +1305,7 @@ void WSCpulldownMenu::_key_handler(void* ptr,WSDkeyboard* keyboard,WSCbool fl){
         obj->onMouseRelease(&pt);
         keyboard->setKey(0); 
       }
-      if (key == WSK_Return){
+      if (key == WSK_Return || key == WSK_KP_Enter){
         WSCpoint pt;
         pt.x = 0;
         pt.y = 0;
@@ -1032,23 +1315,40 @@ void WSCpulldownMenu::_key_handler(void* ptr,WSDkeyboard* keyboard,WSCbool fl){
     }
   }
 }
+WSCstring WSCpulldownMenu::_get_proc_name_by_sc(char* sc,int* val){
+  WSCstring ret;
+  if (_menu != NULL && _menu->getVisible() != False){
+    ret = _menu->_get_proc_name_by_sc(sc,val);
+  }
+  return ret;
+}
+#if 0
 WSCstring WSCpulldownMenu::_get_proc_name_by_sc(char* menustr,char* sc){
   if (menustr == NULL){
     WSCstring ret;
     return ret;
   }
   WSCstring str(menustr);
-  long num = str.getWords(",");
+  long num = str.getWords(_separator2);
   long i;
   for(i=0; i<num; i++){
     WSCstring tmp;
-    tmp = str.getWord(i,",");
+    tmp = str.getWord(i,_separator2);
     WSCstring submenu;
-    submenu = tmp.getWord(1,":"); 
+    submenu = tmp.getWord(1,_separator); 
     submenu.delHeadSpace();
     submenu.delTailSpace();
     char* submenu1 = submenu.getString(WS_EN_UTF8);
-    long sblen = strlen(submenu1);
+//    long sblen = strlen(submenu1);
+    long level = 1;
+    while(submenu1[0] == '{'){
+      level++;
+      submenu = tmp.getWord(level,_separator); 
+      submenu.delHeadSpace();
+      submenu.delTailSpace();
+      submenu1 = submenu.getString(WS_EN_UTF8);
+    }
+#if 0
     if (submenu1[0] == '{' && submenu1[sblen-1] == '}'){
       submenu1[sblen-1] = 0;
       WSCstring t(&submenu1[1],WS_EN_UTF8);
@@ -1057,27 +1357,42 @@ WSCstring WSCpulldownMenu::_get_proc_name_by_sc(char* menustr,char* sc){
         return procn;
       }
     }
+#endif
     WSCstring scname;
-    scname = tmp.getWord(2,":"); 
+    scname = tmp.getWord(level+1,_separator); 
     scname.to_upper();
-    if (!strcmp(scname,sc)){
+    WSCstring valname;
+    valname = tmp.getWord(level+2,_separator); 
+    long val = atol((char*)valname);
+    if (!strncmp(scname.getString(),sc,1)){
+      if (val > -1 && getValueSensitive(val) == False){
+        WSCstring ret;
+        return ret;
+      }
       return submenu;
     }
   }
   WSCstring ret;
   return ret;
 }
+#endif
 void(*WSCpulldownMenu::_ext_draw_proc)(void*) = NULL;
-WSCstring WSCpulldownMenu::_policy_name;
+char* WSCpulldownMenu::_policy_name = NULL;
 
 void WSCpulldownMenu::onGuiPolicyChange(long policy){
   WSCvlabel::onGuiPolicyChange(policy);
   if (WSGIappDev()->getExtGuiPolicySelected() == False){
     _ext_draw_proc = NULL;
   }else{
-    if (!strcmp((char*)_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
+    if (_policy_name != NULL &&
+        !strcmp(_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
     }else{
       _ext_draw_proc = (void(*)(void*))WSGIappDev()->getExtGuiPolicyProc("WSCpulldownMenu::draw");
+      if (_policy_name != NULL){
+        delete _policy_name;
+      }
+      _policy_name = WSGFstrdup(WSGIappDev()->getExtGuiPolicyName());
+
     }
   }
   if (getVisible() != False){
