@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCscrForm.h>
@@ -36,6 +32,10 @@
 #include <WSDmouse.h>
 #include <WSCguiPolicy.h>
 
+#ifdef WS_EMBED
+#include <WSCvscrBar.h>
+#endif
+
 #define NO_PIX
 
 WSMFguiClassInitialize(WSCscrForm,WSCform);
@@ -122,9 +122,10 @@ WSCscrForm::WSCscrForm(WSCbase* base,char* objname):
     WSMFpropertyValueChangeDef( WSCscrForm,WSNshadowType ,char);
 
     WSMFpropertyVisibleOff(WSNshadowType);
+#ifndef NO_GRADATION
     WSMFpropertyDelete(WSNgradation);
     WSMFpropertyDelete(WSNgradationMargin);
-
+#endif
     WSMFaddTrigger(WSEV_VALUE_CH    );
     WSMFaddTrigger(WSEV_ACTIVATE    );
     WSMFaddExtTrigger(WSEV_SCR_MOUSE_PRESS,"SCR-MOUSE-PRESS"  );
@@ -501,8 +502,14 @@ long WSCscrForm::initialize(){
   scrname.setString(getInstanceName());
   addstr.setString("-hscr");
   scrname.addString(addstr);
-
+#ifdef WS_EMBED
+  WSCbase* htmp = new WSCvscrBar(this,scrname.getString());
+#else
   WSCbase* htmp = WSCbase::getNewInstance(getUseScrBarClass(),this,scrname.getString());
+#endif
+  if (htmp == NULL){
+WSMFtrace("Error: WSCscrForm cannot create ScrBarClass:%s.\n",getUseScrBarClass());
+  }
   _h_scr = (WSCvscrBar*)htmp->cast("WSCvscrBar");
   if (_h_scr == NULL){
 WSMFtrace("Error: WSCscrForm cannot create ScrBarClass:%s.\n",getUseScrBarClass());
@@ -525,11 +532,19 @@ WSMFtrace("Error: WSCscrForm cannot create ScrBarClass:%s.\n",getUseScrBarClass(
   _h_scr->setPropertyV(WSNtopShadowColor,_ts_color);
   _h_scr->setPropertyV(WSNbottomShadowColor,_bs_color);
   _h_scr->setPropertyV(WSNbarShadowColor,_bar_back_color);
+  _h_scr->setPropertyV(WSNpageIncrement,_page_increment);
 
   scrname.setString(getInstanceName());
   addstr.setString("-vscr");
   scrname.addString(addstr);
+#ifdef WS_EMBED
+  WSCbase* vtmp = new WSCvscrBar(this,scrname.getString());
+#else
   WSCbase* vtmp = WSCbase::getNewInstance(getUseScrBarClass(),this,scrname.getString());
+#endif
+  if (vtmp == NULL){
+WSMFtrace("Error: WSCscrForm cannot create ScrBarClass:%s.\n",getUseScrBarClass());
+  }
   _v_scr = (WSCvscrBar*)vtmp->cast("WSCvscrBar");
   if (_v_scr == NULL){
 WSMFtrace("Error: WSCscrForm cannot create ScrBarClass:%s.\n",getUseScrBarClass());
@@ -553,6 +568,7 @@ WSMFtrace("Error: WSCscrForm cannot create ScrBarClass:%s.\n",getUseScrBarClass(
   _v_scr->setPropertyV(WSNtopShadowColor,_ts_color);
   _v_scr->setPropertyV(WSNbottomShadowColor,_bs_color);
   _v_scr->setPropertyV(WSNbarShadowColor,_bar_back_color);
+  _v_scr->setPropertyV(WSNpageIncrement,_page_increment);
   if (_virtual_scr != 0){
     setWorkWSNvirtualScroll(1);
   }
@@ -573,6 +589,7 @@ long WSCscrForm::_device_initialize(){
   _adjust();
 
   dev->setEnableEventBit( WSEV_MOUSE_MOVE_BIT | WSEV_MOUSE_OUT_BIT |
+                              WSEV_MOUSE_IN_BIT |
                               WSEV_MOUSE_PRESS_BIT  | WSEV_MOUSE_RELEASE_BIT );
 
   if (_scr_area != NULL){
@@ -620,9 +637,10 @@ WSCushort WSCscrForm::_value_pressed = 0;
 
 void WSCscrForm::_redraw_scr(){
   if (_scr_area != NULL){
-    if (_virtual_scr == False){
-      _scr_area->redraw();
-    }else{
+//    if (_virtual_scr == False){
+//printf("here2!!!!\n");
+//      _scr_area->redraw();
+//    }else{
       if (_pixmap_style == WS_DIRECT_WINDOW){
         _scr_area->redraw();
       }else{
@@ -630,7 +648,7 @@ void WSCscrForm::_redraw_scr(){
           _scr_area->getdev()->exposeArea(0,0,_work_width,_work_height);
         }
       }
-    }
+//    }
   }
 }
 
@@ -646,13 +664,13 @@ long WSCscrForm::redraw(){
   if (getVisible() == False){
     return WS_ERR;
   }
-  long er = dev->beginDraw(0,0,_w,_h,True);
-  if (er != WS_NO_ERR){
-    return WS_NO_ERR;
-  }
+//  long er = dev->beginDraw(0,0,_w,_h,True);
+//  if (er != WS_NO_ERR){
+//    return WS_NO_ERR;
+//  }
   dev->clearArea(_shadow_thick,_shadow_thick,
                  _w - _shadow_thick *2,_h - _shadow_thick *2,True);
-  dev->endDraw();
+//  dev->endDraw();
 #endif //NO_PIX
   return WS_NO_ERR;
 }
@@ -897,11 +915,11 @@ long WSCscrForm::draw(){
   }
 
   if (_scr_area == NULL){
-    //֥ȤνƤʤ
+    //֥ȤνƤʤ...
     return WS_NO_ERR;
   }
 
-  //ץѥƥѹƤСߤ˳ƥѡĤοι
+  //ץѥƥѹƤСߤ˳ƥѡĤοι...
   if (isNeedUpdate() != False){
     _adjust();
   }
@@ -985,8 +1003,10 @@ long WSCscrForm::draw(){
       st.bottom_shadow_color = _bs_color;
       st.work_back_color = _work_back_color;
       st.back_pixmap = _bp_color;
+#ifndef NO_GRADATION
       st.gradation = _grad_no;
       st.gradation_margin = _grad_margin;
+#endif
       st.bar_thickness = _bar_thick;
       st.margin_width = _marginw;
       st.padding = _padding;
@@ -1138,7 +1158,7 @@ WSCbool WSCscrForm::_get_vbar_visible(){
 
   return False;
 }
-
+#ifndef WS_EMBED
 long WSCscrForm::setPropertyEditMode(WSCbool fl){
   long ret = WSCbase::setPropertyEditMode(fl);
   if (_scr_area != NULL){
@@ -1147,7 +1167,7 @@ long WSCscrForm::setPropertyEditMode(WSCbool fl){
   }
   return ret;
 }
-
+#endif
 long WSCscrForm::getVisibleWidth(){
       return _scr_area_w;
 }
@@ -1186,19 +1206,31 @@ void WSCscrForm::_h_scr_work_op(WSCbase* obj){
   WSCulong val;
   WSCushort sld_width;
   WSCushort current;
-  sld_width = scr->getProperty(WSNscrollWidth);
+//  sld_width = scr->getProperty(WSNscrollWidth);
+  sld_width = scr->_scroll_width;
+  if (sld_width == 0){
+    sld_width = 1;
+  }
   scr->_scr_area->getPropertyV(WSNhbarValue,&current);
 
   obj->getPropertyV(WSNvalue,&val);
  
-  if (val == 0 || current - val > sld_width ||
-                  val - current > sld_width ||
+  if (val == 0 || current - val >= sld_width ||
+                  val - current >= sld_width ||
                   bar->getMaxSlideValue() <= val){
     if ( bar->getMaxSlideValue() <= val){
       val = bar->getMaxSlideValue();
     }else{
-      val -= val % sld_width;
+      if ((long)current > (long)val){
+        val -= ((long)current - (long)val) % sld_width;
+      }else{
+        val -= ((long)val - (long)current) % sld_width;
+      }
+      if ((long)val < 0){
+        val = 0;
+      }
     }
+    bar->_set_internal_value(val);
     if ( scr->_wx_pos != val){
       scr->_wx_pos = val;
       scr->_scr_offset_x = -val;
@@ -1235,17 +1267,29 @@ void WSCscrForm::_v_scr_work_op(WSCbase* obj){
   WSCulong val;
   WSCushort sld_width;
   WSCushort current;
-  sld_width = scr->getProperty(WSNscrollHeight);
+//  sld_width = scr->getProperty(WSNscrollHeight);
+  sld_width = scr->_scroll_height;
+  if (sld_width ==0){
+    sld_width = 1;
+  }
   scr->_scr_area->getPropertyV(WSNvbarValue,&current);
   obj->getPropertyV(WSNvalue,&val);
-  if (val == 0 || (long)((long)current - (long)val) > (long)sld_width ||
-                  (long)((long)val - (long)current) > sld_width ||
+  if (val == 0 || (long)((long)current - (long)val) >= (long)sld_width ||
+                  (long)((long)val - (long)current) >= sld_width ||
                   bar->getMaxSlideValue() <= val){
     if ( bar->getMaxSlideValue() <= val){
       val = bar->getMaxSlideValue();
     }else{
-      val -= val % sld_width;
+      if ((long)current > (long)val){
+        val -= ((long)current - (long)val) % sld_width;
+      }else{
+        val -= ((long)val - (long)current) % sld_width;
+      }
+      if ((long)val < 0){
+        val = 0;
+      }
     }
+    bar->_set_internal_value(val);
     if ( scr->_wy_pos != val){
       scr->_wy_pos = val;
       scr->_scr_offset_y = -val;
@@ -1281,7 +1325,7 @@ void WSCscrForm::onChildAdded(WSCbase* child){
 static char* _scr_bar_cname = NULL;
 void WSCscrForm::setUseScrBarClass(char* cname){
   if (_scr_bar_cname != NULL){
-    delete _scr_bar_cname;
+    delete[] _scr_bar_cname;
     _scr_bar_cname = NULL;
   }
   if (cname != NULL){
@@ -1296,16 +1340,21 @@ char* WSCscrForm::getUseScrBarClass(){
 }
 
 void(*WSCscrForm::_ext_draw_proc)(void*) = NULL;
-WSCstring WSCscrForm::_policy_name;
+char* WSCscrForm::_policy_name = NULL;
 
 void WSCscrForm::onGuiPolicyChange(long policy){
   WSCbase::onGuiPolicyChange(policy);
   if (WSGIappDev()->getExtGuiPolicySelected() == False){
     _ext_draw_proc = NULL;
   }else{
-    if (!strcmp((char*)_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
+    if (_policy_name != NULL &&
+        !strcmp(_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
     }else{
       _ext_draw_proc = (void(*)(void*))WSGIappDev()->getExtGuiPolicyProc("WSCscrForm::draw");
+      if (_policy_name != NULL){
+        delete[] _policy_name;
+      }
+      _policy_name = WSGFstrdup(WSGIappDev()->getExtGuiPolicyName());
     }
   }
   setAbsoluteDraw(True);
@@ -1318,9 +1367,11 @@ void WSCscrForm::onMousePress(WSCpoint* pt){
     }
     long val = _wy_pos - _page_increment;
     if (val < 0){
-      setProperty(WSNvbarValue,0);
+//      setProperty(WSNvbarValue,0);
+      _v_scr->setProperty(WSNvalue,0);
     }else{
-      setProperty(WSNvbarValue,val);
+//      setProperty(WSNvbarValue,val);
+      _v_scr->setProperty(WSNvalue,val);
     }
     return;
   }else
@@ -1334,7 +1385,8 @@ void WSCscrForm::onMousePress(WSCpoint* pt){
     if (val > max - slidersize){
       val = max - slidersize;
     }
-    setProperty(WSNvbarValue,val);
+//    setProperty(WSNvbarValue,val);
+    _v_scr->setProperty(WSNvalue,val);
     return;
   }
 
@@ -1414,18 +1466,19 @@ void WSCscrForm::_exec_child_adjust_for_anchors(){
       WSClistData children = getChildren();
       long num = children.getNum();
       long i;
+      WSCbase** buf = (WSCbase**)children._data;
       for(i=0; i<num; i++){
-        WSCbase* child = (WSCbase*)children[i];
+        WSCbase* child = buf[i];
 //        if (child == _scr_area || child == _v_scr || child == _h_scr){
 //          continue;
 //        }
         child->_adjust_for_anchors(_work_width,_work_height);
         if (_pixmap_style == WS_DYNAMIC_PIXMAP){
           if (child->isNeedUpdate() != False){
-            if ((long)child->getUserData("anchor update..") == 0){
-              child->setUserData("anchor update..",(void*)1);
+            if (child->_flag_for_anchor == False){
+              child->_flag_for_anchor = True;
               child->update();
-              child->setUserData("anchor update..",(void*)0);
+              child->_flag_for_anchor = False;
             }
           }
         }
@@ -1435,7 +1488,14 @@ void WSCscrForm::_exec_child_adjust_for_anchors(){
 }
 long WSCscrForm::getObjectType(){
   return (WS_TYPE_MANAGER | WS_TYPE_FORM | WS_TYPE_SCR_FORM);
+}
+WSCbase* WSCscrForm::getVScrBar(){
+  return _v_scr;
 }   
+WSCbase* WSCscrForm::getHScrBar(){
+  return _h_scr;
+}
+
 #ifdef WS_EMBED
 #ifndef NO_EXT_GUI_POLICY
 class _gui_policy_register_{
