@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCscrFrame.h>
@@ -30,6 +26,7 @@
 #include <WSCdevice.h>
 #include <WSDdev.h>
 #include <WSDmouse.h>
+#include <WSDwindowDev.h>
 
 WSMFguiClassInitialize(WSCscrFrame,WSCform);
 WSMFversion(WSCscrFrame,WSCform);
@@ -193,7 +190,7 @@ long WSCscrFrame::_device_initialize(){
       _dev->setValue(WSDEV_WORK_X,     &val);
       _dev->setValue(WSDEV_WORK_Y,     &val);
     }
-    _dev->setEnableEventBit(WSEV_MOUSE_PRESS_BIT | WSEV_MOUSE_RELEASE_BIT);
+    _dev->setEnableEventBit(WSEV_MOUSE_PRESS_BIT | WSEV_MOUSE_RELEASE_BIT | WSEV_MOUSE_IN_BIT);
   }
   return WS_NO_ERR;
 }
@@ -230,13 +227,21 @@ long WSCscrFrame::redraw(){
   if (getVisible() == False){
     return WS_ERR;
   }
-
-  long err = dev->beginDraw(0,0,_w_w,_w_h,True);
-  if (err != WS_NO_ERR){
-    return WS_NO_ERR;
-  }
-  dev->clearArea(0,0,_w_w,_w_h,True);
-  dev->endDraw();
+//  long noclear = False;
+//  if (_pixmap_style == WS_DYNAMIC_PIXMAP){
+//    WSDwindowDev* wdev = (WSDwindowDev*)dev->cast("WSDwindowDev");
+//    if (wdev != NULL && wdev->getPixmapCreated() == False){
+//      noclear = True;
+//    }
+//  }
+//  long err = dev->beginDraw(0,0,_w_w,_w_h,True);
+//  if (err != WS_NO_ERR){
+//    return WS_NO_ERR;
+//  }
+//  if (noclear == False){
+    dev->clearArea(0,0,_w_w,_w_h,True);
+//  }
+//  dev->endDraw();
 
   return WS_NO_ERR;
 }
@@ -329,8 +334,10 @@ long WSCscrFrame::draw(){
 
 void WSCscrFrame::update(){
   if (_no_scroll != False){
-    _dev->setValue(WSDEV_WORK_WIDTH, &_w_w);
-    _dev->setValue(WSDEV_WORK_HEIGHT,&_w_h);
+    if (_dev != NULL){
+      _dev->setValue(WSDEV_WORK_WIDTH, &_w_w);
+      _dev->setValue(WSDEV_WORK_HEIGHT,&_w_h);
+    }
   }
   if (isNeedUpdate() != False){
     WSCbase::update();
