@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCsform.h>
@@ -32,6 +28,7 @@
 #include <WSDwindowDev.h>
 #include <WSDprivateTimer.h>
 #include <WSCwindow.h>
+#include <WSDkeyboard.h>
 
 WSMFguiClassInitialize(WSCsform,WSCform);
 WSMFversion(WSCsform,WSCform);
@@ -112,6 +109,7 @@ void WSCsform::setWorkWSNrefreshing(WSCbool){
     if (data != (void*)this){
       if (_sep_window != NULL &&
           _sep_window->getVisible() != False){
+        WSGIappKeyboard()->resetSpecialInputWorkProc();
         _sep_window->setVisible(False);
       }
       _move_timer->stopTimer();
@@ -147,7 +145,16 @@ void WSCsform::getWorkWSNbarValue(char**){
 void WSCsform::_parse_pos(char* pos){
   _pos_list.clear();
   WSCstring str(pos);
-
+  str.delHeadSpace();
+  str.delTailSpace();
+  if (!strcmp("",(char*)str)){
+    if (_orientation == WS_VERTICAL){
+      _pos_list.add((void*)(WSCulong)_w);
+    }else{
+      _pos_list.add((void*)(WSCulong)_h);
+    }
+    return;
+  }
   long num = str.getWords(_separator);
   long i;
   long pre = 0;
@@ -168,6 +175,13 @@ void WSCsform::_parse_pos(char* pos){
 void WSCsform::setWorkWSNwidth(WSCushort w){
   WSCform::setWorkWSNwidth(w);
   if (_orientation == WS_VERTICAL){
+    if (!strcmp("",(char*)_pos)){
+      _pos_list.clear();
+      _pos_list.add((void*)(WSCulong)w);
+      _need_adjust = True;
+      _adjust();
+      return;
+    }
     long num = (long)_pos_list.getNum();
     if (w < _shadow_thick*2 + _margin_left + _margin_right + _bw * num + 2){
       w =  _shadow_thick*2 + _margin_left + _margin_right + _bw * num + 2;
@@ -199,7 +213,7 @@ void WSCsform::setWorkWSNwidth(WSCushort w){
     }
     if (num != 0 ){
       if (_pos != NULL){
-        delete _pos;
+        delete[] _pos;
       }
       _pos = WSGFstrdup((char*)str);
       _need_adjust = True;
@@ -215,6 +229,13 @@ void WSCsform::setAdjustChildren(WSCbool fl){
 void WSCsform::setWorkWSNheight(WSCushort h){
   WSCform::setWorkWSNheight(h);
   if (_orientation == WS_HORIZONTAL){
+    if (!strcmp("",(char*)_pos)){
+      _pos_list.clear();
+      _pos_list.add((void*)(WSCulong)h);
+      _need_adjust = True;
+      _adjust();
+      return;
+    }
     long num = (long)_pos_list.getNum();
     if (h < _shadow_thick*2 + _margin_top + _margin_bottom + _bw * num + 2){
       h =  _shadow_thick*2 + _margin_top + _margin_bottom + _bw * num + 2;
@@ -245,7 +266,7 @@ void WSCsform::setWorkWSNheight(WSCushort h){
     }
     if (num != 0 ){
       if (_pos != NULL){
-        delete _pos;
+        delete[] _pos;
       }
       _pos = WSGFstrdup((char*)str);
       _need_adjust = True;
@@ -301,6 +322,14 @@ void WSCsform::setWorkWSNorientation(WSCuchar){
 void WSCsform::getWorkWSNorientation(WSCuchar*){}
 
 WSCsform::~WSCsform(){
+  if (_pos != NULL){
+    delete[] _pos;
+    _pos = NULL;
+  }
+  if (_separator != NULL){
+    delete[] _separator;
+    _separator = NULL;
+  }
 }
 
 long WSCsform::_device_initialize(){
@@ -334,6 +363,10 @@ long WSCsform::draw(){
 extern long _get_ext_children_num(WSClistData* children);
 
 void WSCsform::onMousePress(WSCpoint* pt){
+  if (!strcmp(_pos,"")){
+    WSCform::onMousePress(pt);
+    return;
+  }
   long num = _pos_list.getNum();
   WSClistData children = getChildren();
   long cnum = _get_ext_children_num(&children);
@@ -396,9 +429,14 @@ void WSCsform::onValueChange(){
   execProcedure(WSEV_VALUE_CH);
 }
 void WSCsform::onMouseRelease(WSCpoint* pt){
+  if (!strcmp(_pos,"")){
+    WSCform::onMouseRelease(pt);
+    return;
+  }
   if (_pressed != False && _refreshing == False){
     if (_sep_window != NULL &&
         _sep_window->getVisible() != False){
+      WSGIappKeyboard()->resetSpecialInputWorkProc();
       _sep_window->setVisible(False);
     }
     if (_move_timer != NULL){
@@ -434,9 +472,11 @@ void WSCsform::_clear_area(){
 
 void WSCsform::_move_adjust_work(void* ptr){
   WSCsform* sform = (WSCsform*)ptr;
+  WSGIappKeyboard()->setSpecialInputWorkProc(_special_key_handler,sform,sform);
   if (sform->_pressed == False){
     if (_sep_window != NULL &&
         _sep_window->getVisible() != False){
+      WSGIappKeyboard()->resetSpecialInputWorkProc();
       _sep_window->setVisible(False);
     }
     _move_timer->stopTimer();
@@ -483,6 +523,45 @@ void WSCsform::_move_adjust_work(void* ptr){
     _sep_window->setVisible(True);
   }
 }
+WSCbool WSCsform::_special_key_handler(void* ptr,WSDkeyboard* keyb,WSCbool fl){
+  WSCsform* _this = (WSCsform*)ptr;
+  if (_sep_window == NULL){
+    return True;
+  } 
+  if (_sep_window->getVisible() == False){
+    WSGIappKeyboard()->resetSpecialInputWorkProc();
+    return True;
+  }
+  long key = keyb->getKey();
+  if (key == WSK_Escape || key == WSK_Super_L || key == WSK_Super_R ||
+      key == WSK_Control_L || key == WSK_Control_R || key == WSK_Apps ||
+      key == WSK_Alt_L || key == WSK_Alt_R || 
+      key == WSK_Meta_L || key == WSK_Meta_R ){
+    if (fl == False){
+      return False;
+    }
+    if (!strcmp(_this->_pos,"")){
+      return False;
+    }
+    if (_this->_pressed != False && _this->_refreshing == False){
+      if ( _sep_window->getVisible() != False){
+        WSGIappKeyboard()->resetSpecialInputWorkProc();
+        _sep_window->setVisible(False);
+      }
+      if (_move_timer != NULL){
+        _move_timer->stopTimer();
+      }
+    }
+    _this->_pressed = False;
+    _this->_exec_resize = 1;
+    _this->_need_adjust = True;
+    _this->_adjust();
+    _this->_exec_resize = 0;
+    _this->onValueChange();
+    return False;
+  } 
+  return True;
+}
 void WSCsform::_delay_adjust_work(void* ptr){
   WSCsform* sform = (WSCsform*)ptr;
   sform->_need_adjust = True;
@@ -494,6 +573,11 @@ void WSCsform::_delay_adjust_work(void* ptr){
 
 
 void WSCsform::onMouseMove(WSCpoint* pt){
+  if (!strcmp(_pos,"")){
+    WSCform::onMouseMove(pt);
+    return;
+  }
+
   if (_pressed != False){
     long pos = 0;
     if (_orientation == WS_VERTICAL){
@@ -551,7 +635,7 @@ void WSCsform::onMouseMove(WSCpoint* pt){
       str += WSGFltoa(val);
     }
     if (_pos != NULL){
-      delete _pos;
+      delete[] _pos;
     }
     _pos = WSGFstrdup((char*)str); 
     if (_refreshing != False){
@@ -776,7 +860,7 @@ void WSCsform::_adjust_pos(){
 //printf("WSCsform::_adjust_pos i=%d val=%d\n",i,val*(i+1));
     }
     if (_pos != NULL){
-      delete _pos;
+      delete[] _pos;
     }
     _pos = WSGFstrdup((char*)pos_str);
     _need_adjust = True;
@@ -798,9 +882,13 @@ void WSCsform::_adjust(){
       break;
     }
   }
+#ifndef WS_EMBED
   if (getPropertyEditMode() == False){
     num = i++; 
   }
+#else
+  num = i++; 
+#endif
   for(i=0; i<num; i++){
     long pre = -1;
     if(i > 0){
@@ -857,14 +945,22 @@ void WSCsform::_adjust(){
           rect.height = h;
           child->execEventProc(WSEV_RESIZE,&rect);
         }
-#ifdef MSW
-        WSDdev* dev = child->getdev();
+        //Changed following logic to call update()..
+        child->update();
+#if 0
+//#ifdef MSW
+        WSDdev* dev = child->getowndev();
         if (dev != NULL){
           WSDwindowDev* wdev = (WSDwindowDev*)dev->cast("WSDwindowDev");
           if (wdev != NULL){
             dev->exposeArea(0,0,w,h);
+            WSDdev* dev2 = child->getowndev();
+            if (dev2 != NULL && dev != dev2){
+              dev2->exposeArea(0,0,w,h);
+            }
           }
         }
+//#endif
 #endif
 //        WSCrect rect;
 //        rect.setRect(x,y,w,h);
