@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCtextField.h>
@@ -35,6 +31,10 @@
 #include <WSDappDev.h>
 #include <WSDmouse.h>
 
+#ifdef WS_EMBED
+#include <WSCvscrBar.h>
+#endif
+
 WSMFguiClassInitialize(WSCtextField,WSCform);
 WSMFversion(WSCtextField,WSCform);
 
@@ -42,29 +42,30 @@ WSMFpropertyValueChange( WSCtextField, WSNshadowType , char,WS_SHADOW_IN);
 
 WSCtextField::WSCtextField(WSCbase* base,char* objname):
                 WSCform(base,objname){
+  _register_focus_move();
   _ifd = NULL;
   _v_scr = NULL;
   _h_scr = NULL;
 
+  _hbar_visible = True;
+  _vbar_visible = True;
+
   _bw = 16;
-//  _work_back_color = WSGFcolor("gray80");
-//  _bar_back_color    = WSGFcolor("gray75");
   _work_back_color = WS_DF_WORKBACKCOLOR;
   _bar_back_color  = WS_DF_BARSHADOWCOLOR;
   _margin = 4;
   _enable_input = 1;
   _vpos = 0;
   _hpos = 0;
-//  _label_string = WSGFstrdup("");
   _font = (char)WSGIappFontSet()->getDefaultFontNo();
   _fw = 0;
   _fh = 0;
   _shadow_type = WS_SHADOW_IN;
+  _adjusting = False;
 
   WSMFpropertyCreateStart
     WSMFparentCheckVerSrc(WSCtextField);
 
-//    WSMFpropertyCreate(WSNlabelString, char*, _label_string,WSSlabelString );
     WSMFpropertyCreate(WSNbarThickness, WSCushort, _bw,WSSbarThickness );
     WSMFpropertyCreate(WSNworkBackColor,short,_work_back_color,WSSworkBackColor );
     WSMFpropertyCreate(WSNbarShadowColor, short,_bar_back_color, WSSbarShadowColor  );
@@ -72,6 +73,11 @@ WSCtextField::WSCtextField(WSCbase* base,char* objname):
     WSMFpropertyCreate(WSNenableInput,WSCbool,_enable_input,WSSenableInput );
       WSMFpropertySetSelection(WSRbool3,WSRbool3D);
 
+    WSMFpropertyCreate(WSNhbarVisible,WSCbool,_hbar_visible,WSShbarVisible );
+      WSMFpropertySetSelection(WSRbool2,WSRbool2D);
+    WSMFpropertyCreate(WSNvbarVisible,WSCbool, _vbar_visible,WSSvbarVisible );
+      WSMFpropertySetSelection(WSRbool2,WSRbool2D);
+
     WSMFpropertyValueChangeDef( WSCtextField,WSNshadowType ,char);
 
     WSMFpropertyCreate(WSNvbarValue,WSCulong,_vpos,WSSvbarValue );
@@ -90,40 +96,21 @@ WSCtextField::WSCtextField(WSCbase* base,char* objname):
     WSMFaddExtTrigger(WSEV_SCR_MOUSE_PRESS,"SCR-MOUSE-PRESS"  );
     WSMFaddExtTrigger(WSEV_SCR_MOUSE_RELEASE,"SCR-MOUSE-RELEASE"  );
     WSMFaddExtTrigger(WSEV_SCR_MOUSE_MOVE,"SCR-MOUSE-MOVE"  );
+    WSMFaddExtTrigger(WSEV_SCR_FOCUS_CH,"SCR-FOCUS-CH"  );
 
   WSMFpropertyCreateEnd
 }
 
-//WSMFproperty( WSCtextField, WSNlabelString,char*,_label_string,WSGFstrdup(""));
 WSMFproperty( WSCtextField, WSNbarThickness,  WSCushort, _bw, 16);
-//WSMFproperty( WSCtextField, WSNworkBackColor, short, _work_back_color, WSGFcolor("gray80"));
 WSMFproperty( WSCtextField, WSNworkBackColor, short, _work_back_color, WS_DF_WORKBACKCOLOR);
-//WSMFproperty( WSCtextField, WSNbarShadowColor,short, _bar_back_color,WSGFcolor("gray75"));
 WSMFproperty( WSCtextField, WSNbarShadowColor,short, _bar_back_color,WS_DF_BARSHADOWCOLOR);
 WSMFproperty( WSCtextField, WSNmargin, WSCuchar,_margin,4);
 WSMFproperty( WSCtextField, WSNenableInput, WSCbool,_enable_input,1);
 WSMFproperty( WSCtextField, WSNvbarValue, WSCulong,_vpos,0);
 WSMFproperty( WSCtextField, WSNhbarValue, WSCulong,_hpos,0);
 WSMFproperty(WSCtextField, WSNfont, WSCuchar, _font,(char)WSGIappFontSet()->getDefaultFontNo());
-#if 0
-void WSCtextField::setWorkWSNlabelString(char* data){
-//printf("WSCtextField::setWorkWSNlabelString\n");
-//TEST
-  return;
-  _ifd->setPropertyV(WSNlabelString,data);
-  if (_v_scr != NULL){
-    WSCstring str = data;
-    long lines = str.getWords("\n");
-    long h = lines * _get_font_height();
-    _v_scr->setPropertyV(WSNmaximum,(WSCulong)h);
-  }
-}
-void WSCtextField::getWorkWSNlabelString(char** data){
-  delete _label_string;
-  _label_string = WSGFstrdup(_ifd->getProperty(WSNlabelString));
-  *data = _label_string;
-}
-#endif
+WSMFproperty( WSCtextField, WSNvbarVisible, WSCbool,_vbar_visible,True);
+WSMFproperty( WSCtextField, WSNhbarVisible, WSCbool,_hbar_visible,True);
 
 void WSCtextField::setWorkWSNvbarValue(WSCulong data){
   long bx,by;
@@ -150,7 +137,6 @@ void WSCtextField::setWorkWSNhbarValue(WSCulong data){
   long bx,by;
   _ifd->getCursorBase(&bx,&by);
   bx = data;
-//printf("WSCtextField::setWokrWSNhbarValue %d\n",data);
   _ifd->setCursorBase(bx,by);
   _ifd->draw();
 }
@@ -182,6 +168,20 @@ void WSCtextField::setWorkWSNmargin(WSCuchar){
 }
 void WSCtextField::getWorkWSNmargin(WSCuchar*){}
 
+void WSCtextField::setWorkWSNvbarVisible(WSCbool){
+  if (_ifd != NULL){
+    _adjust();
+  }
+}
+void WSCtextField::getWorkWSNvbarVisible(WSCbool*){}
+
+void WSCtextField::setWorkWSNhbarVisible(WSCbool){
+  if (_ifd != NULL){
+    _adjust();
+  }
+}
+void WSCtextField::getWorkWSNhbarVisible(WSCbool*){}
+
 
 void WSCtextField::setWorkWSNtopShadowColor(short data){
   if (_ifd != NULL){
@@ -238,6 +238,10 @@ void WSCtextField::setWorkWSNworkBackColor(short data){
 }
 void WSCtextField::getWorkWSNworkBackColor(short*){ }
 void WSCtextField::_adjust(){
+  if (_adjusting != False){
+    return;
+  }
+  _adjusting = True;
   long mg = _margin;
   if (WSGIappDev()->getGuiPolicy() & WS_POLICY_WINDOWS){
     mg = 0;
@@ -245,17 +249,31 @@ void WSCtextField::_adjust(){
   long st = _shadow_thick;
   _form->setProperty(WSNx,st);
   _form->setProperty(WSNy,st);
-  _form->setPropertyV(WSNwidth,(WSCushort)(_w - _bw - mg -st*2));
-  _form->setPropertyV(WSNheight,(WSCushort)(_h - _bw - mg-st*2));
   _ifd->setProperty(WSNx,0);
   _ifd->setProperty(WSNy,0);
-  _ifd->setPropertyV(WSNwidth,(WSCushort)(_w - _bw - mg-st*2));
-  _ifd->setPropertyV(WSNheight,(WSCushort)(_h - _bw - mg-st*2));
+  if (_vbar_visible != False){
+    _form->setPropertyV(WSNwidth,(WSCushort)(_w - _bw - mg -st*2));
+    _ifd->setPropertyV(WSNwidth,(WSCushort)(_w - _bw - mg-st*2));
+  }else{
+    _form->setPropertyV(WSNwidth,(WSCushort)(_w -st*2));
+    _ifd->setPropertyV(WSNwidth,(WSCushort)(_w -st*2));
+  }
+  if (_hbar_visible != False){
+    _form->setPropertyV(WSNheight,(WSCushort)(_h - _bw - mg -st*2));
+    _ifd->setPropertyV(WSNheight,(WSCushort)(_h - _bw - mg-st*2));
+  }else{
+    _form->setPropertyV(WSNheight,(WSCushort)(_h -st*2));
+    _ifd->setPropertyV(WSNheight,(WSCushort)(_h -st*2));
+  }
   if (WSGIappDev()->getGuiPolicy() & WS_POLICY_ORIGINAL){
     _v_scr->setPropertyV(WSNx,(short)(_w - _bw));
     _v_scr->setPropertyV(WSNy,(short)0);
     _v_scr->setPropertyV(WSNwidth,(WSCushort)(_bw));
-    _v_scr->setPropertyV(WSNheight,(WSCushort)(_h - _bw - mg));
+    if (_hbar_visible != False){
+      _v_scr->setPropertyV(WSNheight,(WSCushort)(_h - _bw - mg));
+    }else{
+      _v_scr->setPropertyV(WSNheight,(WSCushort)(_h));
+    }
     WSCulong ssize = (WSCulong)(_h - _bw - _shadow_thick*2- mg);
     if (ssize > _h){
       ssize = 1;
@@ -263,7 +281,11 @@ void WSCtextField::_adjust(){
     _v_scr->setPropertyV(WSNsliderSize,ssize);
     _h_scr->setPropertyV(WSNx,(short)0);
     _h_scr->setPropertyV(WSNy,(short)(_h - _bw));
-    _h_scr->setPropertyV(WSNwidth,(WSCushort)(_w - _bw - mg));
+    if (_vbar_visible != False){
+      _h_scr->setPropertyV(WSNwidth,(WSCushort)(_w - _bw - mg));
+    }else{
+      _h_scr->setPropertyV(WSNwidth,(WSCushort)(_w));
+    }
     ssize = (WSCulong)(_w - _bw - _shadow_thick*2- mg);
     if (ssize > _w){
       ssize = 1;
@@ -276,7 +298,11 @@ void WSCtextField::_adjust(){
     _v_scr->setPropertyV(WSNx,(short)(_w - _bw -st));
     _v_scr->setPropertyV(WSNy,(short)st);
     _v_scr->setPropertyV(WSNwidth,(WSCushort)(_bw));
-    _v_scr->setPropertyV(WSNheight,(WSCushort)(_h - _bw - mg -st*2));
+    if (_hbar_visible != False){
+      _v_scr->setPropertyV(WSNheight,(WSCushort)(_h - _bw - mg -st*2));
+    }else{
+      _v_scr->setPropertyV(WSNheight,(WSCushort)(_h -st*2));
+    }
     WSCulong ssize = (WSCulong)(_h - _bw - _shadow_thick*2- mg);
     if (ssize > _h){
       ssize = 1;
@@ -285,7 +311,11 @@ void WSCtextField::_adjust(){
 
     _h_scr->setPropertyV(WSNx,(short)st);
     _h_scr->setPropertyV(WSNy,(short)(_h - _bw -st));
-    _h_scr->setPropertyV(WSNwidth,(WSCushort)(_w - _bw - mg -st*2));
+    if (_vbar_visible != False){
+      _h_scr->setPropertyV(WSNwidth,(WSCushort)(_w - _bw - mg -st*2));
+    }else{
+      _h_scr->setPropertyV(WSNwidth,(WSCushort)(_w -st*2));
+    }
     ssize = (WSCulong)(_w - _bw - _shadow_thick*2- mg);
     if (ssize > _w){
       ssize = 1;
@@ -293,15 +323,28 @@ void WSCtextField::_adjust(){
     _h_scr->setPropertyV(WSNsliderSize,ssize);
     _h_scr->setPropertyV(WSNheight,(WSCushort)(_bw));
   }
+  if (_vbar_visible != False){
+    _v_scr->setVisible(True);
+  }else{
+    _v_scr->setVisible(False);
+  }
+  if (_hbar_visible != False){
+    _h_scr->setVisible(True);
+  }else{
+    _h_scr->setVisible(False);
+  }
+  _adjusting = False;
 }
 void WSCtextField::setWorkWSNwidth(WSCushort data){
-  _adjust();
+  //_adjust(); //this generates some resize trouble in WIN32
   WSCform::setWorkWSNwidth(data);
+  _adjust();
 }
 
 void WSCtextField::setWorkWSNheight(WSCushort data){
-  _adjust();
+  //_adjust(); //this generates some resize trouble in WIN32
   WSCform::setWorkWSNheight(data);
+  _adjust();
 }
 
 void WSCtextField::setWorkWSNenableInput(WSCbool data){
@@ -350,6 +393,7 @@ long WSCtextField::initialize(){
   if (WSGIappDev()->getGuiPolicy() & WS_POLICY_WINDOWS){
     mg = 0;
   }
+  WSCform::initialize();
 
   WSCstring scrname(getInstanceName());
   scrname << "-form";
@@ -409,14 +453,26 @@ long WSCtextField::initialize(){
   op03->setInternal(True);
   op03->setFunction(_keyhook_work,"_keyhook_work");
   _ifd->addProcedure(op03);
+  WSCprocedure* op04 = new WSCprocedure("focus-ch",WSEV_FOCUS_CH);
+  op04->setInternal(True);
+  op04->setFunction(_focus_ch_work,"_focus_ch_work");
+  _ifd->addProcedure(op04);
 
 
   scrname.setString(getInstanceName());
   scrname << "-hscr";
+#ifdef WS_EMBED
+  WSCbase* htmp = new WSCvscrBar(this,scrname.getString());
+#else
   WSCbase* htmp = WSCbase::getNewInstance(getUseScrBarClass(),this,scrname.getString());
-  _h_scr = (WSCvscrBar*)htmp->cast("WSCvscrBar");
-  if (_h_scr == NULL){
+#endif
+  if (htmp == NULL){
     _h_scr = new WSCvscrBar(this,scrname.getString());
+  }else{
+    _h_scr = (WSCvscrBar*)htmp->cast("WSCvscrBar");
+    if (_h_scr == NULL){
+      _h_scr = new WSCvscrBar(this,scrname.getString());
+    }
   }
 
   _h_scr->setInternalObject(True);
@@ -446,10 +502,18 @@ long WSCtextField::initialize(){
 
   scrname.setString(getInstanceName());
   scrname << "-vscr";
+#ifdef WS_EMBED
+  WSCbase* vtmp = new WSCvscrBar(this,scrname.getString());
+#else
   WSCbase* vtmp = WSCbase::getNewInstance(getUseScrBarClass(),this,scrname.getString());
-  _v_scr = (WSCvscrBar*)vtmp->cast("WSCvscrBar");
-  if (_v_scr == NULL){
+#endif
+  if (vtmp == NULL){
     _v_scr = new WSCvscrBar(this,scrname.getString());
+  }else{
+    _v_scr = (WSCvscrBar*)vtmp->cast("WSCvscrBar");
+    if (_v_scr == NULL){
+      _v_scr = new WSCvscrBar(this,scrname.getString());
+    }
   }
   _v_scr->setInternalObject(True);
   _v_scr->setUseParentBaseDev(True);
@@ -485,7 +549,6 @@ long WSCtextField::initialize(){
     _v_scr->setPropertyV(WSNincrement,(WSCushort)sh);
   }
 
-  WSCform::initialize();
   _adjust();
   _scr_adjust();
   return WS_NO_ERR;
@@ -565,7 +628,7 @@ void WSCtextField::_v_scr_work_op(WSCbase* obj){
 #endif
   }
 }
-WSCbase* WSCtextField::getTextFrame(){
+WSCvifield* WSCtextField::getTextFrame(){
   return _ifd;
 }
 WSCbase* WSCtextField::getPropertyInheritChild(){
@@ -625,9 +688,17 @@ long WSCtextField::draw(){
   }
   if ( _shadow_thick > 0){
     if (WSGIappDev()->getGuiPolicy() & WS_POLICY_ORIGINAL){
+      WSCushort ww = _w - _bw - _margin;
+      if (_vbar_visible == False){
+        ww = _w;
+      }
+      WSCushort hh = _h - _bw - _margin;
+      if (_hbar_visible == False){
+        hh = _h;
+      }
       WSGFdrawShadow(dev,_shadow_type, _shadow_thick, _fore_color,
                    _ts_color,_bs_color,_back_color,
-                   0,0,_w - _bw - _margin,_h - _bw - _margin);
+                   0,0,ww,hh);
     }else
     if (WSGIappDev()->getGuiPolicy() & WS_POLICY_WINDOWS){
       WSGFdrawShadow(dev,_shadow_type, _shadow_thick, _fore_color,
@@ -760,7 +831,7 @@ WSCushort* WSCtextField::getBuf(){
 static char* _scr_bar_cname = NULL;
 void WSCtextField::setUseScrBarClass(char* cname){
   if (_scr_bar_cname != NULL){
-    delete _scr_bar_cname;
+    delete[] _scr_bar_cname;
     _scr_bar_cname = NULL;
   }
   if (cname != NULL){
@@ -883,6 +954,10 @@ long WSCtextField::getBottomLine(){
   }
   return lines;
 }
+void WSCtextField::_focus_ch_work(WSCbase* ifd){
+  WSCtextField* text = (WSCtextField*)ifd->getUserData("CBASE");
+  text->onScrFocusChange(ifd->getFocus());
+}
 void WSCtextField::_scr_mouse_press(WSCbase* obj){
   WSCbase* parent = obj->getParent();
   WSCtextField* scr = (WSCtextField*)parent->cast("WSCtextField");
@@ -915,6 +990,9 @@ void WSCtextField::_scr_mouse_move(WSCbase* obj){
   scr->onScrMouseMove(&pt);
 }
 
+void WSCtextField::onScrFocusChange(WSCbool fl){
+  execProcedure(WSEV_SCR_FOCUS_CH);
+}
 void WSCtextField::onScrMousePress(WSCpoint* pt){
   execProcedure(WSEV_SCR_MOUSE_PRESS);
 }
@@ -953,3 +1031,16 @@ void WSCtextField::onMousePress(WSCpoint* pt){
  
   WSCform::onMousePress(pt);
 }
+WSCbase* WSCtextField::getVScrBar(){
+  return _v_scr;
+}
+WSCbase* WSCtextField::getHScrBar(){
+  return _h_scr;
+}
+void WSCtextField::onFocusChange(WSCbool fl){
+  if (fl != False){
+    if (_ifd != NULL){
+      _ifd->setFocus();
+    }
+  }
+}
