@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCvarc.h>
@@ -134,7 +130,7 @@ long WSCvarc::draw(){
     char kind;
     switch (_arc_type) {
     case WS_ARC_PIE:
-      // 𷿤ξ٤
+      // 𷿤ξ٤...
       kind = (char)WS_PI;
       break;
     default:
@@ -152,12 +148,12 @@ long WSCvarc::draw(){
         } else {
           dev->setForeColor(_back_color);
         }
-        //öطʿɤ٤
+        //öطʿɤ٤...
         dev->setHatchPattern((char)0);
         dev->drawFillArc(xx, yy, ww, hh, _angle1*64, _angle2*64, kind);
       }
     }
-    // ꤵ줿ϥåѥɽǱߤɽ
+    // ꤵ줿ϥåѥɽǱߤɽ...
     dev->setHatchPattern(_hatch_pattern);
     if (_blink_type != WS_FORE &&
         _bl_fl == True &&
@@ -185,51 +181,52 @@ long WSCvarc::draw(){
   if (line_w != 0) {
     if (_angle2 != (short)360) {
       // ξ...
-      double dw = (double)w;
-      double dh = (double)h;
-      double dx = dw / 2.0;
-      double dy = dh / 2.0;
-      WSCpoint pt[3];
-      WSCpoint pt1, pt2, pt3;
-      double pai = 3.141592653;
 
-      pt1.x = (short)(dx);
-      pt1.y = (short)(dy);
-      pt2.x = (short)(dx + dw/2.0 * cos((double)_angle1/180.0*pai));
-      pt2.y = (short)(dy - dh/2.0 * sin((double)_angle1/180.0*pai));
-      pt3.x = (short)(dx + dw/2.0 * cos((double)(_angle1+_angle2)/180.0*pai));
-      pt3.y = (short)(dy - dh/2.0 * sin((double)(_angle1+_angle2)/180.0*pai));
       switch (_arc_type) {
-      case WS_ARC_OPEN:		// ߸
-        dev->drawArc(xx, yy, ww, hh, _angle1*64, _angle2*64);
-        break;
-      case WS_ARC_CLOSE:	// ı߸
-        pt[0].x = pt2.x;
-        pt[0].y = pt2.y;
-        pt[1].x = pt3.x;
-        pt[1].y = pt3.y;
-        dev->drawLines(pt, 2);
+        case WS_ARC_OPEN:		// ߸...
+          dev->drawArc(xx, yy, ww, hh, _angle1*64, _angle2*64);
+          break;
+        case WS_ARC_CLOSE:{	// ı߸...
+          double dw = (double)w;
+          double dh = (double)h;
+          double dx = dw / 2.0;
+          double dy = dh / 2.0;
+          double pai = 3.141592653;
+          WSCpoint pt[2];
+          pt[0].x = (short)(dx + dw/2.0 * cos((double)_angle1/180.0*pai));
+          pt[0].y = (short)(dy - dh/2.0 * sin((double)_angle1/180.0*pai));
+          pt[1].x = (short)(dx + dw/2.0 * cos((double)(_angle1+_angle2)/180.0*pai));
+          pt[1].y = (short)(dy - dh/2.0 * sin((double)(_angle1+_angle2)/180.0*pai));
+          dev->drawLines(pt, 2);
 //WSMFtrace("*** WSCvarc::draw drawLines [%d, %d] [%d, %d]\n", pt[0].x, pt[0].y, pt[1].x, pt[1].y);
-        dev->drawArc(xx, yy, ww, hh, _angle1*64, _angle2*64);
-        break;
-      case WS_ARC_PIE:	// 
-        pt[1].x = pt1.x;
-        pt[1].y = pt1.y;
-        pt[0].x = pt2.x;
-        pt[0].y = pt2.y;
-        pt[2].x = pt3.x;
-        pt[2].y = pt3.y;
-        dev->drawLines(pt, 3);
+          dev->drawArc(xx, yy, ww, hh, _angle1*64, _angle2*64);
+          break;
+        }
+        case WS_ARC_PIE:{	// ...
+          double dw = (double)w;
+          double dh = (double)h;
+          double dx = dw / 2.0;
+          double dy = dh / 2.0;
+          double pai = 3.141592653;
+          WSCpoint pt[3];
+          pt[0].x = (short)(dx + dw/2.0 * cos((double)_angle1/180.0*pai));
+          pt[0].y = (short)(dy - dh/2.0 * sin((double)_angle1/180.0*pai));
+          pt[1].x = (short)(dx);
+          pt[1].y = (short)(dy);
+          pt[2].x = (short)(dx + dw/2.0 * cos((double)(_angle1+_angle2)/180.0*pai));
+          pt[2].y = (short)(dy - dh/2.0 * sin((double)(_angle1+_angle2)/180.0*pai));
+          dev->drawLines(pt, 3);
 //WSMFtrace("*** WSCvarc::draw drawLines [%d, %d] [%d, %d] [%d, %d]\n", pt[0].x, pt[0].y, pt[1].x, pt[1].y, pt[2].x, pt[2].y);
-        dev->drawArc(xx, yy, ww, hh, _angle1*64, _angle2*64);
-        break;
+          dev->drawArc(xx, yy, ww, hh, _angle1*64, _angle2*64);
+          break;
+        }
       }
     }else{
       dev->drawArc(xx, yy, ww, hh, _angle1*64, _angle2*64);
     }
   }
 
-  //轪λ
+  //轪λ...
   dev->endDraw();
 
   return WS_NO_ERR;
