@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCvfbtn.h>
@@ -43,7 +39,7 @@ WSMFversion(WSCvfbtn, WSCvbtn);
 WSCvfbtn::WSCvfbtn(WSCbase* base, char* objname):
 			WSCvbtn(base, objname) {
 
-  _mouse_in = 0;
+//  _mouse_in = 0;
 
   WSMFpropertyCreateStart
     WSMFparentCheckVerSrc(WSCvfbtn);
@@ -58,14 +54,14 @@ WSCvfbtn::~WSCvfbtn() {
 }
 
 void WSCvfbtn::onMouseIn(WSCpoint* pt){
-  _mouse_in = 1;
+//  _mouse_in = 1;
   WSCvbtn::onMouseIn(pt);
 //printf("WSCvfbtn::onMouseIn...\n");
   redraw();
 }
 
 void WSCvfbtn::onMouseOut() {
-  _mouse_in = 0;
+//  _mouse_in = 0;
   WSCvbtn::onMouseOut();
   redraw();
 }
@@ -93,7 +89,7 @@ void WSCvfbtn::onFocusChange(WSCbool fl){
     return;
   }
   if (fl != False){
-    WSGIappKeyboard()->setFocusInputWorkProc(_key_handler, NULL, this);
+    WSGIappKeyboard()->setFocusInputWorkProc(_key_handler, NULL, this, this);
     if (getowndev() != NULL) {
       WSGIappKeyboard()->resetFep(getowndev());
     }
@@ -112,7 +108,7 @@ void WSCvfbtn::_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool fl) {
     return;
   }
 
-  if (fl == True && key == WSK_Return) {
+  if (fl == True && (key == WSK_Return || key == WSK_KP_Enter)) {
     if (obj->_press_state == False) {
       obj->_press_state = True;
       obj->_mouse_in = 1;
@@ -120,7 +116,7 @@ void WSCvfbtn::_key_handler(void* ptr, WSDkeyboard* keyboard, WSCbool fl) {
       obj->redraw();
     }
   } else
-  if (fl == False && key == WSK_Return) {
+  if (fl == False && (key == WSK_Return || key == WSK_KP_Enter)) {
     if (obj->_press_state == True) {
       obj->_press_state = False;
       obj->_mouse_in = 0;
