@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSkeysym.h>
@@ -38,6 +34,9 @@
 #include <WSDmouse.h>
 #include <WSDappDev.h>
 #include <WSCbaseList.h>
+#ifdef MSW
+#include <WSClocaleSet.h>
+#endif
 
 WSMFguiClassInitialize(WSCvifield,WSCvlabel);
 WSMFversion(WSCvifield,WSCvlabel);
@@ -51,14 +50,14 @@ WSCvifield::WSCvifield(WSCbase* base,char* objname):
   _register_focus_move();
   _cbase = 0;
   _cursor_fore = 1;
-  _upname = WSGFstrdup("");
-  _dwname = WSGFstrdup("");
-  _lname = WSGFstrdup("");
-  _rname = WSGFstrdup("");
-  _rtname = WSGFstrdup("");
-  _tbname = WSGFstrdup("");
-  _btbname = WSGFstrdup("");
-  _data_source_name = WSGFstrdup("");
+  _upname = NULL;
+  _dwname = NULL;
+  _lname = NULL;
+  _rname = NULL;
+  _rtname = NULL;
+  _tbname = NULL;
+  _btbname = NULL;
+  _data_source_name = NULL;
   _data_source_type = WS_DATA_SOURCE_NONE;
   _enable_focus_move = False;
   _cursor_adjust = True;
@@ -67,7 +66,7 @@ WSCvifield::WSCvifield(WSCbase* base,char* objname):
 //  _columns = 0;
   _cur_pos = 0;
   _cur_pos_bk = -1;
-  _kanji_in = True;
+  _enable_fep = WS_FEP_ENABLE;
   _icursor = True;
   _fspace = False;
   _mouse_pressed = False;
@@ -90,16 +89,27 @@ WSCvifield::WSCvifield(WSCbase* base,char* objname):
   _fep_set_focus = 0;
   _display_only = 0;
   _need_ds_update = False;
+  _text_changed = True;
 
   
   WSMFpropertyCreateStart
     WSMFparentCheckVerSrc(WSCvifield);
-
 //    WSMFpropertyCreate(WSNcolumns,         WSCushort, _columns,WSScolumns );
     WSMFpropertyCreate(WSNcursorPos,  short, _cur_pos, WSScursorPos );
-    WSMFpropertyCreate(WSNkanjiIn,  WSCbool, _kanji_in, WSSkanjiIn );
+    WSMFpropertyCreate(WSNenableFep,  WSCuchar, _enable_fep, WSSenableFep );
+#ifdef MSW
+    char* locale_name = WSGIappLocaleSet()->getSystemLocaleName();
+    if(!strcmp(locale_name,"ja_JP.eucjp") ||
+       !strcmp(locale_name,"ja_JP.sjis") ||
+       !strcmp(locale_name,"ja_JP.utf8")){
+      WSMFpropertySetSelection(WSRfepStatus,WSRfepStatusD);
+    }else{
       WSMFpropertySetSelection(WSRbool1,WSRbool1D);
-      WSMFpropertyVisible(False);
+    }
+#else
+    WSMFpropertySetSelection(WSRbool1,WSRbool1D);
+#endif
+//      WSMFpropertyVisible(False);
     WSMFpropertyCreate(WSNinterCur, WSCbool, _icursor,WSSinterCur);
       WSMFpropertySetSelection(WSRbool1,WSRbool1D);
     WSMFpropertyCreate(WSNupward, char*, _upname,WSSupward);
@@ -129,11 +139,11 @@ WSCvifield::WSCvifield(WSCbase* base,char* objname):
       WSMFpropertySetSelection(WSRbool3,WSRbool3D);
 
 
-
     WSMFpropertyDelete(WSNemboss);
     WSMFpropertyDelete(WSNorientation);
     WSMFpropertyDelete(WSNalignmentH);
     WSMFpropertyDelete(WSNalignmentV);
+    WSMFpropertyDelete(WSNautoLineFeeding);
 
     WSMFpropertyValueChangeDef(WSCvifield,WSNbackColor,short);
 
@@ -144,22 +154,23 @@ WSCvifield::WSCvifield(WSCbase* base,char* objname):
     WSMFaddTrigger(WSEV_KEY_HOOK    );
     WSMFaddTrigger(WSEV_FOCUS_CH    );
     WSMFaddTrigger(WSEV_VALUE_CH    );
+
   WSMFpropertyCreateEnd
 
 }
 
 //WSMFproperty( WSCvifield, WSNcolumns,   WSCushort, _columns,   0  );
 WSMFproperty( WSCvifield, WSNcursorPos,  short,          _cur_pos,0      );
-WSMFproperty( WSCvifield, WSNkanjiIn,  WSCbool,    _kanji_in,  True   );
+WSMFproperty( WSCvifield, WSNenableFep,  WSCbool,    _enable_fep,  WS_FEP_ENABLE   );
 WSMFproperty( WSCvifield, WSNinterCur, WSCbool,    _icursor, True  );
 WSMFproperty( WSCvifield, WSNenableFocusMove, WSCbool,    _enable_focus_move, False );
-WSMFproperty( WSCvifield, WSNupward,    char*,   _upname, WSGFstrdup(""));
-WSMFproperty( WSCvifield, WSNdownward, char*,   _dwname, WSGFstrdup(""));
-WSMFproperty( WSCvifield, WSNleftward,   char*,   _lname, WSGFstrdup(""));
-WSMFproperty( WSCvifield, WSNrightward,  char*,   _rname, WSGFstrdup(""));
-WSMFproperty( WSCvifield, WSNreturn, char*,   _rtname, WSGFstrdup(""));
-WSMFproperty( WSCvifield, WSNtab, char*,   _tbname, WSGFstrdup(""));
-WSMFproperty( WSCvifield, WSNbacktab, char*,   _btbname, WSGFstrdup(""));
+WSMFproperty( WSCvifield, WSNupward,    char*,   _upname, NULL);
+WSMFproperty( WSCvifield, WSNdownward, char*,   _dwname, NULL);
+WSMFproperty( WSCvifield, WSNleftward,   char*,   _lname, NULL);
+WSMFproperty( WSCvifield, WSNrightward,  char*,   _rname, NULL);
+WSMFproperty( WSCvifield, WSNreturn, char*,   _rtname, NULL);
+WSMFproperty( WSCvifield, WSNtab, char*,   _tbname, NULL);
+WSMFproperty( WSCvifield, WSNbacktab, char*,   _btbname, NULL);
 WSMFproperty( WSCvifield, WSNfillSpace,   WSCbool,    _fspace,    False );
 WSMFproperty( WSCvifield, WSNcursorAdjust,WSCbool,    _cursor_adjust, True );
 //WSMFproperty( WSCvifield, WSNmultiLine,  WSCbool,    _multi_lines,   False );
@@ -167,7 +178,7 @@ WSMFproperty( WSCvifield, WSNifieldSkipMode, WSCbool,  _skip_mode,     False );
 WSMFproperty( WSCvifield, WSNnoHatch, WSCbool,  _no_hatch,     False );
 WSMFproperty( WSCvifield, WSNreturnKeyFocus, WSCbool, _return_key_focus, False);
 WSMFproperty( WSCvifield, WSNdataSource,  WSCuchar, _data_source_type,WS_DATA_SOURCE_NONE);
-WSMFproperty( WSCvifield, WSNdataSourceName,  char*, _data_source_name,WSGFstrdup(""));
+WSMFproperty( WSCvifield, WSNdataSourceName,  char*, _data_source_name,NULL);
 
 WSMFproperty( WSCvifield, WSNdisplayOnly, WSCbool,  _display_only,     False );
 
@@ -186,20 +197,23 @@ void WSCvifield::getWorkWSNtab(char**){}
 void WSCvifield::setWorkWSNbacktab(char*){}
 void WSCvifield::getWorkWSNbacktab(char**){}
 void WSCvifield::setWorkWSNlabelString(char*){
-  
-  if (_label_string[0] == 0){
+  if (_label_string == NULL || _label_string[0] == 0){
     if (_fspace != False){
       long len = _max_length;
       if (_enable_max != False){
-        delete _label_string;
+        if (_label_string != NULL){
+          delete[] _label_string;
+        }
         _label_string = new char[len+1];
         memset(_label_string,' ',len);
         _label_string[len]=0;
       }
     }
-    WSCushort* str = WSGFgetUCS2(_label_string,WS_EN_DEFAULT);
-    _set_text(str);
+//    WSCushort* str = WSGFgetUCS2(_label_string,WS_EN_DEFAULT);
+    WSCushort* str = new WSCushort[1];
+    str[0] = 0;
     setCurPos(0);
+    _set_text(str);
     return;
   }
 
@@ -207,9 +221,9 @@ void WSCvifield::setWorkWSNlabelString(char*){
   if (_enable_max != False && WSGFstrlenUCS2(ustr) > _max_length-1){
     ustr[_max_length] = 0;
   }
+  setCurPos(0);
   _set_text( ustr );
   _fill_space();
-  setCurPos(0);
 }
 void WSCvifield::getWorkWSNlabelString(char** data){
   if (_label_string == NULL){
@@ -217,15 +231,17 @@ void WSCvifield::getWorkWSNlabelString(char** data){
       _label_string = WSGFgetString(_ucs2,WS_EN_DEFAULT);
       *data = _label_string;
     }else{
-      *data = "";
+      *data = NULL;
     }
   }else{
     *data = _label_string;
   }
+#ifndef WS_EMBED
   if (getPropertyEditMode() != False &&
       _data_source_type != WS_DATA_SOURCE_NONE){
-    *data = "";
+    *data = NULL;
   }
+#endif
 }
 
 //void WSCvifield::setWorkWSNcolumns(WSCushort){}
@@ -257,7 +273,7 @@ void WSCvifield::setWorkWSNmaxLength(WSCushort data){
   }
 }
 
-void WSCvifield::setWorkWSNkanjiIn(WSCbool data){
+void WSCvifield::setWorkWSNenableFep(WSCbool data){
   if (_initialized == False){
     return;
   }
@@ -266,7 +282,7 @@ void WSCvifield::setWorkWSNkanjiIn(WSCbool data){
     return;
   }
   if (getFocus() != False){
-      WSGIappKeyboard()->setFocusInputWorkProc(_key_handler,NULL,this);
+      WSGIappKeyboard()->setFocusInputWorkProc(_key_handler,NULL,this,this);
       if (data == False){
         WSGIappKeyboard()->setEnableFep(dev,False);
       }else{
@@ -275,7 +291,7 @@ void WSCvifield::setWorkWSNkanjiIn(WSCbool data){
   }
 }
 
-void WSCvifield::getWorkWSNkanjiIn(WSCbool*){}
+void WSCvifield::getWorkWSNenableFep(WSCbool*){}
 void WSCvifield::setWorkWSNinterCur(WSCbool fl){
   if (getInitialized() != False && fl != False){
     WSCstring str(getProperty(WSNlabelString));
@@ -325,7 +341,7 @@ void WSCvifield::setWorkWSNdataSource(WSCuchar){
       WSCstring* str = WSGFreadTextFile(_data_source_name);
       if (str != NULL){
         setProperty(WSNlabelString,str->getString());
-        delete str;
+        delete[] str;
       }
     }else
     if (_data_source_type == WS_DATA_SOURCE_INSTANCE){
@@ -357,10 +373,14 @@ void WSCvifield::getWorkWSNdataSourceName(char**){}
 
 
 WSCvifield::~WSCvifield(){
-  if (getFocus() != False){
+  if (getFocus() != False || WSGIappKeyboard()->getFocusInstance() == this){
     WSGIappTimer()->delTimerProc( _cur_timer_id );
     WSGIappKeyboard()->resetFocusInputWorkProc();
   }
+  if (WSGIappKeyboard()->getSpecialFocusInstance() == this){
+    WSGIappKeyboard()->resetSpecialInputWorkProc();
+  } 
+
   if (this == WSGIappDev()->getSelectionOwner()){
     _deleting = True;
     WSGIappDev()->setSelectionOwner(NULL);
@@ -372,41 +392,42 @@ WSCvifield::~WSCvifield(){
   }
  
   setFocus(False);
+  setSpecialFocus(False);
   if (_upname != NULL){
-    delete _upname;
+    delete[] _upname;
     _upname = NULL;
   }
   if (_dwname != NULL){
-    delete _dwname;
+    delete[] _dwname;
     _dwname = NULL;
   }
   if (_lname != NULL){
-    delete _lname;
+    delete[] _lname;
     _lname = NULL;
   }
   if (_rname != NULL){
-    delete _rname;
+    delete[] _rname;
     _rname = NULL;
   }
   if (_rtname != NULL){
-    delete _rtname;
+    delete[] _rtname;
     _rtname = NULL;
   }
   if (_tbname != NULL){
-    delete _tbname;
+    delete[] _tbname;
     _tbname = NULL;
   }
   if (_btbname != NULL){
-    delete _btbname;
+    delete[] _btbname;
     _btbname = NULL;
   }
 
   if (_ucs2 != NULL){
-    delete _ucs2;
+    delete[] _ucs2;
     _ucs2 = NULL;
   }
   if (_data_source_name != NULL){
-    delete _data_source_name;
+    delete[] _data_source_name;
     _data_source_name = NULL;
   }
 }
@@ -414,7 +435,7 @@ long WSCvifield::_cur_timer_id = 0;
 
 void WSCvifield::_cursor_blink_handler(WSCuchar,void* ptr){
   WSCvifield* ifd = (WSCvifield*)ptr;
-  WSGIappKeyboard()->setFocusInputWorkProc(_key_handler,NULL,ifd);
+  WSGIappKeyboard()->setFocusInputWorkProc(_key_handler,NULL,ifd,ifd);
   ifd->setAbsoluteDraw(True);
   ifd->onCursorBlink();
 }
@@ -429,9 +450,11 @@ void WSCvifield::onCursorBlink(){
   }else{
     redraw();
   }
+#ifndef WS_EMBED
   if (_show_grip_hand != False){
     drawGriphand();
   }
+#endif
   return;
 }
 
@@ -477,7 +500,8 @@ void WSCvifield::_fill_space(){
   long i;
   for(i=len2; i<len; i++){
     tmp[i] = (WSCushort)' ';
-  } 
+  }
+  tmp[len] = 0;
   _set_text(tmp);
   return;
 }
@@ -514,7 +538,7 @@ dbprintf("WSCvifield::draw %s:%d  %s dev=NULL\n",__FILE__,__LINE__,getInstanceNa
     if (getFocus() != False){
       shadow_type = WS_SHADOW_IN;
     }else if (_use_pix != False){
-      //USE_PIXMAP = True ˤƤΤ False ˤ롣
+      //USE_PIXMAP = True ˤƤΤ False ˤ롣...
       char fl = False;
       dev->setValue(WSDEV_USE_PIXMAP,&fl);
     }
@@ -595,11 +619,15 @@ dbprintf("WSCvifield::draw %s:%d %s device is not ready\n",__FILE__,__LINE__,get
         dev->setForeColor(bgno);
       }
     }
+#ifndef NO_GRADATION
     if (_grad_no == 0){
+#endif
       dev->drawFillRect(0, 0, w, h);
+#ifndef NO_GRADATION
     }else{
       dev->drawGradation(_grad_no,tsno,bgno,bsno,0, 0, w, h,_grad_margin);
     }
+#endif
   }
 
   short ix = st+ _margin_left;
@@ -680,9 +708,12 @@ dbprintf("WSCvifield::draw %s:%d %s draw shadow..\n",__FILE__,__LINE__,getInstan
   if (_multi_lines != False){
     alignment = WS_LEFT_TOP;
   }
-
+#ifndef WS_EMBED
   if (getSensitive() == False ||
-      (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0)){
+      (getPropertyEditMode() != False && _sensitive_work == False)){
+#else
+  if (getSensitive() == False){
+#endif
     if (_no_hatch == False){
       dev->setHatchPattern(8);
     }
@@ -792,11 +823,6 @@ dbprintf("WSCvifield::draw %s:%d %s draw string..\n",__FILE__,__LINE__,getInstan
   printf("WSCvifield::draw here3 %d\n",WSGFclocktime());
 #endif
       }
-#if 0
-      if (_kanji_in != False){
-        _fep_cursor_adjust(WSGIappKeyboard());
-      }
-#endif
     }else{
       if (_multi_lines == False){
 //printf("WSCvifield::draw drawUString xy3=%d,%d\n",sx,sy);
@@ -849,16 +875,11 @@ dbprintf("WSCvifield::draw %s:%d %s draw string..\n",__FILE__,__LINE__,getInstan
 
       }
 dbprintf("WSCvifield::draw %s:%d %s draw string..\n",__FILE__,__LINE__,getInstanceName());
-#if 0
-      if (_kanji_in != False){
-        _fep_cursor_adjust(WSGIappKeyboard());
-      }
-#endif
     }
   }
 
   if (str != NULL){
-    delete str;
+    delete[] str;
   }
 
 dbprintf("WSCvifield::draw %s:%d  %s end..\n",__FILE__,__LINE__,getInstanceName());
@@ -1049,7 +1070,7 @@ WSCbool WSCvifield::_special_key_handler(void* ptr,WSDkeyboard* keyboard,WSCbool
   if (obj->getSensitive() == False){
     return True;
   }
-  if (keyboard->getKey() == WSK_Return){
+  if (keyboard->getKey() == WSK_Return || keyboard->getKey() == WSK_KP_Enter){
     obj->_on_key_input(keyboard,fl);
     if (fl != False){
       obj->onKey(keyboard,True);
@@ -1088,7 +1109,8 @@ void WSCvifield::onKeyInput(WSDkeyboard* keyb,WSCbool fl){
   _on_key_input(keyb,fl);
 }
 void WSCvifield::onKey(WSDkeyboard* keyb,WSCbool fl){
-  if( fl != False && keyb->getKey() == WSK_Return){
+  if( fl != False &&
+      (keyb->getKey() == WSK_Return || keyb->getKey() == WSK_KP_Enter)){
 //    execEventProc(WSEV_ACTIVATE,NULL);
     onActivate();
   }
@@ -1109,10 +1131,15 @@ long WSCvifield::_get_absolute_pos(long x,long y){
   char alignment =  WS_LEFT;
   long px = x;
   long py = y;
+  if (py < 0){
+    py = 0;
+  }
   if (_multi_lines != False){
     alignment = WS_LEFT_TOP;
     px += _mx;
     py += _my;
+  }else{
+    py = _shadow_thick + margin_top;
   }
   WSCrect area;
   area.setRect(_shadow_thick+ margin_left,
@@ -1159,6 +1186,9 @@ void WSCvifield::onMousePress(WSCpoint* point){
   WSCvlabel::onMousePress(point);
   if (getSensitive() != False){
     _mouse_pressed = True;
+    _out_side_mouse_release = True;
+    _out_side_mouse_move = True;
+
 #ifndef SUN
 #ifndef MSW
     if (WSGIappMouse()->getMouseStatus() & WS_MOUSE_BTN2){
@@ -1183,7 +1213,7 @@ void WSCvifield::onMousePress(WSCpoint* point){
       _set_cur_pos(absolute_pos);
       _reset_select_string();
     }
-    if (_kanji_in == False){
+    if (_enable_fep == WS_FEP_DISABLE){
       WSGIappKeyboard()->resetFep(getowndev());
     }else{
       _fep_cursor_adjust(WSGIappKeyboard());
@@ -1220,6 +1250,9 @@ void WSCvifield::onMouseMove(WSCpoint* point){
            }
         }
 #endif
+        if (absolute_pos == 0 && _cbase > 0){
+          absolute_pos--;
+        }
         _set_cur_pos(absolute_pos);
         _set_select_pos2(absolute_pos);
 
@@ -1238,6 +1271,8 @@ void WSCvifield::onMouseMove(WSCpoint* point){
 void WSCvifield::onMouseRelease(WSCpoint* point){
 //printf("WSCvifield::onMouseRelease ...\n");
   WSCvlabel::onMouseRelease(point);
+  _out_side_mouse_release = True;
+  _out_side_mouse_move = True;
   if (getSensitive() != False){
     _mouse_pressed = False;
 #if 0
@@ -1316,6 +1351,9 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
 //  WSCstring str( _label_string );
   if (_display_only != False){
     WSCbool reject = True;
+    if (key == WSK_Tab){
+      reject = False;
+    }
     if (WSGIappKeyboard()->isCursorKey() != False){
       reject = False;
     }
@@ -1335,7 +1373,7 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
   }
 
   _no_mcur_adjust = 0;
-  if (key == WSK_Return && _return_key_ok() == False){
+  if ((key == WSK_Return || key == WSK_KP_Enter) && _return_key_ok() == False){
      if (_enable_focus_move != False){
        WSCbase* target = getFocusMoveInstance(WS_RET);
        if (target != NULL && target != (WSCbase*)this && target->existProperty(WSNcursorPos) != False){
@@ -1345,13 +1383,13 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
          target->setFocus();
          target->draw();
        }
+       _focus_move_for_combo(WS_RET);
      }
      return;
   }
 
   WSCbool refresh;
   getPropertyV(WSNblinkRefreshing,&refresh);
-
   if ( keyboard->isCursorKey() != False ||
        key == WSK_Tab ||
        key == WSK_BackSpace ||
@@ -1413,7 +1451,8 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
         break;
       case WSK_Left:
         if (_cursor_move(WS_LEFT) != WS_NO_ERR){
-          if (_enable_focus_move != False){
+          if (keyboard->withShift() == False &&
+              _enable_focus_move != False){
             target = getFocusMoveInstance(WS_LEFT);
             if (target != NULL && target != (WSCbase*)this &&
               target->cast("WSCvifield") != NULL){
@@ -1423,6 +1462,7 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
               target->setFocus();
               target->draw();
             }
+            _focus_move_for_combo(WS_LEFT);
           }
         }else{
           if (keyboard->withShift() == False){
@@ -1447,7 +1487,8 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
         break;
       case WSK_Right:
         if (_cursor_move(WS_RIGHT) != WS_NO_ERR){
-          if (_enable_focus_move != False){
+          if (keyboard->withShift() == False &&
+              _enable_focus_move != False){
             target = getFocusMoveInstance(WS_RIGHT);
             if ( target != NULL && target != (WSCbase*)this &&
               target->cast("WSCvifield") != NULL){
@@ -1457,6 +1498,7 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
               target->setFocus();
               target->draw();
             }
+            _focus_move_for_combo(WS_RIGHT);
           }
         }else{
           if (keyboard->withShift() == False){
@@ -1491,6 +1533,7 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
               target->setFocus();
               target->draw();
             }
+            _focus_move_for_combo(WS_UP);
           }
         }else{
           if (keyboard->withShift() == False){
@@ -1525,6 +1568,7 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
               target->setFocus();
               target->draw();
             }
+            _focus_move_for_combo(WS_DOWN);
           }
         }else{
           if (keyboard->withShift() == False){
@@ -1568,21 +1612,24 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
             }
           }
         }
-        _select_pos1 = _cur_pos;
-        _select_pos2 = _cur_pos;
-        _fep_cursor_adjust(keyboard);
-        if (_multi_lines != False){
-          return;
-        }
-        {
+        if (keyboard->withShift() == False){
+          _select_pos1 = _cur_pos;
+          _select_pos2 = _cur_pos;
+        }else{
+          _set_select_pos2(_cur_pos - _cbase);
           WSCbool fl;
           getPropertyV(WSNblinkRefreshing,&fl);
+          setAbsoluteDraw(True);
           if (fl == False){
             draw();
           }else{
             redraw();
           }
         }
+        _fep_cursor_adjust(keyboard);
+        if (_multi_lines != False){
+          return;
+        }
         break;
       case WSK_End:
         if (keyboard->withCntl() != False){
@@ -1614,21 +1661,24 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
             }
           }
         }
-        _select_pos1 = _cur_pos;
-        _select_pos2 = _cur_pos;
-        _fep_cursor_adjust(keyboard);
-        if (_multi_lines != False){
-          return;
-        }
-        {
+        if (keyboard->withShift() == False){
+          _select_pos1 = _cur_pos;
+          _select_pos2 = _cur_pos;
+        }else{
+          _set_select_pos2(_cur_pos - _cbase);
           WSCbool fl;
           getPropertyV(WSNblinkRefreshing,&fl);
+          setAbsoluteDraw(True);
           if (fl == False){
             draw();
           }else{
             redraw();
           }
         }
+        _fep_cursor_adjust(keyboard);
+        if (_multi_lines != False){
+          return;
+        }
         break;
       case WSK_Insert:
         if ( (keyboard->withCntl() != False) ||
@@ -1658,6 +1708,20 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
               target->setFocus();
               target->draw();
             }
+            _focus_move_for_combo(WS_STAB);
+          }else if (_multi_lines != False){
+            WSCstring val = getProperty(WSNbacktab);
+            if (strcmp(val,"")){
+              target = getFocusMoveInstance(WS_STAB);
+              if (target != NULL && target != (WSCbase*)this &&
+                target->cast("WSCvifield") != NULL){
+                target->setPropertyV(WSNcursorPos,(short)0);
+              }
+              if (target != NULL ){
+                target->setFocus();
+                target->draw();
+              }
+            }
           }
         }else{
           if (_enable_focus_move != False){
@@ -1670,18 +1734,34 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
               target->setFocus();
               target->draw();
             }
+            _focus_move_for_combo(WS_TAB);
+          }else if (_multi_lines != False){
+            WSCstring val = getProperty(WSNtab);
+            if (strcmp(val,"")){
+              target = getFocusMoveInstance(WS_TAB);
+              if (target != NULL && target != (WSCbase*)this &&
+                target->cast("WSCvifield") != NULL){
+                target->setPropertyV(WSNcursorPos,(short)0);
+              }
+              if (target != NULL ){
+                target->setFocus();
+                target->draw();
+              }
+            }
           }
         }
         break;
     }
   }else{
     WSCstring* istr1 = keyboard->getTextString();
-//printf("keyboard->getTextString()=%s\n",istr1->getString());
     WSCushort* istr = WSGFgetUCS2(istr1->getString(WS_EN_UTF8),WS_EN_UTF8);
     WSCushort tmp[2];
     tmp[0] = '\n';
     tmp[1] = 0;
-    if (key == WSK_Return ){
+    if (key == WSK_Return || key == WSK_KP_Enter){
+      if (istr != NULL){
+        delete[] istr;
+      }
       istr = WSGFstrdupUCS2(tmp);
     }else{
       WSGFdelLineFeedUCS2(istr);
@@ -1702,8 +1782,8 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
         WSGIappDev()->setSelectionOwner(this);
         char* sstr2 = WSGFgetString(sstr,WS_EN_UTF8); 
         WSGIappKeyboard()->setSelectedString(sstr2,WS_EN_UTF8);
-        delete sstr;
-        delete sstr2;
+        delete[] sstr;
+        delete[] sstr2;
       }
     }else
     if (((key == WSK_V || key == WSK_v) && keyboard->withCntl() != False) || 
@@ -1715,7 +1795,7 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
       _replace_string(istr2);
       _fep_cursor_adjust(keyboard);
       _update_text();
-      delete istr2;
+      delete[] istr2;
     }else{
       if (_key_input_disable == False && istr[0] != 0){
         _replace_string(istr);
@@ -1723,7 +1803,7 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
         _update_text();
       }
     }
-    delete istr; 
+    delete[] istr; 
   }
 
   setCursorFore(True);
@@ -1740,11 +1820,15 @@ void WSCvifield::_on_key_input(WSDkeyboard* keyboard,WSCbool fl){
 void WSCvifield::replaceSelectedString(char* istr,long encode){
   WSCushort* istr2 = WSGFgetUCS2(istr,encode);
   _replace_string(istr2); 
-  delete istr2;
+  delete[] istr2;
+  setAbsoluteDraw(True);
+  needUpdate();
 }
 void WSCvifield::deleteSelectedString(){
   _delete_select_string();
   _update_text();
+  setAbsoluteDraw(True);
+  needUpdate();
 }
 WSCstring WSCvifield::getSelectedString(){
   WSCstring tmp;
@@ -1762,31 +1846,31 @@ WSCstring WSCvifield::getSelectedString(){
   WSGFdelStringUCS2(str,0,pos1);
   char* str2 = WSGFgetString(str,WS_EN_UTF8);
   tmp.setString(str2,WS_EN_UTF8);
-  delete str;
-  delete str2;
+  delete[] str;
+  delete[] str2;
   return tmp;
 }
 void WSCvifield::_replace_string(WSCushort* istr){
   if (_ucs2 == NULL){
     _ucs2 = WSGFstrdupUCS2(istr);
     _text_changed = True;
-    _update_text();
     _cur_pos = WSGFstrlenUCS2(istr);
+    _update_text();
   }else
   if (_ucs2[0] == 0){
-    delete _ucs2;
+    delete[] _ucs2;
     _ucs2 = WSGFstrdupUCS2(istr);
     _text_changed = True;
-    _update_text();
     _cur_pos = WSGFstrlenUCS2(istr);
+    _update_text();
   }else{
     _delete_select_string();
     if (_icursor == False){
       WSGFdelStringUCS2(_ucs2,_cur_pos,1);
     }
     WSCushort* newstr = WSGFinsertStringUCS2(_ucs2,_cur_pos,istr);
-    _set_text(newstr);
     _cur_pos += WSGFstrlenUCS2(istr);
+    _set_text(newstr);
   }
   if (_enable_max != False && _cur_pos >= _max_length - !_icursor){
     _cur_pos = _max_length - !_icursor;
@@ -1802,6 +1886,7 @@ void WSCvifield::_replace_string(WSCushort* istr){
             target->setFocus();
             target->draw();
           }
+          _focus_move_for_combo(WS_RIGHT);
         }
       }
     }
@@ -1865,6 +1950,9 @@ void WSCvifield::_adjust_cbase(){
       WSDfont* font = WSGIappFontSet()->getFont(_font);
       WSCushort* tmp = WSGFstrdupUCS2(_ucs2);
       long len = WSGFstrlenUCS2(_ucs2);
+      if (_cur_pos > len){
+        _cur_pos = len;
+      }
       tmp[_cur_pos] = 0;
       while(1){
         long strw = font->getStringWidthUCS2(&tmp[_cbase]);
@@ -1878,7 +1966,7 @@ void WSCvifield::_adjust_cbase(){
           break;
         }
       }
-      delete tmp;
+      delete[] tmp;
     }
   }
 }
@@ -1939,15 +2027,22 @@ void WSCvifield::onFocusChange(WSCbool fl){
     _dev->setValue(WSDEV_USE_PIXMAP,&fl);
   }
   if (fl != False){
-    WSGIappKeyboard()->setFocusInputWorkProc(_key_handler,NULL,this);
+    WSGIappKeyboard()->setFocusInputWorkProc(_key_handler,NULL,this,this);
     if (_return_key_focus != False){
-      WSGIappKeyboard()->setSpecialInputWorkProc(_special_key_handler,this);
+      WSGIappKeyboard()->setSpecialInputWorkProc(_special_key_handler,this,this);
       setSpecialFocus(True);
     }
-    if (_kanji_in == False){
+    if (_enable_fep == WS_FEP_DISABLE){
       WSGIappKeyboard()->setEnableFep(getowndev(),False);
     }else{
       WSGIappKeyboard()->setEnableFep(getowndev(),True);
+#ifdef MSW
+      if (_enable_fep == WS_FEP_KANJI_ON_FOCUS ||
+          _enable_fep == WS_FEP_KATAKANA_ON_FOCUS ||
+          _enable_fep == WS_FEP_HKATAKANA_ON_FOCUS ){
+        _fep_set_focus = True;
+      }
+#endif
     }
     if (_cur_timer_id != 0){
 WSMFtrace("WSCvifiled::onFocusChange(%d)   _cur_timer_id ERROR!!!!\n",fl);
@@ -1957,22 +2052,37 @@ WSMFtrace("WSCvifiled::onFocusChange(%d)   _cur_timer_id ERROR!!!!\n",fl);
     }
     _fep_cursor_adjust(WSGIappKeyboard());
 
-    if (_kanji_in != False && _fep_set_focus != False){
-      _fep_set_focus = False;
-      long ox,oy;
-      if (_multi_lines == False){
-        _get_disp_cur_pos(&ox,&oy);
-        WSGIappKeyboard()->startKanji(getowndev(),ox + _x,oy + _y,_font,_fore_color,
-                          _back_color);
-      }else{
-        _get_disp_cur_pos(&ox,&oy);
-        WSGIappKeyboard()->startKanji(getowndev(),_shadow_thick + ox + _x - _mx,
+    if (_enable_fep != WS_FEP_DISABLE && _fep_set_focus != False){
+      WSDdev* dev = getowndev();
+      if (dev != NULL){
+        _fep_set_focus = False;
+        long ox,oy;
+        if (_multi_lines == False){
+          _get_disp_cur_pos(&ox,&oy);
+          WSGIappKeyboard()->startFep(getowndev(),ox + _x,oy + _y,_font,_fore_color,
+                            _back_color);
+        }else{
+          _get_disp_cur_pos(&ox,&oy);
+          WSGIappKeyboard()->startFep(getowndev(),_shadow_thick + ox + _x - _mx,
                           _shadow_thick + oy + _y - _my,_font,
                           _fore_color,_back_color);
+        }
+#ifdef MSW
+        if (_enable_fep == WS_FEP_KANJI_ON_FOCUS){
+          WSGIappKeyboard()->setFepStatus(WS_FEP_START);
+        }else
+        if (_enable_fep == WS_FEP_KATAKANA_ON_FOCUS){
+          WSGIappKeyboard()->setFepStatus(WS_FEP_KATAKANA);
+        }else
+        if (_enable_fep == WS_FEP_HKATAKANA_ON_FOCUS){
+          WSGIappKeyboard()->setFepStatus(WS_FEP_HKATAKANA);
+        }
+#endif
       }
     }
 
   }else{
+    WSGIappKeyboard()->setEnableFep(getowndev(),False);
     WSGIappKeyboard()->resetFocusInputWorkProc();
     WSGIappTimer()->delTimerProc( _cur_timer_id );
     _cur_timer_id = 0;
@@ -1993,7 +2103,7 @@ void WSCvifield::onSpecialFocusChange(WSCbool fl){
   if (fl == False){
     WSGIappKeyboard()->resetSpecialInputWorkProc();
   }else{
-    WSGIappKeyboard()->setSpecialInputWorkProc(_special_key_handler,this);
+    WSGIappKeyboard()->setSpecialInputWorkProc(_special_key_handler,this,this);
   }
   WSCvlabel::onSpecialFocusChange(fl);
 }
@@ -2033,20 +2143,21 @@ WSCbool WSCvifield::setCurPos(long pos){
 }
 void WSCvifield::_set_text(WSCushort* str){
   if (_ucs2 != NULL){
-    delete _ucs2;
+    delete[] _ucs2;
     _ucs2 = NULL;
   }
   if (str == NULL){
     _ucs2 = new WSCushort[1];
     _ucs2[0] = 0;
+  }else{
+    _ucs2 = str;
   }
-  _ucs2 = str;
   _text_changed = True;
   _update_text();
 }
 void WSCvifield::_update_text(){
   if (_label_string != NULL){
-    delete _label_string;
+    delete[] _label_string;
     _label_string = NULL;
   }
   if (_ucs2 == NULL){
@@ -2066,7 +2177,7 @@ void WSCvifield::_update_text(){
     dest[2] = 0;
     long num = 0;
     WSCushort* newstr = WSGFreplaceStringUCS2(_ucs2,src,dest,&num);
-    delete _ucs2;
+    delete[] _ucs2;
     _ucs2 = newstr;
   }else{
 #if 0
@@ -2091,7 +2202,7 @@ void WSCvifield::_update_text(){
     onValueChange();
   }
   _text_changed = False;
-  delete bk;
+  delete[] bk;
   return;
 }
 void WSCvifield::onValueChange(){
@@ -2099,7 +2210,7 @@ void WSCvifield::onValueChange(){
     char* ptr = WSGFgetString(_ucs2,WS_EN_DEFAULT);
     WSCvariant val;
     val = ptr;
-    delete ptr;
+    delete[] ptr;
     setDataToDataSourceClients(&val);
   }
   execProcedure(WSEV_VALUE_CH);
@@ -2115,7 +2226,7 @@ void WSCvifield::setBuf(WSCushort* buf){
 void WSCvifield::addString(char* str,long encode){
   WSCushort* str2 = WSGFgetUCS2(str,encode);
   WSCushort* newstr = WSGFaddStringUCS2(_ucs2,str2);
-  delete str2;
+  delete[] str2;
   _set_text(newstr);
 }
 long WSCvifield::setSelect(long pos,long len){
@@ -2200,7 +2311,7 @@ WSCstring WSCvifield::getString(){
   }
   char* str = WSGFgetString(_ucs2,WS_EN_UTF8);
   ret.setString(str,WS_EN_UTF8);
-  delete str;
+  delete[] str;
   return ret;
 }
 void WSCvifield::setData(WSCvariant* val,long code){
@@ -2219,7 +2330,7 @@ char* WSCvifield::getDataSourcePropertyName(){
   return WSNlabelString;
 }
 long WSCvifield::setFepStatus(WSCbool fl){
-  if (_kanji_in == False){
+  if (_enable_fep == WS_FEP_DISABLE){
     return WS_ERR;
   }
   if (getowndev() == NULL){
@@ -2235,14 +2346,28 @@ long WSCvifield::setFepStatus(WSCbool fl){
     long ox,oy;
     if (_multi_lines == False){
       _get_disp_cur_pos(&ox,&oy);
-      WSGIappKeyboard()->startKanji(getowndev(),ox + _x,oy + _y,_font,_fore_color,
+      WSGIappKeyboard()->startFep(getowndev(),ox + _x,oy + _y,_font,_fore_color,
                           _back_color);
     }else{
       _get_disp_cur_pos(&ox,&oy);
-      WSGIappKeyboard()->startKanji(getowndev(),_shadow_thick + ox + _x - _mx,
+      WSGIappKeyboard()->startFep(getowndev(),_shadow_thick + ox + _x - _mx,
                           _shadow_thick + oy + _y - _my,_font,
                           _fore_color,_back_color);
     }
+#ifdef MSW
+    if (_enable_fep == WS_FEP_KANJI_ON_FOCUS){
+      WSGIappKeyboard()->setFepStatus(WS_FEP_STOP);
+      WSGIappKeyboard()->setFepStatus(WS_FEP_START);
+    }else
+    if (_enable_fep == WS_FEP_KATAKANA_ON_FOCUS){
+      WSGIappKeyboard()->setFepStatus(WS_FEP_STOP);
+      WSGIappKeyboard()->setFepStatus(WS_FEP_KATAKANA);
+    }else
+    if (_enable_fep == WS_FEP_HKATAKANA_ON_FOCUS){
+      WSGIappKeyboard()->setFepStatus(WS_FEP_STOP);
+      WSGIappKeyboard()->setFepStatus(WS_FEP_HKATAKANA);
+    }
+#endif
   }
   return WS_NO_ERR;
 }
@@ -2254,22 +2379,30 @@ void WSCvifield::update(){
   WSCvlabel::update();
 }
 void WSCvifield::onVisibleChange(WSCbool fl){
+  WSCvlabel::onVisibleChange(fl);
   if (fl == False){
     if (getFocus() != False){
       setFocus(False);
     }
+  }else{
+    if (getFocus() != False){
+      _fep_cursor_adjust(WSGIappKeyboard());
+    }
   }
-  WSCvlabel::onVisibleChange(fl);
 }
 void WSCvifield::onParentVisibleChange(WSCbool fl){
+  WSCvlabel::onParentVisibleChange(fl);
 //printf("WSCvifield::onParentVisibleChange %d\n",fl);
   if (fl == False){
     if (getFocus() != False){
 //printf("WSCvifield::onParentVisibleChange setFocus False\n");
       setFocus(False);
     }
+  }else{
+    if (getFocus() != False){
+      _fep_cursor_adjust(WSGIappKeyboard());
+    }
   }
-  WSCvlabel::onParentVisibleChange(fl);
 }
 void WSCvifield::onWindowFocusChange(WSCbool fl){
   if (fl == False && _use_pix == False && _dev != NULL){
@@ -2277,14 +2410,21 @@ void WSCvifield::onWindowFocusChange(WSCbool fl){
     _dev->setValue(WSDEV_USE_PIXMAP,&fl);
   }
   if (fl != False){
-    WSGIappKeyboard()->setFocusInputWorkProc(_key_handler,NULL,this);
+    WSGIappKeyboard()->setFocusInputWorkProc(_key_handler,NULL,this,this);
     if (_return_key_focus != False){
-      WSGIappKeyboard()->setSpecialInputWorkProc(_special_key_handler,this);
+      WSGIappKeyboard()->setSpecialInputWorkProc(_special_key_handler,this,this);
     }
-    if (_kanji_in == False){
+    if (_enable_fep == WS_FEP_DISABLE){
       WSGIappKeyboard()->setEnableFep(getowndev(),False);
     }else{
       WSGIappKeyboard()->setEnableFep(getowndev(),True);
+#ifdef MSW
+      if (_enable_fep == WS_FEP_KANJI_ON_FOCUS ||
+          _enable_fep == WS_FEP_KATAKANA_ON_FOCUS ||
+          _enable_fep == WS_FEP_HKATAKANA_ON_FOCUS ){
+        _fep_set_focus = True;
+      }
+#endif
     }
     if (_cur_timer_id != 0){
 WSMFtrace("WSCvifiled::onWindowFocusChange(%d)   _cur_timer_id ERROR!!!!\n",fl);
@@ -2294,35 +2434,54 @@ WSMFtrace("WSCvifiled::onWindowFocusChange(%d)   _cur_timer_id ERROR!!!!\n",fl);
     }
     _fep_cursor_adjust(WSGIappKeyboard());
 
-    if (_kanji_in != False && _fep_set_focus != False){
-      _fep_set_focus = False;
-      long ox,oy;
-      if (_multi_lines == False){
-        _get_disp_cur_pos(&ox,&oy);
-        WSGIappKeyboard()->startKanji(getowndev(),ox + _x,oy + _y,_font,_fore_color,
-                          _back_color);
-      }else{
-        _get_disp_cur_pos(&ox,&oy);
-        WSGIappKeyboard()->startKanji(getowndev(),_shadow_thick + ox + _x - _mx,
-                          _shadow_thick + oy + _y - _my,_font,
-                          _fore_color,_back_color);
+    if (_enable_fep != WS_FEP_DISABLE && _fep_set_focus != False){
+      WSDdev* dev = getowndev();
+      if (dev != NULL){
+        _fep_set_focus = False;
+        long ox,oy;
+        if (_multi_lines == False){
+          _get_disp_cur_pos(&ox,&oy);
+          WSGIappKeyboard()->startFep(getowndev(),ox + _x,oy + _y,_font,_fore_color,
+                            _back_color);
+        }else{
+          _get_disp_cur_pos(&ox,&oy);
+          WSGIappKeyboard()->startFep(getowndev(),_shadow_thick + ox + _x - _mx,
+                            _shadow_thick + oy + _y - _my,_font,
+                            _fore_color,_back_color);
+        }
+#ifdef MSW
+        if (_enable_fep == WS_FEP_KANJI_ON_FOCUS){
+          WSGIappKeyboard()->setFepStatus(WS_FEP_START);
+        }else
+        if (_enable_fep == WS_FEP_KATAKANA_ON_FOCUS){
+          WSGIappKeyboard()->setFepStatus(WS_FEP_KATAKANA);
+        }else
+        if (_enable_fep == WS_FEP_HKATAKANA_ON_FOCUS){
+          WSGIappKeyboard()->setFepStatus(WS_FEP_HKATAKANA);
+        }
+#endif
       }
     }
 
   }else{
-    WSGIappKeyboard()->resetFocusInputWorkProc();
-    WSGIappTimer()->delTimerProc( _cur_timer_id );
-    _cur_timer_id = 0;
-    setCursorFore(False);
-    if (_return_key_focus != False){
-      WSGIappKeyboard()->resetSpecialInputWorkProc();
+    if (WSGIappKeyboard()->getKeepFocus() == False){
+      WSGIappKeyboard()->setEnableFep(getowndev(),False);
+      WSGIappKeyboard()->resetFocusInputWorkProc();
+      WSGIappTimer()->delTimerProc( _cur_timer_id );
+      _cur_timer_id = 0;
+      setCursorFore(False);
+      if (_return_key_focus != False){
+        WSGIappKeyboard()->resetSpecialInputWorkProc();
+      }
     }
   }
   setAbsoluteDraw(True);
   draw();
 
   if (fl == False && getowndev() != NULL){
-    WSGIappKeyboard()->resetFep(getowndev());
+    if (WSGIappKeyboard()->getKeepFocus() == False){
+      WSGIappKeyboard()->resetFep(getowndev());
+    }
   }
 }
 void WSCvifield::setKeyInputDisable(WSCbool fl){
@@ -2331,3 +2490,13 @@ void WSCvifield::setKeyInputDisable(WSCbool fl){
 WSCbool WSCvifield::getKeyInputDisable(){
   return _key_input_disable;
 }
+void WSCvifield::_focus_move_for_combo(long direct){
+  WSCbase* parent = getParent();
+  if (parent->cast("WSCcomboBox") != NULL){
+    WSCbase* target = parent->getFocusMoveInstance(direct);
+    if (target != NULL){
+      target->setFocus(True);
+      target->redraw();
+    }
+  }
+}
