@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCvmeter.h>
@@ -127,15 +123,15 @@ long WSCvmeter::draw(){
     value2 = value1;
   }
   if (value2 != value1){
-    double tmp_v1, tmp_v2;
+    long tmp_v1, tmp_v2;
     if (_orientation == WS_VERTICAL) {
-      tmp_v1 = (double)(yy + hh) - (double)(value1 * hh) / 100.0;
-      tmp_v2 = (double)(yy + hh) - (double)(value2 * hh) / 100.0;
+      tmp_v1 = yy + hh - value1 * hh / 100;
+      tmp_v2 = yy + hh - value2 * hh / 100;
       yy = (short)tmp_v1;
       hh = (WSCushort)(tmp_v2 - tmp_v1);
     }else{
-      tmp_v1 = (double)(xx) + (double)(value1 * ww) / 100.0;
-      tmp_v2 = (double)(xx) + (double)(value2 * ww) / 100.0;
+      tmp_v1 = xx + value1 * ww / 100;
+      tmp_v2 = xx + value2 * ww / 100;
       xx = (short)tmp_v2;
       ww = (WSCushort)(tmp_v1 - tmp_v2);
     }
