@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCvscrBar.h>
@@ -68,8 +64,9 @@ WSCvscrBar::WSCvscrBar(WSCbase* base,char* objname):
   _page_increment = 30;
   _value          = 0;
   _orientation    = WS_HORIZONTAL;
+#ifndef NO_GRADATION
   _grad_on = 0;
-  _mask_images = NULL;
+#endif
   WSGIappObjectList()->addEvent(this,WSEV_GUI_POLICY_CH);
 
   WSMFpropertyCreateStart
@@ -88,8 +85,13 @@ WSCvscrBar::WSCvscrBar(WSCbase* base,char* objname):
     WSMFpropertyCreate(WSNorientation,WSCuchar, _orientation,WSSorientation );
       WSMFpropertySetSelection(WSRorientation,WSRorientationD);
     WSMFpropertyCreate(WSNbackPixmap, short, _bp_no,WSSbackPixmap  );
+#ifndef NO_GRADATION
     WSMFpropertyCreate(WSNgradation, WSCbool, _grad_on,WSSgradation2 );
       WSMFpropertySetSelection(WSRbool3,WSRbool3D);
+#endif
+    WSMFpropertyCreate(WSNkeepPixmapCache,WSCbool,_keep_pixmap_cache,WSSkeepPixmapCache );
+      WSMFpropertySetSelection(WSRbool4,WSRbool4D);
+
 
     WSMFpropertyDelete(WSNblinkFlag);
     WSMFpropertyDelete(WSNblinkRate);
@@ -118,7 +120,6 @@ WSCvscrBar::~WSCvscrBar(){
   WSGIappObjectList()->delEvent(this,WSEV_GUI_POLICY_CH);
 }
 
-//ץѥƥѥץκ
 WSMFproperty( WSCvscrBar, WSNshadowThickness,   WSCuchar, _shadow_thick,2             );
 //WSMFproperty( WSCvscrBar, WSNbackColor,         short,  _bg_no   ,WSGFcolor("gray85") );
 //WSMFproperty( WSCvscrBar, WSNbarShadowColor,    short,  _bg2_no  ,WSGFcolor("gray75") );
@@ -135,10 +136,12 @@ WSMFproperty( WSCvscrBar, WSNpageIncrement,     WSCushort, _page_increment  ,30
 WSMFproperty( WSCvscrBar, WSNvalue,             WSCulong, _value           ,0         );
 WSMFproperty( WSCvscrBar, WSNorientation,       WSCuchar,  _orientation     ,WS_HORIZONTAL   );
 WSMFproperty(WSCvscrBar, WSNbackPixmap,        short,_bp_no    ,-1    );
+#ifndef NO_GRADATION
 WSMFproperty(WSCvscrBar, WSNgradation,        WSCbool,_grad_on    ,0    );
-
+WSMFproperty(WSCvscrBar, WSNkeepPixmapCache, WSCbool,  _keep_pixmap_cache,True );
 void WSCvscrBar::setWorkWSNgradation(WSCbool){ }
 void WSCvscrBar::getWorkWSNgradation(WSCbool*){ }
+#endif
 void WSCvscrBar::setWorkWSNbackPixmap(short){ }
 void WSCvscrBar::getWorkWSNbackPixmap(short*){ }
 void WSCvscrBar::setWorkWSNshadowThickness(WSCuchar){ }
@@ -200,6 +203,8 @@ void WSCvscrBar::setWorkWSNvalue(WSCulong value){
 void WSCvscrBar::getWorkWSNvalue(WSCulong*){}
 void WSCvscrBar::setWorkWSNorientation(WSCuchar){}
 void WSCvscrBar::getWorkWSNorientation(WSCuchar*){}
+void WSCvscrBar::setWorkWSNkeepPixmapCache(WSCbool){}
+void WSCvscrBar::getWorkWSNkeepPixmapCache(WSCbool*){}
 
 void WSCvscrBar::_timer_work(WSCuchar clock,void* data){
   WSCvscrBar* obj = (WSCvscrBar*)data;
@@ -485,8 +490,9 @@ long WSCvscrBar::draw(){
       st.top_shadow_color = _ts_no;
       st.bottom_shadow_color = _bs_no;
       st.back_pixmap = _bp_no;
+#ifndef NO_GRADATION
       st.gradation = _grad_on;
-
+#endif
       _ext_draw_proc(&st);
       return WS_NO_ERR;
     }
@@ -509,13 +515,23 @@ long WSCvscrBar::draw(){
   }
   double bar_area_width  = w - h*3 + st*6;
   double bar_area_height = h - w*3 + st*6;
+  if (bar_area_width == 0.0){
+    bar_area_width = 1;
+  }
+  if (bar_area_height == 0.0){
+    bar_area_height = 1;
+  }
 
   dev->setForeColor( _bg2_no );
+#ifndef NO_GRADATION
   if (_grad_on == False){
+#endif
     dev->drawFillRect( 0,0,w,h);
+#ifndef NO_GRADATION
   }else{
     dev->drawGradation( WS_GR_LT_RB,_ts_no,_bg_no,_bs_no,0-w/2,0-h/2,w*2,h*2,0);
   }
+#endif
 
   if (_bp_no != -1){
     WSDimage* image = WSGIappImageSet()->getImage(_bp_no);
@@ -532,13 +548,21 @@ long WSCvscrBar::draw(){
       if (w > 2  + st*2){
         WSCulong _slider_size_alt = _slider_size;
         long _value_alt = _value;
-        if ((WSCulong)((double)bar_area_width * _slider_size /_maximum) < WS_MINIMUM_WIDTH ){
-          if ( w-h*3 + st*6 != 0){
+        if ((WSCulong)(bar_area_width * _slider_size /_maximum) < WS_MINIMUM_WIDTH ){
+          if ( bar_area_width != 0){
             _slider_size_alt  = (WSCulong)(WS_MINIMUM_WIDTH * _maximum / bar_area_width);
+            if (_slider_size_alt > _maximum){
+              _slider_size_alt = _maximum;
+            }
+
+#if 0
             if ( x +w +st*3 -h*3/2 < 
                x -st*3 +h*3/2 +(long)((double)bar_area_width * (_value+_slider_size_alt)/_maximum)){ 
               _value_alt = _maximum -_slider_size_alt;
             }
+#endif
+            _value_alt = _value * (double)(_maximum - _slider_size_alt)/(_maximum - _slider_size);
+
             if (_value_alt < 0){
               _value_alt = 0;
             }
@@ -598,46 +622,7 @@ long WSCvscrBar::draw(){
                       ah,ah);
         }
         //bar
-#ifndef WS_EMBED
-        if (policy & WS_POLICY_AQUA){
-          WSCushort bw = (long)(x +w + st*3 -((double)h)*3/2 - (((double)bar_area_width) * (_maximum -_value_alt -_slider_size_alt) / _maximum )
-                   -(x +((double)h)*3/2 -st*3 + (((double)bar_area_width) * _value_alt / _maximum )));
-          WSCushort bh = h -st*2;
-          if (_mask_images != NULL){
-            WSCulong val = (WSCulong)((WSCulong)bw << 16 + bh);
-            WSCulong val2 = (WSCulong)_mask_images[4];
-            if (val != val2){
-              delete _mask_images[0];
-              delete _mask_images[1];
-              delete _mask_images[2];
-              delete _mask_images[3];
-              delete _mask_images;
-              _mask_images = NULL;
-            }
-          }
-          if (_mask_images == NULL){
-            _mask_images = new WSDmwindowDev*[5];
-            _mask_images[0] = WSGFgetAquaImage(_ts_no,bw,bh,0);
-            _mask_images[1] = WSGFgetAquaImage(WS_DF_AQUAFORECOLOR,bw,bh,0);
-            _mask_images[2] = WSGFgetAquaImage(WS_DF_AQUAHIGHLIGHTCOLOR,bw,bh,0);
-            _mask_images[3] = WSGFgetAquaMaskImage(bw,bh,0);
-            _mask_images[4] = (WSDmwindowDev*)((WSCulong)((WSCulong)bw << 16 + bh));
-          }
-          short posx =  (short)(x +(double)h*3/2 -st*2 + (bar_area_width * _value_alt / _maximum ));
-          short posy =  y +st*2;
-//          if (_mouse_in != False){
-//            _mask_images[2]->copyToWindowWithMask(dev,0,0,bw,bh,0,0,_mask_images[3]);
-//          }else if (getFocus() != False){
-          if (getFocus() != False){
-            _mask_images[1]->copyToWindowWithMask(dev,0,0,bw,bh,posx,posy,_mask_images[3]);
-          }else{
-            _mask_images[2]->copyToWindowWithMask(dev,0,0,bw,bh,posx,posy,_mask_images[3]);
-//            _mask_images[0]->copyToWindowWithMask(dev,0,0,bw,bh,posx,posy,_mask_images[3]);
-          }
-        }else{
-#else
         {
-#endif
           dev->setForeColor( _bg_no );
           dev->drawFillRect( (short)(x +(double)h*3/2 -st*2 + (bar_area_width * _value_alt / _maximum )),
                    y +st*2 ,
@@ -708,13 +693,19 @@ long WSCvscrBar::draw(){
         WSCulong _slider_size_alt = _slider_size;
         long           _value_alt = _value;
         if ((WSCulong)(bar_area_height * _slider_size /_maximum) < WS_MINIMUM_WIDTH ){
-          if ( h-w*3 + st*6 != 0){
+          if ( bar_area_height != 0){
             _slider_size_alt  = (WSCulong)(WS_MINIMUM_WIDTH * _maximum / bar_area_height);
-
+            if (_slider_size_alt > _maximum){
+              _slider_size_alt = _maximum;
+            }
+#if 0
             if ( y +h +st*3 -w*3/2 < 
                  y -st*3 +w*3/2 +(long)(bar_area_height * (_value+_slider_size_alt)/_maximum)){ 
                _value_alt = _maximum -_slider_size_alt;
             }
+#endif
+            _value_alt = _value * (double)(_maximum - _slider_size_alt)/(_maximum - _slider_size);
+
             if (_value_alt < 0){
               _value_alt = 0;
             }
@@ -773,42 +764,7 @@ long WSCvscrBar::draw(){
                       ah,ah);
         }
         //bar
-#ifndef WS_EMBED
-        if (policy & WS_POLICY_AQUA){
-          WSCushort bw = w -st*2;
-          WSCushort bh = (long)((y +h +st*3 - ((double)w)*3/2 - (((double)bar_area_height) * (_maximum -_value_alt - _slider_size_alt) / _maximum ))
-                    -(y -st*3 + ((double)w)*3/2 + (((double)bar_area_height) * _value_alt / _maximum )));
-          if (_mask_images != NULL){
-            WSCulong val = (WSCulong)((WSCulong)bw << 16 + bh);
-            WSCulong val2 = (WSCulong)_mask_images[4];
-            if (val != val2){
-              delete _mask_images[0];
-              delete _mask_images[1];
-              delete _mask_images[2];
-              delete _mask_images[3];
-              delete _mask_images;
-              _mask_images = NULL;
-            }
-          }
-          if (_mask_images == NULL){
-            _mask_images = new WSDmwindowDev*[5];
-            _mask_images[0] = WSGFgetAquaImage(_ts_no,bw,bh,0);
-            _mask_images[1] = WSGFgetAquaImage(WS_DF_AQUAFORECOLOR,bw,bh,0);
-            _mask_images[2] = WSGFgetAquaImage(WS_DF_AQUAHIGHLIGHTCOLOR,bw,bh,0);
-            _mask_images[3] = WSGFgetAquaMaskImage(bw,bh,0);
-            _mask_images[4] = (WSDmwindowDev*)((WSCulong)((WSCulong)bw << 16 + bh));
-          }
-          short posx = x +st;
-          short posy = (short)(y -st*3 + (double)w*3/2 + ((double)bar_area_height * _value_alt / _maximum ));
-          if (getFocus() != False){
-            _mask_images[1]->copyToWindowWithMask(dev,0,0,bw,bh,posx,posy,_mask_images[3]);
-          }else{
-            _mask_images[2]->copyToWindowWithMask(dev,0,0,bw,bh,posx,posy,_mask_images[3]);
-          }
-        }else{
-#else
         {
-#endif
           dev->setForeColor( _bg_no );
           dev->drawFillRect(
                    x +st*2 ,
@@ -831,7 +787,7 @@ long WSCvscrBar::draw(){
                    w -st*2, bheight);
 
         }
-        //ܥ
+        // btn down
         dev->setForeColor( _bg_no );
         if (WSGIappDev()->getGuiPolicy() & WS_POLICY_ORIGINAL){
           if (_bar_status == WS_BBTN3){
@@ -864,7 +820,7 @@ long WSCvscrBar::draw(){
                       y+h +st*2 -w*3/2 + w/4,
                       ah,ah);
         }
-        //ܥ󲼲
+        // btn down down 
         dev->setForeColor( _bg_no );
         dev->drawFillRect( x + st*2 , y+h +st*2 -w/2, smallbtn_w, smallbtn_h );
         WSGFdrawShadow(dev,_bar_status != WS_BBTN4 ? WS_SHADOW_OUT : WS_SHADOW_IN,st2,
@@ -889,18 +845,31 @@ long WSCvscrBar::_bar_in(short px,short py,
     st = 0;
   }
   if (_orientation == WS_HORIZONTAL){
+    double bar_area_width  = w - h*3 + st*6;
+
     if (w > 2  + st*2){
 
        WSCulong _slider_size_alt = _slider_size;
        long _value_alt = _value;
-       if ((WSCulong)((double)(w - h*3 +st*6) * _slider_size /_maximum) < 5 ){
+       if ((WSCulong)(bar_area_width * _slider_size /_maximum) < WS_MINIMUM_WIDTH ){
+          if ( bar_area_width != 0){ 
+            _slider_size_alt  = (WSCulong)(WS_MINIMUM_WIDTH * _maximum / bar_area_width);
+            if (_slider_size_alt > _maximum){
+              _slider_size_alt = _maximum;
+            }
+
+#if 0
           _slider_size_alt  = 5 * _maximum / (w - h*3 + st*6);
           if ( x +w +st*3 -h*3/2 < 
                x -st*3 +h*3/2 +(long)((double)(w - h*3 +st*6) * (_value+_slider_size_alt)/_maximum)){ 
              _value_alt = (long)((double)(w +st*6 -h*3) * _maximum /(w - h*3 + st*6))-_slider_size_alt;
           }
-          if (_value_alt < 0){
-            _value_alt = 0;
+#endif
+
+            _value_alt = _value * (double)(_maximum - _slider_size_alt)/(_maximum - _slider_size);
+            if (_value_alt < 0){
+              _value_alt = 0;
+            }
           }
        }
 
@@ -942,18 +911,30 @@ long WSCvscrBar::_bar_in(short px,short py,
     }
 
   }else{
+    double bar_area_height = h - w*3 + st*6;
     if (h > 2  + st*2){
 
        WSCulong _slider_size_alt = _slider_size;
        long _value_alt = _value;
-       if ((WSCulong)((double)(h - w*3 +st*6) * _slider_size /_maximum) < 5 ){
+       if ((WSCulong)(bar_area_height * _slider_size /_maximum) < WS_MINIMUM_WIDTH ){
+          if ( bar_area_height != 0){
+            _slider_size_alt  = (WSCulong)(WS_MINIMUM_WIDTH * _maximum / bar_area_height);
+            if (_slider_size_alt > _maximum){
+              _slider_size_alt = _maximum;
+            }
+
+#if 0
           _slider_size_alt  = 5 * _maximum / (h - w*3 + st*6);
           if ( y +h +st*3 -w*3/2 < 
                y -st*3 +w*3/2 +(long)((double)(h - w*3 +st*6) * (_value+_slider_size_alt)/_maximum)){ 
              _value_alt = (long)((double)(h +st*6 -w*3) * _maximum /(h - w*3 + st*6))-_slider_size_alt;
           }
-          if (_value_alt < 0){
-            _value_alt = 0;
+#endif
+            _value_alt = _value * (double)(_maximum - _slider_size_alt)/(_maximum - _slider_size);
+
+            if (_value_alt < 0){
+              _value_alt = 0;
+            }
           }
        }
 
@@ -998,7 +979,7 @@ long WSCvscrBar::_bar_in(short px,short py,
 }
 
 long WSCvscrBar::_draw_sl_bar(){
-  //ԥޥåˤʤäΤɬפʤʤä
+  // this function became no need because of drawing of pixmap style.
   draw();
   return WS_NO_ERR;
 }
@@ -1007,7 +988,7 @@ WSCulong WSCvscrBar::getMaxSlideValue(){
 }
 
 void(*WSCvscrBar::_ext_draw_proc)(void*) = NULL;
-WSCstring WSCvscrBar::_policy_name;
+char* WSCvscrBar::_policy_name = NULL;
 
 void WSCvscrBar::onGuiPolicyChange(long policy){
   if (getVisible() != False){
@@ -1023,9 +1004,14 @@ void WSCvscrBar::onGuiPolicyChange(long policy){
   if (WSGIappDev()->getExtGuiPolicySelected() == False){
     _ext_draw_proc = NULL;
   }else{
-    if (!strcmp((char*)_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
+    if (_policy_name != NULL &&
+        !strcmp(_policy_name,(char*)WSGIappDev()->getExtGuiPolicyName())){
     }else{
       _ext_draw_proc = (void(*)(void*))WSGIappDev()->getExtGuiPolicyProc("WSCvscrBar::draw");
+      if (_policy_name != NULL){
+        delete _policy_name;
+      }
+      _policy_name = WSGFstrdup(WSGIappDev()->getExtGuiPolicyName());
     }
   }
   if (getVisible() != False){
@@ -1038,6 +1024,9 @@ void WSCvscrBar::onGuiPolicyChange(long policy){
 char* WSCvscrBar::getDefaultProperty(){
   return WSNvalue;
 }
+void WSCvscrBar::_set_internal_value(long val){
+  _value = val;
+}
 #ifdef WS_EMBED
 #ifndef NO_EXT_GUI_POLICY
 class _gui_policy_register_{
