@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCcomboBox.h>
@@ -39,6 +35,7 @@
 #include <WSCvifield.h>
 #include <WSCblink.h>
 #include <WSCfontSet.h>
+#include <WSDkeyboard.h>
 
 WSMFguiClassInitialize(WSCcomboBox,WSCform);
 WSMFversion(WSCcomboBox, WSCform);
@@ -56,44 +53,14 @@ WSCcomboBox::WSCcomboBox(WSCbase* base,char* objname):WSCform(base,objname){
   _btn = NULL;
   _if = NULL;
   _shadow_type = WS_SHADOW_IN;
-  _bl_type = WS_FORE;
-//  _bg_blink_color =  WSGFcolor("gray85");
-//  _work_back_color =  WSGFcolor("gray85");
   _bg_blink_color = WS_DF_NWBACKCOLOR;
   _work_back_color = WS_DF_WORKBACKCOLOR;
-  _rv_fl = False;
-  _emboss = False;
-  _use_pix = False;
-
-  _max_length = WS_VLABEL_MAXLEN_DEFAULT;
-  _label_string = WSGFstrdup("");
-  _alignment_v = WS_CENTER;
-  _alignment_h = WS_CENTER;
-  _margin_top = 2;
-  _margin_bottom = 2;
-  _margin_left = 2;
-  _margin_right = 2;
-  _label_pixmap = -1;
-  _blink_pixmap = -1;
   _font = (char)WSGIappFontSet()->getDefaultFontNo();
+  _increment = 10;
+  _page_increment = 100;
 
-  _upname = NULL;
-  _dwname = NULL;
-  _lname = NULL;
-  _rname = NULL;
-  _rtname = NULL;
-  _tbname = NULL;
-  _btbname = NULL;
-  _enable_focus_move = False;
-  _cursor_adjust = True;
-  _skip_mode = False;
-  _columns = 20;
-  _cur_pos = 0;
-  _kanji_in = True;
-  _icursor = True;
-  _fspace = False;
   _item_height = 20;
-  _enable_input = 1;
+  _selected_by_key = False;
 
   WSMFpropertyCreateStart
     WSMFparentCheckVerSrc(WSCcomboBox);
@@ -101,60 +68,19 @@ WSCcomboBox::WSCcomboBox(WSCbase* base,char* objname):WSCform(base,objname){
     WSMFpropertyCreate(WSNmenuItems, char*, _menu_str,WSSmenuItems);
     WSMFpropertyCreate(WSNmaxHeight, WSCushort, _max_height,WSSmaxHeight);
 
-    WSMFpropertyCreate(WSNreverseFlag, WSCbool, _rv_fl,WSSreverseFlag);
-      WSMFpropertySetSelection(WSRbool4, WSRbool4D);
     WSMFpropertyCreate(WSNbackBlinkColor, short, _bg_blink_color,WSSbackBlinkColor);
     WSMFpropertyCreate(WSNworkBackColor,short,_work_back_color,WSSworkBackColor);
-    WSMFpropertyCreate(WSNblinkType, WSCuchar, _bl_type,WSSblinkType);
-      WSMFpropertySetSelection(WSRblinkType, WSRblinkTypeD);
-    WSMFpropertyCreate(WSNemboss,           WSCbool,  _emboss,WSSemboss);
-      WSMFpropertySetSelection(WSRbool1, WSRbool1D);
-    WSMFpropertyCreate(WSNusePixmap,    WSCuchar,  _use_pix,WSSusePixmap);
-      WSMFpropertySetSelection(WSRbool4, WSRbool4D);
-
-    WSMFpropertyCreate(WSNlabelString,  char*,   _label_string,WSSlabelString);
-    WSMFpropertyCreate(WSNalignmentV,   WSCuchar,   _alignment_v,WSSalignmentV );
-      WSMFpropertySetSelection(WSRalignmentV, WSRalignmentVD);
-    WSMFpropertyCreate(WSNalignmentH,   WSCuchar,   _alignment_h,WSSalignmentH);
-      WSMFpropertySetSelection( WSRalignmentH, WSRalignmentHD);
-    WSMFpropertyCreate(WSNmarginTop, WSCuchar, _margin_top,WSSmarginTop );
-    WSMFpropertyCreate(WSNmarginBottom, WSCuchar, _margin_bottom,WSSmarginBottom);
-    WSMFpropertyCreate(WSNmarginLeft, WSCuchar,_margin_left,WSSmarginLeft );
-    WSMFpropertyCreate(WSNmarginRight,WSCuchar,_margin_right,WSSmarginRight);
-    WSMFpropertyCreate(WSNmaxLength, WSCushort, _max_length,WSSmaxLength);
     WSMFpropertyCreate(WSNfont,  WSCuchar, _font,WSSfont  );
-    WSMFpropertyCreate(WSNlabelPixmap, short, _label_pixmap,WSSlabelPixmap);
-    WSMFpropertyCreate(WSNblinkPixmap, short, _blink_pixmap,WSSblinkPixmap);
 
-    WSMFpropertyCreate(WSNcolumns,         WSCushort, _columns,WSScolumns );
-    WSMFpropertyCreate(WSNcursorPos,  short, _cur_pos, WSScursorPos );
-    WSMFpropertyCreate(WSNkanjiIn,  WSCbool, _kanji_in, WSSkanjiIn );
-      WSMFpropertySetSelection(WSRbool1,WSRbool1D);
-
-    WSMFpropertyCreate(WSNinterCur, WSCbool, _icursor,WSSinterCur);
-      WSMFpropertySetSelection(WSRbool1,WSRbool1D);
-    WSMFpropertyCreate(WSNupward, char*, _upname,WSSupward);
-    WSMFpropertyCreate(WSNdownward,char*, _dwname,WSSdownward);
-    WSMFpropertyCreate(WSNleftward,char*,_lname,WSSleftward);
-    WSMFpropertyCreate(WSNrightward, char*,_rname,WSSrightward);
-    WSMFpropertyCreate(WSNreturn,char*,_rtname,WSSreturn);
-    WSMFpropertyCreate(WSNtab,char*,_tbname,WSStab);
-    WSMFpropertyCreate(WSNbacktab,char*,_btbname,WSSbacktab);
-    WSMFpropertyCreate(WSNenableFocusMove, WSCbool,_enable_focus_move,WSSenableFocusMove);
-    WSMFpropertySetSelection(WSRbool1,WSRbool1D);
-
-    WSMFpropertyCreate(WSNfillSpace, WSCbool, _fspace,WSSfillSpace);
-      WSMFpropertySetSelection(WSRbool1,WSRbool1D);
-    WSMFpropertyCreate(WSNcursorAdjust,WSCbool,_cursor_adjust,WSScursorAdjust);
-      WSMFpropertySetSelection(WSRbool1,WSRbool1D);
-    WSMFpropertyCreate(WSNifieldSkipMode,WSCbool,_skip_mode,WSSifieldSkipMode );
-      WSMFpropertySetSelection(WSRbool1,WSRbool1D);
     WSMFpropertyCreate(WSNitemHeight,  WSCuchar, _item_height,WSSitemHeight  );
-    WSMFpropertyCreate(WSNenableInput,WSCbool,_enable_input,WSSenableInput );
-      WSMFpropertySetSelection(WSRbool3,WSRbool3D);
+    WSMFpropertyCreate(WSNincrement,    WSCushort, _increment,WSSincrement );
+    WSMFpropertyCreate(WSNpageIncrement,WSCushort, _page_increment,WSSpageIncrement );
 
+//    WSMFpropertyValueChangeDef( WSCcomboBox,WSNshadowType ,char);
     WSMFpropertyValueChangeDef( WSCcomboBox,WSNshadowType ,char);
-    WSMFpropertyValueChangeDef( WSCcomboBox,WSNshadowType ,char);
+    WSMFpropertyDelete(WSNupward);
+    WSMFpropertyDelete(WSNdownward);
+    WSMFpropertyDelete(WSNreturn);
 
     WSMFaddTrigger(WSEV_VALUE_CH    );
     WSMFaddTrigger(WSEV_ACTIVATE    );
@@ -167,120 +93,30 @@ WSCcomboBox::WSCcomboBox(WSCbase* base,char* objname):WSCform(base,objname){
 WSMFproperty(WSCcomboBox, WSNmenuItems,  char*, _menu_str, WSGFstrdup("item1,item2,item3"));
 WSMFproperty(WSCcomboBox, WSNmaxHeight,  WSCushort, _max_height,       0);
 
-WSMFproperty(WSCcomboBox, WSNblinkType,         WSCuchar,       _bl_type, WS_FORE          );
-//WSMFproperty(WSCcomboBox, WSNworkBackColor,    short,          _work_back_color, WSGFcolor("gray85"));
 WSMFproperty(WSCcomboBox, WSNworkBackColor, short, _work_back_color,WS_DF_NWBACKCOLOR);
-//WSMFproperty(WSCcomboBox, WSNbackBlinkColor,    short,          _bg_blink_color, WSGFcolor("gray85"));
 WSMFproperty(WSCcomboBox, WSNbackBlinkColor, short, _bg_blink_color,WS_DF_WORKBACKCOLOR);
-WSMFproperty(WSCcomboBox, WSNreverseFlag,       WSCbool,        _rv_fl, False              );
-WSMFproperty(WSCcomboBox, WSNemboss,            WSCbool,        _emboss, False             );
-WSMFproperty(WSCcomboBox, WSNusePixmap,         WSCuchar,       _use_pix,       False      );
-WSMFproperty(WSCcomboBox, WSNlabelString, char*,    _label_string,  WSGFstrdup(""));
-WSMFproperty(WSCcomboBox, WSNalignmentV,  WSCuchar, _alignment_v,   WS_CENTER     );
-WSMFproperty(WSCcomboBox, WSNalignmentH,  WSCuchar, _alignment_h,   WS_CENTER     );
-WSMFproperty(WSCcomboBox, WSNmarginTop,   WSCuchar,_margin_top,    2             );
-WSMFproperty(WSCcomboBox, WSNmarginBottom,WSCuchar,_margin_bottom, 2             );
-WSMFproperty(WSCcomboBox, WSNmarginLeft,  WSCuchar,_margin_left,   2             );
-WSMFproperty(WSCcomboBox, WSNmarginRight, WSCuchar,_margin_right,  2             );
-WSMFproperty(WSCcomboBox, WSNmaxLength,   WSCushort,_max_length,    WS_VLABEL_MAXLEN_DEFAULT );
 WSMFproperty(WSCcomboBox, WSNfont,        WSCuchar, _font,(char)WSGIappFontSet()->getDefaultFontNo());
-WSMFproperty(WSCcomboBox, WSNlabelPixmap,  short,   _label_pixmap,  -1  );
-WSMFproperty(WSCcomboBox, WSNblinkPixmap,  short,   _blink_pixmap,  -1  );
-WSMFproperty(WSCcomboBox, WSNcolumns,   WSCushort, _columns,   20  );
-WSMFproperty(WSCcomboBox, WSNcursorPos,  short,          _cur_pos,0      );
-WSMFproperty(WSCcomboBox, WSNkanjiIn,  WSCbool,    _kanji_in,  True   );
-WSMFproperty(WSCcomboBox, WSNinterCur, WSCbool,    _icursor, True  );
-WSMFproperty(WSCcomboBox, WSNenableFocusMove, WSCbool,    _enable_focus_move, False );
-WSMFproperty(WSCcomboBox, WSNupward,    char*,   _upname, WSGFstrdup(""));
-WSMFproperty(WSCcomboBox, WSNdownward, char*,   _dwname, WSGFstrdup(""));
-WSMFproperty(WSCcomboBox, WSNleftward,   char*,   _lname, WSGFstrdup(""));
-WSMFproperty(WSCcomboBox, WSNrightward,  char*,   _rname, WSGFstrdup(""));
-WSMFproperty(WSCcomboBox, WSNreturn, char*,   _rtname, WSGFstrdup(""));
-WSMFproperty(WSCcomboBox, WSNtab, char*,   _tbname, WSGFstrdup(""));
-WSMFproperty(WSCcomboBox, WSNbacktab, char*,   _btbname, WSGFstrdup(""));
-WSMFproperty(WSCcomboBox, WSNfillSpace,   WSCbool,    _fspace,    False );
-WSMFproperty(WSCcomboBox, WSNcursorAdjust,WSCbool,    _cursor_adjust, True );
-WSMFproperty(WSCcomboBox, WSNifieldSkipMode, WSCbool,  _skip_mode,     False );
 WSMFproperty(WSCcomboBox, WSNitemHeight,     WSCuchar,  _item_height, 20);
-WSMFproperty(WSCcomboBox, WSNenableInput, WSCbool,_enable_input,1);
+WSMFproperty( WSCcomboBox, WSNincrement,     WSCushort, _increment,  10);
+WSMFproperty( WSCcomboBox, WSNpageIncrement, WSCushort, _page_increment, 100);
 
 void WSCcomboBox::setWorkWSNdet(WSCbool fl){
   WSCform::setWorkWSNdet(fl);
-  _if->setPropertyV(WSNdet,fl);
+  if (_if != NULL){
+    _if->setPropertyV(WSNdet,fl);
+  }
 }
-void WSCcomboBox::setWorkWSNupward(char* data){
-  _if->setPropertyV(WSNupward,data);
-}
-void WSCcomboBox::getWorkWSNupward(char**){}
-void WSCcomboBox::setWorkWSNdownward(char* data){
-  _if->setPropertyV(WSNdownward,data);
-}
-void WSCcomboBox::getWorkWSNdownward(char**){}
-void WSCcomboBox::setWorkWSNleftward(char* data){
-  _if->setPropertyV(WSNleftward,data);
-}
-void WSCcomboBox::getWorkWSNleftward(char**){}
-void WSCcomboBox::setWorkWSNrightward(char* data){
-  _if->setPropertyV(WSNrightward,data);
-}
-void WSCcomboBox::getWorkWSNrightward(char**){}
-void WSCcomboBox::setWorkWSNreturn(char* data){
-  _if->setPropertyV(WSNreturn,data);
-}
-void WSCcomboBox::getWorkWSNreturn(char**){}
-void WSCcomboBox::setWorkWSNtab(char* data){
-  _if->setPropertyV(WSNtab,data);
-}
-void WSCcomboBox::getWorkWSNtab(char**){}
-void WSCcomboBox::setWorkWSNbacktab(char* data){
-  _if->setPropertyV(WSNbacktab,data);
-}
-void WSCcomboBox::getWorkWSNbacktab(char**){}
-
-void WSCcomboBox::setWorkWSNcolumns(WSCushort data){
-  _if->setPropertyV(WSNcolumns,data);
-}
-void WSCcomboBox::getWorkWSNcolumns(WSCushort*){}
-void WSCcomboBox::setWorkWSNcursorPos(short data){
-  _if->setPropertyV(WSNcursorPos,data);
-}
-void WSCcomboBox::getWorkWSNcursorPos(short*){}
-void WSCcomboBox::setWorkWSNkanjiIn(WSCbool data){
-  _if->setPropertyV(WSNkanjiIn,data);
-}
-void WSCcomboBox::getWorkWSNkanjiIn(WSCbool*){}
-void WSCcomboBox::setWorkWSNinterCur(WSCbool fl){
-  _if->setPropertyV(WSNinterCur,fl);
-}
-void WSCcomboBox::getWorkWSNinterCur(WSCbool*){}
-void WSCcomboBox::setWorkWSNenableFocusMove(WSCbool data){
-  _if->setPropertyV(WSNenableFocusMove,data);
-}
-void WSCcomboBox::getWorkWSNenableFocusMove(WSCbool*){}
-
-void WSCcomboBox::setWorkWSNfillSpace(WSCbool fl){
-  _if->setPropertyV(WSNfillSpace,fl);
-}
-void WSCcomboBox::getWorkWSNfillSpace(WSCbool*){}
-void WSCcomboBox::setWorkWSNcursorAdjust(WSCbool data){
-  _if->setPropertyV(WSNcursorAdjust,data);
-}
-void WSCcomboBox::getWorkWSNcursorAdjust(WSCbool*){}
-void WSCcomboBox::setWorkWSNifieldSkipMode(WSCbool data){
-  _if->setPropertyV(WSNifieldSkipMode,data);
-}
-void WSCcomboBox::getWorkWSNifieldSkipMode(WSCbool*){}
-
 void WSCcomboBox::setWorkWSNmenuItems(char*){}
 void WSCcomboBox::getWorkWSNmenuItems(char**){}
-void WSCcomboBox::setWorkWSNmaxHeight(WSCushort){}
-void WSCcomboBox::getWorkWSNmaxHeight(WSCushort*){}
-
-void WSCcomboBox::setWorkWSNblinkType(WSCuchar data){
-  _if->setPropertyV(WSNblinkType,data);
+void WSCcomboBox::setWorkWSNmaxHeight(WSCushort mh){
+  if (_list != NULL){
+    WSCulong barw = _list->getProperty(WSNbarThickness);
+    if (_max_height > 0 && _max_height < barw *4){
+      _max_height = barw *4;
+    }
+  }
 }
-void WSCcomboBox::getWorkWSNblinkType(WSCuchar*){}
-
+void WSCcomboBox::getWorkWSNmaxHeight(WSCushort*){}
 void WSCcomboBox::setWorkWSNworkBackColor(short data){
   _if->setPropertyV(WSNbackColor,data);
 }
@@ -290,137 +126,57 @@ void WSCcomboBox::setWorkWSNbackBlinkColor(short data){
   _if->setPropertyV(WSNbackBlinkColor,data);
 }
 void WSCcomboBox::getWorkWSNbackBlinkColor(short*){}
-void WSCcomboBox::setWorkWSNreverseFlag(WSCbool fl){
-  _if->setPropertyV(WSNreverseFlag,fl);
-}
-void WSCcomboBox::getWorkWSNreverseFlag(WSCbool*){}
-void WSCcomboBox::setWorkWSNemboss(WSCbool fl){
-  _if->setPropertyV(WSNemboss,fl);
-}
-void WSCcomboBox::getWorkWSNemboss(WSCbool*){}
-
-void WSCcomboBox::setWorkWSNusePixmap(WSCuchar fl) {
-  _if->setPropertyV(WSNusePixmap,fl);
-}
-void WSCcomboBox::getWorkWSNusePixmap(WSCuchar*){}
-void WSCcomboBox::setWorkWSNlabelString(char* data){
-  _if->setPropertyV(WSNlabelString,data);
-}
-void WSCcomboBox::getWorkWSNlabelString(char** str){
-  if (_label_string != NULL){
-    delete _label_string;
-  }
-  _label_string = WSGFstrdup(_if->getProperty(WSNlabelString));
-  *str = _label_string;
-}
-void WSCcomboBox::setWorkWSNalignmentV(WSCuchar data){
-  _if->setPropertyV(WSNalignmentV,data);
-}
-void WSCcomboBox::getWorkWSNalignmentV(WSCuchar*){}
-void WSCcomboBox::setWorkWSNmarginTop(WSCuchar data){
-  _if->setPropertyV(WSNmarginTop,data);
-}
-void WSCcomboBox::getWorkWSNmarginTop(WSCuchar*){}
-void WSCcomboBox::setWorkWSNmarginBottom(WSCuchar data){
-  _if->setPropertyV(WSNmarginBottom,data);
-}
-void WSCcomboBox::getWorkWSNmarginBottom(WSCuchar*){}
-void WSCcomboBox::setWorkWSNalignmentH(WSCuchar data){
-  _if->setPropertyV(WSNalignmentH,data);
-}
-void WSCcomboBox::getWorkWSNalignmentH(WSCuchar*){}
-void WSCcomboBox::setWorkWSNmarginLeft(WSCuchar data){
-  _if->setPropertyV(WSNmarginLeft,data);
-}
-void WSCcomboBox::getWorkWSNmarginLeft(WSCuchar*){}
-void WSCcomboBox::setWorkWSNmarginRight(WSCuchar data){
-  _if->setPropertyV(WSNmarginRight,data);
-}
-void WSCcomboBox::getWorkWSNmarginRight(WSCuchar*){}
-void WSCcomboBox::setWorkWSNmaxLength(WSCushort len) {
-  if (len > 1024) {
-    _max_length = 1024;
-  }
-  _if->setPropertyV(WSNmaxLength,_max_length);
-}
-
-void WSCcomboBox::getWorkWSNmaxLength(WSCushort*){}
 void WSCcomboBox::setWorkWSNfont(WSCuchar data){
   _if->setPropertyV(WSNfont,data);
+  if (_list != NULL){
+    _list->setPropertyV(WSNfont,data);
+  }
 }
 void WSCcomboBox::getWorkWSNfont(WSCuchar*){}
 void WSCcomboBox::setWorkWSNitemHeight(WSCuchar data){
   if (_list != NULL){
     _list->setProperty(WSNitemHeight,data);
-    _list->setProperty(WSNincrement,data);
-    _list->setProperty(WSNscrollHeight,data-1);
+//    _list->setProperty(WSNincrement,data);
+//    _list->setProperty(WSNscrollHeight,data-1);
+    _list->setProperty(WSNscrollHeight,data);
   }
 }
 void WSCcomboBox::getWorkWSNitemHeight(WSCuchar*){}
-void WSCcomboBox::setWorkWSNlabelPixmap(short data){
-  _if->setPropertyV(WSNlabelPixmap,data);
-}
-void WSCcomboBox::getWorkWSNlabelPixmap(short*){}
-void WSCcomboBox::setWorkWSNblinkPixmap(short no){
-  _if->setPropertyV(WSNblinkPixmap,no);
-}
-void WSCcomboBox::getWorkWSNblinkPixmap(short*){}
-
-void WSCcomboBox::setWorkWSNenableInput(WSCbool data){
-  if (data == False){
-    _if->setKeyInputDisable(True);
-  }else{
-    _if->setKeyInputDisable(False);
+void WSCcomboBox::setWorkWSNincrement(WSCushort val){
+  if (_list != NULL){
+    _list->setProperty(WSNincrement,val);
   }
 }
-
-void WSCcomboBox::getWorkWSNenableInput(WSCbool* data){}
+void WSCcomboBox::getWorkWSNincrement(WSCushort*){}
+void WSCcomboBox::setWorkWSNpageIncrement(WSCushort val){
+  if (_list != NULL){
+    _list->setProperty(WSNpageIncrement,val);
+  }
+}
+void WSCcomboBox::getWorkWSNpageIncrement(WSCushort*){}
 
 WSCcomboBox::~WSCcomboBox(){
   _menu_popdown();
   if (_win != NULL){
+#ifndef WS_EMBED
     WSGFcleanupStretchAction(_win);
+#endif
   }
   if (_win != NULL){
     delete _win;
   }
   if (_menu_str != NULL){
-    delete _menu_str;
+    delete[] _menu_str;
     _menu_str = NULL;
   }
-  if (_upname != NULL){
-    delete _upname;
-    _upname = NULL;
-  }
-  if (_dwname != NULL){
-    delete _dwname;
-    _dwname = NULL;
-  }
-  if (_lname != NULL){
-    delete _lname;
-    _lname = NULL;
-  }
-  if (_rname != NULL){
-    delete _rname;
-    _rname = NULL;
-  }
-
-  if (_rtname != NULL){
-    delete _rtname;
-    _rtname = NULL;
-  }
-  if (_tbname != NULL){
-    delete _tbname;
-    _tbname = NULL;
-  }
-  if (_btbname != NULL){
-    delete _btbname;
-    _btbname = NULL;
-  }
+}
 
+WSCbase* WSCcomboBox::getPropertyInheritChild(){
+  return _if;
 }
 
 long WSCcomboBox::initialize(){
+  WSCform::initialize();
   WSCstring name;
   name = (WSCstring)"cw-btn-" + getInstanceName();
   _btn = new WSCvarrow(this,name);
@@ -432,6 +188,10 @@ long WSCcomboBox::initialize(){
   op->setInternal(True);
   op->setFunction(_btn_work,"_btn_work");
   _btn->addProcedure(op);
+  WSCprocedure* op6 = new WSCprocedure("key proc",WSEV_KEY_PRESS);
+  op6->setInternal(True);
+  op6->setFunction(_btn_key_press_work,"_btn_key_press_work");
+  _btn->addProcedure(op6);
 
   name = (WSCstring)"cw-if-" + getInstanceName();
   _if = new WSCvifield(this,name);
@@ -459,13 +219,18 @@ long WSCcomboBox::initialize(){
   op5->setFunction(_value_ch_work,"_value_ch_work");
   _if->addProcedure(op5);
 
+  _btn->setProperty(WSNtab, name);
+  _btn->setProperty(WSNbacktab, name);
+  _btn->setProperty(WSNupward, name);
+  _btn->setProperty(WSNdownward, name);
+  _btn->setProperty(WSNleftward, name);
+  _btn->setProperty(WSNrightward, name);
 
   _adjust();
 
   _btn->setVisible(True);
   _if->setVisible(True);
-
-  return WSCform::initialize();
+  return WS_NO_ERR;
 }
 
 void WSCcomboBox::setWorkWSNwidth(WSCushort data){
@@ -516,7 +281,6 @@ long WSCcomboBox::_menu_popup(){
   dev->getDispAddr(&px,&py);
   py += _h;
   WSCstring str(_menu_str);
-  WSCushort height = _max_height;
   char tmp[2];
   tmp[0] = (char)-1;
   tmp[1] = 0;
@@ -524,9 +288,11 @@ long WSCcomboBox::_menu_popup(){
   str.replaceString("\\,",tmp,0);
 
   long num  = str.getWords(",");
-  if (height == 0){
-    height = num *_h;
+  WSCushort height = num * _item_height + _shadow_thick*2;
+  if (_max_height != 0 && height > _max_height){
+    height = _max_height;
   }
+
   if (height > WSGIappDev()->getHeight() - py){ 
     height = WSGIappDev()->getHeight() -py;
     if (height < 10){
@@ -547,8 +313,9 @@ long WSCcomboBox::_menu_popup(){
     _list->setPropertyV(WSNreverseSelect,(WSCbool)False);
     _list->setPropertyV(WSNmargin,(WSCuchar)_shadow_thick);
     _list->setProperty(WSNitemHeight,_item_height);
-    _list->setProperty(WSNincrement,_item_height);
-    _list->setProperty(WSNscrollHeight,_item_height-1);
+    _list->setProperty(WSNincrement,_increment);
+    _list->setProperty(WSNpageIncrement,_page_increment);
+    _list->setProperty(WSNscrollHeight,_item_height);
 
     _list->setUserData("_CMB",(void*)this);
     _list->setAbsoluteChangeSelectFlag(True);
@@ -559,6 +326,10 @@ long WSCcomboBox::_menu_popup(){
     op2->setFunction(_list_ptr_work,"_list_ptr_work");
     _list->addProcedure(op2);
 
+    WSCulong barw = _list->getProperty(WSNbarThickness);
+    if (_max_height != 0 && _max_height < barw *4){
+      _max_height = barw *4;
+    }
   }
   _win->setPropertyV(WSNforeColor,_fore_color);
   _win->setPropertyV(WSNbackColor,_back_color);
@@ -629,11 +400,17 @@ long WSCcomboBox::_menu_popup(){
   if (ldev != NULL){
     WSCbool val = True;
     ldev->setValue(WSDEV_GRAB_POINTER,(void*)&val);
+    _list->setEnableActivate(False);
+    _list->setFocus();
+    _list->setSelectItemChanged(False);
+    _list->setEnableActivate(True);
+    WSGIappKeyboard()->setSpecialInputWorkProc(_special_key_handler,this,this);
   }
   if (_timer != NULL){
     _timer->setWorkProc(_list_timer_work,this);
     _timer->startTimer();
   }
+  WSGIappMouse()->setMouseFocusClient(_list);
   return WS_NO_ERR;
 }
 void WSCcomboBox::_adjust(){
@@ -662,6 +439,13 @@ void WSCcomboBox::_adjust(){
   _btn->setPropertyV(WSNbottomShadowColor,(short)_bs_color);
 }
 long WSCcomboBox::_menu_popdown(){
+  if (_list != NULL){
+    WSDdev* ldev = _list->getowndev();
+    if (ldev != NULL){
+      WSCbool val = False;
+      ldev->setValue(WSDEV_GRAB_POINTER,(void*)&val);
+    }
+  }
   if (_win != NULL){
     WSCushort height;
     _win->getPropertyV(WSNheight,&height);
@@ -681,9 +465,11 @@ long WSCcomboBox::_menu_popdown(){
   long fl = (long)getUserData("Selected-cleard");
   if (fl != False){
     setFocus(True);
+    WSGIappKeyboard()->resetSpecialInputWorkProc();
     return WS_NO_ERR;
   }
   if (_list == NULL){
+    WSGIappKeyboard()->resetSpecialInputWorkProc();
     setFocus(True);
     return WS_NO_ERR;
   }
@@ -693,6 +479,7 @@ long WSCcomboBox::_menu_popdown(){
     _if->setPropertyV(WSNlabelString,(char*)data);
     onActivate();
   }
+  WSGIappKeyboard()->resetSpecialInputWorkProc();
   setFocus(True);
   return WS_NO_ERR;
 }
@@ -756,6 +543,15 @@ void WSCcomboBox::_list_timer_work(void* ptr){
   cmb->_list->getPropertyV(WSNhbarValue,&vx);
   cmb->_list->getPropertyV(WSNvbarValue,&vy);
 //printf("void WSCcomboBox::_list_timer_work(void* ptr)\n");
+  static short bx =0 ,by = 0;
+  short tx =0 ,ty = 0;
+  WSGIappMouse()->getMousePosition(&tx,&ty,cmb);
+  if (tx != bx || ty != by ){
+    cmb->_selected_by_key = False;
+    bx = tx;
+    by = ty;
+  }
+
   for(i=0; i<num; i++){
     WSCbase* lb = (WSCbase*)(*lbs)[i];
     short px,py;
@@ -767,7 +563,7 @@ void WSCcomboBox::_list_timer_work(void* ptr){
 //    px += vx;
 //    py += vy;
     if (0 < px && px < sw && 
-        0 < py && py < sh ){
+        0 < py && py < sh && cmb->_selected_by_key == False){
       cmb->_list->setEnableActivate(False);
       cmb->_list->setSelectPos(i);
       cmb->_list->setSelectItemChanged(False);
@@ -777,6 +573,114 @@ void WSCcomboBox::_list_timer_work(void* ptr){
     }
   }
 }
+WSCbool WSCcomboBox::_special_key_handler(void* ptr,WSDkeyboard* keyb,WSCbool fl){
+  WSCcomboBox* _this = (WSCcomboBox*)ptr;
+  if (_this->_win == NULL){
+    return True;
+  }
+  if (_this->_win->getVisible() == False){
+    WSGIappKeyboard()->resetSpecialInputWorkProc();
+    return True;
+  }
+  long key = keyb->getKey();
+  if (key == WSK_Escape || key == WSK_Super_L || key == WSK_Super_R ||
+      key == WSK_Control_L || key == WSK_Control_R || key == WSK_Apps ||
+      key == WSK_Alt_L || key == WSK_Alt_R ||
+      key == WSK_Meta_L || key == WSK_Meta_R ){
+    if (fl == False){
+      return False;
+    }
+    WSDdev* ldev = _this->_list->getowndev();
+    if (ldev != NULL){
+      WSCbool val = False;
+      ldev->setValue(WSDEV_GRAB_POINTER,(void*)&val);
+    }
+    _this->setUserData("Selected-cleard",(void*)1);
+    _this->_menu_popdown();
+    return False;
+  }else
+  if (key == WSK_Return){
+    if (fl == False){
+      return False;
+    }
+    _this->_menu_popdown();
+    _this->_list->onActivate();
+    return False;
+  }else
+  if (key == WSK_Up){
+    if (fl == False){
+      return False;
+    }
+    _this->_selected_by_key = True;
+    long spos = _this->_list->getSelectedPos();
+    if (spos > 0){
+      spos--;
+    }else{
+      spos = 0;
+    }
+    _this->_list->setEnableActivate(False);
+    _this->_list->setSelectPos(spos);
+    long tpos = _this->_list->getTopPos();
+    if (tpos > spos){
+      _this->_list->setTopPos(spos);
+    }
+    WSCbase* cl = _this->_list->getLabel(spos);
+    cl->setFocus();
+    _this->_list->setSelectItemChanged(False);
+    _this->_list->setEnableActivate(True);
+    return False;
+  }else if (key == WSK_Down){
+    if (fl == False){
+      return False;
+    }
+    _this->_selected_by_key = True;
+    long spos = _this->_list->getSelectedPos();
+    if (spos > -1){
+      spos++;
+    }else{
+      spos = -1;
+    }
+    _this->_list->setEnableActivate(False);
+    _this->_list->setSelectPos(spos);
+    long bpos = _this->_list->getBottomPos();
+    if (spos > bpos){
+      _this->_list->setBottomPos(spos);
+    }
+    WSCbase* cl = _this->_list->getLabel(spos);
+    cl->setFocus();
+    _this->_list->setSelectItemChanged(False);
+    _this->_list->setEnableActivate(True);
+    return False;
+  }else{
+    if (fl == False){
+      return False;
+    }
+    char* text = keyb->getText();
+    if ( (text[0] > 'A' -1 && text[0] < 'Z' + 1) ||
+         (text[0] > 'a' -1 && text[0] < 'z' + 1) ||
+         (text[0] > '0' -1 && text[0] < '9' + 1) ){
+      int i;
+      int num = _this->_list->getNum();
+      for(i=0; i<num; i++){
+         WSCstring str = _this->_list->getItem(i);
+         char* str2 = str.getString();
+         if (text[0] == str2[0]){
+           _this->_list->setEnableActivate(False);
+           _this->_list->setTopPos(i);
+           _this->_list->setSelectPos(i);
+           WSCbase* cl = _this->_list->getLabel(i);
+           cl->setFocus();
+           _this->_list->setSelectItemChanged(False);
+           _this->_list->setEnableActivate(True);
+           return False;
+         }
+      }
+    }
+  }
+
+  return True;
+}
+
 
 void WSCcomboBox::_list_ptr_work(WSCbase* list){
   WSCcomboBox* cmb = (WSCcomboBox*)list->getUserData("_CMB");
@@ -784,6 +688,9 @@ void WSCcomboBox::_list_ptr_work(WSCbase* list){
   if (cmb == NULL){
     return;
   }
+  // for touch pannel..
+  _list_timer_work(cmb);
+
   short px,py;
   WSGIappMouse()->getMousePosition(&px,&py,list);
 //  short sx=0,sy=0;
@@ -857,7 +764,11 @@ void WSCcomboBox::_list_work(WSCbase* list){
 }
 void WSCcomboBox::onFocusChange(WSCbool fl){
   if (_if != NULL){
-    _if->setFocus(True);
+//    WSCbool readonly = _if->getProperty(WSNdisplayOnly);
+      _if->setFocus(fl);
+  }
+  if (_btn != NULL && fl == False){
+    _btn->setFocus(fl);
   }
 }
 void WSCcomboBox::_value_ch_work(WSCbase* ifield){
@@ -867,6 +778,17 @@ void WSCcomboBox::_value_ch_work(WSCbase* ifield){
   }
   cmb->execProcedure(WSEV_VALUE_CH);
 }
+void WSCcomboBox::_btn_key_press_work(WSCbase* btn){
+  WSCcomboBox* cmb = (WSCcomboBox*)btn->getUserData("_CMB");
+  if (cmb == NULL){
+    return;
+  }
+  long key = WSGIappKeyboard()->getKey();
+  if (key == WSK_Return || key == WSK_Up || key == WSK_Down){
+    _btn_work(btn);
+    btn->onMouseOut();
+  }
+}
 void WSCcomboBox::_key_press_work(WSCbase* ifield){
   WSCcomboBox* cmb = (WSCcomboBox*)ifield->getUserData("_CMB");
   if (cmb == NULL){
@@ -887,17 +809,34 @@ void WSCcomboBox::_key_hook_work(WSCbase* ifield){
     return;
   }
   cmb->execProcedure(WSEV_KEY_HOOK);
+  long key = WSGIappKeyboard()->getKey();
+  if (key == WSK_Up){
+    if (cmb->_win == NULL || cmb->_win->getVisible() == False){
+      cmb->_menu_popup();
+      WSGIappKeyboard()->setKey(0);
+    }
+  }else
+  if (key == WSK_Down){
+    if (cmb->_win == NULL || cmb->_win->getVisible() == False){
+      cmb->_menu_popup();
+      WSGIappKeyboard()->setKey(0);
+    }
+  }
 }
 void WSCcomboBox::_btn_work(WSCbase* btn){
   WSCcomboBox* cmb = (WSCcomboBox*)btn->getUserData("_CMB");
   if (cmb == NULL){
     return;
   }
-  if (cmb->_win == NULL || cmb->_win->getVisible() == False){
-    cmb->_menu_popup();
-  }else{
-    cmb->_menu_popdown();
+  //for popup menu..
+  if ((WSGIappMouse()->getStatus() & WS_MOUSE_BTN3) == 0){
+    if (cmb->_win == NULL || cmb->_win->getVisible() == False){
+      cmb->_menu_popup();
+    }else{
+      cmb->_menu_popdown();
+    }
   }
+
   short x,y;
   WSGIappMouse()->getMousePosition(&x,&y,btn);
   WSCpoint pt;
@@ -908,3 +847,6 @@ void WSCcomboBox::_btn_work(WSCbase* btn){
 char* WSCcomboBox::getDefaultProperty(){
   return WSNlabelString;
 }
+WSCvifield* WSCcomboBox::getTextFrame(){
+  return _if;
+}
