@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCdirTree.h>
@@ -62,6 +58,8 @@ WSCdirTree::WSCdirTree(WSCbase* base, char* objname):
   _dir_name = WSGFstrdup("");
   _file_name = WSGFstrdup("");
   _mask_name = WSGFstrdup("");
+  _root_dir_label = WSGFstrdup("/");
+  _top_label = WSGFstrdup("My computer");
   _select_dir = True;
 
   WSMFpropertyCreateStart
@@ -91,7 +89,7 @@ void WSCdirTree::setWorkWSNdirName(char* dir){
 void WSCdirTree::getWorkWSNdirName(char** dir){
   long pos = getSelectedPos();
   if (pos == -1){
-    delete _dir_name;
+    delete[] _dir_name;
     _dir_name = WSGFstrdup("");
     *dir = _dir_name;
     return;
@@ -100,7 +98,7 @@ void WSCdirTree::getWorkWSNdirName(char** dir){
   WSCstring dirname;
   dirname = item->getProperty(WSNuserString);
   if (strcmp(_dir_name,(char*)dirname)){
-    delete _dir_name;
+    delete[] _dir_name;
     _dir_name = WSGFstrdup((char*)dirname);
   }
   *dir = _dir_name;
@@ -110,7 +108,7 @@ void WSCdirTree::setWorkWSNfileName(char* fname){
 void WSCdirTree::getWorkWSNfileName(char** fname){
   long pos = getSelectedPos();
   if (pos == -1){
-    delete _file_name;
+    delete[] _file_name;
     _file_name = WSGFstrdup("");
     *fname = _file_name;
     return;
@@ -119,7 +117,7 @@ void WSCdirTree::getWorkWSNfileName(char** fname){
   WSCstring filename;
   filename = item->getProperty(WSNuserString);
   if (strcmp(_file_name,(char*)filename)){
-    delete _file_name;
+    delete[] _file_name;
     _file_name = WSGFstrdup((char*)filename);
   }
   *fname = _file_name;
@@ -158,7 +156,6 @@ long WSCdirTree::setSelectedDirName(char* dirname,WSCbool opened){
     item->setUserData(WS_LIST_TREE_OPEN_CH,(void*)1);
     _lb_adjust();
   }
-
   long i;
   long num =  tmp.getWords("/");
   long pre_pos = 0;
@@ -189,8 +186,10 @@ long WSCdirTree::setSelectedDirName(char* dirname,WSCbool opened){
             val &= ~WS_DT_SELECTED;
             _dir_inf.setData(index,(void*)val);
           }
-
-          setSelectPos(j);
+          long spos = getSelectedPos();
+          if (spos != j){
+            setSelectPos(j);
+          }
         }
         if (i == num -1 && opened == False){
         }else{
@@ -216,7 +215,7 @@ long WSCdirTree::setSelectedDirName(char* dirname,WSCbool opened){
 char* WSCdirTree::getSelectedDirName(){
   long pos = getSelectedPos();
   if (pos == -1){
-    delete _dir_name;
+    delete[] _dir_name;
     _dir_name = WSGFstrdup("/");
     return _dir_name;
   }
@@ -224,7 +223,7 @@ char* WSCdirTree::getSelectedDirName(){
   WSCstring dirname;
   dirname = item->getProperty(WSNuserString);
   if (strcmp(_dir_name,(char*)dirname)){
-    delete _dir_name;
+    delete[] _dir_name;
     _dir_name = WSGFstrdup((char*)dirname);
   }
   return _dir_name;
@@ -233,7 +232,24 @@ char* WSCdirTree::getSelectedDirName(){
 
 WSCdirTree::~WSCdirTree() {
   if (_dir_name != NULL){
-    delete _dir_name;
+    delete[] _dir_name;
+    _dir_name = NULL;
+  }
+  if (_file_name != NULL){
+    delete[] _file_name;
+    _file_name = NULL;
+  }
+  if (_mask_name != NULL){
+    delete[] _mask_name;
+    _mask_name = NULL;
+  }
+  if (_root_dir_label != NULL){
+    delete[] _root_dir_label;
+    _root_dir_label = NULL;
+  }
+  if (_top_label != NULL){
+    delete[] _top_label;
+    _top_label = NULL;
   }
 }
 void WSCdirTree::reset(){
@@ -287,7 +303,8 @@ void WSCdirTree::_lb_adjust(){
         }else{
           dstatus |= WS_DT_OPEN;
         }
-        if (item->getVisible() != False){
+//        if (item->getVisible() != False){
+        if ((WSCbool)item->getProperty(WSNvis) != False){
           if (selected_pos == i){
             dstatus |= WS_DT_SELECTED;
           }
@@ -342,9 +359,9 @@ long WSCdirTree::_adjust_dir(char* path,char* subdir,long level,long pos,long di
         (long)item->getUserData(WS_LIST_INDENT_LEVEL) < level){
       if (subdir[0] == 0){
 #ifdef MSW
-        addItem("My computer",pos);
+        addItem(_top_label,pos);
 #else
-        addItem("/",pos);
+        addItem(_root_dir_label,pos);
 #endif
       }else{
         addItem(subdir,pos);
@@ -352,9 +369,9 @@ long WSCdirTree::_adjust_dir(char* path,char* subdir,long level,long pos,long di
     }else{
       if (subdir[0] == 0){
 #ifdef MSW
-        item->setProperty(WSNlabelString,"My computer");
+        item->setProperty(WSNlabelString,_top_label);
 #else
-        item->setProperty(WSNlabelString,"/");
+        item->setProperty(WSNlabelString,_root_dir_label);
 #endif
       }else{
         item->setProperty(WSNlabelString,subdir);
@@ -363,9 +380,9 @@ long WSCdirTree::_adjust_dir(char* path,char* subdir,long level,long pos,long di
   }else{
     if (subdir[0] == 0){
 #ifdef MSW
-      addItem("My computer");
+      addItem(_top_label);
 #else
-      addItem("/");
+      addItem(_root_dir_label);
 #endif
     }else{
       addItem(subdir);
@@ -373,7 +390,7 @@ long WSCdirTree::_adjust_dir(char* path,char* subdir,long level,long pos,long di
   }
 
   setItemValue(pos,WS_INDENT_LEVEL,level);
-  setItemValue(pos,WS_ICON,(int)WSGIappImageSet()->getImageName(_icon_pixmap));
+  setItemValue(pos,WS_ICON,(long)WSGIappImageSet()->getImageName(_icon_pixmap));
 
   WSCbase* item = getLabel(pos);
   if (subdir[0] == 0){
@@ -457,6 +474,17 @@ long WSCdirTree::_adjust_dir(char* path,char* subdir,long level,long pos,long di
     }
     long no_exec = 0; 
 
+    if (dstatus & WS_DT_OPEN){
+      item->setUserData(WS_LIST_TREE_OPEN,(void*)1);
+    }else{
+      item->setUserData(WS_LIST_TREE_OPEN,(void*)0);
+    }
+    if (dstatus & WS_DT_SELECTED){
+      item->setPropertyV(WSNselectColor,(short)_select_color);
+      item->setPropertyV(WSNselectForeColor,(short)_select_fore_color);
+      item->setUserData(SELECT_STATUS,(void*)1);
+    }
+
     if (dir_list != NULL){
       long num = dir_list->getNum();
 //printf("dir_list num=%d\n",num);
@@ -472,22 +500,12 @@ long WSCdirTree::_adjust_dir(char* path,char* subdir,long level,long pos,long di
         }else{
           path2 << path << "/" << subdir;
         }
-        if (dstatus & WS_DT_OPEN){
-          item->setUserData(WS_LIST_TREE_OPEN,(void*)1);
-        }else{
-          item->setUserData(WS_LIST_TREE_OPEN,(void*)0);
-        }
-        if (dstatus & WS_DT_SELECTED){
-          item->setPropertyV(WSNselectColor,(short)_select_color);
-          item->setPropertyV(WSNselectForeColor,(short)_select_fore_color);
-          item->setUserData(SELECT_STATUS,(void*)1);
-        }
         for(i=0; i<num; i++){
 //printf("WSCdirTree::_adjust_dir child for2 %d...\n",i);
           long pos2;
           char* childdir = (char*)dir_list->getData(i);
           if (no_exec != 0){
-            delete childdir;
+            delete[] childdir;
             continue;
           }
           if (dstatus & WS_DT_OPEN){
@@ -497,19 +515,18 @@ long WSCdirTree::_adjust_dir(char* path,char* subdir,long level,long pos,long di
             no_exec = 1;
           }
           pos = pos2;
-          delete childdir;
+          delete[] childdir;
         }
       }
       delete dir_list;
     }
-
     if (file_list != NULL){
       long num = file_list->getNum();
       if (no_exec != 0){
         int i;
         for(i=0; i<num; i++){
           char* fname = (char*)file_list->getData(i);
-          delete fname;
+          delete[] fname;
         }
         delete file_list;
         return pos;
@@ -519,19 +536,19 @@ long WSCdirTree::_adjust_dir(char* path,char* subdir,long level,long pos,long di
         for(i=0; i<num; i++){
           char* fname = (char*)file_list->getData(i);
           if (no_exec != 0){
-            delete fname;
+            delete[] fname;
             continue;
           }
           WSCstring tmp(fname);
           long sufixw = tmp.getWords(".");
           if (_mask_name != NULL && _mask_name[0] != 0 && sufixw < 2){
-            delete fname;
+            delete[] fname;
             continue;
           }
           WSCstring sufix = tmp.getWord(sufixw -1,".");
           if (_mask_name != NULL && _mask_name[0] != 0 &&
               strcmp(sufix.getString(),_mask_name)){
-            delete fname;
+            delete[] fname;
             continue;
           }
           if (dstatus & WS_DT_OPEN){
@@ -571,7 +588,7 @@ long WSCdirTree::_adjust_dir(char* path,char* subdir,long level,long pos,long di
             }
           }
           setItemValue(pos,WS_INDENT_LEVEL,level+1);
-          setItemValue(pos,WS_ICON,(int)WS_DT_FILE_ICON_NAME);
+          setItemValue(pos,WS_ICON,(long)WS_DT_FILE_ICON_NAME);
 
           long fstatus = (long)_dir_inf[(char*)fname];
           fstatus |= WS_DT_FILE;
@@ -586,7 +603,7 @@ long WSCdirTree::_adjust_dir(char* path,char* subdir,long level,long pos,long di
           }
 
           pos++;
-          delete fname;
+          delete[] fname;
         }
       }
       delete file_list;
@@ -689,4 +706,27 @@ void WSCdirTree::setDirInf(WSCstring inf){
   updateList();
 }
 
-
+void WSCdirTree::setRootDirTitle(char* sep){
+  if (_root_dir_label != NULL){
+    delete[] _root_dir_label;
+    _root_dir_label = NULL;
+  }
+  if (sep == NULL){
+    _root_dir_label = WSGFstrdup("/");
+  }else{
+    _root_dir_label = WSGFstrdup(sep);
+  }
+  reset();
+}
+void WSCdirTree::setTopTitle(char* title){
+  if (_top_label != NULL){
+    delete[] _top_label;
+    _top_label = NULL;
+  }
+  if (title == NULL){
+    _top_label = WSGFstrdup("My computer");
+  }else{
+    _top_label = WSGFstrdup(title);
+  }
+  reset();
+}
