@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,17 +14,14 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
+#ifndef NO_FFORM
 #include <WSCfform.h>
 #include <WSCclassInformation.h>
 #include <WSCdevice.h>
@@ -34,6 +31,7 @@
 #include <WSDwindowDev.h>
 #include <WSCfontSet.h>
 #include <WSCwindow.h>
+#include <WSDappDev.h>
 
 WSMFguiClassInitialize(WSCfform,WSCbase);
 WSMFversion(WSCfform,WSCbase);
@@ -58,11 +56,14 @@ WSCfform::WSCfform(WSCbase* base,char* objname):WSCbase(base,objname){
   _bp_no = -1;
   _shadow_type = WS_SHADOW_OUT;
   _sensitive = True;
+  _sensitive_work = True;
   _pixmap_style = WS_DIRECT_WINDOW;
   _no_refresh = False;
   _mouse_no     = 0;
+#ifndef NO_GRADATION
   _grad_no     = 0;
   _grad_margin   = 0;
+#endif
   _update_resize   = 0;
   _anchor_top = 0;
   _anchor_bottom= 0;
@@ -108,15 +109,17 @@ WSCfform::WSCfform(WSCbase* base,char* objname):WSCbase(base,objname){
       WSMFpropertySetSelection(WSRshadowType,WSRshadowTypeD);
     WSMFpropertyCreate(WSNvis,       WSCbool,      _vis,WSSvis );
       WSMFpropertySetSelection(WSRbool2,WSRbool2D);
-    WSMFpropertyCreate(WSNdet,              WSCbool,  _sensitive,WSSdet);
+    WSMFpropertyCreate(WSNdet,              WSCbool,  _sensitive_work,WSSdet);
       WSMFpropertySetSelection(WSRbool1,WSRbool1D);
     WSMFpropertyCreate(WSNpixmapStyle,      WSCuchar,  _pixmap_style,WSSpixmapStyle);
     WSMFpropertySetSelection(WSRpixmapStyle,WSRpixmapStyleD);
     WSMFpropertyCreate( WSNmouse, WSCushort,   _mouse_no,WSSmouse  );
 
+#ifndef NO_GRADATION
     WSMFpropertyCreate(WSNgradation,  WSCuchar,  _grad_no,WSSgradation);
       WSMFpropertySetSelection(WSRgradation,WSRgradationD);
     WSMFpropertyCreate(WSNgradationMargin,WSCuchar,_grad_margin,WSSgradationMargin);
+#endif
 
     WSMFpropertyCreate(WSNanchorTop, WSCushort, _anchor_top,WSSanchorTop);
     WSMFpropertyCreate(WSNanchorBottom,WSCushort,_anchor_bottom,WSSanchorBottom);
@@ -133,15 +136,19 @@ WSCfform::WSCfform(WSCbase* base,char* objname):WSCbase(base,objname){
     WSMFpropertySetSelection(WSRbool3, WSRbool3D);
     WSMFpropertyCreate(WSNorientation,WSCuchar,_orientation,WSSorientation);
     WSMFpropertySetSelection(WSRorientation, WSRorientationD);
-
+#ifndef NO_REMOTE_INSTANCE
     WSMFpropertyCreate(WSNexport,WSCbool,_export_,WSSexport);
     WSMFpropertySetSelection(WSRbool1, WSRbool1D);
+#endif
     WSMFpropertyCreate( WSNindicatorOn,  WSCbool, _indicator,WSSindicatorOn);
       WSMFpropertySetSelection(WSRbool3,WSRbool3D);
     WSMFpropertyCreate( WSNbarThickness,  WSCuchar, _bar_thick,WSSbarThickness); 
     WSMFpropertyCreate(WSNfont,         WSCuchar,   _font,WSSfont  );
     WSMFpropertyCreate(WSNmargin,WSCuchar,_margin,WSSmargin);
 
+    WSMFpropertyCreate(WSNkeepPixmapCache,WSCbool,_keep_pixmap_cache,WSSkeepPixmapCache );
+      WSMFpropertySetSelection(WSRbool4,WSRbool4D);
+
     WSMFaddTrigger(WSEV_INITIALIZE  );
     WSMFaddTrigger(WSEV_MOUSE_MOVE  );
     WSMFaddTrigger(WSEV_MOUSE_IN    );
@@ -175,11 +182,13 @@ WSMFproperty(WSCfform,WSNbottomShadowColor,short, _bs_color, WS_DF_BOTTOMSHADOWC
 WSMFproperty(WSCfform,WSNbackPixmap, short, _bp_no, -1);
 WSMFproperty(WSCfform,WSNshadowType, char,    _shadow_type ,WS_SHADOW_OUT );
 WSMFproperty(WSCfform,WSNvis,WSCbool, _vis ,False  );
-WSMFproperty(WSCfform,WSNdet,WSCbool, _sensitive, True );
+WSMFproperty(WSCfform,WSNdet,WSCbool, _sensitive_work, True );
 WSMFproperty(WSCfform,WSNpixmapStyle, WSCuchar, _pixmap_style ,WS_DIRECT_WINDOW);
 WSMFproperty(WSCfform, WSNmouse,   WSCushort,     _mouse_no     ,0);
+#ifndef NO_GRADATION
 WSMFproperty(WSCfform, WSNgradation, WSCuchar, _grad_no   ,0);
 WSMFproperty(WSCfform, WSNgradationMargin, WSCuchar, _grad_margin ,0);
+#endif
 WSMFproperty(WSCfform, WSNanchorTop,WSCushort,_anchor_top,0 );
 WSMFproperty(WSCfform, WSNanchorBottom,WSCushort,_anchor_bottom,0 );
 WSMFproperty(WSCfform, WSNanchorLeft,WSCushort,_anchor_left,0 );
@@ -189,11 +198,14 @@ WSMFproperty(WSCfform, WSNanchorBottomFlag,WSCbool,_anchor_bottom_use,False );
 WSMFproperty(WSCfform, WSNanchorLeftFlag,WSCbool,_anchor_left_use,False );
 WSMFproperty(WSCfform, WSNanchorRightFlag,WSCbool,_anchor_right_use,False );
 WSMFproperty(WSCfform, WSNorientation,WSCuchar,_orientation,WS_VERTICAL );
+#ifndef NO_REMOTE_INSTANCE
 WSMFproperty(WSCfform, WSNexport,WSCbool,_export_,False );
+#endif
 WSMFproperty(WSCfform, WSNindicatorOn,  WSCbool,  _indicator   ,True);
 WSMFproperty(WSCfform, WSNbarThickness,  WSCuchar,  _bar_thick   ,8);
 WSMFproperty(WSCfform, WSNfont, WSCuchar, _font,(char)WSGIappFontSet()->getDefaultFontNo());
 WSMFproperty(WSCfform, WSNmargin, WSCuchar, _margin ,0);
+WSMFproperty(WSCfform, WSNkeepPixmapCache, WSCbool,  _keep_pixmap_cache,True );
 void WSCfform::setWorkWSNfont(WSCuchar){}
 void WSCfform::getWorkWSNfont(WSCuchar*){}
 void WSCfform::setWorkWSNorientation(WSCuchar ){}
@@ -271,10 +283,12 @@ void WSCfform::setWorkWSNwidth(WSCushort data){
     _geometry_changed = True;
 //printf("WSCfform::setWorkWSNwidth %s ->%d\n",getInstanceName(),_w);
     _update_resize = True;
+#ifndef NO_GRADATION
     if (_grad_no != 0){
       setAbsoluteDraw(True);
       needUpdate();
     }
+#endif
   }else{
     _geometry_changed = True;
     _update_resize = True;
@@ -297,10 +311,12 @@ void WSCfform::setWorkWSNheight(WSCushort data){
     _dev->setValue(WSDEV_HEIGHT,(void*)&data);
     _geometry_changed = True;
     _update_resize = True;
+#ifndef NO_GRADATION
     if (_grad_no != 0){
       setAbsoluteDraw(True);
       needUpdate();
     }
+#endif
   }else{
     _geometry_changed = True;
     _update_resize = True;
@@ -338,27 +354,34 @@ void WSCfform::setWorkWSNvis(WSCbool data){
 }
 void WSCfform::getWorkWSNvis(WSCbool*){}
 void WSCfform::setWorkWSNdet(WSCbool data){
+#ifndef WS_EMBED
   if (getPropertyEditMode() == False){
-    if (data == False){
-      _sensitive = True;
-    }else{
-      _sensitive = False;
-    }
+#endif
+//    if (data == False){
+//      _sensitive = True;
+//    }else{
+//      _sensitive = False;
+//    }
     setSensitive(data);
-  }else{
-    if (data == False){
-      setUserData(WS_DET,(void*)0);
-    }else{
-      setUserData(WS_DET,(void*)1);
-    }
+#ifndef WS_EMBED
+//  }else{
+//    if (data == False){
+//      setUserData(WS_DET,(void*)0);
+//    }else{
+//      setUserData(WS_DET,(void*)1);
+//    }
+//    _sensitive = True;
   }
+#endif
 }
 void WSCfform::getWorkWSNdet(WSCbool* data){
-  if (getPropertyEditMode() != False){
-    long tmp = (long)getUserData(WS_DET);
-    WSCbool fl = (WSCbool)tmp;
-    *data = fl;
-  }
+#ifndef WS_EMBED
+//  if (getPropertyEditMode() != False){
+//    long tmp = (long)getUserData(WS_DET);
+//    WSCbool fl = (WSCbool)tmp;
+//    *data = fl;
+//  }
+#endif
 }
 void WSCfform::setWorkWSNpixmapStyle(WSCuchar data){
   if (_initialized != False && _dev != NULL){
@@ -376,10 +399,12 @@ void WSCfform::setWorkWSNmouse(WSCushort data ){
   }
 }
 void WSCfform::getWorkWSNmouse(WSCushort* ){}
+#ifndef NO_GRADATION
 void WSCfform::setWorkWSNgradation(WSCuchar ){}
 void WSCfform::getWorkWSNgradation(WSCuchar* ){}
 void WSCfform::setWorkWSNgradationMargin(WSCuchar ){}
 void WSCfform::getWorkWSNgradationMargin(WSCuchar* ){}
+#endif
 void WSCfform::setWorkWSNmargin(WSCuchar ){}
 void WSCfform::getWorkWSNmargin(WSCuchar* ){}
 
@@ -456,15 +481,19 @@ void WSCfform::setWorkWSNanchorRightFlag(WSCbool data){
 }
 void WSCfform::getWorkWSNanchorRightFlag(WSCbool* data){}
 
+#ifndef NO_REMOTE_INSTANCE
 void WSCfform::setWorkWSNexport(WSCbool data){
   setExported(data);
 }
 void WSCfform::getWorkWSNexport(WSCbool* data){
 }
+#endif
 void WSCfform::setWorkWSNindicatorOn(WSCbool data){}
 void WSCfform::getWorkWSNindicatorOn(WSCbool* data){}
 void WSCfform::setWorkWSNbarThickness(WSCuchar data){}
 void WSCfform::getWorkWSNbarThickness(WSCuchar* data){}
+void WSCfform::setWorkWSNkeepPixmapCache(WSCbool){}
+void WSCfform::getWorkWSNkeepPixmapCache(WSCbool*){}
 
 
 WSCbool WSCfform::getNoRefresh(){
@@ -476,6 +505,9 @@ void WSCfform::setChildren(WSClistData& src){
 WSClistData& WSCfform::getChildren(){
   return _children;
 }
+WSClistData* WSCfform::getChildrenPtr(){
+  return &_children;
+}
 void WSCfform::onVisibleChange(WSCbool fl){
   long need_resize_event = 0;
   if (fl != False && getowndev() == NULL){
@@ -508,7 +540,9 @@ void WSCfform::onParentVisibleChange(WSCbool fl){
 }
 void WSCfform::onResize(WSCrect* area){
   WSCbool fl = False;
+#ifndef WS_EMBED
   if (getPropertyEditMode() == False){
+#endif
     _x = area->x;
     _y = area->y;
     if (_w != area->width){
@@ -519,7 +553,9 @@ void WSCfform::onResize(WSCrect* area){
       _h = area->height;
       fl = True;
     }
+#ifndef WS_EMBED
   }
+#endif
   if (fl != False){
 #if 0
     WSDdev* dev = getowndev();
@@ -590,9 +626,11 @@ long WSCfform::_device_initialize(){
     dev->setValue(WSDEV_BACKCOLOR,(void*)&_back_color);
     dev->setValue(WSDEV_BACK_PIXMAP,(void*)&_bp_no);
     dev->setValue(WSDEV_PIXMAP_STYLE,(void*)&pstyle); 
-    dev->setEnableEventBit(WSEV_EXPOSE_BIT | WSEV_RESIZE_BIT | WSEV_MOUSE_PRESS_BIT | WSEV_MOUSE_MOVE_BIT | WSEV_MOUSE_RELEASE_BIT);
+    dev->setEnableEventBit(WSEV_EXPOSE_BIT | WSEV_RESIZE_BIT | WSEV_MOUSE_PRESS_BIT | WSEV_MOUSE_MOVE_BIT | WSEV_MOUSE_RELEASE_BIT | WSEV_MOUSE_IN_BIT);
   }else{
+#ifndef WS_EMBED
     if (getPropertyEditMode() == False){
+#endif
 //      char val = WS_FULL_TITLE;
 //      if (_title_bar == WS_NO_MANAGE){
       {
@@ -602,7 +640,9 @@ long WSCfform::_device_initialize(){
 //      }else{
 //        _dev->setValue(WSDEV_TITLE_BAR,(void*)&val);
 //      }
+#ifndef WS_EMBED
     }
+#endif
 
 //    if (_title_bar == WS_NO_MANAGE && getPropertyEditMode() == False){
         WSCbool fl = True;
@@ -631,6 +671,7 @@ long WSCfform::_device_initialize(){
 
     dev->setEnableEventBit(WSEV_EXPOSE_BIT | WSEV_RESIZE_BIT |
                            WSEV_MOUSE_PRESS_BIT | WSEV_MOUSE_MOVE_BIT |
+                           WSEV_MOUSE_IN_BIT |
                            WSEV_MOUSE_RELEASE_BIT |
                            WSEV_KEY_PRESS_BIT | WSEV_KEY_RELEASE_BIT);
     if (_parent != NULL){
@@ -704,12 +745,16 @@ long WSCfform::draw(){
 //printf("WSCfform::draw %s bp=%d ab=%d %d %d %d %d\n",getInstanceName(),_bp_no,absolute,ex,ey,ew,eh);
   if (_bp_no == -1){
 //printf("w=%d h=%d\n",_w,_h);
+#ifndef NO_GRADATION
     if (_grad_no == 0){
+#endif
       dev->setForeColor(_back_color);
       dev->drawFillRect(0,0,_w,_h);
+#ifndef NO_GRADATION
     }else{
       dev->drawGradation(_grad_no,_ts_color,_back_color,_bs_color,0,0,_w,_h,_grad_margin);
     }
+#endif
   }
   if (getDotMode() == True){
     dev->setForeColor(_fore_color);
@@ -797,15 +842,16 @@ void WSCfform::_exec_child_adjust_for_anchors(){
       WSClistData children = getChildren();
       long num = children.getNum();
       long i;
+      WSCbase** buf = (WSCbase**)children._data;
       for(i=0; i<num; i++){
-        WSCbase* child = (WSCbase*)children[i];
+        WSCbase* child = buf[i];
         child->_adjust_for_anchors(_w,_h);
         if (getPixmapStyle() == WS_DYNAMIC_PIXMAP){
           if (child->isNeedUpdate() != False){
-            if ((long)child->getUserData("anchor update..") == 0){
-              child->setUserData("anchor update..",(void*)1);
+            if (child->_flag_for_anchor == False){
+              child->_flag_for_anchor = True;
               child->update();
-              child->setUserData("anchor update..",(void*)0);
+              child->_flag_for_anchor = False;
             }
           }
         }
@@ -870,12 +916,12 @@ long WSCfform::redraw(){
   if (getVisible() == False){
     return WS_ERR;
   }
-  long er = dev->beginDraw(0,0,_w,_h,True);
-  if (er != WS_NO_ERR){
-    return WS_NO_ERR;
-  }
+//  long er = dev->beginDraw(0,0,_w,_h,True);
+//  if (er != WS_NO_ERR){
+//    return WS_NO_ERR;
+//  }
   dev->clearArea(0,0,_w,_h,True);
-  dev->endDraw();
+//  dev->endDraw();
   return WS_NO_ERR;
 }
 
@@ -955,7 +1001,7 @@ void WSCfform::_adjust_for_anchors(WSCushort w,WSCushort h){
       if (_anchor_bottom_use != 0){
         _anchor_top_use = False;
         _anchor_bottom_use = False;
-        clear();
+//        clear();
         if (h < _anchor_bottom + _anchor_top){
           setProperty(WSNheight,1);
         }else{
@@ -1096,10 +1142,12 @@ void _popdown_resize_frame_(){
   _resize_frame_bottom->setVisible(False);
   _resize_frame_left->setVisible(False);
   _resize_frame_right->setVisible(False);
+  WSGIappKeyboard()->resetSpecialInputWorkProc();
 }
 
 void WSCfform::_resize_proc(void* ptr){
   WSCfform* _this = (WSCfform*)ptr;
+  WSGIappKeyboard()->setSpecialInputWorkProc(_special_key_handler,_this,_this);
 
   short x,y;
   WSGIappMouse()->getMousePosition(&x,&y,_this);
@@ -1146,12 +1194,21 @@ void WSCfform::_resize_proc(void* ptr){
     }
     _resize_timer->stopTimer();
     _this->_mouse_pressed_resize = False;
+    WSCushort dw = WSGIappDev()->getWidth();
+    WSCushort dh = WSGIappDev()->getHeight();
+    if (dw < x + 5){
+      x = dw - 5;
+    } 
+    if (dh < y + 5){
+      y = dh - 5;
+    } 
     if (x < 20){
       x = 20;
     }
     if (y < 20){
       y = 20;
     }
+
     _this->setProperty(WSNwidth,x + 5);
     _this->setProperty(WSNheight,y + 5);
   }
@@ -1233,6 +1290,12 @@ void WSCfform::onMouseRelease(WSCpoint* pt){
       if (_resize_frame_top != NULL){
         _popdown_resize_frame_();
       }
+      if (x < 5){
+        x = 5;
+      }
+      if (y < 5){
+        y = 5;
+      }
       setProperty(WSNx,x-5);
       setProperty(WSNy,y-5);
       _update_only_xy();
@@ -1272,7 +1335,9 @@ void WSCfform::_update_only_xy(){
     _update_resize = False;
     execResizeEvent();
   }
-  WSCbase::update();
+  //for updating fforms WSCnwbase children..
+//  redraw();
+//  WSCbase::update();
 }
 
 
@@ -1280,6 +1345,7 @@ void WSCfform::_move_proc(void* ptr){
   WSCfform* _this = (WSCfform*)ptr;
 //printf("move_proc.. %s\n",_this->getInstanceName());
 //fflush(stdout);
+  WSGIappKeyboard()->setSpecialInputWorkProc(_special_key_handler,_this,_this);
   short x,y;
   WSGIappMouse()->getMousePosition(&x,&y);
       x -= _this->_mouse_x;
@@ -1296,6 +1362,12 @@ void WSCfform::_move_proc(void* ptr){
   _this->setProperty(WSNy,y-5);
   _this->_update_only_xy();
 #endif
+  if (x -5 < 0){
+    x = 5;
+  }
+  if (y -5 < 0){
+    y = 5;
+  }
   _move_resize_frame_(x-5,y-5,_this->_w,_this->_h);
 
   WSCulong status = WSGIappMouse()->getMouseStatus();
@@ -1424,6 +1496,12 @@ void WSCfform::_set_floated(){
   }else{
     short x,y;
     WSGIappMouse()->getMousePosition(&x,&y);
+    if (x < 0){
+      x = 5;
+    }
+    if (y < 0){
+      y = 5;
+    }
     setProperty(WSNx,x-5);
     setProperty(WSNy,y-5);
 
@@ -1516,4 +1594,70 @@ void WSCfform::internalUpdate(){
     }
   }
 }
+#ifndef NO_RESTACK_CHILD
+long WSCfform::restackChild(WSCbase* target,int order){
+  long num = _children.getNum();
+  if (order > num ){
+    order = num;
+  }
+  order = num - order;
+  long pos = -1;
+  long i;
+  for(i=0; i< num; i++){
+    WSCbase* child = (WSCbase*)_children[i];
+    if (child == target){
+      pos = i;
+      break;
+    }
+  }
+  if (pos == -1){
+    return WS_ERR;
+  }
+  if (order == pos+1){
+    return WS_NO_ERR;
+  }
+  _children.delPos(pos);
+  if (order > 0){
+    order--;
+  }
+  _children.add(target,order);
+  WSDdev* dev = getowndev();
+  if (dev != NULL){
+    WSDwindowDev* wdev = (WSDwindowDev*)dev->cast("WSDwindowDev");
+    if (wdev != NULL){
+      wdev->restackChildren();
+    }
+  }
+  return WS_NO_ERR;
+}
+#endif
 
+WSCbool WSCfform::_special_key_handler(void* ptr,WSDkeyboard* keyb,WSCbool fl){
+  WSCfform* _this = (WSCfform*)ptr;
+  long key = keyb->getKey();
+  if (key == WSK_Escape || key == WSK_Super_L || key == WSK_Super_R ||
+      key == WSK_Control_L || key == WSK_Control_R || key == WSK_Apps ||
+      key == WSK_Alt_L || key == WSK_Alt_R ||
+      key == WSK_Meta_L || key == WSK_Meta_R ){
+    if (fl == False){
+      return False;
+    }
+//    if (_this->_floated != False){
+      if (_resize_frame_top != NULL){
+        _popdown_resize_frame_();
+      }
+//    }
+    if (_move_timer != NULL){
+      _move_timer->stopTimer();
+    }
+    if (_resize_timer != NULL){
+      _resize_timer->stopTimer();
+    }
+    _this->_start_moving = False;
+    _this->_mouse_pressed = False;
+    _this->_mouse_pressed_resize = False;
+    return False;
+  }
+  return True;
+}
+#endif //NO_FFORM
