@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCvlabel.h>
@@ -43,7 +39,7 @@
 
 #ifdef _WSWINCE
 #include <stdlib.h>
-#define strdup _strdup
+//#define strdup _strdup
 #endif
 
 WSMFguiClassInitialize(WSCgrid,WSCform);
@@ -62,6 +58,7 @@ WSCgrid::WSCgrid(WSCbase* base,char* objname):
   _data_source_type = WS_DATA_SOURCE_NONE;
   _vnum = 3;
   _hnum = 3;
+  _values_num = 0;
   _vselect_pos = 0;
   _hselect_pos = 0;
   _selected = False;
@@ -170,12 +167,12 @@ void WSCgrid::setData(WSCvariant* val,long code){
     WSCstring str;
     str.setString(val->getCharPtr(),code);
     if (_data != NULL){
-      delete _data;
+      delete[] _data;
     }
     _data = WSGFstrdup(str.getString());
   }else{
     if (_data != NULL){
-      delete _data;
+      delete[] _data;
     }
     _data = WSGFstrdup((char*)val);
   }
@@ -224,11 +221,13 @@ void WSCgrid::_set_ds_instance(){
 }
 
 void WSCgrid::getWorkWSNdata(char** ptr){
+#ifndef WS_EMBED
   if (getPropertyEditMode() != False &&
       _data_source_type != WS_DATA_SOURCE_NONE){
-    *ptr = "";
+    *ptr = NULL;
     return;
   }
+#endif
 };
 
 void WSCgrid::setWorkWSNdataSource(WSCuchar){
@@ -277,22 +276,22 @@ long WSCgrid::_device_initialize(){
 
 WSCgrid::~WSCgrid(){
   if (_vline_pos != NULL){
-    delete _vline_pos;
+    delete[] _vline_pos;
   }
   if (_hline_pos != NULL){
-    delete _hline_pos;
+    delete[] _hline_pos;
   }
   if (_vline_list != NULL){
-    delete _vline_list;
+    delete[] _vline_list;
   }
   if (_hline_list != NULL){
-    delete _hline_list;
+    delete[] _hline_list;
   }
   if (_data != NULL){
-    delete _data;
+    delete[] _data;
   }
   if (_data_source_name != NULL){
-    delete _data_source_name;
+    delete[] _data_source_name;
   }
   if (_values != NULL){
     int i;
@@ -301,7 +300,7 @@ WSCgrid::~WSCgrid(){
       WSCvariant* item = _values[i];
       delete item;
     }
-    delete _values;
+    delete[] _values;
     _values = NULL;
   }
   if (_fg_colors != NULL){
@@ -423,7 +422,6 @@ long WSCgrid::getCellGeometry(WSCulong cx,WSCulong cy,short* x,short* y,
   return WS_ERR;
 }
 long WSCgrid::draw(){
-
   if (_no_refresh != False){
     WSCbase::update();
     setAbsoluteDraw(False);
@@ -448,6 +446,7 @@ long WSCgrid::draw(){
       return WS_NO_ERR;
     }
   }
+
   long err = dev->beginDraw(0,0,_w,_h,absolute);
   if (err != WS_NO_ERR){
     return WS_NO_ERR;
@@ -463,23 +462,18 @@ long WSCgrid::draw(){
   setAbsoluteDraw(False);
 
   if (_bp_color == -1){
+#ifndef NO_GRADATION
     if (_grad_no == 0){
-      dev->setForeColor(_back_color);
-      dev->drawFillRect(0,0,_w,_h);
-#ifndef WS_EMBED
-      if ((WSGIappDev()->getGuiPolicy() & WS_POLICY_AQUA) != 0 &&
-           _bp_color == -1){
-        if (_a1_color == -1){
-          WSGFgetShadowColor(_back_color,&_a1_color,&_a2_color,&_a3_color);
-        }
-        WSGFdrawStripes(dev,_w,_h,_a1_color,_a2_color,_a3_color);
-      }
 #endif
       dev->setForeColor(_back_color);
+      dev->drawFillRect(0,0,_w,_h);
+      dev->setForeColor(_back_color);
 
+#ifndef NO_GRADATION
     }else{
       dev->drawGradation(_grad_no,_ts_color,_back_color,_bs_color,0,0,_w,_h,_grad_margin);
     }
+#endif
   }
 
   if (getDotMode() == True){
@@ -494,8 +488,6 @@ long WSCgrid::draw(){
       }
     }
   }
-
-
   if (_values != NULL){
     long i;
     for(i=0; i<(long)_values_num; i++){
@@ -546,33 +538,37 @@ long WSCgrid::draw(){
 //printf("WSCgrid::draw  %d %d %d %d #%s#\n",x,y,w,h,(char*)str);
     }
   }
+
   dev->setForeColor(_fore_color);
   dev->setLineWidth(_line_width);
   dev->setLineDashType(_line_type);
   long i;
-  if (_vline_list != NULL){
-    long val = _vnum-1;
-    if (val < 0){
-      val = 0;
+  if (_line_width != 0){
+    if (_vline_list != NULL){
+      long val = _vnum-1;
+      if (val < 0){
+        val = 0;
+      }
+      for(i=0; i<val; i++){
+        dev->drawLine(0,_vline_list[i],_w,_vline_list[i]); 
+      }
     }
-    for(i=0; i<val; i++){
-      dev->drawLine(0,_vline_list[i],_w,_vline_list[i]); 
-    }
-  }
-  if (_hline_list != NULL){
-    long val = _hnum-1;
-    if (val < 0){
-      val = 0;
-    }
-    for(i=0; i<val; i++){
-      dev->drawLine(_hline_list[i],0,_hline_list[i],_h); 
-//printf("WSCgrid::drawline x=%d i=%d\n",_hline_list[i],i);
+    if (_hline_list != NULL){
+      long val = _hnum-1;
+      if (val < 0){
+        val = 0;
+      }
+      for(i=0; i<val; i++){
+        dev->drawLine(_hline_list[i],0,_hline_list[i],_h); 
+      }
     }
   }
 
   if (_shadow_type == WS_SHADOW_TRANS){
-    long lt = _line_width /2;
-    dev->drawRect(lt,lt,_w - lt*2,_h - lt*2);
+    if (_line_width != 0){
+      long lt = _line_width /2;
+      dev->drawRect(lt,lt,_w - lt*2,_h - lt*2);
+    }
   }else{
     WSGFdrawShadow(dev,_shadow_type,_shadow_thick,_fore_color,_ts_color,_bs_color,_back_color,0,0,_w,_h);
   }
@@ -594,7 +590,7 @@ void WSCgrid::_adjust_value(){
         WSCvariant* val = _values[i];
         delete val;
       }
-      delete _values;
+      delete[] _values;
       _values = NULL;
       _values_num = 0;
     }
@@ -608,28 +604,41 @@ void WSCgrid::_adjust_value(){
       delete _bg_colors;
       _bg_colors = NULL;
     }
-  }
-
-  WSCstring in(_data);
-  for(i=0; i<num; i++){
-    WSCvariant* val = new WSCvariant;
-    WSCstring item = in.getWord(i,",");
-    val->setValue((char*)item);
+    WSCstring in(_data);
+    for(i=0; i<num; i++){
+      WSCvariant* val = new WSCvariant;
+      WSCstring item = in.getWord(i,",");
+      val->setValue((char*)item);
 //printf("WSCgrip::_adjust_value %d #%s#\n",i,(char*)(*val));
-    _values[i] = val;
+      _values[i] = val;
+    }
+    if (_if_status != NULL){
+      delete _if_status;
+      _if_status = NULL; 
+    }
+  }else{
+    WSCstring in(_data);
+    long num2 = in.getWords(",");
+    for(i=0; i<num; i++){
+      if (i<num2){
+        WSCstring item = in.getWord(i,",");
+        _values[i]->setValue((char*)item);
+      }else{
+        _values[i]->setValue("");
+      }
+    }
   }
 }
 
 void WSCgrid::_adjust_line(){
   long i;
   if (_vline_list != NULL){
-    delete _vline_list;
+    delete[] _vline_list;
     _vline_list = NULL;
   }
   if (_vnum <1){
     _vnum = 1;
   }
-
   _vline_list = new long[_vnum];
   long num = _vnum -1;
   if (num < 0){
@@ -649,7 +658,7 @@ void WSCgrid::_adjust_line(){
   }
   _vline_list[_vnum-1] =0;
   if (_hline_list != NULL){
-    delete _hline_list;
+    delete[] _hline_list;
   }
   if (_hnum <1){
     _hnum = 1;
@@ -691,7 +700,11 @@ static long table[3][3];
 
 long WSCgrid::setCellForeColor(WSCulong x,WSCulong y,short color){
   if (_need_update != False){
-    _value_update();
+    long num_back = _values_num;
+    long num = _vnum * _hnum;
+    if (num_back != num){
+      _value_update();
+    }
   }
   if (_fg_colors == NULL){
     _fg_colors = new short[_values_num];
@@ -710,7 +723,11 @@ long WSCgrid::setCellForeColor(WSCulong x,WSCulong y,short color){
 }
 long WSCgrid::setCellForeColor(WSCulong x,WSCulong y,char* color){
   if (_need_update != False){
-    _value_update();
+    long num_back = _values_num;
+    long num = _vnum * _hnum;
+    if (num_back != num){
+      _value_update();
+    }
   }
   if (_fg_colors == NULL){
     _fg_colors = new short[_values_num];
@@ -731,7 +748,11 @@ long WSCgrid::setCellForeColor(WSCulong x,WSCulong y,char* color){
 
 long WSCgrid::setCellBackColor(WSCulong x,WSCulong y,short color){
   if (_need_update != False){
-    _value_update();
+    long num_back = _values_num;
+    long num = _vnum * _hnum;
+    if (num_back != num){
+      _value_update();
+    }
   }
   if (_bg_colors == NULL){
     _bg_colors = new short[_values_num];
@@ -750,7 +771,11 @@ long WSCgrid::setCellBackColor(WSCulong x,WSCulong y,short color){
 }
 long WSCgrid::setCellBackColor(WSCulong x,WSCulong y,char* color){
   if (_need_update != False){
-    _value_update();
+    long num_back = _values_num;
+    long num = _vnum * _hnum;
+    if (num_back != num){
+      _value_update();
+    }
   }
   if (_bg_colors == NULL){
     _bg_colors = new short[_values_num];
@@ -846,7 +871,7 @@ void WSCgrid::_input_focus_lose(WSCbase* ifield){
 }
 void WSCgrid::_input_fix_work(WSCbase* ifield){
   long key = WSGIappKeyboard()->getKey();
-  if (key == WSK_Return){
+  if (key == WSK_Return || key == WSK_KP_Enter){
     WSCgrid* _this = (WSCgrid*)ifield->getUserData("pgrid");
     WSCvariant val = ifield->getProperty(WSNlabelString);
     long cx = (long)ifield->getUserData("cx");
@@ -871,7 +896,7 @@ void WSCgrid::onInputFixed(WSCvariant* val,long cx,long cy){
       tmp << "," << *(_values[i]);
     }
     if (_data != NULL){
-      delete _data;
+      delete[] _data;
       _data = NULL;
     }
     _data = WSGFstrdup((char*)tmp);
@@ -1025,8 +1050,8 @@ long WSCgrid::setItem(WSCulong hpos,WSCulong vpos,WSCvariant val){
     }
     tmp << *(_values[_values_num -1]);
     if (strcmp((char*)tmp,_data)){
-      delete _data;
-      _data = strdup((char*)tmp);
+      delete[] _data;
+      _data = WSGFstrdup((char*)tmp);
     }
   }
   return WS_ERR;
