@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,17 +14,14 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
+#ifndef NO_PSDRAW
 #include <WSCprform.h>
 #include <WSCclassInformation.h>
 #include <WSCdevice.h>
@@ -133,6 +130,16 @@ long WSCprform::draw(){
       }
   }
 
+  if (_update_resize != False){
+    _update_resize = False;
+    execResizeEvent();
+  }
+
+  if (_geometry_changed != False){
+dbprintf("WSCprform::draw %s:%d %s call _exec_child_adjust_for_anchors()..\n",__FILE__,__LINE__,getInstanceName());
+    _exec_child_adjust_for_anchors();
+  }
+
   long err = dev->beginDraw(0,0,_w,_h,absolute);
   if (err != WS_NO_ERR){
     return WS_NO_ERR;
@@ -148,23 +155,18 @@ long WSCprform::draw(){
   setAbsoluteDraw(False);
 
   if (_bp_color == -1){
+#ifndef NO_GRADATION
     if (_grad_no == 0){
-      dev->setForeColor(_back_color);
-      dev->drawFillRect(0,0,_w,_h);
-#ifndef WS_EMBED
-      if ((WSGIappDev()->getGuiPolicy() & WS_POLICY_AQUA) != 0 &&
-           _bp_color == -1){
-        if (_a1_color == -1){
-          WSGFgetShadowColor(_back_color,&_a1_color,&_a2_color,&_a3_color);
-        }
-        WSGFdrawStripes(dev,_w,_h,_a1_color,_a2_color,_a3_color);
-      }
 #endif
       dev->setForeColor(_back_color);
+      dev->drawFillRect(0,0,_w,_h);
+      dev->setForeColor(_back_color);
 
+#ifndef NO_GRADATION
     }else{
       dev->drawGradation(_grad_no,_ts_color,_back_color,_bs_color,0,0,_w,_h,_grad_margin);
     }
+#endif
   }
 
   if (getDotMode() == True){
@@ -325,3 +327,4 @@ long WSCprform::prdraw(){
 
   return WS_NO_ERR;
 }
+#endif //NO_PSDRAW
