@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCvarrow.h>
@@ -94,9 +90,9 @@ long WSCvarrow::draw() {
     return WS_NO_ERR;
   }
 
-  WSCstring str;
-  str.setString(_label_string);
-  str.replaceString("\\n", "\n", 0);
+//  WSCstring str;
+//  str.setString(_label_string);
+//  str.replaceString("\\n", "\n", 0);
 
   long err = dev->beginDraw(x, y, w, h, absolute);
   if (err != WS_NO_ERR){
@@ -119,11 +115,15 @@ long WSCvarrow::draw() {
       dev->setForeColor(_back_color);
     }
   }
+#ifndef NO_GRADATION
   if (_grad_no == 0){
+#endif
     dev->drawFillRect(0, 0, w, h);
+#ifndef NO_GRADATION
   }else{
     dev->drawGradation(_grad_no,_ts_color,_back_color,_bs_color,0, 0, w, h,_grad_margin);
   }
+#endif
 
   short py = st;
   if (_press_state == True) {
@@ -139,11 +139,16 @@ long WSCvarrow::draw() {
           _ts_color, _bs_color,_back_color, 0, 0, w, h);
     }
   }
-
+#ifndef WS_EMBED
   if (getSensitive() == False ||
-      (getPropertyEditMode() != False && (long)getUserData(WS_DET) == 0)){
+      (getPropertyEditMode() != False && _sensitive_work == False)){
     dev->setHatchPattern(8);
   }
+#else
+  if (getSensitive() == False){
+    dev->setHatchPattern(8);
+  }
+#endif
 
   if (_rv_fl != False){
     if (bldraw && _bl_type != WS_BACK && _tw_fl == True) {
