@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCvimage.h>
@@ -51,8 +47,10 @@ WSCvimage::WSCvimage(WSCbase* base, char* objname):
   _margin_left = 2;
   _margin_right = 2;
   _label_pixmap = -1;
+#ifndef NO_GRADATION
   _grad_no = 0;
   _grad_margin = 0;
+#endif
   _fit = False;
 
   WSMFpropertyCreateStart
@@ -70,11 +68,17 @@ WSCvimage::WSCvimage(WSCbase* base, char* objname):
     WSMFpropertyCreate(WSNmarginRight,  WSCuchar,  _margin_right,WSSmarginRight);
     WSMFpropertyCreate(WSNlabelPixmap,  short,     _label_pixmap,WSSlabelPixmap);
 
+#ifndef NO_GRADATION
     WSMFpropertyCreate(WSNgradation,   WSCuchar,   _grad_no,WSSgradation);
       WSMFpropertySetSelection( WSRgradation, WSRgradationD);
     WSMFpropertyCreate(WSNgradationMargin,   WSCuchar,   _grad_margin,WSSgradationMargin);
+#endif
     WSMFpropertyCreate(WSNfit,   WSCbool,    _fit, WSSfit);
     WSMFpropertySetSelection(WSRbool3, WSRbool3D);
+
+    WSMFpropertyCreate(WSNkeepPixmapCache,WSCbool,_keep_pixmap_cache,WSSkeepPixmapCache );
+      WSMFpropertySetSelection(WSRbool4,WSRbool4D);
+
     
     WSMFpropertyDelete(WSNblinkRate);
     WSMFpropertyDelete(WSNblinkColor);
@@ -96,9 +100,12 @@ WSMFproperty(WSCvimage, WSNmarginBottom,WSCuchar,_margin_bottom, 2             )
 WSMFproperty(WSCvimage, WSNmarginLeft,  WSCuchar,_margin_left,   2             );
 WSMFproperty(WSCvimage, WSNmarginRight, WSCuchar,_margin_right,  2             );
 WSMFproperty(WSCvimage, WSNlabelPixmap,  short,   _label_pixmap,  -1  );
+#ifndef NO_GRADATION
 WSMFproperty(WSCvimage, WSNgradation,    WSCuchar, _grad_no,  0  );
 WSMFproperty(WSCvimage, WSNgradationMargin,    WSCuchar, _grad_margin,  0  );
+#endif
 WSMFproperty(WSCvimage, WSNfit,    WSCbool, _fit,  False  );
+WSMFproperty(WSCvimage, WSNkeepPixmapCache, WSCbool,  _keep_pixmap_cache,True );
 
 void WSCvimage::setWorkWSNshadowThickness(WSCuchar){}
 void WSCvimage::getWorkWSNshadowThickness(WSCuchar*){}
@@ -121,12 +128,16 @@ void WSCvimage::setWorkWSNmarginRight(WSCuchar){}
 void WSCvimage::getWorkWSNmarginRight(WSCuchar*){}
 void WSCvimage::setWorkWSNlabelPixmap(short){}
 void WSCvimage::getWorkWSNlabelPixmap(short*){}
+#ifndef NO_GRADATION
 void WSCvimage::setWorkWSNgradation(WSCuchar no){}
 void WSCvimage::getWorkWSNgradation(WSCuchar*){}
 void WSCvimage::setWorkWSNgradationMargin(WSCuchar no){}
 void WSCvimage::getWorkWSNgradationMargin(WSCuchar*){}
+#endif
 void WSCvimage::setWorkWSNfit(WSCbool fl){}
 void WSCvimage::getWorkWSNfit(WSCbool*){}
+void WSCvimage::setWorkWSNkeepPixmapCache(WSCbool){}
+void WSCvimage::getWorkWSNkeepPixmapCache(WSCbool*){}
 
 WSCvimage::~WSCvimage() {
 }
@@ -207,11 +218,15 @@ long WSCvimage::draw(){
   setAbsoluteDraw(False);
   if (shadowtype != WS_SHADOW_TRANS) {
     dev->setForeColor(bgno);
+#ifndef NO_GRADATION
     if (_grad_no == 0){
+#endif
       dev->drawFillRect( 0, 0, w, h);
+#ifndef NO_GRADATION
     }else{
       dev->drawGradation(_grad_no,tsno,bgno,bsno,0, 0, w, h,_grad_margin);
     }
+#endif
   }
 
   WSDimage* image;
@@ -222,12 +237,21 @@ long WSCvimage::draw(){
     }else{
       WSCushort iw = image->getImageWidth();
       WSCushort ih = image->getImageHeight();
-      if ( ((double)iw /(double)ih > (double)ww/(double)hh) ){
-        WSCushort nh = (WSCushort)((double)ww * ih / iw);
+//      if ( ((double)iw /(double)ih > (double)ww/(double)hh) ){
+      if (iw * hh > ww * ih){
+//        WSCushort nh = (WSCushort)((double)ww * ih / iw);
+        if (iw == 0){
+          iw = 1;
+        }  
+        WSCushort nh = (WSCushort)(ww * ih / iw);
         dev->drawStretchedImage (xx, yy + (hh - nh)/2, ww, nh, image);
 
       }else{
-        WSCushort nw = (WSCushort)((double)hh * iw / ih);
+//        WSCushort nw = (WSCushort)((double)hh * iw / ih);
+        if (ih == 0){
+          ih = 1;
+        }  
+        WSCushort nw = (WSCushort)(hh * iw / ih);
         dev->drawStretchedImage (xx + (ww - nw)/2, yy, nw, hh, image);
       }
      
