@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCvlineGraph.h>
@@ -52,7 +48,7 @@ WSCvlineGraph::WSCvlineGraph(WSCbase* base, char* objname):
   _stippled = False;
   _hatch_pattern = 0;
   _hatch_color = WS_DF_NWBACKCOLOR;
-  _data_source_name = WSGFstrdup("");
+  _data_source_name = NULL;
   _data_source_type = WS_DATA_SOURCE_NONE;
   _need_ds_update = False;
   _bg_blink_color = WS_DF_NWBACKCOLOR;
@@ -94,10 +90,15 @@ WSMFproperty( WSCvlineGraph, WSNstippled,  WSCbool,_stippled,     False);
 WSMFproperty( WSCvlineGraph, WSNhatchPattern, WSCuchar,   _hatch_pattern,0    );
 WSMFproperty( WSCvlineGraph, WSNhatchColor,  short,  _hatch_color,WS_DF_NWBACKCOLOR);
 WSMFproperty( WSCvlineGraph, WSNdataSource,  WSCuchar, _data_source_type,WS_DATA_SOURCE_NONE);
-WSMFproperty( WSCvlineGraph, WSNdataSourceName,  char*, _data_source_name,WSGFstrdup(""));
+WSMFproperty( WSCvlineGraph, WSNdataSourceName,  char*, _data_source_name,NULL);
 WSMFproperty( WSCvlineGraph, WSNbackBlinkColor,    short,        _bg_blink_color, WS_DF_NWBACKCOLOR);
 
 void WSCvlineGraph::_set_ds_instance(){
+  char* val = _data_source_name;
+  if (val == NULL){
+    _need_ds_update = True;
+    return;
+  }
   WSCbase* serv = WSGIappObjectList()->searchInstance("WSCbase", _data_source_name);
   if (serv != NULL){
     setDataSourceServer(serv);
@@ -173,11 +174,13 @@ void WSCvlineGraph::getWorkWSNbackBlinkColor(short*){ }
 
 WSCvlineGraph::~WSCvlineGraph() {
   if (_values != NULL){
-    delete _values;
+    delete[] _values;
     _values = NULL;
   }
   if (_data_source_name != NULL){
-    delete _data_source_name;
+    delete[] _data_source_name;
+    _data_source_name = NULL;
+    
   }
 }
 
