@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <unix/unistd.h>
 #include <btron/WSDbtronAppDev.h>
@@ -32,21 +28,35 @@
 #include <WSCimageSet.h>
 #include <WSDdev.h>
 #include <btron/WSDbtronDraw.h>
+#include <WSCbaseList.h>
 #ifdef TE
 #include <btron/unixemu.h>
 //#include <util/tmonitor.h>
 #endif
 
+#ifdef DEBUG
+#include <util/tmonitor.h>
+#endif
+
+//#define TDBG
+
 extern int main1(int argc,char** argv);
 
-int main(int argc,char** argv){
+int WSGFwsMain(int argc,char** argv){
 //#ifdef TE
   chdir("/SYS");
 //#else
 //  chdir("/SYS/WORK");
 //#endif
+#ifdef TDBG
+  printf("main start:\n");
+  WSMFtrace("main start:\n");
+  WSGFsetTimePoint();
+#endif
 
-//  tm_monitor();
+#ifdef DEBUG
+  tm_monitor();
+#endif
 
 #if 0
   LINK    lnk;
@@ -54,6 +64,16 @@ int main(int argc,char** argv){
   dopn_dat(&lnk);
 #endif
 //    W er = opendbox(&lnk, USER_DATA, 0x2000);
+#ifdef _NOTSH
+extern int WSGFdeviceFrameBufferInitialize();
+  WSGFdeviceFrameBufferInitialize();
+#endif
+
+#ifdef TDBG
+  printf("main init point1.:%d\n",WSGFdiffTimePoint());
+  WSMFtrace("main init point1.:%d\n",WSGFdiffTimePoint());
+  WSGFsetTimePoint();
+#endif
 
   _dbprintf_init();
 dbprintf("Main.cpp: MAIN() %s:%d start.\n",__FILE__,__LINE__);
@@ -63,11 +83,30 @@ dbprintf("Main.cpp: MAIN() %s:%d start.\n",__FILE__,__LINE__);
   if (argc != 0){
     btronApp->setInitPrm(argc,argv);
   }
+#ifdef TDBG
+  printf("main init point2.:%d\n",WSGFdiffTimePoint());
+  WSMFtrace("main init point2.:%d\n",WSGFdiffTimePoint());
+  WSGFsetTimePoint();
+#endif
 
 dbprintf("Main.cpp: main() %s:%d here1.\n",__FILE__,__LINE__);
   btronApp->execInitHandler();
 dbprintf("Main.cpp: main() %s:%d here2.\n",__FILE__,__LINE__);
+
+#ifdef TDBG
+  printf("main init done.:%d\n",WSGFdiffTimePoint());
+  WSMFtrace("main init done.:%d\n",WSGFdiffTimePoint());
+  WSGFsetTimePoint();
+#endif
+  WSGIappObjectList()->execInitialize();
   btronApp->handleEvents();
 dbprintf("Main.cpp: main() %s:%d done.\n",__FILE__,__LINE__);
+
   return 0;
 }
+#ifndef NO_MAIN_FUNCTION
+int main(int argc,char** argv){
+  return WSGFwsMain(argc,argv);
+}
+#endif
+
