@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <basic.h>
 #include <assert.h>
@@ -84,38 +80,72 @@ void WSGFdeviceInitialize(){
 dbprintf("WSGFdeviceInitialize.. started..\n");
   static WSCbool initialized = False;
   if (initialized == False){
+#ifndef NO_FILE_SERIALIZE
+extern void WSGFinitFS();
+    WSGFinitFS();
+#endif
+extern void WSGFunixEnvInit();
+    WSGFunixEnvInit();
+#ifndef TE
+extern void WSGFbtronExecuteInit();
+    WSGFbtronExecuteInit();
+#endif
+#ifndef NO_FILE_IO
+extern void WSGFinitializeUnixFile();
+    WSGFinitializeUnixFile();
+#endif
+#ifndef NO_WSDMUTEX
+extern void WSGFinitializeUnixMutex();
+    WSGFinitializeUnixMutex();
+#endif
+#ifndef NO_WSDSEMAPHORE
+extern void WSGFinitializeUnixSemaphore();
+    WSGFinitializeUnixSemaphore();
+#endif
+#ifndef NO_WSDTHREAD
+extern void WSGFinitializeUnixThread();
+    WSGFinitializeUnixThread();
+#endif
+#ifndef NO_FILE_IO
 extern WSDfile* _unix_fp_chandler();
     WSDfile::setCreateInstanceHandler((void*)_unix_fp_chandler);
+#endif
 //extern WSDdragDrop* _xdrag_drop_create_handler_();
 //    WSDdragDrop::setCreateHandler(_xdrag_drop_create_handler_);
 //extern WSDmessage* _xclient_msg_create();
 //    WSDmessage::setCreateHandler(_xclient_msg_create);
 extern WSDmouse* _btronMouse_create_handler();
     WSDmouse::setCreateInstanceHandler(_btronMouse_create_handler);
+#ifndef NO_TIMER
 extern WSDprivateTimer* _btronPrivatetimer_create();
     WSDprivateTimer::setCreateInstanceHandler((void*)_btronPrivatetimer_create);
+#endif
 #ifndef NO_SOCKET
 extern WSDsocket* _btronSocket_create_handler();
     WSDsocket::setCreateHandler(_btronSocket_create_handler);
 #endif
+#ifndef NO_TIMER
 extern WSDtimer* _btronTimer_create_handler();
     WSDtimer::setCreateInstanceHandler(_btronTimer_create_handler);
-
+#endif
 extern WSDdev* _wsdbtronFormdev_init_();
     WSGIappDevice()->setCreateHandler("formDev",_wsdbtronFormdev_init_);
 extern WSDdev* _wsdbtronWindev_init_();
     WSGIappDevice()->setCreateHandler("windowDev",_wsdbtronWindev_init_);
 extern WSDdev* _wsdbtronNwdev_init_();
     WSGIappDevice()->setCreateHandler("nwDev",_wsdbtronNwdev_init_);
+#ifndef NO_SCR
 extern WSDdev* _wsdbtronScrdev_init_();
     WSGIappDevice()->setCreateHandler("scrFrameDev",_wsdbtronScrdev_init_);
+#endif
 #ifndef NO_MWINDEV
 extern WSDdev* _wsdbtronmwindev_init_();
     WSGIappDevice()->setCreateHandler("mwindowDev",_wsdbtronmwindev_init_);
 #endif
+#ifndef NO_KEYBOARD
 extern WSDkeyboard* _btronKeyboard_create_handler();
     WSDkeyboard::setCreateInstanceHandler(_btronKeyboard_create_handler);
-
+#endif
     WSDappDev::setCreateInstanceHandler(_btronAppDev_create_handler);
 extern WSDcodeConvert* _code_convert_create_handler_();
     WSDcodeConvert::_set_code_convert_handler(_code_convert_create_handler_);
@@ -453,6 +483,9 @@ dbprintf("WSDbtronAppDev::handleEvents start dispatchEvent().   XXXXXXXX\n");
     dispatchEvent();
 dbprintf("WSDbtronAppDev::handleEvents start clearDeleteList(). XXXXXXXX\n");
     WSGIappDevice()->clearDeleteList();
+    if (_exit_event_loop != False){
+      break;
+    }
   }
   return WS_NO_ERR;
 }
@@ -607,6 +640,7 @@ void _btn_release(W window,WEVENT* ev){
       WSDbtronFormDev::isValidInstance(tdev) == False){
     WSGIbtronAppDev()->setBtnPressGrabTarget(NULL);
     if (is_own_window(window) == False){
+dbprintf("_btn_release(W=0x%x) is not own.\n",window);
       return;
     }
     W ret = wget_dat(window,(W*)&dev);
@@ -733,6 +767,9 @@ void recvmsg(){
      ret = rcv_msg(MM_TYPE3,msg,sizeof(MESSAGE)+ sizeof(_read_event_send_type)-sizeof(void*),NOWAIT);
   }
 #endif
+  if (ret <0){
+     ret = rcv_msg(MM_TYPE4,msg,sizeof(MESSAGE),NOWAIT);
+  }
   if (ret <0){
   }else if (msg->msg_type == MS_TYPE1){ 
     _accept_send_type* data = NULL;
@@ -743,6 +780,7 @@ void recvmsg(){
     }
 dbprintf("WSDxappDev::_dispatchEvent %s:%d APPL5x done\n",__FILE__,__LINE__);
     delete data;
+#ifndef NO_TIMER
   }else if (msg->msg_type == MS_TYPE2){ 
 dbprintf("WSDxappDev::_dispatchEvent %s:%d APPL5x start\n",__FILE__,__LINE__);
     WSDbtronPrivateTimer* timer = NULL;
@@ -753,6 +791,7 @@ dbprintf("WSDxappDev::_dispatchEvent %s:%d APPL5x start\n",__FILE__,__LINE__);
       timer->_timer_callback(tid);
     }
 dbprintf("WSDxappDev::_dispatchEvent %s:%d APPL5x2 done\n",__FILE__,__LINE__);
+#endif
   }else if (msg->msg_type == MS_TYPE3){ 
 #ifndef TE
 dbprintf("WSDxappDev::_dispatchEvent %s:%d APPL5x2 start\n",__FILE__,__LINE__);
@@ -771,12 +810,16 @@ dbprintf("WSDxappDev::_dispatchEvent %s:%d APPL5x start\n",__FILE__,__LINE__);
 #ifndef NO_WSDTHREAD
     WSDbtronThread* th = NULL;
     memcpy(&th,msg->msg_body.ANYMSG.msg_str,sizeof(W));
+    void* data = NULL;
+    memcpy(&data,&msg->msg_body.ANYMSG.msg_str[sizeof(W)*2],sizeof(void*));
+
     if (th->getCallbackFunction() != NULL){
-//printf("client message !! here! execute exec_read\n");
+//printf("client message !! here! execute thread callback\n");
       void(*fn)(WSDbtronThread*,void*) =
            (void(*)(WSDbtronThread*,void*))th->getCallbackFunction();
       if (fn != NULL){
-        fn(th,th->getCallbackData());
+//        fn(th,th->getCallbackData());
+        fn(th,data);
       }
     }
 #endif
@@ -818,8 +861,9 @@ long WSDbtronAppDev::_dispatchEvent(WSCbool wait){
 //    w |= (NOMSG | RAWEVT | DRGEVT);
     w |= DRGEVT;
   }
-
+dbprintf("before wget_evt\n");
   W ret = wget_evt(&ev,w);
+dbprintf("after wget_evt\n");
   W ty;
 
 //if (ret != EV_NULL){ dbprintf("EV=0x%x\n",ret);
@@ -997,6 +1041,7 @@ dbprintf("default.. wnd=0x%x\n",ev.s.wid);
  
       break;
     }
+#ifndef NO_KEYBOARD
     case EV_KEYDWN:
 //printf("EV_KEYDWN.. key=0x%x code=0x%x status=0x%x\n",ev.s.cmd,ev.s.wid,ev.s.stat);
       WSGIbtronKeyboard()->deliverKeyEvent(&ev,True);
@@ -1015,6 +1060,7 @@ dbprintf("default.. wnd=0x%x\n",ev.s.wid);
 //      }
       break;
     }
+#endif
     case EV_BUTUP:{
 dbprintf("EV_BUTUP.. wnd=0x%x\n",ev.s.wid);
       WSDbtronFormDev* mouse_target = _mouse_target;
@@ -1032,6 +1078,7 @@ dbprintf("EV_BUTUP.. wnd=0x%x\n",ev.s.wid);
 //printf("EV_BUTUP.. mouse_target=NULL! wnd=0x%x\n",mouse_target);
       }
       if (mouse_target != NULL){
+//printf("EV_BUTUP.. mouse_target=0x%x\n",mouse_target);
         mouse_target->_evh_btn_release(&ev);
 #if 0
         long state = WSGIappMouse()->getStatus();
@@ -1042,6 +1089,7 @@ dbprintf("EV_BUTUP.. wnd=0x%x\n",ev.s.wid);
         }
 #endif
       }else{
+//printf("EV_BUTUP.. _btn_release.. wid=%d\n",ev.s.wid);
         _btn_release(ev.s.wid,&ev);
       }
       break;
@@ -1236,7 +1284,7 @@ GID WSDbtronAppDev::getNewMemGid(WSCulong w,WSCulong h,UB** ptr){
   bmp.bounds.c.right = w;
   bmp.bounds.c.top = 0;
   bmp.bounds.c.bottom = h;
-  bmp.baseaddr[0] = (UB*)malloc(bmp.rowbytes * h);
+  bmp.baseaddr[0] = new UB[bmp.rowbytes * h];
   *ptr = bmp.baseaddr[0];
   return gopn_mem(NULL,&bmp,(B*)cspec);
 }
