@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <basic.h>
 #include <WScom.h>
@@ -47,6 +43,9 @@ void _btronColor_init(){
 //  XSetErrorHandler(_ehandler);
   WSGIappColorSet()->setCreateHandler(_btronColor_create_handler);
   WSGFinitUnixFileSystem();
+#ifndef NO_FILE_IO
+  WSGFinitializeUnixFile();
+#endif
 dbprintf("_btronColor_init executed..\n");
 };
 
@@ -68,7 +67,10 @@ long WSDbtronColor::initialize(){
   if (cname == NULL){
     cname = "black";
   }
-  long rval = 0,gval = 0,bval = 0;
+  WSCbool alpha = False;
+  int aval = 255;
+  int rval = 0,gval = 0,bval = 0;
+
   if(cname[0] == '#'){
     long slen = strlen(cname);
     if(slen == 4){
@@ -76,13 +78,30 @@ long WSDbtronColor::initialize(){
       rval = rval *16;
       gval = gval *16;
       bval = bval *16;
+    }else if(slen == 5){
+      sscanf( cname,"#%1x%1x%1x%1x",&rval,&gval,&bval,&aval);
+      rval = rval *16;
+      gval = gval *16;
+      bval = bval *16;
+      aval = aval *16;
+      alpha = True;
     }else if(slen == 7){
       sscanf( cname,"#%2x%2x%2x",&rval,&gval,&bval);
+    }else if(slen == 9){
+      sscanf( cname,"#%2x%2x%2x%2x",&rval,&gval,&bval,&aval);
+      alpha = True;
     }else if(slen == 13){
       sscanf( cname,"#%4x%4x%4x",&rval,&gval,&bval);
       rval = rval /256;
       gval = gval /256;
       bval = bval /256;
+    }else if(slen == 17){
+      sscanf( cname,"#%4x%4x%4x%4x",&rval,&gval,&bval,&aval);
+      rval = rval /256;
+      gval = gval /256;
+      bval = bval /256;
+      aval = aval /256;
+      alpha = True;
     }else{
       //black...
     }
@@ -112,7 +131,18 @@ long WSDbtronColor::initialize(){
       i++ ;
     }
   }
+#ifdef _NOTSH
+  long val = 0;
+  if (alpha == False){
+    val |= 0xff000000;
+    _a = 0xff;
+  }else{
+    val |= aval << 24;
+    _a = aval;
+  }
+#else
   long val = 0x10000000;
+#endif
   val |= rval << 16;
   val |= gval << 8;
   val |= bval;
