@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <basic.h>
 #include <btron/WSDbtronAppDev.h>
@@ -542,6 +538,17 @@ long WSDbtronDraw::drawLine(short x1,short y1,short x2,short y2){
     return WS_ERR;
   }
 //  PAT pat = {0,16,16,_fg_value,_fg_value,FILL100};
+#ifdef _NOTSH
+extern int WSGFdeviceDrawLine(int gid,short x1,short y1,short x2,short y2,
+                              unsigned int fg,int lw,int type);
+  if (_fdev != NULL && _fdev->haveWindow() == False){
+    short dx,dy;
+    _fdev->getAbstPos(&dx,&dy);
+     WSGFdeviceDrawLine(_gid,x1+dx,y1+dy,x2+dx,y2+dy,_fg_value,_line_width ,_line_type_val);
+  }else{
+     WSGFdeviceDrawLine(_gid,x1,y1,x2,y2,_fg_value,_line_width ,_line_type_val);
+  }
+#else
   PAT pat;
   pat.spat.kind = 0;
   pat.spat.hsize = 8;
@@ -565,6 +572,7 @@ long WSDbtronDraw::drawLine(short x1,short y1,short x2,short y2){
     p2.y = y2;
   }
   gdra_lin(_gid,p1,p2,(_line_type_val<<8)+_line_width,&pat,G_STORE);
+#endif
   return WS_NO_ERR;
 }
 long WSDbtronDraw::drawLines(WSCpoint* points,long num){
@@ -575,6 +583,28 @@ long WSDbtronDraw::drawLines(WSCpoint* points,long num){
     return WS_ERR;
   }
 //  PAT pat = {0,16,16,_fg_value,_fg_value,FILL100};
+#ifdef _NOTSH
+extern int WSGFdeviceDrawLine(int gid,short x1,short y1,short x2,short y2,
+                              unsigned int fg,int lw,int type);
+
+  if (_fdev != NULL && _fdev->haveWindow() == False){
+    short dx,dy;
+    _fdev->getAbstPos(&dx,&dy);
+    long i;
+    for(i=0; i<num-1;i++){
+      WSGFdeviceDrawLine(_gid,points[i].x+dx,points[i].y+dy,
+                              points[i+1].x+dx, points[i+1].y+dy,
+                          _fg_value,_line_width ,_line_type_val);
+    }
+  }else{
+    long i;
+    for(i=0; i<num-1;i++){
+      WSGFdeviceDrawLine(_gid,points[i].x,points[i].y,
+                              points[i+1].x, points[i+1].y,
+                          _fg_value,_line_width ,_line_type_val);
+    }
+  }
+#else
   PAT pat;
   pat.spat.kind = 0;
   pat.spat.hsize = 8;
@@ -615,6 +645,7 @@ long WSDbtronDraw::drawLines(WSCpoint* points,long num){
       gdra_lin(_gid,p1,p2,(_line_type_val<<8)+_line_width,&pat,G_STORE);
     }
   }
+#endif
   return WS_NO_ERR;
 }
 long WSDbtronDraw::drawRect(short x,short y,WSCushort w,WSCushort h){
@@ -625,6 +656,19 @@ long WSDbtronDraw::drawRect(short x,short y,WSCushort w,WSCushort h){
     return WS_ERR;
   }
 //  PAT pat = {0,16,16,_fg_value,_fg_value,FILL100};
+#ifdef _NOTSH
+extern int WSGFdeviceDrawRect(int gid,short x,short y,
+                              unsigned short w, unsigned short h,
+                              unsigned int fg,int lw,int type);
+
+  if (_fdev != NULL && _fdev->haveWindow() == False){
+    short dx,dy;
+    _fdev->getAbstPos(&dx,&dy);
+    WSGFdeviceDrawRect(_gid,x+dx, y+dy, w, h, _fg_value,_line_width ,_line_type_val);
+  }else{
+    WSGFdeviceDrawRect(_gid,x, y, w, h, _fg_value,_line_width ,_line_type_val);
+  }
+#else
   PAT pat;
   pat.spat.kind = 0;
   pat.spat.hsize = 8;
@@ -647,6 +691,7 @@ long WSDbtronDraw::drawRect(short x,short y,WSCushort w,WSCushort h){
     r.c.bottom = y + h;
   }
   gfra_rec(_gid,r,(_line_type_val<<8)+_line_width,&pat,0,G_STORE);
+#endif
   return WS_NO_ERR;
 }
 long WSDbtronDraw::drawFillRect(short x,short y,WSCushort w,WSCushort h){
@@ -656,6 +701,19 @@ long WSDbtronDraw::drawFillRect(short x,short y,WSCushort w,WSCushort h){
   if (_gid == 0){
     return WS_ERR;
   }
+#ifdef _NOTSH
+extern int WSGFdeviceDrawFillRect(int gid,short x,short y,
+                              unsigned short w, unsigned short h,
+                              unsigned int bg,int htype);
+
+  if (_fdev != NULL && _fdev->haveWindow() == False){
+    short dx,dy;
+    _fdev->getAbstPos(&dx,&dy);
+    WSGFdeviceDrawFillRect(_gid,x+dx, y+dy, w, h, _fg_value,(int)_fill_mask);
+  }else{
+    WSGFdeviceDrawFillRect(_gid,x, y, w, h, _fg_value,(int)_fill_mask);
+  }
+#else
 //  PAT pat = {0,16,16,_fg_value,_fg_value,FILL100};
   PAT pat;
   pat.spat.kind = 0;
@@ -680,6 +738,7 @@ long WSDbtronDraw::drawFillRect(short x,short y,WSCushort w,WSCushort h){
   }
 //dbprintf("WSDbtronDraw::drawFillRect %d,%d,(real=%d,%d),%d,%d gid=0x%x\n",x,y,r.c.left,r.c.top,w,h,_gid);
   gfil_rec(_gid,r,&pat,0,G_STORE);
+#endif
   return WS_NO_ERR;
 }
 long WSDbtronDraw::drawRects(WSCrect* rects,long num){
@@ -1114,9 +1173,6 @@ long WSDbtronDraw::drawFillUString(long x,long y,WSCulong w,WSCulong h,
 long WSDbtronDraw::drawString(long x,long y,WSCulong w,WSCulong h,
          WSCstring* str,char font,char align,long cur,WSCbool inter_cur,
          long scur1,long scur2){
-#ifdef TDBG
-//printf("WSDbtronDraw::drawString start %d\n",WSGFclocktime());
-#endif
 dbprintf("ST WSDbtronDraw::drawString start %d %s fg=%s xy=%d,%d\n",WSGFclocktime(),str->getString(WS_EN_EUCJP),WSGIappColorSet()->getColor(_fg_cno)->getColorName(),x,y);
   if (_clip_w == 0 || _clip_h == 0){
     return WS_NO_ERR;
@@ -1159,8 +1215,9 @@ dbprintf("ST WSDbtronDraw::drawString start %d %s fg=%s xy=%d,%d\n",WSGFclocktim
       return WS_ERR;
     }
   }
-  FSSPEC* fnt = (FSSPEC*)fn->getValue1();
-  gset_fon(_gid,fnt); //TEST
+
+//  FSSPEC* fnt = (FSSPEC*)fn->getValue1();
+//  gset_fon(_gid,fnt);
 
   long fheight = fn->getFontHeight();
   long fheight2 = fn->getFontHeight();
@@ -1170,6 +1227,17 @@ dbprintf("ST WSDbtronDraw::drawString start %d %s fg=%s xy=%d,%d\n",WSGFclocktim
     area.setRect(x,y,w,h);
     WSCpoint  point;
     WSGFcalcOneLineStringAddr(str,&area,align,fn,&point);
+
+#ifdef _NOTSH
+    TC* tstr2 = str->getInternalUcs2();
+    if (tstr2 == NULL){
+      return 0;
+    }
+    long tclen = WSGFstrlenUCS2(tstr2);
+    WSGFbtronDrawString(tstr2,fn,_gid,_fg_value,_bg_value,
+                        tclen,point.x,point.y+fheight,
+                        cur,inter_cur,_line_width,scur1,scur2);
+#else
     char* tstr;
     tstr = str->getString(WS_EN_UTF8);
 
@@ -1179,14 +1247,14 @@ extern TC* WSGFbtronStringToTc(char*,long*);
     if (tstr2 == NULL){
       return 0;
     }
-dbprintf("ST WSDbtronDraw::drawString draw %s fg=%s xy=%d,%d fh=%d\n",str->getString(WS_EN_EUCJP),WSGIappColorSet()->getColor(_fg_cno)->getColorName(),point.x,point.y+fheight,fheight);
+
+//dbprintf("ST WSDbtronDraw::drawString draw %s fg=%s xy=%d,%d fh=%d\n",str->getString(WS_EN_EUCJP),WSGIappColorSet()->getColor(_fg_cno)->getColorName(),point.x,point.y+fheight,fheight);
 //    gset_chp(_gid,x,y + fheight,(Bool)True);
     WSGFbtronDrawString(tstr2,fn,_gid,_fg_value,_bg_value,
                         tclen,point.x,point.y+fheight,
                         cur,inter_cur,_line_width,scur1,scur2);
-
     delete tstr2;
-
+#endif
     r.c.left = clx_bk;
     r.c.top = cly_bk;
     r.c.right = clx_bk+clw_bk;
@@ -1221,36 +1289,34 @@ dbprintf("ST WSDbtronDraw::drawString draw %s fg=%s xy=%d,%d fh=%d\n",str->getSt
     if ((long)(cly + clh) < points[i].y){
       break;
     }
+    gset_chp(_gid,points[i].x,points[i].y + fn->getFontHeight(),(Bool)True);
+//#ifdef _NOTSH
+//    TC* tstr2 = (TC*)WSGFgetUCS2(lstr.getString(),lstr.getEncoding());
+//    if (tstr2 == NULL){
+//      return 0;
+//    }
+//    long tclen= WSGFstrlenUCS2(tstr2);
+//#else
     char* tstr;
     tstr = lstr.getString(WS_EN_UTF8);
+extern TC* WSGFbtronStringToTc(char*,long*);
+    long tclen=0;
+    TC* tstr2 = WSGFbtronStringToTc(tstr,&tclen);
+    if (tstr2 == NULL){
+      return 0;
+    }
+//#endif
+
     if (cur == -1){
-extern TC* WSGFbtronStringToTc(char*,long*);
-      long tclen=0;
-      TC* tstr2 = WSGFbtronStringToTc(tstr,&tclen);
-      if (tstr2 == NULL){
-//printf("ST WSDbtronDraw::drawString draw tstr=NULL!!!\n");
-        return 0;
-      }
-
-      gset_chp(_gid,points[i].x,points[i].y + fn->getFontHeight(),(Bool)True);
       WSGFbtronDrawString(tstr2,fn,_gid,_fg_value,_bg_value,
-                          tclen,points[i].x,points[i].y + fn->getFontHeight(),
-                          -1,inter_cur,_line_width,scur1 - lpos,scur2 - lpos);
-      delete tstr2;
+                 tclen,points[i].x,points[i].y + fn->getFontHeight(),
+                 -1,inter_cur,_line_width,scur1 - lpos,scur2 - lpos);
     }else{
-extern TC* WSGFbtronStringToTc(char*,long*);
-      long tclen=0;
-      TC* tstr2 = WSGFbtronStringToTc(tstr,&tclen);
-      if (tstr2 == NULL){
-        return 0;
-      }
-
-      gset_chp(_gid,points[i].x,points[i].y + fn->getFontHeight(),(Bool)True);
       WSGFbtronDrawString(tstr2,fn,_gid,_fg_value,_bg_value,
                  tclen,points[i].x,points[i].y + fn->getFontHeight(),
                  cur - lpos,inter_cur,_line_width,scur1 - lpos,scur2 - lpos);
-      delete tstr2;
     }
+    delete tstr2;
   }
   str->seek(0);
   if (num != 0){
@@ -1403,7 +1469,11 @@ static WSDcolor* color = NULL;
       if (val3 > 0xff){
         val3 = 0xff;
       }
+#ifdef _NOTSH
+      long cval = 0xff000000;
+#else
       long cval = 0x10000000;
+#endif
       cval |= val1 << 16;
       cval |= val2 << 8;
       cval |= val3;
@@ -1475,7 +1545,11 @@ static WSDcolor* color = NULL;
       if (val3 > 0xff){
         val3 = 0xff;
       }
+#ifdef _NOTSH
+      long cval = 0xff000000;
+#else
       long cval = 0x10000000;
+#endif
       cval |= val1 << 16;
       cval |= val2 << 8;
       cval |= val3;
@@ -1545,7 +1619,11 @@ static WSDcolor* color = NULL;
       if (val3 > 0xff){
         val3 = 0xff;
       }
+#ifdef _NOTSH
+      long cval = 0xff000000;
+#else
       long cval = 0x10000000;
+#endif
       cval |= val1 << 16;
       cval |= val2 << 8;
       cval |= val3;
@@ -1614,7 +1692,11 @@ static WSDcolor* color = NULL;
       if (val3 > 0xff){
         val3 = 0xff;
       }
+#ifdef _NOTSH
+      long cval = 0xff000000;
+#else
       long cval = 0x10000000;
+#endif
       cval |= val1 << 16;
       cval |= val2 << 8;
       cval |= val3;
