@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 //#include <x11/WSDxappDev.h>
 #include <unistd.h>
@@ -60,12 +56,17 @@ WSMFclassInit(WSDbtronExecute,WSDexecute);
 WSDexecute* _unix_execute_chandler(){
   return new WSDbtronExecute;
 }
+void WSGFbtronExecuteInit(){
+    WSDexecute::setCreateInstanceHandler((void*)_unix_execute_chandler);
+}
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _unix_execute_init {
   public: _unix_execute_init(){
-    WSDexecute::setCreateInstanceHandler((void*)_unix_execute_chandler);
+    WSGFbtronExecuteInit();
   };
 };
 static _unix_execute_init  _unix_execute_init_execute;
+#endif
 
 WSDbtronExecute::WSDbtronExecute(){
 //  _xid = 0;
@@ -638,6 +639,13 @@ void WSDbtronExecute::_read_thread_(int ptr){
     memcpy(msg->msg_body.ANYMSG.msg_str,&data,sizeof(data));
     snd_msg(0,msg,WAIT);
 
+#ifdef _NOTSH
+#ifndef NO_THREAD
+extern int _wake_up_event();
+    _wake_up_event();
+#endif
+#endif
+
     if (ret < 1){
       break;
     }
@@ -705,6 +713,13 @@ void WSDbtronExecute::_read_thread_2(int ptr){
     memcpy(msg->msg_body.ANYMSG.msg_str,&data,sizeof(data));
     snd_msg(0,msg,WAIT);
 
+#ifdef _NOTSH
+#ifndef NO_THREAD
+extern int _wake_up_event();
+    _wake_up_event();
+#endif
+#endif
+
     if (ret < 1){
       break;
     }
