@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <basic.h>
 #include <btron/hmi.h>
@@ -48,7 +44,7 @@ WSDdev* _wsdbtronFormdev_init_(){
 dbprintf("_wsdbtronFormdev_init_() %s;%d started.\n",__FILE__,__LINE__);
   return new WSDbtronFormDev();
 }
-
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _WSDbtronFormDev_init_ {
   public:_WSDbtronFormDev_init_(){
 dbprintf("_WSDbtronFormDev_init_() %s;%d started.\n",__FILE__,__LINE__);
@@ -56,7 +52,7 @@ dbprintf("_WSDbtronFormDev_init_() %s;%d started.\n",__FILE__,__LINE__);
   };
 };
 static _WSDbtronFormDev_init_ _init_to_run_WSDbtronFormDev_;
-
+#endif
 static WSDbtronDraw* draw_of_btronForm = NULL;
 
 static WSClistData _valid_list;
@@ -76,6 +72,13 @@ short WSDbtronFormDev::_exposed_global_region_y = 0;
 WSCushort WSDbtronFormDev::_exposed_global_region_w = 0;
 WSCushort WSDbtronFormDev::_exposed_global_region_h = 0;
 
+#ifdef _NOTSH
+extern void WSGFlockDev();
+extern void WSGFunlockDev();
+extern ERR _gset_vis(GID gid, RECT r);
+extern ERR _gset_for(GID gid, RLIST* rl);
+extern ERR _wget_act(W* pid);
+#endif
 
 WSDbtronFormDev::WSDbtronFormDev(){
 //dbprintf("WSDbtronFormDev::WSDbtronFormDev() %s;%d started.\n",__FILE__,__LINE__);
@@ -93,6 +96,7 @@ WSDbtronFormDev::WSDbtronFormDev(){
   _bg_pix = -1;
   _mno = 0;
   _click_time = 0;
+  _mparent_dev = NULL;
 #ifdef F_MEMGID
   setPixmapStyle(WS_DYNAMIC_PIXMAP);
 //printf("WSDbtronFormDev::WSDbtronFormDev  PIXMAP! \n");
@@ -126,9 +130,10 @@ WSDbtronFormDev::~WSDbtronFormDev(){
   }
 
   long num = _child_dev_list.getNum();
+  WSDbtronFormDev** buf = (WSDbtronFormDev**)_child_dev_list._data;
   long i;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* child = (WSDbtronFormDev*)_child_dev_list[i];
+    WSDbtronFormDev* child = (WSDbtronFormDev*)buf[i];
     child->setChildDevListFlag(False);
   }
   _child_dev_list.clear();
@@ -145,18 +150,20 @@ void WSDbtronFormDev::setChildDevListFlag(WSCbool fl){
 WSCbool WSDbtronFormDev::getChildDevListFlag(){
   return _added_to_child_dev_list;
 }
-WSClistData WSDbtronFormDev::getChildDevList(){
+WSClistData* WSDbtronFormDev::getChildDevList(){
   long num = _child_dev_list.getNum();
   long i;
+  WSDbtronFormDev** child_buf =(WSDbtronFormDev**)_child_dev_list._data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* child =(WSDbtronFormDev*)_child_dev_list[i];
+    WSDbtronFormDev* child = child_buf[i];
     if (isValidInstance(child) == False){
       _child_dev_list.del((void*)child);
+      child_buf =(WSDbtronFormDev**)_child_dev_list._data;
       i--;
       num = _child_dev_list.getNum();
     }
   }
-  return _child_dev_list;
+  return &_child_dev_list;
 }
 void WSDbtronFormDev::addChildDevList(WSDbtronFormDev* child){
   WSCbool fl = child->getChildDevListFlag();
@@ -173,8 +180,9 @@ void WSDbtronFormDev::delChildDevList(WSDbtronFormDev* child){
 WSCbool WSDbtronFormDev::isValidInstance(WSDbtronFormDev* inst){
   long i;
   long num = _valid_list.getNum();
+  WSDbtronFormDev** buf = (WSDbtronFormDev**)_valid_list._data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* item = (WSDbtronFormDev*)_valid_list[i];
+    WSDbtronFormDev* item = (WSDbtronFormDev*)buf[i];
     if (item == inst){
       return True;
     }
@@ -339,13 +347,27 @@ void WSDbtronFormDev::_copy_area(long x,long y,WSCulong w,WSCulong h,long dx,lon
     to.c.bottom = h +dy + py;
 //printf("WSDbtronFormDev::_copy_area px,py=%d,%d\n",px,py);
 //printf("WSDbtronFormDev::_copy_area %d,%d,%d,%d -> %d,%d %s\n",x+px,y+py,w,h,dx+px,dy+py,getAttachedClient()->getInstanceName());
-    gset_for(getParentMemGID(),NULL);
-    gset_vis(getParentMemGID(),from);
-    gset_for(getParentGID(),NULL);
+    W pmgid = getParentMemGID();
+    W pgid = getParentGID();
+#ifdef _NOTSH
+    //for optimizeing..
+    WSGFlockDev();
+    _gset_for(pmgid,NULL);
+    _gset_vis(pmgid,from);
+    _gset_for(pgid,NULL);
+    W ppid;
+    if (_expose_sequence == False || _wget_act(&ppid) != getWindowResource()){
+      _gset_vis(pgid,to);
+      WSGFunlockDev();
+#else
+    gset_for(pmgid,NULL);
+    gset_vis(pmgid,from);
+    gset_for(pgid,NULL);
     W ppid;
     if (_expose_sequence == False || wget_act(&ppid) != getWindowResource()){
+      gset_vis(pgid,to);
+#endif
 //printf("WSDbtronFormDev::_copy_area vis=%d,%d,%d,%d\n",px+dx,py+dy,w,h);
-      gset_vis(getParentGID(),to);
     }else{
       long gx,gy;
       WSCulong gw,gh;
@@ -360,10 +382,15 @@ void WSDbtronFormDev::_copy_area(long x,long y,WSCulong w,WSCulong h,long dx,lon
       vis.c.top = gy;
       vis.c.right = gx + gw;
       vis.c.bottom = gy + gh;
-      gset_vis(getParentGID(),vis);
+#ifdef _NOTSH
+      //for optimizeing..
+      _gset_vis(pgid,vis);
+      WSGFunlockDev();
+#else
+      gset_vis(pgid,vis);
+#endif
 //printf("WSDbtronFormDev::_copy_area vis=%d,%d,%d,%d\n",gx,gy,gw,gh);
     }
-
     setDefaultRegion();
 #if 0
 //this logic cause no drawing in pulldown
@@ -388,7 +415,7 @@ void WSDbtronFormDev::_copy_area(long x,long y,WSCulong w,WSCulong h,long dx,lon
       gcop_bmp(getParentMemGID(),&from,getParentGID(),&to,NULL,G_STORE);
     }
 #else
-    gcop_bmp(getParentMemGID(),&from,getParentGID(),&to,NULL,G_STORE);
+    gcop_bmp(pmgid,&from,pgid,&to,NULL,G_STORE);
 #endif
 #endif
   }else{
@@ -409,11 +436,13 @@ void WSDbtronFormDev::_copy_area(long x,long y,WSCulong w,WSCulong h,long dx,lon
 
 //    gset_for(getParentMemGID(),NULL);
 //    gset_vis(getParentMemGID(),from);
-    gset_for(getParentGID(),NULL);
+    W pmgid = getParentMemGID();
+    W pgid = getParentGID();
+    gset_for(pgid,NULL);
     W ppid;
     if (_expose_sequence == False || wget_act(&ppid) != getWindowResource()){
 //printf("WSDbtronFormDev::_copy_area vis=%d,%d,%d,%d\n",px+dx,py+dy,w,h);
-      gset_vis(getParentGID(),to);
+      gset_vis(pgid,to);
     }else{
       long gx,gy;
       WSCulong gw,gh;
@@ -428,15 +457,15 @@ void WSDbtronFormDev::_copy_area(long x,long y,WSCulong w,WSCulong h,long dx,lon
       vis.c.top = gy;
       vis.c.right = gx + gw;
       vis.c.bottom = gy + gh;
-      gset_vis(getParentGID(),vis);
+      gset_vis(pgid,vis);
 //printf("WSDbtronFormDev::_copy_area vis=%d,%d,%d,%d\n",gx,gy,gw,gh);
     }
 
-    if (getParentGID() != NULL){
-      gcop_bmp(getParentGID(),&from,getParentGID(),&to,NULL,G_STORE);
+    if (pgid != NULL){
+      gcop_bmp(pgid,&from,pgid,&to,NULL,G_STORE);
     }
-    if (getParentMemGID() != NULL){
-      gcop_bmp(getParentMemGID(),&from,getParentMemGID(),&to,NULL,G_STORE);
+    if (pmgid != NULL){
+      gcop_bmp(pmgid,&from,pmgid,&to,NULL,G_STORE);
     }
   }
 }
@@ -449,17 +478,17 @@ void WSDbtronFormDev::_clear_area(long x,long y,WSCulong w,WSCulong h){
 #ifdef F_MEMGID
   if (getPixmapStyle() != WS_DIRECT_WINDOW){
     _setup_pixmap(x,y,w,h,_bg,_bg_pix);
-  }
-  short dx,dy;
-  getAbstPos(&dx,&dy);
-  RECT r;
-  r.c.left = dx+x;
-  r.c.top = dx+y;
-  r.c.right = dx+x+_w_w;
-  r.c.bottom = dy+y+_w_h;
+    short dx,dy;
+    getAbstPos(&dx,&dy);
+    RECT r;
+    r.c.left = dx+x;
+    r.c.top = dx+y;
+    r.c.right = dx+x+_w_w;
+    r.c.bottom = dy+y+_w_h;
 //printf("WSDbtronFormDev::_clear_area.. expose!\n");
 //  _evh_expose(&r);
-  gcop_bmp(getParentMemGID(),&r,getParentGID(),&r,NULL,G_AND);
+    gcop_bmp(getParentMemGID(),&r,getParentGID(),&r,NULL,G_AND);
+  }
 #else 
   if (getUseMemGID() == False){
     drawBackGround(x,y,w,h);
@@ -467,6 +496,9 @@ void WSDbtronFormDev::_clear_area(long x,long y,WSCulong w,WSCulong h){
 #endif
 }
 WSDbtronFormDev* WSDbtronFormDev::getParentAreaDev(){
+  if (_mparent_dev != NULL){
+    return _mparent_dev;
+  }
   WSDdev*      pdev = WSDbtronFormDev::getParentDev();
   if (pdev == NULL){
     return NULL;
@@ -482,6 +514,7 @@ WSDbtronFormDev* WSDbtronFormDev::getParentAreaDev(){
       xdev = (WSDbtronFormDev*)pdev->cast("WSDbtronFormDev");
     }
   }
+  _mparent_dev = xdev;
   return xdev;
 }
 WSDbtronFormDev* WSDbtronFormDev::getMostParentAreaDev(){
@@ -507,7 +540,8 @@ dbprintf("## WSDbtronFormDev::_setup_pixmap vis=False return!!!\n");
   }
 //printf("## WSDbtronFormDev::_setup_pixmap %s %d,%d,%d,%d\n",getAttachedClient()->getInstanceName(),x,y,w,h);
 dbprintf("## WSDbtronFormDev::_setup_pixmap _mem_gid=0x%x\n",_mem_gid);
-  if (getParentMemGID() == 0){
+  W pmgid = getParentMemGID();
+  if (pmgid == 0){
     return;
   }
 
@@ -524,7 +558,11 @@ dbprintf("## WSDbtronFormDev::_setup_pixmap _mem_gid=0x%x\n",_mem_gid);
 //  }
 
 //  W  gid = parent->getContextResource();
+#ifdef _NOTSH
+  W bgc = 0xff000000;
+#else
   W bgc = 0x10000000;
+#endif
   WSDcolor* color = WSGIappColorSet()->getColor(bg);
   if (color != NULL){
     bgc = color->getValue1();
@@ -533,14 +571,12 @@ dbprintf("## WSDbtronFormDev::_setup_pixmap _mem_gid=0x%x\n",_mem_gid);
   WSCushort vw,vh;
   getAbstSize(&vx,&vy,&vw,&vh);
 
-
   RECT v;
   v.c.left = vx;
   v.c.top = vy;
   v.c.right = vx + vw;
   v.c.bottom = vy + vh;
-  gset_vis(getParentMemGID(),v);
-
+  gset_vis(pmgid,v);
 //  PAT pat = {0,16,16,bgc,bgc,FILL100};
   PAT pat;
   pat.spat.kind = 0;
@@ -549,7 +585,6 @@ dbprintf("## WSDbtronFormDev::_setup_pixmap _mem_gid=0x%x\n",_mem_gid);
   pat.spat.fgcol = bgc;
   pat.spat.bgcol = bgc;
   pat.spat.mask = FILL100;
-
   short px,py;
   getAbstPos(&px,&py);
   setDefaultRegion(); 
@@ -564,7 +599,7 @@ dbprintf("## WSDbtronFormDev::_setup_pixmap _mem_gid=0x%x\n",_mem_gid);
 //printf("## WSDbtronFormDev::_setup_pixmap %s fil=%d,%d,%d,%d col=%s\n",getAttachedClient()->getInstanceName(),x,y,w,h,color->getColorName());
 //printf("## WSDbtronFormDev::_setup_pixmap fil=%d,%d,%d,%d bgc=0x%x\n",x,y,w,h,bgc);
 //  gset_for(getContextResource(),NULL);
-  gfil_rec(getParentMemGID(),rect,&pat,0,G_STORE);
+
 
   if (img_no != -1 && img_no != 0){
     WSDimage* image = WSGIappImageSet()->getImage(img_no);
@@ -579,7 +614,14 @@ dbprintf("## WSDbtronFormDev::_setup_pixmap _mem_gid=0x%x\n",_mem_gid);
 //printf("## WSDbtronFormDev::_setup_pixmap %s pix=0x%x pw,ph=%d,%d\n",getAttachedClient()->getInstanceName(),pid,pw,ph);
       if (pid == 0 || pid == -1){
         return;
-      }     
+      }
+      RECT r;
+      r.c.left = x + px;
+      r.c.top = y + py;
+      r.c.right = x+w + px;
+      r.c.bottom = y+h + py;
+      gset_vis(pmgid,r);
+
       while(1){
         short ch = 0;
         while(1){
@@ -593,10 +635,10 @@ dbprintf("## WSDbtronFormDev::_setup_pixmap _mem_gid=0x%x\n",_mem_gid);
           to.c.right = pw +cw + px;
           to.c.bottom = ph +ch + py;
           if (pid2 != 0 && pid2 != -1){
-            gcop_bmp(pid2,&from,getParentMemGID(),&to,NULL,G_AND);
-            gcop_bmp(pid,&from,getParentMemGID(),&to,NULL,G_OR);
+            gcop_bmp(pid2,&from,pmgid,&to,NULL,G_AND);
+            gcop_bmp(pid,&from,pmgid,&to,NULL,G_OR);
           }else{
-            gcop_bmp(pid,&from,getParentMemGID(),&to,NULL,G_STORE);
+            gcop_bmp(pid,&from,pmgid,&to,NULL,G_STORE);
           }
 
           ch += ph;
@@ -611,9 +653,17 @@ dbprintf("## WSDbtronFormDev::_setup_pixmap _mem_gid=0x%x\n",_mem_gid);
         }
         if (cw > w) break;
       }
+      r.c.left = 0;
+      r.c.top = 0;
+      r.c.right = 32768;
+      r.c.bottom = 32768;
+      gset_vis(pmgid,r);
+    }else{
+      gfil_rec(pmgid,rect,&pat,0,G_STORE);
     }
+  }else{
+    gfil_rec(pmgid,rect,&pat,0,G_STORE);
   }
-
   return;
 }
 
@@ -640,10 +690,11 @@ dbprintf("WSDbtronWinDev::createWindow %d,%d,%d,%d\n",_w_x,_w_y,_w_w,_w_h);
 
   drawBackGround(0,0,w,h);
 
-  WSDbtronFormDev* pdev = getParentAreaDev();
-  if (pdev != NULL){
-    pdev->addChildDevList(this);
-  }
+//  WSDbtronFormDev* pdev = getParentAreaDev();
+//  if (pdev != NULL){
+//    pdev->addChildDevList(this);
+//  }
+  parent->addChildDevList(this);
   return WS_NO_ERR;
 }
 
@@ -679,11 +730,12 @@ long WSDbtronFormDev::createPixmap(){
   exposeExecute(0,0,w,h);
 
 #ifdef F_MEMGID
-  WSClistData children = getChildDevList();
-  long num = children.getNum();
+  WSClistData* children = getChildDevList();
+  long num = children->getNum();
   long i;
+  WSDbtronFormDev** buf = (WSDbtronFormDev**)children->_data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* child =(WSDbtronFormDev*)children[i];
+    WSDbtronFormDev* child = buf[i];
     if (WSDbtronFormDev::isValidInstance(child) == False){
       continue;
     }
@@ -695,6 +747,25 @@ long WSDbtronFormDev::createPixmap(){
 
   return WS_NO_ERR;
 }
+void WSDbtronFormDev::_clear_parent_cache(){
+  _mparent_dev = NULL;
+
+  WSClistData* children = getChildDevList();
+  long num = children->getNum();
+  long i;
+  WSDbtronFormDev** buf = (WSDbtronFormDev**)children->_data;
+  for(i=0; i<num; i++){
+    WSDbtronFormDev* child = buf[i];
+    if (WSDbtronFormDev::isValidInstance(child) == False){
+      continue;
+    }
+    child->_clear_parent_cache();
+  }
+}
+long WSDbtronFormDev::detachClient(){
+  _clear_parent_cache();
+  return WSDwindowDev::detachClient();
+}
 void WSDbtronFormDev::recursiveExposeExecute(short x,short y,WSCushort w,WSCushort h){
   if (getParentVisible() == False){
     return;
@@ -719,11 +790,12 @@ void WSDbtronFormDev::recursiveExposeExecute(short x,short y,WSCushort w,WSCusho
   short px,py;
   getAbstPos(&px,&py);
 #ifdef F_MEMGID
-  WSClistData children = getChildDevList();
-  long num = children.getNum();
+  WSClistData* children = getChildDevList();
+  long num = children->getNum();
   long i;
+  WSDbtronFormDev** child_buf =(WSDbtronFormDev**)children->_data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* child =(WSDbtronFormDev*)children[i];
+    WSDbtronFormDev* child = child_buf[i];
     if (WSDbtronFormDev::isValidInstance(child) == False){
       continue;
     }
@@ -800,11 +872,12 @@ long WSDbtronFormDev::destroyWindow(){
 
 long WSDbtronFormDev::destroyPixmap(){
   WSDwindowDev::destroyPixmap();
-  WSClistData children = getChildDevList();
-  long num = children.getNum();
+  WSClistData* children = getChildDevList();
+  long num = children->getNum();
   long i;
+  WSDbtronFormDev** child_buf =(WSDbtronFormDev**)children->_data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* child =(WSDbtronFormDev*)children[i];
+    WSDbtronFormDev* child = child_buf[i];
     if (WSDbtronFormDev::isValidInstance(child) == False){
       continue;
     }
@@ -927,21 +1000,9 @@ dbprintf("WSDbtronFormDev::setVisible fl=%d\n",fl);
        createWindow();
     }
     if (getPixmapStyle() == WS_DYNAMIC_PIXMAP){
-//printf("WSDbtronFormDev::setVisible fl=%d %s\n",fl,getAttachedClient()->getInstanceName());
-
       if (bk2 != fl){
         createPixmap();
-//      if (WSGIbtronAppDev()->getMouseGrabbed() != False){
-#if 1 //TEST
-//        WSDbtronFormDev* pdev = getMostParentAreaDev();
-//        if (pdev != NULL && pdev->getVisible() != False){
-//          pdev->addUpdateList();
-//        }
-        redrawAll();
-#else
         _copy_area(0,0,_w_w,_w_h,0,0,WS_PIXMAP_TO_WINDOW);
-#endif
-//      }
       }
     }
   }
@@ -1021,11 +1082,12 @@ WSDbtronFormDev* WSDbtronFormDev::getChildWithPoint(short x,short y){
   }else{
     return NULL;
   }
-  WSClistData children = getChildDevList();
-  long num = children.getNum();
+  WSClistData* children = getChildDevList();
+  long num = children->getNum();
   long i;
+  WSDbtronFormDev** child_buf = (WSDbtronFormDev**)children->_data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* child = (WSDbtronFormDev*)children[num-1 - i];
+    WSDbtronFormDev* child = child_buf[num-1 - i];
     short cx,cy;
     WSCushort cw,ch;
     child->getAbstSize(&cx,&cy,&cw,&ch);
@@ -1085,11 +1147,12 @@ void WSDbtronFormDev::_evh_expose(RECT* r){
 //printf("WSDbtronForm::_evh_expose %s\n",getAttachedClient()->getInstanceName());
   onExpose(&area);
 //printf("WSDbtronForm::_evh_expose next %s\n",getAttachedClient()->getInstanceName());
-  WSClistData children = getChildDevList();
-  long num = children.getNum();
+  WSClistData* children = getChildDevList();
+  long num = children->getNum();
   long i;
+  WSDbtronFormDev** child_buf =(WSDbtronFormDev**)children->_data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* child =(WSDbtronFormDev*)children[i];
+    WSDbtronFormDev* child = child_buf[i];
     if (child->getParentVisible() == False){
       continue;
     }
@@ -1113,21 +1176,25 @@ void WSDbtronFormDev::_evh_expose(RECT* r){
 //printf("WSDbtronForm::_evh_expose done %s\n",getAttachedClient()->getInstanceName());
 }
 void WSDbtronFormDev::_expose_ex(WSCrect* r){
-   if (getParentVisible() == False){
-     return;
-   }
-   if (getAttachedClient() == NULL){
+  if (getParentVisible() == False){
+    return;
+  }
+  if (getAttachedClient() == NULL){
 dbprintf("WSDbtronFormDev::_expose_ex %d,%d,%d,%d exit1!!!!\n",r->x,r->y,r->width,r->height);
-     setVisible(False);
-     return;
-   }else if (getAttachedClient()->getParentVisible() == False){
+    setVisible(False);
+    return;
+  }else if (getAttachedClient()->getParentVisible() == False){
 //dbprintf("WSDbtronFormDev::_expose_ex %d,%d,%d,%d exit2!!!! client=%s\n",r->x,r->y,r->width,r->height,getAttachedClient()->getInstanceName());
-     setVisible(False);
-     return;
-   }
+    setVisible(False);
+    return;
+  }
 
-   W bgc = 0x10000000;
-   WSDcolor* color = WSGIappColorSet()->getColor(_bg);
+#ifdef _NOTSH
+  W bgc = 0xff000000;
+#else
+  W bgc = 0x10000000;
+#endif
+  WSDcolor* color = WSGIappColorSet()->getColor(_bg);
                                                                                 
   if (color != NULL){
     bgc = color->getValue1();
@@ -1157,11 +1224,12 @@ dbprintf("WSDbtronFormDev::_expose_ex %d,%d,%d,%d exit1!!!!\n",r->x,r->y,r->widt
   area.setRect(r->x - dx,r->y - dy,r->width,r->height);
   onExpose(&area);
 
-  WSClistData children = getChildDevList();
-  long num = children.getNum();
+  WSClistData* children = getChildDevList();
+  long num = children->getNum();
   long i;
+  WSDbtronFormDev** child_buf =(WSDbtronFormDev**)children->_data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* child =(WSDbtronFormDev*)children[i];
+    WSDbtronFormDev* child = child_buf[i];
     if (child->getParentVisible() == False){
       continue;
     }
@@ -1223,11 +1291,12 @@ void WSDbtronFormDev::_evh_expose_abs(RECT* r){
              (WSCushort)(r->c.bottom - r->c.top));
   onExpose(&area);
 
-  WSClistData children = getChildDevList();
-  long num = children.getNum();
+  WSClistData* children = getChildDevList();
+  long num = children->getNum();
   long i;
+  WSDbtronFormDev** child_buf =(WSDbtronFormDev**)children->_data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* child =(WSDbtronFormDev*)children[i];
+    WSDbtronFormDev* child = child_buf[i];
     if (child->getParentVisible() == False){
       continue;
     }
@@ -1244,21 +1313,25 @@ void WSDbtronFormDev::_evh_expose_abs(RECT* r){
   }
 }
 void WSDbtronFormDev::_expose_ex_abs(WSCrect* r){
-   if (getParentVisible() == False){
-     return;
-   }
-   if (getAttachedClient() == NULL){
+  if (getParentVisible() == False){
+    return;
+  }
+  if (getAttachedClient() == NULL){
 //dbprintf("WSDbtronFormDev::_expose_ex %d,%d,%d,%d exit1!!!!\n",r->x,r->y,r->width,r->height);
-     setVisible(False);
-     return;
-   }else if (getAttachedClient()->getParentVisible() == False){
+    setVisible(False);
+    return;
+  }else if (getAttachedClient()->getParentVisible() == False){
 //dbprintf("WSDbtronFormDev::_expose_ex %d,%d,%d,%d exit2!!!! client=%s\n",r->x,r->y,r->width,r->height,getAttachedClient()->getInstanceName());
-     setVisible(False);
-     return;
-   }
+    setVisible(False);
+    return;
+  }
 
-   W bgc = 0x10000000;
-   WSDcolor* color = WSGIappColorSet()->getColor(_bg);
+#ifdef _NOTSH
+  W bgc = 0xff000000;
+#else
+  W bgc = 0x10000000;
+#endif
+  WSDcolor* color = WSGIappColorSet()->getColor(_bg);
                                                                                 
   if (color != NULL){
     bgc = color->getValue1();
@@ -1287,11 +1360,12 @@ void WSDbtronFormDev::_expose_ex_abs(WSCrect* r){
   exposeExecute(r->x - dx,r->y - dy,r->width,r->height);
   onExpose(&area);
 
-  WSClistData children = getChildDevList();
-  long num = children.getNum();
+  WSClistData* children = getChildDevList();
+  long num = children->getNum();
   long i;
+  WSDbtronFormDev** child_buf =(WSDbtronFormDev**)children->_data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* child =(WSDbtronFormDev*)children[i];
+    WSDbtronFormDev* child = child_buf[i];
     if (child->getParentVisible() == False){
       continue;
     }
@@ -1381,7 +1455,7 @@ void WSDbtronFormDev::_evh_btn_press(WEVENT* ev){
    long clk = WSGFclocktime();
    WSGIbtronAppDev()->setBtnPressGrabTarget(this);
    onMousePress(&point);
-   if ((WSCulong)clk - (WSCulong)_click_time < 250){
+   if ((WSCulong)clk - (WSCulong)_click_time < 500){
      onMouseDoubleClick(&point);
    }
    _click_time = clk;
@@ -1500,13 +1574,16 @@ WSCbool WSDbtronFormDev::haveWindow(){
 
 void WSDbtronFormDev::setDefaultRegion(){
 //dbprintf("WSDbtronFormDev::setDefaultRegion this->%s\n",getAttachedClient()->getInstanceName());
+#ifndef NO_KEYBOARD
 extern W _fep_popup;
-  WSClistData dlist = getChildDevList();
-  long num = dlist.getNum();
+#endif
 
+  WSClistData* dlist = getChildDevList();
+  long num = dlist->getNum();
 //printf("fep_popup=0x%x\n",_fep_popup);
   RLIST rlist[2+N_VIEWR];
   long cnt = 0;
+#ifndef NO_KEYBOARD
   if (_fep_popup != 0){
     WDSTAT wst;
     wget_sts(_fep_popup,&wst,NULL);
@@ -1517,6 +1594,7 @@ extern W _fep_popup;
     rlist[cnt].next = NULL;
     cnt++;
   }
+#endif
   if (_global_region_w !=0 && _global_region_h != 0){
 //printf("fep_mask=%d,%d,%d,%d\n",_global_region_x,_global_region_y,_global_region_w,_global_region_h);
     PNT pt;
@@ -1595,12 +1673,45 @@ extern W _fep_popup;
 //  }
 
   WSDbtronFormDev* pdev = getParentAreaDev();
-  WSClistData bdlist;
+  WSClistData* bdlist = NULL;
+  long bnum = 0;
   if (pdev != NULL){
-    WSClistData bdlist2 = pdev->getChildDevList();
-    bdlist = bdlist2;
+    bdlist = pdev->getChildDevList();
+  }
+  if (bdlist != NULL){
+    bnum = bdlist->getNum();
+  }
+  W pmgid = getParentMemGID();
+  W pgid = getParentGID();
+  if (num == 0 && bnum == 0){
+#ifdef _NOTSH
+    //for optimizeing..
+    WSGFlockDev();
+    if (pmgid != 0){
+      _gset_for(pmgid,NULL);
+    }
+    if (pgid != 0){
+      if (cnt > 0){
+        _gset_for(pgid,rlist);
+      }else{
+        _gset_for(pgid,NULL);
+      }
+    }
+    WSGFunlockDev();
+#else
+    if (pmgid != 0){
+      gset_for(pmgid,NULL);
+    }
+    if (pgid != 0){
+      if (cnt > 0){
+        gset_for(pgid,rlist);
+      }else{
+        gset_for(pgid,NULL);
+      }
+    }
+#endif
+    return;
   }
-  long bnum = bdlist.getNum();
 
   long next=0;
   short dx,dy;
@@ -1611,8 +1722,9 @@ extern W _fep_popup;
   RLIST* rc_mem = new RLIST[num+bnum+1];
   rc[0].next = NULL;
   rc_mem[0].next = NULL;
+  void** dlist_buf = dlist->_data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* dev = (WSDbtronFormDev*)dlist[i];
+    WSDbtronFormDev* dev = (WSDbtronFormDev*)dlist_buf[i];
 //printf("setDefaultRegion %d %s vis=%d\n",i,dev->getAttachedClient()->getInstanceName(),dev->getParentVisible());
     if (WSDbtronFormDev::isValidInstance(dev) == False){
       continue;
@@ -1635,7 +1747,7 @@ extern W _fep_popup;
 //dbprintf("WSDbtronFormDev::setDefaultRegion child->%s %d,%d,%d,%d\n",dev->getAttachedClient()->getInstanceName(),cx,cy,cw,ch);
     pt.x = cx;
     pt.y = cy;
-    gcnv_abs((W)getParentGID(),&pt);
+    gcnv_abs(pgid,&pt);
 
     rc[next].rcomp.c.left = pt.x;
     rc[next].rcomp.c.top = pt.y;
@@ -1643,7 +1755,7 @@ extern W _fep_popup;
     rc[next].rcomp.c.bottom = pt.y + ch;
 //printf("WSDbtronFormDev::setDefaultRegion %d,%d,%d,%d\n",pt.x,pt.y,cw,ch);
 
-    if (getParentMemGID() != 0){
+    if (pmgid != 0){
       pt.x = cx;
       pt.y = cy;
       rc_mem[next].rcomp.c.left = pt.x;
@@ -1660,91 +1772,119 @@ extern W _fep_popup;
     }
     next++;
   }
-
-  WSCbool found_self = False;
-  for(i=0; i<bnum; i++){
-    WSDbtronFormDev* dev = (WSDbtronFormDev*)bdlist[i];
+  if (bdlist != NULL){
+    WSCbool found_self = False;
+    void** list_buf = bdlist->getBuf();
+    for(i=0; i<bnum; i++){
+      WSDbtronFormDev* dev = (WSDbtronFormDev*)list_buf[i];
+//    WSDbtronFormDev* dev = (WSDbtronFormDev*)bdlist[i];
 //printf("setDefaultRegion %d %s vis=%d\n",i,dev->getAttachedClient()->getInstanceName(),dev->getParentVisible());
-    if (WSDbtronFormDev::isValidInstance(dev) == False){
-      continue;
-    }
-    if (dev->getParentVisible() == False){
-      continue;
-    }
-    if (dev == this){
-      found_self = True;
-      continue;
-    }
-    if (found_self == False){
-      continue;
-    }
-    short cx,cy;
-    WSCushort cw,ch;
-    PNT pt;
-    dev->getAbstSize(&cx,&cy,&cw,&ch);
-    rc[next].next = NULL;
-    rc_mem[next].next = NULL;
-    if (cw <1){
-      continue;
-    }
-    if (ch <1){
-      continue;
-    }
+      if (WSDbtronFormDev::isValidInstance(dev) == False){
+        continue;
+      }
+      if (dev->getParentVisible() == False){
+        continue;
+      }
+      if (dev == this){
+        found_self = True;
+        continue;
+      }
+      if (found_self == False){
+        continue;
+      }
+      short cx,cy;
+      WSCushort cw,ch;
+      PNT pt;
+      dev->getAbstSize(&cx,&cy,&cw,&ch);
+      rc[next].next = NULL;
+      rc_mem[next].next = NULL;
+      if (cw <1){
+        continue;
+      }
+      if (ch <1){
+        continue;
+      }
 //dbprintf("WSDbtronFormDev::setDefaultRegion child->%s %d,%d,%d,%d\n",dev->getAttachedClient()->getInstanceName(),cx,cy,cw,ch);
-    pt.x = cx;
-    pt.y = cy;
-//    gcnv_abs((W)getContextResource(),&pt);
-    gcnv_abs((W)getParentGID(),&pt);
-
-    rc[next].rcomp.c.left = pt.x;
-    rc[next].rcomp.c.top = pt.y;
-    rc[next].rcomp.c.right = pt.x + cw;
-    rc[next].rcomp.c.bottom = pt.y + ch;
-
-    if (getParentMemGID() != 0){
       pt.x = cx;
       pt.y = cy;
-      rc_mem[next].rcomp.c.left = pt.x;
-      rc_mem[next].rcomp.c.top = pt.y;
-      rc_mem[next].rcomp.c.right = pt.x + cw;
-      rc_mem[next].rcomp.c.bottom = pt.y + ch;
-    }
+//    gcnv_abs((W)getContextResource(),&pt);
+      gcnv_abs(pgid,&pt);
+
+      rc[next].rcomp.c.left = pt.x;
+      rc[next].rcomp.c.top = pt.y;
+      rc[next].rcomp.c.right = pt.x + cw;
+      rc[next].rcomp.c.bottom = pt.y + ch;
+
+      if (pmgid != 0){
+        pt.x = cx;
+        pt.y = cy;
+        rc_mem[next].rcomp.c.left = pt.x;
+        rc_mem[next].rcomp.c.top = pt.y;
+        rc_mem[next].rcomp.c.right = pt.x + cw;
+        rc_mem[next].rcomp.c.bottom = pt.y + ch;
+      }
 
 //dbprintf("WSDbtronFormDev::setDefaultRegion %d,%d,%d,%d\n",pt.x,pt.y,cw,ch);
-    if (next >  0){
-      rc[next-1].next = &rc[next];
-      rc_mem[next-1].next = &rc_mem[next];
+      if (next >  0){
+        rc[next-1].next = &rc[next];
+        rc_mem[next-1].next = &rc_mem[next];
+      }
+      next++;
     }
-    next++;
   }
 //printf("cnt=%d\n",cnt);
   if (next == 0){
-//dbprintf("WSDbtronFormDev::setDefaultRegion set_for=0x%x\n",getContextResource());
-    if (getParentMemGID() != 0){
-      gset_for(getParentMemGID(),NULL);
+#ifdef _NOTSH
+    //for optimizeing..
+    WSGFlockDev();
+    if (pmgid != 0){
+      _gset_for(pmgid,NULL);
     }
-    if (getParentGID() != 0){
+    if (pgid != 0){
       if (cnt > 0){
-//printf("clip cnt=%d\n",cnt);
-        gset_for(getParentGID(),rlist);
+        _gset_for(pgid,rlist);
       }else{
-        gset_for(getParentGID(),NULL);
+        _gset_for(pgid,NULL);
       }
     }
-//    gset_for(getContextResource(),NULL);
-  }else{
-//dbprintf("WSDbtronFormDev::setDefaultRegion set_for=0x%x\n",getContextResource());
-    if (getParentMemGID() != 0){
-      gset_for(getParentMemGID(),rc_mem);
+    WSGFunlockDev();
+#else
+    if (pmgid != 0){
+      gset_for(pmgid,NULL);
     }
-    if (getParentGID() != 0){
-//printf("add cnt=%d\n",cnt);
+    if (pgid != 0){
+      if (cnt > 0){
+        gset_for(pgid,rlist);
+      }else{
+        gset_for(pgid,NULL);
+      }
+    }
+#endif
+  }else{
+#ifdef _NOTSH
+    //for optimizeing..
+    WSGFlockDev();
+    if (pmgid != 0){
+      _gset_for(pmgid,rc_mem);
+    }
+    if (pgid != 0){
       if (cnt > 0){
         rc[next-1].next = &rlist[0];
       }
-      gset_for(getParentGID(),rc);
+      _gset_for(pgid,rc);
     }
-//    gset_for(getContextResource(),rc);
+    WSGFunlockDev();
+#else
+    if (pmgid != 0){
+      gset_for(pmgid,rc_mem);
+    }
+    if (pgid != 0){
+      if (cnt > 0){
+        rc[next-1].next = &rlist[0];
+      }
+      gset_for(pgid,rc);
+    }
+#endif
   }
   delete[] rc;
   delete[] rc_mem;
@@ -1768,7 +1908,11 @@ void WSDbtronFormDev::drawBackGround(long x,long y,WSCulong w,WSCulong h){
     }
   }
 
+#ifdef _NOTSH
+  W bgc = 0xff000000;
+#else
   W bgc = 0x10000000;
+#endif
   WSDcolor* color = WSGIappColorSet()->getColor(_bg);
   if (color != NULL){
     bgc = color->getValue1();
@@ -1900,14 +2044,15 @@ W WSDbtronFormDev::getParentMemGID(){
   }
   return 0;
 }
-void WSDbtronFormDev::exposeExecute(short x,short y,WSCushort w,WSCushort h){
-  WSDwindowDev::exposeExecute(x,y,w,h);
-}
+//void WSDbtronFormDev::exposeExecute(short x,short y,WSCushort w,WSCushort h){
+//  WSDwindowDev::exposeExecute(x,y,w,h);
+//}
 void WSDbtronFormDev::addUpdateList(){
   long num = _update_list.getNum();
+  WSDbtronFormDev** buf =(WSDbtronFormDev**)_update_list._data;
   long i;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* item =(WSDbtronFormDev*)_update_list[i];
+    WSDbtronFormDev* item =(WSDbtronFormDev*)buf[i];
     if (item == this){
       return;
     }
@@ -1916,9 +2061,10 @@ void WSDbtronFormDev::addUpdateList(){
 }
 void WSDbtronFormDev::addRecursiveUpdateList(){
   long num = _recursive_update_list.getNum();
+  WSDbtronFormDev** buf =(WSDbtronFormDev**)_recursive_update_list._data;
   long i;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* item =(WSDbtronFormDev*)_recursive_update_list[i];
+    WSDbtronFormDev* item =(WSDbtronFormDev*)buf[i];
     if (item == this){
       return;
     }
@@ -1927,9 +2073,10 @@ void WSDbtronFormDev::addRecursiveUpdateList(){
 }
 void WSDbtronFormDev::addGeometryUpdateList(){
   long num = _geometry_update_list.getNum();
+  WSDbtronFormDev** buf =(WSDbtronFormDev**)_geometry_update_list._data;
   long i;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* item =(WSDbtronFormDev*)_geometry_update_list[i];
+    WSDbtronFormDev* item =(WSDbtronFormDev*)buf[i];
     if (item == this){
       return;
     }
@@ -1938,9 +2085,10 @@ void WSDbtronFormDev::addGeometryUpdateList(){
 }
 void WSDbtronFormDev::executeUpdate(){
   long num = _geometry_update_list.getNum();
+  WSDbtronFormDev** buf =(WSDbtronFormDev**)_geometry_update_list._data;
   long i;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* item =(WSDbtronFormDev*)_geometry_update_list[i];
+    WSDbtronFormDev* item =(WSDbtronFormDev*)buf[i];
     if (item != NULL && isValidInstance(item) != False){
       short dx,dy;
       WSCushort dw,dh;
@@ -1957,8 +2105,9 @@ void WSDbtronFormDev::executeUpdate(){
           W wid = (W)item->getWindowResource();
           long update = True;
           long unum = _expose_list.getNum();
+          W* ebuf1 = (W*)_expose_list._data;
           for(j=0; j<unum;j++){
-            W item = (W)_expose_list[j];
+            W item = (W)ebuf1[j];
             if (item == wid){
               update = False;
               break;
@@ -1983,8 +2132,9 @@ void WSDbtronFormDev::executeUpdate(){
   _geometry_update_list.clear();
 
   num = _update_list.getNum();
+  WSDbtronFormDev** buf2 =(WSDbtronFormDev**)_update_list._data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* item =(WSDbtronFormDev*)_update_list[i];
+    WSDbtronFormDev* item =(WSDbtronFormDev*)buf2[i];
     if (item != NULL && isValidInstance(item) != False &&
         item->getParentVisible() != False){
 #if 0
@@ -2003,8 +2153,10 @@ void WSDbtronFormDev::executeUpdate(){
       W wid = (W)item->getWindowResource();
       long update = True;
       long unum = _expose_list.getNum();
+      W* ebuf = (W*)_expose_list._data;
+ 
       for(j=0; j<unum;j++){
-        W item = (W)_expose_list[j];
+        W item = (W)ebuf[j];
         if (item == wid){
           update = False;
           break;
@@ -2019,16 +2171,18 @@ void WSDbtronFormDev::executeUpdate(){
 
   WSClistData recursive_ulist;
   num = _recursive_update_list.getNum();
+  WSDbtronFormDev** buf3 =(WSDbtronFormDev**)_recursive_update_list._data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* item =(WSDbtronFormDev*)_recursive_update_list[i];
+    WSDbtronFormDev* item =(WSDbtronFormDev*)buf3[i];
     if (item != NULL && isValidInstance(item) != False &&
         item->getParentVisible() != False){
       long j;
       W wid = (W)item->getWindowResource();
       long update = True;
       long unum = _expose_list.getNum();
+      W* ebuf1 = (W*)_expose_list._data;
       for(j=0; j<unum;j++){
-        W item = (W)_expose_list[j];
+        W item = (W)ebuf1[j];
         if (item == wid){
           update = False;
           break;
@@ -2042,8 +2196,9 @@ void WSDbtronFormDev::executeUpdate(){
   _recursive_update_list.clear();
 
   num = _expose_list.getNum();
+  W* ebuf = (W*)_expose_list._data;
   for(i=0; i<num; i++){
-    W wid = (W)_expose_list[i];
+    W wid = (W)ebuf[i];
 #ifndef SMARTUPDATE
     W ret = wreq_dsp(wid);
 #else
@@ -2074,8 +2229,9 @@ dbprintf("WSDbtronFormDev::execUpdate update wid=0x%x ret=0x%x\n",wid,ret);
   _expose_list.clear();
 
   num = recursive_ulist.getNum();
+  WSDbtronFormDev** buf4 =(WSDbtronFormDev**)recursive_ulist._data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* item =(WSDbtronFormDev*)recursive_ulist[i];
+    WSDbtronFormDev* item =(WSDbtronFormDev*)buf4[i];
     item->recursive_update();
   }
 }
@@ -2128,7 +2284,10 @@ void WSDbtronFormDev::setClipNum(long num){
 void WSDbtronFormDev::_evh_destroy(WEVENT* ev){
   WSCbase* client = getAttachedClient();
   if (client != NULL){
-    client->setVisible(False);
+    if (WSGFgetWindowEnableClose(client) != False){
+      client->setVisible(False);
+    }
+    client->execEventProc(WSEV_WINDOW_CLOSED,NULL);
   }
 }
 WSCbool WSDbtronFormDev::getParentVisible(){
