@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,12 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+
 
 #include <basic.h>
 #include <btron/hmi.h>
@@ -32,6 +29,7 @@
 #include <errcode.h>
 #endif
 #include <WSCdevice.h>
+#ifndef NO_SCR
 #include <btron/WSDbtronScrFrameDev.h>
 #include <WSCbase.h>
 #include <WSCbaseList.h>
@@ -47,12 +45,14 @@ WSDdev* _wsdbtronScrdev_init_(){
   return new WSDbtronScrFrameDev();
 }
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _WSDbtronScrFrameDev_init_ {
   public:_WSDbtronScrFrameDev_init_(){
      WSGIappDevice()->setCreateHandler("scrFrameDev",_wsdbtronScrdev_init_);
   };
 };
 static _WSDbtronScrFrameDev_init_ _init_to_run_WSDbtronScrFrameDev_;
+#endif
 
 WSDbtronScrFrameDev::WSDbtronScrFrameDev(){
   _p1_x = 0;
@@ -541,11 +541,12 @@ long WSDbtronScrFrameDev::createPixmap(){
   _setup_pixmap(0,0,w,h,_bg,_bg_pix);
   exposeExecute(0,0,w,h);
 #ifdef F_MEMGID
-  WSClistData children = getChildDevList();
-  long num = children.getNum();
+  WSClistData* children = getChildDevList();
+  long num = children->getNum();
   long i;
+  WSDbtronFormDev** child_buf =(WSDbtronFormDev**)children->_data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* child =(WSDbtronFormDev*)children[i];
+    WSDbtronFormDev* child = child_buf[i];
     if (WSDbtronFormDev::isValidInstance(child) == False){
       continue;
     }
@@ -604,3 +605,5 @@ void WSDbtronScrFrameDev::_clear_area(long x,long y,WSCulong w,WSCulong h){
 void WSDbtronScrFrameDev::recursive_update(){
   recursiveExposeExecute(_h_scr_pos,_v_scr_pos,_w_w,_w_h);
 }
+
+#endif
