@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 //#include <X11/Xlib.h>
 #define _ANSI_SOURCE
@@ -64,6 +60,7 @@ WSDsocket* _btronSocket_create_handler(){
 };
 
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class  _btronSocket_init {
   public:
   _btronSocket_init(){
@@ -72,6 +69,7 @@ class  _btronSocket_init {
   };
 };
 _btronSocket_init   WSGIappXsocketInit;
+#endif
 
 WSDbtronSocket::WSDbtronSocket(){
   _socket = -1; 
@@ -117,16 +115,24 @@ fprintf(stderr,"WSDbtronSocket::connect socket is udp.\n");
   struct hostent  hptmp;
   struct hostent  *hp = &hptmp;
   ERR ret;
-//printf("WSDbtron::connect begin gethostbyname dest_naddr=#%s#\n",(char*)_dest_naddr);
+  UB s[4];
   if (!strcmp(_dest_naddr,"")){
     B tmpbuf[HBUFLEN];
-    ret = so_gethostbyname("localhost",hp,tmpbuf);
+    if (inet_pton(AF_INET,"localhost",s) == 1){
+      ret = so_gethostbyaddr((B*)s,4,AF_INET,hp,tmpbuf);
+    }else{
+      ret = so_gethostbyname("localhost",hp,tmpbuf);
+    }
   }else{
     B tmpbuf[HBUFLEN];
-    ret = so_gethostbyname((char*)_dest_naddr,hp,tmpbuf);
+    if (inet_pton(AF_INET,(char*)_dest_naddr,s) == 1){
+      ret = so_gethostbyaddr((B*)s,4,AF_INET,hp,tmpbuf);
+    }else{
+      ret = so_gethostbyname((char*)_dest_naddr,hp,tmpbuf);
+    }
   }
   server.sin_family = AF_INET;
-  if (hp == NULL){
+  if (ret < 0){
     server.sin_addr.s_addr = (unsigned long)inet_addr((char*)_dest_naddr);
     if ( server.sin_addr.s_addr == (unsigned long)-1 ) {
 fprintf(stderr,"WSDbtronSocket::connect invarid addr=%s\n",(char*)_dest_naddr);
@@ -135,13 +141,97 @@ fprintf(stderr,"WSDbtronSocket::connect invarid addr=%s\n",(char*)_dest_naddr);
   }else{
     server.sin_addr.s_addr = *(unsigned long *)hp->h_addr;
   }
+  WSCulong timeout = 0;
+//  if (_socket == (long)handle){
+    timeout = _timeout *1000;
+//  }else{
+//    timeout = _cl_timeout *1000;
+//  }
+  if (timeout > 0){
+    int fl = so_fcntl((int)_socket,F_GETFL,0);
+    if (fl == -1){
+      seterr();
+      return 0;
+    }
+    int ret = so_fcntl((int)_socket,F_SETFL,fl | FNDELAY);
+    if (ret == -1){
+      seterr();
+      return 0;
+    }
+  }else{
+    int fl = so_fcntl((int)_socket,F_GETFL,0);
+    if (fl == -1){
+      seterr();
+      return 0;
+    }
+    int ret = so_fcntl((int)_socket,F_SETFL,fl & ~FNDELAY);
+    if (ret == -1){
+      seterr();
+      return 0;
+    }
+  }
+
   server.sin_port = htons( _dest_port );
-//printf("WSDbtron::connect begin connect\n");
+
   ret = so_connect(_socket, (struct sockaddr *)&server,sizeof server);
 //printf("WSDbtron::connect begin connect done\n");
+  if (ret < 0){
+    if (timeout > 0){
+      if (ret == EX_INPROGRESS){
+        struct timeval tm;
+        tm.tv_sec = timeout/1000;
+        tm.tv_usec = 0;
+        fd_set connect_fds;
+        FD_ZERO(&connect_fds);
+        FD_SET((int)_socket,&connect_fds);
+        fd_set err_fds;
+        FD_ZERO(&err_fds);
+        FD_SET((int)_socket,&err_fds);
+        ret = so_select( FD_SETSIZE,NULL,&connect_fds,&err_fds,&tm );
+
+        long err = ret;
+        if (FD_ISSET(_socket,&err_fds)){
+          seterr();
+fprintf(stderr,"WSDbtronSocket::connect failed addr=%s port=%d err=%d\n",(char*)_dest_naddr,_dest_port,ret>>16);
+          return WS_ERR;
+        }
+        if (ret < 0 && err != EX_ALREADY){
+//printf("WSDbtronSocket::read... ret=%d\n",ret>>16);
+          seterr();
+          close();
+fprintf(stderr,"WSDbtronSocket::connect failed addr=%s port=%d err=%d\n",(char*)_dest_naddr,_dest_port,ret>>16);
+          return WS_ERR;
+        }
+        if (ret == 0){
+//printf("WSDbtronSocket::read... ret=%d\n",ret>>16);
+          seterr(EX_TIMEDOUT);
+          close();
+fprintf(stderr,"WSDbtronSocket::connect failed addr=%s port=%d err=%d\n",(char*)_dest_naddr,_dest_port,ret>>16);
+          return WS_ERR;
+        }
+        char buf[4];
+      // This recving is check the connection to the server.
+      // This logic depends on T-Shell's  socket specification.
+        ret = so_recv((int)_socket,(char*)&buf,1,0);
+        if (ret == EX_WOULDBLOCK){
+          return WS_NO_ERR;
+        }
+fprintf(stderr,"WSDbtronSocket::connect failed addr=%s port=%d err=%d\n",(char*)_dest_naddr,_dest_port,ret>>16);
+        seterr();
+        close();
+        return WS_ERR;
+      }
+fprintf(stderr,"WSDbtronSocket::connect failed addr=%s port=%d err=%d\n",(char*)_dest_naddr,_dest_port,ret>>16);
+      seterr();
+      close();
+      return WS_ERR;
+    }
+fprintf(stderr,"WSDbtronSocket::connect failed addr=%s port=%d err=%d\n",(char*)_dest_naddr,_dest_port,ret>>16);
+    seterr();
+    return WS_ERR;
+  }
   if( ret < 0 ){
     seterr();
-    close();
 fprintf(stderr,"WSDbtronSocket::connect failed addr=%s port=%d err=%d\n",(char*)_dest_naddr,_dest_port,ret>>16);
     return WS_ERR;
   }
@@ -265,6 +355,14 @@ void WSDbtronSocket::_accept_thread_(W ptr){
 
 
     W ret = snd_msg(0,msg,WAIT);
+
+#ifdef _NOTSH
+#ifndef NO_THREAD
+extern int _wake_up_event();
+    _wake_up_event();
+#endif
+#endif
+
 //fprintf(stderr,"WSDbtronSocket::_accept_thread data=0x%x ret=%d\n",data,ret>>16);
   }
 }
@@ -353,6 +451,13 @@ void WSDbtronSocket::_udp_read_thread_(W ptr){
       }
       _this->_sem = (void*)sem;
       snd_msg(0,msg,WAIT);
+
+#ifdef _NOTSH
+#ifndef NO_THREAD
+extern int _wake_up_event();
+    _wake_up_event();
+#endif
+#endif
  
 //printf("WSDbtronSocket::_udp_read_thread wait..\n");
       wai_sem(sem,T_FOREVER); 
@@ -473,8 +578,9 @@ long WSDbtronSocket::read(void* handle,WSCuchar* buf,long size){
   }else{
     timeout = _cl_timeout *1000;
   }
+
   if (timeout > 0){
-    if (_no_delay == False){
+//    if (_no_delay == False){
       int fl = so_fcntl((int)handle,F_GETFL,0);
       if (fl == -1){
         seterr();
@@ -485,10 +591,10 @@ long WSDbtronSocket::read(void* handle,WSCuchar* buf,long size){
         seterr();
         return 0;
       }
-      _no_delay = True;
-    }
+//      _no_delay = True;
+//    }
   }else{
-    if (_no_delay != False){
+//    if (_no_delay != False){
       int fl = so_fcntl((int)handle,F_GETFL,0);
       if (fl == -1){
         seterr();
@@ -499,8 +605,8 @@ long WSDbtronSocket::read(void* handle,WSCuchar* buf,long size){
         seterr();
         return 0;
       }
-      _no_delay = False;
-    }
+//      _no_delay = False;
+//    }
   }
 
   long read_len = 0;
@@ -524,7 +630,7 @@ long WSDbtronSocket::read(void* handle,WSCuchar* buf,long size){
 //printf("timeout=%d\n",timeout);
     if (timeout > 0){
       struct timeval tm;
-      tm.tv_sec = timeout;
+      tm.tv_sec = timeout/1000;
       tm.tv_usec = 0;
       fd_set read_fds;
       FD_ZERO(&read_fds);
@@ -613,7 +719,7 @@ long WSDbtronSocket::write(void* handle,WSCuchar* buf,long size){
   }
 
   if (timeout > 0){
-    if (_no_delay == False){
+//    if (_no_delay == False){
       int fl = so_fcntl((int)handle,F_GETFL,0);
       if (fl == -1){
         seterr();
@@ -624,10 +730,10 @@ long WSDbtronSocket::write(void* handle,WSCuchar* buf,long size){
         seterr();
         return 0;
       }
-      _no_delay = True;
-    }
+//      _no_delay = True;
+//    }
   }else{
-    if (_no_delay != False){
+//    if (_no_delay != False){
       int fl = so_fcntl((int)handle,F_GETFL,0);
       if (fl == -1){
         seterr();
@@ -638,8 +744,8 @@ long WSDbtronSocket::write(void* handle,WSCuchar* buf,long size){
         seterr();
         return 0;
       }
-      _no_delay = False;
-    }
+//      _no_delay = False;
+//    }
   }
 
   long write_len = 0;
@@ -668,9 +774,9 @@ long WSDbtronSocket::write(void* handle,WSCuchar* buf,long size){
 //printf("sendto.. size=%d\n",sizeof(toaddr));
 //printf("sendto.. handle=%d ret=%d errno=%d\n",handle,ret,ret >> 16);
     }
-    if (_timeout > 0){
+    if (timeout > 0){
       struct timeval tm;
-      tm.tv_sec = _timeout;
+      tm.tv_sec = timeout/1000;
       tm.tv_usec = 0;
       fd_set write_fds;
       FD_ZERO(&write_fds);
