@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <basic.h>
 #include <WScom.h>
@@ -42,13 +38,14 @@ WSDthread* _btron_th_chandler(){
 void WSGFinitializeUnixThread(){
     WSDthread::setCreateInstanceHandler((void*)_btron_th_chandler);
 }
-
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _btron_th_init {
   public: _btron_th_init(){
-    WSDthread::setCreateInstanceHandler((void*)_btron_th_chandler);
+    WSGFinitializeUnixThread();
   };
 };
 _btron_th_init  _btron_th_init_execute;
+#endif
 
 WSDbtronThread::WSDbtronThread(){
   _mutex = cre_sem(1,SEM_EXCL | DELEXIT);
@@ -133,18 +130,28 @@ long WSDbtronThread::getStatus(){
   return WS_THREAD_RUNNING;
 }
 long WSDbtronThread::execCallback(void* data){
-  _callback_data = data;
+//  _callback_data = data;
 
   W tid = get_tid();
   MESSAGE* msg;
   char buffer[128];
   msg = (MESSAGE*)buffer;
   msg->msg_type = MS_TYPE4;
-  msg->msg_size = sizeof(W)*2;
-  memcpy(msg->msg_body.ANYMSG.msg_str,this,sizeof(W));
-  memcpy(&(msg->msg_body.ANYMSG.msg_str[4]),&tid,sizeof(W));
+  msg->msg_size = sizeof(W)*3;
+  WSDbtronThread* ptr = this;
+
+  memcpy(msg->msg_body.ANYMSG.msg_str,&ptr,sizeof(W));
+  memcpy(&(msg->msg_body.ANYMSG.msg_str[sizeof(W)]),&tid,sizeof(W));
+  memcpy(&(msg->msg_body.ANYMSG.msg_str[sizeof(W)*2]),&data,sizeof(void*));
   W ret = snd_msg(0,msg,WAIT);
 
+#ifdef _NOTSH
+#ifndef NO_THREAD
+extern int _wake_up_event();
+    _wake_up_event();
+#endif
+#endif
+
   return WS_NO_ERR;
 }
 #endif
