@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <basic.h>
 #include <btron/hmi.h>
@@ -50,6 +46,7 @@ dbprintf("_wsdbtronWindev_init_() %s:%d started..\n",__FILE__,__LINE__);
   return new WSDbtronWinDev();
 }
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _WSDbtronWinDev_init_ {
   public:_WSDbtronWinDev_init_(){
 dbprintf("_WSDbtronWinDev_init_() %s:%d started..\n",__FILE__,__LINE__);
@@ -57,6 +54,8 @@ dbprintf("_WSDbtronWinDev_init_() %s:%d started..\n",__FILE__,__LINE__);
   };
 };
 static _WSDbtronWinDev_init_ _init_to_run_WSDbtronWinDev_;
+#endif
+
 //static Atom wm_protocols = 0;
 //static Atom wm_delete = 0;
 extern TC* WSGFbtronStringToTc(char*,long*);
@@ -215,6 +214,7 @@ dbprintf("WSDbtronWinDev::setValue back color..%s:%d bg=%d\n",__FILE__,__LINE__,
              break;
     case WSDEV_TITLE_STRING:
              {
+#ifndef _NOTSH
                char* vstr = (char*)data;
                long encode = WSGIappLocaleSet()->getDefaultEncoding();
                _title_str.setString(vstr,encode);
@@ -228,6 +228,7 @@ dbprintf("WSDbtronWinDev::setValue back color..%s:%d bg=%d\n",__FILE__,__LINE__,
                    delete tstr2;
                  }
                }
+#endif
              }
              break;
     case WSDEV_FLOAT_ON_TOP:
@@ -326,13 +327,17 @@ dbprintf("### WSDbtronWinDev::createWindow %s vis=%d attr=0x%x\n",getAttachedCli
   if ( _top_float != False){
 //    attr |= WA_POPUP;
   }
-
+  if ( _modal != False){
+    attr = WA_SFULL | WA_FMOVE;
+  }
   TC* tstr2 = NULL;
+#ifndef _NOTSH
   if (strcmp(_title_str.getString(),"")){
     long tclen=0;
     tstr2 = WSGFbtronStringToTc2( _title_str.getString(WS_EN_UTF8),&tclen);
 dbprintf("WSDbtronWinDev::createWindow title=#%s# tclen=%d\n",_title_str.getString(),tclen);
   }
+#endif
   W bgc = 0x10000000;
   WSDcolor* color = WSGIappColorSet()->getColor(_bg);
   if (color != NULL){
@@ -544,11 +549,12 @@ dbprintf("WSDbtronFormDev::createPixmap() gcls_env=0x%x\n",ret);
   exposeExecute(0,0,w,h);
 
 #ifdef F_MEMGID
-  WSClistData children = getChildDevList();
-  long num = children.getNum();
+  WSClistData* children = getChildDevList();
+  long num = children->getNum();
   long i;
+  WSDbtronFormDev** child_buf =(WSDbtronFormDev**)children->_data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* child =(WSDbtronFormDev*)children[i];
+    WSDbtronFormDev* child = child_buf[i];
     child->createPixmap();
   }
 #endif
@@ -564,11 +570,12 @@ dbprintf("WSDbtronFormDev::destroyPixmap() gcls_env=0x%x\n",ret);
     _mem_gid = NULL;
   }
   WSDwindowDev::destroyPixmap();
-  WSClistData children = getChildDevList();
-  long num = children.getNum();
+  WSClistData* children = getChildDevList();
+  long num = children->getNum();
   long i;
+  WSDbtronFormDev** child_buf =(WSDbtronFormDev**)children->_data;
   for(i=0; i<num; i++){
-    WSDbtronFormDev* child =(WSDbtronFormDev*)children[i];
+    WSDbtronFormDev* child = child_buf[i];
     child->destroyPixmap();
   } 
   return WS_NO_ERR;
