@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #ifdef TE
 #include <tstring.h>
@@ -33,7 +29,9 @@
 #endif
 
 #include <basic.h>
+#ifndef _NOTSH
 #include <btron/tf.h>
+#endif
 #include <btron/WSDbtronAppDev.h>
 #include <WSDfont.h>
 #include <WSDdev.h>
@@ -68,10 +66,11 @@ char* getenv(const char*){
 #ifdef TE
 #include <btron/unixemu.h>
 #else
+#include <unix/sys/wait.h>
 extern "C" {
 extern int btron_prefork( void );
 extern pid_t btron_forkexecvp( const char *name, char * const *argv );
-
+WERR u_waitpid (int pid,int *status,int option);
 };
 #endif
 
@@ -83,6 +82,7 @@ void WSGFsetRect(RECT* r,short x,short y,WSCushort w,WSCushort h){
 }
 
 
+#ifndef _NOTSH
 TF_CTX tc_to_string_ctx = NULL;
 TF_CTX string_to_tc_ctx = NULL;
 //#define NO_LIBTF
@@ -454,9 +454,9 @@ void _btron_draw_string(TC* str,WSDfont* fn,GID gid,W fg,W bg,long op,long slen,
 
 
 void WSGFbtronDrawString(TC* str,WSDfont* fn,GID gid,W fg,W bg,long len,short x,short y,long cur,WSCbool inter_cur,long lw,long scur1,long scur2){
-  gset_chp(gid,x,y,(Bool)True);
+  gset_chp(gid,x,y-1,(Bool)True);
   _btron_draw_string(str,fn,gid,fg,bg,G_AND,len,cur,inter_cur,lw,scur1,scur2,x,y);
-  gset_chp(gid,x,y,(Bool)True);
+  gset_chp(gid,x,y-1,(Bool)True);
   _btron_draw_string(str,fn,gid,fg,bg,G_OR,len,cur,inter_cur,lw,scur1,scur2,x,y);
 }
 short WSGFbtronGetStringWidth(TC* str,WSDfont* fn,long slen){
@@ -465,7 +465,7 @@ short WSGFbtronGetStringWidth(TC* str,WSDfont* fn,long slen){
   long ptr = 0;
   FSSPEC* fnt = (FSSPEC*)fn->getValue1();
   FSSPEC* fnt_ascii = (FSSPEC*)fn->getValue3();
-  W fdesc = fn->getValue2();
+//  W fdesc = fn->getValue2();
   GID fgid = WSGIbtronAppDev()->getFontGid();
   W ret = 0;
   while(1){
@@ -526,6 +526,7 @@ short WSGFbtronGetStringWidth(TC* str,WSDfont* fn,long slen){
   }
   return ret;
 }
+#endif
 
 int main1(int argc,char** argv){
 #ifdef TDBG
@@ -617,8 +618,52 @@ void WSGFbtronDeleteMemory(char* ptr){
   }
   delete ptr;
 }
+#ifndef TE
+WSClistData _pid_list;
+W _pid_sem = -1;
+void _watch_pid_proc(int val){
+  pid_t pid = (pid_t)val;
+  if (pid > 0){
+    int status;
+    while(1){
+      pid_t ret = waitpid(pid,&status,WNOHANG);
+      if (ret == 0){
+        wai_prc(100);
+        continue;
+      }
+      break;
+    }
+    wai_sem(_pid_sem,T_FOREVER);
+    int i;
+    int num = _pid_list.getNum();
+    for(i=0; i<num; i++){
+      pid_t p = (pid_t)_pid_list[i];
+      if (p == pid){
+        _pid_list.delPos(i);
+        break;
+      }
+    }
+    sig_sem(_pid_sem);
+  }
+  ext_tsk();
+}
+void btron_kill_all_child_proc(){
+  wai_sem(_pid_sem,T_FOREVER);
+  int i;
+  int num = _pid_list.getNum();
+  for(i=0; i<num; i++){
+    pid_t p = (pid_t)_pid_list[i];
+    ter_prc(p,0,TERM_NRM);
+  }
+  _pid_list.clear();
+  sig_sem(_pid_sem);
+}
+#endif
 int btron_system(char* exe_name){
 #ifndef TE
+  if (_pid_sem == -1){
+    _pid_sem = cre_sem(1,SEM_SYNC|DELEXIT );
+  }
   WSCstring exename;
   exename = WSGIappFileSystem()->adjustFileName(exe_name);
   exename.delHeadSpace();
@@ -641,6 +686,17 @@ int btron_system(char* exe_name){
     delete args[i];
   }
   delete args;
+  wai_sem(_pid_sem,T_FOREVER);
+  _pid_list.add((void*)err);
+  sig_sem(_pid_sem);
+  cre_tsk(_watch_pid_proc,-1,(int)err);
+
 #endif
   return WS_NO_ERR;
 }
+
+#ifdef _NOTSH
+W WSGFgetFontGid(){
+  return WSGIbtronAppDev()->getFontGid();
+}
+#endif
