@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #define _ANSI_SOURCE
 #include <basic.h>
@@ -39,12 +35,16 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#define WS_READ_SIZE     1024
+#define WS_READ_MIN_SIZE  256
+#define WS_READ_ERR_SIZE  256
+#define WS_MAX_PATH_LEN  2048
+#define WS_MAX_URLS        64
+
 static char WS_CONTENT_TYPE[]="Content-type:";
 static char WS_CONTENT_LEN[]="Content-length:";
 static char WS_POST_CONTENT_TYPE[]="application/x-www-form-urlencoded";
 
-void WSGFtcpInit(){ }
-
 static void SetSocketBlockingState(int* sock,int blocking){
   int ret;
 //#ifdef O_NONBLOCK
@@ -54,94 +54,24 @@ static void SetSocketBlockingState(int* sock,int blocking){
 //  int val=(!n_blocking);
 //  ret = ioctl(*sock, FIONBIO, &val);
 //#endif
-  if (ret == -1) {
+  if (ret == -1){
     fprintf(stderr, "Connection error. <%s>\n", (blocking ? "blocking" : "non-blocking"));
   }
 }
 
-#define MIN_READ_ERR_SIZE 0x100
-long WSGFtcpRead(int sock,char** outbuf,int* size){
-  long bsize = 0x400;
-  long eof_fl = False;
-  long len = 0;
-
-  char *buf1 = new char[bsize];
-  memset(buf1,0, bsize * sizeof(char));
-
-  if(size != NULL){
-    *size = 0;
-  }
-  *outbuf = NULL;
-  if (buf1 == NULL) {
-    fprintf(stderr, "Memory allocation error.\n");
-    return WS_MEM_ERR;
-  }
-  do{
-    int bytes_read;
-    if (bsize - len < MIN_READ_ERR_SIZE) {
-      bsize += 0x400;
-      delete buf1;
-      buf1 = new char[bsize];
-      if ( buf1 == NULL ){
-        fprintf(stderr, "Memory allocation error.\n");
-        return WS_MEM_ERR;
-      }
-    }
-    bytes_read = so_read(sock, &buf1[len], bsize-len-1);
-    if (bytes_read <= 0) {
-      if (bytes_read < 0 && (bytes_read == EX_NOTCONN || bytes_read == EX_CONNRESET || bytes_read == EX_BADF)) {
-        fprintf(stderr, "Network read error.\n");
-        delete buf1;
-        return WS_READ_ERR;
-      }else if (bytes_read < 0) {
-        fprintf(stderr, "Network error.\n");
-        delete buf1;
-        return WS_NET_ERR;
-      }
-      eof_fl = True;
-    }else{
-      len += bytes_read;
-    }
-  }while(!eof_fl);
-
-  buf1[len] = '\0';
-  *outbuf = buf1;
-  if(size != NULL){
-    *size = (len+1);
-  }
-  return WS_NO_ERR;
-}
-
-
-long WSGFtcpWrite(int sock,char* data,int size){
-  if (data == NULL){
-    return WS_NO_ERR;
-  }
-  int ret = so_write(sock, data, (int)size);
-  if (ret <= 0){
-    if (ret == 0){
-      fprintf(stderr, "0 bytes written.\n");
-    }else if ((ret == EX_NOTCONN || ret == EX_CONNRESET || ret == EX_BADF)) {
-      fprintf(stderr, "Network write error.\n");
-      return WS_WRITE_ERR;
-    }
-  }
-  return WS_NO_ERR;
-}
-
-long WSGFtcpConnect(char* host,int  port,int* sock){
+static long _connect_to(char* host,long port,int* sock){
   int status = WS_NO_ERR;
   struct sockaddr_in addr;
   struct sockaddr_in *sin=(&addr);
   struct hostent hostent1;
   struct hostent *hostent=&hostent1;
 
-  if (*host >= '0' && *host <= '9') {
+  if (*host >= '0' && *host <= '9'){
     sin->sin_addr.s_addr = inet_addr(host);
   }else{
     B tmpbuf[HBUFLEN];
     ERR ret = so_gethostbyname(host,hostent,tmpbuf);
-    if (ret < 0) {
+    if (ret < 0){
       fprintf(stderr, "Cannot find Internet node name.\n");
       return WS_HOSTN_ERR;
     }
@@ -154,26 +84,22 @@ long WSGFtcpConnect(char* host,int  port,int* sock){
   SetSocketBlockingState(sock, False);
 
   status = so_connect(*sock, (struct sockaddr*)&addr, sizeof(addr));
-  if ((status < 0) && (status==EX_INPROGRESS || status==EX_WOULDBLOCK)) {
+  if ((status < 0) && (status==EX_INPROGRESS || status==EX_WOULDBLOCK)){
     struct timeval timeout;
     int ret=0;
 
     timeout.tv_sec = 0;
     timeout.tv_usec = 100000;
-    while (ret <= 0) {
+    while (ret <= 0){
       fd_set writefds;
 
       FD_ZERO(&writefds);
       FD_SET(*sock, &writefds);
-#ifdef HPUX
-      ret = so_select(FD_SETSIZE, NULL, (int*)&writefds, NULL, &timeout);
-#else //HPUX
       ret = so_select(FD_SETSIZE, NULL, &writefds, NULL, &timeout);
-#endif //HPUX
-      if ((ret < 0)&&(ret != EX_ALREADY)) {
+      if ((ret < 0)&&(ret != EX_ALREADY)){
         status = ret;
         break;
-      }else if (ret > 0) {
+      }else if (ret > 0){
         status = so_connect(*sock, (struct sockaddr*)&addr,sizeof(addr));
         if ((status < 0)&&(status == EX_ISCONN)){
           status = WS_NO_ERR;
@@ -191,7 +117,7 @@ long WSGFtcpConnect(char* host,int  port,int* sock){
       }
     }
   }
-  if (status >= 0) {
+  if (status >= 0){
     SetSocketBlockingState(sock, True);
   }else{
     so_close(*sock);
@@ -199,9 +125,6 @@ long WSGFtcpConnect(char* host,int  port,int* sock){
   return status;
 }
 
-
-#define WS_MAX_PATH_LEN 2048
-
 #ifdef DS
 extern "C"{
   extern int gethostname(char*,WSCuint);
@@ -268,24 +191,10 @@ static char          _default_file[] = "index.html";
 static char          _http_proxy[WS_MAX_PATH_LEN+1];
 static WSCurlCache* _first_url_cache = NULL;
 static WSCurlCache* _last_url_cache  = NULL;
-static long          _max_url_cache   = -1;
-static long          _cur_url_cache   = 0;
-
+static long          _max_url_cache   = WS_MAX_URLS;
+static long          _current_url_cache   = 0;
 static long          _http_initialized = 0;
-long WSGFhttpInit(){
-  if (_http_initialized != 0){
-    return 0;
-  }
-  _http_initialized = 1;
-extern void WSGFtcpInit();
-  WSGFtcpInit();
-  char* httpp = getenv("WSHTPROXY");
-  if (httpp != NULL){
-    strcpy(_http_proxy,httpp);
-  }
-  return 0;
-}
-
+static long          _url_cache_initialized = 0;
 
 void _get_user_name(char* buffer,long buf_size){
    char user_name[1024];
@@ -294,7 +203,7 @@ void _get_user_name(char* buffer,long buf_size){
    sprintf(user_name, "ws@");
    total = strlen(user_name);
 #if 0
-   if (gethostname(&user_name[total], sizeof(user_name)-1-total) < 0) {
+   if (gethostname(&user_name[total], sizeof(user_name)-1-total) < 0){
       sprintf(&user_name[total], "UNKNOWN");
    }else{
 #endif
@@ -304,7 +213,7 @@ void _get_user_name(char* buffer,long buf_size){
       ERR ret = so_gethostbyname(&user_name[total],&p_hostent,tmpbuf);
 
       if (ret == 0 && p_hostent.h_name != NULL &&
-            p_hostent.h_name[0] != '\0') {
+            p_hostent.h_name[0] != 0){
          strcpy(&user_name[total], p_hostent.h_name);
       }
    }
@@ -314,66 +223,63 @@ void _get_user_name(char* buffer,long buf_size){
 static char *_get_url_name(char *url){
    char *ptr = strchr(url, '#');
 
-   if (ptr != NULL) {
+   if (ptr != NULL){
       char *return_buf;
 
-      *ptr = '\0';
+      *ptr = 0;
       return_buf = WSGFstrdup(url);
       *ptr = '#';
       return return_buf;
    }
    return WSGFstrdup(url);
 }
-
 static void _init_cache(){
-  if (_max_url_cache == (-1)) {
-    _max_url_cache = 40;
-    _cur_url_cache = 0;
-    _first_url_cache = _last_url_cache = NULL;
+  if (_url_cache_initialized == 0){
+    _url_cache_initialized = 1;
+    _first_url_cache = NULL;
+    _last_url_cache = NULL;
+    _current_url_cache = 0;
   }
 }
 
 static void _remove_cache( struct WSCurlCache *url){
-  if (url == NULL) return;
-  if (url->parent == NULL) {
-    _first_url_cache = url->child;
-  }else{
-    url->parent->child = url->child;
+  if (url == NULL){
+    return;
   }
-  if (url->child == NULL) {
+  if (url->child == NULL){
     _last_url_cache = url->parent;
   }else{
     url->child->parent = url->parent;
   }
+  if (url->parent == NULL){
+    _first_url_cache = url->child;
+  }else{
+    url->parent->child = url->child;
+  }
   url->parent = url->child = NULL;
-  _cur_url_cache--;
+  _current_url_cache--;
 }
 
 
 static void _add_cache(struct WSCurlCache* parent_url,
                            struct WSCurlCache* child_url,
-                           struct WSCurlCache* url) {
-  url->parent = parent_url;
-  url->child = child_url;
-  if (parent_url == NULL) {
+                           struct WSCurlCache* url){
+  if (parent_url == NULL){
     _first_url_cache = url;
   }else{
     parent_url->child = url;
   }
-  if (child_url == NULL) {
+  if (child_url == NULL){
     _last_url_cache = url;
   }else{
     child_url->parent = url;
   }
-  _cur_url_cache++;
+  url->parent = parent_url;
+  url->child = child_url;
+  _current_url_cache++;
 }
 
-void _update_url( struct WSCurlCache *url){
-   _remove_cache(url);
-   _add_cache(_last_url_cache, NULL, url);
-}
-
-static WSCurlCache *_find_url_cache(char* url_name,long update_lru){
+static WSCurlCache *_search_cache(char* url_name,long update_lru){
   char *url_base_name = _get_url_name(url_name);
   struct WSCurlCache *url;
 
@@ -381,11 +287,12 @@ static WSCurlCache *_find_url_cache(char* url_name,long update_lru){
   if (url_base_name == NULL){
     return NULL;
   }
-  for (url=_last_url_cache; url != NULL; url=url->parent) {
-    if (strcmp(url_base_name, url->url_base_name) == 0) {
+  for (url=_last_url_cache; url != NULL; url=url->parent){
+    if (strcmp(url_base_name, url->url_base_name) == 0){
       delete url_base_name;
       if (update_lru){
-        _update_url(url);
+        _remove_cache(url);
+        _add_cache(_last_url_cache, NULL, url);
       }
       return url;
     }
@@ -397,11 +304,11 @@ static WSCurlCache *_find_url_cache(char* url_name,long update_lru){
 static char *_get_proxy_info(char* proxy_spec,long def_port,long* pn_port){
    char *colon_ptr=strchr(proxy_spec, ':'), *return_buf=NULL;
 
-   if (colon_ptr == NULL) {
+   if (colon_ptr == NULL){
       *pn_port = def_port;
       return_buf = WSGFstrdup(proxy_spec);
    }else{
-      *colon_ptr = '\0';
+      *colon_ptr = 0;
       *pn_port = atoi(&colon_ptr[1]);
       return_buf = WSGFstrdup(proxy_spec);
       *colon_ptr = ':';
@@ -411,7 +318,7 @@ static char *_get_proxy_info(char* proxy_spec,long def_port,long* pn_port){
 
 static long _usr_agent_name_inited = False;
 
-static void _init_client_name() {
+static void _init_client_name(){
    if (_usr_agent_name_inited){
      return;
    }
@@ -420,33 +327,6 @@ static void _init_client_name() {
    _get_user_name(_user_name_buf, sizeof(_user_name_buf));
 }
 
-long WSGFhttpConnect(char* host,long port,int* sock) {
-   long rc, len=strlen(host)+80;
-   char *msg=new char[len+1];
-
-   if (msg == NULL) {
-      WSMFtrace("Memory allocation failed.\n");
-      return WS_MEM_ERR;
-   }
-
-#ifdef DTRACE
-   WSMFtrace("Making an HTTP connection to \"%s:%1d\"...\n", host, port);
-#endif
-
-   rc = WSGFtcpConnect(host, port, sock);
-
-   if (rc == WS_NO_ERR) {
-#ifdef DTRACE
-      WSMFtrace("HTTP: connection to \"%s:%1d\" established.\n", host, port);
-#endif
-   }else{
-      WSMFtrace("Fail to connect to HTTP server on \"%s:%1d\".\n", host, port);
-   }
-   delete msg;
-
-   return rc;
-}
-
 static char *accept_strs[] = {
   "text/plain",
   "text/html",
@@ -455,12 +335,12 @@ static char *accept_strs[] = {
   NULL
 };
 
-static char *_append_acstr(char* buf){
+static char *_add_acstr(char* buf){
   char **ptr;
   WSCstring tmp;
   tmp.setString(buf);
   delete buf;
-  for (ptr=accept_strs; *ptr != NULL; ptr++) {
+  for (ptr=accept_strs; *ptr != NULL; ptr++){
     WSCstring tmp2;
     tmp2.setString(tmp.getString());
     tmp2.addString("Accept: ");
@@ -472,7 +352,7 @@ static char *_append_acstr(char* buf){
   return buf;
 }
 
-static char *_append_string(char* buf,char* name,char* value){
+static char *_add_string(char* buf,char* name,char* value){
   WSCstring tmp;
   if (buf != NULL){
     tmp.setString(buf);
@@ -491,49 +371,51 @@ static char *_append_string(char* buf,char* name,char* value){
   }
 }
 
-static char *_append_uastr(char* buf){
+static char *_add_uastr(char* buf){
   _init_client_name();
-  return _append_string(buf, "User-Agent", _client_name);
+  return _add_string(buf, "User-Agent", _client_name);
 }
 
-static char *_append_from(char* buf){
+static char *_add_from(char* buf){
    _init_client_name();
-   return _append_string(buf, "From", _user_name_buf);
+   return _add_string(buf, "From", _user_name_buf);
 }
 
-static char *_append_typestr(char* buf){
-   return _append_string(buf, "Content-type", WS_POST_CONTENT_TYPE);
+static char *_add_typestr(char* buf){
+   return _add_string(buf, "Content-type", WS_POST_CONTENT_TYPE);
 }
 
-static char* _append_lenstr(char* buf,long content_length){
+static char* _add_lenstr(char* buf,long clen){
    char len_str[20];
-   sprintf(len_str, "%1d", content_length);
-   return _append_string(buf, "Content-length", len_str);
+   sprintf(len_str, "%1d", clen);
+   return _add_string(buf, "Content-length", len_str);
 }
 
-static char* _append_connect_str(char* buf,FILE* fp,long content_length){
-  long cur_len=strlen(buf), bytes_read, total_read=0;
-  long new_len=cur_len+content_length+1;
-  char tmp_buf[512];
-  char* new_buf = new char[new_len];
-  if (new_buf == NULL){
+static char* _add_connect_str(char* buf,FILE* fp,long clen){
+  long buf_len=strlen(buf);
+  long new_len=buf_len+clen+1;
+  char read_buf[1024];
+  char* ret_buf = new char[new_len];
+  if (ret_buf == NULL){
     return NULL;
   }
-  strcpy(new_buf,buf);
+  strcpy(ret_buf,buf);
   delete buf;
-  while ((bytes_read=fread(tmp_buf, sizeof(char), sizeof(tmp_buf),fp)) > 0) {
-    if (bytes_read+total_read > content_length) {
-      bytes_read = content_length-total_read;
-      fprintf(stderr, "Lines too long in _append_connect_str().\n");
+  long total_len=0;
+  long read_len;
+  while ((read_len=fread(read_buf, sizeof(char), sizeof(read_buf),fp)) > 0){
+    if (read_len+total_len > clen){
+      read_len = clen-total_len;
+      fprintf(stderr, "Lines too long in _add_connect_str().\n");
     }
-    strncpy(&new_buf[cur_len+total_read], tmp_buf, bytes_read);
-    total_read += bytes_read;
+    strncpy(&ret_buf[buf_len+total_len], read_buf, read_len);
+    total_len += read_len;
   }
-  new_buf[cur_len + content_length] = '\0';
-  return new_buf;
+  ret_buf[buf_len + clen] = 0;
+  return ret_buf;
 }
 
-static char* append_return(char* buf){
+static char* add_linefeed(char* buf){
   if (buf != NULL){
     WSCstring tmp;
     tmp.setString(buf);
@@ -545,59 +427,64 @@ static char* append_return(char* buf){
   }
 }
 
-long WSGFhttpWrite(int sock,char* path){
-  long status=0, total_sz=0;
-  char *buf=(char*)calloc(strlen(path)+5+2+31, sizeof(char));
+static long _write_data(int sock,char* path){
+  long status=0;
+  long total_size=0;
+  char *buf=new char[strlen(path)+64];
   FILE *fp=NULL;
 
-  if (buf == NULL) {
+  if (buf == NULL){
     fprintf(stderr, "Memory allocation failed.\n");
     return WS_MEM_ERR;
   }
-  if (_cgi_query) {
-     sprintf(buf, "POST %s %s\r\n", path, _ws_http_ver);
+  if (_cgi_query){
+    sprintf(buf, "POST %s %s\r\n", path, _ws_http_ver);
   }else{
-     sprintf(buf, "GET %s %s\r\n", path, _ws_http_ver);
+    sprintf(buf, "GET %s %s\r\n", path, _ws_http_ver);
   }
-  buf=_append_acstr(buf);
-  buf=_append_uastr(buf);
-  buf=_append_from(buf);
+  buf=_add_acstr(buf);
+  buf=_add_uastr(buf);
+  buf=_add_from(buf);
 
-  if (_cgi_query && _fname_cgi_query != NULL) {
-     long bytes_read;
-     char tmp_buf[512];
+  if (_cgi_query && _fname_cgi_query != NULL){
+    long read_len;
+    char read_buf[1024];
 
-     if ((fp=fopen(_fname_cgi_query, "r")) == NULL) {
-        fprintf(stderr, "Fail to open '%s' for read.\n",
-              _fname_cgi_query);
-        return WS_READ_ERR;
-     }
-     while ((bytes_read=fread(tmp_buf, sizeof(char), sizeof(tmp_buf),fp)) > 0){
-        total_sz += bytes_read;
-     }
-     rewind(fp);
-     buf=_append_typestr(buf);
-     buf=_append_lenstr(buf, total_sz);
+    if ((fp=fopen(_fname_cgi_query, "r")) == NULL){
+      fprintf(stderr, "Failed to open file:%s.\n",_fname_cgi_query);
+      return WS_READ_ERR;
+    }
+    while ((read_len=fread(read_buf, sizeof(char), sizeof(read_buf),fp)) > 0){
+      total_size += read_len;
+    }
+    rewind(fp);
+    buf=_add_typestr(buf);
+    buf=_add_lenstr(buf, total_size);
   }
-  buf=append_return(buf);
-  if (fp != NULL) {
-      buf = _append_connect_str(buf, fp, total_sz);
-      fclose(fp);
-      if (buf == NULL) {
-         fprintf(stderr, "Memory allocation failed.\n");
-         return WS_MEM_ERR;
-      }
+  buf=add_linefeed(buf);
+  if (fp != NULL){
+    buf = _add_connect_str(buf, fp, total_size);
+    fclose(fp);
+    if (buf == NULL){
+      fprintf(stderr, "Memory allocation failed.\n");
+      return WS_MEM_ERR;
+    }
   }
 
-#ifdef DTRACE
-   WSMFtrace("HTTP: sending requests...\n");
-#endif
-
-  status = WSGFtcpWrite(sock, buf, (long)strlen(buf));
+  status = WS_NO_ERR;
+  int ret = so_write(sock, buf, (int)strlen(buf));
+  if (ret <= 0){
+    if (ret == 0){
+      fprintf(stderr, "0 bytes written.\n");
+    }else if ((ret == EX_NOTCONN || ret == EX_CONNRESET || ret == EX_BADF)){
+      fprintf(stderr, "Network write error.\n");
+      status =  WS_WRITE_ERR;
+    }
+  }
   delete buf;
 
-  if (status != WS_NO_ERR) {
-     WSMFtrace("HTTP: fail to send requests.\n");
+  if (status != WS_NO_ERR){
+    WSMFtrace("HTTP: Failed to send requests.\n");
   }
   return status;
 }
@@ -605,11 +492,10 @@ long WSGFhttpWrite(int sock,char* path){
 char *_http_extxt(char* buf,long* bsize,long* fl,char** ctype){
   char* ptr = strchr(buf, '\n');
   char* content_type = NULL;
-  char* line_ptr = buf;
-  long  content_length = -1;
-  long  len2 = strlen(WS_CONTENT_LEN);
-  long  len1 = strlen(WS_CONTENT_TYPE);
-  long  text_type = False;
+  long  clen = -1;
+  long  cl_len = strlen(WS_CONTENT_LEN);
+  long  ct_len = strlen(WS_CONTENT_TYPE);
+  long  text_fl = False;
 
   if (bsize != NULL){
     *bsize = 0;
@@ -620,109 +506,109 @@ char *_http_extxt(char* buf,long* bsize,long* fl,char** ctype){
   if (ctype != NULL){
     *ctype = NULL;
   }
-  while (ptr != NULL) {
+  char* current_ptr = buf;
+  while (ptr != NULL){
     char *parent_ptr = ptr;
-    if (parent_ptr != line_ptr && *(--parent_ptr) == '\r') {
-      *parent_ptr = '\0';
+    if (parent_ptr != current_ptr && *(--parent_ptr) == '\r'){
+      *parent_ptr = 0;
     }else{
       parent_ptr = NULL;
-      *ptr = '\0';
+      *ptr = 0;
     }
-    if (*line_ptr == '\0') {
-      if (parent_ptr == NULL) {
+    if (*current_ptr == 0){
+      if (parent_ptr == NULL){
         *ptr = '\n';
       }else{
         *parent_ptr = '\r';
       }
-      line_ptr = &ptr[1];
+      current_ptr = &ptr[1];
       break;
     }
-    if (content_type == NULL && WSGFstrnccmp(line_ptr, WS_CONTENT_TYPE, len1) == 0) {
-      content_type = WSGFstrdup(&line_ptr[len1]);
-      if (content_type != NULL) {
+    if (content_type == NULL &&
+        WSGFstrnccmp(current_ptr, WS_CONTENT_TYPE, ct_len) == 0){
+      content_type = WSGFstrdup(&current_ptr[ct_len]);
+      if (content_type != NULL){
         WSGFtrimSpace(content_type);
-        if (ctype != NULL) {
+        if (ctype != NULL){
           *ctype = WSGFstrdup(content_type);
         }
-        if (WSGFstrnccmp(content_type, "text/", 5) == 0) {
-          text_type = True;
-          if (strcmp(&content_type[5], "html") == 0) {
+        if (WSGFstrnccmp(content_type, "text/", 5) == 0){
+          if (strcmp(&content_type[5], "html") == 0){
             if (fl != NULL){
               *fl = True;
             }
           }
+          text_fl = True;
         }
       }
-    }else if (content_length == -1 &&
-           WSGFstrnccmp(line_ptr, WS_CONTENT_LEN, len2) == 0) {
-      char *tmp_ptr = WSGFstrdup(&line_ptr[len2]);
-      if (tmp_ptr != NULL) {
-        WSGFtrimSpace(tmp_ptr);
-        content_length = atoi(tmp_ptr);
-        delete tmp_ptr;
+    }else
+    if (clen == -1 && WSGFstrnccmp(current_ptr,WS_CONTENT_LEN,cl_len) == 0){
+      char *tstr = WSGFstrdup(&current_ptr[cl_len]);
+      if (tstr != NULL){
+        WSGFtrimSpace(tstr);
+        clen = atoi(tstr);
+        delete tstr;
       }
     }
-    if (parent_ptr == NULL) {
+    if (parent_ptr == NULL){
       *ptr = '\n';
     }else{
       *parent_ptr = '\r';
     }
-    line_ptr = &ptr[1];
-    ptr = strchr(line_ptr, '\n');
+    current_ptr = &ptr[1];
+    ptr = strchr(current_ptr, '\n');
   }
   if (content_type != NULL){
     delete content_type;
   }
-  if (text_type){
-    long buf_len = strlen(line_ptr);
+  if (text_fl){
+    long buf_len = strlen(current_ptr);
     char *return_buf;
 
-    if (content_length == (-1)) {
-      content_length = buf_len;
-      return_buf = (char*)calloc(buf_len+1, sizeof(char));
+    if (clen == (-1)){
+      clen = buf_len;
+      return_buf = new char[buf_len+1];
     }else{
-      return_buf = (char*)calloc(content_length+1, sizeof(char));
+      return_buf = new char[clen+1];
     }
 
-    if (return_buf == NULL) {
+    if (return_buf == NULL){
       WSMFtrace("Memory allocation failed.\n");
       return NULL;
     }
-    if (buf_len <= content_length) {
-      memcpy(return_buf, line_ptr, content_length);
+    if (buf_len <= clen){
+      memcpy(return_buf, current_ptr, clen);
     }else{
-      while (buf_len > content_length) {
-      if (*line_ptr == '\r' || *line_ptr == '\n') {
-        line_ptr++;
+      while (buf_len > clen){
+      if (*current_ptr == '\r' || *current_ptr == '\n'){
+        current_ptr++;
         buf_len--;
       }else{
         break;
       }
     }
-    memcpy(return_buf, line_ptr, content_length);
+    memcpy(return_buf, current_ptr, clen);
   }
-  return_buf[content_length] = '\0';
-  if (bsize != NULL) *bsize = (content_length+1);
+  return_buf[clen] = 0;
+  if (bsize != NULL) *bsize = (clen+1);
     return return_buf;
-  }else if (content_length != (-1)) {
-    char *return_buf=(char*)calloc(content_length+1, sizeof(char));
+  }else if (clen != (-1)){
+    char *return_buf= new char[clen+1];
 
-    if (return_buf == NULL) {
+    if (return_buf == NULL){
        WSMFtrace("Memory allocation failed.\n");
        return NULL;
     }
-    memcpy(return_buf, line_ptr, content_length);
-    return_buf[content_length] = '\0';
-    if (bsize != NULL) *bsize = (content_length+1);
+    memcpy(return_buf, current_ptr, clen);
+    return_buf[clen] = 0;
+    if (bsize != NULL) *bsize = (clen+1);
     return return_buf;
   }
   return NULL;
 }
 
-#define MIN_READ_SIZE 0x100
-
-long WSGFhttpRead(int sock,char** pbuffer,long* bsize){
-  long  size = 0x400;
+static long _read_data(int sock,char** pbuffer,long* bsize){
+  long  size = WS_READ_SIZE;
   long  len  = 0;
   char* buf = new char[size];
 
@@ -731,40 +617,40 @@ long WSGFhttpRead(int sock,char** pbuffer,long* bsize){
     *bsize = 0;
   }
   *pbuffer = NULL;
-  if (buf == NULL) {
+  if (buf == NULL){
     WSMFtrace("Memory allocation failed.\n");
     return WS_MEM_ERR;
   }
 
   long  status = WS_NO_ERR;
-  long  end_of_file = False;
-  while (status == WS_NO_ERR && !end_of_file){
-    long bytes_read;
-    if (size - len < MIN_READ_SIZE) {
-      size += 0x400;
+  long  eof_fl = False;
+  while (status == WS_NO_ERR && !eof_fl){
+    long read_len;
+    if (size - len < WS_READ_MIN_SIZE){
+      size += WS_READ_SIZE;
       char* new_buf = new char[size];
-      memcpy(new_buf,buf,size-0x400);
+      memcpy(new_buf,buf,size-WS_READ_SIZE);
       delete buf;
       buf = new_buf;
     }
-    bytes_read = so_read(sock, &buf[len], size-len-1);
-    if (bytes_read <= 0) {
-      if (bytes_read < 0 && (bytes_read == EX_NOTCONN || bytes_read == EX_CONNRESET ||
-          bytes_read == EX_BADF)) {
+    read_len = so_read(sock, &buf[len], size-len-1);
+    if (read_len <= 0){
+      if (read_len < 0 && (read_len == EX_NOTCONN || read_len == EX_CONNRESET ||
+          read_len == EX_BADF)){
         WSMFtrace("Network read error.\n");
         status = WS_READ;
-      }else if (bytes_read < 0) {
+      }else if (read_len < 0){
         WSMFtrace("Network error.\n");
         status = WS_NET_ERR;
       }
-      end_of_file = True;
+      eof_fl = True;
     }else{
-      len += bytes_read;
+      len += read_len;
     }
   }
 
-  if (status == WS_NO_ERR) {
-    buf[len] = '\0';
+  if (status == WS_NO_ERR){
+    buf[len] = 0;
     *pbuffer = buf;
     if (bsize != NULL){
       *bsize = (len+1);
@@ -778,46 +664,51 @@ long WSGFhttpRead(int sock,char** pbuffer,long* bsize){
   return status;
 }
 
-static long _url_parse(char* url,char** protocol,char** host,long* port,char** path){
+static long _get_url_data(char* url,char** protocol,char** host,long* port,char** path){
   char *ptr = strchr(url, ':');
-
-  *protocol = *host = *path = 0;
   if (ptr == NULL){
     return WS_FORMAT_ERR;
   }
 
-  *ptr = '\0';
+  *protocol = NULL;
+  *host = NULL;
+  *path = NULL;
+
+  *ptr = 0;
   *protocol = WSGFstrdup(url);
   *ptr++ = ':';
-  if (strncmp(ptr, "//", 2) == 0) {
-    char *tmp_host=(&ptr[2]), *port_ptr;
+  if (strncmp(ptr, "//", 2) != 0){
+    *path = WSGFstrdup(ptr);
+    *host = WSGFstrdup("localhost");
+  }else{
+    char *port_ptr = NULL;
+    char *host_ptr = &ptr[2];
 
-    if ((ptr=strchr(tmp_host, '/')) == NULL) {
+    if ((ptr=strchr(host_ptr, '/')) == NULL){
       *path = WSGFstrdup("");
     }else{
       *path = WSGFstrdup(ptr);
-      *ptr = '\0';
+      *ptr = 0;
     }
-    if ((port_ptr=strchr(tmp_host, ':')) != NULL) {
-      *port_ptr = '\0';
+    if ((port_ptr=strchr(host_ptr, ':')) == NULL){
+      *host = WSGFstrdup(host_ptr);
+    }else{
+      *port_ptr = 0;
       *port = (long)atoi(&port_ptr[1]);
-      *host = WSGFstrdup(tmp_host);
+      *host = WSGFstrdup(host_ptr);
       *port_ptr = ':';
-      if (*port <= 0) {
+      if (*port <= 0){
         return WS_FORMAT_ERR;
       }
-    }else{
-       *host = WSGFstrdup(tmp_host);
     }
-    if (ptr != NULL) *ptr = '/';
-  }else{
-    *host = WSGFstrdup("localhost");
-    *path = WSGFstrdup(ptr);
+    if (ptr != NULL){
+      *ptr = '/';
+    }
   }
   return WS_NO_ERR;
 }
 
-void _update_url_cache(char* url_name,char* rbuf,char* ctype, long rbuf_size, long fl){
+void _update_cache(char* url_name,char* rbuf,char* ctype, long rbuf_size, long fl){
   if (rbuf == NULL){
     rbuf = WSGFstrdup("");
   }
@@ -828,9 +719,11 @@ void _update_url_cache(char* url_name,char* rbuf,char* ctype, long rbuf_size, lo
   WSCurlCache *url;
 
   _init_cache();
-  if (url_base_name == NULL) return;
-  for (url=_last_url_cache; url != NULL; url=url->parent) {
-    if (strcmp(url_base_name, url->url_base_name) == 0) {
+  if (url_base_name == NULL){
+    return;
+  }
+  for (url=_last_url_cache; url != NULL; url=url->parent){
+    if (strcmp(url_base_name, url->url_base_name) == 0){
       break;
     }
   }
@@ -838,7 +731,7 @@ void _update_url_cache(char* url_name,char* rbuf,char* ctype, long rbuf_size, lo
     _remove_cache(url);
     delete url;
   }else{
-    if (_cur_url_cache >= _max_url_cache) {
+    if (_current_url_cache >= _max_url_cache){
       url = _first_url_cache;
       _remove_cache(url);
       if (url != NULL){
@@ -847,7 +740,7 @@ void _update_url_cache(char* url_name,char* rbuf,char* ctype, long rbuf_size, lo
     }
   }
   url = new WSCurlCache;
-  if (url == NULL) {
+  if (url == NULL){
     WSMFtrace("Memory Allocate error\n");
     delete url_base_name;
     return;
@@ -866,7 +759,13 @@ long WSGFloadRemoteFile(char* url,char** pbuffer,char** ctype,
   char* proxy_host = NULL;
   struct WSCurlCache *curl = NULL;
 
-  WSGFhttpInit();
+  if (_http_initialized == 0){
+    char* httpp = getenv("WSHTPROXY");
+    if (httpp != NULL){
+      strcpy(_http_proxy,httpp);
+    }
+    _http_initialized = 1;
+  }
   if (bsize != NULL){
     *bsize = 0;
   }
@@ -876,9 +775,9 @@ long WSGFloadRemoteFile(char* url,char** pbuffer,char** ctype,
   *pbuffer = NULL;
 
   if (!reload){
-    curl = _find_url_cache(url, True);
+    curl = _search_cache(url, True);
   }
-  if (curl != NULL && curl->rbuf_size > 0 && curl->rbuf != NULL) {
+  if (curl != NULL && curl->rbuf_size > 0 && curl->rbuf != NULL){
     *pbuffer = new char[curl->rbuf_size];
     memset(*pbuffer,0,curl->rbuf_size);
     if (*pbuffer == NULL){
@@ -886,7 +785,7 @@ long WSGFloadRemoteFile(char* url,char** pbuffer,char** ctype,
     }
     memcpy(*pbuffer, curl->rbuf, curl->rbuf_size);
     *bsize = curl->rbuf_size;
-    if (ctype != NULL) {
+    if (ctype != NULL){
       *ctype = WSGFstrdup(curl->content_type);
     }
     *fl = curl->fl;
@@ -897,11 +796,11 @@ long WSGFloadRemoteFile(char* url,char** pbuffer,char** ctype,
   char* host = NULL;
   char* path = NULL;
   char  port_str[20];
-  port_str[0] = '\0';
+  port_str[0] = 0;
   long  port = 0;
   long  proxy_port = 0;
-  long status = _url_parse(url, &protocol, &host, &port, &path);
-  if (status != WS_NO_ERR) {
+  long status = _get_url_data(url, &protocol, &host, &port, &path);
+  if (status != WS_NO_ERR){
     if (host != NULL){
       delete host;
     }
@@ -917,12 +816,12 @@ long WSGFloadRemoteFile(char* url,char** pbuffer,char** ctype,
 
   long  size = 0;
   int   sock = 0;
-  if (WSGFstricmp(protocol, "http") == 0) {
+  if (WSGFstricmp(protocol, "http") == 0){
     long len=strlen(path);
-    if (path[len-1] == '/') {
+    if (path[len-1] == '/'){
       char *work = new char[len+strlen(_default_file)+1];
       memset(work,0, len+strlen(_default_file)+1);
-      if (work == NULL) {
+      if (work == NULL){
         WSMFtrace("Memory allocate error.\n");
         return False;
       }
@@ -934,48 +833,46 @@ long WSGFloadRemoteFile(char* url,char** pbuffer,char** ctype,
       port = 80;
     }
 
-    if (*_http_proxy == '\0') {
+    if (*_http_proxy == 0){
       sprintf(port_str, "%1d", port);
-      status = WSGFhttpConnect(host, port, &sock);
+      status = _connect_to(host, port, &sock);
     }else{
       proxy_host = _get_proxy_info(_http_proxy, 80, &proxy_port);
       sprintf(port_str, "%1d", proxy_port);
-      if (proxy_host == NULL) {
+      if (proxy_host == NULL){
         status = -1;
       }else{
-        status = WSGFhttpConnect(proxy_host, proxy_port, &sock);
+        status = _connect_to(proxy_host, proxy_port, &sock);
       }
     }
-    if (status == WS_INTR) {
-      WSMFtrace("HTTP: connection interrupted.\n");
-    }else if (status < 0) {
-      WSMFtrace("HTTP: unable to connect to remote host.\n");
-      if (*_http_proxy == '\0') {
-        WSMFtrace("Can not connect to %s%s%s.\n", host,
+    if (status == WS_INTR){
+      WSMFtrace("HTTP: Connection interrupted.\n");
+    }else if (status < 0){
+      if (*_http_proxy == 0){
+        WSMFtrace("HTTP: Failed to connect to %s%s%s.\n", host,
               port==80 ? "" : ":", port==80 ? "" : port_str);
       }else{
-        WSMFtrace("Can not connect to %s%s%s.\n", proxy_host,
+        WSMFtrace("HTTP: Failed to connect to %s%s%s.\n", proxy_host,
                   proxy_port==80 ? "" : ":", proxy_port==80 ? "" : port_str);
       }
     }else if (status == WS_NO_ERR){
-//      if (WSGFhttpWrite(sock, (*_http_proxy=='\0' ? path : url)) == WS_NO_ERR) {
-      if (WSGFhttpWrite(sock,url) == WS_NO_ERR) {
-        if ((status=WSGFhttpRead(sock, &buf, &size)) ==
-                WS_NO_ERR && buf != NULL && *buf != '\0') {
-               *pbuffer = _http_extxt(buf, bsize, fl,ctype);
+//      if (_write_data(sock, (*_http_proxy==0 ? path : url)) == WS_NO_ERR){
+      if (_write_data(sock,url) == WS_NO_ERR){
+        if ((status=_read_data(sock, &buf, &size)) == WS_NO_ERR &&
+            buf != NULL && *buf != 0){
+          *pbuffer = _http_extxt(buf, bsize, fl,ctype);
           delete buf;
-        }else if (status == WS_INTR) {
+        }else if (status == WS_INTR){
           WSMFtrace("HTTP: connection longerrupted.\n");
-        }else if (buf == NULL || *buf == '\0') {
-          WSMFtrace( "HTTP: fail to retrieve file from the server.\n");
+        }else if (buf == NULL || *buf == 0){
+          WSMFtrace( "HTTP: Failed to retrieve file from the server.\n");
           WSMFtrace( "Can not get %s.\n", url);
         }else{
-          WSMFtrace( "HTTP: network error occurred when retrieving file.\n");
-          if (*_http_proxy == '\0') {
-            WSMFtrace("Network error while talking to %s%s%s.\n",
+          if (*_http_proxy == 0){
+            WSMFtrace("HTTP: Failed to talk to %s%s%s.\n",
                    host, port==80 ? "" : ":", port==80 ? "" : port_str);
           }else{
-            WSMFtrace("Network error while talking to %s%s%s.\n",
+            WSMFtrace("HTTP: Failed to talk to %s%s%s.\n",
                    proxy_host, proxy_port==80 ? "" : ":",
                    proxy_port==80 ? "" : port_str);
           }
@@ -983,23 +880,22 @@ long WSGFloadRemoteFile(char* url,char** pbuffer,char** ctype,
       }
       so_close(sock);
     }else{
-      WSMFtrace("HTTP: unable to connect to remote host.\n");
-      if (*_http_proxy == '\0') {
-        WSMFtrace("Can not connect to %s%s%s.\n", host,
+      if (*_http_proxy == 0){
+        WSMFtrace("Failed to connect to %s%s%s.\n", host,
                port==80 ? "" : ":", port==80 ? "" : port_str);
       }else{
-        WSMFtrace("Can not connect to %s%s%s.\n", proxy_host,
+        WSMFtrace("Failed to connect to %s%s%s.\n", proxy_host,
                 proxy_port==80 ? "" : ":", proxy_port==80 ? "" : port_str);
       }
     }
-    if (status == WS_NO_ERR) {
-      _update_url_cache(url, *pbuffer,
+    if (status == WS_NO_ERR){
+      _update_cache(url, *pbuffer,
       ctype==(char**)NULL ? (char*)NULL : *ctype,*bsize, *fl);
     }
   }else if (WSGFstricmp(protocol, "ftp") == 0){
-    WSMFtrace("Do not support the ftp protocol.\n");
+    WSMFtrace("Unsupported the ftp protocol.\n");
   }else{
-    WSMFtrace("Do not support the '%s' protocol.\n", url);
+    WSMFtrace("Unsupported the '%s' protocol.\n", url);
   }
   if (host != NULL){
     delete host;
