@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,17 +14,15 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
+#ifndef TE
 #include <devfb/devfb.h>
+#endif
 #include <assert.h>
 #include <WSCbaseList.h>
 #include <WSCbase.h>
@@ -75,6 +73,29 @@ void WSGFdeviceInitialize(){
 dbprintf("WSGFdeviceInitialize.. started..\n");
   static WSCbool initialized = False;
   if (initialized == False){
+#ifndef NO_FILE_SERIALIZE
+extern void WSGFinitFS();
+    WSGFinitFS();
+#endif
+extern void WSGFunixEnvInit();
+    WSGFunixEnvInit();
+#ifndef NO_FILE_SYSTEM
+extern void WSGFinitializeUnixFile();
+    WSGFinitializeUnixFile();
+#endif
+#ifndef NO_WSDMUTEX
+extern void WSGFinitializeUnixMutex();
+    WSGFinitializeUnixMutex();
+#endif
+#ifndef NO_WSDSEMAPHORE
+extern void WSGFinitializeUnixSemaphore();
+    WSGFinitializeUnixSemaphore();
+#endif
+#ifndef NO_WSDTHREAD
+extern void WSGFinitializeUnixThread();
+    WSGFinitializeUnixThread();
+#endif
+
 extern WSDfile* _unix_fp_chandler();
     WSDfile::setCreateInstanceHandler((void*)_unix_fp_chandler);
 //extern WSDdragDrop* _xdrag_drop_create_handler_();
@@ -202,7 +223,7 @@ long is_own_window(W w){
   return False;
 }
 long WSDdevfbAppDev::loadGuiPolicy(){
-#if 0
+#if defined(LINUX) || defined (MacOS)
   WSCstring* envstr = WSGFreadTextFile("$(HOME)/.wsrc");
   if (envstr != NULL){
     while(envstr->eof() == False){
@@ -216,95 +237,101 @@ long WSDdevfbAppDev::loadGuiPolicy(){
       data.delHeadSpace();
       data.delTailSpace();
       data.delLineFeed();
-      if (!strcmp((char*)index,"#POLICY") && strcmp((char*)data,"")){
-        long val = atoi((char*)data);
+      char* istr = (char*)index;
+      char* dstr = (char*)data;
+      char* dstr2 = (char*)data2;
+      if (!strcmp(istr,"#POLICY") && strcmp(dstr,"")){
+        long val = atoi(dstr);
         if (_policy == -1){
           setGuiPolicy(val,False);
         }
       }else
-      if (!strcmp((char*)index,"#TOPSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#EXPOLICY") && strcmp(dstr,"")){
+        setExtGuiPolicy(dstr,True);
+      }else
+      if (!strcmp(istr,"#TOPSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_TOPSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#BOTTOMSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#BOTTOMSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_BOTTOMSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#BACKCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#BACKCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_BACKCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#FORECOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#FORECOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_FORECOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#MENUFORECOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#MENUFORECOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_MENUFORECOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#MENUBACKCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#MENUBACKCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_MENUBACKCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#MENUSELECTCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#MENUSELECTCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_MENUSELECTCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#MENUTOPSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#MENUTOPSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_MENUTOPSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#MENUBOTTOMSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#MENUBOTTOMSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_MENUBOTTOMSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#WORKBACKCOLOR") && strcmp((char*)data,"")){
+      if (!strcmp(istr,"#WORKBACKCOLOR") && strcmp(dstr,"")){
         short cno = WSGIappColorSet()->getColorNo((char*)data);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_WORKBACKCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#DARKBACKCOLOR") && strcmp((char*)data,"")){
+      if (!strcmp(istr,"#DARKBACKCOLOR") && strcmp(dstr,"")){
         short cno = WSGIappColorSet()->getColorNo((char*)data);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_DARKBACKCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#BARSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#BARSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_BARSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#NWTOPSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#NWTOPSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_NWTOPSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#NWBOTTOMSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#NWBOTTOMSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_NWBOTTOMSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#NWBACKCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#NWBACKCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_NWBACKCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#NWFORECOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#NWFORECOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_NWFORECOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#DARKBOTTOMSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#DARKBOTTOMSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_DARKBOTTOMSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#MENUSELECTFORECOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#MENUSELECTFORECOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_MENUSELECTFORECOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#AQUAFORECOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#AQUAFORECOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_AQUAFORECOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#AQUAHIGHLIGHTCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#AQUAHIGHLIGHTCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_AQUAHIGHLIGHTCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#DEFAULTFONT") && strcmp((char*)data2,"")){
+      if (!strcmp(istr,"#DEFAULTFONT") && strcmp(dstr2,"")){
         WSDfont* font = WSGIappFontSet()->getFont(8);
-        font->setFontName(data2);
+        font->setFontName(dstr2);
       }
     }
     if (_policy == -1){
@@ -320,6 +347,8 @@ long WSDdevfbAppDev::saveGuiPolicy(){
   WSCstring str;
   str << "#POLICY\n";
   str << getGuiPolicy() << "\n";
+  str << "#EXPOLICY\n";
+  str << getExtGuiPolicyName() << "\n";
   str << "#TOPSHADOWCOLOR\n";
   str << WSGIappColorSet()->getColor(WS_DF_TOPSHADOWCOLOR)->getSrc() << "\n";
   str << "#BOTTOMSHADOWCOLOR\n";
@@ -445,6 +474,9 @@ dbprintf("WSDdevfbAppDev::handleEvents start dispatchEvent().   XXXXXXXX\n");
     dispatchEvent();
 dbprintf("WSDdevfbAppDev::handleEvents start clearDeleteList(). XXXXXXXX\n");
     WSGIappDevice()->clearDeleteList();
+    if (_exit_event_loop != False){
+      break;
+    }
   }
   return WS_NO_ERR;
 }
@@ -518,13 +550,15 @@ dbprintf("_expose(W=0x%x) has no instance.\n",window);
     rlist[N_VIEWR-1].next = NULL;
 
     dev->clearClipList();
-    W nr2 = gget_for(dev->getParentGID(), dev->getClipList());
+//this cause to fail drawing by noeffective old fore lists..
+//    W nr2 = gget_for(dev->getParentGID(), dev->getClipList());
     W nr  = wsta_dsp(window, &r,rlist);
 
     if (nr <= 0){
       break;
     }
-    dev->setClipNum(nr2);
+//this cause to fail drawing by noeffective old fore lists..
+//    dev->setClipNum(nr2);
 
     if (nr > N_VIEWR){
 dbprintf("need expose %d,%d,%d,%d .. wnd=0x%x\n",r.c.left,r.c.top,r.c.right-r.c.left,r.c.bottom - r.c.top,window);
@@ -732,6 +766,9 @@ void recvmsg(){
   if (ret <0){
      ret = rcv_msg(MM_TYPE3,msg,sizeof(MESSAGE)+ sizeof(_read_event_send_type)-sizeof(void*),NOWAIT);
   }
+  if (ret <0){
+     ret = rcv_msg(MM_TYPE4,msg,sizeof(MESSAGE),NOWAIT);
+  }
 #endif
   if (ret <0){
   }else if (msg->msg_type == MS_TYPE1){ 
@@ -774,13 +811,18 @@ dbprintf("WSDxappDev::_dispatchEvent %s:%d APPL5x2 done\n",__FILE__,__LINE__);
 dbprintf("WSDxappDev::_dispatchEvent %s:%d APPL5x start\n",__FILE__,__LINE__);
 #ifndef NO_WSDTHREAD
     WSDdevfbThread* th = NULL;
+    void* data = NULL;
+//    memcpy(&th,&(msg->msg_body.ANYMSG.msg_str),sizeof(W));
     memcpy(&th,msg->msg_body.ANYMSG.msg_str,sizeof(W));
+    memcpy(&data,&msg->msg_body.ANYMSG.msg_str[sizeof(W)*2],sizeof(void*));
+//printf("client message !! here! th=0x%x\n",th);
     if (th->getCallbackFunction() != NULL){
 //printf("client message !! here! execute exec_read\n");
       void(*fn)(WSDdevfbThread*,void*) =
            (void(*)(WSDdevfbThread*,void*))th->getCallbackFunction();
       if (fn != NULL){
-        fn(th,th->getCallbackData());
+//        fn(th,th->getCallbackData());
+        fn(th,data);
       }
     }
 #endif
@@ -991,6 +1033,7 @@ dbprintf("W_TITL/W_PICT.. need expose..\n");
           break;
         }
         case W_CLOSED:{
+          _destroy_evt(ev.s.wid,&ev);
 dbprintf("need close child window..\n");
           break;
         }
@@ -1166,6 +1209,32 @@ dbprintf("WSDdevfbAppDev::_dispatchEvent start executeUpdate().   XXXXXXXX\n");
 
       return WS_NO_ERR;
     }
+    case EV_CANCEL:{
+      WSCbase* ucl = WSGIappMouse()->getMouseUngrabClient();
+      if (ucl != NULL){
+        ucl->execEventProc(WSEV_GRAB_CANCELED,NULL);
+      }
+      WSDdevfbFormDev::executeUpdate();
+      WSGIappObjectList()->execInitialize();
+      WSGIappObjectList()->execUpdate();
+dbprintf("WSDdevfbAppDev::_dispatchEvent start executeUpdate().   XXXXXXXX\n");
+
+      return WS_NO_ERR;
+    }
+    case EV_RESIZE:{
+      RECT r2;
+      wget_wrk(ev.s.wid,&r2);
+      gset_vis(ev.s.wid,r2);
+      _exec_resize(ev.s.wid,&r2);
+      _expose(ev.s.wid);
+
+      WSDdevfbFormDev::executeUpdate();
+      WSGIappObjectList()->execInitialize();
+      WSGIappObjectList()->execUpdate();
+dbprintf("WSDdevfbAppDev::_dispatchEvent start executeUpdate().   XXXXXXXX\n");
+
+      return WS_NO_ERR;
+    }
     WSDdevfbFormDev::executeUpdate();
     WSGIappObjectList()->execInitialize();
     WSGIappObjectList()->execUpdate();
