@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,18 +14,14 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
-#include <devfb/devfb.h>
 #include <WScom.h>
+#include <devfb/devfb.h>
 #include <WSDenv.h>
 #include <WSCcolorSet.h>
 #include <devfb/WSDdevfbColor.h>
@@ -47,6 +43,7 @@ void _devfbColor_init(){
 //  XSetErrorHandler(_ehandler);
   WSGIappColorSet()->setCreateHandler(_devfbColor_create_handler);
   WSGFinitUnixFileSystem();
+  WSGFinitializeUnixFile();
 dbprintf("_devfbColor_init executed..\n");
 };
 
@@ -70,7 +67,7 @@ long WSDdevfbColor::initialize(){
   }
   _a = 255;
   WSCbool alpha = False;
-  long rval = 0,gval = 0,bval = 0,aval=255;
+  int rval = 0,gval = 0,bval = 0,aval=255;
   if(cname[0] == '#'){
     long slen = strlen(cname);
     if(slen == 4){
