@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,16 +14,13 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
+#include <WScom.h>
 #include <devfb/devfb.h>
 #include <unistd.h>
 #ifndef NO_PNG
@@ -33,16 +30,17 @@
 extern "C" {
 #include <jpeglib.h>
 #include <jerror.h>
+#include <setjmp.h>
 };
 #endif
 
-#include <WScom.h>
 #include <WSCimageSet.h>
 #include <WSDfileSystem.h>
 #include <WStcpcom.h>
 #include <devfb/WSDdevfbImage.h>
 #include <devfb/WSDdevfbAppDev.h>
 #include <devfb/WSDdevfbColor.h>
+#include <WSDfile.h>
 
 extern char* WSGFdevfbAllocNewMemory(long);
 extern void WSGFdevfbDeleteMemory(char*);
@@ -50,7 +48,7 @@ extern void WSGFdevfbDeleteMemory(char*);
 long WSGFdevfbBufToPixmap(unsigned char *buf,W* pixmap,long width,long height,long* vw,long* vh,long row_bytes,UB**,WSCbool reuse_mem);
 long WSGFdevfbBufToPixmap2(unsigned char *buf,W* pixmap,W*,long width,long height,long* vw,long* vh,long row_bytes,UB**,UB**,WSCbool reuse_mem);
 long WSGFdevfbBufToPixmapG(unsigned char *buf,W* pixmap,long width,long height,long* vw,long* vh,long row_bytes,UB**,WSCbool reuse_mem);
-long WSGFdevfbBmpReadFpToPixmap(FILE* ifile,W* pixmap,W*,long* w,long* h,UB**,UB**);
+long WSGFdevfbBmpReadFpToPixmap(WSDfile* ifile,W* pixmap,W*,long* w,long* h,UB**,UB**);
 long WSGFdevfbBmpReadFileToPixmap(char* filename,W* pixmap,W*,long* w,long* h,UB**,UB**);
 long PngReadFileToPixmap(char* filename,GID* mem_gid,GID* mem_gid2,long* w,long* h,UB**,UB**);
 #ifndef NO_JPEG
@@ -240,50 +238,49 @@ long WSDdevfbImage::createImage(){
   return WS_NO_ERR;
 }
 
-int WSGFdevfbgetbyte(FILE *fp) {
-  int c;
-  if ((c = fgetc(fp)) == EOF){
+int WSGFdevfbgetbyte(WSDfile* fp) {
+  WSCuchar c[1];
+  if (fp->read(c,1) < 1){
     return -1;
   }
-  return c;
+  return c[0];
 }
 
-int WSGFdevfbreadLittleShort(FILE *fp, short *d) {
-  int c;
-  if ((c = fgetc(fp)) == EOF){
+int WSGFdevfbreadLittleShort(WSDfile* fp, short *d) {
+  WSCuchar c[1]; 
+  if (fp->read(c,1) < 1){
+    return -1; 
+  }
+  *d = c[0] & 0xff;
+  if (fp->read(c,1) < 1){
     return -1;
   }
-  *d = c & 0xff;
-  if ((c = fgetc(fp)) == EOF){
-    return -1;
-  }
-  *d |= (c & 0xff) << 8;
+  *d |= (c[0] & 0xff) << 8;
   return 0;
 }
 
-int WSGFdevfbreadLittleLong(FILE *fp, long *d) {
-  int c;
-
-  if ((c = fgetc(fp)) == EOF){
+int WSGFdevfbreadLittleLong(WSDfile* fp, long *d) {
+  WSCuchar c[1];
+  if (fp->read(c,1) < 1){
     return -1;
   }
-  *d = c & 0xff;
-  if ((c = fgetc(fp)) == EOF){
+  *d = c[0] & 0xff;
+  if (fp->read(c,1) < 1){
     return -1;
   }
-  *d |= ((c & 0xff) << 8);
-  if ((c = fgetc(fp)) == EOF){
+  *d |= ((c[0] & 0xff) << 8);
+  if (fp->read(c,1) < 1){
     return -1;
   }
-  *d |= ((c & 0xff) << 16);
-  if ((c = fgetc(fp)) == EOF){
+  *d |= ((c[0] & 0xff) << 16);
+  if (fp->read(c,1) < 1){
     return -1;
   }
-  *d |= ((c & 0xff) << 24);
+  *d |= ((c[0] & 0xff) << 24);
   return 0;
 }
 
-long WSGFdevfbBmpReadFpToPixmap(FILE* ifile,W* pixmap,W* pixmap2,long* w,long* h,UB** ptr,UB** ptr2){
+long WSGFdevfbBmpReadFpToPixmap(WSDfile* ifile,W* pixmap,W* pixmap2,long* w,long* h,UB** ptr,UB** ptr2){
 
   // bmp (Windows 3.x) header
   //       implemented
@@ -386,11 +383,11 @@ long WSGFdevfbBmpReadFpToPixmap(FILE* ifile,W* pixmap,W* pixmap2,long* w,long* h
         return -1;
     }
   }
-  if (fseek(ifile, foffs, SEEK_SET) != 0){
+  if (ifile->seek(foffs) != WS_NO_ERR){
     return -1;
   }
   unsigned char* fbuf = new unsigned char[fsize - foffs];
-  if (fread(fbuf, fsize - foffs, 1, ifile) != 1) {
+  if (ifile->read(fbuf, fsize - foffs) != (fsize - foffs)) {
     delete fbuf;
     return -1;
   }
@@ -414,7 +411,6 @@ long WSGFdevfbBmpReadFpToPixmap(FILE* ifile,W* pixmap,W* pixmap2,long* w,long* h
       }
     }
   }
-
   if (cmptype == 1) {
     // BI_RLE8
     x = 0;
@@ -615,6 +611,7 @@ long WSGFdevfbBmpReadFpToPixmap(FILE* ifile,W* pixmap,W* pixmap2,long* w,long* h
     delete buf_mask;
   }else{
     reti = WSGFdevfbBufToPixmap(buf, pixmap, width, height,w,h,width*3,ptr,False);
+    *pixmap2 = 0;
     delete buf;
   }
 
@@ -632,10 +629,10 @@ long WSGFdevfbBufToPixmap(unsigned char *buf,W* mem_gid,long rwidth,long rheight
     vwidth = rwidth;
     vheight = rheight;
   }else{
-    if ((long)((double)rwidth * vheight/vwidth) < rheight){
-      vwidth = (long)((double)vheight * rwidth/rheight);
+    if ((long)(rwidth * vheight/vwidth) < rheight){
+      vwidth = (long)(vheight * rwidth/rheight);
     }else{
-      vheight = (long)((double)vwidth * rheight/rwidth);
+      vheight = (long)(vwidth * rheight/rwidth);
     }
     if (vwidth == rwidth && vheight == rheight){
       stretched = False;
@@ -656,7 +653,7 @@ long WSGFdevfbBufToPixmap(unsigned char *buf,W* mem_gid,long rwidth,long rheight
   int red_bits = dspec->color[0] & 0xf;
   int green_bits = dspec->color[1] & 0xf;
   int blue_bits = dspec->color[2] & 0xf;
-//printf("devspec 0x%x 0x%x 0x%x\n",dspec->color[0],dspec->color[1],dspec->color[2]);
+//printf("devspec bits=0x%x 0x%x 0x%x 0x%x\n",dspec->pixbits,dspec->color[0],dspec->color[1],dspec->color[2]);
   long cnt = 0;
   long ptr = 0;
   WSCbool duplicate = False;
@@ -681,7 +678,6 @@ long WSGFdevfbBufToPixmap(unsigned char *buf,W* mem_gid,long rwidth,long rheight
     }
 #endif
   }
-
   if (reuse_mem){
     buf2 = (UB*)WSGFdevfbAllocNewMemory( rheight * rwidth * ddepth/8);
   }else{
@@ -689,33 +685,55 @@ long WSGFdevfbBufToPixmap(unsigned char *buf,W* mem_gid,long rwidth,long rheight
   }
   unsigned short* buf2_s = (unsigned short*)buf2;
   unsigned int* buf2_i = (unsigned int*)buf2;
+  unsigned char* buf2_c = (unsigned char*)buf2;
 
   unsigned int val_i;
   unsigned int val_s;
-  for(i=0; i<vheight; i++){
-    ptr = row_bytes*i;
-    for(j=0; j<vwidth; j++){
-      UB r = buf[ptr++];
-      UB g = buf[ptr++];
-      UB b = buf[ptr++];
-      if (ddepth == 32){
-        val_i = (r >> (8-red_bits)) << red_shift |
-                           (g >> (8-green_bits)) << green_shift |
-                           (b >> (8-blue_bits)) << blue_shift;
+  int rbt = 8-red_bits;
+  int gbt = 8-green_bits;
+  int bbt = 8-blue_bits;
+  if (ddepth == 32){
+    for(i=0; i<vheight; i++){
+      ptr = row_bytes*i;
+      for(j=0; j<vwidth; j++){
+        UB r = buf[ptr++];
+        UB g = buf[ptr++];
+        UB b = buf[ptr++];
+        val_i = (r >> rbt) << red_shift |
+                (g >> gbt) << green_shift |
+                (b >> bbt) << blue_shift;
         buf2_i[cnt++] = val_i;
-      }else
-      if (ddepth == 16){
-//        unsigned short val = ((r & 0xf8) | ((g & 0xe0) >> 5)) << 8 +
-//                           ((g & 0x1c) << 3 )| (b & 0xf8) >> 3;
-        val_s = (r >> (8-red_bits)) << red_shift |
-                             (g >> (8-green_bits)) << green_shift |
-                             (b >> (8-blue_bits)) << blue_shift;
+      }
+    }
+  }else
+  if (ddepth == 24){
+    for(i=0; i<vheight; i++){
+      ptr = row_bytes*i;
+      for(j=0; j<vwidth; j++){
+        UB r = buf[ptr++];
+        UB g = buf[ptr++];
+        UB b = buf[ptr++];
+        val_i = (r >> rbt) << red_shift |
+                (g >> gbt) << green_shift |
+                (b >> bbt) << blue_shift;
+        buf2_c[cnt++] = (val_i >> 16) & 0xff;
+        buf2_c[cnt++] = (val_i >> 8)  & 0xff;
+        buf2_c[cnt++] = (val_i)       & 0xff;
+      }
+    }
+  }else
+  if (ddepth == 16){
+    for(i=0; i<vheight; i++){
+      ptr = row_bytes*i;
+      for(j=0; j<vwidth; j++){
+        UB r = buf[ptr++];
+        UB g = buf[ptr++];
+        UB b = buf[ptr++];
+        val_s = (r >> rbt) << red_shift |
+                (g >> gbt) << green_shift |
+                (b >> bbt) << blue_shift;
         buf2_s[cnt++] = val_s;
       }
- #if 0
-      buf2[cnt++] = ((g & 0x1c) << 3 )| (b & 0xf8) >> 3;
-      buf2[cnt++] = (r & 0xf8) | ((g & 0xe0) >> 5);
-#endif
     }
   }
 
@@ -805,10 +823,10 @@ long WSGFdevfbBufToPixmapG(unsigned char *buf,W* mem_gid,long rwidth,long rheigh
     vwidth = rwidth;
     vheight = rheight;
   }else{
-    if ((long)((double)rwidth * vheight/vwidth) < rheight){
-      vwidth = (long)((double)vheight * rwidth/rheight);
+    if ((long)(rwidth * vheight/vwidth) < rheight){
+      vwidth = (long)(vheight * rwidth/rheight);
     }else{
-      vheight = (long)((double)vwidth * rheight/rwidth);
+      vheight = (long)(vwidth * rheight/rwidth);
     }
     if (vwidth == rwidth && vheight == rheight){
       stretched = False;
@@ -862,33 +880,50 @@ long WSGFdevfbBufToPixmapG(unsigned char *buf,W* mem_gid,long rwidth,long rheigh
   }
   unsigned short* buf2_s = (unsigned short*)buf2;
   unsigned int* buf2_i = (unsigned int*)buf2;
+  unsigned char* buf2_c = (unsigned char*)buf2;
 
   unsigned int val_i;
   unsigned short val_s;
-  for(i=0; i<vheight; i++){
-    ptr = row_bytes*i;
-    for(j=0; j<vwidth; j++){
-      UB r = buf[ptr];
-      UB g = buf[ptr];
-      UB b = buf[ptr++];
-      if (ddepth == 32){
-        val_i = (r >> (8-red_bits)) << red_shift |
-                           (g >> (8-green_bits)) << green_shift |
-                           (b >> (8-blue_bits)) << blue_shift;
+  int rbt = 8-red_bits;
+  int gbt = 8-green_bits;
+  int bbt = 8-blue_bits;
+
+  if (ddepth == 32){
+    for(i=0; i<vheight; i++){
+      ptr = row_bytes*i;
+      for(j=0; j<vwidth; j++){
+        UB v = buf[ptr++];
+        val_i = (v >> rbt) << red_shift |
+                (v >> gbt) << green_shift |
+                (v >> bbt) << blue_shift;
         buf2_i[cnt++] = val_i;
-      }else
-      if (ddepth == 16){
-//        unsigned short val = ((r & 0xf8) | ((g & 0xe0) >> 5)) << 8 +
-//                           ((g & 0x1c) << 3 )| (b & 0xf8) >> 3;
-        val_s = (r >> (8-red_bits)) << red_shift |
-                             (g >> (8-green_bits)) << green_shift |
-                             (b >> (8-blue_bits)) << blue_shift;
+      }
+    }
+  }else
+  if (ddepth == 24){
+    for(i=0; i<vheight; i++){
+      ptr = row_bytes*i;
+      for(j=0; j<vwidth; j++){
+        UB v = buf[ptr++];
+        val_i = (v >> rbt) << red_shift |
+                (v >> gbt) << green_shift |
+                (v >> bbt) << blue_shift;
+        buf2_c[cnt++] = (val_i >> 16) & 0xff;
+        buf2_c[cnt++] = (val_i >> 8)  & 0xff;
+        buf2_c[cnt++] = (val_i)       & 0xff;
+      }
+    }
+  }else
+  if (ddepth == 16){
+    for(i=0; i<vheight; i++){
+      ptr = row_bytes*i;
+      for(j=0; j<vwidth; j++){
+        UB v = buf[ptr++];
+        val_s = (v >> rbt) << red_shift |
+                (v >> gbt) << green_shift |
+                (v >> bbt) << blue_shift;
         buf2_s[cnt++] = val_s;
       }
-#if 0
-      buf2[cnt++] = ((g & 0x1c) << 3 )| (b & 0xf8) >> 3;
-      buf2[cnt++] = (r & 0xf8) | ((g & 0xe0) >> 5);
-#endif
     }
   }
 
@@ -1034,10 +1069,10 @@ long WSGFdevfbBufToPixmap2(unsigned char *buf,W* mem_gid,W* mem_gid2,long rwidth
     vwidth = rwidth;
     vheight = rheight;
   }else{
-    if ((long)((double)rwidth * vheight/vwidth) < rheight){
-      vwidth = (long)((double)vheight * rwidth/rheight);
+    if ((long)(rwidth * vheight/vwidth) < rheight){
+      vwidth = (long)(vheight * rwidth/rheight);
     }else{
-      vheight = (long)((double)vwidth * rheight/rwidth);
+      vheight = (long)(vwidth * rheight/rwidth);
     }
     if (vwidth == rwidth && vheight == rheight){
       stretched = False;
@@ -1095,36 +1130,78 @@ long WSGFdevfbBufToPixmap2(unsigned char *buf,W* mem_gid,W* mem_gid2,long rwidth
   }
   unsigned short* buf2_s = (unsigned short*)buf2;
   unsigned int* buf2_i = (unsigned int*)buf2;
+  unsigned char* buf2_c = (unsigned char*)buf2;
   unsigned short* buf3_s = (unsigned short*)buf3;
   unsigned int* buf3_i = (unsigned int*)buf3;
+  unsigned char* buf3_c = (unsigned char*)buf3;
 
   unsigned int val_i;
   unsigned int val_s;
-  for(i=0; i<vheight; i++){
-    ptr = row_bytes*i;
-    for(j=0; j<vwidth; j++){
-      UB r = buf[ptr++];
-      UB g = buf[ptr++];
-      UB b = buf[ptr++];
-      UB a = buf[ptr++];
+  int rbt = 8-red_bits;
+  int gbt = 8-green_bits;
+  int bbt = 8-blue_bits;
 
-      if (ddepth == 32){
-        val_i = (r >> (8-red_bits)) << red_shift |
-                           (g >> (8-green_bits)) << green_shift |
-                           (b >> (8-blue_bits)) << blue_shift;
+  if (ddepth == 32){
+    for(i=0; i<vheight; i++){
+      ptr = row_bytes*i;
+      for(j=0; j<vwidth; j++){
+        UB r = buf[ptr++];
+        UB g = buf[ptr++];
+        UB b = buf[ptr++];
+        UB a = buf[ptr++];
+
+        val_i = (r >> rbt) << red_shift |
+                (g >> gbt) << green_shift |
+                (b >> bbt) << blue_shift;
         buf2_i[cnt] = val_i;
         if (a != 0){
           buf3_i[cnt++] = 0;
         }else{
           buf3_i[cnt++] = 0xffffffff;
         }
-      }else
-      if (ddepth == 16){
-//        unsigned short val = ((r & 0xf8) | ((g & 0xe0) >> 5)) << 8 +
-//                           ((g & 0x1c) << 3 )| (b & 0xf8) >> 3;
-        val_s = (r >> (8-red_bits)) << red_shift |
-                             (g >> (8-green_bits)) << green_shift |
-                             (b >> (8-blue_bits)) << blue_shift;
+      }
+    }
+  }else
+  if (ddepth == 24){
+    for(i=0; i<vheight; i++){
+      ptr = row_bytes*i;
+      for(j=0; j<vwidth; j++){
+        UB r = buf[ptr++];
+        UB g = buf[ptr++];
+        UB b = buf[ptr++];
+        UB a = buf[ptr++];
+
+        val_i = (r >> rbt) << red_shift |
+                (g >> gbt) << green_shift |
+                (b >> bbt) << blue_shift;
+        buf2_c[cnt++] = (val_i >> 16) & 0xff;
+        buf2_c[cnt++] = (val_i >> 8)  & 0xff;
+        buf2_c[cnt++] = (val_i)       & 0xff;
+        if (a != 0){
+          buf3_c[cnt++] = 0;
+          buf3_c[cnt++] = 0;
+          buf3_c[cnt++] = 0;
+        }else{
+          buf3_c[cnt++] = 0xff;
+          buf3_c[cnt++] = 0xff;
+          buf3_c[cnt++] = 0xff;
+        }
+      }
+    }
+ 
+  }else
+  if (ddepth == 16){
+    for(i=0; i<vheight; i++){
+      ptr = row_bytes*i;
+      for(j=0; j<vwidth; j++){
+        UB r = buf[ptr++];
+        UB g = buf[ptr++];
+        UB b = buf[ptr++];
+        UB a = buf[ptr++];
+
+        val_s = (r >> rbt) << red_shift |
+                (g >> gbt) << green_shift |
+                (b >> bbt) << blue_shift;
         buf2_s[cnt] = val_s;
         if (a != 0){
           buf3_s[cnt++] = 0;
@@ -1132,32 +1209,10 @@ long WSGFdevfbBufToPixmap2(unsigned char *buf,W* mem_gid,W* mem_gid2,long rwidth
           buf3_s[cnt++] = 0xffff;
         }
       }
-#if 0
-#if 0
-      buf2[cnt] = ((g & 0x1c) << 3 )| (b & 0xf8) >> 3;
-      buf2[cnt+1] = (r & 0xf8) | ((g & 0xe0) >> 5);
-#endif
-      if (a != 0){
-        a=0;
-        buf3_s[cnt++] = 0;
-#if 0
-        buf3[cnt++] = 0;
-        buf3[cnt++] = 0;
-#endif
-      }else{
-        a = 0xff;
-        buf3_s[cnt++] = 0xffff;
-#if 0
-        buf3[cnt++] = 0xff;
-        buf3[cnt++] = 0xff;
-#endif
-      }
-//      buf3[cnt++] = ((a & 0x1c) << 3 )| (a & 0xf8) >> 3;
-//      buf3[cnt++] = (a & 0xf8) | ((a & 0xe0) >> 5);
-#endif
     }
   }
 
+
   BMP bmp;
   bmp.planes = 1;
 #if 0
@@ -1278,10 +1333,10 @@ long WSGFdevfbBufToPixmap2(unsigned char *buf,W* mem_gid,W* mem_gid2,long rwidth
     vwidth = rwidth;
     vheight = rheight;
   }else{
-    if ((long)((double)rwidth * vheight/vwidth) < rheight){
-      vwidth = (long)((double)vheight * rwidth/rheight);
+    if ((long)(rwidth * vheight/vwidth) < rheight){
+      vwidth = (long)(vheight * rwidth/rheight);
     }else{
-      vheight = (long)((double)vwidth * rheight/rwidth);
+      vheight = (long)(vwidth * rheight/rwidth);
     }
     stretched = 1;
   }
@@ -1301,8 +1356,8 @@ long WSGFdevfbBufToPixmap2(unsigned char *buf,W* mem_gid,W* mem_gid2,long rwidth
     ptr = row_bytes*i;
     for(j=0; j<vwidth; j++){
       if (stretched != 0){
-        ptr = (long)((double)i*rheight/vheight)*rwidth
-             + (long)((double)j*rwidth/vwidth);
+        ptr = (long)(i*rheight*rwidth/vheight)
+             + (long)(j*rwidth/vwidth);
         ptr *= 3;
       }
       r = buf[ptr++];
@@ -1336,13 +1391,16 @@ long WSGFdevfbBufToPixmap2(unsigned char *buf,W* mem_gid,W* mem_gid2,long rwidth
 }
 long WSGFdevfbBmpReadFileToPixmap(char* filename,W* pixmap,W* pixmap2,long* w,long* h,UB** ptr,UB** ptr2){
 
-  FILE* ifile = fopen(filename,"r");
+  WSDfile* ifile = WSDfile::getNewInstance();
   if (ifile == NULL){
     return -1;
   }
+  if (ifile->open(filename,"r") != WS_NO_ERR){
+    return -1;
+  }
 
   long reti = WSGFdevfbBmpReadFpToPixmap(ifile, pixmap,pixmap2, w, h,ptr,ptr2);
-  fclose(ifile);
+  delete ifile;
   return reti;
 }
 
@@ -1785,10 +1843,10 @@ long JpegReadFileToPixmap(char* filename,long size,GID* mem_gid,GID* mem_gid2,lo
       *w = rwidth;
       *h = rheight;
     }else{
-      if ((long)((double)rwidth * vheight/vwidth) < rheight){
-        *w = (long)((double)vheight * rwidth/rheight);
+      if ((long)(rwidth * vheight/vwidth) < rheight){
+        *w = (long)(vheight * rwidth/rheight);
       }else{
-        *h = (long)((double)vwidth * rheight/rwidth);
+        *h = (long)(vwidth * rheight/rwidth);
       }
       stretched = 1;
     }
@@ -1829,7 +1887,7 @@ long JpegReadFileToPixmap(char* filename,long size,GID* mem_gid,GID* mem_gid2,lo
       if (reuse_mem){
         WSGFdevfbDeleteMemory((char*)buf);
       }else{
-        delete buf;
+        delete[] buf;
       }
       jpeg_finish_decompress(&jinf);
       jpeg_destroy_decompress(&jinf);
@@ -1842,9 +1900,8 @@ long JpegReadFileToPixmap(char* filename,long size,GID* mem_gid,GID* mem_gid2,lo
     if (reuse_mem){
       WSGFdevfbDeleteMemory((char*)buf);
     }else{
-      delete buf;
+      delete[] buf;
     }
-
     jpeg_finish_decompress(&jinf);
     jpeg_destroy_decompress(&jinf);
     if (size == 0){
