@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 //#include <basic.h>
 //#include <devfb/hmi.h>
@@ -38,10 +34,13 @@
 #include <WSCbase.h>
 #include <WSDmouse.h>
 #include <WSClocaleSet.h>
+#include <WSCbaseList.h>
 #include <devfb/WSDdevfbWinDev.h>
 #include <devfb/WSDdevfbAppDev.h>
 #include <devfb/WSDdevfbMouse.h>
-
+#ifdef MacOS
+#include <WSCbaseList.h>
+#endif
 #define MEMGID
 
 WSMFclassInit(WSDdevfbWinDev,WSDdevfbFormDev);
@@ -50,6 +49,7 @@ dbprintf("_wsddevfbWindev_init_() %s:%d started..\n",__FILE__,__LINE__);
   return new WSDdevfbWinDev();
 }
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _WSDdevfbWinDev_init_ {
   public:_WSDdevfbWinDev_init_(){
 dbprintf("_WSDdevfbWinDev_init_() %s:%d started..\n",__FILE__,__LINE__);
@@ -57,6 +57,8 @@ dbprintf("_WSDdevfbWinDev_init_() %s:%d started..\n",__FILE__,__LINE__);
   };
 };
 static _WSDdevfbWinDev_init_ _init_to_run_WSDdevfbWinDev_;
+#endif
+
 //static Atom wm_protocols = 0;
 //static Atom wm_delete = 0;
 extern TC* WSGFdevfbStringToTc(char*,long*);
@@ -139,8 +141,9 @@ dbprintf("WSDdevfbWinDev::setValue need execute expose..%s:%d\n",__FILE__,__LINE
 #endif
                    createPixmap();
                    clearClipList();
-                   W nr2 = gget_for(getParentGID(),getClipList());
-                   setClipNum(nr2);
+//this cause to fail drawing by noeffective old fore lists..
+//                   W nr2 = gget_for(getParentGID(),getClipList());
+//                   setClipNum(nr2);
                    RECT r;
                    wsta_dsp(_window, &r,NULL);
                    r.c.top =0;
@@ -300,7 +303,13 @@ dbprintf("### WSDdevfbWinDev::createWindow %s vis=%d attr=0x%x\n",getAttachedCli
   }
   _no_frame_first_time = 0;
 //  _top_float = False;
-
+#ifndef NO_MWT_FUNCTION
+  void (*create_handler)(WSCbase*) =
+     (void(*)(WSCbase*))WSGIappObjectList()->getInternalWindowCreateHandler();
+  if (create_handler){
+    create_handler(getAttachedClient());
+  }
+#endif
   short x,y;    
   WSCushort w,h;    
   getWindowSize(&x,&y,&w,&h);
@@ -346,7 +355,7 @@ dbprintf("WSDdevfbWinDev::createWindow title=#%s# tclen=%d\n",_title_str.getStri
   pat.spat.bgcol = bgc;
   pat.spat.mask = FILL100;
 //  wset_bgp(_window,&pat);
- 
+
   if (_top_float != False && _no_frame != False){
 //    UW attr = WA_SUBW | WA_NOFRAME | WA_TITL | WA_WORK | WA_FMOVE | WA_BGDSP;
 //    UW attr = WA_NOFRAME | WA_TITL | WA_WORK | WA_FMOVE | WA_BGDSP;
@@ -381,7 +390,7 @@ dbprintf("WSDdevfbWinDev::createWindow title=#%s# tclen=%d\n",_title_str.getStri
 //printf("wopn_wnd!!! _window=0x%x\n",_window);
   }
   if (tstr2 != NULL){
-    delete tstr2;
+    delete[] tstr2;
     tstr2 = 0;
   }
 dbprintf("WSDdevfbWinDev::createWindow _window=0x%x\n",_window);
@@ -429,6 +438,16 @@ long WSDdevfbWinDev::destroyWindow(){
     wcls_wnd(_window,CLR);
     _window = 0;
     _gid = 0;
+#ifndef NO_MWT_FUNCTION
+    void (*destroy_handler)(WSCbase*) =
+       (void(*)(WSCbase*))WSGIappObjectList()->getInternalWindowDestroyHandler();
+    if (destroy_handler){
+      WSCbase* target = getAttachedClient();
+      if (target != NULL){
+        destroy_handler(target);
+      }
+    }
+#endif
   }
   if (getChildDevListFlag() != False){
     WSDdevfbFormDev* pdev = getParentAreaDev();
@@ -468,10 +487,26 @@ long WSDdevfbWinDev::setVisible(WSCbool fl){
       WSGIdevfbAppDev()->setBtnPressGrabTarget(NULL);
     }
 
+#ifndef NO_MWT_FUNCTION
+    void (*unvis_handler)(WSCbase*) = (void(*)(WSCbase*))WSGIappObjectList()->getInternalWindowUnvisibleHandler();
+    if (unvis_handler){
+      unvis_handler(getAttachedClient());
+    }
+#endif
+
   }else{
     if (_window == 0){
        createWindow();
     }
+
+#ifndef NO_MWT_FUNCTION
+    void (*vis_handler)(WSCbase*) =
+         (void(*)(WSCbase*))WSGIappObjectList()->getInternalWindowVisibleHandler();
+    if (vis_handler){
+      vis_handler(getAttachedClient());
+    }
+#endif
+
     if (WSGIdevfbAppDev()->getMouseGrabbed() != False){
       _copy_area(0,0,_w_w,_w_h,0,0,WS_PIXMAP_TO_WINDOW);
     }
@@ -543,11 +578,12 @@ dbprintf("WSDdevfbFormDev::createPixmap() gcls_env=0x%x\n",ret);
   exposeExecute(0,0,w,h);
 
 #ifdef F_MEMGID
-  WSClistData children = getChildDevList();
-  long num = children.getNum();
+  WSClistData* children = getChildDevList();
+  long num = children->getNum();
   long i;
+  WSDdevfbFormDev** child_buf =(WSDdevfbFormDev**)children->_data;
   for(i=0; i<num; i++){
-    WSDdevfbFormDev* child =(WSDdevfbFormDev*)children[i];
+    WSDdevfbFormDev* child = child_buf[i];
     child->createPixmap();
   }
 #endif
@@ -563,11 +599,12 @@ dbprintf("WSDdevfbFormDev::destroyPixmap() gcls_env=0x%x\n",ret);
     _mem_gid = 0;
   }
   WSDwindowDev::destroyPixmap();
-  WSClistData children = getChildDevList();
-  long num = children.getNum();
+  WSClistData* children = getChildDevList();
+  long num = children->getNum();
   long i;
+  WSDdevfbFormDev** child_buf = (WSDdevfbFormDev**)children->_data;
   for(i=0; i<num; i++){
-    WSDdevfbFormDev* child =(WSDdevfbFormDev*)children[i];
+    WSDdevfbFormDev* child = child_buf[i];
     child->destroyPixmap();
   } 
   return WS_NO_ERR;
