@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -555,7 +551,7 @@ long JpegReadFileToPixmap2(Display* display,Window window,GC gc,char* filename,l
       }
     }
     image->data = data1;
-    delete buf;
+    delete[] buf;
     Pixmap pix = XCreatePixmap(display,window, vwidth,vheight,display_depth);
     *pixmap = pix;
     XPutImage(display,pix,gc,image,0,0,0,0,vwidth,vheight);
@@ -564,7 +560,7 @@ long JpegReadFileToPixmap2(Display* display,Window window,GC gc,char* filename,l
     jpeg_destroy_decompress(&jinf);
 
     if (image->data != NULL){
-      delete image->data;
+      delete[] image->data;
       image->data = NULL;
     }
     XDestroyImage(image);
