@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <mac/WSDmacAppDev.h>
 #include <mac/WSDmacColor.h>
@@ -65,21 +61,6 @@
 #include <../../sys/pixmaps/search2.xpm>
 #include <../../sys/pixmaps/help.xpm>
 #include <../../sys/pixmaps/runx.xpm>
-#include <../../sys/pixmaps/al.xpm>
-#include <../../sys/pixmaps/ml.xpm>
-#include <../../sys/pixmaps/ar.xpm>
-#include <../../sys/pixmaps/mr.xpm>
-#include <../../sys/pixmaps/am.xpm>
-#include <../../sys/pixmaps/alv.xpm>
-#include <../../sys/pixmaps/mlv.xpm>
-#include <../../sys/pixmaps/arv.xpm>
-#include <../../sys/pixmaps/mrv.xpm>
-#include <../../sys/pixmaps/amv.xpm>
-#include <../../sys/pixmaps/rl.xpm>
-#include <../../sys/pixmaps/rm.xpm>
-#include <../../sys/pixmaps/rr.xpm>
-#include <../../sys/pixmaps/mrl.xpm>
-#include <../../sys/pixmaps/mrr.xpm>
 
 //#define TDBG
 
@@ -141,21 +122,6 @@ WSGFsetTimePoint();
    WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/edit.xpm",edit_xpm);
    WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/pset.xpm",pset_xpm);
 
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/al.xpm",al_xpm);
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/ml.xpm",ml_xpm);
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/ar.xpm",ar_xpm);
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/mr.xpm",mr_xpm);
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/am.xpm",am_xpm);
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/alv.xpm",alv_xpm);
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/mlv.xpm",mlv_xpm);
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/arv.xpm",arv_xpm);
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/mrv.xpm",mrv_xpm);
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/amv.xpm",amv_xpm);
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/rr.xpm",rr_xpm);
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/rm.xpm",rm_xpm);
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/rl.xpm",rl_xpm);
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/mrr.xpm",mrr_xpm);
-   WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/mrl.xpm",mrl_xpm);
 
    mapp->execInitHandler();
 //  WSDxcolor::restart_blink_server();
@@ -175,11 +141,16 @@ int WSGFwsInitialize(){
   return 0;
 }
 
-int main(int argc,char** argv){
+int WSGFwsMain(int argc,char** argv){
   main1(argc,argv);
   WSDappDev* app = WSGIappDev();
   WSDmacAppDev* mapp = (WSDmacAppDev*)app->cast("WSDmacAppDev");
   mapp->handleEvents();
   return 0;
 }
+#ifndef NO_MAIN_FUNCTION
+int main(int argc,char** argv){
+  return WSGFwsMain(argc,argv);
+}
+#endif
 
