@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <assert.h>
 //#include <X11/Intrinsic.h>
@@ -96,6 +92,25 @@ _macAppDev_init _macAppDev_init_run_constructor;
 void WSGFdeviceInitialize(){
   static WSCbool initialized = False;
   if (initialized == False){
+extern void WSGFinitFS();
+    WSGFinitFS();
+extern void WSGFunixEnvInit();
+    WSGFunixEnvInit();
+#if 0 //TODO:
+extern void WSGFunixExecuteInit();
+    WSGFunixExecuteInit();
+#endif
+extern void WSGFinitializeUnixFile();
+    WSGFinitializeUnixFile();
+
+#if 0
+extern void WSGFinitializeUnixMutex();
+    WSGFinitializeUnixMutex();
+extern void WSGFinitializeUnixSemaphore();
+    WSGFinitializeUnixSemaphore();
+extern void WSGFinitializeUnixThread();
+    WSGFinitializeUnixThread();
+#endif
 extern WSDfile* _unix_fp_chandler();
     WSDfile::setCreateInstanceHandler((void*)_unix_fp_chandler);
 #if 0 //XXZZ
@@ -442,6 +457,9 @@ long WSDmacAppDev::handleEvents(){
   while(1){
     dispatchEvent();
     WSGIappDevice()->clearDeleteList();
+    if (_exit_event_loop != False){
+      break;
+    }
   }
   return WS_NO_ERR;
 }
@@ -489,6 +507,9 @@ printf("close not supported..\n");
     HiliteMenu( 0 );
 }
 
+long WSDmacAppDev::processEvent(){
+  return WS_ERR;
+}
 long WSDmacAppDev::dispatchEvent(){
     EventRecord event;
     short           part;
