@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <Carbon/Carbon.h>
 #include <WScom.h>
@@ -30,7 +26,7 @@
 #include <WSCcolorSet.h>
 #include <mac/WSDmacColor.h>
 #include <mac/WSDmacAppDev.h>
-//#include <mac/WSDunixFile.h>
+#include <x11/WSDunixFile.h>
 #include <mac/WSDunixFS.h>
 #include <unistd.h>
 #include <signal.h>
@@ -52,6 +48,7 @@ WSDcolor* _macColor_create_handler(){
 void _macColor_init(){
   WSGIappColorSet()->setCreateHandler(_macColor_create_handler);
   WSGFinitUnixFileSystem();
+  WSGFinitializeUnixFile();
 };
 
 
@@ -70,11 +67,11 @@ long WSDmacColor::destroy(){
   return WS_NO_ERR;
 }
 long WSDmacColor::initialize(){
-  long  rval = 0;
-  long  gval = 0;
-  long  bval = 0;
+  int  rval = 0;
+  int  gval = 0;
+  int  bval = 0;
   char* cname = getSrc();
-printf("WSDmacColor::initialize.. cname=%s this=0x%x\n",cname,this);
+//printf("WSDmacColor::initialize.. cname=%s this=0x%x\n",cname,this);
 
   if(cname == NULL){
     cname = "#000000";
