@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <Carbon/Carbon.h>
 #include <WScom.h>
@@ -86,7 +82,7 @@ long _macfont_setup_handler(WSDfont* font,char* str){
   str6[0] = 0;
   str7[0] = 0;
   long no;
-  sscanf(str,"%d %s %s %s %s %s %s %s",&no,str1,str2,str3,str4,str5,str6,str7);
+  sscanf(str,"%ld %s %s %s %s %s %s %s",&no,str1,str2,str3,str4,str5,str6,str7);
   sprintf(fname,"%s %s %s %s %s %s %s",str1,str2,str3,str4,str5,str6,str7);
   font->setFontName(fname);
   font->setId((short)no);
@@ -172,7 +168,7 @@ long WSDmacFont::initialize(){
     strcpy(gfontr,"*");
     strcpy(kfontf,"*");
     strcpy(gfontf,"*");
-    sscanf(fname,"%d %s %s %s %s %s %s",&_font_height,efontr,
+    sscanf(fname,"%ld %s %s %s %s %s %s",&_font_height,efontr,
      fmly,wght,efontf,slant,gfontf);
     _fmly.setString(fmly);
     _wght.setString(wght);
@@ -180,7 +176,7 @@ long WSDmacFont::initialize(){
 //  }else{
 //    //Japanese
 //    japanese = True;
-//    sscanf(fname,"%d %s %s %s %s %s %s",&_font_height,efontr,
+//    sscanf(fname,"%ld %s %s %s %s %s %s",&_font_height,efontr,
 //     kfontr,gfontr,efontf,kfontf,gfontf);
 //  }
 
@@ -345,3 +341,13 @@ printf("WSDmacFont::getStringHeight not supported..\n");
   return  _font_height;
 #endif 
 }
+long WSDmacFont::getAscent(){
+  FontInfo fi;
+  GetFontInfo(&fi);
+  return fi.ascent;
+}
+long WSDmacFont::getDescent(){
+  FontInfo fi;
+  GetFontInfo(&fi);
+  return fi.descent;
+}
