@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <Carbon/Carbon.h>
 #include <WSCdevice.h>
@@ -47,13 +43,14 @@ WSDdev* _wsdmacFormdev_init_(){
   return new WSDmacFormDev();
 }
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _WSDmacFormDev_init_ {
   public:_WSDmacFormDev_init_(){
      WSGIappDevice()->setCreateHandler("formDev",_wsdmacFormdev_init_);
   };
 };
 static _WSDmacFormDev_init_ _init_to_run_WSDmacFormDev_;
-
+#endif
 //static WSDmacDraw* draw_of_macForm = NULL;
 
 WSDmacFormDev::WSDmacFormDev(){
@@ -911,7 +908,15 @@ void WSDmacFormDev::_evh_delete(Widget w,XtPointer ptr,XEvent* ev,WSCbool*){
     if (ev->xclient.message_type ==  XInternAtom(XtDisplay(w),"WM_PROTOCOLS",False) &&
         (WSCulong)ev->xclient.data.l[0] == XInternAtom(XtDisplay(w),"WM_DELETE_WINDOW",False)){
       WSDmacFormDev* dev = (WSDmacFormDev*)ptr;
-      dev->onUnmap();
+      WSCbase* client = dev->getAttachedClient();
+      if (client != NULL){
+        if (WSGFgetWindowEnableClose(client) != False){
+          dev->onUnmap();
+        }
+        client->execEventProc(WSEV_WINDOW_CLOSED,NULL);
+      }else{
+        dev->onUnmap();
+      }
     }
   }
 }
