@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <mac/WSDmacKeyboard.h>
 #include <mac/WSDmacFont.h>
@@ -151,7 +147,7 @@ long WSDmacKeyboard::initialize(){
   return WS_NO_ERR;
 }
 
-long WSDmacKeyboard::startKanji(WSDdev* dev,short x,short y,short fontno,short fgno,short bgno){
+long WSDmacKeyboard::startFep(WSDdev* dev,short x,short y,short fontno,short fgno,short bgno){
 #if 0 //XXZZ
   WSDfont*    font  = WSGIappFontSet()->getFont(fontno);
   WSDcolor*   fgcol = WSGIappColorSet()->getColor(fgno);
@@ -637,7 +633,7 @@ WSCbool WSDmacKeyboard::deliverKeyEvent(XEvent* ev){
         (key == WSK_Zenkaku_Hankaku && withAlt() == True) ||
         (key == WSK_Hiragana_Katakana ) )){
     if (_dev_for_kn != NULL && getEnableFep() == True ){
-      startKanji(_dev_for_kn,_x_kn,_y_kn,_font_kn,_fg_kn,_bg_kn);
+      startFep(_dev_for_kn,_x_kn,_y_kn,_font_kn,_fg_kn,_bg_kn);
       return False;
     }
   }
