@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <Carbon/Carbon.h>
 #include <mac/WSDmacMouse.h>
@@ -39,6 +35,7 @@ WSDmouse* _macmouse_create_handler(){
   return new WSDmacMouse();
 }
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _macmouse_init{
   public: _macmouse_init(){
     WSDmouse::setCreateInstanceHandler(_macmouse_create_handler);
@@ -46,7 +43,7 @@ class _macmouse_init{
 };
 
 _macmouse_init _macmouse_init_run_constructor;
-
+#endif
 
 WSDmacMouse* WSGImacAppMouse(){
   WSDmouse* mouse = WSGIappMouse();
@@ -175,7 +172,7 @@ void WSDmacMouse::setupMouseCursor(char* data){
   char img2[1024];
   char forec[64];
   char backc[64];
-  sscanf(data,"%d %s %s %s %s",&id,img1,img2,forec,backc);
+  sscanf(data,"%ld %s %s %s %s",&id,img1,img2,forec,backc);
   long img = atoi(img1);
   long val = 0;
   if (img != 0){
