@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WSCstring.h>
 #include <WSDenv.h>
@@ -41,14 +37,14 @@ extern void WSGFunixEnvInit();
     WSGFunixEnvInit();
     WSDfileSystem::setCreateInstanceHandler(_unix_fs_chandler);
 }
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _unix_fs_init{
   public: _unix_fs_init(){
-extern void WSGFunixEnvInit();
-    WSGFunixEnvInit();
-    WSDfileSystem::setCreateInstanceHandler(_unix_fs_chandler);
+    WSGFinitUnixFileSystem();
   };
 };
 _unix_fs_init _unix_fs_init_execute;
+#endif
 
 WSDunixFS::WSDunixFS(){}
 WSDunixFS::~WSDunixFS(){}
@@ -151,7 +147,6 @@ WSClistData* WSDunixFS::showDir(char* dname){
     return NULL;
   }
   WSClistData* list = new WSClistData;
-  WSClistData* list2 = new WSClistData;
   seekdir(dptr,0);
   while( (finddata = readdir(dptr)) != NULL){
 //printf(" +-%s\n",finddata->d_name);
@@ -173,9 +168,8 @@ WSClistData* WSDunixFS::showDir(char* dname){
   }
 //printf(" done\n");
   closedir(dptr);
-  WSGFsortCharList(list,list2);
-  delete list;
-  return list2;
+  WSGFsortCharList(list);
+  return list;
 #if 0
   char buf[8192];
   sprintf(buf,"/bin/ls %s",dname);
@@ -208,7 +202,6 @@ WSClistData* WSDunixFS::showFile(char* dname){
     return NULL;
   }
   WSClistData* list = new WSClistData;
-  WSClistData* list2 = new WSClistData;
   while( (finddata = readdir(dptr)) != NULL){
     if (!strcmp(finddata->d_name,"..")){
       continue;
@@ -227,9 +220,8 @@ WSClistData* WSDunixFS::showFile(char* dname){
     }
   }
   closedir(dptr);
-  WSGFsortCharList(list,list2);
-  delete list;
-  return list2;
+  WSGFsortCharList(list);
+  return list;
 #if 0
   char  buf[8192];
   sprintf(buf,"/bin/ls %s",name);
