@@ -1,10 +1,25 @@
-/* XUnicode ver.0.3.2
- * 
- * This software is in the public domain.
- * There are no restrictions on any sort of usage of this software.
- * 
- * $xunicode: xunicode.c,v 1.127.0 2001/04/08 11:10:00 Toshihiro Inoue Exp $
- */
+//
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
+//
+
 #define MacOSX32
 
 #include "xunicode.h"
@@ -153,6 +168,8 @@ printf("xunicode:: get system encoding...\n");
   XUInitSetting();
 #endif
   XUInitTable();
+extern void WSGFsetSystemDefaultEncoding(long);
+  WSGFsetSystemDefaultEncoding(xu_locale_encoding);
 }
 
 
@@ -200,7 +217,7 @@ static void XUInitTable()
   c2 = 0x21;
   for(i = 0; i < 7896; i++) {
     c = table_jis0208[i];
-    if(c) table_rev_jis0208[c] = (c1 << 8) + c2;
+    if(c && table_rev_jis0208[c] == 0) table_rev_jis0208[c] = (c1 << 8) + c2;
     c2++;
     if(c2 == 0x7f) {
       c2 = 0x21;
@@ -269,6 +286,7 @@ static void XUInitLocale()
   xu_lang = getenv("LANG");
   if(!xu_lang) xu_lang = "C";
   strncpy(xu_language, xu_lang, 2);
+  xu_language[2] = 0;
   
   XUInitEncoding();
   
@@ -400,6 +418,9 @@ static void XUInitSetting()
   char fn[64];
   
   home = getenv("HOME");
+  if (home == NULL){
+    home = "";
+  }
   
   snprintf(fn, 64, "%s/.xunicoderc.%s", home, xu_language);
   if(XUReadSetting(fn)) return;
@@ -1015,6 +1036,7 @@ char* XUQueryFont(char* dest, int max, Display* display, const char* family, int
   
   if(count == 1) {
     strncpy(dest, fonts[0], max);
+    dest[max -1] = 0;
   } else {
     XUCompareFonts(dest, max, ideal, fonts, count);
   }
@@ -1053,10 +1075,12 @@ char* XUSearchFont(char* dest, int max, Display* display, const char* font, int
   if(f->none[charset]) return NULL;
   if(f->fontName[charset]) {
     strncpy(dest, f->fontName[charset], max);
+    dest[max -1] = 0;
     return dest;
   }
   
   strncpy(buf, font, 256);
+  buf[255] = 0;
   if(!XUSplitFontName(buf, sep)) {
     f->none[charset] = True;
     return NULL;
@@ -1111,6 +1135,7 @@ char* XUSearchFont(char* dest, int max, Display* display, const char* font, int
   
   if(count == 1) {
     strncpy(dest, fonts[0], max);
+    dest[max-1] = 0;
   } else {
     XUCompareFonts(dest, max, font, fonts, count);
   }
@@ -1170,12 +1195,14 @@ char* XUCompareFonts(char* dest, int max, const char* font, char* fonts[], int c
   int i = 0;
   
   strncpy(buf1, font, 256);
+  buf1[255] = 0;
   if(!XUSplitFontName(buf1, sep1)) return NULL;
   
   s1a = atoi(sep1[6]);
   s1b = atoi(sep1[7]);
   for(i = 0; i < count; i++) {
     strncpy(buf2, fonts[i], 256);
+    buf2[255] = 0;
     if(!XUSplitFontName(buf2, sep2)) continue;
     v = 0;
     if(!strcmp(sep1[3], sep2[3])) v += 400;
@@ -1202,7 +1229,9 @@ char* XUCompareFonts(char* dest, int max, const char* font, char* fonts[], int c
   if(no < 0) return NULL;
   
   strncpy(dest, fonts[no], max);
+  dest[max -1] = 0;
   strncpy(buf2, fonts[no], 256);
+  buf2[255] = 0;
   if(!XUSplitFontName(buf2, sep2)) return dest;
   if(atoi(sep2[6]) != 0) return dest;
   
@@ -1281,6 +1310,7 @@ char* XUGetFontSet(char* dest, int max, const char* separator, Display* display,
   f = XUGetInfoFont(display, font);
   if(f->fontSetName) {
     strncpy(dest, f->fontSetName, max);
+    dest[max -1] = 0;
     return dest;
   }
   
@@ -1839,6 +1869,7 @@ void XUSearchOrder(Display* display, XUInfoFont* f)
   Bool prfBig5;
   
   strncpy(buf, f->name, 256);
+  buf[255] = 0;
   if(!XUSplitFontName(buf, sep)) return;
   
   snprintf(
@@ -1958,6 +1989,7 @@ void XUAddFonts(Display* display, XUInfoFont* f, const char* query)
   
   for(i = 0; i < count; i++) {
     strncpy(buf, fonts[i], 256);
+    buf[255] = 0;
     if(!XUSplitFontName(buf, sep)) continue;
     snprintf(encoding, 32, "%s-%s", sep[12], sep[13]);
     for(j = 0; j < XU_FONT_COUNT; j++) {
@@ -2213,7 +2245,8 @@ int XUctEncode(XUChar* dest, int max, const char* text, int length)
       if(c2 < 0xa1 || c2 > 0xfe) ch = '?';
       if(!ch) ch = table_gb2312[(c1 - 0xa1) * 94 + (c2 - 0xa1)];
     } else if(code == XU_FONT_JIS0208) {
-      if(c1 < 0xa1 || c1 > 0xf4) ch = '?';
+//      if(c1 < 0xa1 || c1 > 0xf4) ch = '?';
+      if(c1 < 0xa1 || c1 > 0xfc) ch = '?';
       if(c2 < 0xa1 || c2 > 0xfe) ch = '?';
       if(!ch) ch = table_jis0208[(c1 - 0xa1) * 94 + (c2 - 0xa1)];
     } else if(code == XU_FONT_KSC5601) {
@@ -2400,7 +2433,8 @@ printf("xunicode.. use mac native... \n");
   if(!c2) return '?';
   if(code == XU_CONV_EUCJP) {
     if(c1 == 0x8e) return c2 + 0xfec0;
-    if(c1 < 0xa1 || c1 > 0xf4) return '?';
+//    if(c1 < 0xa1 || c1 > 0xf4) return '?';
+    if(c1 < 0xa1 || c1 > 0xfc) return '?';
     if(c2 < 0xa1 || c2 > 0xfe) return '?';
     ret = table_jis0208[(c1 - 0xa1) * 94 + (c2 - 0xa1)];
   } else if(code == XU_CONV_SJIS) {
@@ -2571,7 +2605,8 @@ printf("xunicode: must use native unicode conversion...\n");
   if(!c2) return 1;
   if(code == XU_CONV_EUCJP) {
     if(c1 == 0x8e) return 2;
-    if(c1 < 0xa1 || c1 > 0xf4) return 1;
+//    if(c1 < 0xa1 || c1 > 0xf4) return 1;
+    if(c1 < 0xa1 || c1 > 0xfc) return 1;
     if(c2 < 0xa1 || c2 > 0xfe) return 1;
     return 2;
   } else if(code == XU_CONV_SJIS) {
