@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,10 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team 
-// shall not be used in advertising or otherwise to promote the sale, use or 
-// other dealings in this Software without prior written authorization from  
-// WideStudio Development Team.
 
 #include <WSCdb.h>
 #include <WSCdbDrv.h>
@@ -29,9 +25,11 @@
 #include <WSClocaleSet.h>
 
 #include "udebug.h"
+#include <WSDthread.h>
+#include <WSDsemaphore.h>
 
 extern "C" {
-void libwsmysql_func();
+WSDLEX32 void libwsmysql_func();
 };
 void libwsmysql_func(){
 }
@@ -61,6 +59,7 @@ WSCdbMysqlDatabase::WSCdbMysqlDatabase() : WSCdbDrvDatabase()
 	TRACE_U(("WSCdbMysqlDatabase::WSCdbMysqlDatabase start"));
 	
 	_hconn = NULL;
+	_hconn2 = NULL;
 	_open = False;
 	_begin = 0;
 	memset(_dbName, '\0', sizeof(_dbName));
@@ -88,6 +87,9 @@ long WSCdbMysqlDatabase::open(char* host, char* uid, char* pwd, char* dbname, ch
 	if(!(_hconn = mysql_init((MYSQL*)0))) {
 		return WS_ERR;
 	}
+	if(!(_hconn2 = mysql_init((MYSQL*)0))) {
+		return WS_ERR;
+	}
 	if(!mysql_real_connect(_hconn, 
 						   host, 
 						   (uid == NULL ? NULL : strlen(uid) == 0 ? NULL : uid), 
@@ -99,6 +101,19 @@ long WSCdbMysqlDatabase::open(char* host, char* uid, char* pwd, char* dbname, ch
 		getError();
         return WS_ERR;
 	}
+	if(!mysql_real_connect(_hconn2, 
+						   host, 
+						   (uid == NULL ? NULL : strlen(uid) == 0 ? NULL : uid), 
+						   (pwd == NULL ? NULL : strlen(pwd) == 0 ? NULL : pwd), 
+						   (dbname == NULL ? NULL : strlen(dbname) == 0 ? NULL : dbname),
+						   (port == NULL ? 0 : strlen(port) == 0 ? 0 : atoi(port)),
+						   NULL,
+						   0)) {
+		getError();
+        return WS_ERR;
+	}
+
+
 	_open = True;
 	if(dbname) {
 		strcpy(_dbName, dbname);
@@ -119,6 +134,8 @@ long WSCdbMysqlDatabase::close()
 	}
 	mysql_close(_hconn);
 	_hconn = NULL;
+	mysql_close(_hconn2);
+	_hconn2 = NULL;
 	_open = False;
 	memset(_dbName, '\0', sizeof(_dbName));
 	_encoding = -1;
@@ -132,6 +149,9 @@ long WSCdbMysqlDatabase::beginTran()
 		return WS_ERR;
 	}
 	long ret = WS_NO_ERR;
+    if (_hconn == NULL){
+		return WS_ERR;
+    }
 	if(mysql_query(_hconn, "BEGIN")) {
 		getError();
 		ret = WS_ERR;
@@ -145,6 +165,9 @@ long WSCdbMysqlDatabase::commitTran()
 	if(!isOpen()) {
 		return WS_ERR;
 	}
+    if (_hconn == NULL){
+		return WS_ERR;
+    }
 	long ret = WS_NO_ERR;
 	if(mysql_query(_hconn, "COMMIT")) {
 		getError();
@@ -386,6 +409,8 @@ WSCdbMysqlRecord::WSCdbMysqlRecord(WSCdbMysqlDatabase* pDatabase) : WSCdbDrvReco
 	TRACE_U(("WSCdbMysqlRecord::WSCdbMysqlRecord start"));
 
 	Cleanup(pDatabase);
+    _async_thread = NULL;
+    _wait_sem = NULL;
 
 	TRACE_U(("WSCdbMysqlRecord::WSCdbMysqlRecord end"));
 }
@@ -405,6 +430,9 @@ void WSCdbMysqlRecord::Cleanup(WSCdbMysqlDatabase* pDatabase)
 	_FetchRowPos = 0;
 	_encoding = -1;
 	_rowiduse = 0;
+    _is_busy = False;
+    _ret_query = WS_ERR;
+    _tid = 0;
 }
 
 WSCdbMysqlRecord::~WSCdbMysqlRecord()
@@ -475,8 +503,12 @@ long WSCdbMysqlRecord::open(const char* stmt, short eCousor, short eAccess)
 		ret = WS_ERR;
 	} else {
 		_res_stmt = mysql_store_result(_pDB->_hconn);
+        if (_res_stmt == NULL){
+			return WS_ERR;
+        }
 		_RowCount = (unsigned long)mysql_num_rows(_res_stmt);
 		if(BindCols() != WS_NO_ERR) {
+			getError();
 			return WS_ERR;
 		}
 	}
@@ -487,9 +519,95 @@ long WSCdbMysqlRecord::open(const char* stmt, short eCousor, short eAccess)
 	TRACE_U(("WSCdbMysqlRecord::open end"));
 	return WS_NO_ERR;
 }
+void* _async_thread_func(WSDthread* thr,void* ptr){
+  WSCdbMysqlRecord* _this = (WSCdbMysqlRecord*)ptr;
+  if (_this->_pDB != NULL){
+    _this->_wait_sem->lock();
+    my_thread_init();
+    _this->_tid = mysql_thread_id(_this->_pDB->_hconn);
+    _this->_is_busy = True;
+    _this->_ret_query = _this->open(_this->_tmp_stmt, _this->_tmp_ecursor, _this->_tmp_eaccess);
+    my_thread_end();
+    _this->_is_busy = False;
+    _this->_wait_sem->unlock();
+  }
+  return NULL;
+}
+
+long WSCdbMysqlRecord::sendQuery(const char* stmt)
+{
+	return sendQuery(stmt, WSCDB_SQL_CURSOR_STATIC, WSCDB_SQL_CONCUR_LOCK);
+}
+long WSCdbMysqlRecord::sendQuery(const char* stmt, short eCursor, short eAccess)
+{
+  if (_is_busy != False){
+    return WS_DB_BUSY;
+  }
+  close();
+  long ret = WS_ERR;
+  if (_async_thread == NULL){
+    _async_thread = WSDthread::getNewInstance();
+    if (_async_thread == NULL){
+      return WS_ERR;
+    }
+    _async_thread->setFunction(_async_thread_func);
+  }
+  if (_wait_sem == NULL){
+    _wait_sem = WSDsemaphore::getNewInstance();
+    if (_wait_sem == NULL){
+      return WS_ERR;
+    }
+    _wait_sem->initialize();
+  }
+
+  _is_busy = True;
+  _tmp_stmt = stmt;
+  _tmp_ecursor = eCursor;
+  _tmp_eaccess = eAccess;
+  ret = _async_thread->createThread((void*)this);
+  if (ret == WS_NO_ERR){
+    return WS_NO_ERR;
+  }
+//printf("sendQuery returned.. False!!!!!!\n");fflush(stdout);
+  _is_busy = False;
+  return WS_ERR;
+}
+long WSCdbMysqlRecord::getResult(WSCbool wait){
+  if (_is_busy != False){
+    if (wait == False){
+      return WS_DB_BUSY;
+    }
+    _wait_sem->lock();
+    _wait_sem->unlock();
+  }
+  return _ret_query;
+}
+long WSCdbMysqlRecord::cancelQuery(){
+  if (_pDB == NULL){
+    return WS_ERR;
+  }
+  if (_pDB->_hconn2 == NULL){
+    return WS_ERR;
+  }
+  if (_is_busy != False){
+    if (_tid != 0){
+      mysql_kill(_pDB->_hconn2,_tid);
+      _tid = 0;
+    }
+    _wait_sem->lock();
+    _wait_sem->unlock();
+  }
+  return WS_NO_ERR;
+}
+WSCbool WSCdbMysqlRecord::isBusy(){
+  return _is_busy;
+}
 long WSCdbMysqlRecord::close()
 {
 	TRACE_U(("WSCdbMysqlRecord::close start"));
+    if (_is_busy){
+      cancelQuery();
+    }
 	
 	if(_res_stmt) {
 		mysql_free_result(_res_stmt);
@@ -501,7 +619,10 @@ long WSCdbMysqlRecord::close()
 	_FetchRowPos = 0;
 	_encoding = -1;
 	_rowiduse = 0;
-	
+    _is_busy = False;
+    _ret_query = WS_ERR;
+    _tid = 0;
+
 	TRACE_U(("WSCdbMysqlRecord::close end"));
 	return WS_NO_ERR;
 }
@@ -1129,3 +1250,4 @@ long WSCdbMysqlRecord::isNull(const char* name)
 {
 	return isNull(getColIndex(name));
 }
+
