@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <assert.h>
 #include <WSCbaseList.h>
@@ -44,6 +40,9 @@
 #include <nx/WSDnxFormDev.h>
 #include <nx/WSnxcom.h>
 #include <nx/WSDunixThread.h>
+#include <WSDmutex.h>
+#include <WSDthread.h>
+#include <WSDsemaphore.h>
 
 WSMFclassInit(WSDnxAppDev,WSDappDev);
 
@@ -75,6 +74,22 @@ void WSGFdeviceInitialize(){
 dbprintf("WSGFdeviceInitialize.. started..\n");
   static WSCbool initialized = False;
   if (initialized == False){
+extern void WSGFinitFS();
+    WSGFinitFS();
+extern void WSGFunixEnvInit();
+    WSGFunixEnvInit();
+extern void WSGFunixExecuteInit();
+    WSGFunixExecuteInit();
+extern void WSGFinitializeUnixFile();
+    WSGFinitializeUnixFile();
+
+extern void WSGFinitializeUnixMutex();
+    WSGFinitializeUnixMutex();
+extern void WSGFinitializeUnixSemaphore();
+    WSGFinitializeUnixSemaphore();
+extern void WSGFinitializeUnixThread();
+    WSGFinitializeUnixThread();
+
 extern WSDfile* _unix_fp_chandler();
     WSDfile::setCreateInstanceHandler((void*)_unix_fp_chandler);
 //extern WSDdragDrop* _xdrag_drop_create_handler_();
@@ -410,6 +425,9 @@ dbprintf("WSDnxAppDev::handleEvents start dispatchEvent().   XXXXXXXX\n");
     dispatchEvent();
 dbprintf("WSDnxAppDev::handleEvents start clearDeleteList(). XXXXXXXX\n");
     WSGIappDevice()->clearDeleteList();
+    if (_exit_event_loop != False){
+      break;
+    }
   }
   return WS_NO_ERR;
 }
@@ -647,12 +665,15 @@ long WSDnxAppDev::_dispatchEvent(WSCbool wait){
         break;
       }
       case WSEV_NX_THREAD_MESSAGE:{
-        WSDunixThread* ptr = (WSDunixThread*)ev.data;
+        _internal_thr_data_* ptr = (_internal_thr_data_*)ev.data;
+        WSDunixThread* thr = ptr->thr;
+        void* data = ptr->data;
         void(*fn)(WSDunixThread*,void*) =
-             (void(*)(WSDunixThread*,void*))ptr->getCallbackFunction();
+             (void(*)(WSDunixThread*,void*))thr->getCallbackFunction();
         if (fn != NULL){
-          fn(ptr,ptr->getCallbackData());
+          fn(thr,data);
         }
+        delete ptr;
         break;
       }
     }
