@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,17 +14,14 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <xunicode.h>
+#include <WSDappDev.h>
 #include <WScom.h>
 #include <nx/WSDnxCodeConvert.h>
 #include <WSDenv.h>
@@ -41,10 +38,38 @@ int WSDnxCodeConvert::getSystemEncoding(){
 extern int XUGetLocale();
   return XUGetLocale();
 }
+
+static char lc_all_buf[32];
+static char lang_buf[32];
+
 void WSDnxCodeConvert::initialize(){
   XUInit();
-extern char* XUGetLang();
-  setSystemLocaleName(XUGetLang(),getSystemEncoding());
+//extern char* XUGetLang();
+//  setSystemLocaleName(XUGetLang(),getSystemEncoding());
+  char* lang = getenv("LANG");
+  if (lang == NULL){
+    lang = "C";
+  }
+  char buffer[32];
+  strncpy(buffer,lang,32);
+  buffer[31] = 0;
+  char* ptr = strstr(buffer, "@euro");
+  if (ptr != NULL){
+    ptr[0] = 0x2E; // '.'
+    ptr[1] = 0x75; // 'u'
+    ptr[2] = 0x74; // 't'
+    ptr[3] = 0x66; // 'f'
+    ptr[4] = 0x38; // '8'
+
+    strcpy(lc_all_buf,"LC_ALL=");
+    strncat(lc_all_buf,buffer,24);
+    putenv(lc_all_buf);
+    strcpy(lang_buf,"LANG=");
+    strncat(lang_buf,buffer,24);
+    putenv(lang_buf);
+  }
+
+  setSystemLocaleName(buffer,getSystemEncoding());
 }
 int WSDnxCodeConvert::utf8CharLen(const char* str,int max){
   return XUutf8CharLen(str,max); 
