@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCfontSet.h>
@@ -57,8 +53,8 @@ long _nxFont_setup_handler(WSDfont* font,char* str){
   fname[0] = 0;
   long no;
   long fno;
-  sscanf(str,"%d %s %x",&no,str1,&fno);
-  sprintf(fname,"%s %x",str1,fno);
+  sscanf(str,"%ld %s %lx",&no,str1,&fno);
+  sprintf(fname,"%s %lx",str1,fno);
   font->setFontName(fname);
   font->setId((short)no);
   return WS_NO_ERR;
@@ -100,7 +96,7 @@ long WSDnxFont::initialize(){
   char* fname = getSrc();
   _font_height = 14;
   char buffer[1024];
-  sscanf(fname,"%d %s ",&_font_height,buffer);
+  sscanf(fname,"%ld %s ",&_font_height,buffer);
 
   // TODO create font resource from font data..
   setValue1((long)0);
@@ -137,3 +133,11 @@ long WSDnxFont::getStringHeight(WSCstring*){
   getValue1();
   return  _font_height;
 }
+long WSDnxFont::getAscent(){
+  getValue1();
+  return  _font_height*4/5;
+}
+long WSDnxFont::getDescent(){
+  getValue1();
+  return  _font_height - _font_height*4/5;
+}
