@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <nx/WSDnxMouse.h>
 #include <nx/WSDnxAppDev.h>
@@ -39,6 +35,7 @@ dbprintf("_nxMouse_create_handler() %s:%d started..\n",__FILE__,__LINE__);
   return new WSDnxMouse();
 }
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _nxMouse_init{
   public: _nxMouse_init(){
 dbprintf("_nxMouse_init() %s:%d started..\n",__FILE__,__LINE__);
@@ -47,7 +44,7 @@ dbprintf("_nxMouse_init() %s:%d started..\n",__FILE__,__LINE__);
 };
 
 _nxMouse_init _nxMouse_init_run_constructor;
-
+#endif
 
 WSDnxMouse* WSGInxAppMouse(){
   WSDmouse* mouse = WSGIappMouse();
