@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <nx/WSDnxNwDev.h>
 #include <nx/WSDnxAppDev.h>
@@ -36,12 +32,14 @@ WSDdev* _wsdnxNwdev_init_(){
   return new WSDnxNwDev();
 }
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _WSDnxNwDev_init_ {
   public:_WSDnxNwDev_init_(){
      WSGIappDevice()->setCreateHandler("nwDev",_wsdnxNwdev_init_);
   };
 };
 static _WSDnxNwDev_init_ _init_to_run_WSDnxNwDev_;
+#endif
 
 static WSDnxDraw* draw_of_nxNw = NULL;
 
