@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <nx/WSDnxAppDev.h>
@@ -44,13 +40,23 @@
 #include <nx/WSnxcom.h>
 
 #ifdef SUN
-#define SOCKLEN_T  int
-#else //SUN
-#ifdef MacOS
-#define SOCKLEN_T  int
-#else //MacOS
+
+#ifdef __GNUC__
+#if __GNUC__ > 2
 #define SOCKLEN_T  socklen_t
-#endif //MacOS
+#else
+#define SOCKLEN_T  int
+#endif
+#else
+#define SOCKLEN_T  int
+#endif
+
+#else //SUN
+//#ifdef MacOS  //removed for 10.4 tigger
+//#define SOCKLEN_T  int
+//#else //MacOS
+#define SOCKLEN_T  socklen_t
+//#endif //MacOS
 #endif //SUN
 
 WSMFclassInit(WSDnxSocket,WSDnxSocket);
@@ -61,6 +67,7 @@ WSDsocket* _nxSocket_create_handler(){
 };
 
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class  _nxSocket_init {
   public:
   _nxSocket_init(){
@@ -69,7 +76,7 @@ class  _nxSocket_init {
   };
 };
 _nxSocket_init   WSGIappXsocketInit;
-
+#endif
 
 // A new semaphore function for initializing.
 int pthread_sem_init_np(pthread_sem_t* semp){
@@ -419,8 +426,8 @@ long WSDnxSocket::acceptEx(void(*hd)(WSDsocket*,void*,WSCulong)){
 
 long WSDnxSocket::close(void* sock){
   if (sock != 0){
-    ::shutdown((int)sock,0);
-    ::close((int)sock);
+    ::shutdown((long)sock,0);
+    ::close((long)sock);
     seterr();
     return WS_NO_ERR;
   }else{
@@ -463,29 +470,29 @@ long WSDnxSocket::read(void* handle,WSCuchar* buf,long size){
     }
   }
   WSCulong timeout = 0;
-  if (_socket == (int)handle){
+  if (_socket == (long)handle){
     timeout = _timeout *1000;
   }else{
     timeout = _cl_timeout *1000;
   }
   if (timeout > 0){
-    int fl = fcntl((int)handle,F_GETFL,0);
+    int fl = fcntl((long)handle,F_GETFL,0);
     if (fl == -1){
       seterr();
       return 0;
     }
-    int ret = fcntl((int)handle,F_SETFL,fl | FNDELAY);
+    int ret = fcntl((long)handle,F_SETFL,fl | FNDELAY);
     if (ret == -1){
       seterr();
       return 0;
     }
   }else{
-    int fl = fcntl((int)handle,F_GETFL,0);
+    int fl = fcntl((long)handle,F_GETFL,0);
     if (fl == -1){
       seterr();
       return 0;
     }
-    int ret = fcntl((int)handle,F_SETFL,fl & ~FNDELAY);
+    int ret = fcntl((long)handle,F_SETFL,fl & ~FNDELAY);
     if (ret == -1){
       seterr();
       return 0;
@@ -498,26 +505,26 @@ long WSDnxSocket::read(void* handle,WSCuchar* buf,long size){
   while(1){
     long ret = 0;
     if (_udp == False){
-      ret = recv((int)handle,(char*)&buf[read_len],len_to_read,0);
+      ret = recv((long)handle,(char*)&buf[read_len],len_to_read,0);
     }else{
       struct sockaddr_in fromaddr;
       fromaddr.sin_family = AF_INET;
       fromaddr.sin_port = 0;
       fromaddr.sin_addr.s_addr = 0;
       SOCKLEN_T len = sizeof(fromaddr);
-      ret = recvfrom((int)handle,(char*)&buf[read_len],len_to_read,0,(sockaddr*)&fromaddr,&len);
+      ret = recvfrom((long)handle,(char*)&buf[read_len],len_to_read,0,(sockaddr*)&fromaddr,&len);
       _set_dest_addr_(htonl(fromaddr.sin_addr.s_addr));
       _dest_port = htons(fromaddr.sin_port);
     }
     if (timeout > 0){
 #ifndef MacOS
       struct pollfd fds;
-      fds.fd = (int)handle;
+      fds.fd = (long)handle;
       fds.events = POLLIN;
       fds.revents = 0;
 #else //MacOS
       struct timeval tm;
-      tm.tv_sec = timeout;
+      tm.tv_sec = timeout/1000;
       tm.tv_usec = 0;
       fd_set read_fds;
       FD_ZERO(&read_fds);
@@ -596,7 +603,7 @@ long WSDnxSocket::write(void* handle,WSCuchar* buf,long size){
   if (_udp != False){
     int val;
     val = 1;
-    int ret = setsockopt((int)handle, SOL_SOCKET, SO_BROADCAST,
+    int ret = setsockopt((long)handle, SOL_SOCKET, SO_BROADCAST,
                          (char *)&val, sizeof(val));
     if (ret < 0){
       seterr();
@@ -605,30 +612,30 @@ long WSDnxSocket::write(void* handle,WSCuchar* buf,long size){
 //printf("WSDnxSocket::write setsockopt... _socket=%d handle=%d ret=%d\n",_socket,handle,ret);
   }
   WSCulong timeout = 0;
-  if (_socket == (int)handle){
+  if (_socket == (long)handle){
     timeout = _timeout *1000;
   }else{
     timeout = _cl_timeout *1000;
   }
 
   if (timeout > 0){
-    int fl = fcntl((int)handle,F_GETFL,0);
+    int fl = fcntl((long)handle,F_GETFL,0);
     if (fl == -1){
       seterr();
       return 0;
     }
-    int ret = fcntl((int)handle,F_SETFL,fl | FNDELAY);
+    int ret = fcntl((long)handle,F_SETFL,fl | FNDELAY);
     if (ret == -1){
       seterr();
       return 0;
     }
   }else{
-    int fl = fcntl((int)handle,F_GETFL,0);
+    int fl = fcntl((long)handle,F_GETFL,0);
     if (fl == -1){
       seterr();
       return 0;
     }
-    int ret = fcntl((int)handle,F_SETFL,fl & ~FNDELAY);
+    int ret = fcntl((long)handle,F_SETFL,fl & ~FNDELAY);
     if (ret == -1){
       seterr();
       return 0;
@@ -641,7 +648,7 @@ long WSDnxSocket::write(void* handle,WSCuchar* buf,long size){
   while(1){
     long ret = 0;
     if (_udp == False){
-      ret = send((int)handle,(char*)&buf[write_len],len_to_write,0);
+      ret = send((long)handle,(char*)&buf[write_len],len_to_write,0);
     }else if (_udp_binded == False){
       struct sockaddr_in toaddr;
       toaddr.sin_family = AF_INET;
@@ -652,18 +659,18 @@ long WSDnxSocket::write(void* handle,WSCuchar* buf,long size){
         toaddr.sin_addr.s_addr =(unsigned long)inet_addr((char*)_dest_naddr);
       }
 
-      ret = sendto((int)handle,(char*)&buf[write_len],len_to_write,0,
+      ret = sendto((long)handle,(char*)&buf[write_len],len_to_write,0,
                    (sockaddr*)&toaddr,sizeof(toaddr));
     }
-    if (_timeout > 0){
+    if (timeout > 0){
 #ifndef MacOS
       struct pollfd fds;
-      fds.fd = (int)handle;
+      fds.fd = (long)handle;
       fds.events = POLLOUT;
       fds.revents = 0;
 #else //MacOS
       struct timeval tm;
-      tm.tv_sec = _timeout;
+      tm.tv_sec = timeout/1000;
       tm.tv_usec = 0;
       fd_set write_fds;
       FD_ZERO(&write_fds);
