@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,22 +14,19 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team 
-// shall not be used in advertising or otherwise to promote the sale, use or 
-// other dealings in this Software without prior written authorization from  
-// WideStudio Development Team.
-
+//
 
 #include <WSCdb.h>
 #include <WSCdbDrv.h>
 #include <WSCdbOdbc.h>
 #include <WSCodbcDatabase.h>
 #include "udebug.h"
-
+#include <WSDthread.h>
+#include <WSDsemaphore.h>
 
 #ifdef FACTORY_METHOD /* factory method handler..*/
 WSCdbDrvDatabase* _WSCdbOdbcDatabase_create_(){
@@ -158,16 +155,30 @@ WSCdbOdbcRecord::WSCdbOdbcRecord(WSCdbOdbcDatabase* pDatabase) : WSCdbDrvRecord(
 {
 	_pDB = pDatabase;
 	_rs = NULL;
+    _async_thread = NULL;
+    _wait_sem = NULL;
+    _is_busy = False;
+    _ret_query = WS_ERR;
 }
 WSCdbOdbcRecord::~WSCdbOdbcRecord()
 {
 	close();
 	_pDB = NULL;
+    if (_async_thread != NULL){
+      delete _async_thread;
+    }
+    if (_wait_sem != NULL){
+      delete _wait_sem;
+    }
 }
 long WSCdbOdbcRecord::open(const char* stmt)
 {
 	return open(stmt, SQL_CURSOR_STATIC, SQL_CONCUR_LOCK);
 }
+long WSCdbOdbcRecord::sendQuery(const char* stmt)
+{
+	return open(stmt, SQL_CURSOR_STATIC, SQL_CONCUR_LOCK);
+}
 long WSCdbOdbcRecord::open(const char* stmt, short eCousor, short eAccess)
 {
 	close();
@@ -178,8 +189,81 @@ long WSCdbOdbcRecord::open(const char* stmt, short eCousor, short eAccess)
 	}
 	return ret;
 }
+void* _async_thread_func(WSDthread* thr,void* ptr){
+  WSCdbOdbcRecord* _this = (WSCdbOdbcRecord*)ptr;
+  _this->_wait_sem->lock();
+  _this->_is_busy = True;
+  _this->_ret_query = _this->_rs->open(_this->_tmp_stmt, _this->_tmp_ecursor, _this->_tmp_eaccess);
+  _this->_is_busy = False;
+  _this->_wait_sem->unlock();
+  return NULL;
+}
+WSCbool WSCdbOdbcRecord::isBusy(){
+  return _is_busy;
+}
+long WSCdbOdbcRecord::sendQuery(const char* stmt, short eCursor, short eAccess)
+{
+  if (_is_busy != False){
+    return WS_DB_BUSY;
+  }
+  close();
+  long ret = WS_ERR;
+  if (_async_thread == NULL){
+    _async_thread = WSDthread::getNewInstance();
+    if (_async_thread == NULL){
+      return WS_ERR;
+    }
+    _async_thread->setFunction(_async_thread_func);
+    _wait_sem = WSDsemaphore::getNewInstance();
+    if (_wait_sem == NULL){
+      return WS_ERR;
+    }
+    _wait_sem->initialize();
+  }
+  _rs = new WSCodbcRecord(_pDB->_db);
+  if(_rs){
+    _is_busy = True;
+    _tmp_stmt = stmt;
+    _tmp_ecursor = eCursor;
+    _tmp_eaccess = eAccess;
+    ret = _async_thread->createThread((void*)this);
+    if (ret == WS_NO_ERR){
+      return WS_NO_ERR;
+    }
+    _is_busy = False;
+    return WS_ERR;
+  }
+  return WS_ERR;
+}
+long WSCdbOdbcRecord::getResult(WSCbool wait){
+  if (_is_busy != False){
+    if (_rs != NULL){
+      if (wait == False){
+        return WS_DB_BUSY;
+      }
+      _wait_sem->lock();
+      _wait_sem->unlock();
+      return _ret_query;
+    }else{
+      return WS_ERR;
+    }
+  }else{
+    return _ret_query;
+  }
+}
+long WSCdbOdbcRecord::cancelQuery(){
+  if (_is_busy != False){
+    _rs->cancel();
+    _wait_sem->lock();
+    _wait_sem->unlock();
+  }
+  return WS_NO_ERR;
+}
 long WSCdbOdbcRecord::close()
 {
+    if (_is_busy){
+      cancelQuery();
+    }
 	long ret = WS_ERR;
 	if(_rs) {
 		ret = _rs->close();
