@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team 
-// shall not be used in advertising or otherwise to promote the sale, use or 
-// other dealings in this Software without prior written authorization from  
-// WideStudio Development Team.
-
+//
 
 #include <WSCdb.h>
 #include <WSCodbcDatabase.h>
@@ -96,6 +92,7 @@ long WSCodbcDatabase::open(char* dsn, char* uid, char* pwd)
 	TRACE_U(("WSCodbcDatabase::open end"));
 	return WS_NO_ERR;
 }
+
 long WSCodbcDatabase::close()
 {
 	TRACE_U(("WSCodbcDatabase::close start"));
@@ -382,6 +379,14 @@ long WSCodbcRecord::open(const char* stmt, short eCousor, short eAccess)
 	TRACE_U(("WSCodbcRecord::open end"));
 	return WS_NO_ERR;
 }
+long WSCodbcRecord::cancel(){
+  TRACE_U(("WSCodbcRecord::cancelstart"));
+  if(_hstmt) {
+    SQLCancel(_hstmt);
+  }
+  TRACE_U(("WSCodbcRecord::cancel end"));
+  return WS_NO_ERR;
+}
 long WSCodbcRecord::close()
 {
 	TRACE_U(("WSCodbcRecord::close start"));
@@ -626,8 +631,11 @@ long WSCodbcRecord::getRowCount()
 {
 	SQLRETURN	sqlrc = SQL_SUCCESS;
 	long nCount = -1;
-
+#ifndef SQL64
     sqlrc = SQLRowCount(_hstmt, (SQLINTEGER*)&nCount);
+#else
+    sqlrc = SQLRowCount(_hstmt, (SQLLEN*)&nCount);
+#endif
 
 	return nCount;
 }
@@ -640,7 +648,11 @@ long WSCodbcRecord::BindCols()
 	TRACE_U(("WSCodbcRecord::BindCols start"));
 
 	SQLRETURN	sqlrc = SQL_SUCCESS;
+#ifndef SQL64
 	SQLINTEGER	colDataDisplaySize;
+#else
+	SQLLEN	colDataDisplaySize;
+#endif
 	int i;
 
     sqlrc = SQLNumResultCols(_hstmt, &_Cols);
