@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCopenglForm.h>
@@ -56,7 +52,7 @@
 #include <WSDappDev.h>
 
 extern "C" {
-void libwsopengl_func();
+WSDLEX32 void libwsopengl_func();
 };
 void libwsopengl_func(){
 }
@@ -82,6 +78,9 @@ WSCopenglForm::WSCopenglForm(WSCbase* base, char* objname):
     WSMFparentCheckVerSrc(WSCopenglForm);
     WSMFpropertyDelete(WSNpixmapStyle);
 
+    WSMFaddExtTrigger(WSEV_OPENGL_INITIALIZE,"OPENGL-INITIALIZE");
+    WSMFaddExtTrigger(WSEV_OPENGL_DELETE,"OPENGL-DELETE");
+
   WSMFpropertyCreateEnd
 
 }
@@ -101,12 +100,20 @@ WSCopenglForm::~WSCopenglForm(){
   }
 }
 void WSCopenglForm::_delete_context(){
+//printf("WSCopenglForm::_delete_context here!!\n");
+  if (_context_created_fl == False){
+    return;
+  }
+  _context_created_fl = False;
+//printf("WSCopenglForm::_delete_context2 here!!\n");
   if (_opengl_context != NULL){
 #ifndef MSW
     WSDdev* dev = getdev();
     if (dev != NULL){
       Display* disp = (Display*)dev->getDeviceResource();
-      glXDestroyContext(disp,(GLXContext) _opengl_context);
+      if (disp != (Display*)-1){
+        glXDestroyContext(disp,(GLXContext) _opengl_context);
+      }
     }
 #else //MSW
     wglMakeCurrent(NULL, NULL);
@@ -114,6 +121,8 @@ void WSCopenglForm::_delete_context(){
 #endif //MSW
     _opengl_context = NULL;
   }
+  _context_created_fl = False;
+  onOpenGLDelete();
 }
 
 void WSCopenglForm::setCreateContextHandler(long (*hd)()){
@@ -130,7 +139,9 @@ long WSCopenglForm::_create_context(){
   }
   _context_created_fl = True;
   if (_create_context_handler != NULL){
-    return _create_context_handler();
+    int ret = _create_context_handler();
+    onOpenGLInitialize();
+    return ret;
   }
   return _default_create_context();
 }
@@ -180,7 +191,19 @@ long WSCopenglForm::_default_create_context(){
     printf("The XVisual isn't TrueColor.\n");
     return WS_ERR;
   }
-
+#ifndef MacOSX
+//printf("visualid=0x%x 0x%x\n",visual_info->visualid,DefaultVisual(disp,DefaultScreen(disp))->visualid);
+  if ( visual_info->visualid != DefaultVisual(disp,DefaultScreen(disp))->visualid){
+    XFree(visual_info);
+    XVisualInfo vi;
+    vi.visualid = DefaultVisual(disp,DefaultScreen(disp))->visualid;
+    int n_vis = 1;
+    XVisualInfo* vinfo2 = XGetVisualInfo(disp,VisualIDMask,&vi,&n_vis);
+    if(vinfo2 != NULL){
+      visual_info = vinfo2;
+    }
+  }
+#endif
   _opengl_context = glXCreateContext(disp, visual_info, None, True);
   if(_opengl_context == NULL){
     printf("Can't open rendering context.\n");
@@ -226,7 +249,7 @@ long WSCopenglForm::_default_create_context(){
     printf("wglCreateContext fails..\n");
     return WS_ERR;
   }
-  //wglMakeCurrent(disp, (HGLRC)_opengl_context);
+//  wglMakeCurrent(disp, (HGLRC)_opengl_context);
 
 #endif //MSW
   WSCrect rect;
@@ -235,18 +258,29 @@ long WSCopenglForm::_default_create_context(){
   rect.width = _w;
   rect.height = _h;
   onResize(&rect);
-
+  onOpenGLInitialize();
   return WS_NO_ERR;
 }
 
+void WSCopenglForm::onParentVisibleChange(WSCbool vis){
+//printf("WSCopenglForm::onParentVisibleChange vis=%d!!\n",vis);
+  WSCform::onParentVisibleChange(vis);
+  WSCbool fl = getVisible();
+  if (vis == False){
+    _delete_context();
+  }else if (fl != False){
+  }
+}
 void WSCopenglForm::onVisibleChange(WSCbool vis){
+//printf("WSCopenglForm::onVisibleChange vis=%d!!\n",vis);
   WSCform::onVisibleChange(vis);
   WSCbool fl = getVisible();
   if (vis == False){
     _delete_context();
   }else if (fl != False){
 //printf("WSCopenglForm::onVisibleChange _create_context..\n");
-    _create_context();
+    //move to exposed..
+    //_create_context();
   }
 
 }
@@ -286,21 +320,31 @@ long WSCopenglForm::beginDraw(){
   setAbsoluteDraw(False);
   _begin_draw_fl =1;
 
-
+  if (_context_created_fl == False){
+    _create_context();
+  }else{
 #ifndef MSW
-  Window win = (Window)dev->getWindowResource();
-  Display* disp = (Display*)dev->getDeviceResource();
-  glXMakeCurrent(disp, win, (GLXContext)_opengl_context);
-#else
+    Window win = (Window)dev->getWindowResource();
+    Display* disp = (Display*)dev->getDeviceResource();
+    glXMakeCurrent(disp, win, (GLXContext)_opengl_context);
+#endif
+  }
+#ifdef MSW
   HDC disp = (HDC)getUserData("HDC");
   wglMakeCurrent(disp, (HGLRC)_opengl_context);
 #endif
-
   //fprintf(stderr, "_begin_draw\n");
 
 
   return WS_NO_ERR;
 }
+void WSCopenglForm::onOpenGLInitialize(){
+  execProcedure(WSEV_OPENGL_INITIALIZE);
+}
+void WSCopenglForm::onOpenGLDelete(){
+//printf("WSCopenglForm::onOpenGLDelete here!!\n");
+  execProcedure(WSEV_OPENGL_DELETE);
+}
 long WSCopenglForm::endDraw(){
 //printf("SwapBuffers... here1\n");
   if(_double_buffer == True){
@@ -436,32 +480,50 @@ long WSCopenglForm::_default_update_context(){
 }
 long WSCopenglForm::execEventProc(long ev,void* data){
   if (ev == WSEV_EXPOSE){
-#ifndef MSW
-#else
-    HDC disp = (HDC)getUserData("HDC");
-    wglMakeCurrent(disp, (HGLRC)_opengl_context);
-#endif
+//#ifndef MSW  // move to beginDraw()..
+//#else
+//    HDC disp = (HDC)getUserData("HDC");
+//    wglMakeCurrent(disp, (HGLRC)_opengl_context);
+//#endif
     if (data != NULL){
       WSCrect* area = (WSCrect*)data;
       onExpose(area);
       draw();
       if (_expose_op == 1){
         if (beginDraw()== WS_NO_ERR){
+#ifndef WS_EMBED
+          if (getPropertyEditMode() != False){
+            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+#if 0
+            static GLuint teapotdlist;
+            static long init = False;
+            if (init == False){
+              glShadeModel(GL_SMOOTH);
+              glEnable(GL_DEPTH_TEST);
+              glEnable(GL_CULL_FACE);
+              glEnable(GL_TEXTURE_2D);
+
+              init = True;
+            }
+#endif
+          }
+#endif
           execProcedure(ev);
           endDraw();
         }
       }
 #ifndef MSW
 #else
+      HDC disp = (HDC)getUserData("HDC");
       wglMakeCurrent(disp, NULL);
 #endif
       return WS_NO_ERR;
     }else{
-#ifndef MSW
-#else
-    HDC disp = (HDC)getUserData("HDC");
-    wglMakeCurrent(disp, (HGLRC)_opengl_context);
-#endif 
+//#ifndef MSW  // move to beginDraw()..
+//#else
+//    HDC disp = (HDC)getUserData("HDC");
+//    wglMakeCurrent(disp, (HGLRC)_opengl_context);
+//#endif 
       WSCrect area;
       area.setRect(0,0,_w,_h);
       onExpose(&area);
@@ -474,6 +536,7 @@ long WSCopenglForm::execEventProc(long ev,void* data){
       }
 #ifndef MSW
 #else
+      HDC disp = (HDC)getUserData("HDC");
       wglMakeCurrent(disp, NULL);
 #endif
       return WS_NO_ERR;
