@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,20 +14,17 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCbase.h>
 #include <WSCprocedure.h>
 #include <stdio.h>
+#include "wsperl.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -40,19 +37,6 @@ extern "C" {
 }
 #endif
 
-#if 0 //deleted by hirabayashi
-EXTERN_C void boot_DynaLoader (pTHX_ CV* cv);
-EXTERN_C void boot_mpfc_perl (pTHX_ CV* cv);
-
-EXTERN_C void
-xs_init(pTHX)
-{
-  char *file = __FILE__;
-  newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
-  newXS("mpfc::boot_mpfc", boot_mpfc_perl, file);
-}
-#endif
-
 WSCindexData flist;
 void WSGFfunctionRegister(char* fname, void* perlfunc){
   flist.setData(fname,(void*)perlfunc);
@@ -64,7 +48,7 @@ void _procedure_handler(char* fname, WSCbase* object)
 extern PerlInterpreter * current_interpreter;
   PerlInterpreter* my_perl = current_interpreter;
   if (my_perl == NULL){
-printf("my_perl is NULL!\n");
+    fprintf(stderr,"Error: _procedure_handler: PerlInterpreter is NULL.\n");
     return;
   }
 #endif
@@ -76,23 +60,12 @@ printf("my_perl is NULL!\n");
 #else
   snprintf(filename, 4096, "%s.pl", fname);
 #endif
-#if 0   //deleted by hirabayashi
-  char *my_argv[] = { fname,  filename};
-
-  PerlInterpreter *ws_mpfc;
-  ws_mpfc = perl_alloc();
-  
-  perl_construct(ws_mpfc);
-  perl_parse(ws_mpfc, xs_init, 2, my_argv, (char **)NULL);
-
-  //perl_run(ws_mpfc);
-#endif
 
   perl_require_pv(filename); //added by hirabayashi
 
   SV* wstestobj = sv_newmortal();
   char cname1[1024];
-  sprintf(cname1,"mpfc::%s",object->getClassName());
+  sprintf(cname1,cname_header,object->getClassName());
   sv_setref_pv(wstestobj, cname1, (void*)object->cast(object->getClassName()));
 
   dSP;                            /* initialize stack pointer      */
@@ -109,10 +82,6 @@ printf("my_perl is NULL!\n");
   FREETMPS;                       /* free that return value        */
   LEAVE;                          /* ...and the XPUSHed "mortal" args.*/
 
-#if 0  //deleted by hirabayashi
-  //perl_destruct(ws_mpfc);
-  perl_free(ws_mpfc);
-#endif
 
   PL_perl_destruct_level -= 1;
 }
