@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -46,7 +42,6 @@ void png_cexcept_error(png_structp png_ptr, png_const_charp msg){
 
 long PngReadFileToPixmap(Display* display,Window window,GC gc,char* filename,Pixmap* pixmap,Pixmap* pixmap_mask,long* w,long* h){
     *pixmap_mask = (Pixmap)0;
-
     int red_mask, green_mask, blue_mask;
     int red_shift, green_shift, blue_shift;
     int start_shift, msb_flag;
@@ -229,7 +224,7 @@ fprintf(stderr,"png read error: out of memory..\n");
 fprintf(stderr,"png read error: out of memory..\n");
       png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
       fclose(ifile);
-      delete buf;
+      delete[] buf;
       return -1;
     }
     int i;
@@ -242,7 +237,7 @@ fprintf(stderr,"png read error: out of memory..\n");
     }
     png_read_image(png_ptr, png_row_ptrs);
     png_read_end(png_ptr,NULL);
-    delete png_row_ptrs;
+    delete[] png_row_ptrs;
 
     long vwidth = *w;
     long vheight = *h;
@@ -262,7 +257,6 @@ fprintf(stderr,"png read error: out of memory..\n");
     vheight = *h;
 
 
-
     XImage* image = 0;
     XImage* image_mask = 0;
     Visual* visual = XDefaultVisual(display,XDefaultScreen(display));
@@ -289,7 +283,6 @@ fprintf(stderr,"png read error: out of memory..\n");
         fclose(ifile);
                  return -1;
     }
-    //
     if ((win_attr.depth == 24) || (win_attr.depth == 16)) {
       if (win_attr.depth == 24) {
         start_shift = 24;
@@ -335,11 +328,14 @@ fprintf(stderr,"png read error: out of memory..\n");
     if (image->bits_per_pixel ==32){
       if (components == 3 || components == 4){
         for(i=0; i<vheight; i++){
+          ptr = i * png_row_bytes;
           for(j=0; j<vwidth; j++){
             if (stretched != 0){
               ptr = (long)((double)i*rheight/vheight)*rwidth
                    + (long)((double)j*rwidth/vwidth);
-              ptr *= 3;
+              if (components == 4){
+                ptr *= 3;
+              }else
               if (components == 4){
                 ptr *= 4;
               }
@@ -349,7 +345,11 @@ fprintf(stderr,"png read error: out of memory..\n");
             b = buf[ptr++];
             if (components == 4){
               if( buf[ptr++]){
-                ((unsigned char*)data2)[ptr3] |= (1<< (j%8));
+                if (msb_flag ==1){
+                  ((unsigned char*)data2)[ptr3] |= (128>> (j%8));
+                }else{
+                  ((unsigned char*)data2)[ptr3] |= (1<< (j%8));
+                }
               }
               if (j%8 == 7){
                 ptr3++;
@@ -385,10 +385,10 @@ fprintf(stderr,"png read error: out of memory..\n");
               ((unsigned char*)data1)[ptr2++] = (udat & 0xff000000)>>24;
             }
           }
-          ptr = i * png_row_bytes;
         }
       }else{
         for(i=0; i<vheight; i++){
+          ptr = i * png_row_bytes;
           for(j=0; j<vwidth; j++){
             if (stretched != 0){
               ptr = (long)((double)i*rheight/vheight)*rwidth
@@ -409,18 +409,20 @@ fprintf(stderr,"png read error: out of memory..\n");
               ((unsigned char*)data1)[ptr2++] =  0;
             }
           }
-          ptr = i * png_row_bytes;
         }
       }
     }else if (image->bits_per_pixel == 24){
       if (components == 3 || components == 4){
         for(i=0; i<vheight; i++){
+          ptr = i * png_row_bytes;
           ptr2 = i * image->bytes_per_line;
           for(j=0; j<vwidth; j++){
             if (stretched != 0){
               ptr = (long)((double)i*rheight/vheight)*rwidth
                    + (long)((double)j*rwidth/vwidth);
-              ptr *= 3;
+              if (components == 4){
+                ptr *= 3;
+              }else
               if (components == 4){
                 ptr *= 4;
               }
@@ -430,7 +432,11 @@ fprintf(stderr,"png read error: out of memory..\n");
             b = buf[ptr++];
             if (components == 4){
               if( buf[ptr++]){
-                ((unsigned char*)data2)[ptr3] |= (1<< (j%8));
+                if (msb_flag ==1){
+                  ((unsigned char*)data2)[ptr3] |= (128>> (j%8));
+                }else{
+                  ((unsigned char*)data2)[ptr3] |= (1<< (j%8));
+                }
               }
               if (j%8 == 7){
                 ptr3++;
@@ -466,7 +472,6 @@ fprintf(stderr,"png read error: out of memory..\n");
               ((unsigned char*)data1)[ptr2++] = (udat & 0xff0000)>>16;
             }
           }
-          ptr = i * png_row_bytes;
         }
       }else{
         for(i=0; i<vheight; i++){
@@ -493,11 +498,14 @@ fprintf(stderr,"png read error: out of memory..\n");
     }else if (image->bits_per_pixel ==16){
       if (components == 3 || components == 4){
         for(i=0; i<vheight; i++){
+          ptr = i * png_row_bytes;
           for(j=0; j<vwidth; j++){
             if (stretched != 0){
               ptr = (long)((double)i*rheight/vheight+0.5)*rwidth
                    + (long)((double)j*rwidth/vwidth + 0.5);
-              ptr *= 3;
+              if (components == 3){
+                ptr *= 3;
+              }else
               if (components == 4){
                 ptr *= 4;
               }
@@ -507,7 +515,11 @@ fprintf(stderr,"png read error: out of memory..\n");
             unsigned int bb = buf[ptr++];
             if (components == 4){
               if( buf[ptr++]){
-                ((unsigned char*)data2)[ptr3] |= (1<< (j%8));
+                if (msb_flag ==1){
+                  ((unsigned char*)data2)[ptr3] |= (128>> (j%8));
+                }else{
+                  ((unsigned char*)data2)[ptr3] |= (1<< (j%8));
+                }
               }
               if (j%8 == 7){
                 ptr3++;
@@ -540,10 +552,10 @@ fprintf(stderr,"png read error: out of memory..\n");
               ((unsigned char*)data1)[ptr2++] = (udat & 0xff00)>>8;
             }
           }
-          ptr = i * png_row_bytes;
         }
       }else{
         for(i=0; i<vheight; i++){
+          ptr = i * png_row_bytes;
           for(j=0; j<vwidth; j++){
             if (stretched != 0){
               ptr = (long)((double)i*rheight/vheight)*rwidth
@@ -554,7 +566,6 @@ fprintf(stderr,"png read error: out of memory..\n");
             b = buf[ptr++]>>3;
             ((short*)data1)[ptr2++] =  r <<11 | g<<5 | b;
           }
-          ptr = i * png_row_bytes;
         }
       }
     }else if (image->bits_per_pixel == 8){
@@ -595,13 +606,16 @@ fprintf(stderr,"png read error: out of memory..\n");
         dr1 = 0;
         dg1 = 0;
         db1 = 0;
+        ptr = i * png_row_bytes;
         for(j=0; j<vwidth; j++){
 
           if (components == 3 || components== 4){
             if (stretched != 0){
               ptr = (long)((double)i*rheight/vheight)*rwidth
                      + (long)((double)j*rwidth/vwidth);
-              ptr *= 3;
+              if (components == 3){
+                ptr *= 3;
+              }else
               if (components == 4){
                 ptr *= 4;
               }
@@ -611,7 +625,11 @@ fprintf(stderr,"png read error: out of memory..\n");
             tb1 = buf[ptr++];
             if (components == 4){
               if( buf[ptr++]){
-                ((unsigned char*)data2)[ptr3] |= (1<< (j%8));
+                if (msb_flag ==1){
+                  ((unsigned char*)data2)[ptr3] |= (128>> (j%8));
+                }else{
+                  ((unsigned char*)data2)[ptr3] |= (1<< (j%8));
+                }
               }
               if (j%8 == 7){
                 ptr3++;
@@ -673,18 +691,17 @@ fprintf(stderr,"png read error: out of memory..\n");
           colptr = (rr>>6)*5*5  + (gg>>6)*5 + (bb>>6);
           data1[ptr2++] =  col[colptr].pixel;
         }
-        ptr = i * png_row_bytes;
       }
     }
     image->data = data1;
-    delete buf;
+    delete[] buf;
 
     Pixmap pix = XCreatePixmap(display,window, vwidth,vheight,display_depth);
     *pixmap = pix;
     XPutImage(display,pix,gc,image,0,0,0,0,vwidth,vheight);
 
     if (image->data != NULL){
-      delete image->data;
+      delete[] image->data;
       image->data = NULL;
     }
     XDestroyImage(image);
@@ -702,7 +719,7 @@ fprintf(stderr,"png read error: out of memory..\n");
       *pixmap_mask = pix;
       XPutImage(display,pix,gc2,image_mask,0,0,0,0,vwidth,vheight);
       if (image_mask->data != NULL){
-        delete image_mask->data;
+        delete[] image_mask->data;
         image_mask->data = NULL;
       }
       XFreeGC(display,gc2);
