@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,16 +14,13 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team 
-// shall not be used in advertising or otherwise to promote the sale, use or 
-// other dealings in this Software without prior written authorization from  
-// WideStudio Development Team.
-
+//
 
+#include <WScom.h>
 #include <WSCdb.h>
 #include <WSCdbDrv.h>
 #include <WSCdbPgsql.h>
@@ -32,7 +29,7 @@
 #include "udebug.h"
 
 extern "C" {
-void libwspostgres_func();
+WSDLEX32 void libwspostgres_func();
 };
 void libwspostgres_func(){
 }
@@ -98,7 +95,7 @@ long WSCdbPgsqlDatabase::open(char* host, char* uid, char* pwd, char* dbname, ch
 						  (uid == NULL ? NULL : strlen(uid) == 0 ? NULL : uid),
 						  (pwd == NULL ? NULL : strlen(pwd) == 0 ? NULL : pwd));
 	if(PQstatus(_hconn) == CONNECTION_BAD) {
-		getError();
+		getError(NULL);
         return WS_ERR;
 	}
 	_open = True;
@@ -159,7 +156,7 @@ long WSCdbPgsqlDatabase::beginTran()
 	PGresult* res = NULL;
 	res = PQexec(_hconn, "BEGIN");
 	if(IS_PGSQL_ERROR(res)) {
-		getError();
+		getError(res);
 		ret = WS_ERR;
 	} else {
 		_begin = PGSQL_TRAN_FUNCTION;
@@ -178,7 +175,7 @@ long WSCdbPgsqlDatabase::commitTran()
 	PGresult* res = NULL;
 	res = PQexec(_hconn, "COMMIT");
 	if(IS_PGSQL_ERROR(res)) {
-		getError();
+		getError(res);
 		ret = WS_ERR;
 	} else {
 		_begin = False;
@@ -197,7 +194,7 @@ long WSCdbPgsqlDatabase::abortTran()
 	PGresult* res = NULL;
 	res = PQexec(_hconn, "ROLLBACK");
 	if(IS_PGSQL_ERROR(res)) {
-		getError();
+		getError(res);
 		ret = WS_ERR;
 	} else {
 		_begin = False;
@@ -219,28 +216,48 @@ long WSCdbPgsqlDatabase::sqlExecute(const char* stmt)
 
 	res = PQexec(_hconn, decode(stmt));
 	if(IS_PGSQL_ERROR(res)) {
-		getError();
+		getError(res);
 		ret = WS_ERR;
 	}
 	PQclear(res);
 	TRACE_U(("WSCdbPgsqlDatabase::sqlExecute end"));
 	return ret;
 }
+
 WSCbool WSCdbPgsqlDatabase::isOpen()
 {
 	return _open;
 }
-void WSCdbPgsqlDatabase::getError()
+void WSCdbPgsqlDatabase::getError(PGresult* res)
 {
 	TRACE_U(("WSCdbPgsqlDatabase::getError start"));
 
 	strcpy((char*)_szSqlState, "DB001");
 	memset(_szErrorMsg, '\0', sizeof(_szErrorMsg));
-	strncpy((char*)_szErrorMsg, PQerrorMessage(_hconn), sizeof(_szErrorMsg)-1);
-
+    if (res != NULL){
+      char* err = PQerrorMessage(_hconn);
+      char* msg = PQresultErrorMessage(res);
+      if (msg == NULL){
+        msg = "";
+      }
+#if defined(_VC) || defined (_BCC)
+      _snprintf((char*)_szErrorMsg,sizeof(_szErrorMsg)-1,"%s res:%s",err,msg);
+#else
+      snprintf((char*)_szErrorMsg,sizeof(_szErrorMsg)-1,"%s res:%s",err,msg);
+#endif
+    }else{
+	  strncpy((char*)_szErrorMsg, PQerrorMessage(_hconn), sizeof(_szErrorMsg)-1);
+    }
+    _szErrorMsg[PGSQL_MAX_MESSAGE_LENGTH-1] = 0;
+    _szErrorMsg[PGSQL_MAX_MESSAGE_LENGTH] = 0;
 	TRACE_U(("WSCdbPgsqlDatabase::getError State:%s messgae:%s", _szSqlState, _szErrorMsg));
 	TRACE_U(("WSCdbPgsqlDatabase::getError end"));
 }
+void WSCdbPgsqlDatabase::setError(char* msg){
+  strncpy((char*)_szErrorMsg, msg, sizeof(_szErrorMsg)-1);
+  _szErrorMsg[PGSQL_MAX_MESSAGE_LENGTH-1] = 0;
+  _szErrorMsg[PGSQL_MAX_MESSAGE_LENGTH] = 0;
+}
 
 void WSCdbPgsqlDatabase::getErrorMsg(char* szState, char* szMsg)
 {
@@ -445,7 +462,7 @@ long WSCdbPgsqlRecord::open(const char* stmt, short eCousor, short eAccess)
 	long ret = WS_NO_ERR;
 	WSCbool	select = False;
 	PGresult* res = NULL;
-	char*	buf;
+	char*	buf = NULL;
 	char*	sql;
 	char	oid[256];
 	char	buf1[80];
@@ -494,7 +511,9 @@ long WSCdbPgsqlRecord::open(const char* stmt, short eCousor, short eAccess)
 				PQclear(res);
 				sprintf(buf2, "CLOSE %s", _CurrsorName);
 				res = PQexec(_pDB->_hconn, buf2);
-			}
+			}else{
+		      getError(res);
+            }
 			PQclear(res);
 		}
 	} else {
@@ -556,11 +575,14 @@ long WSCdbPgsqlRecord::open(const char* stmt, short eCousor, short eAccess)
 	}
 */
 	buf = new char[strlen(stmt) + strlen(buf1) + sizeof(oid) + 1];
-
+	if (buf == NULL){
+        setError("Memory allocation error..");
+		return WS_ERR;
+	}
 	sprintf(buf, "%s%s%s", buf1, oid, sql);
 	res = PQexec(_pDB->_hconn, buf);
 	if(IS_PGSQL_ERROR(res)) {
-		getError();
+		getError(res);
 		ret = WS_ERR;
 	}
 	PQclear(res);
@@ -568,27 +590,240 @@ long WSCdbPgsqlRecord::open(const char* stmt, short eCousor, short eAccess)
 	if(select && ret == WS_NO_ERR) {
 		sprintf(buf, "FETCH ALL IN %s", _CurrsorName);
 		_res_stmt = PQexec(_pDB->_hconn, buf);
-		if(IS_PGSQL_ERROR(res)) {
-			getError();
+		if(IS_PGSQL_ERROR(_res_stmt)) {
+			getError(res);
 			ret = WS_ERR;
 		} else {
 			_RowCount = PQntuples(_res_stmt);
 		}
 	}
-	delete[] buf;
-
+	if (buf != NULL){
+		delete[] buf;
+		buf = NULL;
+	}
 	if(BindCols() != WS_NO_ERR) {
 		return WS_ERR;
 	}
 	_open = True;
 
-//	delete[] buf;
 
 	moveNext();
 
 	TRACE_U(("WSCdbPgsqlRecord::open end"));
 	return WS_NO_ERR;
 }
+long WSCdbPgsqlRecord::sendQuery(const char* stmt)
+{
+	return sendQuery(stmt, WSCDB_SQL_CURSOR_STATIC, WSCDB_SQL_CONCUR_LOCK);
+}
+#define WS_RETRY_LOOP 100
+long WSCdbPgsqlRecord::sendQuery(const char* stmt, short eCousor, short eAccess)
+{
+  TRACE_U(("WSCdbPgsqlRecord::sendQuery start"));
+  if(_pDB == NULL) {
+    return WS_ERR;
+  }
+  if(_pDB->_hconn == NULL) {
+    return WS_ERR;
+  }
+  if(isOpen()) {
+    close();
+  }
+  _open = False;
+  _Cursor = eCousor;
+  _access = eAccess;
+  _RowCount = 0;
+  _FetchRowPos = 0xffffffff;
+  _encoding = _pDB->getEncode();
+  memset(_tableName, '\0', sizeof(_tableName));
+
+  long ret = WS_NO_ERR;
+  WSCbool	select = False;
+  PGresult* res = NULL;
+  char*	buf = NULL;
+  char*	sql;
+  char	oid[256];
+  char	buf1[80];
+  char	buf2[256];
+  strncpy(buf1, stmt, 6);
+  buf1[6] = '\0';
+
+  oid[0] = '\0';
+  _oiduse = 0;
+
+  if(WSGFstricmp(buf1, "select") == 0) {
+    select = True;
+    if(!_pDB->getbeginTranmode()) {
+      if(_pDB->beginTran() == WS_ERR) {
+        return WS_ERR;
+      }
+      _pDB->setbeginTranmode(PGSQL_TRAN_RECORD_FUNCTION);
+    }
+    sprintf(_CurrsorName, "WSdbCursor%d%d", _pDB->getProcessId(), _pDB->getCursor());
+#ifndef BINARY
+    sprintf(buf1, "DECLARE %s CURSOR FOR ", _CurrsorName);
+#else
+    sprintf(buf1, "DECLARE %s BINARY CURSOR FOR ", _CurrsorName);
+#endif	
+    sql = (char*)&stmt[6];
+    getTableName((char*)stmt, _tableName, sizeof(_tableName));
+    strcpy(oid, "select ");
+
+    if(_access != WSCDB_SQL_CONCUR_READ_ONLY) {
+      sprintf(buf2, "%s select count(a.attname) from "
+                 "pg_class c, pg_attribute a "
+                 "where c.relname = '%s' "
+                 "and a.attrelid = c.oid and attname='oid'", buf1, _tableName);
+      // oid item check
+      res = PQexec(_pDB->_hconn, buf2);
+      TRACE_U(("WSCdbPgsqlRecord::sendRequest oid %s", buf2));
+      if(!IS_PGSQL_ERROR(res)) {
+        PQclear(res);
+        sprintf(buf2, "FETCH ALL IN %s", _CurrsorName);
+        res = PQexec(_pDB->_hconn, buf2);
+        if(atoi((char*)PQgetvalue(res, 0, 0)) > 0) {
+          _oiduse = 1;
+          sprintf(oid, "select %s.oid as oid_%s,", _tableName, _CurrsorName);
+          TRACE_U(("WSCdbPgsqlRecord::open oid use"));
+        }
+        PQclear(res);
+        sprintf(buf2, "CLOSE %s", _CurrsorName);
+        res = PQexec(_pDB->_hconn, buf2);
+      }
+      PQclear(res);
+    }
+  }else{
+    _CurrsorName[0] = '\0';
+    buf1[0] = '\0';
+    sql = (char*)stmt;
+  }
+
+  buf = new char[strlen(stmt) + strlen(buf1) + sizeof(oid) + 1];
+  if (buf == NULL){
+    return WS_ERR;
+  }
+  sprintf(buf, "%s%s%s", buf1, oid, sql);
+  res = PQexec(_pDB->_hconn, buf);
+  if(IS_PGSQL_ERROR(res)) {
+    getError(res);
+    ret = WS_ERR;
+  }
+  PQclear(res);
+	
+  if(select && ret == WS_NO_ERR) {
+    sprintf(buf, "FETCH ALL IN %s", _CurrsorName);
+    PQsetnonblocking(_pDB->_hconn,True);
+    int retv = False;
+    int cnt = 0;
+    while(1){
+      cnt++;
+      retv = PQsendQuery(_pDB->_hconn, buf);
+      if (retv != False){
+        break;
+      }
+      if (cnt > WS_RETRY_LOOP){
+        break;
+      }
+    }
+    if (retv == False){
+      getError(NULL);
+      ret = WS_ERR;
+    }
+  }
+  if (buf != NULL){
+    delete[] buf;
+    buf = NULL;
+  }
+  _open = True;
+  return ret;
+}
+
+#define WS_CONSUME_LOOP 10
+long WSCdbPgsqlRecord::getResult(WSCbool wait){
+  if(_pDB == NULL) {
+    return WS_ERR;
+  }
+  if(_pDB->_hconn == NULL) {
+    return WS_ERR;
+  }
+  int ret = 0;
+  if (wait == False){
+    int i;
+    for(i=0; i<WS_CONSUME_LOOP; i++){
+      int ret = PQconsumeInput(_pDB->_hconn);
+      if (ret == 0){
+#if 0 //removed for WIN32
+        getError();
+        return False;
+#endif
+      }
+    }
+    ret = PQisBusy(_pDB->_hconn);
+    if (ret == 1){
+      return WS_DB_BUSY;
+    }
+  }
+  ret = WS_NO_ERR;
+  _res_stmt = PQgetResult(_pDB->_hconn);
+  if(IS_PGSQL_ERROR(_res_stmt)) {
+    getError(_res_stmt);
+    ret = WS_ERR;
+    PQclear(_res_stmt);
+    _res_stmt = NULL;
+  }else{
+    _RowCount = PQntuples(_res_stmt);
+  }
+  if(BindCols() != WS_NO_ERR) {
+    return WS_ERR;
+  }
+  moveNext();
+
+  TRACE_U(("WSCdbPgsqlRecord::getResult end"));
+  return WS_NO_ERR;
+}
+
+WSCbool WSCdbPgsqlRecord::isBusy(){
+  if(_pDB == NULL) {
+    return False;
+  }
+  if(_pDB->_hconn == NULL) {
+    return False;
+  }
+  int ret = 0;
+  int i;
+  for(i=0; i<WS_CONSUME_LOOP; i++){
+    int ret = PQconsumeInput(_pDB->_hconn);
+
+    if (ret == 0){
+#if 0 //removed for WIN32
+      getError();
+      return False;
+#endif
+    }
+  }
+  ret = PQisBusy(_pDB->_hconn);
+  if (ret == 1){
+    return True;
+  }
+  return False;
+}
+
+long WSCdbPgsqlRecord::cancelQuery(){
+  if(_pDB == NULL) {
+    return WS_ERR;
+  }
+  if(_pDB->_hconn == NULL) {
+    return WS_ERR;
+  }
+  if (_open == False){
+    return WS_ERR;
+  }
+  int ret = PQrequestCancel(_pDB->_hconn);
+  if (ret == 1){
+    close();
+  }
+  return WS_NO_ERR;
+}
 long WSCdbPgsqlRecord::close()
 {
 	TRACE_U(("WSCdbPgsqlRecord::close start"));
@@ -841,7 +1076,7 @@ long WSCdbPgsqlRecord::BindCols()
 //	if(_Cols < 2) {
 	if(_Cols < 1) {
 		TRACE_U(("WSCdbPgsqlRecord::BindCols PQnfields() Error"));
-		getError();
+		getError(NULL);
 		return WS_ERR;
 	}
 	_fd = new _PgsqlCol[_Cols];
@@ -984,17 +1219,36 @@ void WSCdbPgsqlRecord::setValue(int index, WSCvariant* var)
 	TRACE_U(("WSCdbPgsqlRecord::setValue end"));
 }
 
-void WSCdbPgsqlRecord::getError()
+void WSCdbPgsqlRecord::getError(PGresult* res)
 {
 	TRACE_U(("WSCdbPgsqlRecord::getError start"));
 
 	strcpy((char*)_szSqlState, "REC01");
 	memset(_szErrorMsg, '\0', sizeof(_szErrorMsg));
-	strncpy((char*)_szErrorMsg, PQerrorMessage(_pDB->_hconn), sizeof(_szErrorMsg)-1);
-
+    if (res != NULL){
+      char* err = PQerrorMessage(_pDB->_hconn);
+      char* msg = PQresultErrorMessage(res);
+      if (msg == NULL){
+        msg = "";
+      }
+#if defined(_VC) || defined (_BCC)
+      _snprintf((char*)_szErrorMsg,sizeof(_szErrorMsg)-1,"%s res:%s",err,msg);
+#else
+      snprintf((char*)_szErrorMsg,sizeof(_szErrorMsg)-1,"%s res:%s",err,msg);
+#endif
+    }else{
+	  strncpy((char*)_szErrorMsg, PQerrorMessage(_pDB->_hconn), sizeof(_szErrorMsg)-1);
+    }
+    _szErrorMsg[PGSQL_MAX_MESSAGE_LENGTH-1] = 0;
+    _szErrorMsg[PGSQL_MAX_MESSAGE_LENGTH] = 0;
 	TRACE_U(("WSCdbPgsqlRecord::getError State:%s messgae:%s", _szSqlState, _szErrorMsg));
 	TRACE_U(("WSCdbPgsqlRecord::getError end"));
 }
+void WSCdbPgsqlRecord::setError(char* msg){
+  strncpy((char*)_szErrorMsg, msg, sizeof(_szErrorMsg)-1);
+  _szErrorMsg[PGSQL_MAX_MESSAGE_LENGTH-1] = 0;
+  _szErrorMsg[PGSQL_MAX_MESSAGE_LENGTH] = 0;
+}
 
 void WSCdbPgsqlRecord::getErrorMsg(char* szState, char* szMsg)
 {
