@@ -1,8 +1,14 @@
 #ifndef mpfc_H
 #define mpfc_H
 
+#include "../../include/wsconfig.h"
+
 typedef unsigned char  WSCbool;
+#if defined(SWIGOCAML) || defined(SWIGRUBY)
 typedef void*          WSCptr;
+#else
+typedef long           WSCptr;
+#endif
 typedef unsigned short WSCushort;
 typedef unsigned long  WSCulong;
 typedef unsigned char  WSCuchar;
@@ -61,6 +67,12 @@ typedef unsigned int   WSCuint;
 #define WS_POLICY_GRADATION  4
 #define WS_POLICY_AQUA       8
 
+#define WS_TYPE_WINDOW   0x0001
+#define WS_TYPE_MANAGER  0x0002
+#define WS_TYPE_FORM     0x0004
+#define WS_TYPE_SCR_FORM 0x0008
+#define WS_TYPE_MENU_WINDOW 0x0010
+
 typedef struct {
   long x, y;
   inline void setPoint(long vx,long vy){
@@ -121,7 +133,9 @@ struct WSCrect {
 class WSCstring {
   public:
     WSCstring(char*, long en = WS_EN_DEFAULT);
+#ifndef SWIGOCAML
     WSCstring(const WSCvariant&, long en = WS_EN_DEFAULT);
+#endif
     WSCstring();
     ~WSCstring();
 
@@ -136,8 +150,10 @@ class WSCstring {
     long getEncoding();
     long isExist(const char*,long encode = WS_EN_DEFAULT );
 
+#ifndef SWIGOCAML
     void addString(const WSCstring&);
     void addString(const WSCvariant&,long encode = WS_EN_DEFAULT);
+#endif
     void addString(const char*,long encode = WS_EN_DEFAULT);
     void cutString(WSCulong char_pos);
     void insertString(WSCulong char_pos,WSCstring&);
@@ -171,17 +187,18 @@ class WSCstring {
     long eof();
     long getLines();
     WSCstring gets();
-
-#if defined(SWIGPYTHON) || defined(SWIGRUBY)
+#if defined(SWIGPYTHON) || defined(SWIGRUBY) || defined(SWIGOCAML)
+#ifndef SWIGOCAML
     WSCstring operator + (const WSCstring&);
     WSCstring operator + (const WSCvariant&);
-    WSCstring operator + (const char*);
     WSCstring& operator << (const WSCstring&);
     WSCstring& operator << (const WSCvariant&);
+#endif
+    WSCstring operator + (const char*);
     WSCstring& operator << (const char*);
     WSCstring& operator << (long);
     WSCstring& operator << (double);
-#endif
+#endif /* defined(SWIGPYTHON) || defined(SWIGRUBY) || defined(SWIGOCAML) */
 
 #ifdef SWIGPERL
     %rename(eq) operator ==;
@@ -196,10 +213,16 @@ class WSCstring {
 
 class WSCvariant {
   public:
+#ifdef SWIGOCAML
+    WSCvariant(int);
+#else
     WSCvariant(long);
+#endif
     WSCvariant(double);
-    WSCvariant(const WSCvariant&);
     WSCvariant(char*);
+#ifndef SWIGOCAML
+    WSCvariant(const WSCvariant&);
+#endif
     WSCvariant(WSCbase*);
     WSCvariant();
 
@@ -228,9 +251,14 @@ class WSCvariant {
     }
 #endif /* defined(SWIGPYTHON) || defined(SWIGRUBY) */
 
-    void setValue(long);
-    void setValue(char*);
     void setValue(double);
+    void setValue(char*);
+#ifdef SWIGOCAML
+    void setValue(int);
+#endif
+#ifndef SWIGOCAML
+    void setValue(long);
+#endif
     void setValue(WSCbase*);
 
     long getType();
@@ -247,13 +275,36 @@ public:
   WSCushort getHeight();
   virtual void update() = 0;
   void exit(long);
+  long lock();
+  long unlock();
+  void setRebaseHandle(long);
+  long getRebaseHandle();
 };
-
 extern WSDappDev* WSGIappDev();
 extern int WSGFwsInitialize();
+extern int WSGFwsInitialize2(int);
 extern long WSGFdestroyWindow(WSCbase*);
 extern void WSGFdeviceInitialize(); extern void WSGFfunctionRegister(char*,PyObject*);
 //extern void WSGFfunctionRegister(char*,void*);
+class WSCprocedure {
+  public:
+    WSCprocedure(char*,long);
+    ~WSCprocedure();
+    long setProcName(char*);
+    long setProc(void*);
+    long setEffectiveTriggerMask(long);
+    long setTrigger(long);
+    long setFunction(void(*)(WSCbase*),char*);
+    void setInternal(WSCbool);
+    void setUseFile(WSCbool fl);
+    char* getProcName();
+    char* getFunctionName();
+    void* getProc();
+    long  getEffectiveTriggerMask();
+    long  getTrigger();
+    WSCbool getInternal();
+    WSCbool getUseFile();
+};
 
 class WSCbase {
 public:
@@ -288,18 +339,29 @@ public:
   void onDefaultColorChange();
 
   char* getInstanceName();
+  char* getClassName();
   void setVisible(WSCbool);
   WSCbool getVisible();
-
+  WSCbool isParent(WSCbase*);
   WSCvariant getProperty(char*);
+#ifndef SWIGOCAML
   WSCbool setProperty(char*,const WSCvariant&);
+#endif
   WSCbool setProperty(char*,char*);
+
+#ifndef SWIGOCAML
   WSCbool setProperty(char*,long);
+#else
+  WSCbool setProperty(char*,int);
+#endif
+
   WSCbool setProperty(char*,double);
-#ifdef SWIGRUBY
+#if defined(SWIGRUBY)
   WSCbool setProperty(char*,bool);
 #endif
+  WSCbase* getNextChild(WSCbase*);
   long addProcedure(char* pname,char* fname,long);
+  long delProcedure(long id);
 
   void setParent(WSCbase*);
   WSCbase* getParent();
@@ -332,15 +394,32 @@ public:
 
   void setUserData(char*,void*);
   void* getUserData(char*);
+  void setUserLongData(char*,WSCulong);
+  WSCulong getUserLongData(char*);
+  void setUserStringData(char*,char*);
+  char* getUserStringData(char*);
 
+#ifndef SWIGOCAML
   void setVariantData(char*,WSCvariant);
+  void setVariantData(char*,long);
+#else
   void setVariantData(char*,int);
-  void setVariantData(char*,char*);
+#endif
   void setVariantData(char*,double);
+  void setVariantData(char*,char*);
+
+#ifndef SWIGOCAML
   WSCvariant& getVariantData(char*);
+#else
+  WSCvariant getVariantData(char*);
+#endif
 
   void execProcedure(char*);
+#ifndef SWIGOCAML
   void execProcedure(long);
+#else
+  void execProcedure(int);
+#endif
 
   long draw();
   long clear();
@@ -351,11 +430,18 @@ public:
   WSDdev* getowndev();
   WSDdev* getParentDev();
   //WSDdev* attachdev();
-
+  
+  long setShowGriphand(WSCbool);
+  long drawGriphand();
+  long clearGriphand();
+  long setPropertyEditExecute(WSCbool);
 static WSCbase* getNewInstance(char* cname,WSCbase*,char*);
 static WSCbase* getFocusInstance();
 static WSCbase* getSpecialFocusInstance();
-
+  long getObjectType();
+  void update();
+  WSCbool existTrigger(long);
+  WSCindexData* getExtTriggerList();
 };
 
 class WSDmouse {
@@ -425,6 +511,13 @@ public:
   ~WSCvfbtn();
 };
 
+class WSCvendbtn :public WSCvbtn{
+public:
+  WSCvendbtn(WSCbase*,char*);
+  ~WSCvendbtn();
+};
+
+
 class WSCvklabel :public WSCvlabel{
 public:
   WSCvklabel(WSCbase*,char*);
@@ -616,7 +709,7 @@ public:
   void onActivate();
 };
 
-class WSCoption :public WSCform{
+class WSCoption :public WSCvlabel{
 public:
   WSCoption(WSCbase*,char*);
   ~WSCoption();
@@ -967,6 +1060,8 @@ class WSCwindow :public WSCbase{
 public:
   WSCwindow(WSCbase*,char*);
   ~WSCwindow();
+  long getWindowStatus();
+  long setWindowStatus(long status);
 };
 
 class WSCmainWindow :public WSCwindow{
@@ -1166,7 +1261,9 @@ public:
 
   void onSort();
   void onItemSelected();
+#ifndef SWIGOCAML
   void onInputFixed(WSCstring,long,long);
+#endif
   void onInputFixed(char*,long,long);
 
   void setLabelHeight(WSCushort );
@@ -1201,7 +1298,8 @@ public:
   WSCstring getInputString();
   void cancelInput();
   char* getSeparator();
-
+  long getSelectedNextPos(long);
+  WSCstring getSelectedNextItem(long);
 };
 
 class WSCverbList :public WSClist{
@@ -1277,7 +1375,7 @@ public:
   char* getFontName();
   long getStringWidth(char* str,long en = WS_EN_DEFAULT);
   long getStringHeight(char* str,long en = WS_EN_DEFAULT);
-
+  WSCbool getReady(WSCbool loading = False);
 };
 class WSCfontSet{
 public:
@@ -1288,6 +1386,7 @@ public:
   void setDefaultFont(short);
   short getDefaultFontNo();
   WSDfont* getDefaultFont();
+  WSCbool getReady(short no,WSCbool loading = False);
 };
 extern WSCfontSet* WSGIappFontSet();
 extern void WSGFdestroyPixmap(char*);
@@ -1306,7 +1405,7 @@ class WSCbaseList{
   /* virtual */  ~WSCbaseList();
   
   WSCbase* getInstance(char* class_name,char* objname);
-  WSClistData* getClassList();
+//  WSClistData* getClassList();
 
   void addClient(WSCbase*,void* bases = NULL);
   void delClient(WSCbase* ,void* bases = NULL);
@@ -1318,25 +1417,34 @@ class WSCbaseList{
   void execInitialize();
   void  setUserData(char*,void *);
   void* getUserData(char*);
+  void  setUserLongData(char*,WSCulong);
+  WSCulong  getUserLongData(char*);
+  void  setUserStringData(char*,char*);
+  char* getUserStringData(char*);
 
-  WSClistData* getInstanceList(char*);
+//  WSClistData* getInstanceList(char*);
   WSCRbase*    getRemoteInstance(char*);
 
   void         setSearchInstanceHandler(void*);
   WSCbase*     searchInstance(char* class_name,char* objname);
   WSCbool      existInstance(WSCbase*);
+
   void     addEvent(WSCbase*,long ev = WSEV_NONE);
   void     delEvent(WSCbase*,long ev = WSEV_NONE);
   void     execEvent(long,void*);
   void     execEvent(long);
   void     execEvent(char*);
 
+  void     addEventProc(char* fname,char*,long);
+  void     delEventProc(char* fname,char*);
+  void     execEventProc(char* ev);
+  void     delAllEventProc();
+
   long     addRemoteInstanceData(WSCremoteInstanceData* idata);
   long     delRemoteInstanceData(WSCremoteInstanceData* idata);
   WSCbool  getRemoteInstanceListUpdated();
   void     setRemoteInstanceListUpdated(WSCbool);
 
-
   long     addExportInstanceList(WSCbase* inst);
   long     delExportInstanceList(WSCbase* inst);
   long     addExportInstanceUpdateList(WSCbase* inst);
@@ -1345,17 +1453,28 @@ class WSCbaseList{
   long     addExportInstanceDeleteList(WSCbase* inst);
   long     delExportInstanceDeleteList(WSCbase* inst);
 
+//  WSClistData* getExportInstanceDeleteList();
+//  WSClistData* getExportInstanceUpdateList();
+//  WSClistData* getExportInstanceList();
 
-  WSClistData* getExportInstanceDeleteList();
-  WSClistData* getExportInstanceUpdateList();
-  WSClistData* getExportInstanceList();
-  
   WSCremoteInstanceData*  getRemoteInstanceData(char* pname);
-  WSClistData* getRemoteInstanceList();
+//  WSClistData* getRemoteInstanceList();
+
+  void      setNewInstancePositionMode(WSCbool fl);
+  WSCbool   getNewInstancePositionMode();
+  void      setNewInstanceClassName(char*);
+  WSCstring getNewInstanceClassName();
+
+  void      setInstanceId(long kind,WSCulong data);
+  WSCulong  getInstanceId(WSCulong id,long kind,WSCulong data);
+  WSCstring getInstanceData(WSCulong id,long kind,WSCulong data);
+  WSCbase*  getInstanceFromId(WSCulong id);
+  WSCulong  getIdFromInstance(WSCbase* inst);
   
 };
 
 extern WSCbaseList *WSGIappObjectList();
+extern WSCbase* WSGFreadWinFile(char* dirname,char* fname,long encoding);
 
 //WSCsheet
 #define WSCSHEET_DEFAULT               -1
@@ -1457,20 +1576,26 @@ public:
   long      setText(long text);
   long      setText(double text);
   long      setText(const char* text);
+#ifndef SWIGOCAML
   long      setText(const WSCvariant& text);
   long      setText(const WSCstring& text);
+#endif
   long      setText(long col, long row, long text);
   long      setText(long col, long row, double text);
   long      setText(long col, long row, const char* text);
+#ifndef SWIGOCAML
   long      setText(long col, long row, const WSCvariant& text);
   long      setText(long col, long row, const WSCstring& text);
+#endif
   WSCstring getText(long col, long row);
 
   long      setFixedText(long col, long row, long text);
   long      setFixedText(long col, long row, double text);
   long      setFixedText(long col, long row, const char* text);
+#ifndef SWIGOCAML
   long      setFixedText(long col, long row, const WSCvariant& text);
   long      setFixedText(long col, long row, const WSCstring& text);
+#endif
   WSCstring getFixedText(long col, long row);
 
   %apply long *OUTPUT { long *col, long *row };
@@ -1526,23 +1651,23 @@ public:
   void      setHorzAutoNumber(WSCbool val);
   WSCbool   getHorzAutoNumber();
 
-  long      setCellAlignemnt(char align);
-  long      setCellAlignemnt(long col, long row, char align);
-  char      getCellAlignemnt(long col, long row);
+  long      setCellAlignment(char align);
+  long      setCellAlignment(long col, long row, char align);
+  char      getCellAlignment(long col, long row);
 
-  long      setFixedCellAlignemnt(long col, long row, char align);
-  char      getFixedCellAlignemnt(long col, long row);
+  long      setFixedCellAlignment(long col, long row, char align);
+  char      getFixedCellAlignment(long col, long row);
 
 #ifdef SWIGPYTHON
   %extend {
-    long  setCellAlignemnt(short align) {
-      return self->setCellAlignemnt((char)align);
+    long  setCellAlignment(short align) {
+      return self->setCellAlignment((char)align);
     }
-    long  setCellAlignemnt(long col, long row, short align) {
-      return self->setCellAlignemnt(col, row, (char)align);
+    long  setCellAlignment(long col, long row, short align) {
+      return self->setCellAlignment(col, row, (char)align);
     }
-    long  setFixedCellAlignemnt(long col, long row, short align) {
-      return self->setFixedCellAlignemnt(col, row, (char)align);
+    long  setFixedCellAlignment(long col, long row, short align) {
+      return self->setFixedCellAlignment(col, row, (char)align);
     }
   }
 #endif
@@ -1569,44 +1694,60 @@ public:
 
   /// database 
   long      setDataSource(const char* name);
+#ifndef SWIGOCAML
   long      setDataSource(WSCstring& name);
+#endif
   WSCstring getDataSource();
 
   long      setTableName(const char* table);
+#ifndef SWIGOCAML
   long      setTableName(WSCstring& table);
+#endif
   WSCstring getTableName();
   
   long      setDbItem(const char* item);
+#ifndef SWIGOCAML
   long      setDbItem(WSCstring& item);
+#endif
   WSCstring getDbItem();
 
   long      setDbSortItem1(const char* item);
+#ifndef SWIGOCAML
   long      setDbSortItem1(WSCstring& item);
+#endif
   WSCstring getDbSortItem1();
 
   long      setDbSort1(WSCuchar sort);
   WSCuchar  getDbSort1();
 
   long      setDbSortItem2(const char* item);
+#ifndef SWIGOCAML
   long      setDbSortItem2(WSCstring& item);
+#endif
   WSCstring getDbSortItem2();
 
   long      setDbSort2(WSCuchar sort);
   WSCuchar  getDbSort2();
 
   long      setDbSortItem3(const char* item);
+#ifndef SWIGOCAML
   long      setDbSortItem3(WSCstring& item);
+#endif
   WSCstring getDbSortItem3();
 
   long      setDbSort3(WSCuchar sort);
   WSCuchar  getDbSort3();
   
   long      setDbFilter(const char* filter);
+#ifndef SWIGOCAML
   long      setDbFilter(WSCstring& filter);
+#endif
   WSCstring getDbFilter();
 
   long      setDbStmtString(const char* stmt);
+#ifndef SWIGOCAML
   long      setDbStmtString(WSCstring& stmt);
+#endif
   WSCstring getDbStmtString();
 
   long      dbRefresh();
@@ -1655,6 +1796,26 @@ class WSDmwindowDev :public WSDdev{
 static WSDmwindowDev* getNewInstance();
 };
 
+class WSCindexData{
+  public:
+    WSCindexData();
+    ~WSCindexData();
+    long   setData(char*,WSCptr);
+    char*  getIndex(long pos);
+    WSCptr   getData(char*);
+    WSCptr   getData(long pos);
+    long   getNum();
+#ifdef SWIGPYTHON
+    %extend {
+      long delData(char* val) { return self->del(val); };
+    }
+#else
+    long   del(char*);
+#endif
+    long   delPos(long);
+    void   clear();
+};
+
 //kind of mouse button state ,return value of WSDmouse::getMouseStatus()
 #define WS_MOUSE_SHIFT     (1<<0)
 #define WS_MOUSE_LOCK      (1<<1)
@@ -1672,22 +1833,22 @@ static WSDmwindowDev* getNewInstance();
 #define WS_RW        2
 
 //kind of indicator type
-#define WS_IN_OUT     (unsigned char)0
-#define WS_OUT        (unsigned char)1
-#define WS_IN         (unsigned char)2
-#define WS_NONE       (unsigned char)3
+#define WS_IN_OUT     0
+#define WS_OUT        1
+#define WS_IN         2
+#define WS_NONE       3
 
 //kind of orientation
-#define WS_VERTICAL   (unsigned char)1
-#define WS_HORIZONTAL (unsigned char)0
+#define WS_VERTICAL   1
+#define WS_HORIZONTAL 0
 
 //PIXMAP STYLE
-#define WS_DIRECT_WINDOW      (char)(0)
-#define WS_DYNAMIC_PIXMAP     (char)(1)
+#define WS_DIRECT_WINDOW      0
+#define WS_DYNAMIC_PIXMAP     1
 
 //SHADOW TYPE  //notice. exist in WSRdef
-#define WS_SHADOW_NONE     (signed char)(-1)
-#define WS_SHADOW_TRANS    (signed char)(-1)
+#define WS_SHADOW_NONE     (-1)
+#define WS_SHADOW_TRANS    (-1)
 #define WS_SHADOW_OUT      (0)
 #define WS_SHADOW_IN       (1)
 #define WS_SHADOW_EOUT     (2)
@@ -1695,13 +1856,13 @@ static WSDmwindowDev* getNewInstance();
 #define WS_SHADOW_BORDER   (4)
 
 //TITLE BAR  //notice. exist in WSRdef
-#define WS_NO_TITLE       (char)0
-#define WS_FULL_TITLE     (char)1
-#define WS_ONLY_TITLE     (char)2
-#define WS_MINI_BTN       (char)3
-#define WS_MAX_BTN        (char)4
-#define WS_FRAME          (char)5
-#define WS_NO_MANAGE      (char)6
+#define WS_NO_TITLE       0
+#define WS_FULL_TITLE     1
+#define WS_ONLY_TITLE     2
+#define WS_MINI_BTN       3
+#define WS_MAX_BTN        4
+#define WS_FRAME          5
+#define WS_NO_MANAGE      6
 //do not change...
 #define WS_LEFT           0
 #define WS_RIGHT          1
@@ -2108,21 +2269,21 @@ static WSDmwindowDev* getNewInstance();
 #define WSK_Page_Up		0x1246
 #define WSK_Page_Down		0x1247
 #define WSK_End		0x125e
-//#define WSK_Begin		WSK_Begin
+#define WSK_Begin		-1	// dummy
 
-//#define WSK_Select		WSK_Select
+#define WSK_Select		-1	// dummy
 #define WSK_Print		0x1148
 #define WSK_Execute		-1
 #define WSK_Insert		-1
-//#define WSK_Undo		WSK_Undo
-//#define WSK_Redo		WSK_Redo
-//#define WSK_Menu		WSK_Menu
-//#define WSK_Find		WSK_Find
+#define WSK_Undo		-1	// dummy
+#define WSK_Redo		-1	// dummy
+#define WSK_Menu		-1	// dummy
+#define WSK_Find		-1	// dummy
 #define WSK_Cancel		0x0018
 #define WSK_Help		-1
 #define WSK_Break		-1
-//#define WSK_Mode_switch		WSK_Mode_switch
-//#define WSK_script_switch		WSK_script_switch
+#define WSK_Mode_switch		-1	// dummy
+#define WSK_script_switch	-1	// dummy
 #define WSK_Num_Lock		-1
 
 #define WSK_KP_Space		-1
@@ -2308,6 +2469,8 @@ static WSDmwindowDev* getNewInstance();
 #define WSK_underscore		0x2132
 //#define WSK_grave		TK_RQUT
 //#define WSK_quoteleft		TK_RQUT
+#define WSK_grave		-1	// dummy
+#define WSK_quoteleft		-1	// dummy
 #define WSK_a		(0x2361 +0)
 #define WSK_b		(0x2361 +1)
 #define WSK_c		(0x2361 +2)
@@ -2382,14 +2545,21 @@ static WSDmwindowDev* getNewInstance();
 #define WSK_Page_Up				0x21
 #define WSK_Page_Down			        0x22
 #define WSK_End					0x23
+#define WSK_Begin                               -1	// dummy
 
-
+#define WSK_Select				-1	// dummy
 #define WSK_Print				0x2C
 #define WSK_Execute				0x2B
 #define WSK_Insert				0x2D
+#define WSK_Undo				-1	// dummy
+#define WSK_Redo				-1	// dummy
+#define WSK_Menu				-1	// dummy
+#define WSK_Find		                -1	// dummy
 #define WSK_Cancel				0x03
 #define WSK_Help				0x2F
 #define WSK_Break				0x03
+#define WSK_Mode_switch				-1	// dummy
+#define WSK_script_switch			-1	// dummy
 #define WSK_Num_Lock			        0x90
 
 #define WSK_KP_Space				-1
@@ -2617,4 +2787,135 @@ static WSDmwindowDev* getNewInstance();
 #define WS_LIST_LABEL_LIST "list-lb-list"
 #define WS_LIST_TOP "list-lb-top"
 
+
+#ifndef NO_MWT_FUNCTION
+
+#define WS_ROOT_INSTANCE_ID   0
+#define WS_ROOT_PROPERTY_ID   0
+#define WS_ROOT_PROCEDURE_ID  0
+
+#define WS_NEXT_CHILD_INSTANCE_ID       1
+#define WS_PARENT_INSTANCE_ID           2
+
+#define WS_SET_ROOT_INSTANCE            3
+#define WS_REMOVE_ROOT_INSTANCE         4
+#define WS_SET_SELECTED_INSTANCE        5
+#define WS_REMOVE_SELECTED_INSTANCE     6
+#define WS_NEXT_SELECTED_INSTANCE_ID    7
+
+#define WS_INSTANCE_NAME                10
+#define WS_PROPERTY_NAME                11
+#define WS_PROPERTY_VALUE               12
+#define WS_PROPERTY_TYPE                13
+#define WS_PROPERTY_ATTRIBUTE           14
+#define WS_PROCEDURE_FUNCTION_NAME      15
+#define WS_PROCEDURE_NAME                1
+#define WS_OBJECT_NAME               2
+#define WS_OBJECT_ARRAY_NAME         3
+#define WS_OBJECT_ARRAY_NAME_INVALID 4
+#define WS_OTHER_NAME                5
+
+#define WS_PROPERTY_SELECT_VALUE_NAME   16
+#define WS_PROPERTY_SELECT_VALUE        17
+#define WS_PROPERTY_TITLE               18
+
+#define WS_PROPERTY_TYPE_COLOR          1
+#define WS_PROPERTY_TYPE_PIXMAP         2
+#define WS_PROPERTY_TYPE_FILE_SELECT    4
+
+
+#define WS_NEXT_PROPERTY_ID             20
+#define WS_NEXT_PROCEDURE_ID            21
+#define WS_NEXT_PROPERTY_SELECT_ITEM_ID 22
+
+#define WS_NEXT_TARGET_ID               30
+#define WS_TARGET_NATIVE                 0
+#define WS_TARGET_NAME                  31
+#define WS_TARGET_SUFFIX                32
+#define WS_ITEM_ROOT                     0
+
+#define WS_NEXT_LANG_ID                 33
+#define WS_LANG_STR                     34
+
+#define WS_NEXT_ENCODING_ID             35
+#define WS_ENCODING_NAME                36
+
+#define WS_DEFAULT_PROJECT_FILE_NAME    40
+#define WS_LINK_OPT_DEF_FILE_NAME       41
+#define WS_LINK_OPTIONS                 42
+#define WS_STLINK_OPTIONS               43
+#define WS_NEXT_LINK_OPT_ID             44
+#define WS_LINK_OPT_NAME                45
+#define WS_TARGET_INSTANCE_ID           46
+#define WS_LINK_LIB_DEF_FILE_NAME       47
+#define WS_LINK_LIB_NAME                48
+#define WS_LINK_CLASS_DEF_FILE_NAME     49
+#define WS_TARGET_DEF_FILE_NAME         50
+#define WS_ENV_DEF_FILE_NAME            51
+#define WS_LIBNAME_DEF_FILE_NAME        52
+#define WS_TARGET_ENV_DEF_FILE_NAME     53
+#define WS_TARGET_ENV_TITLE             54
+#define WS_TARGET_ENV_NAME              55
+
+
+
+extern long WSGFloadProject(char* dname,char* fname,long encoding);
+extern long WSGFloadProject(char* path,long encoding);
+extern long WSGFreloadProject(WSCulong project,char* dname,char* fname,long encoding);
+extern long WSGFreloadProject(WSCulong project,char* path,long encoding);
+extern long WSGFdestroyProject(WSCulong project);
+extern long WSGFdumpProject(WSCulong project);
+extern long WSGFsaveProject(WSCulong project,char* dname,char* fname,long encoding);
+extern long WSGFsetProjectData(WSCulong prj,char* index,char* val,long encoding);
+extern WSCstring WSGFgetProjectData(WSCulong prj,char* index);
+extern WSCstring WSGFgetProjectDefaultData(WSCulong prj,char* index);
+extern WSCstring WSGFgetProjectEnvironmentData(char* index);
+extern long WSGFsetProjectEnvironmentData(char* index,char* val,long encoding);
+extern long WSGFsaveProjectEnvironment();
+extern long WSGFloadLinkOptions(WSCulong prj);
+
+extern long WSGFloadWinFiles(char* dname,char* fname,WSCulong prj);
+extern long WSGFloadProjectAndWinFiles(char* dname,char* fname,long encoding);
+extern long WSGFsaveWinFile(char* dname,WSCbase* win,long encoding);
+
+extern long WSGFloadTargets(char* path,long encoding);
+extern long WSGFgetMgrData(long kind,WSCulong val);
+extern WSCstring WSGFgetMgrStrData(long kind,WSCulong val);
+extern WSCstring WSGFgetMgrStrStrData(long kind,char* val);
+extern long WSGFsetMgrStrData(long kind,char* val,long encoding);
+extern WSCprocedure* WSGFidToProcedure(WSCulong id);
+extern WSCulong WSGFprocedureToId(WSCprocedure*);
+
+extern long WSGFcreateProjectSrcFile(char* dname,char* fname,WSCulong proj,long encoding);
+extern long WSGFcreateMakefiles(char* dname,char* prj_name,WSCulong proj,long encoding);
+extern long WSGFcreateWindowSrcFiles(char* dname,WSCulong proj,long encoding);
+extern void WSGFsetDefaultVePopupMenus();
+extern void WSGFsetDefaultVePropertyEditProcedures();
+extern WSCbool  WSGFcheckNameString(char*);
+extern WSCbool       WSGFisArrayName(char*);
+extern long          WSGFgetArrayNo(char*);
+extern WSCstring     WSGFgetArrayName(char*);
+extern long WSGFexistName(char*,WSCbase*);
+extern WSCbool WSGFcheckExistFile(char*);
+extern void WSGFsetupInstance(WSCbase*);
+extern void WSGFloadLibrary(long,char* libname);
+extern void WSGFsetFontData(WSCstring* data);
+extern long WSGFgetEncodingValue(char*);
+
+#define WS_LANG_CPP    0
+#define WS_LANG_RUBY   1
+#define WS_LANG_PYTHON 2
+#define WS_LANG_PERL   3
+#define WS_LANG_JAVA   4
+#define WS_LANG_OCAML  5
+
+extern long WSGFcreateProcedure(char* dname,char* fname,long,char* fcname,char* cname,long encode);
+
+extern long WSGFpasteInstance(WSCbase* target,WSCbase* src);
+#define WS_ERR_NOT_MANAGER -3
+#define WS_ERR_OVERWRAPED  -4
+
+
+#endif
+
 #endif //#ifndef mpfc_H
