@@ -14,6 +14,8 @@ have_library("wspostgres","libwspostgres_func")
 have_library("wsmysql","libwsmysql_func")
 have_library("wsodbc","libwsodbc_func")
 have_library("wssheet","libwssheet_func")
+have_library("wse","libwse_func")
+have_library("wsad","libwsad_func")
 
 create_makefile("mpfc")
 
