@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team 
-// shall not be used in advertising or otherwise to promote the sale, use or 
-// other dealings in this Software without prior written authorization from  
-// WideStudio Development Team.
-
+//
 
 #include <WScom.h>
 #include <WSCvlabel.h>
@@ -97,6 +93,13 @@ void _lang_init_proc(){
 #ifndef NO_ES_MSG
 #ifndef _WSWIN32
 #include "Espanol.cpp"
+#include "Espanol-ar.cpp"
+#endif
+#endif
+#ifndef NO_ES_UTF8_MSG
+#ifndef _WSWIN32
+#include "Espanol-utf8.cpp"
+#include "Espanol-ar-utf8.cpp"
 #endif
 #endif
 
@@ -113,6 +116,7 @@ static _lang_init_ _run_lang_init;
 
 
 WSMFguiClassInitialize(WSCsheet, WSCform);
+WSMFversion(WSCsheet,WSCform);
 
 WSCsheet::WSCsheet(WSCbase* base, char* objname) : WSCform(base, objname)
 {
@@ -149,6 +153,7 @@ WSCsheet::WSCsheet(WSCbase* base, char* objname) : WSCform(base, objname)
 	_hautoNumber = True;
 	_initVisible = True;
 	_adjust = False;
+	_auto_refresh = True;
 	
 	_vdb = NULL;
 	_dataSource = WSGFstrdup("");
@@ -208,10 +213,13 @@ WSCsheet::WSCsheet(WSCbase* base, char* objname) : WSCform(base, objname)
     WSMFpropertySetSelection(WSRbool1, WSRbool1D);
     WSMFpropertyCreate(WSNdbCellTitle, WSCbool, _dbCellTitle, WSSdbCellTitle);
     WSMFpropertySetSelection(WSRbool1, WSRbool1D);
+    WSMFpropertyCreate(WSNrefreshing, WSCbool, _auto_refresh, WSSrefreshing);
+    WSMFpropertySetSelection(WSRbool1, WSRbool1D);
 
     WSMFaddExtTrigger(WSEV_SCR_MOUSE_PRESS,"SCR-MOUSE-PRESS");
     WSMFaddExtTrigger(WSEV_SCR_MOUSE_RELEASE,"SCR-MOUSE-RELEASE");
     WSMFaddExtTrigger(WSEV_SCR_MOUSE_MOVE,"SCR-MOUSE-MOVE");
+    WSMFaddExtTrigger(WSEV_SCR_MOUSE_DOUBLE_CLICK,"SCR-MOUSE-DCLICK"  );
 
     WSMFaddTrigger(WSEV_ACTIVATE);
     WSMFaddTrigger(WSEV_KEY_PRESS);
@@ -255,6 +263,7 @@ WSMFproperty(WSCsheet, WSNfilter, char*, _dbFilter, WSGFstrdup(""));
 WSMFproperty(WSCsheet, WSNsql, char*, _dbStmtString, WSGFstrdup("")); 
 WSMFproperty(WSCsheet, WSNdbCellAutoResize, WSCbool, _dbCellAutoResize, True);
 WSMFproperty(WSCsheet, WSNdbCellTitle, WSCbool, _dbCellTitle, True);
+WSMFproperty(WSCsheet, WSNrefreshing, WSCbool, _auto_refresh, True);
 
 void WSCsheet::setWorkWSNrows(long val)
 {
@@ -416,10 +425,21 @@ void WSCsheet::setWorkWSNdbCellAutoResize(WSCbool val){}
 void WSCsheet::getWorkWSNdbCellAutoResize(WSCbool*){}
 void WSCsheet::setWorkWSNdbCellTitle(WSCbool val){}
 void WSCsheet::getWorkWSNdbCellTitle(WSCbool*){}
+void WSCsheet::setWorkWSNrefreshing(WSCbool val){
+  if (val == False){
+	_initVisible = False;
+  }
+}
+void WSCsheet::getWorkWSNrefreshing(WSCbool*){}
 
 
 WSCsheet::~WSCsheet()
 {
+	WSCbase* fclient = WSGIappMouse()->getMouseFocusClient();
+	if (fclient == this){
+		WSGIappMouse()->setMouseFocusClient(NULL);
+	}
+
 	setRowCount(0);
 
 	if(_dataSource) {
@@ -463,11 +483,7 @@ long WSCsheet::initialize()
 	addstr.setString("-sheet");
 	scrname.addString(addstr);
 
-	WSCbase* htmp1 = WSCbase::getNewInstance(getUseScrBarClass(), this, scrname.getString());
-	_sheet = (WSCform*)htmp1->cast("WSCform");
-	if(_sheet == NULL) {
-		_sheet = new WSCform(this, scrname.getString());
-	}
+	_sheet = new WSCform(this, scrname.getString());
 	_sheet->setInternalObject(True);
 	_sheet->setUseParentBaseDev(True);
 	_sheet->initialize();
@@ -495,6 +511,13 @@ long WSCsheet::initialize()
 	op->setFunction(onSheetMouseRelease, "onSheetMouseRelease");
 	_sheet->addProcedure(op);
 
+	op = new WSCprocedure("sheet-mousedclick", WSEV_MOUSE_DOUBLE_CLICK);
+	op->setInternal(True);
+	op->setFunction(onSheetMouseDoubleClick, "onSheetMouseDoubleClick");
+	_sheet->addProcedure(op);
+
+
+
 	_sheet->setVisible(True);
 
 	scrname.setString(getInstanceName());
@@ -512,7 +535,7 @@ long WSCsheet::initialize()
 
 	WSCprocedure* op1 = new WSCprocedure("value-ch", WSEV_VALUE_CH);
 	op1->setInternal(True);
-	op1->setFunction(onHscroolClick, "onHscroolClick");
+	op1->setFunction(onHscrollClick, "onHscrollClick");
 	_hscrBar->addProcedure(op1);
 	_hscrBar->setPropertyV(WSNincrement, (WSCushort)1);
 	_hscrBar->setVisible(True);
@@ -665,6 +688,8 @@ long WSCsheet::draw()
 	if(err != WS_NO_ERR) {
 		return WS_NO_ERR;
 	}
+	WSCbase::update();
+	setAbsoluteDraw(False);
 	if(shadow_thick > 0){
 		if(WSGIappDev()->getGuiPolicy() & WS_POLICY_WINDOWS) {
 			WSGFdrawShadow(dev, _shadow_type, shadow_thick, _fore_color, 
@@ -673,7 +698,8 @@ long WSCsheet::draw()
 	}
 	dev->endDraw();
 
-	_sheet->execProcedure(WSEV_EXPOSE);
+//	_sheet->execProcedure(WSEV_EXPOSE);
+	onSheetExpose(_sheet);
 
 	return WS_NO_ERR;
 }
@@ -687,7 +713,21 @@ long WSCsheet::cdraw()
 }
 void WSCsheet::update()
 {
-  WSCform::update();
+	if(getVisible() != False && _initVisible) {
+		if(_dataSource != NULL && _dataSource[0] != 0 &&
+            !getPropertyEditMode()) {
+			setNoRefresh(True);
+
+			long ret = dbDataSet();
+			if (ret == WS_NO_ERR){
+				_initVisible = False;
+			}
+			setNoRefresh(False);
+		}else{
+			_initVisible = False;
+		}
+	}
+	WSCform::update();
 }
 void WSCsheet::onResize(WSCrect* rect)
 {
@@ -728,6 +768,30 @@ void WSCsheet::onMouseMove(WSCpoint* pt)
 }
 void WSCsheet::onMousePress(WSCpoint* pt)
 {
+	if (WSGIappMouse()->getTargetBtn() == WS_MOUSE_BTN4){
+		if (getVisible() == False){
+			return;
+		}
+		int val = _vscrBar->getProperty(WSNvalue);
+		int inc = _vscrBar->getProperty(WSNincrement);
+		val -= inc;
+		if (val < 0){
+			val = 0;
+		}
+		_vscrBar->setProperty(WSNvalue, val );
+		return;
+	}else
+	if (WSGIappMouse()->getTargetBtn() == WS_MOUSE_BTN5){
+		if (getVisible() == False){
+			return;
+		}
+		int val = _vscrBar->getProperty(WSNvalue);
+		int inc = _vscrBar->getProperty(WSNincrement);
+		val += inc;
+		_vscrBar->setProperty(WSNvalue, val );
+		return;
+	}
+
 	long row, col, oldRow = -1, oldCol = -1;
 
 	if(getPropertyEditMode()) {
@@ -770,6 +834,7 @@ void WSCsheet::onMousePress(WSCpoint* pt)
 		draw();
 	}
 	
+	WSGIappMouse()->setMouseFocusClient(this);
 	WSCform::onMousePress(pt);
 }
 void WSCsheet::onMouseRelease(WSCpoint* pt)
@@ -791,22 +856,48 @@ void WSCsheet::onMouseRelease(WSCpoint* pt)
 void WSCsheet::onVisibleChange(WSCbool vis)
 {
 	if(vis && _initVisible) {
-		_initVisible = False;
 
-		if(!getPropertyEditMode()) {
+		if(_dataSource != NULL && _dataSource[0] != 0 &&
+            !getPropertyEditMode()) {
 			setNoRefresh(True);
 
-			dbDataSet();
-
+			long ret = dbDataSet();
+			if (ret == WS_NO_ERR){
+				_initVisible = False;
+			}
 			setNoRefresh(False);
-		}
+		}else{
+			_initVisible = False;
+        }
 	}
 	WSCform::onVisibleChange(vis);
 }
+
+void WSCsheet::onParentVisibleChange(WSCbool vis)
+{
+	if(vis && _initVisible) {
+
+		if(_dataSource != NULL && _dataSource[0] != 0 &&
+            !getPropertyEditMode()) {
+			setNoRefresh(True);
+
+			long ret = dbDataSet();
+			if (ret == WS_NO_ERR){
+				_initVisible = False;
+			}
+			setNoRefresh(False);
+		}else{
+			_initVisible = False;
+        }
+	}
+	WSCform::onParentVisibleChange(vis);
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // local event
 //
-void WSCsheet::onHscroolClick(WSCbase* obj)
+void WSCsheet::onHscrollClick(WSCbase* obj)
 {
 	WSCulong val;
 	WSCbase* parent = obj->getParent();
@@ -975,6 +1066,20 @@ void WSCsheet::onSheetMouseRelease(WSCbase* obj)
 		ss->onScrMouseRelease(&pt);
 	}
 }
+void WSCsheet::onSheetMouseDoubleClick(WSCbase* obj)
+{
+	WSCbase* parent = obj->getParent();
+	WSCsheet* ss = (WSCsheet*)parent->cast("WSCsheet");
+
+	WSCpoint pt;
+	short    x, y;
+	if(WSGIappMouse()->getMousePosition(&x, &y, obj)) {
+		pt.x = (long)x;
+		pt.y = (long)y;
+		ss->onMouseDoubleClick(&pt);
+		ss->onScrMouseDoubleClick(&pt);
+	}
+}
 void WSCsheet::onScrMouseMove(WSCpoint* pt)
 {
 	execProcedure(WSEV_SCR_MOUSE_MOVE);
@@ -987,6 +1092,10 @@ void WSCsheet::onScrMouseRelease(WSCpoint* pt)
 {
 	execProcedure(WSEV_SCR_MOUSE_RELEASE);
 }
+void WSCsheet::onScrMouseDoubleClick(WSCpoint* pt)
+{
+	execProcedure(WSEV_SCR_MOUSE_DOUBLE_CLICK);
+}
 void WSCsheet::onEditActivate(WSCbase* obj)
 {
 	WSCbase* parent = obj->getParent();
@@ -1911,7 +2020,7 @@ long WSCsheet::setFixedCellFont(short fontno)
 
 	return WS_NO_ERR;
 }
-long WSCsheet::setCellAlignemnt(char align)
+long WSCsheet::setCellAlignment(char align)
 {
 	WSCsheetCell* cell;
 	
@@ -1945,7 +2054,7 @@ long WSCsheet::setCellAlignemnt(char align)
 
 	return WS_NO_ERR;
 }
-long WSCsheet::setCellAlignemnt(long col, long row, char align)
+long WSCsheet::setCellAlignment(long col, long row, char align)
 {
 	WSCsheetCell* cell = getCell(col, row, True);
 	if(!cell) {
@@ -1957,7 +2066,7 @@ long WSCsheet::setCellAlignemnt(long col, long row, char align)
 
 	return WS_NO_ERR;
 }
-char WSCsheet::getCellAlignemnt(long col, long row)
+char WSCsheet::getCellAlignment(long col, long row)
 {
 	WSCsheetCell* cell = getCell(col, row);
 	if(cell) {
@@ -1965,7 +2074,7 @@ char WSCsheet::getCellAlignemnt(long col, long row)
 	}
 	return WS_LEFT;
 }
-long WSCsheet::setFixedCellAlignemnt(long col, long row, char align)
+long WSCsheet::setFixedCellAlignment(long col, long row, char align)
 {
 	if(col < 0 || row < 0) {
 		return WS_ERR;
@@ -1979,7 +2088,7 @@ long WSCsheet::setFixedCellAlignemnt(long col, long row, char align)
 	}
 	return WS_NO_ERR;
 }
-char WSCsheet::getFixedCellAlignemnt(long col, long row)
+char WSCsheet::getFixedCellAlignment(long col, long row)
 {
 	if(col < 0 || row < 0) {
 		return WS_ERR;
@@ -2975,11 +3084,13 @@ long WSCsheet::dbConnect()
 	if(_vdb) {
 		return WS_NO_ERR;
 	}
-	WSCbase* object = WSGIappObjectList()->getInstance("WSCbase", getDataSource());
+    WSCstring str = getDataSource();
+	WSCbase* object = WSGIappObjectList()->getInstance("WSCbase", str);
 	if(object) {
 		_vdb = (WSCvdb*)object->cast("WSCvdb");
 	} else {
 		_vdb = NULL;
+		return WS_ERR;
 	}
 	return WS_NO_ERR;
 }
@@ -3002,7 +3113,7 @@ long WSCsheet::setFixedCelldbItem(WSCdbRecord* rs)
 			WSCstring* word;
 			word = new WSCstring(str.getWord(col, ","));
 			setFixedCellText(col + fixedCols, 0, (char*)word->getString());
-			setFixedCellAlignemnt(col + fixedCols, 0, WS_CENTER);
+			setFixedCellAlignment(col + fixedCols, 0, WS_CENTER);
 			
 			width = getColumnWidth(col + fixedCols);
 			if(width < (font->getStringWidth(word) + 4)) {
@@ -3016,7 +3127,7 @@ long WSCsheet::setFixedCelldbItem(WSCdbRecord* rs)
 		for(col = 0; col < cols; col++){
 			rs->getColName(col, name);
 			setFixedCellText(col + fixedCols, 0, name);
-			setFixedCellAlignemnt(col + fixedCols, 0, WS_CENTER);
+			setFixedCellAlignment(col + fixedCols, 0, WS_CENTER);
 			
 			word = name;
 			width = getColumnWidth(col + fixedCols);
@@ -3035,11 +3146,12 @@ long WSCsheet::dbDataSet()
 	dbConnect();
 
 	if(!_vdb) {
-		return WS_NO_ERR;
-	}
-	if(*((char*)_vdb->getProperty(WSNhostname)) == '\0') {
-		return WS_NO_ERR;
+		return WS_ERR;
 	}
+//In same case, we often use empty hostname property..
+//	if(*((char*)_vdb->getProperty(WSNhostname)) == '\0') {
+//		return WS_NO_ERR;
+//	}
 
 	WSDfont* font = WSGIappFontSet()->getFont(_fixedFont);
 
@@ -3119,8 +3231,9 @@ long WSCsheet::dbDataSet()
 			char* val = new char[32767*2];
 			long col = 0;
 			long row = 1;
-			long width;
+			long width = 0;
 			long fixedCols = getFixedColumnCount();
+			long fixedRows = getFixedRowCount();
 			
 			while (!rs.isEOF()) {
 				for(col = 0; col < rs.getColCount(); col++) {
@@ -3129,12 +3242,29 @@ long WSCsheet::dbDataSet()
 						setRowCount(row+1);
 					}
 					setCellText(col + fixedCols, row, val);
-
 					if(_dbCellAutoResize) {
 						str = val;
 						width = getColumnWidth(col + fixedCols);
-						if(width < (font->getStringWidth(&str)+4)) {
-							setColumnWidth(col + fixedCols, font->getStringWidth(&str)+4);
+						if (row == 1){
+							long dwidth = _colWidth;
+							int frow;
+							for(frow = 0; frow < fixedRows; frow++){
+								WSCstring fstr = getFixedCellText(col+fixedCols,frow);
+								long fwidth = font->getStringWidth(&fstr)+4;
+								if (fwidth > dwidth){
+									dwidth = fwidth;
+								}
+							}
+
+
+                            if (width > dwidth){
+								setColumnWidth(col + fixedCols, dwidth);
+							}
+							width = dwidth;
+						}
+						long twidth = font->getStringWidth(&str)+4;
+						if(width < twidth) {
+							setColumnWidth(col + fixedCols, twidth);
 						}
 					}
 				}
@@ -3207,3 +3337,12 @@ WSCstring WSCsheet::getDbErrorMsg()
 {
 	return _dbError;
 }
+WSCbase* WSCsheet::getScrFrame(){
+	return _sheet;
+}
+WSCbase* WSCsheet::getVScrBar(){
+    return _vscrBar;
+}
+WSCbase* WSCsheet::getHScrBar(){
+    return _hscrBar;
+}
