@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,17 +14,15 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+//
 
 #ifdef _WSWINCE
 #include <winsock.h>
+#include <sipapi.h>
 #endif
 #include <win/WSDwinAppDev.h>
 #include <WSCbaseList.h>
@@ -32,6 +30,7 @@
 #include <WSCbase.h>
 #include <WSCdevice.h>
 #include <win/WSDwinformDev.h>
+#include <win/WSDwinwinDev.h>
 #include <WSCcolorSet.h>
 #include <win/WSDwinPrivateTimer.h>
 #include <win/WSDwinExecute.h>
@@ -45,6 +44,8 @@
 #include <win/WSDwinThread.h>
 #include <win/WSDwinMutex.h>
 #include <win/WSDwinSemaphore.h>
+#include <WSDmouse.h>
+
 #define COLOR_MAX 256
 WSMFclassInit(WSDwinAppDev,WSDappDev);
 WSDappDev* _winAppDev_create_handler(){
@@ -83,6 +84,7 @@ extern WSDcodeConvert* _code_convert_create_handler_();
     WSGIappCodeConvert()->initialize();
     initialized = True;
     WSGIappInitializeManager()->execInitializeProcs();
+#ifndef NOT_LOAD_DEFAULT_ICONS
     WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/psave.ico",(void*)202);
     WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/pend.ico",(void*)203);
     WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/wload.ico",(void*)204);
@@ -118,12 +120,42 @@ extern WSDcodeConvert* _code_convert_create_handler_();
     WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi41.ico",(void*)233);
     WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi42.ico",(void*)234);
     WSGIappImageSet()->setDefaultImageHandle("$(WSDIR)/sys/pixmaps/bi43.ico",(void*)235);
+#endif
+extern void WSGFinitFS();
+    WSGFinitFS();
+extern void WSGFwinEnvInit();
+    WSGFwinEnvInit();
+extern void WSGFwinExecuteInit();
+    WSGFwinExecuteInit();
+extern void WSGFwinExternalInit();
+    WSGFwinExternalInit();
+extern void WSGFinitializeWinFile();
+    WSGFinitializeWinFile();
+
 extern void WSGFinitializeWinMutex();
     WSGFinitializeWinMutex();
-extern void WSGFinitializeWinThread();
-    WSGFinitializeWinThread();
 extern void WSGFinitializeWinSemaphore();
     WSGFinitializeWinSemaphore();
+extern void WSGFinitializeWinThread();
+    WSGFinitializeWinThread();
+
+extern WSDmouse* _winMouse_create_handler();
+    WSDmouse::setCreateInstanceHandler(_winMouse_create_handler);
+extern WSDdev* _wscdwinformdev_init_();
+    WSGIappDevice()->setCreateHandler("formDev",_wscdwinformdev_init_);
+extern WSDdev* _wsdwinwindev_init_();
+    WSGIappDevice()->setCreateHandler("windowDev",_wsdwinwindev_init_);
+extern WSDdev* _wsdwinnwdev_init_();
+    WSGIappDevice()->setCreateHandler("nwDev",_wsdwinnwdev_init_);
+extern WSDdev* _wsdxscrdev_init_();
+    WSGIappDevice()->setCreateHandler("scrFrameDev",_wsdxscrdev_init_);
+extern WSDdev* _wscdwinmwindev_init_();
+    WSGIappDevice()->setCreateHandler("mwindowDev",_wscdwinmwindev_init_);
+extern WSDprivateTimer* _winPrivatetimer_create();
+    WSDprivateTimer::setCreateInstanceHandler((void*)_winPrivatetimer_create);
+extern WSDtimer* _wintimer_create_handler();
+    WSDtimer::setCreateInstanceHandler(_wintimer_create_handler);
+ 
 extern void _win_font_init();
     _win_font_init();
 extern void _win_color_init();
@@ -134,11 +166,19 @@ WSMFtrace("WSGFdeviceInitialize done:\n");
   }
 }
 WSDwinAppDev::WSDwinAppDev(){
-dbprintf("WSDwinAppDev::WSDwinAppDev %s:%d start this=0x%x \n",__FILE__,__LINE__,this);
+//dbprintf("WSDwinAppDev::WSDwinAppDev %s:%d start this=0x%x \n",__FILE__,__LINE__,this);
   _hinst = 0;
   _mfocus_dev = NULL;
   _palette = 0;
   _wnd_proc = NULL;
+  _tid = GetCurrentThreadId();
+  _locked = False;
+  _mutex = NULL;
+  _allowed = False;
+  _show_cursor = True;
+  _grabed = False;
+  _global_cursor = False;
+  _grabed_wnd = 0;
 }
 WSDwinAppDev::~WSDwinAppDev(){}
 void WSDwinAppDev::setMouseFocusDev(WSDwinformDev* mdev){
@@ -148,16 +188,40 @@ WSDwinformDev* WSDwinAppDev::getMouseFocusDev(){
   return _mfocus_dev;
 }
 WSCushort WSDwinAppDev::getWidth(){
-  HWND dw = GetDesktopWindow();
+//  HWND dw = GetDesktopWindow();
+//  RECT r;
+//  GetClientRect(dw,&r);
+//  return (WSCushort)r.right; //DEVELOP...
+//  WSCushort ret = (WSCushort)GetSystemMetrics(SM_CXVIRTUALSCREEN);
+//  if (ret == 0){
+//    ret = (WSCushort)GetSystemMetrics(SM_CXSCREEN);
+//  }
   RECT r;
-  GetClientRect(dw,&r);
-  return (WSCushort)r.right; //DEVELOP...
+  SystemParametersInfo(SPI_GETWORKAREA,NULL,&r,NULL);
+  return r.right - r.left;
+}
+WSCushort WSDwinAppDev::getXOffset(){
+  RECT r;
+  SystemParametersInfo(SPI_GETWORKAREA,NULL,&r,NULL);
+  return r.left;
 }
 WSCushort WSDwinAppDev::getHeight(){
-  HWND dw = GetDesktopWindow();
+//  HWND dw = GetDesktopWindow();
+//  RECT r;
+//  GetClientRect(dw,&r);
+//  return (WSCushort)r.bottom; //DEVELOP...
+//  WSCushort ret = (WSCushort)GetSystemMetrics(SM_CYVIRTUALSCREEN);
+//  if (ret == 0){
+//    ret = (WSCushort)GetSystemMetrics(SM_CYSCREEN);
+//  }
   RECT r;
-  GetClientRect(dw,&r);
-  return (WSCushort)r.bottom; //DEVELOP...
+  SystemParametersInfo(SPI_GETWORKAREA,NULL,&r,NULL);
+  return r.bottom - r.top;
+}
+WSCushort WSDwinAppDev::getYOffset(){
+  RECT r;
+  SystemParametersInfo(SPI_GETWORKAREA,NULL,&r,NULL);
+  return r.top;
 }
 
 long WSDwinAppDev::getDeviceResource(){
@@ -335,6 +399,9 @@ long WSDwinAppDev::handleEvents(){
     if (ret == 0){
       break;
     }
+    if (_exit_event_loop != False){
+      break;
+    }
   }
   return _msg.wParam;
 }
@@ -344,9 +411,54 @@ long WSDwinAppDev::dispatchEvent(){
 long WSDwinAppDev::processEvent(){
  return _dispatchEvent(False);
 }
+#ifndef _WSWINCE
+WSClistData _pending_events;
+void _add_pending_event(WinEvent* ev){
+  if (ev->message == WM_PAINT){
+    int num = _pending_events.getNum();
+    int i;
+    WinEvent** buf =(WinEvent**)_pending_events._data;
+    for(i=0; i<num; i++){
+      WinEvent* item = buf[i];
+      if (ev->hWnd == item->hWnd &&
+          ev->uParam == item->uParam &&
+          ev->lParam == item->lParam){
+        Sleep(10);
+        return;
+      }
+    }
+  }
+  WinEvent* pev = new WinEvent;
+  pev->message = ev->message;
+  pev->uParam = ev->uParam;
+  pev->lParam = ev->lParam;
+  pev->hWnd = ev->hWnd;
+  _pending_events.add((void*)pev);
+}
+extern LRESULT CALLBACK WndProc(HWND hWnd,UINT message,WPARAM uParam,LPARAM lParam);
+void _execute_pending_events(){
+  if (_pending_events.getNum() > 0){
+    WSClistData evlist;
+    evlist = _pending_events;
+    _pending_events.clear();
+    int i;
+    int num = evlist.getNum();
+    WinEvent** buf = (WinEvent**)evlist._data;
+    for(i=0; i<num; i++){
+      WinEvent* ev = buf[i];
+      WndProc(ev->hWnd,ev->message,ev->uParam,ev->lParam);
+      delete ev;
+    }
+  }
+}
+#endif
 long WSDwinAppDev::_dispatchEvent(WSCbool wait){
 //  HANDLE hAccelTable;
   if (wait != False){
+    if (_locked != False && _tid != GetCurrentThreadId()){
+//printf("WSDwinAppDev::_dispatchEvent sub_tid..\n");fflush(stdout);
+      return SendMessage(getHWND(),WM_USER,(WPARAM)WS_WIN_ALLOW_EXEC_EVT,(LPARAM)0);
+    }
     long ret = GetMessage(&_msg,NULL,0,0);
     TranslateMessage(&_msg);
     DispatchMessage(&_msg);
@@ -367,38 +479,148 @@ void WSDwinAppDev::setInstanceHandle(HINSTANCE h){
 HINSTANCE WSDwinAppDev::getInstanceHandle(){
   return _hinst;
 }
+DWORD _activate_ev_start_handler(void* ptr){
+  WinEvent* ev = (WinEvent*)ptr;
+  HRESULT ret1 = DefWindowProc(ev->hWnd,ev->message,ev->uParam,ev->lParam);
+  delete ev;
+  ExitThread((long unsigned int)0);
+  return 0;
+}
 LRESULT CALLBACK WndProc(HWND hWnd,UINT message,WPARAM uParam,LPARAM lParam){
-//  int wmld, wmEvent;
-//  PAINTSTRUCT ps;
-//  RECT rect;
-//  RECT winsize;
-//  char buffer[32];
-dbprintf("WndProc %s:%d start hwnd=0x%x message=0x%x message=%d \n",__FILE__,__LINE__,hWnd,message,message);
+dbprintf("WndProc %s:%d start hwnd=0x%x message=0x%x message=%d uParam=0x%x lParam=0x%x\n",__FILE__,__LINE__,hWnd,message,message,uParam,lParam);
+
+#ifndef _WSWINCE
+  if (message == WM_IME_NOTIFY && uParam == IMN_SETOPENSTATUS){
+    WSGIwinKeyboard()->setHwnd(hWnd);
+    WSGIwinKeyboard()->updateFepStatus();
+  }
+#endif
+
+#ifdef _WSWINCE
+  if (message == WM_SETTINGCHANGE){
+dbprintf("WndProc %s:%d uParam=0x%x 0x%x %d\n",__FILE__,__LINE__,uParam,SPI_SETSIPINFO,(LPARAM)SPI_SETSIPINFO);
+    if (uParam == (LPARAM)SPI_SETSIPINFO){
+dbprintf("WndProc %s:%d uParam=0x%x 0x%x\n",__FILE__,__LINE__,uParam,SPI_SETSIPINFO);
+      SIPINFO sip;
+      memset (&sip,0,sizeof(SIPINFO));
+      sip.cbSize=sizeof(SIPINFO);
+      BOOL reti = SipGetInfo(&sip);
+      DWORD err = GetLastError();
+      WSCulong x = sip.rcVisibleDesktop.left;
+      WSCulong y = sip.rcVisibleDesktop.top;
+      WSCulong w = sip.rcVisibleDesktop.right - sip.rcVisibleDesktop.left;
+      WSCulong h = sip.rcVisibleDesktop.bottom - sip.rcVisibleDesktop.top;
+
+      RECT p;
+      GetClientRect(hWnd,&p);
+      if (p.right - p.left != w || p.bottom - p.top != h){
+        MoveWindow(hWnd,x,y,w,h,True);
+      }
+    }
+dbprintf("WndProc %s:%d done. uParam=0x%x 0x%x %d\n",__FILE__,__LINE__,uParam,SPI_SETSIPINFO,(LPARAM)SPI_SETSIPINFO);
+  }
+  if (message == WM_KILLFOCUS && uParam == 0 && lParam == 0){
+    HWND target = hWnd;
+    while(1){
+      HWND p = GetParent(target);
+      if ( p == NULL){
+        hWnd = target;
+        break;
+      }else{
+        target = p;
+      }
+    }
+    message = WM_CLOSE;
+  }
+#endif
+#ifndef _WSWINCE
+  if (WSGIwinAppDev()->_locked == False && WSGIwinAppDev()->getAppThreadId() == GetCurrentThreadId()){
+    _execute_pending_events();
+  }
+#endif
+
   WinEvent ev;
   ev.message = message;
   ev.uParam = uParam;
   ev.lParam = lParam;
-#if 0
-  if (WSGIwinAppDev()->getGrabedWnd() != 0 &&
-      GetCapture() != WSGIwinAppDev()->getGrabedWnd()){
-    SetCapture(WSGIwinAppDev()->getGrabedWnd());
+  ev.hWnd = hWnd;
+#ifndef _WSWINCE
+  if (message == WM_USER && uParam == WS_WIN_ALLOW_EXEC_EVT){
+    WSGIwinAppDev()->setAllowed(True);
+    MSG _msg;
+    long ret2 = GetMessage(&_msg,NULL,0,0);
+    if (ret2 > 0){
+      TranslateMessage(&_msg);
+      DispatchMessage(&_msg);
+    }
+    ReplyMessage(1);
+    WSGIwinAppDev()->setAllowed(False);
+    return 0;
   }
-WSMFtrace("grabed=0x%x\n",WSGIwinAppDev()->getGrabedWnd());
 #endif
-//printf("message ==0x%x message=%d hwnd=0x%x\n",message,message,hWnd);
+
+#ifndef _WSWINCE
+  if (message == WM_UNLOCK){
+    WSGIwinAppDev()->_locked = False;
+    ReplyMessage(1);
+    WSGIappObjectList()->execInitialize();
+    WSGIappObjectList()->execUpdate();
+    return 0;
+  }
+#endif
+#ifndef _WSWINCE
+  if (message == WM_USER && uParam == WS_WIN_LOCK){
+    WSGIwinAppDev()->_locked = True;
+    ReplyMessage(1);
+    return WS_NO_ERR;
+  }
+#endif
+
+#ifndef _WSWINCE
+  WSCbool gmouse = WSGIappMouse()->getGlobalMouseIdStatus();
+  if (gmouse != WSGIwinAppDev()->_global_cursor){
+    HCURSOR val = (HCURSOR)WSGIappMouse()->getMouseCursor(0);
+    WSGIwinAppDev()->_global_cursor = gmouse;
+    SetCursor(val);
+  }
+#endif
+
   if (message == WM_TIMER && hWnd == WSGIwinAppDev()->getHWND()){
+#ifndef _WSWINCE
+    if (WSGIwinAppDev()->_locked != False &&
+        WSGIwinAppDev()->getAllowed() == 0 &&
+        WSGIwinAppDev()->getAppThreadId() == GetCurrentThreadId()){
+      _add_pending_event(&ev);
+      return 0;
+    }
+#endif
     WSDwinPrivateTimer::_timer_callback(uParam);
 dbprintf("WndProc %s:%d done1\n",__FILE__,__LINE__);
-//printf("WndProc %s:%d done1\n",__FILE__,__LINE__);
     return 0;
   }
   if (message == WM_USER && uParam == WS_WIN_EXECUTE){
+#ifndef _WSWINCE
+    if (WSGIwinAppDev()->_locked != False &&
+        WSGIwinAppDev()->getAllowed() == 0 &&
+        WSGIwinAppDev()->getAppThreadId() == GetCurrentThreadId()){
+      _add_pending_event(&ev);
+      return 0;
+    }
+#endif
     WSDwinExecute* obj = (WSDwinExecute*)lParam;
     obj->_callback();
 dbprintf("WndProc %s:%d done2\n",__FILE__,__LINE__);
     return 0;
   }
   if (message == WM_USER && uParam == WS_WIN_ACCEPT){
+#ifndef _WSWINCE
+    if (WSGIwinAppDev()->_locked != False &&
+        WSGIwinAppDev()->getAllowed() == 0 &&
+        WSGIwinAppDev()->getAppThreadId() == GetCurrentThreadId()){
+      _add_pending_event(&ev);
+      return 0;
+    }
+#endif
     _accept_send_type* data = (_accept_send_type*)lParam;
     data->hd(data->sock,(void*)data->socket,data->addr);
     delete data;
@@ -406,24 +628,57 @@ dbprintf("WndProc %s:%d done3\n",__FILE__,__LINE__);
     return 0;
   }
   if (message == WM_USER && uParam == WS_WIN_THR){
-    WSDwinThread* ptr = (WSDwinThread*)lParam;
+#ifndef _WSWINCE
+    if (WSGIwinAppDev()->_locked != False &&
+        WSGIwinAppDev()->getAllowed() == 0 &&
+        WSGIwinAppDev()->getAppThreadId() == GetCurrentThreadId()){
+      _add_pending_event(&ev);
+      return 0;
+    }
+#endif
+    _internal_thr_data_* ptr = (_internal_thr_data_*)lParam;
+    WSDwinThread* thr = ptr->thr;
+    void* data = ptr->data;
     void (*fn)(WSDthread*,void*) =
-           (void(*)(WSDthread*,void*))ptr->getCallbackFunction();
-    fn(ptr,ptr->getCallbackData());
+           (void(*)(WSDthread*,void*))thr->getCallbackFunction();
+    if (fn != NULL){
+      fn(thr,data);
+    }
+    delete ptr;
     WSGIappObjectList()->execInitialize();
     WSGIappObjectList()->execUpdate();
     return 0;
   }
+
+#ifndef _WSWINCE
+  if (message == WM_CREATE_NEW_WINDOW_EX){
+    WSGFreplyCreateWindowRequestEx((void*)uParam);
+    return 0;
+  }else
+  if (message == WM_CREATE_NEW_WINDOW){
+    WSGFreplyCreateWindowRequest((void*)uParam);
+    return 0;
+  }
+  if (message == WM_DESTROY_WINDOW){
+    WSGFreplyDestroyWindowRequest((HWND)uParam);
+    return 0;
+  }
+#endif
   if (message == WM_USER){
+#ifndef _WSWINCE
+    if (WSGIwinAppDev()->_locked != False &&
+        WSGIwinAppDev()->getAllowed() == 0 &&
+        WSGIwinAppDev()->getAppThreadId() == GetCurrentThreadId()){
+      _add_pending_event(&ev);
+      return 0;
+    }
+#endif
     int ret = WSDwinMessage::receiveMessage((WSDwinMessage*)uParam,(void*)lParam);
     if (ret != False){
 dbprintf("WndProc %s:%d done4\n",__FILE__,__LINE__);
       return 0;
     }
   }
-#if 0
-  long ptr = GetWindowLong(hWnd,GWL_USERDATA);
-#else
 #ifndef _WSWINCE
   HANDLE ptr = GetProp(hWnd,"frm");
 #else
@@ -433,7 +688,6 @@ dbprintf("WndProc %s:%d done4\n",__FILE__,__LINE__);
   str1[2] = 'm';
   str1[3] = 0;
   HANDLE ptr = GetProp(hWnd,str1);
-#endif
 #endif
   WSDwinformDev* frm = (WSDwinformDev*)ptr;
   if (message == WM_CHAR ||
@@ -441,6 +695,14 @@ dbprintf("WndProc %s:%d done4\n",__FILE__,__LINE__);
       message == WM_KEYUP ||
       message == WM_SYSKEYDOWN ||
       message == WM_SYSKEYUP){
+#ifndef _WSWINCE
+    if (WSGIwinAppDev()->_locked != False &&
+        WSGIwinAppDev()->getAllowed() == 0 &&
+        WSGIwinAppDev()->getAppThreadId() == GetCurrentThreadId()){
+      _add_pending_event(&ev);
+      return 0;
+    }
+#endif
     MSG msg;
     msg.message = message;
     msg.wParam = uParam;
@@ -455,9 +717,13 @@ dbprintf("WndProc %s:%d done5\n",__FILE__,__LINE__);
   }
 #ifndef _WSWINCE
   if (message == WM_NCHITTEST){
-//printf("displatch.. nchittest..\n",hWnd);
+    if (WSGIwinAppDev()->_locked != False &&
+        WSGIwinAppDev()->getAllowed() == 0 &&
+        WSGIwinAppDev()->getAppThreadId() == GetCurrentThreadId()){
+      _add_pending_event(&ev);
+      return 0;
+    }
     LRESULT ret = DefWindowProc(hWnd,message,uParam,lParam);
-//printf("displatch.. ht=%d\n",ret);
     if (ret != HTCLIENT){
       WSDwinformDev* mdev = WSGIwinAppDev()->getMouseFocusDev();
       if (mdev != NULL){
@@ -476,20 +742,50 @@ dbprintf("WndProc %s:%d done7\n",__FILE__,__LINE__);
 dbprintf("WndProc %s:%d done0\n",__FILE__,__LINE__);
     return ret1;
   }
-  WSDwinAppDev::_event_wnd = hWnd;
-#if 0 //XXZZXXZZ
-   if (message == WM_LBUTTONDOWN ||
+#ifndef _WSWINCE
+  if (WSGIwinAppDev()->_locked != False &&
+      WSGIwinAppDev()->getAllowed() == 0 &&
+      WSGIwinAppDev()->getAppThreadId() == GetCurrentThreadId() &&
+      (message == WM_MOUSEMOVE ||
+       message == WM_PAINT ||
+       message == WM_ERASEBKGND ||
+       message == WM_LBUTTONDOWN ||
        message == WM_MBUTTONDOWN ||
-       message == WM_RBUTTONDOWN ){
-     HWND fwnd = GetFocus();
-     if (fwnd != hWnd || WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
-       SetFocus(hWnd);
-     }
-       SetActiveWindow(hWnd);
-       SetFocus(hWnd);
-   }
-#endif 
-//printf("WSDwinAppDev::displatch.. wnd=0x%x XXZZff1\n",hWnd);
+       message == WM_RBUTTONDOWN ||
+       (message == WM_MOUSEWHEEL && uParam == 0xff880000) ||
+       (message == WM_MOUSEWHEEL && uParam == 0x780000) ||
+       message == WM_LBUTTONUP ||
+       message == WM_MBUTTONUP ||
+       message == WM_RBUTTONUP ||
+       message == WM_MOVE ||
+       message == WM_SIZE ||
+       message == WM_CLOSE ||
+       message == WM_CANCELMODE ||
+//       message == WM_DESTROY || // this event needs be executed in order to execute _evh_delete handler instead of adding into pending list..
+       message == WM_KILLFOCUS ||
+       message == WM_SETFOCUS)){
+    _add_pending_event(&ev);
+    return 0;
+  }
+#endif
+  WSDwinAppDev::_event_wnd = hWnd;
+#ifndef _WSWINCE
+#ifndef NO_MWT_FUNCTION
+  //
+  WSDwinwinDev* wfrm = (WSDwinwinDev*)frm->cast("WSDwinwinDev");
+  if (message == WM_ACTIVATE && wfrm != NULL && wfrm->getRebaseHandle() != 0){
+    DWORD _tid;
+    WinEvent* ev = new WinEvent;
+    ev->message = message;
+    ev->uParam = uParam;
+    ev->lParam = lParam;
+    ev->hWnd = hWnd;
+    CreateThread(NULL,0,(LPTHREAD_START_ROUTINE)_activate_ev_start_handler,ev,0,(DWORD*)&_tid);
+    return 0;
+  }
+#endif
+#endif
+//printf("WndProc frm->dispatchEvent called.. mes=%d 0x%x\n",message,message);fflush(stdout);
   long ret = frm->dispatchEvent(&ev);
 
 //for IE escape key navigation...
@@ -551,6 +847,9 @@ dbprintf("WndProc %s:%d done11\n",__FILE__,__LINE__);
 
   if (ret != 0){
 //WSMFtrace("displatch.. wnd=0x%x ev=%d   XXZZm1\n",hWnd,message);
+    if (message == WM_SYSCHAR){
+      return 0;
+    }
     HRESULT ret1 = DefWindowProc(hWnd,message,uParam,lParam);
 dbprintf("WndProc %s:%d done11\n",__FILE__,__LINE__);
     return ret1;
@@ -559,34 +858,13 @@ dbprintf("WndProc %s:%d done12\n",__FILE__,__LINE__);
   WSGIappObjectList()->execInitialize();
   WSGIappObjectList()->execUpdate();
   return 0;
-#if 0
-  switch(message){
-    
-    case WM_COMMAND:
-      wmld = LOWORD(uParam);
-      wmEvent = HIWORD(uParam);
-    case WM_DESTROY:
-      PostQuitMessage(0);
-      break;
-    case WM_PAINT:
-      hdc = BeginPaint(hWnd,&ps);
-      GetClientRect(hWnd,&rect);
-      GetWindowRect(hWnd,&winsize);
-      sprintf(buffer,"size = %d,%d",winsize.right - winsize.left,winsize.bottom - winsize.top);
-      DrawText(hdc,buffer,-1,&rect,DT_SINGLELINE | DT_CENTER |DT_VCENTER);
-      EndPaint(hWnd,&ps);
-      break;
-    default:
-      return DefWindowProc(hWnd,message,uParam,lParam);
-  }
-  return 0;
-#endif
 }
 void WSDwinAppDev::addUpdateDev(WSDwinformDev* dev){
   long num = _update_dev_list.getNum();
   long i;
+  WSDwinformDev** buf = (WSDwinformDev**)_update_dev_list._data;
   for(i = 0; i < num; i++){
-    WSDwinformDev* d = (WSDwinformDev*)_update_dev_list[i];
+    WSDwinformDev* d = (WSDwinformDev*)buf[i];
     if (d == dev){
       return;
     }
@@ -600,14 +878,15 @@ void WSDwinAppDev::delUpdateDev(WSDwinformDev* dev){
 void WSDwinAppDev::update(){
   long num = _update_dev_list.getNum();
   long i;
-  WSClistData bk;
-  for(i = 0; i < num; i++){
-    bk.add(_update_dev_list[i]);
-  }
-  for(i = 0; i < num; i++){
-    WSDwinformDev* d = (WSDwinformDev*)bk[i];
-    d->setExposed(True);
-    d->_clear_and_expose();
+  if (num > 0){
+    WSClistData bk;
+    WSDwinformDev** buf = new WSDwinformDev*[num];
+    memcpy(buf,_update_dev_list._data,num*sizeof(WSDwinformDev*));
+    for(i = 0; i < num; i++){
+      buf[i]->setExposed(True);
+      buf[i]->_clear_and_expose();
+    }
+    delete buf;
   }
   while(1){
     long ret = PeekMessage(&_msg,NULL,0,0,PM_REMOVE);
@@ -693,6 +972,7 @@ HWND WSDwinAppDev::getEventWnd(){
   return _event_wnd;
 }
 void WSDwinAppDev::setGrabedWnd(HWND wnd){
+//printf("WSDwinAppDev::setGrabedWnd wnd=0x%x..\n",wnd);fflush(stdout);
   _grabed_wnd = wnd;
   if (wnd == NULL){
     ReleaseCapture();
@@ -715,3 +995,65 @@ long WSDwinAppDev::setWndProc(void* proc){
 void* WSDwinAppDev::getWndProc(){
   return _wnd_proc;
 }
+void WSDwinAppDev::_init_lock(){
+  _mutex = WSDsemaphore::getNewInstance();
+  _mutex->initialize();
+}
+long WSDwinAppDev::lock(){
+  if (_mutex == NULL){
+    _init_lock();
+  }
+  if (_mutex == NULL){
+    return WS_ERR;
+  }
+  _mutex->lock();
+#ifdef THR_DBG
+printf("WSDwinAppDev::lock thr=0x%x started.\n",GetCurrentThreadId());fflush(stdout);
+#endif
+  LRESULT ret = SendMessage(getHWND(),WM_USER,(WPARAM)WS_WIN_LOCK,(LPARAM)0);
+  if (ret == 0){
+    _mutex->unlock();
+    return WS_ERR;
+  }
+#ifdef THR_DBG
+printf("WSDwinAppDev::lock thr=0x%x done.\n",GetCurrentThreadId());fflush(stdout);
+#endif
+  return WS_NO_ERR;
+}
+
+long WSDwinAppDev::unlock(){
+  if (_mutex == NULL){
+    return WS_ERR;
+  }
+#ifdef THR_DBG
+printf("WSDwinAppDev::unlock\n");fflush(stdout);
+#endif
+  LRESULT ret = SendMessage(getHWND(),WM_UNLOCK,(WPARAM)0,(LPARAM)0);
+  if (ret == 0){
+    _mutex->unlock();
+    return WS_ERR;
+  }
+  _mutex->unlock();
+#ifdef THR_DBG
+printf("WSDwinAppDev::unlock thr=0x%x done.\n",GetCurrentThreadId());fflush(stdout);
+#endif
+  return WS_NO_ERR;
+}
+void WSDwinAppDev::setAppThreadId(DWORD tid){
+  _tid = tid;
+}
+DWORD WSDwinAppDev::getAppThreadId(){
+  return _tid;
+}
+WSCbool WSDwinAppDev::getAllowed(){
+  return _allowed;
+}
+void WSDwinAppDev::setAllowed(WSCbool fl){
+  _allowed = fl;
+}
+void WSDwinAppDev::showCursor(WSCbool fl){
+  if (_show_cursor != fl){
+    _show_cursor = fl;
+    ShowCursor(fl);
+  }
+}
