@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+//
 
 #include <WScom.h>
 #include <win/WSDwinDragDrop.h>
@@ -38,6 +35,7 @@ static WSDdragDrop* _winDrag_drop_create_handler_(){
   return new WSDwinDragDrop;
 }
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _wsd_winDragdrop_init{
   public:
     _wsd_winDragdrop_init(){
@@ -45,6 +43,7 @@ class _wsd_winDragdrop_init{
     };
 };
 static _wsd_winDragdrop_init  execute_init;
+#endif
 
 WSDwinDragDrop* WSGIwinAppDragDrop(){
   WSDdragDrop* dd = WSGIappDragDrop();
@@ -109,7 +108,12 @@ WSMFtrace("WSDwinDragDrop search hit. XXZZe1\n");
   if (windev != NULL){
     windev->setValue(WSDEV_MOUSE_NO,&_mouse_back);
     HCURSOR mval = (HCURSOR)WSGIappMouse()->getMouseCursor(_mouse_back);
-    SetCursor(mval);
+    if ((long)mval == -1){
+      WSGIwinAppDev()->showCursor(false);
+    }else{
+      WSGIwinAppDev()->showCursor(true);
+      SetCursor(mval);
+    }
   }
 }
 
@@ -137,7 +141,12 @@ WSMFtrace("WSDwinDragDrop:: drag_start %s  XXZZe1\n",client->getInstanceName());
     WSCushort val = 154;
     windev->setValue(WSDEV_MOUSE_NO,&val);
     HCURSOR mval = (HCURSOR)WSGIappMouse()->getMouseCursor(val);
-    SetCursor(mval);
+    if ((long)mval != -1){
+      WSGIwinAppDev()->showCursor(false);
+    }else{
+      WSGIwinAppDev()->showCursor(true);
+      SetCursor(mval);
+    }
   }else{
     obj->_mouse_back = 0;
   }
