@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+//
 
 #include <win/WSDwinEnv.h>
 #include <WSCstring.h>
@@ -32,16 +29,17 @@
 WSDenv* _win_env_chandler(){
   return new WSDwinEnv();
 }
-class _win_env_init{
-  public: _win_env_init(){
-    WSDenv::setCreateInstanceHandler(_win_env_chandler);
-  };
-};
-static _win_env_init _win_env_init_execute;
-
 void WSGFwinEnvInit(){
   WSDenv::setCreateInstanceHandler(_win_env_chandler);
 }
+#ifndef NO_GLOBAL_CONSTRUCTORS
+class _win_env_init{
+  public: _win_env_init(){
+    WSGFwinEnvInit();
+  };
+};
+static _win_env_init _win_env_init_execute;
+#endif
 
 WSDwinEnv::WSDwinEnv(){}
 WSDwinEnv::~WSDwinEnv(){}
