@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+//
 
 #include <WScom.h>
 #include <WSClocaleSet.h>
@@ -48,13 +45,14 @@ static long _init = 0;
   _init = 1;
   WSDfile::setCreateInstanceHandler((void*)_win_file_chandler);
 }
-
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _win_file_init {
   public: _win_file_init(){
-    WSDfile::setCreateInstanceHandler((void*)_win_file_chandler);
+    WSGFinitializeWinFile();
   };
 };
 _win_file_init  _win_file_init_execute;
+#endif
 
 WSDwinFile::WSDwinFile(){
   _file = NULL;
@@ -70,7 +68,20 @@ long WSDwinFile::open(char* name,char* mode){
   _file_name.setString(name);
   _file_name.delHeadSpace();
 //  _file_name.replaceString("/","\\",0);
+#ifndef NO_VIRTUAL_FS
+  if (WSDfile::getFileHandleRegistered() != False){
+    WSCulong size =0;
+    WSCuchar* buf = WSDfile::getFileHandle((char*)_file_name,&size);
+    if (buf != NULL){
+      setType(WS_VFS);
+      return WSDfile::open(name,mode);
+    }
+  }
+#endif
 
+#ifdef NO_FILE_SYSTEM
+  return WS_ERR;
+#else //NO_FILE_SYSTEM
   char* fn = _file_name.getString(WSGIappLocaleSet()->getSystemLocaleEncoding());
   if (!strncmp("http://",fn,7)){
     char*  pbuf = NULL;
@@ -229,9 +240,19 @@ long WSDwinFile::open(char* name,char* mode){
     }
   }
   return WS_ERR;
+#endif //NO_FILE_SYSTEM
 }
 
 long WSDwinFile::close(){
+#ifndef NO_VIRTUAL_FS
+  if (getType() == WS_VFS){
+    return WSDfile::close();
+  }
+#endif
+
+#ifdef NO_FILE_SYSTEM
+  return WS_ERR;
+#else //NO_FILE_SYSTEM
   if (_file != NULL){
     fclose(_file);
     _file = NULL;
@@ -245,8 +266,18 @@ long WSDwinFile::close(){
     _file2 = INVALID_HANDLE_VALUE;
   }
   return WS_NO_ERR;
+#endif //NO_FILE_SYSTEM
 }
 long WSDwinFile::read(void* buf,long len){
+#ifndef NO_VIRTUAL_FS
+  if (getType() == WS_VFS){
+    return WSDfile::read(buf,len);
+  }
+#endif
+
+#ifdef NO_FILE_SYSTEM
+  return WS_ERR;
+#else //NO_FILE_SYSTEM
   if (_mfile != NULL){
     return _mfile->read(buf,len);
   }
@@ -261,8 +292,18 @@ long WSDwinFile::read(void* buf,long len){
     } 
   }
   return 0;
+#endif //NO_FILE_SYSTEM
 }
 long WSDwinFile::write(void* buf,long len){
+#ifndef NO_VIRTUAL_FS
+  if (getType() == WS_VFS){
+    return WSDfile::write(buf,len);
+  }
+#endif
+
+#ifdef NO_FILE_SYSTEM
+  return WS_ERR;
+#else //NO_FILE_SYSTEM
   if (_mfile != NULL){
     return _mfile->write(buf,len);
   }
@@ -277,8 +318,18 @@ long WSDwinFile::write(void* buf,long len){
     } 
   }
   return 0;
+#endif //NO_FILE_SYSTEM
 }
 WSCbool WSDwinFile::eof(){
+#ifndef NO_VIRTUAL_FS
+  if (getType() == WS_VFS){
+    return WSDfile::eof();
+  }
+#endif
+
+#ifdef NO_FILE_SYSTEM
+  return WS_ERR;
+#else //NO_FILE_SYSTEM
   if (_mfile != NULL){
     return _mfile->eof();
   }
@@ -296,12 +347,32 @@ WSCbool WSDwinFile::eof(){
     }
   }
   return False;
+#endif //NO_FILE_SYSTEM
 }
 
 long WSDwinFile::seekSet(){
+#ifndef NO_VIRTUAL_FS
+  if (getType() == WS_VFS){
+    return WSDfile::seekSet();
+  }
+#endif
+
+#ifdef NO_FILE_SYSTEM
+  return WS_ERR;
+#else //NO_FILE_SYSTEM
   return seek(0);
+#endif //NO_FILE_SYSTEM
 }
 long WSDwinFile::seekEnd(){
+#ifndef NO_VIRTUAL_FS
+  if (getType() == WS_VFS){
+    return WSDfile::seekEnd();
+  }
+#endif
+
+#ifdef NO_FILE_SYSTEM
+  return WS_ERR;
+#else //NO_FILE_SYSTEM
   if (_mfile != NULL){
     _mfile->seekEnd();
     return WS_NO_ERR;
@@ -317,8 +388,18 @@ long WSDwinFile::seekEnd(){
     return WS_ERR;
   }
   return WS_NO_ERR;
+#endif //NO_FILE_SYSTEM
 }
 long WSDwinFile::seek(long addr){
+#ifndef NO_VIRTUAL_FS
+  if (getType() == WS_VFS){
+    return WSDfile::seek(addr);
+  }
+#endif
+
+#ifdef NO_FILE_SYSTEM
+  return WS_ERR;
+#else //NO_FILE_SYSTEM
   if (_mfile != NULL){
     return _mfile->seek(addr);
   }
@@ -334,8 +415,18 @@ long WSDwinFile::seek(long addr){
   }
 WSMFtrace("WSDwinFDB::_seek error file=NULL\n");
   return WS_NO_ERR;
+#endif //NO_FILE_SYSTEM
 }
 long WSDwinFile::getSize(){
+#ifndef NO_VIRTUAL_FS
+  if (getType() == WS_VFS){
+    return WSDfile::getSize();
+  }
+#endif
+
+#ifdef NO_FILE_SYSTEM
+  return WS_ERR;
+#else //NO_FILE_SYSTEM
   if (_mfile != NULL){
     return _mfile->getSize();
   }
@@ -350,8 +441,18 @@ long WSDwinFile::getSize(){
     return GetFileSize(_file2,NULL);
   }
   return 0;
+#endif //NO_FILE_SYSTEM
 }
 long WSDwinFile::tell(){
+#ifndef NO_VIRTUAL_FS
+  if (getType() == WS_VFS){
+    return WSDfile::tell();
+  }
+#endif
+
+#ifdef NO_FILE_SYSTEM
+  return WS_ERR;
+#else //NO_FILE_SYSTEM
   if (_mfile != NULL){
     return _mfile->tell();
   }
@@ -362,4 +463,5 @@ long WSDwinFile::tell(){
     return SetFilePointer(_file2,0,NULL,FILE_CURRENT);
   }
   return 0;
+#endif //NO_FILE_SYSTEM
 }
