@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+//
 
 #include <WScom.h>
 #include <win/WSDwinImage.h>
@@ -51,8 +48,22 @@ int SetDIBits(HDC hdc,HBITMAP bm,UINT st,UINT scanline,void* buf1,BITMAPINFO* bm
   WSCuchar* buf = (WSCuchar*)buf1;
   int w = bmi->bmiHeader.biWidth;
   int h = bmi->bmiHeader.biHeight;
-  int rowbyte = (w+1)*3 & ~3;
+  int rowbyte = ((w*bmi->bmiHeader.biBitCount+ 31) & ~31)/8;
+  HDC tmpdc = GetDC(NULL);
+  void* bbuf = NULL;
+  HBITMAP bmp = CreateDIBSection(tmpdc,bmi,DIB_RGB_COLORS,(void**)&bbuf,NULL,0);
+  if (bbuf != NULL){
+    memcpy(bbuf,buf,rowbyte*h);
+  }
+  ReleaseDC(NULL,tmpdc);
 
+  tmpdc = CreateCompatibleDC(NULL);
+  SelectObject(tmpdc,bmp);
+  BitBlt(hdc,0,0,w,h,tmpdc,0,0,SRCCOPY);
+  DeleteDC(tmpdc);
+  DeleteObject(tmpdc);
+  DeleteObject(bmp);
+#if 0
   int i,j;
   COLORREF c;
   int p;
@@ -80,6 +91,64 @@ int SetDIBits(HDC hdc,HBITMAP bm,UINT st,UINT scanline,void* buf1,BITMAPINFO* bm
       }
     }
   }
+#endif
+  return True;
+}
+int SetDIBits2(HDC hdc,HBITMAP bm,UINT st,UINT scanline,void* buf1,BITMAPINFO* bmi,UINT pl,int fl){
+  WSCuchar* buf = (WSCuchar*)buf1;
+  int w = bmi->bmiHeader.biWidth;
+  int h = bmi->bmiHeader.biHeight;
+  int rowbyte = ((w*bmi->bmiHeader.biBitCount+ 31) & ~31)/8;
+  HDC tmpdc = GetDC(NULL);
+  void* data = NULL;
+  HBITMAP bmp = CreateDIBSection(tmpdc,bmi,DIB_RGB_COLORS,(void**)&data,NULL,0);
+  if (data != NULL){
+    unsigned char r,g,b;
+    long ptr = 0;
+    long ptr2 = 0;
+    long i,j;
+
+    if (fl == 3){
+      long base = 0;
+      unsigned char* buf2 = (unsigned char*)data;
+      for(i=0; i<h; i++){
+        base = (h - 1 -i)*rowbyte;
+        for(j=0; j<w; j++){
+          buf2[base+2] = buf[ptr++];
+          buf2[base+1] = buf[ptr++];
+          buf2[base] = buf[ptr++];
+          base +=3;
+        }
+      }
+    }else if (fl == 2){
+      for(i=0; i<h; i++){
+        for(j=0; j<w; j++){
+          long base = (h - 1 -i)*rowbyte + j*3;
+          ((unsigned char*)data)[base +2] = buf[ptr];
+          ((unsigned char*)data)[base +1] = buf[ptr];
+          ((unsigned char*)data)[base] = buf[ptr++];
+          ptr2 +=3;
+        }
+      }
+    }else{
+        for(i=0; i<h; i++){
+          for(j=0; j<w; j++){
+            r = buf[ptr]>>3;
+            g = buf[ptr]>>2;
+            b = buf[ptr++]>>3;
+            ((short*)data)[ptr2++] =  r <<11 | g<<5 | b;
+          }
+        }
+    }
+  }
+  ReleaseDC(NULL,tmpdc);
+
+  tmpdc = CreateCompatibleDC(NULL);
+  SelectObject(tmpdc,bmp);
+  BitBlt(hdc,0,0,w,h,tmpdc,0,0,SRCCOPY);
+  DeleteDC(tmpdc);
+  DeleteObject(tmpdc);
+  DeleteObject(bmp);
   return True;
 }
 int GetDIBits(HDC hdc,HBITMAP bm,UINT st,UINT scanline,void* buf1,BITMAPINFO* bmi,UINT pl){
@@ -225,19 +294,21 @@ dbprintf("WSDwinImage::initialize %s read done.\n",(char*)in);
 WSMFtrace("XXZZim1 WSDwinImage::init fname=#%s# 0x%x \n",(char*)in,hBitmap);
   if (hBitmap == 0){
     long words = in.getWords("\\");
+    WSCstring in2;
     if (words > 1){
-      WSCstring in2 = in.getWord(words-1,"\\");
+      in2 = in.getWord(words-1,"\\");
       load_file_name = (char*)in2;
+    }
 #ifndef _WSWINCE
-      hBitmap = (HBITMAP)LoadImage( NULL,
+    hBitmap = (HBITMAP)LoadImage( NULL,
         (LPCTSTR)load_file_name,
         IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_CREATEDIBSECTION | LR_LOADREALSIZE );
+//printf("WSDwinImage::initialize loadbmp=#%s# val=%d\n",(char*)load_file_name,hBitmap);
 #else
-      hBitmap = (HBITMAP)LoadImage( NULL,
+    hBitmap = (HBITMAP)LoadImage( NULL,
         (LPCTSTR)load_file_name,
         IMAGE_BITMAP, 0, 0, LR_LOADREALSIZE );
 #endif
-    }
   }
   if (hBitmap == 0){
     WSCstring in2(iname);
@@ -246,9 +317,11 @@ WSMFtrace("XXZZim1 WSDwinImage::init fname=#%s# 0x%x \n",(char*)in,hBitmap);
     in2.replaceString(".jpg", ".bmp", 0);
     in2.replaceString(".JPG", ".bmp", 0);
     long val = (long)WSGIappImageSet()->getDefaultImageHandle((char*)in2);
+//printf("WSDwinImage::initialize default image handle=#%s# val=%d\n",(char*)in2,val);
     if (val != 0){
 #ifndef _WSWINCE
       hBitmap = (HBITMAP)LoadBitmap(WSGIwinAppDev()->getInstanceHandle(), (char*)val);
+//printf("WSDwinImage::initialize loadb=0x%x\n",hBitmap);
 #else
       WSCushort* tmpstr = WSGFgetUCS2((char*)val,WS_EN_DEFAULT);
       hBitmap = (HBITMAP)LoadBitmap(WSGIwinAppDev()->getInstanceHandle(), tmpstr);
@@ -271,19 +344,20 @@ WSMFtrace("XXZZim1 WSDwinImage::init fname=#%s# 0x%x \n",(char*)in,hBitmap);
 #endif
     if (hBitmap == 0){
       long words = in.getWords("\\");
+      WSCstring in2;
       if (words > 1){
         WSCstring in2 = in.getWord(words-1,"\\");
         load_file_name = (char*)in2;
+      }
 #ifndef _WSWINCE
-        hBitmap = (HBITMAP)LoadImage( NULL,
+      hBitmap = (HBITMAP)LoadImage( NULL,
           (LPCTSTR)load_file_name,
           IMAGE_ICON, 0, 0, LR_LOADFROMFILE | LR_CREATEDIBSECTION | LR_LOADREALSIZE );
 #else
-        hBitmap = (HBITMAP)LoadImage( NULL,
+      hBitmap = (HBITMAP)LoadImage( NULL,
           (LPCTSTR)load_file_name,
           IMAGE_ICON, 0, 0, LR_LOADREALSIZE );
 #endif
-      }
     }
     if (hBitmap == 0){
       WSCstring in2(iname);
@@ -295,8 +369,9 @@ WSMFtrace("XXZZim1 WSDwinImage::init fname=#%s# 0x%x \n",(char*)in,hBitmap);
       if (val != 0){
 #ifndef _WSWINCE
         hBitmap = (HBITMAP)LoadIcon(WSGIwinAppDev()->getInstanceHandle(), (char*)val);
+//printf("WSDwinImage::initialize loadicon=0x%x val=%d\n",hBitmap,val);
 #else
-        hBitmap = 0;
+        hBitmap = (HBITMAP)LoadIcon(WSGIwinAppDev()->getInstanceHandle(), (unsigned short*)val);
 #endif
       }
     }
@@ -516,25 +591,58 @@ dbprintf("2 cinf->global_state=%d w,h=%d,%d components=%d\n",jinf.global_state,w
 //    long gap = width*3 + bytes_per_line;
     bytes_per_line = (width+1)*3 & ~3;
 
-    WSCuchar* data = new WSCuchar[bytes_per_line * height];
-
-#if 0
-    short tmp1 = 0xff00;
-    char* tmp2 = (char*)&tmp1; 
-    if (tmp2[0] == 0xff){
-      msb_flag = 1;
-    }else{
-      msb_flag = 0;
+#ifdef _WSWINCE
+    HDC hdc = GetDC((HWND)WSGIwinAppDev()->getWindowResource());
+    HBITMAP hBitmap = CreateCompatibleBitmap(hdc,width,height);
+    ReleaseDC((HWND)WSGIwinAppDev()->getWindowResource(),hdc);
+    mdc = CreateCompatibleDC(NULL);
+    HBITMAP pxbk = (HBITMAP)SelectObject(mdc,(void*)hBitmap);
+    int fl = components;
+    if (fl != 3){
+      if (jinf.jpeg_color_space == JCS_GRAYSCALE){
+        fl = 2;
+      }else{
+        fl = 1;
+      }
     }
+    if (!SetDIBits2(mdc,hBitmap,0,height,buf,&bmi,DIB_RGB_COLORS,fl)){
+      DeleteObject((void*)hBitmap);
+      hBitmap = 0;
+    }
+    SelectObject(mdc,(void*)pxbk);
+    GdiFlush();
+    DeleteDC(mdc);
 #endif
 
+#ifndef _WSWINCE
+    WSCuchar* data = new WSCuchar[bytes_per_line * height];
+    if (data == NULL){
+      jpeg_finish_decompress(&jinf);
+      jpeg_destroy_decompress(&jinf);
+      delete buf;
+      return 0;
+    }
+
     unsigned char r,g,b;
     long ptr = 0;
     long ptr2 = 0;
     long j;
 
     if (components == 3){
-//        long end = height*width-1;
+#if 1 //speed up..
+        long base = 0;
+        unsigned char* buf2 = (unsigned char*)data;
+        for(i=0; i<height; i++){
+          base = (height - 1 -i)*bytes_per_line;
+          for(j=0; j<width; j++){
+            buf2[base+2] = buf[ptr++];
+            buf2[base+1] = buf[ptr++];
+            buf2[base] = buf[ptr++];
+            base +=3;
+          }
+        }
+#endif
+#if 0
         for(i=0; i<height; i++){
           for(j=0; j<width; j++){
             long base = (height - 1 -i)*bytes_per_line + j*3;
@@ -544,6 +652,7 @@ dbprintf("2 cinf->global_state=%d w,h=%d,%d components=%d\n",jinf.global_state,w
             ptr2 +=3;
           }
         }
+#endif
     }else if (jinf.jpeg_color_space == JCS_GRAYSCALE){
       for(i=0; i<height; i++){
         for(j=0; j<width; j++){
@@ -564,32 +673,27 @@ dbprintf("2 cinf->global_state=%d w,h=%d,%d components=%d\n",jinf.global_state,w
           }
         }
     }
+
+
     HDC hdc = GetDC((HWND)WSGIwinAppDev()->getWindowResource());
     HBITMAP hBitmap = CreateCompatibleBitmap(hdc,width,height);
     ReleaseDC((HWND)WSGIwinAppDev()->getWindowResource(),hdc);
-
     mdc = CreateCompatibleDC(NULL);
-#ifdef _WSWINCE
-     HBITMAP pxbk = (HBITMAP)SelectObject(mdc,(void*)hBitmap);
-#endif
     if (!SetDIBits(mdc,hBitmap,0,height,data,&bmi,DIB_RGB_COLORS)){
-WSMFtrace("XXZZim1 setDIBits error %s\n",filename);
       DeleteObject((void*)hBitmap);
       hBitmap = 0;
     }
-#ifdef _WSWINCE
-     SelectObject(mdc,(void*)pxbk);
-#endif
     GdiFlush();
     DeleteDC(mdc);
-
-    jpeg_finish_decompress(&jinf);
-    jpeg_destroy_decompress(&jinf);
-
     if (data != NULL){
       delete data;
       data = NULL;
     }
+#endif
+
+    jpeg_finish_decompress(&jinf);
+    jpeg_destroy_decompress(&jinf);
+
     if (buf != NULL){
       delete buf;
     }
@@ -852,8 +956,9 @@ fprintf(stderr,"png read error: out of memory..\n");
 //    if (components == 3 || components == 4){
       for(i=0; i<height; i++){
         ptr = i * png_row_bytes;
+        long base = (height - 1 -i)*bytes_per_line;
         for(j=0; j<width; j++){
-          long base = (height - 1 -i)*bytes_per_line + j*3;
+//          long base = (height - 1 -i)*bytes_per_line + j*3;
           ((unsigned char*)data)[base +2] = buf[ptr++];
           ((unsigned char*)data)[base +1] = buf[ptr++];
           ((unsigned char*)data)[base] = buf[ptr++];
@@ -863,6 +968,7 @@ fprintf(stderr,"png read error: out of memory..\n");
             ((unsigned char*)data2)[base +1] = a;
             ((unsigned char*)data2)[base] = a;
           }
+          base += 3;
         }
       }
 //    }else{
