@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+//
 
 #include <win/WSDwinKeyboard.h>
 #include <windows.h>
@@ -65,6 +62,7 @@ WSDwinKeyboard::WSDwinKeyboard(){
   _font_no = -1;
   _fep_on = 0;
   _pos = 0;
+  _fep_status_bk = False;
 }
 
 WSDwinKeyboard::~WSDwinKeyboard(){
@@ -73,6 +71,7 @@ WSDwinKeyboard::~WSDwinKeyboard(){
     _hwnd = 0;
   }
   if (_himc != 0){
+    ImmSetOpenStatus(_himc, FALSE);
     ImmDestroyContext(_himc);
     _himc = 0;
   }
@@ -84,7 +83,10 @@ long WSDwinKeyboard::initialize(){
   return WS_NO_ERR;
 }
 
-long WSDwinKeyboard::startKanji(WSDdev* dev,short x,short y,short fontno,short fgno,short bgno){
+long WSDwinKeyboard::startFep(WSDdev* dev,short x,short y,short fontno,short fgno,short bgno){
+  if (_himc == 0){
+    _himc = ImmCreateContext();
+  }
   HWND wnd = (HWND)dev->getWindowResource();
   if (wnd == NULL || wnd == (HWND)-1){
     return WS_ERR;
@@ -97,10 +99,17 @@ long WSDwinKeyboard::startKanji(WSDdev* dev,short x,short y,short fontno,short f
     _hwnd = wnd;
     ImmAssociateContext(_hwnd,_himc);
   }
-  ImmSetOpenStatus(_himc, TRUE);
-  DWORD conversion = IME_CMODE_ROMAN | IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE;
-  DWORD sentence = IME_SMODE_PHRASEPREDICT;
-  ImmSetConversionStatus(_himc,conversion,sentence);
+  if (getEnableFep() == False){
+    ImmSetOpenStatus(_himc, FALSE);
+    DWORD conversion = IME_CMODE_ROMAN;
+    DWORD sentence = IME_SMODE_NONE;
+    ImmSetConversionStatus(_himc,conversion,sentence);
+  }else{
+    ImmSetOpenStatus(_himc, TRUE);
+    DWORD conversion = IME_CMODE_ROMAN | IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE;
+    DWORD sentence = IME_SMODE_PHRASEPREDICT;
+    ImmSetConversionStatus(_himc,conversion,sentence);
+  }
 
 
   short sx,sy;
@@ -148,6 +157,9 @@ long WSDwinKeyboard::resetFep(WSDdev* dev){
   if (wnd == NULL || wnd == (HWND)-1){
     return WS_ERR;
   }
+  if (_himc == 0){
+    return WS_NO_ERR;
+  }
   if (_hwnd != wnd){
     if (_hwnd != NULL){
       ImmAssociateContext(_hwnd,0);
@@ -155,12 +167,51 @@ long WSDwinKeyboard::resetFep(WSDdev* dev){
     _hwnd = wnd;
     ImmAssociateContext(_hwnd,_himc);
   }
+  if (ImmGetOpenStatus(_himc) != False){
+    _fep_status_bk = True;
+    DWORD conv;
+    ImmGetConversionStatus(_himc,&conv,&_sent_bk);
+  }
   ImmSetOpenStatus(_himc, FALSE);
   _fep_on = False;
 
   return WS_NO_ERR;
 }
 
+long WSDwinKeyboard::setFepStatus(long val){
+  if (val == WS_FEP_START){
+    if (_himc != 0){
+      ImmSetOpenStatus(_himc, TRUE);
+      DWORD conversion = IME_CMODE_ROMAN | IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE;
+      DWORD sentence = IME_SMODE_PHRASEPREDICT;
+      ImmSetConversionStatus(_himc,conversion,sentence);
+    }
+    return WS_NO_ERR;
+  }else if (val == WS_FEP_STOP){
+    if (_himc != 0){
+      ImmSetOpenStatus(_himc, FALSE);
+      DWORD conversion = IME_CMODE_ROMAN;
+      DWORD sentence = IME_SMODE_NONE;
+      ImmSetConversionStatus(_himc,conversion,sentence);
+    }
+    return WS_NO_ERR;
+  }else if (val == WS_FEP_KATAKANA){
+    if (_himc != 0){
+      ImmSetOpenStatus(_himc, TRUE);
+      DWORD conversion = IME_CMODE_NATIVE | IME_CMODE_KATAKANA | IME_CMODE_FULLSHAPE;
+      DWORD sentence = IME_SMODE_PHRASEPREDICT;
+      ImmSetConversionStatus(_himc,conversion,sentence);
+    }
+  }else if (val == WS_FEP_HKATAKANA){
+    if (_himc != 0){
+      ImmSetOpenStatus(_himc, TRUE);
+      DWORD conversion = IME_CMODE_NATIVE | IME_CMODE_KATAKANA;
+      DWORD sentence = IME_SMODE_PHRASEPREDICT;
+      ImmSetConversionStatus(_himc,conversion,sentence);
+    }
+  }
+}
+
 WSDdev* _fep_dev = NULL;
 short _fep_pos_x = -1;
 short _fep_pos_y = -1;
@@ -170,7 +221,15 @@ short _fep_bg_no = -1;
 
 long WSDwinKeyboard::setFepPos(WSDdev* dev,short x,short y,short fontno,short fgno,short bgno){
 WSMFtrace("WSDwinKeyboard::setFepPos x,y=%d,%d\n",x,y);
-//printf("WSDwinKeyboard::setFepPos x,y=%d,%d\n",x,y);
+//  if (getEnableFep() == False){
+//    if (_himc != 0){
+//      ImmSetOpenStatus(_himc, FALSE);
+//    }
+//    return WS_NO_ERR;
+//  }
+  if (_himc == 0){
+    _himc = ImmCreateContext();
+  }
   HWND wnd = (HWND)dev->getWindowResource();
   if (wnd == NULL || wnd == (HWND)-1){
     return WS_ERR;
@@ -184,6 +243,32 @@ WSMFtrace("WSDwinKeyboard::setFepPos x,y=%d,%d\n",x,y);
     ImmAssociateContext(_hwnd,_himc);
   }
 
+  if (getEnableFep() == False){
+    ImmSetOpenStatus(_himc, FALSE);
+    DWORD conversion = IME_CMODE_ROMAN;
+    DWORD sentence = IME_SMODE_NONE;
+    ImmSetConversionStatus(_himc,conversion,sentence);
+  }else{
+    if (_fep_status_bk != False){
+      ImmSetOpenStatus(_himc, TRUE);
+      _fep_status_bk = False;
+      DWORD conversion;
+      DWORD sentence;
+      ImmGetConversionStatus(_himc,&conversion,&sentence);
+      if (!(conversion & IME_CMODE_NATIVE)){
+        conversion = IME_CMODE_ROMAN | IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE;
+      }
+      ImmSetConversionStatus(_himc,conversion,_sent_bk);
+    }else{
+      DWORD conversion;
+      ImmGetConversionStatus(_himc,&conversion,&_sent_bk);
+      if (!(conversion & IME_CMODE_NATIVE) && WSGIappLocaleSet()->getSystemLocaleEncoding() == WS_EN_SJIS){
+        conversion = IME_CMODE_ROMAN | IME_CMODE_NATIVE | IME_CMODE_FULLSHAPE;
+        DWORD sentence = IME_SMODE_PHRASEPREDICT;
+        ImmSetConversionStatus(_himc,conversion,sentence);
+      }
+    }
+  }
   short sx,sy;
   dev->getAbsoluteAddr(x,y,&sx,&sy);
   dev->setAbsoluteAddrChangeHandler(_device_move_handler,this);
@@ -222,6 +307,9 @@ WSMFtrace("WSDwinKeyboard::setFepPos() setfl=0x%x\n",lf);
 }
 
 WSCbool WSDwinKeyboard::_dispatch_key(MSG* msg){
+  if (_global_key_sensitive == False){
+    return False;
+  }
   if (msg->message == WM_KEYDOWN && msg->wParam ==VK_PROCESSKEY){
 //printf("process key ev=%d\n",msg->message);
     if (WSGIappLocaleSet()->getSystemLocaleEncoding()==WS_EN_EUCKR &&
@@ -237,8 +325,13 @@ WSCbool WSDwinKeyboard::_dispatch_key(MSG* msg){
 //printf("here001 ev=%d code=0x%x\n",msg->message,msg->wParam);
   if (msg->message == WM_CHAR || msg->message == WM_SYSCHAR){
     setIsPressed(True);
+dbprintf("---- msg=0x%x code=0x%x\n",msg->message,msg->wParam);
 //printf("here001 code=0x%x\n",msg->wParam);
-    if (GetKeyState(VK_CONTROL) <0){
+//--
+    // We need to replace the next line
+    //if (GetKeyState(VK_CONTROL) <0){
+    if (GetKeyState(VK_CONTROL) <0 && GetKeyState(VK_RMENU) > -1){
+//--
       if (msg->message == WM_CHAR ){
 //printf("here001 return..1\n");
         return True;
@@ -258,8 +351,19 @@ WSCbool WSDwinKeyboard::_dispatch_key(MSG* msg){
         msg->wParam != VK_RETURN &&
         msg->wParam != VK_ESCAPE ){
       if (_pos == 0){
+#ifdef _WSWINCE
+        WSCushort ubuf[2];
+        ubuf[0] =(WSCushort)msg->wParam;
+        ubuf[1] = 0;
+        char* str = WSGFgetString(ubuf,WS_EN_UTF8);
+        strcpy(&_buf[_pos],str);
+//        _pos += strlen(str);
+dbprintf("---- string=#%s#\n",str);
+        delete str;
+#else
         _buf[_pos] = (char)msg->wParam;
         _buf[_pos+1] = 0;
+#endif
         if ((((msg->wParam >= 0x80 && msg->wParam < 0xa0 ) ||
              msg->wParam > 0xdf) && 
         WSGIappLocaleSet()->getSystemLocaleEncoding()==WS_EN_SJIS) ||
@@ -272,10 +376,17 @@ WSCbool WSDwinKeyboard::_dispatch_key(MSG* msg){
           setKey(0);
           return True;
         }else{
+          setKey(0);
 WSMFtrace("key pressed key=#%s#  XXZZk1 xx\n",_buf);
-//printf("key pressed key=#%s#  XXZZk1 xx\n",_buf);
+#ifdef _WSWINCE
+          setText(_buf,WS_EN_UTF8);
+#else
           setText(_buf,WS_EN_LOCALE);
+#endif
           _is_tty_function = False;
+          if (_buf[0] == ' '){
+            setKey(WSK_space);
+          }else
           if (_buf[0] == '!'){
             setKey(WSK_exclam);
           }else
@@ -391,22 +502,43 @@ WSMFtrace("key pressed key=#%s#  XXZZk1 xx\n",_buf);
             _is_cursor = False;
           }
 #endif
+//--
+//
+// We need to modificate this code
+/*
           if (msg->wParam > VK_F1 -1 && msg->wParam < VK_F12 +1){
             _is_function = True;
           }else{
             _is_function = False;
           }
+*/
+         if (GetKeyState(VK_RMENU) > -1){
+            if (msg->wParam > VK_F1 -1 && msg->wParam < VK_F12 +1){
+              _is_function = True;
+            }else{
+              _is_function = False;
+            }
+         }else{
+            _is_function = False;
+         }
+//--
         }
       }else{
-//        if(getEnableFep() != False){
+#ifdef _WSWINCE
+        WSCushort ubuf[2];
+        ubuf[0] =(WSCushort)msg->wParam;
+        ubuf[1] = 0;
+        char* str = WSGFgetString(ubuf,WS_EN_UTF8);
+        strcpy(&_buf[_pos],str);
+        _pos += strlen(str);
+dbprintf("---- string=#%s# #%s#\n",str);
+        delete str;
+        setText(_buf,WS_EN_UTF8);
+#else
           _buf[_pos] = (char)msg->wParam;
           _buf[_pos+1] = 0;
-//        }else{
-//          _buf[0] = 0;
-//        }
-WSMFtrace("key pressed key=#%s#  XXZZk1 xx2\n",_buf);
-//printf("key pressed2 key=#%s#  XXZZk1 xx2 pos=%d\n",_buf,_pos);
         setText(_buf,WS_EN_LOCALE);
+#endif
         _pos = 0;
         _is_tty_function = False;
         _is_cursor = False;
@@ -417,7 +549,11 @@ WSMFtrace("key pressed getkey=0x%x  XXZZk1\n",getKey());
       setKey((long) msg->wParam);
 #endif      
       _with_cntl = False;
-      if (GetKeyState(VK_MENU) < 0){
+//--
+      // We need to change the next line
+      //if (GetKeyState(VK_MENU) < 0){
+      if ((GetKeyState(VK_MENU) < 0) && (GetKeyState(VK_RMENU) > -1)){
+//--
         _with_alt = True;
       }else{
         _with_alt = False;
@@ -454,6 +590,7 @@ WSMFtrace("key pressed key=#%s#  XXZZk12\n",_buf);
            msg->wParam == VK_RETURN ||
            msg->wParam == VK_SHIFT ||
            msg->wParam == VK_MENU ||
+           msg->wParam == VK_APPS ||
            msg->wParam == VK_ESCAPE ||
            msg->wParam == VK_INSERT ||
            msg->wParam == VK_DELETE ||
@@ -597,7 +734,11 @@ WSMFtrace("key pressed key=#%s#  XXZZk12\n",_buf);
 WSMFtrace("key pressed setkey=0x%x  XXZZk12\n",msg->wParam);
 //printf("key pressed executed2.. setkey=0x%x  #%s# encode=%d\n",msg->wParam,_buf,WSGIappLocaleSet()->getSystemLocaleEncoding());
       _buf[0] = 0;
+#ifdef _WSWINCE
+      setText(_buf,WS_EN_UTF8);
+#else
       setText(_buf,WS_EN_LOCALE);
+#endif
       _with_cntl = True;
       if (GetKeyState(VK_MENU) < 0){
         _with_alt = True;
@@ -618,6 +759,8 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk12\n",msg->wParam);
           msg->wParam == VK_UP || 
           msg->wParam == VK_DOWN ){
         _is_cursor = True;
+      }else{
+        _is_cursor = False;
       }
       if (msg->wParam > VK_F1 -1 && msg->wParam < VK_F12 +1){
         _is_function = True;
@@ -645,6 +788,7 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk12\n",msg->wParam);
            msg->wParam == VK_RETURN ||
            msg->wParam == VK_SHIFT ||
            msg->wParam == VK_MENU ||
+           msg->wParam == VK_APPS ||
            msg->wParam == VK_ESCAPE ||
            msg->wParam == VK_INSERT ||
            msg->wParam == VK_DELETE ||
@@ -660,6 +804,8 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk12\n",msg->wParam);
            msg->wParam == VK_NUMLOCK ||
            msg->wParam == VK_SCROLL ||
            msg->wParam == VK_SNAPSHOT ||
+           msg->wParam == VK_LWIN ||
+           msg->wParam == VK_RWIN ||
            (msg->wParam > VK_F1 -1 && msg->wParam < VK_F12 +1)){
         long key1 = (long)msg->wParam;
 //printf("key2=0x%x %d\n",key1,key1);
@@ -667,7 +813,11 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk12\n",msg->wParam);
   
 WSMFtrace("key pressed setkey=0x%x  XXZZk14\n",msg->wParam);
         _buf[0] = 0;
+#ifdef _WSWINCE
+        setText(_buf,WS_EN_UTF8);
+#else
         setText(_buf,WS_EN_LOCALE);
+#endif
         _with_cntl = False;
         if (GetKeyState(VK_MENU) < 0){
           _with_alt = True;
@@ -688,6 +838,8 @@ WSMFtrace("key pressed setkey=0x%x  XXZZk14\n",msg->wParam);
             msg->wParam == VK_UP || 
             msg->wParam == VK_DOWN ){
           _is_cursor = True;
+        }else{
+          _is_cursor = False;
         }
         if (msg->wParam > VK_F1 -1 && msg->wParam < VK_F12 +1){
           _is_function = True;
@@ -992,11 +1144,19 @@ WSMFtrace("key pressed XXZZk13 0x%x\n",getKey());
     }
     if (GetKeyState(VK_MENU) < 0){
       _with_alt = True;
+#ifdef _WSWINCE
+      setText(_buf,WS_EN_UTF8);
+#else
       setText(_buf,WS_EN_LOCALE);
+#endif
     }else{
       _with_alt = False;
       _buf[0] = 0;
+#ifdef _WSWINCE
+      setText(_buf,WS_EN_UTF8);
+#else
       setText(_buf,WS_EN_LOCALE);
+#endif
     }
     if (GetKeyState(VK_SHIFT) < 0){
       _with_shift = True;
@@ -1110,3 +1270,48 @@ char* WSDwinKeyboard::getSelectedString(long encode){
     return "";
   }
 }
+void WSDwinKeyboard::updateFepStatus(){
+  WSCbool enable = getEnableFep();
+  if (_himc != 0){
+    BOOL ret = ImmGetOpenStatus(_himc);
+    if (ret == True && enable != False){
+      _fep_on = True;
+    }else
+    if (ret == True && enable == False){
+      setFepStatus(WS_FEP_STOP);
+      _fep_on = False;
+    }else{
+      if (enable == False){
+        setFepStatus(WS_FEP_STOP);
+      }
+      _fep_on = False;
+    }
+  }else{
+    if (enable == False){
+      setFepStatus(WS_FEP_STOP);
+    }
+    _fep_on = False;
+  }
+}
+void WSDwinKeyboard::setEnableFep(WSDdev* dev,WSCbool fl){
+  WSDkeyboard::setEnableFep(dev,fl);
+  if (fl == False){
+    setFepStatus(WS_FEP_STOP);
+#if 0
+    if (_himc != 0){
+      ImmSetOpenStatus(_himc, FALSE);
+      ImmDestroyContext(_himc);
+      _himc = 0;
+    }
+#endif
+  }
+}
+//HIMC WSDwinKeyboard::getHimc(){
+//  return _himc;
+//}
+void WSDwinKeyboard::setHwnd(HWND wnd){
+  _hwnd = wnd;
+  if (_himc != 0){
+    ImmAssociateContext(_hwnd,_himc);
+  } 
+}
