@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+//
 
 #include <win/WSDwinMouse.h>
 #include <win/WSDwinAppDev.h>
@@ -39,6 +36,8 @@ WSDmouse* _winMouse_create_handler(){
   return new WSDwinMouse();
 }
 static long _init_mouse = 0;
+
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _winMouse_init{
   public: _winMouse_init(){
     _init_mouse = 1;
@@ -47,6 +46,7 @@ class _winMouse_init{
 };
 
 _winMouse_init _winMouse_init_run_constructor;
+#endif
 
 #ifdef _WSWINCE
 int _mouse_pos_x =0;
@@ -69,6 +69,7 @@ WSDwinMouse* WSGIwinAppMouse(){
 
 char* dmnames[]={
 "0   IDC_ARROW",
+"1   IDC_NONE",
 "195 IDC_SIZENWSE",
 "197 IDC_SIZENS",
 "196 IDC_SIZENESW",
@@ -80,6 +81,7 @@ char* dmnames[]={
 "154 IDC_SIZEALL",
 "186 IDC_SIZENS",
 "182 IDC_SIZEWE",
+"203 IDC_WAIT",
 NULL
 };
 
@@ -118,7 +120,7 @@ WSMFtrace("WSDwinMouse::setupMouseCursor %s\n",data);
   sscanf(data,"%d %s",&id,img1);
   if (!strncmp(img1,"IDC_",4)){
 #ifndef _WSWINCE
-    char* mstr;
+    char* mstr = NULL;
 #else
     WSCushort* mstr;
 #endif
@@ -163,6 +165,10 @@ WSMFtrace("WSDwinMouse::setupMouseCursor %s\n",data);
     }else
     if (!strcmp("IDC_WAIT",img1)){
       mstr =IDC_WAIT;
+    }else
+    if (!strcmp("IDC_NONE",img1)){
+      addMouseCursor(id,(long)-1,data);
+      return;
     }else{
       mstr =IDC_ARROW;
     }
