@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+//
 
 #include <win/WSDwinPrivateTimer.h>
 #include <win/WSDwinAppDev.h>
@@ -33,6 +30,7 @@ dbprintf("_winPrivatetimer_create.. start/done %s:%d\n",__FILE__,__LINE__);
   return new WSDwinPrivateTimer;
 }
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _winPrivatetimer_init {
   public: _winPrivatetimer_init(){
 _dbprintf_init();
@@ -40,9 +38,8 @@ dbprintf("_winPrivatetimer_init global contructor.. start/done %s:%d \n",__FILE_
     WSDprivateTimer::setCreateInstanceHandler((void*)_winPrivatetimer_create);
   };
 };
-
 static _winPrivatetimer_init  _winPrivatetimer_init_execute;
-
+#endif
 
 WSDwinPrivateTimer::WSDwinPrivateTimer(){
 WSMFtrace("WSDwinPrivateTimer 0x%x constructed.. %s:%d\n",this,__FILE__,__LINE__);
