@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+//
 
 #include <win/WSDwinScrFrameDev.h>
 #include <win/WSDwinappDev.h>
@@ -38,12 +35,14 @@ WSDdev* _wsdxscrdev_init_(){
   return new WSDwinScrFrameDev();
 }
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _WSDwinScrFrameDev_init_ {
   public:_WSDwinScrFrameDev_init_(){
      WSGIappDevice()->setCreateHandler("scrFrameDev",_wsdxscrdev_init_);
   };
 };
 static _WSDwinScrFrameDev_init_ _init_to_run_WSDwinScrFrameDev_;
+#endif
 
 WSDwinScrFrameDev::WSDwinScrFrameDev(){
   _scr_wnd = 0;
@@ -167,11 +166,13 @@ WSMFtrace(" -> %d  w=0x%d\n",p.top,p.bottom - p.top);
 
                if (_wnd != NULL){
                  GetClientRect(_wnd,&p);
-
+#ifndef _WSWINCE
                  MoveWindow(_wnd,-_h_scr_pos,-_v_scr_pos,_work_width,_work_height, True);
-//                 MoveWindow(_wnd,-_h_scr_pos,-_v_scr_pos,_work_width,_work_height, False);
                  exposeArea(_h_scr_pos,_v_scr_pos,_w_w,_w_h,False);
                  _expose_children(_h_scr_pos,_v_scr_pos,_w_w,_w_h);
+#else
+                 MoveWindow(_wnd,-_h_scr_pos,-_v_scr_pos,_work_width,_work_height, True);
+#endif
                }
              }
              break;
@@ -191,11 +192,14 @@ WSMFtrace(" -> %d  w=0x%d\n",p.top,p.bottom - p.top);
                }
                if (_wnd != NULL){
                  GetClientRect(_wnd,&p);
+#ifndef _WSWINCE
                  MoveWindow(_wnd,-_h_scr_pos,-_v_scr_pos,_work_width,_work_height, True);
-//                 MoveWindow(_wnd,-_h_scr_pos,-_v_scr_pos,_work_width,_work_height, False);
-//  invalidate(True);
                  exposeArea(_h_scr_pos,_v_scr_pos,_w_w,_w_h,False);
                  _expose_children(_h_scr_pos,_v_scr_pos,_w_w,_w_h);
+#else
+                 MoveWindow(_wnd,-_h_scr_pos,-_v_scr_pos,_work_width,_work_height, True);
+#endif
+//  invalidate(True);
                }
              }
              break;
@@ -215,7 +219,11 @@ WSMFtrace(" -> %d  w=0x%d\n",p.top,p.bottom - p.top);
              }
              if (_wnd != NULL){
                GetClientRect(_wnd,&p);
+#ifndef _WSWINCE
                MoveWindow(_wnd,-_h_scr_pos,-_v_scr_pos,_work_width,_work_height, True);
+#else
+               MoveWindow(_wnd,-_h_scr_pos,-_v_scr_pos,_work_width,_work_height, False);
+#endif
              }
              break;
     case WSDEV_WORK_HEIGHT:
@@ -234,7 +242,11 @@ WSMFtrace(" -> %d  w=0x%d\n",p.top,p.bottom - p.top);
              }
              if (_wnd != NULL){
                GetClientRect(_wnd,&p);
+#ifndef _WSWINCE
                MoveWindow(_wnd,-_h_scr_pos,-_v_scr_pos,_work_width,_work_height, True);
+#else
+               MoveWindow(_wnd,-_h_scr_pos,-_v_scr_pos,_work_width,_work_height, False);
+#endif
              }
              break;
     case WSDEV_PIXMAP_STYLE:
@@ -249,10 +261,15 @@ WSMFtrace(" -> %d  w=0x%d\n",p.top,p.bottom - p.top);
              if (_wnd == NULL){
                break;
              }
-             _grabed = *(WSCbool*)data;
-             if (_grabed != False){
+             WSGIwinAppDev()->_grabed = *(WSCbool*)data;
+             if (WSGIwinAppDev()->_grabed != False){
                WSGIwinAppDev()->setGrabedWnd(_wnd);
+               WSGIappMouse()->setMouseFocusClient(NULL);
              }else{
+               long state = WSGIappMouse()->getStatus();
+               if (state > 0){
+                 break;
+               }
                WSGIwinAppDev()->setGrabedWnd(NULL);
              }
 #if 0
@@ -319,7 +336,7 @@ extern char szAppName[];
   _pwnd = windev->getHWND();
 WSMFtrace("CreateWindow  x,y,w,h=%d,%d,%d,%d\n",x,y,w,h);
 #ifndef _WSWINCE
-  _scr_wnd = CreateWindow( szAppName,"",
+  _scr_wnd = WSGFcreateWindow( szAppName,"",
                 WS_CHILD | WS_OVERLAPPED | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
                 x,y,w,h,_pwnd,
                 NULL,WSGIwinAppDev()->getInstanceHandle(),NULL);
@@ -349,7 +366,7 @@ WSMFtrace("CreateWindow  x,y,w,h=%d,%d,%d,%d\n",x,y,w,h);
 #endif
 
 #ifndef _WSWINCE
-  _wnd = CreateWindow( szAppName,"",
+  _wnd = WSGFcreateWindow( szAppName,"",
                 WS_CHILD | WS_OVERLAPPED | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
                 -_h_scr_pos,-_v_scr_pos,_work_width,_work_height,_scr_wnd,
                 NULL,WSGIwinAppDev()->getInstanceHandle(),NULL);
@@ -557,11 +574,12 @@ long WSDwinScrFrameDev::setVisible(WSCbool fl){
   return WS_NO_ERR;
 }
 
-void WSDwinScrFrameDev::_evh_delete(WinEvent* ptr){
+long WSDwinScrFrameDev::_evh_delete(WinEvent* ptr){
   _pwnd = 0;
   _scr_wnd = 0;
   _wnd = 0;
   setInitialized(False);
+  return 0;
 }
 void WSDwinScrFrameDev::_evh_resize(WinEvent* ev){
 //  if (_resized == 1){
