@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <win/WSDwinThread.h>
@@ -38,13 +34,14 @@ WSDthread* _win_th_chandler(){
 void WSGFinitializeWinThread(){
     WSDthread::setCreateInstanceHandler((void*)_win_th_chandler);
 }
-
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _win_thread_init {
   public: _win_thread_init(){
-    WSDthread::setCreateInstanceHandler((void*)_win_th_chandler);
+    WSGFinitializeWinThread();
   };
 };
 _win_thread_init  _win_th_init_execute;
+#endif
 
 WSDwinThread::WSDwinThread(){
   _thr = NULL;
@@ -124,11 +121,32 @@ long WSDwinThread::getStatus(){
   return WS_THREAD_RUNNING;
 }
 long WSDwinThread::execCallback(void* data){
-  _callback_data = data;
+//  _callback_data = data;
   DWORD result;
-  SendMessageTimeout(WSGIwinAppDev()->getHWND(),
-                     WM_USER,(WPARAM)WS_WIN_THR,(LPARAM)this,
+  _internal_thr_data_* ptr = new _internal_thr_data_();  
+  ptr->thr = this;
+  ptr->data = data;
+  // Changed from SMTO_NORMAL to SMTO_BLOCK because recursive call generates
+  // some resource error.
+#ifndef _WSWINCE
+  LRESULT ret = SendMessageTimeout(WSGIwinAppDev()->getHWND(),
+                     WM_USER,(WPARAM)WS_WIN_THR,(LPARAM)ptr,
+                     SMTO_BLOCK,1000,&result);
+#else
+  LRESULT ret = SendMessageTimeout(WSGIwinAppDev()->getHWND(),
+                     WM_USER,(WPARAM)WS_WIN_THR,(LPARAM)ptr,
                      SMTO_NORMAL,1000,&result);
+#endif
+  if (ret == 0){
+//    if (ptr != NULL){
+//      delete ptr;
+//    }
+    if (GetLastError() == 1460){ //1460 is ERROR_TIMEOUT
+      return WS_ERR_TIMEDOUT;
+    }else{
+      return WS_ERR;
+    }
+  }
   return WS_NO_ERR;
 }
 
