@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+//
 
 #include <windows.h>
 #include <win/WSDwindraw.h>
@@ -86,6 +83,8 @@ WSDwindraw::WSDwindraw(){
   _clipping = False;
   _dash_no = PS_SOLID;
   _printer = False;
+  _underline_char_pos = 0;
+  _underline_char_len = 0;
 }
 WSDwindraw::~WSDwindraw(){}
 WSCushort _pr_x;
@@ -343,9 +342,11 @@ long WSDwindraw::setHatchPattern(char type){
   return bk;
 }
 long WSDwindraw::beginDraw(WSDdev* dev,long val1,long val2,long val3){
-
+  _underline_char_pos = 0;
+  _underline_char_len = 0;
   _dev = dev;
-
+  _xor_mdc = 0;
+  _operation = WS_OPERATION_SRC_COPY;
   _fg_color = 0;
   _bg_color = 0;
 //printf("windraw::beginDraw hdc=0x%x\n",val3);
@@ -379,6 +380,18 @@ long WSDwindraw::endDraw(){
   if (_hdc == 0){
     return WS_ERR;
   }
+  if (_xor_mdc != 0){
+    if (_dev != NULL){
+      WSCushort w=1;
+      WSCushort h=1;
+      _dev->getWH(&w,&h);
+      BitBlt(_hdc_bk,0,0,w,h,_xor_mdc,0,0,SRCINVERT);
+    }
+    DeleteDC(_xor_mdc);
+    _hdc = _hdc_bk;
+    _hdc_bk = 0;
+    _xor_mdc = 0;
+  }
   if (_hdc == 0 && _palette != 0){
     SelectPalette(_hdc,_palette_bk,False);
   }
@@ -438,10 +451,23 @@ long WSDwindraw::drawArc(short x,short y,WSCushort w,WSCushort h,short a1,short
   HPEN pen_bk = (HPEN)SelectObject(_hdc,(void*)pen);
   SetBkMode(_hdc,TRANSPARENT);
 
-  double vx1 = x + w/2 + cos(3.141592 *a1/64.0/180.0)*w/2;
-  double vy1 = y + h/2 - sin(3.141592 *a1/64.0/180.0)*h/2;
-  double vx2 = x + w/2 + cos(3.141592 *(a1+a2)/64.0/180.0)*w/2;
-  double vy2 = y + h/2 - sin(3.141592 *(a1+a2)/64.0/180.0)*h/2;
+  double w1 = w;
+  double h1 = h;
+  if ((a2 > 0 && a2 < 64) ||
+      (a2 < 0 && a2 > -64)){
+    if (w < 1000 && h < 1000){
+      w1 = w*10;
+      h1 = h*10;
+    }
+    if (w < 100 && h < 100){
+      w1 = w*10;
+      h1 = h*10;
+    }
+  }
+  double vx1 = x + w/2 + cos(3.141592 *a1/64.0/180.0)*w1;
+  double vy1 = y + h/2 - sin(3.141592 *a1/64.0/180.0)*h1;
+  double vx2 = x + w/2 + cos(3.141592 *(a1+a2)/64.0/180.0)*w1;
+  double vy2 = y + h/2 - sin(3.141592 *(a1+a2)/64.0/180.0)*h1;
   Arc(_hdc,x,y,x+w,y+h, (int)vx1,(int)vy1,(int)vx2,(int)vy2);
   SetBkMode(_hdc,OPAQUE);
   SelectObject(_hdc,(void*)pen_bk);
@@ -475,18 +501,23 @@ long WSDwindraw::drawArc(short x,short y,WSCushort w,WSCushort h,short a1,short
       circle = True;
       an2 = 360;
     }
-#define WS_PI_VALUE 3.14159265358979323846
-    long points = (w+h)*2 * WS_PI_VALUE * an2 / 360;
-//  WSCpoint point[WS_PT_NUM+1];
+#define WS_PI_VALUE1 (3.14159265358979323846/360)
+#define WS_PI_VALUE2 (3.14159265358979323846/180)
+  long points = (long)((w+h)*2 * WS_PI_VALUE1 * an2);
+
     WSCpoint* point = new WSCpoint[points+1];
 
     long cnt = 0;
     int i;
+    double aa1 = WS_PI_VALUE2 * an1;
+    double aa2 = WS_PI_VALUE2 * an2 / (points -1);
+    double w2 = w/2;
+    double h2 = h/2;
+
     for(i=0; i<points; i++){
-      point[cnt].x = (long)(x + (double)w/2 +
-           w * cos(WS_PI_VALUE * (an1 + (an2* i)/(points-1))  / 180)/2 +0.5);
-      point[cnt].y = (long)(y + (double)h/2 -
-           h * sin(WS_PI_VALUE * (an1 + (an2* i)/(points-1))  / 180)/2 +0.5);
+      double aa = aa1 + aa2*i;
+      point[cnt].x = x + (long)(w2 * (1 + cos(aa)) + 0.5);
+      point[cnt].y = y + (long)(h2 * (1 - sin(aa)) + 0.5);
       if (cnt > 0){
         if (point[cnt].x == point[cnt-1].x &&
             point[cnt].y == point[cnt-1].y){
@@ -550,10 +581,23 @@ long WSDwindraw::drawFillArc(short x,short y,WSCushort w,WSCushort h,short a1,sh
   HBRUSH brush_bk = (HBRUSH)SelectObject(_hdc,(void*)brush);
 
   SetBkMode(_hdc,TRANSPARENT);
-  double vx1 = x + w/2 + cos(3.141592 *a1/64.0/180.0)*w/2;
-  double vy1 = y + h/2 - sin(3.141592 *a1/64.0/180.0)*h/2;
-  double vx2 = x + w/2 + cos(3.141592 *(a1+a2)/64.0/180.0)*w/2;
-  double vy2 = y + h/2 - sin(3.141592 *(a1+a2)/64.0/180.0)*h/2;
+  double w1 = w;
+  double h1 = h;
+  if ((a2 > 0 && a2 < 64) ||
+      (a2 < 0 && a2 > -64)){
+    if (w < 1000 && h < 1000){
+      w1 = w*10;
+      h1 = h*10;
+    }
+    if (w < 100 && h < 100){
+      w1 = w*10;
+      h1 = h*10;
+    }
+  }
+  double vx1 = x + w/2 + cos(3.141592 *a1/64.0/180.0)*w1;
+  double vy1 = y + h/2 - sin(3.141592 *a1/64.0/180.0)*h1;
+  double vx2 = x + w/2 + cos(3.141592 *(a1+a2)/64.0/180.0)*w1;
+  double vy2 = y + h/2 - sin(3.141592 *(a1+a2)/64.0/180.0)*h1;
   if (kind == WS_PI){
     Pie(_hdc,x,y,x+w,y+h, (int)vx1,(int)vy1,(int)vx2,(int)vy2);
   }else{
@@ -576,17 +620,21 @@ long WSDwindraw::drawFillArc(short x,short y,WSCushort w,WSCushort h,short a1,sh
     an2 = 360;
   }
 //#define WS_PT_NUM 36
-  long points = (w+h)*2 * WS_PI_VALUE * an2 / 360;
-//  WSCpoint point[WS_PT_NUM+1];
+  long points = (long)((w+h)*2 * WS_PI_VALUE1 * an2);
   WSCpoint* point = new WSCpoint[points+1];
 
   long cnt = 0;
   int i;
+  double aa1 = WS_PI_VALUE2 * an1;
+  double aa2 = WS_PI_VALUE2 * an2 / (points -1);
+  double w2 = w/2;
+  double h2 = h/2;
+
   for(i=0; i<points; i++){
-    point[cnt].x = (long)(x + (double)w/2 +
-           w * cos(WS_PI_VALUE * (an1 + (an2* i)/(points-1))  / 180)/2 +0.5);
-    point[cnt].y = (long)(y + (double)h/2 -
-           h * sin(WS_PI_VALUE * (an1 + (an2* i)/(points-1))  / 180)/2 +0.5);
+    double aa = aa1 + aa2*i;
+    point[cnt].x = x + (long)(w2 * (1 + cos(aa)) + 0.5);
+    point[cnt].y = y + (long)(h2 * (1 - sin(aa)) + 0.5);
+
     if (cnt > 0){
       if (point[cnt].x == point[cnt-1].x &&
           point[cnt].y == point[cnt-1].y){
@@ -1070,7 +1118,7 @@ long WSDwindraw::drawFillPoly(WSCpoint* points,long num){
 }
 
 void _draw_stretch_image_with_mask(HDC hdc,HDC mdc,HDC mdc2,HBITMAP pix,HBITMAP pix2,HPALETTE pl,
-                                   int x,int y,int w,int h, int sx,int sy,int iw,int ih){
+                                   int x,int y,int w,int h, int sx,int sy,int iw,int ih,WSCuchar op){
       HDC mdc3 = CreateCompatibleDC(hdc);
       HPALETTE plbk3 = SelectPalette(mdc3,pl,FALSE);
       RealizePalette(mdc3);
@@ -1102,17 +1150,32 @@ void _draw_stretch_image_with_mask(HDC hdc,HDC mdc,HDC mdc2,HBITMAP pix,HBITMAP
       GetDIBits(mdc2,pix2,0,ih,(void*)buf2,&bmi,DIB_RGB_COLORS);
       GetDIBits(mdc3,pix3,0,ih,(void*)buf1,&bmi,DIB_RGB_COLORS);
       int i,j;
-      for(i=0; i<ih; i++){
-        long ptr = i*bytes_per_line;
-        for(j=0; j<bytes_per_line; j++){
-          long val = buf1[ptr] * buf2[ptr] /255 + buf3[ptr] * (255 - buf2[ptr]) /255;
-          buf1[ptr] = val;
-          ptr++;
+      if (op != WS_OPERATION_XOR){
+        for(i=0; i<ih; i++){
+          long ptr = i*bytes_per_line;
+          for(j=0; j<bytes_per_line; j++){
+            long val = buf1[ptr] * buf2[ptr] /255 + buf3[ptr] * (255 - buf2[ptr]) /255;
+            buf1[ptr] = val;
+            ptr++;
+          }
+        }
+      }else{
+        for(i=0; i<ih; i++){
+          long ptr = i*bytes_per_line;
+          for(j=0; j<bytes_per_line; j++){
+            long val = buf1[ptr] * buf2[ptr] /255;
+            buf1[ptr] = val;
+            ptr++;
+          }
         }
       }
-      SetDIBits(mdc3,pix3,0,ih,(void*)buf1,&bmi,DIB_RGB_COLORS);
 
-      StretchBlt(hdc,x,y,w,h,mdc3,0,0,iw,ih,SRCCOPY);
+      SetDIBits(mdc3,pix3,0,ih,(void*)buf1,&bmi,DIB_RGB_COLORS);
+      if (op != WS_OPERATION_XOR){
+        StretchBlt(hdc,x,y,w,h,mdc3,0,0,iw,ih,SRCCOPY);
+      }else{
+        StretchBlt(hdc,x,y,w,h,mdc3,0,0,iw,ih,PATINVERT);
+      }
 
       SelectObject(mdc3,pxbk3);
       DeleteObject(pix3);
@@ -1239,18 +1302,26 @@ long WSDwindraw::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDi
         SetMapMode(mdc2,GetMapMode(mdc));
 #endif
         StretchBlt(mdc2,0,0,w,h,mdc,0,0,iw,ih,SRCCOPY);
-        int ret = StretchBlt(_hdc,x,y,w,h,mdc2,0,0,w,h,SRCCOPY);
-dbprintf("WSDwindraw::drawStretchedImage _hdc=0x%x mdc=0x%x pr ret=%d %d,%d,%d,%d<-%d,%d \n",_hdc,mdc,ret,x,y,w,h,iw,ih);
+        if (_operation != WS_OPERATION_XOR){
+          StretchBlt(_hdc,x,y,w,h,mdc2,0,0,w,h,SRCCOPY);
+        }else{
+          StretchBlt(_hdc,x,y,w,h,mdc2,0,0,w,h,PATINVERT);
+        }
+//dbprintf("WSDwindraw::drawStretchedImage _hdc=0x%x mdc=0x%x pr ret=%d %d,%d,%d,%d<-%d,%d \n",_hdc,mdc,ret,x,y,w,h,iw,ih);
         SelectObject(mdc2,pxbk2);
         DeleteObject(pix2);
         DeleteDC(mdc2);
       }else{
-        int ret = StretchBlt(_hdc,x,y,w,h,mdc,0,0,iw,ih,SRCCOPY);
-dbprintf("WSDwindraw::drawStretchedImage _hdc=0x%x mdc=0x%x ret=%d %d,%d,%d,%d<-%d,%d \n",_hdc,mdc,ret,x,y,w,h,iw,ih);
+        if (_operation != WS_OPERATION_XOR){
+          StretchBlt(_hdc,x,y,w,h,mdc,0,0,iw,ih,SRCCOPY);
+        }else{
+          StretchBlt(_hdc,x,y,w,h,mdc,0,0,iw,ih,PATINVERT);
+        }
+//dbprintf("WSDwindraw::drawStretchedImage _hdc=0x%x mdc=0x%x ret=%d %d,%d,%d,%d<-%d,%d \n",_hdc,mdc,ret,x,y,w,h,iw,ih);
       }
     }else{
       _draw_stretch_image_with_mask(_hdc,mdc,mdc2,pix,pix2,pl,x,y,w,h,
-                                                            0,0,iw,ih);
+                                                            0,0,iw,ih,_operation);
     }
     if (pl != 0){
       SelectPalette(mdc,plbk,FALSE);
@@ -1395,11 +1466,15 @@ dbprintf("WSDwindraw::drawImage _pr_scale_x=%f _pr_scale_y=%f\n",_pr_scale_x,_pr
 #ifndef _WSWINCE
         SetMapMode(mdc2,GetMapMode(mdc));
 #endif
-        int ret = StretchBlt(mdc2,0,0,dw,dh,mdc,dx -x -xoff,dy -y -yoff,iw2,ih2,SRCCOPY);
-dbprintf("WSDwindraw::drawImage _hdc=0x%x mdc=0x%x ret=%d %d,%d,%d,%d->%d,%d err=0x%x err=%d\n",_hdc,mdc,ret,dx2,dy2,dw,dh,dx -x -xoff,dy-y-yoff,GetLastError(),GetLastError());
-        ret = BitBlt(_hdc,dx2,dy2,dw,dh,mdc2,0,0,SRCCOPY);
+        StretchBlt(mdc2,0,0,dw,dh,mdc,dx -x -xoff,dy -y -yoff,iw2,ih2,SRCCOPY);
+//dbprintf("WSDwindraw::drawImage _hdc=0x%x mdc=0x%x ret=%d %d,%d,%d,%d->%d,%d err=0x%x err=%d\n",_hdc,mdc,ret,dx2,dy2,dw,dh,dx -x -xoff,dy-y-yoff,GetLastError(),GetLastError());
+        if (_operation != WS_OPERATION_XOR){
+          BitBlt(_hdc,dx2,dy2,dw,dh,mdc2,0,0,SRCCOPY);
+        }else{
+          BitBlt(_hdc,dx2,dy2,dw,dh,mdc2,0,0,PATINVERT);
+        }
 //        int ret = StretchBlt(_hdc,dx2,dy2,dw,dh,mdc,dx -x -xoff,dy-y-yoff,iw2,ih2,SRCCOPY);
-dbprintf("WSDwindraw::drawImage _hdc=0x%x mdc=0x%x st ret=%d %d,%d,%d,%d->%d,%d err=0x%x err=%d\n",_hdc,mdc,ret,dx2,dy2,dw,dh,dx -x -xoff,dy-y-yoff,GetLastError(),GetLastError());
+//dbprintf("WSDwindraw::drawImage _hdc=0x%x mdc=0x%x st ret=%d %d,%d,%d,%d->%d,%d err=0x%x err=%d\n",_hdc,mdc,ret,dx2,dy2,dw,dh,dx -x -xoff,dy-y-yoff,GetLastError(),GetLastError());
         SelectObject(mdc2,pxbk2);
         DeleteObject(pix2);
         DeleteDC(mdc2);
@@ -1411,12 +1486,16 @@ dbprintf("WSDwindraw::drawImage _hdc=0x%x mdc=0x%x st ret=%d %d,%d,%d,%d->%d,%d
 //        BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
 
         _draw_stretch_image_with_mask(_hdc,mdc,mdc2,pix,pix2,pl,dx,dy,dw,dh,
-                                      dx -x -xoff, dy -y -yoff,iw2,ih2);
+                                      dx -x -xoff, dy -y -yoff,iw2,ih2,_operation);
       }
     }else{
       if (pix2 == 0){
-        int ret = BitBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy -y -yoff,SRCCOPY);
-dbprintf("WSDwindraw::drawImage _hdc=0x%x mdc=0x%x ret=%d %d,%d,%d,%d->%d,%d \n",_hdc,mdc,ret,dx,dy,dw,dh,dx -x -xoff,dy-y-yoff);
+        if (_operation != WS_OPERATION_XOR){
+          BitBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy -y -yoff,SRCCOPY);
+        }else{
+          BitBlt(_hdc,dx,dy,dw,dh,mdc,dx -x -xoff,dy -y -yoff,PATINVERT);
+        }
+//dbprintf("WSDwindraw::drawImage _hdc=0x%x mdc=0x%x ret=%d %d,%d,%d,%d->%d,%d \n",_hdc,mdc,ret,dx,dy,dw,dh,dx -x -xoff,dy-y-yoff);
       }else{
         HDC mdc3 = CreateCompatibleDC(_hdc);
         HPALETTE plbk3 = SelectPalette(mdc3,pl,FALSE);
@@ -1449,19 +1528,31 @@ dbprintf("WSDwindraw::drawImage _hdc=0x%x mdc=0x%x ret=%d %d,%d,%d,%d->%d,%d \n"
         GetDIBits(mdc2,pix2,0,ph,(void*)buf2,&bmi,DIB_RGB_COLORS);
         GetDIBits(mdc,pix,0,ph,(void*)buf3,&bmi,DIB_RGB_COLORS);
         int i,j;
-        for(i=0; i<ph; i++){
-          long ptr = i*bytes_per_line;
-          for(j=0; j<bytes_per_line; j++){
-            long val = buf1[ptr] * buf2[ptr] /255 + buf3[ptr] * (255 - buf2[ptr]) /255;
-            buf1[ptr] = val;
-            ptr++;
+        if (_operation != WS_OPERATION_XOR){
+          for(i=0; i<ph; i++){
+            long ptr = i*bytes_per_line;
+            for(j=0; j<bytes_per_line; j++){
+              long val = buf1[ptr] * buf2[ptr] /255 + buf3[ptr] * (255 - buf2[ptr]) /255;
+              buf1[ptr] = val;
+              ptr++;
+            }
+          }
+        }else{
+          for(i=0; i<ph; i++){
+            long ptr = i*bytes_per_line;
+            for(j=0; j<bytes_per_line; j++){
+              long val = buf1[ptr] * buf2[ptr] /255;
+              buf1[ptr] = val;
+              ptr++;
+            }
           }
         }
-
         SetDIBits(mdc3,pix3,0,ph,(void*)buf1,&bmi,DIB_RGB_COLORS);
-        BitBlt(_hdc,dx,dy,dw,dh,mdc3,dx -x -xoff,dy -y -yoff,SRCCOPY);
-
-
+        if (_operation != WS_OPERATION_XOR){
+          BitBlt(_hdc,dx,dy,dw,dh,mdc3,dx -x -xoff,dy -y -yoff,SRCCOPY);
+        }else{
+          BitBlt(_hdc,dx,dy,dw,dh,mdc3,dx -x -xoff,dy -y -yoff,SRCINVERT);
+        }
         SelectObject(mdc3,pxbk3);
         DeleteObject(pix3);
         DeleteDC(mdc3);
@@ -1583,7 +1674,11 @@ long WSDwindraw::drawImage(short x,short y,short sx,short sy,WSCushort w,WSCusho
       SetMapMode(mdc2,GetMapMode(mdc));
 #endif
       StretchBlt(mdc2,0,0,dw,dh,mdc,dx -x +sx,dy -y +sy,iw2,ih2,SRCCOPY);
-      BitBlt(_hdc,dx2,dy2,dw,dh,mdc2,0,0,SRCCOPY);
+      if (_operation != WS_OPERATION_XOR){
+        BitBlt(_hdc,dx2,dy2,dw,dh,mdc2,0,0,SRCCOPY);
+      }else{
+        BitBlt(_hdc,dx2,dy2,dw,dh,mdc2,0,0,PATINVERT);
+      }
 //      StretchBlt(_hdc,dx2,dy2,dw,dh,mdc,dx -x +sx,dy -y +sy,iw2,ih2,SRCCOPY);
       SelectObject(mdc2,pxbk2);
       DeleteObject(pix2);
@@ -1596,7 +1691,7 @@ long WSDwindraw::drawImage(short x,short y,short sx,short sy,WSCushort w,WSCusho
 //      BitBlt(mdc2,0,0,iw,ih,mdc2,0,0,DSTINVERT);
 
       _draw_stretch_image_with_mask(_hdc,mdc,mdc2,pix,pix2,pl,dx,dy,dw,dh,
-                                      dx -x +sx, dy -y +sx,iw2,ih2);
+                                      dx -x +sx, dy -y +sx,iw2,ih2,_operation);
     }
   }else{
     if (pix2 == 0){
@@ -1643,18 +1738,32 @@ long WSDwindraw::drawImage(short x,short y,short sx,short sy,WSCushort w,WSCusho
       GetDIBits(mdc2,pix2,0,ih,(void*)buf2,&bmi,DIB_RGB_COLORS);
       GetDIBits(mdc,pix,0,ih,(void*)buf3,&bmi,DIB_RGB_COLORS);
       int i,j;
-      for(i=0; i<ih; i++){
-        long ptr = i*bytes_per_line;
-        for(j=0; j<bytes_per_line; j++){
-          long val = buf1[ptr] * buf2[ptr] /255 + buf3[ptr] * (255 - buf2[ptr]) /255;
-          buf1[ptr] = val;
-          ptr++;
+      if (_operation != WS_OPERATION_XOR){
+        for(i=0; i<ih; i++){
+          long ptr = i*bytes_per_line;
+          for(j=0; j<bytes_per_line; j++){
+            long val = buf1[ptr] * buf2[ptr] /255 + buf3[ptr] * (255 - buf2[ptr]) /255;
+            buf1[ptr] = val;
+            ptr++;
+          }
+        }
+      }else{
+        for(i=0; i<ih; i++){
+          long ptr = i*bytes_per_line;
+          for(j=0; j<bytes_per_line; j++){
+            long val = buf1[ptr] * buf2[ptr] /255;
+            buf1[ptr] = val;
+            ptr++;
+          }
         }
       }
 
       SetDIBits(mdc3,pix3,0,ih,(void*)buf1,&bmi,DIB_RGB_COLORS);
-      BitBlt(_hdc,dx,dy,dw,dh,mdc3,dx -x +sx,dy -y +sy,SRCCOPY);
-
+      if (_operation != WS_OPERATION_XOR){
+        BitBlt(_hdc,dx,dy,dw,dh,mdc3,dx -x +sx,dy -y +sy,SRCCOPY);
+      }else{
+        BitBlt(_hdc,dx,dy,dw,dh,mdc3,dx -x +sx,dy -y +sy,PATINVERT);
+      }
       SelectObject(mdc3,pxbk3);
       DeleteObject(pix3);
       DeleteDC(mdc3);
@@ -1784,6 +1893,27 @@ WSMFtrace("XXZZr1 setRegion %d,%d,%d,%d bk %d,%d,%d,%d\n",clx,cly,clw,clh,clx_bk
       ExtTextOutW(_hdc,point.x,point.y,ETO_OPAQUE,&r,(WCHAR*)tstr2,slen,NULL);
 #endif
     }
+    if (_underline_char_len > 0 && _underline_char_pos+_underline_char_len > 0){
+      long ustart = _underline_char_pos;
+      long upos   = _underline_char_len;
+      if (ustart < 0){
+        ustart = 0;
+        upos = ustart + upos;
+      }
+      long posx = 0;
+      if (ustart > 0){
+        WSCstring str2(*str);
+        str2.cutString(ustart);
+        posx = fn->getStringWidth(&str2);
+      }
+      WSCstring str3(*str);
+      str3.cutString(ustart+upos);
+      long posx2 = fn->getStringWidth(&str3);
+      long lwbk =_line_width;
+      _line_width = 1;
+      drawLine(point.x + posx, point.y-1 + fheight,point.x + posx2, point.y-1 + fheight);
+      _line_width = lwbk;
+    }
     if (inter_cur != False){
       if (cur > -1){
         WSCstring str2(*str);
@@ -2463,4 +2593,52 @@ static WSDcolor* color = NULL;
   setRegion(clx_bk,cly_bk,clw_bk,clh_bk);
   return WS_NO_ERR;
 }
+long WSDwindraw::setOperation(WSCuchar op){
+  _operation = op;
+  if (op == WS_OPERATION_XOR){
+    if (_xor_mdc == 0){
+      if (_dev != NULL){
+        _xor_mdc = CreateCompatibleDC(_hdc);
+        WSCushort w=1;
+        WSCushort h=1;
+        _dev->getWH(&w,&h);
+        HBITMAP pix = CreateCompatibleBitmap(_hdc,w,h);
+        HBITMAP pxbk = (HBITMAP)SelectObject(_xor_mdc,(void*)pix);
+        DeleteObject(pxbk);
+        WSDcolor*  color = WSGIappColorSet()->getColor("#000000");
+        HPEN pen = CreatePen(PS_SOLID,1,color->getValue1());
+        HPEN pen_bk = (HPEN)SelectObject(_xor_mdc,(void*)pen);
+        DeleteObject(pen_bk);
+        HBRUSH brush = CreateSolidBrush(color->getValue1());
+        HPEN br_bk = (HPEN)SelectObject(_xor_mdc,(void*)brush);
+        DeleteObject(br_bk);
+        
+#ifndef _WSWINCE
+        SetMapMode(_xor_mdc,GetMapMode(_hdc));
+#endif
+        Rectangle(_xor_mdc,0,0,w,h);
+        _hdc_bk = _hdc;
+        _hdc = _xor_mdc;
+      }
+    }
+  }else{
+    if (_xor_mdc != 0){
+      if (_dev != NULL){
+        WSCushort w=1;
+        WSCushort h=1;
+        _dev->getWH(&w,&h);
+        BitBlt(_hdc_bk,0,0,w,h,_xor_mdc,0,0,SRCINVERT);
+      }
+      DeleteDC(_xor_mdc);
+      _hdc = _hdc_bk;
+      _hdc_bk = 0;
+      _xor_mdc = 0;
+    }
+  }
+  return WS_NO_ERR;
+}
+void WSDwindraw::setUnderLineChar(long pos,long len){
+  _underline_char_pos = pos;
+  _underline_char_len = len;
+}
 
