@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+//
 
 #include <windows.h>
 #include <WScom.h>
@@ -70,7 +67,7 @@ char* dfnames_jp[] = {
 "5,24,0,0,0,400,0,0,0,128,3,2,1,1,lr SVbN",
 "6,26,0,0,0,400,0,0,0,128,3,2,1,1,lr SVbN",
 "7,34,0,0,0,400,0,0,0,128,3,2,1,1,lr SVbN",
-"8,12,0,0,0,400,0,0,0,128,3,2,1,1,lr SVbN",
+"8,13,0,0,0,400,0,0,0,128,3,2,1,1,lr SVbN",
 NULL
 };
 #define DEFAULT_FONT_SET_KR  "12,0,0,0,400,0,0,0,129,3,2,1,49,ü"
@@ -324,4 +321,34 @@ HFONT WSDwinfont::getScaledFont(double sx,double sy){
   _hfont2 = CreateFontIndirect(&_lgfont2);
   return _hfont2;
 }
+long WSDwinfont::getAscent(){
+  if (_hfont == 0){
+    initialize();
+  }
+  TEXTMETRICW metric;
 
+  HWND wnd = WSGIwinAppDev()->getHWND();
+  HDC hdc = GetDC(wnd);
+  HFONT fbk = (HFONT)SelectObject(hdc,(void*)_hfont);
+  GetTextMetricsW(hdc,&metric);
+  SelectObject(hdc,(void*)fbk);
+  ReleaseDC(wnd,hdc);
+  return metric.tmAscent;
+//  return  _lgfont.lfHeight*4/5;
+}
+
+long WSDwinfont::getDescent(){
+  if (_hfont == 0){
+    initialize();
+  }
+  TEXTMETRICW metric;
+
+  HWND wnd = WSGIwinAppDev()->getHWND();
+  HDC hdc = GetDC(wnd);
+  HFONT fbk = (HFONT)SelectObject(hdc,(void*)_hfont);
+  GetTextMetricsW(hdc,&metric);
+  SelectObject(hdc,(void*)fbk);
+  ReleaseDC(wnd,hdc);
+  return metric.tmDescent;
+//  return  _lgfont.lfHeight/5;
+}
