@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+//
 
 #include <windows.h>
 #include <WSCdevice.h>
@@ -49,13 +46,14 @@ WSDdev* _wscdwinmwindev_init_(){
   return new WSDwinmwinDev();
 }
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _WSDwinmwinDev_init_ {
   public:_WSDwinmwinDev_init_(){
      WSGIappDevice()->setCreateHandler("mwindowDev",_wscdwinmwindev_init_);
   };
 };
 static _WSDwinmwinDev_init_ _init_to_run_WSDwinmwinDev_;
-
+#endif
 //static WSDwindraw* draw_of_winmwin = NULL;
 
 int red_mask, green_mask, blue_mask;
@@ -86,6 +84,19 @@ long WSDwinmwinDev::copyToWindow(WSDdev* dest,short x,short y,WSCushort w,WSCush
   if (_hBitmap == 0 || _mdc == 0){
     return WS_ERR;
   }
+  DWORD op = SRCCOPY;
+  if (_operation == WS_OPERATION_SRC_COPY){
+  }else
+  if (_operation == WS_OPERATION_XOR){
+    op = PATINVERT;
+  }
+
+  WSDmwindowDev* mdev = (WSDmwindowDev*)dest->cast("WSDmwindowDev");
+  if (mdev != NULL){
+    HDC hdc = (HDC)mdev->getContextResource();
+    BitBlt(hdc,dx,dy,w,h,_mdc,x,y,op);
+    return WS_NO_ERR;
+  }
   HDC hdc = GetDC((HWND)dest->getWindowResource());
   HPALETTE palette = WSGIwinAppDev()->getPalette();
   HPALETTE palette_bk = 0;
@@ -104,13 +115,13 @@ long WSDwinmwinDev::copyToWindow(WSDdev* dest,short x,short y,WSCushort w,WSCush
     BitBlt(hdc,vx+dx,vy+dy,w,h,_mdc,x,y,SRCCOPY);
     WSDwinformDev* wdev = (WSDwinformDev*)dest->getParentDev()->cast("WSDwinformDev");
     if (wdev != NULL && wdev->getPixmapStyle() != WS_DIRECT_WINDOW){
-      BitBlt((HDC)wdev->getContextResource(),vx+dx,vy+dy,w,h,_mdc,x,y,SRCCOPY);
+      BitBlt((HDC)wdev->getContextResource(),vx+dx,vy+dy,w,h,_mdc,x,y,op);
     }
   }else{
     BitBlt(hdc,dx,dy,w,h,_mdc,x,y,SRCCOPY);
     WSDwinformDev* wdev = (WSDwinformDev*)dest->cast("WSDwinformDev");
     if (wdev != NULL && wdev->getPixmapStyle() != WS_DIRECT_WINDOW){
-      BitBlt((HDC)wdev->getContextResource(),dx,dy,w,h,_mdc,x,y,SRCCOPY);
+      BitBlt((HDC)wdev->getContextResource(),dx,dy,w,h,_mdc,x,y,op);
     }
   }
   if (palette != 0){
@@ -126,11 +137,17 @@ long WSDwinmwinDev::copyToWindowWithMask(WSDdev* dest,short x,short y,
     return WS_ERR;
   }
   HDC hdc = GetDC((HWND)dest->getWindowResource());
+
   HPALETTE palette = WSGIwinAppDev()->getPalette();
   HPALETTE palette_bk = 0;
-  if (palette != 0 ){
-     palette_bk = SelectPalette(hdc,palette,False);
-     RealizePalette(hdc);
+  WSDmwindowDev* mdev = (WSDmwindowDev*)dest->cast("WSDmwindowDev");
+  if (mdev != NULL){
+    hdc = (HDC)mdev->getContextResource();
+  }else{
+    if (palette != 0 ){
+      palette_bk = SelectPalette(hdc,palette,False);
+      RealizePalette(hdc);
+    }
   }
   HDC src = CreateCompatibleDC(hdc);
   HBITMAP srcp = CreateCompatibleBitmap(hdc,w,h);
@@ -156,19 +173,30 @@ long WSDwinmwinDev::copyToWindowWithMask(WSDdev* dest,short x,short y,
         vx = dest->getAttachedClient()->getProperty(WSNx);
         vy = dest->getAttachedClient()->getProperty(WSNy);
       }
-      BitBlt(hdc,vx+dx,vy+dy,w,h,mask2,0,0,SRCAND);
-      WSDwinformDev* wdev = (WSDwinformDev*)dest->getParentDev()->cast("WSDwinformDev");
-      if (wdev != NULL && wdev->getPixmapStyle() != WS_DIRECT_WINDOW){
-        BitBlt((HDC)wdev->getContextResource(),vx+dx,vy+dy,w,h,mask2,0,0,SRCAND);
+      if (_operation != WS_OPERATION_XOR){
+        BitBlt(hdc,vx+dx,vy+dy,w,h,mask2,0,0,SRCAND);
+        WSDwinformDev* wdev = (WSDwinformDev*)dest->getParentDev()->cast("WSDwinformDev");
+        if (wdev != NULL && wdev->getPixmapStyle() != WS_DIRECT_WINDOW){
+          BitBlt((HDC)wdev->getContextResource(),vx+dx,vy+dy,w,h,mask2,0,0,SRCAND);
+        }
       }
     }else{
-      BitBlt(hdc,dx,dy,w,h,mask2,0,0,SRCAND);
-      WSDwinformDev* wdev = (WSDwinformDev*)dest->cast("WSDwinformDev");
-      if (wdev != NULL && wdev->getPixmapStyle() != WS_DIRECT_WINDOW){
-        BitBlt((HDC)wdev->getContextResource(),dx,dy,w,h,mask2,0,0,SRCAND);
+      if (_operation != WS_OPERATION_XOR){
+        BitBlt(hdc,dx,dy,w,h,mask2,0,0,SRCAND);
+        WSDwinformDev* wdev = (WSDwinformDev*)dest->cast("WSDwinformDev");
+        if (wdev != NULL && wdev->getPixmapStyle() != WS_DIRECT_WINDOW){
+          BitBlt((HDC)wdev->getContextResource(),dx,dy,w,h,mask2,0,0,SRCAND);
+        }
       }
     }
   }
+  DWORD op = SRCPAINT;
+  if (_operation == WS_OPERATION_SRC_COPY){
+  }else
+  if (_operation == WS_OPERATION_XOR){
+    op = PATINVERT;
+  }
+
   if (dest->cast("WSDnwDev") != NULL){
     int vx = 0;
     int vy = 0;
@@ -176,19 +204,19 @@ long WSDwinmwinDev::copyToWindowWithMask(WSDdev* dest,short x,short y,
       vx = dest->getAttachedClient()->getProperty(WSNx);
       vy = dest->getAttachedClient()->getProperty(WSNy);
     }
-    BitBlt(hdc,vx+dx,vy+dy,w,h,src,0,0,SRCPAINT);
+    BitBlt(hdc,vx+dx,vy+dy,w,h,src,0,0,op);
     WSDwinformDev* wdev = (WSDwinformDev*)dest->getParentDev()->cast("WSDwinformDev");
     if (wdev != NULL && wdev->getPixmapStyle() != WS_DIRECT_WINDOW){
-      BitBlt((HDC)wdev->getContextResource(),vx+dx,vy+dy,w,h,src,0,0,SRCPAINT);
+      BitBlt((HDC)wdev->getContextResource(),vx+dx,vy+dy,w,h,src,0,0,op);
     }
   }else{
-    BitBlt(hdc,dx,dy,w,h,src,0,0,SRCPAINT);
+    BitBlt(hdc,dx,dy,w,h,src,0,0,op);
     WSDwinformDev* wdev = (WSDwinformDev*)dest->cast("WSDwinformDev");
     if (wdev != NULL && wdev->getPixmapStyle() != WS_DIRECT_WINDOW){
-      BitBlt((HDC)wdev->getContextResource(),dx,dy,w,h,src,0,0,SRCPAINT);
+      BitBlt((HDC)wdev->getContextResource(),dx,dy,w,h,src,0,0,op);
     }
   }
-  if (palette != 0){
+  if (mdev == NULL && palette != 0){
     SelectPalette(hdc,palette_bk,False);
   }
 
@@ -197,8 +225,9 @@ long WSDwinmwinDev::copyToWindowWithMask(WSDdev* dest,short x,short y,
   DeleteObject(maskp2);
   DeleteDC(mask2);
   DeleteDC(mask);
-
-  ReleaseDC((HWND)dest->getWindowResource(), hdc);
+  if (mdev == NULL){
+    ReleaseDC((HWND)dest->getWindowResource(), hdc);
+  }
   return WS_NO_ERR;
 }
 
@@ -211,9 +240,14 @@ long WSDwinmwinDev::copyToWindowWithMask(WSDdev* dest,short x,short y,
   HDC hdc = GetDC((HWND)dest->getWindowResource());
   HPALETTE palette = WSGIwinAppDev()->getPalette();
   HPALETTE palette_bk = 0;
-  if (palette != 0 ){
-     palette_bk = SelectPalette(hdc,palette,False);
-     RealizePalette(hdc);
+  WSDmwindowDev* mdev = (WSDmwindowDev*)dest->cast("WSDmwindowDev");
+  if (mdev != NULL){
+    hdc = (HDC)mdev->getContextResource();
+  }else{
+    if (palette != 0 ){
+      palette_bk = SelectPalette(hdc,palette,False);
+      RealizePalette(hdc);
+    }
   }
   HDC src = CreateCompatibleDC(hdc);
   HBITMAP srcp = CreateCompatibleBitmap(hdc,w,h);
@@ -237,26 +271,34 @@ long WSDwinmwinDev::copyToWindowWithMask(WSDdev* dest,short x,short y,
   if (_mdc != (HDC)NULL){
     BitBlt(mask2,0,0,w,h,mask,0,0,NOTSRCCOPY);
     BitBlt(src,0,0,w,h,mask,0,0,SRCAND);
-    if (dest->cast("WSDnwDev") != NULL){
-      int vx = 0;
-      int vy = 0;
-      if (dest->getAttachedClient() != NULL){
-        vx = dest->getAttachedClient()->getProperty(WSNx);
-        vy = dest->getAttachedClient()->getProperty(WSNy);
-      }
-      BitBlt(hdc,vx+dx,vy+dy,w,h,mask2,0,0,SRCAND);
-      WSDwinformDev* wdev = (WSDwinformDev*)dest->getParentDev()->cast("WSDwinformDev");
-      if (wdev != NULL && wdev->getPixmapStyle() != WS_DIRECT_WINDOW){
-        BitBlt((HDC)wdev->getContextResource(),vx+dx,vy+dy,w,h,mask2,0,0,SRCAND);
-      }
-    }else{
-      BitBlt(hdc,dx,dy,w,h,mask2,0,0,SRCAND);
-      WSDwinformDev* wdev = (WSDwinformDev*)dest->cast("WSDwinformDev");
-      if (wdev != NULL && wdev->getPixmapStyle() != WS_DIRECT_WINDOW){
-        BitBlt((HDC)wdev->getContextResource(),dx,dy,w,h,mask2,0,0,SRCAND);
+    if (_operation != WS_OPERATION_XOR){
+      if (dest->cast("WSDnwDev") != NULL){
+        int vx = 0;
+        int vy = 0;
+        if (dest->getAttachedClient() != NULL){
+          vx = dest->getAttachedClient()->getProperty(WSNx);
+          vy = dest->getAttachedClient()->getProperty(WSNy);
+        }
+        BitBlt(hdc,vx+dx,vy+dy,w,h,mask2,0,0,SRCAND);
+        WSDwinformDev* wdev = (WSDwinformDev*)dest->getParentDev()->cast("WSDwinformDev");
+        if (wdev != NULL && wdev->getPixmapStyle() != WS_DIRECT_WINDOW){
+          BitBlt((HDC)wdev->getContextResource(),vx+dx,vy+dy,w,h,mask2,0,0,SRCAND);
+        }
+      }else{
+        BitBlt(hdc,dx,dy,w,h,mask2,0,0,SRCAND);
+        WSDwinformDev* wdev = (WSDwinformDev*)dest->cast("WSDwinformDev");
+        if (wdev != NULL && wdev->getPixmapStyle() != WS_DIRECT_WINDOW){
+          BitBlt((HDC)wdev->getContextResource(),dx,dy,w,h,mask2,0,0,SRCAND);
+        }
       }
     }
   }
+  DWORD op = SRCPAINT;
+  if (_operation == WS_OPERATION_SRC_COPY){
+  }else
+  if (_operation == WS_OPERATION_XOR){
+    op = SRCINVERT;
+  }
   if (dest->cast("WSDnwDev") != NULL){
     int vx = 0;
     int vy = 0;
@@ -264,19 +306,19 @@ long WSDwinmwinDev::copyToWindowWithMask(WSDdev* dest,short x,short y,
       vx = dest->getAttachedClient()->getProperty(WSNx);
       vy = dest->getAttachedClient()->getProperty(WSNy);
     }
-    BitBlt(hdc,vx+dx,vy+dy,w,h,src,0,0,SRCPAINT);
+    BitBlt(hdc,vx+dx,vy+dy,w,h,src,0,0,op);
     WSDwinformDev* wdev = (WSDwinformDev*)dest->getParentDev()->cast("WSDwinformDev");
     if (wdev != NULL && wdev->getPixmapStyle() != WS_DIRECT_WINDOW){
-      BitBlt((HDC)wdev->getContextResource(),vx+dx,vy+dy,w,h,src,0,0,SRCPAINT);
+      BitBlt((HDC)wdev->getContextResource(),vx+dx,vy+dy,w,h,src,0,0,op);
     }
   }else{
-    BitBlt(hdc,dx,dy,w,h,src,0,0,SRCPAINT);
+    BitBlt(hdc,dx,dy,w,h,src,0,0,op);
     WSDwinformDev* wdev = (WSDwinformDev*)dest->cast("WSDwinformDev");
     if (wdev != NULL && wdev->getPixmapStyle() != WS_DIRECT_WINDOW){
-      BitBlt((HDC)wdev->getContextResource(),dx,dy,w,h,src,0,0,SRCPAINT);
+      BitBlt((HDC)wdev->getContextResource(),dx,dy,w,h,src,0,0,op);
     }
   }
-  if (palette != 0){
+  if (mdev == NULL && palette != 0){
     SelectPalette(hdc,palette_bk,False);
   }
 
@@ -284,8 +326,9 @@ long WSDwinmwinDev::copyToWindowWithMask(WSDdev* dest,short x,short y,
   DeleteDC(src);
   DeleteObject(maskp2);
   DeleteDC(mask2);
-
-  ReleaseDC((HWND)dest->getWindowResource(), hdc);
+  if (mdev == NULL){
+    ReleaseDC((HWND)dest->getWindowResource(), hdc);
+  }
   return WS_NO_ERR;
 }
 long WSDwinmwinDev::copyFromWindow(WSDdev* dest,short x,short y,WSCushort w,WSCushort h,short dx,short dy){
@@ -393,7 +436,7 @@ long WSDwinmwinDev::getSpecialResource(){
 }
 
 long WSDwinmwinDev::getWindowResource(){
-  return (long) WSGIwinAppDev()->getWindowResource();
+  return getContextResource();
 }
 
 long WSDwinmwinDev::initBuffer(){
@@ -531,6 +574,19 @@ long WSDwinmwinDev::saveFile(char* fname,long type){
   }
   return WS_ERR;
 }
+HBITMAP WSDwinmwinDev::getBitmap(){
+  return _hBitmap;
+}
+long WSDwinmwinDev::setClipboardData(){
+  if (_hBitmap == 0){
+    return WS_ERR;
+  }
+  OpenClipboard(WSGIwinAppDev()->getHWND());
+  EmptyClipboard();
+  SetClipboardData(CF_BITMAP,_hBitmap);
+  CloseClipboard();
+  return WS_NO_ERR;
+}
 void png_write_fn(png_structp png_ptr, png_bytep buf, png_size_t len) {
   FILE* fp = (FILE*)png_get_io_ptr(png_ptr);
   int ret = fwrite(buf, 1, len, fp);
