@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+//
 
 #include <win/WSDwinnwDev.h>
 #include <win/WSDwinAppDev.h>
@@ -40,6 +37,7 @@ WSDdev* _wsdwinnwdev_init_(){
 //static WSDwindraw* draw_of_winnw = NULL;
 
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _WSDwinnwDev_init_ {
   public:_WSDwinnwDev_init_(){
 //WSMFtrace("set create handler of WSDwinnwDev \n");
@@ -47,6 +45,7 @@ class _WSDwinnwDev_init_ {
   };
 };
 static _WSDwinnwDev_init_ _init_to_run_WSDwinnwDev_;
+#endif
 
 WSDwinnwDev::WSDwinnwDev(){
 //WSMFtrace("WSDwinnwDev::WSDwinnwDev 0x%x 0x%x\n",this,*(long*)this);
