@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <windows.h>
 #ifdef _WSWINCE
@@ -116,7 +112,59 @@ dbprintf("WSDwinsocket::connect() dest addr is %s\n",(char*)_dest_naddr);
   server.sin_port = htons( _dest_port );
 dbprintf("WSDwinsocket::connect() dest port is %d\n",_dest_port);
 
-  if( ::connect(_socket, (struct sockaddr *)&server,sizeof server) < 0 ){
+  WSCulong timeout = _timeout;
+  if (timeout > 0){
+    WSCulong val =1;
+    int fl = ioctlsocket((SOCKET)_socket,FIONBIO,&val);
+    if (fl == SOCKET_ERROR){
+      seterr();
+      return 0;
+    }
+  }else{
+    WSCulong val =0;
+    int fl = ioctlsocket((SOCKET)_socket,FIONBIO,&val);
+    if (fl == SOCKET_ERROR){
+      seterr();
+      return 0;
+    }
+  }
+  int ret = ::connect(_socket, (struct sockaddr *)&server,sizeof server);
+  if( ret < 0 ){
+    long err = WSAGetLastError();
+    if (timeout > 0){
+      if (err == WSAEINPROGRESS || err == WSAEWOULDBLOCK){
+        struct timeval tm;
+        tm.tv_sec = timeout;
+        tm.tv_usec = 0;
+
+        fd_set write_fds;
+        FD_ZERO(&write_fds);
+        FD_SET((SOCKET)_socket,&write_fds);
+        fd_set err_fds;
+        FD_ZERO(&err_fds);
+        FD_SET((SOCKET)_socket,&err_fds);
+        ret = select(FD_SETSIZE,NULL,&write_fds,&err_fds,&tm);
+        err = WSAGetLastError();
+        if (ret < 0 && err != WSAEALREADY){
+          seterr();
+          close();
+          return WS_ERR;
+        }
+        if (ret == 0){
+          seterr();
+          close();
+          return WS_ERR;
+        }
+        if (FD_ISSET(_socket,&err_fds)){
+          seterr();
+          close();
+          return WS_ERR;
+        }
+        return WS_NO_ERR;
+      }
+      close();
+    }
+
 dbprintf("WSDwinsocket::connect() connect err.\n");
     seterr();
     return WS_ERR;
