@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,14 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
+//
 
 #include <win/WSDwinAppDev.h>
 #include <win/WSDwinColor.h>
@@ -29,6 +26,7 @@
 #include <WSCcolorSet.h>
 #include <win/WSDwindraw.h>
 #include <WSDdev.h>
+#include <WSCbaseList.h>
 
 HINSTANCE hInst;
 char szAppName[] = "Wide Studio Application";
@@ -117,7 +115,7 @@ dbprintf("InitInstance %s:%d hWnd=0x%x\n",__FILE__,__LINE__,hWnd);
   return TRUE;
 }
 
-
+int app_type = WS_APP_TYPE_GENERIC;
 int _win_main2(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow){
 
   WSDdev::setDraw(new WSDwindraw());
@@ -205,7 +203,12 @@ WSMFtrace("main 2:\n");
 dbprintf("_win_main2 %s:%d done\n",__FILE__,__LINE__);
    return 0;
 }
-
+extern WSDLEX int WSGFwsInitialize();
+extern WSDLEX int WSGFwsInitialize2(int type);
+int WSGFwsInitialize2(int type){
+  app_type = type;
+  return WSGFwsInitialize();
+}
 int WSGFwsInitialize(){
 static long cnt = 0;
   HINSTANCE hInst = GetModuleHandle(NULL);
@@ -224,14 +227,20 @@ cnt++;
 }
 
 extern "C" {
-WSDLEX32 int _win_main(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow){
+WSDLEX32 int WSGFwsMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow){
   _dbprintf_init();
   _win_main2(hInstance,hPrevInstance,lpCmdLine,nCmdShow);
 
 
   WSDwinAppDev* winApp = WSGIwinAppDev();
   winApp->execInitHandler();
+  WSGIappObjectList()->execInitialize();
   winApp->handleEvents();
   return 0;
 }
+#ifndef NO_MAIN_FUNCTION
+WSDLEX32 int _win_main(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow){
+  return WSGFwsMain(hInstance,hPrevInstance,lpCmdLine,nCmdShow);
+}
+#endif
 };
