@@ -1,6 +1,26 @@
+//
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
+//
+
   WSGIappLocaleSet()->setLocaleString("zh_CN","","");
-  WSGIappLocaleSet()->setLocaleString("zh_CN","A4:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:","    :COPY:,ճ:PASTE:,SP,:PROP:,:OP:,:EXPROP:,SP,ɾ:DEL:");
-  WSGIappLocaleSet()->setLocaleString("zh_CN","A4:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:","    :COPY:,ճ:PASTE:,SP,:PROP:,:OP:,:EXPROP:,SP,ǰ:FRONT:,ú:BACK:,SP,ɾ:DEL:");
   WSGIappLocaleSet()->setLocaleString("zh_CN","B4:WSSTR Confirmation","ѯ");
   WSGIappLocaleSet()->setLocaleString("zh_CN","C4:WSSTR ObjectBox","");
   WSGIappLocaleSet()->setLocaleString("zh_CN","D4:WSSTR ProjectSettings","Ŀ");
@@ -23,21 +43,21 @@
   WSGIappLocaleSet()->setLocaleString("zh_CN","T4:WSSTR Find files","ļ");
   WSGIappLocaleSet()->setLocaleString("zh_CN","U4:WSSTR Help","");
   WSGIappLocaleSet()->setLocaleString("zh_CN","V4:WSSTR  File(F)"," ļ(F)");
-  WSGIappLocaleSet()->setLocaleString("zh_CN","W4:WSSTR New window(N)         :m11:N,Open window(O):m12:O,Save window(S):m13:S,Save as(R):m14:R,Save all(A):m15:A,SP,Exit(X):m16:X","½(N)      :m11:N,򿪴(O):m12:O,洰(S):m13:S,Ϊ(R):m14:R,ȫ(A):m15:A,SP,˳(X):m16:X");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","W4:WSSTR New window(N)         :m11:N,Open window(O):m12:O,Save window(S):m13:S,Save as(R):m14:R,Save all(A):m15:A,SP,Exit(X):m16:X","½(N)      :m11:N:0:$(WSDIR)/sys/pixmaps/wcreate.png,򿪴(O):m12:O:0:$(WSDIR)/sys/pixmaps/wload.png,洰(S):m13:S:0:$(WSDIR)/sys/pixmaps/wsave.png,Ϊ(R):m14:R,ȫ(A):m15:A,SP,˳(X):m16:X:0:$(WSDIR)/sys/pixmaps/pend.png");
   WSGIappLocaleSet()->setLocaleString("zh_CN","X4:WSSTR  Edit(E)"," ༭(E)");
-  WSGIappLocaleSet()->setLocaleString("zh_CN","Y4:WSSTR Copy(C):m21:C,Paste(P):m22:P,SP,Display(I):m27:I,Find(F):m28:F,Delete(D):m23:D,SP,Properties(R):m24:R,Procedures:{List up(L):m25:L,New procedure(N):m251:N,Properties(S):m252:S,Edit(E):m253:E,Delete(X):m254:X},Attributes(A):m26:A,Files(L):m29:L","(C):m21:C,ճ(P):m22:P,SP,ʾ(I):m27:I,ʵ(F):m28:F,ɾ(D):m23:D,SP,Ա༭(R):m24:R,̱༭:{һ(L):m25:L,½(N):m251:N,(S):m252:S,༭(E):m253:E,ɾ(X):m254:X},(A):m26:A,ļһ(L):m29:L");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","Y4:WSSTR Copy(C):m21:C,Paste(P):m22:P,SP,Display(I):m27:I,Find(F):m28:F,Delete(D):m23:D,SP,Properties(R):m24:R,Procedures:{List up(L):m25:L,New procedure(N):m251:N,Properties(S):m252:S,Edit(E):m253:E,Delete(X):m254:X},Attributes(A):m26:A,Files(L):m29:L","ȡ(U):m291:U:1,(C):m21:C,ճ(P):m22:P,SP,ʾ(I):m27:I,ʵ(F):m28:F,ɾ(D):m23:D,SP,Ա༭(R):m24:R,̱༭:{һ(L):m25:L,½(N):m251:N,(S):m252:S,༭(E):m253:E,ɾ(X):m254:X},(A):m26:A,ļһ(L):m29:L");
   WSGIappLocaleSet()->setLocaleString("zh_CN","Z4:WSSTR  Build(B)"," (B)");
-  WSGIappLocaleSet()->setLocaleString("zh_CN","a4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Execute(X):m36:X,Execute with Trace(T):m39:T,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","(C)    :m31:C,ȫ(B):m32:B,(R):m33:R,(E):m34:E,SP,(D):m35:D,(X):m36:X,(T):m39:T,SP,Դ(G):m37:G,Ϣ(I):m38:I");
-  WSGIappLocaleSet()->setLocaleString("zh_CN","b4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Abort(X):m36:X,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","(C)    :m31:C,ȫ(B):m32:B,(R):m33:R,(E):m34:E,SP,(D):m35:D,ֹͣ(X):m36:X,SP,Դ(G):m37:G,Ϣ(I):m38:I");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","a4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Execute(X):m36:X,Execute with Trace(T):m39:T,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","(C)    :m31:C,ȫ(B):m32:B:0:$(WSDIR)/sys/pixmaps/make.png,(R):m33:R,(E):m34:E,SP,(D):m35:D,(X):m36:X:0:$(WSDIR)/sys/pixmaps/run.png,(T):m39:T,SP,Դ(G):m37:G,Ϣ(I):m38:I");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","b4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Abort(X):m36:X,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","(C)    :m31:C,ȫ(B):m32:B:0:$(WSDIR)/sys/pixmaps/make.png,(R):m33:R,(E):m34:E,SP,(D):m35:D,ֹͣ(X):m36:X:0:$(WSDIR)/sys/pixmaps/runx.png,SP,Դ(G):m37:G,Ϣ(I):m38:I");
   WSGIappLocaleSet()->setLocaleString("zh_CN","c4:WSSTR  View(V)"," ʾ(V)");
-  WSGIappLocaleSet()->setLocaleString("zh_CN","d4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Terminal(T):m45:T","ļ(D) :m46:D,(I) :m47:I,ʵ༭(E) :m48:E,Ӵ(L) :m49:L,(O)     :m41:O,(C):m42:C,(B):m43:B,ն(T):m45:T");
-  WSGIappLocaleSet()->setLocaleString("zh_CN","e4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Find Files(F):m44:F,Terminal(T):m45:T","ļ(D) :m46:D,(I) :m47:I,ʵ༭(E) :m48:E,Ӵ(L) :m49:L,(O)     :m41:O,(C):m42:C,(B):m43:B,ļ(F):m44:F,ն(T):m45:T");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","d4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Terminal(T):m45:T","ļ(D) :m46:D,(I) :m47:I,ʵ༭(E) :m48:E,Ӵ(L) :m49:L,(O)     :m41:O:0:$(WSDIR)/sys/pixmaps/obox.png,(C):m42:C,(B):m43:B,SP,ն(T):m45:T");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","e4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Find Files(F):m44:F,Terminal(T):m45:T","ļ(D) :m46:D,(I) :m47:I,ʵ༭(E) :m48:E,Ӵ(L) :m49:L,(O)     :m41:O:0:$(WSDIR)/sys/pixmaps/obox.png,(C):m42:C,(B):m43:B,SP,ļ(F):m44:F,ն(T):m45:T");
   WSGIappLocaleSet()->setLocaleString("zh_CN","f4:WSSTR  Project(P)"," Ŀ(P)");
-  WSGIappLocaleSet()->setLocaleString("zh_CN","g4:WSSTR New project(N)        :m51:N,Open project(O):m52:O,Save project(S):m53:S,Rename project(R):m54:R,SP,Project settings(E):m55:E,SP,Close project(C):m56:C","½Ŀ(N)      :m51:N,Ŀ(O) :m52:O,Ŀ(S):m53:S,ĿΪ(R):m54:R,SP,Ŀ(E):m55:E,SP,رĿ(C):m56:C");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","g4:WSSTR New project(N)        :m51:N,Open project(O):m52:O,Save project(S):m53:S,Rename project(R):m54:R,SP,Project settings(E):m55:E,SP,Close project(C):m56:C","½Ŀ(N)      :m51:N:0:$(WSDIR)/sys/pixmaps/pcreate.png,Ŀ(O) :m52:O:0:$(WSDIR)/sys/pixmaps/pload.png,Ŀ(S):m53:S:0:$(WSDIR)/sys/pixmaps/psave.png,ĿΪ(R):m54:R,SP,Ŀ(E):m55:E:0:$(WSDIR)/sys/pixmaps/pset.png,SP,رĿ(C):m56:C:0:$(WSDIR)/sys/pixmaps/pend.png");
   WSGIappLocaleSet()->setLocaleString("zh_CN","h4:WSSTR  Option(O)"," ѡ(O)");
-  WSGIappLocaleSet()->setLocaleString("zh_CN","i4:WSSTR Environment(E)    :m61:E,SP,Colors(C):m62:C,Fonts(F):m63:F,Look and Feel(L):m64:L","(E)      :m61:E,SP,ɫ(C):m62:C,(F):m63:F,(L):m64:L");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","i4:WSSTR Environment(E)    :m61:E,SP,Colors(C):m62:C,Fonts(F):m63:F,Look and Feel(L):m64:L","(E)      :m61:E:0:$(WSDIR)/sys/pixmaps/pset.png,SP,ɫ(C):m62:C,(F):m63:F,(L):m64:L");
   WSGIappLocaleSet()->setLocaleString("zh_CN","j4:WSSTR  Help(H)"," (H)");
-  WSGIappLocaleSet()->setLocaleString("zh_CN","k4:WSSTR Index(I):m81:I,Reference(R)     :m82:R,User's guide(U):m83:U,About WideStudio(V):m84:V,License(L):m85:L","Ŀ¼(I):m81:I,ο(R)     :m82:R,ʹ÷(U):m83:U,汾Ϣ(V):m84:V,֤Ϣ(L):m85:L");
+  WSGIappLocaleSet()->setLocaleString("zh_CN","k4:WSSTR Index(I):m81:I,Reference(R)     :m82:R,User's guide(U):m83:U,About WideStudio(V):m84:V,License(L):m85:L","Ŀ¼(I):m81:I:0:$(WSDIR)/sys/pixmaps/help.png,ο(R)     :m82:R,ʹ÷(U):m83:U,汾Ϣ(V):m84:V,֤Ϣ(L):m85:L");
   WSGIappLocaleSet()->setLocaleString("zh_CN","l4:WSSTR Please specify a window to save.","ѡҪĴڡ");
   WSGIappLocaleSet()->setLocaleString("zh_CN","m4:WSSTR Continue saving?","ȷϱ");
   WSGIappLocaleSet()->setLocaleString("zh_CN","n4:WSSTR Please specify some instances to copy.","ѡҪƵʵ");
