@@ -1,6 +1,26 @@
+//
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
+//
+
   WSGIappLocaleSet()->setLocaleString("en_US","","");
-  WSGIappLocaleSet()->setLocaleString("en_US","A4:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:","Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:");
-  WSGIappLocaleSet()->setLocaleString("en_US","A4:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:","Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:");
   WSGIappLocaleSet()->setLocaleString("en_US","B4:WSSTR Confirmation","Confirmation");
   WSGIappLocaleSet()->setLocaleString("en_US","C4:WSSTR ObjectBox","ObjectBox");
   WSGIappLocaleSet()->setLocaleString("en_US","D4:WSSTR ProjectSettings","ProjectSettings");
@@ -23,21 +43,21 @@
   WSGIappLocaleSet()->setLocaleString("en_US","T4:WSSTR Find files","Find files");
   WSGIappLocaleSet()->setLocaleString("en_US","U4:WSSTR Help","Help");
   WSGIappLocaleSet()->setLocaleString("en_US","V4:WSSTR  File(F)"," File(F)");
-  WSGIappLocaleSet()->setLocaleString("en_US","W4:WSSTR New window(N)         :m11:N,Open window(O):m12:O,Save window(S):m13:S,Save as(R):m14:R,Save all(A):m15:A,SP,Exit(X):m16:X","New window(N)         :m11:N,Open window(O):m12:O,Save window(S):m13:S,Save as(R):m14:R,Save all(A):m15:A,SP,Exit(X):m16:X");
+  WSGIappLocaleSet()->setLocaleString("en_US","W4:WSSTR New window(N)         :m11:N,Open window(O):m12:O,Save window(S):m13:S,Save as(R):m14:R,Save all(A):m15:A,SP,Exit(X):m16:X","New window(N)         :m11:N:0:$(WSDIR)/sys/pixmaps/wcreate.png,Open window(O):m12:O:0:$(WSDIR)/sys/pixmaps/wload.png,Save window(S):m13:S:0:$(WSDIR)/sys/pixmaps/wsave.png,Save as(R):m14:R,Save all(A):m15:A,SP,Exit(X):m16:X:0:$(WSDIR)/sys/pixmaps/pend.png");
   WSGIappLocaleSet()->setLocaleString("en_US","X4:WSSTR  Edit(E)"," Edit(E)");
-  WSGIappLocaleSet()->setLocaleString("en_US","Y4:WSSTR Copy(C):m21:C,Paste(P):m22:P,SP,Display(I):m27:I,Find(F):m28:F,Delete(D):m23:D,SP,Properties(R):m24:R,Procedures:{List up(L):m25:L,New procedure(N):m251:N,Properties(S):m252:S,Edit(E):m253:E,Delete(X):m254:X},Attributes(A):m26:A,Files(L):m29:L","Copy(C):m21:C,Paste(P):m22:P,SP,Display(I):m27:I,Find(F):m28:F,Delete(D):m23:D,SP,Properties(R):m24:R,Procedures:{List up(L):m25:L,New procedure(N):m251:N,Properties(S):m252:S,Edit(E):m253:E,Delete(X):m254:X},Attributes(A):m26:A,Files(L):m29:L");
+  WSGIappLocaleSet()->setLocaleString("en_US","Y4:WSSTR Copy(C):m21:C,Paste(P):m22:P,SP,Display(I):m27:I,Find(F):m28:F,Delete(D):m23:D,SP,Properties(R):m24:R,Procedures:{List up(L):m25:L,New procedure(N):m251:N,Properties(S):m252:S,Edit(E):m253:E,Delete(X):m254:X},Attributes(A):m26:A,Files(L):m29:L","Undo(U):m291:U:1,Copy(C):m21:C,Paste(P):m22:P,SP,Display(I):m27:I,Find(F):m28:F,Delete(D):m23:D,SP,Properties(R):m24:R,Procedures:{List up(L):m25:L,New procedure(N):m251:N,Properties(S):m252:S,Edit(E):m253:E,Delete(X):m254:X},Attributes(A):m26:A,Files(L):m29:L");
   WSGIappLocaleSet()->setLocaleString("en_US","Z4:WSSTR  Build(B)"," Build(B)");
-  WSGIappLocaleSet()->setLocaleString("en_US","a4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Execute(X):m36:X,Execute with Trace(T):m39:T,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Execute(X):m36:X,Execute with Trace(T):m39:T,SP,Generate source(G):m37:G,Generate inf file(I):m38:I");
-  WSGIappLocaleSet()->setLocaleString("en_US","b4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Abort(X):m36:X,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Abort(X):m36:X,SP,Generate source(G):m37:G,Generate inf file(I):m38:I");
+  WSGIappLocaleSet()->setLocaleString("en_US","a4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Execute(X):m36:X,Execute with Trace(T):m39:T,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","Compile(C)    :m31:C,Build all(B):m32:B:0:$(WSDIR)/sys/pixmaps/make.png,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Execute(X):m36:X:0:$(WSDIR)/sys/pixmaps/run.png,Execute with Trace(T):m39:T,SP,Generate source(G):m37:G,Generate inf file(I):m38:I");
+  WSGIappLocaleSet()->setLocaleString("en_US","b4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Abort(X):m36:X,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","Compile(C)    :m31:C,Build all(B):m32:B:0:$(WSDIR)/sys/pixmaps/make.png,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Abort(X):m36:X:0:$(WSDIR)/sys/pixmaps/runx.png,SP,Generate source(G):m37:G,Generate inf file(I):m38:I");
   WSGIappLocaleSet()->setLocaleString("en_US","c4:WSSTR  View(V)"," View(V)");
-  WSGIappLocaleSet()->setLocaleString("en_US","d4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Terminal(T):m45:T","Folders(D) :m46:D,Inspector(I) :m47:I,InstanceEditor(E) :m48:E,ClassList(L) :m49:L,ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Terminal(T):m45:T");
-  WSGIappLocaleSet()->setLocaleString("en_US","e4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Find Files(F):m44:F,Terminal(T):m45:T","Folders(D) :m46:D,Inspector(I) :m47:I,InstanceEditor(E) :m48:E,ClassList(L) :m49:L,ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Find Files(F):m44:F,Terminal(T):m45:T");
+  WSGIappLocaleSet()->setLocaleString("en_US","d4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Terminal(T):m45:T","Folders(D) :m46:D,Inspector(I) :m47:I,InstanceEditor(E) :m48:E,ClassList(L) :m49:L,ObjectBox(O) :m41:O:0:$(WSDIR)/sys/pixmaps/obox.png,Component(C)    :m42:C,ToolBar(B):m43:B,SP,Terminal(T):m45:T");
+  WSGIappLocaleSet()->setLocaleString("en_US","e4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Find Files(F):m44:F,Terminal(T):m45:T","Folders(D) :m46:D,Inspector(I) :m47:I,InstanceEditor(E) :m48:E,ClassList(L) :m49:L,ObjectBox(O) :m41:O:0:$(WSDIR)/sys/pixmaps/obox.png,Component(C)    :m42:C,ToolBar(B):m43:B,SP,Find Files(F):m44:F,Terminal(T):m45:T");
   WSGIappLocaleSet()->setLocaleString("en_US","f4:WSSTR  Project(P)"," Project(P)");
-  WSGIappLocaleSet()->setLocaleString("en_US","g4:WSSTR New project(N)        :m51:N,Open project(O):m52:O,Save project(S):m53:S,Rename project(R):m54:R,SP,Project settings(E):m55:E,SP,Close project(C):m56:C","New project(N)        :m51:N,Open project(O):m52:O,Save project(S):m53:S,Rename project(R):m54:R,SP,Project settings(E):m55:E,SP,Close project(C):m56:C");
+  WSGIappLocaleSet()->setLocaleString("en_US","g4:WSSTR New project(N)        :m51:N,Open project(O):m52:O,Save project(S):m53:S,Rename project(R):m54:R,SP,Project settings(E):m55:E,SP,Close project(C):m56:C","New project(N)        :m51:N:0:$(WSDIR)/sys/pixmaps/pcreate.png,Open project(O):m52:O:0:$(WSDIR)/sys/pixmaps/pload.png,Save project(S):m53:S:0:$(WSDIR)/sys/pixmaps/psave.png,Rename project(R):m54:R,SP,Project settings(E):m55:E:0:$(WSDIR)/sys/pixmaps/pset.png,SP,Close project(C):m56:C:0:$(WSDIR)/sys/pixmaps/pend.png");
   WSGIappLocaleSet()->setLocaleString("en_US","h4:WSSTR  Option(O)"," Option(O)");
-  WSGIappLocaleSet()->setLocaleString("en_US","i4:WSSTR Environment(E)    :m61:E,SP,Colors(C):m62:C,Fonts(F):m63:F,Look and Feel(L):m64:L","Environment(E)    :m61:E,SP,Colors(C):m62:C,Fonts(F):m63:F,Look and Feel(L):m64:L");
+  WSGIappLocaleSet()->setLocaleString("en_US","i4:WSSTR Environment(E)    :m61:E,SP,Colors(C):m62:C,Fonts(F):m63:F,Look and Feel(L):m64:L","Environment(E)    :m61:E:0:$(WSDIR)/sys/pixmaps/pset.png,SP,Colors(C):m62:C,Fonts(F):m63:F,Look and Feel(L):m64:L");
   WSGIappLocaleSet()->setLocaleString("en_US","j4:WSSTR  Help(H)"," Help(H)");
-  WSGIappLocaleSet()->setLocaleString("en_US","k4:WSSTR Index(I):m81:I,Reference(R)     :m82:R,User's guide(U):m83:U,About WideStudio(V):m84:V,License(L):m85:L","Index(I):m81:I,Reference(R)     :m82:R,User's guide(U):m83:U,About WideStudio(V):m84:V,License(L):m85:L");
+  WSGIappLocaleSet()->setLocaleString("en_US","k4:WSSTR Index(I):m81:I,Reference(R)     :m82:R,User's guide(U):m83:U,About WideStudio(V):m84:V,License(L):m85:L","Index(I):m81:I:0:$(WSDIR)/sys/pixmaps/help.png,Reference(R)     :m82:R,User's guide(U):m83:U,About WideStudio(V):m84:V,License(L):m85:L");
   WSGIappLocaleSet()->setLocaleString("en_US","l4:WSSTR Please specify a window to save.","Please specify a window to save.");
   WSGIappLocaleSet()->setLocaleString("en_US","m4:WSSTR Continue saving?","Continue saving?");
   WSGIappLocaleSet()->setLocaleString("en_US","n4:WSSTR Please specify some instances to copy.","Please specify some instances to copy.");
