@@ -1,6 +1,26 @@
+//
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
+//
+
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","","");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","A4:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:","    :COPY:,\x93\\t:PASTE:,SP,vpeB:PROP:,vV[W:OP:,eݒ:EXPROP:,SP,폜:DEL:");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","A4:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:","    :COPY:,\x93\\t:PASTE:,SP,vpeB:PROP:,vV[W:OP:,eݒ:EXPROP:,SP,Oʂ:FRONT:,wʂ:BACK:,SP,폜:DEL:");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","B4:WSSTR Confirmation","");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","C4:WSSTR ObjectBox","IuWFNg{bNX");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","D4:WSSTR ProjectSettings","vWFNgݒ");
@@ -23,21 +43,21 @@
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","T4:WSSTR Find files","t@C");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","U4:WSSTR Help","wv");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","V4:WSSTR  File(F)"," t@C(F)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","W4:WSSTR New window(N)         :m11:N,Open window(O):m12:O,Save window(S):m13:S,Save as(R):m14:R,Save all(A):m15:A,SP,Exit(X):m16:X","VKEBhE(N)      :m11:N,EBhEǂݍ(O):m12:O,EBhEۑ(S):m13:S,EBhEʖۑ(R):m14:R,EBhESۑ(A):m15:A,SP,r_I(X):m16:X");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","W4:WSSTR New window(N)         :m11:N,Open window(O):m12:O,Save window(S):m13:S,Save as(R):m14:R,Save all(A):m15:A,SP,Exit(X):m16:X","VKEBhE(N)      :m11:N:0:$(WSDIR)/sys/pixmaps/wcreate.png,EBhEǂݍ(O):m12:O:0:$(WSDIR)/sys/pixmaps/wload.png,EBhEۑ(S):m13:S:0:$(WSDIR)/sys/pixmaps/wsave.png,EBhEʖۑ(R):m14:R:0:,EBhESۑ(A):m15:A:0:,SP,r_I(X):m16:X:0:$(WSDIR)/sys/pixmaps/pend.png");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","X4:WSSTR  Edit(E)"," ҏW(E)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","Y4:WSSTR Copy(C):m21:C,Paste(P):m22:P,SP,Display(I):m27:I,Find(F):m28:F,Delete(D):m23:D,SP,Properties(R):m24:R,Procedures:{List up(L):m25:L,New procedure(N):m251:N,Properties(S):m252:S,Edit(E):m253:E,Delete(X):m254:X},Attributes(A):m26:A,Files(L):m29:L","(C):m21:C,\x93\\t(P):m22:P,SP,\x95\\(I):m27:I,CX^X̌(F):m28:F,폜(D):m23:D,SP,vpeBҏW(R):m24:R,vV[WҏW:{vV[Wꗗ(L):m25:L,vV[WVK쐬(N):m251:N,vV[Wݒ(S):m252:S,vV[WҏW(E):m253:E,vV[W폜(X):m254:X},{ݒ(A):m26:A,t@Cꗗ(L):m29:L");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","Y4:WSSTR Copy(C):m21:C,Paste(P):m22:P,SP,Display(I):m27:I,Find(F):m28:F,Delete(D):m23:D,SP,Properties(R):m24:R,Procedures:{List up(L):m25:L,New procedure(N):m251:N,Properties(S):m252:S,Edit(E):m253:E,Delete(X):m254:X},Attributes(A):m26:A,Files(L):m29:L","ɖ߂(U):m291:U:1,(C):m21:C,\x93\\t(P):m22:P,SP,\x95\\(I):m27:I,CX^X̌(F):m28:F,폜(D):m23:D,SP,vpeBҏW(R):m24:R,vV[WҏW:{vV[Wꗗ(L):m25:L,vV[WVK쐬(N):m251:N,vV[Wݒ(S):m252:S,vV[WҏW(E):m253:E,vV[W폜(X):m254:X},{ݒ(A):m26:A,t@Cꗗ(L):m29:L");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","Z4:WSSTR  Build(B)"," rh(B)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","a4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Execute(X):m36:X,Execute with Trace(T):m39:T,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","RpC(C)    :m31:C,rhI[(B):m32:B,rh(R):m33:R,N[(E):m34:E,SP,fobO(D):m35:D,s(X):m36:X,g[Xs(T):m39:T,SP,\x83\\[X̐(G):m37:G,\x8d\\̏o(I):m38:I");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","b4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Abort(X):m36:X,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","RpC(C)    :m31:C,rhI[(B):m32:B,rh(R):m33:R,N[(E):m34:E,SP,fobO(D):m35:D,s~(X):m36:X,SP,\x83\\[X̐(G):m37:G,\x8d\\̏o(I):m38:I");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","a4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Execute(X):m36:X,Execute with Trace(T):m39:T,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","RpC(C)    :m31:C,rhI[(B):m32:B:0:$(WSDIR)/sys/pixmaps/make.png,rh(R):m33:R,N[(E):m34:E,SP,fobO(D):m35:D,s(X):m36:X:0:$(WSDIR)/sys/pixmaps/run.png,g[Xs(T):m39:T,SP,\x83\\[X̐(G):m37:G,\x8d\\̏o(I):m38:I");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","b4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Abort(X):m36:X,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","RpC(C)    :m31:C,rhI[(B):m32:B:0:$(WSDIR)/sys/pixmaps/make.png,rh(R):m33:R,N[(E):m34:E,SP,fobO(D):m35:D,s~(X):m36:X:0:$(WSDIR)/sys/pixmaps/runx.png,SP,\x83\\[X̐(G):m37:G,\x8d\\̏o(I):m38:I");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","c4:WSSTR  View(V)"," \x95\\(V)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","d4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Terminal(T):m45:T","tH_(D) :m46:D,CXyN^(I) :m47:I,CX^XGfB^(E) :m48:E,NXXg(L) :m49:L,IuWFNg{bNX(O)     :m41:O,R|[lg(C):m42:C,c[o[(B):m43:B,^[~i(T):m45:T");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","e4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Find Files(F):m44:F,Terminal(T):m45:T","tH_(D) :m46:D,CXyN^(I) :m47:I,CX^XGfB^(E) :m48:E,NXXg(L) :m49:L,IuWFNg{bNX(O)     :m41:O,R|[lg(C):m42:C,c[o[(B):m43:B,t@C(F):m44:F,^[~i(T):m45:T");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","d4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Terminal(T):m45:T","tH_(D) :m46:D,CXyN^(I) :m47:I,CX^XGfB^(E) :m48:E,NXXg(L) :m49:L,IuWFNg{bNX(O)     :m41:O:0:$(WSDIR)/sys/pixmaps/obox.png,R|[lg(C):m42:C,c[o[(B):m43:B,SP,^[~i(T):m45:T");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","e4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Find Files(F):m44:F,Terminal(T):m45:T","tH_(D) :m46:D,CXyN^(I) :m47:I,CX^XGfB^(E) :m48:E,NXXg(L) :m49:L,IuWFNg{bNX(O)     :m41:O:0:$(WSDIR)/sys/pixmaps/obox.png,R|[lg(C):m42:C,c[o[(B):m43:B,SP,t@C(F):m44:F,^[~i(T):m45:T");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","f4:WSSTR  Project(P)"," vWFNg(P)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","g4:WSSTR New project(N)        :m51:N,Open project(O):m52:O,Save project(S):m53:S,Rename project(R):m54:R,SP,Project settings(E):m55:E,SP,Close project(C):m56:C","VKvWFNg(N)      :m51:N,vWFNgǂݍ(O) :m52:O,vWFNgۑ(S):m53:S,vWFNgʖۑ(R):m54:R,SP,vWFNgݒ(E):m55:E,SP,vWFNgI(C):m56:C");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","g4:WSSTR New project(N)        :m51:N,Open project(O):m52:O,Save project(S):m53:S,Rename project(R):m54:R,SP,Project settings(E):m55:E,SP,Close project(C):m56:C","VKvWFNg(N)      :m51:N:0:$(WSDIR)/sys/pixmaps/pcreate.png,vWFNgǂݍ(O) :m52:O:0:$(WSDIR)/sys/pixmaps/pload.png,vWFNgۑ(S):m53:S:0:$(WSDIR)/sys/pixmaps/psave.png,vWFNgʖۑ(R):m54:R,SP,vWFNgݒ(E):m55:E:0:$(WSDIR)/sys/pixmaps/pset.png,SP,vWFNgI(C):m56:C:0:$(WSDIR)/sys/pixmaps/pend.png");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","h4:WSSTR  Option(O)"," IvV(O)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","i4:WSSTR Environment(E)    :m61:E,SP,Colors(C):m62:C,Fonts(F):m63:F,Look and Feel(L):m64:L","ݒ(E)      :m61:E,SP,Fݒ(C):m62:C,tHgݒ(F):m63:F,bNAhtB[(L):m64:L");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","i4:WSSTR Environment(E)    :m61:E,SP,Colors(C):m62:C,Fonts(F):m63:F,Look and Feel(L):m64:L","ݒ(E)      :m61:E:0:$(WSDIR)/sys/pixmaps/pset.png,SP,Fݒ(C):m62:C,tHgݒ(F):m63:F,bNAhtB[(L):m64:L");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","j4:WSSTR  Help(H)"," wv(H)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","k4:WSSTR Index(I):m81:I,Reference(R)     :m82:R,User's guide(U):m83:U,About WideStudio(V):m84:V,License(L):m85:L","ڎ(I):m81:I,t@X(R)     :m82:R,g(U):m83:U,o[W(V):m84:V,CZX(L):m85:L");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","k4:WSSTR Index(I):m81:I,Reference(R)     :m82:R,User's guide(U):m83:U,About WideStudio(V):m84:V,License(L):m85:L","ڎ(I):m81:I:0:$(WSDIR)/sys/pixmaps/help.png,t@X(R)     :m82:R,g(U):m83:U,o[W(V):m84:V,CZX(L):m85:L");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","l4:WSSTR Please specify a window to save.","ۑEBhEIĉ.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","m4:WSSTR Continue saving?","ۑ𑱂܂.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.sjis","n4:WSSTR Please specify some instances to copy.","Rs[CX^XIĂ.");
