@@ -1,6 +1,26 @@
+//
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
+//
+
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","","");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","A4:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:","複写    :COPY:,貼り付け:PASTE:,SP,プロパティ:PROP:,プロシージャ:OP:,各種設定:EXPROP:,SP,削除:DEL:");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","A4:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:","複写    :COPY:,貼り付け:PASTE:,SP,プロパティ:PROP:,プロシージャ:OP:,各種設定:EXPROP:,SP,前面へ:FRONT:,背面へ:BACK:,SP,削除:DEL:");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","B4:WSSTR Confirmation","質問");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","C4:WSSTR ObjectBox","オブジェクトボックス");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","D4:WSSTR ProjectSettings","プロジェクト設定");
@@ -23,21 +43,21 @@
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","T4:WSSTR Find files","ファイル検索");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","U4:WSSTR Help","ヘルプ");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","V4:WSSTR  File(F)"," ファイル(F)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","W4:WSSTR New window(N)         :m11:N,Open window(O):m12:O,Save window(S):m13:S,Save as(R):m14:R,Save all(A):m15:A,SP,Exit(X):m16:X","新規ウィンドウ(N)      :m11:N,ウィンドウ読み込み(O):m12:O,ウィンドウ保存(S):m13:S,ウィンドウ別名保存(R):m14:R,ウィンドウ全保存(A):m15:A,SP,ビルダ終了(X):m16:X");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","W4:WSSTR New window(N)         :m11:N,Open window(O):m12:O,Save window(S):m13:S,Save as(R):m14:R,Save all(A):m15:A,SP,Exit(X):m16:X","新規ウィンドウ(N)      :m11:N:0:$(WSDIR)/sys/pixmaps/wcreate.png,ウィンドウ読み込み(O):m12:O:0:$(WSDIR)/sys/pixmaps/wload.png,ウィンドウ保存(S):m13:S:0:$(WSDIR)/sys/pixmaps/wsave.png,ウィンドウ別名保存(R):m14:R:0:,ウィンドウ全保存(A):m15:A:0:,SP,ビルダ終了(X):m16:X:0:$(WSDIR)/sys/pixmaps/pend.png");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","X4:WSSTR  Edit(E)"," 編集(E)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","Y4:WSSTR Copy(C):m21:C,Paste(P):m22:P,SP,Display(I):m27:I,Find(F):m28:F,Delete(D):m23:D,SP,Properties(R):m24:R,Procedures:{List up(L):m25:L,New procedure(N):m251:N,Properties(S):m252:S,Edit(E):m253:E,Delete(X):m254:X},Attributes(A):m26:A,Files(L):m29:L","複写(C):m21:C,貼り付け(P):m22:P,SP,表示(I):m27:I,インスタンスの検索(F):m28:F,削除(D):m23:D,SP,プロパティ編集(R):m24:R,プロシージャ編集:{プロシージャ一覧(L):m25:L,プロシージャ新規作成(N):m251:N,プロシージャ設定(S):m252:S,プロシージャ編集(E):m253:E,プロシージャ削除(X):m254:X},基本設定(A):m26:A,ファイル一覧(L):m29:L");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","Y4:WSSTR Copy(C):m21:C,Paste(P):m22:P,SP,Display(I):m27:I,Find(F):m28:F,Delete(D):m23:D,SP,Properties(R):m24:R,Procedures:{List up(L):m25:L,New procedure(N):m251:N,Properties(S):m252:S,Edit(E):m253:E,Delete(X):m254:X},Attributes(A):m26:A,Files(L):m29:L","元に戻す(U):m291:U:1,複写(C):m21:C,貼り付け(P):m22:P,SP,表示(I):m27:I,インスタンスの検索(F):m28:F,削除(D):m23:D,SP,プロパティ編集(R):m24:R,プロシージャ編集:{プロシージャ一覧(L):m25:L,プロシージャ新規作成(N):m251:N,プロシージャ設定(S):m252:S,プロシージャ編集(E):m253:E,プロシージャ削除(X):m254:X},基本設定(A):m26:A,ファイル一覧(L):m29:L");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","Z4:WSSTR  Build(B)"," ビルド(B)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","a4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Execute(X):m36:X,Execute with Trace(T):m39:T,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","コンパイル(C)    :m31:C,ビルドオール(B):m32:B,リビルド(R):m33:R,クリーン(E):m34:E,SP,デバッグ(D):m35:D,実行(X):m36:X,トレース実行(T):m39:T,SP,ソースの生成(G):m37:G,構成情報の出力(I):m38:I");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","b4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Abort(X):m36:X,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","コンパイル(C)    :m31:C,ビルドオール(B):m32:B,リビルド(R):m33:R,クリーン(E):m34:E,SP,デバッグ(D):m35:D,実行中止(X):m36:X,SP,ソースの生成(G):m37:G,構成情報の出力(I):m38:I");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","a4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Execute(X):m36:X,Execute with Trace(T):m39:T,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","コンパイル(C)    :m31:C,ビルドオール(B):m32:B:0:$(WSDIR)/sys/pixmaps/make.png,リビルド(R):m33:R,クリーン(E):m34:E,SP,デバッグ(D):m35:D,実行(X):m36:X:0:$(WSDIR)/sys/pixmaps/run.png,トレース実行(T):m39:T,SP,ソースの生成(G):m37:G,構成情報の出力(I):m38:I");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","b4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Abort(X):m36:X,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","コンパイル(C)    :m31:C,ビルドオール(B):m32:B:0:$(WSDIR)/sys/pixmaps/make.png,リビルド(R):m33:R,クリーン(E):m34:E,SP,デバッグ(D):m35:D,実行中止(X):m36:X:0:$(WSDIR)/sys/pixmaps/runx.png,SP,ソースの生成(G):m37:G,構成情報の出力(I):m38:I");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","c4:WSSTR  View(V)"," 表示(V)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","d4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Terminal(T):m45:T","フォルダ(D) :m46:D,インスペクタ(I) :m47:I,インスタンスエディタ(E) :m48:E,クラスリスト(L) :m49:L,オブジェクトボックス(O)     :m41:O,コンポーネント(C):m42:C,ツールバー(B):m43:B,ターミナル(T):m45:T");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","e4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Find Files(F):m44:F,Terminal(T):m45:T","フォルダ(D) :m46:D,インスペクタ(I) :m47:I,インスタンスエディタ(E) :m48:E,クラスリスト(L) :m49:L,オブジェクトボックス(O)     :m41:O,コンポーネント(C):m42:C,ツールバー(B):m43:B,ファイル検索(F):m44:F,ターミナル(T):m45:T");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","d4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Terminal(T):m45:T","フォルダ(D) :m46:D,インスペクタ(I) :m47:I,インスタンスエディタ(E) :m48:E,クラスリスト(L) :m49:L,オブジェクトボックス(O)     :m41:O:0:$(WSDIR)/sys/pixmaps/obox.png,コンポーネント(C):m42:C,ツールバー(B):m43:B,SP,ターミナル(T):m45:T");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","e4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Find Files(F):m44:F,Terminal(T):m45:T","フォルダ(D) :m46:D,インスペクタ(I) :m47:I,インスタンスエディタ(E) :m48:E,クラスリスト(L) :m49:L,オブジェクトボックス(O)     :m41:O:0:$(WSDIR)/sys/pixmaps/obox.png,コンポーネント(C):m42:C,ツールバー(B):m43:B,SP,ファイル検索(F):m44:F,ターミナル(T):m45:T");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","f4:WSSTR  Project(P)"," プロジェクト(P)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","g4:WSSTR New project(N)        :m51:N,Open project(O):m52:O,Save project(S):m53:S,Rename project(R):m54:R,SP,Project settings(E):m55:E,SP,Close project(C):m56:C","新規プロジェクト(N)      :m51:N,プロジェクト読み込み(O) :m52:O,プロジェクト保存(S):m53:S,プロジェクト別名保存(R):m54:R,SP,プロジェクト設定(E):m55:E,SP,プロジェクト終了(C):m56:C");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","g4:WSSTR New project(N)        :m51:N,Open project(O):m52:O,Save project(S):m53:S,Rename project(R):m54:R,SP,Project settings(E):m55:E,SP,Close project(C):m56:C","新規プロジェクト(N)      :m51:N:0:$(WSDIR)/sys/pixmaps/pcreate.png,プロジェクト読み込み(O) :m52:O:0:$(WSDIR)/sys/pixmaps/pload.png,プロジェクト保存(S):m53:S:0:$(WSDIR)/sys/pixmaps/psave.png,プロジェクト別名保存(R):m54:R,SP,プロジェクト設定(E):m55:E:0:$(WSDIR)/sys/pixmaps/pset.png,SP,プロジェクト終了(C):m56:C:0:$(WSDIR)/sys/pixmaps/pend.png");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","h4:WSSTR  Option(O)"," オプション(O)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","i4:WSSTR Environment(E)    :m61:E,SP,Colors(C):m62:C,Fonts(F):m63:F,Look and Feel(L):m64:L","環境設定(E)      :m61:E,SP,色設定(C):m62:C,フォント設定(F):m63:F,ルックアンドフィール(L):m64:L");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","i4:WSSTR Environment(E)    :m61:E,SP,Colors(C):m62:C,Fonts(F):m63:F,Look and Feel(L):m64:L","環境設定(E)      :m61:E:0:$(WSDIR)/sys/pixmaps/pset.png,SP,色設定(C):m62:C,フォント設定(F):m63:F,ルックアンドフィール(L):m64:L");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","j4:WSSTR  Help(H)"," ヘルプ(H)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","k4:WSSTR Index(I):m81:I,Reference(R)     :m82:R,User's guide(U):m83:U,About WideStudio(V):m84:V,License(L):m85:L","目次(I):m81:I,リファレンス(R)     :m82:R,使い方(U):m83:U,バージョン情報(V):m84:V,ライセンス情報(L):m85:L");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","k4:WSSTR Index(I):m81:I,Reference(R)     :m82:R,User's guide(U):m83:U,About WideStudio(V):m84:V,License(L):m85:L","目次(I):m81:I:0:$(WSDIR)/sys/pixmaps/help.png,リファレンス(R)     :m82:R,使い方(U):m83:U,バージョン情報(V):m84:V,ライセンス情報(L):m85:L");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","l4:WSSTR Please specify a window to save.","保存したいウィンドウを選択して下さい.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","m4:WSSTR Continue saving?","保存を続けますか.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.utf8","n4:WSSTR Please specify some instances to copy.","コピーしたいインスタンスを選択してください.");
