@@ -1,6 +1,26 @@
+//
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
+//
+
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","","");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","A4:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:","ʣ    :COPY:,Žդ:PASTE:,SP,ץѥƥ:PROP:,ץ:OP:,Ƽ:EXPROP:,SP,:DEL:");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","A4:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:","ʣ    :COPY:,Žդ:PASTE:,SP,ץѥƥ:PROP:,ץ:OP:,Ƽ:EXPROP:,SP,̤:FRONT:,̤:BACK:,SP,:DEL:");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","B4:WSSTR Confirmation","");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","C4:WSSTR ObjectBox","֥ȥܥå");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","D4:WSSTR ProjectSettings","ץ");
@@ -23,21 +43,21 @@
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","T4:WSSTR Find files","ե븡");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","U4:WSSTR Help","إ");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","V4:WSSTR  File(F)"," ե(F)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","W4:WSSTR New window(N)         :m11:N,Open window(O):m12:O,Save window(S):m13:S,Save as(R):m14:R,Save all(A):m15:A,SP,Exit(X):m16:X","ɥ(N)      :m11:N,ɥɤ߹(O):m12:O,ɥ¸(S):m13:S,ɥ̾¸(R):m14:R,ɥ¸(A):m15:A,SP,ӥλ(X):m16:X");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","W4:WSSTR New window(N)         :m11:N,Open window(O):m12:O,Save window(S):m13:S,Save as(R):m14:R,Save all(A):m15:A,SP,Exit(X):m16:X","ɥ(N)      :m11:N:0:$(WSDIR)/sys/pixmaps/wcreate.png,ɥɤ߹(O):m12:O:0:$(WSDIR)/sys/pixmaps/wload.png,ɥ¸(S):m13:S:0:$(WSDIR)/sys/pixmaps/wsave.png,ɥ̾¸(R):m14:R:0:,ɥ¸(A):m15:A:0:,SP,ӥλ(X):m16:X:0:$(WSDIR)/sys/pixmaps/pend.png");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","X4:WSSTR  Edit(E)"," Խ(E)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","Y4:WSSTR Copy(C):m21:C,Paste(P):m22:P,SP,Display(I):m27:I,Find(F):m28:F,Delete(D):m23:D,SP,Properties(R):m24:R,Procedures:{List up(L):m25:L,New procedure(N):m251:N,Properties(S):m252:S,Edit(E):m253:E,Delete(X):m254:X},Attributes(A):m26:A,Files(L):m29:L","ʣ(C):m21:C,Žդ(P):m22:P,SP,ɽ(I):m27:I,󥹥󥹤θ(F):m28:F,(D):m23:D,SP,ץѥƥԽ(R):m24:R,ץԽ:{ץ(L):m25:L,ץ㿷(N):m251:N,ץ(S):m252:S,ץԽ(E):m253:E,ץ(X):m254:X},(A):m26:A,ե(L):m29:L");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","Y4:WSSTR Copy(C):m21:C,Paste(P):m22:P,SP,Display(I):m27:I,Find(F):m28:F,Delete(D):m23:D,SP,Properties(R):m24:R,Procedures:{List up(L):m25:L,New procedure(N):m251:N,Properties(S):m252:S,Edit(E):m253:E,Delete(X):m254:X},Attributes(A):m26:A,Files(L):m29:L","᤹(U):m291:U:1,ʣ(C):m21:C,Žդ(P):m22:P,SP,ɽ(I):m27:I,󥹥󥹤θ(F):m28:F,(D):m23:D,SP,ץѥƥԽ(R):m24:R,ץԽ:{ץ(L):m25:L,ץ㿷(N):m251:N,ץ(S):m252:S,ץԽ(E):m253:E,ץ(X):m254:X},(A):m26:A,ե(L):m29:L");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","Z4:WSSTR  Build(B)"," ӥ(B)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","a4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Execute(X):m36:X,Execute with Trace(T):m39:T,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","ѥ(C)    :m31:C,ӥɥ(B):m32:B,ӥ(R):m33:R,꡼(E):m34:E,SP,ǥХå(D):m35:D,¹(X):m36:X,ȥ졼¹(T):m39:T,SP,(G):m37:G,ν(I):m38:I");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","b4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Abort(X):m36:X,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","ѥ(C)    :m31:C,ӥɥ(B):m32:B,ӥ(R):m33:R,꡼(E):m34:E,SP,ǥХå(D):m35:D,¹(X):m36:X,SP,(G):m37:G,ν(I):m38:I");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","a4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Execute(X):m36:X,Execute with Trace(T):m39:T,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","ѥ(C)    :m31:C,ӥɥ(B):m32:B:0:$(WSDIR)/sys/pixmaps/make.png,ӥ(R):m33:R,꡼(E):m34:E,SP,ǥХå(D):m35:D,¹(X):m36:X:0:$(WSDIR)/sys/pixmaps/run.png,ȥ졼¹(T):m39:T,SP,(G):m37:G,ν(I):m38:I");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","b4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Abort(X):m36:X,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","ѥ(C)    :m31:C,ӥɥ(B):m32:B:0:$(WSDIR)/sys/pixmaps/make.png,ӥ(R):m33:R,꡼(E):m34:E,SP,ǥХå(D):m35:D,¹(X):m36:X:0:$(WSDIR)/sys/pixmaps/runx.png,SP,(G):m37:G,ν(I):m38:I");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","c4:WSSTR  View(V)"," ɽ(V)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","d4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Terminal(T):m45:T","ե(D) :m46:D,󥹥ڥ(I) :m47:I,󥹥󥹥ǥ(E) :m48:E,饹ꥹ(L) :m49:L,֥ȥܥå(O)     :m41:O,ݡͥ(C):m42:C,ġС(B):m43:B,ߥʥ(T):m45:T");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","e4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Find Files(F):m44:F,Terminal(T):m45:T","ե(D) :m46:D,󥹥ڥ(I) :m47:I,󥹥󥹥ǥ(E) :m48:E,饹ꥹ(L) :m49:L,֥ȥܥå(O)     :m41:O,ݡͥ(C):m42:C,ġС(B):m43:B,ե븡(F):m44:F,ߥʥ(T):m45:T");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","d4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Terminal(T):m45:T","ե(D) :m46:D,󥹥ڥ(I) :m47:I,󥹥󥹥ǥ(E) :m48:E,饹ꥹ(L) :m49:L,֥ȥܥå(O)     :m41:O:0:$(WSDIR)/sys/pixmaps/obox.png,ݡͥ(C):m42:C,ġС(B):m43:B,SP,ߥʥ(T):m45:T");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","e4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Find Files(F):m44:F,Terminal(T):m45:T","ե(D) :m46:D,󥹥ڥ(I) :m47:I,󥹥󥹥ǥ(E) :m48:E,饹ꥹ(L) :m49:L,֥ȥܥå(O)     :m41:O:0:$(WSDIR)/sys/pixmaps/obox.png,ݡͥ(C):m42:C,ġС(B):m43:B,SP,ե븡(F):m44:F,ߥʥ(T):m45:T");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","f4:WSSTR  Project(P)"," ץ(P)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","g4:WSSTR New project(N)        :m51:N,Open project(O):m52:O,Save project(S):m53:S,Rename project(R):m54:R,SP,Project settings(E):m55:E,SP,Close project(C):m56:C","ץ(N)      :m51:N,ץɤ߹(O) :m52:O,ץ¸(S):m53:S,ץ̾¸(R):m54:R,SP,ץ(E):m55:E,SP,ץȽλ(C):m56:C");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","g4:WSSTR New project(N)        :m51:N,Open project(O):m52:O,Save project(S):m53:S,Rename project(R):m54:R,SP,Project settings(E):m55:E,SP,Close project(C):m56:C","ץ(N)      :m51:N:0:$(WSDIR)/sys/pixmaps/pcreate.png,ץɤ߹(O) :m52:O:0:$(WSDIR)/sys/pixmaps/pload.png,ץ¸(S):m53:S:0:$(WSDIR)/sys/pixmaps/psave.png,ץ̾¸(R):m54:R,SP,ץ(E):m55:E:0:$(WSDIR)/sys/pixmaps/pset.png,SP,ץȽλ(C):m56:C:0:$(WSDIR)/sys/pixmaps/pend.png");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","h4:WSSTR  Option(O)"," ץ(O)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","i4:WSSTR Environment(E)    :m61:E,SP,Colors(C):m62:C,Fonts(F):m63:F,Look and Feel(L):m64:L","Ķ(E)      :m61:E,SP,(C):m62:C,ե(F):m63:F,åɥե(L):m64:L");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","i4:WSSTR Environment(E)    :m61:E,SP,Colors(C):m62:C,Fonts(F):m63:F,Look and Feel(L):m64:L","Ķ(E)      :m61:E:0:$(WSDIR)/sys/pixmaps/pset.png,SP,(C):m62:C,ե(F):m63:F,åɥե(L):m64:L");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","j4:WSSTR  Help(H)"," إ(H)");
-  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","k4:WSSTR Index(I):m81:I,Reference(R)     :m82:R,User's guide(U):m83:U,About WideStudio(V):m84:V,License(L):m85:L","ܼ(I):m81:I,ե(R)     :m82:R,Ȥ(U):m83:U,С(V):m84:V,饤󥹾(L):m85:L");
+  WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","k4:WSSTR Index(I):m81:I,Reference(R)     :m82:R,User's guide(U):m83:U,About WideStudio(V):m84:V,License(L):m85:L","ܼ(I):m81:I:0:$(WSDIR)/sys/pixmaps/help.png,ե(R)     :m82:R,Ȥ(U):m83:U,С(V):m84:V,饤󥹾(L):m85:L");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","l4:WSSTR Please specify a window to save.","¸ɥ򤷤Ʋ.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","m4:WSSTR Continue saving?","¸³ޤ.");
   WSGIappLocaleSet()->setLocaleString("ja_JP.eucjp","n4:WSSTR Please specify some instances to copy.","ԡ󥹥󥹤򤷤Ƥ.");
