@@ -1,6 +1,26 @@
+//
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
+//
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to
+// deal in the Software without restriction, including without limitation the
+// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+// sell copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
+//
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
+//
+
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","","");
-  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","A4:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:","    :COPY:,ٿֱ:PASTE:,SP,Ƽ:PROP:,ν:OP:,:EXPROP:,SP,:DEL:");
-  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","A4:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:","    :COPY:,ٿֱ:PASTE:,SP,Ƽ:PROP:,ν:OP:,:EXPROP:,SP,ո鿡:FRONT:,ڸ鿡:BACK:,SP,:DEL:");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","B4:WSSTR Confirmation","");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","C4:WSSTR ObjectBox","Ʈ ڽ");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","D4:WSSTR ProjectSettings","Ʈ ");
@@ -23,21 +43,21 @@
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","T4:WSSTR Find files"," ˻");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","U4:WSSTR Help","");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","V4:WSSTR  File(F)"," (F)");
-  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","W4:WSSTR New window(N)         :m11:N,Open window(O):m12:O,Save window(S):m13:S,Save as(R):m14:R,Save all(A):m15:A,SP,Exit(X):m16:X"," (N)      :m11:N, (O):m12:O, (S):m13:S, ٸ ̸ (R):m14:R,  (A):m15:A,SP, (X):m16:X");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","W4:WSSTR New window(N)         :m11:N,Open window(O):m12:O,Save window(S):m13:S,Save as(R):m14:R,Save all(A):m15:A,SP,Exit(X):m16:X"," (N)      :m11:N:0:$(WSDIR)/sys/pixmaps/wcreate.png, (O):m12:O:0:$(WSDIR)/sys/pixmaps/wload.png, (S):m13:S:0:$(WSDIR)/sys/pixmaps/wsave.png, ٸ ̸ (R):m14:R,  (A):m15:A,SP, (X):m16:X:0:$(WSDIR)/sys/pixmaps/pend.png");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","X4:WSSTR  Edit(E)"," (E)");
-  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","Y4:WSSTR Copy(C):m21:C,Paste(P):m22:P,SP,Display(I):m27:I,Find(F):m28:F,Delete(D):m23:D,SP,Properties(R):m24:R,Procedures:{List up(L):m25:L,New procedure(N):m251:N,Properties(S):m252:S,Edit(E):m253:E,Delete(X):m254:X},Attributes(A):m26:A,Files(L):m29:L","(C):m21:C,ٿֱ(P):m22:P,SP,ǥ(I):m27:I,νϽ ˻(F):m28:F,(D):m23:D,SP,Ƽ (R):m24:R,ν :{ν ϶(L):m25:L,ν  (N):m251:N,ν (S):m252:S,ν (E):m253:E,ν (X):m254:X},⺻(A):m26:A, ϶(L):m29:L");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","Y4:WSSTR Copy(C):m21:C,Paste(P):m22:P,SP,Display(I):m27:I,Find(F):m28:F,Delete(D):m23:D,SP,Properties(R):m24:R,Procedures:{List up(L):m25:L,New procedure(N):m251:N,Properties(S):m252:S,Edit(E):m253:E,Delete(X):m254:X},Attributes(A):m26:A,Files(L):m29:L"," (U):m291:U:1,(C):m21:C,ٿֱ(P):m22:P,SP,ǥ(I):m27:I,νϽ ˻(F):m28:F,(D):m23:D,SP,Ƽ (R):m24:R,ν :{ν ϶(L):m25:L,ν  (N):m251:N,ν (S):m252:S,ν (E):m253:E,ν (X):m254:X},⺻(A):m26:A, ϶(L):m29:L");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","Z4:WSSTR  Build(B)"," (B)");
-  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","a4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Execute(X):m36:X,Execute with Trace(T):m39:T,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","(C)    :m31:C, (B):m32:B,(R):m33:R,Ŭ(E):m34:E,SP,(D):m35:D,(X):m36:X,Ʈ̽ (T):m39:T,SP,ҽ (G):m37:G, (I):m38:I");
-  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","b4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Abort(X):m36:X,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","(C)    :m31:C, (B):m32:B,(R):m33:R,Ŭ(E):m34:E,SP,(D):m35:D,(X):m36:X,SP,ҽ (G):m37:G, (I):m38:I");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","a4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Execute(X):m36:X,Execute with Trace(T):m39:T,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","(C)    :m31:C, (B):m32:B:0:$(WSDIR)/sys/pixmaps/make.png,(R):m33:R,Ŭ(E):m34:E,SP,(D):m35:D,(X):m36:X:0:$(WSDIR)/sys/pixmaps/run.png,Ʈ̽ (T):m39:T,SP,ҽ (G):m37:G, (I):m38:I");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","b4:WSSTR Compile(C)    :m31:C,Build all(B):m32:B,Rebuild(R):m33:R,Clean(E):m34:E,SP,Debug(D):m35:D,Abort(X):m36:X,SP,Generate source(G):m37:G,Generate inf file(I):m38:I","(C)    :m31:C, (B):m32:B:0:$(WSDIR)/sys/pixmaps/make.png,(R):m33:R,Ŭ(E):m34:E,SP,(D):m35:D,(X):m36:X:0:$(WSDIR)/sys/pixmaps/runx.png,SP,ҽ (G):m37:G, (I):m38:I");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","c4:WSSTR  View(V)"," (V)");
-  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","d4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Terminal(T):m45:T","޴(D) :m46:D,ν(I) :m47:I,νϽ (E) :m48:E,Ŭ Ʈ(L) :m49:L,Ʈ ڽ(O)     :m41:O,Ʈ(C):m42:C, (B):m43:B,͹̳(T):m45:T");
-  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","e4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Find Files(F):m44:F,Terminal(T):m45:T","޴(D) :m46:D,ν(I) :m47:I,νϽ (E) :m48:E,Ŭ Ʈ(L) :m49:L,Ʈ ڽ(O)     :m41:O,Ʈ(C):m42:C, (B):m43:B, ˻(F):m44:F,͹̳(T):m45:T");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","d4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Terminal(T):m45:T","޴(D) :m46:D,ν(I) :m47:I,νϽ (E) :m48:E,Ŭ Ʈ(L) :m49:L,Ʈ ڽ(O)     :m41:O:0:$(WSDIR)/sys/pixmaps/obox.png,Ʈ(C):m42:C, (B):m43:B,SP,͹̳(T):m45:T");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","e4:WSSTR ObjectBox(O) :m41:O,Component(C)    :m42:C,ToolBar(B):m43:B,Find Files(F):m44:F,Terminal(T):m45:T","޴(D) :m46:D,ν(I) :m47:I,νϽ (E) :m48:E,Ŭ Ʈ(L) :m49:L,Ʈ ڽ(O)     :m41:O:0:$(WSDIR)/sys/pixmaps/obox.png,Ʈ(C):m42:C, (B):m43:B,SP, ˻(F):m44:F,͹̳(T):m45:T");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","f4:WSSTR  Project(P)"," Ʈ(P)");
-  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","g4:WSSTR New project(N)        :m51:N,Open project(O):m52:O,Save project(S):m53:S,Rename project(R):m54:R,SP,Project settings(E):m55:E,SP,Close project(C):m56:C","Ʈ  (N)      :m51:N,Ʈ (O) :m52:O,Ʈ (S):m53:S,Ʈ ٸ ̸ (R):m54:R,SP,Ʈ (E):m55:E,SP,Ʈ ݱ(C):m56:C");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","g4:WSSTR New project(N)        :m51:N,Open project(O):m52:O,Save project(S):m53:S,Rename project(R):m54:R,SP,Project settings(E):m55:E,SP,Close project(C):m56:C","Ʈ  (N)      :m51:N:0:$(WSDIR)/sys/pixmaps/pcreate.png,Ʈ (O) :m52:O:0:$(WSDIR)/sys/pixmaps/pload.png,Ʈ (S):m53:S:0:$(WSDIR)/sys/pixmaps/psave.png,Ʈ ٸ ̸ (R):m54:R,SP,Ʈ (E):m55:E:0:$(WSDIR)/sys/pixmaps/pset.png,SP,Ʈ ݱ(C):m56:C:0:$(WSDIR)/sys/pixmaps/pend.png");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","h4:WSSTR  Option(O)"," ɼ(O)");
-  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","i4:WSSTR Environment(E)    :m61:E,SP,Colors(C):m62:C,Fonts(F):m63:F,Look and Feel(L):m64:L","ȯ漳(E)      :m61:E,SP, (C):m62:C,۲ (F):m63:F,Ѻ(L):m64:L");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","i4:WSSTR Environment(E)    :m61:E,SP,Colors(C):m62:C,Fonts(F):m63:F,Look and Feel(L):m64:L","ȯ漳(E)      :m61:E:0:$(WSDIR)/sys/pixmaps/pset.png,SP, (C):m62:C,۲ (F):m63:F,Ѻ(L):m64:L");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","j4:WSSTR  Help(H)"," (H)");
-  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","k4:WSSTR Index(I):m81:I,Reference(R)     :m82:R,User's guide(U):m83:U,About WideStudio(V):m84:V,License(L):m85:L","(I):m81:I,۷(R)     :m82:R,(U):m83:U,(V):m84:V,̼(L):m85:L");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","k4:WSSTR Index(I):m81:I,Reference(R)     :m82:R,User's guide(U):m83:U,About WideStudio(V):m84:V,License(L):m85:L","(I):m81:I:0:$(WSDIR)/sys/pixmaps/help.png,۷(R)     :m82:R,(U):m83:U,(V):m84:V,̼(L):m85:L");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","l4:WSSTR Please specify a window to save."," 带 ּ.");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","m4:WSSTR Continue saving?"," մϱ?");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","n4:WSSTR Please specify some instances to copy."," νϽ ּ.");
@@ -51,7 +71,7 @@
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","v4:WSSTR Please specify a application window to compile.","   ּ.");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","w4:WSSTR Please check the debugger settings or confirm whether the project is build.","  ùٸ, ũǾ ִ Ȯּ.");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","x4:WSSTR Can not execute the class library...","̺귯  մϴ.");
-  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","y4:WSSTR Can not execute the load module.\nPlease confirm whether the project is collect or build.","ε մϴ.\nƮ ùٸ, ũǾ ִ Ȯּ.");
+  WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","y4:WSSTR Can not execute the load module.\nPlease confirm whether the project is collect or built.","ε մϴ.\nƮ ùٸ, ũǾ ִ Ȯּ.");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","z4:WSSTR Please specify a application window to generate source code.","ҽ   ּ.");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","A4:WSSTR Please wait for a while to start web browser.."," ֽϴ. ٷּ...");
   WSGIappLocaleSet()->setLocaleString("ko_KR.euckr","B4:WSSTR NoName","");
