@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCwindow.h>
@@ -85,7 +81,7 @@ WSCwizardDialog*     newwin;
 WSCwizardDialog*     newprj;
 WSCwizardDialog*     searchwz;
 WSCbaseDialog*     objsearchwz;
-WSCform*           toolbar;
+WSCfform*           toolbar;
 WSCpopupMenu*      popupmenu;
 WSCpopupMenu*      popupmenu2;
 //WSCprocessBox*     procbox;
@@ -101,14 +97,21 @@ WSCvlabel*         show_dir1;
 WSCvlabel*         show_prj2;
 WSCvlabel*         show_dir2;
 
-#define WS_PM_MENU1 WSGFgettext("A4:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:")
-#define WS_PM_MENU2 WSGFgettext("A4:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:")
+#define WS_PM_MENU1 WSGFgettext("A3:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Delete:DEL:")
+#define WS_PM_MENU2 WSGFgettext("A3:WSSTR Copy    :COPY:,Paste:PASTE:,SP,Properties:PROP:,Procedures:OP:,Attributes:EXPROP:,SP,Front:FRONT:,Back:BACK:,SP,Delete:DEL:")
 
 #define WS_TITLE_WIDTH   200
 #define WS_TITLE_HEIGHT  200
 WSCwindow* title =  NULL;
 WSCform*       titlei = NULL;
 
+WSCstring _get_java_package_name_handler(){
+  WSCstring pkgname = pe->getProjectData("#JAVA_PKG_NAME");
+  pkgname.delHeadSpace();
+  pkgname.delTailSpace();
+  return pkgname;
+}
+WSClistData _dll_list;
 WSCbase* _create_instance_dll_load_handler(char* cname,WSCbase* parent,char* name){
   WSCstring reqlib = objectbox->getRequiredLibrary(cname);
   if (!strcmp((char*)reqlib,"")){
@@ -119,21 +122,34 @@ extern WSCstring get_complete_libname(char*);
 //printf("reqlib=%s\n",(char*)reqlib);
   WSCstring libname = get_complete_libname((char*)reqlib);
 //printf("comp_reqlib=%s\n",(char*)libname);
-  long ret = newext->open((char*)libname);
+  long ret = newext->open((char*)libname.getString(WSGIappLocaleSet()->getSystemLocaleEncoding()));
   if (ret != WS_NO_ERR){
 //printf("Library %s not found.\n",(char*)libname);
     return NULL;
   }
+  _dll_list.add((void*)newext);
   WSCbase* obj = WSCbase::getNewInstance(cname,parent,name);
   return obj;
 
 }
+void pe_update_work(WSCbase* obj){
+  if (obj->getVisible() == False){
+    inspector->updateList();
+  }
+}
 
 long _get_lang_hd(){
   return pe->getLanguage();
 }
+long _get_mapping_hd(){
+  if (pe->useMpfcMapping() != False){
+    return WS_MAPPING_MPFC;
+  }else{
+    return WS_MAPPING_MWT;
+  }
+}
 
-#define WS_TITLE_COMMAND "$(WSDIR)/bin/wstitle"
+#define WS_TITLE_COMMAND "$(WSDIR)/bin/wstitle &"
 extern char* get_project_dir();
 long create_proc_for_template(char* pname,char* base_cname){
   WSCstring procstr;
@@ -161,6 +177,20 @@ long create_proc_for_template(char* pname,char* base_cname){
       fpname.replaceString(".cpp",".pl",1);
     }
     pname2.replaceString(".pl","",1);
+  }else
+  if (pe->getLanguage() == WS_LANG_JAVA){
+    if (pname2.isExist(".cpp") != -1){
+      pname2.replaceString(".cpp","",1);
+      fpname.replaceString(".cpp",".java",1);
+    }
+    pname2.replaceString(".java","",1);
+  }else
+  if (pe->getLanguage() == WS_LANG_OCAML){
+    if (pname2.isExist(".cpp") != -1){
+      pname2.replaceString(".cpp","",1);
+      fpname.replaceString(".cpp",".ml",1);
+    }
+    pname2.replaceString(".ml","",1);
   }else{
     pname2.replaceString(".cpp","",1);
   }
@@ -178,6 +208,12 @@ long create_proc_for_template(char* pname,char* base_cname){
     }else
     if (lang == WS_LANG_PERL){
       WSGFopCreatePerl(&procstr,pname2,NULL);
+    }else
+    if (lang == WS_LANG_JAVA){
+      WSGFopCreateJava(&procstr,pname2,NULL);
+    }else
+    if (lang == WS_LANG_OCAML){
+      WSGFopCreateOcaml(&procstr,pname2,NULL);
     }
   }else{
     WSGFclassProcedureCreate(&procstr,pname2,NULL,base_cname);
@@ -210,7 +246,7 @@ WSCbool _arrow_key_for_selected_obj(long key,WSCbool fl){
         key == WSK_Right){
       WSCbase* focusobj = WSCbase::getFocusInstance();
       if (focusobj == NULL){
-        return False;
+        return True;
       }
       long hit=0;
       long i;
@@ -338,14 +374,15 @@ extern int _system(char*);
   system(WSGIappFileSystem()->adjustFileName(WS_TITLE_COMMAND));
 #endif
 
+  WSGFenablePropertyBuilderTitle();
   WSCscrForm::setUseScrBarClass("WSCvbscrBar");
   WSCtextField::setUseScrBarClass("WSCvbscrBar");
 
   WSGFsetGetLanguageHandler(_get_lang_hd);
+  WSGFsetGetMappingHandler(_get_mapping_hd);
 
   //changed by hirabayashi 1999 02/10
 //  WSGIappColorSet()->initialize("$(WSDIR)/sys/data/color.data");
-  //ѥեȤӥѥեȤ˥åȤ롣
   WSGIappFontSet()->setDefaultFont(8);
 
   WSGIappColorSelect();
@@ -381,6 +418,7 @@ extern void create_main_window();
   message->setPropertyV(WSNheight,(WSCushort)125);
   message->setPropertyV(WSNlabelPixmap,WS_MSG_PIX);
   message->setPropertyV(WSNtitleString,"Message");
+  message->setPropertyV(WSNcancel,False);
 
   question = new WSCmessageDialog(NULL,"bl-question");
   question->initialize();
@@ -392,8 +430,17 @@ extern void create_main_window();
   question->setPropertyV(WSNlabelPixmap,WS_MSG_PIX);
   question->setPropertyV(WSNtitleString,WSGFgettext("B4:WSSTR Confirmation"));
 
+  WSCstring loc = WSGIappLocaleSet()->getSystemLocaleName();
+  if (!strcmp((char*)loc,"C")){
+    loc = "en_US";
+  }
   WSCstring datadir;
-  datadir << "$(WSDIR)/sys/data/locale/" << WSGIappLocaleSet()->getSystemLocaleName();
+  datadir << "$(WSDIR)/sys/data/locale/" << loc << "/lib1.data";
+  if (WSGFcheckExistFile(WSGIappFileSystem()->adjustFileName((char*)datadir)) == False){
+    loc = "en_US";
+  }
+  datadir ="";
+  datadir << "$(WSDIR)/sys/data/locale/" << loc;
   WSCstring data;
   data << datadir << "/lib1.data,";
   data << datadir << "/lib2.data,";
@@ -417,7 +464,7 @@ extern void create_main_window();
   pe->setPropertyV(WSNy,(short)10);
 
   pe->setPropertyV(WSNwidth,(WSCushort)840);
-  pe->setPropertyV(WSNheight,(WSCushort)350);
+  pe->setPropertyV(WSNheight,(WSCushort)400);
   pe->setPropertyV(WSNtitleString,WSGFgettext("D4:WSSTR ProjectSettings"));
   pe->setPropertyV(WSNlabelPixmap, WS_DIALOG_PIX1 );
   pe->setObjectBox(objectbox);
@@ -425,6 +472,11 @@ extern void create_main_window();
   pe->loadEditorOptions("$(WSDIR)/sys/data/editors");
   pe->loadLinkOptions("$(WSDIR)/sys/data/linkoptions");
 
+  WSCprocedure* updateop = new WSCprocedure("updateop",WSEV_VISIBLE_CH);
+  updateop->setFunction(pe_update_work,"pe_update_work");
+  pe->addProcedure(updateop);
+
+
 extern void create_newprj_dialog();
   create_newprj_dialog();
 
@@ -432,22 +484,26 @@ extern void _create_win_tracewin();
   _create_win_tracewin();
 
   WSCstring defprj;
-  defprj << "$(WSDIR)/sys/data/locale/" << WSGIappLocaleSet()->getSystemLocaleName()
-         << "/default.prj";
+  defprj << "$(WSDIR)/sys/data/locale/" << loc << "/default.prj";
   pe->setDefaultProjectName(defprj);
   pe->open(defprj);
-#if MSW
+
+  WSCstring defrc;
+  defrc << "$(WSDIR)/sys/data/locale/" << loc << "/default.rc";
+  pe->open_global(defrc);
+
+#ifdef MSW
   long ret = pe->open_global("$(WINDIR)/.wsbuilderrc");
 #else
   long ret = pe->open_global("$(HOME)/.wsbuilderrc");
 #endif
+#if 0 //move above..
   if (ret != WS_NO_ERR){
     WSCstring defrc;
-    defrc << "$(WSDIR)/sys/data/locale/" << WSGIappLocaleSet()->getSystemLocaleName()
-           << "/default.rc";
+    defrc << "$(WSDIR)/sys/data/locale/" << loc << "/default.rc";
     pe->open_global(defrc);
   }
-
+#endif
 extern void create_newwin_dialog();
   create_newwin_dialog();
 
@@ -582,7 +638,6 @@ extern void _create_win_policy_select();
 
 
 #if 0
-  //̾ΥåϥɥϿ
 extern long check_name_strings(char*,WSCbase*);
   WSGFsetExistNameHandler(check_name_strings);
 #endif
@@ -598,7 +653,6 @@ extern void load_window_style();
 
 extern void adjust_mainwindow(WSCbase*);
   adjust_mainwindow(mainwindow);  
-  //ѥեȤȤ᤹
 //  WSGIappFontSet()->setDefaultFont(0);
 extern long check_name_strings(char*,WSCbase*);
   WSGFsetExistNameHandler(check_name_strings);
@@ -606,6 +660,8 @@ extern long check_name_strings(char*,WSCbase*);
   WSCtextField::setUseScrBarClass(NULL);
   WSCbase::setCreateInstanceHandler(_create_instance_dll_load_handler);
 
+  WSGFsetJavaPackageNameHandler((void*)_get_java_package_name_handler);
+
 extern void _gui_policy_change();
   _gui_policy_change();
 
