@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,20 +14,23 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSClocaleSet.h>
 #include <WSCinitializeManager.h>
 
+#if defined (_VC) || defined (_BCC)
+#define NO_JAPANESE_EUC_MSG
+#define NO_JAPANESE_UTF8_MSG
+#define NO_KSC5601_MSG
+#define NO_GB2312_MSG
+#endif
+
 #ifndef NO_ENGLISH_MSG
 void _wsbuilder_lang_init_proc(){
 #include "English.cpp"
@@ -61,6 +64,13 @@ void _wsbuilder_lang_init_proc5(){
 #ifndef NO_ES_MSG
 void _wsbuilder_lang_init_proc6(){
 #include "Espanol.cpp"
+#include "Espanol-ar.cpp"
+}
+#endif
+#ifndef NO_ES_UTF8_MSG
+void _wsbuilder_lang_init_proc7(){
+#include "Espanol-utf8.cpp"
+#include "Espanol-ar-utf8.cpp"
 }
 #endif
 
@@ -87,6 +97,9 @@ public:
 #endif
 #ifndef NO_ES_MSG
     WSGIappInitializeManager()->addInitializeProc(_wsbuilder_lang_init_proc6);
+#endif
+#ifndef NO_ES_UTF8_MSG
+    WSGIappInitializeManager()->addInitializeProc(_wsbuilder_lang_init_proc7);
 #endif
   };
 };
