@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
 #include <WSCbase.h>
@@ -46,6 +42,7 @@
 #include <WSCoption.h>
 #include <WSClocaleSet.h>
 #include <WSDappDev.h>
+#include <WSCundoManager.h>
 
 extern WSClistData       selected_clients;
 extern WSClistData       copy_clients;
@@ -91,6 +88,12 @@ void setup_instance(WSCbase* target){
 //printf("setup handler:  %s\n",target->getInstanceName());
   _register_drop_site(target);
   _register_popup_menu(target);
+#ifndef NO_MWT_FUNCTION
+#ifndef NO_INDEX_VARIANT_DATA
+  WSGIappUndoManager()->clear();
+#endif
+#endif
+
 #if 0 /* move to make_execute*/
 extern WSCobjectBox* objectbox;
   WSCstring reqlib = objectbox->getRequiredLibrary(target->getClassName());
@@ -109,6 +112,62 @@ void del_instance(WSCbase* target){
   if (parent == NULL){
     winlist.del(target);
   }
+  long val = (long)target->getUserData(WS_NO_DEL_BUILDER_DATA);
+  if (val == 0){
+    WSClistData* property_list =(WSClistData*)target->getUserData(WS_PROP_LIST);
+    if (property_list != NULL){
+      long num = property_list->getNum();
+      long i;
+      for(i=0; i< num; i++){
+        prop_data* propdata = (prop_data*)(*property_list)[i];
+        delete propdata;
+      }
+      delete property_list;
+      target->setUserData(WS_PROP_LIST,NULL);
+    }
+    WSClistData* utrg_list =(WSClistData*)target->getUserData(WS_UTRG_LIST);
+    if (utrg_list != NULL){
+      long num = utrg_list->getNum();
+      long i;
+      for(i=0; i< num; i++){
+        utrg_data* utrgdata = (utrg_data*)(*utrg_list)[i];
+        delete utrgdata;
+      }
+      delete utrg_list;
+      target->setUserData(WS_UTRG_LIST,NULL);
+    }
+    WSClistData* trg_list =(WSClistData*)target->getUserData(WS_TRG_LIST);
+    if (trg_list != NULL){
+      delete trg_list;
+      target->setUserData(WS_TRG_LIST,NULL);
+    }
+    WSClistData* rmtd_list =(WSClistData*)target->getUserData(WS_RMETHOD_LIST);
+    if (rmtd_list != NULL){
+      long num = rmtd_list->getNum();
+      long i;
+      for(i=0; i< num; i++){
+        rmethod_data* rmdata = (rmethod_data*)(*rmtd_list)[i];
+        delete rmdata;
+      }
+      delete rmtd_list;
+      target->setUserData(WS_RMETHOD_LIST,NULL);
+    }
+    WSClistData* rcname =(WSClistData*)target->getUserData(WS_RCLASS_NAME);
+    if (rcname != NULL){
+      delete rcname;
+      target->setUserData(WS_RCLASS_NAME,NULL);
+    }
+    WSClistData* rcbname =(WSClistData*)target->getUserData(WS_RCLASS_BNAME);
+    if (rcbname != NULL){
+      delete rcbname;
+      target->setUserData(WS_RCLASS_BNAME,NULL);
+    }
+  }
+#ifndef NO_MWT_FUNCTION
+#ifndef NO_INDEX_VARIANT_DATA
+  WSGIappUndoManager()->clear();
+#endif
+#endif
 //printf("delete handler: done.\n");
 }
 
@@ -273,7 +332,9 @@ void idel_proc(WSCbase* target){
     str << WSGFgettext("C4:WSSTR Delete instance:") << client->getInstanceName()
         << WSGFgettext("D4:WSSTR ?");
     message->setPropertyV(WSNlabelString,(char*)str);  
+    message->setPropertyV(WSNcancel,(WSCbool)True);
     long ret = message->popup();
+    message->setPropertyV(WSNcancel,(WSCbool)False);
     if (ret == WS_DIALOG_OK){
       selected_clients.clear();
       copy_clients.clear();
@@ -384,14 +445,18 @@ void ipaste_work2(WSCbase* menu){
     ipaste_proc(target);
   }
 }
-
+void _delay_ipaste_proc(WSCuchar clk,void* ptr){
+  WSCbase* target = (WSCbase*)ptr;
+  ipaste_proc(target);
+}
 void ipaste_work(WSCbase*){
 //printf("ipaste..\n");
   WSCbase* target = (WSCbase*)inspector->getPopupMenuSelectedClient();
   if (target == NULL){
     return;
   }
-  ipaste_proc(target);
+  WSGIappTimer()->addTriggerProc(_delay_ipaste_proc,WS2000MS,target);
+//  ipaste_proc(target);
 }
 
 void icopy_proc(WSCbase* target){
@@ -532,14 +597,14 @@ extern char* get_make_name();
 #else /* MSW */
   if (target->getUserData(WS_WIN_CLASS)){
     fname = target->getInstanceName();
-    str << "cd " << get_project_dir() << ";";
+    str << "cd \"" << get_project_dir() << "\";";
     str << "/bin/rm -f " << fname << ".o " << fname << "P.o ;";
 extern char* get_make_name();
 //    str << "make " << fname << ".o " << fname << "P.o 2>&1 | tee log.txt";
     str << get_make_name() << " " << fname << ".o " << fname << "P.o 2>&1 | tee log.txt";
   }else{
     fname = target->getInstanceName();
-    str << "cd " << get_project_dir() << ";";
+    str << "cd \"" << get_project_dir() << "\";";
     str << "/bin/rm -f " << fname << ".o ;";
 extern char* get_make_name();
 //    str << "make " << fname << ".o 2>&1 | tee log.txt";
@@ -674,7 +739,9 @@ long read_win_file(char* dirname,char* fname,WSClistData* wlist){
       message->setPropertyV(WSNreplyHandler,(void*)NULL);
       message->setPropertyV(WSNlabelString,(char*)buffer);
       message->setPropertyV(WSNno,(WSCbool)True);
+      message->setPropertyV(WSNcancel,(WSCbool)True);
       long ret = message->popup();
+      message->setPropertyV(WSNcancel,(WSCbool)False);
       message->setPropertyV(WSNwidth,(WSCushort)450);
       message->setPropertyV(WSNheight,(WSCushort)125);
       message->setPropertyV(WSNno,(WSCbool)False);
@@ -693,7 +760,9 @@ long read_win_file(char* dirname,char* fname,WSClistData* wlist){
     message->setPropertyV(WSNwidth,(WSCushort)500);
     message->setPropertyV(WSNheight,(WSCushort)135);
     message->setPropertyV(WSNno,(WSCbool)True);
+    message->setPropertyV(WSNcancel,(WSCbool)True);
     long ret = message->popup();
+    message->setPropertyV(WSNcancel,(WSCbool)False);
     message->setPropertyV(WSNno,(WSCbool)False);
     message->setPropertyV(WSNwidth,(WSCushort)450);
     message->setPropertyV(WSNheight,(WSCushort)125);
@@ -1078,9 +1147,9 @@ void _drop_proc(WSCbase* b1,WSCbase* b2){
     newobj->setPropertyEditMode(True);
     newobj->setVisible(False);
 
-    //֥ȤФƲΤǤФΥϥɥ¹
+    //֥ȤФƲΤǤФΥϥɥ¹...
     WSGFexecCreateHandler(newobj);
-    //֥Ȥ˲ΤǤФΥϥɥϿ
+    //֥Ȥ˲ΤǤФΥϥɥϿ...
     void* delete_handler = (void*)WSGFgetDeleteHandler();
     if (delete_handler != NULL){
          WSCprocedure* ac = new WSCprocedure("delete-handler",WSEV_DELETE);
@@ -1118,7 +1187,7 @@ extern WSCprojectEditor* pe;
     if ( dw != 0){
       if (newobj->existProperty(WSNwidth) != False){
         WSCproperty* prop = newobj->getPropObj(WSNwidth);
-        if (prop->getBuilderVisible() != False){
+        if (prop != NULL && prop->getBuilderVisible() != False){
           newobj->setPropertyV(WSNwidth,dw);
         }
       }
@@ -1126,14 +1195,14 @@ extern WSCprojectEditor* pe;
     if ( dh != 0){
       if (newobj->existProperty(WSNheight) != False){
         WSCproperty* prop = newobj->getPropObj(WSNheight);
-        if (prop->getBuilderVisible() != False){
+        if (prop != NULL && prop->getBuilderVisible() != False){
           newobj->setPropertyV(WSNheight,dh);
         }
       }
     }
     if (newobj->existProperty(WSNshadowThickness) != False){
       WSCproperty* prop = newobj->getPropObj(WSNshadowThickness);
-      if (prop->getBuilderVisible() != False){
+      if (prop != NULL && prop->getBuilderVisible() != False){
         newobj->setPropertyV(WSNshadowThickness,(WSCuchar)dt);
       }
     }
@@ -1308,7 +1377,7 @@ extern WSCbase* copy_recursive(WSCbase*,WSCbase*);
   }
   inspector->updateList();
 }
-
+#if 0// moved to advance/WSadcom.cpp
 WSCbase* copy_recursive(WSCbase* item,WSCbase* target){
   WSCbase* newobj = WSCbase::getNewInstance(item->getClassName(),target,
          WSGFcreateName(target->getParentWindow()->getInstanceName(),
@@ -1370,7 +1439,7 @@ WSCbase* copy_recursive(WSCbase* item,WSCbase* target){
   return newobj;
 
 }
-
+#endif
 WSCstring _get_structure_inf_work(WSCbase* base,int level){
   WSCstring ret;
   WSClistData children = base->getChildren();
@@ -1411,6 +1480,9 @@ WSCstring _get_structure_inf_work(WSCbase* base,int level){
       }else
       if (lang == WS_LANG_PERL){
         ret << ac->getFunctionName()<<".pl\n";
+      }else
+      if (lang == WS_LANG_JAVA){
+        ret << ac->getFunctionName()<<".java\n";
       }
     }
   }
