@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <x11/WSDxappDev.h>
 #include <WScom.h>
@@ -49,12 +45,17 @@ WSMFclassInit(WSDunixExecute,WSDexecute);
 WSDexecute* _unix_execute_chandler(){
   return new WSDunixExecute;
 }
+void WSGFunixExecuteInit(){
+    WSDexecute::setCreateInstanceHandler((void*)_unix_execute_chandler);
+}
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _unix_execute_init {
   public: _unix_execute_init(){
-    WSDexecute::setCreateInstanceHandler((void*)_unix_execute_chandler);
+    WSGFunixExecuteInit();
   };
 };
 static _unix_execute_init  _unix_execute_init_execute;
+#endif
 
 WSDunixExecute::WSDunixExecute(){
 //  _xid = 0;
@@ -100,6 +101,7 @@ long WSDunixExecute::execute(char* exename,long mode){
   _exe_name.delHeadSpace();
   char* exe_name = _exe_name.getString(WSGIappLocaleSet()->getSystemLocaleEncoding());
   char* dir_name = _dir_name.getString(WSGIappLocaleSet()->getSystemLocaleEncoding());
+//printf("WSDunixExecute::execute exe_name=#%s# dir_name=#%s#\n",exe_name,dir_name);
 
 #ifdef MacOS
   WSCstring tmp1;
@@ -150,7 +152,7 @@ extern char* utf8_to_macutf8(char*);
                 (void*)(XtInputReadMask | XtInputWriteMask),
                  _callback,(void*)this);
 #endif
-      pthread_t thr;
+      pthread_t thr = 0;
       int ret = pthread_create(&thr,NULL,_read_thread_,this);
       if (ret != 0){
         return WS_ERR;
@@ -344,14 +346,14 @@ void* WSDunixExecute::_read_thread_(void* ptr){
       cm.display = disp;
       cm.window = win;
       cm.message_type = atom;
-      cm.format = 32;
+      cm.format = 8;
 
       pthread_sem_t* sem = new pthread_sem_t;
       _this->_sem = (void*)sem;
       pthread_sem_init_np(sem);
       pthread_sem_lock_np(sem);
 
-      memcpy(cm.data.l,&data,sizeof(_read_event_send_type));
+      memcpy(cm.data.b,&data,sizeof(_read_event_send_type));
       XSendEvent(disp,win,False,NoEventMask,(_XEvent *)&cm);
 //printf("send message... win=0x%x\n",win);
       Window  win2  = XCreateSimpleWindow(disp,DefaultRootWindow(disp),400,400,
