@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,17 +14,14 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <WScom.h>
+#ifndef WS_EMBED
 #include <x11/WSDunixExternal.h>
 #include <stdio.h>
 #include <WSClocaleSet.h>
@@ -38,28 +35,85 @@
 
 #include <mach-o/dyld.h>
 #define RTLD_LAZY 0
-void* dlopen(char* fname,long){
-    int result;
-    NSObjectFileImage handle;
-    result = NSCreateObjectFileImageFromFile(fname, &handle);
-    if (result != NSObjectFileImageSuccess) {
-      printf("Failed to load file:%s\n", fname);
-      return NULL;
+
+#include <WSClistData.h>
+#include <WSDenv.h>
+
+static void expand_list(char* fname, WSCstring* path_list, WSClistData* outlist) {
+  if (path_list->getChars() <= 0) return;
+
+  for (long i = 0; i < path_list->getWords(":"); i++) {
+    WSCstring tmp = path_list->getWord(i, ":");
+    if (tmp.getChars() > 0) {
+      tmp << "/" << fname;
+      WSCstring* path = new WSCstring(tmp);
+      outlist->add((void*)path);
     }
-    NSLinkModule(handle,fname, TRUE);
-    return handle;
+  }
+}
+
+static void make_dl_path_list(char* fname, WSClistData* outlist) {
+  if (fname[0] == '/') {
+    outlist->add((void*)(new WSCstring(fname)));
+    return;
+  }
+
+  static char* env_list[] = {
+    "LD_LIBRARY_PATH",
+    "DYLD_LIBRARY_PATH"
+  };
+  static char* env_fallback = "DYLD_FALLBACK_LIBRARY_PATH";
+
+  WSCstring path_list;
+
+  for (unsigned long i = 0; i < sizeof(env_list); i++) {
+    path_list = WSGIappEnvironment()->getEnv(env_list[i]);
+    expand_list(fname, &path_list, outlist);
+  }
+
+  path_list = WSGIappEnvironment()->getEnv(env_fallback);
+  if (path_list.getChars() <= 0) {
+    path_list = WSGIappEnvironment()->getPlaneString("$(HOME)/lib");
+    path_list << ":" << "/usr/local/lib" << ":" << "/usr/lib";
+  }
+  expand_list(fname, &path_list, outlist);
+  outlist->add((void*)(new WSCstring(fname)));
+
+  return;
+}
+static void delete_items(WSClistData* list) {
+  for (long i = 0; i < list->getNum(); i++) {
+    delete (WSCstring*)list->getData(i);
+  }
+}
+
+void* dlopen(char* fname,long){
+  WSClistData list;
+  make_dl_path_list(fname, &list);
+
+  for (long i = 0; i < list.getNum(); i++) {
+    NSObjectFileImage handle;
+    WSCstring* path = (WSCstring*)list.getData(i);
+    int result = NSCreateObjectFileImageFromFile(path->getString(), &handle);
+    if (result == NSObjectFileImageSuccess) {
+      NSLinkModule(handle,fname, NSLINKMODULE_OPTION_NONE);
+      delete_items(&list);
+      return handle;
+    }
+  }
+  fprintf(stderr,"Failed to load file:%s\n", fname);
+  delete_items(&list);
+  return NULL;
 };
 void* dlsym(void* ptr,char* procn){
-    WSCstring procn2("_");
-    procn2 << procn;
-    if(NSIsSymbolNameDefined(procn)) {
-      printf("Failed to find symbol=%s\n",procn);
-    }
-    /* NSLookupAndBindSymbol require function name with "_" !! */
-    NSSymbol sym = NSLookupAndBindSymbol(procn2.getString());
-    void* ret = (void*)NSAddressOfSymbol(sym);
-
-    return ret;
+  WSCstring procn2("_");
+  procn2 << procn;
+  if(NSIsSymbolNameDefined(procn)) {
+    printf("Failed to find symbol=%s\n",procn);
+  }
+  /* NSLookupAndBindSymbol require function name with "_" !! */
+  NSSymbol sym = NSLookupAndBindSymbol(procn2.getString());
+  void* ret = (void*)NSAddressOfSymbol(sym);
   return (void*)ret;
 }
 #endif //MacOS
@@ -69,14 +123,17 @@ WSMFclassInit(WSDunixExternal,WSDexternal);
 WSDexternal* _unix_external_create(){
   return new WSDunixExternal;
 }
-
+void WSGFunixExternalInit(){
+  WSDexternal::setCreateInstanceHandler((void*)_unix_external_create);
+}
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _unix_external_init {
   public: _unix_external_init(){
-    WSDexternal::setCreateInstanceHandler((void*)_unix_external_create);
+    WSGFunixExternalInit();
   };
 };
-
 _unix_external_init  _unix_external_init_execute;
+#endif
 
 WSDunixExternal::WSDunixExternal(){
   _dl_ptr = NULL;
@@ -155,4 +212,4 @@ WSCstring* WSDunixExternal::getObjectSymbols(){
   }
   return NULL;
 }
-
+#endif //WS_EMBED
