@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <assert.h>
 #include <X11/Intrinsic.h>
@@ -53,7 +49,7 @@
 #include <x11/WSDunixThread.h>
 #include <x11/WSDunixMutex.h>
 #include <x11/WSDunixSemaphore.h>
-
+#include <unistd.h>
 #if 1 //UNICODE
 #include <xunicode.h>
 XUInfoDisplay *_info_disp = NULL;
@@ -63,14 +59,16 @@ int _ehandler(Display *d,XErrorEvent *e);
 extern char *_err_strings[];
 
 WSMFclassInit(WSDxappDev,WSDappDev);
-
+static WSDxappDev* _the_xappdev = NULL;
 WSDxappDev* WSGIxwinAppDev(){
-  WSDappDev* appdev = WSGIappDev();
-  if (appdev == NULL){
-    return NULL;
+  if (_the_xappdev == NULL){
+    WSDappDev* appdev = WSGIappDev();
+    if (appdev == NULL){
+      return NULL;
+    }
+    _the_xappdev =(WSDxappDev*)appdev->cast("WSDxappDev");
   }
-  WSDxappDev* xappdev =(WSDxappDev*)appdev->cast("WSDxappDev");
-  return xappdev;
+  return _the_xappdev;
 }
 
 WSDappDev* _xappDev_create_handler(){
@@ -88,16 +86,35 @@ _xappDev_init _xappDev_init_run_constructor;
 void WSGFdeviceInitialize(){
   static WSCbool initialized = False;
   if (initialized == False){
-extern WSDsemaphore* _unix_sem_chandler();
-    WSDmutex::setCreateInstanceHandler((void*)_unix_sem_chandler);
-extern WSDmutex* _unix_mutex_chandler();
-    WSDmutex::setCreateInstanceHandler((void*)_unix_mutex_chandler);
-extern WSDthread* _unix_th_chandler();
-    WSDthread::setCreateInstanceHandler((void*)_unix_th_chandler);
+extern void WSGFinitFS();
+    WSGFinitFS();
+extern void WSGFunixEnvInit();
+    WSGFunixEnvInit();
+extern void WSGFunixExecuteInit();
+    WSGFunixExecuteInit();
+#ifndef WS_EMBED
+extern void WSGFunixExternalInit();
+    WSGFunixExternalInit();
+#endif
+extern void WSGFinitializeUnixFile();
+    WSGFinitializeUnixFile();
+
+extern void WSGFinitializeUnixMutex();
+    WSGFinitializeUnixMutex();
+extern void WSGFinitializeUnixSemaphore();
+    WSGFinitializeUnixSemaphore();
+extern void WSGFinitializeUnixThread();
+    WSGFinitializeUnixThread();
+
+extern void WSGFxmessageInitialize();
+    WSGFxmessageInitialize();
+
 extern WSDfile* _unix_fp_chandler();
     WSDfile::setCreateInstanceHandler((void*)_unix_fp_chandler);
+#ifndef NO_DRAG_DROP
 extern WSDdragDrop* _xdrag_drop_create_handler_();
     WSDdragDrop::setCreateHandler(_xdrag_drop_create_handler_);
+#endif
 extern WSDmessage* _xclient_msg_create();
     WSDmessage::setCreateHandler(_xclient_msg_create);
 extern WSDmouse* _xmouse_create_handler();
@@ -135,6 +152,10 @@ extern void _xcolor_init();
 extern void _ximage_init();
     _ximage_init();
     WSGIappInitializeManager()->execInitializeProcs();
+    if (WSGIxwinAppDev() == NULL){
+      return;
+    }
+    WSGIxwinAppDev()->_init_lock();
   }
 }
 WSDxappDev::WSDxappDev(){
@@ -143,6 +164,17 @@ WSDxappDev::WSDxappDev(){
   _accept_atom_for_sock = 0;
   _read_event_for_execute = 0;
   _callback_event_for_thread = 0;
+  _lock_event_for_thread = 0;
+  _sem_lock = NULL;
+  _sem_wait = NULL;
+  _mutex = NULL;
+//  _msg_pipe[0] = -1;
+//  _msg_pipe[1] = -1;
+  _thr_id = (int)0;
+  _lock_events = 0;
+#ifndef NO_MWT_FUNCTION
+  _grabbed_disp = NULL;
+#endif
 }
 WSDxappDev::~WSDxappDev(){}
 
@@ -189,98 +221,101 @@ long WSDxappDev::loadGuiPolicy(){
       data.delHeadSpace();
       data.delTailSpace();
       data.delLineFeed();
-      if (!strcmp((char*)index,"#POLICY") && strcmp((char*)data,"")){
-        long val = atoi((char*)data);
+      char* istr = (char*)index;
+      char* dstr = (char*)data;
+      char* dstr2 = (char*)data2;
+      if (!strcmp(istr,"#POLICY") && strcmp(dstr,"")){
+        long val = atoi(dstr);
         if (_policy == -1){
           setGuiPolicy(val,False);
         }
       }else
-      if (!strcmp((char*)index,"#EXPOLICY") && strcmp((char*)data,"")){
-        setExtGuiPolicy((char*)data,True);
+      if (!strcmp(istr,"#EXPOLICY") && strcmp(dstr,"")){
+        setExtGuiPolicy(dstr,True);
       }else
-      if (!strcmp((char*)index,"#TOPSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#TOPSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_TOPSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#BOTTOMSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#BOTTOMSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_BOTTOMSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#BACKCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#BACKCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_BACKCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#FORECOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#FORECOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_FORECOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#MENUFORECOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#MENUFORECOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_MENUFORECOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#MENUBACKCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#MENUBACKCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_MENUBACKCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#MENUSELECTCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#MENUSELECTCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_MENUSELECTCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#MENUTOPSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#MENUTOPSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_MENUTOPSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#MENUBOTTOMSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#MENUBOTTOMSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_MENUBOTTOMSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#WORKBACKCOLOR") && strcmp((char*)data,"")){
+      if (!strcmp(istr,"#WORKBACKCOLOR") && strcmp(dstr,"")){
         short cno = WSGIappColorSet()->getColorNo((char*)data);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_WORKBACKCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#DARKBACKCOLOR") && strcmp((char*)data,"")){
+      if (!strcmp(istr,"#DARKBACKCOLOR") && strcmp(dstr,"")){
         short cno = WSGIappColorSet()->getColorNo((char*)data);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_DARKBACKCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#BARSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#BARSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_BARSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#NWTOPSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#NWTOPSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_NWTOPSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#NWBOTTOMSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#NWBOTTOMSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_NWBOTTOMSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#NWBACKCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#NWBACKCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_NWBACKCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#NWFORECOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#NWFORECOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_NWFORECOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#DARKBOTTOMSHADOWCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#DARKBOTTOMSHADOWCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_DARKBOTTOMSHADOWCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#MENUSELECTFORECOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#MENUSELECTFORECOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_MENUSELECTFORECOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#AQUAFORECOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#AQUAFORECOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_AQUAFORECOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#AQUAHIGHLIGHTCOLOR") && strcmp((char*)data,"")){
-        short cno = WSGIappColorSet()->getColorNo((char*)data);
+      if (!strcmp(istr,"#AQUAHIGHLIGHTCOLOR") && strcmp(dstr,"")){
+        short cno = WSGIappColorSet()->getColorNo(dstr);
         WSGIappColorSet()->setDefaultColorNo(WS_DF_AQUAHIGHLIGHTCOLOR,cno);
       }else
-      if (!strcmp((char*)index,"#DEFAULTFONT") && strcmp((char*)data2,"")){
+      if (!strcmp(istr,"#DEFAULTFONT") && strcmp(dstr2,"")){
         WSDfont* font = WSGIappFontSet()->getFont(8);
-        font->setFontName(data2);
+        font->setFontName(dstr2);
       }
     }
     if (_policy == -1){
@@ -345,7 +380,9 @@ long WSDxappDev::saveGuiPolicy(){
 
 long WSDxappDev::initialize(){
   WSDappDev::initialize();
-
+//  if (pipe(_msg_pipe) != 0){
+//fprintf(stderr,"The file discripter for message passing can not be created.\n");
+//  }
 
   short cno;
 
@@ -440,14 +477,43 @@ long WSDxappDev::initialize(){
            XtNx, 0, XtNy, 0, XtNwidth,  1, XtNheight, 1, NULL );
 
   XtRealizeWidget( _app_widget );
+//  if (_msg_pipe[0] != -1){
+//    XtAppAddInput(_app_context,_msg_pipe[0],(XtPointer)XtInputReadMask,_msg_recv,this);
+//  }
 
   return WS_NO_ERR;
 }
-
-long WSDxappDev::handleEvents(){
+#if 0
+void WSDxappDev::_msg_recv(void* ptr,int* src,XtInputId*){
+  WSDxappDev* _this = (WSDxappDev*)ptr;
+  char buf[1];
   while(1){
+  int len = read(*src,buf,1);
+    if (len == 1){
+      break;
+    }
+  }
+  if (_this->_sem_lock != NULL){
+    _this->_sem_lock->unlock();
+  }else{
+    return;
+  }
+  if (_this->_sem_wait != NULL){
+    _this->_sem_wait->lock();
+  }
+  WSGIappObjectList()->execInitialize();
+  WSGIappObjectList()->execUpdate();
+}
+#endif
+long WSDxappDev::handleEvents(){
+  _thr_id = (int)pthread_self();
+  while(1){
+//printf("_lock_count=%d\n",_lock_count);
     dispatchEvent();
     WSGIappDevice()->clearDeleteList();
+    if (_exit_event_loop != False){
+      break;
+    }
   }
   return WS_NO_ERR;
 }
@@ -480,11 +546,30 @@ WSCuchar hex_to_char(char* buffer){
   }
   return val;
 }
+
 long WSDxappDev::_dispatchEvent(WSCbool wait,XEvent* ev){
 dbprintf("WSDxappDev::_dispatchEvent %s:%d start\n",__FILE__,__LINE__);
 
-static long cnt = 0;
-cnt++;
+  if (_thr_id == (int)pthread_self() && _lock_events > 0){
+    _lock_events--;
+    if (_sem_lock != NULL){
+      _sem_lock->unlock();
+    }else{
+      return WS_NO_ERR;
+    }
+    if (_sem_wait != NULL){
+#ifdef THR_DBG
+  printf("WSDxappDev::_dispatchEvent main thread locked thr=0x%x\n",pthread_self());fflush(stdout);
+#endif
+      _sem_wait->lock();
+    }
+#ifdef THR_DBG
+  printf("WSDxappDev::_dispatchEvent main thread unlocked thr=0x%x\n",pthread_self());fflush(stdout);
+#endif
+  }
+ 
+//static long cnt = 0;
+//cnt++;
   XEvent e;
   if (ev == NULL){
     if (wait != False){
@@ -508,7 +593,14 @@ dbprintf("WSDxappDev::_dispatchEvent %s:%d done1\n",__FILE__,__LINE__);
     memcpy(&e,ev,sizeof(e));
   }
   WSCbool dispatch = True; 
-
+#ifndef NO_MWT_FUNCTION
+  if (getOwnerGrab() != False && getGrabbedDisplay() != NULL){
+    if (WSGIappMouse()->getMouseStatus() == 0){
+      XUngrabServer(getGrabbedDisplay());
+      setGrabbedDisplay(NULL);
+    }
+  }
+#endif
 dbprintf("WSDxappDev::_dispatchEvent %s:%d ev=%d\n",__FILE__,__LINE__,e.type);
   if (e.type == ClientMessage){
 //printf("client message !! here! xappdev \n");
@@ -521,10 +613,13 @@ dbprintf("WSDxappDev::_dispatchEvent %s:%d ev=%d\n",__FILE__,__LINE__,e.type);
     if (_callback_event_for_thread == 0){
       _callback_event_for_thread = XInternAtom(_disp,"THREAD-MESSAGE",False);
     }
+    if (_lock_event_for_thread == 0){
+      _lock_event_for_thread = XInternAtom(_disp,"LOCK-MESSAGE",False);
+    }
     if (e.xclient.message_type == _accept_atom_for_sock){
 //printf("client message !! here! execute xappdev \n");
       _accept_send_type data;
-      memcpy(&data,e.xclient.data.l,sizeof(_accept_send_type));
+      memcpy(&data,e.xclient.data.b,sizeof(_accept_send_type));
       void(*hd)(WSDsocket*,void*,WSCulong) = data.hd;
       if (hd != NULL){
         hd(data.sock,(void*)data.socket,data.addr);
@@ -535,7 +630,7 @@ dbprintf("WSDxappDev::_dispatchEvent %s:%d done2\n",__FILE__,__LINE__);
     if (e.xclient.message_type == _read_event_for_execute){
 //printf("client message !! here! execute xappdev for exec_read\n");
       _read_event_send_type data;
-      memcpy(&data,e.xclient.data.l,sizeof(_read_event_send_type));
+      memcpy(&data,e.xclient.data.b,sizeof(_read_event_send_type));
       WSDunixExecute* obj = data.obj;
 
       if (obj->getId() == data.id){
@@ -548,14 +643,17 @@ dbprintf("WSDxappDev::_dispatchEvent %s:%d done3\n",__FILE__,__LINE__);
     if (e.xclient.message_type == _callback_event_for_thread){
 //printf("client message !! here! execute xappdev for exec_read\n");
       WSDunixThread* ptr;
-      memcpy(&ptr,e.xclient.data.l,sizeof(WSDunixThread*));
+      memcpy(&ptr,e.xclient.data.b,sizeof(WSDunixThread*));
+      void* data;
+      memcpy(&data,&e.xclient.data.b[sizeof(ptr)],sizeof(void*));
 
       if (ptr->getCallbackFunction() != NULL){
 //printf("client message !! here! execute exec_read\n");
         void(*fn)(WSDunixThread*,void*) =
              (void(*)(WSDunixThread*,void*))ptr->getCallbackFunction();
         if (fn != NULL){
-          fn(ptr,ptr->getCallbackData());
+//          fn(ptr,ptr->getCallbackData());
+          fn(ptr,data);
         }
         WSGIappObjectList()->execInitialize();
         WSGIappObjectList()->execUpdate();
@@ -564,7 +662,30 @@ dbprintf("WSDxappDev::_dispatchEvent %s:%d done3\n",__FILE__,__LINE__);
       return WS_NO_ERR;
     }
 
-
+    if (e.xclient.message_type == _lock_event_for_thread){
+      if (_thr_id == (int)pthread_self()){
+        if (_sem_lock != NULL){
+          _sem_lock->unlock();
+        }else{
+          return WS_NO_ERR;
+        }
+        if (_sem_wait != NULL){
+#ifdef THR_DBG
+  printf("WSDxappDev::_dispatchEvent main thread locked thr=0x%x\n",pthread_self());fflush(stdout);
+#endif
+          _sem_wait->lock();
+        }
+#ifdef THR_DBG
+  printf("WSDxappDev::_dispatchEvent main thread unlocked thr=0x%x\n",pthread_self());fflush(stdout);
+#endif
+        WSGIappObjectList()->execInitialize();
+        WSGIappObjectList()->execUpdate();
+        return WS_NO_ERR;
+      }else{
+        _lock_events++;
+        return WS_NO_ERR;
+      }
+    }
   }
 
   if (e.type == SelectionClear){
@@ -649,84 +770,6 @@ extern int XUctDecode(char* dest, int max, const XUChar* text, int length);
                       8,PropModeReplace,(unsigned char*)str2,strlen(str2));
       delete ustr;
       delete str2;
-#endif
-#if 0
-      char* str2 = NULL;
-      WSCbool need_convert = False;
-      WSCushort* ustr = NULL;
-      if ( WSGIappLocaleSet()->getSystemLocaleEncoding() == WS_EN_EUCJP){
-        char b1[5];
-        b1[0] = 0x1b;
-        b1[1] = 0x24;
-        b1[2] = 0x29;
-        b1[3] = 0x42;
-        b1[4] = 0;
-        str2 = new char[strlen(str)+8];
-        strcpy(str2,b1);
-        strcpy(&(str2[4]),str);
-        need_convert = True;
-      }else{
-        ustr = WSGFgetUCS2(str,WSGIappLocaleSet()->getSystemLocaleEncoding());
-        int i;
-        long ulen = WSGFstrlenUCS2(ustr);
-        str2 = new char[ulen*8+1];
-        int cnt = 0;
-        for(i=0; i<ulen; i++){
-          if (ustr[i] & 0xff00){
-            need_convert = True;
-            str2[cnt++] = '\\';
-            str2[cnt++] = 'x';
-            str2[cnt++] = '{';
-            WSCuchar val = ((ustr[i] >> 12) & 0xf);
-            if (val >= 10){
-              val += ('A' -10);
-            }else{
-              val += '0';
-            }
-            str2[cnt++] = val;
-            val = ((ustr[i] >> 8) & 0xf);
-            if (val >= 10){
-              val += ('A' -10);
-            }else{
-              val += '0';
-            }
-            str2[cnt++] = val;
-            val = ((ustr[i] >> 4) & 0xf);
-            if (val >= 10){
-              val += ('A' -10);
-            }else{
-              val += '0';
-            }
-            str2[cnt++] = val;
-            val = (ustr[i] & 0xf);
-            if (val >= 10){
-              val += ('A' -10);
-            }else{
-              val += '0';
-            }
-            str2[cnt++] = val;
-            str2[cnt++] = '}';
-          }else{
-            str2[cnt] = ustr[i];
-            cnt++;
-          }
-          str2[cnt] = 0;
-        }
-      }
-      if (need_convert == False){
-        XChangeProperty(_disp,se.requestor,se.property,se.target,
-                      8,PropModeReplace,(unsigned char*)str,strlen(str));
-      }else{
-        XChangeProperty(_disp,se.requestor,se.property,se.target,
-                      8,PropModeReplace,(unsigned char*)str2,strlen(str2));
-int i;
-for(i=0; i<strlen(str2);i++){
-printf("0x%x ",str2[i]);
-}
-printf("\n",str2[i]);
-      }
-      delete str2;
-      delete ustr;
 #endif
     }else if (e.xselectionrequest.selection == XA_PRIMARY &&
          e.xselectionrequest.target == prop5 ){
@@ -950,15 +993,6 @@ dbprintf("WSDxappDev::_dispatchEvent %s:%d done5\n",__FILE__,__LINE__);
 #endif
 //printf("0x%x 0x%x 0x%x 0x%x nitems=%d\n",buffer[0],buffer[1],buffer[2],buffer[3],nitems);
         long encoding = WSGIappLocaleSet()->getSystemLocaleEncoding();
-#if 0
-{
-  int i;
-  for(i=0; i<strlen(buffer); i++){
-printf("0x%x ",buffer[i]);
-  }
-printf("\n");
-}
-#endif
         long len = strlen(buffer)+1;
         WSCushort* ustr = new WSCushort[len];
         if (ustr != NULL){
@@ -1045,11 +1079,29 @@ extern int XUctEncode(XUChar* dest, int max, const char* text, int length);
     }else if (e.xbutton.button == 4){
       state |= WS_MOUSE_BTN4;
       WSGIappMouse()->setTargetBtn(WS_MOUSE_BTN4);
+      dispatch = False;
     }else if (e.xbutton.button == 5){
       state |= WS_MOUSE_BTN5;
       WSGIappMouse()->setTargetBtn(WS_MOUSE_BTN5);
+      dispatch = False;
     }
     WSGIappMouse()->setStatus(state);
+    if (dispatch == False){
+      WSCpoint point;
+      point.setPoint( e.xbutton.x,e.xbutton.y );
+
+      if (e.xbutton.button == 4){
+        WSCbase* fcl = WSGIappMouse()->getMouseFocusClient();
+        if (fcl != NULL){
+          fcl->onMousePress(&point);
+        }
+      }else if (e.xbutton.button == 5){
+        WSCbase* fcl = WSGIappMouse()->getMouseFocusClient();
+        if (fcl != NULL){
+          fcl->onMousePress(&point);
+        }
+      }
+    }
   }else if (e.type == ButtonRelease){
     long state = WSGIappMouse()->getStatus();
     if (e.xbutton.button == 1){
@@ -1064,11 +1116,29 @@ extern int XUctEncode(XUChar* dest, int max, const char* text, int length);
     }else if (e.xbutton.button == 4){
       state &= ~WS_MOUSE_BTN4;
       WSGIappMouse()->setTargetBtn(WS_MOUSE_BTN4);
+      dispatch = False;
     }else if (e.xbutton.button == 5){
       state &= ~WS_MOUSE_BTN5;
       WSGIappMouse()->setTargetBtn(WS_MOUSE_BTN5);
+      dispatch = False;
     }
     WSGIappMouse()->setStatus(state);
+    if (dispatch == False){
+      WSCpoint point;
+      point.setPoint( e.xbutton.x,e.xbutton.y );
+
+      if (e.xbutton.button == 4){
+        WSCbase* fcl = WSGIappMouse()->getMouseFocusClient();
+        if (fcl != NULL){
+          fcl->onMouseRelease(&point);
+        }
+      }else if (e.xbutton.button == 5){
+        WSCbase* fcl = WSGIappMouse()->getMouseFocusClient();
+        if (fcl != NULL){
+          fcl->onMouseRelease(&point);
+        }
+      }
+    }
   }
 
   if (dispatch == True && ev == NULL){
@@ -1110,15 +1180,26 @@ void WSDxappDev::setXEvent(XEvent* ev){
 XEvent* WSDxappDev::getEvent(){
   return _ev;
 }
-
+int WSDxappDev::_get_main_loop_thr_id(){
+  return _thr_id;
+}
 void WSDxappDev::update(){
+  if (_thr_id != (int)pthread_self()){
+    XFlush(display());
+    return;
+  }
   while(1){
     Boolean fl = XtAppPending(_app_context);
     if (fl == 0){
 	  break;
 	}
+//    processEvent();
     XEvent e;
-    XtAppNextEvent(_app_context,&e);
+//    XtAppNextEvent(_app_context,&e);
+    Boolean ret = XCheckMaskEvent(display(),VisibilityChangeMask | ExposureMask,&e);
+    if (ret == 0){
+      break;
+    }
     WSCbool dispatch = True; 
 
 #ifndef KINPUT2
@@ -1136,11 +1217,11 @@ void WSDxappDev::update(){
           continue;
       }
     }
-#endif
 
     if (e.type == KeyPress || e.type == KeyRelease || e.type == FocusIn){
        dispatch = WSGIxkeyboard()->deliverKeyEvent(&e);
     }
+#endif
     if (e.type == GraphicsExpose){
      e.type = Expose;
     }
@@ -1150,14 +1231,88 @@ void WSDxappDev::update(){
   }
   XFlush(display());
 }
+void WSDxappDev::_init_lock(){
+  _sem_lock = WSDsemaphore::getNewInstance();
+  _sem_lock->initialize();
+  _sem_lock->lock();
+  _sem_wait = WSDsemaphore::getNewInstance();
+  _sem_wait->initialize();
+  _sem_wait->lock();
+  _mutex = WSDmutex::getNewInstance();
+  _mutex->initialize();
+}
+long WSDxappDev::lock(){
+  if (_thr_id == (int)pthread_self()){
+    return WS_ERR;
+  }
+  if (_mutex == NULL){
+    return WS_ERR;
+  }
+  if (_sem_lock == NULL || _sem_wait == NULL){
+    return WS_ERR;
+  }
+#ifdef THR_DBG
+  printf("WSDxappDev::lock thr=0x%x\n",pthread_self());fflush(stdout);
+#endif
+  _mutex->lock();
+
+//  char buf[1];
+//  buf[0] = 0;
+//  write(_msg_pipe[1],buf,1);
+
+static Display* disp = NULL;
+  if (disp == NULL){
+    disp = XOpenDisplay((char*)NULL);
+  }
+//  Display* disp = XOpenDisplay((char*)NULL);
+  if (disp == NULL){
+    _mutex->unlock();
+    return WS_ERR;
+  }
+  Atom atom = XInternAtom(disp,"LOCK-MESSAGE",False);
+  Window win = XtWindow(WSGIxwinAppDev()->appWidget());
+  XClientMessageEvent  cm;
+  cm.type = ClientMessage;
+  cm.display = disp;
+  cm.window = win;
+  cm.message_type = atom;
+  cm.format = 8;
+  XSendEvent(disp,win,False,NoEventMask,(_XEvent *)&cm);
+
+  XFlush(disp);
+
+  _sem_lock->lock();
+  _locked = True;
+  _mutex->unlock();
+  return WS_NO_ERR;
+}
+long WSDxappDev::unlock(){
+  if (_thr_id == (int)pthread_self()){
+    return WS_ERR;
+  }
+  if (_sem_lock == NULL || _sem_wait == NULL){
+    return WS_ERR;
+  }
+
+#ifdef THR_DBG
+  printf("WSDxappDev::unlock thr=0x%x\n",pthread_self());fflush(stdout);
+//MEM trap!
+if (_locked == False){
+  fprintf(stderr,"WSDxappDev::unlock thr=0x%x err..\n",pthread_self());fflush(stdout);
+void* ptr=NULL;
+memset(ptr,0,1024);
+}
+#endif
+  _locked = False;
+  _sem_wait->unlock();
+  return WS_NO_ERR;
+}
 
 int _ehandler(Display *d,XErrorEvent *e){
 //TEST
 //assert(0);
 //TEST
-#if 1 //UNICODE
- if (!XUErrorHandler(d, e)) return 0;
-#endif //UNICODE
+// if (!XUErrorHandler(d, e)) return 0;
 
  char string[64];
  XGetErrorText(d,e->error_code,string,64);
@@ -1170,7 +1325,33 @@ int _ehandler(Display *d,XErrorEvent *e){
  printf("minor   code  = %d\n",e->minor_code);
  return 0;
 }
-
+void WSDxappDev::checkThreadSafe(){
+  if (_get_main_loop_thr_id() != 0 &&
+      pthread_self() != (pthread_t)_get_main_loop_thr_id() &&
+      getLockStatus() == False){
+  fprintf(stderr,"WSDxappDEv::checkTheadSafe() thread safe ERROR!!!\n");fflush(stdout);
+#ifdef THR_DBG
+void* ptr = 0;
+      memset(ptr,0,1024);
+#endif
+//    assert(0);
+//    while(1){}
+  }
+}
+#ifndef NO_MWT_FUNCTION
+void WSDxappDev::setGrabbedDisplay(Display* disp){
+  _grabbed_disp = disp;
+}
+Display* WSDxappDev::getGrabbedDisplay(){
+  return _grabbed_disp;
+}
+void WSDxappDev::setOwnerGrab(WSCbool fl){
+  _owner_grab = fl;
+}
+WSCbool WSDxappDev::getOwnerGrab(){
+  return _owner_grab;
+}
+#endif
 /* Request codes */
 char *_err_strings[] = {
 "---------------------------",
