@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <X11/Xlib.h>
 #include <WSDfont.h>
@@ -38,6 +34,7 @@
 #include <x11/WSDxappDev.h>
 #include <x11/WSDxdraw.h>
 #include <x11/WSDximage.h>
+#include <x11/WSDxfont.h>
 
 //#define TDBG
 
@@ -88,6 +85,9 @@ WSDxdraw::WSDxdraw(){
   _line_width = -1;
   _line_type = -1;
   _clipping = False;
+  _gc = 0;
+  _underline_char_pos = 0;
+  _underline_char_len = 0;
 }
 WSDxdraw::~WSDxdraw(){}
 
@@ -403,10 +403,14 @@ long WSDxdraw::setHatchPattern(char type){
   return bk;
 }
 long WSDxdraw::beginDraw(WSDdev* dev,long val1,long val2,long val3){
+  _underline_char_pos = 0;
+  _underline_char_len = 0;
   _dev = dev;
   _display = (Display*)val1;
   _window = (Window)val2;
 
+  WS_CHECK_THREAD_SAFE
+
   if (_gc != (GC)val3){
     _fg_color = 0;
     _bg_color = 0;
@@ -479,6 +483,12 @@ long WSDxdraw::setRegion(short x,short y,WSCushort w,WSCushort h){
   if (_gc == 0 || _window == 0){
     return WS_ERR;
   }
+  if ((long)x + (long)w < 0 || (long)y + (long)h <0){
+    x = 0;
+    w = 0;
+    y = 0;
+    h = 0;
+  }
   if (_clip_x != x || _clip_y != y || _clip_w != w || _clip_h != h){
 //printf("WSDxdraw::setRegion win=0x%x gc=0x%x %d %d %d %d\n",_window,_gc,x,y,w,h);
     _clip_x = x;
@@ -556,6 +566,12 @@ long WSDxdraw::drawFillRect(short x,short y,WSCushort w,WSCushort h){
   if (_gc == 0 || _window == 0){
     return WS_ERR;
   }
+  if ((long)x + (long)w < 0){
+    return WS_NO_ERR;
+  }
+  if ((long)y + (long)h < 0){
+    return WS_NO_ERR;
+  }
 //printf("WSDxdraw::drawFillRect win=0x%x gc=0x%x %d %d %d %d\n",_window,_gc,x,y,w,h);
   XFillRectangle(_display,_window,_gc,(int)x,(int)y,(WSCuint)w,(WSCuint)h); 
   return WS_NO_ERR;
@@ -611,7 +627,7 @@ long WSDxdraw::drawFillPoly(WSCpoint* points,long num){
     pts[i].y = points[i].y;
   }
   XFillPolygon(_display,_window,_gc,pts,num,Nonconvex,CoordModeOrigin);
-  delete pts;
+  delete[] pts;
   return WS_NO_ERR;
 }
 
@@ -755,7 +771,9 @@ long WSDxdraw::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDima
   int msb_flag = (ImageByteOrder(_display) == MSBFirst)?1:0;
   WSCulong iw = image->getImageWidth();
   WSCulong ih = image->getImageHeight();
-  img2->data = new char[img2->bytes_per_line * dh ];
+//  img2->data = new char[img2->bytes_per_line * dh ];
+  //for valgrind mem checker.
+  img2->data = (char*)malloc(sizeof(char)*(img2->bytes_per_line * dh ));
 //printf("img2->bytes_per_line=%d\n",img2->bytes_per_line);
   long bit = img2->bits_per_pixel;
 
@@ -769,7 +787,10 @@ long WSDxdraw::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDima
                        ZPixmap,0,0,
                        dw, dh,8,0);
     XInitImage(img_mask2);
-    img_mask2->data = new char[img_mask2->bytes_per_line * dh ];
+//    img_mask2->data = new char[img_mask2->bytes_per_line * dh ];
+    //for valgrind mem checker.
+    img_mask2->data =
+          (char*)malloc(sizeof(char)*(img_mask2->bytes_per_line * dh ));
     iwm = img_mask->bytes_per_line;
     iwm2 = img_mask2->bytes_per_line;
 //printf("img_mask2->bytes_per_line=%d\n",img_mask2->bytes_per_line);
@@ -863,7 +884,6 @@ long WSDxdraw::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDima
           WSCuchar m = 1;
           m <<= shiftd;
 //printf("ptr=%d %d,pt=%d %d %d 0x%x\n",ptr,shiftd,pt,shiftd,val2&m,~m);
-//TEST
           val &= ~m;
         }
         img_mask2->data[ptr] = val;
@@ -897,7 +917,8 @@ long WSDxdraw::drawStretchedImage(short x,short y,WSCushort w,WSCushort h,WSDima
     XFreePixmap(XtDisplay(app->appWidget()),pixm);
   }
   if (img2->data != NULL){
-    delete img2->data;
+    //for valgrind mem checker
+    free(img2->data);
     img2->data = NULL;
   }
   XDestroyImage(img2);
@@ -947,7 +968,7 @@ long WSDxdraw::drawUString(long x,long y,WSCulong w,WSCulong h,
   char* str1 = WSGFgetString(str,WS_EN_UTF8);
   WSCstring str2(str1,WS_EN_UTF8);
   long ret = drawString(x,y,w,h,&str2,font,align,cur,inter_cur,scur1,scur2);
-  delete str1;
+  delete[] str1;
   return ret;
 }
 long WSDxdraw::drawFillUString(long x,long y,WSCulong w,WSCulong h,
@@ -955,7 +976,7 @@ long WSDxdraw::drawFillUString(long x,long y,WSCulong w,WSCulong h,
   char* str1 = WSGFgetString(str,WS_EN_UTF8);
   WSCstring str2(str1,WS_EN_UTF8);
   long ret = drawFillString(x,y,w,h,&str2,font,align,cur,inter_cur);
-  delete str1;
+  delete[] str1;
   return ret;
 }
 long WSDxdraw::drawString(long x,long y,WSCulong w,WSCulong h,
@@ -992,6 +1013,7 @@ long WSDxdraw::drawString(long x,long y,WSCulong w,WSCulong h,
       return WS_ERR;
     }
   }
+#if 0 //old version
   XFontStruct* efs = (XFontStruct*)fn->getValue1();
   XFontStruct* kfs = (XFontStruct*)fn->getValue2();
   XFontStruct* gfs = (XFontStruct*)fn->getValue3();
@@ -1018,15 +1040,20 @@ long WSDxdraw::drawString(long x,long y,WSCulong w,WSCulong h,
   }else{
     gfid = gfs->fid;
   }
-
   long fheight = (long)efs->max_bounds.ascent;
   long fheight2 = (long)kfs->max_bounds.ascent;
-
+#endif
+  long fheight = (long)fn->getFontHeight();
+  WSDxfont* xfn = (WSDxfont*)fn->cast("WSDxfont");
+  if (xfn == NULL){
+    return WS_ERR;
+  }
   if (str->isExist("\n") == -1){
     WSCrect   area;
     area.setRect(x,y,w,h);
     WSCpoint  point;
     WSGFcalcOneLineStringAddr(str,&area,align,fn,&point);
+#if 0
     char* tstr;
 //    if (WSGIappLocaleSet()->getDefaultEncoding() == WS_EN_EUCJP){
 //      tstr = str->getString(WS_EN_EUCJP);
@@ -1036,10 +1063,16 @@ long WSDxdraw::drawString(long x,long y,WSCulong w,WSCulong h,
     char* tstr2 = new char[strlen(tstr) + 8];
     strcpy(tstr2,tstr);
     strcat(tstr2," ");
-    WSGFxdrawTextString( _display, _window, _gc, efs, kfs, gfs, _fg_color,
-      _bg_color,point.x, point.y + fheight,
-      tstr2, kfontWidth, efontWidth, fheight2,fheight,cur, inter_cur,_line_width,scur1,scur2);
-    delete tstr2;
+#endif
+//    WSGFxdrawTextString( _display, _window, _gc, efs, kfs, gfs, _fg_color,
+//      _bg_color,point.x, point.y + fheight,
+//      tstr2, kfontWidth, efontWidth, fheight2,fheight,cur, inter_cur,_line_width,scur1,scur2);
+    char* tstr2 = str->getString(WS_EN_UTF8);
+    WSGFxdrawTextString( _display, _window, _gc, xfn->getFontSetName(),
+                         _fg_color,_bg_color,point.x, point.y + fheight,
+                         tstr2, fheight,cur,inter_cur,_line_width,scur1,scur2,
+                         _underline_char_pos,_underline_char_len);
+//    delete[] tstr2;
     setRegion(clx_bk,cly_bk,clw_bk,clh_bk);
     return WS_NO_ERR;
   }
@@ -1083,24 +1116,32 @@ long WSDxdraw::drawString(long x,long y,WSCulong w,WSCulong h,
 //    }
 //printf("multiline...#%s#\n",tstr);
     if (cur == -1){
-      WSGFxdrawTextString( _display, _window, _gc, efs, kfs, gfs, _fg_color,
-        _bg_color,points[i].x, points[i].y + fheight,
-        tstr, kfontWidth, efontWidth, fheight2,fheight, -1,
-        inter_cur,_line_width,
-        scur1 - lpos,scur2 - lpos);
+//      WSGFxdrawTextString( _display, _window, _gc, efs, kfs, gfs, _fg_color,
+//        _bg_color,points[i].x, points[i].y + fheight,
+//        tstr, kfontWidth, efontWidth, fheight2,fheight, -1,
+//        inter_cur,_line_width,
+//        scur1 - lpos,scur2 - lpos);
+      WSGFxdrawTextString( _display, _window, _gc, xfn->getFontSetName(),
+                    _fg_color,_bg_color,points[i].x, points[i].y + fheight,
+                    tstr, fheight, -1, inter_cur,_line_width,
+                    scur1 - lpos,scur2 - lpos);
 //printf("cur=%d c1=%d c2=%d  lpos=%d pre=%d\n",-1,scur1-lpos,scur2-lpos,lpos,prelpos);
     }else{
-      WSGFxdrawTextString( _display, _window, _gc, efs, kfs, gfs, _fg_color,
-        _bg_color,points[i].x, points[i].y + fheight,
-        tstr, kfontWidth, efontWidth, fheight2,fheight,cur - lpos,
-        inter_cur,_line_width,
+//      WSGFxdrawTextString( _display, _window, _gc, efs, kfs, gfs, _fg_color,
+//        _bg_color,points[i].x, points[i].y + fheight,
+//        tstr, kfontWidth, efontWidth, fheight2,fheight,cur - lpos,
+//        inter_cur,_line_width,
+//        scur1 - lpos,scur2 - lpos);
+      WSGFxdrawTextString( _display, _window, _gc, xfn->getFontSetName(),
+        _fg_color, _bg_color,points[i].x, points[i].y + fheight,
+        tstr, fheight, cur - lpos, inter_cur,_line_width,
         scur1 - lpos,scur2 - lpos);
 //printf("cur=%d c1=%d c2=%d  lpos=%d pre=%d\n",cur-lpos,scur1-lpos,scur2-lpos,lpos,prelpos);
     }
   }
   str->seek(0);
   if (num != 0){
-    delete points;
+    delete[] points;
   }
   setRegion(clx_bk,cly_bk,clw_bk,clh_bk);
 
@@ -1137,6 +1178,7 @@ long WSDxdraw::drawFillString(long x,long y,WSCulong w,WSCulong h,
       return WS_ERR;
     }
   }
+#if 0
   XFontStruct* efs = (XFontStruct*)fn->getValue1();
   XFontStruct* kfs = (XFontStruct*)fn->getValue2();
   XFontStruct* gfs = (XFontStruct*)fn->getValue3();
@@ -1165,6 +1207,13 @@ long WSDxdraw::drawFillString(long x,long y,WSCulong w,WSCulong h,
   }
   long fheight = (long)efs->max_bounds.ascent;
   long fheight2 = (long)kfs->max_bounds.ascent;
+#endif
+  long fheight = (long)fn->getFontHeight();
+  WSDxfont* xfn = (WSDxfont*)fn->cast("WSDxfont");
+  if (xfn == NULL){
+    return WS_ERR;
+  }
+
   if (str->isExist("\n") == -1){
     WSCrect area;
     area.setRect(x,y,w,h);
@@ -1176,10 +1225,14 @@ long WSDxdraw::drawFillString(long x,long y,WSCulong w,WSCulong h,
     strcpy(tstr2,tstr);
     strcat(tstr2," ");
 
-    WSGFxdrawTextString( _display, _window, _gc, efs, kfs, gfs, _fg_color,
-      _bg_color,point.x, point.y + fheight,
-      tstr2, kfontWidth, efontWidth, fheight2, fheight, cur, inter_cur,_line_width);
-    delete tstr2;
+//    WSGFxdrawTextString( _display, _window, _gc, efs, kfs, gfs, _fg_color,
+//      _bg_color,point.x, point.y + fheight,
+//      tstr2, kfontWidth, efontWidth, fheight2, fheight, cur, inter_cur,_line_width);
+    WSGFxdrawTextString( _display, _window, _gc, xfn->getFontSetName(),
+        _fg_color, _bg_color,point.x, point.y + fheight,
+        tstr2, fheight, cur, inter_cur,_line_width,
+        _underline_char_pos,_underline_char_len);
+    delete[] tstr2;
     setRegion(clx_bk,cly_bk,clw_bk,clh_bk);
     return WS_NO_ERR;
   }
@@ -1210,9 +1263,12 @@ long WSDxdraw::drawFillString(long x,long y,WSCulong w,WSCulong h,
     char* tstr2 = new char[strlen(tstr) + 8];
     strcpy(tstr2,tstr);
 //    strcat(tstr2," ");
-    WSGFxdrawTextString( _display, _window, _gc, efs, kfs, gfs, _fg_color,
-       _bg_color,points[i].x, points[i].y + fheight,
-       tstr2, kfontWidth, efontWidth, fheight2, fheight, cur - lpos, inter_cur,_line_width);
+//    WSGFxdrawTextString( _display, _window, _gc, efs, kfs, gfs, _fg_color,
+//       _bg_color,points[i].x, points[i].y + fheight,
+//       tstr2, kfontWidth, efontWidth, fheight2, fheight, cur - lpos, inter_cur,_line_width);
+    WSGFxdrawTextString( _display, _window, _gc, xfn->getFontSetName(),
+            _fg_color, _bg_color,points[i].x, points[i].y + fheight,
+            tstr2, fheight, cur - lpos, inter_cur,_line_width);
     delete tstr2;
   }
   str->seek(0);
@@ -1530,4 +1586,20 @@ static WSDcolor* color = NULL;
 
   return WS_NO_ERR;
 }
+long WSDxdraw::setOperation(WSCuchar op){
+  if (_gc == 0 || _window == 0){
+    return WS_ERR;
+  }
+  if (op == WS_OPERATION_SRC_COPY){
+    XSetFunction(_display,_gc,GXcopy);
+  }else
+  if (op == WS_OPERATION_XOR){
+    XSetFunction(_display,_gc,GXxor);
+  }
+  return WS_NO_ERR;
+}
+void WSDxdraw::setUnderLineChar(long pos,long len){
+  _underline_char_pos = pos;
+  _underline_char_len = len;
+}
 
