@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <X11/Xlib.h>
 #include <WScom.h>
@@ -33,6 +29,22 @@
 #include <xunicode.h>
 
 
+//size fmly-name bold italic
+#define DEFAULT_FONT_SET  "14 * "
+//new version
+char* dfnames[] = { 
+"0 14 * 0 0", //No. size fmly-name bold italic
+"1 16 * 0 0",
+"2 18 * 0 0",
+"3 20 * 0 0",
+"4 22 * 0 0",
+"5 24 * 0 0",
+"6 26 * 0 0",
+"7 30 * 0 0",
+"8 14 * 0 0",
+NULL
+};
+#if 0 //old version
 #define DEFAULT_FONT_SET  "14 iso8859 * * * r * "
 char* dfnames[] = { 
 "0 14 iso8859 * * * r * ", //No. size rgstry fmly wght fndry slant not-use
@@ -46,7 +58,8 @@ char* dfnames[] = {
 "8 14 iso8859 * * * r * ",
 NULL
 };
-
+#endif
+#if 0 //old version
 #define DEFAULT_FONT_SET_J "14  jisx0201.1976  jisx0208.1983  *   misc misc * "
 char* dfnames_j[] = { 
 "0 14 jisx0201.1976 jisx0208.1983 *  misc misc misc ",
@@ -60,7 +73,7 @@ char* dfnames_j[] = {
 "8 14 jisx0201.1976 jisx0208.1983 *  misc misc misc ",
 NULL
 };
-
+#endif
 
 WSMFclassInit(WSDxfont,WSDfont);
 WSDfont* _xfont_chandler(){
@@ -84,8 +97,14 @@ long _xfont_setup_handler(WSDfont* font,char* str){
   str6[0] = 0;
   str7[0] = 0;
   long no;
-  sscanf(str,"%d %s %s %s %s %s %s %s",&no,str1,str2,str3,str4,str5,str6,str7);
-  sprintf(fname,"%s %s %s %s %s %s %s",str1,str2,str3,str4,str5,str6,str7);
+  sscanf(str,"%ld %s %s %s %s %s %s %s",&no,str1,str2,str3,str4,str5,str6,str7);
+
+//printf("font=%ld %s %s %s %s %s %s %s\n",no,str1,str2,str3,str4,str5,str6,str7);
+  if (str6[0] != 0){ //backward compatible..
+    sprintf(fname,"%s *",str1);
+  }else{
+    sprintf(fname,"%s %s %s %s",str1,str2,str3,str4);
+  }
   font->setFontName(fname);
   font->setId((short)no);
   return WS_NO_ERR;
@@ -97,7 +116,6 @@ long _xfont_default_init_handler(WSCfontSet* fontset){
 //  if (!strcmp(WSGIappLocaleSet()->getDefaultLocaleName(),WS_LOCALE_JAPANESE)){
 //    _dfnames = dfnames_j;
 //  } 
- 
   while(_dfnames[i]){
     WSDfont* font = new WSDxfont();
     _xfont_setup_handler(font,_dfnames[i]);
@@ -114,10 +132,10 @@ void _xfont_init(){
   WSGIappFontSet()->initializeDefault();
 };
 
-
-
 WSDxfont::WSDxfont(){
+  _font_set_name = NULL;
   _xfont_set = 0;
+#if 0
   _ergs.setString("jisx0201.1976");
   _krgs.setString("jisx0208.1983");
   _grgs.setString("jisx0208.1983");
@@ -135,7 +153,9 @@ WSDxfont::WSDxfont(){
 //    //lang Japanese
 //    //do nothing
 //  }
-
+#endif
+  _fmly.setString("*");
+  _font_height = 14;
 }
 
 WSDxfont::~WSDxfont(){
@@ -145,6 +165,12 @@ WSDxfont::~WSDxfont(){
 static Display* disp = NULL;
 
 long WSDxfont::initialize(){
+  if (_xfont_set != 0){
+    WSDxappDev* app = WSGIxwinAppDev(); 
+    Display* disp = app->display();
+    XFreeFontSet(disp, _xfont_set);
+    _xfont_set = 0;
+  }
   char* fname = getSrc();
   if (fname == NULL || !strcmp(fname,"") ){
     fname = DEFAULT_FONT_SET;
@@ -152,6 +178,47 @@ long WSDxfont::initialize(){
 //      fname = DEFAULT_FONT_SET_J;
 //    }
   }
+  char str1[128];
+  char str2[128];
+  char str3[128];
+  char str4[128];
+  char str5[128];
+  char str6[128];
+  char str7[128];
+  str1[0] = 0;
+  str2[0] = 0;
+  str3[0] = 0;
+  str4[0] = 0;
+  str5[0] = 0;
+  str6[0] = 0;
+  str7[0] = 0;
+  long no;
+  sscanf(fname,"%ld %s %s %s %s %s %s %s",&no,str1,str2,str3,str4,str5,str6,str7);
+
+//printf("font=%ld %s %s %s %s %s %s %s\n",no,str1,str2,str3,str4,str5,str6,str7);
+
+  char font_set_name[256];
+  int bold = 0;
+  int italic = 0;
+  font_set_name[0] = 0;
+
+  if (str6[0] != 0){ //backward compatible..
+    strcpy(font_set_name,"*");
+//    sprintf(fname,"*",str1,str2,str3,str4,str5,str6,str7);
+  }else{
+    sscanf(fname,"%ld %s %d %d",&_font_height,font_set_name,&bold,&italic);
+//printf("bold=%d italic=%d\n",bold,italic);
+  }
+  _fmly.setString(font_set_name);
+  _style = 0;
+  if (bold != 0){
+    _style |= WS_FONT_BOLD;
+  }
+  if (italic != 0){
+    _style |= WS_FONT_ITALIC;
+  }
+
+#if 0 //old version
   char efontr[128];
   char kfontr[128];
   char gfontr[128];
@@ -168,7 +235,7 @@ long WSDxfont::initialize(){
     strcpy(gfontr,"*");
     strcpy(kfontf,"*");
     strcpy(gfontf,"*");
-    sscanf(fname,"%d %s %s %s %s %s %s",&_font_height,efontr,
+    sscanf(fname,"%ld %s %s %s %s %s %s",&_font_height,efontr,
      fmly,wght,efontf,slant,gfontf);
     _fmly.setString(fmly);
     _wght.setString(wght);
@@ -177,14 +244,14 @@ long WSDxfont::initialize(){
     char dust[1024];
     if (_virtual_flag != False){
       char* fname2 = getFontName();
-      sscanf(fname2,"%d %s",&_font_height,dust);
+      sscanf(fname2,"%ld %s",&_font_height,dust);
     }
 
 
 //  }else{
 //    //Japanese
 //    japanese = True;
-//    sscanf(fname,"%d %s %s %s %s %s %s",&_font_height,efontr,
+//    sscanf(fname,"%ld %s %s %s %s %s %s",&_font_height,efontr,
 //     kfontr,gfontr,efontf,kfontf,gfontf);
 //  }
 
@@ -252,10 +319,33 @@ fprintf(stderr,WSGFgettext("A2:WSSTR Invalid font name: %s\n"),fname);
   }else{
     setValue3(0);
   }
+#endif
+
+  WSCstring ret;
+  ret = _fmly;
+  if (_style == WS_FONT_BOLD){
+    ret << ".b";
+  }else
+  if (_style == (WS_FONT_BOLD | WS_FONT_ITALIC)){
+    ret << ".b.i";
+  }else
+  if (_style == WS_FONT_ITALIC){
+    ret << ".i";
+  }
+//printf("WSDxfont::getFontSetName ret=%s 0x%x\n",(char*)ret,_style);
+  if (_font_set_name != NULL){
+    delete[] _font_set_name; 
+    _font_set_name = NULL;
+  }
+  _font_set_name = WSGFstrdup((char*)ret);
+
+  setValue1(0);
+  setValue2(0);
+  setValue3(0);
   return WS_NO_ERR;
 }
 
-long WSDxfont::getXFontSet(){
+XFontSet WSDxfont::getXFontSet(){
   if (_xfont_set == 0){
     char **miss_font;
     int  miss_cnt;
@@ -265,20 +355,24 @@ long WSDxfont::getXFontSet(){
     }
     char fname[256];
     sprintf(fname,"-*-*-medium-r-*-*-%d-*-*-*-*-*-*-*",_font_height);
-    _xfont_set = (long)XCreateFontSet(disp,fname,&miss_font,&miss_cnt,NULL);
+    _xfont_set = XCreateFontSet(disp,fname,&miss_font,&miss_cnt,NULL);
 
-    if (miss_cnt != 0){
-WSMFtrace("WSDxfont::getXFontSet  miss=%d \n",miss_cnt);
-      long i;
-      for(i=0; i < miss_cnt;i++){
-WSMFtrace("   ----> miss font=%s \n",miss_font[i]);
-      }
+    if (miss_cnt > 0){
+      XFreeStringList(miss_font);
     }
   }
   return _xfont_set;
 }
 
 long WSDxfont::destroy(){
+  if (_font_set_name != NULL){
+    delete[] _font_set_name; 
+    _font_set_name = NULL;
+  }
+  setValue1(-1);
+  setValue2(-1);
+  setValue3(-1);
+#if 0 //old version
   XFontStruct* xfs1 = (XFontStruct*)getValue1();
   if ( xfs1 != (XFontStruct*)-1 && xfs1 != NULL){
     XFreeFont(disp,xfs1);
@@ -294,28 +388,66 @@ long WSDxfont::destroy(){
     XFreeFont(disp,xfs3);
     setValue3(-1);
   }
+#endif
+  if (_xfont_set != 0){
+    WSDxappDev* app = WSGIxwinAppDev(); 
+    Display* disp = app->display();
+    XFreeFontSet(disp, _xfont_set);
+    _xfont_set = 0;
+  }
   return WS_NO_ERR;
 }
+char* WSDxfont::getFontSetName(){
+#if 0
+  WSCstring ret;
+  ret = _fmly;
+  if (_style == WS_FONT_BOLD){
+    ret << ".b";
+  }else
+  if (_style == (WS_FONT_BOLD | WS_FONT_ITALIC)){
+    ret << ".b.i";
+  }else
+  if (_style == WS_FONT_ITALIC){
+    ret << ".i";
+  }
+//printf("WSDxfont::getFontSetName ret=%s 0x%x\n",(char*)ret,_style);
+  return ret;
+#endif
+  getValue1();
+  return _font_set_name;
+}
 long WSDxfont::getStringWidthUCS2(WSCushort* str){
+  getValue1();
   if (disp == NULL){
     WSDxappDev* app   = WSGIxwinAppDev(); 
     disp = app->display();
   }
-  XFontStruct* xfs1 = (XFontStruct*)getValue1();
-  return XUTextWidth(disp, xfs1, str,-1);
+//  XFontStruct* xfs1 = (XFontStruct*)getValue1();
+//  return XUTextWidth(disp, xfs1, str,-1);
+  WSCstring fname = getFontSetName();
+  return XUTextWidth(disp, fname, getFontHeight(),str,-1);
 }
 long WSDxfont::getStringHeightUCS2(WSCushort* str){
   getValue1();
   return  _font_height;
 }
 long WSDxfont::getStringWidth(WSCstring* str){
+  getValue1();
   if (disp == NULL){
     WSDxappDev* app   = WSGIxwinAppDev(); 
     disp = app->display();
   }
-  XFontStruct* xfs1 = (XFontStruct*)getValue1();
-  char* str1 = str->getString(WS_EN_UTF8);
-  return XUutf8TextWidth(disp, xfs1,str1,-1);
+  WSCushort* ucs2 = str->getInternalUcs2();
+  if (ucs2 == NULL){
+    return 0;
+  }
+  WSCstring fname = getFontSetName();
+  long ret = XUTextWidth(disp, fname,getFontHeight(),ucs2,-1);
+  return ret;
+
+//  char* str1 = str->getString(WS_EN_UTF8);
+//  XFontStruct* xfs1 = (XFontStruct*)getValue1();
+//  return XUutf8TextWidth(disp, xfs1,str1,-1);
 
 #if 0
   long dx = 0;
@@ -363,3 +495,34 @@ long WSDxfont::getStringHeight(WSCstring*){
   getValue1();
   return  _font_height;
 }
+long WSDxfont::getAscent(){
+  getValue1();
+  if (disp == NULL){
+    WSDxappDev* app   = WSGIxwinAppDev(); 
+    disp = app->display();
+  }
+//  XFontStruct* xfs1 = (XFontStruct*)getValue1();
+//  return XUTextWidth(disp, xfs1, str,-1);
+  WSCstring fname = getFontSetName();
+  int direction = 0;
+  int ascent = 0;
+  int descent = 0;
+  XUFontExtents(disp, fname, getFontHeight(),&direction,&ascent,&descent);
+  return  ascent;
+}
+long WSDxfont::getDescent(){
+  getValue1();
+  if (disp == NULL){
+    WSDxappDev* app   = WSGIxwinAppDev(); 
+    disp = app->display();
+  }
+//  XFontStruct* xfs1 = (XFontStruct*)getValue1();
+//  return XUTextWidth(disp, xfs1, str,-1);
+  WSCstring fname = getFontSetName();
+  int direction = 0;
+  int ascent = 0;
+  int descent = 0;
+  XUFontExtents(disp, fname, getFontHeight(),&direction,&ascent,&descent);
+  return  descent;
+}
+
