@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,18 +14,15 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <X11/Xlib.h>
 #include <WScom.h>
+#ifndef WS_EMBED
 #include <WSDappDev.h>
 #include <WSCbaseDialog.h>
 #include <WSCform.h>
@@ -82,7 +79,8 @@ WSCbaseDialog(base,objName){
   opt004 = NULL;
   opt005 = NULL;
   opt006 = NULL;
-  _font_name = WSGFstrdup("14 jisx0201.1976 jisx0208.1983 * misc misc misc");
+//  _font_name = WSGFstrdup("14 jisx0201.1976 jisx0208.1983 * misc misc misc");
+  _font_name = WSGFstrdup("14 * 0 0");
   _target_name = WSGFstrdup("");
   _rep = NULL;
   _font_no = 0;
@@ -101,7 +99,8 @@ WSCbaseDialog(base,objName){
     WSMFaddTrigger(3);
   WSMFpropertyCreateEnd
 }
-WSMFproperty(WSDxfontSelect,WSNfontName,char*,_font_name,WSGFstrdup("14 jisx0201.1976 jisx0208.1983 * misc misc misc"));
+//WSMFproperty(WSDxfontSelect,WSNfontName,char*,_font_name,WSGFstrdup("14 jisx0201.1976 jisx0208.1983 * misc misc misc"));
+WSMFproperty(WSDxfontSelect,WSNfontName,char*,_font_name,WSGFstrdup("14 * 0 0"));
 void WSDxfontSelect::setWorkWSNfontName(char* data){
   WSCstring str(data);
   _size = atoi((char*)str.getWord(0));
@@ -120,15 +119,15 @@ void WSDxfontSelect::setWorkWSNfontName(char* data){
   }
 //  if (strcmp(WSGIappLocaleSet()->getDefaultLocaleName(),WS_LOCALE_JAPANESE)){
     //C
-    _ec = str.getWord(1);
+    _ec = "*";
     _kc = "*";
     _gc = "*";
-    _ev = str.getWord(4);
+    _ev = "*";
     _kv = "*";
     _gv = "*";
-    _fmly = str.getWord(2);
-    _wght = str.getWord(3);
-    _slant = str.getWord(5);
+    _fmly = str.getWord(1);
+    _wght = "*";
+    _slant = "*";
 //  }else{
 //    //japanese
 //    _ec = str.getWord(1);
@@ -144,18 +143,47 @@ void WSDxfontSelect::setWorkWSNfontName(char* data){
 
 void WSDxfontSelect::getWorkWSNfontName(char** fn){
   if (_target_name != NULL && _target_name[0] != 0){
+    WSCstring filename("$(WSDIR)/sys/data/fontdef.");
+    filename << _target_name;
+    WSCstring* str = WSGFreadTextFile(filename);
+    if (str != NULL){
+      str->delLineFeed();
+      if (!strcmp(str->getString(),"")){
+        return;
+      }
+      WSCstring tmp( WSGFltoa(_size) );
+      str->replaceString("%d",tmp,1);
+      delete _font_name;
+      _font_name = WSGFstrdup(str->getString());
+      delete str;
+      *fn = _font_name;
+      return;
+    }
+#if 0
     WSCstring str( WSGFltoa(_size) );
     str << " 60c6";
     delete _font_name;
     _font_name = WSGFstrdup((char*)str);
     *fn = _font_name;
-    return;
+#endif
   }
   WSCstring str( WSGFltoa(_size));
 //  if (strcmp(WSGIappLocaleSet()->getDefaultLocaleName(),WS_LOCALE_JAPANESE)){
     //C
-    str << " " << _ec << " " << _fmly << " " << _wght
-        << " " << _ev << " " << _slant << " " << "*"; 
+    str << " " << _fmly << " ";
+    if (!strcmp("*",(char*)_wght) || !strcmp("medium",_wght)){
+      str << "0 ";
+    }else{
+      str << "1 ";
+    }
+    if (!strcmp("*",(char*)_slant) || !strcmp("r",_slant)){
+      str << "0 ";
+    }else{
+      str << "1 ";
+    }
+
+//    str << " " << _ec << " " << _fmly << " " << _wght
+//        << " " << _ev << " " << _slant << " " << "*"; 
 //  }else{
 //    //japanese
 //    str << " " << _ec << " " << _kc << " " << _gc
@@ -325,13 +353,14 @@ long WSDxfontSelect::initialize(){
 //  opt004->setPropertyV(WSNbottomShadowColor,"gray55");
   opt004->setPropertyV(WSNalignmentH,(WSCuchar)0);
   opt004->setPropertyV(WSNx,(short)285);
-  opt004->setPropertyV(WSNy,(short)85);
+  opt004->setPropertyV(WSNy,(short)45);
+//  opt004->setPropertyV(WSNy,(short)85);
   opt004->setPropertyV(WSNwidth,(WSCushort)150);
   opt004->setPropertyV(WSNheight,(WSCushort)20);
   WSCprocedure* op_opt004_0 = new WSCprocedure("new_ep",WSEV_VALUE_CH);
   op_opt004_0->setFunction(xfdVch,"xfdVch");
   opt004->addProcedure(op_opt004_0);
-//  opt004->setVisible(True);
+  opt004->setVisible(True);
 
   opt005 = new  WSCoption(xfonarea001,"opt005");
   opt005->initialize();
@@ -342,13 +371,14 @@ long WSDxfontSelect::initialize(){
 //  opt005->setPropertyV(WSNbottomShadowColor,"gray55");
   opt005->setPropertyV(WSNalignmentH,(WSCuchar)0);
   opt005->setPropertyV(WSNx,(short)285);
-  opt005->setPropertyV(WSNy,(short)105);
+  opt005->setPropertyV(WSNy,(short)65);
+//  opt005->setPropertyV(WSNy,(short)105);
   opt005->setPropertyV(WSNwidth,(WSCushort)150);
   opt005->setPropertyV(WSNheight,(WSCushort)20);
   WSCprocedure* op_opt005_0 = new WSCprocedure("new_ep",WSEV_VALUE_CH);
   op_opt005_0->setFunction(xfdVch,"xfdVch");
   opt005->addProcedure(op_opt005_0);
-//  opt005->setVisible(True);
+  opt005->setVisible(True);
 
   opt006 = new  WSCoption(xfonarea001,"opt006");
   opt006->initialize();
@@ -362,8 +392,8 @@ long WSDxfontSelect::initialize(){
   opt006->setPropertyV(WSNy,(short)125);
   opt006->setPropertyV(WSNwidth,(WSCushort)150);
   opt006->setPropertyV(WSNheight,(WSCushort)20);
-  WSCprocedure* op_opt006_0 = new WSCprocedure("new_ep",WSEV_VALUE_CH);
-  op_opt006_0->setFunction(xfdVch,"xfdVch");
+//  WSCprocedure* op_opt006_0 = new WSCprocedure("new_ep",WSEV_VALUE_CH);
+//  op_opt006_0->setFunction(xfdVch,"xfdVch");
 //  opt006->addProcedure(op_opt006_0);
 
 //  if (!strcmp(WSGIappLocaleSet()->getDefaultLocaleName(),WS_LOCALE_JAPANESE)){
@@ -426,35 +456,37 @@ long WSDxfontSelect::initialize(){
   xfonvtex017->setPropertyV(WSNheight,(WSCushort)20);
   xfonvtex017->setPropertyV(WSNshadowType,(signed char)WS_SHADOW_TRANS);
   xfonvtex017->setVisible(True);
-#if 0
-  WSCvlabel* xfonvtex018 = new  WSCvlabel(xfonarea001,"xfonvtex018");
+
+  xfonvtex018 = new  WSCvlabel(xfonarea001,"xfonvtex018");
   xfonvtex018->initialize();
   xfonvtex018->setInternalObject(True);
   xfonvtex018->setUserData(WS_BASE_CLASS,(void*)this);
   xfonvtex018->setPropertyV(WSNlabelString,WSGFgettext(_WS_STR5));
   xfonvtex018->setPropertyV(WSNalignmentH,(WSCuchar)2);
   xfonvtex018->setPropertyV(WSNx,(short)210);
-  xfonvtex018->setPropertyV(WSNy,(short)85);
+  xfonvtex018->setPropertyV(WSNy,(short)45);
+//  xfonvtex018->setPropertyV(WSNy,(short)85);
   xfonvtex018->setPropertyV(WSNwidth,(WSCushort)70);
   xfonvtex018->setPropertyV(WSNheight,(WSCushort)20);
   xfonvtex018->setPropertyV(WSNshadowType,(signed char)WS_SHADOW_TRANS);
   xfonvtex018->setVisible(True);
-#endif
 
-#if 0
-  WSCvlabel* xfonvtex019 = new  WSCvlabel(xfonarea001,"xfonvtex019");
+
+
+  xfonvtex019 = new  WSCvlabel(xfonarea001,"xfonvtex019");
   xfonvtex019->initialize();
   xfonvtex019->setInternalObject(True);
   xfonvtex019->setUserData(WS_BASE_CLASS,(void*)this);
   xfonvtex019->setPropertyV(WSNlabelString,WSGFgettext(_WS_STR6));
   xfonvtex019->setPropertyV(WSNalignmentH,(WSCuchar)2);
   xfonvtex019->setPropertyV(WSNx,(short)210);
-  xfonvtex019->setPropertyV(WSNy,(short)105);
+  xfonvtex019->setPropertyV(WSNy,(short)65);
+//  xfonvtex019->setPropertyV(WSNy,(short)105);
   xfonvtex019->setPropertyV(WSNwidth,(WSCushort)70);
   xfonvtex019->setPropertyV(WSNheight,(WSCushort)20);
   xfonvtex019->setPropertyV(WSNshadowType,(signed char)WS_SHADOW_TRANS);
   xfonvtex019->setVisible(True);
-#endif
+
 
   WSCvlabel* xfonvtex020 = new  WSCvlabel(xfonarea001,"xfonvtex020");
   xfonvtex020->initialize();
@@ -580,6 +612,7 @@ void WSDxfontSelect::xfdVch(WSCbase* object){
   base->updateList();
 
   WSCstring buf( WSGFltoa(base->_size));
+#if 0
   if (japanese == False){
     //C
     buf << " " << base->_ec << " " << base->_fmly << " " << base->_wght
@@ -589,6 +622,20 @@ void WSDxfontSelect::xfdVch(WSCbase* object){
     buf << " " << base->_ec << " " << base->_kc << " " << base->_gc
         << " " << base->_ev << " " << base->_kv << " " << base->_gv; 
   }
+#endif
+  buf << " " << base->_fmly << " ";
+  if (!strcmp("*",(char*)base->_wght) || !strcmp("medium",(char*)base->_wght)){
+    buf << "0 ";
+  }else{
+    buf << "1 ";
+  }
+  if (!strcmp("*",(char*)base->_slant) || !strcmp("r",(char*)base->_slant)){
+    buf << "0 ";
+  }else{
+    buf << "1 ";
+  }
+
+
   WSCuchar fontn;
   base->getPropertyV(WSNfont,&fontn);
 //printf("WSDxfontSelect::xfdVch fno=%d %s\n",fontn,(char*)buf);
@@ -663,6 +710,8 @@ void WSDxfontSelect::onVisibleChange(WSCbool fl){
     lb004->setPropertyV(WSNlabelString,WSGFgettext(_WS_STR8));
     xfonvtex014->setPropertyV(WSNlabelString,WSGFgettext(_WS_STR1));
     xfonvtex017->setPropertyV(WSNlabelString,WSGFgettext(_WS_STR4));
+    xfonvtex018->setPropertyV(WSNlabelString,WSGFgettext(_WS_STR5));
+    xfonvtex019->setPropertyV(WSNlabelString,WSGFgettext(_WS_STR6));
   }
   updateList();
 }
@@ -1143,3 +1192,4 @@ void WSDxfontSelect::onResize(WSCrect* area){
     xfonarea001->setProperty(WSNheight,1);
   }
 }
+#endif
