@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -38,23 +34,21 @@
 #include <x11/WSxcom.h>
 #include <x11/WSDxdragDrop.h>
 #include <form.h>
-//TEST
-#include <x11/WSDxscrFrameDev.h>
-//TEST
+#include <x11/WSDxkeyboard.h>
 
 WSMFclassInit(WSDxformDev,WSDwindowDev);
 
 WSDdev* _wsdxformdev_init_(){
   return new WSDxformDev();
 }
-
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _WSDxformDev_init_ {
   public:_WSDxformDev_init_(){
      WSGIappDevice()->setCreateHandler("formDev",_wsdxformdev_init_);
   };
 };
 static _WSDxformDev_init_ _init_to_run_WSDxformDev_;
-
+#endif
 //static WSDxdraw* draw_of_xform = NULL;
 
 WSDxformDev::WSDxformDev(){
@@ -66,6 +60,11 @@ WSDxformDev::WSDxformDev(){
   _bg_pix = -1;
   _mno = 0;
   _click_time = 0;
+  _global_mouse = False;
+  _mouse_val = 0;
+#ifndef NO_MWT_FUNCTION
+  _need_grab = False;
+#endif
 
 //  if (draw_of_xform == NULL){
 //      draw_of_xform = new WSDxdraw();
@@ -225,14 +224,31 @@ long WSDxformDev::setValue(long kind,void* data){
              }
              fl = *(WSCbool*)data;
              if (fl != False){
+#ifndef NO_MWT_FUNCTION
+               if (getNeedGrab() != False){
+                 WSGIxwinAppDev()->setGrabbedDisplay(XtDisplay(_widget));
+                 XGrabServer(XtDisplay(_widget));
+                 if (WSGIxwinAppDev()->getOwnerGrab() != False){
+                   WSGIxwinAppDev()->setOwnerGrab(False);
+                 }
+               }
+#endif
                XGrabPointer(XtDisplay(_widget),XtWindow(_widget),False,
                   ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                   GrabModeAsync,GrabModeAsync,None,None,CurrentTime);
                XGrabKeyboard(XtDisplay(_widget),XtWindow(_widget),False,
                   GrabModeAsync,GrabModeAsync,CurrentTime);
+               WSGIappMouse()->setMouseFocusClient(NULL);
              }else{
                XUngrabPointer(XtDisplay(_widget),CurrentTime);
                XUngrabKeyboard(XtDisplay(_widget),CurrentTime);
+#ifndef NO_MWT_FUNCTION
+               if (getNeedGrab() != False){
+                 XUngrabServer(XtDisplay(_widget));
+                 WSGIxwinAppDev()->setGrabbedDisplay(NULL);
+                 WSGIxwinAppDev()->setOwnerGrab(False);
+               }
+#endif
              }
              break;
      case WSDEV_MOUSE_NO:
@@ -240,7 +256,11 @@ long WSDxformDev::setValue(long kind,void* data){
              if (_widget != NULL && XtWindow(_widget) != 0){
                long val = WSGIappMouse()->getMouseCursor(_mouse_no);
                if (val != 0){
-                 XDefineCursor(XtDisplay(_widget),XtWindow(_widget),val);
+                 if (_mouse_val != val){
+                   _global_mouse = WSGIappMouse()->getGlobalMouseIdStatus();
+                   _mouse_val = val;
+                   XDefineCursor(XtDisplay(_widget),XtWindow(_widget),val);
+                 }
                }
              }
              break;
@@ -326,7 +346,6 @@ WSDxformDev* WSDxformDev::getParentAreaDev(){
 void WSDxformDev::_setup_pixmap(short x,short y,WSCushort w,WSCushort h,short bg,short img_no){
   WSDcolor*  color = WSGIappColorSet()->getColor(bg);
 
-
   GC gc = WSGIxwinAppDev()->appGC2();
   if (gc == 0){
       return;
@@ -345,6 +364,15 @@ void WSDxformDev::_setup_pixmap(short x,short y,WSCushort w,WSCushort h,short bg
          WSCushort pw = image->getImageWidth();
          WSCushort ph = image->getImageHeight();
          Pixmap pid = (Pixmap)image->getValue1();
+
+         XRectangle xrect[1];
+         xrect[0].x = (WSCuint)x;
+         xrect[0].y = (WSCuint)y;
+         xrect[0].width = (WSCuint)w;
+         xrect[0].height = (WSCuint)h;
+
+         XSetClipRectangles(disp,gc,0,0,xrect,1,Unsorted);
+
          short cw = 0;
          while(1){
              short ch = 0;
@@ -358,7 +386,7 @@ void WSDxformDev::_setup_pixmap(short x,short y,WSCushort w,WSCushort h,short bg
              cw += pw;
              if (cw > w) break;
          }
-
+         XSetClipMask(disp,gc,None);
       }else{
         XFillRectangle(disp,pix, gc,(int)x,(int)y,(WSCuint)w,(WSCuint)h);
       }
@@ -379,7 +407,7 @@ long WSDxformDev::createWindow(){
       return WS_NO_ERR;
     }
   }
-
+  _mouse_val = 0;
   short x,y;    
   WSCushort w,h;    
   getWindowSize(&x,&y,&w,&h);
@@ -470,7 +498,6 @@ long WSDxformDev::createPixmap(){
   if (h <1){
     h = 1;
   }
-
   Pixmap pix = WSGFxGetPixmap(w,h);
 
   setPixmap(pix);
@@ -591,16 +618,22 @@ long WSDxformDev::setVisible(WSCbool fl){
 //        createPixmap();
 //DEVELOP//////////////////////////////////////////////////////////////////////
     }
-
     XtManageChild(_widget);
   }
   return WS_NO_ERR;
 }
 
 void WSDxformDev::_evh_mouse_in(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
+  if (WSGIappMouse()->getGlobalMouseSensitive() == False){
+    WSGIxwinAppDev()->setXEvent(NULL);
+    return;
+  }
   if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
     WSGIxwinAppDev()->_dispatchEvent(False,ev);
   }
+  if (ev->xcrossing.mode != NotifyNormal){
+    return;
+  }
   WSGIxwinAppDev()->setXEvent(ev);
   WSDxformDev* dev = (WSDxformDev*)ptr;
 
@@ -610,7 +643,11 @@ void WSDxformDev::_evh_mouse_in(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
   long val = WSGIappMouse()->getMouseCursor(dev->_mouse_no);
   Display* disp = WSGIxwinAppDev()->display();
   if (val != 0){
-    XDefineCursor(disp,XtWindow((Widget)dev->_widget),val);
+    if (dev->_mouse_val != val){
+      dev->_mouse_val = val;
+      dev->_global_mouse = WSGIappMouse()->getGlobalMouseIdStatus();
+      XDefineCursor(disp,XtWindow((Widget)dev->_widget),val);
+    }
   }
 
   dev->onMouseIn(&point);
@@ -618,47 +655,68 @@ void WSDxformDev::_evh_mouse_in(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
 }
 
 void WSDxformDev::_evh_mouse_out(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
+  if (WSGIappMouse()->getGlobalMouseSensitive() == False){
+    WSGIxwinAppDev()->setXEvent(NULL);
+    return;
+  }
   if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
     WSGIxwinAppDev()->_dispatchEvent(False,ev);
   }
-   if (ev->xcrossing.mode != NotifyNormal){
-     return;
-   }
-   WSGIxwinAppDev()->setXEvent(ev);
-   WSDxformDev* dev = (WSDxformDev*)ptr;
+  if (ev->xcrossing.mode != NotifyNormal){
+    return;
+  }
+  WSGIxwinAppDev()->setXEvent(ev);
+  WSDxformDev* dev = (WSDxformDev*)ptr;
 
   dev->onMouseOut();
   WSGIxwinAppDev()->setXEvent(NULL);
 }
 
 void WSDxformDev::_evh_mouse_move(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
+  if (WSGIappMouse()->getGlobalMouseSensitive() == False){
+    WSGIxwinAppDev()->setXEvent(NULL);
+    return;
+  }
   if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
     WSGIxwinAppDev()->_dispatchEvent(False,ev);
   }
-   WSGIxwinAppDev()->setXEvent(ev);
-   WSDxformDev* dev = (WSDxformDev*)ptr;
+  WSGIxwinAppDev()->setXEvent(ev);
+  WSDxformDev* dev = (WSDxformDev*)ptr;
+  WSCbool gmouse = WSGIappMouse()->getGlobalMouseIdStatus();
+  if (gmouse != dev->_global_mouse){
+    if (dev->_widget != NULL && XtWindow(dev->_widget) != 0){
+      long val = WSGIappMouse()->getMouseCursor(dev->_mouse_no);
+      if (val != 0){
+        if (dev->_mouse_val != val){
+          dev->_mouse_val = val;
+          dev->_global_mouse = gmouse;
+          XDefineCursor(XtDisplay(dev->_widget),XtWindow(dev->_widget),val);
+        }
+      }
+    }
+  }
 
-   WSCpoint point;
-   point.setPoint( ev->xmotion.x, ev->xmotion.y );
+  WSCpoint point;
+  point.setPoint( ev->xmotion.x, ev->xmotion.y );
 
-   WSCbool b1 =  (ev->xmotion.state & Button1Mask) != 0 ? True:False;
-   WSCbool b2 =  (ev->xmotion.state & Button2Mask) != 0 ? True:False;
-   WSCbool b3 =  (ev->xmotion.state & Button3Mask) != 0 ? True:False;
+  WSCbool b1 =  (ev->xmotion.state & Button1Mask) != 0 ? True:False;
+  WSCbool b2 =  (ev->xmotion.state & Button2Mask) != 0 ? True:False;
+  WSCbool b3 =  (ev->xmotion.state & Button3Mask) != 0 ? True:False;
 
-   long state = WSGIappMouse()->getStatus();
-   if (b1){
-     state |= WS_MOUSE_BTN1;
-   }
-   if (b2){
-     state |= WS_MOUSE_BTN2;
-   }
-   if (b3){
-     state |= WS_MOUSE_BTN3;
-   }
-   WSGIappMouse()->setStatus(state);
+  long state = WSGIappMouse()->getStatus();
+  if (b1){
+    state |= WS_MOUSE_BTN1;
+  }
+  if (b2){
+    state |= WS_MOUSE_BTN2;
+  }
+  if (b3){
+    state |= WS_MOUSE_BTN3;
+  }
+  WSGIappMouse()->setStatus(state);
 
-   dev->onMouseMove(&point);
-   WSGIxwinAppDev()->setXEvent(NULL);
+  dev->onMouseMove(&point);
+  WSGIxwinAppDev()->setXEvent(NULL);
 
 }
 
@@ -713,16 +771,35 @@ void WSDxformDev::_evh_resize(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
 }
 
 void WSDxformDev::_evh_btn_press(Widget w,XtPointer ptr,XEvent* ev,WSCbool*){
+  if (WSGIappMouse()->getGlobalMouseSensitive() == False){
+    WSGIxwinAppDev()->setXEvent(NULL);
+    return;
+  }
   if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
 //printf("XtSetKeyboardFocus..\n");
     Widget w2 = XtGetKeyboardFocusWidget(w);
     XtSetKeyboardFocus(w2,w);
     WSGIxwinAppDev()->_dispatchEvent(False,ev);
   }
-   WSGIxwinAppDev()->setXEvent(ev);
-   WSDxformDev* dev = (WSDxformDev*)ptr;
-   WSCpoint point;
-   point.setPoint( ev->xbutton.x,ev->xbutton.y );
+  WSGIxwinAppDev()->setXEvent(ev);
+  WSDxformDev* dev = (WSDxformDev*)ptr;
+  WSCpoint point;
+  point.setPoint( ev->xbutton.x,ev->xbutton.y );
+
+#ifndef NO_MWT_FUNCTION
+  if (dev->getNeedGrab() != False){
+    // for avoiding x11's dead lock bug in XRegisterIMInstantiateCallback()
+    WSGIxkeyboard();
+
+    WSGIxwinAppDev()->setGrabbedDisplay(XtDisplay(w));
+    WSGIxwinAppDev()->setOwnerGrab(True);
+    int ret = XGrabServer(XtDisplay(w));
+//    int ret = XGrabPointer(XtDisplay(w),XtWindow(w),False,
+//                  ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
+//                  GrabModeAsync,GrabModeAsync,None,None,CurrentTime);
+  }
+#endif
+
 #if 0
    long state = WSGIappMouse()->getStatus();
    if (ev->xbutton.button == 1){
@@ -737,42 +814,76 @@ void WSDxformDev::_evh_btn_press(Widget w,XtPointer ptr,XEvent* ev,WSCbool*){
    }
    WSGIappMouse()->setStatus(state);
 #endif
-   long clk = WSGFclocktime();
+  long clk = WSGFclocktime();
+#if 0 // moved to WSDxappDev::_dispatchEvent
+  if (ev->xbutton.button == 4){
+    WSCbase* fcl = WSGIappMouse()->getMouseFocusClient();
+    if (fcl != NULL){
+      fcl->onMousePress(&point);
+    }
+  }else if (ev->xbutton.button == 5){
+    WSCbase* fcl = WSGIappMouse()->getMouseFocusClient();
+    if (fcl != NULL){
+      fcl->onMousePress(&point);
+    }
+  }
+#endif
+  dev->onMousePress(&point);
+  if ((WSCulong)clk - (WSCulong)dev->_click_time < 250){
+    dev->onMouseDoubleClick(&point);
+  }
+  dev->_click_time = clk;
 
-   if (ev->xbutton.button == 4){
-     WSCbase* fcl = WSGIappMouse()->getMouseFocusClient();
-     if (fcl != NULL){
-       fcl->onMousePress(&point);
-     }
-   }else if (ev->xbutton.button == 5){
-     WSCbase* fcl = WSGIappMouse()->getMouseFocusClient();
-     if (fcl != NULL){
-       fcl->onMousePress(&point);
-     }
-   }
-   dev->onMousePress(&point);
-   if ((WSCulong)clk - (WSCulong)dev->_click_time < 250){
-     dev->onMouseDoubleClick(&point);
-   }
-   dev->_click_time = clk;
-   WSGIxwinAppDev()->setXEvent(NULL);
+  WSCbool gmouse = WSGIappMouse()->getGlobalMouseIdStatus();
+  if (gmouse != dev->_global_mouse){
+    if (dev->_widget != NULL && XtWindow(dev->_widget) != 0){
+      long val = WSGIappMouse()->getMouseCursor(dev->_mouse_no);
+      if (val != 0){
+        if (dev->_mouse_val != val){
+          dev->_mouse_val = val;
+          dev->_global_mouse = gmouse;
+          XDefineCursor(XtDisplay(dev->_widget),XtWindow(dev->_widget),val);
+        }
+      }
+    }
+  }
+
+  WSGIxwinAppDev()->setXEvent(NULL);
 }
 
-void WSDxformDev::_evh_btn_release(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
+void WSDxformDev::_evh_btn_release(Widget w,XtPointer ptr,XEvent* ev,WSCbool*){
+  if (WSGIappMouse()->getGlobalMouseSensitive() == False){
+    WSGIxwinAppDev()->setXEvent(NULL);
+    return;
+  }
   if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
     WSGIxwinAppDev()->_dispatchEvent(False,ev);
   }
-   WSGIxwinAppDev()->setXEvent(ev);
-   WSDxformDev* dev = (WSDxformDev*)ptr;
-   WSDxdragDrop* xdd = WSGIxappDragDrop();
-   if (xdd != NULL){
-     if (xdd->getDragState() != 0){
-       xdd->executeDrop();
-	   return;
-	 }
-   }
-   WSCpoint point;
-   point.setPoint( ev->xbutton.x, ev->xbutton.y );
+  WSGIxwinAppDev()->setXEvent(ev);
+  WSDxformDev* dev = (WSDxformDev*)ptr;
+
+
+#ifndef NO_MWT_FUNCTION
+  if (dev->getNeedGrab() != False){
+//    XUngrabPointer(XtDisplay(w),CurrentTime);
+    if (WSGIappMouse()->getStatus() == 0){
+      XUngrabServer(XtDisplay(w));
+      WSGIxwinAppDev()->setGrabbedDisplay(NULL);
+      WSGIxwinAppDev()->setOwnerGrab(False);
+    }
+  }
+#endif
+#ifndef NO_DRAG_DROP
+  WSDxdragDrop* xdd = WSGIxappDragDrop();
+  if (xdd != NULL){
+    if (xdd->getDragState() != 0){
+      xdd->executeDrop();
+      return;
+    }
+  }
+#endif //NO_DRAG_DROP
+  WSCpoint point;
+  point.setPoint( ev->xbutton.x, ev->xbutton.y );
 #if 0
    long state = WSGIappMouse()->getStatus();
    if (ev->xbutton.button == 1){
@@ -787,8 +898,23 @@ void WSDxformDev::_evh_btn_release(Widget,XtPointer ptr,XEvent* ev,WSCbool*){
    }
    WSGIappMouse()->setStatus(state);
 #endif
-   dev->onMouseRelease(&point);
-   WSGIxwinAppDev()->setXEvent(NULL);
+  dev->onMouseRelease(&point);
+
+  WSCbool gmouse = WSGIappMouse()->getGlobalMouseIdStatus();
+  if (gmouse != dev->_global_mouse){
+    if (dev->_widget != NULL && XtWindow(dev->_widget) != 0){
+      long val = WSGIappMouse()->getMouseCursor(dev->_mouse_no);
+      if (val != 0){
+        if (dev->_mouse_val != val){
+          dev->_mouse_val = val;
+          dev->_global_mouse = gmouse;
+          XDefineCursor(XtDisplay(dev->_widget),XtWindow(dev->_widget),val);
+        }
+      }
+    }
+  }
+
+  WSGIxwinAppDev()->setXEvent(NULL);
 }
 
 
@@ -861,3 +987,64 @@ void WSDxformDev::clearChildWidgets(){
   }
   _clear_widget();
 }
+
+#ifndef NO_RESTACK_CHILD
+void WSDxformDev::restackChildren(){
+  if (getAttachedClient() == NULL){
+    return;
+  }
+  if (_widget == 0){
+    return;
+  }
+  WSClistData children = getAttachedClient()->getChildren();
+  long num = children.getNum();
+  long i;
+  WSClistData wlist;
+  for(i=0; i<num; i++){
+    WSCbase* child = (WSCbase*)children[num - i -1];
+    WSDdev* dev = child->getowndev();
+    if (dev == NULL){
+      continue;
+    }
+    WSDwindowDev* wdev = (WSDwindowDev*)dev->cast("WSDwindowDev");
+    if (wdev == NULL){
+      continue;
+    }
+    Widget widget = (Widget)wdev->getSpecialResource();
+    if (widget == 0){
+      continue;
+    }
+    while(1){
+      Widget pwidget = XtParent(widget);
+      if (pwidget == 0){
+        break;
+      }
+      if (_widget == pwidget){
+        Window w = XtWindow(widget);
+        if (w != 0){
+          wlist.add((void*)w);
+          break;
+        }
+      }
+      widget = XtParent(widget);
+    }
+  }
+  num = wlist.getNum();
+  Display* disp = WSGIxwinAppDev()->display();
+  XRestackWindows(disp,(Window*)wlist.getBuf(),num);
+}
+#endif
+#ifndef NO_MWT_FUNCTION
+WSCbool WSDxformDev::getNeedGrab(){
+  if (_need_grab != False){
+    return True;
+  }
+  WSDxformDev* parent = getParentAreaDev();
+  if (parent == NULL){
+    return False;
+  }
+  return parent->getNeedGrab();
+}
+#endif
+
+
