@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <X11/Xlib.h>
 #include <unistd.h>
@@ -33,6 +29,7 @@
 #define RTLD_LAZY 0
 extern void* dlopen(char* fname,long);
 extern void* dlsym(void*,char* procn);
+#include <WSDenv.h>
 #endif //MacOS
 #include <WScom.h>
 #include <WSCimageSet.h>
@@ -41,10 +38,12 @@ extern void* dlsym(void*,char* procn);
 #include <x11/WSDximage.h>
 #include <x11/WSDxappDev.h>
 #include <x11/WSDxcolor.h>
+#include <WSDfile.h>
 
 long WSGFxBufToPixmap(Display* display,Window window,GC gc,unsigned char *buf,Pixmap* pixmap,long width,long height,long* vw,long* vh);
 long WSGFxBufToBitmap(Display* display,Window window,GC gc,unsigned char *buf,Pixmap* pixmap,long width,long height,long* vw,long* vh);
-long WSGFxBmpReadFpToPixmap(Display* display,Window window,GC gc,FILE* ifile,Pixmap* pixmap,Pixmap* pixmap2,long* w,long* h);
+//long WSGFxBmpReadFpToPixmap(Display* display,Window window,GC gc,FILE* ifile,Pixmap* pixmap,Pixmap* pixmap2,long* w,long* h);
+long WSGFxBmpReadFpToPixmap(Display* display,Window window,GC gc,WSDfile* ifile,Pixmap* pixmap,Pixmap* pixmap2,long* w,long* h);
 long WSGFxBmpReadFileToPixmap(Display* display,Window window,GC gc,char* filename,Pixmap* pixmap,Pixmap* pixmap2,long* w,long* h);
 
 WSMFclassInit(WSDximage,WSDimage);
@@ -84,7 +83,6 @@ long WSDximage::initialize(){
                        long*,long*) = NULL;
      static long (*fn2)(Display*,Window,Colormap,char**,Pixmap*,Pixmap*,
                        long*,long*) = NULL;
-
      if (dl_init == 0){
 #ifndef MacOS
        dl = dlopen("libwsxpm.so",RTLD_LAZY);
@@ -92,7 +90,12 @@ long WSDximage::initialize(){
          dl = dlopen("./libwsxpm.so",RTLD_LAZY);
        }
 #else //MacOS
-       dl = dlopen("/usr/lib/xpm.bundle",RTLD_LAZY);
+       dl = dlopen(
+           WSGIappEnvironment()->getPlaneString("$(WSDIR)/lib/xpm.bundle"),
+           RTLD_LAZY);
+       if (dl == NULL){
+         dl = dlopen("/usr/lib/xpm.bundle",RTLD_LAZY);
+       }
        if (dl == NULL){
          dl = dlopen("./xpm.bundle",RTLD_LAZY);
        }
@@ -102,7 +105,7 @@ long WSDximage::initialize(){
 #ifndef MacOS
 WSMFtrace("ERROR. in WSDximage::initialize(). Library libwsxpm.so open error. \nso, Xpm image can not support...\n");
 #else //MacOS
-WSMFtrace("ERROR. in WSDximage::initialize(). Library /usr/lib/xpm.bundle open error. \nso, Xpm image can not support...\n");
+WSMFtrace("ERROR. in WSDximage::initialize(). Library xpm.bundle open error. \nso, Xpm image can not support...\n");
 #endif //MacOS
        }else{
          fn = (long (*)(Display*,Window,Colormap,char*,
@@ -185,7 +188,12 @@ WSMFtrace("ERROR. in WSDximage::initialize(). Library libXpm.so is invalid. \nso
          dl = dlopen("./libwsjpg.so",RTLD_LAZY);
        }
 #else //MacOS
-       dl = dlopen("/usr/lib/jpg.bundle",RTLD_LAZY);
+       dl = dlopen(
+           WSGIappEnvironment()->getPlaneString("$(WSDIR)/lib/jpg.bundle"),
+           RTLD_LAZY);
+       if (dl == NULL){
+         dl = dlopen("/usr/lib/jpg.bundle",RTLD_LAZY);
+       }
        if (dl == NULL){
          dl = dlopen("./jpg.bundle",RTLD_LAZY);
        }
@@ -255,7 +263,12 @@ WSMFtrace("ERROR. in WSDximage::initialize(). Library libwsjpg.so is invalid. \n
          dl = dlopen("./libwspng.so",RTLD_LAZY);
        }
 #else //MacOS
-       dl = dlopen("/usr/lib/png.bundle",RTLD_LAZY);
+       dl = dlopen(
+           WSGIappEnvironment()->getPlaneString("$(WSDIR)/lib/png.bundle"),
+           RTLD_LAZY);
+       if (dl == NULL){
+         dl = dlopen("/usr/lib/png.bundle",RTLD_LAZY);
+       }
        if (dl == NULL){
          dl = dlopen("./png.bundle",RTLD_LAZY);
        }
@@ -290,6 +303,7 @@ WSMFtrace("ERROR. in WSDximage::initialize(). Library libwspng.so is invalid. \n
      }
 
   }else if (in.isExist(".bmp") != -1 || in.isExist(".BMP") != -1){
+#ifndef NO_FILE_SYSTEM
 //WSMFtrace("WSDximage::initialize %s\n",in.getString());
      Pixmap pixmap_file;
      Pixmap pixmap_file2;
@@ -312,6 +326,7 @@ WSMFtrace("ERROR. in WSDximage::initialize(). Library libwspng.so is invalid. \n
      }else{
 //WSMFtrace("ERROR. WSDximage::initialize file open error. file=%s\n",load_file_name);
      }
+#endif //NO_FILE_SYSTEM
   }
 
   return WS_NO_ERR;
@@ -336,7 +351,7 @@ long WSDximage::destroy(){
   XImage* image = (XImage*)_value3;
   if ((long)image != 0 && (long)image != -1){
     if ( image->data != NULL){
-      delete image->data;
+      free(image->data);
       image->data = NULL;
     }
     XDestroyImage(image);
@@ -353,7 +368,7 @@ long WSDximage::createImage(){
   XImage* image = (XImage*)getValue3();
   if ((long)image != 0 && (long)image != -1){
     if ( image->data != NULL){
-      delete image->data;
+      free(image->data);
       image->data = NULL;
     }
     XDestroyImage(image);
@@ -396,50 +411,51 @@ XImage* WSDximage::createMaskImage(){
                     getImageHeight(),0xffffffff,ZPixmap);
   return image;
 }
-int WSGFxgetbyte(FILE *fp) {
-  int c;
-  if ((c = fgetc(fp)) == EOF){
+
+
+int WSGFxgetbyte(WSDfile* fp) {
+  WSCuchar c[1];
+  if (fp->read(c,1) < 1){
     return -1;
   }
-  return c;
+  return c[0];
 }
 
-int WSGFxreadLittleShort(FILE *fp, short *d) {
-  int c;
-  if ((c = fgetc(fp)) == EOF){
+int WSGFxreadLittleShort(WSDfile* fp, short *d) {
+  WSCuchar c[1];
+  if (fp->read(c,1) < 1){
     return -1;
   }
-  *d = c & 0xff;
-  if ((c = fgetc(fp)) == EOF){
+  *d = c[0] & 0xff;
+  if (fp->read(c,1) < 1){
     return -1;
   }
-  *d |= (c & 0xff) << 8;
+  *d |= (c[0] & 0xff) << 8;
   return 0;
 }
 
-int WSGFxreadLittleLong(FILE *fp, long *d) {
-  int c;
-
-  if ((c = fgetc(fp)) == EOF){
+int WSGFxreadLittleLong(WSDfile* fp, long *d) {
+  WSCuchar c[1];
+  if (fp->read(c,1) < 1){
     return -1;
   }
-  *d = c & 0xff;
-  if ((c = fgetc(fp)) == EOF){
+  *d = c[0] & 0xff;
+  if (fp->read(c,1) < 1){
     return -1;
   }
-  *d |= ((c & 0xff) << 8);
-  if ((c = fgetc(fp)) == EOF){
+  *d |= ((c[0] & 0xff) << 8);
+  if (fp->read(c,1) < 1){
     return -1;
   }
-  *d |= ((c & 0xff) << 16);
-  if ((c = fgetc(fp)) == EOF){
+  *d |= ((c[0] & 0xff) << 16);
+  if (fp->read(c,1) < 1){
     return -1;
   }
-  *d |= ((c & 0xff) << 24);
+  *d |= ((c[0] & 0xff) << 24);
   return 0;
 }
 
-long WSGFxBmpReadFpToPixmap(Display* display,Window window,GC gc,FILE* ifile,Pixmap* pixmap,Pixmap* pixmap2,long* w,long* h){
+long WSGFxBmpReadFpToPixmap(Display* display,Window window,GC gc,WSDfile* ifile,Pixmap* pixmap,Pixmap* pixmap2,long* w,long* h){
 
   // bmp (Windows 3.x) header
   //       implemented
@@ -543,11 +559,11 @@ long WSGFxBmpReadFpToPixmap(Display* display,Window window,GC gc,FILE* ifile,Pix
         return -1;
     }
   }
-  if (fseek(ifile, foffs, SEEK_SET) != 0){
+  if (ifile->seek(foffs) != WS_NO_ERR){
     return -1;
   }
   unsigned char* fbuf = new unsigned char[fsize - foffs];
-  if (fread(fbuf, fsize - foffs, 1, ifile) != 1) {
+  if (ifile->read(fbuf, fsize - foffs) != (fsize - foffs)) {
     delete fbuf;
     return -1;
   }
@@ -558,7 +574,6 @@ long WSGFxBmpReadFpToPixmap(Display* display,Window window,GC gc,FILE* ifile,Pix
   }
   unsigned char *tmp, *ftmp;
   unsigned char *tmp_mask;
-
   int line, dat, x, y;
 //printf("cmptype=%d w,h=%d,%d bit=%d\n",cmptype,width,height,bit_cnt);
   if (cmptype) {
@@ -775,6 +790,7 @@ long WSGFxBmpReadFpToPixmap(Display* display,Window window,GC gc,FILE* ifile,Pix
   return reti;
 }
 
+
 long WSGFxBufToPixmap(Display* display,Window window,GC gc,unsigned char *buf,Pixmap* pixmap,long rwidth,long rheight,long* w,long* h){
 
   int red_mask, green_mask, blue_mask;
@@ -855,7 +871,7 @@ long WSGFxBufToPixmap(Display* display,Window window,GC gc,unsigned char *buf,Pi
                          ZPixmap,0,0,vwidth,vheight,8,0);
   }
 
-  image->data = new char[image->bytes_per_line * vheight];
+  image->data = (char*)malloc(image->bytes_per_line * vheight);
 
   unsigned char r,g,b;
   long ptr = 0;
@@ -864,6 +880,7 @@ long WSGFxBufToPixmap(Display* display,Window window,GC gc,unsigned char *buf,Pi
 
   if (image->bits_per_pixel ==32){
     for(i=0; i<vheight; i++){
+      ptr2 = i * image->bytes_per_line;
       for(j=0; j<vwidth; j++){
         if (stretched != 0){
           ptr = (long)((double)i*rheight/vheight)*rwidth
@@ -946,6 +963,7 @@ long WSGFxBufToPixmap(Display* display,Window window,GC gc,unsigned char *buf,Pi
     }
   }else if (image->bits_per_pixel ==16){
     for(i=0; i<vheight; i++){
+      ptr2 = i * image->bytes_per_line;
       for(j=0; j<vwidth; j++){
         if (stretched != 0){
           ptr = (long)((double)i*rheight/vheight)*rwidth
@@ -982,99 +1000,151 @@ long WSGFxBufToPixmap(Display* display,Window window,GC gc,unsigned char *buf,Pi
       }
     }
   }else if (image->bits_per_pixel == 8){
-    XColor col[5*5*5];
-    Colormap cm = DefaultColormap(display,DefaultScreen(display));
-    long k;
-    long cnt=0;
-    for(i=0; i<5; i++){
-      for(j=0; j<5; j++){
-        for(k=0; k<5; k++){
-          if (i != 4){
-            col[cnt].red = i*64*256;
-          }else{
-            col[cnt].red = 0xffff;
-          }
-          if (j != 4){
-            col[cnt].green = j*64*256;
-          }else{
-            col[cnt].green = 0xffff;
-          }
-          if (k != 4){
-            col[cnt].blue = k*64*256;
-          }else{
-            col[cnt].blue = 0xffff;
-          }
-          XAllocColor(display,cm,&(col[cnt]));
-          cnt++;
-        }
+    const int nr=5, ng=5, nb=5;
+    const unsigned int maxcols=nr*ng*nb;
+    XColor col[maxcols];
+    unsigned long collist[maxcols], c;
+    unsigned long k, cnt;
+
+    cnt=0;
+    for(i=0; i<rheight; i++){
+      for(j=0; j<rwidth; j++){
+	c=buf[i*rwidth+j*3];
+	c=(c<<8)|buf[i*rwidth+j*3+1];
+	c=(c<<8)|buf[i*rwidth+j*3+2];
+	for( k=0 ; k<cnt ; k++ ){
+	    if( collist[k]==c ){
+		break;
+	    }
+	}
+	if( k>=cnt ){
+	    if( cnt<maxcols ){
+		collist[cnt++]=c;
+	    }else{
+		break;
+	    }
+	}
       }
+      if( j<rwidth )break;
     }
+    if( i<rheight ){
+	Colormap cm = DefaultColormap(display,DefaultScreen(display));
+	long k;
+	long cnt=0;
+	for(i=0; i<nr; i++){
+	  for(j=0; j<ng; j++){
+	    for(k=0; k<nb; k++){
+	      col[cnt].red = ((i*255)/(nr-1))*0x101;
+	      col[cnt].green = ((j*255)/(ng-1))*0x101;
+	      col[cnt].blue = ((k*255)/(nb-1))*0x101;
+	      XAllocColor(display,cm,&col[cnt]);
+	      cnt++;
+	    }
+	  }
+        }
+        long colptr = 0;
+	long rr = 0,gg = 0,bb = 0;
+	long tr1,tg1,tb1;
+	signed char dr1,dg1,db1;
+	for(i=0; i<vheight; i++){
+	  dr1 = 0;
+	  dg1 = 0;
+	  db1 = 0;
+	  for(j=0; j<vwidth; j++){
+	    if (stretched != 0){
+	      ptr = (long)((double)i*rheight/vheight)*rwidth
+		+ (long)((double)j*rwidth/vwidth);
+	      ptr *= 3;
+	    }
+	    tr1 = buf[ptr++];
+	    tg1 = buf[ptr++];
+	    tb1 = buf[ptr++];
+	    if ( 0 < tr1 + dr1 && tr1 + dr1 < 256 ){
+	      if ( -128/(nr-1) < dr1 && dr1 < 128/(nr-1) ){ 
+		rr = tr1 + dr1 + 128/(nr-1)-1 ;
+	      }else if (dr1 > 128/(nr-1)-1 ){
+		rr = tr1 + 256/(nr-1)-1 ;
+	      }else{
+		rr = tr1 ;
+	      }
+	      rr -= rr % (256/(nr-1));
+	    }else if (tr1 + dr1 > 255){
+	      rr = 0x100;
+	    }else{
+	      rr = 0;
+	    }
+	    dr1 += (tr1 - rr);
+	    if (0 < tg1 + dg1  && tg1 + dg1 < 256){
+		if (-128/(ng-1) < dg1 && dg1 < 128/(ng-1) ){
+		    gg = tg1 + dg1 + 128/(ng-1)-1 ;
+		}else if (dg1 > 128/(ng-1)-1){
+		    gg = tg1 + 256/(ng-1)-1 ;
+		}else{
+		    gg = tg1 ;
+		}
+		gg -= gg % (256/(ng-1));
+	    }else if (tg1 + dg1 > 255){
+		gg = 0x100;
+	    }else{
+		gg = 0;
+	    }
+	    dg1 += (tg1 - gg);
 
-    long colptr = 0;
-    long rr = 0,gg = 0,bb = 0;
-    long tr1,tg1,tb1;
-    signed char dr1,dg1,db1;
-    for(i=0; i<vheight; i++){
-      dr1 = 0;
-      dg1 = 0;
-      db1 = 0;
-      for(j=0; j<vwidth; j++){
-        if (stretched != 0){
-          ptr = (long)((double)i*rheight/vheight)*rwidth
-               + (long)((double)j*rwidth/vwidth);
-          ptr *= 3;
-        }
-        tr1 = buf[ptr++];
-        tg1 = buf[ptr++];
-        tb1 = buf[ptr++];
+	    if (0 < tb1 + db1 && tb1 + db1 < 256){
+		if (-(128/(nb-1)) < db1 && db1 < 128/(nb-1)){
+		    bb = tb1 + db1 + 128/(nb-1)-1 ;
+		}else if (db1 > 31){
+		    bb = tb1 + 256/(nb-1)-1 ;
+		}else{
+		    bb = tb1 ;
+		}
+		bb -= bb % (256/(nb-1));
+	    }else if (tb1 + db1 > 255){
+		bb = 0x100;
+	    }else{
+		bb = 0;
+	    }
+	    db1 += (tb1 - bb);
 
-        if ((0 < tr1 + dr1) && (tr1 + dr1) < 256){
-          if ((-32 < dr1) && (dr1 < 32)){
-            rr = (tr1 + dr1 + 31) & 0x1c0;
-          }else if (dr1 > 31){
-            rr = (tr1 + 63) & 0x1c0;
-          }else{
-            rr = tr1  & 0x1c0;
-          }
-        }else if (tr1 + dr1 > 255){
-          rr = 0x100;
-        }else{
-          rr = 0;
-        }
-        dr1 += (tr1 - rr);
-        if (0 < tg1 + dg1  &&tg1 + dg1 < 256){
-          if (-32 < dg1 && dg1 < 32 ){
-            gg = (tg1 + dg1 + 31) & 0x1c0;
-          }else if (dg1 > 31){
-            gg = (tg1 + 63) & 0x1c0;
-          }else{
-            gg = tg1  & 0x1c0;
-          }
-        }else if (tg1 + dg1 > 255){
-          gg = 0x100;
-        }else{
-          gg = 0;
-        }
-        dg1 += (tg1 - gg);
-
-        if (0 < tb1 + db1 && tb1 + db1 < 256){
-          if (-32 < db1 && db1 < 32){
-            bb = (tb1 + db1 + 31) & 0x1c0;
-          }else if (db1 > 31){
-            bb = (tb1 + 63) & 0x1c0;
-          }else{
-            bb = tb1 & 0x1c0;
-          }
-        }else if (tb1 + db1 > 255){
-            bb = 0x100;
-        }else{
-          bb = 0;
-        }
-        db1 += (tb1 - bb);
-        db1 += (tb1 - bb);
-        colptr = (rr>>6)*5*5  + (gg>>6)*5 + (bb>>6);
-        image->data[ptr2++] =  col[colptr].pixel;
+	    k=rr/(256/(nr-1)); if( k>=nr )k=nr-1;
+	    colptr=k;
+	    k=gg/(256/(ng-1)); if( k>=ng )k=ng-1;
+	    colptr=colptr*ng+k;
+	    k=bb/(256/(nr-1)); if( k>=nb )k=nb-1;
+	    colptr=colptr*nb+k;
+	    image->data[ptr2++] =  col[colptr].pixel;
+	}
       }
+    }else{
+	for(i=0;i<cnt;i++){
+	    c=collist[i];
+	    col[i].red=0x101*((c>>16)&0xFF);
+	    col[i].green=0x101*((c>>8)&0xFF);
+	    col[i].blue=0x101*(c&0xFF);
+	}
+	Colormap cm = DefaultColormap(display,DefaultScreen(display));
+	for( i=0 ; i<cnt ; i++ ){
+	    XAllocColor( display, cm, &col[i] );
+	}
+	for( i=0 ; i<vheight ; i++ ){
+	    for( j=0 ; j<vwidth ; j++ ){
+		if (stretched != 0){
+		    ptr = (long)((double)i*rheight/vheight)*rwidth
+			    + (long)((double)j*rwidth/vwidth);
+		    ptr *= 3;
+		}
+
+		c=buf[ptr++];
+		c=(c<<8)|buf[ptr++];
+		c=(c<<8)|buf[ptr++];
+		for( k=0 ; k<cnt-1 ; k++ ){
+		    if( collist[k]==c ){
+			break;
+		    }
+		}
+		image->data[j+i*image->bytes_per_line]=col[k].pixel;
+	    }
+	}
     }
   }
 
@@ -1082,7 +1152,7 @@ long WSGFxBufToPixmap(Display* display,Window window,GC gc,unsigned char *buf,Pi
   *pixmap = pix;
   XPutImage(display,pix,gc,image,0,0,0,0,vwidth,vheight);
   if (image->data != NULL){
-    delete image->data;
+    free(image->data);
     image->data = NULL;
   }
   XDestroyImage(image);
@@ -1112,7 +1182,7 @@ long WSGFxBufToBitmap(Display* display,Window window,GC gc,unsigned char *buf,Pi
                          ZPixmap,0,0,vwidth,vheight,8,0);
 
 //printf("here1 image=0x%x bytes_per_line=%d w.h\n",image,image->bytes_per_line,vwidth,vheight);
-  image->data = new char[image->bytes_per_line * vheight];
+  image->data = (char*)malloc(image->bytes_per_line * vheight);
 
   unsigned char v;
   long ptr = 0;
@@ -1129,7 +1199,11 @@ long WSGFxBufToBitmap(Display* display,Window window,GC gc,unsigned char *buf,Pi
         v = 0;
       }
       if (buf[ptr++] == 0){
-        v |= (1<<(j%8));
+	  if( image->bitmap_bit_order==LSBFirst ){
+	      v |= (1<<(j%8));
+	  }else{
+	      v |= (1<<(8-1-(j%8)));
+	  }
       }
 //printf("i=%d j=%d pt=%d pt2=%d bit=%d val=0x%x v=0x%x\n",i,j,ptr,ptr2,j%8,buf[ptr-1],v);
       if ( (j%8) == 7){
@@ -1153,7 +1227,7 @@ long WSGFxBufToBitmap(Display* display,Window window,GC gc,unsigned char *buf,Pi
   *pixmap = pix;
   XPutImage(display,pix,gc2,image,0,0,0,0,vwidth,vheight);
   if (image->data != NULL){
-    delete image->data;
+    free(image->data);
     image->data = NULL;
   }
   XFreeGC(display,gc2);
@@ -1163,13 +1237,15 @@ long WSGFxBufToBitmap(Display* display,Window window,GC gc,unsigned char *buf,Pi
 
 long WSGFxBmpReadFileToPixmap(Display* display,Window window,GC gc,char* filename,Pixmap* pixmap,Pixmap* pixmap2,long* w,long* h){
 
-  FILE* ifile = fopen(filename,"r");
+  WSDfile* ifile = WSDfile::getNewInstance();
   if (ifile == NULL){
     return -1;
   }
+  if (ifile->open(filename,"r") != WS_NO_ERR){
+    return -1;
+  }
 
   long reti = WSGFxBmpReadFpToPixmap(display, window, gc, ifile, pixmap,pixmap2, w, h);
-  fclose(ifile);
+  delete ifile;
   return reti;
 }
-
