@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <x11/WSDxkeyboard.h>
 #include <X11/Xutil.h>
@@ -37,6 +33,7 @@
 #include <WSCcolorSet.h>
 #include <WSCfontSet.h>
 #include <WSClocaleSet.h>
+#include <xunicode.h>
 
 WSMFclassInit(WSDxkeyboard,WSDkeyboard);
 
@@ -184,73 +181,13 @@ extern long WSGFgetEncodingValue(char*);
   return WS_NO_ERR;
 }
 
-long WSDxkeyboard::startKanji(WSDdev* dev,short x,short y,short fontno,short fgno,short bgno){
+long WSDxkeyboard::startFep(WSDdev* dev,short x,short y,short fontno,short fgno,short bgno){
   WSDfont*    font  = WSGIappFontSet()->getFont(fontno);
   WSDcolor*   fgcol = WSGIappColorSet()->getColor(fgno);
   WSDcolor*   bgcol = WSGIappColorSet()->getColor(bgno);
 #ifdef KINPUT2
-
-  setDevForKI(dev);
-  _x_kn = x;
-  _y_kn = y;
-  _font_kn = fontno;
-  _fg_kn = fgno;
-  _bg_kn = bgno;
-  WSDxappDev* app   = WSGIxwinAppDev(); 
-  if (_atom1 == 0){
-     _atom1 = XInternAtom(app->display(),"_JAPANESE_CONVERSION",False);
-  }
-  if (_atom2 == 0){
-     _atom2 = XInternAtom(app->display(),"_CONPOUND_TEXT",False);
-  }
-  XFontStruct* _ef = (XFontStruct*)font->getValue1();
-  XFontStruct* _kf = (XFontStruct*)font->getValue2();
-  XFontStruct* _gf = (XFontStruct*)font->getValue3();
-  XFontStruct*  fonts[4];
-  long          num = 0;
-  if (_ef != (XFontStruct*)-1 && _ef != NULL){
-    fonts[num] = _ef;
-    num++;
-  }
-  if (_kf != (XFontStruct*)-1 && _kf != NULL){
-    fonts[num] = _kf;
-    num++;
-  }
-  if (_gf != (XFontStruct*)-1 && _gf != NULL){
-    fonts[num] = _gf;
-    num++;
-  }
-  fonts[num] = 0;
-  WSCstring tmpstr("ABjpy");
-  Arg args[10];
-
-  short px,py;
-  dev->getAbsoluteAddr(x,y + font->getStringHeight(&tmpstr),&px,&py);
-  dev->setAbsoluteAddrChangeHandler(_device_move_handler,this);
-
-  XtSetArg(args[0],"inputStyle","over");
-  XtSetArg(args[1],"spotX",px);
-  XtSetArg(args[2],"spotY",py); 
-  XtSetArg(args[3],"eventCaptureMethod","focusSelect");
-  XtSetArg(args[4],"fonts",fonts);
-  XtSetArg(args[5],"foreground",fgcol->getValue1());
-  XtSetArg(args[6],"background",bgcol->getValue1());
-  _beginConversionWithAttributes((Widget)dev->getSpecialResource(),_atom1,_atom2,
-                     _kanji_input, _kanji_start_end,this,args,7);
-  _fep_on = True; 
+  // not support
 #else //XIM
-
-#if 0
-  if(_xic != 0 ){
-    if (_focused == True){
-      XUnsetICFocus(_xic);
-      XDestroyIC(_xic);
-      _xic = NULL;
-      _focused = False;
-    }
-  }
-#endif
-
   setDevForXIM(dev);
   WSDxfont* xfont = (WSDxfont*)font->cast("WSDxfont");
   if (xfont != NULL){
@@ -275,8 +212,10 @@ long WSDxkeyboard::startKanji(WSDdev* dev,short x,short y,short fontno,short fgn
     if (_xic != 0){
       XVaNestedList list;
       XVaNestedList list2;
-      XFontSet* font_set = (XFontSet*)xfont->getXFontSet();
+      Display* disp =  XtDisplay((Widget)dev->getSpecialResource());
 
+      XFontSet font_set = xfont->getXFontSet();
+//printf("font_set2=0x%x\n",font_set);
       list = XVaCreateNestedList(0, XNFontSet,font_set,
                XNSpotLocation,&point, XNArea,&area, 
                XNForeground,fgcol->getValue1(),
@@ -364,13 +303,7 @@ long WSDxkeyboard::startKanji(WSDdev* dev,short x,short y,short fontno,short fgn
 
 long WSDxkeyboard::startAscii(WSDdev* dev,short ,short ,short ,short ,short ){
 #ifdef KINPUT2
-  setDevForKI(dev);
-  if (_fep_on == True){
-    _endConversion((Widget)dev->getSpecialResource(), _atom1, True);
-    if (dev != NULL){
-      dev->setAbsoluteAddrChangeHandler(NULL,NULL);
-    }
-  }
+  // not support
 #else  //XIM
   if(_xic != 0 ){
     if (_focused == True){
@@ -386,18 +319,25 @@ long WSDxkeyboard::startAscii(WSDdev* dev,short ,short ,short ,short ,short ){
   return WS_NO_ERR;
 }
 
-
-long WSDxkeyboard::resetFep(WSDdev* dev){
-#ifdef KINPUT2
-  if (_fep_on == True){
-    _endConversion((Widget)dev->getSpecialResource(), _atom1, True);
-    if (dev != NULL){
-      dev->setAbsoluteAddrChangeHandler(NULL,NULL);
+long WSDxkeyboard::setFepStatus(long val){
+  if (val == WS_FEP_STOP){
+    if (_xic != 0){
+      if (_focused == True){
+        XUnsetICFocus(_xic);
+        _focused = False;
+      }
+      XDestroyIC(_xic);
+      _xic = NULL;
+    }
+    if (_xim != 0){
+      XCloseIM( _xim );
+      _xim = 0;
     }
   }
-  _fep_on = False;
-  setDevForKI(NULL);
-  return WS_NO_ERR;
+}
+long WSDxkeyboard::resetFep(WSDdev* dev){
+#ifdef KINPUT2
+  // not support
 #else  //KINPUT2
   if(_xic != 0 ){
     if (_focused == True){
@@ -495,24 +435,7 @@ XIC WSDxkeyboard::_get_xic(Window win){
 
 long WSDxkeyboard::setFepPos(WSDdev* dev,short x,short y,short fontno,short fgno,short bgno){
 #ifdef KINPUT2
-  setDevForKI(dev);
-  _x_kn = x;
-  _y_kn = y;
-  _font_kn = fontno;
-  _fg_kn = fgno;
-  _bg_kn = bgno;
-  if (dev != NULL && _fep_on == True){
-//    WSCstring tmpstr("ABjpy");
-    WSDfont*  font  = WSGIappFontSet()->getFont(fontno);
-    short px,py;
-    dev->getAbsoluteAddr(x,y + font->getFontHeight(),&px,&py);
-    dev->setAbsoluteAddrChangeHandler(_device_move_handler,this);
-    Arg args[2];
-    XtSetArg(args[0],"spotX",px);
-    XtSetArg(args[1],"spotY",py); 
-    _changeConversionAttributes((Widget)dev->getSpecialResource(), _atom1,args,2);
-  }
-
+  // not support
 #else  //XIM
 static short x_bk = 0;
 static short y_bk = 0;
@@ -544,8 +467,13 @@ static short y_bk = 0;
     if (_xim_init != False){
       XVaNestedList list;
       XVaNestedList list2;
+      Widget w = (Widget)dev->getSpecialResource();
+      if (w == 0){
+        return WS_ERR;
+      }
+      Display* disp =  XtDisplay(w);
 
-      XFontSet* font_set = (XFontSet*)xfont->getXFontSet();
+      XFontSet font_set = xfont->getXFontSet();
       list = XVaCreateNestedList(0,
                XNFontSet,font_set,
                XNSpotLocation,&point,
@@ -570,13 +498,14 @@ static short y_bk = 0;
                   XNStatusDoneCallback,&callback_status_done, 
                   XNStatusDrawCallback,&callback_status_draw, NULL,NULL);
       if (_xic == 0){
-        _xic = _create_xic(XtWindow((Widget)dev->getSpecialResource()),
-                         list,list2);
-        if (_xic != 0 && _has_no_focus_win == False){
-          Window win = 0;
-          XGetICValues(_xic,XNFocusWindow,&win, NULL);
-          if (win == 0){
-            _has_no_focus_win = True;
+        if (getEnableFep() != False){
+          _xic = _create_xic(XtWindow(w),list,list2);
+          if (_xic != 0 && _has_no_focus_win == False){
+            Window win = 0;
+            XGetICValues(_xic,XNFocusWindow,&win, NULL);
+            if (win == 0){
+              _has_no_focus_win = True;
+            }
           }
         }
       }else{
@@ -596,14 +525,14 @@ static short y_bk = 0;
           }
           XSetICValues(_xic, XNPreeditAttributes,list, XNStatusAttributes,
                        list2, NULL);
-       }else if(win != XtWindow((Widget)dev->getSpecialResource())){
-          Display* disp =  XtDisplay((Widget)dev->getSpecialResource());
-          Window   target =  XtWindow((Widget)dev->getSpecialResource());
+       }else if(win != XtWindow((Widget)w)){
+          Display* disp =  XtDisplay((Widget)w);
+          Window   target =  XtWindow((Widget)w);
           XWindowAttributes att;
           XGetWindowAttributes(disp, target, &att);
           XDestroyIC(_xic);
           _xic = NULL;
-          _xic = _create_xic(XtWindow((Widget)dev->getSpecialResource()),
+          _xic = _create_xic(XtWindow((Widget)w),
                            list,list2);
           WSCulong mask;
           XGetICValues(_xic,XNFilterEvents,&mask,NULL);
@@ -648,52 +577,35 @@ void WSDxkeyboard::setModeWindowPos(short x,short y,WSDdev* dev){
 }
 
 
-#ifdef KINPUT2
-void WSDxkeyboard::_kanji_input(Widget,Atom,Atom,int pformat,
-                    WSCulong psize,WSCuchar *data,XtPointer dt){
-  WSDxkeyboard* obj = (WSDxkeyboard*)dt;
-  if (pformat == 8){
-    char *buf = new char[psize+1];
-    memcpy(buf,data,(WSCuint)psize);
-    buf[psize] = 0;
-    char* kanji_euc_str = WSGFcreateEucStringFromJisStr(buf);
-    delete buf;
-    obj->setKey(0);
-    obj->_with_cntl = False;
-    obj->_with_lock =  False;
-    obj->_with_shift = False;
-    obj->_with_alt = False;
-    obj->_is_function = False;
-    obj->_is_cursor = False;
-    obj->_is_tty_function = False;
-    obj->setText(kanji_euc_str,WSGIappCodeConvert()->getSystemEncoding());
-    delete kanji_euc_str;
-    obj->executeHandler(True);
-    obj->executeHandler(False);
-  }
-  return;
-}
-
-void WSDxkeyboard::_kanji_start_end(Widget,Atom,int state,XtPointer dt,Window){
-  WSDxkeyboard* obj = (WSDxkeyboard*)dt;
-  if (state == 1 || state == -1){
-    obj->_fep_on = False;
-  }
-}
-#endif
 #ifndef KINPUT2
 void WSDxkeyboard::setEnableFep(WSDdev* dev,WSCbool fl){
   WSDkeyboard::setEnableFep(dev,fl);
-  return;
-#if 0
+//  return;
   if (fl == False){
     if (_xic != 0){
       if (_focused == True){
         XUnsetICFocus(_xic);
         _focused = False;
       }
+      XDestroyIC(_xic);
+      _xic = NULL;
     }
+    if (_xim != 0){
+      XCloseIM( _xim );
+      _xim = 0;
+    }
+  }
+#if 0
   }else if (dev != NULL){
+    if (_xic == 0){
+      if (_xic != 0 && _has_no_focus_win == False){
+        Window win = 0;
+        XGetICValues(_xic,XNFocusWindow,&win, NULL);
+        if (win == 0){
+          _has_no_focus_win = True;
+        }
+      }
+    }
     setDevForXIM(dev);
     Window win ;
     if(_xic != 0){
@@ -738,6 +650,9 @@ void WSDxkeyboard::setEnableFep(WSDdev* dev,WSCbool fl){
 #endif
 
 WSCbool WSDxkeyboard::deliverKeyEvent(XEvent* ev){
+  if (WSGIappKeyboard()->getGlobalKeySensitive() == False){
+    return False;
+  }
   WSCbool press = False;
   if ( ev->type == KeyPress){
      press = True;
@@ -745,40 +660,10 @@ WSCbool WSDxkeyboard::deliverKeyEvent(XEvent* ev){
   setIsPressed(press);
 
   char buffer[1024];
+  memset(buffer,0,sizeof(buffer));
   KeySym key = 0;
 #ifdef KINPUT2 
-  if (ev->type == FocusIn ){
-    return True;
-  }
-  if (_fep_on != False ){
-    static WSCulong time_bk = 0;
-    if (_dev_for_kn != NULL && _dev_for_kn->getVisible() != False){
-      Window win = XtWindow((Widget)_dev_for_kn->getSpecialResource());
-      if (ev->xany.window != win){
-        ev->xany.window = win;
-        if (ev->type == KeyPress){
-          XSendEvent( XtDisplay((Widget)_dev_for_kn->getSpecialResource()),
-                  win,False,KeyPressMask,ev);
-        }else{
-          XSendEvent( XtDisplay((Widget)_dev_for_kn->getSpecialResource()),
-                  win,False,KeyReleaseMask,ev);
-        }
-        return False;
-      }
-    }
-    if (ev->xkey.time != time_bk){
-      time_bk = ev->xkey.time;
-      return False;
-    }
-  }
-
-  int len = XLookupString((XKeyEvent *)ev,buffer,1022,&key,(XComposeStatus*)NULL);
-  buffer[len]=0;
-  if (len == 1 && ((WSCuchar)buffer[0] < 0x20 || (WSCuchar)buffer[0] > 0x7f)){
-     buffer[0] = 0;
-     len = 0;
-  }
-
+  // not support
 #else //XIM
   if (_xic == 0){
     if (ev->type == FocusIn){
@@ -790,6 +675,11 @@ WSCbool WSDxkeyboard::deliverKeyEvent(XEvent* ev){
        buffer[0] = 0;
        len = 0;
     }
+    if (len == 0 && 0x81 < key && key < 0x100){
+       buffer[0] = key;
+       buffer[1] = 0;
+       len = 1;
+    }
 
   }else{
     if (_dev_for_xim != NULL ){ 
@@ -828,14 +718,22 @@ WSCbool WSDxkeyboard::deliverKeyEvent(XEvent* ev){
       }
     }
     Status status = XLookupNone;
-    int len = XmbLookupString(_xic,(XKeyEvent *)ev,buffer,1022, &key,&status);
-    buffer[len] = 0;
-    if(status == XLookupNone){
-         len = XLookupString((XKeyEvent *)ev,buffer,1022,&key,(XComposeStatus*)NULL);
-    }
-    if (len == 1 && (buffer[0] < 0x20 || (unsigned char)buffer[0] > 0x7f)){
-       buffer[0] = 0;
-       len = 0;
+    int len = 0;
+    if (ev->type != FocusIn){
+      len = XmbLookupString(_xic,(XKeyEvent *)ev,buffer,1022, &key,&status);
+      if (len > -1 && len < 1023){
+        buffer[len] = 0;
+      }    
+      if(status == XLookupNone){
+        //this causes a segmentation fault.. because it calls with FocusIn event?
+        len = XLookupString((XKeyEvent *)ev,buffer,1022,&key,(XComposeStatus*)NULL);
+      }
+      if (len == 1 && (buffer[0] < 0x20 || (unsigned char)buffer[0] > 0x7f)){
+         buffer[0] = 0;
+         len = 0;
+      }
+    }else{
+       return True;
     }
 
   }
@@ -864,22 +762,8 @@ WSCbool WSDxkeyboard::deliverKeyEvent(XEvent* ev){
   }
 
 #ifdef KINPUT2
-  if ( ev->type == KeyPress && (
-   (key == WSK_space && withShift() == True ) ||
-#ifndef SUN
-               key == WSK_Kanji ||
-#else 
-               key == WSK_Henkan ||
+  // not support
 #endif
-        (key == WSK_Zenkaku_Hankaku && withAlt() == True) ||
-        (key == WSK_Hiragana_Katakana ) )){
-    if (_dev_for_kn != NULL && getEnableFep() == True ){
-      startKanji(_dev_for_kn,_x_kn,_y_kn,_font_kn,_fg_kn,_bg_kn);
-      return False;
-    }
-  }
-#endif
-
   WSCbool executed = executeHandler(press);
   if (executed == True){
     return False;
