@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
@@ -36,13 +32,19 @@ WSMFclassInit(WSDxmessage,WSDmessage);
 WSDmessage* _xclient_msg_create(){
   return new WSDxmessage;
 }
-
+void WSGFxmessageInitialize(){
+  WSDmessage::setCreateHandler(_xclient_msg_create);
+  WSDmessage::_setup_ext_send_msg_proc((void*)WSDxmessage::extSendMessage,
+                                       (void*)WSDxmessage::extSendNewMessage);
+}
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _xclient_msg_init {
   public: _xclient_msg_init(){
-    WSDmessage::setCreateHandler(_xclient_msg_create);
+    WSGFxmessageInitialize();
   };
 };
 _xclient_msg_init  _xclient_msg_init_execute;
+#endif
 
 WSDxmessage::WSDxmessage(){
  _ext_send_msg1 = extSendMessage;
