@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <X11/Intrinsic.h>
 #include <x11/WSDxscrFrameDev.h>
@@ -33,7 +29,8 @@
 #include <WSCcolorSet.h>
 #include <X11/StringDefs.h>
 #include <form.h>
-#include <WSCbase.h>
+#include <WSDmouse.h>
+
 
 
 WSMFclassInit(WSDxscrFrameDev,WSDxformDev);
@@ -41,12 +38,14 @@ WSDdev* _wsdxscrdev_init_(){
   return new WSDxscrFrameDev();
 }
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _WSDxscrFrameDev_init_ {
   public:_WSDxscrFrameDev_init_(){
      WSGIappDevice()->setCreateHandler("scrFrameDev",_wsdxscrdev_init_);
   };
 };
 static _WSDxscrFrameDev_init_ _init_to_run_WSDxscrFrameDev_;
+#endif
 
 WSDxscrFrameDev::WSDxscrFrameDev(){
   _scr_form = 0;
@@ -234,14 +233,31 @@ long WSDxscrFrameDev::setValue(long kind,void* data){
              }
              fl = *(WSCbool*)data;
              if (fl != False){
+#ifndef NO_MWT_FUNCTION
+               if (getNeedGrab() != False){
+                 WSGIxwinAppDev()->setGrabbedDisplay(XtDisplay(_widget));
+                 XGrabServer(XtDisplay(_widget));
+                 if (WSGIxwinAppDev()->getOwnerGrab() != False){
+                   WSGIxwinAppDev()->setOwnerGrab(False);
+                 }
+               }
+#endif
                XGrabPointer(XtDisplay(_widget),XtWindow(_widget),False,
                   ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                   GrabModeAsync,GrabModeAsync,None,None,CurrentTime);
                XGrabKeyboard(XtDisplay(_widget),XtWindow(_widget),False,
                   GrabModeAsync,GrabModeAsync,CurrentTime);
+               WSGIappMouse()->setMouseFocusClient(NULL);
              }else{
                XUngrabPointer(XtDisplay(_widget),CurrentTime);
                XUngrabKeyboard(XtDisplay(_widget),CurrentTime);
+#ifndef NO_MWT_FUNCTION
+               if (getNeedGrab() != False){
+                 XUngrabServer(XtDisplay(_widget));
+                 WSGIxwinAppDev()->setGrabbedDisplay(NULL);
+                 WSGIxwinAppDev()->setOwnerGrab(False);
+               }
+#endif
              }
              break;
 
