@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <X11/Xlib.h>
 #include <WScom.h>
@@ -44,13 +40,23 @@
 #include <x11/WSxcom.h>
 
 #ifdef SUN
-#define SOCKLEN_T  int
-#else //SUN
-#ifdef MacOS
-#define SOCKLEN_T  int
-#else //MacOS
+
+#ifdef __GNUC__
+#if __GNUC__ > 2
 #define SOCKLEN_T  socklen_t
-#endif //MacOS
+#else
+#define SOCKLEN_T  int
+#endif
+#else
+#define SOCKLEN_T  int
+#endif
+
+#else //SUN
+//#ifdef MacOS //removed for 10.4 tigger.
+//#define SOCKLEN_T  int
+//#else //MacOS
+#define SOCKLEN_T  socklen_t
+//#endif //MacOS
 #endif //SUN
 
 WSMFclassInit(WSDxsocket,WSDxsocket);
@@ -61,6 +67,7 @@ WSDsocket* _xsocket_create_handler(){
 };
 
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class  _xsocket_init {
   public:
   _xsocket_init(){
@@ -69,6 +76,7 @@ class  _xsocket_init {
   };
 };
 _xsocket_init   WSGIappXsocketInit;
+#endif
 
 //typedef struct pthread_sem_struct{
 //  int locked;
@@ -179,13 +187,90 @@ fprintf(stderr,"WSDxsocket::connect invarid addr=%s\n",(char*)_dest_naddr);
       return WS_ERR;
     }
   }else{
-    server.sin_addr.s_addr = *(unsigned long *)hp->h_addr;
+//    server.sin_addr.s_addr = *(unsigned long *)hp->h_addr;
+    memcpy(&server.sin_addr.s_addr,hp->h_addr,hp->h_length);
   }
   server.sin_port = htons( _dest_port );
+  WSCulong timeout = _timeout *1000;
 
-  if( ::connect(_socket, (struct sockaddr *)&server,sizeof server) < 0 ){
+  if (timeout > 0){
+    int fl = fcntl((long)_socket,F_GETFL,0);
+    if (fl == -1){
+      seterr();
+      return 0;
+    }
+    int ret = fcntl((long)_socket,F_SETFL,fl | FNDELAY);
+    if (ret == -1){
+      seterr();
+      return 0;
+    }
+  }else{
+    int fl = fcntl((long)_socket,F_GETFL,0);
+    if (fl == -1){
+      seterr();
+      return 0;
+    }
+    int ret = fcntl((long)_socket,F_SETFL,fl & ~FNDELAY);
+    if (ret == -1){
+      seterr();
+      return 0;
+    }
+  }
+
+  int ret = ::connect(_socket, (struct sockaddr *)&server,sizeof server);
+  if( ret < 0){
+    long err = errno;
+    if (timeout > 0){
+      if (err == EINPROGRESS && timeout > 0){
+#ifndef MacOS
+        struct pollfd fds;
+        fds.fd = (long)_socket;
+        fds.events = POLLOUT;
+        fds.revents = 0;
+        ret = poll( &fds, 1, timeout );
+        if (ret < 1){
+          seterr();
+          close();
+          return WS_ERR;
+        }
+        if ((fds.revents & POLLOUT) != POLLOUT){
+          seterr();
+          close();
+          return WS_ERR;
+        }
+#else //MacOS
+        struct timeval tm;
+        tm.tv_sec = timeout/1000;
+        tm.tv_usec = 0;
+        fd_set write_fds;
+        FD_ZERO(&write_fds);
+        FD_SET((int)_socket,&write_fds);
+        ret = select(FD_SETSIZE,NULL,&write_fds,NULL,&tm);
+        long err = errno;
+        if (ret < 0 && err != EALREADY){
+          seterr();
+          close();
+          return WS_ERR;
+        }
+        if (ret == 0){
+          seterr(ETIMEDOUT);
+          close();
+          return WS_ERR;
+        }
+#endif //MacOS
+        int etmp = 0;
+        SOCKLEN_T len = 4;
+        ret = getsockopt(_socket,SOL_SOCKET,SO_ERROR,(void*)&etmp,&len);
+        if (ret != 0 || etmp != 0){
+          seterr(ETIMEDOUT);
+          close();
+          return WS_ERR;
+        }
+        return WS_NO_ERR;
+      }
+      close();
+    }
     seterr();
-    close();
 fprintf(stderr,"WSDxsocket::connect failed addr=%s port=%d\n",(char*)_dest_naddr,_dest_port);
     return WS_ERR;
   }
@@ -308,10 +393,10 @@ void* WSDxsocket::_accept_thread_(void* ptr){
     cm.display = disp;
     cm.window = win;
     cm.message_type = atom;
-    cm.format = 32;
+    cm.format = 8;
 
 
-    memcpy(cm.data.l,&data,sizeof(_accept_send_type));
+    memcpy(cm.data.b,&data,sizeof(_accept_send_type));
     XSendEvent(disp,win,False,NoEventMask,(_XEvent *)&cm);
 //printf("send message... win=0x%x\n",win);
     Window  win2  = XCreateSimpleWindow(disp,DefaultRootWindow(disp),400,400,
@@ -411,14 +496,14 @@ void* WSDxsocket::_udp_read_thread_(void* ptr){
       cm.display = disp;
       cm.window = win;
       cm.message_type = atom;
-      cm.format = 32;
+      cm.format = 8;
 
       pthread_sem_t* sem = new pthread_sem_t;
       _this->_sem = (void*)sem;
       pthread_sem_init_np(sem);
       pthread_sem_lock_np(sem);    
   
-      memcpy(cm.data.l,&data,sizeof(_accept_send_type));
+      memcpy(cm.data.b,&data,sizeof(_accept_send_type));
       XSendEvent(disp,win,False,NoEventMask,(_XEvent *)&cm);
 //printf("send message... win=0x%x\n",win);
       Window  win2  = XCreateSimpleWindow(disp,DefaultRootWindow(disp),400,400,
@@ -491,8 +576,8 @@ long WSDxsocket::acceptEx(void(*hd)(WSDsocket*,void*,WSCulong)){
 
 long WSDxsocket::close(void* sock){
   if (sock != 0){
-    ::shutdown((int)sock,0);
-    ::close((int)sock);
+    ::shutdown((long)sock,0);
+    ::close((long)sock);
     seterr();
     return WS_NO_ERR;
   }else{
@@ -540,29 +625,29 @@ long WSDxsocket::read(void* handle,WSCuchar* buf,long size){
     }
   }
   WSCulong timeout = 0;
-  if (_socket == (int)handle){
+  if (_socket == (long)handle){
     timeout = _timeout *1000;
   }else{
     timeout = _cl_timeout *1000;
   }
   if (timeout > 0){
-    int fl = fcntl((int)handle,F_GETFL,0);
+    int fl = fcntl((long)handle,F_GETFL,0);
     if (fl == -1){
       seterr();
       return 0;
     }
-    int ret = fcntl((int)handle,F_SETFL,fl | FNDELAY);
+    int ret = fcntl((long)handle,F_SETFL,fl | FNDELAY);
     if (ret == -1){
       seterr();
       return 0;
     }
   }else{
-    int fl = fcntl((int)handle,F_GETFL,0);
+    int fl = fcntl((long)handle,F_GETFL,0);
     if (fl == -1){
       seterr();
       return 0;
     }
-    int ret = fcntl((int)handle,F_SETFL,fl & ~FNDELAY);
+    int ret = fcntl((long)handle,F_SETFL,fl & ~FNDELAY);
     if (ret == -1){
       seterr();
       return 0;
@@ -575,7 +660,7 @@ long WSDxsocket::read(void* handle,WSCuchar* buf,long size){
   while(1){
     long ret = 0;
     if (_udp == False){
-      ret = recv((int)handle,(char*)&buf[read_len],len_to_read,0);
+      ret = recv((long)handle,(char*)&buf[read_len],len_to_read,0);
 //printf("WSDxsocket::read ret=%d len=%d eno=%d buf=0x%x handle=0x%x\n",ret,len_to_read,errno,buf,handle);
     }else{
       struct sockaddr_in fromaddr;
@@ -583,7 +668,7 @@ long WSDxsocket::read(void* handle,WSCuchar* buf,long size){
       fromaddr.sin_port = 0;
       fromaddr.sin_addr.s_addr = 0;
       SOCKLEN_T len = sizeof(fromaddr);
-      ret = recvfrom((int)handle,(char*)&buf[read_len],len_to_read,0,(sockaddr*)&fromaddr,&len);
+      ret = recvfrom((long)handle,(char*)&buf[read_len],len_to_read,0,(sockaddr*)&fromaddr,&len);
       _set_dest_addr_(htonl(fromaddr.sin_addr.s_addr));
       _dest_port = htons(fromaddr.sin_port);
 //printf("WSDxsocket::read udp ret=%d len=%d eno=%d buf=0x%x handle=0x%x\n",ret,len_to_read,errno,buf,handle);
@@ -592,12 +677,12 @@ long WSDxsocket::read(void* handle,WSCuchar* buf,long size){
     if (timeout > 0){
 #ifndef MacOS
       struct pollfd fds;
-      fds.fd = (int)handle;
+      fds.fd = (long)handle;
       fds.events = POLLIN;
       fds.revents = 0;
 #else //MacOS
       struct timeval tm;
-      tm.tv_sec = timeout;
+      tm.tv_sec = timeout/1000;
       tm.tv_usec = 0;
       fd_set read_fds;
       FD_ZERO(&read_fds);
@@ -685,7 +770,7 @@ long WSDxsocket::write(void* handle,WSCuchar* buf,long size){
   if (_udp != False){
     int val;
     val = 1;
-    int ret = setsockopt((int)handle, SOL_SOCKET, SO_BROADCAST,
+    int ret = setsockopt((long)handle, SOL_SOCKET, SO_BROADCAST,
                          (char *)&val, sizeof(val));
     if (ret < 0){
       seterr();
@@ -694,30 +779,30 @@ long WSDxsocket::write(void* handle,WSCuchar* buf,long size){
 //printf("WSDxsocket::write setsockopt... _socket=%d handle=%d ret=%d\n",_socket,handle,ret);
   }
   WSCulong timeout = 0;
-  if (_socket == (int)handle){
+  if (_socket == (long)handle){
     timeout = _timeout *1000;
   }else{
     timeout = _cl_timeout *1000;
   }
 
   if (timeout > 0){
-    int fl = fcntl((int)handle,F_GETFL,0);
+    int fl = fcntl((long)handle,F_GETFL,0);
     if (fl == -1){
       seterr();
       return 0;
     }
-    int ret = fcntl((int)handle,F_SETFL,fl | FNDELAY);
+    int ret = fcntl((long)handle,F_SETFL,fl | FNDELAY);
     if (ret == -1){
       seterr();
       return 0;
     }
   }else{
-    int fl = fcntl((int)handle,F_GETFL,0);
+    int fl = fcntl((long)handle,F_GETFL,0);
     if (fl == -1){
       seterr();
       return 0;
     }
-    int ret = fcntl((int)handle,F_SETFL,fl & ~FNDELAY);
+    int ret = fcntl((long)handle,F_SETFL,fl & ~FNDELAY);
     if (ret == -1){
       seterr();
       return 0;
@@ -730,7 +815,7 @@ long WSDxsocket::write(void* handle,WSCuchar* buf,long size){
   while(1){
     long ret = 0;
     if (_udp == False){
-      ret = send((int)handle,(char*)&buf[write_len],len_to_write,0);
+      ret = send((long)handle,(char*)&buf[write_len],len_to_write,MSG_NOSIGNAL);
     }else if (_udp_binded == False){
       struct sockaddr_in toaddr;
       toaddr.sin_family = AF_INET;
@@ -744,20 +829,20 @@ long WSDxsocket::write(void* handle,WSCuchar* buf,long size){
 //printf("here2..... addr=0x%x port=%d\n",toaddr.sin_addr.s_addr, toaddr.sin_port);
       }
 
-      ret = sendto((int)handle,(char*)&buf[write_len],len_to_write,0,
+      ret = sendto((long)handle,(char*)&buf[write_len],len_to_write,MSG_NOSIGNAL,
                    (sockaddr*)&toaddr,sizeof(toaddr));
 //printf("sendto.. size=%d\n",sizeof(toaddr));
 //printf("sendto.. handle=%d ret=%d errno=%d\n",handle,ret,errno);
     }
-    if (_timeout > 0){
+    if (timeout > 0){
 #ifndef MacOS
       struct pollfd fds;
-      fds.fd = (int)handle;
+      fds.fd = (long)handle;
       fds.events = POLLOUT;
       fds.revents = 0;
 #else //MacOS
       struct timeval tm;
-      tm.tv_sec = _timeout;
+      tm.tv_sec = timeout/1000;
       tm.tv_usec = 0;
       fd_set write_fds;
       FD_ZERO(&write_fds);
