@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <X11/Intrinsic.h>
 #include <WSCdevice.h>
@@ -39,6 +35,7 @@
 #include <X11/Shell.h>
 #include <X11/StringDefs.h>
 #include <form.h>
+#include <WSCbaseList.h>
 
 #if 1 //UNICODE
 #include <xunicode.h>
@@ -49,12 +46,15 @@ WSDdev* _wsdxwindev_init_(){
   return new WSDxwinDev();
 }
 
+#ifndef NO_GLOBAL_CONSTRUCTORS
 class _WSDxwinDev_init_ {
   public:_WSDxwinDev_init_(){
      WSGIappDevice()->setCreateHandler("windowDev",_wsdxwindev_init_);
   };
 };
 static _WSDxwinDev_init_ _init_to_run_WSDxwinDev_;
+#endif
+
 static Atom wm_protocols = 0;
 static Atom wm_delete = 0;
 
@@ -70,9 +70,18 @@ WSDxwinDev::WSDxwinDev(){
   _modal = 0;
   _sizehints = NULL;
   _title_bar = 0;
+#ifndef NO_MWT_FUNCTION
+  _rebase_handle = 0;
+#endif
+  _timer_handle = 0;
 }
 
 WSDxwinDev::~WSDxwinDev(){
+  if (_timer_handle != 0 && _requested_top != 0){
+    WSGIappTimer()->delTriggerProc(_timer_handle);
+    _timer_handle = 0;
+    _requested_top = 0;
+  }
   destroyWindow();
 }
 
@@ -99,6 +108,11 @@ long WSDxwinDev::setValue(long kind,void* data){
              if (getVisible() == False){
                _diff_x = 0;
              }
+#ifndef NO_MWT_FUNCTION
+             if (_rebase_handle != 0){
+               break;
+             }
+#endif
              if (_swidget != 0){
                if (getVisible() != False){
                  XtSetArg(args[0],XtNx,*(short*)data);
@@ -110,6 +124,11 @@ long WSDxwinDev::setValue(long kind,void* data){
              if (getVisible() == False){
                _diff_y = 0;
              }
+#ifndef NO_MWT_FUNCTION
+             if (_rebase_handle != 0){
+               break;
+             }
+#endif
              if (_swidget != 0){
                if (getVisible() != False){
                  XtSetArg(args[0],XtNy,*(short*)data);
@@ -119,8 +138,7 @@ long WSDxwinDev::setValue(long kind,void* data){
              break;
     case WSDEV_WIDTH:
              if (_swidget != 0){
-               if (getVisible() != False && *(WSCushort*)data > 0){
-
+               if (*(WSCushort*)data > 0){
                  WSCushort val;
                  XtSetArg(args[0],XtNwidth,&val);
                  XtGetValues((Widget)_swidget,args,1);
@@ -134,7 +152,7 @@ long WSDxwinDev::setValue(long kind,void* data){
     case WSDEV_HEIGHT:
 WSMFtrace("WSCxwinDev::setValue height=%d _w_h=%d\n",*(WSCushort*)data,_w_h);
              if (_swidget != 0){
-               if (getVisible() != False && *(WSCushort*)data > 0){
+               if (*(WSCushort*)data > 0){
                  WSCushort val;
                  XtSetArg(args[0],XtNheight,&val);
                  XtGetValues((Widget)_swidget,args,1);
@@ -189,12 +207,12 @@ WSMFtrace("WSCxwinDev::setValue height=%d _w_h=%d\n",*(WSCushort*)data,_w_h);
                case WS_FRAME:      vl = 4; break;
                default: vl = 1;
              };
-#if 0 //DEVELOP
-             if ((Widget)_widget != (Widget)NULL){
+
+             if ((Widget)_swidget != (Widget)NULL){
                XtSetArg(args[0],"mwmDecorations",vl);
                XtSetValues((Widget)_swidget,args,1);
              }
-#endif
+
              _title_bar_style = vl;
              if (vl == 0){
                _diff_x = 0;
@@ -291,14 +309,31 @@ WSMFtrace("WSCxwinDev::setValue height=%d _w_h=%d\n",*(WSCushort*)data,_w_h);
              }
              fl = *(WSCbool*)data;
              if (fl != False){
+#ifndef NO_MWT_FUNCTION
+               if (getNeedGrab() != False){
+                 WSGIxwinAppDev()->setGrabbedDisplay(XtDisplay(_widget));
+                 XGrabServer(XtDisplay(_widget));
+                 if (WSGIxwinAppDev()->getOwnerGrab() != False){
+                   WSGIxwinAppDev()->setOwnerGrab(False);
+                 }
+               }
+#endif
                XGrabPointer(XtDisplay(_widget),XtWindow(_widget),False,
                             ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
                             GrabModeAsync,GrabModeAsync,None,None,CurrentTime);
                XGrabKeyboard(XtDisplay(_widget),XtWindow(_widget),False,
                             GrabModeAsync,GrabModeAsync,CurrentTime);
+               WSGIappMouse()->setMouseFocusClient(NULL);
              }else{
                XUngrabPointer(XtDisplay(_widget),CurrentTime);
                XUngrabKeyboard(XtDisplay(_widget),CurrentTime);
+#ifndef NO_MWT_FUNCTION
+               if (getNeedGrab() != False){
+                 XUngrabServer(XtDisplay(_widget));
+                 WSGIxwinAppDev()->setGrabbedDisplay(NULL);
+                 WSGIxwinAppDev()->setOwnerGrab(False);
+               }
+#endif
              }
              break;
     case WSDEV_MODAL:
@@ -329,15 +364,28 @@ WSMFtrace("WSCxwinDev::setValue height=%d _w_h=%d\n",*(WSCushort*)data,_w_h);
              }else{
                return WS_ERR;
              }
-    case WSDEV_MOUSE_NO:
+    case WSDEV_MOUSE_NO:{
              _mno = *(WSCushort*)data;
-             if (_widget != NULL && XtWindow(_widget) != 0){
-               long val = WSGIappMouse()->getMouseCursor(_mouse_no);
-               if (val != 0){
-                 XDefineCursor(XtDisplay(_widget),XtWindow(_widget),val);
+             long val = WSGIappMouse()->getMouseCursor(_mouse_no);
+             if (val != 0){
+               WSCbool gmouse = WSGIappMouse()->getGlobalMouseIdStatus();
+               if (_swidget != NULL && XtWindow(_swidget) != 0){
+                 if (_mouse_val != val){
+                   _mouse_val = val;
+                   _global_mouse = gmouse;
+                   XDefineCursor(XtDisplay(_swidget),XtWindow(_swidget),val);
+                 }
+               }
+               if (_widget != NULL && XtWindow(_widget) != 0){
+                 if (_mouse_val != val){
+                   _mouse_val = val;
+                   _global_mouse = gmouse;
+                   XDefineCursor(XtDisplay(_widget),XtWindow(_widget),val);
+                 }
                }
              }
              return WS_ERR;
+    }
     default:
       return WS_ERR;
   }
@@ -350,6 +398,7 @@ long WSDxwinDev::createWindow(){
   }
   _no_frame_first_time = 0;
 //  _top_float = False;
+  _mouse_val = 0;
 
   short x,y;    
   WSCushort w,h;    
@@ -372,11 +421,30 @@ long WSDxwinDev::createWindow(){
   XtSetArg(args[anum],XtNheight,h);anum++;
   XtSetArg(args[anum],"mwmDecorations",_title_bar_style);anum++;
   XtSetArg(args[anum],"dialogStyle",_modal);anum++;
+#ifndef NO_MWT_FUNCTION
+  void (*create_handler)(WSCbase*) =
+     (void(*)(WSCbase*))WSGIappObjectList()->getInternalWindowCreateHandler();
+  if (create_handler){
+    create_handler(getAttachedClient());
+  }
+  if (WSGIappDev()->getRebaseHandle() != 0){
+    _rebase_handle = WSGIappDev()->getRebaseHandle();
+    WSGIappDev()->setRebaseHandle(0);
+  }
+#endif
 
   if (_title_bar_style == 0 && _no_frame == 1){
     XtSetArg(args[anum],XtNoverrideRedirect,True);anum++;
   }else{
+#ifndef NO_MWT_FUNCTION
+    if (_rebase_handle != 0){
+      XtSetArg(args[anum],XtNoverrideRedirect,True);anum++;
+    }else{
+      XtSetArg(args[anum],XtNoverrideRedirect,False);anum++;
+    }
+#else
     XtSetArg(args[anum],XtNoverrideRedirect,False);anum++;
+#endif
   }
   XtSetArg(args[anum],XtNborderWidth,0); anum++;
   WSDxappDev* app = WSGIxwinAppDev(); 
@@ -389,8 +457,24 @@ long WSDxwinDev::createWindow(){
   if (alt != NULL){
     _swidget = alt;
   }else{
-    _swidget = XtCreatePopupShell ( "wsshell", transientShellWidgetClass, parent,
+#ifndef NO_MWT_FUNCTION
+    if (_rebase_handle != 0){
+      _swidget = XtCreatePopupShell( "wsshell",shellWidgetClass,parent,
                (ArgList)args, anum );
+      XtManageChild(_swidget);
+      if (XtWindow(_swidget) != 0){
+        XReparentWindow(XtDisplay(_swidget), XtWindow(_swidget),
+                        (Window)_rebase_handle, 0,0);
+        _need_grab = True;
+      }
+    }else{
+      _swidget = XtCreatePopupShell( "wsshell",transientShellWidgetClass,parent,
+               (ArgList)args, anum );
+    }
+#else
+    _swidget = XtCreatePopupShell( "wsshell",transientShellWidgetClass,parent,
+               (ArgList)args, anum );
+#endif
   }
   XtAddEventHandler(_swidget,StructureNotifyMask | SubstructureNotifyMask,False,(XtEventHandler)_evh_resize,this);
   XtAddEventHandler(_swidget,FocusChangeMask,False,(XtEventHandler)_evh_focus_ch,this);
@@ -439,6 +523,9 @@ long WSDxwinDev::createWindow(){
 
 long WSDxwinDev::destroyWindow(){
 //printf("WSCxwinDev::destroyWindow dev=0x%x _swidget=0x%x vis=%d\n",this,_swidget,getVisible());
+#ifndef NO_MWT_FUNCTION
+  _rebase_handle = 0;
+#endif
   WSCbool destroy = False;
   if (_widget != 0){
     XtRemoveEventHandler(_widget,ExposureMask,False,(XtEventHandler)_evh_expose,this);
@@ -449,18 +536,18 @@ long WSDxwinDev::destroyWindow(){
     XtRemoveEventHandler(_widget,LeaveWindowMask,False,(XtEventHandler)_evh_mouse_out,(WSDxformDev*)this);
     XtRemoveEventHandler(_widget,KeyPressMask | KeyReleaseMask,False,(XtEventHandler)_evh_dummy,(WSDxformDev*)this);
     XtRemoveCallback(_widget,XtNdestroyCallback,_evh_delete2,this);
-#if 1 //XUNICODE
-    XUFreeInfoWindow(XtDisplay(_widget),XtWindow(_widget));
-#endif //UNICODE
+
+//    XUFreeInfoWindow(XtDisplay(_widget),XtWindow(_widget));
     destroy = True;
     XtDestroyWidget(_widget);
     _widget = 0;
   }
   if (_swidget != 0){
-#if 1 //USE_XUNICODE
-    XUFreeInfoWindow(XtDisplay(_swidget),XtWindow(_swidget));
-#endif
- 
+//    XUFreeInfoWindow(XtDisplay(_swidget),XtWindow(_swidget));
+    XtRemoveEventHandler(_swidget,StructureNotifyMask | SubstructureNotifyMask,False,(XtEventHandler)_evh_resize,this);
+    XtAddEventHandler(_swidget,FocusChangeMask,False,(XtEventHandler)_evh_focus_ch,this);
+    XtAddEventHandler(_swidget,NoEventMask,True,(XtEventHandler)_evh_client_message,this);
+
     XtDestroyWidget(_swidget);
     _swidget = 0;
   }
@@ -468,11 +555,25 @@ long WSDxwinDev::destroyWindow(){
     clearChildWidgets();
   }
   setInitialized(False);
-  WSGIxwinAppDev()->dispatchEvent();
+// dispatchEvent() causes a bug of hung-up on eclipse.
+//  WSGIxwinAppDev()->dispatchEvent();
+  WSGIxwinAppDev()->processEvent();
   if (_sizehints != NULL){
     XFree(_sizehints);
     _sizehints = NULL;
   }
+#ifndef NO_MWT_FUNCTION
+  if (destroy != False){
+    void (*destroy_handler)(WSCbase*) =
+       (void(*)(WSCbase*))WSGIappObjectList()->getInternalWindowDestroyHandler();
+    if (destroy_handler){
+      WSCbase* target = getAttachedClient();
+      if (target != NULL){
+        destroy_handler(target);
+      }
+    }
+  }
+#endif
   return WS_NO_ERR;
 }
 
@@ -480,6 +581,7 @@ void WSDxwinDev::_float_top_timer_proc(WSCuchar,void* data){
   WSDxwinDev* dev = (WSDxwinDev*)data;
   WSCbool fl = _chk_popup(dev);
   if (fl == False){
+    dev->_timer_handle = 0;
     dev->_requested_top = 0;
     return;
   }
@@ -491,9 +593,10 @@ void WSDxwinDev::_float_top_timer_proc(WSCuchar,void* data){
       XConfigureWindow(XtDisplay(dev->_swidget),
                        XtWindow(dev->_swidget),CWStackMode,&wc);
     }
-    WSGIappTimer()->addTriggerProc(_float_top_timer_proc,WS500MS,dev);
+    dev->_timer_handle = WSGIappTimer()->addTriggerProc(_float_top_timer_proc,WS500MS,dev);
     dev->_requested_top = 1;
   }else{
+    dev->_timer_handle = 0;
     dev->_requested_top = 0;
   }
 
@@ -511,17 +614,18 @@ WSMFtrace("WSCxwinDev::setVisible _w_h=%d\n",_w_h);
     if (_swidget != 0){
       if (_no_frame == 0){
         XFlush(XtDisplay(_swidget));
+//        XtUnmapWidget(_swidget);
         XUnmapWindow(XtDisplay(_swidget),XtWindow(_swidget));
         XtUnmanageChild(_swidget);
+//printf("WSCxwinDev::setVisible(False) dev=0x%x _swidget=0x%x unmanaged\n",this,_swidget);
       }else{
-         _del_list(this);
+        _del_list(this);
         XUnmapWindow(XtDisplay(_swidget),XtWindow(_swidget));
         XFlush(XtDisplay(_swidget));
         XtRemoveCallback(_widget,XtNdestroyCallback,_evh_delete2,this);
         XtDestroyWidget(_swidget);
-#if 1 //USE_XUNICODE
-        XUFreeInfoWindow(XtDisplay(_swidget),XtWindow(_swidget));
-#endif
+//printf("WSCxwinDev::setVisible(False) dev=0x%x _swidget=0x%x destroyed\n",this,_swidget);
+//        XUFreeInfoWindow(XtDisplay(_swidget),XtWindow(_swidget));
         clearChildWidgets();
         onUnmap();
         if (_sizehints != NULL){
@@ -529,12 +633,25 @@ WSMFtrace("WSCxwinDev::setVisible _w_h=%d\n",_w_h);
           _sizehints = NULL;
         }
       }
-
     }
+#ifndef NO_MWT_FUNCTION
+    void (*unvis_handler)(WSCbase*) =
+         (void(*)(WSCbase*))WSGIappObjectList()->getInternalWindowUnvisibleHandler();
+    if (unvis_handler){
+      unvis_handler(getAttachedClient());
+    }
+#endif
   }else{
     if (_swidget == 0){
        createWindow();
     }
+#ifndef NO_MWT_FUNCTION
+    void (*vis_handler)(WSCbase*) =
+         (void(*)(WSCbase*))WSGIappObjectList()->getInternalWindowVisibleHandler();
+    if (vis_handler){
+      vis_handler(getAttachedClient());
+    }
+#endif
 
     if (getPixmapStyle() != WS_DIRECT_WINDOW){
       if (bk == False){
@@ -566,13 +683,18 @@ WSMFtrace("WSDxwinDev::setVisible %d,%d,%d,%d  %d,%d,%d,%d\n",x,y,w,h,x1,y1,w1,h
       XtSetArg(args[1],XtNheight,h);
       XtSetArg(args[2],XtNx,x);
       XtSetArg(args[3],XtNy,y);
-//printf("WSDxwinDev::setVisible value set..\n");
       XtSetValues(_swidget,args,4);
 //printf("WSDxwinDev::setVisible here2\n");
       XtSetValues(_widget,args,2);
       XFlush(XtDisplay(_widget));
 	}
     XtManageChild(_swidget);
+#ifndef NO_MWT_FUNCTION
+    if (_rebase_handle != 0){
+      XReparentWindow(XtDisplay(_swidget), XtWindow(_swidget),(Window)_rebase_handle, 0,0);
+      _need_grab = True;
+    }
+#endif
   }
   return WS_NO_ERR;
 }
@@ -594,7 +716,9 @@ void WSDxwinDev::_evh_focus_ch(Widget w,XtPointer ptr,XEvent* ev,WSCbool* fl){
     if (fcl != NULL){
       WSCbase* cl = dev->getAttachedClient();
       WSCbase* pcl = fcl;
-      if (cl != NULL){
+      if (cl != NULL && cl->getVisible() != False){
+extern void _set_active_instance(WSCbase*);
+        _set_active_instance(cl);
         WSCbool fl = False;
         while(1){
           if (pcl == NULL){
@@ -604,12 +728,25 @@ void WSDxwinDev::_evh_focus_ch(Widget w,XtPointer ptr,XEvent* ev,WSCbool* fl){
             fl = True;
             break;
           }
-          pcl = pcl->getParent();
+//          pcl = pcl->getParent();
+          WSDdev* dev = pcl->getowndev();
+          if (dev == NULL){
+            pcl = pcl->getParent();
+            continue;
+          }
+          WSDdev* pdev = dev->getParentDev();
+          if (pdev == NULL){
+            break;
+          }
+          pcl = pdev->getAttachedClient();
         }
         if (fl != False){
           fcl->onWindowFocusChange(True);
+//          dev->onFocusChange();
         }else{
+          fcl->setSpecialFocus(False);
           fcl->setFocus(False);
+//          dev->onFocusChange();
         }
       }else{
         fcl->onWindowFocusChange(True);
@@ -620,12 +757,44 @@ void WSDxwinDev::_evh_focus_ch(Widget w,XtPointer ptr,XEvent* ev,WSCbool* fl){
   }else if (ev->type == FocusOut){
     WSCbase* fcl = WSCbase::getFocusInstance();
     if (fcl != NULL){
-      fcl->onWindowFocusChange(False);
+      WSCbase* cl = dev->getAttachedClient();
+      WSCbase* pcl = fcl;
+      if (cl != NULL){
+        WSCbool fl = False;
+        while(1){
+          if (pcl == NULL){
+            break;
+          }
+          if (cl == pcl){
+            fl = True;
+            break;
+          }
+//          pcl = pcl->getParent();
+          WSDdev* dev = pcl->getowndev();
+          if (dev == NULL){
+            pcl = pcl->getParent();
+            continue;
+          }
+          WSDdev* pdev = dev->getParentDev();
+          if (pdev == NULL){
+            break;
+          }
+          pcl = pdev->getAttachedClient();
+        }
+        if (fl != False){
+          fcl->onWindowFocusChange(False);
+        }
+      }
     }
   }
 }
 
 void WSDxwinDev::_evh_mouse_in(Widget w,XtPointer ptr,XEvent* ev,WSCbool* fl){
+  if (WSGIappMouse()->getGlobalMouseSensitive() == False){
+    WSGIxwinAppDev()->setXEvent(NULL);
+    return;
+  }
+
   if (WSGIappDev()->getMode() == WS_MODE_NS_PLUGIN){
     WSGIxwinAppDev()->_dispatchEvent(False,ev);
   }
@@ -635,9 +804,14 @@ void WSDxwinDev::_evh_mouse_in(Widget w,XtPointer ptr,XEvent* ev,WSCbool* fl){
   }
 //printf("dev=0x%x _swidget=0x%x\n",dev,dev->_swidget);
 //fflush(stdout);
+
   long val = WSGIappMouse()->getMouseCursor(dev->_mouse_no);
   if (val != 0){
-    XDefineCursor(XtDisplay((Widget)dev->_swidget),XtWindow((Widget)dev->_swidget),val);
+    if (dev->_mouse_val != val){
+      dev->_mouse_val = val;
+      dev->_global_mouse = WSGIappMouse()->getGlobalMouseIdStatus();
+      XDefineCursor(XtDisplay((Widget)dev->_swidget),XtWindow((Widget)dev->_swidget),val);
+    }
   }
   WSDxformDev::_evh_mouse_in(w,(WSDxformDev*)dev,ev,fl);
 }
@@ -649,6 +823,7 @@ void WSDxwinDev::_evh_delete2(Widget w,XtPointer ptr,XtPointer ){
    }
    dev->_clear_widget();
    dev->onUnmap();
+   dev->execEvent(WSEV_WINDOW_CLOSED,NULL);
 }
 
 WSClistData WSDxwinDev::_no_frame_list;
@@ -722,7 +897,36 @@ void WSDxwinDev::_evh_resize(Widget w,XtPointer ptr,XEvent* ev,WSCbool* ){
     WSGIxwinAppDev()->_dispatchEvent(False,ev);
   }
   WSDxwinDev* dev = (WSDxwinDev*)ptr;
+#ifndef NO_MWT_FUNCTION
+//  if (dev->_rebase_handle != 0 && ev->type == MapNotify &&
+//      XtWindow(dev->_swidget) == ev->xconfigure.window){
+  if (dev->_rebase_handle != 0){
+    WSCuint child_num = 0;
+    Window* children;
+    Window root;
+    Window parent;
+    long ret = XQueryTree(XtDisplay(dev->_swidget),
+                          XtWindow(dev->_swidget),
+                          &root,&parent,&children,&child_num);
+    if (ret != 0){
+      if (child_num > 0){
+        XFree(children);
+      }
+      if (parent != dev->_rebase_handle){
+        XReparentWindow(XtDisplay(dev->_swidget), XtWindow(dev->_swidget),
+                        (Window)dev->_rebase_handle, 0,0);
+        dev->_need_grab = True;
+      }
+    }else{
+      XReparentWindow(XtDisplay(dev->_swidget), XtWindow(dev->_swidget),
+                      (Window)dev->_rebase_handle, 0,0);
+      dev->_need_grab = True;
+    }
+  }
+  if (ev->type == ReparentNotify && dev->_rebase_handle == 0){
+#else
   if (ev->type == ReparentNotify){
+#endif
     if (dev->getVisible() == False){
       return;
     }
@@ -758,11 +962,14 @@ void WSDxwinDev::_evh_resize(Widget w,XtPointer ptr,XEvent* ev,WSCbool* ){
     }
     
 //printf("WSDxwinDev::_evh_resize here! %d\n",dev->_title_bar);
-    if (dev->_title_bar == WS_ONLY_TITLE && dev->_sizehints == NULL){
-//printf("WSDxwinDev::_evh_resize here\n");
-      dev->_sizehints = XAllocSizeHints();
+    if (dev->_title_bar == WS_ONLY_TITLE){
+      if (dev->_sizehints == NULL){
+        dev->_sizehints = XAllocSizeHints();
+      }
 //      dev->_sizehints->flags = USPosition | USSize | PMinSize | PMaxSize ;
-      dev->_sizehints->flags = USSize | PMinSize | PMaxSize ;
+      dev->_sizehints->flags = USPosition | USSize | PMinSize | PMaxSize ;
+      dev->_sizehints->x = dev->_w_x;
+      dev->_sizehints->y = dev->_w_y;
       dev->_sizehints->min_width = nw;
       dev->_sizehints->max_width = nw;
       dev->_sizehints->base_width = nw;
@@ -778,7 +985,7 @@ void WSDxwinDev::_evh_resize(Widget w,XtPointer ptr,XEvent* ev,WSCbool* ){
 WSMFtrace("WSDxwinDev::_evh_resize %d,%d,%d,%d\n",area.x,area.y,area.width,area.height);
     dev->onResize(&area);
   }
- 
+   
   if (dev->_swidget == NULL){
     return;
   }
@@ -789,7 +996,6 @@ WSMFtrace("WSDxwinDev::_evh_resize %d,%d,%d,%d\n",area.x,area.y,area.width,area.
                                RevertToParent,CurrentTime);
   }
 
-
   if (dev->_title_bar_style == 0 && dev->_no_frame == 1 &&
       ev->type == ConfigureNotify){
     _popup_window(dev);
@@ -804,11 +1010,17 @@ WSMFtrace("WSDxwinDev::_evh_resize %d,%d,%d,%d\n",area.x,area.y,area.width,area.
     XGetGeometry( XtDisplay((Widget)dev->_swidget),
                   XtWindow((Widget)dev->_swidget),&root,&dustx,&dusty,
                   &vw,&vh,&vborder,&vdepth);
+//printf("WSDxwinDev::_evh_resize2 x,y=%d,%d border=%d,%d\n",dustx,dusty,vborder,vdepth);
     if (ev->xconfigure.width != (long)vw ||
         ev->xconfigure.height != (long)vh ){
       return;
     }
-
+    if (dev->getVisible() == False){
+      //To reject an old configure event which
+      //are often generated after calling
+      //setVisible(False) and setValue(size).
+      return;
+    }
     short x,y;
 //    WSCushort w,h;
     Arg args[4];
@@ -865,10 +1077,14 @@ void WSDxwinDev::_evh_client_message(Widget w,XtPointer ptr,XEvent* ev,WSCbool*
   }
 //printf("WSDxwinDev::_evh_client_message !! XXXZZ\n");
   WSDxwinDev* dev = (WSDxwinDev*)ptr;
+//printf("WSDxwinDev::_evh_client_message !! dev=0x%x\n",dev);
   if (ev->type == ClientMessage){
     WSCbase* client = dev->getAttachedClient();
 	if (client != NULL){
-	  client->setVisible(False);
+      if (WSGFgetWindowEnableClose(client) != False){
+	    client->setVisible(False);
+      }
+      client->execEventProc(WSEV_WINDOW_CLOSED,NULL);
 	}
   }
 }
@@ -893,3 +1109,45 @@ long WSDxwinDev::getValue(long kind,void* val){
   }
   return WSDwindowDev::getValue(kind,val);
 }
+
+#ifndef NO_MWT_FUNCTION
+void WSDxwinDev::getDiffCoordinates(long* ox,long* oy){
+  *ox = 0;
+  *oy = 0;
+  short x,y;    
+  WSCushort w,h;    
+  getWindowSize(&x,&y,&w,&h);
+  if (_swidget == 0){
+    return;
+  }
+  Display* disp = XtDisplay(_swidget);
+  Window win = XtWindow(_swidget);
+  int total_x = 0;
+  int total_y = 0;
+  while(1){
+    WSCuint child_num = 0;
+    Window* children;
+    Window root;
+    Window parent;
+    long ret = XQueryTree(disp,win,&root,&parent,&children,&child_num);
+    if (ret == 0){
+      break;
+    }
+    if (child_num > 0){
+      XFree(children);
+    }
+    int dx,dy;
+    WSCuint dw,dh;
+    WSCuint border,depth;
+    XGetGeometry(disp,win,&root,&dx,&dy,&dw,&dh,&border,&depth);
+    total_x += dx;
+    total_y += dy;
+    if (parent == root){
+      break;
+    }
+    win = parent;
+  }
+  *ox = total_x - x;
+  *oy = total_y - y;
+}
+#endif
