@@ -1,5 +1,5 @@
 //
-// Copyright (C) 1999-2004 WideStudio Development Team
+// Copyright (C) 1999-2006 WideStudio/MWT Project Team
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to
@@ -14,15 +14,11 @@
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-// WIDESTUDIO DEVELOPMENT TEAM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 // LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// Except as contained in this notice, the name of WideStudio Development Team
-// shall not be used in advertising or otherwise to promote the sale, use or
-// other dealings in this Software without prior written authorization from
-// WideStudio Development Team.
 
 #include <x11/WSxcom.h>
 #include <x11/WSDxappDev.h>
@@ -38,7 +34,7 @@
 
 //#define DBG
 
-extern int WSGFpickupString(unsigned char *dest,unsigned  char *src, int *n);
+//extern int WSGFpickupString(unsigned char *dest,unsigned  char *src, int *n);
 extern char *EUCToJIS(char *input);
 
 extern "C" {
@@ -59,9 +55,9 @@ static void _init_plist(){
   _pix_list = new WSClistData();
   _pix_ws = new WSClistData();
   _pix_hs = new WSClistData();
-  _used_pix_list = new WSClistData();
-  _used_pix_ws = new WSClistData();
-  _used_pix_hs = new WSClistData();
+  _used_pix_list = new WSClistData(64);
+  _used_pix_ws = new WSClistData(64);
+  _used_pix_hs = new WSClistData(64);
 }
 
 
@@ -69,6 +65,12 @@ Pixmap WSGFxGetPixmap(WSCushort w,WSCushort h){
   if (_pix_list == NULL){
     _init_plist();
   }
+  if (w > 32767){
+    w = 32767;
+  }
+  if (h > 32767){
+    h = 32767;
+  }
   long pw = (long)w;
   long ph = (long)h;
   long iw = 0;
@@ -194,66 +196,84 @@ char *EUCToJIS(char *input){
 //static long _lang_inited = False;
 //static long _lang = 0;
 
-void WSGFxdrawTextString(Display* disp,Window win,GC gc,
-                        XFontStruct* efs,XFontStruct* kf,XFontStruct* gf,
+void WSGFxdrawTextString(Display* disp,Window win,GC gc,char* fname,
                         WSCulong fg,WSCulong bg,
                         long x,long y,char *string,
-                        WSCushort kwidth,WSCushort ewidth,
-                        WSCushort kheight,WSCushort eheight,
+                        WSCushort font_height,
                         long cur,WSCbool inter_cur,WSCushort linew,
-                        long scur1,long scur2 ){
+                        long scur1,long scur2,long ustart,long ulen){
 
-  y -= 1;
 
   XUChar work[8192];
   long len = strlen(string);
   if (len > 0){
     int slen = XUutf8Encode(work, 8192, string, -1);
+
+    XCharStruct overAll;
+    int dir = 0;
+    int ascent = 0;
+    int descent = 0;
+    XUTextExtents(disp,fname,font_height,work,slen,&dir,&ascent,&descent,&overAll);
+    y -= descent;
+    if (ulen > 0 && ustart + ulen > 0){
+      if (ustart < 0){
+        ustart = 0;
+        ulen = ustart + ulen;
+      }
+      long posx = 0;
+      if (ustart > 0){
+        posx = XUTextWidth(disp, fname, font_height,work, ustart);
+      }
+      long posx2 = XUTextWidth(disp, fname, font_height,work, ustart+ulen);
+      XSetLineAttributes(disp,gc,1, LineSolid,CapButt,WS_JOIN);
+      XDrawLine(disp,win,gc,x+posx, y+2, x+posx2, y+2);
+    }
     if (scur1 <= 0 && scur2 >= slen){
       XSetForeground(disp,gc,bg);
       XSetBackground(disp,gc,fg);
-      XUDrawString(disp,win, efs,gc, x, y, work, -1, True);
+      XUDrawString(disp,win, fname,font_height,gc, x, y, work, -1, True);
       XSetForeground(disp,gc,fg);
       XSetBackground(disp,gc,bg);
     }else{
-      XUDrawString(disp,win, efs,gc, x, y, work, -1, False);
+      XUDrawString(disp,win,fname,font_height,gc, x, y, work, -1, False);
       if (scur1 < 0) scur1 = 0;
       if (scur2 < 0) scur2 = 0;
       if (scur1 > slen) scur1 = slen;
       if (scur2 > slen) scur2 = slen;
       if (scur1 != scur2){
         int x2 = x;
-        if (scur1 > 0) x2 += XUTextWidth(disp, efs, work, scur1);
+        if (scur1 > 0) x2 += XUTextWidth(disp, fname, font_height,work, scur1);
         XSetForeground(disp,gc,bg);
         XSetBackground(disp,gc,fg);
-        XUDrawString(disp,win, efs,gc, x2, y, &work[scur1], scur2 - scur1, True);
+        XUDrawString(disp,win, fname,font_height,gc, x2, y, &work[scur1], scur2 - scur1, True);
         XSetForeground(disp,gc,fg);
         XSetBackground(disp,gc,bg);
       }
     }
     if (cur >= 0 && cur < slen + inter_cur ){
       int x2 = x;
-      if (cur > 0) x2 += XUTextWidth(disp, efs, work, cur);
+      if (cur > 0) x2 += XUTextWidth(disp, fname, font_height,work, cur);
       if (inter_cur == False){
         XSetForeground(disp,gc,bg);
         XSetBackground(disp,gc,fg);
-        XUDrawString(disp,win, efs,gc, x2, y, &work[cur], 1, True);
+        XUDrawString(disp,win, fname,font_height,gc, x2, y, &work[cur], 1, True);
         XSetForeground(disp,gc,fg);
         XSetBackground(disp,gc,bg);
       }else{
         if (work[cur] != '\n'){
-          draw_cur(disp,win,gc,linew,x2,y,eheight);
+          draw_cur(disp,win,gc,linew,x2,y,font_height);
         }
       }
     }
   }else if (inter_cur == 1 && cur == 0){
-    draw_cur(disp,win,gc,linew,x,y,eheight);
+    y -= 2;
+    draw_cur(disp,win,gc,linew,x,y,font_height);
   }
 
   return;
 }
 
-
+#if 0
 int WSGFpickupString(unsigned char *dest,unsigned char *src, int *n) {
    int jflag;
    
@@ -261,7 +281,7 @@ int WSGFpickupString(unsigned char *dest,unsigned char *src, int *n) {
 
    if (src[0] == 0) return 1;
 
-   if (src[0] == 0x8f || src[0] > 0xf4){
+   if (src[0] == 0x8f || src[0] > 0xfc){
      jflag = WS_GAIJI;
    }else if (src[0] == 0x8e){
      jflag = WS_HKANA;
@@ -301,7 +321,7 @@ int WSGFpickupString(unsigned char *dest,unsigned char *src, int *n) {
          dest[*n] = src[*n];
          *n = *n +1;
        }
-     }else if (src[*n] == 0x8f || src[*n] > 0xf4){ //is gaiji?
+     }else if (src[*n] == 0x8f || src[*n] > 0xfc){ //is gaiji?
        if (jflag != WS_GAIJI){
          dest[*n] = 0;
          if (jflag == WS_KANJI){
@@ -347,15 +367,16 @@ int WSGFpickupString(unsigned char *dest,unsigned char *src, int *n) {
      *n = *n +1;
    }
 }
-
+#endif
 WSCulong WSGFxcolorval(WSDcolor* work,WSCuchar r,WSCuchar g,WSCuchar b){
 static WSClistData cval;
 static WSClistData clist;
   long num = cval.getNum();
   long val = r << 16 | g << 8 | b;
   long i;
+  long* cval_list = (long*)cval.getBuf();
   for(i=0; i<num; i++){
-    if (val == (long)cval[i]){
+    if (val == cval_list[i]){
       return (WSCulong)clist[i];
     }
   }
